requirejs.config({
    paths: {
        text: '../Scripts/text',
        durandal: '../Scripts/durandal',
        plugins: '../Scripts/durandal/plugins',
        transitions: '../Scripts/durandal/transitions',
        ace: '../Scripts/ace',
        moment: '../Scripts/moment',
        'd3': '../Scripts/d3',
        forge: '../Scripts/forge',
        jszip: '../Scripts/jszip'
    },
    // 0 disables the timeout completely, default is 7 seconds
    waitSeconds: 30
});

define('jquery', function() { return jQuery; });
define('knockout', ko);
define('nvd3', ['d3/d3', 'd3/nv', 'd3/models/timelines', 'd3/models/timelinesChart'], function (d3, nv, timelines, chart) { return nv; });
define('dagre', ['d3/d3', 'd3/dagre'], function (d3, dagre) { return dagre; });

// Do not remove the below comment, as it's used by the optimized build to inline Durandal scripts.
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The activator module encapsulates all logic related to screen/component activation.
 * An activator is essentially an asynchronous state machine that understands a particular state transition protocol.
 * The protocol ensures that the following series of events always occur: `canDeactivate` (previous state), `canActivate` (new state), `deactivate` (previous state), `activate` (new state).
 * Each of the _can_ callbacks may return a boolean, affirmative value or promise for one of those. If either of the _can_ functions yields a false result, then activation halts.
 * @module activator
 * @requires system
 * @requires knockout
 */
define("durandal/activator", ['durandal/system', 'knockout'], function (system, ko) {
    var activator;
    var defaultOptions = {
        canDeactivate:true
    };

    function ensureSettings(settings) {
        if (settings == undefined) {
            settings = {};
        }

        if (!system.isBoolean(settings.closeOnDeactivate)) {
            settings.closeOnDeactivate = activator.defaults.closeOnDeactivate;
        }

        if (!settings.beforeActivate) {
            settings.beforeActivate = activator.defaults.beforeActivate;
        }

        if (!settings.afterDeactivate) {
            settings.afterDeactivate = activator.defaults.afterDeactivate;
        }

        if(!settings.affirmations){
            settings.affirmations = activator.defaults.affirmations;
        }

        if (!settings.interpretResponse) {
            settings.interpretResponse = activator.defaults.interpretResponse;
        }

        if (!settings.areSameItem) {
            settings.areSameItem = activator.defaults.areSameItem;
        }

        if (!settings.findChildActivator) {
            settings.findChildActivator = activator.defaults.findChildActivator;
        }

        return settings;
    }

    function invoke(target, method, data) {
        if (system.isArray(data)) {
            return target[method].apply(target, data);
        }

        return target[method](data);
    }

    function deactivate(item, close, settings, dfd, setter) {
        if (item && item.deactivate) {
            system.log('Deactivating', item);

            var result;
            try {
                result = item.deactivate(close);
            } catch(error) {
                system.log('ERROR: ' + error.message, error);
                dfd.resolve(false);
                return;
            }

            if (result && result.then) {
                result.then(function() {
                    settings.afterDeactivate(item, close, setter);
                    dfd.resolve(true);
                }, function(reason) {
                    system.log(reason);
                    dfd.resolve(false);
                });
            } else {
                settings.afterDeactivate(item, close, setter);
                dfd.resolve(true);
            }
        } else {
            if (item) {
                settings.afterDeactivate(item, close, setter);
            }

            dfd.resolve(true);
        }
    }

    function activate(newItem, activeItem, callback, activationData) {
        var result;

        if(newItem && newItem.activate) {
            system.log('Activating', newItem);

            try {
                result = invoke(newItem, 'activate', activationData);
            } catch(error) {
                system.log('ERROR: ' + error.message, error);
                callback(false);
                return;
            }
        }

        if(result && result.then) {
            result.then(function() {
                activeItem(newItem);
                callback(true);
            }, function(reason) {
                system.log('ERROR: ' + reason.message, reason);
                callback(false);
            });
        } else {
            activeItem(newItem);
            callback(true);
        }
    }

    function canDeactivateItem(item, close, settings, options) {
        options = system.extend({}, defaultOptions, options);
        settings.lifecycleData = null;

        return system.defer(function (dfd) {
            function continueCanDeactivate() {
                if (item && item.canDeactivate && options.canDeactivate) {
                    var resultOrPromise;
                    try {
                        resultOrPromise = item.canDeactivate(close);
                    } catch (error) {
                        system.log('ERROR: ' + error.message, error);
                        dfd.resolve(false);
                        return;
                    }

                    if (resultOrPromise.then) {
                        resultOrPromise.then(function (result) {
                            settings.lifecycleData = result;
                            dfd.resolve(settings.interpretResponse(result));
                        }, function (reason) {
                            system.log('ERROR: ' + reason.message, reason);
                            dfd.resolve(false);
                        });
                    } else {
                        settings.lifecycleData = resultOrPromise;
                        dfd.resolve(settings.interpretResponse(resultOrPromise));
                    }
                } else {
                    dfd.resolve(true);
                }
            }

            var childActivator = settings.findChildActivator(item);
            if (childActivator) {
                childActivator.canDeactivate().then(function(result) {
                    if (result) {
                        continueCanDeactivate();
                    } else {
                        dfd.resolve(false);
                    }
                });
            } else {
                continueCanDeactivate();
            }
        }).promise();
    };

    function canActivateItem(newItem, activeItem, settings, activeData, newActivationData) {
        settings.lifecycleData = null;

        return system.defer(function (dfd) {
            if (settings.areSameItem(activeItem(), newItem, activeData, newActivationData)) {
                dfd.resolve(true);
                return;
            }

            if (newItem && newItem.canActivate) {
                var resultOrPromise;
                try {
                    resultOrPromise = invoke(newItem, 'canActivate', newActivationData);
                } catch (error) {
                    system.log('ERROR: ' + error.message, error);
                    dfd.resolve(false);
                    return;
                }

                if (resultOrPromise.then) {
                    resultOrPromise.then(function(result) {
                        settings.lifecycleData = result;
                        dfd.resolve(settings.interpretResponse(result));
                    }, function(reason) {
                        system.log('ERROR: ' + reason.message, reason);
                        dfd.resolve(false);
                    });
                } else {
                    settings.lifecycleData = resultOrPromise;
                    dfd.resolve(settings.interpretResponse(resultOrPromise));
                }
            } else {
                dfd.resolve(true);
            }
        }).promise();
    };

    /**
     * An activator is a read/write computed observable that enforces the activation lifecycle whenever changing values.
     * @class Activator
     */
    function createActivator(initialActiveItem, settings) {
        var activeItem = ko.observable(null);
        var activeData;

        settings = ensureSettings(settings);

        var computed = ko.computed({
            read: function () {
                return activeItem();
            },
            write: function (newValue) {
                computed.viaSetter = true;
                computed.activateItem(newValue);
            }
        });

        computed.__activator__ = true;

        /**
         * The settings for this activator.
         * @property {ActivatorSettings} settings
         */
        computed.settings = settings;
        settings.activator = computed;

        /**
         * An observable which indicates whether or not the activator is currently in the process of activating an instance.
         * @method isActivating
         * @return {boolean}
         */
        computed.isActivating = ko.observable(false);

        computed.forceActiveItem = function (item) {
            activeItem(item);
        };

        /**
         * Determines whether or not the specified item can be deactivated.
         * @method canDeactivateItem
         * @param {object} item The item to check.
         * @param {boolean} close Whether or not to check if close is possible.
         * @param {object} options Options for controlling the activation process.
         * @return {promise}
         */
        computed.canDeactivateItem = function (item, close, options) {
            return canDeactivateItem(item, close, settings, options);
        };

        /**
         * Deactivates the specified item.
         * @method deactivateItem
         * @param {object} item The item to deactivate.
         * @param {boolean} close Whether or not to close the item.
         * @return {promise}
         */
        computed.deactivateItem = function (item, close) {
            return system.defer(function(dfd) {
                computed.canDeactivateItem(item, close).then(function(canDeactivate) {
                    if (canDeactivate) {
                        deactivate(item, close, settings, dfd, activeItem);
                    } else {
                        computed.notifySubscribers();
                        dfd.resolve(false);
                    }
                });
            }).promise();
        };

        /**
         * Determines whether or not the specified item can be activated.
         * @method canActivateItem
         * @param {object} item The item to check.
         * @param {object} activationData Data associated with the activation.
         * @return {promise}
         */
        computed.canActivateItem = function (newItem, activationData) {
            return canActivateItem(newItem, activeItem, settings, activeData, activationData);
        };

        /**
         * Activates the specified item.
         * @method activateItem
         * @param {object} newItem The item to activate.
         * @param {object} newActivationData Data associated with the activation.
         * @param {object} options Options for controlling the activation process.
         * @return {promise}
         */
        computed.activateItem = function (newItem, newActivationData, options) {
            var viaSetter = computed.viaSetter;
            computed.viaSetter = false;

            return system.defer(function (dfd) {
                if (computed.isActivating()) {
                    dfd.resolve(false);
                    return;
                }

                computed.isActivating(true);

                var currentItem = activeItem();
                if (settings.areSameItem(currentItem, newItem, activeData, newActivationData)) {
                    computed.isActivating(false);
                    dfd.resolve(true);
                    return;
                }

                computed.canDeactivateItem(currentItem, settings.closeOnDeactivate, options).then(function (canDeactivate) {
                    if (canDeactivate) {
                        computed.canActivateItem(newItem, newActivationData).then(function (canActivate) {
                            if (canActivate) {
                                system.defer(function (dfd2) {
                                    deactivate(currentItem, settings.closeOnDeactivate, settings, dfd2);
                                }).promise().then(function () {
                                        newItem = settings.beforeActivate(newItem, newActivationData);
                                        activate(newItem, activeItem, function (result) {
                                            activeData = newActivationData;
                                            computed.isActivating(false);
                                            dfd.resolve(result);
                                        }, newActivationData);
                                    });
                            } else {
                                if (viaSetter) {
                                    computed.notifySubscribers();
                                }

                                computed.isActivating(false);
                                dfd.resolve(false);
                            }
                        });
                    } else {
                        if (viaSetter) {
                            computed.notifySubscribers();
                        }

                        computed.isActivating(false);
                        dfd.resolve(false);
                    }
                });
            }).promise();
        };

        /**
         * Determines whether or not the activator, in its current state, can be activated.
         * @method canActivate
         * @return {promise}
         */
        computed.canActivate = function () {
            var toCheck;

            if (initialActiveItem) {
                toCheck = initialActiveItem;
                initialActiveItem = false;
            } else {
                toCheck = computed();
            }

            return computed.canActivateItem(toCheck);
        };

        /**
         * Activates the activator, in its current state.
         * @method activate
         * @return {promise}
         */
        computed.activate = function () {
            var toActivate;

            if (initialActiveItem) {
                toActivate = initialActiveItem;
                initialActiveItem = false;
            } else {
                toActivate = computed();
            }

            return computed.activateItem(toActivate);
        };

        /**
         * Determines whether or not the activator, in its current state, can be deactivated.
         * @method canDeactivate
         * @return {promise}
         */
        computed.canDeactivate = function (close) {
            return computed.canDeactivateItem(computed(), close);
        };

        /**
         * Deactivates the activator, in its current state.
         * @method deactivate
         * @return {promise}
         */
        computed.deactivate = function (close) {
            return computed.deactivateItem(computed(), close);
        };

        computed.includeIn = function (includeIn) {
            includeIn.canActivate = function () {
                return computed.canActivate();
            };

            includeIn.activate = function () {
                return computed.activate();
            };

            includeIn.canDeactivate = function (close) {
                return computed.canDeactivate(close);
            };

            includeIn.deactivate = function (close) {
                return computed.deactivate(close);
            };
        };

        if (settings.includeIn) {
            computed.includeIn(settings.includeIn);
        } else if (initialActiveItem) {
            computed.activate();
        }

        computed.forItems = function (items) {
            settings.closeOnDeactivate = false;

            settings.determineNextItemToActivate = function (list, lastIndex) {
                var toRemoveAt = lastIndex - 1;

                if (toRemoveAt == -1 && list.length > 1) {
                    return list[1];
                }

                if (toRemoveAt > -1 && toRemoveAt < list.length - 1) {
                    return list[toRemoveAt];
                }

                return null;
            };

            settings.beforeActivate = function (newItem) {
                var currentItem = computed();

                if (!newItem) {
                    newItem = settings.determineNextItemToActivate(items, currentItem ? items.indexOf(currentItem) : 0);
                } else {
                    var index = items.indexOf(newItem);

                    if (index == -1) {
                        items.push(newItem);
                    } else {
                        newItem = items()[index];
                    }
                }

                return newItem;
            };

            settings.afterDeactivate = function (oldItem, close) {
                if (close) {
                    items.remove(oldItem);
                }
            };

            var originalCanDeactivate = computed.canDeactivate;
            computed.canDeactivate = function (close) {
                if (close) {
                    return system.defer(function (dfd) {
                        var list = items();
                        var results = [];

                        function finish() {
                            for (var j = 0; j < results.length; j++) {
                                if (!results[j]) {
                                    dfd.resolve(false);
                                    return;
                                }
                            }

                            dfd.resolve(true);
                        }

                        for (var i = 0; i < list.length; i++) {
                            computed.canDeactivateItem(list[i], close).then(function (result) {
                                results.push(result);
                                if (results.length == list.length) {
                                    finish();
                                }
                            });
                        }
                    }).promise();
                } else {
                    return originalCanDeactivate();
                }
            };

            var originalDeactivate = computed.deactivate;
            computed.deactivate = function (close) {
                if (close) {
                    return system.defer(function (dfd) {
                        var list = items();
                        var results = 0;
                        var listLength = list.length;

                        function doDeactivate(item) {
                            setTimeout(function () {
                                computed.deactivateItem(item, close).then(function () {
                                    results++;
                                    items.remove(item);
                                    if (results == listLength) {
                                        dfd.resolve();
                                    }
                                });
                            }, 1);
                        }

                        for (var i = 0; i < listLength; i++) {
                            doDeactivate(list[i]);
                        }
                    }).promise();
                } else {
                    return originalDeactivate();
                }
            };

            return computed;
        };

        return computed;
    }

    /**
     * @class ActivatorSettings
     * @static
     */
    var activatorSettings = {
        /**
         * The default value passed to an object's deactivate function as its close parameter.
         * @property {boolean} closeOnDeactivate
         * @default true
         */
        closeOnDeactivate: true,
        /**
         * Lower-cased words which represent a truthy value.
         * @property {string[]} affirmations
         * @default ['yes', 'ok', 'true']
         */
        affirmations: ['yes', 'ok', 'true'],
        /**
         * Interprets the response of a `canActivate` or `canDeactivate` call using the known affirmative values in the `affirmations` array.
         * @method interpretResponse
         * @param {object} value
         * @return {boolean}
         */
        interpretResponse: function(value) {
            if(system.isObject(value)) {
                value = value.can || false;
            }

            if(system.isString(value)) {
                return ko.utils.arrayIndexOf(this.affirmations, value.toLowerCase()) !== -1;
            }

            return value;
        },
        /**
         * Determines whether or not the current item and the new item are the same.
         * @method areSameItem
         * @param {object} currentItem
         * @param {object} newItem
         * @param {object} currentActivationData
         * @param {object} newActivationData
         * @return {boolean}
         */
        areSameItem: function(currentItem, newItem, currentActivationData, newActivationData) {
            return currentItem == newItem;
        },
        /**
         * Called immediately before the new item is activated.
         * @method beforeActivate
         * @param {object} newItem
         */
        beforeActivate: function(newItem) {
            return newItem;
        },
        /**
         * Called immediately after the old item is deactivated.
         * @method afterDeactivate
         * @param {object} oldItem The previous item.
         * @param {boolean} close Whether or not the previous item was closed.
         * @param {function} setter The activate item setter function.
         */
        afterDeactivate: function(oldItem, close, setter) {
            if(close && setter) {
                setter(null);
            }
        },
        findChildActivator: function(item){
            return null;
        }
    };

    /**
     * @class ActivatorModule
     * @static
     */
    activator = {
        /**
         * The default settings used by activators.
         * @property {ActivatorSettings} defaults
         */
        defaults: activatorSettings,
        /**
         * Creates a new activator.
         * @method create
         * @param {object} [initialActiveItem] The item which should be immediately activated upon creation of the ativator.
         * @param {ActivatorSettings} [settings] Per activator overrides of the default activator settings.
         * @return {Activator} The created activator.
         */
        create: createActivator,
        /**
         * Determines whether or not the provided object is an activator or not.
         * @method isActivator
         * @param {object} object Any object you wish to verify as an activator or not.
         * @return {boolean} True if the object is an activator; false otherwise.
         */
        isActivator:function(object){
            return object && object.__activator__;
        }
    };

    return activator;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The app module controls app startup, plugin loading/configuration and root visual display.
 * @module app
 * @requires system
 * @requires viewEngine
 * @requires composition
 * @requires events
 * @requires jquery
 */
define("durandal/app", ['durandal/system', 'durandal/viewEngine', 'durandal/composition', 'durandal/events', 'jquery'], function(system, viewEngine, composition, Events, $) {
    var app,
        allPluginIds = [],
        allPluginConfigs = [];

    function loadPlugins(){
        return system.defer(function(dfd){
            if(allPluginIds.length == 0){
                dfd.resolve();
                return;
            }

            system.acquire(allPluginIds).then(function(loaded){
                for(var i = 0; i < loaded.length; i++){
                    var currentModule = loaded[i];

                    if(currentModule.install){
                        var config = allPluginConfigs[i];
                        if(!system.isObject(config)){
                            config = {};
                        }

                        currentModule.install(config);
                        system.log('Plugin:Installed ' + allPluginIds[i]);
                    }else{
                        system.log('Plugin:Loaded ' + allPluginIds[i]);
                    }
                }

                dfd.resolve();
            }).fail(function(err){
                system.error('Failed to load plugin(s). Details: ' + err.message);
            });
        }).promise();
    }

    /**
     * @class AppModule
     * @static
     * @uses Events
     */
    app = {
        /**
         * The title of your application.
         * @property {string} title
         */
        title: 'Application',
        /**
         * Configures one or more plugins to be loaded and installed into the application.
         * @method configurePlugins
         * @param {object} config Keys are plugin names. Values can be truthy, to simply install the plugin, or a configuration object to pass to the plugin.
         * @param {string} [baseUrl] The base url to load the plugins from.
         */
        configurePlugins:function(config, baseUrl){
            var pluginIds = system.keys(config);
            baseUrl = baseUrl || 'plugins/';

            if(baseUrl.indexOf('/', baseUrl.length - 1) === -1){
                baseUrl += '/';
            }

            for(var i = 0; i < pluginIds.length; i++){
                var key = pluginIds[i];
                allPluginIds.push(baseUrl + key);
                allPluginConfigs.push(config[key]);
            }
        },
        /**
         * Starts the application.
         * @method start
         * @return {promise}
         */
        start: function() {
            system.log('Application:Starting');

            if (this.title) {
                document.title = this.title;
            }

            return system.defer(function (dfd) {
                $(function() {
                    loadPlugins().then(function(){
                        dfd.resolve();
                        system.log('Application:Started');
                    });
                });
            }).promise();
        },
        /**
         * Sets the root module/view for the application.
         * @method setRoot
         * @param {string} root The root view or module.
         * @param {string} [transition] The transition to use from the previous root (or splash screen) into the new root.
         * @param {string} [applicationHost] The application host element or id. By default the id 'applicationHost' will be used.
         */
        setRoot: function(root, transition, applicationHost) {
            var hostElement, settings = { activate:true, transition: transition };

            if (!applicationHost || system.isString(applicationHost)) {
                hostElement = document.getElementById(applicationHost || 'applicationHost');
            } else {
                hostElement = applicationHost;
            }

            if (system.isString(root)) {
                if (viewEngine.isViewUrl(root)) {
                    settings.view = root;
                } else {
                    settings.model = root;
                }
            } else {
                settings.model = root;
            }

            function finishComposition() {
                if(settings.model) {
                    if (settings.model.canActivate) {
                        try {
                            var result = settings.model.canActivate();
                            if (result && result.then) {
                                result.then(function (actualResult) {
                                    if (actualResult) {
                                        composition.compose(hostElement, settings);
                                    }
                                }).fail(function (err) {
                                    system.error(err);
                                });
                            } else if (result) {
                                composition.compose(hostElement, settings);
                            }
                        } catch (er) {
                            system.error(er);
                        }
                    } else {
                        composition.compose(hostElement, settings);
                    }
                } else {
                    composition.compose(hostElement, settings);
                }
            }

            if(system.isString(settings.model)) {
                system.acquire(settings.model).then(function(module) {
                    settings.model = system.resolveObject(module);
                    finishComposition();
                }).fail(function(err) {
                    system.error('Failed to load root module (' + settings.model + '). Details: ' + err.message);
                });
            } else {
                finishComposition();
            }
        }
    };

    Events.includeIn(app);

    return app;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The binder joins an object instance and a DOM element tree by applying databinding and/or invoking binding lifecycle callbacks (binding and bindingComplete).
 * @module binder
 * @requires system
 * @requires knockout
 */
define("durandal/binder", ['durandal/system', 'knockout'], function (system, ko) {
    var binder,
        insufficientInfoMessage = 'Insufficient Information to Bind',
        unexpectedViewMessage = 'Unexpected View Type',
        bindingInstructionKey = 'durandal-binding-instruction',
        koBindingContextKey = '__ko_bindingContext__';

    function normalizeBindingInstruction(result){
        if(result === undefined){
            return { applyBindings: true };
        }

        if(system.isBoolean(result)){
            return { applyBindings:result };
        }

        if(result.applyBindings === undefined){
            result.applyBindings = true;
        }

        return result;
    }

    function doBind(obj, view, bindingTarget, data){
        if (!view || !bindingTarget) {
            if (binder.throwOnErrors) {
                system.error(insufficientInfoMessage);
            } else {
                system.log(insufficientInfoMessage, view, data);
            }
            return;
        }

        if (!view.getAttribute) {
            if (binder.throwOnErrors) {
                system.error(unexpectedViewMessage);
            } else {
                system.log(unexpectedViewMessage, view, data);
            }
            return;
        }

        var viewName = view.getAttribute('data-view');

        try {
            var instruction;

            if (obj && obj.binding) {
                instruction = obj.binding(view);
            }

            instruction = normalizeBindingInstruction(instruction);
            binder.binding(data, view, instruction);

            if(instruction.applyBindings){
                system.log('Binding', viewName, data);
                ko.applyBindings(bindingTarget, view);
            }else if(obj){
                ko.utils.domData.set(view, koBindingContextKey, { $data:obj });
            }

            binder.bindingComplete(data, view, instruction);

            if (obj && obj.bindingComplete) {
                obj.bindingComplete(view);
            }

            ko.utils.domData.set(view, bindingInstructionKey, instruction);
            return instruction;
        } catch (e) {
            e.message = e.message + ';\nView: ' + viewName + ";\nModuleId: " + system.getModuleId(data);
            if (binder.throwOnErrors) {
                system.error(e);
            } else {
                system.log(e.message);
            }
        }
    }

    /**
     * @class BinderModule
     * @static
     */
    return binder = {
        /**
         * Called before every binding operation. Does nothing by default.
         * @method binding
         * @param {object} data The data that is about to be bound.
         * @param {DOMElement} view The view that is about to be bound.
         * @param {object} instruction The object that carries the binding instructions.
         */
        binding: system.noop,
        /**
         * Called after every binding operation. Does nothing by default.
         * @method bindingComplete
         * @param {object} data The data that has just been bound.
         * @param {DOMElement} view The view that has just been bound.
         * @param {object} instruction The object that carries the binding instructions.
         */
        bindingComplete: system.noop,
        /**
         * Indicates whether or not the binding system should throw errors or not.
         * @property {boolean} throwOnErrors
         * @default false The binding system will not throw errors by default. Instead it will log them.
         */
        throwOnErrors: false,
        /**
         * Gets the binding instruction that was associated with a view when it was bound.
         * @method getBindingInstruction
         * @param {DOMElement} view The view that was previously bound.
         * @return {object} The object that carries the binding instructions.
         */
        getBindingInstruction:function(view){
            return ko.utils.domData.get(view, bindingInstructionKey);
        },
        /**
         * Binds the view, preserving the existing binding context. Optionally, a new context can be created, parented to the previous context.
         * @method bindContext
         * @param {KnockoutBindingContext} bindingContext The current binding context.
         * @param {DOMElement} view The view to bind.
         * @param {object} [obj] The data to bind to, causing the creation of a child binding context if present.
         * @param {string} [dataAlias] An alias for $data if present.
         */
        bindContext: function(bindingContext, view, obj, dataAlias) {
            if (obj && bindingContext) {
                bindingContext = bindingContext.createChildContext(obj, typeof(dataAlias) === 'string' ? dataAlias : null);
            }

            return doBind(obj, view, bindingContext, obj || (bindingContext ? bindingContext.$data : null));
        },
        /**
         * Binds the view, preserving the existing binding context. Optionally, a new context can be created, parented to the previous context.
         * @method bind
         * @param {object} obj The data to bind to.
         * @param {DOMElement} view The view to bind.
         */
        bind: function(obj, view) {
            return doBind(obj, view, obj, obj);
        }
    };
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The composition module encapsulates all functionality related to visual composition.
 * @module composition
 * @requires system
 * @requires viewLocator
 * @requires binder
 * @requires viewEngine
 * @requires activator
 * @requires jquery
 * @requires knockout
 */
define("durandal/composition", ['durandal/system', 'durandal/viewLocator', 'durandal/binder', 'durandal/viewEngine', 'durandal/activator', 'jquery', 'knockout'], function (system, viewLocator, binder, viewEngine, activator, $, ko) {
    var dummyModel = {},
        activeViewAttributeName = 'data-active-view',
        composition,
        compositionCompleteCallbacks = [],
        compositionCount = 0,
        compositionDataKey = 'durandal-composition-data',
        partAttributeName = 'data-part',
        bindableSettings = ['model', 'view', 'transition', 'area', 'strategy', 'activationData', 'onError'],
        visibilityKey = "durandal-visibility-data",
        composeBindings = ['compose:'];
    
    function onError(context, error, element) {
        try {
            if (context.onError) {
                try {
                    context.onError(error, element);
                } catch (e) {
                    system.error(e);
                }
            } else {
                system.error(error);
            }
        } finally {
            endComposition(context, element, true);
        }
    }

    function getHostState(parent) {
        var elements = [];
        var state = {
            childElements: elements,
            activeView: null
        };

        var child = ko.virtualElements.firstChild(parent);

        while (child) {
            if (child.nodeType == 1) {
                elements.push(child);
                if (child.getAttribute(activeViewAttributeName)) {
                    state.activeView = child;
                }
            }

            child = ko.virtualElements.nextSibling(child);
        }

        if(!state.activeView){
            state.activeView = elements[0];
        }

        return state;
    }

    function endComposition(context, element, error) {
        compositionCount--;

        if(compositionCount === 0) {
            var callBacks = compositionCompleteCallbacks;
            compositionCompleteCallbacks = [];
            
            if (!error) {
                setTimeout(function () {
                    var i = callBacks.length;

                    while (i--) {
                        try {
                            callBacks[i]();
                        } catch (e) {
                            onError(context, e, element);
                        }
                    }
                }, 1);
            }
        }

        cleanUp(context);
    }

    function cleanUp(context){
        delete context.activeView;
        delete context.viewElements;
    }

    function tryActivate(context, successCallback, skipActivation, element) {
        if(skipActivation){
            successCallback();
        } else if (context.activate && context.model && context.model.activate) {
            var result;

            try{
                if(system.isArray(context.activationData)) {
                    result = context.model.activate.apply(context.model, context.activationData);
                } else {
                    result = context.model.activate(context.activationData);
                }

                if(result && result.then) {
                    result.then(successCallback, function(reason) {
                        onError(context, reason, element);
                        successCallback();
                    });
                } else if(result || result === undefined) {
                    successCallback();
                } else {
                    endComposition(context, element);
                }
            }
            catch(e){
                onError(context, e, element);
            }
        } else {
            successCallback();
        }
    }

    function triggerAttach(context, element) {
        var context = this;

        if (context.activeView) {
            context.activeView.removeAttribute(activeViewAttributeName);
        }

        if (context.child) {
            try{
                if (context.model && context.model.attached) {
                    if (context.composingNewView || context.alwaysTriggerAttach) {
                        context.model.attached(context.child, context.parent, context);
                    }
                }

                if (context.attached) {
                    context.attached(context.child, context.parent, context);
                }

                context.child.setAttribute(activeViewAttributeName, true);

                if (context.composingNewView && context.model && context.model.detached) {
                    ko.utils.domNodeDisposal.addDisposeCallback(context.child, function () {
                        try{
                            context.model.detached(context.child, context.parent, context);
                        }catch(e2){
                            onError(context, e2, element);
                        }
                    });
                }
            }catch(e){
                onError(context, e, element);
            }
        }

        context.triggerAttach = system.noop;
    }

    function shouldTransition(context) {
        if (system.isString(context.transition)) {
            if (context.activeView) {
                if (context.activeView == context.child) {
                    return false;
                }

                if (!context.child) {
                    return true;
                }

                if (context.skipTransitionOnSameViewId) {
                    var currentViewId = context.activeView.getAttribute('data-view');
                    var newViewId = context.child.getAttribute('data-view');
                    return currentViewId != newViewId;
                }
            }

            return true;
        }

        return false;
    }

    function cloneNodes(nodesArray) {
        for (var i = 0, j = nodesArray.length, newNodesArray = []; i < j; i++) {
            var clonedNode = nodesArray[i].cloneNode(true);
            newNodesArray.push(clonedNode);
        }
        return newNodesArray;
    }

    function replaceParts(context){
        var parts = cloneNodes(context.parts);
        var replacementParts = composition.getParts(parts);
        var standardParts = composition.getParts(context.child);

        for (var partId in replacementParts) {
            var toReplace = standardParts[partId];
            if (!toReplace) {
                toReplace = $('[data-part="' + partId + '"]', context.child).get(0);
                if (!toReplace) {
                    system.log('Could not find part to override: ' + partId);
                    continue;
                }
            }

            toReplace.parentNode.replaceChild(replacementParts[partId], toReplace);
        }
    }

    function removePreviousView(context){
        var children = ko.virtualElements.childNodes(context.parent), i, len;

        if(!system.isArray(children)){
            var arrayChildren = [];
            for(i = 0, len = children.length; i < len; i++){
                arrayChildren[i] = children[i];
            }
            children = arrayChildren;
        }

        for(i = 1,len = children.length; i < len; i++){
            ko.removeNode(children[i]);
        }
    }

    function hide(view) {
        ko.utils.domData.set(view, visibilityKey, view.style.display);
        view.style.display = 'none';
    }

    function show(view) {
        var displayStyle = ko.utils.domData.get(view, visibilityKey);
        view.style.display = displayStyle === 'none' ? 'block' : displayStyle;
    }

    function hasComposition(element){
        var dataBind = element.getAttribute('data-bind');
        if(!dataBind){
            return false;
        }

        for(var i = 0, length = composeBindings.length; i < length; i++){
            if(dataBind.indexOf(composeBindings[i]) > -1){
                return true;
            }
        }

        return false;
    }

    /**
     * @class CompositionTransaction
     * @static
     */
    var compositionTransaction = {
        /**
         * Registers a callback which will be invoked when the current composition transaction has completed. The transaction includes all parent and children compositions.
         * @method complete
         * @param {function} callback The callback to be invoked when composition is complete.
         */
        complete: function (callback) {
            compositionCompleteCallbacks.push(callback);
        }
    };

    /**
     * @class CompositionModule
     * @static
     */
    composition = {
        /**
         * An array of all the binding handler names (includeing :) that trigger a composition.
         * @property {string} composeBindings
         * @default ['compose:']
         */
        composeBindings:composeBindings,
        /**
         * Converts a transition name to its moduleId.
         * @method convertTransitionToModuleId
         * @param {string} name The name of the transtion.
         * @return {string} The moduleId.
         */
        convertTransitionToModuleId: function (name) {
            return 'transitions/' + name;
        },
        /**
         * The name of the transition to use in all compositions.
         * @property {string} defaultTransitionName
         * @default null
         */
        defaultTransitionName: null,
        /**
         * Represents the currently executing composition transaction.
         * @property {CompositionTransaction} current
         */
        current: compositionTransaction,
        /**
         * Registers a binding handler that will be invoked when the current composition transaction is complete.
         * @method addBindingHandler
         * @param {string} name The name of the binding handler.
         * @param {object} [config] The binding handler instance. If none is provided, the name will be used to look up an existing handler which will then be converted to a composition handler.
         * @param {function} [initOptionsFactory] If the registered binding needs to return options from its init call back to knockout, this function will server as a factory for those options. It will receive the same parameters that the init function does.
         */
        addBindingHandler:function(name, config, initOptionsFactory){
            var key,
                dataKey = 'composition-handler-' + name,
                handler;

            config = config || ko.bindingHandlers[name];
            initOptionsFactory = initOptionsFactory || function(){ return undefined;  };

            handler = ko.bindingHandlers[name] = {
                init: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    if(compositionCount > 0){
                        var data = {
                            trigger:ko.observable(null)
                        };

                        composition.current.complete(function(){
                            if(config.init){
                                config.init(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                            }

                            if(config.update){
                                ko.utils.domData.set(element, dataKey, config);
                                data.trigger('trigger');
                            }
                        });

                        ko.utils.domData.set(element, dataKey, data);
                    }else{
                        ko.utils.domData.set(element, dataKey, config);

                        if(config.init){
                            config.init(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                        }
                    }

                    return initOptionsFactory(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                },
                update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var data = ko.utils.domData.get(element, dataKey);

                    if(data.update){
                        return data.update(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                    }

                    if(data.trigger){
                        data.trigger();
                    }
                }
            };

            for (key in config) {
                if (key !== "init" && key !== "update") {
                    handler[key] = config[key];
                }
            }
        },
        /**
         * Gets an object keyed with all the elements that are replacable parts, found within the supplied elements. The key will be the part name and the value will be the element itself.
         * @method getParts
         * @param {DOMElement\DOMElement[]} elements The element(s) to search for parts.
         * @return {object} An object keyed by part.
         */
        getParts: function(elements, parts) {
            parts = parts || {};

            if (!elements) {
                return parts;
            }

            if (elements.length === undefined) {
                elements = [elements];
            }

            for (var i = 0, length = elements.length; i < length; i++) {
                var element = elements[i],
                    id;

                if (element.getAttribute) {
                    id = element.getAttribute(partAttributeName);
                    if (id) {
                        parts[id] = element;
                    }

                    if (element.hasChildNodes() && !hasComposition(element)) {
                        composition.getParts(element.childNodes, parts);
                    }
                }
            }

            return parts;
        },
        cloneNodes:cloneNodes,
        finalize: function (context, element) {
            if(context.transition === undefined) {
                context.transition = this.defaultTransitionName;
            }

            if(!context.child && !context.activeView){
                if (!context.cacheViews) {
                    ko.virtualElements.emptyNode(context.parent);
                }

                context.triggerAttach(context, element);
                endComposition(context, element);
            } else if (shouldTransition(context)) {
                var transitionModuleId = this.convertTransitionToModuleId(context.transition);

                system.acquire(transitionModuleId).then(function (transition) {
                    context.transition = transition;

                    transition(context).then(function () {
                        if (!context.cacheViews) {
                            if(!context.child){
                                ko.virtualElements.emptyNode(context.parent);
                            }else{
                                removePreviousView(context);
                            }
                        }else if(context.activeView){
                            var instruction = binder.getBindingInstruction(context.activeView);
                            if(instruction && instruction.cacheViews != undefined && !instruction.cacheViews){
                                ko.removeNode(context.activeView);
                            }else{
                                hide(context.activeView);
                            }
                        }

                        if (context.child) {
                            show(context.child);
                        }

                        context.triggerAttach(context, element);
                        endComposition(context, element);
                    });
                }).fail(function(err){
                    onError(context, 'Failed to load transition (' + transitionModuleId + '). Details: ' + err.message, element);
                });
            } else {
                if (context.child != context.activeView) {
                    if (context.cacheViews && context.activeView) {
                        var instruction = binder.getBindingInstruction(context.activeView);
                        if(!instruction || (instruction.cacheViews != undefined && !instruction.cacheViews)){
                            ko.removeNode(context.activeView);
                        }else{
                            hide(context.activeView);
                        }
                    }

                    if (!context.child) {
                        if (!context.cacheViews) {
                            ko.virtualElements.emptyNode(context.parent);
                        }
                    } else {
                        if (!context.cacheViews) {
                            removePreviousView(context);
                        }

                        show(context.child);
                    }
                }

                context.triggerAttach(context, element);
                endComposition(context, element);
            }
        },
        bindAndShow: function (child, element, context, skipActivation) {
            context.child = child;
            context.parent.__composition_context = context;

            if (context.cacheViews) {
                context.composingNewView = (ko.utils.arrayIndexOf(context.viewElements, child) == -1);
            } else {
                context.composingNewView = true;
            }

            tryActivate(context, function () {
                if (context.parent.__composition_context == context) {
                    delete context.parent.__composition_context;

                    if (context.binding) {
                        context.binding(context.child, context.parent, context);
                    }

                    if (context.preserveContext && context.bindingContext) {
                        if (context.composingNewView) {
                            if(context.parts){
                                replaceParts(context);
                            }

                            hide(child);
                            ko.virtualElements.prepend(context.parent, child);

                        binder.bindContext(context.bindingContext, child, context.model, context.as);
                        }
                    } else if (child) {
                        var modelToBind = context.model || dummyModel;
                        var currentModel = ko.dataFor(child);

                        if (currentModel != modelToBind) {
                            if (!context.composingNewView) {
                                ko.removeNode(child);
                                viewEngine.createView(child.getAttribute('data-view')).then(function(recreatedView) {
                                    composition.bindAndShow(recreatedView, element, context, true);
                                });
                                return;
                            }

                            if(context.parts){
                                replaceParts(context);
                            }

                            hide(child);
                            ko.virtualElements.prepend(context.parent, child);

                            binder.bind(modelToBind, child);
                        }
                    }

                    composition.finalize(context, element);
                } else {
                    endComposition(context, element);
                }
            }, skipActivation, element);
        },
        /**
         * Eecutes the default view location strategy.
         * @method defaultStrategy
         * @param {object} context The composition context containing the model and possibly existing viewElements.
         * @return {promise} A promise for the view.
         */
        defaultStrategy: function (context) {
            return viewLocator.locateViewForObject(context.model, context.area, context.viewElements);
        },
        getSettings: function (valueAccessor, element) {
            var value = valueAccessor(),
                settings = ko.utils.unwrapObservable(value) || {},
                activatorPresent = activator.isActivator(value),
                moduleId;

            if (system.isString(settings)) {
                if (viewEngine.isViewUrl(settings)) {
                    settings = {
                        view: settings
                    };
                } else {
                    settings = {
                        model: settings,
                        activate: !activatorPresent
                    };
                }

                return settings;
            }

            moduleId = system.getModuleId(settings);
            if (moduleId) {
                settings = {
                    model: settings,
                    activate: !activatorPresent
                };

                return settings;
            }

            if(!activatorPresent && settings.model) {
                activatorPresent = activator.isActivator(settings.model);
            }

            for (var attrName in settings) {
                if (ko.utils.arrayIndexOf(bindableSettings, attrName) != -1) {
                    settings[attrName] = ko.utils.unwrapObservable(settings[attrName]);
                } else {
                    settings[attrName] = settings[attrName];
                }
            }

            if (activatorPresent) {
                settings.activate = false;
            } else if (settings.activate === undefined) {
                settings.activate = true;
            }

            return settings;
        },
        executeStrategy: function (context, element) {
            context.strategy(context).then(function (child) {
                composition.bindAndShow(child, element, context);
            });
        },
        inject: function (context, element) {
            if (!context.model) {
                this.bindAndShow(null, element, context);
                return;
            }

            if (context.view) {
                viewLocator.locateView(context.view, context.area, context.viewElements).then(function (child) {
                    composition.bindAndShow(child, element, context);
                });
                return;
            }

            if (!context.strategy) {
                context.strategy = this.defaultStrategy;
            }

            if (system.isString(context.strategy)) {
                system.acquire(context.strategy).then(function (strategy) {
                    context.strategy = strategy;
                    composition.executeStrategy(context, element);
                }).fail(function (err) {
                    onError(context, 'Failed to load view strategy (' + context.strategy + '). Details: ' + err.message, element);
                });
            } else {
                this.executeStrategy(context, element);
            }
        },
        /**
         * Initiates a composition.
         * @method compose
         * @param {DOMElement} element The DOMElement or knockout virtual element that serves as the parent for the composition.
         * @param {object} settings The composition settings.
         * @param {object} [bindingContext] The current binding context.
         */
        compose: function (element, settings, bindingContext, fromBinding) {
            compositionCount++;

            if(!fromBinding){
                settings = composition.getSettings(function() { return settings; }, element);
            }

            if (settings.compositionComplete) {
                compositionCompleteCallbacks.push(function () {
                    settings.compositionComplete(settings.child, settings.parent, settings);
                });
            }

            compositionCompleteCallbacks.push(function () {
                if(settings.composingNewView && settings.model && settings.model.compositionComplete){
                    settings.model.compositionComplete(settings.child, settings.parent, settings);
                }
            });

            var hostState = getHostState(element);

            settings.activeView = hostState.activeView;
            settings.parent = element;
            settings.triggerAttach = triggerAttach;
            settings.bindingContext = bindingContext;

            if (settings.cacheViews && !settings.viewElements) {
                settings.viewElements = hostState.childElements;
            }

            if (!settings.model) {
                if (!settings.view) {
                    this.bindAndShow(null, element, settings);
                } else {
                    settings.area = settings.area || 'partial';
                    settings.preserveContext = true;

                    viewLocator.locateView(settings.view, settings.area, settings.viewElements).then(function (child) {
                        composition.bindAndShow(child, element, settings);
                    });
                }
            } else if (system.isString(settings.model)) {
                system.acquire(settings.model).then(function (module) {
                    settings.model = system.resolveObject(module);
                    composition.inject(settings, element);
                }).fail(function (err) {
                    onError(settings, 'Failed to load composed module (' + settings.model + '). Details: ' + err.message, element);
                });
            } else {
                composition.inject(settings, element);
            }
        }
    };

    ko.bindingHandlers.compose = {
        init: function() {
            return { controlsDescendantBindings: true };
        },
        update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
            var settings = composition.getSettings(valueAccessor, element);
            if(settings.mode){
                var data = ko.utils.domData.get(element, compositionDataKey);
                if(!data){
                    var childNodes = ko.virtualElements.childNodes(element);
                    data = {};

                    if(settings.mode === 'inline'){
                        data.view = viewEngine.ensureSingleElement(childNodes);
                    }else if(settings.mode === 'templated'){
                        data.parts = cloneNodes(childNodes);
                    }

                    ko.virtualElements.emptyNode(element);
                    ko.utils.domData.set(element, compositionDataKey, data);
                }

                if(settings.mode === 'inline'){
                    settings.view = data.view.cloneNode(true);
                }else if(settings.mode === 'templated'){
                    settings.parts = data.parts;
                }

                settings.preserveContext = true;
            }

            composition.compose(element, settings, bindingContext, true);
        }
    };

    ko.virtualElements.allowedBindings.compose = true;

    return composition;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Durandal events originate from backbone.js but also combine some ideas from signals.js as well as some additional improvements.
 * Events can be installed into any object and are installed into the `app` module by default for convenient app-wide eventing.
 * @module events
 * @requires system
 */
define("durandal/events", ['durandal/system'], function (system) {
    var eventSplitter = /\s+/;
    var Events = function() { };

    /**
     * Represents an event subscription.
     * @class Subscription
     */
    var Subscription = function(owner, events) {
        this.owner = owner;
        this.events = events;
    };

    /**
     * Attaches a callback to the event subscription.
     * @method then
     * @param {function} callback The callback function to invoke when the event is triggered.
     * @param {object} [context] An object to use as `this` when invoking the `callback`.
     * @chainable
     */
    Subscription.prototype.then = function (callback, context) {
        this.callback = callback || this.callback;
        this.context = context || this.context;
        
        if (!this.callback) {
            return this;
        }

        this.owner.on(this.events, this.callback, this.context);
        return this;
    };

    /**
     * Attaches a callback to the event subscription.
     * @method on
     * @param {function} [callback] The callback function to invoke when the event is triggered. If `callback` is not provided, the previous callback will be re-activated.
     * @param {object} [context] An object to use as `this` when invoking the `callback`.
     * @chainable
     */
    Subscription.prototype.on = Subscription.prototype.then;

    /**
     * Cancels the subscription.
     * @method off
     * @chainable
     */
    Subscription.prototype.off = function () {
        this.owner.off(this.events, this.callback, this.context);
        return this;
    };

    /**
     * Creates an object with eventing capabilities.
     * @class Events
     */

    /**
     * Creates a subscription or registers a callback for the specified event.
     * @method on
     * @param {string} events One or more events, separated by white space.
     * @param {function} [callback] The callback function to invoke when the event is triggered. If `callback` is not provided, a subscription instance is returned.
     * @param {object} [context] An object to use as `this` when invoking the `callback`.
     * @return {Subscription|Events} A subscription is returned if no callback is supplied, otherwise the events object is returned for chaining.
     */
    Events.prototype.on = function(events, callback, context) {
        var calls, event, list;

        if (!callback) {
            return new Subscription(this, events);
        } else {
            calls = this.callbacks || (this.callbacks = {});
            events = events.split(eventSplitter);

            while (event = events.shift()) {
                list = calls[event] || (calls[event] = []);
                list.push(callback, context);
            }

            return this;
        }
    };

    /**
     * Removes the callbacks for the specified events.
     * @method off
     * @param {string} [events] One or more events, separated by white space to turn off. If no events are specified, then the callbacks will be removed.
     * @param {function} [callback] The callback function to remove. If `callback` is not provided, all callbacks for the specified events will be removed.
     * @param {object} [context] The object that was used as `this`. Callbacks with this context will be removed.
     * @chainable
     */
    Events.prototype.off = function(events, callback, context) {
        var event, calls, list, i;

        // No events
        if (!(calls = this.callbacks)) {
            return this;
        }

        //removing all
        if (!(events || callback || context)) {
            delete this.callbacks;
            return this;
        }

        events = events ? events.split(eventSplitter) : system.keys(calls);

        // Loop through the callback list, splicing where appropriate.
        while (event = events.shift()) {
            if (!(list = calls[event]) || !(callback || context)) {
                delete calls[event];
                continue;
            }

            for (i = list.length - 2; i >= 0; i -= 2) {
                if (!(callback && list[i] !== callback || context && list[i + 1] !== context)) {
                    list.splice(i, 2);
                }
            }
        }

        return this;
    };

    /**
     * Triggers the specified events.
     * @method trigger
     * @param {string} [events] One or more events, separated by white space to trigger.
     * @chainable
     */
    Events.prototype.trigger = function(events) {
        var event, calls, list, i, length, args, all, rest;
        if (!(calls = this.callbacks)) {
            return this;
        }

        rest = [];
        events = events.split(eventSplitter);
        for (i = 1, length = arguments.length; i < length; i++) {
            rest[i - 1] = arguments[i];
        }

        // For each event, walk through the list of callbacks twice, first to
        // trigger the event, then to trigger any `"all"` callbacks.
        while (event = events.shift()) {
            // Copy callback lists to prevent modification.
            if (all = calls.all) {
                all = all.slice();
            }

            if (list = calls[event]) {
                list = list.slice();
            }

            // Execute event callbacks.
            if (list) {
                for (i = 0, length = list.length; i < length; i += 2) {
                    list[i].apply(list[i + 1] || this, rest);
                }
            }

            // Execute "all" callbacks.
            if (all) {
                args = [event].concat(rest);
                for (i = 0, length = all.length; i < length; i += 2) {
                    all[i].apply(all[i + 1] || this, args);
                }
            }
        }

        return this;
    };

    /**
     * Creates a function that will trigger the specified events when called. Simplifies proxying jQuery (or other) events through to the events object.
     * @method proxy
     * @param {string} events One or more events, separated by white space to trigger by invoking the returned function.
     * @return {function} Calling the function will invoke the previously specified events on the events object.
     */
    Events.prototype.proxy = function(events) {
        var that = this;
        return (function(arg) {
            that.trigger(events, arg);
        });
    };

    /**
     * Creates an object with eventing capabilities.
     * @class EventsModule
     * @static
     */

    /**
     * Adds eventing capabilities to the specified object.
     * @method includeIn
     * @param {object} targetObject The object to add eventing capabilities to.
     */
    Events.includeIn = function(targetObject) {
        targetObject.on = Events.prototype.on;
        targetObject.off = Events.prototype.off;
        targetObject.trigger = Events.prototype.trigger;
        targetObject.proxy = Events.prototype.proxy;
    };

    return Events;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The system module encapsulates the most basic features used by other modules.
 * @module system
 * @requires require
 * @requires jquery
 */
define("durandal/system", ['require', 'jquery'], function(require, $) {
    var isDebugging = false,
        nativeKeys = Object.keys,
        hasOwnProperty = Object.prototype.hasOwnProperty,
        toString = Object.prototype.toString,
        system,
        treatAsIE8 = false,
        nativeIsArray = Array.isArray,
        slice = Array.prototype.slice;

    //polyfill from https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/Trim
    if (!String.prototype.trim) {
        String.prototype.trim = function () {
            return this.replace(/^\s+|\s+$/g, '');
        };
    }

    //see http://patik.com/blog/complete-cross-browser-console-log/
    // Tell IE9 to use its built-in console
    if (Function.prototype.bind && (typeof console === 'object' || typeof console === 'function') && typeof console.log == 'object') {
        try {
            ['log', 'info', 'warn', 'error', 'assert', 'dir', 'clear', 'profile', 'profileEnd']
                .forEach(function(method) {
                    console[method] = this.call(console[method], console);
                }, Function.prototype.bind);
        } catch (ex) {
            treatAsIE8 = true;
        }
    }

    // callback for dojo's loader 
    // note: if you wish to use Durandal with dojo's AMD loader,
    // currently you must fork the dojo source with the following
    // dojo/dojo.js, line 1187, the last line of the finishExec() function: 
    //  (add) signal("moduleLoaded", [module.result, module.mid]);
    // an enhancement request has been submitted to dojo to make this
    // a permanent change. To view the status of this request, visit:
    // http://bugs.dojotoolkit.org/ticket/16727

    if (require.on) {
        require.on("moduleLoaded", function(module, mid) {
            system.setModuleId(module, mid);
        });
    }

    // callback for require.js loader
    if (typeof requirejs !== 'undefined') {
        requirejs.onResourceLoad = function(context, map, depArray) {
            system.setModuleId(context.defined[map.id], map.id);
        };
    }

    var noop = function() { };

    var log = function() {
        try {
            // Modern browsers
            if (typeof console != 'undefined' && typeof console.log == 'function') {
                // Opera 11
                if (window.opera) {
                    var i = 0;
                    while (i < arguments.length) {
                        console.log('Item ' + (i + 1) + ': ' + arguments[i]);
                        i++;
                    }
                }
                // All other modern browsers
                else if ((slice.call(arguments)).length == 1 && typeof slice.call(arguments)[0] == 'string') {
                    console.log((slice.call(arguments)).toString());
                } else {
                    console.log.apply(console, slice.call(arguments));
                }
            }
            // IE8
            else if ((!Function.prototype.bind || treatAsIE8) && typeof console != 'undefined' && typeof console.log == 'object') {
                Function.prototype.call.call(console.log, console, slice.call(arguments));
            }

            // IE7 and lower, and other old browsers
        } catch (ignore) { }
    };

    var logError = function(error, err) {
        var exception;
        
        if(error instanceof Error){
            exception = error;
        } else {
            exception = new Error(error);
        }
        
        exception.innerError = err;
        
        //Report the error as an error, not as a log
        try {
            // Modern browsers (it's only a single item, no need for argument splitting as in log() above)
            if (typeof console != 'undefined' && typeof console.error == 'function') {
                console.error(exception);
            }
            // IE8
            else if ((!Function.prototype.bind || treatAsIE8) && typeof console != 'undefined' && typeof console.error == 'object') {
                Function.prototype.call.call(console.error, console, exception);
            }
            // IE7 and lower, and other old browsers
        } catch (ignore) { }

        throw exception;
    };

    /**
     * @class SystemModule
     * @static
     */
    system = {
        /**
         * Durandal's version.
         * @property {string} version
         */
        version: "2.1.0",
        /**
         * A noop function.
         * @method noop
         */
        noop: noop,
        /**
         * Gets the module id for the specified object.
         * @method getModuleId
         * @param {object} obj The object whose module id you wish to determine.
         * @return {string} The module id.
         */
        getModuleId: function(obj) {
            if (!obj) {
                return null;
            }

            if (typeof obj == 'function' && obj.prototype) {
                return obj.prototype.__moduleId__;
            }

            if (typeof obj == 'string') {
                return null;
            }

            return obj.__moduleId__;
        },
        /**
         * Sets the module id for the specified object.
         * @method setModuleId
         * @param {object} obj The object whose module id you wish to set.
         * @param {string} id The id to set for the specified object.
         */
        setModuleId: function(obj, id) {
            if (!obj) {
                return;
            }

            if (typeof obj == 'function' && obj.prototype) {
                obj.prototype.__moduleId__ = id;
                return;
            }

            if (typeof obj == 'string') {
                return;
            }

            obj.__moduleId__ = id;
        },
        /**
         * Resolves the default object instance for a module. If the module is an object, the module is returned. If the module is a function, that function is called with `new` and it's result is returned.
         * @method resolveObject
         * @param {object} module The module to use to get/create the default object for.
         * @return {object} The default object for the module.
         */
        resolveObject: function(module) {
            if (system.isFunction(module)) {
                return new module();
            } else {
                return module;
            }
        },
        /**
         * Gets/Sets whether or not Durandal is in debug mode.
         * @method debug
         * @param {boolean} [enable] Turns on/off debugging.
         * @return {boolean} Whether or not Durandal is current debugging.
         */
        debug: function(enable) {
            if (arguments.length == 1) {
                isDebugging = enable;
                if (isDebugging) {
                    this.log = log;
                    this.error = logError;
                    this.log('Debug:Enabled');
                } else {
                    this.log('Debug:Disabled');
                    this.log = noop;
                    this.error = noop;
                }
            }

            return isDebugging;
        },
        /**
         * Logs data to the console. Pass any number of parameters to be logged. Log output is not processed if the framework is not running in debug mode.
         * @method log
         * @param {object} info* The objects to log.
         */
        log: noop,
        /**
         * Logs an error.
         * @method error
         * @param {string|Error} obj The error to report.
         */
        error: noop,
        /**
         * Asserts a condition by throwing an error if the condition fails.
         * @method assert
         * @param {boolean} condition The condition to check.
         * @param {string} message The message to report in the error if the condition check fails.
         */
        assert: function (condition, message) {
            if (!condition) {
                system.error(new Error(message || 'Assert:Failed'));
            }
        },
        /**
         * Creates a deferred object which can be used to create a promise. Optionally pass a function action to perform which will be passed an object used in resolving the promise.
         * @method defer
         * @param {function} [action] The action to defer. You will be passed the deferred object as a paramter.
         * @return {Deferred} The deferred object.
         */
        defer: function(action) {
            return $.Deferred(action);
        },
        /**
         * Creates a simple V4 UUID. This should not be used as a PK in your database. It can be used to generate internal, unique ids. For a more robust solution see [node-uuid](https://github.com/broofa/node-uuid).
         * @method guid
         * @return {string} The guid.
         */
        guid: function() {
            var d = new Date().getTime();
            return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = (d + Math.random() * 16) % 16 | 0;
                d = Math.floor(d/16);
                return (c == 'x' ? r : (r & 0x7 | 0x8)).toString(16);
            });
        },
        /**
         * Uses require.js to obtain a module. This function returns a promise which resolves with the module instance. You can pass more than one module id to this function or an array of ids. If more than one or an array is passed, then the promise will resolve with an array of module instances.
         * @method acquire
         * @param {string|string[]} moduleId The id(s) of the modules to load.
         * @return {Promise} A promise for the loaded module(s).
         */
        acquire: function() {
            var modules,
                first = arguments[0],
                arrayRequest = false;

            if(system.isArray(first)){
                modules = first;
                arrayRequest = true;
            }else{
                modules = slice.call(arguments, 0);
            }

            return this.defer(function(dfd) {
                require(modules, function() {
                    var args = arguments;
                    setTimeout(function() {
                        if(args.length > 1 || arrayRequest){
                            dfd.resolve(slice.call(args, 0));
                        }else{
                            dfd.resolve(args[0]);
                        }
                    }, 1);
                }, function(err){
                    dfd.reject(err);
                });
            }).promise();
        },
        /**
         * Extends the first object with the properties of the following objects.
         * @method extend
         * @param {object} obj The target object to extend.
         * @param {object} extension* Uses to extend the target object.
         */
        extend: function(obj) {
            var rest = slice.call(arguments, 1);

            for (var i = 0; i < rest.length; i++) {
                var source = rest[i];

                if (source) {
                    for (var prop in source) {
                        obj[prop] = source[prop];
                    }
                }
            }

            return obj;
        },
        /**
         * Uses a setTimeout to wait the specified milliseconds.
         * @method wait
         * @param {number} milliseconds The number of milliseconds to wait.
         * @return {Promise}
         */
        wait: function(milliseconds) {
            return system.defer(function(dfd) {
                setTimeout(dfd.resolve, milliseconds);
            }).promise();
        }
    };

    /**
     * Gets all the owned keys of the specified object.
     * @method keys
     * @param {object} object The object whose owned keys should be returned.
     * @return {string[]} The keys.
     */
    system.keys = nativeKeys || function(obj) {
        if (obj !== Object(obj)) {
            throw new TypeError('Invalid object');
        }

        var keys = [];

        for (var key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                keys[keys.length] = key;
            }
        }

        return keys;
    };

    /**
     * Determines if the specified object is an html element.
     * @method isElement
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */
    system.isElement = function(obj) {
        return !!(obj && obj.nodeType === 1);
    };

    /**
     * Determines if the specified object is an array.
     * @method isArray
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */
    system.isArray = nativeIsArray || function(obj) {
        return toString.call(obj) == '[object Array]';
    };

    /**
     * Determines if the specified object is...an object. ie. Not an array, string, etc.
     * @method isObject
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */
    system.isObject = function(obj) {
        return obj === Object(obj);
    };

    /**
     * Determines if the specified object is a boolean.
     * @method isBoolean
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */
    system.isBoolean = function(obj) {
        return typeof(obj) === "boolean";
    };

    /**
     * Determines if the specified object is a promise.
     * @method isPromise
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */
    system.isPromise = function(obj) {
        return obj && system.isFunction(obj.then);
    };

    /**
     * Determines if the specified object is a function arguments object.
     * @method isArguments
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    /**
     * Determines if the specified object is a function.
     * @method isFunction
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    /**
     * Determines if the specified object is a string.
     * @method isString
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    /**
     * Determines if the specified object is a number.
     * @method isNumber
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    /**
     * Determines if the specified object is a date.
     * @method isDate
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    /**
     * Determines if the specified object is a boolean.
     * @method isBoolean
     * @param {object} object The object to check.
     * @return {boolean} True if matches the type, false otherwise.
     */

    //isArguments, isFunction, isString, isNumber, isDate, isRegExp.
    var isChecks = ['Arguments', 'Function', 'String', 'Number', 'Date', 'RegExp'];

    function makeIsFunction(name) {
        var value = '[object ' + name + ']';
        system['is' + name] = function(obj) {
            return toString.call(obj) == value;
        };
    }

    for (var i = 0; i < isChecks.length; i++) {
        makeIsFunction(isChecks[i]);
    }

    return system;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The viewEngine module provides information to the viewLocator module which is used to locate the view's source file. The viewEngine also transforms a view id into a view instance.
 * @module viewEngine
 * @requires system
 * @requires jquery
 */
define("durandal/viewEngine", ['durandal/system', 'jquery'], function (system, $) {
    var parseMarkup;

    if ($.parseHTML) {
        parseMarkup = function (html) {
            return $.parseHTML(html);
        };
    } else {
        parseMarkup = function (html) {
            return $(html).get();
        };
    }

    /**
     * @class ViewEngineModule
     * @static
     */
    return {
        cache:{},
        /**
         * The file extension that view source files are expected to have.
         * @property {string} viewExtension
         * @default .html
         */
        viewExtension: '.html',
        /**
         * The name of the RequireJS loader plugin used by the viewLocator to obtain the view source. (Use requirejs to map the plugin's full path).
         * @property {string} viewPlugin
         * @default text
         */
        viewPlugin: 'text',
        /**
         * Parameters passed to the RequireJS loader plugin used by the viewLocator to obtain the view source.
         * @property {string} viewPluginParameters
         * @default The empty string by default.
         */
        viewPluginParameters: '',
        /**
         * Determines if the url is a url for a view, according to the view engine.
         * @method isViewUrl
         * @param {string} url The potential view url.
         * @return {boolean} True if the url is a view url, false otherwise.
         */
        isViewUrl: function (url) {
            return url.indexOf(this.viewExtension, url.length - this.viewExtension.length) !== -1;
        },
        /**
         * Converts a view url into a view id.
         * @method convertViewUrlToViewId
         * @param {string} url The url to convert.
         * @return {string} The view id.
         */
        convertViewUrlToViewId: function (url) {
            return url.substring(0, url.length - this.viewExtension.length);
        },
        /**
         * Converts a view id into a full RequireJS path.
         * @method convertViewIdToRequirePath
         * @param {string} viewId The view id to convert.
         * @return {string} The require path.
         */
        convertViewIdToRequirePath: function (viewId) {
            var plugin = this.viewPlugin ? this.viewPlugin + '!' : '';
            return plugin + viewId + this.viewExtension + this.viewPluginParameters;
        },
        /**
         * Parses the view engine recognized markup and returns DOM elements.
         * @method parseMarkup
         * @param {string} markup The markup to parse.
         * @return {DOMElement[]} The elements.
         */
        parseMarkup: parseMarkup,
        /**
         * Calls `parseMarkup` and then pipes the results through `ensureSingleElement`.
         * @method processMarkup
         * @param {string} markup The markup to process.
         * @return {DOMElement} The view.
         */
        processMarkup: function (markup) {
            var allElements = this.parseMarkup(markup);
            return this.ensureSingleElement(allElements);
        },
        /**
         * Converts an array of elements into a single element. White space and comments are removed. If a single element does not remain, then the elements are wrapped.
         * @method ensureSingleElement
         * @param {DOMElement[]} allElements The elements.
         * @return {DOMElement} A single element.
         */
        ensureSingleElement:function(allElements){
            if (allElements.length == 1) {
                return allElements[0];
            }

            var withoutCommentsOrEmptyText = [];

            for (var i = 0; i < allElements.length; i++) {
                var current = allElements[i];
                if (current.nodeType != 8) {
                    if (current.nodeType == 3) {
                        var result = /\S/.test(current.nodeValue);
                        if (!result) {
                            continue;
                        }
                    }

                    withoutCommentsOrEmptyText.push(current);
                }
            }

            if (withoutCommentsOrEmptyText.length > 1) {
                return $(withoutCommentsOrEmptyText).wrapAll('<div class="durandal-wrapper"></div>').parent().get(0);
            }

            return withoutCommentsOrEmptyText[0];
        },
        /**
         * Gets the view associated with the id from the cache of parsed views.
         * @method tryGetViewFromCache
         * @param {string} id The view id to lookup in the cache.
         * @return {DOMElement|null} The cached view or null if it's not in the cache.
         */
        tryGetViewFromCache:function(id) {
            return this.cache[id];
        },
        /**
         * Puts the view associated with the id into the cache of parsed views.
         * @method putViewInCache
         * @param {string} id The view id whose view should be cached.
         * @param {DOMElement} view The view to cache.
         */
        putViewInCache: function (id, view) {
            this.cache[id] = view;
        },
        /**
         * Creates the view associated with the view id.
         * @method createView
         * @param {string} viewId The view id whose view should be created.
         * @return {Promise} A promise of the view.
         */
        createView: function(viewId) {
            var that = this;
            var requirePath = this.convertViewIdToRequirePath(viewId);
            var existing = this.tryGetViewFromCache(requirePath);

            if (existing) {
                return system.defer(function(dfd) {
                    dfd.resolve(existing.cloneNode(true));
                }).promise();
            }

            return system.defer(function(dfd) {
                system.acquire(requirePath).then(function(markup) {
                    var element = that.processMarkup(markup);
                    element.setAttribute('data-view', viewId);
                    that.putViewInCache(requirePath, element);
                    dfd.resolve(element.cloneNode(true));
                }).fail(function(err) {
                    that.createFallbackView(viewId, requirePath, err).then(function(element) {
                        element.setAttribute('data-view', viewId);
                        that.cache[requirePath] = element;
                        dfd.resolve(element.cloneNode(true));
                    });
                });
            }).promise();
        },
        /**
         * Called when a view cannot be found to provide the opportunity to locate or generate a fallback view. Mainly used to ease development.
         * @method createFallbackView
         * @param {string} viewId The view id whose view should be created.
         * @param {string} requirePath The require path that was attempted.
         * @param {Error} requirePath The error that was returned from the attempt to locate the default view.
         * @return {Promise} A promise for the fallback view.
         */
        createFallbackView: function (viewId, requirePath, err) {
            var that = this,
                message = 'View Not Found. Searched for "' + viewId + '" via path "' + requirePath + '".';

            return system.defer(function(dfd) {
                dfd.resolve(that.processMarkup('<div class="durandal-view-404">' + message + '</div>'));
            }).promise();
        }
    };
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The viewLocator module collaborates with the viewEngine module to provide views (literally dom sub-trees) to other parts of the framework as needed. The primary consumer of the viewLocator is the composition module.
 * @module viewLocator
 * @requires system
 * @requires viewEngine
 */
define("durandal/viewLocator", ['durandal/system', 'durandal/viewEngine'], function (system, viewEngine) {
    function findInElements(nodes, url) {
        for (var i = 0; i < nodes.length; i++) {
            var current = nodes[i];
            var existingUrl = current.getAttribute('data-view');
            if (existingUrl == url) {
                return current;
            }
        }
    }
    
    function escape(str) {
        return (str + '').replace(/([\\\.\+\*\?\[\^\]\$\(\)\{\}\=\!\<\>\|\:])/g, "\\$1");
    }

    /**
     * @class ViewLocatorModule
     * @static
     */
    return {
        /**
         * Allows you to set up a convention for mapping module folders to view folders. It is a convenience method that customizes `convertModuleIdToViewId` and `translateViewIdToArea` under the covers.
         * @method useConvention
         * @param {string} [modulesPath] A string to match in the path and replace with the viewsPath. If not specified, the match is 'viewmodels'.
         * @param {string} [viewsPath] The replacement for the modulesPath. If not specified, the replacement is 'views'.
         * @param {string} [areasPath] Partial views are mapped to the "views" folder if not specified. Use this parameter to change their location.
         */
        useConvention: function(modulesPath, viewsPath, areasPath) {
            modulesPath = modulesPath || 'viewmodels';
            viewsPath = viewsPath || 'views';
            areasPath = areasPath || viewsPath;

            var reg = new RegExp(escape(modulesPath), 'gi');

            this.convertModuleIdToViewId = function (moduleId) {
                return moduleId.replace(reg, viewsPath);
            };

            this.translateViewIdToArea = function (viewId, area) {
                if (!area || area == 'partial') {
                    return areasPath + '/' + viewId;
                }
                
                return areasPath + '/' + area + '/' + viewId;
            };
        },
        /**
         * Maps an object instance to a view instance.
         * @method locateViewForObject
         * @param {object} obj The object to locate the view for.
         * @param {string} [area] The area to translate the view to.
         * @param {DOMElement[]} [elementsToSearch] An existing set of elements to search first.
         * @return {Promise} A promise of the view.
         */
        locateViewForObject: function(obj, area, elementsToSearch) {
            var view;

            if (obj.getView) {
                view = obj.getView();
                if (view) {
                    return this.locateView(view, area, elementsToSearch);
                }
            }

            if (obj.viewUrl) {
                return this.locateView(obj.viewUrl, area, elementsToSearch);
            }

            var id = system.getModuleId(obj);
            if (id) {
                return this.locateView(this.convertModuleIdToViewId(id), area, elementsToSearch);
            }

            return this.locateView(this.determineFallbackViewId(obj), area, elementsToSearch);
        },
        /**
         * Converts a module id into a view id. By default the ids are the same.
         * @method convertModuleIdToViewId
         * @param {string} moduleId The module id.
         * @return {string} The view id.
         */
        convertModuleIdToViewId: function(moduleId) {
            return moduleId;
        },
        /**
         * If no view id can be determined, this function is called to genreate one. By default it attempts to determine the object's type and use that.
         * @method determineFallbackViewId
         * @param {object} obj The object to determine the fallback id for.
         * @return {string} The view id.
         */
        determineFallbackViewId: function (obj) {
            var funcNameRegex = /function (.{1,})\(/;
            var results = (funcNameRegex).exec((obj).constructor.toString());
            var typeName = (results && results.length > 1) ? results[1] : "";
            typeName = typeName.trim();
            return 'views/' + typeName;
        },
        /**
         * Takes a view id and translates it into a particular area. By default, no translation occurs.
         * @method translateViewIdToArea
         * @param {string} viewId The view id.
         * @param {string} area The area to translate the view to.
         * @return {string} The translated view id.
         */
        translateViewIdToArea: function (viewId, area) {
            return viewId;
        },
        /**
         * Locates the specified view.
         * @method locateView
         * @param {string|DOMElement} viewOrUrlOrId A view, view url or view id to locate.
         * @param {string} [area] The area to translate the view to.
         * @param {DOMElement[]} [elementsToSearch] An existing set of elements to search first.
         * @return {Promise} A promise of the view.
         */
        locateView: function(viewOrUrlOrId, area, elementsToSearch) {
            if (typeof viewOrUrlOrId === 'string') {
                var viewId;

                if (viewEngine.isViewUrl(viewOrUrlOrId)) {
                    viewId = viewEngine.convertViewUrlToViewId(viewOrUrlOrId);
                } else {
                    viewId = viewOrUrlOrId;
                }

                if (area) {
                    viewId = this.translateViewIdToArea(viewId, area);
                }

                if (elementsToSearch) {
                    var existing = findInElements(elementsToSearch, viewId);
                    if (existing) {
                        return system.defer(function(dfd) {
                            dfd.resolve(existing);
                        }).promise();
                    }
                }

                return viewEngine.createView(viewId);
            }

            return system.defer(function(dfd) {
                dfd.resolve(viewOrUrlOrId);
            }).promise();
        }
    };
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The dialog module enables the display of message boxes, custom modal dialogs and other overlays or slide-out UI abstractions. Dialogs are constructed by the composition system which interacts with a user defined dialog context. The dialog module enforced the activator lifecycle.
 * @module dialog
 * @requires system
 * @requires app
 * @requires composition
 * @requires activator
 * @requires viewEngine
 * @requires jquery
 * @requires knockout
 */
define("plugins/dialog", ['durandal/system', 'durandal/app', 'durandal/composition', 'durandal/activator', 'durandal/viewEngine', 'jquery', 'knockout'], function (system, app, composition, activator, viewEngine, $, ko) {
    var contexts = {},
        dialogCount = ko.observable(0),
        dialog;

    /**
     * Models a message box's message, title and options.
     * @class MessageBox
     */
    var MessageBox = function (message, title, options, autoclose, settings) {
        this.message = message;
        this.title = title || MessageBox.defaultTitle;
        this.options = options || MessageBox.defaultOptions;
        this.autoclose = autoclose || false;
        this.settings = $.extend({}, MessageBox.defaultSettings, settings);
    };

    /**
     * Selects an option and closes the message box, returning the selected option through the dialog system's promise.
     * @method selectOption
     * @param {string} dialogResult The result to select.
     */
    MessageBox.prototype.selectOption = function (dialogResult) {
        dialog.close(this, dialogResult);
    };

    /**
     * Provides the view to the composition system.
     * @method getView
     * @return {DOMElement} The view of the message box.
     */
    MessageBox.prototype.getView = function () {
        return viewEngine.processMarkup(MessageBox.defaultViewMarkup);
    };

    /**
     * Configures a custom view to use when displaying message boxes.
     * @method setViewUrl
     * @param {string} viewUrl The view url relative to the base url which the view locator will use to find the message box's view.
     * @static
     */
    MessageBox.setViewUrl = function (viewUrl) {
        delete MessageBox.prototype.getView;
        MessageBox.prototype.viewUrl = viewUrl;
    };

    /**
     * The title to be used for the message box if one is not provided.
     * @property {string} defaultTitle
     * @default Application
     * @static
     */
    MessageBox.defaultTitle = app.title || 'Application';

    /**
     * The options to display in the message box if none are specified.
     * @property {string[]} defaultOptions
     * @default ['Ok']
     * @static
     */
    MessageBox.defaultOptions = ['Ok'];

    
    MessageBox.defaultSettings = { buttonClass: "btn btn-default", primaryButtonClass: "btn-primary autofocus", secondaryButtonClass: "", "class": "modal-content messageBox", style: null };

    /**
    * Sets the classes and styles used throughout the message box markup.
    * @method setDefaults
    * @param {object} settings A settings object containing the following optional properties: buttonClass, primaryButtonClass, secondaryButtonClass, class, style.
    */
    MessageBox.setDefaults = function (settings) {
        $.extend(MessageBox.defaultSettings, settings);
    };

    MessageBox.prototype.getButtonClass = function ($index) {
        var c = "";
        if (this.settings) {
            if (this.settings.buttonClass) {
                c = this.settings.buttonClass;
            }
            if ($index() === 0 && this.settings.primaryButtonClass) {
                if (c.length > 0) {
                    c += " ";
                }
                c += this.settings.primaryButtonClass;
            }
            if ($index() > 0 && this.settings.secondaryButtonClass) {
                if (c.length > 0) {
                    c += " ";
                }
                c += this.settings.secondaryButtonClass;
            }
        }
        return c;
    };

    MessageBox.prototype.getClass = function () {
        if (this.settings) {
            return this.settings["class"];
        }
        return "messageBox";
    };

    MessageBox.prototype.getStyle = function () {
        if (this.settings) {
            return this.settings.style;
        }
        return null;
    };

    MessageBox.prototype.getButtonText = function (stringOrObject) {
        var t = $.type(stringOrObject);
        if (t === "string") {
            return stringOrObject;
        }
        else if (t === "object") {
            if ($.type(stringOrObject.text) === "string") {
                return stringOrObject.text;
            } else {
                system.error('The object for a MessageBox button does not have a text property that is a string.');
                return null;
            }
        }
        system.error('Object for a MessageBox button is not a string or object but ' + t + '.');
        return null;
    };

    MessageBox.prototype.getButtonValue = function (stringOrObject) {
        var t = $.type(stringOrObject);
        if (t === "string") {
            return stringOrObject;
        }
        else if (t === "object") {
            if ($.type(stringOrObject.text) === "undefined") {
                system.error('The object for a MessageBox button does not have a value property defined.');
                return null;
            } else {
                return stringOrObject.value;
            }
        }
        system.error('Object for a MessageBox button is not a string or object but ' + t + '.');
        return null;
    };

    /**
     * The markup for the message box's view.
     * @property {string} defaultViewMarkup
     * @static
     */
    MessageBox.defaultViewMarkup = [
        '<div data-view="plugins/messageBox" data-bind="css: getClass(), style: getStyle()">',
            '<div class="modal-header">',
                '<h3 data-bind="html: title"></h3>',
            '</div>',
            '<div class="modal-body">',
                '<p class="message" data-bind="html: message"></p>',
            '</div>',
            '<div class="modal-footer">',
                '<!-- ko foreach: options -->',
                '<button data-bind="click: function () { $parent.selectOption($parent.getButtonValue($data)); }, text: $parent.getButtonText($data), css: $parent.getButtonClass($index)"></button>',
                '<!-- /ko -->',
                '<div style="clear:both;"></div>',
            '</div>',
        '</div>'
    ].join('\n');

    function ensureDialogInstance(objOrModuleId) {
        return system.defer(function (dfd) {
            if (system.isString(objOrModuleId)) {
                system.acquire(objOrModuleId).then(function (module) {
                    dfd.resolve(system.resolveObject(module));
                }).fail(function (err) {
                    system.error('Failed to load dialog module (' + objOrModuleId + '). Details: ' + err.message);
                });
            } else {
                dfd.resolve(objOrModuleId);
            }
        }).promise();
    }

    /**
     * @class DialogModule
     * @static
     */
    dialog = {
        /**
         * The constructor function used to create message boxes.
         * @property {MessageBox} MessageBox
         */
        MessageBox: MessageBox,
        /**
         * The css zIndex that the last dialog was displayed at.
         * @property {number} currentZIndex
         */
        currentZIndex: 1050,
        /**
         * Gets the next css zIndex at which a dialog should be displayed.
         * @method getNextZIndex
         * @return {number} The next usable zIndex.
         */
        getNextZIndex: function () {
            return ++this.currentZIndex;
        },
        /**
         * Determines whether or not there are any dialogs open.
         * @method isOpen
         * @return {boolean} True if a dialog is open. false otherwise.
         */
        isOpen: ko.computed(function() {
            return dialogCount() > 0;
        }),
        /**
         * Gets the dialog context by name or returns the default context if no name is specified.
         * @method getContext
         * @param {string} [name] The name of the context to retrieve.
         * @return {DialogContext} True context.
         */
        getContext: function (name) {
            return contexts[name || 'default'];
        },
        /**
         * Adds (or replaces) a dialog context.
         * @method addContext
         * @param {string} name The name of the context to add.
         * @param {DialogContext} dialogContext The context to add.
         */
        addContext: function (name, dialogContext) {
            dialogContext.name = name;
            contexts[name] = dialogContext;

            var helperName = 'show' + name.substr(0, 1).toUpperCase() + name.substr(1);
            this[helperName] = function (obj, activationData) {
                return this.show(obj, activationData, name);
            };
        },
        createCompositionSettings: function (obj, dialogContext) {
            var settings = {
                model: obj,
                activate: false,
                transition: false
            };

            if (dialogContext.binding) {
                settings.binding = dialogContext.binding;
            }

            if (dialogContext.attached) {
                settings.attached = dialogContext.attached;
            }

            if (dialogContext.compositionComplete) {
                settings.compositionComplete = dialogContext.compositionComplete;
            }

            return settings;
        },
        /**
         * Gets the dialog model that is associated with the specified object.
         * @method getDialog
         * @param {object} obj The object for whom to retrieve the dialog.
         * @return {Dialog} The dialog model.
         */
        getDialog: function (obj) {
            if (obj) {
                return obj.__dialog__;
            }

            return undefined;
        },
        /**
         * Closes the dialog associated with the specified object.
         * @method close
         * @param {object} obj The object whose dialog should be closed.
         * @param {object} results* The results to return back to the dialog caller after closing.
         */
        close: function (obj) {
            var theDialog = this.getDialog(obj);
            if (theDialog) {
                var rest = Array.prototype.slice.call(arguments, 1);
                theDialog.close.apply(theDialog, rest);
            }
        },
        /**
         * Shows a dialog.
         * @method show
         * @param {object|string} obj The object (or moduleId) to display as a dialog.
         * @param {object} [activationData] The data that should be passed to the object upon activation.
         * @param {string} [context] The name of the dialog context to use. Uses the default context if none is specified.
         * @return {Promise} A promise that resolves when the dialog is closed and returns any data passed at the time of closing.
         */
        show: function (obj, activationData, context) {
            var that = this;
            var dialogContext = contexts[context || 'default'];

            return system.defer(function (dfd) {
                ensureDialogInstance(obj).then(function (instance) {
                    var dialogActivator = activator.create();

                    dialogActivator.activateItem(instance, activationData).then(function (success) {
                        if (success) {
                            var theDialog = instance.__dialog__ = {
                                owner: instance,
                                context: dialogContext,
                                activator: dialogActivator,
                                close: function () {
                                    var args = arguments;
                                    dialogActivator.deactivateItem(instance, true).then(function (closeSuccess) {
                                        if (closeSuccess) {
                                            dialogCount(dialogCount() - 1);
                                            dialogContext.removeHost(theDialog);
                                            delete instance.__dialog__;

                                            if (args.length === 0) {
                                                dfd.resolve();
                                            } else if (args.length === 1) {
                                                dfd.resolve(args[0]);
                                            } else {
                                                dfd.resolve.apply(dfd, args);
                                            }
                                        }
                                    });
                                }
                            };

                            theDialog.settings = that.createCompositionSettings(instance, dialogContext);
                            dialogContext.addHost(theDialog);

                            dialogCount(dialogCount() + 1);
                            composition.compose(theDialog.host, theDialog.settings);
                        } else {
                            dfd.resolve(false);
                        }
                    });
                });
            }).promise();
        },
        /**
         * Shows a message box.
         * @method showMessage
         * @param {string} message The message to display in the dialog.
         * @param {string} [title] The title message.
         * @param {string[]} [options] The options to provide to the user.
         * @param {boolean} [autoclose] Automatically close the the message box when clicking outside?
         * @param {Object} [settings] Custom settings for this instance of the messsage box, used to change classes and styles.
         * @return {Promise} A promise that resolves when the message box is closed and returns the selected option.
         */
        showMessage: function (message, title, options, autoclose, settings) {
            if (system.isString(this.MessageBox)) {
                return dialog.show(this.MessageBox, [
                    message,
                    title || MessageBox.defaultTitle,
                    options || MessageBox.defaultOptions,
                    autoclose || false,
                    settings || {}
                ]);
            }

            return dialog.show(new this.MessageBox(message, title, options, autoclose, settings));
        },
        /**
         * Installs this module into Durandal; called by the framework. Adds `app.showDialog` and `app.showMessage` convenience methods.
         * @method install
         * @param {object} [config] Add a `messageBox` property to supply a custom message box constructor. Add a `messageBoxView` property to supply custom view markup for the built-in message box. You can also use messageBoxViewUrl to specify the view url.
         */
        install: function (config) {
            app.showDialog = function (obj, activationData, context) {
                return dialog.show(obj, activationData, context);
            };

            app.closeDialog = function () {
                return dialog.close.apply(dialog, arguments);
            };

            app.showMessage = function (message, title, options, autoclose, settings) {
                return dialog.showMessage(message, title, options, autoclose, settings);
            };

            if (config.messageBox) {
                dialog.MessageBox = config.messageBox;
            }

            if (config.messageBoxView) {
                dialog.MessageBox.prototype.getView = function () {
                    return viewEngine.processMarkup(config.messageBoxView);
                };
            }

            if (config.messageBoxViewUrl) {
                dialog.MessageBox.setViewUrl(config.messageBoxViewUrl);
            }
        }
    };

    /**
     * @class DialogContext
     */
    dialog.addContext('default', {
        blockoutOpacity: 0.2,
        removeDelay: 200,
        /**
         * In this function, you are expected to add a DOM element to the tree which will serve as the "host" for the modal's composed view. You must add a property called host to the modalWindow object which references the dom element. It is this host which is passed to the composition module.
         * @method addHost
         * @param {Dialog} theDialog The dialog model.
         */
        addHost: function (theDialog) {
            var body = $('body');
            var blockout = $('<div class="modalBlockout"></div>')
                .css({ 'z-index': dialog.getNextZIndex(), 'opacity': this.blockoutOpacity })
                .appendTo(body);

            var host = $('<div class="modalHost"></div>')
                .css({ 'z-index': dialog.getNextZIndex() })
                .appendTo(body);

            theDialog.host = host.get(0);
            theDialog.blockout = blockout.get(0);

            if (!dialog.isOpen()) {
                theDialog.oldBodyMarginRight = body.css("margin-right");
                theDialog.oldInlineMarginRight = body.get(0).style.marginRight;

                var html = $("html");
                var oldBodyOuterWidth = body.outerWidth(true);
                var oldScrollTop = html.scrollTop();
                $("html").css("overflow-y", "hidden");
                var newBodyOuterWidth = $("body").outerWidth(true);
                body.css("margin-right", (newBodyOuterWidth - oldBodyOuterWidth + parseInt(theDialog.oldBodyMarginRight, 10)) + "px");
                html.scrollTop(oldScrollTop); // necessary for Firefox
            }
        },
        /**
         * This function is expected to remove any DOM machinery associated with the specified dialog and do any other necessary cleanup.
         * @method removeHost
         * @param {Dialog} theDialog The dialog model.
         */
        removeHost: function (theDialog) {
            $(theDialog.host).css('opacity', 0);
            $(theDialog.blockout).css('opacity', 0);

            setTimeout(function () {
                ko.removeNode(theDialog.host);
                ko.removeNode(theDialog.blockout);
            }, this.removeDelay);

            if (!dialog.isOpen()) {
                var html = $("html");
                var oldScrollTop = html.scrollTop(); // necessary for Firefox.
                html.css("overflow-y", "").scrollTop(oldScrollTop);

                if (theDialog.oldInlineMarginRight) {
                    $("body").css("margin-right", theDialog.oldBodyMarginRight);
                } else {
                    $("body").css("margin-right", '');
                }
            }
        },
        attached: function (view) {
            //To prevent flickering in IE8, we set visibility to hidden first, and later restore it
            $(view).css("visibility", "hidden");
        },
        /**
         * This function is called after the modal is fully composed into the DOM, allowing your implementation to do any final modifications, such as positioning or animation. You can obtain the original dialog object by using `getDialog` on context.model.
         * @method compositionComplete
         * @param {DOMElement} child The dialog view.
         * @param {DOMElement} parent The parent view.
         * @param {object} context The composition context.
         */
        compositionComplete: function (child, parent, context) {
            var theDialog = dialog.getDialog(context.model);
            var $child = $(child);
            var loadables = $child.find("img").filter(function () {
                //Remove images with known width and height
                var $this = $(this);
                return !(this.style.width && this.style.height) && !($this.attr("width") && $this.attr("height"));
            });

            $child.data("predefinedWidth", $child.get(0).style.width);

            var setDialogPosition = function (childView, objDialog) {
                //Setting a short timeout is need in IE8, otherwise we could do this straight away
                setTimeout(function () {
                    var $childView = $(childView);

                    objDialog.context.reposition(childView);

                    $(objDialog.host).css('opacity', 1);
                    $childView.css("visibility", "visible");

                    $childView.find('.autofocus').first().focus();
                }, 1);
            };

            setDialogPosition(child, theDialog);
            loadables.load(function () {
                setDialogPosition(child, theDialog);
            });

            if ($child.hasClass('autoclose') || context.model.autoclose) {
                $(theDialog.blockout).click(function () {
                    theDialog.close();
                });
            }
        },
        /**
         * This function is called to reposition the model view.
         * @method reposition
         * @param {DOMElement} view The dialog view.
         */
        reposition: function (view) {
            var $view = $(view),
                $window = $(window);

            //We will clear and then set width for dialogs without width set 
            if (!$view.data("predefinedWidth")) {
                $view.css({ width: '' }); //Reset width
            }
            var width = $view.outerWidth(false),
                height = $view.outerHeight(false),
                windowHeight = $window.height() - 10, //leave at least 10 pixels free
                windowWidth = $window.width() - 10, //leave at least 10 pixels free
                constrainedHeight = Math.min(height, windowHeight),
                constrainedWidth = Math.min(width, windowWidth);

            $view.css({
                'margin-top': (-constrainedHeight / 2).toString() + 'px',
                'margin-left': (-constrainedWidth / 2).toString() + 'px'
            });

            if (height > windowHeight) {
                $view.css("overflow-y", "auto").outerHeight(windowHeight);
            } else {
                $view.css({
                    "overflow-y": "",
                    "height": ""
                });
            }

            if (width > windowWidth) {
                $view.css("overflow-x", "auto").outerWidth(windowWidth);
            } else {
                $view.css("overflow-x", "");

                if (!$view.data("predefinedWidth")) {
                    //Ensure the correct width after margin-left has been set
                    $view.outerWidth(constrainedWidth);
                } else {
                    $view.css("width", $view.data("predefinedWidth"));
                }
            }
        }
    });

    return dialog;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * This module is based on Backbone's core history support. It abstracts away the low level details of working with browser history and url changes in order to provide a solid foundation for a router.
 * @module history
 * @requires system
 * @requires jquery
 */
define("plugins/history", ['durandal/system', 'jquery'], function (system, $) {
    // Cached regex for stripping a leading hash/slash and trailing space.
    var routeStripper = /^[#\/]|\s+$/g;

    // Cached regex for stripping leading and trailing slashes.
    var rootStripper = /^\/+|\/+$/g;

    // Cached regex for detecting MSIE.
    var isExplorer = /msie [\w.]+/;

    // Cached regex for removing a trailing slash.
    var trailingSlash = /\/$/;

    // Update the hash location, either replacing the current entry, or adding
    // a new one to the browser history.
    function updateHash(location, fragment, replace) {
        if (replace) {
            var href = location.href.replace(/(javascript:|#).*$/, '');

            if (history.history.replaceState) {
                history.history.replaceState({}, document.title, href + '#' + fragment); // using history.replaceState instead of location.replace to work around chrom bug
            } else {
                location.replace(href + '#' + fragment);
            }
        } else {
            // Some browsers require that `hash` contains a leading #.
            location.hash = '#' + fragment;
        }
    };

    /**
     * @class HistoryModule
     * @static
     */
    var history = {
        /**
         * The setTimeout interval used when the browser does not support hash change events.
         * @property {string} interval
         * @default 50
         */
        interval: 50,
        /**
         * Indicates whether or not the history module is actively tracking history.
         * @property {string} active
         */
        active: false
    };
    
    // Ensure that `History` can be used outside of the browser.
    if (typeof window !== 'undefined') {
        history.location = window.location;
        history.history = window.history;
    }

    /**
     * Gets the true hash value. Cannot use location.hash directly due to a bug in Firefox where location.hash will always be decoded.
     * @method getHash
     * @param {string} [window] The optional window instance
     * @return {string} The hash.
     */
    history.getHash = function(window) {
        var match = (window || history).location.href.match(/#(.*)$/);
        return match ? match[1] : '';
    };
    
    /**
     * Get the cross-browser normalized URL fragment, either from the URL, the hash, or the override.
     * @method getFragment
     * @param {string} fragment The fragment.
     * @param {boolean} forcePushState Should we force push state?
     * @return {string} he fragment.
     */
    history.getFragment = function(fragment, forcePushState) {
        if (fragment == null) {
            if (history._hasPushState || !history._wantsHashChange || forcePushState) {
                fragment = history.location.pathname + history.location.search;
                var root = history.root.replace(trailingSlash, '');
                if (!fragment.indexOf(root)) {
                    fragment = fragment.substr(root.length);
                }
            } else {
                fragment = history.getHash();
            }
        }
        
        return fragment.replace(routeStripper, '');
    };

    /**
     * Activate the hash change handling, returning `true` if the current URL matches an existing route, and `false` otherwise.
     * @method activate
     * @param {HistoryOptions} options.
     * @return {boolean|undefined} Returns true/false from loading the url unless the silent option was selected.
     */
    history.activate = function(options) {
        if (history.active) {
            system.error("History has already been activated.");
        }

        history.active = true;

        // Figure out the initial configuration. Do we need an iframe?
        // Is pushState desired ... is it available?
        history.options = system.extend({}, { root: '/' }, history.options, options);
        history.root = history.options.root;
        history._wantsHashChange = history.options.hashChange !== false;
        history._wantsPushState = !!history.options.pushState;
        history._hasPushState = !!(history.options.pushState && history.history && history.history.pushState);

        var fragment = history.getFragment();
        var docMode = document.documentMode;
        var oldIE = (isExplorer.exec(navigator.userAgent.toLowerCase()) && (!docMode || docMode <= 7));

        // Normalize root to always include a leading and trailing slash.
        history.root = ('/' + history.root + '/').replace(rootStripper, '/');

        if (oldIE && history._wantsHashChange) {
            history.iframe = $('<iframe src="javascript:0" tabindex="-1" />').hide().appendTo('body')[0].contentWindow;
            history.navigate(fragment, false);
        }

        // Depending on whether we're using pushState or hashes, and whether
        // 'onhashchange' is supported, determine how we check the URL state.
        if (history._hasPushState) {
            $(window).on('popstate', history.checkUrl);
        } else if (history._wantsHashChange && ('onhashchange' in window) && !oldIE) {
            $(window).on('hashchange', history.checkUrl);
        } else if (history._wantsHashChange) {
            history._checkUrlInterval = setInterval(history.checkUrl, history.interval);
        }

        // Determine if we need to change the base url, for a pushState link
        // opened by a non-pushState browser.
        history.fragment = fragment;
        var loc = history.location;
        var atRoot = loc.pathname.replace(/[^\/]$/, '$&/') === history.root;

        // Transition from hashChange to pushState or vice versa if both are requested.
        if (history._wantsHashChange && history._wantsPushState) {
            // If we've started off with a route from a `pushState`-enabled
            // browser, but we're currently in a browser that doesn't support it...
            if (!history._hasPushState && !atRoot) {
                history.fragment = history.getFragment(null, true);
                history.location.replace(history.root + history.location.search + '#' + history.fragment);
                // Return immediately as browser will do redirect to new url
                return true;

            // Or if we've started out with a hash-based route, but we're currently
            // in a browser where it could be `pushState`-based instead...
            } else if (history._hasPushState && atRoot && loc.hash) {
                this.fragment = history.getHash().replace(routeStripper, '');
                this.history.replaceState({}, document.title, history.root + history.fragment + loc.search);
            }
        }

        if (!history.options.silent) {
            return history.loadUrl(options.startRoute);
        }
    };

    /**
     * Disable history, perhaps temporarily. Not useful in a real app, but possibly useful for unit testing Routers.
     * @method deactivate
     */
    history.deactivate = function() {
        $(window).off('popstate', history.checkUrl).off('hashchange', history.checkUrl);
        clearInterval(history._checkUrlInterval);
        history.active = false;
    };

    /**
     * Checks the current URL to see if it has changed, and if it has, calls `loadUrl`, normalizing across the hidden iframe.
     * @method checkUrl
     * @return {boolean} Returns true/false from loading the url.
     */
    history.checkUrl = function() {
        var current = history.getFragment();
        if (current === history.fragment && history.iframe) {
            current = history.getFragment(history.getHash(history.iframe));
        }

        if (current === history.fragment) {
            return false;
        }

        if (history.iframe) {
            history.navigate(current, false);
        }
        
        history.loadUrl();
    };
    
    /**
     * Attempts to load the current URL fragment. A pass-through to options.routeHandler.
     * @method loadUrl
     * @return {boolean} Returns true/false from the route handler.
     */
    history.loadUrl = function(fragmentOverride) {
        var fragment = history.fragment = history.getFragment(fragmentOverride);

        return history.options.routeHandler ?
            history.options.routeHandler(fragment) :
            false;
    };

    /**
     * Save a fragment into the hash history, or replace the URL state if the
     * 'replace' option is passed. You are responsible for properly URL-encoding
     * the fragment in advance.
     * The options object can contain `trigger: false` if you wish to not have the
     * route callback be fired, or `replace: true`, if
     * you wish to modify the current URL without adding an entry to the history.
     * @method navigate
     * @param {string} fragment The url fragment to navigate to.
     * @param {object|boolean} options An options object with optional trigger and replace flags. You can also pass a boolean directly to set the trigger option. Trigger is `true` by default.
     * @return {boolean} Returns true/false from loading the url.
     */
    history.navigate = function(fragment, options) {
        if (!history.active) {
            return false;
        }

        if(options === undefined) {
            options = {
                trigger: true
            };
        }else if(system.isBoolean(options)) {
            options = {
                trigger: options
            };
        }

        fragment = history.getFragment(fragment || '');

        if (history.fragment === fragment) {
            return;
        }

        history.fragment = fragment;

        var url = history.root + fragment;

        // Don't include a trailing slash on the root.
        if(fragment === '' && url !== '/') {
            url = url.slice(0, -1);
        }

        // If pushState is available, we use it to set the fragment as a real URL.
        if (history._hasPushState) {
            history.history[options.replace ? 'replaceState' : 'pushState']({}, document.title, url);

            // If hash changes haven't been explicitly disabled, update the hash
            // fragment to store history.
        } else if (history._wantsHashChange) {
            updateHash(history.location, fragment, options.replace);
            
            if (history.iframe && (fragment !== history.getFragment(history.getHash(history.iframe)))) {
                // Opening and closing the iframe tricks IE7 and earlier to push a
                // history entry on hash-tag change.  When replace is true, we don't
                // want history.
                if (!options.replace) {
                    history.iframe.document.open().close();
                }
                
                updateHash(history.iframe.location, fragment, options.replace);
            }

            // If you've told us that you explicitly don't want fallback hashchange-
            // based history, then `navigate` becomes a page refresh.
        } else {
            return history.location.assign(url);
        }

        if (options.trigger) {
            return history.loadUrl(fragment);
        }
    };

    /**
     * Navigates back in the browser history.
     * @method navigateBack
     */
    history.navigateBack = function() {
        history.history.back();
    };

    /**
     * @class HistoryOptions
     * @static
     */

    /**
     * The function that will be called back when the fragment changes.
     * @property {function} routeHandler
     */

    /**
     * The url root used to extract the fragment when using push state.
     * @property {string} root
     */

    /**
     * Use hash change when present.
     * @property {boolean} hashChange
     * @default true
     */

    /**
     * Use push state when present.
     * @property {boolean} pushState
     * @default false
     */

    /**
     * Prevents loading of the current url when activating history.
     * @property {boolean} silent
     * @default false
     */

    return history;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Enables common http request scenarios.
 * @module http
 * @requires jquery
 * @requires knockout
 */
define("plugins/http", ['jquery', 'knockout'], function ($, ko) {
    /**
     * @class HTTPModule
     * @static
     */
    return {
        /**
         * The name of the callback parameter to inject into jsonp requests by default.
         * @property {string} callbackParam
         * @default callback
         */
        callbackParam: 'callback',
        /**
         * Converts the data to JSON.
         * @method toJSON
         * @param {object} data The data to convert to JSON.
         * @return {string} JSON.
         */
        toJSON: function(data) {
            return ko.toJSON(data);
        },
        /**
         * Makes an HTTP GET request.
         * @method get
         * @param {string} url The url to send the get request to.
         * @param {object} [query] An optional key/value object to transform into query string parameters.
         * @param {object} [headers] The data to add to the request header.  It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @return {Promise} A promise of the get response data.
         */
        get: function (url, query, headers) {
            return $.ajax(url, { data: query, headers: ko.toJS(headers) });
        },
        /**
         * Makes an JSONP request.
         * @method jsonp
         * @param {string} url The url to send the get request to.
         * @param {object} [query] An optional key/value object to transform into query string parameters.
         * @param {string} [callbackParam] The name of the callback parameter the api expects (overrides the default callbackParam).
         * @param {object} [headers] The data to add to the request header.  It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @return {Promise} A promise of the response data.
         */
        jsonp: function (url, query, callbackParam, headers) {
            if (url.indexOf('=?') == -1) {
                callbackParam = callbackParam || this.callbackParam;

                if (url.indexOf('?') == -1) {
                    url += '?';
                } else {
                    url += '&';
                }

                url += callbackParam + '=?';
            }

            return $.ajax({
                url: url,
                dataType: 'jsonp',
                data: query,
                headers: ko.toJS(headers)
            });
        },
        /**
         * Makes an HTTP PUT request.
         * @method put
         * @param {string} url The url to send the put request to.
         * @param {object} data The data to put. It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @param {object} [headers] The data to add to the request header.  It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @return {Promise} A promise of the response data.
         */
        put:function(url, data, headers) {
            return $.ajax({
                url: url,
                data: this.toJSON(data),
                type: 'PUT',
                contentType: 'application/json',
                dataType: 'json',
                headers: ko.toJS(headers)
            });
        },
        /**
         * Makes an HTTP POST request.
         * @method post
         * @param {string} url The url to send the post request to.
         * @param {object} data The data to post. It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @param {object} [headers] The data to add to the request header.  It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @return {Promise} A promise of the response data.
         */
        post: function (url, data, headers) {
            return $.ajax({
                url: url,
                data: this.toJSON(data),
                type: 'POST',
                contentType: 'application/json',
                dataType: 'json',
                headers: ko.toJS(headers)
            });
        },
        /**
         * Makes an HTTP DELETE request.
         * @method remove
         * @param {string} url The url to send the delete request to.
         * @param {object} [query] An optional key/value object to transform into query string parameters.
         * @param {object} [headers] The data to add to the request header.  It will be converted to JSON. If the data contains Knockout observables, they will be converted into normal properties before serialization.
         * @return {Promise} A promise of the get response data.
         */
        remove:function(url, query, headers) {
            return $.ajax({
                url: url,
                data: query,
                type: 'DELETE',
                headers: ko.toJS(headers)
            });
        }
    };
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Enables automatic observability of plain javascript object for ES5 compatible browsers. Also, converts promise properties into observables that are updated when the promise resolves.
 * @module observable
 * @requires system
 * @requires binder
 * @requires knockout
 */
define("plugins/observable", ['durandal/system', 'durandal/binder', 'knockout'], function(system, binder, ko) {
    var observableModule,
        toString = Object.prototype.toString,
        nonObservableTypes = ['[object Function]', '[object String]', '[object Boolean]', '[object Number]', '[object Date]', '[object RegExp]'],
        observableArrayMethods = ['remove', 'removeAll', 'destroy', 'destroyAll', 'replace'],
        arrayMethods = ['pop', 'reverse', 'sort', 'shift', 'slice'],
        additiveArrayFunctions = ['push', 'unshift'],
        es5Functions = ['filter', 'map', 'reduce', 'reduceRight', 'forEach', 'every', 'some'],
        arrayProto = Array.prototype,
        observableArrayFunctions = ko.observableArray.fn,
        logConversion = false,
        changeDetectionMethod = undefined,
        skipPromises = false,
        shouldIgnorePropertyName;

    /**
     * You can call observable(obj, propertyName) to get the observable function for the specified property on the object.
     * @class ObservableModule
     */

    if (!('getPropertyDescriptor' in Object)) {
        var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;
        var getPrototypeOf = Object.getPrototypeOf;

        Object['getPropertyDescriptor'] = function(o, name) {
            var proto = o, descriptor;

            while(proto && !(descriptor = getOwnPropertyDescriptor(proto, name))) {
                proto = getPrototypeOf(proto);
            }

            return descriptor;
        };
    }

    function defaultShouldIgnorePropertyName(propertyName){
        var first = propertyName[0];
        return first === '_' || first === '$' || (changeDetectionMethod && propertyName === changeDetectionMethod);
    }

    function isNode(obj) {
        return !!(obj && obj.nodeType !== undefined && system.isNumber(obj.nodeType));
    }

    function canConvertType(value) {
        if (!value || isNode(value) || value.ko === ko || value.jquery) {
            return false;
        }

        var type = toString.call(value);

        return nonObservableTypes.indexOf(type) == -1 && !(value === true || value === false);
    }

    function createLookup(obj) {
        var value = {};

        Object.defineProperty(obj, "__observable__", {
            enumerable: false,
            configurable: false,
            writable: false,
            value: value
        });

        return value;
    }

    function makeObservableArray(original, observable, hasChanged) {
        var lookup = original.__observable__, notify = true;

        if(lookup && lookup.__full__){
            return;
        }

        lookup = lookup || createLookup(original);
        lookup.__full__ = true;

        es5Functions.forEach(function (methodName) {
            observable[methodName] = function () {
                return arrayProto[methodName].apply(original, arguments);
            };
        });

        observableArrayMethods.forEach(function(methodName) {
            original[methodName] = function() {
                notify = false;
                var methodCallResult = observableArrayFunctions[methodName].apply(observable, arguments);
                notify = true;
                return methodCallResult;
            };
        });

        arrayMethods.forEach(function(methodName) {
            original[methodName] = function() {
                if(notify){
                    observable.valueWillMutate();
                }

                var methodCallResult = arrayProto[methodName].apply(original, arguments);

                if(notify){
                    observable.valueHasMutated();
                }

                return methodCallResult;
            };
        });

        additiveArrayFunctions.forEach(function(methodName){
            original[methodName] = function() {
                for (var i = 0, len = arguments.length; i < len; i++) {
                    convertObject(arguments[i], hasChanged);
                }

                if(notify){
                    observable.valueWillMutate();
                }

                var methodCallResult = arrayProto[methodName].apply(original, arguments);

                if(notify){
                    observable.valueHasMutated();
                }

                return methodCallResult;
            };
        });

        original['splice'] = function() {
            for (var i = 2, len = arguments.length; i < len; i++) {
                convertObject(arguments[i], hasChanged);
            }

            if(notify){
                observable.valueWillMutate();
            }

            var methodCallResult = arrayProto['splice'].apply(original, arguments);

            if(notify){
                observable.valueHasMutated();
            }

            return methodCallResult;
        };

        for (var i = 0, len = original.length; i < len; i++) {
            convertObject(original[i], hasChanged);
        }
    }

    /**
     * Converts an entire object into an observable object by re-writing its attributes using ES5 getters and setters. Attributes beginning with '_' or '$' are ignored.
     * @method convertObject
     * @param {object} obj The target object to convert.
     */
    function convertObject(obj, hasChanged) {
        var lookup, value;

        if (changeDetectionMethod) {
            if(obj && obj[changeDetectionMethod]) {
                if (hasChanged) {
                    hasChanged = hasChanged.slice(0);
                } else {
                    hasChanged = [];
                }
                hasChanged.push(obj[changeDetectionMethod]);
            }
        }

        if(!canConvertType(obj)){
            return;
        }

        lookup = obj.__observable__;

        if(lookup && lookup.__full__){
            return;
        }

        lookup = lookup || createLookup(obj);
        lookup.__full__ = true;

        if (system.isArray(obj)) {
            var observable = ko.observableArray(obj);
            makeObservableArray(obj, observable, hasChanged);
        } else {
            for (var propertyName in obj) {
                if(shouldIgnorePropertyName(propertyName)){
                    continue;
                }

                if (!lookup[propertyName]) {
                    var descriptor = Object.getPropertyDescriptor(obj, propertyName);
                    if (descriptor && (descriptor.get || descriptor.set)) {
                        defineProperty(obj, propertyName, {
                            get:descriptor.get,
                            set:descriptor.set
                        });
                    } else {
                        value = obj[propertyName];

                        if(!system.isFunction(value)) {
                            convertProperty(obj, propertyName, value, hasChanged);
                        }
                    }
                }
            }
        }

        if(logConversion) {
            system.log('Converted', obj);
        }
    }

    function innerSetter(observable, newValue, isArray) {
        //if this was originally an observableArray, then always check to see if we need to add/replace the array methods (if newValue was an entirely new array)
        if (isArray) {
            if (!newValue) {
                //don't allow null, force to an empty array
                newValue = [];
                makeObservableArray(newValue, observable);
            }
            else if (!newValue.destroyAll) {
                makeObservableArray(newValue, observable);
            }
        } else {
            convertObject(newValue);
        }

        //call the update to the observable after the array as been updated.
        observable(newValue);
    }

    /**
     * Converts a normal property into an observable property using ES5 getters and setters.
     * @method convertProperty
     * @param {object} obj The target object on which the property to convert lives.
     * @param {string} propertyName The name of the property to convert.
     * @param {object} [original] The original value of the property. If not specified, it will be retrieved from the object.
     * @return {KnockoutObservable} The underlying observable.
     */
    function convertProperty(obj, propertyName, original, hasChanged) {
        var observable,
            isArray,
            lookup = obj.__observable__ || createLookup(obj);

        if(original === undefined){
            original = obj[propertyName];
        }

        if (system.isArray(original)) {
            observable = ko.observableArray(original);
            makeObservableArray(original, observable, hasChanged);
            isArray = true;
        } else if (typeof original == "function") {
            if(ko.isObservable(original)){
                observable = original;
            }else{
                return null;
            }
        } else if(!skipPromises && system.isPromise(original)) {
            observable = ko.observable();

            original.then(function (result) {
                if(system.isArray(result)) {
                    var oa = ko.observableArray(result);
                    makeObservableArray(result, oa, hasChanged);
                    result = oa;
                }

                observable(result);
            });
        } else {
            observable = ko.observable(original);
            convertObject(original, hasChanged);
        }

        if (hasChanged && hasChanged.length > 0) {
            hasChanged.forEach(function (func) {
                if (system.isArray(original)) {
                    observable.subscribe(function (arrayChanges) {
                        func(obj, propertyName, null, arrayChanges);
                    }, null, "arrayChange");
                } else {
                    observable.subscribe(function (newValue) {
                        func(obj, propertyName, newValue, null);
                    });
                }
            });
        }

        Object.defineProperty(obj, propertyName, {
            configurable: true,
            enumerable: true,
            get: observable,
            set: ko.isWriteableObservable(observable) ? (function (newValue) {
                if (newValue && system.isPromise(newValue) && !skipPromises) {
                    newValue.then(function (result) {
                        innerSetter(observable, result, system.isArray(result));
                    });
                } else {
                    innerSetter(observable, newValue, isArray);
                }
            }) : undefined
        });

        lookup[propertyName] = observable;
        return observable;
    }

    /**
     * Defines a computed property using ES5 getters and setters.
     * @method defineProperty
     * @param {object} obj The target object on which to create the property.
     * @param {string} propertyName The name of the property to define.
     * @param {function|object} evaluatorOrOptions The Knockout computed function or computed options object.
     * @return {KnockoutObservable} The underlying computed observable.
     */
    function defineProperty(obj, propertyName, evaluatorOrOptions) {
        var computedOptions = { owner: obj, deferEvaluation: true },
            computed;

        if (typeof evaluatorOrOptions === 'function') {
            computedOptions.read = evaluatorOrOptions;
        } else {
            if ('value' in evaluatorOrOptions) {
                system.error('For defineProperty, you must not specify a "value" for the property. You must provide a "get" function.');
            }

            if (typeof evaluatorOrOptions.get !== 'function' && typeof evaluatorOrOptions.read !== 'function') {
                system.error('For defineProperty, the third parameter must be either an evaluator function, or an options object containing a function called "get".');
            }

            computedOptions.read = evaluatorOrOptions.get || evaluatorOrOptions.read;
            computedOptions.write = evaluatorOrOptions.set || evaluatorOrOptions.write;
        }

        computed = ko.computed(computedOptions);
        obj[propertyName] = computed;

        return convertProperty(obj, propertyName, computed);
    }

    observableModule = function(obj, propertyName){
        var lookup, observable, value;

        if (!obj) {
            return null;
        }

        lookup = obj.__observable__;
        if(lookup){
            observable = lookup[propertyName];
            if(observable){
                return observable;
            }
        }

        value = obj[propertyName];

        if(ko.isObservable(value)){
            return value;
        }

        return convertProperty(obj, propertyName, value);
    };

    observableModule.defineProperty = defineProperty;
    observableModule.convertProperty = convertProperty;
    observableModule.convertObject = convertObject;

    /**
     * Installs the plugin into the view model binder's `beforeBind` hook so that objects are automatically converted before being bound.
     * @method install
     */
    observableModule.install = function(options) {
        var original = binder.binding;

        binder.binding = function(obj, view, instruction) {
            if(instruction.applyBindings && !instruction.skipConversion){
                convertObject(obj);
            }

            original(obj, view);
        };

        logConversion = options.logConversion;
        if (options.changeDetection) {
            changeDetectionMethod = options.changeDetection;
        }

        skipPromises = options.skipPromises;
        shouldIgnorePropertyName = options.shouldIgnorePropertyName || defaultShouldIgnorePropertyName;
    };

    return observableModule;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Connects the history module's url and history tracking support to Durandal's activation and composition engine allowing you to easily build navigation-style applications.
 * @module router
 * @requires system
 * @requires app
 * @requires activator
 * @requires events
 * @requires composition
 * @requires history
 * @requires knockout
 * @requires jquery
 */
define("plugins/router", ['durandal/system', 'durandal/app', 'durandal/activator', 'durandal/events', 'durandal/composition', 'plugins/history', 'knockout', 'jquery'], function(system, app, activator, events, composition, history, ko, $) {
    var optionalParam = /\((.*?)\)/g;
    var namedParam = /(\(\?)?:\w+/g;
    var splatParam = /\*\w+/g;
    var escapeRegExp = /[\-{}\[\]+?.,\\\^$|#\s]/g;
    var startDeferred, rootRouter;
    var trailingSlash = /\/$/;
    var routesAreCaseSensitive = false;
    var lastUrl = '/', lastTryUrl = '/';

    function routeStringToRegExp(routeString) {
        routeString = routeString.replace(escapeRegExp, '\\$&')
            .replace(optionalParam, '(?:$1)?')
            .replace(namedParam, function(match, optional) {
                return optional ? match : '([^\/]+)';
            })
            .replace(splatParam, '(.*?)');

        return new RegExp('^' + routeString + '$', routesAreCaseSensitive ? undefined : 'i');
    }

    function stripParametersFromRoute(route) {
        var colonIndex = route.indexOf(':');
        var length = colonIndex > 0 ? colonIndex - 1 : route.length;
        return route.substring(0, length);
    }

    function endsWith(str, suffix) {
        return str.indexOf(suffix, str.length - suffix.length) !== -1;
    }

    function compareArrays(first, second) {
        if (!first || !second){
            return false;
        }

        if (first.length != second.length) {
            return false;
        }

        for (var i = 0, len = first.length; i < len; i++) {
            if (first[i] != second[i]) {
                return false;
            }
        }

        return true;
    }

    function reconstructUrl(instruction){
        if(!instruction.queryString){
            return instruction.fragment;
        }

        return instruction.fragment + '?' + instruction.queryString;
    }

    /**
     * @class Router
     * @uses Events
     */

    /**
     * Triggered when the navigation logic has completed.
     * @event router:navigation:complete
     * @param {object} instance The activated instance.
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered when the navigation has been cancelled.
     * @event router:navigation:cancelled
     * @param {object} instance The activated instance.
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered when navigation begins.
     * @event router:navigation:processing
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered right before a route is activated.
     * @event router:route:activating
     * @param {object} instance The activated instance.
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered right before a route is configured.
     * @event router:route:before-config
     * @param {object} config The route config.
     * @param {Router} router The router.
     */

    /**
     * Triggered just after a route is configured.
     * @event router:route:after-config
     * @param {object} config The route config.
     * @param {Router} router The router.
     */

    /**
     * Triggered when the view for the activated instance is attached.
     * @event router:navigation:attached
     * @param {object} instance The activated instance.
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered when the composition that the activated instance participates in is complete.
     * @event router:navigation:composition-complete
     * @param {object} instance The activated instance.
     * @param {object} instruction The routing instruction.
     * @param {Router} router The router.
     */

    /**
     * Triggered when the router does not find a matching route.
     * @event router:route:not-found
     * @param {string} fragment The url fragment.
     * @param {Router} router The router.
     */

    var createRouter = function() {
        var queue = [],
            isProcessing = ko.observable(false),
            currentActivation,
            currentInstruction,
            activeItem = activator.create();

        var router = {
            /**
             * The route handlers that are registered. Each handler consists of a `routePattern` and a `callback`.
             * @property {object[]} handlers
             */
            handlers: [],
            /**
             * The route configs that are registered.
             * @property {object[]} routes
             */
            routes: [],
            /**
             * The route configurations that have been designated as displayable in a nav ui (nav:true).
             * @property {KnockoutObservableArray} navigationModel
             */
            navigationModel: ko.observableArray([]),
            /**
             * The active item/screen based on the current navigation state.
             * @property {Activator} activeItem
             */
            activeItem: activeItem,
            /**
             * Indicates that the router (or a child router) is currently in the process of navigating.
             * @property {KnockoutComputed} isNavigating
             */
            isNavigating: ko.computed(function() {
                var current = activeItem();
                var processing = isProcessing();
                var currentRouterIsProcesing = current
                    && current.router
                    && current.router != router
                    && current.router.isNavigating() ? true : false;
                return  processing || currentRouterIsProcesing;
            }),
            /**
             * An observable surfacing the active routing instruction that is currently being processed or has recently finished processing.
             * The instruction object has `config`, `fragment`, `queryString`, `params` and `queryParams` properties.
             * @property {KnockoutObservable} activeInstruction
             */
            activeInstruction:ko.observable(null),
            __router__:true
        };

        events.includeIn(router);

        activeItem.settings.areSameItem = function (currentItem, newItem, currentActivationData, newActivationData) {
            if (currentItem == newItem) {
                return compareArrays(currentActivationData, newActivationData);
            }

            return false;
        };

        activeItem.settings.findChildActivator = function(item) {
            if (item && item.router && item.router.parent == router) {
                return item.router.activeItem;
            }

            return null;
        };

        function hasChildRouter(instance, parentRouter) {
            return instance.router && instance.router.parent == parentRouter;
        }

        function setCurrentInstructionRouteIsActive(flag) {
            if (currentInstruction && currentInstruction.config.isActive) {
                currentInstruction.config.isActive(flag);
            }
        }

        function completeNavigation(instance, instruction, mode) {
            system.log('Navigation Complete', instance, instruction);

            var fromModuleId = system.getModuleId(currentActivation);
            if (fromModuleId) {
                router.trigger('router:navigation:from:' + fromModuleId);
            }

            currentActivation = instance;

            setCurrentInstructionRouteIsActive(false);
            currentInstruction = instruction;
            setCurrentInstructionRouteIsActive(true);

            var toModuleId = system.getModuleId(currentActivation);
            if (toModuleId) {
                router.trigger('router:navigation:to:' + toModuleId);
            }

            if (!hasChildRouter(instance, router)) {
                router.updateDocumentTitle(instance, instruction);
            }

            switch (mode) {
                case 'rootRouter':
                    lastUrl = reconstructUrl(currentInstruction);
                    break;
                case 'rootRouterWithChild':
                    lastTryUrl = reconstructUrl(currentInstruction);
                    break;
                case 'lastChildRouter':
                    lastUrl = lastTryUrl;
                    break;
            }

            rootRouter.explicitNavigation = false;
            rootRouter.navigatingBack = false;

            router.trigger('router:navigation:complete', instance, instruction, router);
        }

        function cancelNavigation(instance, instruction) {
            system.log('Navigation Cancelled');

            router.activeInstruction(currentInstruction);

            router.navigate(lastUrl, false);

            isProcessing(false);
            rootRouter.explicitNavigation = false;
            rootRouter.navigatingBack = false;
            router.trigger('router:navigation:cancelled', instance, instruction, router);
        }

        function redirect(url) {
            system.log('Navigation Redirecting');

            isProcessing(false);
            rootRouter.explicitNavigation = false;
            rootRouter.navigatingBack = false;
            router.navigate(url, { trigger: true, replace: true });
        }

        function activateRoute(activator, instance, instruction) {
            rootRouter.navigatingBack = !rootRouter.explicitNavigation && currentActivation != instruction.fragment;
            router.trigger('router:route:activating', instance, instruction, router);

            var options = {
                canDeactivate: !router.parent
            };

            activator.activateItem(instance, instruction.params, options).then(function(succeeded) {
                if (succeeded) {
                    var previousActivation = currentActivation;
                    var withChild = hasChildRouter(instance, router);
                    var mode = '';

                    if (router.parent) {
                        if(!withChild) {
                            mode = 'lastChildRouter';
                        }
                    } else {
                        if (withChild) {
                            mode = 'rootRouterWithChild';
                        } else {
                            mode = 'rootRouter';
                        }
                    }

                    completeNavigation(instance, instruction, mode);

                    if (withChild) {
                        instance.router.trigger('router:route:before-child-routes', instance, instruction, router);

                        var fullFragment = instruction.fragment;
                        if (instruction.queryString) {
                            fullFragment += "?" + instruction.queryString;
                        }

                        instance.router.loadUrl(fullFragment);
                    }

                    if (previousActivation == instance) {
                        router.attached();
                        router.compositionComplete();
                    }
                } else if(activator.settings.lifecycleData && activator.settings.lifecycleData.redirect){
                    redirect(activator.settings.lifecycleData.redirect);
                }else{
                    cancelNavigation(instance, instruction);
                }

                if (startDeferred) {
                    startDeferred.resolve();
                    startDeferred = null;
                }
            }).fail(function(err){
                system.error(err);
            });
        }

        /**
         * Inspects routes and modules before activation. Can be used to protect access by cancelling navigation or redirecting.
         * @method guardRoute
         * @param {object} instance The module instance that is about to be activated by the router.
         * @param {object} instruction The route instruction. The instruction object has config, fragment, queryString, params and queryParams properties.
         * @return {Promise|Boolean|String} If a boolean, determines whether or not the route should activate or be cancelled. If a string, causes a redirect to the specified route. Can also be a promise for either of these value types.
         */
        function handleGuardedRoute(activator, instance, instruction) {
            var resultOrPromise = router.guardRoute(instance, instruction);
            if (resultOrPromise || resultOrPromise === '') {
                if (resultOrPromise.then) {
                    resultOrPromise.then(function(result) {
                        if (result) {
                            if (system.isString(result)) {
                                redirect(result);
                            } else {
                                activateRoute(activator, instance, instruction);
                            }
                        } else {
                            cancelNavigation(instance, instruction);
                        }
                    });
                } else {
                    if (system.isString(resultOrPromise)) {
                        redirect(resultOrPromise);
                    } else {
                        activateRoute(activator, instance, instruction);
                    }
                }
            } else {
                cancelNavigation(instance, instruction);
            }
        }

        function ensureActivation(activator, instance, instruction) {
            if (router.guardRoute) {
                handleGuardedRoute(activator, instance, instruction);
            } else {
                activateRoute(activator, instance, instruction);
            }
        }

        function canReuseCurrentActivation(instruction) {
            return currentInstruction
                && currentInstruction.config.moduleId == instruction.config.moduleId
                && currentActivation
                && ((currentActivation.canReuseForRoute && currentActivation.canReuseForRoute.apply(currentActivation, instruction.params))
                || (!currentActivation.canReuseForRoute && currentActivation.router && currentActivation.router.loadUrl));
        }

        function dequeueInstruction() {
            if (isProcessing()) {
                return;
            }

            var instruction = queue.shift();
            queue = [];

            if (!instruction) {
                return;
            }

            isProcessing(true);
            router.activeInstruction(instruction);
            router.trigger('router:navigation:processing', instruction, router);

            if (canReuseCurrentActivation(instruction)) {
                var tempActivator = activator.create();
                tempActivator.forceActiveItem(currentActivation); //enforce lifecycle without re-compose
                tempActivator.settings.areSameItem = activeItem.settings.areSameItem;
                tempActivator.settings.findChildActivator = activeItem.settings.findChildActivator;
                ensureActivation(tempActivator, currentActivation, instruction);
            } else if(!instruction.config.moduleId) {
                ensureActivation(activeItem, {
                    viewUrl:instruction.config.viewUrl,
                    canReuseForRoute:function() {
                        return true;
                    }
                }, instruction);
            } else {
                system.acquire(instruction.config.moduleId).then(function(m) {
                    var instance = system.resolveObject(m);

                    if(instruction.config.viewUrl) {
                        instance.viewUrl = instruction.config.viewUrl;
                    }

                    ensureActivation(activeItem, instance, instruction);
                }).fail(function(err) {
                    cancelNavigation(null, instruction);
                    system.error('Failed to load routed module (' + instruction.config.moduleId + '). Details: ' + err.message, err);
                });
            }
        }

        function queueInstruction(instruction) {
            queue.unshift(instruction);
            dequeueInstruction();
        }

        // Given a route, and a URL fragment that it matches, return the array of
        // extracted decoded parameters. Empty or unmatched parameters will be
        // treated as `null` to normalize cross-browser behavior.
        function createParams(routePattern, fragment, queryString) {
            var params = routePattern.exec(fragment).slice(1);

            for (var i = 0; i < params.length; i++) {
                var current = params[i];
                params[i] = current ? decodeURIComponent(current) : null;
            }

            var queryParams = router.parseQueryString(queryString);
            if (queryParams) {
                params.push(queryParams);
            }

            return {
                params:params,
                queryParams:queryParams
            };
        }

        function configureRoute(config){
            router.trigger('router:route:before-config', config, router);

            if (!system.isRegExp(config.route)) {
                config.title = config.title || router.convertRouteToTitle(config.route);

                if (!config.viewUrl) {
                    config.moduleId = config.moduleId || router.convertRouteToModuleId(config.route);
                }
                
                config.hash = config.hash || router.convertRouteToHash(config.route);

                if (config.hasChildRoutes) {
                    config.route = config.route + '*childRoutes';
                }

                config.routePattern = routeStringToRegExp(config.route);
            }else{
                config.routePattern = config.route;
            }

            config.isActive = config.isActive || ko.observable(false);
            router.trigger('router:route:after-config', config, router);
            router.routes.push(config);

            router.route(config.routePattern, function(fragment, queryString) {
                var paramInfo = createParams(config.routePattern, fragment, queryString);
                queueInstruction({
                    fragment: fragment,
                    queryString:queryString,
                    config: config,
                    params: paramInfo.params,
                    queryParams:paramInfo.queryParams
                });
            });
        };

        function mapRoute(config) {
            if(system.isArray(config.route)){
                var isActive = config.isActive || ko.observable(false);

                for(var i = 0, length = config.route.length; i < length; i++){
                    var current = system.extend({}, config);

                    current.route = config.route[i];
                    current.isActive = isActive;

                    if(i > 0){
                        delete current.nav;
                    }

                    configureRoute(current);
                }
            }else{
                configureRoute(config);
            }

            return router;
        }

        /**
         * Parses a query string into an object.
         * @method parseQueryString
         * @param {string} queryString The query string to parse.
         * @return {object} An object keyed according to the query string parameters.
         */
        router.parseQueryString = function (queryString) {
            var queryObject, pairs;

            if (!queryString) {
                return null;
            }

            pairs = queryString.split('&');

            if (pairs.length == 0) {
                return null;
            }

            queryObject = {};

            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i];
                if (pair === '') {
                    continue;
                }

                var parts = pair.split(/=(.+)?/),
                    key = parts[0],
                    value = parts[1] && decodeURIComponent(parts[1].replace(/\+/g, ' '));

                var existing = queryObject[key];

                if (existing) {
                    if (system.isArray(existing)) {
                        existing.push(value);
                    } else {
                        queryObject[key] = [existing, value];
                    }
                }
                else {
                    queryObject[key] = value;
                }
            }

            return queryObject;
        };

        /**
         * Add a route to be tested when the url fragment changes.
         * @method route
         * @param {RegEx} routePattern The route pattern to test against.
         * @param {function} callback The callback to execute when the route pattern is matched.
         */
        router.route = function(routePattern, callback) {
            router.handlers.push({ routePattern: routePattern, callback: callback });
        };

        /**
         * Attempt to load the specified URL fragment. If a route succeeds with a match, returns `true`. If no defined routes matches the fragment, returns `false`.
         * @method loadUrl
         * @param {string} fragment The URL fragment to find a match for.
         * @return {boolean} True if a match was found, false otherwise.
         */
        router.loadUrl = function(fragment) {
            var handlers = router.handlers,
                queryString = null,
                coreFragment = fragment,
                queryIndex = fragment.indexOf('?');

            if (queryIndex != -1) {
                coreFragment = fragment.substring(0, queryIndex);
                queryString = fragment.substr(queryIndex + 1);
            }

            if(router.relativeToParentRouter){
                var instruction = this.parent.activeInstruction();
                coreFragment = queryIndex == -1 ? instruction.params.join('/') : instruction.params.slice(0, -1).join('/');

                if(coreFragment && coreFragment.charAt(0) == '/'){
                    coreFragment = coreFragment.substr(1);
                }

                if(!coreFragment){
                    coreFragment = '';
                }

                coreFragment = coreFragment.replace('//', '/').replace('//', '/');
            }

            coreFragment = coreFragment.replace(trailingSlash, '');

            for (var i = 0; i < handlers.length; i++) {
                var current = handlers[i];
                if (current.routePattern.test(coreFragment)) {
                    current.callback(coreFragment, queryString);
                    return true;
                }
            }

            system.log('Route Not Found', fragment, currentInstruction);
            router.trigger('router:route:not-found', fragment, router);

            if (router.parent) {
                lastUrl = lastTryUrl;
            }

            history.navigate(lastUrl, { trigger:false, replace:true });

            rootRouter.explicitNavigation = false;
            rootRouter.navigatingBack = false;

            return false;
        };

        var titleSubscription;
        function setTitle(value) {
            var appTitle = ko.unwrap(app.title);

            if (appTitle) {
                document.title = value + " | " + appTitle;
            } else {
                document.title = value;
            }
        }  
        
        // Allow observable to be used for app.title
        if(ko.isObservable(app.title)) {
            app.title.subscribe(function () {
                var instruction = router.activeInstruction();
                var title = instruction != null ? ko.unwrap(instruction.config.title) : '';
                setTitle(title);
            });
        }
        
        /**
         * Updates the document title based on the activated module instance, the routing instruction and the app.title.
         * @method updateDocumentTitle
         * @param {object} instance The activated module.
         * @param {object} instruction The routing instruction associated with the action. It has a `config` property that references the original route mapping config.
         */
        router.updateDocumentTitle = function (instance, instruction) {
            var appTitle = ko.unwrap(app.title),
                title = instruction.config.title;
                
            if (titleSubscription) {
                titleSubscription.dispose();
            }

            if (title) {
                if (ko.isObservable(title)) {
                    titleSubscription = title.subscribe(setTitle);
                    setTitle(title());
                } else {
                    setTitle(title);
                }
            } else if (appTitle) {
                document.title = appTitle;
            }
        };

        /**
         * Save a fragment into the hash history, or replace the URL state if the
         * 'replace' option is passed. You are responsible for properly URL-encoding
         * the fragment in advance.
         * The options object can contain `trigger: false` if you wish to not have the
         * route callback be fired, or `replace: true`, if
         * you wish to modify the current URL without adding an entry to the history.
         * @method navigate
         * @param {string} fragment The url fragment to navigate to.
         * @param {object|boolean} options An options object with optional trigger and replace flags. You can also pass a boolean directly to set the trigger option. Trigger is `true` by default.
         * @return {boolean} Returns true/false from loading the url.
         */
        router.navigate = function(fragment, options) {
            if(fragment && fragment.indexOf('://') != -1) {
                window.location.href = fragment;
                return true;
            }

            if(options === undefined || (system.isBoolean(options) && options) || (system.isObject(options) && options.trigger)) {
                rootRouter.explicitNavigation = true;
            }

            if ((system.isBoolean(options) && !options) || (options && options.trigger != undefined && !options.trigger)) {
                lastUrl = fragment;
            }

            return history.navigate(fragment, options);
        };

        /**
         * Navigates back in the browser history.
         * @method navigateBack
         */
        router.navigateBack = function() {
            history.navigateBack();
        };

        router.attached = function() {
            router.trigger('router:navigation:attached', currentActivation, currentInstruction, router);
        };

        router.compositionComplete = function(){
            isProcessing(false);
            router.trigger('router:navigation:composition-complete', currentActivation, currentInstruction, router);
            dequeueInstruction();
        };

        /**
         * Converts a route to a hash suitable for binding to a link's href.
         * @method convertRouteToHash
         * @param {string} route
         * @return {string} The hash.
         */
        router.convertRouteToHash = function(route) {
            route = route.replace(/\*.*$/, '');

            if(router.relativeToParentRouter){
                var instruction = router.parent.activeInstruction(),
                    hash = route ? instruction.config.hash + '/' + route : instruction.config.hash;

                if(history._hasPushState){
                    hash = '/' + hash;
                }

                hash = hash.replace('//', '/').replace('//', '/');
                return hash;
            }

            if(history._hasPushState){
                return route;
            }

            return "#" + route;
        };

        /**
         * Converts a route to a module id. This is only called if no module id is supplied as part of the route mapping.
         * @method convertRouteToModuleId
         * @param {string} route
         * @return {string} The module id.
         */
        router.convertRouteToModuleId = function(route) {
            return stripParametersFromRoute(route);
        };

        /**
         * Converts a route to a displayable title. This is only called if no title is specified as part of the route mapping.
         * @method convertRouteToTitle
         * @param {string} route
         * @return {string} The title.
         */
        router.convertRouteToTitle = function(route) {
            var value = stripParametersFromRoute(route);
            return value.substring(0, 1).toUpperCase() + value.substring(1);
        };

        /**
         * Maps route patterns to modules.
         * @method map
         * @param {string|object|object[]} route A route, config or array of configs.
         * @param {object} [config] The config for the specified route.
         * @chainable
         * @example
         router.map([
         { route: '', title:'Home', moduleId: 'homeScreen', nav: true },
         { route: 'customer/:id', moduleId: 'customerDetails'}
         ]);
         */
        router.map = function(route, config) {
            if (system.isArray(route)) {
                for (var i = 0; i < route.length; i++) {
                    router.map(route[i]);
                }

                return router;
            }

            if (system.isString(route) || system.isRegExp(route)) {
                if (!config) {
                    config = {};
                } else if (system.isString(config)) {
                    config = { moduleId: config };
                }

                config.route = route;
            } else {
                config = route;
            }

            return mapRoute(config);
        };

        /**
         * Builds an observable array designed to bind a navigation UI to. The model will exist in the `navigationModel` property.
         * @method buildNavigationModel
         * @param {number} defaultOrder The default order to use for navigation visible routes that don't specify an order. The default is 100 and each successive route will be one more than that.
         * @chainable
         */
        router.buildNavigationModel = function(defaultOrder) {
            var nav = [], routes = router.routes;
            var fallbackOrder = defaultOrder || 100;

            for (var i = 0; i < routes.length; i++) {
                var current = routes[i];

                if (current.nav) {
                    if (!system.isNumber(current.nav)) {
                        current.nav = ++fallbackOrder;
                    }

                    nav.push(current);
                }
            }

            nav.sort(function(a, b) { return a.nav - b.nav; });
            router.navigationModel(nav);

            return router;
        };

        /**
         * Configures how the router will handle unknown routes.
         * @method mapUnknownRoutes
         * @param {string|function} [config] If not supplied, then the router will map routes to modules with the same name.
         * If a string is supplied, it represents the module id to route all unknown routes to.
         * Finally, if config is a function, it will be called back with the route instruction containing the route info. The function can then modify the instruction by adding a moduleId and the router will take over from there.
         * @param {string} [replaceRoute] If config is a module id, then you can optionally provide a route to replace the url with.
         * @chainable
         */
        router.mapUnknownRoutes = function(config, replaceRoute) {
            var catchAllRoute = "*catchall";
            var catchAllPattern = routeStringToRegExp(catchAllRoute);

            router.route(catchAllPattern, function (fragment, queryString) {
                var paramInfo = createParams(catchAllPattern, fragment, queryString);
                var instruction = {
                    fragment: fragment,
                    queryString: queryString,
                    config: {
                        route: catchAllRoute,
                        routePattern: catchAllPattern
                    },
                    params: paramInfo.params,
                    queryParams: paramInfo.queryParams
                };

                if (!config) {
                    instruction.config.moduleId = fragment;
                } else if (system.isString(config)) {
                    instruction.config.moduleId = config;
                    if(replaceRoute){
                        history.navigate(replaceRoute, { trigger:false, replace:true });
                    }
                } else if (system.isFunction(config)) {
                    var result = config(instruction);
                    if (result && result.then) {
                        result.then(function() {
                            router.trigger('router:route:before-config', instruction.config, router);
                            router.trigger('router:route:after-config', instruction.config, router);
                            queueInstruction(instruction);
                        });
                        return;
                    }
                } else {
                    instruction.config = config;
                    instruction.config.route = catchAllRoute;
                    instruction.config.routePattern = catchAllPattern;
                }

                router.trigger('router:route:before-config', instruction.config, router);
                router.trigger('router:route:after-config', instruction.config, router);
                queueInstruction(instruction);
            });

            return router;
        };

        /**
         * Resets the router by removing handlers, routes, event handlers and previously configured options.
         * @method reset
         * @chainable
         */
        router.reset = function() {
            currentInstruction = currentActivation = undefined;
            router.handlers = [];
            router.routes = [];
            router.off();
            delete router.options;
            return router;
        };

        /**
         * Makes all configured routes and/or module ids relative to a certain base url.
         * @method makeRelative
         * @param {string|object} settings If string, the value is used as the base for routes and module ids. If an object, you can specify `route` and `moduleId` separately. In place of specifying route, you can set `fromParent:true` to make routes automatically relative to the parent router's active route.
         * @chainable
         */
        router.makeRelative = function(settings){
            if(system.isString(settings)){
                settings = {
                    moduleId:settings,
                    route:settings
                };
            }

            if(settings.moduleId && !endsWith(settings.moduleId, '/')){
                settings.moduleId += '/';
            }

            if(settings.route && !endsWith(settings.route, '/')){
                settings.route += '/';
            }

            if(settings.fromParent){
                router.relativeToParentRouter = true;
            }

            router.on('router:route:before-config').then(function(config){
                if(settings.moduleId){
                    config.moduleId = settings.moduleId + config.moduleId;
                }

                if(settings.route){
                    if(config.route === ''){
                        config.route = settings.route.substring(0, settings.route.length - 1);
                    }else{
                        config.route = settings.route + config.route;
                    }
                }
            });

            if (settings.dynamicHash) {
                router.on('router:route:after-config').then(function (config) {
                    config.routePattern = routeStringToRegExp(config.route ? settings.dynamicHash + '/' + config.route : settings.dynamicHash);
                    config.dynamicHash = config.dynamicHash || ko.observable(config.hash);
                });

                router.on('router:route:before-child-routes').then(function(instance, instruction, parentRouter) {
                    var childRouter = instance.router;

                    for(var i = 0; i < childRouter.routes.length; i++) {
                        var route = childRouter.routes[i];
                        var params = instruction.params.slice(0);

                        route.hash = childRouter.convertRouteToHash(route.route)
                            .replace(namedParam, function(match) {
                                return params.length > 0 ? params.shift() : match;
                            });

                        route.dynamicHash(route.hash);
                    }
                });
            }

            return router;
        };

        /**
         * Creates a child router.
         * @method createChildRouter
         * @return {Router} The child router.
         */
        router.createChildRouter = function() {
            var childRouter = createRouter();
            childRouter.parent = router;
            return childRouter;
        };

        return router;
    };

    /**
     * @class RouterModule
     * @extends Router
     * @static
     */
    rootRouter = createRouter();
    rootRouter.explicitNavigation = false;
    rootRouter.navigatingBack = false;

    /**
     * Makes the RegExp generated for routes case sensitive, rather than the default of case insensitive.
     * @method makeRoutesCaseSensitive
     */
    rootRouter.makeRoutesCaseSensitive = function(){
        routesAreCaseSensitive = true;
    };

    /**
     * Verify that the target is the current window
     * @method targetIsThisWindow
     * @return {boolean} True if the event's target is the current window, false otherwise.
     */
    rootRouter.targetIsThisWindow = function(event) {
        var targetWindow = $(event.target).attr('target');

        if (!targetWindow ||
            targetWindow === window.name ||
            targetWindow === '_self' ||
            (targetWindow === 'top' && window === window.top)) { return true; }

        return false;
    };

    /**
     * Activates the router and the underlying history tracking mechanism.
     * @method activate
     * @return {Promise} A promise that resolves when the router is ready.
     */
    rootRouter.activate = function(options) {
        return system.defer(function(dfd) {
            startDeferred = dfd;
            rootRouter.options = system.extend({ routeHandler: rootRouter.loadUrl }, rootRouter.options, options);

            history.activate(rootRouter.options);

            if(history._hasPushState){
                var routes = rootRouter.routes,
                    i = routes.length;

                while(i--){
                    var current = routes[i];
                    current.hash = current.hash.replace('#', '/');
                }
            }

            var rootStripper = rootRouter.options.root && new RegExp("^" + rootRouter.options.root + "/");

            $(document).delegate("a", 'click', function(evt){
                if(history._hasPushState){
                    if(!evt.altKey && !evt.ctrlKey && !evt.metaKey && !evt.shiftKey && rootRouter.targetIsThisWindow(evt)){
                        var href = $(this).attr("href");

                        // Ensure the protocol is not part of URL, meaning its relative.
                        // Stop the event bubbling to ensure the link will not cause a page refresh.
                        if (href != null && !(href.charAt(0) === "#" || /^[a-z]+:/i.test(href))) {
                            rootRouter.explicitNavigation = true;
                            evt.preventDefault();

                            if (rootStripper) {
                                href = href.replace(rootStripper, "");
                            }

                            history.navigate(href);
                        }
                    }
                }else{
                    rootRouter.explicitNavigation = true;
                }
            });

            if(history.options.silent && startDeferred){
                startDeferred.resolve();
                startDeferred = null;
            }
        }).promise();
    };

    /**
     * Disable history, perhaps temporarily. Not useful in a real app, but possibly useful for unit testing Routers.
     * @method deactivate
     */
    rootRouter.deactivate = function() {
        history.deactivate();
    };

    /**
     * Installs the router's custom ko binding handler.
     * @method install
     */
    rootRouter.install = function(){
        ko.bindingHandlers.router = {
            init: function() {
                return { controlsDescendantBindings: true };
            },
            update: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var settings = ko.utils.unwrapObservable(valueAccessor()) || {};

                if (settings.__router__) {
                    settings = {
                        model:settings.activeItem(),
                        attached:settings.attached,
                        compositionComplete:settings.compositionComplete,
                        activate: false
                    };
                } else {
                    var theRouter = ko.utils.unwrapObservable(settings.router || viewModel.router) || rootRouter;
                    settings.model = theRouter.activeItem();
                    settings.attached = theRouter.attached;
                    settings.compositionComplete = theRouter.compositionComplete;
                    settings.activate = false;
                }

                composition.compose(element, settings, bindingContext);
            }
        };

        ko.virtualElements.allowedBindings.router = true;
    };

    return rootRouter;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Serializes and deserializes data to/from JSON.
 * @module serializer
 * @requires system
 */
define("plugins/serializer", ['durandal/system'], function(system) {
    /**
     * @class SerializerModule
     * @static
     */
    return {
        /**
         * The name of the attribute that the serializer should use to identify an object's type.
         * @property {string} typeAttribute
         * @default type
         */
        typeAttribute: 'type',
        /**
         * The amount of space to use for indentation when writing out JSON.
         * @property {string|number} space
         * @default undefined
         */
        space:undefined,
        /**
         * The default replacer function used during serialization. By default properties starting with '_' or '$' are removed from the serialized object.
         * @method replacer
         * @param {string} key The object key to check.
         * @param {object} value The object value to check.
         * @return {object} The value to serialize.
         */
        replacer: function(key, value) {
            if(key){
                var first = key[0];
                if(first === '_' || first === '$'){
                    return undefined;
                }
            }

            return value;
        },
        /**
         * Serializes the object.
         * @method serialize
         * @param {object} object The object to serialize.
         * @param {object} [settings] Settings can specify a replacer or space to override the serializer defaults.
         * @return {string} The JSON string.
         */
        serialize: function(object, settings) {
            settings = (settings === undefined) ? {} : settings;

            if(system.isString(settings) || system.isNumber(settings)) {
                settings = { space: settings };
            }

            return JSON.stringify(object, settings.replacer || this.replacer, settings.space || this.space);
        },
        /**
         * Gets the type id for an object instance, using the configured `typeAttribute`.
         * @method getTypeId
         * @param {object} object The object to serialize.
         * @return {string} The type.
         */
        getTypeId: function(object) {
            if (object) {
                return object[this.typeAttribute];
            }

            return undefined;
        },
        /**
         * Maps type ids to object constructor functions. Keys are type ids and values are functions.
         * @property {object} typeMap.
         */
        typeMap: {},
        /**
         * Adds a type id/constructor function mampping to the `typeMap`.
         * @method registerType
         * @param {string} typeId The type id.
         * @param {function} constructor The constructor.
         */
        registerType: function() {
            var first = arguments[0];

            if (arguments.length == 1) {
                var id = first[this.typeAttribute] || system.getModuleId(first);
                this.typeMap[id] = first;
            } else {
                this.typeMap[first] = arguments[1];
            }
        },
        /**
         * The default reviver function used during deserialization. By default is detects type properties on objects and uses them to re-construct the correct object using the provided constructor mapping.
         * @method reviver
         * @param {string} key The attribute key.
         * @param {object} value The object value associated with the key.
         * @param {function} getTypeId A custom function used to get the type id from a value.
         * @param {object} getConstructor A custom function used to get the constructor function associated with a type id.
         * @return {object} The value.
         */
        reviver: function(key, value, getTypeId, getConstructor) {
            var typeId = getTypeId(value);
            if (typeId) {
                var ctor = getConstructor(typeId);
                if (ctor) {
                    if (ctor.fromJSON) {
                        return ctor.fromJSON(value);
                    }

                    return new ctor(value);
                }
            }

            return value;
        },
        /**
         * Deserialize the JSON.
         * @method deserialize
         * @param {string} text The JSON string.
         * @param {object} [settings] Settings can specify a reviver, getTypeId function or getConstructor function.
         * @return {object} The deserialized object.
         */
        deserialize: function(text, settings) {
            var that = this;
            settings = settings || {};

            var getTypeId = settings.getTypeId || function(object) { return that.getTypeId(object); };
            var getConstructor = settings.getConstructor || function(id) { return that.typeMap[id]; };
            var reviver = settings.reviver || function(key, value) { return that.reviver(key, value, getTypeId, getConstructor); };

            return JSON.parse(text, reviver);
        },
        /**
         * Clone the object.
         * @method clone
         * @param {object} obj The object to clone.
         * @param {object} [settings] Settings can specify any of the options allowed by the serialize or deserialize methods.
         * @return {object} The new clone.
         */
        clone:function(obj, settings) {
            return this.deserialize(this.serialize(obj, settings), settings);
        }
    };
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * Layers the widget sugar on top of the composition system.
 * @module widget
 * @requires system
 * @requires composition
 * @requires jquery
 * @requires knockout
 */
define("plugins/widget", ['durandal/system', 'durandal/composition', 'jquery', 'knockout'], function(system, composition, $, ko) {
    var kindModuleMaps = {},
        kindViewMaps = {},
        bindableSettings = ['model', 'view', 'kind'],
        widgetDataKey = 'durandal-widget-data';

    function extractParts(element, settings){
        var data = ko.utils.domData.get(element, widgetDataKey);

        if(!data){
            data = {
                parts:composition.cloneNodes(ko.virtualElements.childNodes(element))
            };

            ko.virtualElements.emptyNode(element);
            ko.utils.domData.set(element, widgetDataKey, data);
        }

        settings.parts = data.parts;
    }

    /**
     * @class WidgetModule
     * @static
     */
    var widget = {
        getSettings: function(valueAccessor) {
            var settings = ko.utils.unwrapObservable(valueAccessor()) || {};

            if (system.isString(settings)) {
                return { kind: settings };
            }

            for (var attrName in settings) {
                if (ko.utils.arrayIndexOf(bindableSettings, attrName) != -1) {
                    settings[attrName] = ko.utils.unwrapObservable(settings[attrName]);
                } else {
                    settings[attrName] = settings[attrName];
                }
            }

            return settings;
        },
        /**
         * Creates a ko binding handler for the specified kind.
         * @method registerKind
         * @param {string} kind The kind to create a custom binding handler for.
         */
        registerKind: function(kind) {
            ko.bindingHandlers[kind] = {
                init: function() {
                    return { controlsDescendantBindings: true };
                },
                update: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var settings = widget.getSettings(valueAccessor);
                    settings.kind = kind;
                    extractParts(element, settings);
                    widget.create(element, settings, bindingContext, true);
                }
            };

            ko.virtualElements.allowedBindings[kind] = true;
            composition.composeBindings.push(kind + ':');
        },
        /**
         * Maps views and module to the kind identifier if a non-standard pattern is desired.
         * @method mapKind
         * @param {string} kind The kind name.
         * @param {string} [viewId] The unconventional view id to map the kind to.
         * @param {string} [moduleId] The unconventional module id to map the kind to.
         */
        mapKind: function(kind, viewId, moduleId) {
            if (viewId) {
                kindViewMaps[kind] = viewId;
            }

            if (moduleId) {
                kindModuleMaps[kind] = moduleId;
            }
        },
        /**
         * Maps a kind name to it's module id. First it looks up a custom mapped kind, then falls back to `convertKindToModulePath`.
         * @method mapKindToModuleId
         * @param {string} kind The kind name.
         * @return {string} The module id.
         */
        mapKindToModuleId: function(kind) {
            return kindModuleMaps[kind] || widget.convertKindToModulePath(kind);
        },
        /**
         * Converts a kind name to it's module path. Used to conventionally map kinds who aren't explicitly mapped through `mapKind`.
         * @method convertKindToModulePath
         * @param {string} kind The kind name.
         * @return {string} The module path.
         */
        convertKindToModulePath: function(kind) {
            return 'widgets/' + kind + '/viewmodel';
        },
        /**
         * Maps a kind name to it's view id. First it looks up a custom mapped kind, then falls back to `convertKindToViewPath`.
         * @method mapKindToViewId
         * @param {string} kind The kind name.
         * @return {string} The view id.
         */
        mapKindToViewId: function(kind) {
            return kindViewMaps[kind] || widget.convertKindToViewPath(kind);
        },
        /**
         * Converts a kind name to it's view id. Used to conventionally map kinds who aren't explicitly mapped through `mapKind`.
         * @method convertKindToViewPath
         * @param {string} kind The kind name.
         * @return {string} The view id.
         */
        convertKindToViewPath: function(kind) {
            return 'widgets/' + kind + '/view';
        },
        createCompositionSettings: function(element, settings) {
            if (!settings.model) {
                settings.model = this.mapKindToModuleId(settings.kind);
            }

            if (!settings.view) {
                settings.view = this.mapKindToViewId(settings.kind);
            }

            settings.preserveContext = true;
            settings.activate = true;
            settings.activationData = settings;
            settings.mode = 'templated';

            return settings;
        },
        /**
         * Creates a widget.
         * @method create
         * @param {DOMElement} element The DOMElement or knockout virtual element that serves as the target element for the widget.
         * @param {object} settings The widget settings.
         * @param {object} [bindingContext] The current binding context.
         */
        create: function(element, settings, bindingContext, fromBinding) {
            if(!fromBinding){
                settings = widget.getSettings(function() { return settings; }, element);
            }

            var compositionSettings = widget.createCompositionSettings(element, settings);

            composition.compose(element, compositionSettings, bindingContext);
        },
        /**
         * Installs the widget module by adding the widget binding handler and optionally registering kinds.
         * @method install
         * @param {object} config The module config. Add a `kinds` array with the names of widgets to automatically register. You can also specify a `bindingName` if you wish to use another name for the widget binding, such as "control" for example.
         */
        install:function(config){
            config.bindingName = config.bindingName || 'widget';

            if(config.kinds){
                var toRegister = config.kinds;

                for(var i = 0; i < toRegister.length; i++){
                    widget.registerKind(toRegister[i]);
                }
            }

            ko.bindingHandlers[config.bindingName] = {
                init: function() {
                    return { controlsDescendantBindings: true };
                },
                update: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var settings = widget.getSettings(valueAccessor);
                    extractParts(element, settings);
                    widget.create(element, settings, bindingContext, true);
                }
            };

            composition.composeBindings.push(config.bindingName + ':');
            ko.virtualElements.allowedBindings[config.bindingName] = true;
        }
    };

    return widget;
});
/**
 * Durandal 2.1.0 Copyright (c) 2012 Blue Spire Consulting, Inc. All Rights Reserved.
 * Available via the MIT license.
 * see: http://durandaljs.com or https://github.com/BlueSpire/Durandal for details.
 */
/**
 * The entrance transition module.
 * @module entrance
 * @requires system
 * @requires composition
 * @requires jquery
 */
define("transitions/entrance", ['durandal/system', 'durandal/composition', 'jquery'], function(system, composition, $) {
    var fadeOutDuration = 100;
    var endValues = {
        left: '0px',
        opacity: 1
    };
    var clearValues = {
        left: '',
        top: '',
        right: '',
        bottom:'',
        position:'',
        opacity: ''
    };

    var isIE = navigator.userAgent.match(/Trident/) || navigator.userAgent.match(/MSIE/);

    var animation = false,
        domPrefixes = 'Webkit Moz O ms Khtml'.split(' '),
        elm = document.createElement('div');

    if(elm.style.animationName !== undefined) {
        animation = true;
    }

    if(!animation) {
        for(var i = 0; i < domPrefixes.length; i++) {
            if(elm.style[domPrefixes[i] + 'AnimationName'] !== undefined) {
                animation = true;
                break;
            }
        }
    }

    if(animation) {
        if(isIE){
            system.log('Using CSS3/jQuery mixed animations.');
        }else{
            system.log('Using CSS3 animations.');
        }
    } else {
        system.log('Using jQuery animations.');
    }

    function removeAnimationClasses(ele, fadeOnly){
        ele.classList.remove(fadeOnly ? 'entrance-in-fade' : 'entrance-in');
        ele.classList.remove('entrance-out');
    }

    /**
     * @class EntranceModule
     * @constructor
     */
    var entrance = function(context) {
        return system.defer(function(dfd) {
            function endTransition() {
                dfd.resolve();
            }

            function scrollIfNeeded() {
                if (!context.keepScrollPosition) {
                    $(document).scrollTop(0);
                }
            }

            if (!context.child) {
                $(context.activeView).fadeOut(fadeOutDuration, endTransition);
            } else {
                var duration = context.duration || 500;
                var $child = $(context.child);
                var fadeOnly = !!context.fadeOnly;
                var startValues = {
                    display: 'block',
                    opacity: 0,
                    position: 'absolute',
                    left: fadeOnly || animation ? '0px' : '20px',
                    right: 0,
                    top: 0,
                    bottom: 0
                };

                function startTransition() {
                    scrollIfNeeded();
                    context.triggerAttach();

                    if (animation) {
                        removeAnimationClasses(context.child, fadeOnly);
                        context.child.classList.add(fadeOnly ? 'entrance-in-fade' : 'entrance-in');
                        setTimeout(function () {
                            removeAnimationClasses(context.child, fadeOnly);
                            if(context.activeView){
                                removeAnimationClasses(context.activeView, fadeOnly);
                            }
                            $child.css(clearValues);
                            endTransition();
                        }, duration);
                    } else {
                        $child.animate(endValues, {
                            duration: duration,
                            easing: 'swing',
                            always: function() {
                                $child.css(clearValues);
                                endTransition();
                            }
                        });
                    }
                }

                $child.css(startValues);

                if(context.activeView) {
                    if (animation && !isIE) {
                        removeAnimationClasses(context.activeView, fadeOnly);
                        context.activeView.classList.add('entrance-out');
                        setTimeout(startTransition, fadeOutDuration);
                    } else {
                        $(context.activeView).fadeOut({ duration: fadeOutDuration, always: startTransition });
                    }
                } else {
                    startTransition();
                }
            }
        }).promise();
    };

    return entrance;
});
define("transitions/fadeIn", ['durandal/system', 'transitions/transitionHelper'], function (system, helper) {
    var settings = {
        inAnimation: 'fadeIn',
        outAnimation: 'fadeOut'
    },
    fadeIn = function (context) {
        system.extend(context, settings);
        return helper.create(context);
    };

    return fadeIn;
});
define("transitions/fadeInVerticalDown", ['durandal/system', 'transitions/transitionHelper'], function (system, helper) {
    var settings = {
        inAnimation: 'fadeInDownBig',
        outAnimation: 'fadeOutUpBig'
    },
    fadeInVerticalDown = function (context) {
        system.extend(context, settings);
        return helper.create(context);
    };

    return fadeInVerticalDown;
});
define("transitions/transitionHelper", ['durandal/system', 'jquery'], function (system, $) {

    var animationTypes = [
        'bounce',
        'bounceIn',
        'bounceInDown',
        'bounceInLeft',
        'bounceInRight',
        'bounceInUp',
        'bounceOut',
        'bounceOutDown',
        'bounceOutLeft',
        'bounceOutRight',
        'bounceOutUp',
        'fadeIn',
        'fadeInDown',
        'fadeInDownBig',
        'fadeInLeft',
        'fadeInLeftBig',
        'fadeInRight',
        'fadeInRightBight',
        'fadeInUp',
        'fadeInUpBig',
        'fadeOut',
        'fadeOutDown',
        'fadeOutDownBig',
        'fadeOutLeft',
        'fadeOutLeftBig',
        'fadeOutRight',
        'fadeOutRightBig',
        'fadeOutUp',
        'fadeOutUpBig',
        'flash',
        'flip',
        'flipInX',
        'flipInY',
        'flipOutX',
        'flipOutY',
        'hinge',
        'lightSpeedIn',
        'lightSpeedOut',
        'pulse',
        'rollIn',
        'rollOut',
        'rotateIn',
        'rotateInDownLeft',
        'rotateInDownRight',
        'rotateInUpLeft',
        'rotateInUpRight',
        'rotateOut',
        'rotateOutDownLeft',
        'rotateOutDownRight',
        'rotateOutUpLeft',
        'roateOutUpRight',
        'shake',
        'swing',
        'tada',
        'wiggle',
        'wobble'
    ];

    var skippedModuleIds = [
        "viewmodels/indexing",
        "viewmodels/indexPerformance",
        "viewmodels/indexStats",
        "viewmodels/indexBatchSize",
        "viewmodels/indexPrefetches",
        "viewmodels/requests",
        "viewmodels/requestsCount",
        "viewmodels/requestTracing"
    ];

    function animValue(type) {
        if (Object.prototype.toString.call(type) == '[object String]') {
            return type;
        } else {
            return animationTypes[type];
        }
    }

    function ensureSettings(settings) {
        settings.inAnimation = settings.inAnimation || 'fadeInRight';
        settings.outAnimation = settings.outAnimation || 'fadeOut';
        return settings;
    }

    function doTrans(settings) {
        var parent = settings.parent,
          activeView = settings.activeView,
          newChild = settings.child,
          outAn = animValue(settings.outAnimation),
          inAn = animValue(settings.inAnimation),
          $previousView,
          $newView = $(newChild).removeClass(outAn); // just need to remove outAn here, keeping the animated class so we don't get a "flash"

        return system.defer(function (dfd) {
            function outTransition(callback) {
                $previousView = $(activeView);
                $previousView.addClass('animated');
                $previousView.addClass(outAn);
                setTimeout(function () {
                    if (callback) {
                        callback();
                    }
                }, App.duration);
            }

            function inTransition() {
                if ($previousView) {
                    $previousView.css('display', 'none');
                }
                settings.triggerAttach();

                $newView.addClass('animated');  //moved the adding of the animated class here so it keeps it together
                $newView.addClass(inAn);
                $newView.css('display', '');

                setTimeout(function () {
                    $newView.removeClass(inAn + ' animated'); // just need to remove inAn here, that's all we'll have
                    dfd.resolve(true);
                }, App.duration);
            }

            if (newChild) {
                if (!!settings.model && !!settings.model.__moduleId__ && skippedModuleIds.indexOf(settings.model.__moduleId__) > -1) {
                    dfd.resolve(true);
                    return;
                }

                $newView = $(newChild);
                if (settings.activeView) {
                    outTransition(inTransition);
                } else {
                    inTransition();
                } 
            }

        }).promise();
    }

    return App = {
        duration: 1000 * .13, // seconds
        create: function (settings) {
            settings = ensureSettings(settings);
            return doTrans(settings);
        }
    };
});


define(["durandal/system", "durandal/app", "durandal/viewLocator", "plugins/dialog", "durandal/composition"], function (system, app, viewLocator, dialog, composition) {
    //system.debug(true);
    
    NProgress.configure({ showSpinner: false });

    app.title = 'Raven.Studio';
    dialog.MessageBox.setViewUrl('views/dialog.html');

    app.configurePlugins({
        router: true,
        dialog: true,
        widget: true
    });

    app.start().then(function() {
        //Replace 'viewmodels' in the moduleId with 'views' to locate the view.
        //Look for partial views in a 'views' folder in the root.
        viewLocator.useConvention();

        if ("WebSocket" in window || "EventSource" in window) {
            //Show the app by setting the root view model for our application with a transition.
            app.setRoot("viewmodels/shell", "entrance");
            composition.defaultTransitionName = "fadeIn";
        } else {
            //The browser doesn't support nor WebSocket nor EventSource. IE 9, Firefox 6, Chrome 6 and below.
            app.showMessage("Your browser isn't supported. Please use a modern browser!", ":-(", []);
            NProgress.done();
        }
    });
});
/*import commandBase = require("commands/commandBase");
import pointChange = require("models/timeSeries/timeSeriesChange");
import timeSeries = require("models/timeSeries/timeSeries");

class addPointCommand extends commandBase {

    constructor(private ts: timeSeries, private type: string, private key: string, private at: number, private values: number[]) {
        super();
    }

    execute(): JQueryPromise<timeSeriesChange[]> {
        var args = {
            group: this.group,
            timeSeriesName: this.timeSeriesName,
            delta: this.delta
        };
        var url = "/change/" + this.group + "/" + this.timeSeriesName + this.urlEncodeArgs({delta: this.delta });
        var action = this.post(url, null, this.ts, { dataType: undefined });

        var successMessage = this.isNew ? "Successfully created a new timeSeries!" : "Successfully updated a timeSeries!";
        action.done(() => this.reportSuccess(successMessage));
        var failMessage = this.isNew ? "Failed to create a new timeSeries!" : "Successfully to update timeSeries!";
        action.fail((response: JQueryXHR) => this.reportError(failMessage, response.responseText, response.statusText));
        return action;
    }
}

export = addPointCommand;  */
// Want Intellisense comments for your extensions? Use JSDoc format:
/**
 * Summary goes here.
 * @someArg Description of someArg here.
 */
var TenantType;
(function (TenantType) {
    TenantType[TenantType["Database"] = 0] = "Database";
    TenantType[TenantType["FileSystem"] = 1] = "FileSystem";
    TenantType[TenantType["CounterStorage"] = 2] = "CounterStorage";
    TenantType[TenantType["TimeSeries"] = 3] = "TimeSeries";
})(TenantType || (TenantType = {}));
var checkbox;
(function (checkbox) {
    checkbox[checkbox["UnChecked"] = 0] = "UnChecked";
    checkbox[checkbox["SomeChecked"] = 1] = "SomeChecked";
    checkbox[checkbox["Checked"] = 2] = "Checked";
})(checkbox || (checkbox = {}));
var searchResults = (function () {
    function searchResults() {
    }
    return searchResults;
})();
/// <reference path="../models/dto.ts" />
define("commands/commandBase", ["require", "exports", "common/messagePublisher", "models/resources/database", "common/appUrl", "common/oauthContext", "forge/forge_custom.min", "plugins/router"], function (require, exports, messagePublisher, database, appUrl, oauthContext, forge, router) {
    /// Commands encapsulate a read or write operation to the database and support progress notifications and common AJAX related functionality.
    var commandBase = (function () {
        function commandBase() {
        }
        commandBase.prototype.execute = function () {
            throw new Error("Execute must be overridden.");
        };
        commandBase.prototype.urlEncodeArgs = function (args) {
            return appUrl.urlEncodeArgs(args);
        };
        commandBase.prototype.getTimeToAlert = function (longWait) {
            return longWait ? 60000 : 9000;
        };
        commandBase.prototype.query = function (relativeUrl, args, resource, resultsSelector, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            var ajax = this.ajax(relativeUrl, args, "GET", resource, options, timeToAlert);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    //if we fetched a database document, save the etag from the header
                    if (results.hasOwnProperty('SecuredSettings')) {
                        results['__metadata'] = { '@etag': xhr.getResponseHeader('Etag') };
                    }
                    var transformedResults = resultsSelector(results);
                    task.resolve(transformedResults);
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            }
            else {
                return ajax;
            }
        };
        commandBase.prototype.head = function (relativeUrl, args, resource, resultsSelector) {
            var ajax = this.ajax(relativeUrl, args, "HEAD", resource);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    var allHeaders = xhr.getAllResponseHeaders();
                    if (allHeaders) {
                        var headersObject = {};
                        var headersArray = xhr.getAllResponseHeaders().trim().split(/\r?\n/);
                        for (var n = 0; n < headersArray.length; n++) {
                            var keyValue = headersArray[n].split(": ");
                            if (keyValue.length == 2) {
                                //keyValue[1] = keyValue[1].replaceAll("\"", "");
                                headersObject[keyValue[0]] = keyValue[1];
                            }
                        }
                        var transformedResults = resultsSelector(headersObject);
                        task.resolve(transformedResults);
                    }
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            }
            else {
                return ajax;
            }
        };
        commandBase.prototype.put = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "PUT", resource, options, timeToAlert);
        };
        commandBase.prototype.reset = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "RESET", resource, options);
        };
        /*
         * Performs a DELETE rest call.
        */
        commandBase.prototype.del = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "DELETE", resource, options, timeToAlert);
        };
        commandBase.prototype.post = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "POST", resource, options, timeToAlert);
        };
        commandBase.prototype.patch = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "PATCH", resource, options);
        };
        commandBase.prototype.evalJs = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "EVAL", resource, options);
        };
        commandBase.prototype.ajax = function (relativeUrl, args, method, resource, options, timeToAlert) {
            var _this = this;
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            var originalArguments = arguments;
            // ContentType:
            //
            // Can't use application/json in cross-domain requests, otherwise it 
            // issues OPTIONS preflight request first, which doesn't return proper 
            // headers(e.g.Etag header, etc.)
            // 
            // So, for GETs, we issue text/plain requests, which skip the OPTIONS
            // request and goes straight for the GET request.
            var contentType = method === "GET" ?
                "text/plain; charset=utf-8" :
                "application/json; charset=utf-8";
            var defaultOptions = {
                url: appUrl.forResourceQuery(resource) + relativeUrl,
                data: args,
                dataType: "json",
                contentType: contentType,
                type: method,
                headers: undefined,
                xhr: function () {
                    var xhr = new XMLHttpRequest();
                    xhr.upload.addEventListener("progress", function (evt) {
                        if (evt.lengthComputable) {
                            var percentComplete = (evt.loaded / evt.total) * 100;
                            if (percentComplete < 100) {
                                // waiting for upload progress to complete
                                clearTimeout(commandBase.alertTimeout);
                                commandBase.alertTimeout = setTimeout(commandBase.showServerNotRespondingAlert, timeToAlert);
                            }
                            ko.postbox.publish("UploadProgress", percentComplete);
                        }
                    }, false);
                    return xhr;
                }
            };
            if (options) {
                for (var prop in options) {
                    defaultOptions[prop] = options[prop];
                }
            }
            var isBiggestTimeToAlertUpdated = timeToAlert > commandBase.biggestTimeToAlert;
            if ((commandBase.loadingCounter == 0 && timeToAlert > 0) || isBiggestTimeToAlertUpdated) {
                commandBase.biggestTimeToAlert = timeToAlert;
                clearTimeout(commandBase.splashTimerHandle);
                commandBase.splashTimerHandle = setTimeout(commandBase.showSpin, 1000, timeToAlert, isBiggestTimeToAlertUpdated);
            }
            if (oauthContext.apiKey()) {
                if (!defaultOptions.headers) {
                    var newHeaders = {};
                    defaultOptions.headers = newHeaders;
                }
                defaultOptions.headers["Has-Api-Key"] = "True";
            }
            if (oauthContext.authHeader()) {
                if (!defaultOptions.headers) {
                    var newHeaders = {};
                    defaultOptions.headers = newHeaders;
                }
                defaultOptions.headers["Authorization"] = oauthContext.authHeader();
            }
            commandBase.loadingCounter++;
            var ajaxTask = $.Deferred();
            $.ajax(defaultOptions).always(function () {
                --commandBase.loadingCounter;
                if (commandBase.loadingCounter == 0) {
                    clearTimeout(commandBase.splashTimerHandle);
                    clearTimeout(commandBase.alertTimeout);
                    commandBase.alertTimeout = 0;
                    commandBase.hideSpin();
                }
            }).done(function (results, status, xhr) {
                ajaxTask.resolve(results, status, xhr);
            }).fail(function (request, status, error) {
                var dbBeingUpdated = request.getResponseHeader("Raven-Database-Load-In-Progress");
                if (dbBeingUpdated) {
                    ajaxTask.reject(request, status, error);
                    var currentDb = appUrl.getDatabase();
                    if (currentDb != null && currentDb.name == dbBeingUpdated) {
                        router.navigate(appUrl.forUpgrade(new database(dbBeingUpdated)));
                    }
                }
                else if (request.status == 412 /* PreconditionFailed */ && oauthContext.apiKey()) {
                    _this.handleOAuth(ajaxTask, request, originalArguments);
                }
                else {
                    ajaxTask.reject(request, status, error);
                }
            });
            return ajaxTask.promise();
        };
        commandBase.prototype.handleOAuth = function (task, request, originalArguments) {
            var _this = this;
            var oauthSource = request.getResponseHeader('OAuth-Source');
            // issue request to oauth source endpoint to get RSA exponent and modulus
            $.ajax({
                type: 'POST',
                url: oauthSource,
                headers: {
                    grant_type: 'client_credentials'
                }
            }).fail(function (request, status, error) {
                if (request.status != 412 /* PreconditionFailed */) {
                    task.reject(request, status, error);
                }
                else {
                    var wwwAuth = request.getResponseHeader('WWW-Authenticate');
                    var tokens = wwwAuth.split(',');
                    var authRequest = {};
                    tokens.forEach(function (token) {
                        var eqPos = token.indexOf("=");
                        var kv = [token.substring(0, eqPos), token.substring(eqPos + 1)];
                        var m = kv[0].match(/[a-zA-Z]+$/g);
                        if (m) {
                            authRequest[m[0]] = kv[1];
                        }
                        else {
                            authRequest[kv[0]] = kv[1];
                        }
                    });
                    // form oauth request
                    var data = _this.objectToString({
                        exponent: authRequest.exponent,
                        modulus: authRequest.modulus,
                        data: _this.encryptAsymmetric(authRequest.exponent, authRequest.modulus, _this.objectToString({
                            "api key name": oauthContext.apiKeyName(),
                            "challenge": authRequest.challenge,
                            "response": _this.prepareResponse(authRequest.challenge)
                        }))
                    });
                    $.ajax({
                        type: 'POST',
                        url: oauthSource,
                        data: data,
                        headers: {
                            grant_type: 'client_credentials'
                        }
                    }).done(function (results, status, xhr) {
                        var resultsAsString = JSON.stringify(results, null, 0);
                        oauthContext.authHeader("Bearer " + resultsAsString.replace(/(\r\n|\n|\r)/gm, ""));
                        _this.retryOriginalRequest(task, originalArguments);
                    }).fail(function (request, status, error) {
                        task.reject(request, status, error);
                    });
                }
            });
        };
        commandBase.prototype.retryOriginalRequest = function (task, orignalArguments) {
            this.ajax.apply(this, orignalArguments).done(function (results, status, xhr) {
                task.resolve(results, status, xhr);
            }).fail(function (request, status, error) {
                task.reject(request, status, error);
            });
        };
        commandBase.prototype.prepareResponse = function (challenge) {
            var input = challenge + ";" + oauthContext.apiKeySecret();
            var md = forge.md.sha1.create();
            md.update(input);
            return forge.util.encode64(md.digest().getBytes());
        };
        commandBase.prototype.objectToString = function (input) {
            return $.map(input, function (value, key) { return key + "=" + value; }).join(',');
        };
        commandBase.prototype.base64ToBigInt = function (input) {
            input = forge.util.decode64(input);
            var hex = forge.util.bytesToHex(input);
            return new forge.jsbn.BigInteger(hex, 16);
        };
        commandBase.prototype.encryptAsymmetric = function (exponent, modulus, data) {
            var e = this.base64ToBigInt(exponent);
            var n = this.base64ToBigInt(modulus);
            var rsa = forge.pki.rsa;
            var publicKey = rsa.setPublicKey(n, e);
            var key = forge.random.getBytesSync(32);
            var iv = forge.random.getBytesSync(16);
            var keyAndIvEncrypted = publicKey.encrypt(key + iv, 'RSA-OAEP');
            var cipher = forge.cipher.createCipher('AES-CBC', key);
            cipher.start({ iv: iv });
            cipher.update(forge.util.createBuffer(data));
            cipher.finish();
            var encrypted = cipher.output;
            return forge.util.encode64(keyAndIvEncrypted + encrypted.data);
        };
        commandBase.showSpin = function (timeToAlert, isBiggestTimeToAlertUpdated) {
            ko.postbox.publish("LoadProgress", 2 /* warning */);
            if (commandBase.alertTimeout == 0 || isBiggestTimeToAlertUpdated) {
                clearTimeout(commandBase.alertTimeout);
                commandBase.biggestTimeToAlert = timeToAlert;
                commandBase.alertTimeout = setTimeout(commandBase.showServerNotRespondingAlert, timeToAlert);
            }
        };
        commandBase.showServerNotRespondingAlert = function () {
            ko.postbox.publish("LoadProgress", 3 /* danger */);
        };
        commandBase.hideSpin = function () {
            ko.postbox.publish("LoadProgress", null);
            $.unblockUI();
        };
        commandBase.prototype.reportInfo = function (title, details) {
            messagePublisher.reportInfo(title, details);
        };
        commandBase.prototype.reportError = function (title, details, httpStatusText) {
            messagePublisher.reportError(title, details, httpStatusText);
        };
        commandBase.prototype.reportSuccess = function (title, details) {
            messagePublisher.reportSuccess(title, details);
        };
        commandBase.prototype.reportWarning = function (title, details, httpStatusText) {
            messagePublisher.reportWarning(title, details, httpStatusText);
        };
        commandBase.prototype.reportWarningWithButton = function (title, details, buttonName, action) {
            messagePublisher.reportWarningWithButton(title, details, buttonName, action);
        };
        // TODO: better place for this?
        commandBase.ravenClientVersion = '3.5.0.0';
        commandBase.splashTimerHandle = 0;
        commandBase.alertTimeout = 0;
        commandBase.loadingCounter = 0;
        commandBase.biggestTimeToAlert = 0;
        return commandBase;
    })();
    return commandBase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/getStatusDebugTasksSummaryCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugTasksSummaryCommand = (function (_super) {
        __extends(getStatusDebugTasksSummaryCommand, _super);
        function getStatusDebugTasksSummaryCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugTasksSummaryCommand.prototype.execute = function () {
            var url = "/debug/tasks/summary";
            return this.query(url, null, this.db);
        };
        return getStatusDebugTasksSummaryCommand;
    })(commandBase);
    return getStatusDebugTasksSummaryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/getStudioConfig", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStudioConfig = (function (_super) {
        __extends(getStudioConfig, _super);
        function getStudioConfig(db) {
            _super.call(this);
            this.db = db;
        }
        getStudioConfig.prototype.execute = function () {
            var url = "/studio-tasks/config";
            var getTask = this.query(url, null, this.db);
            return getTask;
        };
        return getStudioConfig;
    })(commandBase);
    return getStudioConfig;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/saveTransformerLockModeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveTransformerLockModeCommand = (function (_super) {
        __extends(saveTransformerLockModeCommand, _super);
        /*
         * @param lockMode Should be either "Unlock", "LockedIngore", or "LockedError".
        */
        function saveTransformerLockModeCommand(transformerName, lockMode, db) {
            _super.call(this);
            this.transformerName = transformerName;
            this.lockMode = lockMode;
            this.db = db;
        }
        saveTransformerLockModeCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving " + this.transformerName + "...");
            var args = {
                op: "lockModeChange",
                mode: this.lockMode
            };
            var url = "/transformers/" + this.transformerName + this.urlEncodeArgs(args);
            var saveTask = this.post(url, JSON.stringify(args), this.db, { dataType: 'text' });
            saveTask.done(function () { return _this.reportSuccess("Saved " + _this.transformerName); });
            saveTask.fail(function (response) { return _this.reportError("Failed to save this " + _this.transformerName, response.responseText); });
            return saveTask;
        };
        return saveTransformerLockModeCommand;
    })(commandBase);
    return saveTransformerLockModeCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/forceLicenseUpdate", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var forceLicenseUpdate = (function (_super) {
        __extends(forceLicenseUpdate, _super);
        function forceLicenseUpdate() {
            _super.apply(this, arguments);
        }
        forceLicenseUpdate.prototype.execute = function () {
            var url = "/admin/license/forceUpdate";
            return this.query(url, null, appUrl.getSystemDatabase(), function (r) { return r.Success; });
        };
        return forceLicenseUpdate;
    })(commandBase);
    return forceLicenseUpdate;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/getApiKeysCommand", ["require", "exports", "commands/commandBase", "models/auth/apiKey"], function (require, exports, commandBase, apiKey) {
    var getApiKeysCommand = (function (_super) {
        __extends(getApiKeysCommand, _super);
        function getApiKeysCommand() {
            _super.apply(this, arguments);
        }
        getApiKeysCommand.prototype.execute = function () {
            var args = {
                startsWith: "Raven/ApiKeys/",
                exclude: null,
                start: 0,
                pageSize: 256
            };
            return this.query("/docs", args, null, function (dtos) { return dtos.map(function (dto) { return new apiKey(dto); }); });
        };
        return getApiKeysCommand;
    })(commandBase);
    return getApiKeysCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/getLicenseStatusCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getLicenseStatusCommand = (function (_super) {
        __extends(getLicenseStatusCommand, _super);
        function getLicenseStatusCommand() {
            _super.apply(this, arguments);
        }
        getLicenseStatusCommand.prototype.execute = function () {
            return this.query("/license/status", null);
        };
        return getLicenseStatusCommand;
    })(commandBase);
    return getLicenseStatusCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/getSingleAuthTokenCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSingleAuthTokenCommand = (function (_super) {
        __extends(getSingleAuthTokenCommand, _super);
        function getSingleAuthTokenCommand(resource, checkIfMachineAdmin) {
            if (checkIfMachineAdmin === void 0) { checkIfMachineAdmin = false; }
            _super.call(this);
            this.resource = resource;
            this.checkIfMachineAdmin = checkIfMachineAdmin;
            if (this.resource == null) {
                throw new Error("Must specify resource");
            }
        }
        getSingleAuthTokenCommand.prototype.execute = function () {
            var url = "/singleAuthToken";
            var args = null;
            if (this.checkIfMachineAdmin) {
                args = {
                    CheckIfMachineAdmin: true
                };
            }
            var getTask = this.query(url, args, this.resource);
            return getTask;
        };
        return getSingleAuthTokenCommand;
    })(commandBase);
    return getSingleAuthTokenCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/getSupportCoverageCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSupportCoverageCommand = (function (_super) {
        __extends(getSupportCoverageCommand, _super);
        function getSupportCoverageCommand() {
            _super.apply(this, arguments);
        }
        getSupportCoverageCommand.prototype.execute = function () {
            return this.query("/license/support", null);
        };
        return getSupportCoverageCommand;
    })(commandBase);
    return getSupportCoverageCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/getWindowsAuthCommand", ["require", "exports", "commands/commandBase", "models/auth/windowsAuthSetup"], function (require, exports, commandBase, windowsAuthSetup) {
    var getWindowsAuthCommand = (function (_super) {
        __extends(getWindowsAuthCommand, _super);
        function getWindowsAuthCommand() {
            _super.apply(this, arguments);
        }
        getWindowsAuthCommand.prototype.execute = function () {
            return this.query("/docs", { id: "Raven/Authorization/WindowsSettings" }, null, function (dto) { return new windowsAuthSetup(dto); });
        };
        return getWindowsAuthCommand;
    })(commandBase);
    return getWindowsAuthCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/licenseCheckConnectivityCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var licenseCheckConnectivityCommand = (function (_super) {
        __extends(licenseCheckConnectivityCommand, _super);
        function licenseCheckConnectivityCommand() {
            _super.apply(this, arguments);
        }
        licenseCheckConnectivityCommand.prototype.execute = function () {
            var url = "/admin/license/connectivity";
            return this.query(url, null, appUrl.getSystemDatabase(), function (r) { return r.Success; });
        };
        return licenseCheckConnectivityCommand;
    })(commandBase);
    return licenseCheckConnectivityCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/saveApiKeysCommand", ["require", "exports", "commands/database/documents/executeBulkDocsCommand", "common/appUrl"], function (require, exports, executeBulkDocsCommand, appUrl) {
    var saveApiKeysCommand = (function (_super) {
        __extends(saveApiKeysCommand, _super);
        function saveApiKeysCommand(apiKeys, deletedApiKeys) {
            var newApiKeysBulkDocs = apiKeys.map(function (k) { return k.toBulkDoc("PUT"); });
            var deletedApiKeysBulkDocs = deletedApiKeys.map(function (k) { return k.toBulkDoc("DELETE"); });
            _super.call(this, newApiKeysBulkDocs.concat(deletedApiKeysBulkDocs), appUrl.getSystemDatabase());
        }
        saveApiKeysCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving API keys...");
            return _super.prototype.execute.call(this)
                .fail(function (result) { return _this.reportError("Unable to save API keys.", result.responseText, result.statusText); })
                .done(function () { return _this.reportSuccess("Saved API keys."); });
        };
        return saveApiKeysCommand;
    })(executeBulkDocsCommand);
    return saveApiKeysCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/saveWindowsAuthCommand", ["require", "exports", "commands/commandBase", "commands/auth/getLicenseStatusCommand"], function (require, exports, commandBase, getLicenseStatusCommand) {
    var saveWindowsAuthCommand = (function (_super) {
        __extends(saveWindowsAuthCommand, _super);
        function saveWindowsAuthCommand(dto) {
            _super.call(this);
            this.dto = dto;
        }
        saveWindowsAuthCommand.prototype.execute = function () {
            return this.fetchLicenseStatus();
        };
        saveWindowsAuthCommand.prototype.fetchLicenseStatus = function () {
            var _this = this;
            return new getLicenseStatusCommand()
                .execute()
                .done(function (result) { return _this.handleLicenseStatus(result); });
        };
        saveWindowsAuthCommand.prototype.handleLicenseStatus = function (licenseStatus) {
            //if (licenseStatus.IsCommercial || licenseStatus.ValidCommercialLicenseSeen) {
            this.performWithCommercialLicense();
            //} else {
            //this.reportWarning("Cannot setup Windows Authentication without a valid commercial license.");
            //}
        };
        saveWindowsAuthCommand.prototype.performWithCommercialLicense = function () {
            if (this.dto.RequiredUsers.concat(this.dto.RequiredGroups).every(function (element) { return (element.Name.indexOf("\\") !== -1); })) {
                this.performSave();
            }
            else {
                this.reportWarning("Windows Authentication not saved! All names must have \"\\\" in them.");
            }
        };
        saveWindowsAuthCommand.prototype.performSave = function () {
            var _this = this;
            this.reportInfo("Saving Windows Authentication settings.");
            this.saveSetup()
                .done(function () { return _this.reportSuccess("Saved Windows Authentication settings."); })
                .fail(function (response) { return _this.reportError("Failed to save Windows Authentication settings.", response.responseText, response.statusText); });
        };
        saveWindowsAuthCommand.prototype.saveSetup = function () {
            var id = "Raven/Authorization/WindowsSettings";
            var url = "/docs/" + id;
            var putArgs = JSON.stringify(this.dto);
            return this.put(url, putArgs, null);
        };
        return saveWindowsAuthCommand;
    })(commandBase);
    return saveWindowsAuthCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/auth/verifyPrincipalCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var verifyPrincipalCommand = (function (_super) {
        __extends(verifyPrincipalCommand, _super);
        function verifyPrincipalCommand(mode, principal) {
            _super.call(this);
            this.mode = mode;
            this.principal = principal;
        }
        verifyPrincipalCommand.prototype.execute = function () {
            return this.query("/admin/verify-principal", { mode: this.mode, principal: this.principal }, appUrl.getSystemDatabase(), function (r) { return r.Valid; }, null, 25000);
        };
        return verifyPrincipalCommand;
    })(commandBase);
    return verifyPrincipalCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/backupCounterStorageCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var backupFilesystemCommand = (function (_super) {
        __extends(backupFilesystemCommand, _super);
        function backupFilesystemCommand(cs, backupLocation, updateBackupStatus, incremental) {
            _super.call(this);
            this.cs = cs;
            this.backupLocation = backupLocation;
            this.updateBackupStatus = updateBackupStatus;
            this.incremental = incremental;
        }
        backupFilesystemCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            var args = {
                BackupLocation: this.backupLocation,
                DatabaseDocument: null
            };
            this.post('/admin/backup?incremental=' + this.incremental, JSON.stringify(args), this.cs, { dataType: 'text' })
                .fail(function (response) {
                _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function () { return _this.getBackupStatus(result); });
            return result;
        };
        backupFilesystemCommand.prototype.getBackupStatus = function (result) {
            /*new getConfigurationByKeyCommand(this.cs, "Raven/Backup/Status")
                .execute()
                .fail((response: JQueryXHR) => {
                    this.reportError("Failed to fetch backup status!", response.responseText, response.statusText);
                    result.reject();
                })
                .done((backupStatusAsString: string) => {
                    var backupStatus: backupStatusDto = JSON.parse(backupStatusAsString);
                    this.updateBackupStatus(backupStatus);
                    if (backupStatus.IsRunning) {
                        setTimeout(() => this.getBackupStatus(result), 1000);
                    } else {
                        this.reportSuccess("Counter Storage backup was successfully created!");
                        result.resolve();
                    }
                });*/
        };
        return backupFilesystemCommand;
    })(commandBase);
    return backupFilesystemCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/deleteCountersByGroupCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteCountersByGroupCommand = (function (_super) {
        __extends(deleteCountersByGroupCommand, _super);
        function deleteCountersByGroupCommand(group, cs) {
            _super.call(this);
            this.group = group;
            this.cs = cs;
        }
        deleteCountersByGroupCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                groupName: this.group.isAllGroupsGroup ? "" : this.group.name
            };
            var url = "/delete-by-group/" + this.urlEncodeArgs(args);
            var deleteTask = this.del(url, null, this.cs, { dataType: undefined }, (9000 * this.group.countersCount()) / 2);
            deleteTask.done(function (numOfDeletedCounters) { return _this.reportSuccess("Successfully deleted " + numOfDeletedCounters + " counters in '" + _this.group.name + "'"); });
            deleteTask.fail(function (response) { return _this.reportError("Failed to delete '" + _this.group.name + "'", response.responseText, response.statusText); });
            return deleteTask;
        };
        return deleteCountersByGroupCommand;
    })(commandBase);
    return deleteCountersByGroupCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/deleteCountersCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteCountersCommand = (function (_super) {
        __extends(deleteCountersCommand, _super);
        function deleteCountersCommand(groupAndNames, cs) {
            _super.call(this);
            this.groupAndNames = groupAndNames;
            this.cs = cs;
        }
        deleteCountersCommand.prototype.execute = function () {
            var _this = this;
            var deletionTasks = new Array();
            ;
            for (var i = 0; i < this.groupAndNames.length; i++) {
                var deleteCommand = this.deleteCounter(this.groupAndNames[i]);
                deletionTasks.push(deleteCommand);
            }
            var successMessage = "Deleted " + (this.groupAndNames.length > 1 ? +this.groupAndNames.length + " counters" : this.getCounterDeleteText(this.groupAndNames[0]));
            var failMessage = "Failed to delete " + (this.groupAndNames.length > 1 ? this.groupAndNames.length + " counters" : this.getCounterDeleteText(this.groupAndNames[0]));
            var combinedTask = $.when.apply($, deletionTasks)
                .done(function () { return _this.reportSuccess(successMessage); })
                .fail(function (response) { return _this.reportError(failMessage, response.responseText, response.statusText); });
            return combinedTask;
        };
        deleteCountersCommand.prototype.deleteCounter = function (groupAndName) {
            var url = "/delete/" + this.urlEncodeArgs(groupAndName);
            return this.del(url, null, this.cs, { dataType: undefined }, 9000);
        };
        deleteCountersCommand.prototype.getCounterDeleteText = function (groupAndName) {
            return "counter name: " + groupAndName.counterName + ", group: " + groupAndName.groupName;
        };
        return deleteCountersCommand;
    })(commandBase);
    return deleteCountersCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/deleteCounterStorageCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteCounterStorageCommand = (function (_super) {
        __extends(deleteCounterStorageCommand, _super);
        function deleteCounterStorageCommand(counterStorageName, isHardDelete) {
            _super.call(this);
            this.counterStorageName = counterStorageName;
            this.isHardDelete = isHardDelete;
        }
        deleteCounterStorageCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Deleting " + this.counterStorageName + "...");
            var url = "/counterstorage/admin/" + encodeURIComponent(this.counterStorageName) + "?hard-delete=" + this.isHardDelete;
            var deleteTask = this.del(url, null, null, { dataType: undefined });
            deleteTask.done(function () { return _this.reportSuccess("Deleted " + _this.counterStorageName); });
            deleteTask.fail(function (response) { return _this.reportError("Failed to delete counter storage", response.responseText, response.statusText); });
            return deleteTask;
        };
        return deleteCounterStorageCommand;
    })(commandBase);
    return deleteCounterStorageCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCounterCommand", ["require", "exports", "commands/commandBase", "models/counter/counter"], function (require, exports, commandBase, counter) {
    var getCounterCommand = (function (_super) {
        __extends(getCounterCommand, _super);
        function getCounterCommand(cs, groupName, counterName) {
            _super.call(this);
            this.cs = cs;
            this.groupName = groupName;
            this.counterName = counterName;
        }
        getCounterCommand.prototype.execute = function () {
            var args = {
                groupName: this.groupName,
                counterName: this.counterName
            };
            var url = "/getCounter";
            var selector = function (dto) { return new counter(dto); };
            return this.query(url, args, this.cs, selector);
        };
        return getCounterCommand;
    })(commandBase);
    return getCounterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCounterGroupsCommand", ["require", "exports", "commands/commandBase", "models/counter/counterGroup"], function (require, exports, commandBase, counterGroup) {
    var getCounterGroupsCommand = (function (_super) {
        __extends(getCounterGroupsCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getCounterGroupsCommand(cs, skip, take) {
            _super.call(this);
            this.cs = cs;
            this.skip = skip;
            this.take = take;
        }
        getCounterGroupsCommand.prototype.execute = function () {
            var _this = this;
            var selector = function (groups) { return groups.map(function (g) { return counterGroup.fromDto(g, _this.cs); }); };
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            return this.query("/groups", args, this.cs, selector);
        };
        return getCounterGroupsCommand;
    })(commandBase);
    return getCounterGroupsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCountersCommand", ["require", "exports", "commands/commandBase", "models/counter/counterSummary", "common/pagedResultSet"], function (require, exports, commandBase, counterSummary, pagedResultSet) {
    var getCountersCommand = (function (_super) {
        __extends(getCountersCommand, _super);
        /**
        * @param counterStorage - the counter storage that is being used
        * @param skip - number of entries to skip
        * @param take - number of entries to take
        * @param counterGroupName - the counter group to take the entries from
        */
        function getCountersCommand(cs, skip, take, group) {
            if (group === void 0) { group = null; }
            _super.call(this);
            this.cs = cs;
            this.skip = skip;
            this.take = take;
            this.group = group;
        }
        getCountersCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                skip: this.skip,
                take: this.take,
                group: this.group
            };
            var url = "/counters";
            var doneTask = $.Deferred();
            var selector = function (dtos) { return dtos.map(function (d) { return new counterSummary(d, _this.group == null); }); };
            var task = this.query(url, args, this.cs, selector);
            task.done(function (summaries) { return doneTask.resolve(new pagedResultSet(summaries, summaries.length)); });
            task.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        return getCountersCommand;
    })(commandBase);
    return getCountersCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCounterStorageReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getCounterStorageReplicationCommand = (function (_super) {
        __extends(getCounterStorageReplicationCommand, _super);
        function getCounterStorageReplicationCommand(counterStorage, reportRefreshProgress) {
            if (reportRefreshProgress === void 0) { reportRefreshProgress = false; }
            _super.call(this);
            this.counterStorage = counterStorage;
            this.reportRefreshProgress = reportRefreshProgress;
            if (!counterStorage) {
                throw new Error("Must specify counter storage");
            }
        }
        getCounterStorageReplicationCommand.prototype.execute = function () {
            var _this = this;
            var url = "/replications/get";
            var getTask = this.query(url, null, this.counterStorage);
            if (this.reportRefreshProgress) {
                getTask.done(function () { return _this.reportSuccess("Replication Destionations of '" + _this.counterStorage.name + "' were successfully refreshed!"); });
                getTask.fail(function (response) { return _this.reportWarning("There are no saved replication destinations on the server!", response.responseText, response.statusText); });
            }
            return getTask;
        };
        return getCounterStorageReplicationCommand;
    })(commandBase);
    return getCounterStorageReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCounterStoragesCommand", ["require", "exports", "models/counter/counterStorage", "commands/commandBase"], function (require, exports, counterStorage, commandBase) {
    var getCounterStoragesCommand = (function (_super) {
        __extends(getCounterStoragesCommand, _super);
        function getCounterStoragesCommand() {
            _super.apply(this, arguments);
        }
        getCounterStoragesCommand.prototype.execute = function () {
            var args = {
                skip: 0,
                take: 1024,
                getAdditionalData: true
            };
            var url = "/cs";
            var resultsSelector = function (counterStorages) {
                return counterStorages.map(function (cs) { return new counterStorage(cs.Name, cs.IsAdminCurrentTenant, cs.Disabled, cs.Bundles); });
            };
            return this.query(url, args, null, resultsSelector);
        };
        return getCounterStoragesCommand;
    })(commandBase);
    return getCounterStoragesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/getCounterStorageStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getCounterStorageStatsCommand = (function (_super) {
        __extends(getCounterStorageStatsCommand, _super);
        function getCounterStorageStatsCommand(cs, longWait) {
            if (longWait === void 0) { longWait = false; }
            _super.call(this);
            this.cs = cs;
            this.longWait = longWait;
        }
        getCounterStorageStatsCommand.prototype.execute = function () {
            var url = "/stats";
            return this.query(url, null, this.cs, null, this.getTimeToAlert(this.longWait));
        };
        return getCounterStorageStatsCommand;
    })(commandBase);
    return getCounterStorageStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/resetCounterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resetCounterCommand = (function (_super) {
        __extends(resetCounterCommand, _super);
        function resetCounterCommand(cs, groupName, counterName) {
            _super.call(this);
            this.cs = cs;
            this.groupName = groupName;
            this.counterName = counterName;
        }
        resetCounterCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                groupName: this.groupName,
                counterName: this.counterName
            };
            var url = "/reset" + this.urlEncodeArgs(args);
            var action = this.post(url, args, this.cs, { dataType: undefined });
            action.done(function () { return _this.reportSuccess("Successfully reset counter!"); });
            action.fail(function (response) { return _this.reportError("Failed to reset counter!", response.responseText, response.statusText); });
            return action;
        };
        return resetCounterCommand;
    })(commandBase);
    return resetCounterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/saveCounterStorageReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveCounterStorageReplicationCommand = (function (_super) {
        __extends(saveCounterStorageReplicationCommand, _super);
        function saveCounterStorageReplicationCommand(dto, counterStorage) {
            _super.call(this);
            this.dto = dto;
            this.counterStorage = counterStorage;
        }
        saveCounterStorageReplicationCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving counters replication");
            return this.saveSetup()
                .done(function () { return _this.reportSuccess("Saved counters replication"); })
                .fail(function (response) { return _this.reportError("Failed to save counters replication", response.responseText, response.statusText); });
        };
        saveCounterStorageReplicationCommand.prototype.saveSetup = function () {
            var url = "/replications/save";
            var putArgs = JSON.stringify(this.dto);
            return this.post(url, putArgs, this.counterStorage, { dataType: undefined });
        };
        return saveCounterStorageReplicationCommand;
    })(commandBase);
    return saveCounterStorageReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/counter/updateCounterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var updateCounterCommand = (function (_super) {
        __extends(updateCounterCommand, _super);
        function updateCounterCommand(cs, group, counterName, delta, isNew) {
            _super.call(this);
            this.cs = cs;
            this.group = group;
            this.counterName = counterName;
            this.delta = delta;
            this.isNew = isNew;
        }
        updateCounterCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                groupName: this.group,
                counterName: this.counterName,
                delta: this.delta
            };
            var url = "/change/" + this.urlEncodeArgs(args);
            var action = this.post(url, null, this.cs, { dataType: undefined });
            var successMessage = this.isNew ? "Successfully created a new counter!" : "Successfully updated a counter!";
            action.done(function () { return _this.reportSuccess(successMessage); });
            var failMessage = this.isNew ? "Failed to create a new counter!" : "Successfully to update counter!";
            action.fail(function (response) { return _this.reportError(failMessage, response.responseText, response.statusText); });
            return action;
        };
        return updateCounterCommand;
    })(commandBase);
    return updateCounterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/changeNodeVotingModeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var ChangeNodeVotingModeCommand = (function (_super) {
        __extends(ChangeNodeVotingModeCommand, _super);
        function ChangeNodeVotingModeCommand(db, connectionInfo) {
            _super.call(this);
            this.db = db;
            this.connectionInfo = connectionInfo;
        }
        ChangeNodeVotingModeCommand.prototype.execute = function () {
            var _this = this;
            return this.post("/admin/cluster/changeVotingMode?isVoting=" + !(this.connectionInfo.IsNoneVoter), ko.toJSON(this.connectionInfo), this.db, { dataType: 'text' })
                .done(function () { return _this.reportSuccess("Node was promoted to voter."); })
                .fail(function (response) { return _this.reportError("Failed to promote node to voter", response.responseText, response.statusText); });
        };
        return ChangeNodeVotingModeCommand;
    })(commandBase);
    return ChangeNodeVotingModeCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/extendRaftClusterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var extendRaftClusterCommand = (function (_super) {
        __extends(extendRaftClusterCommand, _super);
        function extendRaftClusterCommand(db, connectionInfo, create, force) {
            _super.call(this);
            this.db = db;
            this.connectionInfo = connectionInfo;
            this.create = create;
            this.force = force;
        }
        extendRaftClusterCommand.prototype.execute = function () {
            var _this = this;
            if (this.create) {
                return this.post("/admin/cluster/create", ko.toJSON(this.connectionInfo), this.db, { dataType: undefined })
                    .done(function () { return _this.reportSuccess("Cluster was created"); })
                    .fail(function (response) { return _this.reportError("Failed to create cluster", response.responseText, response.statusText); });
            }
            else {
                var url = "/admin/cluster/join";
                if (this.force) {
                    url += "?force=true";
                }
                return this.post(url, ko.toJSON(this.connectionInfo), this.db, { dataType: undefined })
                    .done(function () { return _this.reportSuccess("Server was added to cluster."); })
                    .fail(function (response) { return _this.reportError("Unable to add server to cluster", response.responseText, response.statusText); });
            }
        };
        return extendRaftClusterCommand;
    })(commandBase);
    return extendRaftClusterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/getClusterNodesStatusCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getClusterNodesStatusCommand = (function (_super) {
        __extends(getClusterNodesStatusCommand, _super);
        function getClusterNodesStatusCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getClusterNodesStatusCommand.prototype.execute = function () {
            return this.query("/cluster/status", null, this.db);
        };
        return getClusterNodesStatusCommand;
    })(commandBase);
    return getClusterNodesStatusCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/getClusterTopologyCommand", ["require", "exports", "commands/commandBase", "models/database/replication/topology"], function (require, exports, commandBase, topology) {
    var getClusterTopologyCommand = (function (_super) {
        __extends(getClusterTopologyCommand, _super);
        function getClusterTopologyCommand(ownerDb) {
            _super.call(this);
            this.ownerDb = ownerDb;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getClusterTopologyCommand.prototype.execute = function () {
            var task = $.Deferred();
            this.query("/cluster/topology", null, this.ownerDb, function (x) { return new topology(x); })
                .done(function (result) {
                task.resolve(result);
            })
                .fail(function (result) {
                if (result.status === 200 && !result.responseText) {
                    task.resolve(null);
                }
                else {
                    task.reject(result);
                }
            });
            return task;
        };
        return getClusterTopologyCommand;
    })(commandBase);
    return getClusterTopologyCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/initializeNewClusterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var initializeNewClusterCommand = (function (_super) {
        __extends(initializeNewClusterCommand, _super);
        function initializeNewClusterCommand(db) {
            _super.call(this);
            this.db = db;
        }
        initializeNewClusterCommand.prototype.execute = function () {
            var _this = this;
            return this.patch("/admin/cluster/initialize-new-cluster", null, this.db)
                .done(function () { return _this.reportSuccess("New cluster has been initialized"); })
                .fail(function (response) { return _this.reportError("Unable to initialize new cluster", response.responseText, response.statusText); });
        };
        return initializeNewClusterCommand;
    })(commandBase);
    return initializeNewClusterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/leaveRaftClusterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var leaveRaftClusterCommand = (function (_super) {
        __extends(leaveRaftClusterCommand, _super);
        function leaveRaftClusterCommand(db, connectionInfo) {
            _super.call(this);
            this.db = db;
            this.connectionInfo = connectionInfo;
        }
        leaveRaftClusterCommand.prototype.execute = function () {
            var _this = this;
            return this.query("/admin/cluster/leave", { name: this.connectionInfo.Name }, this.db)
                .done(function () { return _this.reportSuccess("Node was removed from cluster."); })
                .fail(function (response) { return _this.reportError("Failed to remove node from cluster", response.responseText, response.statusText); });
        };
        return leaveRaftClusterCommand;
    })(commandBase);
    return leaveRaftClusterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/removeClusteringCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var removeClusteringCommand = (function (_super) {
        __extends(removeClusteringCommand, _super);
        function removeClusteringCommand(db) {
            _super.call(this);
            this.db = db;
        }
        removeClusteringCommand.prototype.execute = function () {
            var _this = this;
            return this.patch("/admin/cluster/remove-clustering", null, this.db)
                .done(function () { return _this.reportSuccess("Removed cluster entirely and performed cleanup."); })
                .fail(function (response) { return _this.reportError("Unable to remove cluster and perform cleanup", response.responseText, response.statusText); });
        };
        return removeClusteringCommand;
    })(commandBase);
    return removeClusteringCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/saveClusterConfigurationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveClusterConfigurationCommand = (function (_super) {
        __extends(saveClusterConfigurationCommand, _super);
        function saveClusterConfigurationCommand(dto, db) {
            _super.call(this);
            this.dto = dto;
            this.db = db;
        }
        saveClusterConfigurationCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Cluster configuration.");
            return this.saveSetup()
                .done(function () { return _this.reportSuccess("Saved Cluster configuration."); })
                .fail(function (response) { return _this.reportError("Failed to save cluster configuration.", response.responseText, response.statusText); });
        };
        saveClusterConfigurationCommand.prototype.saveSetup = function () {
            var putArgs = JSON.stringify(this.dto);
            return this.put("/admin/cluster/commands/configuration", putArgs, this.db, { dataType: undefined });
        };
        return saveClusterConfigurationCommand;
    })(commandBase);
    return saveClusterConfigurationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/cluster/updateRaftClusterCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var updateRaftClusterCommand = (function (_super) {
        __extends(updateRaftClusterCommand, _super);
        function updateRaftClusterCommand(db, connectionInfo) {
            _super.call(this);
            this.db = db;
            this.connectionInfo = connectionInfo;
        }
        updateRaftClusterCommand.prototype.execute = function () {
            var _this = this;
            return this.post("/admin/cluster/update", ko.toJSON(this.connectionInfo), this.db, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Configuration was updated"); })
                .fail(function (response) { return _this.reportError("Failed to update node configuration", response.responseText, response.statusText); });
        };
        return updateRaftClusterCommand;
    })(commandBase);
    return updateRaftClusterCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/adminLogsConfigureCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var adminLogsConfigureCommand = (function (_super) {
        __extends(adminLogsConfigureCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function adminLogsConfigureCommand(ownerDb, logConfig, eventsId) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.logConfig = logConfig;
            this.eventsId = eventsId;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        adminLogsConfigureCommand.prototype.execute = function () {
            var args = {
                'watch-category': $.map(this.logConfig, function (item) { return item.category + ":" + item.level + ":" + (item.includeStackTrace ? "watch-stack" : "no-watch-stack"); }),
                id: this.eventsId
            };
            var url = "/admin/logs/configure" + this.urlEncodeArgs(args);
            return this.query(url, null, this.ownerDb);
        };
        return adminLogsConfigureCommand;
    })(commandBase);
    return adminLogsConfigureCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getDebugMetricsCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var getDebugMetricsCommand = (function (_super) {
        __extends(getDebugMetricsCommand, _super);
        function getDebugMetricsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getDebugMetricsCommand.prototype.execute = function () {
            var url = this.getQueryUrlFragment();
            return this.query(url, null, this.db);
        };
        getDebugMetricsCommand.prototype.getQueryUrl = function () {
            return appUrl.forResourceQuery(this.db) + this.getQueryUrlFragment();
        };
        getDebugMetricsCommand.prototype.getQueryUrlFragment = function () {
            return "/debug/metrics";
        };
        return getDebugMetricsCommand;
    })(commandBase);
    return getDebugMetricsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getDocRefsCommand", ["require", "exports", "common/pagedResultSet", "commands/commandBase"], function (require, exports, pagedResultSet, commandBase) {
    var getDocRefsCommand = (function (_super) {
        __extends(getDocRefsCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getDocRefsCommand(ownerDb, docId, skip, take) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.docId = docId;
            this.skip = skip;
            this.take = take;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getDocRefsCommand.prototype.execute = function () {
            var args = {
                id: this.docId,
                start: this.skip,
                pageSize: this.take
            };
            var url = "/debug/docrefs";
            var docRefsTask = $.Deferred();
            this.query(url, args, this.ownerDb).
                fail(function (response) { return docRefsTask.reject(response); }).
                done(function (docRefs) {
                var items = $.map(docRefs.Results, function (r) {
                    return {
                        getId: function () {
                            return r;
                        },
                        getDocumentPropertyNames: function () {
                            return ["Id"];
                        }
                    };
                });
                var resultsSet = new pagedResultSet(items, docRefs.TotalCount);
                docRefsTask.resolve(resultsSet);
            });
            return docRefsTask;
        };
        return getDocRefsCommand;
    })(commandBase);
    return getDocRefsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getFilteredOutIndexStatsCommand", ["require", "exports", "commands/commandBase", "d3/d3"], function (require, exports, commandBase, d3) {
    var getFilteredOutIndexStatsCommand = (function (_super) {
        __extends(getFilteredOutIndexStatsCommand, _super);
        function getFilteredOutIndexStatsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getFilteredOutIndexStatsCommand.prototype.execute = function () {
            var url = "/debug/filtered-out-indexes";
            var parser = d3.time.format.iso;
            return this.query(url, null, this.db, function (result) {
                result.map(function (item) {
                    item.TimestampParsed = parser.parse(item.Timestamp);
                });
                return result;
            });
        };
        return getFilteredOutIndexStatsCommand;
    })(commandBase);
    return getFilteredOutIndexStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getIdentitiesCommand", ["require", "exports", "common/pagedResultSet", "commands/commandBase"], function (require, exports, pagedResultSet, commandBase) {
    var getIdentitiesCommand = (function (_super) {
        __extends(getIdentitiesCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getIdentitiesCommand(ownerDb, skip, take) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.skip = skip;
            this.take = take;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getIdentitiesCommand.prototype.execute = function () {
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            var url = "/debug/identities";
            var identitiesTask = $.Deferred();
            this.query(url, args, this.ownerDb).
                fail(function (response) { return identitiesTask.reject(response); }).
                done(function (identities) {
                var items = $.map(identities.Identities, function (r) {
                    return {
                        getId: function () { return r.Key; },
                        getUrl: function () { return r.Key; },
                        'Value': r.Value,
                        'Key': r.Key,
                        getDocumentPropertyNames: function () { return ["Key", "Value"]; }
                    };
                });
                console.log(items); //TODO: dlete me
                var resultsSet = new pagedResultSet(items, identities.TotalCount);
                identitiesTask.resolve(resultsSet);
            });
            return identitiesTask;
        };
        return getIdentitiesCommand;
    })(commandBase);
    return getIdentitiesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getIndexingBatchStatsCommand", ["require", "exports", "commands/commandBase", "d3/d3", "common/appUrl"], function (require, exports, commandBase, d3, appUrl) {
    var getIndexingBatchStatsCommand = (function (_super) {
        __extends(getIndexingBatchStatsCommand, _super);
        function getIndexingBatchStatsCommand(db, lastId) {
            if (lastId === void 0) { lastId = 0; }
            _super.call(this);
            this.db = db;
            this.lastId = lastId;
        }
        getIndexingBatchStatsCommand.prototype.execute = function () {
            var url = "/debug/indexing-batch-stats";
            var entryMapping = function (entry) {
                return {
                    indexName: entry.key,
                    stats: entry.value
                };
            };
            var args = { lastId: this.lastId };
            var parser = d3.time.format.iso;
            return this.query(url, args, this.db, function (result) {
                var mappedResult = result.map(function (item) {
                    return {
                        Id: item.Id,
                        BatchType: item.BatchType,
                        IndexesToWorkOn: item.IndexesToWorkOn,
                        TotalDocumentCount: item.TotalDocumentCount,
                        TotalDocumentSize: item.TotalDocumentSize,
                        StartedAt: item.StartedAt,
                        StartedAtDate: parser.parse(item.StartedAt),
                        TotalDurationMs: item.TotalDurationMs,
                        PerfStats: d3.map(item.PerformanceStats).entries().map(entryMapping)
                    };
                });
                mappedResult.forEach(function (r) {
                    r.PerfStats.forEach(function (p) {
                        p.stats.Operations.filter(function (x) { return !("Name" in x); }).forEach(function (o) {
                            o.BatchedOperations.forEach(function (b) {
                                b.Operations.forEach(function (x) {
                                    x.ParallelParent = b;
                                });
                                b.Parent = o;
                            });
                        });
                    });
                });
                return mappedResult;
            });
        };
        getIndexingBatchStatsCommand.prototype.getQueryUrl = function () {
            return appUrl.forResourceQuery(this.db) + this.getQueryUrlFragment();
        };
        getIndexingBatchStatsCommand.prototype.getQueryUrlFragment = function () {
            return "/debug/indexing-perf-stats-with-timings";
        };
        return getIndexingBatchStatsCommand;
    })(commandBase);
    return getIndexingBatchStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getIndexingPerfStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexingPerfStatsCommand = (function (_super) {
        __extends(getIndexingPerfStatsCommand, _super);
        function getIndexingPerfStatsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getIndexingPerfStatsCommand.prototype.execute = function () {
            var url = "/debug/indexing-perf-stats-with-timings";
            return this.query(url, null, this.db);
        };
        return getIndexingPerfStatsCommand;
    })(commandBase);
    return getIndexingPerfStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getIndexMergeSuggestionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexMergeSuggestionsCommand = (function (_super) {
        __extends(getIndexMergeSuggestionsCommand, _super);
        function getIndexMergeSuggestionsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getIndexMergeSuggestionsCommand.prototype.execute = function () {
            var url = "/debug/suggest-index-merge";
            return this.query(url, null, this.db);
        };
        return getIndexMergeSuggestionsCommand;
    })(commandBase);
    return getIndexMergeSuggestionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getInfoPackage", ["require", "exports", "commands/commandBase", 'jszip/jszip-utils.min', 'common/appUrl', "d3/d3", "commands/auth/getSingleAuthTokenCommand"], function (require, exports, commandBase, zipUtils, appUrl, d3, getSingleAuthTokenCommand) {
    var getInfoPackage = (function (_super) {
        __extends(getInfoPackage, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getInfoPackage(db, withStackTrace) {
            _super.call(this);
            this.db = db;
            this.withStackTrace = withStackTrace;
        }
        getInfoPackage.prototype.execute = function () {
            var _this = this;
            var task = $.Deferred();
            var getTokenTask = new getSingleAuthTokenCommand(this.db).execute();
            getTokenTask
                .done(function (tokenObject) {
                var token = tokenObject.Token;
                var url = appUrl.forResourceQuery(_this.db) + (_this.db.isSystem ? '/admin/debug/info-package' : '/debug/info-package');
                url += '?singleUseAuthToken=' + token;
                if (_this.withStackTrace && _this.db.isSystem) {
                    url += "&stacktrace=true";
                }
                var now = d3.time.format("%Y-%m-%d_%H:%M:%S")(new Date());
                var filename = _this.db.isSystem ? "Admin-Debug-Info-" + now + ".zip" : "Debug-Info-" + _this.db.name + "-" + now + ".zip";
                zipUtils.getBinaryContent(url, function (err, data) {
                    if (err) {
                        task.reject(err);
                    }
                    else {
                        task.resolve(data, filename);
                    }
                });
            })
                .fail(function (response) {
                _this.reportError("Failed to get token!", response.responseText, response.statusText);
                task.reject(response);
            });
            return task.promise();
        };
        return getInfoPackage;
    })(commandBase);
    return getInfoPackage;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getLogsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getLogsCommand = (function (_super) {
        __extends(getLogsCommand, _super);
        function getLogsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getLogsCommand.prototype.execute = function () {
            var url = "/logs";
            return this.query(url, null, this.db);
        };
        return getLogsCommand;
    })(commandBase);
    return getLogsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getPluginsInfoCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getLogsCommand = (function (_super) {
        __extends(getLogsCommand, _super);
        function getLogsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getLogsCommand.prototype.execute = function () {
            var url = "/debug/plugins";
            return this.query(url, null, this.db);
        };
        return getLogsCommand;
    })(commandBase);
    return getLogsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getReducingBatchStatsCommand", ["require", "exports", "commands/commandBase", "d3/d3"], function (require, exports, commandBase, d3) {
    var getReducingBatchStatsCommand = (function (_super) {
        __extends(getReducingBatchStatsCommand, _super);
        function getReducingBatchStatsCommand(db, lastId) {
            _super.call(this);
            this.db = db;
            this.lastId = lastId;
        }
        getReducingBatchStatsCommand.prototype.execute = function () {
            var url = "/debug/reducing-batch-stats";
            var args = { lastId: this.lastId };
            var inlinePerfStats = function (entry) {
                var result = [];
                d3.map(entry).entries().forEach(function (e) {
                    e.value.forEach(function (s) {
                        result.push({
                            indexName: e.key,
                            stats: s
                        });
                    });
                });
                return result;
            };
            var parser = d3.time.format.iso;
            return this.query(url, args, this.db, function (result) {
                var mappedResult = result.map(function (item) {
                    return {
                        Id: item.Id,
                        IndexesToWorkOn: item.IndexesToWorkOn,
                        TotalDurationMs: item.TotalDurationMs,
                        StartedAt: item.StartedAt,
                        StartedAtDate: parser.parse(item.StartedAt),
                        PerfStats: inlinePerfStats(item.PerformanceStats),
                        TimeSinceFirstReduceInBatchCompletedMs: item.TimeSinceFirstReduceInBatchCompletedMs
                    };
                });
                mappedResult.forEach(function (r) {
                    r.PerfStats.forEach(function (s) {
                        s.stats.LevelStats.forEach(function (l) {
                            l.Operations.filter(function (x) { return !("Name" in x); }).forEach(function (o) {
                                o.BatchedOperations.forEach(function (b) {
                                    b.Operations.forEach(function (x) {
                                        x.ParallelParent = b;
                                    });
                                    b.Parent = o;
                                });
                            });
                            l.parent = s;
                        });
                        s.parent = r;
                    });
                });
                return mappedResult;
            });
        };
        return getReducingBatchStatsCommand;
    })(commandBase);
    return getReducingBatchStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getReplicationPerfStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getReplicationPerfStatsCommand = (function (_super) {
        __extends(getReplicationPerfStatsCommand, _super);
        function getReplicationPerfStatsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getReplicationPerfStatsCommand.prototype.execute = function () {
            var url = "/debug/replication-perf-stats";
            return this.query(url, null, this.db);
        };
        return getReplicationPerfStatsCommand;
    })(commandBase);
    return getReplicationPerfStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getRequestTracingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getRequestTracingCommand = (function (_super) {
        __extends(getRequestTracingCommand, _super);
        function getRequestTracingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getRequestTracingCommand.prototype.execute = function () {
            var url = "/debug/request-tracing";
            return this.query(url, null, this.db);
        };
        return getRequestTracingCommand;
    })(commandBase);
    return getRequestTracingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getSlowDocCountsCommand", ["require", "exports", "commands/commandBase", "models/database/debug/debugDocumentStats"], function (require, exports, commandBase, debugDocumentStats) {
    var getSlowDocCountsCommand = (function (_super) {
        __extends(getSlowDocCountsCommand, _super);
        function getSlowDocCountsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getSlowDocCountsCommand.prototype.execute = function () {
            var url = "/debug/sl0w-d0c-c0unts";
            var resultSelector = function (result) { return new debugDocumentStats(result); };
            return this.query(url, null, this.db, resultSelector);
        };
        return getSlowDocCountsCommand;
    })(commandBase);
    return getSlowDocCountsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getSqlReplicationPerfStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSqlReplicationPerfStatsCommand = (function (_super) {
        __extends(getSqlReplicationPerfStatsCommand, _super);
        function getSqlReplicationPerfStatsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getSqlReplicationPerfStatsCommand.prototype.execute = function () {
            var url = "/debug/sql-replication-perf-stats";
            return this.query(url, null, this.db);
        };
        return getSqlReplicationPerfStatsCommand;
    })(commandBase);
    return getSqlReplicationPerfStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugChangesCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugChangesCommand = (function (_super) {
        __extends(getStatusDebugChangesCommand, _super);
        function getStatusDebugChangesCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugChangesCommand.prototype.execute = function () {
            var url = "/debug/changes";
            return this.query(url, null, this.db);
        };
        return getStatusDebugChangesCommand;
    })(commandBase);
    return getStatusDebugChangesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugConfigCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugConfigCommand = (function (_super) {
        __extends(getStatusDebugConfigCommand, _super);
        function getStatusDebugConfigCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugConfigCommand.prototype.execute = function () {
            var url = "/debug/config";
            return this.query(url, null, this.db);
        };
        return getStatusDebugConfigCommand;
    })(commandBase);
    return getStatusDebugConfigCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugCurrentlyIndexingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugCurrentlyIndexingCommand = (function (_super) {
        __extends(getStatusDebugCurrentlyIndexingCommand, _super);
        function getStatusDebugCurrentlyIndexingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugCurrentlyIndexingCommand.prototype.execute = function () {
            var url = "/debug/currently-indexing";
            return this.query(url, null, this.db);
        };
        return getStatusDebugCurrentlyIndexingCommand;
    })(commandBase);
    return getStatusDebugCurrentlyIndexingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugDataSubscriptionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugDataSubscriptionsCommand = (function (_super) {
        __extends(getStatusDebugDataSubscriptionsCommand, _super);
        function getStatusDebugDataSubscriptionsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugDataSubscriptionsCommand.prototype.execute = function () {
            var url = "/debug/subscriptions";
            return this.query(url, null, this.db);
        };
        return getStatusDebugDataSubscriptionsCommand;
    })(commandBase);
    return getStatusDebugDataSubscriptionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugIndexFieldsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugIndexFieldsCommand = (function (_super) {
        __extends(getStatusDebugIndexFieldsCommand, _super);
        function getStatusDebugIndexFieldsCommand(db, indexStr) {
            _super.call(this);
            this.db = db;
            this.indexStr = indexStr;
        }
        getStatusDebugIndexFieldsCommand.prototype.execute = function () {
            var url = "/debug/index-fields";
            return this.post(url, this.indexStr, this.db);
        };
        return getStatusDebugIndexFieldsCommand;
    })(commandBase);
    return getStatusDebugIndexFieldsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugQueriesCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugQueriesCommand = (function (_super) {
        __extends(getStatusDebugQueriesCommand, _super);
        function getStatusDebugQueriesCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugQueriesCommand.prototype.execute = function () {
            var url = "/debug/queries";
            var resultSelector = function (result) { return $.map(result, function (queries, key) { return { "IndexName": key, "Queries": queries }; }); };
            return this.query(url, null, this.db, resultSelector);
        };
        return getStatusDebugQueriesCommand;
    })(commandBase);
    return getStatusDebugQueriesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugRoutesCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var getStatusDebugConfigCommand = (function (_super) {
        __extends(getStatusDebugConfigCommand, _super);
        function getStatusDebugConfigCommand() {
            _super.call(this);
        }
        getStatusDebugConfigCommand.prototype.execute = function () {
            var url = "/debug/routes";
            var db = appUrl.getSystemDatabase();
            return this.query(url, null, db);
        };
        return getStatusDebugConfigCommand;
    })(commandBase);
    return getStatusDebugConfigCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusDebugSqlReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusDebugSqlReplicationCommand = (function (_super) {
        __extends(getStatusDebugSqlReplicationCommand, _super);
        function getStatusDebugSqlReplicationCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusDebugSqlReplicationCommand.prototype.execute = function () {
            var url = "/debug/sql-replication-stats";
            return this.query(url, null, this.db);
        };
        return getStatusDebugSqlReplicationCommand;
    })(commandBase);
    return getStatusDebugSqlReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusStorageBreakdownCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusStorageBreakdownCommand = (function (_super) {
        __extends(getStatusStorageBreakdownCommand, _super);
        function getStatusStorageBreakdownCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusStorageBreakdownCommand.prototype.execute = function () {
            var url = "/admin/detailed-storage-breakdown";
            return this.query(url, null, this.db);
        };
        return getStatusStorageBreakdownCommand;
    })(commandBase);
    return getStatusStorageBreakdownCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getStatusStorageOnDiskCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getStatusStorageOnDiskCommand = (function (_super) {
        __extends(getStatusStorageOnDiskCommand, _super);
        function getStatusStorageOnDiskCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getStatusStorageOnDiskCommand.prototype.execute = function () {
            var url = "/database/storage/sizes";
            return this.query(url, null, this.db);
        };
        return getStatusStorageOnDiskCommand;
    })(commandBase);
    return getStatusStorageOnDiskCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/getUserInfoCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getUserInfoCommand = (function (_super) {
        __extends(getUserInfoCommand, _super);
        function getUserInfoCommand(db) {
            if (db === void 0) { db = null; }
            _super.call(this);
            this.db = db;
        }
        getUserInfoCommand.prototype.execute = function () {
            var url = "/debug/user-info";
            return this.query(url, null, this.db);
        };
        return getUserInfoCommand;
    })(commandBase);
    return getUserInfoCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/ioTestCommand", ["require", "exports", "commands/commandBase", "common/appUrl", 'commands/operations/getOperationStatusCommand'], function (require, exports, commandBase, appUrl, getOperationStatusCommand) {
    var ioTestCommand = (function (_super) {
        __extends(ioTestCommand, _super);
        function ioTestCommand(db, testParameters, onStatus) {
            _super.call(this);
            this.db = db;
            this.testParameters = testParameters;
            this.onStatus = onStatus;
            this.operationIdTask = $.Deferred();
        }
        ioTestCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var url = '/admin/ioTest';
            this.post(url, ko.toJSON(this.testParameters), null)
                .done(function (result) {
                _this.operationIdTask.resolve(result.OperationId);
                _this.monitorIoTest(promise, result.OperationId);
            })
                .fail(function (response) {
                _this.reportError("Failed to start disk IO test!", response.responseText, response.statusText);
                promise.reject();
            });
            return promise;
        };
        ioTestCommand.prototype.monitorIoTest = function (parentPromise, operationId) {
            var _this = this;
            new getOperationStatusCommand(appUrl.getSystemDatabase(), operationId)
                .execute()
                .done(function (result) {
                if (result.Completed) {
                    if (result.Faulted || result.Canceled) {
                        _this.reportError("Failed to perform disk IO test!", result.State.Error);
                        parentPromise.reject();
                    }
                    else {
                        _this.reportSuccess("Disk IO test completed");
                        parentPromise.resolve();
                    }
                }
                else {
                    _this.onStatus(result.State);
                    setTimeout(function () { return _this.monitorIoTest(parentPromise, operationId); }, 500);
                }
            });
        };
        return ioTestCommand;
    })(commandBase);
    return ioTestCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/queryIndexDebugAfterReduceCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var queryIndexDebugAfterReduceCommand = (function (_super) {
        __extends(queryIndexDebugAfterReduceCommand, _super);
        function queryIndexDebugAfterReduceCommand(indexName, db, reduceKeys) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.reduceKeys = reduceKeys;
        }
        queryIndexDebugAfterReduceCommand.prototype.execute = function () {
            var args = {
                debug: "entries",
                reduceKeys: this.reduceKeys
            };
            var url = "/indexes/" + this.indexName;
            return this.query(url, args, this.db, function (r) { return r.Results; });
        };
        return queryIndexDebugAfterReduceCommand;
    })(commandBase);
    return queryIndexDebugAfterReduceCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/queryIndexDebugDocsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var queryIndexDebugDocsCommand = (function (_super) {
        __extends(queryIndexDebugDocsCommand, _super);
        function queryIndexDebugDocsCommand(indexName, db, startsWith, skip, take) {
            if (skip === void 0) { skip = 0; }
            if (take === void 0) { take = 256; }
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.startsWith = startsWith;
            this.skip = skip;
            this.take = take;
        }
        queryIndexDebugDocsCommand.prototype.execute = function () {
            var args = {
                start: this.skip,
                pageSize: this.take,
                debug: "docs",
                startsWith: this.startsWith
            };
            var url = "/indexes/" + this.indexName;
            return this.query(url, args, this.db, function (r) { return r.Results; });
        };
        return queryIndexDebugDocsCommand;
    })(commandBase);
    return queryIndexDebugDocsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/queryIndexDebugMapCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var queryIndexDebugMapCommand = (function (_super) {
        __extends(queryIndexDebugMapCommand, _super);
        function queryIndexDebugMapCommand(indexName, db, args, skip, take) {
            if (skip === void 0) { skip = 0; }
            if (take === void 0) { take = 256; }
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.args = args;
            this.skip = skip;
            this.take = take;
        }
        queryIndexDebugMapCommand.prototype.execute = function () {
            var baseArgs = {
                start: this.skip,
                pageSize: this.take,
                debug: "map"
            };
            var url = "/indexes/" + this.indexName;
            return this.query(url, $.extend({}, baseArgs, this.args), this.db, function (r) { return r.Results; });
        };
        return queryIndexDebugMapCommand;
    })(commandBase);
    return queryIndexDebugMapCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/debug/queryIndexDebugReduceCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var queryIndexDebugReduceCommand = (function (_super) {
        __extends(queryIndexDebugReduceCommand, _super);
        function queryIndexDebugReduceCommand(indexName, db, level, key, skip, take) {
            if (skip === void 0) { skip = 0; }
            if (take === void 0) { take = 256; }
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.level = level;
            this.key = key;
            this.skip = skip;
            this.take = take;
        }
        queryIndexDebugReduceCommand.prototype.execute = function () {
            var args = {
                start: this.skip,
                pageSize: this.take,
                debug: "reduce",
                key: this.key,
                level: this.level
            };
            var url = "/indexes/" + this.indexName;
            return this.query(url, args, this.db, function (r) { return r.Results; });
        };
        return queryIndexDebugReduceCommand;
    })(commandBase);
    return queryIndexDebugReduceCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/deleteCollectionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteCollectionCommand = (function (_super) {
        __extends(deleteCollectionCommand, _super);
        function deleteCollectionCommand(collectionName, db) {
            _super.call(this);
            this.collectionName = collectionName;
            this.db = db;
            this.displayCollectionName = (collectionName === "*") ? "All Documents" : collectionName;
        }
        deleteCollectionCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Deleting " + this.displayCollectionName);
            var url = "/bulk_docs/Raven/DocumentsByEntityName";
            var urlParams = "?query=Tag%3A" + encodeURIComponent(this.collectionName) + "&allowStale=true";
            var deleteTask = this.del(url + urlParams, null, this.db);
            // deletion is made asynchronically so we infom user about operation start - not about actual completion. 
            deleteTask.done(function () { return _this.reportSuccess("Scheduled deletion of " + _this.displayCollectionName); });
            deleteTask.fail(function (response) { return _this.reportError("Failed to delete " + _this.displayCollectionName, response.responseText, response.statusText); });
            return deleteTask;
        };
        return deleteCollectionCommand;
    })(commandBase);
    return deleteCollectionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/deleteDocsMatchingQueryCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteDocsMatchingQueryCommand = (function (_super) {
        __extends(deleteDocsMatchingQueryCommand, _super);
        function deleteDocsMatchingQueryCommand(indexName, queryText, db) {
            _super.call(this);
            this.indexName = indexName;
            this.queryText = queryText;
            this.db = db;
        }
        deleteDocsMatchingQueryCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Deleting docs matching query...");
            var args = {
                query: this.queryText,
                pageSize: 128,
                allowStale: false
            };
            var url = "/bulk_docs/" + this.indexName + this.urlEncodeArgs(args);
            var task = this.del(url, null, this.db);
            task.done(function () { return _this.reportSuccess("Docs deleted"); });
            task.fail(function (response) { return _this.reportError("Error deleting docs matching query", response.responseText, response.statusText); });
            return task;
        };
        return deleteDocsMatchingQueryCommand;
    })(commandBase);
    return deleteDocsMatchingQueryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/deleteDocumentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteDocumentCommand = (function (_super) {
        __extends(deleteDocumentCommand, _super);
        function deleteDocumentCommand(docId, db) {
            _super.call(this);
            this.docId = docId;
            this.db = db;
        }
        deleteDocumentCommand.prototype.execute = function () {
            return this.del('/docs/' + this.docId, null, this.db);
        };
        return deleteDocumentCommand;
    })(commandBase);
    return deleteDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/deleteDocumentsCommand", ["require", "exports", "commands/database/documents/executeBulkDocsCommand"], function (require, exports, executeBulkDocsCommand) {
    var deleteDocumentsCommand = (function (_super) {
        __extends(deleteDocumentsCommand, _super);
        function deleteDocumentsCommand(docIds, db) {
            var bulkDocs = docIds.map(function (id) { return deleteDocumentsCommand.createDeleteDocument(id); });
            _super.call(this, bulkDocs, db);
        }
        deleteDocumentsCommand.prototype.execute = function () {
            var _this = this;
            var docCount = this.docs.length;
            var docsDescription = docCount === 1 ? this.docs[0].Key : docCount + " docs";
            var alertInfoTitle = "Deleting " + docsDescription + "...";
            this.reportInfo(alertInfoTitle);
            var deleteTask = _super.prototype.execute.call(this);
            deleteTask.done(function () { return _this.reportSuccess("Deleted " + docsDescription); });
            deleteTask.fail(function (response) { return _this.reportError("Failed to delete " + docsDescription, response.responseText, response.statusText); });
            return deleteTask;
        };
        deleteDocumentsCommand.createDeleteDocument = function (id) {
            return {
                Key: id,
                Method: "DELETE",
                Etag: null,
                AdditionalData: null
            };
        };
        return deleteDocumentsCommand;
    })(executeBulkDocsCommand);
    return deleteDocumentsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/executeBulkDocsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var executeBulkDocsCommand = (function (_super) {
        __extends(executeBulkDocsCommand, _super);
        function executeBulkDocsCommand(docs, db) {
            _super.call(this);
            this.docs = docs;
            this.db = db;
        }
        executeBulkDocsCommand.prototype.execute = function () {
            return this.post("/bulk_docs", JSON.stringify(this.docs), this.db);
        };
        return executeBulkDocsCommand;
    })(commandBase);
    return executeBulkDocsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/generateClassCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var generateClassCommand = (function (_super) {
        __extends(generateClassCommand, _super);
        function generateClassCommand(db, docId, lang) {
            _super.call(this);
            this.db = db;
            this.docId = docId;
            this.lang = lang;
            this.isGeneratingCode = ko.observable(true);
        }
        generateClassCommand.prototype.execute = function () {
            var _this = this;
            var url = "/generate/code";
            var args = {
                docId: this.docId,
                lang: this.lang
            };
            return this.query(url, args, this.db)
                .done(function (result) {
                return result.Code;
            })
                .fail(function (response) { return _this.reportError("Failed to create class code", response.responseText, response.statusText); });
        };
        generateClassCommand.prototype.activateGenerateCode = function () {
            this.isGeneratingCode(true);
        };
        return generateClassCommand;
    })(commandBase);
    return generateClassCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getCollectionsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/collection", "commands/database/index/getIndexTermsCommand", "commands/database/documents/getCollectionsCountCommand", "commands/database/studio/getCachedCollectionsCount"], function (require, exports, commandBase, collection, getIndexTermsCommand, getCollectionsCountCommand, getCachedCollectionsCount) {
    var getCollectionsCommand = (function (_super) {
        __extends(getCollectionsCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getCollectionsCommand(ownerDb, previousValues, lastQueryDate) {
            if (previousValues === void 0) { previousValues = []; }
            if (lastQueryDate === void 0) { lastQueryDate = null; }
            _super.call(this);
            this.ownerDb = ownerDb;
            this.previousValues = previousValues;
            this.lastQueryDate = lastQueryDate;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getCollectionsCommand.prototype.execute = function () {
            var _this = this;
            var finalResult = $.Deferred();
            this.runQuery()
                .fail(function (xhr) { return _this.createSystemIndexAndTryAgain(finalResult, xhr); })
                .done(function (results) { return finalResult.resolve(results); });
            return finalResult;
        };
        getCollectionsCommand.prototype.runQuery = function () {
            var _this = this;
            var task = $.Deferred();
            new getIndexTermsCommand("Raven/DocumentsByEntityName", "Tag", this.ownerDb)
                .execute()
                .done(function (terms, status, xhr) {
                if (_this.previousValues.length > 0 && _this.lastQueryDate()) {
                    var collections = _this.previousValues.filter(function (v) { return !v.isAllDocuments && !v.isSystemDocuments; }).map(function (v) { return new collection(v.name, _this.ownerDb, v.documentCount()); });
                    // apply optimalization fetch only changes, based on last query time. 
                    new getCachedCollectionsCount(_this.ownerDb, _this.lastQueryDate())
                        .execute()
                        .done(function (counts) {
                        counts.forEach(function (countData) {
                            var c = collections.first(function (p) { return p.name == countData.CollectionName; });
                            if (c) {
                                c.documentCount(countData.Count);
                            }
                            else {
                                collections.push(new collection(countData.CollectionName, _this.ownerDb, countData.Count));
                            }
                        });
                        task.resolve(collections);
                    })
                        .fail(function (result) { return task.reject(result); });
                }
                else {
                    var collections = terms.map(function (term) { return new collection(term, _this.ownerDb, 0); });
                    new getCollectionsCountCommand(collections, _this.ownerDb)
                        .execute()
                        .done(function (result) { return task.resolve(result); })
                        .fail(function (result) { return task.reject(result); });
                }
                if (_this.lastQueryDate != null) {
                    _this.lastQueryDate(xhr.getResponseHeader('Date'));
                }
            })
                .fail(function (response) {
                _this.createSystemIndexAndTryAgain(task, response);
            });
            return task;
        };
        getCollectionsCommand.prototype.createSystemIndexAndTryAgain = function (deferred, originalReadError) {
            // Most often, failure to get the collections is due to the missing system index, Raven/DocumentsByEntityName.
            // This appears to be new behavior as of 3.0: Raven doesn't create this index automatically for the system database.
            var _this = this;
            // Calling silverlight/ensureStartup creates the system index.
            this.query("/silverlight/ensureStartup", null, this.ownerDb)
                .done(function () { return _this.retryQuery(deferred, originalReadError); })
                .fail(function () { return _this.onErrorReadingCollections(deferred, originalReadError); });
        };
        getCollectionsCommand.prototype.retryQuery = function (deferred, originalReadError) {
            var _this = this;
            this.runQuery()
                .fail(function () { return _this.onErrorReadingCollections(deferred, originalReadError); })
                .done(function (results) { return deferred.resolve(results); });
        };
        getCollectionsCommand.prototype.onErrorReadingCollections = function (deferred, xhr) {
            this.reportError("Failed to read collections", xhr.responseText, xhr.statusText);
            deferred.reject(xhr);
        };
        return getCollectionsCommand;
    })(commandBase);
    return getCollectionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getCollectionsCountCommand", ["require", "exports", "commands/commandBase", "common/queryUtil"], function (require, exports, commandBase, queryUtil) {
    var getCollectionsCountCommand = (function (_super) {
        __extends(getCollectionsCountCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getCollectionsCountCommand(collections, ownerDb) {
            _super.call(this);
            this.collections = collections;
            this.ownerDb = ownerDb;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getCollectionsCountCommand.prototype.execute = function () {
            var _this = this;
            var task = $.Deferred();
            var requests = this.collections.map(function (collection) {
                return {
                    Url: "/indexes/Raven/DocumentsByEntityName",
                    Headers: {},
                    Query: "?&query=Tag:" + queryUtil.escapeTerm(collection.name) + "&pageSize=0"
                };
            });
            this.post("/multi_get?parallel=yes", JSON.stringify(requests), this.ownerDb, null, 0)
                .done(function (result) {
                for (var i = 0; i < _this.collections.length; i++) {
                    _this.collections[i].documentCount(result[i].Result.TotalResults);
                }
                task.resolve(_this.collections);
            })
                .fail(function (response) {
                _this.reportError("Failed to fetch collections count", response.responseText, response.statusText);
                task.reject(response);
            });
            return task;
        };
        return getCollectionsCountCommand;
    })(commandBase);
    return getCollectionsCountCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getCustomColumnsCommand", ["require", "exports", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, getDocumentWithMetadataCommand) {
    var getCustomColumnsCommand = (function (_super) {
        __extends(getCustomColumnsCommand, _super);
        function getCustomColumnsCommand(db, docName) {
            _super.call(this, docName, db, true);
            this.docName = docName;
        }
        getCustomColumnsCommand.prototype.execute = function () {
            return _super.prototype.execute.call(this);
        };
        getCustomColumnsCommand.forIndex = function (indexName, db) {
            return new getCustomColumnsCommand(db, getCustomColumnsCommand.docsPrefix + "Index/" + indexName);
        };
        getCustomColumnsCommand.forCollection = function (collection, db) {
            return new getCustomColumnsCommand(db, getCustomColumnsCommand.docsPrefix + "Collection/" + collection);
        };
        getCustomColumnsCommand.forAllDocuments = function (db) {
            return new getCustomColumnsCommand(db, getCustomColumnsCommand.docsPrefix + "AllDocuments");
        };
        getCustomColumnsCommand.docsPrefix = "Raven/Studio/Columns/";
        return getCustomColumnsCommand;
    })(getDocumentWithMetadataCommand);
    return getCustomColumnsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getCustomFunctionsCommand", ["require", "exports", "models/database/documents/customFunctions", "commands/commandBase"], function (require, exports, customFunctions, commandBase) {
    var getCustomFunctionsCommand = (function (_super) {
        __extends(getCustomFunctionsCommand, _super);
        function getCustomFunctionsCommand(db, global) {
            if (global === void 0) { global = false; }
            _super.call(this);
            this.db = db;
            this.global = global;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        getCustomFunctionsCommand.prototype.execute = function () {
            var resultsSelector = function (queryResult) { return new customFunctions(queryResult); };
            var url = this.global ? "/docs/Raven/Global/Javascript/Functions" : "/docs/Raven/Javascript/Functions";
            return this.query(url, null, this.db, resultsSelector);
        };
        return getCustomFunctionsCommand;
    })(commandBase);
    return getCustomFunctionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getDocumentsByEntityNameCommand", ["require", "exports", "commands/commandBase", "models/database/documents/collectionInfo", "common/pagedResultSet", "common/queryUtil"], function (require, exports, commandBase, collectionInfo, pagedResultSet, queryUtil) {
    var getDocumentsByEntityNameCommand = (function (_super) {
        __extends(getDocumentsByEntityNameCommand, _super);
        function getDocumentsByEntityNameCommand(collection, skip, take) {
            _super.call(this);
            this.collection = collection;
            this.skip = skip;
            this.take = take;
        }
        getDocumentsByEntityNameCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                query: "Tag:" + queryUtil.escapeTerm(this.collection.name),
                start: this.skip,
                pageSize: this.take,
                sort: "-LastModifiedTicks"
            };
            var resultsSelector = function (dto) { return new collectionInfo(dto); };
            var url = "/indexes/Raven/DocumentsByEntityName";
            var documentsTask = $.Deferred();
            this.query(url, args, this.collection.ownerDatabase, resultsSelector)
                .fail(function (response) {
                if (response.status == 500 /* InternalServerError */) {
                    // old style index, probably, try again without the LastModifiedTicks
                    args.sort = "-LastModified";
                    _this.query(url, args, _this.collection.ownerDatabase, resultsSelector)
                        .fail(function (_) { return documentsTask.reject(); })
                        .then(function (collection) {
                        var items = collection.results;
                        var resultSet = new pagedResultSet(items, collection.totalResults);
                        documentsTask.resolve(resultSet);
                    });
                    return;
                }
                documentsTask.reject();
            })
                .then(function (collection) {
                var items = collection.results;
                var resultSet = new pagedResultSet(items, collection.totalResults);
                documentsTask.resolve(resultSet);
            });
            return documentsTask;
        };
        return getDocumentsByEntityNameCommand;
    })(commandBase);
    return getDocumentsByEntityNameCommand;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getDocumentsMetadataByIDPrefixCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getDocumentsMetadataByIDPrefixCommand = (function (_super) {
        __extends(getDocumentsMetadataByIDPrefixCommand, _super);
        function getDocumentsMetadataByIDPrefixCommand(prefix, resultsAmount, db) {
            _super.call(this);
            this.prefix = prefix;
            this.resultsAmount = resultsAmount;
            this.db = db;
        }
        getDocumentsMetadataByIDPrefixCommand.prototype.execute = function () {
            var url = '/docs';
            var args = {
                'startsWith': this.prefix,
                'exclude': null,
                'start': 0,
                'pageSize': this.resultsAmount,
                'metadata-only': true
            };
            return this.query(url, args, this.db);
        };
        return getDocumentsMetadataByIDPrefixCommand;
    })(commandBase);
    return getDocumentsMetadataByIDPrefixCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getDocumentsPreviewCommand", ["require", "exports", "commands/commandBase", "models/database/documents/collectionInfo", "common/pagedResultSet"], function (require, exports, commandBase, collectionInfo, pagedResultSet) {
    var getDocumentsPreviewCommand = (function (_super) {
        __extends(getDocumentsPreviewCommand, _super);
        function getDocumentsPreviewCommand(database, skip, take, collectionName, bindings) {
            _super.call(this);
            this.database = database;
            this.skip = skip;
            this.take = take;
            this.collectionName = collectionName;
            this.bindings = bindings;
        }
        getDocumentsPreviewCommand.prototype.execute = function () {
            var args = {
                collection: this.collectionName,
                start: this.skip,
                pageSize: this.take,
                binding: this.bindings
            };
            var resultsSelector = function (dto) {
                var collection = new collectionInfo(dto);
                var items = collection.results;
                return new pagedResultSet(items, collection.totalResults);
            };
            var url = "/doc-preview";
            return this.query(url, args, this.database, resultsSelector);
        };
        return getDocumentsPreviewCommand;
    })(commandBase);
    return getDocumentsPreviewCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getDocumentWithMetadataCommand", ["require", "exports", "commands/commandBase", "models/database/documents/document"], function (require, exports, commandBase, document) {
    var getDocumentWithMetadataCommand = (function (_super) {
        __extends(getDocumentWithMetadataCommand, _super);
        function getDocumentWithMetadataCommand(id, db, shouldResolveNotFoundAsNull) {
            _super.call(this);
            this.id = id;
            this.db = db;
            if (!id) {
                throw new Error("Must specify ID");
            }
            if (!db) {
                throw new Error("Must specify database");
            }
            this.shouldResolveNotFoundAsNull = shouldResolveNotFoundAsNull || false;
        }
        getDocumentWithMetadataCommand.prototype.execute = function () {
            var _this = this;
            // Executing /queries will return the doc with the metadata. 
            // We can do a GET call to /docs/<id>, however, it returns the metadata only as headers, 
            // which can have some issues when querying via CORS.
            var documentResult = $.Deferred();
            var postResult = this.post("/queries", JSON.stringify([this.id]), this.db);
            postResult.fail(function (xhr) { return documentResult.fail(xhr); });
            postResult.done(function (queryResult) {
                if (queryResult.Results.length === 0) {
                    if (_this.shouldResolveNotFoundAsNull) {
                        documentResult.resolve(null);
                    }
                    else {
                        documentResult.reject("Unable to find document with ID " + _this.id);
                    }
                }
                else {
                    documentResult.resolve(new document(queryResult.Results[0]));
                }
            });
            return documentResult;
        };
        return getDocumentWithMetadataCommand;
    })(commandBase);
    return getDocumentWithMetadataCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getPeriodicExportSetupCommand", ["require", "exports", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, getDocumentWithMetadataCommand) {
    var getPeriodicExportSetupCommand = (function (_super) {
        __extends(getPeriodicExportSetupCommand, _super);
        function getPeriodicExportSetupCommand(db, getGlobalConfig) {
            if (getGlobalConfig === void 0) { getGlobalConfig = false; }
            _super.call(this, getGlobalConfig ? "Raven/Global/Backup/Periodic/Setup" : "Raven/Backup/Periodic/Setup", db);
        }
        getPeriodicExportSetupCommand.prototype.execute = function () {
            return _super.prototype.execute.call(this);
        };
        return getPeriodicExportSetupCommand;
    })(getDocumentWithMetadataCommand);
    return getPeriodicExportSetupCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getServerPrefixForHiLoCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getServerPrefixForHiLoCommand = (function (_super) {
        __extends(getServerPrefixForHiLoCommand, _super);
        function getServerPrefixForHiLoCommand(db) {
            _super.call(this);
            this.db = db;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        getServerPrefixForHiLoCommand.prototype.execute = function () {
            var resultsSelector = function (queryResult) { return queryResult.ServerPrefix; };
            var url = "/docs/Raven/ServerPrefixForHilo";
            return this.query(url, null, this.db, resultsSelector);
        };
        return getServerPrefixForHiLoCommand;
    })(commandBase);
    return getServerPrefixForHiLoCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getSystemDocumentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSystemDocumentCommand = (function (_super) {
        __extends(getSystemDocumentCommand, _super);
        function getSystemDocumentCommand(id) {
            _super.call(this);
            this.id = id;
        }
        getSystemDocumentCommand.prototype.execute = function () {
            var deferred = $.Deferred();
            var url = "/docs/" + this.id;
            var docQuery = this.query(url, null, null);
            docQuery.done(function (dto) { return deferred.resolve(dto); });
            docQuery.fail(function (response) { return deferred.reject(response); });
            return deferred;
        };
        return getSystemDocumentCommand;
    })(commandBase);
    return getSystemDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getSystemDocumentsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/document", "common/pagedResultSet"], function (require, exports, commandBase, document, pagedResultSet) {
    var getSystemDocumentsCommand = (function (_super) {
        __extends(getSystemDocumentsCommand, _super);
        function getSystemDocumentsCommand(db, skip, take, totalResultCount) {
            _super.call(this);
            this.db = db;
            this.skip = skip;
            this.take = take;
            this.totalResultCount = totalResultCount;
        }
        getSystemDocumentsCommand.prototype.execute = function () {
            var _this = this;
            // To get system docs, we just call /docs asking for docs that start with "Raven/".
            // Unfortunately, this returns a plain array; it doesn't tell how many *total* system docs there are.
            // This means we can't really do proper paging. 
            var args = {
                startsWith: "Raven/",
                exclude: null,
                start: this.skip,
                pageSize: this.take
            };
            var deferred = $.Deferred();
            var docsQuery = this.query("/docs", args, this.db, function (dtos) { return dtos.map(function (dto) { return new document(dto); }); });
            docsQuery.done(function (results) {
                var documents = results.map(function (dto) { return new document(dto); });
                //var totalResultCount = documents.length; 
                var resultSet = new pagedResultSet(documents, _this.totalResultCount);
                deferred.resolve(resultSet);
            });
            docsQuery.fail(function (response) { return deferred.reject(response); });
            return deferred;
        };
        return getSystemDocumentsCommand;
    })(commandBase);
    return getSystemDocumentsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/getVersioningsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/versioningEntry"], function (require, exports, commandBase, versioningEntry) {
    var getVersioningsCommand = (function (_super) {
        __extends(getVersioningsCommand, _super);
        function getVersioningsCommand(db, getGlobalConfig) {
            if (getGlobalConfig === void 0) { getGlobalConfig = false; }
            _super.call(this);
            this.db = db;
            this.getGlobalConfig = getGlobalConfig;
        }
        getVersioningsCommand.prototype.execute = function () {
            var documentResult = $.Deferred();
            var resultsSelector = function (result) {
                return result.map(function (dto) { return new versioningEntry(dto, true); });
            };
            var url = "/docs";
            var args = {
                startsWith: this.getGlobalConfig ? "Raven/Global/Versioning" : "Raven/Versioning",
                start: 0,
                pageSize: 1024
            };
            var postResult = this.query(url, args, this.db, resultsSelector);
            postResult.fail(function (xhr) { return documentResult.fail(xhr); });
            postResult.done(function (entries) { return documentResult.resolve(entries); });
            return documentResult;
        };
        return getVersioningsCommand;
    })(commandBase);
    return getVersioningsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/saveBulkOfDocuments", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveBulkOfDocuments = (function (_super) {
        __extends(saveBulkOfDocuments, _super);
        function saveBulkOfDocuments(dataTypes, bulkDocuments, db) {
            _super.call(this);
            this.dataTypes = dataTypes;
            this.bulkDocuments = bulkDocuments;
            this.db = db;
        }
        // performs bulk save of an array of objects 
        saveBulkOfDocuments.prototype.execute = function () {
            var _this = this;
            var saveBulkTask = this.post("/bulk_docs", ko.toJSON(this.bulkDocuments), this.db);
            this.reportInfo("Performing bulk save of " + this.dataTypes);
            saveBulkTask.done(function () { return _this.reportSuccess("Saved all bulk of documents"); });
            saveBulkTask.fail(function (response) { return _this.reportError("Failed to save bulk of documents", response.responseText, response.statusText); });
            return saveBulkTask;
        };
        return saveBulkOfDocuments;
    })(commandBase);
    return saveBulkOfDocuments;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/saveCustomFunctionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveCustomFunctionsCommand = (function (_super) {
        __extends(saveCustomFunctionsCommand, _super);
        function saveCustomFunctionsCommand(db, toSave, global) {
            if (global === void 0) { global = false; }
            _super.call(this);
            this.db = db;
            this.toSave = toSave;
            this.global = global;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        saveCustomFunctionsCommand.prototype.validateCustomFunctions = function (document) {
            return this.post("/studio-tasks/validateCustomFunctions", document, this.db, { dataType: 'text' });
        };
        saveCustomFunctionsCommand.prototype.execute = function () {
            var _this = this;
            var args = JSON.stringify(this.toSave.toDto());
            return this.validateCustomFunctions(args)
                .fail(function (response) { return _this.reportError("Failed to validate custom functions!", response.responseText, response.statusText); })
                .then(function () {
                var url = _this.global ? "/docs/Raven/Global/Javascript/Functions" : "/docs/Raven/Javascript/Functions";
                var saveTask = _this.put(url, args, _this.db, null);
                saveTask.done(function () { return _this.reportSuccess("Custom functions saved!"); });
                saveTask.fail(function (response) { return _this.reportError("Failed to save custom functions!", response.responseText, response.statusText); });
                return saveTask;
            });
        };
        return saveCustomFunctionsCommand;
    })(commandBase);
    return saveCustomFunctionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/saveDocumentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveDocumentCommand = (function (_super) {
        __extends(saveDocumentCommand, _super);
        function saveDocumentCommand(id, document, db, reportSaveProgress) {
            if (reportSaveProgress === void 0) { reportSaveProgress = true; }
            _super.call(this);
            this.id = id;
            this.document = document;
            this.db = db;
            this.reportSaveProgress = reportSaveProgress;
        }
        saveDocumentCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportSaveProgress) {
                this.reportInfo("Saving " + this.id + "...");
            }
            var customHeaders = {
                'Raven-Client-Version': commandBase.ravenClientVersion,
            };
            var metadataDto = this.document.__metadata.toDto();
            var etag = metadataDto["@etag"];
            if (!etag)
                etag = "";
            for (var key in metadataDto) {
                if (key.indexOf('@') !== 0) {
                    customHeaders[key] = metadataDto[key];
                }
            }
            var jQueryOptions = {
                headers: customHeaders
            };
            var documentDto = this.document.toDto(false);
            var commands = [];
            if (etag !== "") {
                commands.push({
                    Method: "PUT",
                    Key: this.id,
                    Document: documentDto,
                    Metadata: metadataDto,
                    Etag: etag
                });
            }
            else {
                commands.push({
                    Method: "PUT",
                    Key: this.id,
                    Document: documentDto,
                    Metadata: metadataDto
                });
            }
            var args = ko.toJSON(commands);
            var url = "/bulk_docs";
            var saveTask = this.post(url, args, this.db, jQueryOptions);
            if (this.reportSaveProgress) {
                saveTask.done(function () { return _this.reportSuccess("Saved " + _this.id); });
                saveTask.fail(function (response) { return _this.reportError("Failed to save " + _this.id, response.responseText, response.statusText); });
            }
            return saveTask;
        };
        return saveDocumentCommand;
    })(commandBase);
    return saveDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/saveScriptedIndexesCommand", ["require", "exports", "commands/database/documents/executeBulkDocsCommand"], function (require, exports, executeBulkDocsCommand) {
    var saveScriptedIndexesCommand = (function (_super) {
        __extends(saveScriptedIndexesCommand, _super);
        function saveScriptedIndexesCommand(scriptedIndexes, db) {
            _super.call(this, scriptedIndexes.map(function (idx) { return (idx.isMarkedToDelete()) ? idx.toBulkDoc("DELETE") : idx.toBulkDoc("PUT"); }), db);
            this.scriptedIndexes = scriptedIndexes;
        }
        saveScriptedIndexesCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Scripted Index...");
            return _super.prototype.execute.call(this)
                .fail(function (result) { return _this.reportError("Unable to save Scripted Index.", result.responseText, result.statusText); })
                .done(function () { return _this.reportSuccess("Saved Scripted Index."); });
        };
        return saveScriptedIndexesCommand;
    })(executeBulkDocsCommand);
    return saveScriptedIndexesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/saveVersioningCommand", ["require", "exports", "models/database/documents/document", "commands/commandBase", "commands/database/documents/saveBulkOfDocuments"], function (require, exports, document, commandBase, saveBulkOfDocuments) {
    var saveVersioningCommand = (function (_super) {
        __extends(saveVersioningCommand, _super);
        function saveVersioningCommand(db, versioningEntries, removeEntries, globalConfig) {
            if (removeEntries === void 0) { removeEntries = []; }
            if (globalConfig === void 0) { globalConfig = false; }
            _super.call(this);
            this.db = db;
            this.versioningEntries = versioningEntries;
            this.removeEntries = removeEntries;
            this.globalConfig = globalConfig;
        }
        saveVersioningCommand.prototype.execute = function () {
            var _this = this;
            var commands = [];
            this.versioningEntries.forEach(function (dto) {
                var entry = new document(dto);
                commands.push({
                    Key: (_this.globalConfig ? "Raven/Global/Versioning/" : "Raven/Versioning/") + entry["Id"],
                    Method: "PUT",
                    Document: entry.toDto(false),
                    Metadata: entry.__metadata.toDto(),
                    Etag: entry.__metadata.etag
                });
            });
            this.removeEntries.forEach(function (dto) {
                var entry = new document(dto);
                commands.push({
                    Key: (_this.globalConfig ? "Raven/Global/Versioning/" : "Raven/Versioning/") + entry["Id"],
                    Method: "DELETE",
                    Etag: entry.__metadata.etag
                });
            });
            var saveTask = new saveBulkOfDocuments("versioning", commands, this.db).execute();
            return saveTask;
        };
        return saveVersioningCommand;
    })(commandBase);
    return saveVersioningCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/updateServerPrefixHiLoCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/deleteDocumentCommand"], function (require, exports, commandBase, deleteDocumentCommand) {
    var updateServerPrefixHiLoCommand = (function (_super) {
        __extends(updateServerPrefixHiLoCommand, _super);
        function updateServerPrefixHiLoCommand(serverPrefix, db) {
            _super.call(this);
            this.serverPrefix = serverPrefix;
            this.db = db;
        }
        updateServerPrefixHiLoCommand.prototype.execute = function () {
            var _this = this;
            if (this.serverPrefix) {
                var doc = {
                    ServerPrefix: this.serverPrefix
                };
                return this.put("/docs/Raven/ServerPrefixForHilo", JSON.stringify(doc), this.db)
                    .done(function () { return _this.reportSuccess("Saved ServerPrefix configuration"); })
                    .fail(function (response) { return _this.reportError("Failed to save ServerPrefix configuration", response.responseText, response.statusText); });
            }
            else {
                return new deleteDocumentCommand("Raven/ServerPrefixForHilo", this.db)
                    .execute()
                    .done(function () { return _this.reportSuccess("Saved ServerPrefix configuration"); })
                    .fail(function (response) { return _this.reportError("Failed to save ServerPrefix configuration", response.responseText, response.statusText); });
            }
        };
        return updateServerPrefixHiLoCommand;
    })(commandBase);
    return updateServerPrefixHiLoCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/documents/verifyDocumentsIDsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var verifyDocumentsIDsCommand = (function (_super) {
        __extends(verifyDocumentsIDsCommand, _super);
        function verifyDocumentsIDsCommand(docIDs, db, queryLocalStorage, storeResultsInLocalStorage) {
            _super.call(this);
            this.docIDs = docIDs;
            this.db = db;
            this.queryLocalStorage = queryLocalStorage;
            this.storeResultsInLocalStorage = storeResultsInLocalStorage;
            if (!docIDs) {
                throw new Error("Must specify IDs");
            }
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        verifyDocumentsIDsCommand.prototype.execute = function () {
            var _this = this;
            var verifyResult = $.Deferred();
            var verifiedIDs = [];
            // if required to check with locally stored document ids first, remove known non existing documet ids first and confirm verified ids later
            if (this.queryLocalStorage === true) {
                if (!!verifyDocumentsIDsCommand.InvalidIDsLocal && verifyDocumentsIDsCommand.InvalidIDsLocal.length > 0) {
                    this.docIDs.removeAll(verifyDocumentsIDsCommand.InvalidIDsLocal);
                }
                if (!!verifyDocumentsIDsCommand.IDsLocalStorage && verifyDocumentsIDsCommand.IDsLocalStorage.length > 0) {
                    this.docIDs.forEach(function (curId) {
                        if (!!verifyDocumentsIDsCommand.IDsLocalStorage.first(function (x) { return x === curId; })) {
                            verifiedIDs.push(curId);
                        }
                    });
                    this.docIDs.removeAll(verifyDocumentsIDsCommand.IDsLocalStorage);
                }
            }
            if (this.docIDs.length > 0) {
                var postResult = this.post("/queries?metadata-only=true", JSON.stringify(this.docIDs), this.db);
                postResult.fail(function (xhr) { return verifyResult.fail(xhr); });
                postResult.done(function (queryResult) {
                    if (!!queryResult && !!queryResult.Results) {
                        queryResult.Results.forEach(function (curVerifiedID) {
                            verifiedIDs.push(curVerifiedID['@metadata']['@id']);
                            if (_this.queryLocalStorage === true) {
                                verifyDocumentsIDsCommand.IDsLocalStorage.push(curVerifiedID);
                            }
                        });
                        if (_this.queryLocalStorage === true) {
                            _this.docIDs.removeAll(queryResult.Results.map(function (curResult) { return curResult['@metadata']['@id']; }));
                            verifyDocumentsIDsCommand.InvalidIDsLocal.pushAll(_this.docIDs);
                        }
                    }
                    verifyResult.resolve(verifiedIDs);
                });
                return verifyResult;
            }
            else {
                return verifiedIDs;
            }
        };
        verifyDocumentsIDsCommand.IDsLocalStorage = [];
        verifyDocumentsIDsCommand.InvalidIDsLocal = [];
        return verifyDocumentsIDsCommand;
    })(commandBase);
    return verifyDocumentsIDsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/deleteLocalPeriodicExportSetupCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteLocalPeriodicExportSetupCommand = (function (_super) {
        __extends(deleteLocalPeriodicExportSetupCommand, _super);
        function deleteLocalPeriodicExportSetupCommand(setupToPersist, db) {
            _super.call(this);
            this.setupToPersist = setupToPersist;
            this.db = db;
        }
        deleteLocalPeriodicExportSetupCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Updating Periodic Export setup.");
            return $.when(this.saveAccountInformation(), this.deleteSetup())
                .done(function () { return _this.reportSuccess("Updated Periodic Export setup."); })
                .fail(function (response) { return _this.reportError("Failed to update Periodic Export setup.", response.responseText); });
        };
        deleteLocalPeriodicExportSetupCommand.prototype.saveAccountInformation = function () {
            var jQueryOptions = {};
            if (this.setupToPersist.getEtag()) {
                jQueryOptions.headers = {
                    'If-None-Match': this.setupToPersist.getEtag()
                };
            }
            var url = "/admin/databases/" + this.db.name;
            var putArgs = JSON.stringify(this.setupToPersist.removeDatabaseSettings());
            return this.put(url, putArgs, null, jQueryOptions);
        };
        deleteLocalPeriodicExportSetupCommand.prototype.deleteSetup = function () {
            var url = "/docs/Raven/Backup/Periodic/Setup";
            return this.del(url, null, this.db);
        };
        return deleteLocalPeriodicExportSetupCommand;
    })(commandBase);
    return deleteLocalPeriodicExportSetupCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/deleteLocalReplicationsSetupCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteLocalReplicationsSetupCommand = (function (_super) {
        __extends(deleteLocalReplicationsSetupCommand, _super);
        function deleteLocalReplicationsSetupCommand(db) {
            _super.call(this);
            this.db = db;
        }
        deleteLocalReplicationsSetupCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Updating Replication config setup.");
            return $.when(this.deleteConflictResolution(), this.deleteLocalDestinations())
                .done(function () { return _this.reportSuccess("Updated Replication config setup."); })
                .fail(function (response) { return _this.reportError("Failed to update replication config setup.", response.responseText); });
        };
        deleteLocalReplicationsSetupCommand.prototype.deleteConflictResolution = function () {
            var url = "/docs/Raven/Replication/Config";
            return this.del(url, null, this.db);
        };
        deleteLocalReplicationsSetupCommand.prototype.deleteLocalDestinations = function () {
            var url = "/docs/Raven/Replication/Destinations";
            return this.del(url, null, this.db);
        };
        return deleteLocalReplicationsSetupCommand;
    })(commandBase);
    return deleteLocalReplicationsSetupCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getConfigurationSettingsCommand", ["require", "exports", "commands/commandBase", 'models/database/globalConfig/configurationSettings'], function (require, exports, commandBase, configurationSettings) {
    var getConfigurationSettingsCommand = (function (_super) {
        __extends(getConfigurationSettingsCommand, _super);
        function getConfigurationSettingsCommand(db, keys) {
            _super.call(this);
            this.db = db;
            this.keys = keys;
        }
        getConfigurationSettingsCommand.prototype.execute = function () {
            var url = "/configuration/settings";
            var args = {
                key: this.keys
            };
            return this.query(url, args, this.db, function (dto) { return new configurationSettings(dto); });
        };
        return getConfigurationSettingsCommand;
    })(commandBase);
    return getConfigurationSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectiveConflictResolutionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getEffectiveConflictResolutionCommand = (function (_super) {
        __extends(getEffectiveConflictResolutionCommand, _super);
        function getEffectiveConflictResolutionCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getEffectiveConflictResolutionCommand.prototype.execute = function () {
            var url = "/configuration/document/Raven/Replication/Config";
            return this.query(url, null, this.db);
        };
        return getEffectiveConflictResolutionCommand;
    })(commandBase);
    return getEffectiveConflictResolutionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectiveCustomFunctionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getEffectiveCustomFunctionsCommand = (function (_super) {
        __extends(getEffectiveCustomFunctionsCommand, _super);
        function getEffectiveCustomFunctionsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getEffectiveCustomFunctionsCommand.prototype.execute = function () {
            var url = "/configuration/document/Raven/Javascript/Functions";
            return this.query(url, null, this.db);
        };
        return getEffectiveCustomFunctionsCommand;
    })(commandBase);
    return getEffectiveCustomFunctionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectivePeriodicExportCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getEffectivePriodicExportCommand = (function (_super) {
        __extends(getEffectivePriodicExportCommand, _super);
        function getEffectivePriodicExportCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getEffectivePriodicExportCommand.prototype.execute = function () {
            var url = "/configuration/document/Raven/Backup/Periodic/Setup";
            return this.query(url, null, this.db);
        };
        return getEffectivePriodicExportCommand;
    })(commandBase);
    return getEffectivePriodicExportCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectiveSettingsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/document"], function (require, exports, commandBase, document) {
    var getEffectiveSettingsCommand = (function (_super) {
        __extends(getEffectiveSettingsCommand, _super);
        function getEffectiveSettingsCommand(db) {
            _super.call(this);
            this.db = db;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        getEffectiveSettingsCommand.prototype.execute = function () {
            var resultsSelector = function (queryResult) { return new document(queryResult); };
            var url = "/configuration/global/settings";
            return this.query(url, null, null, resultsSelector);
        };
        return getEffectiveSettingsCommand;
    })(commandBase);
    return getEffectiveSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectiveSqlReplicationConnectionStringsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getEffectiveSqlReplicationConnectionStringsCommand = (function (_super) {
        __extends(getEffectiveSqlReplicationConnectionStringsCommand, _super);
        function getEffectiveSqlReplicationConnectionStringsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getEffectiveSqlReplicationConnectionStringsCommand.prototype.execute = function () {
            var url = "/configuration/document/Raven/SqlReplication/Connections";
            return this.query(url, null, this.db);
        };
        return getEffectiveSqlReplicationConnectionStringsCommand;
    })(commandBase);
    return getEffectiveSqlReplicationConnectionStringsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getEffectiveVersioningsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/versioningEntry", "models/database/globalConfig/configurationDocument"], function (require, exports, commandBase, versioningEntry, configurationDocument) {
    var getEffectiveVersioningsCommand = (function (_super) {
        __extends(getEffectiveVersioningsCommand, _super);
        function getEffectiveVersioningsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getEffectiveVersioningsCommand.prototype.execute = function () {
            var url = "/configuration/versioning";
            var mapper = function (configs) {
                return configs.map(function (config) {
                    return configurationDocument.fromDtoWithTransform(config, function (x) { return new versioningEntry(x, true); });
                });
            };
            return this.query(url, null, this.db, mapper);
        };
        return getEffectiveVersioningsCommand;
    })(commandBase);
    return getEffectiveVersioningsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/getGlobalConfigReplicationsCommand", ["require", "exports", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, getDocumentWithMetadataCommand) {
    var getGlobalConfigReplicationsCommand = (function (_super) {
        __extends(getGlobalConfigReplicationsCommand, _super);
        function getGlobalConfigReplicationsCommand(db) {
            _super.call(this, "Raven/Global/Replication/Destinations", db);
        }
        getGlobalConfigReplicationsCommand.prototype.execute = function () {
            return _super.prototype.execute.call(this);
        };
        return getGlobalConfigReplicationsCommand;
    })(getDocumentWithMetadataCommand);
    return getGlobalConfigReplicationsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/globalConfig/saveGlobalSettingsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveGlobalSettingsCommand = (function (_super) {
        __extends(saveGlobalSettingsCommand, _super);
        function saveGlobalSettingsCommand(db, document) {
            _super.call(this);
            this.db = db;
            this.document = document;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        saveGlobalSettingsCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Global Settings...");
            var jQueryOptions = {};
            if (this.document.__metadata.etag) {
                jQueryOptions.headers = {
                    'If-None-Match': this.document.__metadata.etag
                };
            }
            var args = JSON.stringify(this.document.toDto());
            var url = "/configuration/global/settings";
            var saveTask = this.put(url, args, null, jQueryOptions);
            saveTask.done(function () { return _this.reportSuccess("Global Settings were successfully saved!"); });
            saveTask.fail(function (response) { return _this.reportError("Failed to save global settings!", response.responseText, response.statusText); });
            return saveTask;
        };
        return saveGlobalSettingsCommand;
    })(commandBase);
    return saveGlobalSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/deleteIndexCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteIndexCommand = (function (_super) {
        __extends(deleteIndexCommand, _super);
        function deleteIndexCommand(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
        }
        deleteIndexCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Deleting " + this.indexName + "...");
            return this.del("/indexes/" + this.indexName, null, this.db)
                .fail(function (response) { return _this.reportError("Failed to delete index " + _this.indexName, response.responseText); })
                .done(function () { return _this.reportSuccess("Deleted " + _this.indexName); });
        };
        return deleteIndexCommand;
    })(commandBase);
    return deleteIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/disableIndexingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var disableIndexingCommand = (function (_super) {
        __extends(disableIndexingCommand, _super);
        /**
        * @param names - The array of resource names to toggle
        * @param isSettingDisabled - Status of disabled to set
        * @param resourceType - The resource type
        */
        function disableIndexingCommand(dbName, isSettingIndexingDisabled) {
            _super.call(this);
            this.dbName = dbName;
            this.isSettingIndexingDisabled = isSettingIndexingDisabled;
        }
        disableIndexingCommand.prototype.execute = function () {
            var _this = this;
            var action = !this.isSettingIndexingDisabled ? "enable" : "disable";
            var args = {
                id: this.dbName,
                isSettingIndexingDisabled: this.isSettingIndexingDisabled
            };
            var url = "/admin/databases-toggle-indexing" + this.urlEncodeArgs(args);
            var toggleTask = this.post(url, null, null, { dataType: undefined });
            toggleTask.done(function () { return _this.reportSuccess("Successfully " + action + "d " + " indexing in " + _this.dbName); });
            toggleTask.fail(function (response) { return _this.reportError("Failed to " + action + " indexing in " + _this.dbName, response.responseText, response.statusText); });
            return toggleTask;
        };
        return disableIndexingCommand;
    })(commandBase);
    return disableIndexingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/forceIndexReplace", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var forceIndexReplace = (function (_super) {
        __extends(forceIndexReplace, _super);
        function forceIndexReplace(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
        }
        forceIndexReplace.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Replacing index (forced)");
            var url = "/indexes/" + this.indexName + "?op=forceReplace";
            return this.post(url, null, this.db, { dataType: undefined }).done(function () {
                _this.reportSuccess("Replaced index " + _this.indexName);
            }).fail(function (response) { return _this.reportError("Failed to replace index.", response.responseText, response.statusText); });
        };
        return forceIndexReplace;
    })(commandBase);
    return forceIndexReplace;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/formatIndexCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var formatIndexCommand = (function (_super) {
        __extends(formatIndexCommand, _super);
        function formatIndexCommand(db, mapReduceArray) {
            _super.call(this);
            this.db = db;
            this.mapReduceArray = mapReduceArray;
        }
        formatIndexCommand.prototype.execute = function () {
            var url = "/debug/format-index";
            return this.post(url, JSON.stringify(this.mapReduceArray), this.db);
        };
        return formatIndexCommand;
    })(commandBase);
    return formatIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getCSharpIndexDefinitionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getCSharpIndexDefinitionCommand = (function (_super) {
        __extends(getCSharpIndexDefinitionCommand, _super);
        function getCSharpIndexDefinitionCommand(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
        }
        getCSharpIndexDefinitionCommand.prototype.execute = function () {
            var url = "/c-sharp-index-definition/" + this.indexName;
            return this.query(url, null, this.db);
        };
        return getCSharpIndexDefinitionCommand;
    })(commandBase);
    return getCSharpIndexDefinitionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getIndexDefinitionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexDefinitionCommand = (function (_super) {
        __extends(getIndexDefinitionCommand, _super);
        function getIndexDefinitionCommand(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
        }
        getIndexDefinitionCommand.prototype.execute = function () {
            var url = "/indexes/" + this.indexName + "?definition=yes";
            return this.query(url, null, this.db);
        };
        return getIndexDefinitionCommand;
    })(commandBase);
    return getIndexDefinitionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getIndexesDefinitionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexesDefinitionsCommand = (function (_super) {
        __extends(getIndexesDefinitionsCommand, _super);
        function getIndexesDefinitionsCommand(db, skip, take) {
            if (skip === void 0) { skip = 0; }
            if (take === void 0) { take = 256; }
            _super.call(this);
            this.db = db;
            this.skip = skip;
            this.take = take;
        }
        getIndexesDefinitionsCommand.prototype.execute = function () {
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            var url = "/indexes" + this.urlEncodeArgs(args);
            return this.query(url, null, this.db);
        };
        return getIndexesDefinitionsCommand;
    })(commandBase);
    return getIndexesDefinitionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getIndexingStatusCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexingStatusCommand = (function (_super) {
        __extends(getIndexingStatusCommand, _super);
        function getIndexingStatusCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getIndexingStatusCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            var url = "/admin/indexingStatus";
            this.query(url, null, this.db)
                .done(function (indexingStatus) { return result.resolve(indexingStatus); })
                .fail(function (response) {
                _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                result.reject();
            });
            return result;
        };
        return getIndexingStatusCommand;
    })(commandBase);
    return getIndexingStatusCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getIndexSuggestionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexSuggestionsCommand = (function (_super) {
        __extends(getIndexSuggestionsCommand, _super);
        function getIndexSuggestionsCommand(db, indexName, field, term, distance, accuracy, maxSuggestions, popularity) {
            if (distance === void 0) { distance = "Default"; }
            if (accuracy === void 0) { accuracy = 0.4; }
            if (maxSuggestions === void 0) { maxSuggestions = 5; }
            if (popularity === void 0) { popularity = true; }
            _super.call(this);
            this.db = db;
            this.indexName = indexName;
            this.field = field;
            this.term = term;
            this.distance = distance;
            this.accuracy = accuracy;
            this.maxSuggestions = maxSuggestions;
            this.popularity = popularity;
        }
        getIndexSuggestionsCommand.prototype.execute = function () {
            var url = '/suggest/' + this.indexName;
            var args = {
                term: this.term,
                field: this.field,
                max: this.maxSuggestions,
                distance: this.distance,
                accuracy: this.accuracy,
                popularity: this.popularity
            };
            return this.query(url, args, this.db);
        };
        return getIndexSuggestionsCommand;
    })(commandBase);
    return getIndexSuggestionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getIndexTermsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getIndexTermsCommand = (function (_super) {
        __extends(getIndexTermsCommand, _super);
        function getIndexTermsCommand(indexName, field, db) {
            _super.call(this);
            this.indexName = indexName;
            this.field = field;
            this.db = db;
        }
        getIndexTermsCommand.prototype.execute = function () {
            var urlArgs = {
                field: this.field,
                pageSize: 1024
            };
            var url = "/terms/" + this.indexName + this.urlEncodeArgs(urlArgs);
            return this.query(url, null, this.db);
        };
        return getIndexTermsCommand;
    })(commandBase);
    return getIndexTermsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getPendingIndexReplacementsCommand", ["require", "exports", "commands/commandBase", "models/database/index/indexReplaceDocument"], function (require, exports, commandBase, indexReplaceDocument) {
    var getPendingIndexReplacementsCommand = (function (_super) {
        __extends(getPendingIndexReplacementsCommand, _super);
        function getPendingIndexReplacementsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getPendingIndexReplacementsCommand.prototype.execute = function () {
            var resultsSelector = function (result) {
                return result.map(function (dto) { return new indexReplaceDocument(dto); });
            };
            var url = "/docs";
            var args = {
                startsWith: indexReplaceDocument.replaceDocumentPrefix,
                start: 0,
                pageSize: 1024
            };
            return this.query(url, args, this.db, resultsSelector);
        };
        return getPendingIndexReplacementsCommand;
    })(commandBase);
    return getPendingIndexReplacementsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/getScriptedIndexesCommand", ["require", "exports", "commands/commandBase", "models/database/index/scriptedIndex"], function (require, exports, commandBase, scriptedIndex) {
    var getScriptedIndexesCommand = (function (_super) {
        __extends(getScriptedIndexesCommand, _super);
        function getScriptedIndexesCommand(db, indexName) {
            if (indexName === void 0) { indexName = ""; }
            _super.call(this);
            this.db = db;
            this.indexName = indexName;
        }
        getScriptedIndexesCommand.prototype.execute = function () {
            var args = {
                startsWith: "Raven/ScriptedIndexResults/" + this.indexName,
                exclude: null,
                start: 0,
                pageSize: 256
            };
            return this.query("/docs", args, this.db, function (dtos) { return dtos.map(function (dto) { return new scriptedIndex(dto); }); });
        };
        return getScriptedIndexesCommand;
    })(commandBase);
    return getScriptedIndexesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/persistIndexCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var persistIndexCommand = (function (_super) {
        __extends(persistIndexCommand, _super);
        function persistIndexCommand(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
        }
        persistIndexCommand.prototype.execute = function () {
            var _this = this;
            var url = "/indexes/" + this.indexName + "?op=forceWriteToDisk";
            return this.post(url, null, this.db, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Successfully persisted index: " + _this.indexName); })
                .fail(function (response) { return _this.reportError("Error persisting index", response.responseText, response.statusText); });
        };
        return persistIndexCommand;
    })(commandBase);
    return persistIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/resetIndexCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resetIndexCommand = (function (_super) {
        __extends(resetIndexCommand, _super);
        function resetIndexCommand(indexNameToReset, db) {
            _super.call(this);
            this.indexNameToReset = indexNameToReset;
            this.db = db;
        }
        resetIndexCommand.prototype.execute = function () {
            var _this = this;
            var url = "/indexes/" + this.indexNameToReset;
            return this.reset(url, null, this.db)
                .done(function () { return _this.reportSuccess("Index " + _this.indexNameToReset + " successfully reset"); })
                .fail(function (response) { return _this.reportError("Failed to reset index: " + _this.indexNameToReset, response.responseText, response.statusText); });
        };
        return resetIndexCommand;
    })(commandBase);
    return resetIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/saveIndexAsPersistentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveIndexAsPersistentCommand = (function (_super) {
        __extends(saveIndexAsPersistentCommand, _super);
        function saveIndexAsPersistentCommand(indexToPersist, db) {
            _super.call(this);
            this.indexToPersist = indexToPersist;
            this.db = db;
        }
        saveIndexAsPersistentCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Persisting " + this.indexToPersist.name + "...");
            // Deliberately not URL-encoding index name, as it causes the operation to fail on the server.
            var url = "/indexes/" + this.indexToPersist.name + "?op=forceWriteToDisk";
            var saveTask = this.post(url, null, this.db);
            saveTask.done(function () {
                _this.reportSuccess("Persisted " + _this.indexToPersist.name);
                _this.indexToPersist.isOnRam("false");
            });
            saveTask.fail(function (response) { return _this.reportError("Failed to persist " + _this.indexToPersist.name, response.responseText, response.statusText); });
            return saveTask;
        };
        return saveIndexAsPersistentCommand;
    })(commandBase);
    return saveIndexAsPersistentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/saveIndexDefinitionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveIndexDefinitionCommand = (function (_super) {
        __extends(saveIndexDefinitionCommand, _super);
        function saveIndexDefinitionCommand(index, db) {
            _super.call(this);
            this.index = index;
            this.db = db;
        }
        saveIndexDefinitionCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving " + this.index.Name + "...");
            return this.saveDefinition()
                .fail(function (response) {
                _this.reportError("Failed to save " + _this.index.Name, response.responseText, response.statusText);
            })
                .done(function () {
                _this.reportSuccess("Saved " + _this.index.Name + ".");
            });
        };
        saveIndexDefinitionCommand.prototype.saveDefinition = function () {
            var urlArgs = {
                definition: "yes"
            };
            var putArgs = JSON.stringify(this.index);
            var url = "/indexes/" + this.index.Name + this.urlEncodeArgs(urlArgs);
            return this.put(url, putArgs, this.db);
        };
        return saveIndexDefinitionCommand;
    })(commandBase);
    return saveIndexDefinitionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/saveIndexLockModeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveIndexLockModeCommand = (function (_super) {
        __extends(saveIndexLockModeCommand, _super);
        /*
         * @param lockMode Should be either "Unlock", "LockedIngore", or "LockedError".
        */
        function saveIndexLockModeCommand(index, lockMode, db) {
            _super.call(this);
            this.index = index;
            this.lockMode = lockMode;
            this.db = db;
        }
        saveIndexLockModeCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving " + this.index.name + "...");
            var args = {
                op: "lockModeChange",
                mode: this.lockMode
            };
            // Deliberately not encoding index name in URI. Doing this breaks Raven's URL handling, resulting in a 400 bad request.
            var url = "/indexes/" + this.index.name + this.urlEncodeArgs(args);
            var saveTask = this.post(url, JSON.stringify(args), this.db, { dataType: 'text' });
            saveTask.done(function () { return _this.reportSuccess("Saved " + _this.index.name); });
            saveTask.fail(function (response) { return _this.reportError("Failed to save this" + _this.index.name, response.responseText); });
            return saveTask;
        };
        return saveIndexLockModeCommand;
    })(commandBase);
    return saveIndexLockModeCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/saveIndexPriorityCommand", ["require", "exports", "commands/commandBase", "models/database/index/index"], function (require, exports, commandBase, index) {
    var saveIndexPriorityCommand = (function (_super) {
        __extends(saveIndexPriorityCommand, _super);
        function saveIndexPriorityCommand(indexName, priority, db) {
            _super.call(this);
            this.indexName = indexName;
            this.priority = priority;
            this.db = db;
        }
        saveIndexPriorityCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving " + this.indexName + " priority ...");
            // Note: don't URL encode the priority. Doing so busts Raven's handler.
            var url = "/indexes-set-priority/" + this.indexName + "?priority=" + index.priorityToString(this.priority);
            return this.post(url, null, this.db, { dataType: undefined })
                .done(function () {
                _this.reportSuccess("Saved " + _this.indexName + ".");
            });
        };
        return saveIndexPriorityCommand;
    })(commandBase);
    return saveIndexPriorityCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/startIndexingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var startIndexingCommand = (function (_super) {
        __extends(startIndexingCommand, _super);
        function startIndexingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        startIndexingCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Enabling indexing...");
            var url = '/admin/startIndexing';
            var createTask = this.post(url, null, this.db);
            createTask.done(function () { return _this.reportSuccess("Indexing was enabled"); });
            createTask.fail(function (response) { return _this.reportError("Failed to enable indexing", response.responseText, response.statusText); });
            return createTask;
        };
        return startIndexingCommand;
    })(commandBase);
    return startIndexingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/startReducingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var startReducingCommand = (function (_super) {
        __extends(startReducingCommand, _super);
        function startReducingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        startReducingCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Enabling reduction...");
            var url = '/admin/startReducing';
            var createTask = this.post(url, null, this.db);
            createTask.done(function () { return _this.reportSuccess("Reduction was enabled"); });
            createTask.fail(function (response) { return _this.reportError("Failed to enable reduction", response.responseText, response.statusText); });
            return createTask;
        };
        return startReducingCommand;
    })(commandBase);
    return startReducingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/stopIndexingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var stopIndexingCommand = (function (_super) {
        __extends(stopIndexingCommand, _super);
        function stopIndexingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        stopIndexingCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Disabling Indexing... (will wait for current indexing batch to complete)");
            var url = '/admin/stopIndexing';
            var createTask = this.post(url, null, this.db);
            createTask.done(function () { return _this.reportSuccess("Indexing was disabled"); });
            createTask.fail(function (response) { return _this.reportError("Failed to disable indexing", response.responseText, response.statusText); });
            return createTask;
        };
        return stopIndexingCommand;
    })(commandBase);
    return stopIndexingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/stopReducingCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var stopReducingCommand = (function (_super) {
        __extends(stopReducingCommand, _super);
        function stopReducingCommand(db) {
            _super.call(this);
            this.db = db;
        }
        stopReducingCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Disabling reduction... (will wait for current reduction batch to complete)");
            var url = '/admin/stopReducing';
            var createTask = this.post(url, null, this.db);
            createTask.done(function () { return _this.reportSuccess("Reduction was disabled"); });
            createTask.fail(function (response) { return _this.reportError("Failed to disable reduction", response.responseText, response.statusText); });
            return createTask;
        };
        return stopReducingCommand;
    })(commandBase);
    return stopReducingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/index/tryRecoverCorruptedIndexes", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var tryRecoverCorruptedIndexes = (function (_super) {
        __extends(tryRecoverCorruptedIndexes, _super);
        function tryRecoverCorruptedIndexes(db) {
            _super.call(this);
            this.db = db;
        }
        tryRecoverCorruptedIndexes.prototype.execute = function () {
            var _this = this;
            var url = "/indexes/try-recover-corrupted";
            return this.patch(url, null, this.db)
                .done(function () { return _this.reportSuccess("The recovery operation started in the background"); })
                .fail(function (response) { return _this.reportError("Failed to start recovery operation", response.responseText, response.statusText); });
        };
        return tryRecoverCorruptedIndexes;
    })(commandBase);
    return tryRecoverCorruptedIndexes;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/patch/evalByQueryCommand", ["require", "exports", "commands/commandBase", "commands/operations/getOperationStatusCommand"], function (require, exports, commandBase, getOperationStatusCommand) {
    var evalByQueryCommand = (function (_super) {
        __extends(evalByQueryCommand, _super);
        function evalByQueryCommand(indexName, queryStr, patchPayload, db, updatePatchingProgress) {
            _super.call(this);
            this.indexName = indexName;
            this.queryStr = queryStr;
            this.patchPayload = patchPayload;
            this.db = db;
            this.updatePatchingProgress = updatePatchingProgress;
        }
        /*
         * Promise returned by this method is initial request promise - this is resolved after opearation is scheduled (but NOT completed yet)
         */
        evalByQueryCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Patching documents...");
            var url = "/bulk_docs/" + encodeURIComponent(this.indexName);
            var urlParams = "?query=" + encodeURIComponent(this.queryStr) + "&allowStale=true";
            var patchTask = this.evalJs(url + urlParams, this.patchPayload, this.db);
            // patch is made asynchronically so we infom user about operation start - not about actual completion. 
            patchTask.done(function (response) {
                _this.reportSuccess("Scheduled patch of index: " + _this.indexName);
                _this.monitorPatching(response.OperationId);
            });
            patchTask.fail(function (response) { return _this.reportError("Failed to schedule patch of index " + _this.indexName, response.responseText, response.statusText); });
            return patchTask;
        };
        evalByQueryCommand.prototype.monitorPatching = function (operationId) {
            var _this = this;
            new getOperationStatusCommand(this.db, operationId)
                .execute()
                .done(function (result) {
                _this.updatePatchingProgress(result);
                if (result.Completed) {
                    if (result.Faulted || result.Canceled) {
                        _this.reportError("Patch failed", result.State.Error);
                    }
                    else {
                        _this.reportSuccess("Patching completed");
                    }
                }
                else {
                    setTimeout(function () { return _this.monitorPatching(operationId); }, 500);
                }
            });
        };
        return evalByQueryCommand;
    })(commandBase);
    return evalByQueryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/patch/executePatchCommand", ["require", "exports", "commands/database/documents/executeBulkDocsCommand"], function (require, exports, executeBulkDocsCommand) {
    var executePatchCommand = (function (_super) {
        __extends(executePatchCommand, _super);
        function executePatchCommand(bulkDocsDto, db, test) {
            _super.call(this, bulkDocsDto, db);
            this.test = test;
        }
        executePatchCommand.prototype.execute = function () {
            var _this = this;
            var failInfo = (this.test) ? "Patch test failed." : "Unable to patch documents.";
            var doneInfo = (this.test) ? "Patch tested. See the results below." : "Documents patched.";
            if (!this.test) {
                this.reportInfo("Patching documents...");
            }
            return _super.prototype.execute.call(this)
                .fail(function (result) { return _this.reportError(failInfo, result.responseText, result.statusText); })
                .done(function () { return _this.reportSuccess(doneInfo); });
        };
        return executePatchCommand;
    })(executeBulkDocsCommand);
    return executePatchCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/patch/getPatchesCommand", ["require", "exports", "commands/commandBase", "models/database/patch/patchDocument"], function (require, exports, commandBase, patchDocument) {
    var getPatchesCommand = (function (_super) {
        __extends(getPatchesCommand, _super);
        function getPatchesCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getPatchesCommand.prototype.execute = function () {
            var args = {
                startsWith: "Studio/Patch/",
                exclude: null,
                start: 0,
                pageSize: 256
            };
            return this.query("/docs", args, this.db, function (dtos) { return dtos.map(function (dto) { return new patchDocument(dto); }); });
        };
        return getPatchesCommand;
    })(commandBase);
    return getPatchesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/patch/savePatchCommand", ["require", "exports", "commands/commandBase", 'commands/database/documents/saveDocumentCommand'], function (require, exports, commandBase, saveDocumentCommand) {
    var savePatchCommand = (function (_super) {
        __extends(savePatchCommand, _super);
        function savePatchCommand(patchName, patchDocument, db) {
            _super.call(this);
            this.patchName = patchName;
            this.patchDocument = patchDocument;
            this.db = db;
        }
        savePatchCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Patch.");
            return new saveDocumentCommand("Studio/Patch/" + this.patchName, this.patchDocument, this.db, false).execute()
                .done(function () { return _this.reportSuccess("Saved Patch."); })
                .fail(function (response) { return _this.reportError("Failed to save Patch.", response.responseText, response.statusText); });
        };
        return savePatchCommand;
    })(commandBase);
    return savePatchCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/query/dataExplorationCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var dataExplorationCommand = (function (_super) {
        __extends(dataExplorationCommand, _super);
        function dataExplorationCommand(request, db) {
            _super.call(this);
            this.request = request;
            this.db = db;
        }
        dataExplorationCommand.prototype.execute = function () {
            var _this = this;
            var self = this;
            var options = {
                xhr: function () {
                    self.xhr = new XMLHttpRequest();
                    // disable alerts as this request might take a while 
                    clearTimeout(commandBase.alertTimeout);
                    clearTimeout(commandBase.splashTimerHandle);
                    return self.xhr;
                }
            };
            var queryTask = this.query(this.getUrl(), null, this.db, null, options);
            queryTask.fail(function (response) { return _this.reportError("Error during query", response.responseText, response.statusText); });
            return queryTask;
        };
        dataExplorationCommand.prototype.getUrl = function () {
            return "/streams/exploration/" + this.urlEncodeArgs(this.request);
        };
        dataExplorationCommand.prototype.getCsvUrl = function () {
            var requestWithCsvDownload = this.request;
            requestWithCsvDownload.download = "true";
            requestWithCsvDownload.format = "excel";
            return appUrl.forResourceQuery(this.db) + "/streams/exploration/" + this.urlEncodeArgs(requestWithCsvDownload);
        };
        return dataExplorationCommand;
    })(commandBase);
    return dataExplorationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/query/enableQueryTimings", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var enableQueryTimings = (function (_super) {
        __extends(enableQueryTimings, _super);
        function enableQueryTimings(db) {
            _super.call(this);
            this.db = db;
        }
        enableQueryTimings.prototype.execute = function () {
            var url = "/debug/enable-query-timing";
            return this.query(url, null, this.db);
        };
        return enableQueryTimings;
    })(commandBase);
    return enableQueryTimings;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/query/getKillQueryCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getKillQueryCommand = (function (_super) {
        __extends(getKillQueryCommand, _super);
        function getKillQueryCommand(db, queryId) {
            _super.call(this);
            this.db = db;
            this.queryId = queryId;
        }
        getKillQueryCommand.prototype.execute = function () {
            var _this = this;
            var url = "/admin/killQuery";
            var args = {
                id: this.queryId
            };
            return this.query(url, args, this.db)
                .fail(function (response) { return _this.reportError("Failed to kill query", response.responseText, response.statusText); });
        };
        return getKillQueryCommand;
    })(commandBase);
    return getKillQueryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/query/queryFacetsCommand", ["require", "exports", "commands/commandBase", "common/pagedResultSet", "models/database/query/facet", "models/database/documents/document"], function (require, exports, commandBase, pagedResultSet, facet, document) {
    var queryFacetsCommand = (function (_super) {
        __extends(queryFacetsCommand, _super);
        function queryFacetsCommand(indexName, queryText, skip, take, facets, db, disableCache) {
            if (disableCache === void 0) { disableCache = false; }
            _super.call(this);
            this.indexName = indexName;
            this.queryText = queryText;
            this.skip = skip;
            this.take = take;
            this.facets = facets;
            this.db = db;
            this.disableCache = disableCache;
            this.argsUrl = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                facetStart: this.skip,
                facetPageSize: this.take,
                disableCache: this.disableCache ? Date.now() : undefined,
                facets: JSON.stringify(this.facets)
            });
        }
        queryFacetsCommand.prototype.execute = function () {
            var _this = this;
            var url = "/facets/" + this.indexName + this.argsUrl;
            // Querying facets returns a facetResultSetDto. We need to massage that
            // data into something that can be displayed in the grid: the pagedResultSet.
            var finishedTask = $.Deferred();
            this.query(url, null, this.db)
                .fail(function (response) {
                _this.reportError("Unable to run query.", response.responseText, response.statusText);
                finishedTask.reject(response);
            })
                .done(function (results) {
                finishedTask.resolve(_this.parseResults(results));
            });
            return finishedTask;
        };
        queryFacetsCommand.prototype.parseResults = function (resultSet) {
            var items = [];
            var totalItemCount = 0;
            // The .Results property contains properties in the form of [facet name]-[facet aggregation field].
            // For example: Company-Total
            // Each of these properties will be of type facetResultDto.
            // In the UI, we display these as "[facet aggregation] of [facet aggregation field]", e.g. "Count of Total".
            var propIndex = 0;
            for (var prop in resultSet.Results) {
                var facetResult = resultSet.Results[prop];
                var propNameParts = prop.split('-');
                var facetName = propNameParts[0];
                var aggregateField = propNameParts[1];
                var correspondingFacet = this.facets[propIndex];
                var facetAggregationLabel = facet.getLabelForAggregation(correspondingFacet.Aggregation);
                totalItemCount = facetResult.Values.length + facetResult.RemainingTermsCount + this.skip;
                // Construct our result set, an array objects that take the following shape:
                // { 'Key': [facet range], 'Count of Foobar': 3, 'Min of Blah': 4, ... }
                for (var i = 0; i < facetResult.Values.length; i++) {
                    var facetValue = facetResult.Values[i];
                    var item = items[i];
                    if (!item) {
                        item = new document({});
                        items[i] = item;
                    }
                    item['Key'] = facetValue.Range;
                    //item[facetAggregationLabel + " of " + aggregateField] = facetValue.Hits;
                    for (var power = 0; power < 5; power++) {
                        var curFieldName = facet.getLabelForAggregation(Math.pow(2, power));
                        var curFieldVal = facetValue[curFieldName];
                        if (!!curFieldVal) {
                            if (typeof curFieldVal === "number") {
                                var fixedVal = curFieldVal.toFixed(2);
                                if (fixedVal != curFieldVal) {
                                    curFieldVal = fixedVal;
                                }
                            }
                            item[curFieldName + " of " + aggregateField] = curFieldVal;
                        }
                    }
                }
                propIndex++;
            }
            return new pagedResultSet(items, totalItemCount, resultSet.Duration);
        };
        return queryFacetsCommand;
    })(commandBase);
    return queryFacetsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/query/queryIndexCommand", ["require", "exports", "commands/commandBase", "models/database/documents/document", "common/pagedResultSet"], function (require, exports, commandBase, document, pagedResultSet) {
    var queryIndexCommand = (function (_super) {
        __extends(queryIndexCommand, _super);
        function queryIndexCommand(indexName, db, skip, take, queryText, sorts, transformerQuery, showFields, indexEntries, useAndOperator, disableCache) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.skip = skip;
            this.take = take;
            this.queryText = queryText;
            this.sorts = sorts;
            this.transformerQuery = transformerQuery;
            this.showFields = showFields;
            this.indexEntries = indexEntries;
            this.useAndOperator = useAndOperator;
            this.disableCache = disableCache;
        }
        queryIndexCommand.prototype.execute = function () {
            var _this = this;
            var selector = function (results) { return new pagedResultSet(results.Results.map(function (d) { return new document(d); }), results.TotalResults, results); };
            var queryTask = this.query(this.getUrl(), null, this.db, selector);
            queryTask.fail(function (response) { return _this.reportError("Error querying index", response.responseText, response.statusText); });
            return queryTask;
        };
        queryIndexCommand.prototype.getUrl = function () {
            var url = "/indexes/" + this.indexName;
            //var resultsTransformerUrlFragment = this.transformer && this.transformer.name() ? "&resultsTransformer=" + this.transformer.name() : ""; // This should not be urlEncoded, as it breaks the query.
            var resultsTransformerUrlFragment = (this.transformerQuery ? this.transformerQuery.toUrl() : "");
            var urlArgs = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                start: this.skip,
                pageSize: this.take,
                sort: this.sorts.map(function (s) { return s.toQuerySortString(); }),
                fetch: this.showFields ? "__all_fields" : undefined,
                debug: this.indexEntries ? "entries" : undefined,
                operator: this.useAndOperator ? "AND" : undefined,
                disableCache: this.disableCache ? Date.now() : undefined
            }) + resultsTransformerUrlFragment;
            return url + urlArgs;
        };
        queryIndexCommand.prototype.getCsvUrl = function () {
            var url = "/streams/query/" + this.indexName;
            var resultsTransformerUrlFragment = (this.transformerQuery ? this.transformerQuery.toUrl() : "");
            var urlArgs = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                sort: this.sorts.map(function (s) { return s.toQuerySortString(); }),
                fetch: this.showFields ? "__all_fields" : undefined,
                debug: this.indexEntries ? "entries" : undefined,
                operator: this.useAndOperator ? "AND" : undefined,
                format: "excel",
                download: true
            }) + resultsTransformerUrlFragment;
            return url + urlArgs;
        };
        return queryIndexCommand;
    })(commandBase);
    return queryIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/conflictsResolveCommand", ["require", "exports", "commands/commandBase", 'commands/operations/getOperationStatusCommand'], function (require, exports, commandBase, getOperationStatusCommand) {
    var conflictsResolveCommand = (function (_super) {
        __extends(conflictsResolveCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function conflictsResolveCommand(ownerDb, resolution) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.resolution = resolution;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        conflictsResolveCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var url = "/studio-tasks/replication/conflicts/resolve?resolution=" + this.resolution;
            this.post(url, null, this.ownerDb)
                .fail(function (response) {
                _this.reportError("Failed to start conflict resolution!", response.responseText, response.statusText);
                promise.reject();
            }).done(function (result) {
                _this.monitorOperation(promise, result.OperationId);
            });
            return promise;
        };
        conflictsResolveCommand.prototype.monitorOperation = function (parentPromise, operationId) {
            var _this = this;
            new getOperationStatusCommand(this.ownerDb, operationId)
                .execute()
                .done(function (result) {
                if (result.Completed) {
                    if (result.Faulted || result.Canceled) {
                        _this.reportError("Failed to perform conflict resolution!", result.State.Error);
                        parentPromise.reject();
                    }
                    else {
                        _this.reportSuccess("Conflict resolution was completed");
                        parentPromise.resolve();
                    }
                }
                else {
                    setTimeout(function () { return _this.monitorOperation(parentPromise, operationId); }, 500);
                }
            });
        };
        return conflictsResolveCommand;
    })(commandBase);
    return conflictsResolveCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/enableReplicationCommand", ["require", "exports", "commands/commandBase", "commands/resources/getDatabaseSettingsCommand", "commands/resources/saveDatabaseSettingsCommand"], function (require, exports, commandBase, getDatabaseSettingsCommand, saveDatabaseSettingsCommand) {
    var enableReplicationCommand = (function (_super) {
        __extends(enableReplicationCommand, _super);
        function enableReplicationCommand(db) {
            _super.call(this);
            this.db = db;
        }
        enableReplicationCommand.prototype.execute = function () {
            var _this = this;
            var task = $.Deferred();
            // query for database document
            new getDatabaseSettingsCommand(this.db, false)
                .execute()
                .done(function (dbSettings) {
                var activeBundles = dbSettings.Settings["Raven/ActiveBundles"];
                if (activeBundles.indexOf("Replication") > -1) {
                    // looks like we already have replication enabled - nothing to do
                    task.resolve(activeBundles.split(","));
                    _this.reportSuccess("Replication was already enabled for this database");
                }
                else {
                    if (!activeBundles) {
                        activeBundles = "Replication";
                    }
                    else {
                        activeBundles += ",Replication";
                    }
                    dbSettings.__metadata.etag = dbSettings.__metadata["@etag"];
                    // and save updated settings back to server
                    dbSettings.Settings["Raven/ActiveBundles"] = activeBundles;
                    dbSettings.__metadata["Raven-Temp-Allow-Bundles-Change"] = true;
                    new saveDatabaseSettingsCommand(_this.db, dbSettings)
                        .execute()
                        .done(function () {
                        task.resolve(activeBundles.split(","));
                    }).fail(function () {
                        task.reject();
                    });
                }
            })
                .fail(function (response) {
                _this.reportError("Failed to enable replication bundle!", response.responseText, response.statusText);
                task.reject();
            });
            return task.promise();
        };
        return enableReplicationCommand;
    })(commandBase);
    return enableReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/explainReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var explainReplicationCommand = (function (_super) {
        __extends(explainReplicationCommand, _super);
        function explainReplicationCommand(db, documentId, url, databaseName) {
            _super.call(this);
            this.db = db;
            this.documentId = documentId;
            this.url = url;
            this.databaseName = databaseName;
        }
        explainReplicationCommand.prototype.execute = function () {
            var args = {
                destinationUrl: this.url,
                databaseName: this.databaseName
            };
            return this.query("/replication/explain/" + this.documentId, args, this.db);
        };
        return explainReplicationCommand;
    })(commandBase);
    return explainReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getAutomaticConflictResolutionDocumentCommand", ["require", "exports", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, getDocumentWithMetadataCommand) {
    var getAutomaticConflictResolutionDocumentCommand = (function (_super) {
        __extends(getAutomaticConflictResolutionDocumentCommand, _super);
        function getAutomaticConflictResolutionDocumentCommand(db, global) {
            if (global === void 0) { global = false; }
            _super.call(this, global ? "Raven/Global/Replication/Config" : "Raven/Replication/Config", db);
        }
        getAutomaticConflictResolutionDocumentCommand.prototype.execute = function () {
            return _super.prototype.execute.call(this);
        };
        return getAutomaticConflictResolutionDocumentCommand;
    })(getDocumentWithMetadataCommand);
    return getAutomaticConflictResolutionDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getConflictsCommand", ["require", "exports", "common/pagedResultSet", "commands/commandBase", "models/database/replication/conflictsInfo"], function (require, exports, pagedResultSet, commandBase, conflictsInfo) {
    var getConflictsCommand = (function (_super) {
        __extends(getConflictsCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getConflictsCommand(ownerDb, skip, take) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.skip = skip;
            this.take = take;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getConflictsCommand.prototype.execute = function () {
            var args = {
                sort: "-ConflictDetectedAt",
                start: this.skip,
                pageSize: this.take,
                resultsTransformer: "Raven/ConflictDocumentsTransformer"
            };
            var resultsSelector = function (dto) { return new conflictsInfo(dto); };
            var url = "/indexes/Raven/ConflictDocuments";
            var conflictsTask = $.Deferred();
            this.query(url, args, this.ownerDb, resultsSelector).
                fail(function (response) { return conflictsTask.reject(response); }).
                done(function (conflicts) {
                var items = conflicts.results;
                var resultsSet = new pagedResultSet(items, conflicts.totalResults);
                conflictsTask.resolve(resultsSet);
            });
            return conflictsTask;
        };
        return getConflictsCommand;
    })(commandBase);
    return getConflictsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getReplicationsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getReplicationsCommand = (function (_super) {
        __extends(getReplicationsCommand, _super);
        function getReplicationsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getReplicationsCommand.prototype.execute = function () {
            var url = "/configuration/document/Raven/Replication/Destinations";
            return this.query(url, null, this.db);
        };
        return getReplicationsCommand;
    })(commandBase);
    return getReplicationsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getReplicationSourcesCommand", ["require", "exports", "commands/commandBase", "models/resources/database"], function (require, exports, commandBase, database) {
    var getReplicationSourcesCommand = (function (_super) {
        __extends(getReplicationSourcesCommand, _super);
        function getReplicationSourcesCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getReplicationSourcesCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                startsWith: "Raven/Replication/Sources",
                exclude: null,
                start: 0,
                pageSize: 1024
            };
            return this.query("/docs", args, this.db, function (dtos) {
                var result = {};
                // insert remote databases info
                dtos.forEach(function (v) { return result[v.ServerInstanceId] = database.getNameFromUrl(v.Source); });
                // ... and insert local database instance id
                result[_this.db.statistics().databaseId()] = _this.db.name;
                return result;
            });
        };
        return getReplicationSourcesCommand;
    })(commandBase);
    return getReplicationSourcesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getReplicationStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getReplicationStatsCommand = (function (_super) {
        __extends(getReplicationStatsCommand, _super);
        function getReplicationStatsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getReplicationStatsCommand.prototype.execute = function () {
            return this.query("/replication/info", null, this.db);
        };
        return getReplicationStatsCommand;
    })(commandBase);
    return getReplicationStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/getReplicationTopology", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getReplicationTopology = (function (_super) {
        __extends(getReplicationTopology, _super);
        function getReplicationTopology(db) {
            _super.call(this);
            this.db = db;
        }
        getReplicationTopology.prototype.execute = function () {
            return this.post("/admin/replication/topology/view", null, this.db).then(function (result) {
                return result;
            });
        };
        return getReplicationTopology;
    })(commandBase);
    return getReplicationTopology;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/replicateAllIndexesCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var replicateAllIndexesCommand = (function (_super) {
        __extends(replicateAllIndexesCommand, _super);
        function replicateAllIndexesCommand(db) {
            _super.call(this);
            this.db = db;
        }
        replicateAllIndexesCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var indexesUrl = '/databases/' + this.db.name + '/replication/replicate-indexes?op=replicate-all';
            this.post(indexesUrl, null, appUrl.getSystemDatabase(), { dataType: undefined })
                .fail(function (response) {
                _this.reportError("Failed to send replicate all indexes command!", response.responseText, response.statusText);
                promise.reject();
            }).done(function () {
                _this.reportSuccess("Sent replicate all indexes command.");
                promise.resolve();
            });
            return promise;
        };
        return replicateAllIndexesCommand;
    })(commandBase);
    return replicateAllIndexesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/replicateAllTransformersCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var replicateAllTransformersCommand = (function (_super) {
        __extends(replicateAllTransformersCommand, _super);
        function replicateAllTransformersCommand(db) {
            _super.call(this);
            this.db = db;
        }
        replicateAllTransformersCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var transformersUrl = '/databases/' + this.db.name + '/replication/replicate-transformers?op=replicate-all';
            this.post(transformersUrl, null, appUrl.getSystemDatabase(), { dataType: undefined })
                .fail(function (response) {
                _this.reportError("Failed to send replicate all transformers command!", response.responseText, response.statusText);
                promise.reject();
            }).done(function () {
                _this.reportSuccess("Sent replicate all transformers command");
                promise.resolve();
            });
            return promise;
        };
        return replicateAllTransformersCommand;
    })(commandBase);
    return replicateAllTransformersCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/replicateIndexesCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var replicateIndexesCommand = (function (_super) {
        __extends(replicateIndexesCommand, _super);
        function replicateIndexesCommand(db, destination) {
            _super.call(this);
            this.db = db;
            this.destination = destination;
        }
        replicateIndexesCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var indexesUrl = '/databases/' + this.db.name + '/replication/replicate-indexes?op=replicate-all-to-destination';
            var destinationJson = JSON.stringify(this.destination.toDto());
            this.post(indexesUrl, destinationJson, appUrl.getSystemDatabase())
                .fail(function (response) {
                _this.reportError("Failed to send replicate indexes command!", response.responseText, response.statusText);
                promise.reject();
            }).done(function () {
                _this.reportSuccess("Sent replicate indexes command.");
                promise.resolve();
            });
            return promise;
        };
        return replicateIndexesCommand;
    })(commandBase);
    return replicateIndexesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/replicateTransformersCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var replicateTransformersCommand = (function (_super) {
        __extends(replicateTransformersCommand, _super);
        function replicateTransformersCommand(db, destination) {
            _super.call(this);
            this.db = db;
            this.destination = destination;
        }
        replicateTransformersCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var transformersUrl = '/databases/' + this.db.name + '/replication/replicate-transformers?op=replicate-all-to-destination';
            var destinationJson = JSON.stringify(this.destination.toDto());
            this.post(transformersUrl, destinationJson, appUrl.getSystemDatabase())
                .fail(function (response) {
                _this.reportError("Failed to send replicate transformers command!", response.responseText, response.statusText);
                promise.reject();
            }).done(function () {
                _this.reportSuccess("Sent replicate transformers command");
                promise.resolve();
            });
            return promise;
        };
        return replicateTransformersCommand;
    })(commandBase);
    return replicateTransformersCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/resolveAllConflictsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resolveAllConflictsCommand = (function (_super) {
        __extends(resolveAllConflictsCommand, _super);
        function resolveAllConflictsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        resolveAllConflictsCommand.prototype.execute = function () {
            var _this = this;
            var promise = $.Deferred();
            var url = '/replication/forceConflictResolution';
            this.query(url, null, this.db)
                .fail(function (response) {
                _this.reportError("Failed to start conflict resolution task", response.responseText, response.statusText);
                promise.reject();
            }).done(function () {
                _this.reportSuccess("Started conflict resolution task (Status->Running Tasks)");
                promise.resolve();
            });
            return promise;
        };
        return resolveAllConflictsCommand;
    })(commandBase);
    return resolveAllConflictsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/saveAutomaticConflictResolutionDocument", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveAutomaticConflictResolutionDocument = (function (_super) {
        __extends(saveAutomaticConflictResolutionDocument, _super);
        function saveAutomaticConflictResolutionDocument(dto, db) {
            _super.call(this);
            this.dto = dto;
            this.db = db;
        }
        saveAutomaticConflictResolutionDocument.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving automatic conflict resolution settings.");
            return this.saveConfig()
                .done(function () { return _this.reportSuccess("Saved automatic conflict resolution settings."); })
                .fail(function (response) { return _this.reportError("Failed to save automatic conflict resolution settings.", response.responseText); });
        };
        saveAutomaticConflictResolutionDocument.prototype.saveConfig = function () {
            var id = "Raven/Replication/Config";
            var url = "/docs/" + id;
            var putArgs = JSON.stringify(this.dto);
            return this.put(url, putArgs, this.db);
        };
        return saveAutomaticConflictResolutionDocument;
    })(commandBase);
    return saveAutomaticConflictResolutionDocument;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/saveAutomaticConflictResolutionDocumentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveAutomaticConflictResolutionDocumentCommand = (function (_super) {
        __extends(saveAutomaticConflictResolutionDocumentCommand, _super);
        function saveAutomaticConflictResolutionDocumentCommand(dto, db, globalConfig) {
            if (globalConfig === void 0) { globalConfig = false; }
            _super.call(this);
            this.dto = dto;
            this.db = db;
            this.globalConfig = globalConfig;
        }
        saveAutomaticConflictResolutionDocumentCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving automatic conflict resolution settings.");
            return this.saveConfig()
                .done(function () { return _this.reportSuccess("Saved automatic conflict resolution settings."); })
                .fail(function (response) { return _this.reportError("Failed to save automatic conflict resolution settings.", response.responseText, response.statusText); });
        };
        saveAutomaticConflictResolutionDocumentCommand.prototype.saveConfig = function () {
            var id = this.globalConfig ? "Raven/Global/Replication/Config" : "Raven/Replication/Config";
            var url = "/docs/" + id;
            var putArgs = JSON.stringify(this.dto);
            return this.put(url, putArgs, this.db);
        };
        return saveAutomaticConflictResolutionDocumentCommand;
    })(commandBase);
    return saveAutomaticConflictResolutionDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/replication/saveReplicationDocumentCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveReplicationDocumentCommand = (function (_super) {
        __extends(saveReplicationDocumentCommand, _super);
        function saveReplicationDocumentCommand(dto, db, globalConfig) {
            if (globalConfig === void 0) { globalConfig = false; }
            _super.call(this);
            this.dto = dto;
            this.db = db;
            this.globalConfig = globalConfig;
        }
        saveReplicationDocumentCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Replication settings.");
            return this.saveSetup()
                .done(function () { return _this.reportSuccess("Saved Replication settings."); })
                .fail(function (response) { return _this.reportError("Failed to save Replication settings.", response.responseText, response.statusText); });
        };
        saveReplicationDocumentCommand.prototype.saveSetup = function () {
            var id = this.globalConfig ? "Raven/Global/Replication/Destinations" : "Raven/Replication/Destinations";
            var url = "/docs/" + id;
            var putArgs = JSON.stringify(this.dto);
            return this.put(url, putArgs, this.db);
        };
        return saveReplicationDocumentCommand;
    })(commandBase);
    return saveReplicationDocumentCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/sqlReplication/getSqlReplicationsCommand", ["require", "exports", "commands/commandBase", "models/database/sqlReplication/sqlReplication"], function (require, exports, commandBase, sqlReplication) {
    var getSqlReplicationsCommand = (function (_super) {
        __extends(getSqlReplicationsCommand, _super);
        function getSqlReplicationsCommand(db, sqlReplicationName) {
            if (sqlReplicationName === void 0) { sqlReplicationName = null; }
            _super.call(this);
            this.db = db;
            this.sqlReplicationName = sqlReplicationName;
        }
        getSqlReplicationsCommand.prototype.execute = function () {
            var args = {
                startsWith: "Raven/SqlReplication/Configuration/",
                exclude: null,
                start: 0,
                pageSize: 256
            };
            return this.query("/docs", args, this.db, function (dtos) { return dtos.map(function (dto) { return new sqlReplication(dto); }); });
        };
        return getSqlReplicationsCommand;
    })(commandBase);
    return getSqlReplicationsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/sqlReplication/resetSqlReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resetSqlReplicationCommand = (function (_super) {
        __extends(resetSqlReplicationCommand, _super);
        function resetSqlReplicationCommand(db, sqlReplicationName) {
            _super.call(this);
            this.db = db;
            this.sqlReplicationName = sqlReplicationName;
        }
        resetSqlReplicationCommand.prototype.execute = function () {
            var args = { sqlReplicationName: this.sqlReplicationName };
            var url = "/studio-tasks/reset-sql-replication" + _super.prototype.urlEncodeArgs.call(this, args);
            return this.post(url, null, this.db);
        };
        return resetSqlReplicationCommand;
    })(commandBase);
    return resetSqlReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/sqlReplication/saveSqlReplicationsCommand", ["require", "exports", "commands/database/documents/executeBulkDocsCommand"], function (require, exports, executeBulkDocsCommand) {
    var saveSqlReplicationsCommand = (function (_super) {
        __extends(saveSqlReplicationsCommand, _super);
        function saveSqlReplicationsCommand(onScreenReplications, deletedReplications, db) {
            var bulkPutReplications = onScreenReplications.map(function (sr) { return sr.toBulkDoc("PUT"); });
            var bulkDeleteReplications = deletedReplications.map(function (sr) { return sr.toBulkDoc("DELETE"); });
            _super.call(this, bulkPutReplications.concat(bulkDeleteReplications), db);
        }
        saveSqlReplicationsCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving SQL replications...");
            return _super.prototype.execute.call(this)
                .fail(function (result) { return _this.reportError("Unable to save SQL replications.", result.responseText, result.statusText); })
                .done(function () { return _this.reportSuccess("Saved SQL replications."); });
        };
        return saveSqlReplicationsCommand;
    })(executeBulkDocsCommand);
    return saveSqlReplicationsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/sqlReplication/testSqlConnectionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var testSqlConnectionCommand = (function (_super) {
        __extends(testSqlConnectionCommand, _super);
        function testSqlConnectionCommand(db, factoryName, connectionString) {
            _super.call(this);
            this.db = db;
            this.factoryName = factoryName;
            this.connectionString = connectionString;
        }
        testSqlConnectionCommand.prototype.execute = function () {
            var args = {
                factoryName: this.factoryName,
                connectionString: this.connectionString
            };
            return this.query("/studio-tasks/test-sql-replication-connection", args, this.db, null, 60000);
        };
        return testSqlConnectionCommand;
    })(commandBase);
    return testSqlConnectionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/checkEncryptionKey", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var checkEncryptionKey = (function (_super) {
        __extends(checkEncryptionKey, _super);
        function checkEncryptionKey(key) {
            _super.call(this);
            this.key = key;
        }
        checkEncryptionKey.prototype.execute = function () {
            var _this = this;
            var keyObject = { "key": this.key };
            var result = this.post("/studio-tasks/is-base-64-key", keyObject, null);
            result.fail(function (response) { return _this.reportError("Failed to create encryption", response.responseText, response.statusText); });
            return result;
        };
        return checkEncryptionKey;
    })(commandBase);
    return checkEncryptionKey;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/checksufficientdiskspaceCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var checkSufficientDiskSpaceCommand = (function (_super) {
        __extends(checkSufficientDiskSpaceCommand, _super);
        function checkSufficientDiskSpaceCommand(fileSize, database) {
            _super.call(this);
            this.fileSize = fileSize;
            this.database = database;
        }
        checkSufficientDiskSpaceCommand.prototype.execute = function () {
            var args = {
                fileSize: this.fileSize
            };
            var url = "/studio-tasks/check-sufficient-diskspace";
            var checkTask = this.query(url, args, this.database, undefined, { dataType: undefined });
            return checkTask;
        };
        return checkSufficientDiskSpaceCommand;
    })(commandBase);
    return checkSufficientDiskSpaceCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/createSampleDataClassCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createSampleDataClassCommand = (function (_super) {
        __extends(createSampleDataClassCommand, _super);
        function createSampleDataClassCommand(db) {
            _super.call(this);
            this.db = db;
        }
        createSampleDataClassCommand.prototype.execute = function () {
            return this.query("/studio-tasks/createSampleDataClass", null, this.db);
        };
        return createSampleDataClassCommand;
    })(commandBase);
    return createSampleDataClassCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/createSampleDataCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createSampleDataCommand = (function (_super) {
        __extends(createSampleDataCommand, _super);
        function createSampleDataCommand(db) {
            _super.call(this);
            this.db = db;
        }
        createSampleDataCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Creating Sample Data, Please wait...");
            return this.post("/studio-tasks/createSampleData", null, this.db, { dataType: 'text' })
                .fail(function (response) { return _this.reportError("Failed to create sample data", response.responseText, response.statusText); })
                .done(function () { return _this.reportSuccess("Sample data creation completed"); });
        };
        return createSampleDataCommand;
    })(commandBase);
    return createSampleDataCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getCachedCollectionsCount", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getCachedCollectionsCount = (function (_super) {
        __extends(getCachedCollectionsCount, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function getCachedCollectionsCount(ownerDb, lastQueryDate) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.lastQueryDate = lastQueryDate;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getCachedCollectionsCount.prototype.execute = function () {
            return this.query("/studio-tasks/collection/counts", { fromDate: this.lastQueryDate }, this.ownerDb);
        };
        return getCachedCollectionsCount;
    })(commandBase);
    return getCachedCollectionsCount;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getClientBuildVersionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getClientBuildVersionCommand = (function (_super) {
        __extends(getClientBuildVersionCommand, _super);
        function getClientBuildVersionCommand() {
            _super.apply(this, arguments);
        }
        getClientBuildVersionCommand.prototype.execute = function () {
            return this.query("/studio/version.json", null);
        };
        return getClientBuildVersionCommand;
    })(commandBase);
    return getClientBuildVersionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getLatestServerBuildVersionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getLatestServerBuildVersionCommand = (function (_super) {
        __extends(getLatestServerBuildVersionCommand, _super);
        function getLatestServerBuildVersionCommand(stableOnly, min, max) {
            _super.call(this);
            this.stableOnly = stableOnly;
            this.min = min;
            this.max = max;
        }
        getLatestServerBuildVersionCommand.prototype.execute = function () {
            var args = {
                stableOnly: this.stableOnly,
                min: this.min,
                max: this.max
            };
            return this.query("/studio-tasks/latest-server-build-version", args, null, null, this.getTimeToAlert(true));
        };
        return getLatestServerBuildVersionCommand;
    })(commandBase);
    return getLatestServerBuildVersionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getNewEncryptionKey", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getNewEncryptionKey = (function (_super) {
        __extends(getNewEncryptionKey, _super);
        function getNewEncryptionKey() {
            _super.call(this);
        }
        getNewEncryptionKey.prototype.execute = function () {
            var key = this.query("/studio-tasks/new-encryption-key", null, null);
            return key;
        };
        return getNewEncryptionKey;
    })(commandBase);
    return getNewEncryptionKey;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getServerConfigsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getServerConfigsCommand = (function (_super) {
        __extends(getServerConfigsCommand, _super);
        function getServerConfigsCommand() {
            _super.apply(this, arguments);
        }
        getServerConfigsCommand.prototype.execute = function () {
            return this.query("/studio-tasks/server-configs", null);
        };
        return getServerConfigsCommand;
    })(commandBase);
    return getServerConfigsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/getSqlReplicationStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSqlReplicationStatsCommand = (function (_super) {
        __extends(getSqlReplicationStatsCommand, _super);
        function getSqlReplicationStatsCommand(ownerDb, sqlReplicationName) {
            _super.call(this);
            this.ownerDb = ownerDb;
            this.sqlReplicationName = sqlReplicationName;
            if (!this.ownerDb) {
                throw new Error("Must specify a database.");
            }
        }
        getSqlReplicationStatsCommand.prototype.execute = function () {
            var args = {
                sqlReplicationName: this.sqlReplicationName
            };
            var url = "/studio-tasks/get-sql-replication-stats";
            var resultsSelector = function (result) {
                result.Value["Name"] = result.Key;
                var replicationDto = result.Value;
                return replicationDto;
            };
            return this.query(url, args, this.ownerDb, resultsSelector);
        };
        return getSqlReplicationStatsCommand;
    })(commandBase);
    return getSqlReplicationStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/importDatabaseCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var importDatabaseCommand = (function (_super) {
        __extends(importDatabaseCommand, _super);
        function importDatabaseCommand(fileData, batchSize, includeExpiredDocuments, stripReplicationInformation, shouldDisableVersioningBundle, operateOnTypes, filters, transformScript, db) {
            _super.call(this);
            this.fileData = fileData;
            this.batchSize = batchSize;
            this.includeExpiredDocuments = includeExpiredDocuments;
            this.stripReplicationInformation = stripReplicationInformation;
            this.shouldDisableVersioningBundle = shouldDisableVersioningBundle;
            this.operateOnTypes = operateOnTypes;
            this.filters = filters;
            this.transformScript = transformScript;
            this.db = db;
        }
        importDatabaseCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Importing data...");
            var args = {
                batchSize: this.batchSize,
                includeExpiredDocuments: this.includeExpiredDocuments,
                stripReplicationInformation: this.stripReplicationInformation,
                shouldDisableVersioningBundle: this.shouldDisableVersioningBundle,
                operateOnTypes: this.operateOnTypes.reduce(function (first, second) { return first | second; }, 0),
                filtersPipeDelimited: this.filters.map(function (f) { return f.Path + ";;;" + f.Values[0] + ";;;" + f.ShouldMatch; }).join("|||") || "",
                transformScript: this.transformScript || ""
            };
            var url = "/studio-tasks/import" + this.urlEncodeArgs(args);
            var ajaxOptions = {
                processData: false,
                contentType: false
            };
            var importTask = this.post(url, this.fileData, this.db, ajaxOptions);
            importTask.done(function () { return _this.reportInfo("Data was uploaded successfully, processing..."); });
            importTask.fail(function (response) { return _this.reportError("Failed to upload data", response.responseText, response.statusText); });
            return importTask;
        };
        return importDatabaseCommand;
    })(commandBase);
    return importDatabaseCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/resolveMergeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resolveMergeCommand = (function (_super) {
        __extends(resolveMergeCommand, _super);
        function resolveMergeCommand(db, documentId) {
            _super.call(this);
            this.db = db;
            this.documentId = documentId;
        }
        resolveMergeCommand.prototype.execute = function () {
            var url = "/databases/" + this.db.name + "/studio-tasks/resolveMerge";
            var args = {
                documentId: this.documentId
            };
            var task = this.query(url, args);
            return task;
        };
        return resolveMergeCommand;
    })(commandBase);
    return resolveMergeCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/saveCsvFileCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveCsvFileCommand = (function (_super) {
        __extends(saveCsvFileCommand, _super);
        function saveCsvFileCommand(fileData, fileName, db) {
            _super.call(this);
            this.fileData = fileData;
            this.fileName = fileName;
            this.db = db;
        }
        saveCsvFileCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Importing...");
            var customHeaders = {
                'X-FileName': this.fileName
            };
            var jQueryOptions = {
                headers: customHeaders,
                processData: false,
                contentType: false,
                dataType: 'text'
            };
            var saveTask = this.post("/studio-tasks/loadCsvFile", this.fileData, this.db, jQueryOptions);
            saveTask.done(function () { return _this.reportSuccess("CSV file imported"); });
            saveTask.fail(function (response) { return _this.reportError("Failed to import CSV file", response.responseText, response.statusText); });
            return saveTask;
        };
        return saveCsvFileCommand;
    })(commandBase);
    return saveCsvFileCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/simulateSqlReplicationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var simulateSqlReplicationCommand = (function (_super) {
        __extends(simulateSqlReplicationCommand, _super);
        function simulateSqlReplicationCommand(db, simulatedSqlReplication, documentId, performRolledbackTransaction) {
            _super.call(this);
            this.db = db;
            this.simulatedSqlReplication = simulatedSqlReplication;
            this.documentId = documentId;
            this.performRolledbackTransaction = performRolledbackTransaction;
        }
        simulateSqlReplicationCommand.prototype.execute = function () {
            var args = {
                documentId: this.documentId,
                performRolledBackTransaction: this.performRolledbackTransaction,
                sqlReplication: JSON.stringify(this.simulatedSqlReplication.toDto())
            };
            return this.post("/studio-tasks/simulate-sql-replication", JSON.stringify(args), this.db, { dataType: undefined }, 60000);
        };
        return simulateSqlReplicationCommand;
    })(commandBase);
    return simulateSqlReplicationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/studio/validateExportDatabaseOptionsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var validateExportDatabaseOptionsCommand = (function (_super) {
        __extends(validateExportDatabaseOptionsCommand, _super);
        function validateExportDatabaseOptionsCommand(smugglerOptions, db) {
            _super.call(this);
            this.smugglerOptions = smugglerOptions;
            this.db = db;
        }
        validateExportDatabaseOptionsCommand.prototype.execute = function () {
            return this.post("/studio-tasks/validateExportOptions", JSON.stringify(this.smugglerOptions), this.db);
        };
        return validateExportDatabaseOptionsCommand;
    })(commandBase);
    return validateExportDatabaseOptionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/subscriptions/getSubscriptionCommand", ["require", "exports", "commands/commandBase", "models/database/subscription/subscription"], function (require, exports, commandBase, Subscription) {
    var getSubscriptionCommand = (function (_super) {
        __extends(getSubscriptionCommand, _super);
        function getSubscriptionCommand(db) {
            _super.call(this);
            this.db = db;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        getSubscriptionCommand.prototype.execute = function () {
            var url = "/debug/subscriptions";
            var resultsSelector = function (subscriptions) { return subscriptions.map(function (subscription) {
                return new Subscription(subscription.SubscriptionId, subscription.AckEtag);
            }); };
            var rc = this.query(url, null, this.db, resultsSelector);
            return rc;
        };
        return getSubscriptionCommand;
    })(commandBase);
    return getSubscriptionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/subscriptions/setSubscriptionAckEtagCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var setSubscriptionAckEtagCommand = (function (_super) {
        __extends(setSubscriptionAckEtagCommand, _super);
        function setSubscriptionAckEtagCommand(db, id, newEtag) {
            _super.call(this);
            this.db = db;
            this.id = id;
            this.newEtag = newEtag;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        setSubscriptionAckEtagCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                id: this.id,
                newEtag: this.newEtag
            };
            var url = "/subscriptions/setSubscriptionAckEtag" + this.urlEncodeArgs(args);
            return this.post(url, null, this.db, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Successfully set acknowledged etag"); })
                .fail(function (response) { return _this.reportError("Failed to set subscription acknowledged etag!", response.responseText, response.statusText); });
        };
        return setSubscriptionAckEtagCommand;
    })(commandBase);
    return setSubscriptionAckEtagCommand;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/transformers/deleteTransformerCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteTransformerCommand = (function (_super) {
        __extends(deleteTransformerCommand, _super);
        function deleteTransformerCommand(transName, db) {
            _super.call(this);
            this.transName = transName;
            this.db = db;
        }
        deleteTransformerCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Deleting " + this.transName + "...");
            return this.del("/transformers/" + this.transName, null, this.db)
                .fail(function (response) { return _this.reportError("Failed to delete transformer " + _this.transName, response.responseText); })
                .done(function () { return _this.reportSuccess("Deleted " + _this.transName); });
        };
        return deleteTransformerCommand;
    })(commandBase);
    return deleteTransformerCommand;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/transformers/getSingleTransformerCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getSingleTransformerCommand = (function (_super) {
        __extends(getSingleTransformerCommand, _super);
        function getSingleTransformerCommand(tranName, db) {
            _super.call(this);
            this.tranName = tranName;
            this.db = db;
        }
        getSingleTransformerCommand.prototype.execute = function () {
            var getTransformerUrl = "/transformers/" + this.tranName;
            return this.query(getTransformerUrl, null, this.db);
        };
        return getSingleTransformerCommand;
    })(commandBase);
    return getSingleTransformerCommand;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/transformers/getTransformersCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getTransformersCommand = (function (_super) {
        __extends(getTransformersCommand, _super);
        function getTransformersCommand(db, skip, take) {
            if (skip === void 0) { skip = 0; }
            if (take === void 0) { take = 256; }
            _super.call(this);
            this.db = db;
            this.skip = skip;
            this.take = take;
        }
        getTransformersCommand.prototype.execute = function () {
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            var url = "/transformers" + this.urlEncodeArgs(args);
            return this.query(url, null, this.db);
        };
        return getTransformersCommand;
    })(commandBase);
    return getTransformersCommand;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/database/transformers/saveTransformerCommand", ["require", "exports", "commands/commandBase", "commands/database/transformers/getSingleTransformerCommand", "models/database/index/transformer"], function (require, exports, commandBase, getSingleTransformerCommand, transformer) {
    var saveTransformerCommand = (function (_super) {
        __extends(saveTransformerCommand, _super);
        function saveTransformerCommand(trans, db) {
            _super.call(this);
            this.trans = trans;
            this.db = db;
        }
        saveTransformerCommand.prototype.execute = function () {
            var _this = this;
            var doneTask = $.Deferred();
            var saveTransformerUrl = "/transformers/" + this.trans.name();
            var saveTransformerPutArgs = JSON.stringify(this.trans.toSaveDto());
            this.put(saveTransformerUrl, saveTransformerPutArgs, this.db)
                .fail(function (result) {
                _this.reportError("Unable to save transformer", result.responseText, result.statusText);
                doneTask.reject(result);
            })
                .done(function (result) {
                doneTask.resolve();
                new getSingleTransformerCommand(result.Transformer, _this.db).execute()
                    .fail(function (xhr) { return doneTask.reject(xhr); })
                    .done(function (savedTransformer) {
                    _this.reportSuccess("Saved " + _this.trans.name());
                    doneTask.resolve(new transformer().initFromSave(savedTransformer));
                });
            });
            return doneTask;
        };
        return saveTransformerCommand;
    })(commandBase);
    return saveTransformerCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/backupFilesystemCommand", ["require", "exports", "commands/commandBase", "commands/filesystem/getConfigurationByKeyCommand"], function (require, exports, commandBase, getConfigurationByKeyCommand) {
    var backupFilesystemCommand = (function (_super) {
        __extends(backupFilesystemCommand, _super);
        function backupFilesystemCommand(fs, backupLocation, updateBackupStatus, incremental) {
            _super.call(this);
            this.fs = fs;
            this.backupLocation = backupLocation;
            this.updateBackupStatus = updateBackupStatus;
            this.incremental = incremental;
        }
        backupFilesystemCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            var args = {
                BackupLocation: this.backupLocation,
                DatabaseDocument: null
            };
            this.post('/admin/backup?incremental=' + this.incremental, JSON.stringify(args), this.fs, { dataType: 'text' })
                .fail(function (response) {
                _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function () { return _this.getBackupStatus(result); });
            return result;
        };
        backupFilesystemCommand.prototype.getBackupStatus = function (result) {
            var _this = this;
            new getConfigurationByKeyCommand(this.fs, "Raven/Backup/Status")
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to fetch backup status!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (backupStatusAsString) {
                var backupStatus = JSON.parse(backupStatusAsString);
                _this.updateBackupStatus(backupStatus);
                if (backupStatus.IsRunning) {
                    setTimeout(function () { return _this.getBackupStatus(result); }, 1000);
                }
                else {
                    if (backupStatus.Success) {
                        _this.reportSuccess("Filesystem backup was successfully created!");
                    }
                    else {
                        var cause = backupStatus.Messages.last(function (x) { return x.Severity === 'Error'; });
                        _this.reportError("Failed to backup filesystem: " + cause.Message);
                    }
                    result.resolve();
                }
            });
        };
        return backupFilesystemCommand;
    })(commandBase);
    return backupFilesystemCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/createDefaultSettingsCommand", ["require", "exports", "models/filesystem/versioningEntry", "commands/commandBase", "commands/filesystem/saveVersioningCommand"], function (require, exports, versioningEntry, commandBase, saveVersioningCommand) {
    var createDefaultSettingsCommand = (function (_super) {
        __extends(createDefaultSettingsCommand, _super);
        function createDefaultSettingsCommand(fs, bundles) {
            _super.call(this);
            this.fs = fs;
            this.bundles = bundles;
        }
        createDefaultSettingsCommand.prototype.execute = function () {
            this.reportInfo("Creating default settings for '" + this.fs.name + "'...");
            var tasksToWatch = [];
            if (this.bundles.contains("Versioning")) {
                tasksToWatch.push(this.saveVersioningConfiguration());
            }
            if (tasksToWatch.length > 0) {
                return $.when.apply(null, tasksToWatch);
            }
            else {
                return $.Deferred().resolve();
            }
        };
        createDefaultSettingsCommand.prototype.createDefaultVersioning = function () {
            return new versioningEntry({
                Id: "Raven/Versioning/DefaultConfiguration",
                MaxRevisions: 5,
                Exclude: false,
                ExcludeUnlessExplicit: false,
                PurgeOnDelete: false
            });
        };
        createDefaultSettingsCommand.prototype.saveVersioningConfiguration = function () {
            return new saveVersioningCommand(this.fs, this.createDefaultVersioning().toDto()).execute();
        };
        return createDefaultSettingsCommand;
    })(commandBase);
    return createDefaultSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/createFilesystemCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createFilesystemCommand = (function (_super) {
        __extends(createFilesystemCommand, _super);
        /**
        * @param filesystemName The file system name we are creating.
        */
        function createFilesystemCommand(filesystemName, settings, securedSettings) {
            _super.call(this);
            this.filesystemName = filesystemName;
            this.settings = settings;
            this.securedSettings = securedSettings;
            if (!filesystemName) {
                this.reportError("File System must have a name!");
                throw new Error("File System must have a name!");
            }
        }
        createFilesystemCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Creating File System '" + this.filesystemName + "'");
            var filesystemDoc = {
                "Settings": this.settings,
                "SecuredSettings": this.securedSettings,
                "Disabled": false
            };
            var url = "/admin/fs/" + this.filesystemName;
            var createTask = this.put(url, JSON.stringify(filesystemDoc), null, { dataType: undefined });
            createTask.done(function () { return _this.reportSuccess(_this.filesystemName + " created"); });
            createTask.fail(function (response) { return _this.reportError("Failed to create file system", response.responseText, response.statusText); });
            return createTask;
        };
        return createFilesystemCommand;
    })(commandBase);
    return createFilesystemCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/createFolderCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createFilesystemFolderCommand = (function (_super) {
        __extends(createFilesystemFolderCommand, _super);
        function createFilesystemFolderCommand(fs) {
            _super.call(this);
            this.fs = fs;
            throw new Error("Not Implemented");
        }
        createFilesystemFolderCommand.prototype.execute = function () {
            throw new Error("Not Implemented");
        };
        return createFilesystemFolderCommand;
    })(commandBase);
    return createFilesystemFolderCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/deleteConfigurationKeyCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteConfigurationKeyCommand = (function (_super) {
        __extends(deleteConfigurationKeyCommand, _super);
        function deleteConfigurationKeyCommand(fs, name) {
            _super.call(this);
            this.fs = fs;
            this.name = name;
        }
        deleteConfigurationKeyCommand.prototype.execute = function () {
            var url = "/config?name=" + this.name;
            return this.del(url, null, this.fs);
        };
        return deleteConfigurationKeyCommand;
    })(commandBase);
    return deleteConfigurationKeyCommand;
});
/// <reference path="../../../Scripts/typings/jquery/jquery.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/deleteDestinationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteDestinationCommand = (function (_super) {
        __extends(deleteDestinationCommand, _super);
        function deleteDestinationCommand(fs, destination) {
            _super.call(this);
            this.fs = fs;
            this.destination = destination;
        }
        deleteDestinationCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            var serverUrl = this.destination.ServerUrl;
            var fileSystem = this.destination.FileSystem;
            this.query("/config", { name: "Raven/Synchronization/Destinations" }, this.fs)
                .done(function (data) {
                if (data && data.hasOwnProperty('Destinations')) {
                    var value = data['Destinations'];
                    if (!(value instanceof Array))
                        value = [value];
                    var dtos = value.map(function (x) { return x; });
                    dtos = dtos.filter(function (x) { return x.ServerUrl != serverUrl || x.FileSystem != fileSystem; });
                    data.Destinations = dtos;
                    var url = "/config?name=" + encodeURIComponent("Raven/Synchronization/Destinations");
                    _this.put(url, JSON.stringify(data), _this.fs)
                        .done(function () { return result.resolve(data); })
                        .fail(function (xhr, statusText, error) {
                        _this.reportError("Could not delete destination (server: " + serverUrl + ", filesystem: " + fileSystem + ")", error, statusText);
                    });
                }
            });
            return result;
        };
        return deleteDestinationCommand;
    })(commandBase);
    return deleteDestinationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/deleteFilesCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteFilesCommand = (function (_super) {
        __extends(deleteFilesCommand, _super);
        function deleteFilesCommand(fileIds, fs) {
            _super.call(this);
            this.fileIds = fileIds;
            this.fs = fs;
        }
        deleteFilesCommand.prototype.execute = function () {
            var _this = this;
            var deletionTasks = new Array();
            ;
            for (var i = 0; i < this.fileIds.length; i++) {
                var deleteCommand = this.deleteFile(this.fileIds[i]);
                deletionTasks.push(deleteCommand);
            }
            var successMessage = this.fileIds.length > 1 ? "Deleted " + this.fileIds.length + " files" : "Deleted " + this.fileIds[0];
            var combinedTask = $.when.apply($, deletionTasks)
                .done(function (x) { return _this.reportSuccess(successMessage); })
                .fail(function (response) { return _this.reportError("Failed to delete files", response.responseText, response.statusText); });
            return combinedTask;
        };
        deleteFilesCommand.prototype.deleteFile = function (fileId) {
            var url = "/files/" + encodeURIComponent(fileId);
            return this.del(url, null, this.fs, null, 9000 * this.fileIds.length);
        };
        return deleteFilesCommand;
    })(commandBase);
    return deleteFilesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/deleteFilesMatchingQueryCommand", ["require", "exports", "commands/commandBase", "commands/operations/getOperationStatusCommand"], function (require, exports, commandBase, getOperationStatusCommand) {
    var deleteFilesMatchingQueryCommand = (function (_super) {
        __extends(deleteFilesMatchingQueryCommand, _super);
        function deleteFilesMatchingQueryCommand(queryText, fs) {
            _super.call(this);
            this.queryText = queryText;
            this.fs = fs;
        }
        deleteFilesMatchingQueryCommand.prototype.execute = function () {
            var _this = this;
            var deleteTaskWithWait = $.Deferred();
            this.reportInfo("Deleting files matching query...");
            var args = {
                query: this.queryText
            };
            var url = "/search/" + this.urlEncodeArgs(args);
            var task = this.del(url, null, this.fs);
            task.done(function (result) { return _this.waitForOperationToComplete(_this.fs, result.OperationId, deleteTaskWithWait); });
            task.fail(function (response) { return _this.reportError("Error deleting files matching query", response.responseText, response.statusText); });
            return deleteTaskWithWait;
        };
        deleteFilesMatchingQueryCommand.prototype.waitForOperationToComplete = function (fs, operationId, task) {
            var _this = this;
            new getOperationStatusCommand(fs, operationId)
                .execute()
                .done(function (result) { return _this.deletionStatusRetrieved(fs, operationId, result, task); });
        };
        deleteFilesMatchingQueryCommand.prototype.deletionStatusRetrieved = function (fs, operationId, result, task) {
            var _this = this;
            if (result.Completed) {
                if (!result.Faulted && !result.Canceled) {
                    this.reportSuccess("Files deleted");
                    task.resolve();
                }
                else {
                    this.reportError("Error deleting files matching query");
                    task.reject();
                }
                fs.isImporting(false);
            }
            else {
                setTimeout(function () { return _this.waitForOperationToComplete(fs, operationId, task); }, 1000);
            }
        };
        return deleteFilesMatchingQueryCommand;
    })(commandBase);
    return deleteFilesMatchingQueryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/fsCheckSufficientDiskSpaceCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var fsCheckSufficientDiskSpaceCommand = (function (_super) {
        __extends(fsCheckSufficientDiskSpaceCommand, _super);
        function fsCheckSufficientDiskSpaceCommand(fileSize, filesystem) {
            _super.call(this);
            this.fileSize = fileSize;
            this.filesystem = filesystem;
        }
        fsCheckSufficientDiskSpaceCommand.prototype.execute = function () {
            var args = {
                fileSize: this.fileSize
            };
            var url = "/studio-tasks/check-sufficient-diskspace";
            var checkTask = this.query(url, args, this.filesystem, undefined, { dataType: undefined });
            return checkTask;
        };
        return fsCheckSufficientDiskSpaceCommand;
    })(commandBase);
    return fsCheckSufficientDiskSpaceCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getConfigurationByKeyCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getConfigurationByKeyCommand = (function (_super) {
        __extends(getConfigurationByKeyCommand, _super);
        function getConfigurationByKeyCommand(fs, name) {
            _super.call(this);
            this.fs = fs;
            this.name = name;
        }
        getConfigurationByKeyCommand.prototype.execute = function () {
            var _this = this;
            var url = "/config";
            var args = {
                name: this.name
            };
            var task = $.Deferred();
            this.query(url, args, this.fs)
                .done(function (data) {
                var prettifySpacing = 4;
                var configText = JSON.stringify(data, null, prettifySpacing);
                task.resolve(configText);
            })
                .fail(function (response) { return _this.reportError("Failed to retrieve filesystem configuration key. Does exist in this filesystem?", response.responseText, response.statusText); });
            return task;
        };
        return getConfigurationByKeyCommand;
    })(commandBase);
    return getConfigurationByKeyCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getConfigurationCommand", ["require", "exports", "commands/commandBase", "models/filesystem/configurationKey"], function (require, exports, commandBase, configurationKey) {
    var getConfigurationCommand = (function (_super) {
        __extends(getConfigurationCommand, _super);
        function getConfigurationCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        getConfigurationCommand.prototype.execute = function () {
            var _this = this;
            var url = "/config/non-generated";
            return this.query(url, null, this.fs, function (results) { return results.map(function (x) { return new configurationKey(_this.fs, x); }); })
                .fail(function (response) { return _this.reportError("Failed to retrieve filesystem configuration.", response.responseText, response.statusText); });
        };
        return getConfigurationCommand;
    })(commandBase);
    return getConfigurationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getConfigurationsByPrefixCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getConfigurationNamesByPrefixCommand = (function (_super) {
        __extends(getConfigurationNamesByPrefixCommand, _super);
        function getConfigurationNamesByPrefixCommand(fs, prefix) {
            _super.call(this);
            this.fs = fs;
            this.prefix = prefix;
        }
        getConfigurationNamesByPrefixCommand.prototype.execute = function () {
            var _this = this;
            var url = "/config/search";
            var args = {
                prefix: this.prefix
            };
            var task = $.Deferred();
            this.query(url, args, this.fs)
                .done(function (data) {
                task.resolve(data);
            })
                .fail(function (response) { return _this.reportError("Failed to search configuration keys for prefix: " + _this.prefix, response.responseText, response.statusText); });
            return task;
        };
        return getConfigurationNamesByPrefixCommand;
    })(commandBase);
    return getConfigurationNamesByPrefixCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getDestinationsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getDestinationsCommand = (function (_super) {
        __extends(getDestinationsCommand, _super);
        function getDestinationsCommand(fs, shouldResolveNotFoundAsNull) {
            _super.call(this);
            this.fs = fs;
            if (!fs) {
                throw new Error("Must specify filesystem");
            }
            this.shouldResolveNotFoundAsNull = shouldResolveNotFoundAsNull || false;
        }
        getDestinationsCommand.prototype.execute = function () {
            var url = "/config";
            var args = {
                name: "Raven/Synchronization/Destinations"
            };
            return this.query(url, args, this.fs);
        };
        return getDestinationsCommand;
    })(commandBase);
    return getDestinationsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFileCommand", ["require", "exports", "commands/commandBase", "models/filesystem/file", "models/filesystem/fileMetadata"], function (require, exports, commandBase, file, fileMetadata) {
    var getFileCommand = (function (_super) {
        __extends(getFileCommand, _super);
        function getFileCommand(fs, name) {
            _super.call(this);
            this.fs = fs;
            this.name = name;
        }
        getFileCommand.prototype.execute = function () {
            var _this = this;
            var url = "/files/" + encodeURIComponent(this.name);
            var resultsSelector = function (metadata) {
                var fileHeaders = new file();
                fileHeaders.id(_this.name);
                for (var property in metadata) {
                    var value = metadata[property];
                    if (value[0] === "{" || value[0] === "[")
                        metadata[property] = JSON.parse(value);
                }
                fileHeaders.__metadata = new fileMetadata(metadata);
                return fileHeaders;
            };
            return this.head(url, null, this.fs, resultsSelector);
        };
        return getFileCommand;
    })(commandBase);
    return getFileCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFilesCommand", ["require", "exports", "commands/commandBase", "models/filesystem/file", "common/pagedResultSet", "common/queryUtil"], function (require, exports, commandBase, file, pagedResultSet, queryUtil) {
    var getFilesystemFilesCommand = (function (_super) {
        __extends(getFilesystemFilesCommand, _super);
        function getFilesystemFilesCommand(fs, directory, skip, take) {
            _super.call(this);
            this.fs = fs;
            this.directory = directory;
            this.skip = skip;
            this.take = take;
        }
        getFilesystemFilesCommand.prototype.execute = function () {
            var filesTask = this.fetchFiles();
            var doneTask = $.Deferred();
            filesTask.done(function (results) {
                var files = results.Files.map(function (d) { return new file(d, true); });
                var totalCount = results.FileCount;
                doneTask.resolve(new pagedResultSet(files, totalCount));
            });
            filesTask.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        getFilesystemFilesCommand.prototype.fetchFiles = function () {
            var level = 1;
            if (this.directory) {
                var slashMatches = this.directory.count("/");
                if (slashMatches) {
                    level = level + slashMatches;
                }
            }
            var levelQuery = "__level:" + level;
            var args = {
                query: this.directory ? "__directoryName:" + queryUtil.escapeTerm(this.directory) + " AND " + levelQuery : levelQuery,
                start: this.skip,
                pageSize: this.take
            };
            var url = "/search";
            var task = this.query(url, args, this.fs, null);
            return task;
        };
        return getFilesystemFilesCommand;
    })(commandBase);
    return getFilesystemFilesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFilesConflictsCommand", ["require", "exports", "commands/commandBase", "models/filesystem/conflictItem"], function (require, exports, commandBase, conflictItem) {
    var getFilesConflictsCommand = (function (_super) {
        __extends(getFilesConflictsCommand, _super);
        function getFilesConflictsCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        getFilesConflictsCommand.prototype.execute = function () {
            var url = "/synchronization/Conflicts";
            var resultsSelector = function (page) { return page.Items.map(function (x) { return conflictItem.fromConflictItemDto(x); }); };
            return this.query(url, null, this.fs, resultsSelector);
        };
        return getFilesConflictsCommand;
    })(commandBase);
    return getFilesConflictsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFilesystemRevisionsCommand", ["require", "exports", "commands/commandBase", "models/filesystem/file", "common/pagedResultSet"], function (require, exports, commandBase, file, pagedResultSet) {
    var getFilesystemRevisionsCommand = (function (_super) {
        __extends(getFilesystemRevisionsCommand, _super);
        function getFilesystemRevisionsCommand(fs, skip, take) {
            _super.call(this);
            this.fs = fs;
            this.skip = skip;
            this.take = take;
        }
        getFilesystemRevisionsCommand.prototype.execute = function () {
            var _this = this;
            var filesTask = this.fetchFiles();
            var doneTask = $.Deferred();
            filesTask.done(function (results) {
                var wrappedResult = {
                    Files: results,
                    FileCount: (results.length === _this.take) ? _this.take + _this.skip + 10 : _this.skip + results.length,
                    Start: _this.skip,
                    PageSize: _this.take
                };
                var files = wrappedResult.Files.map(function (d) { return new file(d, false); });
                var totalCount = wrappedResult.FileCount;
                doneTask.resolve(new pagedResultSet(files, totalCount));
            });
            filesTask.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        getFilesystemRevisionsCommand.prototype.fetchFiles = function () {
            var args = {
                startsWith: '/',
                matches: '*/revisions/*',
                start: this.skip,
                pageSize: this.take
            };
            var url = "/files";
            var task = this.query(url, args, this.fs, null);
            return task;
        };
        return getFilesystemRevisionsCommand;
    })(commandBase);
    return getFilesystemRevisionsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFileSystemsCommand", ["require", "exports", "commands/commandBase", "models/filesystem/filesystem"], function (require, exports, commandBase, filesystem) {
    var getFileSystemsCommand = (function (_super) {
        __extends(getFileSystemsCommand, _super);
        function getFileSystemsCommand() {
            _super.apply(this, arguments);
        }
        getFileSystemsCommand.prototype.execute = function () {
            var args = {
                pageSize: 1024,
                getAdditionalData: true
            };
            var url = "/fs";
            var resultsSelector = function (fileSystems) {
                return fileSystems.map(function (fs) { return new filesystem(fs.Name, fs.IsAdminCurrentTenant, fs.Disabled, fs.IsLoaded, fs.Bundles); });
            };
            return this.query(url, args, null, resultsSelector);
        };
        return getFileSystemsCommand;
    })(commandBase);
    return getFileSystemsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFileSystemsStatusCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getFileSystemsStatusCommand = (function (_super) {
        __extends(getFileSystemsStatusCommand, _super);
        function getFileSystemsStatusCommand() {
            _super.apply(this, arguments);
        }
        getFileSystemsStatusCommand.prototype.execute = function () {
            var url = "/fs/status";
            var resultsSelector = function (response) { return response.Status; };
            return this.query(url, null, null, resultsSelector);
        };
        return getFileSystemsStatusCommand;
    })(commandBase);
    return getFileSystemsStatusCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFileSystemStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getFileSystemStatsCommand = (function (_super) {
        __extends(getFileSystemStatsCommand, _super);
        function getFileSystemStatsCommand(fs, longWait) {
            if (longWait === void 0) { longWait = false; }
            _super.call(this);
            this.fs = fs;
            this.longWait = longWait;
        }
        getFileSystemStatsCommand.prototype.execute = function () {
            var url = "/stats";
            return this.query(url, null, this.fs, null, this.getTimeToAlert(this.longWait));
        };
        return getFileSystemStatsCommand;
    })(commandBase);
    return getFileSystemStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getFoldersCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getFoldersStatsCommand = (function (_super) {
        __extends(getFoldersStatsCommand, _super);
        function getFoldersStatsCommand(fs, skip, take, directory) {
            _super.call(this);
            this.fs = fs;
            this.skip = skip;
            this.take = take;
            this.directory = directory;
        }
        getFoldersStatsCommand.prototype.execute = function () {
            var url = "/folders/Subdirectories";
            var args = {
                start: this.skip,
                pageSize: this.take
            };
            if (this.directory) {
                args.directory = this.directory;
            }
            return this.query(url, args, this.fs, function (result) { return result.map(function (x) {
                return {
                    key: x,
                    title: x.substring(x.lastIndexOf("/") + 1),
                    isLazy: true,
                    isFolder: true
                };
            }); });
        };
        return getFoldersStatsCommand;
    })(commandBase);
    return getFoldersStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getSynchronizationConfigCommand", ["require", "exports", "commands/commandBase", "models/filesystem/synchronizationConfig"], function (require, exports, commandBase, synchronizationConfig) {
    var getSynchronizationConfigCommand = (function (_super) {
        __extends(getSynchronizationConfigCommand, _super);
        function getSynchronizationConfigCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        getSynchronizationConfigCommand.prototype.execute = function () {
            var _this = this;
            var url = "/config";
            var args = {
                name: "Raven/Synchronization/Config"
            };
            var task = $.Deferred();
            this.query(url, args, this.fs)
                .done(function (data) {
                task.resolve(new synchronizationConfig(data));
            })
                .fail(function (response) {
                if (response.status !== 404) {
                    _this.reportError("Failed to retrieve filesystem synchronization config.", response.responseText, response.statusText);
                    task.reject();
                }
                else {
                    task.resolve(synchronizationConfig.empty());
                }
            });
            return task;
        };
        return getSynchronizationConfigCommand;
    })(commandBase);
    return getSynchronizationConfigCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getSyncIncomingActivitiesCommand", ["require", "exports", "commands/commandBase", "models/filesystem/synchronizationDetail"], function (require, exports, commandBase, synchronizationDetail) {
    var getSyncIncomingActivitiesCommand = (function (_super) {
        __extends(getSyncIncomingActivitiesCommand, _super);
        function getSyncIncomingActivitiesCommand(fs, activity, skip, take) {
            _super.call(this);
            this.fs = fs;
            this.activity = activity;
            this.skip = skip;
            this.take = take;
        }
        getSyncIncomingActivitiesCommand.prototype.execute = function () {
            var _this = this;
            var doneTask = $.Deferred();
            this.getActivities()
                .done(function (list) { return doneTask.resolve(list); })
                .fail(function (xhr) {
                _this.reportError("Failed to get outgoing and " + _this.activity + " synchronization activities.");
                doneTask.reject(xhr);
            });
            return doneTask;
        };
        getSyncIncomingActivitiesCommand.prototype.getActivities = function () {
            var _this = this;
            var url;
            var resultsSelector;
            switch (this.activity) {
                case 1 /* Active */:
                    url = "/synchronization/incoming";
                    resultsSelector = function (x) {
                        return {
                            TotalCount: x.TotalCount,
                            Items: x.Items.map(function (item) { return new synchronizationDetail(item, 1 /* Incoming */, _this.activity); })
                        };
                    };
                    break;
                case 3 /* Finished */:
                    url = "/synchronization/finished";
                    resultsSelector = function (x) {
                        return {
                            TotalCount: x.TotalCount,
                            Items: x.Items.map(function (item) {
                                var detail = new synchronizationDetail({
                                    FileName: item.FileName,
                                    DestinationUrl: "",
                                    FileETag: item.FileETag,
                                    Type: item.Type
                                }, 1 /* Incoming */, _this.activity);
                                if (item.Exception != null) {
                                    detail.AdditionalInfo(item.Exception);
                                }
                                return detail;
                            })
                        };
                    };
                    break;
                default:
                    throw new TypeError("Not supported synchronization activity type when attempting to retrieve incoming activities. Given type: " + this.activity);
            }
            return this.query(url, { start: this.skip, pageSize: this.take }, this.fs, resultsSelector, 15000);
        };
        return getSyncIncomingActivitiesCommand;
    })(commandBase);
    return getSyncIncomingActivitiesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getSyncOutgoingActivitiesCommand", ["require", "exports", "commands/commandBase", "models/filesystem/synchronizationDetail"], function (require, exports, commandBase, synchronizationDetail) {
    var getSyncOutgoingActivitiesCommand = (function (_super) {
        __extends(getSyncOutgoingActivitiesCommand, _super);
        function getSyncOutgoingActivitiesCommand(fs, activity, skip, take) {
            _super.call(this);
            this.fs = fs;
            this.activity = activity;
            this.skip = skip;
            this.take = take;
        }
        getSyncOutgoingActivitiesCommand.prototype.execute = function () {
            var _this = this;
            var doneTask = $.Deferred();
            this.getActivities()
                .done(function (list) { return doneTask.resolve(list); })
                .fail(function (xhr) {
                _this.reportError("Failed to get outgoing and " + _this.activity + " synchronization activities.");
                doneTask.reject(xhr);
            });
            return doneTask;
        };
        getSyncOutgoingActivitiesCommand.prototype.getActivities = function () {
            var _this = this;
            var url;
            switch (this.activity) {
                case 1 /* Active */:
                    url = "/synchronization/active";
                    break;
                case 2 /* Pending */:
                    url = "/synchronization/pending";
                    break;
                default:
                    throw new TypeError("Not supported synchronization activity type when attempting to retrieve outgoing activities. Given type: " + this.activity);
            }
            var resultsSelector = function (x) {
                return {
                    TotalCount: x.TotalCount,
                    Items: x.Items.map(function (item) { return new synchronizationDetail(item, 0 /* Outgoing */, _this.activity); })
                };
            };
            return this.query(url, { start: this.skip, pageSize: this.take }, this.fs, resultsSelector, 15000);
        };
        return getSyncOutgoingActivitiesCommand;
    })(commandBase);
    return getSyncOutgoingActivitiesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/getVersioningCommand", ["require", "exports", "commands/commandBase", "models/filesystem/versioningEntry", 'commands/filesystem/getConfigurationByKeyCommand'], function (require, exports, commandBase, versioningEntry, getConfigurationByKeyCommand) {
    var getVersioningCommand = (function (_super) {
        __extends(getVersioningCommand, _super);
        function getVersioningCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        getVersioningCommand.prototype.execute = function () {
            var result = $.Deferred();
            var getCommand = new getConfigurationByKeyCommand(this.fs, 'Raven/Versioning/DefaultConfiguration')
                .execute();
            getCommand.fail(function (xhr) { return result.fail(xhr); });
            getCommand.done(function (config) {
                result.resolve(new versioningEntry(JSON.parse(config)));
            });
            return result;
        };
        return getVersioningCommand;
    })(commandBase);
    return getVersioningCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/importFilesystemCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var importFilesystemCommand = (function (_super) {
        __extends(importFilesystemCommand, _super);
        function importFilesystemCommand(fileData, batchSize, stripReplicationInformation, shouldDisableVersioiningBundle, fs) {
            _super.call(this);
            this.fileData = fileData;
            this.batchSize = batchSize;
            this.stripReplicationInformation = stripReplicationInformation;
            this.shouldDisableVersioiningBundle = shouldDisableVersioiningBundle;
            this.fs = fs;
        }
        importFilesystemCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Importing data...");
            var args = {
                batchSize: this.batchSize,
                stripReplicationInformation: this.stripReplicationInformation,
                shouldDisableVersioningBundle: this.shouldDisableVersioiningBundle
            };
            var url = "/studio-tasks/import" + this.urlEncodeArgs(args);
            var ajaxOptions = {
                processData: false,
                contentType: false
            };
            var importTask = this.post(url, this.fileData, this.fs, ajaxOptions);
            importTask.done(function () { return _this.reportInfo("Data was uploaded successfully, processing..."); });
            importTask.fail(function (response) { return _this.reportError("Failed to upload data", response.responseText, response.statusText); });
            return importTask;
        };
        return importFilesystemCommand;
    })(commandBase);
    return importFilesystemCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/monitorCompactCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl"], function (require, exports, commandBase, getDocumentWithMetadataCommand, appUrl) {
    var monitorCompactCommand = (function (_super) {
        __extends(monitorCompactCommand, _super);
        function monitorCompactCommand(parentPromise, fsName, updateCompactStatus) {
            _super.call(this);
            this.parentPromise = parentPromise;
            this.fsName = fsName;
            this.updateCompactStatus = updateCompactStatus;
            this.db = appUrl.getSystemDatabase();
        }
        monitorCompactCommand.prototype.execute = function () {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/FileSystem/Compact/Status/" + this.fsName, this.db)
                .execute()
                .fail(function (response) {
                setTimeout(function () { return _this.execute(); }, 1000);
            })
                .done(function (compactStatus) {
                _this.updateCompactStatus(compactStatus);
                if (compactStatus.State == "Running") {
                    setTimeout(function () { return _this.execute(); }, 1000);
                }
                else {
                    if (compactStatus.State == "Completed") {
                        _this.reportSuccess("File system was successfully compacted!");
                        _this.parentPromise.resolve();
                    }
                    else {
                        _this.reportError("File system wasn't compacted!");
                        _this.parentPromise.reject();
                    }
                }
            });
            return this.parentPromise;
        };
        return monitorCompactCommand;
    })(commandBase);
    return monitorCompactCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/monitorRestoreCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl"], function (require, exports, commandBase, getDocumentWithMetadataCommand, appUrl) {
    var monitorRestoreCommand = (function (_super) {
        __extends(monitorRestoreCommand, _super);
        function monitorRestoreCommand(parentPromise, filesystemName, updateRestoreStatus) {
            _super.call(this);
            this.parentPromise = parentPromise;
            this.filesystemName = filesystemName;
            this.updateRestoreStatus = updateRestoreStatus;
            this.db = appUrl.getSystemDatabase();
        }
        monitorRestoreCommand.prototype.execute = function () {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/FileSystem/Restore/Status/" + this.filesystemName, this.db)
                .execute()
                .fail(function (response) {
                setTimeout(function () { return _this.execute(); }, 1000);
            })
                .done(function (restoreStatus) {
                _this.updateRestoreStatus(restoreStatus);
                if (restoreStatus.State == "Running") {
                    setTimeout(function () { return _this.execute(); }, 1000);
                }
                else {
                    if (restoreStatus.State == "Completed") {
                        _this.reportSuccess("Filesystem was successfully restored!");
                        _this.parentPromise.resolve();
                    }
                    else {
                        _this.reportError("Filesystem wasn't restored!");
                        _this.parentPromise.reject();
                    }
                }
            });
            return this.parentPromise;
        };
        return monitorRestoreCommand;
    })(commandBase);
    return monitorRestoreCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/renameFileCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var renameFileCommand = (function (_super) {
        __extends(renameFileCommand, _super);
        function renameFileCommand(fs, oldName, newName) {
            _super.call(this);
            this.fs = fs;
            this.oldName = oldName;
            this.newName = newName;
        }
        renameFileCommand.prototype.execute = function () {
            var _this = this;
            var url = "/files/" + encodeURIComponent(this.oldName) + "?rename=" + encodeURIComponent(this.newName);
            return this.patch(url, null, this.fs)
                .done(function () { return _this.reportSuccess("Successfully renamed file"); })
                .fail(function (response) { return _this.reportError("Unable to rename file name", response.responseText, response.statusText); });
        };
        return renameFileCommand;
    })(commandBase);
    return renameFileCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/resetIndexCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resetIndexCommand = (function (_super) {
        __extends(resetIndexCommand, _super);
        function resetIndexCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        resetIndexCommand.prototype.execute = function () {
            var _this = this;
            var url = "/admin/reset-index";
            return this.post(url, null, this.fs, { dataType: 'text' })
                .done(function () { return _this.reportSuccess("File system index successfully reset"); })
                .fail(function (response) { return _this.reportError("Failed to reset file system index.", response.responseText, response.statusText); });
        };
        return resetIndexCommand;
    })(commandBase);
    return resetIndexCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/resolveConflictCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var resolveConflictCommand = (function (_super) {
        __extends(resolveConflictCommand, _super);
        function resolveConflictCommand(fileName, strategy, fs, reportResolveProgress) {
            if (reportResolveProgress === void 0) { reportResolveProgress = true; }
            _super.call(this);
            this.fileName = fileName;
            this.strategy = strategy;
            this.fs = fs;
            this.reportResolveProgress = reportResolveProgress;
        }
        resolveConflictCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportResolveProgress) {
                this.reportInfo("Resolving conflicts for file " + this.fileName + "...");
            }
            var url = "/synchronization/resolveConflict/" + encodeURIComponent(this.fileName) + "?strategy=" + this.strategy;
            var resolveTask = this.patch(url, null, this.fs);
            if (this.reportResolveProgress) {
                resolveTask.done(function () { return _this.reportSuccess("Resolved conflicts for file " + _this.fileName); });
                resolveTask.fail(function (response) {
                    _this.reportError("Failed to resolve conflicts for file " + _this.fileName, response.responseText, response.statusText);
                });
            }
            return resolveTask;
        };
        return resolveConflictCommand;
    })(commandBase);
    return resolveConflictCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/saveConfigurationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveFilesystemConfigurationCommand = (function (_super) {
        __extends(saveFilesystemConfigurationCommand, _super);
        function saveFilesystemConfigurationCommand(fs, key, args) {
            _super.call(this);
            this.fs = fs;
            this.key = key;
            this.args = args;
        }
        saveFilesystemConfigurationCommand.prototype.execute = function () {
            var _this = this;
            var url = "/config?name=" + encodeURIComponent(this.key.key);
            return this.put(url, JSON.stringify(this.args), this.fs)
                .done(function () { return _this.reportSuccess("Saved configuration"); });
        };
        return saveFilesystemConfigurationCommand;
    })(commandBase);
    return saveFilesystemConfigurationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/saveDestinationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var saveDestinationCommand = (function (_super) {
        __extends(saveDestinationCommand, _super);
        function saveDestinationCommand(dto, fs) {
            _super.call(this);
            this.dto = dto;
            this.fs = fs;
        }
        saveDestinationCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Replication destinations.");
            return this.saveSetup()
                .done(function () { return _this.reportSuccess("Saved Replication destinations."); })
                .fail(function (response) { return _this.reportError("Failed to save Replication destinations.", response.responseText, response.statusText); });
        };
        saveDestinationCommand.prototype.saveSetup = function () {
            var name = "Raven/Synchronization/Destinations";
            var url = "/config?name=" + encodeURIComponent(name);
            var putArgs = JSON.stringify(this.dto);
            return this.put(url, putArgs, this.fs);
        };
        return saveDestinationCommand;
    })(commandBase);
    return saveDestinationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/saveVersioningCommand", ["require", "exports", "commands/commandBase", 'commands/filesystem/saveConfigurationCommand', 'models/filesystem/configurationKey'], function (require, exports, commandBase, saveConfigurationCommand, configurationKey) {
    var saveVersioningCommand = (function (_super) {
        __extends(saveVersioningCommand, _super);
        function saveVersioningCommand(fs, versioningEntry) {
            _super.call(this);
            this.fs = fs;
            this.versioningEntry = versioningEntry;
        }
        saveVersioningCommand.prototype.execute = function () {
            return new saveConfigurationCommand(this.fs, new configurationKey(this.fs, 'Raven/Versioning/DefaultConfiguration'), this.versioningEntry)
                .execute();
        };
        return saveVersioningCommand;
    })(commandBase);
    return saveVersioningCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/searchByQueryCommand", ["require", "exports", "commands/commandBase", "common/pagedResultSet", "models/filesystem/file"], function (require, exports, commandBase, pagedResultSet, file) {
    var searchByQueryCommand = (function (_super) {
        __extends(searchByQueryCommand, _super);
        function searchByQueryCommand(fs, queryParameter, skip, take) {
            _super.call(this);
            this.fs = fs;
            this.queryParameter = queryParameter;
            this.skip = skip;
            this.take = take;
        }
        searchByQueryCommand.prototype.execute = function () {
            var filesTask = this.fetchFiles();
            var doneTask = $.Deferred();
            filesTask.done(function (results) { return doneTask.resolve(new pagedResultSet(results.Files.map(function (d) { return new file(d, false); }), results.FileCount)); });
            filesTask.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        searchByQueryCommand.prototype.fetchFiles = function () {
            var args = {
                query: this.queryParameter,
                start: this.skip,
                pageSize: this.take
            };
            var url = "/search";
            return this.query(url, args, this.fs);
        };
        return searchByQueryCommand;
    })(commandBase);
    return searchByQueryCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/searchByTermCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var searchFilesystemByTermCommand = (function (_super) {
        __extends(searchFilesystemByTermCommand, _super);
        function searchFilesystemByTermCommand(fs, queryParameter) {
            _super.call(this);
            this.fs = fs;
            this.queryParameter = queryParameter;
        }
        searchFilesystemByTermCommand.prototype.execute = function () {
            var url = "/search/Terms";
            return this.query(url, { query: this.queryParameter }, this.fs);
        };
        return searchFilesystemByTermCommand;
    })(commandBase);
    return searchFilesystemByTermCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/searchConfigurationByPrefixCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var searchFilesystemConfigurationByPrefixCommand = (function (_super) {
        __extends(searchFilesystemConfigurationByPrefixCommand, _super);
        function searchFilesystemConfigurationByPrefixCommand(fs, prefix) {
            _super.call(this);
            this.fs = fs;
            this.prefix = prefix;
        }
        searchFilesystemConfigurationByPrefixCommand.prototype.execute = function () {
            var url = "/config";
            var args = {
                prefix: this.prefix
            };
            return this.query(url, args, this.fs);
        };
        return searchFilesystemConfigurationByPrefixCommand;
    })(commandBase);
    return searchFilesystemConfigurationByPrefixCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/startCompactCommand", ["require", "exports", "commands/commandBase", "common/appUrl", "commands/database/documents/deleteDocumentCommand", "commands/filesystem/monitorCompactCommand"], function (require, exports, commandBase, appUrl, deleteDocumentCommand, monitorCompactCommand) {
    var startCompactCommand = (function (_super) {
        __extends(startCompactCommand, _super);
        function startCompactCommand(fsToCompact, updateCompactStatus) {
            _super.call(this);
            this.fsToCompact = fsToCompact;
            this.updateCompactStatus = updateCompactStatus;
            this.db = appUrl.getSystemDatabase();
        }
        startCompactCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            new deleteDocumentCommand('Raven/Database/FileSystem/Status/' + this.fsToCompact, this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to delete compact status document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (_) {
                var url = '/admin/fs/compact' + _this.urlEncodeArgs({ filesystem: _this.fsToCompact });
                _this.post(url, null, _this.db)
                    .fail(function (response) {
                    _this.reportError("Failed to compact filesystem!", response.responseText, response.statusText);
                    _this.logError(response, result);
                })
                    .done(function () { return new monitorCompactCommand(result, _this.fsToCompact, _this.updateCompactStatus).execute(); });
            });
            return result;
        };
        startCompactCommand.prototype.logError = function (response, result) {
            var r = JSON.parse(response.responseText);
            var compactStatus = { Messages: [r.Error], LastProgressMessage: "", State: "Faulted" };
            this.updateCompactStatus(compactStatus);
            result.reject();
        };
        return startCompactCommand;
    })(commandBase);
    return startCompactCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/startRestoreCommand", ["require", "exports", "commands/database/documents/deleteDocumentCommand", "commands/commandBase", "commands/filesystem/monitorRestoreCommand", "common/appUrl"], function (require, exports, deleteDocumentCommand, commandBase, monitorRestoreCommand, appUrl) {
    var startRestoreCommand = (function (_super) {
        __extends(startRestoreCommand, _super);
        function startRestoreCommand(defrag, restoreRequest, updateRestoreStatus) {
            _super.call(this);
            this.defrag = defrag;
            this.restoreRequest = restoreRequest;
            this.updateRestoreStatus = updateRestoreStatus;
            this.db = appUrl.getSystemDatabase();
        }
        startRestoreCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            new deleteDocumentCommand('Raven/FileSystem/Restore/Status/' + this.restoreRequest.FilesystemName, this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to delete restore status document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (_) {
                _this.post('/admin/fs/restore?defrag=' + _this.defrag, ko.toJSON(_this.restoreRequest), null, { dataType: 'text' })
                    .fail(function (response) {
                    _this.reportError("Failed to restore backup!", response.responseText, response.statusText);
                    _this.logError(response, result);
                    result.reject();
                })
                    .done(function () { return new monitorRestoreCommand(result, _this.restoreRequest.FilesystemName, _this.updateRestoreStatus).execute(); });
            });
            return result;
        };
        startRestoreCommand.prototype.logError = function (response, result) {
            var r = JSON.parse(response.responseText);
            var restoreStatus = { Messages: [r.Error], State: "Faulted" };
            this.updateRestoreStatus(restoreStatus);
            result.reject();
        };
        return startRestoreCommand;
    })(commandBase);
    return startRestoreCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/synchronizeNowCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var synchronizeNowCommand = (function (_super) {
        __extends(synchronizeNowCommand, _super);
        function synchronizeNowCommand(fs) {
            _super.call(this);
            this.fs = fs;
        }
        synchronizeNowCommand.prototype.execute = function () {
            var _this = this;
            var synchronizeUrl = "/synchronization/ToDestinations?forceSyncingAll=true";
            return this.post(synchronizeUrl, null, this.fs)
                .fail(function (x) {
                _this.reportError("Synchronization endpoint returned with an error.");
            });
        };
        return synchronizeNowCommand;
    })(commandBase);
    return synchronizeNowCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/synchronizeWithDestinationCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var synchronizeWithDestinationCommand = (function (_super) {
        __extends(synchronizeWithDestinationCommand, _super);
        function synchronizeWithDestinationCommand(fs, destination) {
            _super.call(this);
            this.fs = fs;
            this.destination = destination;
        }
        synchronizeWithDestinationCommand.prototype.execute = function () {
            var _this = this;
            var synchronizeUrl = "/synchronization/ToDestination?destination=" + this.destination + "&forceSyncingAll=true";
            return this.post(synchronizeUrl, null, this.fs)
                .fail(function (x) {
                _this.reportError("Synchronization endpoint returned with an error.");
            });
        };
        return synchronizeWithDestinationCommand;
    })(commandBase);
    return synchronizeWithDestinationCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/updateFileMetadataCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var updateFileMetadataCommand = (function (_super) {
        __extends(updateFileMetadataCommand, _super);
        function updateFileMetadataCommand(fileName, metadata, fs, reportSaveProgress) {
            if (reportSaveProgress === void 0) { reportSaveProgress = true; }
            _super.call(this);
            this.fileName = fileName;
            this.metadata = metadata;
            this.fs = fs;
            this.reportSaveProgress = reportSaveProgress;
        }
        updateFileMetadataCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportSaveProgress) {
                this.reportInfo("Saving " + this.fileName + "...");
            }
            var customHeaders = {};
            //We only want to stringify when the header's value is a json doc.
            for (var key in this.metadata) {
                var value = this.metadata[key];
                if (typeof (value) != "string" && typeof (value) != "number")
                    customHeaders[key] = JSON.stringify(value);
                else {
                    customHeaders[key] = this.metadata[key];
                }
            }
            var jQueryOptions = {
                headers: customHeaders
            };
            var url = "/files/" + encodeURIComponent(this.fileName);
            var updateTask = this.post(url, null, this.fs, jQueryOptions);
            if (this.reportSaveProgress) {
                updateTask.done(function () { return _this.reportSuccess("Saved " + _this.fileName); });
                updateTask.fail(function (response) {
                    _this.reportError("Failed to save " + _this.fileName, response.responseText, response.statusText);
                });
            }
            return updateTask;
        };
        return updateFileMetadataCommand;
    })(commandBase);
    return updateFileMetadataCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/filesystem/uploadFileToFilesystemCommand", ["require", "exports", "commands/commandBase", "models/filesystem/uploadItem"], function (require, exports, commandBase, uploadItem) {
    var uploadFileToFilesystemCommand = (function (_super) {
        __extends(uploadFileToFilesystemCommand, _super);
        /**
        * @param ownerDb The database the collections will belong to.
        */
        function uploadFileToFilesystemCommand(source, directory, uploadId, fs, progressHandlingFunction, reportUploadProgress) {
            if (reportUploadProgress === void 0) { reportUploadProgress = true; }
            _super.call(this);
            this.source = source;
            this.directory = directory;
            this.uploadId = uploadId;
            this.fs = fs;
            this.reportUploadProgress = reportUploadProgress;
        }
        uploadFileToFilesystemCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportUploadProgress) {
                this.reportInfo("File " + this.source.name + "queued for upload...");
            }
            var fileName = this.directory + "/" + this.source.name;
            var url = '/files?name=' + encodeURIComponent(fileName) + '&uploadId=' + this.uploadId;
            var customHeaders = {
                'RavenFS-Size': this.source.size
            };
            var deferred = $.Deferred();
            var jQueryOptions = {
                headers: customHeaders,
                processData: false,
                xhr: function () {
                    var myXhr = $.ajaxSettings.xhr();
                    if (myXhr.upload) {
                        myXhr.upload.addEventListener('progress', this.progressHandlingFunction, false); // For handling the progress of the upload
                    }
                    if (myXhr.upload) {
                        myXhr.upload.onprogress = this.progressHandlingFunction;
                    }
                    return myXhr;
                },
                cache: false,
                contentType: "application/json; charset=UTF-8",
                dataType: ''
            };
            var uploadTask = this.put(url, this.source, this.fs, jQueryOptions, 0);
            if (this.reportUploadProgress) {
                uploadTask.done(function () {
                    _this.reportSuccess("Uploaded " + fileName);
                    return deferred.resolve(new uploadItem(_this.uploadId, fileName, "Uploaded", _this.fs));
                });
                uploadTask.fail(function (response) {
                    _this.reportError("Failed to upload " + fileName, response.responseText, response.statusText);
                    return deferred.reject(new uploadItem(_this.uploadId, fileName, "Failed", _this.fs));
                });
            }
            return deferred;
        };
        return uploadFileToFilesystemCommand;
    })(commandBase);
    return uploadFileToFilesystemCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/licensing/activateHotSpareCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var activateHotSpareCommand = (function (_super) {
        __extends(activateHotSpareCommand, _super);
        function activateHotSpareCommand() {
            _super.apply(this, arguments);
        }
        activateHotSpareCommand.prototype.execute = function () {
            var _this = this;
            return this.query("/admin/activate-hotspare", null, null, null, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Activated Hot Spare mode"); })
                .fail(function (response) {
                _this.reportError("Failed to activate Hot Spare mode!", response.responseText, response.statusText);
            });
        };
        return activateHotSpareCommand;
    })(commandBase);
    return activateHotSpareCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/licensing/GetHotSpareInformation", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var GetHotSpareInformation = (function (_super) {
        __extends(GetHotSpareInformation, _super);
        function GetHotSpareInformation() {
            _super.apply(this, arguments);
        }
        GetHotSpareInformation.prototype.execute = function () {
            return this.query("/admin/get-hotspare-information", null, null, null);
        };
        return GetHotSpareInformation;
    })(commandBase);
    return GetHotSpareInformation;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/licensing/testHotSpareCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var testHotSpareCommand = (function (_super) {
        __extends(testHotSpareCommand, _super);
        function testHotSpareCommand() {
            _super.apply(this, arguments);
        }
        testHotSpareCommand.prototype.execute = function () {
            var _this = this;
            return this.query("/admin/test-hotspare", null, null, null, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Activated Hot Spare test mode"); })
                .fail(function (response) {
                _this.reportError("Failed to enable Hot Spare test mode!", response.responseText, response.statusText);
            });
        };
        return testHotSpareCommand;
    })(commandBase);
    return testHotSpareCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/adminJsScriptCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var adminJsScriptCommand = (function (_super) {
        __extends(adminJsScriptCommand, _super);
        function adminJsScriptCommand(script, targetDatabase) {
            _super.call(this);
            this.script = script;
            this.targetDatabase = targetDatabase;
            this.db = appUrl.getSystemDatabase();
        }
        adminJsScriptCommand.prototype.execute = function () {
            var _this = this;
            var url = "/admin/console/" + this.targetDatabase.toString();
            return this.post(url, ko.toJSON({ script: this.script }), this.db)
                .done(function () { return _this.reportSuccess("Script executed"); })
                .fail(function (response) { return _this.reportError("Script failed", response.responseText, response.statusText); });
        };
        return adminJsScriptCommand;
    })(commandBase);
    return adminJsScriptCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/backupDatabaseCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, commandBase, getDocumentWithMetadataCommand) {
    var backupDatabaseCommand = (function (_super) {
        __extends(backupDatabaseCommand, _super);
        function backupDatabaseCommand(db, backupLocation, updateBackupStatus, incremental) {
            _super.call(this);
            this.db = db;
            this.backupLocation = backupLocation;
            this.updateBackupStatus = updateBackupStatus;
            this.incremental = incremental;
        }
        backupDatabaseCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            this.query('/admin/databases/' + this.db.name, null, null /* We should query the system URL here */)
                .fail(function (response) {
                _this.reportError("Failed to fetch database document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (doc) {
                var args = {
                    BackupLocation: _this.backupLocation,
                    DatabaseDocument: doc
                };
                _this.post('/admin/backup?incremental=' + _this.incremental, JSON.stringify(args), _this.db, { dataType: 'text' })
                    .fail(function (response) {
                    _this.reportError("Failed to create backup!", response.responseText, response.statusText);
                    result.reject();
                })
                    .done(function () { return _this.getBackupStatus(result); });
            });
            return result;
        };
        backupDatabaseCommand.prototype.getBackupStatus = function (result) {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/Backup/Status", this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to fetch backup status!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (backupStatus) {
                _this.updateBackupStatus(backupStatus);
                if (backupStatus.IsRunning) {
                    setTimeout(function () { return _this.getBackupStatus(result); }, 1000);
                }
                else {
                    if (backupStatus.Success) {
                        _this.reportSuccess("Database backup was successfully created!");
                    }
                    else {
                        var cause = backupStatus.Messages.last(function (x) { return x.Severity === 'Error'; });
                        _this.reportError("Failed to backup database: " + cause.Message);
                    }
                    result.resolve();
                }
            });
        };
        return backupDatabaseCommand;
    })(commandBase);
    return backupDatabaseCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/listDiskPerformanceRunsCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var listDiskPerformanceRunsCommand = (function (_super) {
        __extends(listDiskPerformanceRunsCommand, _super);
        function listDiskPerformanceRunsCommand() {
            _super.call(this);
            this.db = appUrl.getSystemDatabase();
        }
        listDiskPerformanceRunsCommand.prototype.execute = function () {
            var args = {
                fetch: "DisplayName"
            };
            var mapper = function (x) {
                return {
                    displayName: x.DisplayName,
                    documentId: x["__document_id"]
                };
            };
            return this.query("/indexes/dynamic/DiskIoPerformanceRuns", args, this.db, function (r) { return r.Results.map(mapper); });
        };
        return listDiskPerformanceRunsCommand;
    })(commandBase);
    return listDiskPerformanceRunsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/monitorCompactCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl"], function (require, exports, commandBase, getDocumentWithMetadataCommand, appUrl) {
    var monitorCompactCommand = (function (_super) {
        __extends(monitorCompactCommand, _super);
        function monitorCompactCommand(parentPromise, dbName, updateCompactStatus) {
            _super.call(this);
            this.parentPromise = parentPromise;
            this.dbName = dbName;
            this.updateCompactStatus = updateCompactStatus;
            this.db = appUrl.getSystemDatabase();
        }
        monitorCompactCommand.prototype.execute = function () {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/Database/Compact/Status/" + this.dbName, this.db)
                .execute()
                .fail(function (response) {
                setTimeout(function () { return _this.execute(); }, 1000);
            })
                .done(function (compactStatus) {
                _this.updateCompactStatus(compactStatus);
                if (compactStatus.State == "Running") {
                    setTimeout(function () { return _this.execute(); }, 1000);
                }
                else {
                    if (compactStatus.State == "Completed") {
                        _this.reportSuccess("Database was successfully compacted!");
                        _this.parentPromise.resolve();
                    }
                    else {
                        _this.reportError("Database wasn't compacted!");
                        _this.parentPromise.reject();
                    }
                }
            });
            return this.parentPromise;
        };
        return monitorCompactCommand;
    })(commandBase);
    return monitorCompactCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/monitorRestoreCommand", ["require", "exports", "commands/commandBase", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl"], function (require, exports, commandBase, getDocumentWithMetadataCommand, appUrl) {
    var monitorRestoreCommand = (function (_super) {
        __extends(monitorRestoreCommand, _super);
        function monitorRestoreCommand(parentPromise, updateRestoreStatus) {
            _super.call(this);
            this.parentPromise = parentPromise;
            this.updateRestoreStatus = updateRestoreStatus;
            this.db = appUrl.getSystemDatabase();
        }
        monitorRestoreCommand.prototype.execute = function () {
            var _this = this;
            new getDocumentWithMetadataCommand("Raven/Restore/Status", this.db)
                .execute()
                .fail(function (response) {
                setTimeout(function () { return _this.execute(); }, 1000);
            })
                .done(function (restoreStatus) {
                _this.updateRestoreStatus(restoreStatus);
                if (restoreStatus.State == "Running") {
                    setTimeout(function () { return _this.execute(); }, 1000);
                }
                else {
                    if (restoreStatus.State == "Completed") {
                        _this.reportSuccess("Database was successfully restored!");
                        _this.parentPromise.resolve();
                    }
                    else {
                        _this.reportError("Database wasn't restored!");
                        _this.parentPromise.reject();
                    }
                }
            });
            return this.parentPromise;
        };
        return monitorRestoreCommand;
    })(commandBase);
    return monitorRestoreCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/savePeriodicExportSetupCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var savePeriodicExportSetupCommand = (function (_super) {
        __extends(savePeriodicExportSetupCommand, _super);
        function savePeriodicExportSetupCommand(setupToPersist, db, globalConfig) {
            if (globalConfig === void 0) { globalConfig = false; }
            _super.call(this);
            this.setupToPersist = setupToPersist;
            this.db = db;
            this.globalConfig = globalConfig;
        }
        savePeriodicExportSetupCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Periodic Export setup.");
            return $.when(this.saveAccountInformation(), this.saveSetup())
                .done(function () { return _this.reportSuccess("Saved Periodic Export setup."); })
                .fail(function (response) { return _this.reportError("Failed to save Periodic Export setup.", response.responseText); });
        };
        savePeriodicExportSetupCommand.prototype.saveAccountInformation = function () {
            var jQueryOptions = {};
            if (this.setupToPersist.getEtag()) {
                jQueryOptions.headers = {
                    'If-None-Match': this.setupToPersist.getEtag()
                };
            }
            var url = this.globalConfig ? "/configuration/global/settings" : "/admin/databases/" + this.db.name;
            var putArgs = JSON.stringify(this.setupToPersist.toDatabaseSettingsDto());
            return this.put(url, putArgs, null, jQueryOptions);
        };
        savePeriodicExportSetupCommand.prototype.saveSetup = function () {
            var url = this.globalConfig ? "/docs/Raven/Global/Backup/Periodic/Setup" : "/docs/Raven/Backup/Periodic/Setup";
            var putArgs = JSON.stringify(this.setupToPersist.toDto());
            return this.put(url, putArgs, this.db);
        };
        return savePeriodicExportSetupCommand;
    })(commandBase);
    return savePeriodicExportSetupCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/startCompactCommand", ["require", "exports", "commands/commandBase", "common/appUrl", "commands/database/documents/deleteDocumentCommand", "commands/maintenance/monitorCompactCommand"], function (require, exports, commandBase, appUrl, deleteDocumentCommand, monitorCompactCommand) {
    var startCompactCommand = (function (_super) {
        __extends(startCompactCommand, _super);
        function startCompactCommand(dbToCompact, updateCompactStatus) {
            _super.call(this);
            this.dbToCompact = dbToCompact;
            this.updateCompactStatus = updateCompactStatus;
            this.db = appUrl.getSystemDatabase();
        }
        startCompactCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            new deleteDocumentCommand('Raven/Database/Compact/Status/' + this.dbToCompact, this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to delete compact status document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (_) {
                var url = '/admin/compact' + _this.urlEncodeArgs({ database: _this.dbToCompact });
                _this.post(url, null, _this.db)
                    .fail(function (response) {
                    _this.reportError("Failed to compact database!", response.responseText, response.statusText);
                    _this.logError(response, result);
                })
                    .done(function () { return new monitorCompactCommand(result, _this.dbToCompact, _this.updateCompactStatus).execute(); });
            });
            return result;
        };
        startCompactCommand.prototype.logError = function (response, result) {
            var r = JSON.parse(response.responseText);
            var compactStatus = { Messages: [r.Error], LastProgressMessage: "", State: "Faulted" };
            this.updateCompactStatus(compactStatus);
            result.reject();
        };
        return startCompactCommand;
    })(commandBase);
    return startCompactCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/startRestoreCommand", ["require", "exports", "commands/database/documents/deleteDocumentCommand", "commands/commandBase", "commands/maintenance/monitorRestoreCommand", "common/appUrl"], function (require, exports, deleteDocumentCommand, commandBase, monitorRestoreCommand, appUrl) {
    var startRestoreCommand = (function (_super) {
        __extends(startRestoreCommand, _super);
        function startRestoreCommand(defrag, restoreRequest, updateRestoreStatus) {
            _super.call(this);
            this.defrag = defrag;
            this.restoreRequest = restoreRequest;
            this.updateRestoreStatus = updateRestoreStatus;
            this.db = appUrl.getSystemDatabase();
        }
        startRestoreCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            new deleteDocumentCommand('Raven/Restore/Status', this.db)
                .execute()
                .fail(function (response) {
                _this.reportError("Failed to delete restore status document!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (_) {
                _this.post('/admin/restore?defrag=' + _this.defrag, ko.toJSON(_this.restoreRequest), null, { dataType: 'text' })
                    .fail(function (response) {
                    _this.reportError("Failed to restore backup!", response.responseText, response.statusText);
                    _this.logError(response, result);
                    result.reject();
                })
                    .done(function () { return new monitorRestoreCommand(result, _this.updateRestoreStatus).execute(); });
            });
            return result;
        };
        startRestoreCommand.prototype.logError = function (response, result) {
            var r = JSON.parse(response.responseText);
            var restoreStatus = { Messages: [r.Error], State: "Faulted" };
            this.updateRestoreStatus(restoreStatus);
            result.reject();
        };
        return startRestoreCommand;
    })(commandBase);
    return startRestoreCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/maintenance/toggleRejectDatabaseClients", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var toggleRejectDatabaseClients = (function (_super) {
        __extends(toggleRejectDatabaseClients, _super);
        /**
        * @param names - The array of resource names to toggle
        * @param isSettingDisabled - Status of disabled to set
        * @param resourceType - The resource type
        */
        function toggleRejectDatabaseClients(dbName, rejectClientsEnabled) {
            _super.call(this);
            this.dbName = dbName;
            this.rejectClientsEnabled = rejectClientsEnabled;
        }
        toggleRejectDatabaseClients.prototype.execute = function () {
            var _this = this;
            var action = this.rejectClientsEnabled ? "reject clients mode" : "accept clients mode";
            var args = {
                id: this.dbName,
                isRejectClientsEnabled: this.rejectClientsEnabled
            };
            var url = "/admin/databases-toggle-reject-clients" + this.urlEncodeArgs(args);
            var toggleTask = this.post(url, null, null, { dataType: undefined });
            toggleTask.done(function () { return _this.reportSuccess("Successfully switched to " + action + " in " + _this.dbName); });
            toggleTask.fail(function (response) { return _this.reportError("Failed switching to " + action + " in " + _this.dbName, response.responseText, response.statusText); });
            return toggleTask;
        };
        return toggleRejectDatabaseClients;
    })(commandBase);
    return toggleRejectDatabaseClients;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/dismissAlertCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var dismissAlertCommand = (function (_super) {
        __extends(dismissAlertCommand, _super);
        function dismissAlertCommand(db, alertUniqueKey) {
            _super.call(this);
            this.db = db;
            this.alertUniqueKey = alertUniqueKey;
        }
        dismissAlertCommand.prototype.execute = function () {
            var _this = this;
            var url = "/operation/alert/dismiss";
            var args = {
                key: this.alertUniqueKey
            };
            return this.post(url, JSON.stringify(args), this.db, { dataType: 'text' })
                .fail(function (response) { return _this.reportError("Failed to dismiss alert", response.responseText, response.statusText); });
        };
        return dismissAlertCommand;
    })(commandBase);
    return dismissAlertCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/getAlertsCommand", ["require", "exports", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, getDocumentWithMetadataCommand) {
    var getAlertsCommand = (function (_super) {
        __extends(getAlertsCommand, _super);
        function getAlertsCommand(db) {
            _super.call(this, "Raven/Alerts", db);
        }
        getAlertsCommand.prototype.execute = function () {
            return _super.prototype.execute.call(this);
        };
        return getAlertsCommand;
    })(getDocumentWithMetadataCommand);
    return getAlertsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/getOperationAlertsCommand", ["require", "exports", "commands/commandBase", "models/database/debug/alert"], function (require, exports, commandBase, alert) {
    var getOperationAlertsCommand = (function (_super) {
        __extends(getOperationAlertsCommand, _super);
        function getOperationAlertsCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getOperationAlertsCommand.prototype.execute = function () {
            var url = "/operation/alerts";
            return this.query(url, null, this.db, function (result) { return result.map(function (a) { return new alert(a); }); });
        };
        return getOperationAlertsCommand;
    })(commandBase);
    return getOperationAlertsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/getOperationStatusCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getOperationStatusCommand = (function (_super) {
        __extends(getOperationStatusCommand, _super);
        /**
        * @param db - The database the collection belongs to.
        * @param operationId - The id of the operation.
        */
        function getOperationStatusCommand(rs, operationId) {
            _super.call(this);
            this.rs = rs;
            this.operationId = operationId;
            if (!this.rs) {
                throw new Error("Must specify a resource.");
            }
        }
        getOperationStatusCommand.prototype.execute = function () {
            var url = "/operation/status";
            var args = {
                id: this.operationId
            };
            return this.query(url, args, this.rs);
        };
        return getOperationStatusCommand;
    })(commandBase);
    return getOperationStatusCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/getRunningTasksCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getRunningTasksCommand = (function (_super) {
        __extends(getRunningTasksCommand, _super);
        function getRunningTasksCommand(db) {
            _super.call(this);
            this.db = db;
        }
        getRunningTasksCommand.prototype.execute = function () {
            var url = "/operations";
            return this.query(url, null, this.db);
        };
        return getRunningTasksCommand;
    })(commandBase);
    return getRunningTasksCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/killRunningTaskCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var killRunningTaskCommand = (function (_super) {
        __extends(killRunningTaskCommand, _super);
        function killRunningTaskCommand(db, taskId) {
            _super.call(this);
            this.db = db;
            this.taskId = taskId;
        }
        killRunningTaskCommand.prototype.execute = function () {
            var url = "/operation/kill";
            var args = {
                id: this.taskId
            };
            return this.query(url, args, this.db);
        };
        return killRunningTaskCommand;
    })(commandBase);
    return killRunningTaskCommand;
});
/// <reference path="../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/performSmugglingCommand", ["require", "exports", "commands/commandBase", "common/appUrl", "commands/operations/getOperationStatusCommand"], function (require, exports, commandBase, appUrl, getOperationStatusCommand) {
    var performSmugglingCommand = (function (_super) {
        __extends(performSmugglingCommand, _super);
        function performSmugglingCommand(migration, db, updateMigrationStatus) {
            _super.call(this);
            this.migration = migration;
            this.db = db;
            this.updateMigrationStatus = updateMigrationStatus;
        }
        performSmugglingCommand.prototype.execute = function () {
            var _this = this;
            var result = $.Deferred();
            this.post("/admin/serverSmuggling", JSON.stringify(this.migration), this.db)
                .fail(function (response) {
                _this.reportError("Failed to perform server migration!", response.responseText, response.statusText);
                result.reject();
            })
                .done(function (operationId) {
                _this.monitorOperation(result, operationId.OperationId);
            });
            return result;
        };
        performSmugglingCommand.prototype.monitorOperation = function (parentPromise, operationId) {
            var _this = this;
            new getOperationStatusCommand(appUrl.getSystemDatabase(), operationId)
                .execute()
                .done(function (result) {
                _this.updateMigrationStatus(result);
                if (result.Completed) {
                    if (result.Faulted || result.Canceled) {
                        _this.reportError("Failed to perform server migration!", result.State.Error);
                        parentPromise.reject();
                    }
                    else {
                        _this.reportSuccess("Server migration completed");
                        parentPromise.resolve();
                    }
                }
                else {
                    setTimeout(function () { return _this.monitorOperation(parentPromise, operationId); }, 500);
                }
            });
        };
        return performSmugglingCommand;
    })(commandBase);
    return performSmugglingCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/operations/saveAlertsCommand", ["require", "exports", "commands/database/documents/saveDocumentCommand", "models/database/documents/document"], function (require, exports, saveDocumentCommand, document) {
    var saveAlertsCommand = (function (_super) {
        __extends(saveAlertsCommand, _super);
        function saveAlertsCommand(alertDoc, db) {
            var doc = new document(alertDoc);
            _super.call(this, "Raven/Alerts", doc, db);
        }
        return saveAlertsCommand;
    })(saveDocumentCommand);
    return saveAlertsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/createCounterStorageCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createCounterStorageCommand = (function (_super) {
        __extends(createCounterStorageCommand, _super);
        /**
        * @param filesystemName The file system name we are creating.
        */
        function createCounterStorageCommand(counterStorageName, settings) {
            _super.call(this);
            this.counterStorageName = counterStorageName;
            this.settings = settings;
            if (!counterStorageName) {
                this.reportError("Counter Storage must have a name!");
                throw new Error("Counter Storage must have a name!");
            }
        }
        createCounterStorageCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Creating Counter Storage '" + this.counterStorageName + "'");
            var filesystemDoc = {
                "Settings": this.settings,
                "Disabled": false
            };
            var url = "/admin/cs/" + this.counterStorageName;
            var createTask = this.put(url, JSON.stringify(filesystemDoc), null, { dataType: undefined });
            createTask.done(function () { return _this.reportSuccess(_this.counterStorageName + " created"); });
            createTask.fail(function (response) { return _this.reportError("Failed to create counter storage", response.responseText, response.statusText); });
            return createTask;
        };
        return createCounterStorageCommand;
    })(commandBase);
    return createCounterStorageCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/createDatabaseCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createDatabaseCommand = (function (_super) {
        __extends(createDatabaseCommand, _super);
        function createDatabaseCommand(databaseName, settings, securedSettings) {
            _super.call(this);
            this.databaseName = databaseName;
            this.settings = settings;
            this.securedSettings = securedSettings;
            if (!databaseName) {
                this.reportError("Database must have a name!");
                throw new Error("Database must have a name!");
            }
        }
        createDatabaseCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Creating " + this.databaseName);
            var databaseDoc = {
                "Settings": this.settings,
                "SecuredSettings": this.securedSettings,
                "Disabled": false
            };
            var url = "/admin/databases/" + this.databaseName;
            return this.put(url, JSON.stringify(databaseDoc), null, { dataType: undefined })
                .then(function () { return _this.query("/databases/" + _this.databaseName + "/silverlight/ensureStartup", null, null); }) // Forces creation of system indexes.
                .done(function () { return _this.reportSuccess(_this.databaseName + " created"); })
                .fail(function (response) { return _this.reportError("Failed to create database", response.responseText, response.statusText); });
        };
        return createDatabaseCommand;
    })(commandBase);
    return createDatabaseCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/createDefaultSettingsCommand", ["require", "exports", "models/database/documents/document", "models/database/documents/versioningEntry", "commands/commandBase", "commands/resources/getDatabaseSettingsCommand", "commands/resources/saveDatabaseSettingsCommand", "commands/database/documents/saveVersioningCommand", "commands/database/globalConfig/getEffectiveVersioningsCommand", "commands/database/globalConfig/getConfigurationSettingsCommand"], function (require, exports, document, versioningEntry, commandBase, getDatabaseSettingsCommand, saveDatabaseSettingsCommand, saveVersioningCommand, getEffectiveVersioningsCommand, getConfigurationSettingsCommand) {
    var createDefaultSettingsCommand = (function (_super) {
        __extends(createDefaultSettingsCommand, _super);
        function createDefaultSettingsCommand(db, bundles) {
            _super.call(this);
            this.db = db;
            this.bundles = bundles;
        }
        createDefaultSettingsCommand.prototype.execute = function () {
            this.reportInfo("Creating default settings for '" + this.db.name + "'...");
            var tasksToWatch = [];
            if (this.bundles.contains("Quotas")) {
                tasksToWatch.push(this.updateQuotasSettings());
            }
            if (this.bundles.contains("Versioning")) {
                tasksToWatch.push(this.saveVersioningConfiguration());
            }
            if (tasksToWatch.length > 0) {
                return $.when.apply(null, tasksToWatch);
            }
            else {
                return $.Deferred().resolve();
            }
        };
        createDefaultSettingsCommand.prototype.fillDefaultQuotasSettings = function (doc) {
            var result = new document(doc.toDto(true));
            result["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = (50 * 1024).toString();
            result["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = (45 * 1024).toString();
            result["Settings"]["Raven/Quotas/Documents/HardLimit"] = (10000).toString();
            result["Settings"]["Raven/Quotas/Documents/SoftLimit"] = (8000).toString();
            return result;
        };
        createDefaultSettingsCommand.prototype.saveDatabaseSettings = function (databaseSettings) {
            return new saveDatabaseSettingsCommand(this.db, databaseSettings).execute();
        };
        createDefaultSettingsCommand.prototype.updateQuotasSettings = function () {
            var _this = this;
            var taskDone = $.Deferred();
            this.hasGlobalQuotaSettings().fail(function () { return taskDone.fail(); })
                .done(function (has) {
                if (has) {
                    // use global settings - nothing to do
                    taskDone.resolve();
                }
                else {
                    new getDatabaseSettingsCommand(_this.db, false)
                        .execute()
                        .fail(function () { return taskDone.fail(); })
                        .then(_this.fillDefaultQuotasSettings)
                        .then(function (doc) { return _this.saveDatabaseSettings(doc); })
                        .fail(function () { return taskDone.fail(); })
                        .then(function () { return taskDone.resolve(); });
                }
            });
            return taskDone;
        };
        createDefaultSettingsCommand.prototype.createDefaultVersioningSettings = function () {
            return [
                new versioningEntry({
                    Id: "DefaultConfiguration",
                    MaxRevisions: 5,
                    Exclude: false,
                    ExcludeUnlessExplicit: false,
                    PurgeOnDelete: false
                })
            ];
        };
        createDefaultSettingsCommand.prototype.saveVersioningConfiguration = function () {
            var _this = this;
            var saveTask = $.Deferred();
            this.hasGlobalVersioningSettings().fail(function () { return saveTask.fail(); })
                .done(function (has) {
                if (has) {
                    // use global settings - nothing to do 
                    saveTask.resolve();
                }
                else {
                    var entries = _this.createDefaultVersioningSettings()
                        .map(function (ve) { return ve.toDto(true); });
                    new saveVersioningCommand(_this.db, entries).execute()
                        .done(function () { return saveTask.resolve(); })
                        .fail(function () { return saveTask.reject(); });
                }
            });
            return saveTask;
        };
        createDefaultSettingsCommand.prototype.hasGlobalVersioningSettings = function () {
            var hasGlobal = $.Deferred();
            new getEffectiveVersioningsCommand(this.db)
                .execute()
                .done(function (data) {
                hasGlobal.resolve(!!data.first(function (config) { return config.globalExists(); }));
            })
                .fail(function () { return hasGlobal.reject(); });
            return hasGlobal.promise();
        };
        createDefaultSettingsCommand.prototype.hasGlobalQuotaSettings = function () {
            var hasGlobal = $.Deferred();
            new getConfigurationSettingsCommand(this.db, ["Raven/Quotas/Size/HardLimitInKB"])
                .execute()
                .done(function (result) {
                // note: we detect presence of global config based on single property!
                var hardLimit = result.results["Raven/Quotas/Size/HardLimitInKB"];
                hasGlobal.resolve(hardLimit.globalExists());
            })
                .fail(function () { return hasGlobal.reject(); });
            return hasGlobal.promise();
        };
        return createDefaultSettingsCommand;
    })(commandBase);
    return createDefaultSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/createTimeSeriesCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var createTimeSeriesCommand = (function (_super) {
        __extends(createTimeSeriesCommand, _super);
        /**
        * @param timeSeriesName The file system name we are creating.
        */
        function createTimeSeriesCommand(timeSeriesName, settings) {
            _super.call(this);
            this.timeSeriesName = timeSeriesName;
            this.settings = settings;
            if (!timeSeriesName) {
                this.reportError("Time series must have a name!");
                throw new Error("Time series must have a name!");
            }
        }
        createTimeSeriesCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Creating Time Series '" + this.timeSeriesName + "'");
            var timeSeriesDoc = {
                "Settings": this.settings,
                "Disabled": false
            };
            var url = "/admin/ts/" + this.timeSeriesName;
            var createTask = this.put(url, JSON.stringify(timeSeriesDoc), null, { dataType: undefined });
            createTask.done(function () { return _this.reportSuccess(_this.timeSeriesName + " created"); });
            createTask.fail(function (response) { return _this.reportError("Failed to create time series", response.responseText, response.statusText); });
            return createTask;
        };
        return createTimeSeriesCommand;
    })(commandBase);
    return createTimeSeriesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/deleteResourceCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteDatabaseCommand = (function (_super) {
        __extends(deleteDatabaseCommand, _super);
        function deleteDatabaseCommand(resources, isHardDelete) {
            _super.call(this);
            this.resources = resources;
            this.isHardDelete = isHardDelete;
            this.oneDatabasePath = "/admin/databases/";
            this.multipleDatabasesPath = "/admin/databases/batch-delete";
            this.oneFileSystemPath = "/admin/fs/";
            this.multipleFileSystemsPath = "/admin/fs/batch-delete";
            this.oneCounterStoragePath = "/admin/cs/";
            this.multipleCounterStoragesPath = "/admin/cs/batch-delete";
            this.oneTimeSeriesPath = "/admin/ts/";
            this.multipleTimeSeriesPath = "/admin/ts/batch-delete";
        }
        deleteDatabaseCommand.prototype.execute = function () {
            var deleteTask;
            if (this.resources.length === 1) {
                deleteTask = this.deleteOneResource();
            }
            else {
                deleteTask = this.deleteMultipleResources();
            }
            return deleteTask;
        };
        deleteDatabaseCommand.prototype.deleteOneResource = function () {
            var _this = this;
            var resource = this.resources[0];
            this.reportInfo("Deleting " + resource.name + "...");
            var args = {
                "hard-delete": this.isHardDelete
            };
            var disableOneResourcePath = (resource.type === TenantType.Database) ? this.oneDatabasePath :
                resource.type === TenantType.FileSystem ? this.oneFileSystemPath :
                    resource.type === TenantType.CounterStorage ? this.oneCounterStoragePath : this.oneTimeSeriesPath;
            var url = disableOneResourcePath + encodeURIComponent(resource.name) + this.urlEncodeArgs(args);
            var deleteTask = this.del(url, null, null, { dataType: undefined });
            deleteTask.done(function () { return _this.reportSuccess("Successfully deleted " + resource.name); });
            deleteTask.fail(function (response) { return _this.reportError("Failed to delete " + resource.name, response.responseText, response.statusText); });
            return deleteTask;
        };
        deleteDatabaseCommand.prototype.deleteMultipleResources = function () {
            var _this = this;
            var _arguments = arguments;
            this.reportInfo("Deleting " + this.resources.length + " resources...");
            var dbToDelete = this.resources.filter(function (r) { return r.type === TenantType.Database; });
            var fsToDelete = this.resources.filter(function (r) { return r.type === TenantType.FileSystem; });
            var csToDelete = this.resources.filter(function (r) { return r.type === TenantType.CounterStorage; });
            var tsToDelete = this.resources.filter(function (r) { return r.type === TenantType.TimeSeries; });
            var deleteTasks = [];
            if (dbToDelete.length > 0) {
                deleteTasks.push(this.deleteTask(dbToDelete, this.multipleDatabasesPath));
            }
            if (fsToDelete.length > 0) {
                deleteTasks.push(this.deleteTask(fsToDelete, this.multipleFileSystemsPath));
            }
            if (csToDelete.length > 0) {
                deleteTasks.push(this.deleteTask(csToDelete, this.multipleCounterStoragesPath));
            }
            if (tsToDelete.length > 0) {
                deleteTasks.push(this.deleteTask(tsToDelete, this.multipleTimeSeriesPath));
            }
            var mergedPromise = $.Deferred();
            var combinedPromise = $.when.apply(null, deleteTasks);
            combinedPromise.done(function () {
                var resources = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    resources[_i - 0] = arguments[_i];
                }
                var deletedResources = [].concat.apply([], resources);
                _this.reportSuccess("Successfully deleted " + deletedResources.length + " resources!");
                mergedPromise.resolve(deletedResources);
            });
            combinedPromise.fail(function (response) {
                _this.reportError("Failed to delete resources", response.responseText, response.statusText);
                mergedPromise.reject(response);
            });
            return mergedPromise;
        };
        deleteDatabaseCommand.prototype.deleteTask = function (resources, deletePath) {
            var _arguments = arguments;
            var args = {
                ids: resources.map(function (d) { return d.name; }),
                "hard-delete": this.isHardDelete
            };
            var url = deletePath + this.urlEncodeArgs(args);
            var task = $.Deferred();
            this.del(url, null, null, null, 9000 * resources.length)
                .done(function (resourceNames) {
                task.resolve(resources.filter(function (r) { return resourceNames.contains(r.name); }));
            })
                .fail(function () { return task.reject(_arguments); });
            return task;
        };
        return deleteDatabaseCommand;
    })(commandBase);
    return deleteDatabaseCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/disableResourceToggleCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var disableResourceToggleCommand = (function (_super) {
        __extends(disableResourceToggleCommand, _super);
        /**
        * @param resources - The array of resources to toggle
        * @param isSettingDisabled - Status of disabled to set
        */
        function disableResourceToggleCommand(resources, isSettingDisabled) {
            _super.call(this);
            this.resources = resources;
            this.isSettingDisabled = isSettingDisabled;
            this.oneDatabasePath = "/admin/databases-toggle-disable";
            this.multipleDatabasesPath = "/admin/databases/batch-toggle-disable";
            this.oneFileSystemPath = "/admin/fs/";
            this.multipleFileSystemsPath = "/admin/fs-batch-toggle-disable";
            this.oneCounterStoragePath = "/admin/cs/";
            this.multipleCounterStoragesPath = "/admin/cs/batch-toggle-disable";
            this.oneTimeSeriesPath = "/admin/ts/";
            this.multipleTimeSeriesPath = "/admin/ts/batch-toggle-disable";
        }
        disableResourceToggleCommand.prototype.execute = function () {
            var action = this.isSettingDisabled ? "disable" : "enable";
            var toggleTask;
            if (this.resources.length == 1) {
                toggleTask = this.disableOneResource(action);
            }
            else {
                toggleTask = this.disableMultipleResources(action);
            }
            return toggleTask;
        };
        disableResourceToggleCommand.prototype.disableOneResource = function (action) {
            var _this = this;
            var resource = this.resources[0];
            this.reportInfo("Trying to " + action + " " + resource.name + "...");
            var args = (resource.type === TenantType.Database) ? {
                id: resource.name,
                isSettingDisabled: this.isSettingDisabled
            } : {
                isSettingDisabled: this.isSettingDisabled
            };
            var disableOneResourcePath = (resource.type === TenantType.Database) ? this.oneDatabasePath :
                resource.type === TenantType.FileSystem ? this.oneFileSystemPath :
                    resource.type === TenantType.CounterStorage ? this.oneCounterStoragePath : this.oneTimeSeriesPath;
            var resourceName = (resource.type === TenantType.Database) ? "" : resource.name;
            var url = disableOneResourcePath + resourceName + this.urlEncodeArgs(args);
            var toggleTask = this.post(url, null, null, { dataType: undefined });
            toggleTask.done(function () { return _this.reportSuccess("Successfully " + action + "d " + name); });
            toggleTask.fail(function (response) { return _this.reportError("Failed to " + action + " " + name, response.responseText, response.statusText); });
            return toggleTask;
        };
        disableResourceToggleCommand.prototype.disableMultipleResources = function (action) {
            var _this = this;
            var _arguments = arguments;
            this.reportInfo("Trying to " + action + " " + this.resources.length + " resources...");
            var dbToToggle = this.resources.filter(function (r) { return r.type === TenantType.Database; });
            var fsToToggle = this.resources.filter(function (r) { return r.type === TenantType.FileSystem; });
            var csToToggle = this.resources.filter(function (r) { return r.type === TenantType.CounterStorage; });
            var tsToToggle = this.resources.filter(function (r) { return r.type === TenantType.TimeSeries; });
            var toggleTasks = [];
            if (dbToToggle.length > 0) {
                toggleTasks.push(this.toggleTask(dbToToggle, this.multipleDatabasesPath));
            }
            if (fsToToggle.length > 0) {
                toggleTasks.push(this.toggleTask(fsToToggle, this.multipleFileSystemsPath));
            }
            if (csToToggle.length > 0) {
                toggleTasks.push(this.toggleTask(csToToggle, this.multipleCounterStoragesPath));
            }
            if (tsToToggle.length > 0) {
                toggleTasks.push(this.toggleTask(tsToToggle, this.multipleTimeSeriesPath));
            }
            var mergedPromise = $.Deferred();
            var combinedPromise = $.when.apply(null, toggleTasks);
            combinedPromise.done(function () {
                var resources = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    resources[_i - 0] = arguments[_i];
                }
                var toggledResources = [].concat.apply([], resources);
                _this.reportSuccess("Successfully " + action + "d " + toggledResources.length + " resources!");
                mergedPromise.resolve(toggledResources);
            });
            combinedPromise.fail(function (response) {
                _this.reportError("Failed to " + action + " resources", response.responseText, response.statusText);
                mergedPromise.reject(response);
            });
            return mergedPromise;
        };
        disableResourceToggleCommand.prototype.toggleTask = function (resources, togglePath) {
            var _arguments = arguments;
            var args = {
                ids: resources.map(function (d) { return d.name; }),
                isSettingDisabled: this.isSettingDisabled
            };
            var url = togglePath + this.urlEncodeArgs(args);
            var task = $.Deferred();
            this.post(url, null, null, null, 9000 * resources.length)
                .done(function (resourceNames) {
                task.resolve(resources.filter(function (r) { return resourceNames.contains(r.name); }));
            })
                .fail(function () { return task.reject(_arguments); });
            return task;
        };
        return disableResourceToggleCommand;
    })(commandBase);
    return disableResourceToggleCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getDatabasesCommand", ["require", "exports", "commands/commandBase", "models/resources/database"], function (require, exports, commandBase, database) {
    var getDatabasesCommand = (function (_super) {
        __extends(getDatabasesCommand, _super);
        function getDatabasesCommand() {
            _super.apply(this, arguments);
        }
        getDatabasesCommand.prototype.execute = function () {
            var args = {
                pageSize: 1024,
                getAdditionalData: true
            };
            var url = "/databases";
            var resultsSelector = function (databases) { return databases.map(function (db) {
                return new database(db.Name, db.IsAdminCurrentTenant, db.Disabled, db.Bundles, db.IndexingDisabled, db.RejectClientsEnabled, db.IsLoaded, db.ClusterWide);
            }); };
            return this.query(url, args, null, resultsSelector);
        };
        return getDatabasesCommand;
    })(commandBase);
    return getDatabasesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getDatabaseSettingsCommand", ["require", "exports", "commands/commandBase", "models/database/documents/document"], function (require, exports, commandBase, document) {
    var getDatabaseSettingsCommand = (function (_super) {
        __extends(getDatabaseSettingsCommand, _super);
        function getDatabaseSettingsCommand(db, reportRefreshProgress) {
            if (reportRefreshProgress === void 0) { reportRefreshProgress = false; }
            _super.call(this);
            this.db = db;
            this.reportRefreshProgress = reportRefreshProgress;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        getDatabaseSettingsCommand.prototype.execute = function () {
            var _this = this;
            if (this.reportRefreshProgress) {
                this.reportInfo("Fetching Database Settings for '" + this.db.name + "'");
            }
            var resultsSelector = function (queryResult) { return new document(queryResult); };
            var url = "/admin/databases/" + this.db.name;
            var getTask = this.query(url, null, null, resultsSelector);
            if (this.reportRefreshProgress) {
                getTask.done(function () { return _this.reportSuccess("Database Settings of '" + _this.db.name + "' were successfully refreshed!"); });
                getTask.fail(function (response) { return _this.reportError("Failed to refresh Database Settings!", response.responseText, response.statusText); });
            }
            return getTask;
        };
        return getDatabaseSettingsCommand;
    })(commandBase);
    return getDatabaseSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getDatabaseStatsCommand", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var getDatabaseStatsCommand = (function (_super) {
        __extends(getDatabaseStatsCommand, _super);
        function getDatabaseStatsCommand(db, longWait) {
            if (longWait === void 0) { longWait = false; }
            _super.call(this);
            this.db = db;
            this.longWait = longWait;
        }
        getDatabaseStatsCommand.prototype.execute = function () {
            var url = this.getQueryUrlFragment();
            return this.query(url, null, this.db, null, this.getTimeToAlert(this.longWait));
        };
        getDatabaseStatsCommand.prototype.getQueryUrl = function () {
            return appUrl.forResourceQuery(this.db) + this.getQueryUrlFragment();
        };
        getDatabaseStatsCommand.prototype.getQueryUrlFragment = function () {
            return "/stats";
        };
        return getDatabaseStatsCommand;
    })(commandBase);
    return getDatabaseStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getGlobalReplicationTopology", ["require", "exports", "commands/commandBase", "common/appUrl"], function (require, exports, commandBase, appUrl) {
    var getGlobalReplicationTopology = (function (_super) {
        __extends(getGlobalReplicationTopology, _super);
        function getGlobalReplicationTopology(databases, filesystems, counters) {
            _super.call(this);
            this.databases = databases;
            this.filesystems = filesystems;
            this.counters = counters;
        }
        getGlobalReplicationTopology.prototype.execute = function () {
            var args = {
                Databases: this.databases,
                Filesystems: this.filesystems,
                Counters: this.counters
            };
            return this.post("/admin/replication/topology/global", ko.toJSON(args), appUrl.getSystemDatabase()).then(function (result) {
                return result;
            });
        };
        return getGlobalReplicationTopology;
    })(commandBase);
    return getGlobalReplicationTopology;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getResourceDrives", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getResourceDrives = (function (_super) {
        __extends(getResourceDrives, _super);
        function getResourceDrives(name, type) {
            _super.call(this);
            this.name = name;
            this.type = type;
        }
        getResourceDrives.prototype.execute = function () {
            var args = {
                type: this.type,
                name: this.name
            };
            var url = "/debug/resource-drives";
            return this.query(url, args);
        };
        return getResourceDrives;
    })(commandBase);
    return getResourceDrives;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/getServerBuildVersionCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getServerBuildVersionCommand = (function (_super) {
        __extends(getServerBuildVersionCommand, _super);
        function getServerBuildVersionCommand() {
            _super.apply(this, arguments);
        }
        getServerBuildVersionCommand.prototype.execute = function () {
            return this.query("/build/version", null);
        };
        return getServerBuildVersionCommand;
    })(commandBase);
    return getServerBuildVersionCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/resources/saveDatabaseSettingsCommand", ["require", "exports", "commands/commandBase", "viewmodels/shell", "common/appUrl", "commands/database/documents/getDocumentWithMetadataCommand"], function (require, exports, commandBase, shell, appUrl, getDocumentWithMetadataCommand) {
    var saveDatabaseSettingsCommand = (function (_super) {
        __extends(saveDatabaseSettingsCommand, _super);
        function saveDatabaseSettingsCommand(db, document) {
            _super.call(this);
            this.db = db;
            this.document = document;
            if (!db) {
                throw new Error("Must specify database");
            }
        }
        saveDatabaseSettingsCommand.prototype.execute = function () {
            var _this = this;
            this.reportInfo("Saving Database Settings for '" + this.db.name + "'...");
            var jQueryOptions = {
                headers: {
                    'If-None-Match': this.document.__metadata.etag,
                    'Raven-Temp-Allow-Bundles-Change': this.document.__metadata['Raven-Temp-Allow-Bundles-Change']
                }
            };
            var args = JSON.stringify(this.document.toDto());
            var url = "/admin/databases/" + this.db.name;
            var savedAndAppliedTask;
            if (shell.clusterMode()) {
                // in case of cluster mode we don't get etag and key
                // pool for document until etag will be changed
                jQueryOptions.dataType = undefined;
                var saveTask = this.put(url, args, null, jQueryOptions);
                var changesAppliedTask = $.Deferred();
                savedAndAppliedTask = changesAppliedTask;
                saveTask.fail(function () { return changesAppliedTask.reject(); });
                saveTask.done(function () {
                    var documentKey = "Raven/Databases/" + _this.db.name;
                    new getDocumentWithMetadataCommand(documentKey, appUrl.getSystemDatabase())
                        .execute()
                        .fail(function (request, status, error) { return changesAppliedTask.reject(request, status, error); })
                        .done(function (result) {
                        var documentEtag = result.__metadata.etag;
                        changesAppliedTask.resolve({
                            ETag: documentEtag,
                            Key: documentKey
                        });
                    });
                });
            }
            else {
                savedAndAppliedTask = this.put(url, args, null, jQueryOptions);
            }
            savedAndAppliedTask.done(function () { return _this.reportSuccess("Database Settings of '" + _this.db.name + "' were successfully saved!"); });
            savedAndAppliedTask.fail(function (response) { return _this.reportError("Failed to save Database Settings!", response.responseText, response.statusText); });
            return savedAndAppliedTask;
        };
        return saveDatabaseSettingsCommand;
    })(commandBase);
    return saveDatabaseSettingsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/deleteKeyCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteKeyCommand = (function (_super) {
        __extends(deleteKeyCommand, _super);
        function deleteKeyCommand(type, key, ts) {
            _super.call(this);
            this.type = type;
            this.key = key;
            this.ts = ts;
        }
        deleteKeyCommand.prototype.execute = function () {
            var _this = this;
            var args = {
                key: this.key
            };
            var url = "/delete-key/" + this.type + this.urlEncodeArgs(args);
            return this.del(url, null, this.ts, { dataType: undefined })
                .done(function (numOfDeletedPoints) { return _this.reportSuccess("Successfully deleted '" + _this.key + "' key, with " + numOfDeletedPoints + " points"); })
                .fail(function (response) { return _this.reportError("Failed to delete '" + _this.key + "' key", response.responseText, response.statusText); });
        };
        return deleteKeyCommand;
    })(commandBase);
    return deleteKeyCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/deletePointsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deletePointsCommand = (function (_super) {
        __extends(deletePointsCommand, _super);
        function deletePointsCommand(points, ts) {
            _super.call(this);
            this.points = points;
            this.ts = ts;
        }
        deletePointsCommand.prototype.execute = function () {
            var _this = this;
            var successMessage = this.points.length > 1 ? this.points.length + " points deleted" : "Point deleted";
            var task = this.del("/delete-points", JSON.stringify(this.points), this.ts, null, 9000 * this.points.length)
                .done(function (x) { return _this.reportSuccess(successMessage); })
                .fail(function (response) { return _this.reportError("Failed to delete points", response.responseText, response.statusText); });
            return task;
        };
        return deletePointsCommand;
    })(commandBase);
    return deletePointsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/deleteTypeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteTypeCommand = (function (_super) {
        __extends(deleteTypeCommand, _super);
        function deleteTypeCommand(type, ts) {
            _super.call(this);
            this.type = type;
            this.ts = ts;
        }
        deleteTypeCommand.prototype.execute = function () {
            var _this = this;
            var task = this.del("/types/" + this.type, null, this.ts, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Successfully deleted '" + _this.type + "' type"); })
                .fail(function (response) { return _this.reportError("Failed to delete '" + _this.type + "' type", response.responseText, response.statusText); });
            return task;
        };
        return deleteTypeCommand;
    })(commandBase);
    return deleteTypeCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/deleteTypesCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var deleteTypesCommand = (function (_super) {
        __extends(deleteTypesCommand, _super);
        function deleteTypesCommand(types, ts) {
            _super.call(this);
            this.types = types;
            this.ts = ts;
        }
        deleteTypesCommand.prototype.execute = function () {
            var _this = this;
            var deletionTasks = new Array();
            ;
            for (var i = 0; i < this.types.length; i++) {
                var deleteCommand = this.deleteType(this.types[i]);
                deletionTasks.push(deleteCommand);
            }
            var successMessage = this.types.length > 1 ? "Deleted " + this.types.length + " types" : "Deleted " + this.types[0];
            var combinedTask = $.when.apply($, deletionTasks)
                .done(function (x) { return _this.reportSuccess(successMessage); })
                .fail(function (response) { return _this.reportError("Failed to delete type", response.responseText, response.statusText); });
            return combinedTask;
        };
        deleteTypesCommand.prototype.deleteType = function (type) {
            var url = "/types/" + encodeURIComponent(type);
            return this.del(url, null, this.ts, { dataType: undefined }, 9000 * this.types.length);
        };
        return deleteTypesCommand;
    })(commandBase);
    return deleteTypesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getKeyCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getKeyCommand = (function (_super) {
        __extends(getKeyCommand, _super);
        function getKeyCommand(ts, type, key) {
            _super.call(this);
            this.ts = ts;
            this.type = type;
            this.key = key;
        }
        getKeyCommand.prototype.execute = function () {
            var doneTask = $.Deferred();
            var task = this.query("/key/" + this.type + "?key=" + this.key, null, this.ts);
            task.done(function (key) { return doneTask.resolve(key); });
            task.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        return getKeyCommand;
    })(commandBase);
    return getKeyCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getKeysCommand", ["require", "exports", "commands/commandBase", "models/timeSeries/timeSeriesKey", "common/pagedResultSet"], function (require, exports, commandBase, timeSeriesKey, pagedResultSet) {
    var getKeysCommand = (function (_super) {
        __extends(getKeysCommand, _super);
        function getKeysCommand(ts, skip, take, type, keysCount) {
            _super.call(this);
            this.ts = ts;
            this.skip = skip;
            this.take = take;
            this.type = type;
            this.keysCount = keysCount;
        }
        getKeysCommand.prototype.execute = function () {
            var _this = this;
            var doneTask = $.Deferred();
            var selector = function (keys) { return keys.map(function (key) { return new timeSeriesKey(key, _this.ts); }); };
            var task = this.query("/keys/" + this.type, {
                skip: this.skip,
                take: this.take
            }, this.ts, selector);
            task.done(function (keys) { return doneTask.resolve(new pagedResultSet(keys, _this.keysCount)); });
            task.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        return getKeysCommand;
    })(commandBase);
    return getKeysCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getPointsCommand", ["require", "exports", "commands/commandBase", "models/timeSeries/timeSeriesPoint"], function (require, exports, commandBase, timeSeriesPoint) {
    var getPointsCommand = (function (_super) {
        __extends(getPointsCommand, _super);
        /**
        * @param skip - number of entries to skip
        * @param take - number of entries to take
        */
        function getPointsCommand(ts, skip, take, type, fields, key, start, end) {
            _super.call(this);
            this.ts = ts;
            this.skip = skip;
            this.take = take;
            this.type = type;
            this.fields = fields;
            this.key = key;
            this.start = start;
            this.end = end;
        }
        getPointsCommand.prototype.execute = function () {
            var _this = this;
            var url = "/points/" + this.type + "?key=" + this.key;
            var doneTask = $.Deferred();
            var selector = function (dtos) { return dtos.map(function (d) { return new timeSeriesPoint(_this.type, _this.fields, _this.key, d.At, d.Values); }); };
            var args = {
                skip: this.skip,
                take: this.take,
                start: this.start,
                end: this.end
            };
            var task = this.query(url, args, this.ts, selector);
            task.done(function (points) { return doneTask.resolve(points); });
            task.fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        return getPointsCommand;
    })(commandBase);
    return getPointsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getTimeSeriesCommand", ["require", "exports", "models/timeSeries/timeSeries", "commands/commandBase"], function (require, exports, timeSeries, commandBase) {
    var getTimeSeriesCommand = (function (_super) {
        __extends(getTimeSeriesCommand, _super);
        function getTimeSeriesCommand() {
            _super.apply(this, arguments);
        }
        getTimeSeriesCommand.prototype.execute = function () {
            var args = {
                pageSize: 1024,
                getAdditionalData: true
            };
            var url = "/ts";
            var resultsSelector = function (ts) {
                return ts.map(function (ts) { return new timeSeries(ts.Name, ts.IsAdminCurrentTenant, ts.Disabled, ts.Bundles); });
            };
            return this.query(url, args, null, resultsSelector);
        };
        return getTimeSeriesCommand;
    })(commandBase);
    return getTimeSeriesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getTimeSeriesStatsCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var getTimeSeriesStatsCommand = (function (_super) {
        __extends(getTimeSeriesStatsCommand, _super);
        function getTimeSeriesStatsCommand(ts, longWait) {
            if (longWait === void 0) { longWait = false; }
            _super.call(this);
            this.ts = ts;
            this.longWait = longWait;
        }
        getTimeSeriesStatsCommand.prototype.execute = function () {
            var url = "/stats";
            return this.query(url, null, this.ts, null, this.getTimeToAlert(this.longWait));
        };
        return getTimeSeriesStatsCommand;
    })(commandBase);
    return getTimeSeriesStatsCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/getTypesCommand", ["require", "exports", "commands/commandBase", "models/timeSeries/timeSeriesType"], function (require, exports, commandBase, timeSeriesType) {
    var getTypesCommand = (function (_super) {
        __extends(getTypesCommand, _super);
        function getTypesCommand(ts) {
            _super.call(this);
            this.ts = ts;
        }
        getTypesCommand.prototype.execute = function () {
            var _this = this;
            var selector = function (keys) { return keys.map(function (key) { return timeSeriesType.fromDto(key, _this.ts); }); };
            return this.query("/types", {
                skip: 0,
                take: 1024
            }, this.ts, selector);
        };
        return getTypesCommand;
    })(commandBase);
    return getTypesCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/putPointCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var putPointCommand = (function (_super) {
        __extends(putPointCommand, _super);
        function putPointCommand(type, key, at, values, ts) {
            _super.call(this);
            this.type = type;
            this.key = key;
            this.at = at;
            this.values = values;
            this.ts = ts;
        }
        putPointCommand.prototype.execute = function () {
            var _this = this;
            var url = "/append/" + this.type;
            var args = JSON.stringify({ type: this.type, key: this.key, at: this.at, values: this.values });
            return this.put(url, args, this.ts, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Point saved"); })
                .fail(function (response) {
                _this.reportError("Failed to save point", response.responseText, response.statusText);
            });
        };
        return putPointCommand;
    })(commandBase);
    return putPointCommand;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("commands/timeSeries/putTypeCommand", ["require", "exports", "commands/commandBase"], function (require, exports, commandBase) {
    var putTypeCommand = (function (_super) {
        __extends(putTypeCommand, _super);
        function putTypeCommand(type, fields, ts) {
            _super.call(this);
            this.type = type;
            this.fields = fields;
            this.ts = ts;
        }
        putTypeCommand.prototype.execute = function () {
            var _this = this;
            var url = "/types/" + this.type;
            var dto = { Type: this.type, Fields: this.fields, KeysCount: 0 };
            return this.put(url, JSON.stringify(dto), this.ts, { dataType: undefined })
                .done(function () { return _this.reportSuccess("Type saved"); })
                .fail(function (response) {
                _this.reportError("Failed to save type", response.responseText, response.statusText);
            });
        };
        return putTypeCommand;
    })(commandBase);
    return putTypeCommand;
});
define("common/adminLogsClient", ["require", "exports", 'common/appUrl', 'common/changeSubscription', 'common/changesCallback', 'commands/commandBase', "common/changesApi", "common/idGenerator", "commands/database/debug/adminLogsConfigureCommand"], function (require, exports, appUrl, changeSubscription, changesCallback, commandBase, changesApi, idGenerator, adminLogsConfigureCommand) {
    var adminLogsClient = (function () {
        function adminLogsClient(token) {
            this.token = token;
            this.readyStateOpen = 1;
            this.isCleanClose = false;
            this.normalClosureCode = 1000;
            this.normalClosureMessage = "CLOSE_NORMAL";
            this.successfullyConnectedOnce = false;
            this.commandBase = new commandBase();
            this.adminLogsHandlers = ko.observableArray();
            this.eventsId = idGenerator.generateId();
            this.resourcePath = appUrl.forResourceQuery(appUrl.getSystemDatabase());
            this.connectionOpeningTask = $.Deferred();
            this.connectionClosingTask = $.Deferred();
        }
        adminLogsClient.prototype.connect = function () {
            var connectionString = 'singleUseAuthToken=' + this.token + '&id=' + this.eventsId;
            if ("WebSocket" in window && changesApi.isServerSupportingWebSockets) {
                this.connectWebSocket(connectionString);
            }
            else if ("EventSource" in window) {
                this.connectEventSource(connectionString);
            }
            else {
                //The browser doesn't support nor websocket nor eventsource
                //or we are in IE10 or IE11 and the server doesn't support WebSockets.
                //Anyway, at this point a warning message was already shown. 
                this.connectionOpeningTask.reject();
            }
        };
        adminLogsClient.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === 'https:' ? 'wss://' : 'ws://';
            this.webSocket = new WebSocket(wsProtocol + window.location.host + this.resourcePath + '/admin/logs/events?' + connectionString);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.connectionClosingTask.resolve({ Error: e });
                }
            };
            this.webSocket.onclose = function (e) {
                _this.connectionClosingTask.resolve();
            };
            this.webSocket.onopen = function () {
                console.log("Connected to WebSocket admin logs");
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        adminLogsClient.prototype.connectEventSource = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            this.eventSource = new EventSource(this.resourcePath + '/admin/logs/events?' + connectionString);
            this.eventSource.onmessage = function (e) { return _this.onMessage(e); };
            this.eventSource.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.eventSource.close();
                    _this.connectionClosingTask.resolve(e);
                }
            };
            this.eventSource.onopen = function () {
                console.log("Connected to WebSocket admin logs");
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        adminLogsClient.prototype.send = function (command, value, needToSaveSentMessages) {
            var _this = this;
            if (needToSaveSentMessages === void 0) { needToSaveSentMessages = true; }
            this.connectionOpeningTask.done(function () {
                var args = {
                    id: _this.eventsId,
                    command: command
                };
                if (value !== undefined) {
                    args["value"] = value;
                }
                //TODO: exception handling?
                _this.commandBase.query('/admin/logs/configure', args, appUrl.getSystemDatabase());
            });
        };
        adminLogsClient.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        adminLogsClient.prototype.onMessage = function (e) {
            var eventDto = JSON.parse(e.data);
            if (!!eventDto.Type && eventDto.Type == 'Heartbeat') {
                return;
            }
            this.fireEvents(this.adminLogsHandlers(), eventDto, function (e) { return true; });
        };
        adminLogsClient.prototype.watchAdminLogs = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.adminLogsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.adminLogsHandlers.remove(callback);
            });
        };
        adminLogsClient.prototype.configureCategories = function (categoriesConfig) {
            new adminLogsConfigureCommand(appUrl.getSystemDatabase(), categoriesConfig, this.eventsId).execute();
        };
        adminLogsClient.prototype.dispose = function () {
            var _this = this;
            this.connectionOpeningTask.done(function () {
                var isCloseNeeded;
                if (isCloseNeeded = _this.webSocket && _this.webSocket.readyState == _this.readyStateOpen) {
                    console.log("Disconnecting from WebSocket Logs API");
                    _this.webSocket.close(_this.normalClosureCode, _this.normalClosureMessage);
                }
                else if (isCloseNeeded = _this.eventSource && _this.eventSource.readyState == _this.readyStateOpen) {
                    console.log("Disconnecting from EventSource Logs API");
                    _this.eventSource.close();
                    _this.connectionClosingTask.resolve();
                }
                if (isCloseNeeded) {
                    _this.send('disconnect', undefined, false);
                    _this.isCleanClose = true;
                }
            });
        };
        adminLogsClient.messageWasShownOnce = false;
        return adminLogsClient;
    })();
    return adminLogsClient;
});
define("common/alertArgs", ["require", "exports"], function (require, exports) {
    var alertArgs = (function () {
        function alertArgs(type, title, details, httpStatusText, displayInRecentErrors) {
            if (details === void 0) { details = ""; }
            if (httpStatusText === void 0) { httpStatusText = ""; }
            if (displayInRecentErrors === void 0) { displayInRecentErrors = true; }
            this.type = type;
            this.title = title;
            this.details = details;
            this.httpStatusText = httpStatusText;
            this.displayInRecentErrors = displayInRecentErrors;
            this.buttonName = null;
            this.onClickAction = null;
            var hashString = (title + details)
                .hashCode()
                .toString();
            this.id = "alert_" + hashString;
        }
        alertArgs.prototype.setupButtton = function (buttonName, onClickAction) {
            this.buttonName = buttonName;
            this.onClickAction = onClickAction;
        };
        Object.defineProperty(alertArgs.prototype, "errorMessage", {
            get: function () {
                var error = this.errorInfo;
                if (error && error.message) {
                    return error.message;
                }
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(alertArgs.prototype, "errorInfo", {
            get: function () {
                if (this.parsedErrorInfo) {
                    return this.parsedErrorInfo;
                }
                if (this.type !== 3 /* danger */ && this.type !== 2 /* warning */) {
                    return null;
                }
                // See if we can tease out an error message from the details string.
                var detailsObj = this.getDetailsObject();
                if (detailsObj) {
                    var error = detailsObj.Error;
                    if (error && typeof error === "string") {
                        var indexOfStackTrace = error.indexOf("\r\n");
                        if (indexOfStackTrace !== -1) {
                            this.parsedErrorInfo = {
                                message: detailsObj.Message ? detailsObj.Message : error.substr(0, indexOfStackTrace),
                                stackTrace: detailsObj.Message ? error : error.substr(indexOfStackTrace + "\r\n".length),
                                url: detailsObj.Url || ""
                            };
                        }
                        else {
                            this.parsedErrorInfo = {
                                message: detailsObj.Message ? detailsObj.Message : error,
                                stackTracke: error,
                                url: detailsObj.Url
                            };
                        }
                    }
                }
                return this.parsedErrorInfo;
            },
            enumerable: true,
            configurable: true
        });
        alertArgs.prototype.getDetailsObject = function () {
            if (this.detailsObject) {
                return this.detailsObject;
            }
            if (this.details) {
                try {
                    this.detailsObject = JSON.parse(this.details);
                }
                catch (error) {
                    return null;
                }
            }
            return this.detailsObject;
        };
        return alertArgs;
    })();
    return alertArgs;
});
define("common/alertType", ["require", "exports"], function (require, exports) {
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/apiKeyLocalStorage", ["require", "exports"], function (require, exports) {
    var apiKeyLocalStorage = (function () {
        function apiKeyLocalStorage() {
        }
        apiKeyLocalStorage.get = function () {
            return localStorage.getObject(apiKeyLocalStorage.localStorageName);
        };
        apiKeyLocalStorage.setValue = function (apiKey) {
            localStorage.setObject(apiKeyLocalStorage.localStorageName, apiKey);
        };
        apiKeyLocalStorage.clean = function () {
            localStorage.removeItem(apiKeyLocalStorage.localStorageName);
        };
        apiKeyLocalStorage.notifyAboutLogOut = function () {
            var event = document.createEvent('StorageEvent');
            event.initStorageEvent('storage', false, false, apiKeyLocalStorage.localStorageName, null, null, null, window.localStorage);
            window.dispatchEvent(event);
        };
        apiKeyLocalStorage.localStorageName = "apiKey";
        return apiKeyLocalStorage;
    })();
    return apiKeyLocalStorage;
});
define("common/appUrl", ["require", "exports", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "plugins/router", "common/messagePublisher"], function (require, exports, database, filesystem, counterStorage, timeSeries, router, messagePublisher) {
    // Helper class with static methods for generating app URLs.
    var appUrl = (function () {
        function appUrl() {
        }
        appUrl.detectAppUrl = function () {
            var path = window.location.pathname.replace("\\", "/").replace("%5C", "/");
            var suffix = "studio/index.html";
            if (path.indexOf(suffix, path.length - suffix.length) !== -1) {
                return path.substring(0, path.length - suffix.length - 1);
            }
            return "";
        };
        appUrl.checkIsAreaActive = function (routeRoot) {
            var items = router.routes.filter(function (m) { return m.isActive() && m.route != null && m.route != ''; });
            var isThereAny = items.some(function (m) { return m.route.substring(0, routeRoot.length) === routeRoot; });
            return isThereAny;
        };
        appUrl.getEncodedCounterStoragePart = function (cs) {
            return cs ? "&counterstorage=" + encodeURIComponent(cs.name) : "";
        };
        appUrl.forCounterStorageCounters = function (gruopName, cs) {
            var groupPart = gruopName ? "group=" + encodeURIComponent(gruopName) : "";
            var counterStoragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/counters?" + groupPart + counterStoragePart;
        };
        appUrl.forCounterStorageReplication = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/replication?" + counterStroragePart;
        };
        appUrl.forCounterStorageTasks = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#counterstorages/tasks?" + counterStroragePart;
        };
        appUrl.forImportCounterStorage = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#databases/tasks/importCounterStorage?" + counterStroragePart;
        };
        appUrl.forExportCounterStorage = function (cs) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(cs);
            return "#databases/tasks/exportCounterStorage?" + counterStroragePart;
        };
        appUrl.forCounterStorageStats = function (counterStorage) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(counterStorage);
            return "#counterstorages/stats?" + counterStroragePart;
        };
        appUrl.forCounterStorageConfiguration = function (counterStorage) {
            var counterStroragePart = appUrl.getEncodedCounterStoragePart(counterStorage);
            return "#counterstorages/configuration?" + counterStroragePart;
        };
        appUrl.forTimeSeriesType = function (type, ts) {
            var url = "";
            if (type) {
                url = "type=" + encodeURIComponent(type);
            }
            var timeSeriesPart = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/types?" + url + timeSeriesPart;
        };
        appUrl.forTimeSeriesKey = function (type, key, ts) {
            var url = "";
            if (type && key) {
                url = "type=" + encodeURIComponent(type) + "&key=" + encodeURIComponent(key);
            }
            var timeSeriesPart = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/points?" + url + timeSeriesPart;
        };
        appUrl.forTimeSeriesStats = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/stats?" + part;
        };
        appUrl.forTimeSeriesConfiguration = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/configuration?" + part;
        };
        appUrl.forTimeSeriesConfigurationTypes = function (ts) {
            var part = appUrl.getEncodedTimeSeriesPart(ts);
            return "#timeseries/configuration/types?" + part;
        };
        appUrl.getEncodedTimeSeriesPart = function (ts) {
            return ts ? "&timeseries=" + encodeURIComponent(ts.name) : "";
        };
        appUrl.forUpgrade = function (db) {
            return "#databases/upgrade?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forAdminSettings = function () {
            return "#admin/settings";
        };
        appUrl.forApiKeys = function () {
            return "#admin/settings/apiKeys";
        };
        appUrl.forCluster = function () {
            return "#admin/settings/cluster";
        };
        appUrl.forWindowsAuth = function () {
            return "#admin/settings/windowsAuth";
        };
        appUrl.forGlobalConfig = function () {
            return '#admin/settings/globalConfig';
        };
        appUrl.forServerSmugging = function () {
            return "#admin/settings/serverSmuggling";
        };
        appUrl.forGlobalConfigPeriodicExport = function () {
            return '#admin/settings/globalConfig';
        };
        appUrl.forGlobalConfigReplication = function () {
            return '#admin/settings/globalConfigReplication';
        };
        appUrl.forGlobalConfigSqlReplication = function () {
            return "#admin/settings/globalConfigSqlReplication";
        };
        appUrl.forGlobalConfigQuotas = function () {
            return '#admin/settings/globalConfigQuotas';
        };
        appUrl.forGlobalConfigCustomFunctions = function () {
            return '#admin/settings/globalConfigCustomFunctions';
        };
        appUrl.forGlobalConfigVersioning = function () {
            return "#admin/settings/globalConfigVersioning";
        };
        appUrl.forBackup = function () {
            return "#admin/settings/backup";
        };
        appUrl.forHotSpare = function () {
            return "#admin/settings/hotSpare";
        };
        appUrl.forCompact = function () {
            return "#admin/settings/compact";
        };
        appUrl.forRestore = function () {
            return "#admin/settings/restore";
        };
        appUrl.forAdminLogs = function () {
            return "#admin/settings/adminLogs";
        };
        appUrl.forServerTopology = function () {
            return "#admin/settings/topology";
        };
        appUrl.forTrafficWatch = function () {
            return "#admin/settings/trafficWatch";
        };
        appUrl.forLicenseInformation = function () {
            return "#admin/settings/licenseInformation";
        };
        appUrl.forDebugInfo = function () {
            return "#admin/settings/debugInfo";
        };
        appUrl.forIoTest = function () {
            return "#admin/settings/ioTest";
        };
        appUrl.forDiskIoViewer = function () {
            return "#admin/settings/diskIoViewer";
        };
        appUrl.forAdminJsConsole = function () {
            return "#admin/settings/console";
        };
        appUrl.forStudioConfig = function () {
            return "#admin/settings/studioConfig";
        };
        appUrl.forResources = function () {
            return "#resources";
        };
        appUrl.forHasApiKey = function () {
            return "#has-api-key";
        };
        appUrl.forCounterStorages = function () {
            return "#counterstorages";
        };
        appUrl.forTimeSeries = function () {
            return "#timeseries";
        };
        /**
        * Gets the URL for edit document.
        * @param id The ID of the document to edit, or null to edit a new document.
        * @param collectionName The name of the collection to page through on the edit document, or null if paging will be disabled.
        * @param docIndexInCollection The 0-based index of the doc to edit inside the paged collection, or null if paging will be disabled.
        * @param database The database to use in the URL. If null, the current database will be used.
        */
        appUrl.forEditDoc = function (id, collectionName, docIndexInCollection, db) {
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            var docIdUrlPart = id ? "&id=" + encodeURIComponent(id) : "";
            var pagedListInfo = collectionName && docIndexInCollection != null ? "&list=" + encodeURIComponent(collectionName) + "&item=" + docIndexInCollection : "";
            return "#databases/edit?" + docIdUrlPart + databaseUrlPart + pagedListInfo;
        };
        appUrl.forEditItem = function (itemId, rs, itemIndex, collectionName) {
            var urlPart = appUrl.getEncodedResourcePart(rs);
            var itemIdUrlPart = itemId ? "&id=" + encodeURIComponent(itemId) : "";
            var pagedListInfo = collectionName && itemIndex != null ? "&list=" + encodeURIComponent(collectionName) + "&item=" + itemIndex : "";
            var resourceTag = rs instanceof filesystem ? "#filesystems" : rs instanceof counterStorage ? "#counterstorages" : "#databases";
            return resourceTag + "/edit?" + itemIdUrlPart + urlPart + pagedListInfo;
        };
        appUrl.forEditCounter = function (rs, groupName, counterName) {
            var urlPart = appUrl.getEncodedResourcePart(rs);
            var itemIdUrlPart = "&groupName=" + encodeURIComponent(groupName) + "&counterName=" + encodeURIComponent(counterName);
            return "#counterstorages/edit?" + itemIdUrlPart + urlPart;
        };
        appUrl.forEditQueryItem = function (itemNumber, res, index, query, sort) {
            var databaseUrlPart = appUrl.getEncodedResourcePart(res);
            var indexUrlPart = "&index=" + index;
            var itemNumberUrlPart = "&item=" + itemNumber;
            var queryInfoUrlPart = query ? "&query=" + encodeURIComponent(query) : "";
            var sortInfoUrlPart = sort ? "&sorts=" + sort : "";
            var resourceTag = res instanceof filesystem ? "#filesystems" : "#databases";
            return resourceTag + "/edit?" + databaseUrlPart + indexUrlPart + itemNumberUrlPart + queryInfoUrlPart + sortInfoUrlPart;
        };
        appUrl.forNewDoc = function (db) {
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            return "#databases/edit?" + databaseUrlPart;
        };
        /**
        * Gets the URL for status page.
        * @param database The database to use in the URL. If null, the current database will be used.
        */
        appUrl.forStatus = function (db) {
            return "#databases/status?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplicationPerfStats = function (db) {
            return "#databases/status/replicationPerfStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSqlReplicationPerfStats = function (db) {
            return "#databases/status/sqlReplicationPerfStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestsCount = function (db) {
            return "#databases/status/requests?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestsTracing = function (db) {
            return "#databases/status/requests/tracing?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexPerformance = function (db) {
            return "#databases/status/indexing?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexStats = function (db) {
            return "#databases/status/indexing/stats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexBatchSize = function (db) {
            return "#databases/status/indexing/batchSize?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexPrefetches = function (db) {
            return "#databases/status/indexing/prefetches?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebug = function (db) {
            return "#databases/status/debug?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugChanges = function (db) {
            return "#databases/status/debug?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugMetrics = function (db) {
            return "#databases/status/debug/metrics?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugConfig = function (db) {
            return "#databases/status/debug/config?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugDocrefs = function (db) {
            return "#databases/status/debug/docrefs?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugCurrentlyIndexing = function (db) {
            return "#databases/status/debug/currentlyIndexing?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugQueries = function (db) {
            return "#databases/status/debug/queries?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugTasks = function (db) {
            return "#databases/status/debug/tasks?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugRoutes = function (db) {
            return "#databases/status/debug/routes?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRequestTracing = function (db) {
            return "#databases/status/requests/tracking?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugSqlReplication = function (db) {
            return "#databases/status/debug/sqlReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugIndexFields = function (db) {
            return "#databases/status/debug/indexFields?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugIdentities = function (db) {
            return "#databases/status/debug/identities?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugWebSocket = function (db) {
            return "#databases/status/debug/webSocket?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugPersistAutoIndex = function (db) {
            return "#databases/status/debug/persist?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusDebugExplainReplication = function (db) {
            return "#databases/status/debug/explainReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forInfoPackage = function (db) {
            return '#databases/status/infoPackage?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageOnDisk = function (db) {
            return '#databases/status/storage?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageBreakdown = function (db) {
            return '#databases/status/storage/storageBreakdown?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forStatusStorageCollections = function (db) {
            return '#databases/status/storage/collections?' + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSettings = function (db) {
            return "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forLogs = function (db) {
            return "#databases/status/logs?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forRunningTasks = function (db) {
            return "#databases/status/runningTasks?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forAlerts = function (db) {
            return "#databases/status/alerts?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forIndexErrors = function (db) {
            return "#databases/status/indexErrors?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplicationStats = function (db) {
            return "#databases/status/replicationStats?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forUserInfo = function (db) {
            return "#databases/status/userInfo?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forVisualizer = function (db, index) {
            if (index === void 0) { index = null; }
            var url = "#databases/status/visualizer?" + appUrl.getEncodedDbPart(db);
            if (index) {
                url += "&index=" + index;
            }
            return url;
        };
        appUrl.forDataSubscriptions = function (db) {
            return "#databases/status/debug/dataSubscriptions?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDatabaseSettings = function (db) {
            return "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forQuotas = function (db) {
            return "#databases/settings/quotas?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forPeriodicExport = function (db) {
            return "#databases/settings/periodicExport?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forReplications = function (db) {
            return "#databases/settings/replication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forVersioning = function (db) {
            return "#databases/settings/versioning?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forSqlReplications = function (db) {
            return "#databases/settings/sqlReplication?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forEditSqlReplication = function (sqlReplicationName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/settings/editSqlReplication/" + encodeURIComponent(sqlReplicationName) + "?" + databasePart;
        };
        appUrl.forSqlReplicationConnections = function (db) {
            return "#databases/settings/sqlReplicationConnectionStringsManagement?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forScriptedIndexes = function (db) {
            return "#databases/settings/scriptedIndex?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forCustomFunctionsEditor = function (db) {
            return "#databases/settings/customFunctionsEditor?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDatabaseStudioConfig = function (db) {
            return "#databases/settings/databaseStudioConfig?" + appUrl.getEncodedDbPart(db);
        };
        appUrl.forDocuments = function (collection, db) {
            var collectionPart = collection ? "collection=" + encodeURIComponent(collection) : "";
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/documents?" + collectionPart + databasePart;
        };
        appUrl.forConflicts = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/conflicts?" + databasePart;
        };
        appUrl.forPatch = function (db, hashOfRecentPatch) {
            var databasePart = appUrl.getEncodedDbPart(db);
            if (hashOfRecentPatch) {
                var patchPath = "recentpatch-" + hashOfRecentPatch;
                return "#databases/patch/" + encodeURIComponent(patchPath) + "?" + databasePart;
            }
            else {
                return "#databases/patch?" + databasePart;
            }
        };
        appUrl.forIndexes = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes?" + databasePart;
        };
        appUrl.forNewIndex = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit?" + databasePart;
        };
        appUrl.forEditIndex = function (indexName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit/" + encodeURIComponent(indexName) + "?" + databasePart;
        };
        appUrl.forNewTransformer = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit?" + databasePart;
        };
        appUrl.forEditTransformer = function (transformerName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit/" + encodeURIComponent(transformerName) + "?" + databasePart;
        };
        appUrl.forTransformers = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers?" + databasePart;
        };
        appUrl.forQuery = function (db, indexNameOrHashToQuery) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexToQueryComponent = indexNameOrHashToQuery;
            if (typeof indexNameOrHashToQuery === "number") {
                indexToQueryComponent = "recentquery-" + indexNameOrHashToQuery;
            }
            var indexPart = indexToQueryComponent ? "/" + encodeURIComponent(indexToQueryComponent) : "";
            return "#databases/query/index" + indexPart + "?" + databasePart;
        };
        appUrl.forReporting = function (db, indexName) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexPart = indexName ? "/" + encodeURIComponent(indexName) : "";
            return "#databases/query/reporting" + indexPart + "?" + databasePart;
        };
        appUrl.forExploration = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/query/exploration?" + databasePart;
        };
        appUrl.forTasks = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks?" + databasePart;
        };
        appUrl.forResourceQuery = function (res) {
            if (res && res instanceof database && !res.isSystem) {
                return appUrl.baseUrl + "/databases/" + res.name;
            }
            else if (res && res instanceof filesystem) {
                return appUrl.baseUrl + "/fs/" + res.name;
            }
            else if (res && res instanceof counterStorage) {
                return appUrl.baseUrl + "/cs/" + res.name;
            }
            else if (res && res instanceof timeSeries) {
                return appUrl.baseUrl + "/ts/" + res.name;
            }
            return this.baseUrl;
        };
        appUrl.forTerms = function (indexName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/terms/" + encodeURIComponent(indexName) + "?" + databasePart;
        };
        appUrl.forMegeSuggestions = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/mergeSuggestions?" + databasePart;
        };
        appUrl.forImportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/importDatabase?" + databasePart;
        };
        appUrl.forExportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/exportDatabase?" + databasePart;
        };
        appUrl.forImportFilesystem = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks/importFilesystem?" + filesystemPart;
        };
        appUrl.forExportFilesystem = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks/exportFilesystem?" + filesystemPart;
        };
        appUrl.forExportCollectionCsv = function (collection, db, customColumns) {
            if (collection.isAllDocuments || collection.isSystemDocuments) {
                return null;
            }
            var args = {
                format: "excel",
                download: true,
                query: "Tag:" + collection.name,
                column: customColumns
            };
            return appUrl.forResourceQuery(db) + "/streams/query/Raven/DocumentsByEntityName" + appUrl.urlEncodeArgs(args);
        };
        appUrl.forToggleIndexing = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/toggleIndexing?" + databasePart;
        };
        appUrl.forSetAcknowledgedEtag = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/subscriptionsTask?" + databasePart;
        };
        appUrl.forSampleData = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/sampleData?" + databasePart;
        };
        appUrl.forCsvImport = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/csvImport?" + databasePart;
        };
        appUrl.forCounterStorage = function (cs) {
            var counterStoragePart = appUrl.getEncodedCounterPart(cs);
            return "#counterstorages?" + counterStoragePart;
        };
        appUrl.forIndexesRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes";
        };
        appUrl.forIndexQueryRawData = function (db, indexName) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes/" + indexName;
        };
        appUrl.forReportingRawData = function (db, indexName) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/facets/" + indexName;
        };
        appUrl.forTransformersRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/transformers";
        };
        appUrl.forDatabasesRawData = function () {
            return window.location.protocol + "//" + window.location.host + "/databases";
        };
        appUrl.forDocumentRawData = function (db, docId) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/docs/" + docId;
        };
        appUrl.forFilesystemFiles = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/files?" + filesystemPart;
        };
        appUrl.forFilesystemSearch = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/search?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronization = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronizationDestinations = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization/destinations?" + filesystemPart;
        };
        appUrl.forFilesystemSynchronizationConfiguration = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization/configuration?" + filesystemPart;
        };
        appUrl.forFilesystemStatus = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/status?" + filesystemPart;
        };
        appUrl.forFilesystemTasks = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/tasks?" + filesystemPart;
        };
        appUrl.forFilesystemSettings = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/settings?" + filesystemPart;
        };
        appUrl.forFilesystemConfiguration = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/configuration?" + filesystemPart;
        };
        appUrl.forFilesystemVersioning = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/settings?" + filesystemPart;
        };
        appUrl.forFilesystemConfigurationWithKey = function (fs, key) {
            var filesystemPart = appUrl.getEncodedFsPart(fs) + "&key=" + encodeURIComponent(key);
            return "#filesystems/configuration?" + filesystemPart;
        };
        appUrl.forEditFile = function (id, fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            var fileIdPart = id ? "&id=" + encodeURIComponent(id) : "";
            return "#filesystems/edit?" + fileIdPart + filesystemPart;
        };
        /**
        * Gets the resource from the current web browser address. Returns the system database if no resource name is found.
        */
        appUrl.getResource = function () {
            var appFileSystem = appUrl.getFileSystem();
            var appCounterStorage = appUrl.getCounterStorage();
            var appTimeSeries = appUrl.getTimeSeries();
            if (!!appFileSystem) {
                return appFileSystem;
            }
            else if (!!appCounterStorage) {
                return appCounterStorage;
            }
            else if (!!appTimeSeries) {
                return appTimeSeries;
            }
            else {
                return appUrl.getDatabase();
            }
        };
        /**
        * Gets the database from the current web browser address. Returns the system database if no database name was found.
        */
        appUrl.getDatabase = function () {
            // TODO: instead of string parsing, can we pull this from durandal.activeInstruction()?
            var dbIndicator = "database=";
            var hash = window.location.hash;
            var dbIndex = hash.indexOf(dbIndicator);
            if (dbIndex >= 0) {
                // A database is specified in the address.
                var dbSegmentEnd = hash.indexOf("&", dbIndex);
                if (dbSegmentEnd === -1) {
                    dbSegmentEnd = hash.length;
                }
                var databaseName = hash.substring(dbIndex + dbIndicator.length, dbSegmentEnd);
                var unescapedDatabaseName = decodeURIComponent(databaseName);
                var db = new database(unescapedDatabaseName);
                db.isSystem = unescapedDatabaseName === "<system>";
                return db;
            }
            else {
                // No database is specified in the URL. Assume it's the system database.
                return null;
            }
        };
        appUrl.getSystemDatabase = function () {
            var db = new database("<system>");
            db.isSystem = true;
            db.isVisible(false);
            return db;
        };
        /**
        * Gets the file system from the current web browser address. Returns null if no file system name was found.
        */
        appUrl.getFileSystem = function () {
            // TODO: instead of string parsing, can we pull this from durandal.activeInstruction()?
            var fileSystemIndicator = "filesystem=";
            var hash = window.location.hash;
            var fsIndex = hash.indexOf(fileSystemIndicator);
            if (fsIndex >= 0) {
                // A database is specified in the address.
                var fsSegmentEnd = hash.indexOf("&", fsIndex);
                if (fsSegmentEnd === -1) {
                    fsSegmentEnd = hash.length;
                }
                var fileSystemName = hash.substring(fsIndex + fileSystemIndicator.length, fsSegmentEnd);
                var unescapedFileSystemName = decodeURIComponent(fileSystemName);
                var fs = new filesystem(unescapedFileSystemName);
                return fs;
            }
            else {
                // No file system is specified in the URL.
                return null;
            }
        };
        /**
        * Gets the counter storage from the current web browser address. Returns null if no counter storage name was found.
        */
        appUrl.getCounterStorage = function () {
            // TODO: instead of string parsing, can we pull this from durandal.activeInstruction()?
            var counterStorageIndicator = "counterstorage=";
            var hash = window.location.hash;
            var csIndex = hash.indexOf(counterStorageIndicator);
            if (csIndex >= 0) {
                // A database is specified in the address.
                var csSegmentEnd = hash.indexOf("&", csIndex);
                if (csSegmentEnd === -1) {
                    csSegmentEnd = hash.length;
                }
                var counterStorageName = hash.substring(csIndex + counterStorageIndicator.length, csSegmentEnd);
                var unescapedCounterStorageName = decodeURIComponent(counterStorageName);
                var cs = new counterStorage(unescapedCounterStorageName);
                return cs;
            }
            else {
                // No counter storage is specified in the URL.
                return null;
            }
        };
        /**
        * Gets the time series from the current web browser address. Returns null if no time series name was found.
        */
        appUrl.getTimeSeries = function () {
            // TODO: instead of string parsing, can we pull this from durandal.activeInstruction()?
            var timeSeriesIndicator = "timeseries=";
            var hash = window.location.hash;
            var tsIndex = hash.indexOf(timeSeriesIndicator);
            if (tsIndex >= 0) {
                // A database is specified in the address.
                var tsSegmentEnd = hash.indexOf("&", tsIndex);
                if (tsSegmentEnd === -1) {
                    tsSegmentEnd = hash.length;
                }
                var timeSeriesName = hash.substring(tsIndex + timeSeriesIndicator.length, tsSegmentEnd);
                var unescapedTimeSeriesName = decodeURIComponent(timeSeriesName);
                var ts = new timeSeries(unescapedTimeSeriesName);
                return ts;
            }
            else {
                // No time series is specified in the URL.
                return null;
            }
        };
        /**
        * Gets the server URL.
        */
        appUrl.forServer = function () {
            return window.location.protocol + "//" + window.location.host + appUrl.baseUrl;
        };
        /**
        * Gets the address for the current page but for the specified resource.
        */
        appUrl.forCurrentPage = function (rs) {
            var routerInstruction = router.activeInstruction();
            if (routerInstruction) {
                var currentResourceName = null;
                var currentResourceType = null;
                var dbInUrl = routerInstruction.queryParams[database.type];
                if (dbInUrl) {
                    currentResourceName = dbInUrl;
                    currentResourceType = database.type;
                }
                else {
                    var fsInUrl = routerInstruction.queryParams[filesystem.type];
                    if (fsInUrl) {
                        currentResourceName = fsInUrl;
                        currentResourceType = filesystem.type;
                    }
                    else {
                        var csInUrl = routerInstruction.queryParams[counterStorage.type];
                        if (csInUrl) {
                            currentResourceName = csInUrl;
                            currentResourceType = counterStorage.type;
                        }
                        else {
                            var tsInUrl = routerInstruction.queryParams[timeSeries.type];
                            if (tsInUrl) {
                                currentResourceName = tsInUrl;
                                currentResourceType = timeSeries.type;
                            }
                        }
                    }
                }
                if (currentResourceType && currentResourceType != rs.type) {
                    // user changed resource type - navigate to resources page and preselect resource
                    return appUrl.forResources() + "?" + rs.type + "=" + encodeURIComponent(rs.name);
                }
                var isDifferentDbInAddress = !currentResourceName || currentResourceName !== rs.name.toLowerCase();
                if (isDifferentDbInAddress) {
                    var existingAddress = window.location.hash;
                    var existingDbQueryString = currentResourceName ? rs.type + "=" + encodeURIComponent(currentResourceName) : null;
                    var newDbQueryString = rs.type + "=" + encodeURIComponent(rs.name);
                    var newUrlWithDatabase = existingDbQueryString ?
                        existingAddress.replace(existingDbQueryString, newDbQueryString) :
                        existingAddress + (window.location.hash.indexOf("?") >= 0 ? "&" : "?") + rs.type + "=" + encodeURIComponent(rs.name);
                    return newUrlWithDatabase;
                }
            }
        };
        /**
        * Gets an object containing computed URLs that update when the current database updates.
        */
        appUrl.forCurrentDatabase = function () {
            return appUrl.currentDbComputeds;
        };
        appUrl.forCurrentFilesystem = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.forCurrentCounterStorage = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.forCurrentTimeSeries = function () {
            return appUrl.currentDbComputeds; //This is all mixed. maybe there should be separate structures for Db and Fs and Cs.
        };
        appUrl.getEncodedResourcePart = function (res) {
            if (!res)
                return "";
            if (res instanceof filesystem) {
                return appUrl.getEncodedFsPart(res);
            }
            if (res instanceof counterStorage) {
                return appUrl.getEncodedCounterStoragePart(res);
            }
            if (res instanceof timeSeries) {
                return appUrl.getEncodedTimeSeriesPart(res);
            }
            else {
                return appUrl.getEncodedDbPart(res);
            }
        };
        appUrl.getEncodedDbPart = function (db) {
            return db ? "&database=" + encodeURIComponent(db.name) : "";
        };
        appUrl.getEncodedFsPart = function (fs) {
            return fs ? "&filesystem=" + encodeURIComponent(fs.name) : "";
        };
        appUrl.getEncodedCounterPart = function (cs) {
            return cs ? "&counterstorage=" + encodeURIComponent(cs.name) : "";
        };
        appUrl.mapUnknownRoutes = function (router) {
            router.mapUnknownRoutes(function (instruction) {
                var queryString = !!instruction.queryString ? ("?" + instruction.queryString) : "";
                if (instruction.fragment === "has-api-key" || instruction.fragment.indexOf("api-key") === 0) {
                    // reload page to reinitialize shell and properly consume/provide OAuth token
                    location.reload();
                }
                else {
                    messagePublisher.reportError("Unknown route", "The route " + instruction.fragment + queryString + " doesn't exist, redirecting...");
                    var fragment = instruction.fragment;
                    var appUrls = appUrl.currentDbComputeds;
                    var newLoationHref;
                    if (fragment.indexOf("admin/settings") == 0) {
                        newLoationHref = appUrls.adminSettings();
                    }
                    else {
                        newLoationHref = appUrls.resourcesManagement();
                    }
                    location.href = newLoationHref;
                }
            });
        };
        appUrl.urlEncodeArgs = function (args) {
            var propNameAndValues = [];
            for (var prop in args) {
                var value = args[prop];
                if (value instanceof Array) {
                    for (var i = 0; i < value.length; i++) {
                        propNameAndValues.push(prop + "=" + encodeURIComponent(value[i]));
                    }
                }
                else if (value !== undefined) {
                    propNameAndValues.push(prop + "=" + encodeURIComponent(value));
                }
            }
            return "?" + propNameAndValues.join("&");
        };
        //private static baseUrl = "http://localhost:8080"; // For debugging purposes, uncomment this line to point Raven at an already-running Raven server. Requires the Raven server to have it's config set to <add key="Raven/AccessControlAllowOrigin" value="*" />
        appUrl.baseUrl = appUrl.detectAppUrl(); // This should be used when serving HTML5 Studio from the server app.
        appUrl.currentDatabase = ko.observable().subscribeTo("ActivateDatabase", true);
        appUrl.currentFilesystem = ko.observable().subscribeTo("ActivateFilesystem", true);
        appUrl.currentCounterStorage = ko.observable().subscribeTo("ActivateCounterStorage", true);
        appUrl.currentTimeSeries = ko.observable().subscribeTo("ActivateTimeSeries", true);
        // Stores some computed values that update whenever the current database updates.
        appUrl.currentDbComputeds = {
            adminSettings: ko.computed(function () { return appUrl.forAdminSettings(); }),
            adminSettingsCluster: ko.computed(function () { return appUrl.forCluster(); }),
            hasApiKey: ko.computed(function () { return appUrl.forHasApiKey(); }),
            resources: ko.computed(function () { return appUrl.forResources(); }),
            documents: ko.computed(function () { return appUrl.forDocuments(null, appUrl.currentDatabase()); }),
            conflicts: ko.computed(function () { return appUrl.forConflicts(appUrl.currentDatabase()); }),
            patch: ko.computed(function () { return appUrl.forPatch(appUrl.currentDatabase()); }),
            indexes: ko.computed(function () { return appUrl.forIndexes(appUrl.currentDatabase()); }),
            megeSuggestions: ko.computed(function () { return appUrl.forMegeSuggestions(appUrl.currentDatabase()); }),
            upgrade: ko.computed(function () { return appUrl.forUpgrade(appUrl.currentDatabase()); }),
            transformers: ko.computed(function () { return appUrl.forTransformers(appUrl.currentDatabase()); }),
            newIndex: ko.computed(function () { return appUrl.forNewIndex(appUrl.currentDatabase()); }),
            editIndex: function (indexName) { return ko.computed(function () { return appUrl.forEditIndex(indexName, appUrl.currentDatabase()); }); },
            newTransformer: ko.computed(function () { return appUrl.forNewTransformer(appUrl.currentDatabase()); }),
            editTransformer: function (transformerName) { return ko.computed(function () { return appUrl.forEditTransformer(transformerName, appUrl.currentDatabase()); }); },
            query: function (indexName) { return ko.computed(function () { return appUrl.forQuery(appUrl.currentDatabase(), indexName); }); },
            reporting: ko.computed(function () { return appUrl.forReporting(appUrl.currentDatabase()); }),
            exploration: ko.computed(function () { return appUrl.forExploration(appUrl.currentDatabase()); }),
            tasks: ko.computed(function () { return appUrl.forTasks(appUrl.currentDatabase()); }),
            status: ko.computed(function () { return appUrl.forStatus(appUrl.currentDatabase()); }),
            replicationPerfStats: ko.computed(function () { return appUrl.forReplicationPerfStats(appUrl.currentDatabase()); }),
            sqlReplicationPerfStats: ko.computed(function () { return appUrl.forSqlReplicationPerfStats(appUrl.currentDatabase()); }),
            requestsCount: ko.computed(function () { return appUrl.forRequestsCount(appUrl.currentDatabase()); }),
            requestsTracing: ko.computed(function () { return appUrl.forRequestsTracing(appUrl.currentDatabase()); }),
            indexPerformance: ko.computed(function () { return appUrl.forIndexPerformance(appUrl.currentDatabase()); }),
            indexStats: ko.computed(function () { return appUrl.forIndexStats(appUrl.currentDatabase()); }),
            indexBatchSize: ko.computed(function () { return appUrl.forIndexBatchSize(appUrl.currentDatabase()); }),
            indexPrefetches: ko.computed(function () { return appUrl.forIndexPrefetches(appUrl.currentDatabase()); }),
            settings: ko.computed(function () { return appUrl.forSettings(appUrl.currentDatabase()); }),
            logs: ko.computed(function () { return appUrl.forLogs(appUrl.currentDatabase()); }),
            runningTasks: ko.computed(function () { return appUrl.forRunningTasks(appUrl.currentDatabase()); }),
            alerts: ko.computed(function () { return appUrl.forAlerts(appUrl.currentDatabase()); }),
            indexErrors: ko.computed(function () { return appUrl.forIndexErrors(appUrl.currentDatabase()); }),
            replicationStats: ko.computed(function () { return appUrl.forReplicationStats(appUrl.currentDatabase()); }),
            userInfo: ko.computed(function () { return appUrl.forUserInfo(appUrl.currentDatabase()); }),
            visualizer: ko.computed(function () { return appUrl.forVisualizer(appUrl.currentDatabase()); }),
            databaseSettings: ko.computed(function () { return appUrl.forDatabaseSettings(appUrl.currentDatabase()); }),
            quotas: ko.computed(function () { return appUrl.forQuotas(appUrl.currentDatabase()); }),
            periodicExport: ko.computed(function () { return appUrl.forPeriodicExport(appUrl.currentDatabase()); }),
            replications: ko.computed(function () { return appUrl.forReplications(appUrl.currentDatabase()); }),
            hotSpare: ko.computed(function () { return appUrl.forHotSpare(); }),
            versioning: ko.computed(function () { return appUrl.forVersioning(appUrl.currentDatabase()); }),
            sqlReplications: ko.computed(function () { return appUrl.forSqlReplications(appUrl.currentDatabase()); }),
            editSqlReplication: ko.computed(function (sqlReplicationName) { return appUrl.forEditSqlReplication(sqlReplicationName, appUrl.currentDatabase()); }),
            sqlReplicationsConnections: ko.computed(function () { return appUrl.forSqlReplicationConnections(appUrl.currentDatabase()); }),
            scriptedIndexes: ko.computed(function () { return appUrl.forScriptedIndexes(appUrl.currentDatabase()); }),
            customFunctionsEditor: ko.computed(function () { return appUrl.forCustomFunctionsEditor(appUrl.currentDatabase()); }),
            databaseStudioConfig: ko.computed(function () { return appUrl.forDatabaseStudioConfig(appUrl.currentDatabase()); }),
            statusDebug: ko.computed(function () { return appUrl.forStatusDebug(appUrl.currentDatabase()); }),
            statusDebugChanges: ko.computed(function () { return appUrl.forStatusDebugChanges(appUrl.currentDatabase()); }),
            statusDebugMetrics: ko.computed(function () { return appUrl.forStatusDebugMetrics(appUrl.currentDatabase()); }),
            statusDebugConfig: ko.computed(function () { return appUrl.forStatusDebugConfig(appUrl.currentDatabase()); }),
            statusDebugDocrefs: ko.computed(function () { return appUrl.forStatusDebugDocrefs(appUrl.currentDatabase()); }),
            statusDebugCurrentlyIndexing: ko.computed(function () { return appUrl.forStatusDebugCurrentlyIndexing(appUrl.currentDatabase()); }),
            statusDebugQueries: ko.computed(function () { return appUrl.forStatusDebugQueries(appUrl.currentDatabase()); }),
            statusDebugTasks: ko.computed(function () { return appUrl.forStatusDebugTasks(appUrl.currentDatabase()); }),
            statusDebugRoutes: ko.computed(function () { return appUrl.forStatusDebugRoutes(appUrl.currentDatabase()); }),
            statusDebugSqlReplication: ko.computed(function () { return appUrl.forStatusDebugSqlReplication(appUrl.currentDatabase()); }),
            statusDebugIndexFields: ko.computed(function () { return appUrl.forStatusDebugIndexFields(appUrl.currentDatabase()); }),
            statusDebugIdentities: ko.computed(function () { return appUrl.forStatusDebugIdentities(appUrl.currentDatabase()); }),
            statusDebugWebSocket: ko.computed(function () { return appUrl.forStatusDebugWebSocket(appUrl.currentDatabase()); }),
            statusDebugPersistAutoIndex: ko.computed(function () { return appUrl.forStatusDebugPersistAutoIndex(appUrl.currentDatabase()); }),
            statusDebugExplainReplication: ko.computed(function () { return appUrl.forStatusDebugExplainReplication(appUrl.currentDatabase()); }),
            infoPackage: ko.computed(function () { return appUrl.forInfoPackage(appUrl.currentDatabase()); }),
            dataSubscriptions: ko.computed(function () { return appUrl.forDataSubscriptions(appUrl.currentDatabase()); }),
            statusStorageOnDisk: ko.computed(function () { return appUrl.forStatusStorageOnDisk(appUrl.currentDatabase()); }),
            statusStorageBreakdown: ko.computed(function () { return appUrl.forStatusStorageBreakdown(appUrl.currentDatabase()); }),
            statusStorageCollections: ko.computed(function () { return appUrl.forStatusStorageCollections(appUrl.currentDatabase()); }),
            isAreaActive: function (routeRoot) { return ko.pureComputed(function () { return appUrl.checkIsAreaActive(routeRoot); }); },
            isActive: function (routeTitle) { return ko.pureComputed(function () { return router.navigationModel().first(function (m) { return m.isActive() && m.title === routeTitle; }) != null; }); },
            resourcesManagement: ko.computed(function () { return appUrl.forResources(); }),
            filesystemFiles: ko.computed(function () { return appUrl.forFilesystemFiles(appUrl.currentFilesystem()); }),
            filesystemSearch: ko.computed(function () { return appUrl.forFilesystemSearch(appUrl.currentFilesystem()); }),
            filesystemSynchronization: ko.computed(function () { return appUrl.forFilesystemSynchronization(appUrl.currentFilesystem()); }),
            filesystemStatus: ko.computed(function () { return appUrl.forFilesystemStatus(appUrl.currentFilesystem()); }),
            filesystemTasks: ko.computed(function () { return appUrl.forFilesystemTasks(appUrl.currentFilesystem()); }),
            filesystemSettings: ko.computed(function () { return appUrl.forFilesystemSettings(appUrl.currentFilesystem()); }),
            filesystemSynchronizationDestinations: ko.computed(function () { return appUrl.forFilesystemSynchronizationDestinations(appUrl.currentFilesystem()); }),
            filesystemSynchronizationConfiguration: ko.computed(function () { return appUrl.forFilesystemSynchronizationConfiguration(appUrl.currentFilesystem()); }),
            filesystemConfiguration: ko.computed(function () { return appUrl.forFilesystemConfiguration(appUrl.currentFilesystem()); }),
            filesystemVersioning: ko.computed(function () { return appUrl.forFilesystemVersioning(appUrl.currentFilesystem()); }),
            counterStorages: ko.computed(function () { return appUrl.forCounterStorages(); }),
            counterStorageCounters: ko.computed(function () { return appUrl.forCounterStorageCounters(null, appUrl.currentCounterStorage()); }),
            counterStorageReplication: ko.computed(function () { return appUrl.forCounterStorageReplication(appUrl.currentCounterStorage()); }),
            counterStorageTasks: ko.computed(function () { return appUrl.forCounterStorageTasks(appUrl.currentCounterStorage()); }),
            counterStorageStats: ko.computed(function () { return appUrl.forCounterStorageStats(appUrl.currentCounterStorage()); }),
            counterStorageConfiguration: ko.computed(function () { return appUrl.forCounterStorageConfiguration(appUrl.currentCounterStorage()); }),
            timeSeriesType: ko.computed(function () { return appUrl.forTimeSeriesType(null, appUrl.currentTimeSeries()); }),
            timeSeriesPoints: ko.computed(function () { return appUrl.forTimeSeriesKey(null, null, appUrl.currentTimeSeries()); }),
            timeSeriesStats: ko.computed(function () { return appUrl.forTimeSeriesStats(appUrl.currentTimeSeries()); }),
            timeSeriesConfiguration: ko.computed(function () { return appUrl.forTimeSeriesConfiguration(appUrl.currentTimeSeries()); }),
            timeSeriesConfigurationTypes: ko.computed(function () { return appUrl.forTimeSeriesConfigurationTypes(appUrl.currentTimeSeries()); })
        };
        appUrl.warnWhenUsingSystemDatabase = true;
        return appUrl;
    })();
    return appUrl;
});
define("common/autoCompleterSupport", ["require", "exports", 'common/inputCursor'], function (require, exports, inputCursor) {
    var autoCompleterSupport = (function () {
        function autoCompleterSupport(autoCompleteBase, autoCompleteResults, showAllOptionOnEmptyInput) {
            if (showAllOptionOnEmptyInput === void 0) { showAllOptionOnEmptyInput = false; }
            this.autoCompleteBase = autoCompleteBase;
            this.autoCompleteResults = autoCompleteResults;
            this.showAllOptionOnEmptyInput = showAllOptionOnEmptyInput;
        }
        autoCompleterSupport.prototype.searchForCompletions = function (input) {
            this.autoCompleteResults([]);
            var typedWord = this.getWordUserIsTyping(input);
            if (typedWord.length >= 1 || this.showAllOptionOnEmptyInput) {
                this.autoCompleteResults(this.autoCompleteBase().filter(function (value) {
                    return autoCompleterSupport.wordMatches(typedWord, value()) &&
                        (value() !== input.val()) &&
                        (value() !== typedWord) &&
                        (value().indexOf(' ') === -1);
                }));
            }
        };
        autoCompleterSupport.prototype.completeTheWord = function (input, selectedCompletion, updateObservableClouse) {
            if (updateObservableClouse === void 0) { updateObservableClouse = null; }
            if (input.length > 0) {
                var inputValue = input.val();
                var typedWord = this.getWordUserIsTyping(input);
                var cursorPosition = inputCursor.getPosition(input);
                var beginIndex = this.findWordStartWithEndPosition(inputValue, cursorPosition - 1) + 1;
                // update observable here as input has on key up update
                var newValue = inputValue.substring(0, beginIndex) +
                    selectedCompletion +
                    inputValue.substring(cursorPosition);
                input.val(newValue);
                if (updateObservableClouse) {
                    updateObservableClouse(newValue);
                }
                var positionCorrection = 0;
                if (selectedCompletion[selectedCompletion.length - 1] === ")") {
                    positionCorrection = -1;
                }
                inputCursor.setPosition(input, beginIndex + selectedCompletion.length + positionCorrection);
                this.autoCompleteResults([]);
            }
        };
        autoCompleterSupport.prototype.findWordStartWithEndPosition = function (inputValue, endPosition) {
            var beginIndex = 0;
            for (beginIndex = endPosition; beginIndex >= 0; beginIndex--) {
                var charCode = inputValue.charCodeAt(beginIndex);
                // going back skip every alphanumeric characters
                if ((48 <= charCode && charCode <= 57) // char in range from '0' to '9'
                    || (65 <= charCode && charCode <= 90) // char in range from 'A' to 'Z'
                    || (97 <= charCode && charCode <= 122) // char in range from 'a' to 'z'
                    || (charCode == 95) // char is '_'
                ) {
                    continue;
                }
                else {
                    break;
                }
            }
            return beginIndex;
        };
        autoCompleterSupport.prototype.getWordUserIsTyping = function ($input) {
            var cursorPosition = inputCursor.getPosition($input);
            //var beginIndex = $input.val().lastIndexOf(' ', cursorPosition-1);
            var beginIndex = this.findWordStartWithEndPosition($input.val(), cursorPosition - 1) + 1;
            var endIndex = $input.val().indexOf(' ', cursorPosition);
            if (endIndex === -1) {
                endIndex = $input.val().length;
            }
            return $input.val().substring(beginIndex, cursorPosition).trim();
        };
        autoCompleterSupport.wordMatches = function (toCheck, toMatch) {
            // ignore the case
            toCheck = toCheck.toLowerCase();
            toMatch = toMatch.toLowerCase();
            // match as long as the letters are in correct order
            var matchedChars = 0;
            for (var i = 0; i < toMatch.length; i++) {
                if (toCheck[matchedChars] === toMatch[i]) {
                    matchedChars++;
                }
                if (matchedChars >= toCheck.length) {
                    return true;
                }
            }
            return false;
        };
        return autoCompleterSupport;
    })();
    return autoCompleterSupport;
});
define("common/awesomeMultiselect", ["require", "exports"], function (require, exports) {
    /**
     * This is helper class to build boostrap multiselect with support for custom awesome checkboxes
     *
     * https://github.com/davidstutz/bootstrap-multiselect/issues/576
     */
    var awesomeMultiselect = (function () {
        function awesomeMultiselect() {
        }
        /**
         * Build dropdown
         * @param object
         */
        awesomeMultiselect.build = function (object) {
            object.multiselect({
                templates: {
                    li: '<li><div class="checkbox"><label></label></div></li>'
                }
            });
            if (object.data('instanceData')) {
                throw new Error("Object was already initialized. Use rebuild instead.");
            }
            object.data('instanceData', awesomeMultiselect.instanceNumber);
            awesomeMultiselect.instanceNumber++;
            awesomeMultiselect.fixAwesomeCheckboxes(object);
        };
        /**
         * Update multiselect
         * @param object
         */
        awesomeMultiselect.rebuild = function (object) {
            if (!object.data('instanceData')) {
                throw new Error("Please intialize multiselect using awesomeMultiselect.build before calling rebuild");
            }
            object.multiselect('rebuild');
            awesomeMultiselect.fixAwesomeCheckboxes(object);
        };
        awesomeMultiselect.fixAwesomeCheckboxes = function (object) {
            var instanceId = object.data('instanceData');
            $('.multiselect-container .checkbox', object.parent()).each(function (index) {
                var $self = $(this);
                var id = 'multiselect-' + instanceId + "-" + index;
                var $input = $self.find('input');
                var $label = $self.find('label');
                // check if DOM was already modified
                if (!$label.attr('for')) {
                    $input.detach();
                    $input.prependTo($self);
                    $self.click(function (e) { return e.stopPropagation(); });
                }
                $label.attr('for', id);
                $input.attr('id', id);
            });
        };
        // used for generating unique label/ids
        awesomeMultiselect.instanceNumber = 1;
        return awesomeMultiselect;
    })();
    return awesomeMultiselect;
});
/// <reference path="../../Scripts/typings/jquery/jquery.d.ts" />
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/changesApi", ["require", "exports", "common/appUrl", "common/changeSubscription", "common/changesCallback", "commands/commandBase", "models/filesystem/folder", "commands/auth/getSingleAuthTokenCommand", "common/idGenerator", "common/eventSourceSettingStorage", "common/changesApiWarnStorage", "common/messagePublisher"], function (require, exports, appUrl, changeSubscription, changesCallback, commandBase, folder, getSingleAuthTokenCommand, idGenerator, eventSourceSettingStorage, changesApiWarnStorage, messagePublisher) {
    var changesApi = (function () {
        function changesApi(rs, coolDownWithDataLoss, isMultyTenantTransport) {
            if (coolDownWithDataLoss === void 0) { coolDownWithDataLoss = 0; }
            if (isMultyTenantTransport === void 0) { isMultyTenantTransport = false; }
            this.rs = rs;
            this.readyStateOpen = 1;
            this.isDisposing = false;
            this.disposed = false;
            this.isCleanClose = false;
            this.normalClosureCode = 1000;
            this.normalClosureMessage = "CLOSE_NORMAL";
            this.successfullyConnectedOnce = false;
            this.sentMessages = [];
            this.commandBase = new commandBase();
            this.allReplicationConflicts = ko.observableArray();
            this.allDocsHandlers = ko.observableArray();
            this.allIndexesHandlers = ko.observableArray();
            this.allTransformersHandlers = ko.observableArray();
            this.watchedPrefixes = {};
            this.allBulkInsertsHandlers = ko.observableArray();
            this.allFsSyncHandlers = ko.observableArray();
            this.allFsConflictsHandlers = ko.observableArray();
            this.allFsConfigHandlers = ko.observableArray();
            this.allFsDestinationsHandlers = ko.observableArray();
            this.watchedFolders = {};
            this.allCountersHandlers = ko.observableArray();
            this.watchedCounter = {};
            this.watchedCountersInGroup = {};
            this.allCounterBulkOperationsHandlers = ko.observableArray();
            this.allTimeSeriesHandlers = ko.observableArray();
            this.watchedTimeSeries = {};
            this.allTimeSeriesBulkOperationsHandlers = ko.observableArray();
            this.eventsId = idGenerator.generateId();
            this.coolDownWithDataLoss = coolDownWithDataLoss;
            this.isMultyTenantTransport = isMultyTenantTransport;
            this.resourcePath = appUrl.forResourceQuery(this.rs);
            this.connectToChangesApiTask = $.Deferred();
            if ("WebSocket" in window && changesApi.isServerSupportingWebSockets) {
                this.connect(this.connectWebSocket);
            }
            else if ("EventSource" in window) {
                if (eventSourceSettingStorage.useEventSource()) {
                    this.connect(this.connectEventSource);
                }
                else {
                    this.connectToChangesApiTask.reject();
                }
            }
            else {
                //The browser doesn't support nor websocket nor eventsource
                //or we are in IE10 or IE11 and the server doesn't support WebSockets.
                //Anyway, at this point a warning message was already shown. 
                this.connectToChangesApiTask.reject();
            }
        }
        changesApi.prototype.connect = function (action, recoveringFromWebsocketFailure) {
            var _this = this;
            if (recoveringFromWebsocketFailure === void 0) { recoveringFromWebsocketFailure = false; }
            if (this.disposed) {
                if (!!this.connectToChangesApiTask)
                    this.connectToChangesApiTask.resolve();
                return;
            }
            if (!recoveringFromWebsocketFailure) {
                this.connectToChangesApiTask = $.Deferred();
            }
            var getTokenTask = new getSingleAuthTokenCommand(this.rs).execute();
            getTokenTask
                .done(function (tokenObject) {
                _this.rs.isLoading(false);
                var token = tokenObject.Token;
                var connectionString = "singleUseAuthToken=" + token + "&id=" + _this.eventsId + "&coolDownWithDataLoss=" + _this.coolDownWithDataLoss + "&isMultyTenantTransport=" + _this.isMultyTenantTransport;
                action.call(_this, connectionString);
            })
                .fail(function (e) {
                if (_this.isDisposing) {
                    _this.connectToChangesApiTask.reject();
                    return;
                }
                var error = !!e.responseJSON ? e.responseJSON.Error : e.responseText;
                if (e.status === 0) {
                    // Connection has closed so try to reconnect every 3 seconds.
                    setTimeout(function () { return _this.connect(action); }, 3 * 1000);
                }
                else if (e.status === 503 /* ServiceUnavailable */) {
                    // We're still loading the database, try to reconnect every 2 seconds.
                    if (_this.rs.isLoading() === false) {
                        _this.commandBase.reportError(error || "Failed to connect to changes", e.responseText, e.statusText);
                    }
                    _this.rs.isLoading(true);
                    setTimeout(function () { return _this.connect(action, true); }, 2 * 1000);
                }
                else if (e.status !== 403 /* Forbidden */) {
                    _this.commandBase.reportError(error || "Failed to connect to changes", e.responseText, e.StatusText);
                    _this.connectToChangesApiTask.reject();
                }
            });
        };
        changesApi.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === "https:" ? "wss://" : "ws://";
            var url = wsProtocol + window.location.host + this.resourcePath + "/changes/websocket?" + connectionString;
            this.webSocket = new WebSocket(url);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened === false) {
                    _this.serverNotSupportingWebsocketsErrorHandler();
                }
                else {
                    _this.onError(e);
                }
            };
            this.webSocket.onclose = function (e) {
                if (_this.isCleanClose === false && changesApi.isServerSupportingWebSockets) {
                    // Connection has closed uncleanly, so try to reconnect.
                    _this.connect(_this.connectWebSocket);
                }
            };
            this.webSocket.onopen = function () {
                console.log("Connected to WebSocket changes API (" + _this.rs.fullTypeName + " = " + _this.rs.name + ")");
                _this.reconnect();
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectToChangesApiTask.resolve();
            };
        };
        changesApi.prototype.connectEventSource = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            this.eventSource = new EventSource(this.resourcePath + "/changes/events?" + connectionString);
            this.eventSource.onmessage = function (e) { return _this.onMessage(e); };
            this.eventSource.onerror = function (e) {
                if (connectionOpened === false) {
                    _this.connectToChangesApiTask.reject();
                }
                else {
                    _this.onError(e);
                    _this.eventSource.close();
                    _this.connect(_this.connectEventSource);
                }
            };
            this.eventSource.onopen = function () {
                console.log("Connected to EventSource changes API (" + _this.rs.fullTypeName + " = " + _this.rs.name + ")");
                _this.reconnect();
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectToChangesApiTask.resolve();
            };
        };
        changesApi.prototype.reconnect = function () {
            var _this = this;
            if (this.successfullyConnectedOnce) {
                //send changes connection args after reconnecting
                this.sentMessages.forEach(function (args) { return _this.send(args.command, args.value, false); });
                ko.postbox.publish("ChangesApiReconnected", this.rs);
                if (changesApi.messageWasShownOnce) {
                    this.commandBase.reportSuccess("Successfully reconnected to changes stream!");
                    changesApi.messageWasShownOnce = false;
                }
            }
        };
        changesApi.prototype.onError = function (e) {
            if (changesApi.messageWasShownOnce === false) {
                this.commandBase.reportError("Changes stream was disconnected!", "Retrying connection shortly.");
                changesApi.messageWasShownOnce = true;
            }
        };
        changesApi.prototype.serverNotSupportingWebsocketsErrorHandler = function () {
            var warningMessage;
            var details;
            if ("EventSource" in window) {
                if (eventSourceSettingStorage.useEventSource()) {
                    this.connect(this.connectEventSource, true);
                    warningMessage = "Your server doesn't support the WebSocket protocol!";
                    details = "EventSource API is going to be used instead. However, multi tab usage isn't supported.\r\n" +
                        "WebSockets are only supported on servers running on Windows Server 2012 and equivalent. \r\n" +
                        " If you have issues with WebSockets on Windows Server 2012 and equivalent use Status > Debug > WebSocket to debug.";
                    if (changesApi.isServerSupportingWebSockets) {
                        changesApi.isServerSupportingWebSockets = false;
                        if (changesApiWarnStorage.showChangesApiWarning()) {
                            this.commandBase.reportWarningWithButton(warningMessage, details, "Don't warn me again", function () {
                                if (changesApiWarnStorage.showChangesApiWarning()) {
                                    changesApiWarnStorage.setValue(true);
                                    messagePublisher.reportInfo("Disabled notification about WebSocket.");
                                }
                            });
                        }
                    }
                }
                else {
                    changesApi.isServerSupportingWebSockets = false;
                    this.connectToChangesApiTask.reject();
                }
            }
            else {
                this.connectToChangesApiTask.reject();
                warningMessage = "Changes API is Disabled!";
                details = "Your server doesn't support the WebSocket protocol and your browser doesn't support the EventSource API.\r\n" +
                    "In order to use it, please use a browser that supports the EventSource API.";
                if (changesApi.isServerSupportingWebSockets) {
                    changesApi.isServerSupportingWebSockets = false;
                    this.commandBase.reportWarning(warningMessage, details);
                }
            }
        };
        changesApi.prototype.send = function (command, value, needToSaveSentMessages) {
            var _this = this;
            if (needToSaveSentMessages === void 0) { needToSaveSentMessages = true; }
            this.connectToChangesApiTask.done(function () {
                var args = {
                    id: _this.eventsId,
                    command: command
                };
                if (value !== undefined) {
                    args["value"] = value;
                }
                //TODO: exception handling?
                _this.commandBase.query("/changes/config", args, _this.rs)
                    .done(function () { return _this.saveSentMessages(needToSaveSentMessages, command, args); });
            });
        };
        changesApi.prototype.saveSentMessages = function (needToSaveSentMessages, command, args) {
            if (needToSaveSentMessages) {
                if (command.slice(0, 2) === "un") {
                    var commandName = command.slice(2, command.length);
                    this.sentMessages = this.sentMessages.filter(function (msg) { return msg.command != commandName; });
                }
                else {
                    this.sentMessages.push(args);
                }
            }
        };
        changesApi.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        changesApi.prototype.onMessage = function (e) {
            var eventDto = JSON.parse(e.data);
            var eventType = eventDto.Type;
            if (eventType === "Heartbeat")
                return;
            var value = eventDto.Value;
            if (eventType === "DocumentChangeNotification") {
                this.fireEvents(this.allDocsHandlers(), value, function (event) { return true; });
                for (var key in this.watchedPrefixes) {
                    var docCallbacks = this.watchedPrefixes[key];
                    this.fireEvents(docCallbacks(), value, function (event) { return event.Id != null && event.Id.match("^" + key); });
                }
            }
            else if (eventType === "IndexChangeNotification") {
                this.fireEvents(this.allIndexesHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "TransformerChangeNotification") {
                this.fireEvents(this.allTransformersHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "BulkInsertChangeNotification") {
                this.fireEvents(this.allBulkInsertsHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "SynchronizationUpdateNotification") {
                this.fireEvents(this.allFsSyncHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "ReplicationConflictNotification") {
                this.fireEvents(this.allReplicationConflicts(), value, function (event) { return true; });
            }
            else if (eventType === "ConflictNotification") {
                this.fireEvents(this.allFsConflictsHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "FileChangeNotification") {
                for (var key in this.watchedFolders) {
                    var folderCallbacks = this.watchedFolders[key];
                    this.fireEvents(folderCallbacks(), value, function (event) {
                        var notifiedFolder = folder.getFolderFromFilePath(event.File);
                        var match = null;
                        if (notifiedFolder && notifiedFolder.path) {
                            match = notifiedFolder.path.match(key);
                        }
                        return match && match.length > 0;
                    });
                }
            }
            else if (eventType === "ConfigurationChangeNotification") {
                if (value.Name.indexOf("Raven/Synchronization/Destinations") >= 0) {
                    this.fireEvents(this.allFsDestinationsHandlers(), value, function (e) { return true; });
                }
                this.fireEvents(this.allFsConfigHandlers(), value, function (e) { return true; });
            }
            else if (eventType === "ChangeNotification") {
                this.fireEvents(this.allCountersHandlers(), value, function (event) { return true; });
            }
            else if (eventType === "KeyChangeNotification") {
                this.fireEvents(this.allTimeSeriesHandlers(), value, function (event) { return true; });
            }
            else {
                console.log("Unhandled Changes API notification type: " + eventType);
            }
        };
        changesApi.prototype.watchAllIndexes = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allIndexesHandlers().length === 0) {
                this.send("watch-indexes");
            }
            this.allIndexesHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allIndexesHandlers.remove(callback);
                if (_this.allIndexesHandlers().length === 0) {
                    _this.send("unwatch-indexes");
                }
            });
        };
        changesApi.prototype.watchAllTransformers = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allTransformersHandlers().length === 0) {
                this.send("watch-transformers");
            }
            this.allTransformersHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allTransformersHandlers.remove(callback);
                if (_this.allTransformersHandlers().length === 0) {
                    _this.send("unwatch-transformers");
                }
            });
        };
        changesApi.prototype.watchAllReplicationConflicts = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allReplicationConflicts().length === 0) {
                this.send("watch-replication-conflicts");
            }
            this.allReplicationConflicts.push(callback);
            return new changeSubscription(function () {
                _this.allReplicationConflicts.remove(callback);
                if (_this.allReplicationConflicts().length === 0) {
                    _this.send("unwatch-replication-conflicts");
                }
            });
        };
        changesApi.prototype.watchAllDocs = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allDocsHandlers().length === 0) {
                this.send("watch-docs");
            }
            this.allDocsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allDocsHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send("unwatch-docs");
                }
            });
        };
        changesApi.prototype.watchDocsStartingWith = function (docIdPrefix, onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (typeof (this.watchedPrefixes[docIdPrefix]) === "undefined") {
                this.send("watch-prefix", docIdPrefix);
                this.watchedPrefixes[docIdPrefix] = ko.observableArray();
            }
            this.watchedPrefixes[docIdPrefix].push(callback);
            return new changeSubscription(function () {
                _this.watchedPrefixes[docIdPrefix].remove(callback);
                if (_this.watchedPrefixes[docIdPrefix]().length === 0) {
                    delete _this.watchedPrefixes[docIdPrefix];
                    _this.send("unwatch-prefix", docIdPrefix);
                }
            });
        };
        changesApi.prototype.watchBulks = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allBulkInsertsHandlers().length === 0) {
                this.send("watch-bulk-operation");
            }
            this.allBulkInsertsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allBulkInsertsHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send('unwatch-bulk-operation');
                }
            });
        };
        changesApi.prototype.watchFsSync = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allFsSyncHandlers().length === 0) {
                this.send("watch-sync");
            }
            this.allFsSyncHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allFsSyncHandlers.remove(callback);
                if (_this.allFsSyncHandlers().length === 0) {
                    _this.send("unwatch-sync");
                }
            });
        };
        changesApi.prototype.watchFsConflicts = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allFsConflictsHandlers().length === 0) {
                this.send("watch-conflicts");
            }
            this.allFsConflictsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allFsConflictsHandlers.remove(callback);
                if (_this.allFsConflictsHandlers().length === 0) {
                    _this.send("unwatch-conflicts");
                }
            });
        };
        changesApi.prototype.watchFsFolders = function (folder, onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (typeof (this.watchedFolders[folder]) === "undefined") {
                this.send("watch-folder", folder);
                this.watchedFolders[folder] = ko.observableArray();
            }
            this.watchedFolders[folder].push(callback);
            return new changeSubscription(function () {
                _this.watchedFolders[folder].remove(callback);
                if (_this.watchedFolders[folder].length === 0) {
                    delete _this.watchedFolders[folder];
                    _this.send("unwatch-folder", folder);
                }
            });
        };
        changesApi.prototype.watchFsConfig = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allFsConfigHandlers().length === 0) {
                this.send("watch-config");
            }
            this.allFsConfigHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allFsConfigHandlers.remove(callback);
                if (_this.allFsConfigHandlers().length === 0) {
                    _this.send("unwatch-config");
                }
            });
        };
        changesApi.prototype.watchFsDestinations = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allFsDestinationsHandlers().length === 0) {
                this.send("watch-config");
            }
            this.allFsDestinationsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allFsDestinationsHandlers.remove(callback);
                if (_this.allFsDestinationsHandlers().length === 0) {
                    _this.send("unwatch-config");
                }
            });
        };
        changesApi.prototype.watchAllCounters = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allDocsHandlers().length === 0) {
                this.send("watch-counters");
            }
            this.allCountersHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allCountersHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send("unwatch-counters");
                }
            });
        };
        changesApi.prototype.watchCounterChange = function (groupName, counterName, onChange) {
            var _this = this;
            var counterId = groupName + "/" + counterName;
            var callback = new changesCallback(onChange);
            if (typeof (this.watchedCounter[counterId]) === "undefined") {
                this.send("watch-counter-change", counterId);
                this.watchedCounter[counterId] = ko.observableArray();
            }
            this.watchedCounter[counterId].push(callback);
            return new changeSubscription(function () {
                _this.watchedCounter[counterId].remove(callback);
                if (_this.watchedCounter[counterId]().length === 0) {
                    delete _this.watchedCounter[counterId];
                    _this.send("unwatch-counter-change", counterId);
                }
            });
        };
        changesApi.prototype.watchCountersInGroup = function (group, onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (typeof (this.watchedCountersInGroup[group]) === "undefined") {
                this.send("watch-counters-in-group", group);
                this.watchedCountersInGroup[group] = ko.observableArray();
            }
            this.watchedCountersInGroup[group].push(callback);
            return new changeSubscription(function () {
                _this.watchedCountersInGroup[group].remove(callback);
                if (_this.watchedCountersInGroup[group]().length === 0) {
                    delete _this.watchedCountersInGroup[group];
                    _this.send("unwatch-counters-in-group", group);
                }
            });
        };
        changesApi.prototype.watchCounterBulkOperation = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allCounterBulkOperationsHandlers().length === 0) {
                this.send("watch-bulk-operation");
            }
            this.allCounterBulkOperationsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allCounterBulkOperationsHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send("unwatch-bulk-operation");
                }
            });
        };
        changesApi.prototype.watchTimeSeriesChange = function (type, key, onChange) {
            var _this = this;
            var fullId = type + "/" + key;
            var callback = new changesCallback(onChange);
            if (typeof (this.watchedTimeSeries[fullId]) === "undefined") {
                this.send("watch-time-series-key-change", fullId);
                this.watchedTimeSeries[fullId] = ko.observableArray();
            }
            this.watchedTimeSeries[fullId].push(callback);
            return new changeSubscription(function () {
                _this.watchedTimeSeries[fullId].remove(callback);
                if (_this.watchedTimeSeries[fullId]().length === 0) {
                    delete _this.watchedTimeSeries[fullId];
                    _this.send("unwatch-time-series-key-change", fullId);
                }
            });
        };
        changesApi.prototype.watchAllTimeSeries = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allDocsHandlers().length === 0) {
                this.send("watch-time-series");
            }
            this.allTimeSeriesHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allTimeSeriesHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send("unwatch-time-series");
                }
            });
        };
        changesApi.prototype.watchTimeSeriesBulkOperation = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allTimeSeriesBulkOperationsHandlers().length === 0) {
                this.send("watch-bulk-operation");
            }
            this.allTimeSeriesBulkOperationsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allTimeSeriesBulkOperationsHandlers.remove(callback);
                if (_this.allDocsHandlers().length === 0) {
                    _this.send("unwatch-bulk-operation");
                }
            });
        };
        changesApi.prototype.dispose = function () {
            var _this = this;
            this.isDisposing = true;
            this.disposed = true;
            this.connectToChangesApiTask.done(function () {
                var isCloseNeeded;
                if (_this.webSocket && _this.webSocket.readyState === _this.readyStateOpen) {
                    console.log("Disconnecting from WebSocket changes API for (" + _this.rs.fullTypeName + " = " + _this.rs.name + ")");
                    _this.webSocket.close(_this.normalClosureCode, _this.normalClosureMessage);
                    isCloseNeeded = true;
                }
                else if (_this.eventSource && _this.eventSource.readyState === _this.readyStateOpen) {
                    console.log("Disconnecting from EventSource changes API for (" + _this.rs.fullTypeName + " = " + _this.rs.name + ")");
                    _this.eventSource.close();
                    isCloseNeeded = true;
                }
                if (isCloseNeeded) {
                    _this.send("disconnect", undefined, false);
                    _this.isCleanClose = true;
                }
            });
        };
        changesApi.prototype.getResourceName = function () {
            return this.rs.name;
        };
        changesApi.isServerSupportingWebSockets = true;
        changesApi.messageWasShownOnce = false;
        return changesApi;
    })();
    return changesApi;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/changesApiWarnStorage", ["require", "exports"], function (require, exports) {
    var changesApiWarnStorage = (function () {
        function changesApiWarnStorage() {
        }
        changesApiWarnStorage.get = function () {
            return localStorage.getObject(changesApiWarnStorage.localStorageName);
        };
        changesApiWarnStorage.setValue = function (disabled) {
            if (disabled) {
                localStorage.setObject(changesApiWarnStorage.localStorageName, true);
            }
            else {
                localStorage.removeItem(changesApiWarnStorage.localStorageName);
            }
        };
        changesApiWarnStorage.showChangesApiWarning = function () {
            return !changesApiWarnStorage.get();
        };
        changesApiWarnStorage.localStorageName = "ChangesApiWarnDisabled";
        return changesApiWarnStorage;
    })();
    return changesApiWarnStorage;
});
/// <reference path="../../Scripts/typings/jquery/jquery.d.ts" />
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/changesCallback", ["require", "exports"], function (require, exports) {
    // we use this function wrapper as knockout calls functions stored directly in observableArray
    var changesCallback = (function () {
        function changesCallback(onFire) {
            this.onFire = onFire;
        }
        changesCallback.prototype.fire = function (arg) {
            this.onFire(arg);
        };
        return changesCallback;
    })();
    return changesCallback;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/changesContext", ["require", "exports"], function (require, exports) {
    var changesContext = (function () {
        function changesContext() {
        }
        changesContext.currentResourceChangesApi = ko.observable(null);
        return changesContext;
    })();
    return changesContext;
});
define("common/changeSubscription", ["require", "exports"], function (require, exports) {
    var changeSubscription = (function () {
        function changeSubscription(onOff) {
            this.onOff = onOff;
            this.executed = false;
        }
        changeSubscription.prototype.off = function () {
            if (!this.executed) {
                this.executed = true;
                this.onOff();
            }
        };
        return changeSubscription;
    })();
    return changeSubscription;
});
define("common/chunkFetcher", ["require", "exports"], function (require, exports) {
    var chunkFetcher = (function () {
        function chunkFetcher(fetcher) {
            this.fetcher = fetcher;
            this.task = $.Deferred();
            this.result = [];
            if (!fetcher) {
                throw new Error("Fetcher must be specified.");
            }
        }
        chunkFetcher.prototype.execute = function () {
            var skip = 0;
            var take = 1024;
            this.fetchChunk(skip, take);
            return this.task;
        };
        chunkFetcher.prototype.fetchChunk = function (skip, take) {
            var _this = this;
            this.fetcher(skip, take)
                .fail(function (x) { return _this.task.reject(x); })
                .done(function (data) {
                _this.result.pushAll(data);
                if (data.length == take) {
                    _this.fetchChunk(skip + take, take);
                }
                else {
                    _this.task.resolve(_this.result);
                }
            });
        };
        return chunkFetcher;
    })();
    return chunkFetcher;
});
define("common/cssGenerator", ["require", "exports"], function (require, exports) {
    var cssGenerator = (function () {
        function cssGenerator() {
        }
        cssGenerator.getCssClass = function (entityName, colorMaps, rs) {
            var resourceStyleMap = colorMaps.first(function (map) { return map.resourceName === rs.name; });
            if (!resourceStyleMap) {
                resourceStyleMap = {
                    resourceName: rs.name,
                    styleMap: {}
                };
                colorMaps.push(resourceStyleMap);
            }
            var existingStyle = resourceStyleMap.styleMap[entityName];
            if (existingStyle) {
                return existingStyle;
            }
            // We don't have an existing style. Assign one in the form of 'collection-style-X', where X is a number between 0 and maxStyleCount. These styles are found in app.less.
            var maxStyleCount = 32;
            var styleNumber = Object.keys(resourceStyleMap.styleMap).length % maxStyleCount;
            var style = "collection-style-" + styleNumber;
            resourceStyleMap.styleMap[entityName] = style;
            return style;
        };
        return cssGenerator;
    })();
    return cssGenerator;
});
define("common/dialogResult", ["require", "exports"], function (require, exports) {
    var dialogResult = (function () {
        function dialogResult(cancelled, task) {
            this.cancelled = cancelled;
            this.task = task;
        }
        return dialogResult;
    })();
    return dialogResult;
});
define("common/downloader", ["require", "exports", "commands/auth/getSingleAuthTokenCommand", "common/messagePublisher"], function (require, exports, getSingleAuthTokenCommand, messagePublisher) {
    var downloader = (function () {
        function downloader() {
            this.$downloadFrame = $('#downloadFrame');
        }
        downloader.prototype.download = function (resource, url) {
            var _this = this;
            new getSingleAuthTokenCommand(resource).execute().done(function (token) {
                var authToken = (url.indexOf("?") === -1 ? "?" : "&") + "singleUseAuthToken=" + token.Token;
                _this.$downloadFrame.attr("src", url + authToken);
            }).fail(function (qXHR, textStatus, errorThrown) { return messagePublisher.reportError("Could not get Single Auth Token for download.", errorThrown); });
        };
        downloader.prototype.reset = function () {
            this.$downloadFrame.attr("src", "");
        };
        return downloader;
    })();
    return downloader;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/eventSourceSettingStorage", ["require", "exports"], function (require, exports) {
    var eventSourceSettingStorage = (function () {
        function eventSourceSettingStorage() {
        }
        eventSourceSettingStorage.get = function () {
            return localStorage.getObject(eventSourceSettingStorage.localStorageName);
        };
        eventSourceSettingStorage.setValue = function (disabled) {
            if (disabled) {
                localStorage.setObject(eventSourceSettingStorage.localStorageName, true);
            }
            else {
                localStorage.removeItem(eventSourceSettingStorage.localStorageName);
            }
            var event = document.createEvent('StorageEvent');
            event.initStorageEvent('storage', false, false, eventSourceSettingStorage.localStorageName, disabled, !disabled, null, window.sessionStorage);
            window.dispatchEvent(event);
        };
        eventSourceSettingStorage.useEventSource = function () {
            var disabled = !!eventSourceSettingStorage.get();
            return !disabled;
        };
        eventSourceSettingStorage.localStorageName = "EventSourceDisabled";
        return eventSourceSettingStorage;
    })();
    return eventSourceSettingStorage;
});
define("common/execJs", ["require", "exports"], function (require, exports) {
    var execJs = (function () {
        function execJs() {
        }
        execJs.createSimpleCallableCode = function (script, context) {
            return new Function("with(this) { try { " + script + " } catch(err) { return 'Evaluation Error:' +  err; } }").bind(context);
        };
        execJs.validateSyntax = function (script) {
            try {
                eval("try { " + script + " } catch (e) {}");
            }
            catch (e) {
                if (e instanceof SyntaxError) {
                    return "Invalid binding syntax";
                }
            }
            return "";
        };
        return execJs;
    })();
    return execJs;
});
define("common/extensions", ["require", "exports", "moment"], function (require, exports, moment) {
    var extensions = (function () {
        function extensions() {
        }
        extensions.install = function () {
            extensions.installArrayExtensions();
            extensions.installDateExtensions();
            extensions.installFunctionExtensions();
            extensions.installObservableExtensions();
            extensions.installStringExtension();
            extensions.installStorageExtension();
            extensions.installBindingHandlers();
            extensions.installJqueryExtensions();
            // Want Intellisense for your extensions?
            // Go to extensionInterfaces.ts and add the function signature there.
        };
        extensions.installDateExtensions = function () {
            var datePrototype = Date.prototype;
            var formatNumber = function (num) {
                return num < 10 ? "0" + num : num;
            };
            datePrototype.getUTCDateFormatted = function () {
                var date = this.getUTCDate();
                return formatNumber(date);
            };
            datePrototype.getUTCMonthFormatted = function () {
                var month = this.getUTCMonth() + 1;
                return formatNumber(month);
            };
            datePrototype.getUTCHoursFormatted = function () {
                var hours = this.getUTCHours();
                return formatNumber(hours);
            };
            datePrototype.getUTCMinutesFormatted = function () {
                var minutes = this.getUTCMinutes();
                return formatNumber(minutes);
            };
            datePrototype.getUTCSecondsFormatted = function () {
                var seconds = this.getUTCSeconds();
                return formatNumber(seconds);
            };
        };
        extensions.installObservableExtensions = function () {
            var subscribableFn = ko.subscribable.fn;
            var observableArrayFn = ko.observableArray.fn;
            // observable.where
            subscribableFn.where = function (predicate) {
                var observable = this;
                var matches = ko.observable();
                observable.subscribe(function (val) {
                    if (predicate(val)) {
                        matches(val);
                    }
                });
                return matches;
            };
            // observable.distinctUntilChanged
            subscribableFn.distinctUntilChanged = function () {
                var observable = this;
                var matches = ko.observable();
                var lastMatch = observable();
                observable.subscribe(function (val) {
                    if (val !== lastMatch) {
                        lastMatch = val;
                        matches(val);
                    }
                });
                return matches;
            };
            // observable.throttled
            subscribableFn.throttle = function (throttleTimeMs) {
                var observable = this;
                return ko.computed(function () { return observable(); }).extend({ throttle: throttleTimeMs });
            };
            // observable.select
            subscribableFn.select = function (selector) {
                var observable = this;
                var selectedResults = ko.observable();
                observable.subscribe(function (val) { return selectedResults(selector(val)); });
                return selectedResults;
            };
            // observable.toggle
            subscribableFn.toggle = function () {
                var observable = this;
                observable(!observable());
                return observable;
            };
            // observableArray.pushAll
            observableArrayFn.pushAll = function (items) {
                this.push.apply(this, items);
            };
            // observableArray.contains
            observableArrayFn.contains = function (item) {
                return this.indexOf(item) !== -1;
            };
            // observableArray.first
            observableArrayFn.first = function (filter) {
                return this().first(filter);
            };
            // observableArray.last
            observableArrayFn.last = function (filter) {
                return this().last(filter);
            };
        };
        extensions.installArrayExtensions = function () {
            var arrayPrototype = Array.prototype;
            // Array.remove
            arrayPrototype.remove = function (item) {
                var self = this;
                var index = self.indexOf(item);
                if (index >= 0) {
                    self.splice(index, 1);
                }
                return index;
            };
            // Array.removeAll
            arrayPrototype.removeAll = function (items) {
                var i = 0;
                var self = this;
                for (var i = self.length - 1; i >= 0 && items.length > 0; i--) {
                    var itemsIndex = items.indexOf(self[i]);
                    if (itemsIndex >= 0) {
                        self.splice(i, 1);
                        items.splice(itemsIndex);
                    }
                }
            };
            // Array.first
            arrayPrototype.first = function (filter) {
                var self = this;
                if (self.length > 0) {
                    if (filter) {
                        return ko.utils.arrayFirst(self, filter);
                    }
                    else if (self.length > 0) {
                        return self[0];
                    }
                }
                return null;
            };
            // Array.last
            arrayPrototype.last = function (filter) {
                var self = this;
                if (filter) {
                    for (var i = self.length - 1; i > 0; i--) {
                        if (filter(self[i])) {
                            return self[i];
                        }
                    }
                }
                else if (self.length > 0) {
                    return self[self.length - 1];
                }
                return null;
            };
            // Array.pushAll
            arrayPrototype.pushAll = function (items) {
                this.push.apply(this, items);
            };
            // Array.contains
            arrayPrototype.contains = function (item) {
                var self = this;
                return self.indexOf(item) !== -1;
            };
            // Array.count
            arrayPrototype.count = function (filter) {
                var self = this;
                if (filter) {
                    var matches = 0;
                    for (var i = 0; i < self.length; i++) {
                        if (filter(self[i])) {
                            matches++;
                        }
                    }
                    return matches;
                }
                return self.length;
            };
            // Array.distinct
            arrayPrototype.distinct = function () {
                var distinctElements = [];
                for (var i = 0; i < this.length; i++) {
                    var element = this[i];
                    if (!distinctElements.contains(element)) {
                        distinctElements.push(element);
                    }
                }
                return distinctElements;
            };
            // Array.distinct
            arrayPrototype.concatUnique = function (values) {
                for (var i = 0; i < values.length; i++)
                    if (this.indexOf(values[i]) === -1)
                        this.push(values[i]);
            };
        };
        extensions.installStringExtension = function () {
            String.prototype.fixedCharCodeAt = function (idx) {
                idx = idx || 0;
                var code = this.charCodeAt(idx);
                var hi, low;
                if (0xD800 <= code && code <= 0xDBFF) {
                    hi = code;
                    low = this.charCodeAt(idx + 1);
                    if (isNaN(low)) {
                        throw 'No valid character or memory error!';
                    }
                    return ((hi - 0xD800) * 0x400) + (low - 0xDC00) + 0x10000;
                }
                if (0xDC00 <= code && code <= 0xDFFF) {
                    // We return false to allow loops to skip this iteration since should have already handled high surrogate above in the previous iteration
                    return false;
                }
                return code;
            };
            String.prototype.getSizeInBytesAsUTF8 = function () {
                var result = 0;
                var isQuoted = false;
                var prevChar = 0;
                for (var n = 0; n < this.length; n++) {
                    var charCode = this.fixedCharCodeAt(n);
                    if (charCode === 34 /*quates*/) {
                        if (!(isQuoted === true && prevChar === 92 /*backslash*/)) {
                            isQuoted = !isQuoted;
                        }
                    }
                    prevChar = charCode;
                    // whiteSpaceCharacters list from : https://en.wikipedia.org/wiki/Whitespace_character
                    var whiteSpaceCharacters = [9, 10, 11, 12, 13, 32, 133, 160, 5760, 8192, 8193, 8194, 8195, 8196, 8197, 8198, 8199, 8200, 8201, 8202, 8232, 8233, 8239, 8287, 12288, 6158, 8203, 8204, 8205, 8288, 65279];
                    if (isQuoted === false && $.inArray(charCode, whiteSpaceCharacters) > -1) {
                        continue;
                    }
                    if (typeof charCode === "number") {
                        if (charCode < 128) {
                            result = result + 1;
                        }
                        else if (charCode < 2048) {
                            result = result + 2;
                        }
                        else if (charCode < 65536) {
                            result = result + 3;
                        }
                        else if (charCode < 2097152) {
                            result = result + 4;
                        }
                        else if (charCode < 67108864) {
                            result = result + 5;
                        }
                        else {
                            result = result + 6;
                        }
                    }
                }
                return result;
            };
            String.prototype.hashCode = function () {
                var hash = 0;
                if (this.length == 0)
                    return hash;
                for (var i = 0; i < this.length; i++) {
                    var char = this.charCodeAt(i);
                    hash = ((hash << 5) - hash) + char;
                    hash |= 0; // Convert to 32bit integer
                }
                return hash;
            };
            String.prototype.replaceAll = function (find, replace) {
                return this.replace(new RegExp(find, 'g'), replace);
            };
            String.prototype.reverse = function (input) {
                var chars = new Array;
                if (input != null) {
                    chars = input.split('');
                }
                return chars.reverse().join('');
            };
            String.prototype.count = function (regex) {
                var re = new RegExp(regex, 'g');
                var results = this.match(re);
                return results ? results.length : 0;
            };
            String.prototype.startsWith = String.prototype.startsWith || function (str) {
                return this.indexOf(str) == 0;
            };
            String.prototype.contains = String.prototype.contains || function (str) {
                return this.indexOf(str) > -1;
            };
            String.prototype.multiply = function (amount) {
                var returnedString = new Array(amount + 1).join(this);
                return returnedString;
            };
            String.prototype.paddingLeft = function (paddingChar, paddingLength) {
                var paddingString = paddingChar.multiply(paddingLength);
                return String(paddingString + this).slice(-paddingString.length);
            };
            String.prototype.paddingRight = function (paddingChar, paddingLength) {
                var paddingString = paddingChar.multiply(paddingLength);
                return String(paddingString + this).slice(paddingString.length);
            };
            String.prototype.toHumanizedDate = function () {
                var dateMoment = moment(this.toString());
                if (dateMoment.isValid()) {
                    var now = moment();
                    var agoInMs = dateMoment.diff(now);
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                }
                return this;
            };
        };
        extensions.installStorageExtension = function () {
            Storage.prototype.getObject = function (key) {
                var value = this.getItem(key);
                return value && JSON.parse(value);
            };
            Storage.prototype.setObject = function (key, value) {
                this.setItem(key, ko.toJSON(value));
            };
        };
        extensions.installFunctionExtensions = function () {
            // Function.memoize
            var functionPrototype = Function.prototype;
            functionPrototype.memoize = function (thisVal) {
                var self = this;
                var cache = {};
                return function (arg1, arg2) {
                    if (arg2 in cache) {
                        return cache[arg2];
                    }
                    else {
                        return cache[arg2] = self.call(thisVal, arg1, arg2);
                    }
                };
            };
        };
        extensions.installBindingHandlers = function () {
            ko.bindingHandlers["numericValue"] = {
                init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var underlyingObservable = valueAccessor();
                    var interceptor = ko.computed({
                        read: underlyingObservable,
                        write: function (value) {
                            if (!isNaN(value)) {
                                underlyingObservable(parseFloat(value));
                            }
                        },
                        disposeWhenNodeIsRemoved: element
                    });
                    ko.bindingHandlers.value.init(element, function () { return interceptor; }, allBindingsAccessor, viewModel, bindingContext);
                },
                update: ko.bindingHandlers.value.update
            };
            ko.bindingHandlers["customValidity"] = {
                update: function (element, valueAccessor) {
                    var errorMessage = ko.unwrap(valueAccessor()); //unwrap to get subscription
                    element.setCustomValidity(errorMessage);
                }
            };
            ko.bindingHandlers["bsChecked"] = {
                init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var value = valueAccessor();
                    var newValueAccessor = function () {
                        return {
                            change: function () {
                                value(element.value);
                            }
                        };
                    };
                    if ($(element).val() === ko.unwrap(valueAccessor())) {
                        $(element).closest(".btn").button("toggle");
                    }
                    ko.bindingHandlers.event.init(element, newValueAccessor, allBindingsAccessor, viewModel, bindingContext);
                }
            };
            var key = "_my_init_key_";
            ko.bindingHandlers["updateHighlighting"] = {
                init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    ko.utils.domData.set(element, key, true);
                },
                update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                    var value = valueAccessor();
                    var isInit = ko.utils.domData.get(element, key);
                    var data = ko.dataFor(element);
                    var skip = !!data.isAllDocuments || !!data.isSystemDocuments || !!data.isAllGroupsGroup;
                    if (skip === false && isInit === false) {
                        $($(element).parents("li")).highlight();
                    }
                    else {
                        ko.utils.domData.set(element, key, false);
                    }
                }
            };
            ko.bindingHandlers["checkboxTriple"] = {
                update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                    var checkboxValue = ko.unwrap(valueAccessor());
                    switch (checkboxValue) {
                        case checkbox.Checked:
                            element.checked = true;
                            element.readOnly = false;
                            element.indeterminate = false;
                            break;
                        case checkbox.SomeChecked:
                            element.readOnly = true;
                            element.indeterminate = true;
                            element.checked = false;
                            break;
                        case checkbox.UnChecked:
                            element.checked = false;
                            element.readOnly = false;
                            element.indeterminate = false;
                            break;
                    }
                }
            };
        };
        extensions.installJqueryExtensions = function () {
            jQuery.fn.highlight = function () {
                $(this).each(function () {
                    var el = $(this);
                    el.before("<div/>");
                    el.prev()
                        .width(el.width())
                        .height(el.height())
                        .css({
                        "position": "absolute",
                        "background-color": "#ffff99",
                        "opacity": ".9"
                    })
                        .fadeOut(1500);
                });
            };
        };
        return extensions;
    })();
    return extensions;
});
define("common/fileDownloader", ["require", "exports"], function (require, exports) {
    var fileDownloader = (function () {
        function fileDownloader() {
        }
        fileDownloader.cleanup = function (domCacheElementName) {
            // clean previous elements (in any)
            $("#" + domCacheElementName).remove();
        };
        fileDownloader.createLinkAndStartDownload = function (blob, filename, domCacheElementName) {
            if (navigator && navigator.msSaveBlob) {
                navigator.msSaveBlob(blob, filename);
            }
            else {
                var blobUrl = URL.createObjectURL(blob);
                var a = document.createElement('a');
                a.id = "#" + domCacheElementName;
                a.download = filename;
                a.href = blobUrl;
                document.body.appendChild(a); // required by firefox
                a.click();
            }
        };
        fileDownloader.downloadAsJson = function (object, filename, domCacheElementName) {
            if (domCacheElementName === void 0) { domCacheElementName = "link"; }
            fileDownloader.cleanup(domCacheElementName);
            var modelAsString = JSON.stringify(object, null, 2);
            var blob = new Blob([modelAsString], { type: 'application/json' });
            fileDownloader.createLinkAndStartDownload(blob, filename, domCacheElementName);
        };
        fileDownloader.downloadAsZip = function (buffer, filename, domCacheElementName) {
            if (domCacheElementName === void 0) { domCacheElementName = "link"; }
            fileDownloader.cleanup(domCacheElementName);
            var blob = new Blob([buffer], { type: 'application/zip' });
            fileDownloader.createLinkAndStartDownload(blob, filename, domCacheElementName);
        };
        return fileDownloader;
    })();
    return fileDownloader;
});
define("common/generalUtils", ["require", "exports"], function (require, exports) {
    var genUtils = (function () {
        function genUtils() {
        }
        genUtils.formatAsCommaSeperatedString = function (input, digitsAfterDecimalPoint) {
            var parts = input.toString().split(".");
            parts[0] = parts[0].replace(/\B(?=(\d{3})+(?!\d))/g, ",");
            if (parts.length == 2 && parts[1].length > digitsAfterDecimalPoint) {
                parts[1] = parts[1].substring(0, digitsAfterDecimalPoint);
            }
            return parts.join(".");
        };
        genUtils.formatTimeSpan = function (input) {
            var timeParts = input.split(":");
            var miliPart;
            var sec = 0, milisec = 0;
            if (timeParts.length == 3) {
                miliPart = timeParts[2].split(".");
                sec = parseInt(miliPart[0]);
                var tmpMili;
                if (miliPart[1][0] == '0') {
                    tmpMili = miliPart[1].substring(1, 3);
                }
                else {
                    tmpMili = miliPart[1].substring(0, 3);
                }
                milisec = parseInt(tmpMili);
            }
            var hours = parseInt(timeParts[0]);
            var min = parseInt(timeParts[1]);
            var timeStr = "";
            if (hours > 0) {
                timeStr = hours + " Hours ";
            }
            if (min > 0) {
                timeStr += min + " Min ";
            }
            if (sec > 0) {
                timeStr += sec + " Sec ";
            }
            if ((timeStr == "") && (milisec > 0)) {
                timeStr = milisec + " Ms ";
            }
            return timeStr;
        };
        genUtils.formatBytesToSize = function (bytes) {
            var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
            if (bytes == 0)
                return 'n/a';
            var i = Math.floor(Math.log(bytes) / Math.log(1024));
            if (i < 0) {
                // number < 1
                return genUtils.formatAsCommaSeperatedString(bytes, 4) + ' Bytes';
            }
            var res = bytes / Math.pow(1024, i);
            var newRes = genUtils.formatAsCommaSeperatedString(res, 2);
            return newRes + ' ' + sizes[i];
        };
        // replace characters with their char codes, but leave A-Za-z0-9 and - in place. 
        genUtils.escape = function (input) {
            var output = "";
            for (var i = 0; i < input.length; i++) {
                var ch = input.charCodeAt(i);
                if (ch == 0x2F) {
                    output += '-';
                }
                else if (ch >= 0x30 && ch <= 0x39 || ch >= 0x41 && ch <= 0x5A || ch >= 0x61 && ch <= 0x7A || ch == 0x2D) {
                    output += input[i];
                }
                else {
                    output += ch;
                }
            }
            return output;
        };
        return genUtils;
    })();
    return genUtils;
});
define("common/idGenerator", ["require", "exports"], function (require, exports) {
    var idGenerator = (function () {
        function idGenerator() {
        }
        idGenerator.generateId = function (idLength) {
            if (idLength === void 0) { idLength = 5; }
            var text = "";
            var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
            for (var i = 0; i < idLength; i++)
                text += chars.charAt(Math.floor(Math.random() * chars.length));
            return text;
        };
        return idGenerator;
    })();
    return idGenerator;
});
define("common/inputCursor", ["require", "exports"], function (require, exports) {
    var inputCursor = (function () {
        function inputCursor() {
        }
        inputCursor.setPosition = function ($input, position) {
            var input = $input[0];
            if (!input)
                return null;
            if ('createTextRange' in input) {
                var textRange = input['createTextRange']();
                textRange.collapse(true);
                textRange.moveEnd(position);
                textRange.moveStart(position);
                textRange.select();
            }
            else if ('setSelectionRange' in input) {
                input['setSelectionRange'](position, position);
            }
        };
        inputCursor.getPosition = function ($input) {
            var input = $input[0];
            if (!input)
                return null;
            var cursorPosition = 0;
            if ('selectionStart' in input) {
                // Normal browsers
                cursorPosition = input["selectionStart"];
            }
            else {
                // IE
                input.focus();
                var sel = document["selection"].createRange();
                var selLen = document["selection"].createRange().text.length;
                sel.moveStart('character', -input["value"].length);
                cursorPosition = sel.text.length - selLen;
            }
            return cursorPosition;
        };
        return inputCursor;
    })();
    return inputCursor;
});
define("common/jsonUtil", ["require", "exports"], function (require, exports) {
    var jsonUtil = (function () {
        function jsonUtil() {
        }
        jsonUtil.syntaxHighlight = function (json) {
            if (typeof json != 'string') {
                json = JSON.stringify(json, undefined, 2);
            }
            json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
            return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
                var cls = 'number';
                if (/^"/.test(match)) {
                    if (/:$/.test(match)) {
                        cls = 'key';
                    }
                    else {
                        cls = 'string';
                    }
                }
                else if (/true|false/.test(match)) {
                    cls = 'boolean';
                }
                else if (/null/.test(match)) {
                    cls = 'null';
                }
                return '<span class="' + cls + '">' + match + '</span>';
            });
        };
        jsonUtil.newLineNormalizingHashFunction = function (object) {
            return ko.toJSON(object).replace(/\\r\\n/g, '\\n');
        };
        jsonUtil.newLineNormalizingHashFunctionWithIgnoredFields = function (ignoredFields) {
            return function (object) {
                return ko.toJSON(object, function (k, v) { return ignoredFields.indexOf(k) == -1 ? v : null; }).replace(/\\r\\n/g, '\\n');
            };
        };
        return jsonUtil;
    })();
    return jsonUtil;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/mergedIndexesStorage", ["require", "exports", "models/database/index/indexMergeSuggestion"], function (require, exports, indexMergeSuggestion) {
    var mergedIndexesStorage = (function () {
        function mergedIndexesStorage() {
        }
        mergedIndexesStorage.getMergedIndex = function (db, mergedIndexName) {
            var newSuggestion = null;
            try {
                if (!!mergedIndexName && mergedIndexName.indexOf(db.mergedIndexLocalStoragePrefix) == 0) {
                    var suggestion = localStorage.getObject(mergedIndexName);
                    localStorage.removeItem(mergedIndexName);
                    newSuggestion = new indexMergeSuggestion(suggestion);
                }
            }
            catch (e) {
                return null;
            }
            return newSuggestion;
        };
        mergedIndexesStorage.saveMergedIndex = function (db, id, suggestion) {
            var localStorageName = mergedIndexesStorage.getMergedIndexName(db, id);
            localStorage.setObject(localStorageName, suggestion.toDto());
        };
        mergedIndexesStorage.getMergedIndexName = function (db, id) {
            return db.mergedIndexLocalStoragePrefix + '.' + id;
        };
        return mergedIndexesStorage;
    })();
    return mergedIndexesStorage;
});
define("common/messagePublisher", ["require", "exports", "common/alertArgs"], function (require, exports, alertArgs) {
    var messagePublisher = (function () {
        function messagePublisher() {
        }
        messagePublisher.reportInfo = function (title, details) {
            this.reportProgress(0 /* info */, title, details);
        };
        messagePublisher.reportError = function (title, details, httpStatusText, displayInRecentErrors) {
            if (displayInRecentErrors === void 0) { displayInRecentErrors = true; }
            this.reportProgress(3 /* danger */, title, details, httpStatusText, displayInRecentErrors);
            if (console && console.log && typeof console.log === "function") {
                console.log("Error during command execution", title, details, httpStatusText);
            }
        };
        messagePublisher.reportSuccess = function (title, details) {
            this.reportProgress(1 /* success */, title, details);
        };
        messagePublisher.reportWarning = function (title, details, httpStatusText) {
            this.reportProgress(2 /* warning */, title, details, httpStatusText);
        };
        messagePublisher.reportWarningWithButton = function (title, details, buttonName, action) {
            var alert = new alertArgs(2 /* warning */, title, details, null, true);
            alert.setupButtton(buttonName, action);
            ko.postbox.publish("Alert", alert);
        };
        messagePublisher.reportProgress = function (type, title, details, httpStatusText, displayInRecentErrors) {
            if (displayInRecentErrors === void 0) { displayInRecentErrors = true; }
            ko.postbox.publish("Alert", new alertArgs(type, title, details, httpStatusText, displayInRecentErrors));
        };
        return messagePublisher;
    })();
    return messagePublisher;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/numberFormattingStorage", ["require", "exports"], function (require, exports) {
    var numberFormattingStorage = (function () {
        function numberFormattingStorage() {
        }
        numberFormattingStorage.shouldUseRaw = function () {
            return localStorage.getObject(numberFormattingStorage.localStorageName);
        };
        numberFormattingStorage.save = function (useRawFormat) {
            if (useRawFormat) {
                localStorage.setObject(numberFormattingStorage.localStorageName, true);
            }
            else {
                localStorage.removeItem(numberFormattingStorage.localStorageName);
            }
            var event = document.createEvent('StorageEvent');
            event.initStorageEvent('storage', false, false, numberFormattingStorage.localStorageName, useRawFormat, !useRawFormat, null, window.sessionStorage);
            window.dispatchEvent(event);
        };
        numberFormattingStorage.localStorageName = "NumberFormatting";
        return numberFormattingStorage;
    })();
    return numberFormattingStorage;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/oauthContext", ["require", "exports"], function (require, exports) {
    var oauthContext = (function () {
        function oauthContext() {
        }
        oauthContext.clean = function () {
            oauthContext.apiKey(null);
            oauthContext.authHeader(null);
        };
        oauthContext.apiKey = ko.observable(null);
        oauthContext.authHeader = ko.observable(null);
        oauthContext.apiKeyName = ko.computed(function () {
            var key = oauthContext.apiKey();
            if (key === null) {
                return null;
            }
            var slashPos = key.indexOf('/');
            if (slashPos >= 0) {
                return key.substring(0, slashPos);
            }
            return null;
        });
        oauthContext.apiKeySecret = ko.computed(function () {
            var key = oauthContext.apiKey();
            if (key === null) {
                return null;
            }
            var slashPos = key.indexOf('/');
            if (slashPos >= 0) {
                return key.substring(slashPos + 1);
            }
            return null;
        });
        return oauthContext;
    })();
    return oauthContext;
});
define("common/optional", ["require", "exports"], function (require, exports) {
    var optional = (function () {
        function optional(instance) {
            this.instance = instance;
        }
        optional.val = function (value) {
            return new optional(value);
        };
        optional.prototype.bind = function (fetcher) {
            if (this.instance !== null && this.instance !== undefined) {
                var val = fetcher(this.instance);
                return new optional(val);
            }
            return new optional(null);
        };
        optional.prototype.toString = function () {
            if (this.instance !== null && this.instance !== undefined) {
                return this.instance.toString();
            }
            return "";
        };
        return optional;
    })();
    return optional;
});
/// <reference path="../../Scripts/typings/jquery/jquery.d.ts" />
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/pagedList", ["require", "exports", "common/pagedResultSet"], function (require, exports, pagedResultSet) {
    var pagedList = (function () {
        function pagedList(fetcher) {
            this.fetcher = fetcher;
            this.totalResultCount = ko.observable(0);
            this.items = [];
            this.isFetching = false;
            this.queuedFetch = null;
            this.collectionName = "";
            this.currentItemIndex = ko.observable(0);
            if (!fetcher) {
                throw new Error("Fetcher must be specified.");
            }
        }
        pagedList.prototype.clear = function () {
            if (!!this.queuedFetch) {
                this.queuedFetch.task.reject("data is being reloaded");
                this.queuedFetch = null;
            }
            while (this.items.length > 0) {
                this.items.pop();
            }
        };
        pagedList.prototype.itemCount = function () {
            return this.items ? this.items.length : 0;
        };
        pagedList.prototype.fetch = function (skip, take) {
            var _this = this;
            if (this.isFetching) {
                this.queuedFetch = { skip: skip, take: take, task: $.Deferred() };
                return this.queuedFetch.task;
            }
            var cachedItemsSlice = this.getCachedSliceOrNull(skip, take);
            if (cachedItemsSlice) {
                // We've already fetched these items. Just return them immediately.
                var deferred = $.Deferred();
                var results = new pagedResultSet(cachedItemsSlice, this.totalResultCount());
                deferred.resolve(results);
                return deferred;
            }
            else {
                // We haven't fetched some of the items. Fetch them now from remote.
                this.isFetching = true;
                var remoteFetch = this.fetcher(skip, take)
                    .done(function (resultSet) {
                    _this.totalResultCount(resultSet.totalResultCount);
                    resultSet.items.forEach(function (r, i) { return _this.items[i + skip] = r; });
                })
                    .always(function () {
                    _this.isFetching = false;
                    _this.runQueuedFetch();
                });
                return remoteFetch;
            }
        };
        pagedList.prototype.getCachedSliceOrNull = function (skip, take) {
            for (var i = skip; i < skip + take; i++) {
                if (!this.items[i]) {
                    return null;
                }
            }
            return this.items.slice(skip, skip + take);
        };
        pagedList.prototype.getNthItem = function (nth) {
            var deferred = $.Deferred();
            var cachedItemArray = this.getCachedSliceOrNull(nth, 1);
            if (cachedItemArray) {
                deferred.resolve(cachedItemArray[0]);
            }
            else {
                this.fetch(nth, 1)
                    .done(function (result) {
                    deferred.resolve(result.items[0]);
                })
                    .fail(function (error) { return deferred.reject(error); });
            }
            return deferred;
        };
        pagedList.prototype.getCachedItemsAt = function (indices) {
            var _this = this;
            return indices
                .filter(function (index) { return _this.items[index]; })
                .map(function (validIndex) { return _this.items[validIndex]; });
        };
        pagedList.prototype.getCachedIndices = function (indices) {
            var _this = this;
            return indices.filter(function (index) { return _this.items[index]; });
        };
        pagedList.prototype.getAllCachedItems = function () {
            return this.items;
        };
        pagedList.prototype.runQueuedFetch = function () {
            if (this.queuedFetch) {
                var queuedSkip = this.queuedFetch.skip;
                var queuedTake = this.queuedFetch.take;
                var queuedTask = this.queuedFetch.task;
                this.queuedFetch = null;
                var fetchTask = this.fetch(queuedSkip, queuedTake);
                fetchTask.done(function (results) { return queuedTask.resolve(results); });
                fetchTask.fail(function (error) { return queuedTask.reject(error); });
            }
        };
        pagedList.prototype.invalidateCache = function () {
            this.items.length = 0;
        };
        pagedList.prototype.indexOf = function (item) {
            return this.items.indexOf(item);
        };
        pagedList.prototype.hasIds = function () {
            return this.items && this.items.length > 0 && this.items[0] && this.items[0].getId && this.items[0].getId();
        };
        pagedList.prototype.getItem = function (i) {
            if (i > this.items.length - 1)
                return null;
            return this.items[0];
        };
        return pagedList;
    })();
    return pagedList;
});
define("common/pagedResultSet", ["require", "exports"], function (require, exports) {
    var pagedResultSet = (function () {
        function pagedResultSet(items, totalResultCount, additionalResultInfo) {
            this.items = items;
            this.totalResultCount = totalResultCount;
            this.additionalResultInfo = additionalResultInfo;
        }
        return pagedResultSet;
    })();
    return pagedResultSet;
});
define("common/pair", ["require", "exports"], function (require, exports) {
    var Pair = (function () {
        function Pair(item1, item2) {
            this.item1 = item1;
            this.item2 = item2;
        }
        return Pair;
    })();
    return Pair;
});
define("common/queryUtil", ["require", "exports", "commands/database/index/getIndexTermsCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand"], function (require, exports, getIndexTermsCommand, getDocumentsMetadataByIDPrefixCommand) {
    var queryUtil = (function () {
        function queryUtil() {
        }
        /**
         * Escapes lucene single term
         *
         * Note: Do not use this method for escaping entire query unless you want to end up with: query\:value\ AND\ a\:b
         * @param query query to escape
         */
        queryUtil.escapeTerm = function (term) {
            var output = "";
            for (var i = 0; i < term.length; i++) {
                var c = term.charAt(i);
                if (c === '\\' || c === '+' || c === '-' || c === '!' || c === '(' || c === ')'
                    || c === ':' || c === '^' || c === '[' || c === ']' || c === '\"'
                    || c === '{' || c === '}' || c === '~' || c === '*' || c === '?'
                    || c === '|' || c === '&' || c === ' ') {
                    output += "\\";
                }
                output += c;
            }
            return output;
        };
        queryUtil.queryCompleter = function (indexFields, selectedIndex, dynamicPrefix, activeDatabase, editor, session, pos, prefix, callback) {
            var currentToken = session.getTokenAt(pos.row, pos.column);
            if (!currentToken || typeof currentToken.type === "string") {
                // if in beginning of text or in free text token
                if (!currentToken || currentToken.type === "text") {
                    callback(null, indexFields().map(function (curColumn) {
                        return { name: curColumn, value: curColumn, score: 10, meta: "field" };
                    }));
                }
                else if (currentToken.type === "keyword" || currentToken.type === "value") {
                    // if right after, or a whitespace after keyword token ([column name]:)
                    // first, calculate and validate the column name
                    var currentColumnName = null;
                    var currentValue = "";
                    if (currentToken.type == "keyword") {
                        currentColumnName = currentToken.value.substring(0, currentToken.value.length - 1);
                    }
                    else {
                        currentValue = currentToken.value.trim();
                        var rowTokens = session.getTokens(pos.row);
                        if (!!rowTokens && rowTokens.length > 1) {
                            currentColumnName = rowTokens[rowTokens.length - 2].value.trim();
                            currentColumnName = currentColumnName.substring(0, currentColumnName.length - 1);
                        }
                    }
                    // for non dynamic indexes query index terms, for dynamic indexes, try perform general auto complete
                    if (!!currentColumnName && !!indexFields.first(function (x) { return x === currentColumnName; })) {
                        if (selectedIndex().indexOf(dynamicPrefix) !== 0) {
                            new getIndexTermsCommand(selectedIndex(), currentColumnName, activeDatabase())
                                .execute()
                                .done(function (terms) {
                                if (!!terms && terms.length > 0) {
                                    callback(null, terms.map(function (curVal) {
                                        return { name: curVal, value: curVal, score: 10, meta: "value" };
                                    }));
                                }
                            });
                        }
                        else {
                            if (currentValue.length > 0) {
                                new getDocumentsMetadataByIDPrefixCommand(currentValue, 10, activeDatabase())
                                    .execute()
                                    .done(function (results) {
                                    if (!!results && results.length > 0) {
                                        callback(null, results.map(function (curVal) {
                                            return { name: curVal["@metadata"]["@id"], value: curVal["@metadata"]["@id"], score: 10, meta: "value" };
                                        }));
                                    }
                                });
                            }
                            else {
                                callback([{ error: "notext" }], null);
                            }
                        }
                    }
                }
            }
        };
        return queryUtil;
    })();
    return queryUtil;
});
define("common/recentPatchesStorage", ["require", "exports"], function (require, exports) {
    var recentPatchesStorage = (function () {
        function recentPatchesStorage() {
        }
        recentPatchesStorage.getRecentPatches = function (db) {
            var localStorageName = db.recentPatchesLocalStorageName;
            var recentPatchesFromLocalStorage = this.getRecentPatchesFromLocalStorage(localStorageName);
            if (recentPatchesFromLocalStorage == null || recentPatchesFromLocalStorage instanceof Array === false) {
                localStorage.setObject(localStorageName, []);
                recentPatchesFromLocalStorage = [];
            }
            return recentPatchesFromLocalStorage;
        };
        recentPatchesStorage.saveRecentPatches = function (db, recentPatches) {
            var localStorageName = db.recentPatchesLocalStorageName;
            localStorage.setObject(localStorageName, recentPatches);
        };
        recentPatchesStorage.removeIndexFromRecentPatches = function (db, indexName) {
            var localStorageName = db.recentPatchesLocalStorageName;
            var recentPatchesFromLocalStorage = this.getRecentPatchesFromLocalStorage(localStorageName);
            if (recentPatchesFromLocalStorage == null)
                return;
            var newRecentPatches = recentPatchesFromLocalStorage.filter(function (query) { return query.PatchOnOption !== "Index" || query.SelectedItem !== indexName; });
            localStorage.setObject(localStorageName, newRecentPatches);
        };
        recentPatchesStorage.removeRecentPatches = function (db) {
            var localStorageName = db.recentPatchesLocalStorageName;
            localStorage.setObject(localStorageName, []);
        };
        recentPatchesStorage.getRecentPatchesFromLocalStorage = function (localStorageName) {
            var recentPatchesFromLocalStorage = null;
            try {
                recentPatchesFromLocalStorage = localStorage.getObject(localStorageName);
            }
            catch (err) {
            }
            return recentPatchesFromLocalStorage;
        };
        return recentPatchesStorage;
    })();
    return recentPatchesStorage;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/recentQueriesStorage", ["require", "exports"], function (require, exports) {
    var recentQueriesStorage = (function () {
        function recentQueriesStorage() {
        }
        recentQueriesStorage.getRecentQueries = function (db) {
            var localStorageName = db.recentQueriesLocalStorageName;
            var recentQueriesFromLocalStorage = this.getRecentQueriesFromLocalStorage(localStorageName);
            if (recentQueriesFromLocalStorage == null || recentQueriesFromLocalStorage instanceof Array === false) {
                localStorage.setObject(localStorageName, []);
                recentQueriesFromLocalStorage = [];
            }
            // we have restore properties coming from local storage, 
            // when saving object.prop = undefined in localstorage 
            // we don't get 'prop' property during read.
            recentQueriesFromLocalStorage.forEach(function (entry) {
                if (entry.TransformerQuery) {
                    if (("transformerName" in entry.TransformerQuery) === false) {
                        entry.TransformerQuery.transformerName = undefined;
                    }
                    ;
                }
            });
            return recentQueriesFromLocalStorage;
        };
        recentQueriesStorage.saveRecentQueries = function (db, recentQueries) {
            var localStorageName = db.recentQueriesLocalStorageName;
            localStorage.setObject(localStorageName, recentQueries);
        };
        recentQueriesStorage.removeIndexFromRecentQueries = function (db, indexName) {
            var localStorageName = db.recentQueriesLocalStorageName;
            var recentQueriesFromLocalStorage = this.getRecentQueriesFromLocalStorage(localStorageName);
            if (recentQueriesFromLocalStorage == null)
                return;
            var newRecentQueries = recentQueriesFromLocalStorage.filter(function (query) { return query.IndexName != indexName; });
            localStorage.setObject(localStorageName, newRecentQueries);
        };
        recentQueriesStorage.removeRecentQueries = function (db) {
            var localStorageName = db.recentQueriesLocalStorageName;
            localStorage.setObject(localStorageName, []);
        };
        recentQueriesStorage.getRecentQueriesFromLocalStorage = function (localStorageName) {
            var recentQueriesFromLocalStorage = null;
            try {
                recentQueriesFromLocalStorage = localStorage.getObject(localStorageName);
            }
            catch (err) {
            }
            return recentQueriesFromLocalStorage;
        };
        return recentQueriesStorage;
    })();
    return recentQueriesStorage;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/serverBuildReminder", ["require", "exports"], function (require, exports) {
    var serverBuildReminder = (function () {
        function serverBuildReminder() {
        }
        serverBuildReminder.get = function () {
            return localStorage.getObject(serverBuildReminder.localStorageName);
        };
        serverBuildReminder.mute = function (isMuteNeeded) {
            if (isMuteNeeded) {
                localStorage.setObject(serverBuildReminder.localStorageName, new Date());
            }
            else {
                localStorage.removeItem(serverBuildReminder.localStorageName);
            }
            var event = document.createEvent('StorageEvent');
            event.initStorageEvent('storage', false, false, serverBuildReminder.localStorageName, isMuteNeeded, !isMuteNeeded, null, window.sessionStorage);
            window.dispatchEvent(event);
        };
        serverBuildReminder.isReminderNeeded = function () {
            var lastBuildCheck = serverBuildReminder.get();
            var timestamp = Date.parse(lastBuildCheck);
            var difference = 0;
            if (!isNaN(timestamp)) {
                var lastBuildCheckMoment = moment(lastBuildCheck);
                var currentDateMoment = moment(new Date());
                difference = currentDateMoment.diff(lastBuildCheckMoment, 'days', true);
            }
            if (isNaN(timestamp) || difference > 7) {
                serverBuildReminder.mute(false);
                return true;
            }
            return false;
        };
        serverBuildReminder.localStorageName = "LastServerBuildReminder";
        return serverBuildReminder;
    })();
    return serverBuildReminder;
});
/// <reference path="../../Scripts/typings/knockout/knockout.d.ts" />
define("common/serverSmugglingLocalStorage", ["require", "exports"], function (require, exports) {
    var serverSmugglingLocalStorage = (function () {
        function serverSmugglingLocalStorage() {
        }
        serverSmugglingLocalStorage.get = function () {
            return localStorage.getObject(serverSmugglingLocalStorage.localStorageName);
        };
        serverSmugglingLocalStorage.setValue = function (value) {
            localStorage.setObject(serverSmugglingLocalStorage.localStorageName, value);
        };
        serverSmugglingLocalStorage.clean = function () {
            localStorage.removeItem(serverSmugglingLocalStorage.localStorageName);
        };
        serverSmugglingLocalStorage.localStorageName = "serverSmuggling";
        return serverSmugglingLocalStorage;
    })();
    return serverSmugglingLocalStorage;
});
define("common/settingsAccessAuthorizer", ["require", "exports", "viewmodels/shell"], function (require, exports, shell) {
    var settingsAccessAuthorizer = (function () {
        function settingsAccessAuthorizer() {
            this.isForbidden = ko.computed(function () {
                var globalAdmin = shell.isGlobalAdmin();
                var canReadWriteSettings = shell.canReadWriteSettings();
                var canReadSettings = shell.canReadSettings();
                return !globalAdmin && !canReadWriteSettings && !canReadSettings;
            });
            this.isReadOnly = ko.computed(function () {
                var globalAdmin = shell.isGlobalAdmin();
                var canReadWriteSettings = shell.canReadWriteSettings();
                var canReadSettings = shell.canReadSettings();
                return !globalAdmin && !canReadWriteSettings && canReadSettings;
            });
            this.canReadOrWrite = ko.computed(function () {
                var globalAdmin = shell.isGlobalAdmin();
                var canReadWriteSettings = shell.canReadWriteSettings();
                var canReadSettings = shell.canReadSettings();
                return globalAdmin || canReadWriteSettings || canReadSettings;
            });
            this.canWrite = ko.computed(function () {
                var globalAdmin = shell.isGlobalAdmin();
                var canReadWriteSettings = shell.canReadWriteSettings();
                return globalAdmin || canReadWriteSettings;
            });
        }
        return settingsAccessAuthorizer;
    })();
    return settingsAccessAuthorizer;
});
define("common/sqlReplicationStats", ["require", "exports", "common/generalUtils"], function (require, exports, genUtils) {
    var sqlReplicationStats = (function () {
        function sqlReplicationStats(replicationStats) {
            var _this = this;
            this.statistics = ko.observable();
            this.metrics = ko.observable();
            this.name = ko.observable("");
            if (!!replicationStats.Statistics.LastErrorTime && replicationStats.Statistics.LastErrorTime == "0001-01-01T00:00:00.0000000") {
                replicationStats.Statistics.LastErrorTime = "No Errors";
            }
            this.statistics(replicationStats.Statistics);
            this.name(replicationStats.Name);
            this.metrics(replicationStats.Metrics);
            this.rateMetrics = ko.computed(function () {
                var computedRateMetrics = [];
                var generalMetrics = _this.metrics().GeneralMetrics;
                var tablesMetrics = _this.metrics().TablesMetrics;
                if (!!generalMetrics) {
                    $.map(generalMetrics, function (value, key) {
                        if (value.Type == "Meter") {
                            value["Name"] = key;
                            value["Table"] = "";
                            $.map(value, function (propertyValue, propertyName) {
                                if (!isNaN(propertyValue)) {
                                    value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                }
                            });
                            computedRateMetrics.push(value);
                        }
                    });
                }
                if (!!tablesMetrics) {
                    $.map(tablesMetrics, function (tablesMetricsData, tableMetricsKey) {
                        $.map(tablesMetricsData, function (value, key) {
                            if (value.Type == "Meter") {
                                var newMetric = value;
                                newMetric["Name"] = key;
                                value["Table"] = tableMetricsKey;
                                $.map(value, function (propertyValue, propertyName) {
                                    if (!isNaN(propertyValue)) {
                                        value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                    }
                                });
                                computedRateMetrics.push(newMetric);
                            }
                        });
                    });
                }
                return computedRateMetrics;
            });
            this.histogramMetrics = ko.computed(function () {
                var computedHistogramMetrics = [];
                var generalMetrics = _this.metrics().GeneralMetrics;
                var tablesMetrics = _this.metrics().TablesMetrics;
                if (!!generalMetrics) {
                    $.map(generalMetrics, function (value, key) {
                        if (value.Type == "Histogram") {
                            value["Name"] = key;
                            value["Table"] = "";
                            value["Percentiles"] = $.map(value["Percentiles"], function (percentileValue, percentile) {
                                return {
                                    percentileValue: genUtils.formatAsCommaSeperatedString(percentileValue, 2),
                                    percentile: percentile
                                };
                            });
                            $.map(value, function (propertyValue, propertyName) {
                                if (!isNaN(propertyValue)) {
                                    value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                }
                            });
                            computedHistogramMetrics.push(value);
                        }
                    });
                }
                if (!!tablesMetrics) {
                    $.map(tablesMetrics, function (tablesMetricsData, tableMetricsKey) {
                        $.map(tablesMetricsData, function (value, key) {
                            if (value.Type == "Histogram") {
                                value["Name"] = key;
                                value["Table"] = tableMetricsKey;
                                value["Percentiles"] = $.map(value["Percentiles"], function (percentileValue, percentile) {
                                    return {
                                        percentileValue: genUtils.formatAsCommaSeperatedString(percentileValue, 2),
                                        percentile: percentile
                                    };
                                });
                                $.map(value, function (propertyValue, propertyName) {
                                    if (!isNaN(propertyValue)) {
                                        value[propertyName] = genUtils.formatAsCommaSeperatedString(propertyValue, 2);
                                    }
                                });
                                computedHistogramMetrics.push(value);
                            }
                        });
                    });
                }
                return computedHistogramMetrics;
            });
        }
        sqlReplicationStats.ALL_TABLES = 'All Tables';
        return sqlReplicationStats;
    })();
    return sqlReplicationStats;
});
define("common/svgDownloader", ["require", "exports", 'd3/d3'], function (require, exports, d3) {
    var svgDownloader = (function () {
        function svgDownloader() {
        }
        svgDownloader.convertToData = function (svgElement, cssInliner) {
            var svgClone = svgElement.cloneNode(true);
            svgDownloader.fixAttributes(svgClone);
            var svgContainer = document.createElement('div');
            svgContainer.appendChild(svgClone);
            var targetStyles = cssInliner(svgClone);
            var s = document.createElement('style');
            s.setAttribute('type', 'text/css');
            s.innerHTML = "<![CDATA[\n" + targetStyles + "\n]]>";
            var defs = d3.select(svgClone).select('defs').node();
            if (!defs) {
                defs = document.createElement('defs');
                svgClone.insertBefore(defs, svgClone.firstChild);
            }
            defs.appendChild(s);
            return svgDownloader.svgHeader + "\n" + svgContainer.innerHTML;
        };
        svgDownloader.cleanup = function () {
            // clean previous image (in any)
            $("#downloadSvg").remove();
        };
        svgDownloader.b64toBlob = function (b64Data, contentType, sliceSize) {
            contentType = contentType || '';
            sliceSize = sliceSize || 512;
            var byteCharacters = atob(b64Data);
            var byteArrays = [];
            for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
                var slice = byteCharacters.slice(offset, offset + sliceSize);
                var byteNumbers = new Array(slice.length);
                for (var i = 0; i < slice.length; i++) {
                    byteNumbers[i] = slice.charCodeAt(i);
                }
                var byteArray = new Uint8Array(byteNumbers);
                byteArrays.push(byteArray);
            }
            var blob = new Blob(byteArrays, { type: contentType });
            return blob;
        };
        svgDownloader.createLinkAndStartDownload = function (blob, filename) {
            if (navigator && navigator.msSaveBlob) {
                navigator.msSaveBlob(blob, filename);
            }
            else {
                var blobUrl = URL.createObjectURL(blob);
                var a = document.createElement('a');
                a.id = "downloadSvg";
                a.download = filename;
                a.href = blobUrl;
                document.body.appendChild(a); // required by firefox
                a.click();
            }
        };
        svgDownloader.downloadSvg = function (svgElement, filename, cssInliner) {
            svgDownloader.cleanup();
            var textSvgData = svgDownloader.convertToData(svgElement, cssInliner);
            var encodedImage = window.btoa(textSvgData);
            var blob = svgDownloader.b64toBlob(encodedImage, 'image/svg+xml');
            svgDownloader.createLinkAndStartDownload(blob, filename);
        };
        svgDownloader.downloadPng = function (svgElement, filename, cssInliner) {
            svgDownloader.cleanup();
            var textSvgData = svgDownloader.convertToData(svgElement, cssInliner);
            var image = new Image();
            image.src = 'data:image/svg+xml;base64,' + window.btoa(textSvgData);
            image.onerror = function () {
                alert("Unable to download image!");
            };
            image.onload = function () {
                var canvas = document.createElement('canvas');
                canvas.width = image.width;
                canvas.height = image.height;
                var context = canvas.getContext('2d');
                context.drawImage(image, 0, 0);
                var dataUrlStripped = canvas.toDataURL('image/png').replace(/^data:image\/png;base64,/, "");
                var blob = svgDownloader.b64toBlob(dataUrlStripped, 'image/png');
                svgDownloader.createLinkAndStartDownload(blob, filename);
            };
        };
        // helper method to extract css from element
        svgDownloader.extractInlineCss = function (element) {
            var targetStyles = "";
            var sheets = document.styleSheets;
            for (var i = 0; i < sheets.length; i++) {
                var rules = sheets[i].cssRules;
                for (var j = 0; j < rules.length; j++) {
                    var rule = rules[j];
                    if (typeof (rule.style) != "undefined") {
                        var elems = element.querySelectorAll(rule.selectorText);
                        if (elems.length > 0) {
                            targetStyles += rule.selectorText + " { " + rule.style.cssText + " }\n";
                        }
                    }
                }
            }
            return targetStyles;
        };
        svgDownloader.fixAttributes = function (el) {
            var viewBox = el.getAttribute('viewBox');
            svgDownloader.removeAttributes(el);
            el.setAttribute("version", "1.1");
            el.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns', 'http://www.w3.org/2000/svg');
            el.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
            if (viewBox) {
                var splitedViewBox = viewBox.split(/\s+|,/);
                el.setAttribute("width", splitedViewBox[2]);
                el.setAttribute("height", splitedViewBox[3]);
            }
        };
        svgDownloader.removeAttributes = function (el) {
            var attributes = $.map(el.attributes, function (item) {
                return item.name;
            });
            var selection = $(el);
            $.each(attributes, function (i, item) {
                selection.removeAttr(item);
            });
        };
        svgDownloader.svgHeader = '<?xml version="1.0" standalone="no"?>\n' +
            '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">';
        return svgDownloader;
    })();
    return svgDownloader;
});
define("common/trafficWatchClient", ["require", "exports", 'common/changeSubscription', 'common/changesCallback', 'commands/commandBase', "common/changesApi", "common/idGenerator"], function (require, exports, changeSubscription, changesCallback, commandBase, changesApi, idGenerator) {
    var trafficWatchClient = (function () {
        function trafficWatchClient(resourcePath, token) {
            this.resourcePath = resourcePath;
            this.token = token;
            this.readyStateOpen = 1;
            this.isCleanClose = false;
            this.normalClosureCode = 1000;
            this.normalClosureMessage = "CLOSE_NORMAL";
            this.successfullyConnectedOnce = false;
            this.sentMessages = [];
            this.commandBase = new commandBase();
            this.adminLogsHandlers = ko.observableArray();
            this.connectionOpeningTask = $.Deferred();
            this.connectionClosingTask = $.Deferred();
            this.eventsId = idGenerator.generateId();
        }
        trafficWatchClient.prototype.connect = function () {
            var connectionString = 'singleUseAuthToken=' + this.token + '&id=' + this.eventsId;
            if ("WebSocket" in window && changesApi.isServerSupportingWebSockets) {
                this.connectWebSocket(connectionString);
            }
            else if ("EventSource" in window) {
                this.connectEventSource(connectionString);
            }
            else {
                //The browser doesn't support nor websocket nor eventsource
                //or we are in IE10 or IE11 and the server doesn't support WebSockets.
                //Anyway, at this point a warning message was already shown. 
                this.connectionOpeningTask.reject();
            }
        };
        trafficWatchClient.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === 'https:' ? 'wss://' : 'ws://';
            this.webSocket = new WebSocket(wsProtocol + window.location.host + this.resourcePath + '/traffic-watch/websocket?' + connectionString);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.connectionClosingTask.resolve({ Error: e });
                }
            };
            this.webSocket.onclose = function (e) {
                _this.connectionClosingTask.resolve();
            };
            this.webSocket.onopen = function () {
                console.log("Connected to WebSockets HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        trafficWatchClient.prototype.connectEventSource = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            this.eventSource = new EventSource(this.resourcePath + '/traffic-watch/events?' + connectionString);
            this.eventSource.onmessage = function (e) { return _this.onMessage(e); };
            this.eventSource.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.connectionOpeningTask.reject();
                }
                else {
                    _this.eventSource.close();
                    _this.connectionClosingTask.resolve(e);
                }
            };
            this.eventSource.onopen = function () {
                console.log("Connected to EventSource HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                _this.successfullyConnectedOnce = true;
                connectionOpened = true;
                _this.connectionOpeningTask.resolve();
            };
        };
        trafficWatchClient.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        trafficWatchClient.prototype.onMessage = function (e) {
            var eventDto = JSON.parse(e.data);
            var type = eventDto.Type;
            var value = eventDto.Value;
            if (type !== "Heartbeat") {
                if (type === "LogNotification") {
                    this.fireEvents(this.adminLogsHandlers(), value, function (e) { return true; });
                }
                else {
                    console.log("Unhandled Changes API notification type: " + type);
                }
            }
        };
        trafficWatchClient.prototype.watchTraffic = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            this.adminLogsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.adminLogsHandlers.remove(callback);
            });
        };
        trafficWatchClient.prototype.disconnect = function () {
            var _this = this;
            this.connectionOpeningTask.done(function () {
                if (_this.webSocket && _this.webSocket.readyState == _this.readyStateOpen) {
                    console.log("Disconnecting from WebSocket HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                    _this.isCleanClose = true;
                    _this.webSocket.close(_this.normalClosureCode, _this.normalClosureMessage);
                }
                else if (_this.eventSource && _this.eventSource.readyState == _this.readyStateOpen) {
                    console.log("Disconnecting from EventSource HTTP Trace for " + ((!!_this.resourcePath) ? (_this.resourcePath) : "admin"));
                    _this.isCleanClose = true;
                    _this.eventSource.close();
                    _this.connectionClosingTask.resolve();
                }
            });
        };
        trafficWatchClient.isServerSupportingWebSockets = true;
        trafficWatchClient.messageWasShownOnce = false;
        return trafficWatchClient;
    })();
    return trafficWatchClient;
});
define("common/uploadQueueHelper", ["require", "exports", "models/filesystem/uploadItem"], function (require, exports, uploadItem) {
    var uploadQueueHelper = (function () {
        function uploadQueueHelper() {
        }
        uploadQueueHelper.stringifyUploadQueue = function (queue) {
            return ko.toJSON(queue);
        };
        uploadQueueHelper.parseUploadQueue = function (queue, fs) {
            var stringArray = JSON.parse(queue);
            var uploadQueue = [];
            for (var i = 0; i < stringArray.length; i++) {
                uploadQueue.push(new uploadItem(stringArray[i]["id"], stringArray[i]["fileName"], stringArray[i]["status"], fs));
            }
            return uploadQueueHelper.sortUploadQueue(uploadQueue);
        };
        uploadQueueHelper.sortUploadQueue = function (uploadQueue) {
            return uploadQueue.sort(function (a, b) {
                if (a.status() === uploadQueueHelper.failedStatus) {
                    return -1;
                }
                else if (a.status() === uploadQueueHelper.uploadingStatus)
                    if (b.status() === uploadQueueHelper.failedStatus)
                        return 1;
                    else
                        return -1;
                else if (a.status() === uploadQueueHelper.queuedStatus) {
                    if (b.status() === uploadQueueHelper.failedStatus || b.status() === uploadQueueHelper.uploadingStatus)
                        return 1;
                    else
                        return -1;
                }
                else
                    return 1;
            });
        };
        uploadQueueHelper.updateLocalStorage = function (x, fs) {
            window.localStorage.setItem(uploadQueueHelper.localStorageUploadQueueKey + fs.name, uploadQueueHelper.stringifyUploadQueue(x));
        };
        uploadQueueHelper.updateQueueStatus = function (guid, status, queue) {
            var items = ko.utils.arrayFilter(queue, function (i) {
                return i.id() === guid;
            });
            if (items) {
                items[0].status(status);
            }
            queue = uploadQueueHelper.sortUploadQueue(queue);
            ;
        };
        uploadQueueHelper.uploadingStatus = "Uploading...";
        uploadQueueHelper.failedStatus = "Failed";
        uploadQueueHelper.uploadedStatus = "Uploaded";
        uploadQueueHelper.queuedStatus = "Queued";
        uploadQueueHelper.localStorageUploadQueueKey = "ravenFs-uploadQueue.";
        return uploadQueueHelper;
    })();
    return uploadQueueHelper;
});
define("common/bindingHelpers/aceEditorBindingHandler", ["require", "exports", "durandal/composition", "ace/ace", "ace/ext/language_tools", "ace/mode/lucene", "ace/theme/xcode", "ace/mode/json_newline_friendly", "ace/mode/json"], function (require, exports, composition, ace) {
    /*
     * A custom Knockout binding handler transforms the target element (usually a <pre>) into a code editor, powered by Ace. http://ace.c9.io
     * Usage: data-bind="aceEditor: { code: someObservableString, lang: 'ace/mode/csharp', theme: 'ace/theme/xcode', fontSize: '16px' }"
     * All params are optional, except code.
     */
    var aceEditorBindingHandler = (function () {
        function aceEditorBindingHandler() {
            this.defaults = {
                theme: "ace/theme/xcode",
                fontSize: "16px",
                lang: "ace/mode/csharp",
                readOnly: false,
                selectAll: false,
                bubbleEscKey: false,
                bubbleEnterKey: false
            };
            this.previousLinesCount = -1;
        }
        aceEditorBindingHandler.install = function () {
            if (!ko.bindingHandlers["aceEditor"]) {
                ko.bindingHandlers["aceEditor"] = new aceEditorBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("aceEditor");
                var Editor = require("ace/editor").Editor;
                require("ace/config").defineOptions(Editor.prototype, "editor", {
                    editorType: {
                        set: function (val) {
                        },
                        value: "general"
                    }
                });
                /// taken from https://github.com/ajaxorg/ace-demos/blob/master/scrolling-editor.html
                aceEditorBindingHandler.commands.push({
                    name: "Toggle Fullscreen",
                    bindKey: "Shift+F11",
                    exec: function (editor) {
                        aceEditorBindingHandler.dom.toggleCssClass(document.body, "fullScreen");
                        aceEditorBindingHandler.dom.toggleCssClass(editor.container, "fullScreen-editor");
                        editor.resize();
                        if (aceEditorBindingHandler.dom.hasCssClass(document.body, "fullScreen") === true) {
                            $(".fullScreenModeLabel").text(aceEditorBindingHandler.leaveFullScreenText);
                            $(".fullScreenModeLabel").hide();
                            $(editor.container).find(".fullScreenModeLabel").show();
                            editor.setOption("maxLines", null);
                        }
                        else {
                            $(".fullScreenModeLabel").text(aceEditorBindingHandler.goToFullScreenText);
                            $(".fullScreenModeLabel").show();
                            editor.setOption("maxLines", 10 * 1000);
                        }
                    }
                });
                aceEditorBindingHandler.commands.push({
                    name: "Exit FullScreen",
                    bindKey: "Esc",
                    exec: function (editor) {
                        if (aceEditorBindingHandler.dom.hasCssClass(document.body, "fullScreen") === true) {
                            aceEditorBindingHandler.dom.toggleCssClass(document.body, "fullScreen");
                            aceEditorBindingHandler.dom.toggleCssClass(editor.container, "fullScreen-editor");
                            $(".fullScreenModeLabel").text(aceEditorBindingHandler.goToFullScreenText);
                            $(".fullScreenModeLabel").show();
                        }
                        editor.resize();
                    }
                });
            }
        };
        aceEditorBindingHandler.detached = function () {
            aceEditorBindingHandler.customCompleters = [];
        };
        aceEditorBindingHandler.autoCompleteHub = function (editor, session, pos, prefix, callback) {
            var curEditorType = editor.getOption("editorType");
            var completerThreesome = aceEditorBindingHandler.customCompleters.first(function (x) { return x.editorType === curEditorType; });
            if (!!completerThreesome) {
                completerThreesome.completer.call(completerThreesome.completerHostObject, editor, session, pos, prefix, callback);
            }
            else {
                callback(null, []);
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        aceEditorBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var self = this;
            var bindingValues = valueAccessor();
            var theme = bindingValues.theme || this.defaults.theme;
            var fontSize = bindingValues.fontSize || this.defaults.fontSize;
            var lang = bindingValues.lang || this.defaults.lang;
            var readOnly = bindingValues.readOnly || this.defaults.readOnly;
            var typeName = bindingValues.typeName;
            var code = typeof bindingValues.code === "function" ? bindingValues.code : bindingContext.$rawData;
            var langTools = null;
            var completerHostObject = bindingValues.completerHostObject;
            this.minHeight = bindingValues.minHeight ? bindingValues.minHeight : 140;
            this.maxHeight = bindingValues.maxHeight ? bindingValues.maxHeight : 400;
            this.allowResize = bindingValues.allowResize ? bindingValues.allowResize : false;
            var selectAll = bindingValues.selectAll || this.defaults.selectAll;
            var bubbleEscKey = bindingValues.bubbleEscKey || this.defaults.bubbleEscKey;
            var bubbleEnterKey = bindingValues.bubbleEnterKey || this.defaults.bubbleEnterKey;
            var getFocus = bindingValues.getFocus;
            if (typeof code !== "function") {
                throw new Error("code should be an observable");
            }
            if (!!bindingValues.completer) {
                langTools = ace.require("ace/ext/language_tools");
            }
            var aceEditor = ace.edit(element);
            aceEditor.setOption("enableBasicAutocompletion", true);
            aceEditor.setOption("newLineMode", "windows");
            aceEditor.setTheme(theme);
            aceEditor.setFontSize(fontSize);
            aceEditor.getSession().setMode(lang);
            aceEditor.setReadOnly(readOnly);
            // Setup key bubbling 
            if (bubbleEscKey) {
                aceEditor.commands.addCommand({
                    name: "RavenStudioBubbleEsc",
                    bindKey: "esc",
                    exec: function () { return false; } // Returning false causes the event to bubble up.
                });
            }
            // setup the autocomplete mechanism, bind recieved function with recieved type, will only work if both were recieved
            if (!!typeName) {
                aceEditor.setOption("editorType", typeName);
                if (!!langTools) {
                    if (!aceEditorBindingHandler.customCompleters.first(function (x) { return x.editorType === typeName; })) {
                        aceEditorBindingHandler.customCompleters.push({ editorType: typeName, completerHostObject: completerHostObject, completer: bindingValues.completer });
                    }
                    if (!!aceEditor.completers) {
                        var completersList = aceEditor.completers;
                        if (!completersList.first(function (x) { return x.moduleId === "aceEditoBindingHandler"; })) {
                            langTools.addCompleter({ moduleId: "aceEditoBindingHandler", getCompletions: aceEditorBindingHandler.autoCompleteHub });
                        }
                    }
                    else {
                        langTools.addCompleter({ moduleId: "aceEditoBindingHandler", getCompletions: aceEditorBindingHandler.autoCompleteHub });
                    }
                }
            }
            // In the event of keyup or lose focus, push the value into the observable.
            var aceFocusElement = ".ace_text-input";
            $(element).on('keyup', aceFocusElement, function () {
                code(aceEditor.getSession().getValue());
                self.alterHeight(element, aceEditor);
            });
            $(element).on('focus', aceFocusElement, function () { return aceEditorBindingHandler.currentEditor = aceEditor; });
            // Initialize ace resizeble text box
            aceEditor.setOption('vScrollBarAlwaysVisible', true);
            aceEditor.setOption('hScrollBarAlwaysVisible', true);
            if ($(element).height() < this.minHeight) {
                $(element).height(this.minHeight);
            }
            $(element).resizable({
                minHeight: this.minHeight,
                handles: "s, se",
                grid: [10000000000000000, 1],
                resize: function (event, ui) {
                    aceEditor.resize();
                }
            });
            this.alterHeight(element, aceEditor);
            $(element).find('.ui-resizable-se').removeClass('ui-icon-gripsmall-diagonal-se');
            $(element).find('.ui-resizable-se').addClass('ui-icon-carat-1-s');
            $('.ui-resizable-se').css('cursor', 's-resize');
            $(element).append('<span class="fullScreenModeLabel" style="font-size:90%; z-index: 1000; position: absolute; bottom: 22px; right: 22px; opacity: 0.4">Press Shift+F11 to enter full screen mode</span>');
            // When the element is removed from the DOM, unhook our keyup and focus event handlers and remove the  resizable functionality completely. lest we leak memory.
            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                $(element).off('keyup', aceFocusElement);
                $(element).off('focus', aceFocusElement);
                $(element).resizable("destroy");
                aceEditor.getSession().setUseWorker(false);
                aceEditor.destroy();
            });
            // Keep track of the editor for this element.
            ko.utils.domData.set(element, "aceEditor", aceEditor);
            if (bindingValues.getFocus) {
                setTimeout(function () { return aceEditor.focus(); }, 0);
            }
            if (selectAll) {
                setTimeout(function () { return aceEditor.selectAll(); }, 0);
            }
        };
        // Called by Knockout each time the dependent observable value changes.
        aceEditorBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValues = valueAccessor();
            var code = ko.unwrap(bindingValues.code);
            var aceEditor = ko.utils.domData.get(element, "aceEditor");
            var editorCode = aceEditor.getSession().getValue();
            if (code !== editorCode) {
                aceEditor.getSession().setValue(code || "");
            }
            aceEditor.setReadOnly(bindingValues.readOnly);
            if (this.allowResize) {
                this.alterHeight(element, aceEditor);
            }
        };
        aceEditorBindingHandler.prototype.alterHeight = function (element, aceEditor) {
            if (!this.allowResize) {
                return;
            }
            // update only if line count changes
            var currentLinesCount = aceEditor.getSession().getScreenLength();
            if (this.previousLinesCount != currentLinesCount) {
                var newHeight = currentLinesCount
                    * aceEditor.renderer.lineHeight
                    + aceEditor.renderer.scrollBar.getWidth();
                +10; // few pixels extra padding
                if (newHeight < this.minHeight) {
                    newHeight = this.minHeight;
                }
                else if (newHeight > this.maxHeight) {
                    newHeight = this.maxHeight;
                }
                $(element).height(newHeight);
                aceEditor.resize();
                this.previousLinesCount = currentLinesCount;
            }
        };
        aceEditorBindingHandler.dom = require("ace/lib/dom");
        aceEditorBindingHandler.commands = require("ace/commands/default_commands").commands;
        aceEditorBindingHandler.isInFullScreeenMode = ko.observable(false);
        aceEditorBindingHandler.goToFullScreenText = "Press Shift + F11  to enter full screen mode";
        aceEditorBindingHandler.leaveFullScreenText = "Press Shift + F11 or Esc to leave full screen mode";
        aceEditorBindingHandler.customCompleters = [];
        return aceEditorBindingHandler;
    })();
    return aceEditorBindingHandler;
});
define("common/bindingHelpers/autoCompleteBindingHandler", ["require", "exports", "durandal/composition"], function (require, exports, composition) {
    /*
     * A KnockoutJS binding handler that transforms a div into the auto-complete items container for a text box.
     *
     * Usage:
     *           <input id="myAutoComplete" type="text" data-bind="value: mySearchValue, valueUpdate: 'afterkeydown'" />
     *           <div style="display: none" data-bind="autoComplete: '#myAutoComplete', foreach: yourOwnResults">
     *               <div data-bind="text: name"></div>
     *           </div>
     *
     * In the above sample, yourOwnResults is an array that you are responsible for populating. And 'name' is the property on the items in that array.
     */
    var autoCompleteBindingHandler = (function () {
        function autoCompleteBindingHandler() {
        }
        autoCompleteBindingHandler.install = function () {
            if (!ko.bindingHandlers["autoComplete"]) {
                ko.bindingHandlers["autoComplete"] = new autoCompleteBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("autoComplete");
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        autoCompleteBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var _this = this;
            var inputId = valueAccessor();
            var input = $(inputId);
            if (input.length !== 1) {
                // Don't throw an error here, because we may cancel navigation, and Durandal may pull the element out.
                // Instead, we'll just issue a warning in the console and return.
                //throw new Error("Expected 1 auto complete element, '" + inputId + "', but found " + input.length);
                console.warn("Expected 1 auto complete element, '" + inputId + "', but found " + input.length);
                return;
            }
            // Hide the auto complete container and size it to the same size as the textbox.
            var $element = $(element);
            element.style.display = "none";
            element.style.position = "absolute";
            element.style.left = "auto";
            element.style.top = (input.height() + 20) + "px";
            //This makes elements with long names overflow the container... commenting it for the moment
            //element.style.width = input.width() + "px";
            // Clicking an element in the auto complete list should hide it.
            $element.on('click', function () { return setTimeout(function () { return element.style.display = "none"; }, 0); });
            // Leaving the textbox should hide the auto complete list.
            input.on('blur', function (args) { return setTimeout(function () { return element.style.display = "none"; }, 200); });
            // Putting the focus back on the textbox should show the auto complete list if we have items.
            input.on('focus', function (args) { return setTimeout(function () {
                return element.style.display = _this.getAllAutoCompleteItems($element).length > 0 ? "block" : "none";
            }); });
            // Up, down, enter all have special meaning.
            input.on('keydown', function (args) { return _this.handleKeyPress(element, $element, input, args); });
            // When the results change and we have 1 or more, display the auto complete container.
            var results = allBindings()['foreach'];
            if (!results) {
                throw new Error("Unable to find results list for auto complete.");
            }
            var subscription = results.subscribe(function (array) {
                element.style.display = array.length === 0 || !input.is(":focus") ? "none" : "block";
            });
            // Clean up after ourselves when the node is removed from the DOM.
            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                input.off('blur');
                $element.off('click');
                input.off('keydown');
                subscription.dispose();
            });
        };
        autoCompleteBindingHandler.prototype.getAllAutoCompleteItems = function (resultContainer) {
            return resultContainer.children("li");
        };
        autoCompleteBindingHandler.prototype.findAutoCompleteItemMatching = function (resultContainer, text) {
            var textLower = text.toLowerCase();
            return this.getAllAutoCompleteItems(resultContainer)
                .toArray()
                .filter(function (el) { return el.textContent && el.textContent.trim().toLowerCase().indexOf(textLower) >= 0; })[0];
        };
        autoCompleteBindingHandler.prototype.handleKeyPress = function (element, $element, $input, args) {
            var enter = 13;
            var escape = 27;
            var downArrow = 40;
            var upArrow = 38;
            if (args.which === escape) {
                element.style.display = "none";
            }
            var lis, curSelected;
            if (element.style.display == "none" && args.which === downArrow) {
                if ($element.children("li").length > 0 && $input.is(":focus")) {
                    setTimeout(function () { return element.style.display = "block"; }, 0);
                    return true;
                }
            }
            if (args.which === downArrow || args.which === upArrow || args.which === enter) {
                lis = this.getAllAutoCompleteItems($element);
                curSelected = $element.find(".active");
            }
            if (args.which === downArrow) {
                if (curSelected.length > 0) {
                    curSelected.removeClass("active");
                    var nextSelected = curSelected.next();
                    if (nextSelected.length) {
                        nextSelected.addClass("active");
                        $element.scrollTop((nextSelected.index() - 1) * 30);
                    }
                    else {
                        lis.first().addClass("active");
                        $element.scrollTop(0);
                    }
                }
                else {
                    curSelected = lis.first().addClass("active");
                }
            }
            else if (args.which === upArrow) {
                if (curSelected.length > 0) {
                    curSelected.removeClass("active");
                    var prevSelected = curSelected.prev();
                    if (prevSelected.length) {
                        prevSelected.addClass("active");
                        $element.scrollTop((prevSelected.index() - 1) * 30);
                    }
                    else {
                        lis.last().addClass("active");
                        $element.scrollTop($element.children("li").length * 30);
                    }
                }
                else {
                    curSelected = lis.last().addClass("active");
                }
            }
            else if (args.which === enter) {
                var itemToSelect = curSelected.length ? curSelected : $(this.findAutoCompleteItemMatching($element, $input.val()));
                if (itemToSelect.length) {
                    itemToSelect.click();
                }
            }
        };
        return autoCompleteBindingHandler;
    })();
    return autoCompleteBindingHandler;
});
define("common/bindingHelpers/autoRefreshBindingHandler", ["require", "exports", "durandal/composition", "d3/d3"], function (require, exports, composition, d3) {
    var autoRefreshBindingHandler = (function () {
        function autoRefreshBindingHandler() {
        }
        autoRefreshBindingHandler.install = function () {
            if (!ko.bindingHandlers["autoRefresh"]) {
                ko.bindingHandlers["autoRefresh"] = new autoRefreshBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler('help');
            }
        };
        autoRefreshBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var config = ko.unwrap(valueAccessor());
            if (!config.onRefresh) {
                throw new Error("onRefresh attribute is required.");
            }
            if (!config.duration) {
                throw new Error("duration attribute is required.");
            }
            var context = autoRefreshBindingHandler.initRefresh(element, config);
            $(element).click(function () { return autoRefreshBindingHandler.toggleAutorefresh(context); });
            var active = ko.unwrap(config.active);
            if (active) {
                autoRefreshBindingHandler.animatePath(context);
            }
            else {
                context.refreshing(true);
                context.autorefreshEnabled(false);
            }
            var childBindingContext = bindingContext.createChildContext(context, null, function (context) { return ko.utils.extend(context, autoRefreshBindingHandler); });
            ko.applyBindingsToDescendants(childBindingContext, element);
            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                // break animation (if any)
                context.path
                    .transition()
                    .duration(0);
                context.disposed = true;
            });
            return { controlsDescendantBindings: true };
        };
        autoRefreshBindingHandler.initRefresh = function (element, config) {
            var container = d3.select(element);
            container.classed('auto-refresh', true);
            if (!container.attr('title')) {
                container.attr('title', 'Auto refresh');
            }
            container.append("i")
                .attr({
                'class': 'fa fa-fw',
                'data-bind': "css: { 'fa-pause': autorefreshEnabled(), 'fa-refresh': !autorefreshEnabled() }"
            });
            var divContainer = container.append("div")
                .attr({
                'class': 'circleRefresh',
                'data-bind': "css: { 'timer': autorefreshEnabled(), 'dashed': refreshing() }"
            });
            divContainer.append('svg')
                .attr({
                'class': 'inner-svg',
                'width': '80px',
                'height': '80px',
                'viewBox': '0 0 80 80'
            });
            var svg = container.select("svg");
            var group = svg
                .append('g')
                .attr('transform', 'translate(40,40)');
            group.append('circle')
                .attr({
                'class': 'circleRefresh-circlebg',
                cx: 0,
                cy: 0,
                r: 30
            });
            var path = group
                .append('path')
                .attr({
                'class': 'circleRefresh-circle'
            });
            var arc = d3.svg.arc()
                .innerRadius(30)
                .outerRadius(30)
                .startAngle(0)
                .endAngle(function (d) { return d * 2 * Math.PI; });
            var autorefreshEnabled = ko.isObservable(config.active) ? config.active : ko.observable(true);
            return {
                svg: svg,
                path: path,
                arc: arc,
                autorefreshEnabled: autorefreshEnabled,
                refreshing: ko.observable(false),
                duration: config.duration,
                onRefresh: config.onRefresh,
                disposed: false
            };
        };
        autoRefreshBindingHandler.animatePath = function (context) {
            var _this = this;
            context.refreshing(false);
            context.path.transition()
                .duration(context.duration)
                .ease('linear')
                .attrTween("d", function () { return function (t) { return context.arc(t); }; })
                .each('end', function () {
                context.refreshing(true);
                context.onRefresh()
                    .always(function () {
                    if (!context.disposed) {
                        _this.animatePath(context);
                    }
                });
            });
        };
        autoRefreshBindingHandler.toggleAutorefresh = function (context) {
            if (context.autorefreshEnabled()) {
                // stop current animation if any
                if (!context.refreshing()) {
                    context.path
                        .transition()
                        .duration(0);
                }
            }
            else {
                // start with refresh and then animate refresh
                context.path
                    .attr('d', context.arc(1));
                context.refreshing(true);
                context.onRefresh()
                    .always(function () { return autoRefreshBindingHandler.animatePath(context); });
            }
            context.autorefreshEnabled(!context.autorefreshEnabled());
        };
        autoRefreshBindingHandler.prototype.update = function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
        };
        return autoRefreshBindingHandler;
    })();
    return autoRefreshBindingHandler;
});
/// <reference path="../../../Scripts/typings/bootstrap.v3.datetimepicker/bootstrap.v3.datetimepicker.d.ts" />
define("common/bindingHelpers/datePickerBindingHandler", ["require", "exports", "durandal/composition", "moment"], function (require, exports, composition, moment) {
    var datePickerBindingHandler = (function () {
        function datePickerBindingHandler() {
        }
        datePickerBindingHandler.install = function () {
            if (!ko.bindingHandlers["datePicker"]) {
                ko.bindingHandlers["datePicker"] = new datePickerBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("datePicker");
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        datePickerBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = allBindings().datepickerOptions || {};
            var endDateElement = options.endDateElement;
            var startDateElement = options.startDateElement;
            delete options.endDateElement;
            delete options.startDateElement;
            var dpicker = $(element).datetimepicker(options);
            dpicker.on('dp.change', function (ev) {
                if (endDateElement) {
                    $("#" + endDateElement).data("DateTimePicker").minDate(ev.date);
                }
                if (startDateElement) {
                    $("#" + startDateElement).data("DateTimePicker").maxDate(ev.date);
                }
                var newDate = moment(ev.date);
                var value = valueAccessor();
                value(newDate);
            });
        };
        // Called by Knockout each time the dependent observable value changes.
        datePickerBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var date = ko.unwrap(valueAccessor());
            if (date) {
                $(element).data("DateTimePicker").date(date);
            }
        };
        return datePickerBindingHandler;
    })();
    return datePickerBindingHandler;
});
define("common/bindingHelpers/dynamicHeightBindingHandler", ["require", "exports", "durandal/composition"], function (require, exports, composition) {
    /*
     * A custom Knockout binding handler that causes a DOM element to change its height so that its bottom reaches to a target element.
     * Usage: data-bind="dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: 'footer' }"
     * Target can be whatever element you choose.
     */
    var dynamicHeightBindingHandler = (function () {
        function dynamicHeightBindingHandler() {
            var _this = this;
            this.throttleTimeMs = 100;
            var $window = $(window);
            this.windowHeightObservable = ko.observable($window.height());
            window['ravenStudioWindowHeight'] = this.windowHeightObservable.throttle(this.throttleTimeMs);
            $window.resize(function (ev) { return _this.windowHeightObservable($window.height()); });
        }
        dynamicHeightBindingHandler.install = function () {
            if (!ko.bindingHandlers["dynamicHeight"]) {
                ko.bindingHandlers["dynamicHeight"] = new dynamicHeightBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // This is required so that we know the correct height for the element.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler('dynamicHeight');
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        dynamicHeightBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        };
        // Called by Knockout each time the dependent observable value changes.
        dynamicHeightBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValue = valueAccessor(); // Necessary to register knockout dependency. Without it, update won't fire when window height changes.
            if (bindingValue.target) {
                var newWindowHeight = bindingValue.resizeTrigger;
                var targetSelector = bindingValue.target || "footer";
                var bottomMargin = ko.unwrap(bindingValue.bottomMargin) || 0;
                var container = bindingValue.container;
                // Check what was the last dispatched height to this element.
                var lastWindowHeightKey = "ravenStudioLastDispatchedHeight";
                var lastWindowHeight = ko.utils.domData.get(element, lastWindowHeightKey);
                if (lastWindowHeight !== newWindowHeight) {
                    ko.utils.domData.set(element, lastWindowHeightKey, newWindowHeight);
                    dynamicHeightBindingHandler.stickToTarget(element, targetSelector, bottomMargin, container);
                }
            }
        };
        dynamicHeightBindingHandler.stickToTarget = function (element, targetSelector, bottomMargin, container) {
            var targetElement = $(targetSelector);
            if (targetSelector.length === 0) {
                throw new Error("Couldn't configure dynamic height because the target element isn't on the page. Target element: " + targetSelector);
            }
            var $element = $(element);
            var isVisible = $element.is(":visible");
            if (isVisible) {
                var containerOffset = 0;
                if (container) {
                    var $container = $(container);
                    var topOffsetDiff = $element.offset().top - $container.offset().top;
                    containerOffset = $container.outerHeight() - topOffsetDiff - $element.outerHeight();
                }
                var elementTop = $element.offset().top;
                var footerTop = targetElement.position().top;
                var padding = 5 + bottomMargin;
                var desiredElementHeight = footerTop - elementTop - padding - containerOffset;
                if ($(document).fullScreen()) {
                    var windowHeightKey = "ravenStudioLastDispatchedHeight";
                    var windowHeight = ko.utils.domData.get(element, windowHeightKey);
                    desiredElementHeight = windowHeight - elementTop - padding;
                }
                var minimumHeight = 100;
                if (desiredElementHeight >= minimumHeight) {
                    $element.innerHeight(desiredElementHeight);
                    $element.trigger("DynamicHeightSet", desiredElementHeight);
                }
            }
        };
        return dynamicHeightBindingHandler;
    })();
    return dynamicHeightBindingHandler;
});
define("common/bindingHelpers/enableResizeBindingHandler", ["require", "exports", "durandal/composition"], function (require, exports, composition) {
    /*
     * A custom Knockout binding handler that causes the collections labels window to become resizable
     * Usage: data-bind="enableResize: true"
     */
    var enableResizeBindingHandler = (function () {
        function enableResizeBindingHandler() {
        }
        enableResizeBindingHandler.install = function () {
            if (!ko.bindingHandlers["enableResize"]) {
                ko.bindingHandlers["enableResize"] = new enableResizeBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // This is required so that we know the correct height for the element.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler('enableResize');
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        enableResizeBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = {
                handles: 'e, w',
                helper: function (string) { return "resizable-helper"; },
                resize: function (event, ui) {
                    $(element).find('.col-resizable-target').width(ui.size.width - 40);
                    $(element).find('.col-resizable-affected').width($(element).width() - ui.size.width - 30);
                }
            };
            // initialize the resizable control
            $(element).find('.col-resizable-target').resizable(options);
        };
        return enableResizeBindingHandler;
    })();
    return enableResizeBindingHandler;
});
define("common/bindingHelpers/fileUploadBindingHandler", ["require", "exports", "durandal/composition", "durandal/system", "models/filesystem/uploadItem", "commands/filesystem/uploadFileToFilesystemCommand", "common/uploadQueueHelper"], function (require, exports, composition, system, uploadItem, uploadFileToFilesystemCommand, uploadQueueHelper) {
    // Usage: <input type="file" data-bind="fileUpload: { files: files, uploads: uploadQueue, success: uploadSuccess.bind($data), fail: uploadFailed.bind($data) }" />
    // files: KnockoutObservable<File[]>
    // uploads: KnockoutObservableArray<uploadItem>
    // success: (i: uploadItem) => void;
    // fail: (i: uploadItem) => void;
    var fileUploadBindingHandler = (function () {
        function fileUploadBindingHandler() {
        }
        fileUploadBindingHandler.install = function () {
            if (!ko.bindingHandlers["fileUpload"]) {
                ko.bindingHandlers["fileUpload"] = new fileUploadBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("fileUpload");
            }
        };
        fileUploadBindingHandler.prototype.init = function (element, valueAccessor) {
        };
        fileUploadBindingHandler.prototype.update = function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
            var _this = this;
            var options = ko.utils.unwrapObservable(valueAccessor());
            var context = viewModel;
            var filesystem = ko.utils.unwrapObservable(bindingContext.$data["activeFilesystem"]);
            if (options) {
                options.files();
                // Access the files observable now so that .update is called whenever it changes.
                if (element.files.length) {
                    if (options.before) {
                        options.before();
                    }
                    var files = element.files;
                    for (var i = 0; i < files.length; i++) {
                        var file = files[i];
                        var guid = system.guid();
                        var directory = options.directory() ? options.directory() : "";
                        var item = new uploadItem(guid, directory + "/" + file.name, uploadQueueHelper.queuedStatus, context.activeFilesystem());
                        options.uploads.push(item);
                        new uploadFileToFilesystemCommand(file, directory, guid, filesystem, function (e) { return _this.uploadProgressReported(e); }, true)
                            .execute()
                            .done(function (x) { return options.success(x); })
                            .fail(function (x) { return options.fail(x); });
                        item.status(uploadQueueHelper.uploadingStatus);
                        options.uploads(uploadQueueHelper.sortUploadQueue(options.uploads()));
                        options.uploads.notifySubscribers(options.uploads());
                    }
                }
                options.files(null);
            }
        };
        fileUploadBindingHandler.prototype.uploadProgressReported = function (e) {
            if (e.lengthComputable) {
                var percentComplete = e.loaded / e.total;
            }
        };
        return fileUploadBindingHandler;
    })();
    return fileUploadBindingHandler;
});
define("common/bindingHelpers/helpBindingHandler", ["require", "exports", "durandal/composition", "viewmodels/viewModelBase"], function (require, exports, composition, viewModelBase) {
    var helpBindingHandler = (function () {
        function helpBindingHandler() {
        }
        helpBindingHandler.install = function () {
            if (!ko.bindingHandlers["help"]) {
                ko.bindingHandlers["help"] = new helpBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // This is required so that we know the correct height for the element.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler('help');
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        helpBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            this.link = $('<a class="help_link"><i class="fa fa-question-circle"></i></a>').attr('target', '_blank');
            $(element).append(this.link);
        };
        helpBindingHandler.prototype.update = function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
            var value = valueAccessor();
            var hashUnwrapped = ko.unwrap(value.hash);
            var version = viewModelBase.clientVersion();
            var titleUnwrapped = ko.unwrap(value.title);
            var href = "http://ravendb.net/l/" + hashUnwrapped + "/" + version + "/";
            this.link.attr('href', href);
            this.link.attr('title', titleUnwrapped);
        };
        return helpBindingHandler;
    })();
    return helpBindingHandler;
});
/// <reference path="../../../Scripts/typings/jquery.dynatree/jquery.dynatree.d.ts" />
define("common/bindingHelpers/treeBindingHandler", ["require", "exports", "durandal/composition", "common/appUrl", "commands/filesystem/getFoldersCommand"], function (require, exports, composition, appUrl, getFoldersCommand) {
    /*
     * A custom Knockout binding handler transforms the target element (a <div>) into a directory tree, powered by jquery-dynatree
     * Usage: data-bind="tree: { value: someObservableTreeObject }"
     */
    var treeBindingHandler = (function () {
        function treeBindingHandler() {
        }
        treeBindingHandler.install = function () {
            if (!ko.bindingHandlers["tree"]) {
                ko.bindingHandlers["tree"] = new treeBindingHandler();
                // This tells Durandal to fire this binding handler only after composition 
                // is complete and attached to the DOM.
                // See http://durandaljs.com/documentation/Interacting-with-the-DOM/
                composition.addBindingHandler("tree");
            }
        };
        // Called by Knockout a single time when the binding handler is setup.
        treeBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = ko.utils.unwrapObservable(valueAccessor());
            var tree = $(element).dynatree({
                children: [{ title: appUrl.getFileSystem().name, key: "/", isLazy: true, isFolder: true }],
                onLazyRead: function (node) {
                    treeBindingHandler.loadNodeChildren("#" + element.id, node, options);
                    node.activate();
                },
                onExpand: function (expanded, node) {
                    if (expanded) {
                        treeBindingHandler.loadNodeChildren("#" + element.id, node, options);
                        node.activate();
                    }
                },
                selectMode: 1,
                onSelect: function (flag, node) {
                    treeBindingHandler.onActivateAndSelect(node, valueAccessor());
                },
                onActivate: function (node) {
                    treeBindingHandler.onActivateAndSelect(node, valueAccessor());
                },
            });
            var firstNode = $(element).dynatree("getRoot", []).getChildren()[0];
            firstNode.activate();
            firstNode.expand(null);
        };
        treeBindingHandler.loadNodeChildren = function (tree, node, options) {
            var _this = this;
            var dir;
            if (node && node.data && node.data.key != "/") {
                dir = node.data.key;
            }
            var command = new getFoldersCommand(appUrl.getFileSystem(), 0, 1024, dir);
            command.execute().done(function (results) {
                node.setLazyNodeStatus(0);
                var versioningEnabled = _this.includeRevisionsFunc();
                if (!dir && versioningEnabled) {
                    results.unshift({ key: "/$$revisions$$", title: "$revisions", isLazy: false, isFolder: true });
                }
                var newSet = {};
                var differenceSet = {};
                //this is being done creating a new dictionary to reduce complexity
                for (var i = 0; i < results.length; i++) {
                    newSet[results[i].key] = results[i];
                }
                if (node.hasChildren()) {
                    //calculate deleted and transient nodes
                    for (var j = 0; j < node.getChildren().length; j++) {
                        if (!newSet[node.getChildren()[j].data.key]) {
                            differenceSet[node.getChildren()[j].data.key] = node.getChildren()[j];
                        }
                    }
                    var nodesToRemove = [];
                    //mark deleted nodes filtering transient
                    for (var k = 0; k < node.getChildren().length; k++) {
                        var nodeK = node.getChildren()[k];
                        if (!newSet[nodeK.data.key] && differenceSet[nodeK.data.key] && differenceSet[nodeK.data.key].data.addClass != treeBindingHandler.transientNodeStyle) {
                            nodesToRemove.push(k);
                        }
                        else {
                            newSet[nodeK.data.key] = null;
                        }
                    }
                    //remove deleted nodes
                    for (var m = 0; m < nodesToRemove.length; m++) {
                        node.getChildren()[nodesToRemove[m]].remove();
                    }
                }
                //add new nodes
                for (var key in newSet) {
                    if (newSet[key]) {
                        node.addChild(newSet[key]);
                    }
                }
                if (options && node.hasChildren()) {
                    options.currentLevelNodes.push(node.getChildren().map(function (x) { return x.data.key; }));
                }
            });
        };
        treeBindingHandler.reloadNode = function (tree, nodeKey) {
            var dynaTree = $(tree).dynatree("getTree");
            var node = dynaTree.getNodeByKey(nodeKey);
            treeBindingHandler.loadNodeChildren(tree, node, null);
        };
        treeBindingHandler.onActivateAndSelect = function (node, valueAccessor) {
            var options = ko.utils.unwrapObservable(valueAccessor);
            var selectedNode = node.data && node.data.key != "/" ? node.data.key : null;
            options.selectedNode(selectedNode);
            if (node.data) {
                var siblings = [];
                if (node.hasChildren()) {
                    siblings = node.getChildren();
                }
                var mappedNodes = siblings.map(function (x) { return x.data.title; });
                options.currentLevelNodes(mappedNodes);
            }
        };
        // Called by Knockout each time the dependent observable value changes.
        treeBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var options = ko.utils.unwrapObservable(valueAccessor());
            if (options.addedNode()) {
                var newNode = options.addedNode();
                var activeNode = $(element).dynatree("getActiveNode", []);
                var parentOfNewNode = newNode.key.substring(0, newNode.key.lastIndexOf("/"));
                if (parentOfNewNode === "") {
                    parentOfNewNode = "/";
                }
                var parentNode = $(element).dynatree("getTree").getNodeByKey(parentOfNewNode);
                if (parentNode) {
                    parentNode.addChild(newNode);
                    if (parentNode == activeNode) {
                        options.currentLevelNodes(activeNode.getChildren().map(function (x) { return x.data.title; }));
                    }
                    options.addedNode(null);
                }
            }
        };
        treeBindingHandler.updateNodeHierarchyStyle = function (tree, key, styleClass) {
            var theTree = $(tree).dynatree("getTree");
            var slashPosition = key.length;
            while (slashPosition > 0) {
                key = key.substring(0, slashPosition);
                var temporaryNode = theTree.getNodeByKey(key);
                if (temporaryNode && temporaryNode.data.addClass != styleClass) {
                    temporaryNode.data.addClass = styleClass;
                    temporaryNode.reloadChildren();
                }
                slashPosition = key.lastIndexOf("/");
            }
        };
        treeBindingHandler.setNodeLoadStatus = function (tree, key, status) {
            // Set node load status
            // -1 = error
            // 0 = load OK
            // 1 = loading
            var object = $(tree);
            if (!object || object.length <= 0)
                return;
            var theTree = object.dynatree("getTree");
            if (!theTree) {
                return;
            }
            var slashPosition = key.length;
            while (slashPosition > 0) {
                key = key.substring(0, slashPosition);
                var temporaryNode = theTree.getNodeByKey(key);
                if (temporaryNode) {
                    temporaryNode.setLazyNodeStatus(status);
                }
                slashPosition = key.lastIndexOf("/");
            }
        };
        treeBindingHandler.nodeExists = function (tree, key) {
            var dynaTree = $(tree).dynatree("getTree");
            var node = dynaTree.getNodeByKey(key);
            if (node) {
                return true;
            }
            return false;
        };
        treeBindingHandler.isNodeExpanded = function (tree, key) {
            var dynaTree = $(tree).dynatree("getTree");
            if (!dynaTree)
                return false;
            var node = dynaTree.getNodeByKey(key);
            return node && node.isExpanded();
        };
        treeBindingHandler.transientNodeStyle = "temp-folder";
        return treeBindingHandler;
    })();
    return treeBindingHandler;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/auth/apiKey", ["require", "exports", "models/resources/databaseAccess", "common/appUrl", "models/database/documents/documentMetadata", "models/database/documents/document"], function (require, exports, databaseAccess, appUrl, documentMetadata, document) {
    var apiKey = (function (_super) {
        __extends(apiKey, _super);
        function apiKey(dto) {
            var _this = this;
            _super.call(this, dto);
            this.name = ko.observable();
            this.secret = ko.observable();
            this.enabled = ko.observable();
            this.databases = ko.observableArray();
            this.visible = ko.observable(true);
            this.nameCustomValidity = ko.observable('');
            this.name(dto.Name);
            this.savedName = dto.Name;
            this.secret(dto.Secret);
            this.enabled(dto.Enabled);
            this.databases(dto.Databases.map(function (d) { return new databaseAccess(d); }));
            this.metadata = new documentMetadata(dto['@metadata']);
            this.fullApiKey = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return _this.name() + "/" + _this.secret();
            });
            this.connectionString = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return "Url = " + appUrl.forServer() + "; ApiKey = " + _this.fullApiKey() + "; Database = ";
            });
            this.directLink = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }
                return appUrl.forServer() + "/studio/index.html#api-key=" + _this.fullApiKey();
            });
            this.needToShowSystemDatabaseWarning = ko.computed(function () {
                var resources = _this.databases();
                var hasAllDatabasesAccess = resources.filter(function (x) { return x.tenantId() === "*"; }).length > 0;
                var hasSystemDatabaseAccess = resources.filter(function (x) { return x.tenantId() === "<system>"; }).length > 0;
                return hasAllDatabasesAccess && !hasSystemDatabaseAccess;
            });
        }
        apiKey.empty = function () {
            return new apiKey({
                Databases: [databaseAccess.empty().toDto()],
                Enabled: false,
                Name: "",
                Secret: ""
            });
        };
        apiKey.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            meta['@id'] = "Raven/ApiKeys/" + this.name();
            return {
                '@metadata': meta,
                Databases: this.databases().map(function (db) { return db.toDto(); }),
                Enabled: this.enabled(),
                Name: this.name(),
                Secret: this.secret()
            };
        };
        apiKey.prototype.enable = function () {
            this.enabled(true);
        };
        apiKey.prototype.disable = function () {
            this.enabled(false);
        };
        apiKey.prototype.generateSecret = function () {
            // The old Silverlight Studio would create a new GUID, strip out the 
            // dashes, and convert to base62.
            //
            // For the time being (is there a better way?), we're just creating a 
            // random string of alpha numeric characters.
            var minimumLength = 10;
            var maxLength = 32;
            var randomLength = Math.max(minimumLength, Math.random() * maxLength);
            var randomSecret = apiKey.randomString(randomLength, '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ');
            this.secret(randomSecret);
        };
        apiKey.prototype.addEmptyDatabase = function () {
            var newItem = { TenantId: '', Admin: false, ReadOnly: false };
            this.databases.push(new databaseAccess(newItem));
        };
        apiKey.prototype.removeDatabase = function (database) {
            this.databases.remove(database);
        };
        apiKey.prototype.setIdFromName = function () {
            this.__metadata.id = "Raven/ApiKeys/" + this.name();
            if (this.savedName !== this.name()) {
                this.__metadata.etag = null;
                this.savedName = this.name();
            }
        };
        apiKey.prototype.isValid = function (index) {
            var isApiKeyNameValid = this.name().indexOf("\\") === -1;
            var requiredValues = [this.name(), this.secret()];
            return requiredValues.every(function (v) { return v != null && v.length > 0; }) && isApiKeyNameValid;
        };
        apiKey.randomString = function (length, chars) {
            var result = '';
            for (var i = length; i > 0; --i)
                result += chars[Math.round(Math.random() * (chars.length - 1))];
            return result;
        };
        apiKey.prototype.getKey = function () {
            return "Raven/ApiKeys/" + this.name();
        };
        return apiKey;
    })(document);
    return apiKey;
});
define("models/auth/license", ["require", "exports"], function (require, exports) {
    var license = (function () {
        function license() {
        }
        license.licenseStatus = ko.observable();
        license.supportCoverage = ko.observable();
        license.hotSpare = ko.observable();
        license.licenseCssClass = ko.computed(function () {
            var status = license.licenseStatus();
            var hotSpare = license.hotSpare();
            if (hotSpare) {
                return 'hot-spare';
            }
            if (status == null || !status.IsCommercial) {
                return 'dev-only';
            }
            if (status.Status.contains("Expired")) {
                return 'expired';
            }
            else {
                return 'commercial';
            }
        });
        license.supportCssClass = ko.computed(function () {
            var support = license.supportCoverage();
            if (support == null) {
                return 'no-support';
            }
            switch (support.Status) {
                case 'ProductionSupport':
                    return 'production-support';
                case 'ProfessionalSupport':
                    return 'professional-support';
                case 'PartialSupport':
                    return 'partial-support';
                default:
                    return 'no-support';
            }
        });
        return license;
    })();
    return license;
});
define("models/auth/windowsAuthData", ["require", "exports", "models/resources/databaseAccess"], function (require, exports, databaseAccess) {
    var windowsAuthData = (function () {
        function windowsAuthData(dto) {
            var _this = this;
            this.name = ko.observable();
            this.enabled = ko.observable();
            this.databases = ko.observableArray();
            this.nameCustomValidity = ko.observable('');
            this.invalidName = ko.observable(false);
            this.verificationInProgress = ko.observable(false);
            this.name(dto.Name);
            this.enabled(dto.Enabled);
            this.databases(dto.Databases.map(function (dbDto) { return new databaseAccess(dbDto); }));
            this.needToShowSystemDatabaseWarning = ko.computed(function () {
                var resources = _this.databases();
                var hasAllDatabasesAdminAccess = resources.filter(function (x) { return x.admin() && x.tenantId() === "*"; }).length > 0;
                var hasSystemDatabaseAdminAcces = resources.filter(function (x) { return x.admin() && x.tenantId() === "<system>"; }).length > 0;
                return hasAllDatabasesAdminAccess && hasSystemDatabaseAdminAcces === false;
            });
        }
        windowsAuthData.prototype.toDto = function () {
            return {
                Name: this.name(),
                Enabled: this.enabled(),
                Databases: this.databases().map(function (db) { return db.toDto(); })
            };
        };
        windowsAuthData.empty = function () {
            return new windowsAuthData({
                Name: "",
                Enabled: false,
                Databases: [databaseAccess.empty().toDto()]
            });
        };
        windowsAuthData.prototype.enable = function () {
            this.enabled(true);
        };
        windowsAuthData.prototype.disable = function () {
            this.enabled(false);
        };
        windowsAuthData.prototype.addEmptyDatabase = function () {
            this.databases.push(databaseAccess.empty());
        };
        windowsAuthData.prototype.removeDatabase = function (dba) {
            this.databases.remove(dba);
        };
        return windowsAuthData;
    })();
    return windowsAuthData;
});
define("models/auth/windowsAuthSetup", ["require", "exports", "models/auth/windowsAuthData", "commands/auth/verifyPrincipalCommand"], function (require, exports, windowsAuthData, verifyPrincipalCommand) {
    var windowsAuthSetup = (function () {
        function windowsAuthSetup(dto) {
            var _this = this;
            this.requiredUsers = ko.observableArray();
            this.requiredGroups = ko.observableArray();
            this.requiredUsers(dto.RequiredUsers.map(function (winAuthDto) { return new windowsAuthData(winAuthDto); }));
            this.requiredGroups(dto.RequiredGroups.map(function (winAuthDto) { return new windowsAuthData(winAuthDto); }));
            this.requiredUsers().forEach(function (data) { return windowsAuthSetup.subscribeToObservableName(data, _this.requiredUsers, windowsAuthSetup.ModeUser); });
            this.requiredGroups().forEach(function (data) { return windowsAuthSetup.subscribeToObservableName(data, _this.requiredGroups, windowsAuthSetup.ModeGroup); });
        }
        windowsAuthSetup.prototype.toDto = function () {
            return {
                RequiredGroups: this.requiredGroups().map(function (grp) { return grp.toDto(); }),
                RequiredUsers: this.requiredUsers().map(function (usr) { return usr.toDto(); })
            };
        };
        windowsAuthSetup.subscribeToObservableName = function (data, observableArray, mode) {
            data.name.subscribe(function (previousName) {
                var existingWindowsAuthDataExceptCurrent = observableArray().filter(function (w) { return w !== data && w.name() == previousName; });
                if (existingWindowsAuthDataExceptCurrent.length == 1) {
                    existingWindowsAuthDataExceptCurrent[0].nameCustomValidity('');
                }
            }, this, "beforeChange");
            data.name.throttle(500).subscribe(function (name) {
                var errorMessage = '';
                var isApiKeyNameValid = name.indexOf("\\") > 0;
                var existingApiKeys = observableArray().filter(function (w) { return w !== data && w.name() == name; });
                if (isApiKeyNameValid == false) {
                    errorMessage = "Name must contain '\\'";
                }
                else if (existingApiKeys.length > 0) {
                    errorMessage = "Name already exists!";
                }
                data.nameCustomValidity(errorMessage);
                if (!errorMessage) {
                    data.verificationInProgress(true);
                    data.invalidName(false);
                    var queryId = ++windowsAuthSetup.lastQueryId;
                    new verifyPrincipalCommand(mode, name)
                        .execute()
                        .done(function (valid) {
                        if (windowsAuthSetup.lastQueryId === queryId) {
                            data.invalidName(!valid);
                        }
                    })
                        .always(function () {
                        if (windowsAuthSetup.lastQueryId === queryId) {
                            data.verificationInProgress(false);
                        }
                    });
                }
                else {
                    data.invalidName(false);
                }
            });
        };
        windowsAuthSetup.lastQueryId = 0; // used to discard non-latest responses, at verify-principal can take even 20 seconds :/
        windowsAuthSetup.ModeUser = "user";
        windowsAuthSetup.ModeGroup = "group";
        return windowsAuthSetup;
    })();
    return windowsAuthSetup;
});
define("models/counter/counter", ["require", "exports", "models/counter/serverValue"], function (require, exports, serverValue) {
    var counter = (function () {
        function counter(dto) {
            this.serverValues = ko.observableArray();
            this.localServerId = ko.observable();
            this.lastUpdateByServer = ko.observable();
            this.total = ko.observable(0);
            this.numOfServers = ko.observable(0);
            this.serverValues(dto.ServerValues.map(function (x) { return new serverValue(x); }));
            this.localServerId(dto.LocalServerId);
            this.lastUpdateByServer(dto.LastUpdateByServer);
            this.total(dto.Total);
            this.numOfServers(dto.NumOfServers);
        }
        return counter;
    })();
    return counter;
});
define("models/counter/counterChange", ["require", "exports"], function (require, exports) {
    var counterChange = (function () {
        function counterChange(dto, isNew) {
            var _this = this;
            if (isNew === void 0) { isNew = false; }
            this.value = ko.observable(0);
            this.counterName = ko.observable("");
            this.group = ko.observable("");
            this.delta = ko.observable();
            this.isNew = ko.observable(false);
            this.maxNameLength = 200;
            this.value(dto.CurrentValue);
            this.group(dto.Group);
            this.counterName(dto.CounterName);
            this.delta("0");
            this.isNew(isNew);
            this.counterNameCustomValidityError = ko.computed(function () {
                var counterName = _this.counterName();
                return _this.checkName(counterName, "counter name");
            });
            this.groupCustomValidityError = ko.computed(function () {
                var group = _this.group();
                return _this.checkName(group, "group name");
            });
            this.deltaCustomValidityError = ko.computed(function () {
                var delta = _this.delta();
                if (_this.isNew() === false && delta === "0")
                    return "The change must be different than 0!";
                if (_this.isNumber(delta) === false)
                    return "Please enter a valid natural number!";
                return "";
            });
        }
        counterChange.empty = function () {
            return new counterChange({
                CurrentValue: 0,
                Group: "",
                CounterName: "",
                Delta: 0
            }, true);
        };
        counterChange.prototype.checkName = function (name, fieldName) {
            var message = "";
            if (!$.trim(name)) {
                message = "An empty " + fieldName + " is forbidden for use!";
            }
            else if (name.length > this.maxNameLength) {
                message = "The  " + fieldName + " length can't exceed " + this.maxNameLength + " characters!";
            }
            return message;
        };
        counterChange.prototype.isNumber = function (num) {
            if (num < 0)
                return true;
            var n1 = Math.abs(num);
            var n2 = parseInt(num, 10);
            return !isNaN(n1) && n2 === n1 && n1.toString() === num;
        };
        counterChange.prototype.getValue = function () {
            return this.value().toLocaleString();
        };
        return counterChange;
    })();
    return counterChange;
});
define("models/counter/counterGroup", ["require", "exports", "common/pagedList", "commands/counter/getCountersCommand", "common/cssGenerator"], function (require, exports, pagedList, getCountersCommand, cssGenerator) {
    var counterGroup = (function () {
        function counterGroup(name, ownerCounterStorage, count) {
            var _this = this;
            if (count === void 0) { count = 0; }
            this.name = name;
            this.ownerCounterStorage = ownerCounterStorage;
            this.colorClass = "";
            this.countersCount = ko.observable(0);
            this.countersCountWithThousandsSeparator = ko.computed(function () { return _this.countersCount().toLocaleString(); });
            this.isAllGroupsGroup = false;
            this.countersCount(count);
            this.isAllGroupsGroup = name === counterGroup.allGroupsGroupName;
            this.colorClass = counterGroup.getGroupCssClass(name, ownerCounterStorage);
        }
        counterGroup.prototype.activate = function () {
            ko.postbox.publish("ActivateGroup", this);
        };
        counterGroup.prototype.getCounters = function () {
            if (!this.countersList) {
                this.countersList = this.createPagedList();
            }
            return this.countersList;
        };
        counterGroup.prototype.invalidateCache = function () {
            var countersList = this.getCounters();
            countersList.invalidateCache();
        };
        counterGroup.createAllGroupsCollection = function (ownerCounterStorage) {
            return new counterGroup(counterGroup.allGroupsGroupName, ownerCounterStorage);
        };
        counterGroup.fromDto = function (dto, storage) {
            return new counterGroup(dto.Name, storage, dto.Count);
        };
        counterGroup.getGroupCssClass = function (entityName, cs) {
            if (entityName === counterGroup.allGroupsGroupName) {
                return "all-documents-collection";
            }
            return cssGenerator.getCssClass(entityName, counterGroup.groupColorMaps, cs);
        };
        counterGroup.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchCounters(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        counterGroup.prototype.fetchCounters = function (skip, take) {
            var group = this.isAllGroupsGroup ? null : this.name;
            return new getCountersCommand(this.ownerCounterStorage, skip, take, group).execute();
        };
        counterGroup.allGroupsGroupName = "All Groups";
        counterGroup.groupColorMaps = [];
        return counterGroup;
    })();
    return counterGroup;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/counter/counterStorage", ["require", "exports", "models/resources/resource", "models/auth/license", "models/counter/counterStorageStatistics"], function (require, exports, resource, license, counterStorageStatistics) {
    var counterStorage = (function (_super) {
        __extends(counterStorage, _super);
        function counterStorage(name, isAdminCurrentTenant, isDisabled, bundles) {
            var _this = this;
            if (isAdminCurrentTenant === void 0) { isAdminCurrentTenant = true; }
            if (isDisabled === void 0) { isDisabled = false; }
            if (bundles === void 0) { bundles = []; }
            _super.call(this, name, TenantType.CounterStorage, isAdminCurrentTenant);
            this.statistics = ko.observable();
            this.iconName = "fa fa-fw fa-sort-numeric-desc";
            this.fullTypeName = "Counter Storage";
            this.disabled(isDisabled);
            this.activeBundles(bundles);
            this.itemCountText = ko.computed(function () { return !!_this.statistics() ? _this.statistics().counterCountText() : ""; });
            this.isLicensed = ko.computed(function () {
                if (!!license.licenseStatus() && license.licenseStatus().IsCommercial) {
                    var counterStorageValue = license.licenseStatus().Attributes.counterStorage;
                    return /^true$/i.test(counterStorageValue);
                }
                return true;
            });
        }
        counterStorage.prototype.activate = function () {
            this.isLoaded(true);
            ko.postbox.publish("ActivateCounterStorage", this);
        };
        counterStorage.prototype.saveStatistics = function (dto) {
            if (!this.statistics()) {
                this.statistics(new counterStorageStatistics());
            }
            this.statistics().fromDto(dto);
        };
        counterStorage.type = "counterstorage";
        return counterStorage;
    })(resource);
    return counterStorage;
});
define("models/counter/counterStorageReplicationDestination", ["require", "exports"], function (require, exports) {
    var counterStorageReplicationDestination = (function () {
        function counterStorageReplicationDestination(dto) {
            var _this = this;
            this.disabled = ko.observable().extend({ required: true });
            this.serverUrl = ko.observable().extend({ required: true });
            this.counterStorageName = ko.observable().extend({ required: true });
            this.username = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.domain = ko.observable().extend({ required: true });
            this.apiKey = ko.observable().extend({ required: true });
            this.name = ko.computed(function () {
                if (_this.serverUrl() && _this.counterStorageName()) {
                    return _this.counterStorageName() + " on " + _this.serverUrl();
                }
                else if (_this.serverUrl()) {
                    return _this.serverUrl();
                }
                else if (_this.counterStorageName()) {
                    return _this.counterStorageName();
                }
                return "[empty]";
            });
            this.isValid = ko.computed(function () { return _this.serverUrl() != null && _this.serverUrl().length > 0; });
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.disabled(dto.Disabled);
            this.serverUrl(dto.ServerUrl);
            this.counterStorageName(dto.CounterStorageName);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        counterStorageReplicationDestination.prototype.toggleUserCredentials = function () {
            this.isUserCredentials.toggle();
            if (this.isUserCredentials()) {
                this.isApiKeyCredentials(false);
            }
        };
        counterStorageReplicationDestination.prototype.toggleApiKeyCredentials = function () {
            this.isApiKeyCredentials.toggle();
            if (this.isApiKeyCredentials()) {
                this.isUserCredentials(false);
            }
        };
        counterStorageReplicationDestination.empty = function (counterStorageName) {
            return new counterStorageReplicationDestination({
                Disabled: false,
                ServerUrl: null,
                CounterStorageName: counterStorageName,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null
            });
        };
        counterStorageReplicationDestination.prototype.enable = function () {
            this.disabled(false);
        };
        counterStorageReplicationDestination.prototype.disable = function () {
            this.disabled(true);
        };
        counterStorageReplicationDestination.prototype.toDto = function () {
            return {
                Disabled: this.disabled(),
                ServerUrl: this.prepareUrl(),
                CounterStorageName: this.counterStorageName(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
            };
        };
        counterStorageReplicationDestination.prototype.prepareUrl = function () {
            var url = this.serverUrl();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        return counterStorageReplicationDestination;
    })();
    return counterStorageReplicationDestination;
});
define("models/counter/counterStorageReplicationSetup", ["require", "exports", "models/counter/counterStorageReplicationDestination"], function (require, exports, counterStorageReplicationDestination) {
    var counterStorageReplicationSetup = (function () {
        function counterStorageReplicationSetup(dto) {
            this.destinations = ko.observableArray().extend({ required: true });
            this.destinations(dto.Destinations.map(function (dest) { return new counterStorageReplicationDestination(dest); }));
        }
        counterStorageReplicationSetup.prototype.toDto = function () {
            return {
                Destinations: this.destinations().map(function (dest) { return dest.toDto(); })
            };
        };
        return counterStorageReplicationSetup;
    })();
    return counterStorageReplicationSetup;
});
define("models/counter/counterStorageStatistics", ["require", "exports"], function (require, exports) {
    var counterStorageStatistics = (function () {
        function counterStorageStatistics() {
            this.countersCount = ko.observable();
            this.counterCountText = ko.observable("");
            this.groupCountText = ko.observable("");
            this.requestsPerSecondText = ko.observable("");
        }
        counterStorageStatistics.prototype.getItemCountText = function (itemCount, singularText, suffix) {
            var itemCountText = itemCount.toLocaleString() + " " + singularText;
            if (itemCount !== 1) {
                itemCountText += suffix;
            }
            return itemCountText;
        };
        counterStorageStatistics.prototype.fromDto = function (dto) {
            this.countersCount(dto.CountersCount);
            this.counterCountText(this.getItemCountText(dto.CountersCount, "counter", "s"));
            this.groupCountText(this.getItemCountText(dto.GroupsCount, "group", "s"));
            this.requestsPerSecondText(dto.RequestsPerSecond + " requests per second");
        };
        return counterStorageStatistics;
    })();
    return counterStorageStatistics;
});
define("models/counter/counterSummary", ["require", "exports"], function (require, exports) {
    var counterSummary = (function () {
        function counterSummary(dto, isAllGroupsGroup) {
            if (isAllGroupsGroup === void 0) { isAllGroupsGroup = false; }
            this.isAllGroupsGroup = isAllGroupsGroup;
            this["Counter Name"] = dto.CounterName;
            this["Group Name"] = dto.GroupName;
            this.Total = dto.Total;
        }
        counterSummary.prototype.getEntityName = function () {
            return this["Group Name"];
        };
        counterSummary.prototype.getDocumentPropertyNames = function () {
            var properties = ["Counter Name"];
            if (this.isAllGroupsGroup)
                properties.push("Group Name");
            properties.push("Total");
            return properties;
        };
        counterSummary.prototype.getId = function () {
            return this["Counter Name"];
        };
        counterSummary.prototype.getUrl = function () {
            return this.getId();
        };
        return counterSummary;
    })();
    return counterSummary;
});
define("models/counter/serverValue", ["require", "exports"], function (require, exports) {
    var serverValue = (function () {
        function serverValue(dto) {
            this.serverId = ko.observable();
            this.value = ko.observable(0);
            this.etag = ko.observable(0);
            this.serverId(dto.ServerId);
            this.value(dto.Value);
            this.etag(dto.Etag);
        }
        return serverValue;
    })();
    return serverValue;
});
define("models/database/cluster/clusterConfiguration", ["require", "exports"], function (require, exports) {
    var clusterConfiguration = (function () {
        function clusterConfiguration(dto) {
            this.enableReplication = ko.observable();
            this.enableReplication(dto.EnableReplication);
        }
        clusterConfiguration.prototype.toDto = function () {
            return {
                EnableReplication: this.enableReplication()
            };
        };
        clusterConfiguration.empty = function () {
            return new clusterConfiguration({
                EnableReplication: false
            });
        };
        return clusterConfiguration;
    })();
    return clusterConfiguration;
});
define("models/database/cluster/nodeConnectionInfo", ["require", "exports"], function (require, exports) {
    var nodeConnectionInfo = (function () {
        function nodeConnectionInfo(dto) {
            var _this = this;
            this.uri = ko.observable();
            this.name = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            this.state = ko.observable(); // used to store owning collection name (voting, non-voting, promotable)
            this.status = ko.observable("Loading");
            this.isNonVoting = ko.observable();
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.uri(dto.Uri);
            this.name(dto.Name);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.isNonVoting(dto.IsNoneVoter);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        nodeConnectionInfo.prototype.toDto = function () {
            return {
                Uri: this.uri(),
                Name: this.name(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                IsNoneVoter: this.isNonVoting()
            };
        };
        nodeConnectionInfo.empty = function () {
            return new nodeConnectionInfo({
                Uri: null,
                Name: null,
                IsNoneVoter: false
            });
        };
        nodeConnectionInfo.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        nodeConnectionInfo.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        nodeConnectionInfo.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
            this.clearUserCredentials();
        };
        nodeConnectionInfo.prototype.clearApiKeyCredentials = function () {
            this.apiKey(null);
        };
        nodeConnectionInfo.prototype.clearUserCredentials = function () {
            this.username(null);
            this.password(null);
            this.domain(null);
        };
        return nodeConnectionInfo;
    })();
    return nodeConnectionInfo;
});
define("models/database/cluster/serverConnectionInfo", ["require", "exports"], function (require, exports) {
    var serverConnectionInfo = (function () {
        function serverConnectionInfo() {
            var _this = this;
            this.url = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
        }
        serverConnectionInfo.prototype.guessCredentialsType = function () {
            if (this.apiKey()) {
                this.useApiKeyCredentials();
            }
            else if (this.username()) {
                this.useUserCredentials();
            }
        };
        serverConnectionInfo.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        serverConnectionInfo.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        serverConnectionInfo.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
            this.clearUserCredentials();
        };
        serverConnectionInfo.prototype.clearApiKeyCredentials = function () {
            this.apiKey(undefined);
        };
        serverConnectionInfo.prototype.clearUserCredentials = function () {
            this.username(undefined);
            this.password(undefined);
            this.domain(undefined);
        };
        serverConnectionInfo.prototype.toDto = function () {
            return {
                Url: this.url(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey()
            };
        };
        return serverConnectionInfo;
    })();
    return serverConnectionInfo;
});
define("models/database/debug/adminLogsConfig", ["require", "exports"], function (require, exports) {
    var adminLogsConfig = (function () {
        function adminLogsConfig() {
            this.entries = ko.observableArray();
            this.maxEntries = ko.observable();
            this.singleAuthToken = ko.observable();
        }
        adminLogsConfig.prototype.clone = function () {
            var newConfig = new adminLogsConfig();
            newConfig.maxEntries(this.maxEntries());
            newConfig.entries($.map(this.entries() || [], function (e, idx) { return e.clone(); }));
            return newConfig;
        };
        return adminLogsConfig;
    })();
    return adminLogsConfig;
});
define("models/database/debug/adminLogsConfigEntry", ["require", "exports"], function (require, exports) {
    var adminLogsConfigEntry = (function () {
        function adminLogsConfigEntry(category, level, includeStackTrace) {
            if (includeStackTrace === void 0) { includeStackTrace = false; }
            this.includeStackTrace = ko.observable();
            this.category = ko.observable();
            this.level = ko.observable();
            this.category(category);
            this.level(level);
            this.includeStackTrace(includeStackTrace);
        }
        adminLogsConfigEntry.prototype.clone = function () {
            return new adminLogsConfigEntry(this.category(), this.level(), this.includeStackTrace());
        };
        adminLogsConfigEntry.empty = function () {
            return new adminLogsConfigEntry(null, null, false);
        };
        adminLogsConfigEntry.prototype.toDto = function () {
            return {
                category: this.category(),
                level: this.level(),
                includeStackTrace: this.includeStackTrace()
            };
        };
        return adminLogsConfigEntry;
    })();
    return adminLogsConfigEntry;
});
define("models/database/debug/alert", ["require", "exports"], function (require, exports) {
    var alert = (function () {
        function alert(dto) {
            this.observed = ko.observable(false);
            this.alertLevel = dto.AlertLevel;
            this.createdAt = dto.CreatedAt;
            this.exception = dto.Exception;
            this.message = dto.Message;
            this.observed(dto.Observed);
            this.lastDismissedAt = dto.LastDismissedAt;
            this.title = dto.Title;
            this.uniqueKey = dto.UniqueKey;
        }
        alert.prototype.toDto = function () {
            return {
                AlertLevel: this.alertLevel,
                CreatedAt: this.createdAt,
                Exception: this.exception,
                Message: this.message,
                Observed: this.observed(),
                LastDismissedAt: this.lastDismissedAt,
                Title: this.title,
                UniqueKey: this.uniqueKey
            };
        };
        return alert;
    })();
    return alert;
});
define("models/database/debug/debugDocumentStats", ["require", "exports"], function (require, exports) {
    var debugDocumentStats = (function () {
        function debugDocumentStats(dto) {
            this.total = dto.Total;
            this.totalSize = dto.TotalSize;
            this.tombstones = dto.Tombstones;
            this.system = dto.System;
            this.noCollection = dto.NoCollection;
            this.collections = $.map(dto.Collections, function (data, name) { return { "name": name, "stats": data.Stats, "size": data.TotalSize, "topDocs": data.TopDocs }; });
            this.timeToGenerate = dto.TimeToGenerate;
        }
        return debugDocumentStats;
    })();
    return debugDocumentStats;
});
define("models/database/debug/logEntry", ["require", "exports"], function (require, exports) {
    var logEntry = (function () {
        function logEntry(dto, nowProvider) {
            var _this = this;
            this.nowProvider = nowProvider;
            this.level = ko.observable();
            this.timeStamp = ko.observable();
            this.message = ko.observable();
            this.exception = ko.observable();
            this.loggerName = ko.observable();
            this.database = ko.observable();
            this.stackTrace = ko.observable();
            this.dateMoment = ko.observable();
            this.dateRaw = ko.observable();
            this.level(dto.Level);
            this.timeStamp(dto.TimeStamp);
            this.message(dto.Message);
            this.exception(dto.Exception);
            this.loggerName(dto.LoggerName);
            this.database(dto.Database);
            this.stackTrace(dto.StackTrace);
            // since we know date is in ISO format parse it to date and then pass to moment - it will skip format detection
            this.dateMoment(moment(new Date(this.timeStamp())));
            this.dateRaw(this.dateMoment().toDate().getTime());
            this.humanizedTimestamp = ko.pureComputed(function () {
                var dateMoment = _this.dateMoment();
                var agoInMs = dateMoment.diff(_this.nowProvider());
                var humanized = moment.duration(agoInMs).humanize(true);
                return humanized;
            });
            this.timeStampText = ko.pureComputed(function () {
                var dateMoment = _this.dateMoment();
                return dateMoment.format(" (MM/DD/YY, h:mma)");
            });
        }
        logEntry.prototype.copyFrom = function (newEntry) {
            this.level(newEntry.level());
            this.timeStamp(newEntry.timeStamp());
            this.message(newEntry.message());
            this.exception(newEntry.exception());
            this.loggerName(newEntry.loggerName());
            this.database(newEntry.database());
            this.stackTrace(newEntry.stackTrace());
            this.dateMoment(moment(new Date(newEntry.timeStamp())));
            this.dateRaw(this.dateMoment().toDate().getTime());
        };
        return logEntry;
    })();
    return logEntry;
});
define("models/database/debug/performanceTestRequest", ["require", "exports"], function (require, exports) {
    var performanceTestRequest = (function () {
        function performanceTestRequest(dto) {
            this.path = ko.observable();
            this.fileSize = ko.observable();
            this.testType = ko.observable("generic");
            // generic type specific options:
            this.operationType = ko.observable();
            this.bufferingType = ko.observable();
            this.sequential = ko.observable();
            this.threadCount = ko.observable();
            this.timeToRunInSeconds = ko.observable();
            //random seed - we don't expose on UI
            this.chunkSize = ko.observable();
            // batch type specific options:
            this.numberOfDocuments = ko.observable();
            this.sizeOfDocuments = ko.observable();
            this.numberOfDocumentsInBatch = ko.observable();
            this.waitBetweenBatches = ko.observable();
            this.path(dto.Path);
            this.fileSize(dto.FileSize);
            this.testType(dto.TestType);
            this.operationType(dto.OperationType);
            this.bufferingType(dto.BufferingType);
            this.sequential(dto.Sequential);
            this.threadCount(dto.ThreadCount);
            this.timeToRunInSeconds(dto.TimeToRunInSeconds);
            this.chunkSize(dto.ChunkSize);
            this.numberOfDocuments(dto.NumberOfDocuments);
            this.sizeOfDocuments(dto.SizeOfDocuments);
            this.numberOfDocumentsInBatch(dto.NumberOfDocumentsInBatch);
            this.waitBetweenBatches(dto.WaitBetweenBatches);
        }
        performanceTestRequest.empty = function () {
            return new performanceTestRequest({
                TestType: "generic",
                Path: "c:\\temp\\",
                FileSize: 1024 * 1024 * 1024,
                OperationType: "Write",
                BufferingType: "None",
                Sequential: true,
                ThreadCount: 4,
                TimeToRunInSeconds: 30,
                ChunkSize: 4 * 1024,
                NumberOfDocuments: 300000,
                SizeOfDocuments: 8 * 1024,
                NumberOfDocumentsInBatch: 200,
                WaitBetweenBatches: 0
            });
        };
        performanceTestRequest.prototype.toDto = function () {
            if (this.testType() == "generic") {
                return {
                    TestType: this.testType(),
                    Path: this.path(),
                    FileSize: this.fileSize(),
                    OperationType: this.operationType(),
                    BufferingType: this.bufferingType(),
                    Sequential: this.sequential(),
                    ThreadCount: this.threadCount(),
                    TimeToRunInSeconds: this.timeToRunInSeconds(),
                    ChunkSize: this.chunkSize()
                };
            }
            else {
                return {
                    TestType: this.testType(),
                    Path: this.path(),
                    FileSize: this.fileSize(),
                    NumberOfDocuments: this.numberOfDocuments(),
                    SizeOfDocuments: this.sizeOfDocuments(),
                    NumberOfDocumentsInBatch: this.numberOfDocumentsInBatch(),
                    WaitBetweenBatches: this.waitBetweenBatches()
                };
            }
        };
        return performanceTestRequest;
    })();
    return performanceTestRequest;
});
define("models/database/debug/performanceTestResultWrapped", ["require", "exports", "d3/d3"], function (require, exports, d3) {
    var performanceTestResultWrapped = (function () {
        function performanceTestResultWrapped(dto) {
            var _this = this;
            this.formatter = d3.format(",.2f");
            this.totalRead = ko.computed(function () { return _this.formatter(dto.Result.TotalRead / 1024 / 1024) + " MB"; });
            this.totalWrite = ko.computed(function () { return _this.formatter(dto.Result.TotalWrite / 1024 / 1024) + " MB"; });
            this.hasReads = ko.computed(function () { return dto.Request.TestType == "generic" && (dto.Request.OperationType == "Read" || dto.Request.OperationType == "Mix"); });
            this.hasWrites = ko.computed(function () { return dto.Request.TestType == "batch" || dto.Request.OperationType == "Write" || dto.Request.OperationType == "Mix"; });
            this.testTime = ko.computed(function () { return _this.formatter(dto.Result.TotalTimeMs / 1000) + "s"; });
            this.avgRead = ko.computed(function () { return _this.formatter(dto.Result.TotalRead / 1024 / 1024 / (dto.Result.TotalTimeMs / 1000)) + "MB/s"; });
            this.avgWrite = ko.computed(function () { return _this.formatter(dto.Result.TotalWrite / 1024 / 1024 / (dto.Result.TotalTimeMs / 1000)) + "MB/s"; });
            this.readLatencyMin = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Min) + "ms"; });
            this.readLatencyMax = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Max) + "ms"; });
            this.readLatencyMean = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Mean) + "ms"; });
            this.readLatencyStdev = ko.computed(function () { return _this.formatter(dto.Result.ReadLatency.Stdev) + "ms"; });
            this.writeLatencyMin = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Min) + "ms"; });
            this.writeLatencyMax = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Max) + "ms"; });
            this.writeLatencyMean = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Mean) + "ms"; });
            this.writeLatencyStdev = ko.computed(function () { return _this.formatter(dto.Result.WriteLatency.Stdev) + "ms"; });
            this.readPercentiles = ko.computed(function () { return $.map(dto.Result.ReadLatency.Percentiles, function (value, key) {
                return {
                    key: key,
                    value: _this.formatter(value)
                };
            }); });
            this.writePercentiles = ko.computed(function () { return $.map(dto.Result.WriteLatency.Percentiles, function (value, key) {
                return {
                    key: key,
                    value: _this.formatter(value)
                };
            }); });
        }
        return performanceTestResultWrapped;
    })();
    return performanceTestResultWrapped;
});
define("models/database/debug/runningTask", ["require", "exports"], function (require, exports) {
    var runningTask = (function () {
        function runningTask(dto) {
            this.id = dto.Id;
            this.status = dto.Status;
            this.taskType = dto.TaskType;
            this.description = dto.Description;
            this.completed = dto.Completed;
            this.faulted = dto.Faulted;
            this.canceled = dto.Canceled;
            this.fullStatus = runningTask.createFullStatus(dto);
            this.startTime = dto.StartTime; // used for sorting
            this.timeStampText = this.createHumanReadableTime(dto.StartTime);
            this.exceptionText = dto.Exception ? JSON.stringify(dto.Exception, null, 2) : null;
            this.killable = dto.Killable;
        }
        runningTask.createFullStatus = function (dto) {
            var taskState = "Running";
            if (dto.Canceled) {
                taskState = "Canceled";
            }
            else if (dto.Faulted) {
                taskState = "Faulted";
            }
            else if (dto.Completed) {
                taskState = "Completed";
            }
            var status = null;
            if (dto.Status) {
                status = dto.Faulted || dto.Canceled ? dto.Status.Error : dto.Status.Progress;
            }
            return status ? taskState + ": " + status : taskState;
        };
        runningTask.prototype.createHumanReadableTime = function (time) {
            var now = moment();
            if (time) {
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(now);
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        return runningTask;
    })();
    return runningTask;
});
define("models/database/debug/statusDebugQueriesGroup", ["require", "exports", "models/database/debug/statusDebugQueriesQuery"], function (require, exports, statusDebugQueriesQuery) {
    var statusDebugQueriesGroup = (function () {
        function statusDebugQueriesGroup(dto) {
            this.queries = ko.observableArray();
            this.indexName = dto.IndexName;
            this.queries($.map(dto.Queries, function (q) { return new statusDebugQueriesQuery(q); }));
        }
        return statusDebugQueriesGroup;
    })();
    return statusDebugQueriesGroup;
});
define("models/database/debug/statusDebugQueriesQuery", ["require", "exports"], function (require, exports) {
    var statusDebugQueriesQuery = (function () {
        function statusDebugQueriesQuery(dto) {
            this.queryInfo = ko.observable();
            this.duration = ko.observable(null);
            this.startTime = dto.StartTime;
            this.queryInfo(JSON.stringify(dto.QueryInfo, null, 4));
            this.queryId = dto.QueryId;
            this.duration(dto.Duration);
        }
        return statusDebugQueriesQuery;
    })();
    return statusDebugQueriesQuery;
});
define("models/database/documents/collection", ["require", "exports", "common/pagedList", "commands/database/documents/getDocumentsPreviewCommand", "commands/database/documents/getSystemDocumentsCommand", "common/cssGenerator"], function (require, exports, pagedList, getDocumentsPreviewCommand, getSystemDocumentsCommand, cssGenerator) {
    var collection = (function () {
        function collection(name, ownerDatabase, docCount) {
            var _this = this;
            if (docCount === void 0) { docCount = 0; }
            this.name = name;
            this.ownerDatabase = ownerDatabase;
            this.colorClass = "";
            this.documentCount = ko.observable(0);
            this.documentsCountWithThousandsSeparator = ko.computed(function () { return _this.documentCount().toLocaleString(); });
            this.isAllDocuments = false;
            this.isSystemDocuments = false;
            this.bindings = ko.observable();
            this.collectionName = name;
            this.isAllDocuments = name === collection.allDocsCollectionName;
            this.isSystemDocuments = name === collection.systemDocsCollectionName;
            this.colorClass = collection.getCollectionCssClass(name, ownerDatabase);
            this.documentCount(docCount);
        }
        // Notifies consumers that this collection should be the selected one.
        // Called from the UI when a user clicks a collection the documents page.
        collection.prototype.activate = function () {
            ko.postbox.publish("ActivateCollection", this);
        };
        collection.prototype.prettyLabel = function (text) {
            return text.replace(/__/g, '/');
        };
        collection.prototype.getDocuments = function () {
            if (!this.documentsList) {
                this.documentsList = this.createPagedList();
            }
            return this.documentsList;
        };
        collection.prototype.invalidateCache = function () {
            var documentsList = this.getDocuments();
            documentsList.invalidateCache();
        };
        collection.prototype.clearCollection = function () {
            if (this.isAllDocuments && !!this.documentsList) {
                this.documentsList.clear();
            }
        };
        collection.prototype.fetchDocuments = function (skip, take) {
            if (this.isSystemDocuments) {
                // System documents don't follow the normal paging rules. See getSystemDocumentsCommand.execute() for more info.
                return new getSystemDocumentsCommand(this.ownerDatabase, skip, take, this.documentCount()).execute();
            }
            if (this.isAllDocuments) {
                return new getDocumentsPreviewCommand(this.ownerDatabase, skip, take, undefined, this.bindings()).execute();
            }
            else {
                return new getDocumentsPreviewCommand(this.ownerDatabase, skip, take, this.name, this.bindings()).execute();
            }
        };
        collection.createSystemDocsCollection = function (ownerDatabase) {
            return new collection(collection.systemDocsCollectionName, ownerDatabase);
        };
        collection.createAllDocsCollection = function (ownerDatabase) {
            return new collection(collection.allDocsCollectionName, ownerDatabase);
        };
        collection.getCollectionCssClass = function (entityName, db) {
            if (entityName === collection.allDocsCollectionName) {
                return "all-documents-collection";
            }
            if (!entityName || entityName === collection.systemDocsCollectionName) {
                return "system-documents-collection";
            }
            return cssGenerator.getCssClass(entityName, collection.collectionColorMaps, db);
        };
        collection.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchDocuments(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        collection.allDocsCollectionName = "All Documents";
        collection.systemDocsCollectionName = "System Documents";
        collection.collectionColorMaps = [];
        return collection;
    })();
    return collection;
});
define("models/database/documents/collectionInfo", ["require", "exports", "models/database/documents/document"], function (require, exports, document) {
    var collectionInfo = (function () {
        function collectionInfo(dto) {
            this.results = dto.Results.map(function (d) { return new document(d); });
            this.totalResults = dto.TotalResults;
        }
        return collectionInfo;
    })();
    return collectionInfo;
});
define("models/database/documents/customColumnParams", ["require", "exports", "common/execJs"], function (require, exports, execJs) {
    var customColumnParams = (function () {
        function customColumnParams(dto) {
            var _this = this;
            this.header = ko.observable();
            this.binding = ko.observable();
            this.template = ko.observable();
            this.width = ko.observable();
            this.bindingCustomValidity = ko.computed(function () {
                var binding = _this.binding();
                return execJs.validateSyntax(binding) || "";
            });
            this.header(dto.Header);
            this.binding(dto.Binding);
            this.template(dto.Template || "defaultTemplate");
            this.width(dto.DefaultWidth);
        }
        customColumnParams.empty = function () {
            return new customColumnParams({ Header: '', Binding: '', DefaultWidth: 200 });
        };
        customColumnParams.prototype.toDto = function () {
            return {
                'Header': this.header(),
                'Binding': this.binding(),
                'DefaultWidth': this.width(),
            };
        };
        return customColumnParams;
    })();
    return customColumnParams;
});
define("models/database/documents/customColumns", ["require", "exports", "models/database/documents/customColumnParams"], function (require, exports, customColumnParams) {
    var customColumns = (function () {
        function customColumns(dto) {
            this.columns = ko.observableArray();
            this.customMode = ko.observable(false);
            this.enabled = ko.observable(true);
            this.columns($.map(dto.Columns, function (c) { return new customColumnParams(c); }));
        }
        customColumns.empty = function () {
            return new customColumns({ Columns: [] });
        };
        customColumns.prototype.copyFrom = function (src) {
            this.columns(src.columns());
            this.customMode(src.customMode());
        };
        customColumns.prototype.clone = function () {
            var copy = new customColumns(this.toDto());
            copy.customMode(this.customMode());
            return copy;
        };
        customColumns.prototype.getBindings = function () {
            if (!this.hasOverrides()) {
                return undefined;
            }
            return this.columns().map(function (x) { return x.binding(); });
        };
        customColumns.prototype.hasOverrides = function () {
            return this.enabled() && this.customMode() && this.columns().length > 0;
        };
        customColumns.prototype.toDto = function () {
            return {
                'Columns': $.map(this.columns(), function (c) { return c.toDto(); })
            };
        };
        customColumns.prototype.findConfigFor = function (binding) {
            var colParams = this.columns();
            for (var i = 0; i < colParams.length; i++) {
                var colParam = colParams[i];
                if (colParam.binding() === binding) {
                    return colParam;
                }
            }
            return null;
        };
        return customColumns;
    })();
    return customColumns;
});
define("models/database/documents/customFunctions", ["require", "exports", "models/database/documents/documentMetadata"], function (require, exports, documentMetadata) {
    var customFunctions = (function () {
        function customFunctions(dto) {
            if (!dto) {
                dto = {
                    Functions: ""
                };
                this.__metadata = new documentMetadata();
            }
            else {
                this.__metadata = new documentMetadata(dto["@metadata"]);
            }
            this.functions = dto.Functions;
        }
        customFunctions.prototype.toDto = function (includeMetadata) {
            var dto = {
                Functions: this.functions
            };
            if (includeMetadata && this.__metadata) {
                dto['@metadata'] = this.__metadata.toDto();
            }
            return dto;
        };
        customFunctions.prototype.clone = function () {
            var copy = new customFunctions(this.toDto());
            copy.functions = this.functions;
            return copy;
        };
        customFunctions.empty = function () {
            return new customFunctions({
                Functions: ""
            });
        };
        return customFunctions;
    })();
    return customFunctions;
});
define("models/database/documents/document", ["require", "exports", "models/database/documents/documentMetadata"], function (require, exports, documentMetadata) {
    var document = (function () {
        function document(dto) {
            this.__metadata = new documentMetadata(dto["@metadata"]);
            for (var property in dto) {
                if (property !== "@metadata") {
                    this[property] = dto[property];
                }
            }
        }
        document.prototype.getEntityName = function () {
            return this.__metadata.ravenEntityName;
        };
        document.prototype.getId = function () {
            return this.__metadata.id;
        };
        document.prototype.getUrl = function () {
            return this.getId();
        };
        document.prototype.getDocumentPropertyNames = function () {
            var propertyNames = [];
            for (var property in this) {
                var isMeta = property === "__metadata" || property === "__moduleId__";
                var isFunction = typeof this[property] === "function";
                if (!isMeta && !isFunction) {
                    propertyNames.push(property);
                }
            }
            return propertyNames;
        };
        document.prototype.toDto = function (includeMeta) {
            if (includeMeta === void 0) { includeMeta = false; }
            var dto = { '@metadata': undefined };
            var properties = this.getDocumentPropertyNames();
            for (var i = 0; i < properties.length; i++) {
                var property = properties[i];
                dto[property] = this[property];
            }
            if (includeMeta && this.__metadata) {
                var newDocumentMetadata = new documentMetadata(this.__metadata);
                var metadataDto = newDocumentMetadata.toDto();
                dto["@metadata"] = metadataDto;
            }
            return dto;
        };
        document.prototype.toBulkDoc = function (method) {
            var dto = this.toDto();
            var bulkDoc = {
                Document: dto,
                Key: this.getId(),
                Method: method,
                AdditionalData: null
            };
            var meta = dto["@metadata"];
            if (meta) {
                bulkDoc.Metadata = meta;
                if (meta["@etag"]) {
                    bulkDoc.Etag = meta["@etag"];
                }
            }
            return bulkDoc;
        };
        document.empty = function () {
            var emptyDto = {
                '@metadata': {}
            };
            return new document(emptyDto);
        };
        document.getEntityNameFromId = function (id) {
            if (!id) {
                return null;
            }
            // TODO: is there a better/more reliable way to do this?
            var slashIndex = id.lastIndexOf("/");
            if (slashIndex >= 1) {
                return id.substring(0, 1).toUpperCase() + id.substring(1, slashIndex);
            }
            return id;
        };
        return document;
    })();
    return document;
});
/// <reference path="../../../models/dto.ts" />
define("models/database/documents/documentMetadata", ["require", "exports"], function (require, exports) {
    var documentMetadata = (function () {
        function documentMetadata(dto) {
            var _this = this;
            this.now = ko.observable(new Date());
            if (dto) {
                this.ravenEntityName = dto['Raven-Entity-Name'];
                this.ravenClrType = dto['Raven-Clr-Type'];
                this.nonAuthoritativeInfo = dto['Non-Authoritative-Information'];
                this.id = dto['@id'];
                this.tempIndexScore = dto['Temp-Index-Score'];
                this.lastModified = dto['Last-Modified'];
                this.lastModifiedFullDate = ko.computed(function () {
                    if (!!_this.lastModified) {
                        var lastModifiedMoment = moment(_this.lastModified);
                        var timeSince = lastModifiedMoment.from(_this.now());
                        var fullTimeSinceUtc = lastModifiedMoment.utc().format("DD/MM/YYYY HH:mm (UTC)");
                        return timeSince + " (" + fullTimeSinceUtc + ")";
                    }
                    return "";
                });
                setInterval(function () { return _this.now(new Date()); }, 60 * 1000);
                this.ravenLastModified = dto['Raven-Last-Modified'];
                this.etag = dto['@etag'];
                for (var property in dto) {
                    if (property.toUpperCase() !== 'Raven-Entity-Name'.toUpperCase() &&
                        property.toUpperCase() !== 'Raven-Clr-Type'.toUpperCase() &&
                        property.toUpperCase() !== 'Non-Authoritative-Information'.toUpperCase() &&
                        property.toUpperCase() !== '@id'.toUpperCase() &&
                        property.toUpperCase() !== 'Temp-Index-Score'.toUpperCase() &&
                        property.toUpperCase() !== 'Last-Modified'.toUpperCase() &&
                        property.toUpperCase() !== 'Raven-Last-Modified'.toUpperCase() &&
                        property.toUpperCase() !== '@etag'.toUpperCase() &&
                        property.toUpperCase() !== 'toDto'.toUpperCase()) {
                        this.nonStandardProps = this.nonStandardProps || [];
                        this[property] = dto[property];
                        this.nonStandardProps.push(property);
                    }
                }
            }
        }
        documentMetadata.prototype.toDto = function () {
            var _this = this;
            var dto = {
                'Raven-Entity-Name': this.ravenEntityName,
                'Raven-Clr-Type': this.ravenClrType,
                'Non-Authoritative-Information': this.nonAuthoritativeInfo,
                '@id': this.id,
                'Temp-Index-Score': this.tempIndexScore,
                'Last-Modified': this.lastModified,
                'Raven-Last-Modified': this.ravenLastModified,
                '@etag': this.etag
            };
            if (this.nonStandardProps) {
                this.nonStandardProps.forEach(function (p) { return dto[p] = _this[p]; });
            }
            return dto;
        };
        documentMetadata.filterMetadata = function (metaDto, removedProps) {
            if (removedProps === void 0) { removedProps = null; }
            // We don't want to show certain reserved properties in the metadata text area.
            // Remove them from the DTO, restore them on save.
            var metaPropsToRemove = ["@id", "@etag", "Origin", "Raven-Server-Build", "Raven-Client-Version", "Non-Authoritative-Information", "Raven-Timer-Request",
                "Raven-Authenticated-User", "Raven-Last-Modified", "Has-Api-Key", "Access-Control-Allow-Origin", "Access-Control-Max-Age", "Access-Control-Allow-Methods",
                "Access-Control-Request-Headers", "Access-Control-Allow-Headers", "Reverse-Via", "Persistent-Auth", "Allow", "Content-Disposition", "Content-Encoding",
                "Content-Language", "Content-Location", "Content-MD5", "Content-Range", "Content-Type", "Expires", "Last-Modified", "Content-Length", "Keep-Alive", "X-Powered-By",
                "X-AspNet-Version", "X-Requested-With", "X-SourceFiles", "Accept-Charset", "Accept-Encoding", "Accept", "Accept-Language", "Authorization", "Cookie", "Expect",
                "From", "Host", "If-MatTemp-Index-Scorech", "If-Modified-Since", "If-None-Match", "If-Range", "If-Unmodified-Since", "Max-Forwards", "Referer", "TE", "User-Agent", "Accept-Ranges",
                "Age", "Allow", "ETag", "Location", "Retry-After", "Server", "Set-Cookie2", "Set-Cookie", "Vary", "Www-Authenticate", "Cache-Control", "Connection", "Date", "Pragma",
                "Trailer", "Transfer-Encoding", "Upgrade", "Via", "Warning", "X-ARR-LOG-ID", "X-ARR-SSL", "X-Forwarded-For", "X-Original-URL", "Size-In-Kb"];
            for (var property in metaDto) {
                if (metaDto.hasOwnProperty(property) && metaPropsToRemove.contains(property)) {
                    if (metaDto[property] && removedProps) {
                        removedProps.push({ name: property, value: metaDto[property].toString() });
                    }
                    delete metaDto[property];
                }
            }
            return metaDto;
        };
        return documentMetadata;
    })();
    return documentMetadata;
});
define("models/database/documents/periodicExportSetup", ["require", "exports"], function (require, exports) {
    var periodicExportSetup = (function () {
        function periodicExportSetup() {
            var _this = this;
            this.onDiskExportEnabled = ko.observable(false);
            this.remoteUploadEnabled = ko.observable(false);
            this.localFolderName = ko.observable();
            this.unsupported = ko.observable(false);
            this.disabled = ko.observable(true);
            this.type = ko.observable();
            this.mainValue = ko.observable();
            this.azureRemoteFolderName = ko.observable();
            this.s3RemoteFolderName = ko.observable();
            this.awsAccessKey = ko.observable();
            this.awsSecretKey = ko.observable();
            this.awsRegionEndpoint = ko.observable();
            this.awsSecretKeyDecryptionFailed = ko.observable(false);
            this.azureStorageKeyDecryptionFailed = ko.observable(false);
            this.azureStorageAccount = ko.observable();
            this.azureStorageKey = ko.observable();
            this.incrementalBackupInterval = ko.observable();
            this.incrementalBackupIntervalUnit = ko.observable();
            this.fullBackupInterval = ko.observable();
            this.fullBackupIntervalUnit = ko.observable();
            this.FILE_SYSTEM = "fileSystem";
            this.GLACIER_VAULT = "glacierVault";
            this.S3_BUCKET = "s3bucket";
            this.AZURE_STORAGE = "azureStorage";
            this.TU_MINUTES = "minutes";
            this.TU_HOURS = "hours";
            this.TU_DAYS = "days";
            this.availablePeriodicExports = [
                { label: "Glacier Vault Name:", value: this.GLACIER_VAULT },
                { label: "S3 Bucket Name:", value: this.S3_BUCKET },
                { label: "Azure Storage Container:", value: this.AZURE_STORAGE }
            ];
            this.availableAwsRegionEndpoints = [
                { label: "US East (Virginia)", value: "us-east-1" },
                { label: "US West (N. California)", value: "us-west-1" },
                { label: "US West (Oregon)", value: "us-west-2" },
                { label: "EU West (Ireland)", value: "eu-west-1" },
                { label: "Asia Pacific (Tokyo)", value: "ap-northeast-1" },
                { label: "Asia Pacific (Singapore)", value: "ap-southeast-1" },
                { label: "South America (Sao Paulo)", value: "sa-east-1" }
            ];
            this.availableIntervalUnits = [this.TU_MINUTES, this.TU_HOURS, this.TU_DAYS];
            this.mainPlaceholder = ko.computed(function () {
                switch (_this.type()) {
                    case _this.GLACIER_VAULT:
                        return "vault name only e.g. myvault";
                    case _this.S3_BUCKET:
                        return "bucket name only e.g. mybucket";
                    case _this.AZURE_STORAGE:
                        return "container name only e.g. mycontainer";
                }
            }, this);
            this.additionalAwsInfoRequired = ko.computed(function () {
                var type = _this.type();
                return _this.remoteUploadEnabled() && jQuery.inArray(type, [_this.GLACIER_VAULT, _this.S3_BUCKET]) !== -1;
            }, this);
            this.isGlaceirVault = ko.computed(function () { return _this.remoteUploadEnabled() && _this.type() === _this.GLACIER_VAULT; });
            this.isS3Bucket = ko.computed(function () { return _this.remoteUploadEnabled() && _this.type() === _this.S3_BUCKET; });
            this.additionalAzureInfoRequired = ko.computed(function () {
                var type = _this.type();
                return _this.remoteUploadEnabled() && type === _this.AZURE_STORAGE;
            }, this);
            this.mainValueCustomValidity = ko.computed(function () {
                var mainValue = _this.mainValue();
                switch (_this.type()) {
                    case _this.GLACIER_VAULT:
                        return _this.validateGlacierVaultName(mainValue);
                    case _this.S3_BUCKET:
                        return _this.validateS3Bucket(mainValue);
                    case _this.AZURE_STORAGE:
                        return _this.validateAzureContainerNAme(mainValue);
                }
                return "";
            });
        }
        /*
        Names can be between 1 and 255 characters long.
    
        Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), and '.' (period).
        */
        periodicExportSetup.prototype.validateGlacierVaultName = function (vaultName) {
            if (vaultName == null || vaultName.length < 1 || vaultName.length > 255) {
                return "Vault name must be at least 1 and no more than 255 characters long.";
            }
            var regEx = /^[A-Za-z0-9_\.-]+$/;
            if (!regEx.test(vaultName)) {
                return "Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), and '.' (period).";
            }
            return "";
        };
        /*
        Bucket names must :
            - be at least 3 and no more than 63 characters long.
            - be a series of one or more labels.
                Adjacent labels are separated by a single period (.).
                Bucket names can contain lowercase letters, numbers, and hyphens.
                Each label must start and end with a lowercase letter or a number.
            - not be formatted as an IP address (e.g., 192.168.5.4).
        */
        periodicExportSetup.prototype.validateS3Bucket = function (bucketName) {
            if (bucketName == null || bucketName.length < 3 || bucketName.length > 63) {
                return "Bucket name must be at least 3 and no more than 63 characters long";
            }
            if (bucketName[0] == '.') {
                return 'Bucket name cannot start with a period (.)';
            }
            if (bucketName[bucketName.length - 1] == '.') {
                return 'Bucket name cannot end with a period (.)';
            }
            if (bucketName.indexOf("..") > -1) {
                return 'There can be only one period between labels';
            }
            var labels = bucketName.split(".");
            var labelRegExp = /^[a-z0-9-]+$/;
            var validLabel = function (label) {
                if (label == null || label.length == 0) {
                    return false;
                }
                if (labelRegExp.test(label) == false) {
                    return false;
                }
                if (label[0] == "-" || label[label.length - 1] == "-") {
                    return false;
                }
                return true;
            };
            if (labels.some(function (l) { return !validLabel(l); })) {
                return "Bucket name is invalid";
            }
            var ipRegExp = /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/;
            if (ipRegExp.test(bucketName)) {
                return "Bucket name must not be formatted as an IP address (e.g., 192.168.5.4).";
            }
            return "";
        };
        /*
        Container names must:
            - start with a letter or number, and can contain only letters, numbers, and the dash (-) character.
            - every dash (-) character must be immediately preceded and followed by a letter or number;
            - consecutive dashes are not permitted in container names.
            - all letters in a container name must be lowercase.
            - container names must be from 3 through 63 characters long.
        */
        periodicExportSetup.prototype.validateAzureContainerNAme = function (containerName) {
            if (containerName == null || containerName.length < 3 || containerName.length > 63) {
                return "Container name must be at least 3 and no more than 63 characters long";
            }
            var regExp = /^[a-z0-9-]+$/;
            if (!regExp.test(containerName)) {
                return "Allowed characters are a-z,0-9 and '-' (hyphen).";
            }
            if (containerName[0] == "-" || containerName[containerName.length - 1] == "-") {
                return "Container name must start and end with a letter or number";
            }
            var twoDashes = /--/;
            if (twoDashes.test(containerName)) {
                return "Consecutive dashes are not permitted in container names";
            }
            return "";
        };
        periodicExportSetup.prototype.fromDto = function (dto) {
            this.awsRegionEndpoint(dto.AwsRegionEndpoint);
            this.setupTypeAndMainValue(dto);
            this.s3RemoteFolderName(dto.S3RemoteFolderName);
            this.azureRemoteFolderName(dto.AzureRemoteFolderName);
            var incr = this.prepareBackupInterval(dto.IntervalMilliseconds);
            this.incrementalBackupInterval(incr[0]);
            this.incrementalBackupIntervalUnit(incr[1]);
            var full = this.prepareBackupInterval(dto.FullBackupIntervalMilliseconds);
            this.fullBackupInterval(full[0]);
            this.fullBackupIntervalUnit(full[1]);
            this.disabled(dto.Disabled);
        };
        periodicExportSetup.prototype.toDto = function () {
            return {
                Disabled: this.disabled(),
                GlacierVaultName: this.prepareMainValue(this.GLACIER_VAULT),
                S3BucketName: this.prepareMainValue(this.S3_BUCKET),
                AwsRegionEndpoint: this.awsRegionEndpoint(),
                AzureStorageContainer: this.prepareMainValue(this.AZURE_STORAGE),
                LocalFolderName: this.onDiskExportEnabled() ? this.localFolderName() : null,
                S3RemoteFolderName: this.isS3Bucket() ? this.s3RemoteFolderName() : null,
                AzureRemoteFolderName: this.additionalAzureInfoRequired() ? this.azureRemoteFolderName() : null,
                IntervalMilliseconds: this.convertToMilliseconds(this.incrementalBackupInterval(), this.incrementalBackupIntervalUnit()),
                FullBackupIntervalMilliseconds: this.convertToMilliseconds(this.fullBackupInterval(), this.fullBackupIntervalUnit()),
            };
        };
        periodicExportSetup.prototype.prepareMainValue = function (expectedType) {
            return ((this.type() === expectedType && this.remoteUploadEnabled()) ? this.mainValue() : null);
        };
        periodicExportSetup.prototype.convertToMilliseconds = function (value, unit) {
            if (value && unit) {
                switch (unit) {
                    case this.TU_MINUTES:
                        return value * 1000 * 60;
                    case this.TU_HOURS:
                        return value * 1000 * 60 * 60;
                    case this.TU_DAYS:
                        return value * 1000 * 60 * 60 * 24;
                }
            }
            return null;
        };
        periodicExportSetup.prototype.fromDatabaseSettingsDto = function (dbSettingsDto) {
            this.dbSettingsDto = dbSettingsDto;
            this.awsAccessKey(dbSettingsDto["Settings"]["Raven/AWSAccessKey"]);
            this.awsSecretKey(dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"]);
            this.azureStorageAccount(dbSettingsDto["Settings"]["Raven/AzureStorageAccount"]);
            this.azureStorageKey(dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"]);
            if (periodicExportSetup.decryptFailedValue === this.awsSecretKey()) {
                this.awsSecretKey("");
                this.awsSecretKeyDecryptionFailed(true);
            }
            if (periodicExportSetup.decryptFailedValue === this.azureStorageKey()) {
                this.azureStorageKey("");
                this.azureStorageKeyDecryptionFailed(true);
            }
        };
        periodicExportSetup.prototype.toDatabaseSettingsDto = function () {
            if (this.additionalAwsInfoRequired()) {
                this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"] = this.awsAccessKey();
                this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"] = this.awsSecretKey();
            }
            else {
                delete this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"];
                delete this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"];
            }
            if (this.additionalAzureInfoRequired()) {
                this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"] = this.azureStorageAccount();
                this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"] = this.azureStorageKey();
            }
            else {
                delete this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"];
                delete this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"];
            }
            return this.dbSettingsDto;
        };
        periodicExportSetup.prototype.removeDatabaseSettings = function () {
            delete this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"];
            delete this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"];
            delete this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"];
            delete this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"];
            return this.dbSettingsDto;
        };
        periodicExportSetup.prototype.getEtag = function () {
            return this.toDatabaseSettingsDto()["@metadata"]["@etag"];
        };
        periodicExportSetup.prototype.setEtag = function (newEtag) {
            this.toDatabaseSettingsDto()["@metadata"]["@etag"] = newEtag;
        };
        periodicExportSetup.prototype.resetDecryptionFailures = function () {
            this.awsSecretKeyDecryptionFailed(false);
            this.azureStorageKeyDecryptionFailed(false);
        };
        periodicExportSetup.prototype.setupTypeAndMainValue = function (dto) {
            var count = 0;
            if (dto.LocalFolderName) {
                this.localFolderName(dto.LocalFolderName);
                this.onDiskExportEnabled(true);
            }
            if (dto.GlacierVaultName) {
                count += 1;
                this.type(this.GLACIER_VAULT);
                this.mainValue(dto.GlacierVaultName);
            }
            if (dto.S3BucketName) {
                count += 1;
                this.type(this.S3_BUCKET);
                this.mainValue(dto.S3BucketName);
            }
            if (dto.AzureStorageContainer) {
                count += 1;
                this.type(this.AZURE_STORAGE);
                this.mainValue(dto.AzureStorageContainer);
            }
            if (count > 0) {
                this.remoteUploadEnabled(true);
            }
            this.unsupported(count > 1);
        };
        periodicExportSetup.prototype.prepareBackupInterval = function (milliseconds) {
            if (milliseconds) {
                var seconds = milliseconds / 1000;
                var minutes = seconds / 60;
                var hours = minutes / 60;
                if (this.isValidTimeValue(hours)) {
                    var days = hours / 24;
                    if (this.isValidTimeValue(days)) {
                        return [days, this.TU_DAYS];
                    }
                    return [hours, this.TU_HOURS];
                }
                return [minutes, this.TU_MINUTES];
            }
            return [0, this.TU_MINUTES];
        };
        periodicExportSetup.prototype.isValidTimeValue = function (value) {
            return value >= 1 && value % 1 === 0;
        };
        periodicExportSetup.prototype.copyFrom = function (from) {
            this.onDiskExportEnabled(from.onDiskExportEnabled());
            this.remoteUploadEnabled(from.remoteUploadEnabled());
            this.localFolderName(from.localFolderName());
            this.unsupported(from.unsupported());
            this.disabled(from.disabled());
            this.type(from.type());
            this.mainValue(from.mainValue());
            this.awsAccessKey(from.awsAccessKey());
            this.awsSecretKey(from.awsSecretKey());
            this.awsRegionEndpoint(from.awsRegionEndpoint());
            this.s3RemoteFolderName(from.s3RemoteFolderName());
            this.azureStorageAccount(from.azureStorageAccount());
            this.azureStorageKey(from.azureStorageKey());
            this.azureRemoteFolderName(from.azureRemoteFolderName());
            this.incrementalBackupInterval(from.incrementalBackupInterval());
            this.incrementalBackupIntervalUnit(from.incrementalBackupIntervalUnit());
            this.fullBackupInterval(from.fullBackupInterval());
            this.fullBackupIntervalUnit(from.fullBackupIntervalUnit());
        };
        periodicExportSetup.decryptFailedValue = "<data could not be decrypted>";
        return periodicExportSetup;
    })();
    return periodicExportSetup;
});
define("models/database/documents/versioningEntry", ["require", "exports", "models/database/documents/documentMetadata"], function (require, exports, documentMetadata) {
    var versioningEntry = (function () {
        function versioningEntry(dto, fromDatabse) {
            var _this = this;
            if (fromDatabse === void 0) { fromDatabse = false; }
            this.collection = ko.observable();
            this.maxRevisions = ko.observable();
            this.exclude = ko.observable();
            this.excludeUnlessExplicit = ko.observable();
            this.purgeOnDelete = ko.observable();
            this.fromDatabase = ko.observable();
            if (!dto) {
                // Default settings for new entries
                dto = {
                    Id: "",
                    MaxRevisions: 214748368,
                    Exclude: false,
                    ExcludeUnlessExplicit: false,
                    PurgeOnDelete: false
                };
                this.__metadata = new documentMetadata();
            }
            else {
                this.__metadata = new documentMetadata(dto["@metadata"]);
            }
            this.fromDatabase(fromDatabse);
            var id = dto.Id;
            if (!id && this.__metadata.id) {
                // pre 3.0 backward compatibility
                var prefix = "Raven/Versioning/";
                id = this.__metadata.id;
                if (id && id.startsWith(prefix)) {
                    id = id.substring(prefix.length);
                }
            }
            this.collection(id);
            this.maxRevisions(dto.MaxRevisions);
            this.exclude(dto.Exclude);
            this.excludeUnlessExplicit(dto.ExcludeUnlessExplicit);
            this.purgeOnDelete(dto.PurgeOnDelete);
            this.removable = ko.computed(function () {
                return (_this.collection() !== "DefaultConfiguration");
            });
            this.isValid = ko.computed(function () {
                return _this.collection() != null && _this.collection().length > 0 && _this.collection().indexOf(' ') === -1;
            });
            this.disabled = ko.computed(function () { return _this.exclude(); });
        }
        versioningEntry.prototype.toDto = function (includeMetadata) {
            var dto = {
                '@metadata': undefined,
                Id: this.collection(),
                MaxRevisions: this.maxRevisions(),
                Exclude: this.exclude(),
                ExcludeUnlessExplicit: this.excludeUnlessExplicit(),
                PurgeOnDelete: this.purgeOnDelete()
            };
            if (includeMetadata && this.__metadata) {
                dto["@metadata"] = (this.__metadata.toDto());
            }
            return dto;
        };
        versioningEntry.prototype.copyFromParent = function (parent) {
            this.collection(parent.collection());
            this.maxRevisions(parent.maxRevisions());
            this.exclude(parent.exclude());
            this.excludeUnlessExplicit(parent.excludeUnlessExplicit());
            this.purgeOnDelete(parent.purgeOnDelete());
            this.fromDatabase(true);
            this.__metadata = parent.__metadata;
        };
        return versioningEntry;
    })();
    return versioningEntry;
});
define("models/database/globalConfig/configurationDocument", ["require", "exports"], function (require, exports) {
    var configurationDocument = (function () {
        function configurationDocument(dto) {
            this.localExists = ko.observable();
            this.globalExists = ko.observable();
            this.mergedDocument = ko.observable();
            this.globalDocument = ko.observable();
            this.etag = ko.observable();
            this.metadata = ko.observable();
            this.localExists(dto.LocalExists);
            this.globalExists(dto.GlobalExists);
            this.mergedDocument(dto.MergedDocument);
            this.globalDocument(dto.GlobalDocument);
            this.etag(dto.Etag);
            this.metadata(dto.Metadata);
        }
        configurationDocument.fromDtoWithTransform = function (dto, transform) {
            return new configurationDocument({
                Etag: dto.Etag,
                MergedDocument: transform(dto.MergedDocument),
                Metadata: dto.Metadata,
                GlobalDocument: transform(dto.GlobalDocument),
                LocalExists: dto.LocalExists,
                GlobalExists: dto.GlobalExists
            });
        };
        configurationDocument.prototype.isUsingGlobal = function () {
            var g = this.globalExists();
            var l = this.localExists();
            return g && !l;
        };
        configurationDocument.prototype.copyFromGlobal = function () {
            if (this.globalExists()) {
                this.localExists(false);
                this.mergedDocument().copyFromParent(this.globalDocument());
            }
        };
        return configurationDocument;
    })();
    return configurationDocument;
});
define("models/database/globalConfig/configurationSetting", ["require", "exports"], function (require, exports) {
    var configurationSetting = (function () {
        function configurationSetting(dto) {
            this.localExists = ko.observable();
            this.globalExists = ko.observable();
            this.effectiveValue = ko.observable();
            this.globalValue = ko.observable();
            this.localExists(dto.LocalExists);
            this.globalExists(dto.GlobalExists);
            this.effectiveValue(dto.EffectiveValue);
            this.globalValue(dto.GlobalValue);
        }
        configurationSetting.prototype.isUsingGlobal = function () {
            var g = this.globalExists();
            var l = this.localExists();
            return g && !l;
        };
        configurationSetting.prototype.copyFromGlobal = function () {
            if (this.globalExists()) {
                this.localExists(false);
                this.effectiveValue(this.globalValue());
            }
        };
        return configurationSetting;
    })();
    return configurationSetting;
});
define("models/database/globalConfig/configurationSettings", ["require", "exports", "models/database/globalConfig/configurationSetting"], function (require, exports, configurationSetting) {
    var configurationSettings = (function () {
        function configurationSettings(dto) {
            this.results = {};
            for (var key in dto.Results) {
                this.results[key] = new configurationSetting(dto.Results[key]);
            }
        }
        return configurationSettings;
    })();
    return configurationSettings;
});
define("models/database/index/index", ["require", "exports", "common/appUrl"], function (require, exports, appUrl) {
    var index = (function () {
        function index(dto) {
            var _this = this;
            this.isOnRam = ko.observable();
            this.lockMode = ko.observable();
            this.isMapReduce = ko.observable();
            this.isIdle = ko.observable(false);
            this.isAbandoned = ko.observable(false);
            this.isErrored = ko.observable(false);
            this.isDisabled = ko.observable(false);
            this.isInvalid = ko.observable(false);
            this.isTestIndex = ko.observable(false);
            this.filteredOut = ko.observable(false); //UI only property
            this.willReplaceIndex = ko.observable();
            this.willBeReplacedByIndex = ko.observable();
            this.isSideBySideIndex = ko.computed(function () { return _this.willReplaceIndex() != null; });
            this.createdTimestamp = dto.CreatedTimestamp;
            this.docsCount = dto.DocsCount;
            this.forEntityName = dto.ForEntityName;
            this.indexingAttempts = dto.IndexingAttempts;
            this.indexingErrors = dto.IndexingErrors;
            this.indexingSuccesses = dto.IndexingSuccesses;
            this.isOnRam(dto.IsOnRam);
            this.lastIndexedEtag = dto.LastIndexedEtag;
            this.lastIndexedTimestamp = dto.LastIndexedTimestamp;
            this.lastIndexingTime = dto.LastIndexingTime;
            this.lastQueryTimestamp = dto.LastQueryTimestamp;
            this.lastReducedEtag = dto.LastReducedEtag;
            this.lastReducedTimestamp = dto.LastReducedTimestamp;
            this.lockMode(dto.LockMode);
            this.isMapReduce(dto.IsMapReduce);
            this.name = dto.Name;
            this.priority = dto.Priority;
            this.reduceIndexingAttempts = dto.ReduceIndexingAttempts;
            this.reduceIndexingErrors = dto.ReduceIndexingErrors;
            this.reduceIndexingSuccesses = dto.ReduceIndexingSuccesses;
            this.touchCount = dto.TouchCount;
            this.isInvalid(dto.IsInvalidIndex);
            this.isTestIndex(dto.IsTestIndex);
            this.isAbandoned(this.priority && this.priority.indexOf(index.priorityAbandoned) !== -1);
            this.isDisabled(this.priority && this.priority.indexOf(index.priorityDisabled) !== -1);
            this.isErrored(this.priority && this.priority.indexOf(index.priorityErrored) !== -1);
            this.isIdle(this.priority && this.priority.indexOf(index.priorityIdle) !== -1);
            this.editUrl = appUrl.forCurrentDatabase().editIndex(this.name);
            this.queryUrl = appUrl.forCurrentDatabase().query(this.name);
        }
        index.priorityFromString = function (priority) {
            switch (priority) {
                case index.priorityIdle: return 1 /* idle */;
                case index.priorityDisabled: return 2 /* disabled */;
                case index.priorityAbandoned: return 3 /* abandoned */;
                case index.priorityIdleForced: return 4 /* idleForced */;
                case index.priorityDisabledForced: return 5 /* disabledForced */;
                case index.priorityAbandonedForced: return 6 /* abandonedForced */;
                default: return 0 /* normal */;
            }
        };
        index.priorityToString = function (priority) {
            switch (priority) {
                case 3 /* abandoned */: return index.priorityAbandoned;
                case 6 /* abandonedForced */: return index.priorityAbandonedForced;
                case 2 /* disabled */: return index.priorityDisabled;
                case 5 /* disabledForced */: return index.priorityDisabledForced;
                case 1 /* idle */: return index.priorityIdle;
                case 4 /* idleForced */: return index.priorityIdleForced;
                default: return index.priorityNormal;
            }
        };
        index.priorityNormal = "Normal";
        index.priorityIdle = "Idle";
        index.priorityDisabled = "Disabled";
        index.priorityErrored = "Error";
        index.priorityAbandoned = "Abandoned";
        index.priorityIdleForced = "Idle,Forced";
        index.priorityDisabledForced = "Disabled,Forced";
        index.priorityAbandonedForced = "Abandoned,Forced";
        index.SideBySideIndexPrefix = "ReplacementOf/";
        index.TestIndexPrefix = "Test/";
        return index;
    })();
    return index;
});
define("models/database/index/indexAceAutoCompleteProvider", ["require", "exports", "models/database/documents/collection", "models/database/documents/document", "commands/database/documents/getCollectionsCommand", "commands/database/documents/getDocumentsByEntityNameCommand"], function (require, exports, collection, document, getCollectionsCommand, getDocumentsByEntityNameCommand) {
    var indexAceAutoCompleteProvider = (function () {
        function indexAceAutoCompleteProvider(activeDatabase, editedIndex) {
            this.activeDatabase = activeDatabase;
            this.editedIndex = editedIndex;
        }
        indexAceAutoCompleteProvider.prototype.getCollectionAliasesInsideIndexText = function (session) {
            var aliases = [];
            var curAliasKey = null;
            var curAliasValuePrefix = null;
            var curAliasValueSuffix = null;
            for (var curRow = 0; curRow < session.getLength(); curRow++) {
                var curRowTokens = session.getTokens(curRow);
                for (var curTokenInRow = 0; curTokenInRow < curRowTokens.length; curTokenInRow++) {
                    if (curRowTokens[curTokenInRow].type == "from.alias") {
                        curAliasKey = curRowTokens[curTokenInRow].value.trim();
                    }
                    else if (!!curAliasKey) {
                        if (curRowTokens[curTokenInRow].type == "docs" || curRowTokens[curTokenInRow].type == "collections") {
                            curAliasValuePrefix = curRowTokens[curTokenInRow].value;
                        }
                        else if (curRowTokens[curTokenInRow].type == "collectionName") {
                            curAliasValueSuffix = curRowTokens[curTokenInRow].value;
                            aliases.push({ aliasKey: curAliasKey, aliasValuePrefix: curAliasValuePrefix.replace('.', '').trim(), aliasValueSuffix: curAliasValueSuffix.replace('.', '').trim() });
                            curAliasKey = null;
                            curAliasValuePrefix = null;
                            curAliasValueSuffix = null;
                        }
                    }
                }
            }
            return aliases;
        };
        indexAceAutoCompleteProvider.prototype.getIndexMapCollectionFieldsForAutocomplete = function (session, currentToken) {
            var deferred = $.Deferred();
            var collectionAliases = this.getCollectionAliasesInsideIndexText(session);
            // find the matching alias and get list of fields
            if (collectionAliases.length > 0) {
                var matchingAliasKeyValue = collectionAliases.first(function (x) { return x.aliasKey.replace('.', '').trim() === currentToken.value.replace('.', '').trim(); });
                if (!!matchingAliasKeyValue) {
                    // get list of fields according to it's collection's first row
                    if (matchingAliasKeyValue.aliasValuePrefix.toLowerCase() === "docs") {
                        new getDocumentsByEntityNameCommand(new collection(matchingAliasKeyValue.aliasValueSuffix, this.activeDatabase), 0, 1)
                            .execute()
                            .done(function (result) {
                            if (!!result && result.totalResultCount > 0) {
                                var documentPattern = new document(result.items[0]);
                                deferred.resolve(documentPattern.getDocumentPropertyNames());
                            }
                            else {
                                deferred.reject();
                            }
                        }).fail(function () { return deferred.reject(); });
                    }
                    else {
                        deferred.reject();
                    }
                }
            }
            else {
                deferred.reject();
            }
            return deferred;
        };
        indexAceAutoCompleteProvider.prototype.getIndexMapCompleterValues = function (editor, session, pos) {
            var currentToken = session.getTokenAt(pos.row, pos.column);
            var completedToken;
            var TokenIterator = require("ace/token_iterator").TokenIterator;
            var curPosIterator = new TokenIterator(editor.getSession(), pos.row, pos.column);
            var prevToken = curPosIterator.stepBackward();
            var returnedDeferred = $.Deferred();
            var suggestionsArray = [];
            // validation: if is null or it's type is represented by a string
            if (!currentToken || typeof currentToken.type == "string") {
                // if in beginning of text or in free text token
                if (!currentToken || currentToken.type == "text") {
                    suggestionsArray.push("from");
                    suggestionsArray.push("docs");
                }
                else if (!!currentToken.value && (currentToken.type == "docs" || (!!prevToken && prevToken.type == "docs"))) {
                    new getCollectionsCommand(this.activeDatabase)
                        .execute()
                        .done(function (collections) {
                        collections.forEach(function (curCollection) {
                            return suggestionsArray.push(curCollection.name);
                        });
                        returnedDeferred.resolve(suggestionsArray);
                    })
                        .fail(function () { return returnedDeferred.reject(); });
                }
                else if (currentToken.type == "collections" || currentToken.type == "collectionName") {
                    if (currentToken.type == "collections") {
                        completedToken = currentToken;
                    }
                    else {
                        completedToken = prevToken;
                    }
                    this.getIndexMapCollectionFieldsForAutocomplete(session, completedToken)
                        .done(function (x) { return returnedDeferred.resolve(x); })
                        .fail(function () { return returnedDeferred.reject(); });
                }
                else if (currentToken.type == "data.prefix" || currentToken.type == "data.suffix") {
                    if (currentToken.type == "data.prefix") {
                        completedToken = currentToken;
                    }
                    else {
                        completedToken = prevToken;
                    }
                    var firstToken = session.getTokenAt(0, 0);
                    // treat a "from [foo] in [bar] type of index syntax
                    if (firstToken.value == "from") {
                        var aliases = this.getCollectionAliasesInsideIndexText(session);
                        this.getIndexMapCollectionFieldsForAutocomplete(session, completedToken)
                            .done(function (x) { return returnedDeferred.resolve(x); })
                            .fail(function () { return returnedDeferred.reject(); });
                    }
                    else {
                        returnedDeferred.resolve(["Methodical Syntax Not Supported"]);
                    }
                }
                else {
                    returnedDeferred.reject();
                }
            }
            else {
                returnedDeferred.reject();
            }
            return returnedDeferred;
        };
        indexAceAutoCompleteProvider.prototype.indexMapCompleter = function (editor, session, pos, prefix, callback) {
            this.getIndexMapCompleterValues(editor, session, pos)
                .done(function (x) {
                callback(null, x.map(function (val) {
                    return { name: val, value: val, score: 100, meta: "suggestion" };
                }));
            })
                .fail(function () {
                callback([{ error: "notext" }], null);
            });
        };
        indexAceAutoCompleteProvider.prototype.getIndexReduceCompleterValues = function () {
            var firstMapSrting = this.editedIndex().maps()[0]();
            var dotPrefixes = firstMapSrting.match(/[.]\w*/g);
            var equalPrefixes = firstMapSrting.match(/\w*\s*=\s*/g);
            var autoCompletes = [];
            if (!!dotPrefixes) {
                dotPrefixes.forEach(function (curPrefix) {
                    autoCompletes.push(curPrefix.replace(".", "").trim());
                });
            }
            if (!!equalPrefixes) {
                equalPrefixes.forEach(function (curPrefix) {
                    autoCompletes.push(curPrefix.replace("=", "").trim());
                });
            }
            return autoCompletes;
        };
        indexAceAutoCompleteProvider.prototype.indexReduceCompleter = function (editor, session, pos, prefix, callback) {
            var autoCompletes = this.getIndexReduceCompleterValues();
            callback(null, autoCompletes.map(function (curField) {
                return { name: curField, value: curField, score: 100, meta: "suggestion" };
            }));
        };
        return indexAceAutoCompleteProvider;
    })();
    return indexAceAutoCompleteProvider;
});
define("models/database/index/indexDefinition", ["require", "exports", "models/database/index/luceneField", "models/database/index/spatialIndexField"], function (require, exports, luceneField, spatialIndexField) {
    var indexDefinition = (function () {
        function indexDefinition(dto) {
            var _this = this;
            this.name = ko.observable().extend({ required: true });
            this.map = ko.observable().extend({ required: true });
            this.maps = ko.observableArray().extend({ required: true });
            this.reduce = ko.observable().extend({ required: true });
            this.luceneFields = ko.observableArray();
            this.isTestIndex = ko.observable(false);
            this.isSideBySideIndex = ko.observable(false);
            this.numOfLuceneFields = ko.computed(function () { return _this.luceneFields().length; }).extend({ required: true });
            // This is an amalgamation of several properties from the index (Fields, Stores, Indexes, SortOptions, Analyzers, Suggestions, TermVectors) 
            // Stored as multiple luceneFields for the sake of data binding.
            // Each luceneField corresponds to a Field box in the index editor UI.
            this.spatialFields = ko.observableArray();
            this.numOfSpatialFields = ko.computed(function () { return _this.spatialFields().length; }).extend({ required: true });
            this.maxIndexOutputsPerDocument = ko.observable(0).extend({ required: true });
            this.storeAllFields = ko.observable(false);
            this.fields = ko.observableArray();
            this.analyzers = dto.Analyzers;
            this.fields(dto.Fields);
            this.indexes = dto.Indexes;
            this.internalFieldsMapping = dto.InternalFieldsMapping;
            this.isTestIndex(dto.IsTestIndex);
            this.isSideBySideIndex(dto.IsSideBySideIndex);
            this.isCompiled = dto.IsCompiled;
            this.isMapReduce = dto.IsMapReduce;
            this.lockMode = dto.LockMode;
            this.map(dto.Map);
            this.maps(dto.Maps.map(function (m) { return ko.observable(m); }));
            this.name(dto.Name);
            this.reduce(dto.Reduce);
            this.sortOptions = dto.SortOptions;
            this.spatialIndexes = dto.SpatialIndexes;
            this.stores = dto.Stores;
            this.suggestionsOptions = dto.SuggestionsOptions;
            this.termVectors = dto.TermVectors;
            this.type = dto.Type;
            this.luceneFields(this.parseFields());
            this.spatialFields(this.parseSpatialFields());
            this.maxIndexOutputsPerDocument(dto.MaxIndexOutputsPerDocument ? dto.MaxIndexOutputsPerDocument : 0);
            this.storeAllFields(this.isStoreAllFields());
        }
        indexDefinition.prototype.isStoreAllFields = function () {
            if (this.stores.hasOwnProperty("__all_fields")) {
                return this.stores["__all_fields"] === "Yes";
            }
            return false;
        };
        indexDefinition.prototype.setOrRemoveStoreAllFields = function (add) {
            if (add) {
                this.stores["__all_fields"] = "Yes";
            }
            else {
                delete this.stores["__all_fields"];
            }
            this.storeAllFields(this.isStoreAllFields());
        };
        indexDefinition.prototype.setStoreAllFieldsToObject = function (obj) {
            if (this.isStoreAllFields())
                obj["__all_fields"] = "Yes";
            return obj;
        };
        indexDefinition.prototype.toDto = function () {
            return {
                Analyzers: this.makeFieldObject(function (f) { return f.indexing() === "Analyzed"; }, function (f) { return f.analyzer(); }),
                Fields: this.fields(),
                Indexes: this.makeFieldObject(function (f) { return f.indexing() !== "Default"; }, function (f) { return f.indexing(); }),
                InternalFieldsMapping: this.internalFieldsMapping,
                IsTestIndex: this.isTestIndex(),
                IsSideBySideIndex: this.isSideBySideIndex(),
                IsCompiled: this.isCompiled,
                IsMapReduce: this.isMapReduce,
                LockMode: this.lockMode,
                Map: this.maps()[0](),
                Maps: this.maps().map(function (m) { return m(); }).filter(function (m) { return m && m.length > 0; }),
                Name: this.name(),
                Reduce: this.reduce(),
                SortOptions: this.makeFieldObject(function (f) { return f.sort() !== "None"; }, function (f) { return f.sort(); }),
                SpatialIndexes: this.makeSpatialIndexesObject(),
                Stores: this.setStoreAllFieldsToObject(this.makeFieldObject(function (f) { return f.stores() === "Yes"; }, function (f) { return f.stores(); })),
                SuggestionsOptions: this.luceneFields().filter(function (x) { return x.suggestionEnabled(); }).map(function (x) { return x.name(); }),
                TermVectors: this.makeFieldObject(function (f) { return f.termVector() !== "No"; }, function (f) { return f.termVector(); }),
                Type: this.type,
                MaxIndexOutputsPerDocument: this.maxIndexOutputsPerDocument() ? this.maxIndexOutputsPerDocument() > 0 ? this.maxIndexOutputsPerDocument() : null : null
            };
        };
        indexDefinition.empty = function () {
            return new indexDefinition({
                Analyzers: {},
                Fields: [],
                Indexes: {},
                InternalFieldsMapping: {},
                IsTestIndex: false,
                IsSideBySideIndex: false,
                IsCompiled: false,
                IsMapReduce: false,
                LockMode: "Unlock",
                Map: " ",
                Maps: [" "],
                Name: "",
                Reduce: null,
                SortOptions: {},
                SpatialIndexes: {},
                Stores: {},
                SuggestionsOptions: [],
                TermVectors: {},
                Type: "Map",
                MaxIndexOutputsPerDocument: null
            });
        };
        indexDefinition.prototype.makeSpatialIndexesObject = function () {
            var spatialIndexesObj = {};
            this.spatialFields().forEach(function (f) { return spatialIndexesObj[f.name()] = f.toDto(); });
            return spatialIndexesObj;
        };
        indexDefinition.prototype.makeFieldObject = function (filter, selector) {
            var obj = {};
            this.luceneFields()
                .filter(filter)
                .forEach(function (f) { return obj[f.name()] = selector(f); });
            return obj;
        };
        indexDefinition.prototype.parseFields = function () {
            var _this = this;
            var fieldSources = [
                this.analyzers,
                this.indexes,
                this.sortOptions,
                this.stores,
                this.suggestionsOptions,
                this.termVectors
            ];
            var keys = [];
            for (var i = 0; i < fieldSources.length; i++) {
                var src = fieldSources[i];
                if (src == null)
                    continue;
                for (var key in src) {
                    if (src.hasOwnProperty(key) === true && !keys.contains(key))
                        keys.push(key);
                }
            }
            return keys
                .map(function (fieldName) {
                var suggestionsEnabled = _this.suggestionsOptions && _this.suggestionsOptions.indexOf(fieldName) >= 0;
                return new luceneField(fieldName, _this.stores[fieldName], _this.indexes[fieldName], _this.sortOptions[fieldName], _this.analyzers[fieldName], suggestionsEnabled, _this.termVectors[fieldName], _this.fields());
            });
        };
        indexDefinition.prototype.parseSpatialFields = function () {
            // The spatial fields are stored as properties on the .spatialIndexes object.
            // The property names will be one of the .fields.
            var fields = [];
            if (this.spatialIndexes == null)
                return fields;
            for (var key in this.spatialIndexes) {
                var spatialIndex = this.spatialIndexes[key];
                fields.push(new spatialIndexField(key, spatialIndex));
            }
            return fields;
        };
        return indexDefinition;
    })();
    return indexDefinition;
});
define("models/database/index/indexMergeSuggestion", ["require", "exports", "models/database/index/indexDefinition", "common/idGenerator"], function (require, exports, indexDefinition, idGenerator) {
    var indexMergeSuggestion = (function () {
        function indexMergeSuggestion(dto) {
            this.id = ko.observable();
            this.canMerge = dto.CanMerge;
            this.collection = dto.Collection;
            this.mergedIndexDefinition = dto.MergedIndex ? new indexDefinition(dto.MergedIndex) : null;
            this.id(idGenerator.generateId());
            this.canDelete = dto.CanDelete;
            this.surpassingIndex = dto.SurpassingIndex;
        }
        indexMergeSuggestion.prototype.isSurpassingIndex = function () {
            return this.mergedIndexDefinition == null;
        };
        indexMergeSuggestion.prototype.toDto = function () {
            return {
                CanMerge: this.canMerge,
                Collection: this.collection,
                MergedIndex: this.mergedIndexDefinition ? this.mergedIndexDefinition.toDto() : null,
                CanDelete: this.canDelete,
                SurpassingIndex: this.surpassingIndex,
            };
        };
        return indexMergeSuggestion;
    })();
    return indexMergeSuggestion;
});
define("models/database/index/indexPriority", ["require", "exports"], function (require, exports) {
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/index/indexReplaceDocument", ["require", "exports", "models/database/documents/document"], function (require, exports, document) {
    var indexReplaceDocument = (function (_super) {
        __extends(indexReplaceDocument, _super);
        function indexReplaceDocument(dto) {
            _super.call(this, dto);
            this.indexToReplace = dto.IndexToReplace;
            this.minimumEtagBeforeReplace = dto.MinimumEtagBeforeReplace;
            this.replaceTimeUtc = dto.ReplaceTimeUtc;
        }
        indexReplaceDocument.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            return {
                '@metadata': meta,
                IndexToReplace: this.indexToReplace,
                MinimumEtagBeforeReplace: this.minimumEtagBeforeReplace,
                ReplaceTimeUtc: this.replaceTimeUtc
            };
        };
        indexReplaceDocument.prototype.extractReplaceWithIndexName = function () {
            var fullKey = this.getId();
            return fullKey.substr(indexReplaceDocument.replaceDocumentPrefix.length);
        };
        indexReplaceDocument.replaceDocumentPrefix = "Raven/Indexes/Replace/";
        return indexReplaceDocument;
    })(document);
    return indexReplaceDocument;
});
define("models/database/index/luceneField", ["require", "exports"], function (require, exports) {
    var luceneField = (function () {
        function luceneField(name, stores, indexing, sort, analyzer, suggestionEnabled, termVector, indexFieldNames) {
            var _this = this;
            if (stores === void 0) { stores = "No"; }
            if (indexing === void 0) { indexing = "Default"; }
            if (sort === void 0) { sort = "None"; }
            if (analyzer === void 0) { analyzer = 'StandardAnalyzer'; }
            if (suggestionEnabled === void 0) { suggestionEnabled = false; }
            if (termVector === void 0) { termVector = "No"; }
            this.indexFieldNames = indexFieldNames;
            this.name = ko.observable();
            this.stores = ko.observable();
            this.sort = ko.observable();
            this.termVector = ko.observable();
            this.indexing = ko.observable();
            this.analyzer = ko.observable();
            this.suggestionEnabled = ko.observable();
            this.fieldNameAutocompletes = ko.observableArray();
            this.name(name);
            this.stores(stores);
            this.indexing(indexing);
            this.sort(sort);
            this.analyzer(analyzer);
            this.suggestionEnabled(suggestionEnabled);
            this.termVector(termVector);
            this.name.subscribe(function () { return _this.calculateFieldNamesAutocomplete(); });
        }
        luceneField.prototype.calculateFieldNamesAutocomplete = function () {
            var _this = this;
            if (!!this.indexFieldNames && this.indexFieldNames.length > 0) {
                if (this.name().length > 0) {
                    this.fieldNameAutocompletes(this.indexFieldNames.filter(function (x) { return x.toLowerCase().indexOf(_this.name().toLowerCase()) >= 0; }));
                }
                else {
                    this.fieldNameAutocompletes(this.indexFieldNames);
                }
            }
        };
        luceneField.prototype.setName = function (curName) {
            this.name(curName);
        };
        return luceneField;
    })();
    return luceneField;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/index/scriptedIndex", ["require", "exports", "models/database/documents/document", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, document, aceEditorBindingHandler) {
    var scriptedIndex = (function (_super) {
        __extends(scriptedIndex, _super);
        function scriptedIndex(dto) {
            var _this = this;
            _super.call(this, dto);
            this.indexName = ko.observable();
            this.indexScript = ko.observable();
            this.deleteScript = ko.observable();
            this.deleteLater = ko.observable();
            var scriptedIndexName = dto['@metadata']['@id'].slice(scriptedIndex.PREFIX.length);
            this.indexName(scriptedIndexName);
            this.indexScript(dto.IndexScript);
            this.deleteScript(dto.DeleteScript);
            this.subscribeToObservable(this.indexScript);
            this.subscribeToObservable(this.deleteScript);
            this.indexName.subscribe(function (v) { return _this.__metadata.id = scriptedIndex.PREFIX + v; });
        }
        scriptedIndex.emptyForIndex = function (indexName) {
            var meta = [];
            meta['@id'] = this.PREFIX + indexName;
            meta['Raven-Entity-Name'] = 'ScriptedIndexResults';
            return new scriptedIndex({
                '@metadata': meta,
                IndexScript: "",
                DeleteScript: ""
            });
        };
        scriptedIndex.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            return {
                '@metadata': meta,
                IndexScript: this.indexScript(),
                DeleteScript: this.deleteScript()
            };
        };
        scriptedIndex.prototype.markToDelete = function () {
            this.indexScript("");
            this.deleteScript("");
            this.deleteLater(true);
        };
        scriptedIndex.prototype.cancelDeletion = function () {
            this.deleteLater(false);
        };
        scriptedIndex.prototype.isMarkedToDelete = function () {
            return this.deleteLater();
        };
        scriptedIndex.prototype.subscribeToObservable = function (observable) {
            observable.subscribe(function (newValue) {
                var message = "";
                var currentEditor = aceEditorBindingHandler.currentEditor;
                if (currentEditor != undefined) {
                    var textarea = $(currentEditor.container).find('textarea')[0];
                    if (newValue === "") {
                        message = "Please fill out this field.";
                    }
                    textarea.setCustomValidity(message);
                    setTimeout(function () {
                        var annotations = currentEditor.getSession().getAnnotations();
                        var isErrorExists = false;
                        for (var i = 0; i < annotations.length; i++) {
                            var annotationType = annotations[i].type;
                            if (annotationType === "error" || annotationType === "warning") {
                                isErrorExists = true;
                                break;
                            }
                        }
                        if (isErrorExists) {
                            message = "The script isn't a javascript legal expression!";
                            textarea.setCustomValidity(message);
                        }
                    }, 700);
                }
            });
        };
        scriptedIndex.PREFIX = 'Raven/ScriptedIndexResults/';
        return scriptedIndex;
    })(document);
    return scriptedIndex;
});
define("models/database/index/spatialIndexField", ["require", "exports"], function (require, exports) {
    var spatialIndexField = (function () {
        function spatialIndexField(fieldName, dto) {
            var _this = this;
            this.name = ko.observable().extend({ required: true });
            this.type = ko.observable().extend({ required: true });
            this.strategy = ko.observable().extend({ required: true });
            this.minX = ko.observable().extend({ required: true });
            this.maxX = ko.observable().extend({ required: true });
            this.circleRadiusUnits = ko.observable().extend({ required: true });
            this.maxTreeLevel = ko.observable().extend({ required: true });
            this.minY = ko.observable().extend({ required: true });
            this.maxY = ko.observable().extend({ required: true });
            this.units = ko.observable();
            this.availableStrategies = ko.observableArray();
            this.name(fieldName);
            this.type(dto.Type);
            this.strategy(dto.Strategy);
            this.maxTreeLevel(dto.MaxTreeLevel);
            this.minX(dto.MinX);
            this.maxX(dto.MaxX);
            this.minY(dto.MinY);
            this.maxY(dto.MaxY);
            this.units(dto.Units);
            this.availableStrategies(this.getAvailableStrategies());
            this.canSpecifyUnits = ko.computed(function () { return _this.type() === spatialIndexField.typeGeo; });
            this.canSpecifyTreeLevel = ko.computed(function () { return _this.strategy() !== spatialIndexField.strategyBounding; });
            this.precision = ko.computed(function () { return _this.getPrecisionString(); });
            this.canSpecifyCoordinates = ko.computed(function () { return _this.type() === spatialIndexField.typeCart; });
            this.type.subscribe(function (newType) { return _this.resetCoordinates(); });
            this.type.subscribe(function () { return _this.availableStrategies(_this.getAvailableStrategies()); });
            this.strategy.subscribe(function (newStrategy) { return _this.updateMaxTreeLevelFromStrategy(newStrategy); });
        }
        spatialIndexField.prototype.toDto = function () {
            return {
                Type: this.type(),
                Strategy: this.strategy(),
                MaxTreeLevel: this.maxTreeLevel(),
                MinX: this.minX(),
                MaxX: this.maxX(),
                MinY: this.minY(),
                MaxY: this.maxY(),
                Units: this.units()
            };
        };
        spatialIndexField.empty = function () {
            var dto = {
                Type: spatialIndexField.typeGeo,
                MaxTreeLevel: 9,
                MinX: -180,
                MaxX: 180,
                MinY: -90,
                MaxY: 90,
                Strategy: spatialIndexField.strategyGeo,
                Units: "Kilometers"
            };
            return new spatialIndexField("", dto);
        };
        spatialIndexField.prototype.resetCoordinates = function () {
            this.minX(-180);
            this.maxX(180);
            this.minY(-90);
            this.maxY(90);
        };
        spatialIndexField.prototype.getPrecisionString = function () {
            var strategy = this.strategy();
            var minX = this.minX();
            var maxX = this.maxX();
            var minY = this.minY();
            var maxY = this.maxY();
            var maxTreeLevel = this.maxTreeLevel();
            var type = this.type();
            var units = this.units();
            if (strategy === spatialIndexField.strategyBounding) {
                return "";
            }
            var x = maxX - minX;
            var y = maxY - minY;
            for (var i = 0; i < maxTreeLevel; i++) {
                if (strategy === spatialIndexField.strategyGeo) {
                    if (i % 2 == 0) {
                        x /= 8;
                        y /= 4;
                    }
                    else {
                        x /= 4;
                        y /= 8;
                    }
                }
                else if (strategy === spatialIndexField.strategyQuad) {
                    x /= 2;
                    y /= 2;
                }
            }
            if (type === spatialIndexField.typeGeo) {
                var earthMeanRadiusKm = 6371.0087714;
                var milesToKm = 1.60934;
                var factor = (earthMeanRadiusKm * Math.PI * 2) / 360;
                x = x * factor;
                y = y * factor;
                if (units === "Miles") {
                    x /= milesToKm;
                    y /= milesToKm;
                }
                return "Precision at equator; X: " + x.toFixed(6) + ", Y: " + y.toFixed(6) + " " + units.toLowerCase();
            }
            else {
                return "Precision; X: " + x.toFixed(6) + ", Y: " + y.toFixed(6);
            }
        };
        spatialIndexField.prototype.getAvailableStrategies = function () {
            if (this.type() === spatialIndexField.typeGeo) {
                return [spatialIndexField.strategyGeo, spatialIndexField.strategyQuad, spatialIndexField.strategyBounding];
            }
            else {
                return [spatialIndexField.strategyQuad, spatialIndexField.strategyBounding];
            }
        };
        spatialIndexField.prototype.updateMaxTreeLevelFromStrategy = function (strategy) {
            if (strategy === spatialIndexField.strategyGeo) {
                this.maxTreeLevel(9);
            }
            else if (strategy === spatialIndexField.strategyQuad) {
                this.maxTreeLevel(23);
            }
        };
        spatialIndexField.strategyGeo = "GeohashPrefixTree";
        spatialIndexField.strategyBounding = "BoundingBox";
        spatialIndexField.strategyQuad = "QuadPrefixTree";
        spatialIndexField.typeGeo = "Geography";
        spatialIndexField.typeCart = "Cartesian";
        return spatialIndexField;
    })();
    return spatialIndexField;
});
/// <reference path="../../../models/dto.ts" />
define("models/database/index/transformer", ["require", "exports", "common/appUrl"], function (require, exports, appUrl) {
    var transformer = (function () {
        function transformer() {
            var _this = this;
            this.name = ko.observable().extend({ required: true });
            this.transformResults = ko.observable().extend({ required: true });
            this.lockMode = ko.observable();
            this.originalName = ko.observable();
            this.wasNameChanged = ko.computed(function () { return _this.name() != _this.originalName(); });
        }
        transformer.prototype.initFromLoad = function (dto) {
            this.originalName(dto.name.toString());
            this.name(dto.name);
            this.editUrl = appUrl.forCurrentDatabase().editTransformer(this.name());
            this.transformResults(dto.definition.TransformResults);
            this.lockMode(dto.definition.LockMode);
            return this;
        };
        transformer.prototype.initFromSave = function (dto) {
            this.originalName(dto.Transformer.Name.toString());
            this.name(dto.Transformer.Name);
            this.editUrl = appUrl.forCurrentDatabase().editTransformer(this.name());
            this.transformResults(dto.Transformer.TransformResults);
            this.lockMode(dto.Transformer.LockMode);
            return this;
        };
        transformer.prototype.toDto = function () {
            return {
                'name': this.name(),
                'definition': {
                    'Name': this.name(),
                    'TransformResults': this.transformResults(),
                    'LockMode': this.lockMode()
                }
            };
        };
        transformer.prototype.toSaveDto = function () {
            return {
                'Name': this.name(),
                'TransformResults': this.transformResults()
            };
        };
        transformer.prototype.extractInputs = function () {
            var matcher = /(Query|Parameter)\(["'].*?["']\)/g;
            var defaultMatcher = /(Query|Parameter)OrDefault\(["'].*?["'],\s+["'].*?["']\)/g;
            if (this.transformResults()) {
                var parameters = this.transformResults().match(matcher);
                var parametersWithDefault = this.transformResults().match(defaultMatcher);
                var results = [];
                if (parameters !== null) {
                    parameters.forEach(function (value) { return results.push({
                        name: value.substring(value.indexOf('(') + 2, value.length - 2),
                        hasDefault: false
                    }); });
                }
                if (parametersWithDefault !== null) {
                    parametersWithDefault.forEach(function (value) { return results.push({
                        name: value.substring(value.indexOf('(') + 2, value.indexOf(',') - 1),
                        hasDefault: true
                    }); });
                }
                return results;
            }
            return [];
        };
        transformer.empty = function () {
            return new transformer().initFromLoad({
                'name': "",
                'definition': {
                    'Name': "",
                    'TransformResults': "",
                    'LockMode': "Unlock"
                }
            });
        };
        transformer.prototype.isLocked = function () {
            return this.lockMode() === "LockedIgnore";
        };
        return transformer;
    })();
    return transformer;
});
define("models/database/index/transformerParam", ["require", "exports"], function (require, exports) {
    var transformerParam = (function () {
        function transformerParam(name, value) {
            this.name = name;
            this.value = value;
        }
        return transformerParam;
    })();
    return transformerParam;
});
define("models/database/index/transformerQuery", ["require", "exports", "models/database/index/transformerParam"], function (require, exports, transformerParam) {
    var transformerQuery = (function () {
        function transformerQuery(dto) {
            this.transformerName = dto.transformerName;
            this.queryParams = dto.queryParams;
        }
        transformerQuery.prototype.addParamByNameAndValue = function (name, value) {
            this.queryParams.push(new transformerParam(name, value));
        };
        transformerQuery.prototype.addParam = function (param) {
            this.queryParams.push(param);
        };
        transformerQuery.prototype.toUrl = function () {
            if (this.transformerName) {
                var paramsUrl = this.queryParams
                    .map(function (param) { return "tp-" + param.name + "=" + param.value; }) // transform transformerParam array into "tp-NAME=VALUE" strings (also array)
                    .join("&");
                return "&resultsTransformer=" + this.transformerName + (paramsUrl.length > 0 ? "&" + paramsUrl : "");
            }
            else {
                return "";
            }
        };
        transformerQuery.prototype.toDto = function () {
            return {
                transformerName: this.transformerName,
                queryParams: this.queryParams
            };
        };
        return transformerQuery;
    })();
    return transformerQuery;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/patch/patchDocument", ["require", "exports", "models/database/patch/patchParam", "models/database/documents/document", "models/database/documents/documentMetadata"], function (require, exports, patchParam, document, documentMetadata) {
    var patchDocument = (function (_super) {
        __extends(patchDocument, _super);
        function patchDocument(dto) {
            var _this = this;
            _super.call(this, dto);
            this.patchOnOption = ko.observable();
            this.selectedItem = ko.observable();
            this.query = ko.observable();
            this.script = ko.observable();
            this.parameters = ko.observableArray();
            this.allPatchOptions = ["Document", "Collection", "Index"];
            this.patchOnOption(dto.PatchOnOption);
            this.query(dto.Query);
            this.script(dto.Script);
            this.selectedItem(dto.SelectedItem);
            this.parameters(dto.Values.map(function (val) { return new patchParam(val); }));
            this.patchOptions = ko.computed(function () { return _this.allPatchOptions.filter(function (x) { return x !== _this.patchOnOption(); }); });
        }
        patchDocument.empty = function () {
            var meta = {};
            meta['Raven-Entity-Name'] = 'PatchDocuments';
            return new patchDocument({
                '@metadata': meta,
                PatchOnOption: "Document",
                Query: "",
                Script: "",
                SelectedItem: "",
                Values: []
            });
        };
        patchDocument.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            return {
                '@metadata': meta,
                PatchOnOption: this.patchOnOption(),
                Query: this.query(),
                Script: this.script(),
                SelectedItem: this.selectedItem(),
                Values: this.parameters().map(function (val) { return val.toDto(); })
            };
        };
        patchDocument.prototype.isDocumentPatch = function () {
            return this.patchOnOption() === "Document";
        };
        patchDocument.prototype.isCollectionPatch = function () {
            return this.patchOnOption() === "Collection";
        };
        patchDocument.prototype.isIndexPatch = function () {
            return this.patchOnOption() === "Index";
        };
        patchDocument.prototype.createParameter = function () {
            this.parameters.push(patchParam.empty());
        };
        patchDocument.prototype.removeParameter = function (key) {
            this.parameters.remove(key);
        };
        patchDocument.prototype.name = function () {
            return this.__metadata.id.replace('Studio/Patch/', '');
        };
        patchDocument.prototype.resetMetadata = function () {
            this.__metadata = new documentMetadata();
            this.__metadata.ravenEntityName = 'PatchDocuments';
        };
        patchDocument.prototype.clone = function () {
            return new patchDocument(this.toDto());
        };
        return patchDocument;
    })(document);
    return patchDocument;
});
define("models/database/patch/patchDocuments", ["require", "exports", "models/database/patch/patchParam"], function (require, exports, patchParam) {
    var patchDocuments = (function () {
        function patchDocuments(dto) {
            this.patchOnOption = ko.observable();
            this.selectedItem = ko.observable();
            this.query = ko.observable();
            this.script = ko.observable();
            this.parameters = ko.observableArray();
            this.patchOnOption(dto.PatchOnOption);
            this.query(dto.Query);
            this.script(dto.Script);
            this.selectedItem(dto.SelectedItem);
            this.parameters(dto.Values.map(function (val) { return new patchParam(val); }));
        }
        patchDocuments.empty = function () {
            return new patchDocuments({
                PatchOnOption: "Document",
                Query: "",
                Script: "",
                SelectedItem: "",
                Values: []
            });
        };
        patchDocuments.prototype.isDocumentPatch = function () {
            return this.patchOnOption() === "Document";
        };
        patchDocuments.prototype.isCollectionPatch = function () {
            return this.patchOnOption() === "Collection";
        };
        patchDocuments.prototype.isIndexPatch = function () {
            return this.patchOnOption() === "Index";
        };
        patchDocuments.prototype.createParameter = function () {
            this.parameters.push(patchParam.empty());
        };
        patchDocuments.prototype.removeParameter = function (key) {
            this.parameters.remove(key);
        };
        return patchDocuments;
    })();
    return patchDocuments;
});
define("models/database/patch/patchParam", ["require", "exports"], function (require, exports) {
    var patchParam = (function () {
        function patchParam(dto) {
            this.key = ko.observable();
            this.value = ko.observable();
            this.key(dto.Key);
            this.value(dto.Value);
        }
        patchParam.empty = function () {
            return new patchParam({ Key: "", Value: "" });
        };
        patchParam.prototype.toDto = function () {
            return {
                Key: this.key(),
                Value: this.value()
            };
        };
        return patchParam;
    })();
    return patchParam;
});
define("models/database/query/abstractQueryResult", ["require", "exports"], function (require, exports) {
    var abstractQueryResult = (function () {
        function abstractQueryResult(dto) {
            this.includes = dto.Includes;
            this.isStale = dto.IsStale;
            this.indexTimestamp = new Date(dto.IndexTimestamp);
            this.totalResults = dto.TotalResults;
            this.skippedResults = dto.SkippedResults;
            this.indexName = dto.IndexName;
            this.indexEtag = dto.IndexEtag;
            this.resultEtag = dto.ResultEtag;
            this.highlightings = dto.Highlightings;
            this.nonAuthoritativeInformation = dto.NonAuthoritativeInformation;
            this.lastQueryTime = new Date(dto.LastQueryTime);
            this.durationMilliseconds = dto.DurationMilliseconds;
        }
        return abstractQueryResult;
    })();
    return abstractQueryResult;
});
define("models/database/query/dataExplorationRequest", ["require", "exports"], function (require, exports) {
    var dataExplorationRequest = (function () {
        function dataExplorationRequest(dto) {
            this.linq = ko.observable();
            this.collection = ko.observable();
            this.timeoutSeconds = ko.observable();
            this.pageSize = ko.observable();
            this.linq(dto.Linq);
            this.collection(dto.Collection);
            this.timeoutSeconds(dto.TimeoutSeconds);
            this.pageSize(dto.PageSize);
        }
        dataExplorationRequest.prototype.toDto = function () {
            return {
                Linq: this.linq(),
                Collection: this.collection(),
                TimeoutSeconds: this.timeoutSeconds(),
                PageSize: this.pageSize()
            };
        };
        dataExplorationRequest.empty = function () {
            return new dataExplorationRequest({
                Linq: "from result in results \nselect result",
                Collection: null,
                PageSize: 1000,
                TimeoutSeconds: 60
            });
        };
        return dataExplorationRequest;
    })();
    return dataExplorationRequest;
});
define("models/database/query/facet", ["require", "exports"], function (require, exports) {
    var facet = (function () {
        function facet(dto) {
            var _this = this;
            this.aggregation = ko.observable(); // None = 0, Count = 1, Max = 2, Min = 4, Average = 8, Sum = 16
            this.aggregationLabel = ko.computed(function () { return facet.getLabelForAggregation(_this.aggregation()); });
            this.aggregation(dto.Aggregation);
            this.aggregationField = dto.AggregationField;
            this.aggregationType = dto.AggregationType;
            this.displayName = dto.DisplayName;
            this.includeRemainingTerms = dto.IncludeRemainingTerms;
            this.maxResults = dto.MaxResults;
            this.mode = dto.Mode;
            this.name = dto.Name;
            this.ranges = dto.Ranges;
            this.termSortMode = dto.TermSortMode;
        }
        facet.prototype.toDto = function () {
            return {
                Aggregation: this.aggregation(),
                AggregationField: this.aggregationField,
                AggregationType: this.aggregationType,
                DisplayName: this.displayName,
                IncludeRemainingTerms: this.includeRemainingTerms,
                MaxResults: this.maxResults,
                Mode: this.mode,
                Name: this.name,
                Ranges: this.ranges,
                TermSortMode: this.termSortMode
            };
        };
        facet.fromNameAndAggregation = function (name, aggregationField, aggregationType) {
            var dto = {
                Aggregation: 0,
                AggregationField: aggregationField,
                AggregationType: aggregationType,
                DisplayName: name + "-" + aggregationField,
                IncludeRemainingTerms: false,
                MaxResults: null,
                Mode: 0,
                Name: name,
                Ranges: [],
                TermSortMode: 0
            };
            return new facet(dto);
        };
        facet.getLabelForAggregation = function (aggregation) {
            // None = 0, Count = 1, Max = 2, Min = 4, Average = 8, Sum = 16
            return aggregation === 1 ? "Count" :
                aggregation === 2 ? "Max" :
                    aggregation === 4 ? "Min" :
                        aggregation === 8 ? "Average" :
                            aggregation === 16 ? "Sum" :
                                "None";
        };
        facet.prototype.setAggregationToCount = function () {
            this.aggregation(1);
        };
        facet.prototype.setAggregationToMax = function () {
            this.aggregation(2);
        };
        facet.prototype.setAggregationToMin = function () {
            this.aggregation(4);
        };
        facet.prototype.setAggregationToAverage = function () {
            this.aggregation(8);
        };
        facet.prototype.setAggregationToSum = function () {
            this.aggregation(16);
        };
        return facet;
    })();
    return facet;
});
define("models/database/query/querySort", ["require", "exports"], function (require, exports) {
    var querySort = (function () {
        function querySort() {
            var _this = this;
            this.fieldName = ko.observable();
            this.isRange = ko.observable(false);
            this.isAscending = ko.observable(true);
            this.fieldNameOrDefault = ko.computed(function () { return _this.fieldName() ? _this.fieldName() : "Select a field"; });
        }
        querySort.prototype.toggleAscending = function () {
            this.isAscending.toggle();
        };
        querySort.prototype.toggleRange = function () {
            this.isRange.toggle();
        };
        querySort.prototype.toQuerySortString = function () {
            var querySortString;
            if (this.isRange()) {
                if (this.isAscending()) {
                    querySortString = this.fieldName() + querySort.rangeIndicator; //ascending range
                }
                else {
                    querySortString = "-" + this.fieldName() + querySort.rangeIndicator; //descending range
                }
            }
            else {
                if (this.isAscending()) {
                    querySortString = this.fieldName(); // ascending
                }
                else {
                    querySortString = "-" + this.fieldName(); // descending
                }
            }
            return querySortString;
        };
        querySort.prototype.toHumanizedString = function () {
            var str;
            if (this.isRange()) {
                if (this.isAscending()) {
                    str = this.fieldName() + " range"; //ascending range
                }
                else {
                    str = this.fieldName() + " range descending"; //descending range
                }
            }
            else {
                if (this.isAscending()) {
                    str = this.fieldName(); // ascending
                }
                else {
                    str = this.fieldName() + " descending"; // descending
                }
            }
            return str;
        };
        querySort.fromQuerySortString = function (querySortText) {
            var isDescending = querySortText.slice(0, 1) === "-";
            var isRange = querySortText.length > querySort.rangeIndicator.length
                && querySortText.substr(querySortText.length - querySort.rangeIndicator.length) === querySort.rangeIndicator;
            var sortField = querySort.getSortField(querySortText, isDescending, isRange);
            var q = new querySort();
            q.isAscending(!isDescending);
            q.isRange(isRange);
            q.fieldName(sortField);
            return q;
        };
        querySort.getSortField = function (querySortText, isDescending, isRange) {
            var sortField;
            if (isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1 - querySort.rangeIndicator.length);
            }
            else if (isRange && !isDescending) {
                sortField = querySortText.substr(0, querySortText.length - 1 - querySort.rangeIndicator.length);
            }
            else if (!isRange && isDescending) {
                sortField = querySortText.substr(1, querySortText.length - 1);
            }
            else {
                sortField = querySortText;
            }
            return sortField;
        };
        querySort.rangeIndicator = "_Range";
        return querySort;
    })();
    return querySort;
});
define("models/database/replication/conflict", ["require", "exports", "models/database/replication/conflictVersion"], function (require, exports, conflictVersion) {
    var conflict = (function () {
        function conflict(dto) {
            this.id = dto.Id;
            this.conflictDetectedAt = dto.ConflictDetectedAt;
            this.versions = $.map(dto.Versions, function (v) { return new conflictVersion(v); });
        }
        conflict.prototype.getId = function () {
            return this.id;
        };
        conflict.prototype.getUrl = function () {
            return this.id;
        };
        conflict.prototype.getDocumentPropertyNames = function () {
            return ["Id", "conflictDetectedAt", "versions"];
        };
        return conflict;
    })();
    return conflict;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/replication/conflictsInfo", ["require", "exports", "models/database/replication/conflict", "models/database/query/abstractQueryResult"], function (require, exports, conflict, abstractQueryResult) {
    var conflictsInfo = (function (_super) {
        __extends(conflictsInfo, _super);
        function conflictsInfo(dto) {
            _super.call(this, dto);
            this.results = dto.Results.map(function (d) { return new conflict(d); });
        }
        return conflictsInfo;
    })(abstractQueryResult);
    return conflictsInfo;
});
define("models/database/replication/conflictVersion", ["require", "exports"], function (require, exports) {
    var conflictVersion = (function () {
        function conflictVersion(dto) {
            this.id = dto.Id;
            this.sourceId = dto.SourceId;
        }
        return conflictVersion;
    })();
    return conflictVersion;
});
define("models/database/replication/replicationConfig", ["require", "exports"], function (require, exports) {
    var replicationConfig = (function () {
        function replicationConfig(dto) {
            var _this = this;
            this.documentConflictResolution = ko.observable().extend({ required: true });
            this.attachmentConflictResolution = ko.observable().extend({ required: true });
            this.documentConflictResolution(dto.DocumentConflictResolution);
            this.attachmentConflictResolution(dto.AttachmentConflictResolution);
            this.documentConflictResolution.subscribe(function (val) { return _this.attachmentConflictResolution(val); }); // todo: remove that if decided to treat attachments differently
        }
        replicationConfig.prototype.toDto = function () {
            return {
                DocumentConflictResolution: this.documentConflictResolution(),
                AttachmentConflictResolution: this.attachmentConflictResolution()
            };
        };
        replicationConfig.prototype.clear = function () {
            this.documentConflictResolution("None");
            this.attachmentConflictResolution("None");
        };
        replicationConfig.DO_NOT_RESOLVE_AUTOMATICALLY = "None";
        replicationConfig.RESOLVE_TO_LOCAL = "ResolveToLocal";
        replicationConfig.RESOLVE_TO_REMOTE = "ResolveToRemote";
        replicationConfig.RESOLVE_TO_LATEST = "ResolveToLatest";
        return replicationConfig;
    })();
    return replicationConfig;
});
define("models/database/replication/replicationDestination", ["require", "exports", "models/database/replication/replicationPatchScript"], function (require, exports, replicationPatchScript) {
    var replicationDestination = (function () {
        function replicationDestination(dto) {
            var _this = this;
            this.url = ko.observable().extend({ required: true });
            this.username = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.domain = ko.observable().extend({ required: true });
            this.apiKey = ko.observable().extend({ required: true });
            this.database = ko.observable().extend({ required: true });
            this.transitiveReplicationBehavior = ko.observable().extend({ required: true });
            this.ignoredClient = ko.observable().extend({ required: true });
            this.disabled = ko.observable().extend({ required: true });
            this.clientVisibleUrl = ko.observable().extend({ required: true });
            this.skipIndexReplication = ko.observable().extend({ required: true });
            this.hasGlobal = ko.observable(false);
            this.hasLocal = ko.observable(true);
            this.globalConfiguration = ko.observable();
            this.specifiedCollections = ko.observableArray().extend({ required: false });
            this.specifiedCollectionsNames = ko.observableArray([]);
            this.withScripts = ko.observableArray([]);
            this.enableReplicateOnlyFromCollections = ko.observable();
            this.name = ko.computed(function () {
                var prefix = _this.disabled() ? "[disabled]" : null;
                var database = _this.database();
                var on = _this.database() && _this.url() ? "on" : null;
                var url = _this.url();
                return [prefix, database, on, url]
                    .filter(function (s) { return !!s; })
                    .join(" ")
                    || "[empty]";
            });
            this.isValid = ko.computed(function () { return _this.url() != null && _this.url().length > 0; });
            this.canEdit = ko.computed(function () { return _this.hasLocal(); });
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.url(dto.Url);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.database(dto.Database);
            this.transitiveReplicationBehavior(dto.TransitiveReplicationBehavior);
            this.ignoredClient(dto.IgnoredClient);
            this.disabled(dto.Disabled);
            this.clientVisibleUrl(dto.ClientVisibleUrl);
            this.skipIndexReplication(dto.SkipIndexReplication);
            this.specifiedCollections(this.mapSpecifiedCollections(dto.SpecifiedCollections));
            this.specifiedCollectionsNames(this.specifiedCollections().map(function (x) { return x.collection(); }));
            this.withScripts(this.specifiedCollections().filter(function (x) { return typeof (x.script()) !== "undefined"; }).map(function (x) { return x.collection(); }));
            this.enableReplicateOnlyFromCollections = ko.observable(this.specifiedCollections().length > 0);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
            this.skipIndexReplication.subscribe(function () { return ko.postbox.publish('skip-index-replication'); });
            this.hasGlobal(dto.HasGlobal);
            this.hasLocal(dto.HasLocal);
        }
        replicationDestination.prototype.clearApiKeyCredentials = function () {
            this.apiKey(null);
        };
        replicationDestination.prototype.clearUserCredentials = function () {
            this.username(null);
            this.password(null);
            this.domain(null);
        };
        replicationDestination.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        replicationDestination.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        replicationDestination.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearUserCredentials();
            this.clearApiKeyCredentials();
        };
        replicationDestination.prototype.toggleIsAdvancedShows = function (item, event) {
            $(event.target).next().toggle();
        };
        replicationDestination.prototype.mapSpecifiedCollections = function (input) {
            var result = [];
            if (!input) {
                return result;
            }
            for (var key in input) {
                if (input.hasOwnProperty(key)) {
                    var item = replicationPatchScript.empty();
                    item.collection(key);
                    var script = input[key];
                    if (script === null) {
                        script = undefined;
                    }
                    item.script(script);
                    result.push(item);
                }
            }
            return result;
        };
        replicationDestination.prototype.isSelected = function (col) {
            var cols = this.specifiedCollections();
            return !!cols.first(function (c) { return c.collection() === col.name; });
        };
        replicationDestination.prototype.hasScript = function (col) {
            var cols = this.specifiedCollections();
            var item = cols.first(function (c) { return c.collection() === col.name; });
            if (!item) {
                return false;
            }
            return typeof (item.script()) !== "undefined";
        };
        replicationDestination.prototype.toggleSkipIndexReplication = function () {
            this.skipIndexReplication.toggle();
        };
        replicationDestination.empty = function (databaseName) {
            return new replicationDestination({
                Url: location.protocol + "//" + location.host,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null,
                Database: databaseName,
                TransitiveReplicationBehavior: "Replicate",
                IgnoredClient: false,
                Disabled: false,
                ClientVisibleUrl: null,
                SkipIndexReplication: false,
                SpecifiedCollections: {},
                HasGlobal: false,
                HasLocal: true
            });
        };
        replicationDestination.prototype.enable = function () {
            this.disabled(false);
        };
        replicationDestination.prototype.disable = function () {
            this.disabled(true);
        };
        replicationDestination.prototype.includeFailover = function () {
            this.ignoredClient(false);
        };
        replicationDestination.prototype.skipFailover = function () {
            this.ignoredClient(true);
        };
        replicationDestination.prototype.toDto = function () {
            return {
                Url: this.prepareUrl(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                Database: this.database(),
                TransitiveReplicationBehavior: this.transitiveReplicationBehavior(),
                IgnoredClient: this.ignoredClient(),
                Disabled: this.disabled(),
                ClientVisibleUrl: this.clientVisibleUrl(),
                SkipIndexReplication: this.skipIndexReplication(),
                SpecifiedCollections: this.enableReplicateOnlyFromCollections() ? this.specifiedCollectionsToMap() : {}
            };
        };
        replicationDestination.prototype.specifiedCollectionsToMap = function () {
            var result = {};
            var collections = this.specifiedCollections();
            for (var i = 0; i < collections.length; i++) {
                var item = collections[i];
                var script = item.script();
                if (!script) {
                    script = null;
                }
                result[item.collection()] = script;
            }
            return result;
        };
        replicationDestination.prototype.prepareUrl = function () {
            var url = this.url();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        replicationDestination.prototype.findEditor = function (coll) {
            var collections = this.specifiedCollections();
            var item = collections.first(function (c) { return c.collection() === coll.name; });
            return item.script;
        };
        replicationDestination.prototype.copyFromGlobal = function () {
            if (this.globalConfiguration()) {
                var gConfig = this.globalConfiguration();
                this.url(gConfig.url());
                this.username(gConfig.username());
                this.password(gConfig.password());
                this.domain(gConfig.domain());
                this.apiKey(gConfig.apiKey());
                this.database(gConfig.database());
                this.transitiveReplicationBehavior(gConfig.transitiveReplicationBehavior());
                this.ignoredClient(gConfig.ignoredClient());
                this.disabled(gConfig.disabled());
                this.clientVisibleUrl(gConfig.clientVisibleUrl());
                this.skipIndexReplication(gConfig.skipIndexReplication());
                this.hasGlobal(true);
                this.hasLocal(false);
                this.isUserCredentials(gConfig.isUserCredentials());
                this.isApiKeyCredentials(gConfig.isApiKeyCredentials());
            }
        };
        return replicationDestination;
    })();
    return replicationDestination;
});
define("models/database/replication/replicationPatchScript", ["require", "exports"], function (require, exports) {
    var replicationPatchScript = (function () {
        function replicationPatchScript(collection, script) {
            var _this = this;
            this.collection = ko.observable();
            this.script = ko.observable();
            this.collectionWithLabel = ko.computed(function () { return _this.collection() || "Select collection"; });
            this.collection(collection);
            this.script(script);
        }
        replicationPatchScript.empty = function () {
            return new replicationPatchScript(undefined, undefined);
        };
        return replicationPatchScript;
    })();
    return replicationPatchScript;
});
define("models/database/replication/replicationsSetup", ["require", "exports", "models/database/replication/replicationDestination"], function (require, exports, replicationDestination) {
    var replicationsSetup = (function () {
        function replicationsSetup(dto) {
            var _this = this;
            this.source = ko.observable();
            this.destinations = ko.observableArray().extend({ required: true });
            this.clientFailoverBehaviour = ko.observable(null);
            this.requestTimeSlaThreshold = ko.observable(null);
            this.readFromAllAllowWriteToSecondaries = ko.computed(function () {
                var behaviour = _this.clientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return tokens.contains("ReadFromAllServers") && tokens.contains("AllowReadsFromSecondariesAndWritesToSecondaries");
            });
            this.readFromAllButSwitchWhenRequestTimeSlaThresholdIsReached = ko.computed(function () {
                var behaviour = _this.clientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return tokens.contains("ReadFromAllServers") && tokens.contains("AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
            });
            this.source(dto.MergedDocument.Source);
            this.destinations(dto.MergedDocument.Destinations.map(function (dest) {
                var result = new replicationDestination(dest);
                if (dto.GlobalDocument) {
                    var foundParent = dto.GlobalDocument.Destinations
                        .first(function (x) { return x.Url.toLowerCase() === dest.Url.toLowerCase() && x.Database.toLowerCase() === dest.Database.toLowerCase(); });
                    if (foundParent) {
                        result.globalConfiguration(new replicationDestination(foundParent));
                    }
                }
                return result;
            }));
            var clientConfiguration = dto.MergedDocument.ClientConfiguration;
            if (clientConfiguration) {
                if (clientConfiguration.FailoverBehavior) {
                    this.clientFailoverBehaviour(clientConfiguration.FailoverBehavior);
                }
                if (clientConfiguration.RequestTimeSlaThresholdInMilliseconds) {
                    this.requestTimeSlaThreshold(clientConfiguration.RequestTimeSlaThresholdInMilliseconds);
                }
            }
            this.showRequestTimeSlaThreshold = ko.computed(function () {
                return _this.clientFailoverBehaviour() && _this.clientFailoverBehaviour().contains("AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
            });
            this.clientFailoverBehaviour.subscribe(function (newValue) {
                if (!newValue.contains('AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached')) {
                    _this.requestTimeSlaThreshold(undefined);
                }
                else if (!_this.requestTimeSlaThreshold()) {
                    _this.requestTimeSlaThreshold(100);
                }
            });
        }
        replicationsSetup.prototype.toDto = function (filterLocal) {
            if (filterLocal === void 0) { filterLocal = true; }
            var dto = {
                Destinations: this.destinations().filter(function (dest) { return !filterLocal || dest.hasLocal(); }).map(function (dest) { return dest.toDto(); }),
                Source: this.source()
            };
            dto.ClientConfiguration = {
                RequestTimeSlaThresholdInMilliseconds: undefined,
                FailoverBehavior: undefined
            };
            if (this.clientFailoverBehaviour()) {
                dto.ClientConfiguration.FailoverBehavior = this.clientFailoverBehaviour();
            }
            if (this.showRequestTimeSlaThreshold()) {
                dto.ClientConfiguration.RequestTimeSlaThresholdInMilliseconds = this.requestTimeSlaThreshold();
            }
            return dto;
        };
        replicationsSetup.prototype.copyFromParent = function (parentClientFailover, parentRequestTimeSlaThreshold) {
            this.clientFailoverBehaviour(parentClientFailover);
            this.requestTimeSlaThreshold(parentRequestTimeSlaThreshold);
            this.destinations(this.destinations().filter(function (d) { return d.hasGlobal(); }));
            this.destinations().forEach(function (d) { return d.copyFromGlobal(); });
        };
        replicationsSetup.prototype.clear = function () {
            this.destinations.removeAll();
            this.clientFailoverBehaviour(null);
            this.source(null);
        };
        return replicationsSetup;
    })();
    return replicationsSetup;
});
define("models/database/replication/topology", ["require", "exports", "models/database/cluster/nodeConnectionInfo"], function (require, exports, nodeConnectionInfo) {
    var topology = (function () {
        function topology(dto) {
            var _this = this;
            this.currentLeader = ko.observable();
            this.currentTerm = ko.observable();
            this.state = ko.observable();
            this.commitIndex = ko.observable();
            this.allNodes = ko.observableArray();
            this.allVotingNodes = ko.observableArray();
            this.promotableNodes = ko.observableArray();
            this.nonVotingNodes = ko.observableArray();
            this.topologyId = ko.observable();
            this.clusterMode = ko.computed(function () {
                var voting = _this.allVotingNodes().length;
                var promatable = _this.promotableNodes().length;
                var nonVoting = _this.nonVotingNodes().length;
                return voting + promatable + nonVoting > 0;
            });
            this.currentLeader(dto.CurrentLeader);
            this.currentTerm(dto.CurrentTerm);
            this.state(dto.State);
            this.commitIndex(dto.CommitIndex);
            dto.AllVotingNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("voting");
                _this.allVotingNodes.push(nci);
                _this.allNodes.push(nci);
            });
            dto.PromotableNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("promotable");
                _this.promotableNodes.push(nci);
                _this.allNodes.push(nci);
            });
            dto.NonVotingNodes.map(function (n) {
                var nci = new nodeConnectionInfo(n);
                nci.state("non-voting");
                _this.nonVotingNodes.push(nci);
                _this.allNodes.push(nci);
            });
            var leader = this.currentLeader();
            if (leader) {
                var leaderNode = this.allNodes.first(function (n) { return n.name() === leader; });
                if (leaderNode) {
                    leaderNode.state("leader");
                }
            }
            this.topologyId(dto.TopologyId);
        }
        return topology;
    })();
    return topology;
});
define("models/database/sqlReplication/predefinedSqlConnection", ["require", "exports", "durandal/app", "commands/database/sqlReplication/testSqlConnectionCommand"], function (require, exports, app, testSqlConnectionCommand) {
    var predefinedSqlConnection = (function () {
        function predefinedSqlConnection(dto) {
            var _this = this;
            this.name = ko.observable();
            this.factoryName = ko.observable();
            this.connectionString = ko.observable();
            this.connectionTestState = ko.observable(this.CONNECTION_STATE_STAND_BY);
            this.hasGlobal = ko.observable(false);
            this.hasLocal = ko.observable(true);
            this.globalConfiguration = ko.observable();
            this.canEdit = ko.computed(function () { return _this.hasLocal(); });
            this.CONNECTION_STATE_STAND_BY = "stand-by";
            this.CONNECTION_STATE_CONNECTING = "connecting";
            this.CONNECTION_STATE_CONNECTED = "connected";
            this.name(dto.Name);
            this.factoryName(dto.FactoryName);
            this.connectionString(dto.ConnectionString);
            this.connectionTestState = ko.observable(this.CONNECTION_STATE_STAND_BY);
            this.hasGlobal(dto.HasGlobal);
            this.hasLocal(dto.HasLocal);
        }
        predefinedSqlConnection.empty = function () {
            return new predefinedSqlConnection({
                Name: "",
                FactoryName: "",
                ConnectionString: "",
                HasGlobal: false,
                HasLocal: true
            });
        };
        predefinedSqlConnection.prototype.toDto = function () {
            return {
                Name: this.name(),
                FactoryName: this.factoryName(),
                ConnectionString: this.connectionString()
            };
        };
        predefinedSqlConnection.prototype.testConnection = function (dbObservable) {
            var _this = this;
            this.connectionTestState(this.CONNECTION_STATE_CONNECTING);
            new testSqlConnectionCommand(dbObservable(), this.factoryName(), this.connectionString())
                .execute()
                .done(function () {
                _this.connectionTestState(_this.CONNECTION_STATE_CONNECTED);
                app.showMessage("Connection " + _this.name() + " is valid", "SQL Connection test");
            })
                .fail(function (request, status, error) {
                var errorText = !!request.responseJSON ? !!request.responseJSON.Exception ? request.responseJSON.Exception.Message : error : error;
                app.showMessage("Connection " + _this.name() + " is not valid, error: " + errorText, "SQL Connection test");
            }).
                always(function () { return setTimeout(_this.connectionTestState, 500, _this.CONNECTION_STATE_STAND_BY); });
        };
        predefinedSqlConnection.prototype.copyFromGlobal = function () {
            if (this.globalConfiguration()) {
                var gConfig = this.globalConfiguration();
                this.name(gConfig.name());
                this.factoryName(gConfig.factoryName());
                this.connectionString(gConfig.connectionString());
                this.connectionTestState(gConfig.connectionTestState());
                this.hasGlobal(true);
                this.hasLocal(false);
            }
        };
        return predefinedSqlConnection;
    })();
    return predefinedSqlConnection;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/sqlReplication/sqlReplication", ["require", "exports", "models/database/sqlReplication/sqlReplicationTable", "models/database/documents/document", "models/database/documents/documentMetadata", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, sqlReplicationTable, document, documentMetadata, aceEditorBindingHandler) {
    var sqlReplication = (function (_super) {
        __extends(sqlReplication, _super);
        function sqlReplication(dto) {
            var _this = this;
            _super.call(this, dto);
            this.CONNECTION_STRING = "Connection String";
            this.PREDEFINED_CONNECTION_STRING_NAME = "Predefined Connection String Name";
            this.CONNECTION_STRING_NAME = "Connection String Name";
            this.CONNECTION_STRING_SETTING_NAME = "Connection String Setting Name";
            this.availableConnectionStringTypes = [
                this.PREDEFINED_CONNECTION_STRING_NAME,
                this.CONNECTION_STRING,
                this.CONNECTION_STRING_NAME,
                this.CONNECTION_STRING_SETTING_NAME
            ];
            this.name = ko.observable().extend({ required: true });
            this.disabled = ko.observable().extend({ required: true });
            this.factoryName = ko.observable().extend({ required: true });
            this.connectionStringType = ko.observable().extend({ required: true });
            this.connectionStringValue = ko.observable(null).extend({ required: true });
            this.ravenEntityName = ko.observable("").extend({ required: true });
            this.parameterizeDeletesDisabled = ko.observable(false).extend({ required: true });
            this.forceSqlServerQueryRecompile = ko.observable(false);
            this.quoteTables = ko.observable(true);
            this.sqlReplicationTables = ko.observableArray().extend({ required: true });
            this.script = ko.observable("").extend({ required: true });
            this.connectionString = ko.observable(null);
            this.connectionStringName = ko.observable(null);
            this.connectionStringSettingName = ko.observable(null);
            this.collections = ko.observableArray();
            this.showReplicationConfiguration = ko.observable(false);
            this.hasAnyInsertOnlyOption = ko.computed(function () {
                var hasAny = false;
                _this.sqlReplicationTables().forEach(function (s) {
                    // don't use return here to register all deps in knockout
                    if (s.insertOnly()) {
                        hasAny = true;
                    }
                });
                return hasAny;
            });
            this.name(dto.Name);
            this.disabled(dto.Disabled);
            this.factoryName(dto.FactoryName);
            this.ravenEntityName(dto.RavenEntityName != null ? dto.RavenEntityName : "");
            this.parameterizeDeletesDisabled(dto.ParameterizeDeletesDisabled);
            this.sqlReplicationTables(dto.SqlReplicationTables.map(function (tab) { return new sqlReplicationTable(tab); }));
            this.script(dto.Script);
            this.forceSqlServerQueryRecompile(!!dto.ForceSqlServerQueryRecompile ? dto.ForceSqlServerQueryRecompile : false);
            this.quoteTables(("PerformTableQuatation" in dto) ? dto.PerformTableQuatation : ("QuoteTables" in dto ? dto.QuoteTables : true));
            this.setupConnectionString(dto);
            this.metadata = new documentMetadata(dto["@metadata"]);
            this.connectionStringSourceFieldName = ko.computed(function () {
                if (_this.connectionStringType() == _this.CONNECTION_STRING) {
                    return "Connection String Text";
                }
                else if (_this.connectionStringType() == _this.PREDEFINED_CONNECTION_STRING_NAME) {
                    return "Predefined connection string name";
                }
                else if (_this.connectionStringType() == _this.CONNECTION_STRING_NAME) {
                    return "Setting name in local machine configuration";
                }
                else {
                    return "Setting name in memory/remote configuration";
                }
            });
            this.searchResults = ko.computed(function () {
                var newRavenEntityName = _this.ravenEntityName();
                return _this.collections().filter(function (name) { return name.toLowerCase().indexOf(newRavenEntityName.toLowerCase()) > -1; });
            });
            this.script.subscribe(function (newValue) {
                var message = "";
                var currentEditor = aceEditorBindingHandler.currentEditor;
                var textarea = $(currentEditor.container).find("textarea")[0];
                if (newValue === "") {
                    message = "Please fill out this field.";
                }
                textarea.setCustomValidity(message);
                setTimeout(function () {
                    var annotations = currentEditor.getSession().getAnnotations();
                    var isErrorExists = false;
                    for (var i = 0; i < annotations.length; i++) {
                        var annotationType = annotations[i].type;
                        if (annotationType === "error" || annotationType === "warning") {
                            isErrorExists = true;
                            break;
                        }
                    }
                    if (isErrorExists) {
                        message = "The script isn't a javascript legal expression!";
                        textarea.setCustomValidity(message);
                    }
                }, 700);
            });
        }
        sqlReplication.prototype.setupConnectionString = function (dto) {
            if (dto.ConnectionStringName) {
                this.connectionStringType(this.CONNECTION_STRING_NAME);
                this.connectionStringValue(dto.ConnectionStringName);
            }
            else if (dto.ConnectionStringSettingName) {
                this.connectionStringType(this.CONNECTION_STRING_SETTING_NAME);
                this.connectionStringValue(dto.ConnectionStringSettingName);
            }
            else if (dto.ConnectionString) {
                this.connectionStringType(this.CONNECTION_STRING);
                this.connectionStringValue(dto.ConnectionString);
            }
            else {
                this.connectionStringType(this.PREDEFINED_CONNECTION_STRING_NAME);
                this.connectionStringValue(dto.PredefinedConnectionStringSettingName);
            }
        };
        sqlReplication.prototype.setConnectionStringType = function (strType) {
            this.connectionStringType(strType);
        };
        sqlReplication.empty = function () {
            return new sqlReplication({
                Name: "",
                Disabled: true,
                ParameterizeDeletesDisabled: false,
                RavenEntityName: "",
                Script: "",
                FactoryName: null,
                ConnectionString: null,
                PredefinedConnectionStringSettingName: null,
                ConnectionStringName: null,
                ConnectionStringSettingName: null,
                SqlReplicationTables: [sqlReplicationTable.empty().toDto()],
                ForceSqlServerQueryRecompile: false,
                QuoteTables: true
            });
        };
        sqlReplication.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            meta["@id"] = "Raven/SqlReplication/Configuration/" + this.name();
            return {
                '@metadata': meta,
                Name: this.name(),
                Disabled: this.disabled(),
                ParameterizeDeletesDisabled: this.parameterizeDeletesDisabled(),
                RavenEntityName: this.ravenEntityName(),
                Script: this.script(),
                FactoryName: this.factoryName(),
                ConnectionString: this.prepareConnectionString(this.CONNECTION_STRING),
                PredefinedConnectionStringSettingName: this.prepareConnectionString(this.PREDEFINED_CONNECTION_STRING_NAME),
                ConnectionStringName: this.prepareConnectionString(this.CONNECTION_STRING_NAME),
                ConnectionStringSettingName: this.prepareConnectionString(this.CONNECTION_STRING_SETTING_NAME),
                ForceSqlServerQueryRecompile: this.forceSqlServerQueryRecompile(),
                QuoteTables: this.quoteTables(),
                SqlReplicationTables: this.sqlReplicationTables().map(function (tab) { return tab.toDto(); })
            };
        };
        sqlReplication.prototype.prepareConnectionString = function (expectedType) {
            return ((this.connectionStringType() === expectedType) ? this.connectionStringValue() : null);
        };
        sqlReplication.prototype.enable = function () {
            this.disabled(false);
        };
        sqlReplication.prototype.disable = function () {
            this.disabled(true);
        };
        sqlReplication.prototype.enableParameterizeDeletes = function () {
            this.parameterizeDeletesDisabled(false);
        };
        sqlReplication.prototype.disableParameterizeDeletes = function () {
            this.parameterizeDeletesDisabled(true);
        };
        sqlReplication.prototype.addNewTable = function () {
            this.sqlReplicationTables.push(sqlReplicationTable.empty());
        };
        sqlReplication.prototype.removeTable = function (table) {
            this.sqlReplicationTables.remove(table);
        };
        sqlReplication.prototype.setIdFromName = function () {
            this.__metadata.id = "Raven/SqlReplication/Configuration/" + this.name();
        };
        sqlReplication.prototype.saveNewRavenEntityName = function (newRavenEntityName) {
            this.ravenEntityName(newRavenEntityName);
        };
        sqlReplication.prototype.isSqlServerKindOfFactory = function (factoryName) {
            if (factoryName == "System.Data.SqlClient" || factoryName == "System.Data.SqlServerCe.4.0" || factoryName == "System.Data.SqlServerCe.3.5") {
                return true;
            }
            return false;
        };
        return sqlReplication;
    })(document);
    return sqlReplication;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/database/sqlReplication/sqlReplicationConnections", ["require", "exports", "models/database/documents/document", "models/database/sqlReplication/predefinedSqlConnection"], function (require, exports, document, predefinedConnection) {
    var sqlReplicationConnections = (function (_super) {
        __extends(sqlReplicationConnections, _super);
        function sqlReplicationConnections(dto) {
            _super.call(this, dto);
            this.predefinedConnections = ko.observableArray();
            this.predefinedConnections(dto.MergedDocument.PredefinedConnections.map(function (c) {
                var result = new predefinedConnection(c);
                if (dto.GlobalDocument) {
                    var foundParent = dto.GlobalDocument.PredefinedConnections.first(function (x) { return x.Name.toLowerCase() === c.Name.toLowerCase(); });
                    if (foundParent) {
                        result.globalConfiguration(new predefinedConnection(foundParent));
                    }
                }
                return result;
            }));
        }
        sqlReplicationConnections.empty = function () {
            return new sqlReplicationConnections({
                MergedDocument: {
                    PredefinedConnections: []
                },
                LocalExists: true,
                GlobalExists: false
            });
        };
        sqlReplicationConnections.prototype.toDto = function (filterLocal) {
            if (filterLocal === void 0) { filterLocal = true; }
            var meta = this.__metadata.toDto();
            meta["@id"] = "Raven/SqlReplication/Connections/";
            return {
                PredefinedConnections: this.predefinedConnections().filter(function (dest) { return !filterLocal || dest.hasLocal(); }).map(function (x) { return x.toDto(); })
            };
        };
        sqlReplicationConnections.prototype.copyFromParent = function () {
            this.predefinedConnections(this.predefinedConnections().filter(function (c) { return c.hasGlobal(); }));
            this.predefinedConnections().forEach(function (c) { return c.copyFromGlobal(); });
        };
        sqlReplicationConnections.sqlProvidersConnectionStrings = [
            { ProviderName: "System.Data.SqlClient", ConnectionString: "Server=[Server Address];Database=[Database Name];User Id=[User ID];Password=[Password];" },
            { ProviderName: "System.Data.SqlServerCe.4.0", ConnectionString: "Data Source=[path of .sdf file];Persist Security Info=False;" },
            { ProviderName: "System.Data.SqlServerCe.3.5", ConnectionString: "Provider=Microsoft.SQLSERVER.CE.OLEDB.3.5;Data Source=[path of .sdf file];" },
            { ProviderName: "System.Data.OleDb", ConnectionString: "" },
            { ProviderName: "System.Data.OracleClient", ConnectionString: "Data Source=[TNSNames name];User Id=[User ID];Password=[Password];Integrated Security=no;" },
            { ProviderName: "MySql.Data.MySqlClient", ConnectionString: "Server=[Server Address];Port=[Server Port(default:3306)];Database=[Database Name];Uid=[User ID];Pwd=[Password];" },
            { ProviderName: "Npgsql", ConnectionString: "Server=[Server Address];Port=[Port];Database=[Database Name];User Id=[User ID];Password=[Password];" }
        ];
        return sqlReplicationConnections;
    })(document);
    return sqlReplicationConnections;
});
define("models/database/sqlReplication/sqlReplicationSimulatedCommand", ["require", "exports"], function (require, exports) {
    var sqlReplicationSimulatedCommand = (function () {
        function sqlReplicationSimulatedCommand(showParamsValues, commandData) {
            var _this = this;
            this.showParamsValues = ko.observable(false);
            this.commandData = ko.observable();
            this.showParamsValues(showParamsValues);
            this.commandData(commandData);
            this.commandText = ko.computed(function () {
                if (_this.showParamsValues() && !!_this.commandData().Params) {
                    var processedCommandText = _this.commandData().CommandText.slice(0);
                    _this.commandData().Params.forEach(function (x) {
                        if (x.Key.slice(0, 1) === '@') {
                            processedCommandText = processedCommandText.replace(x.Key, x.Value);
                        }
                        else {
                            processedCommandText = processedCommandText.replace('@' + x.Key, x.Value);
                        }
                    });
                    return processedCommandText;
                }
                else {
                    return _this.commandData().CommandText;
                }
            });
        }
        return sqlReplicationSimulatedCommand;
    })();
    return sqlReplicationSimulatedCommand;
});
define("models/database/sqlReplication/sqlReplicationTable", ["require", "exports"], function (require, exports) {
    var sqlReplicationTable = (function () {
        function sqlReplicationTable(dto) {
            this.tableName = ko.observable().extend({ required: true });
            this.documentKeyColumn = ko.observable().extend({ required: true });
            this.insertOnly = ko.observable(false);
            this.tableName(dto.TableName);
            this.documentKeyColumn(dto.DocumentKeyColumn);
            this.insertOnly(dto.InsertOnlyMode);
        }
        sqlReplicationTable.empty = function () {
            return new sqlReplicationTable({
                TableName: null,
                DocumentKeyColumn: null,
                InsertOnlyMode: false
            });
        };
        sqlReplicationTable.prototype.toDto = function () {
            return {
                TableName: this.tableName(),
                DocumentKeyColumn: this.documentKeyColumn(),
                InsertOnlyMode: this.insertOnly()
            };
        };
        return sqlReplicationTable;
    })();
    return sqlReplicationTable;
});
define("models/database/subscription/subscription", ["require", "exports"], function (require, exports) {
    var Subscription = (function () {
        function Subscription(id, etag) {
            var _this = this;
            this.id = id;
            this.etag = etag;
            this.ackEtag = ko.observable();
            this.newAckEtag = ko.observable();
            this.isChangeInProgress = ko.observable();
            this.subscriptionId = id;
            this.ackEtag(etag);
            this.newAckEtag(etag);
            this.isButtonDisabled = ko.computed(function () { return _this.ackEtag() === _this.newAckEtag() || _this.isChangeInProgress(); });
            this.etagCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var currentNewAckEtag = _this.newAckEtag();
                if (currentNewAckEtag.length !== 36) {
                    errorMessage = "Etag length is invalid";
                }
                else if (currentNewAckEtag[8] !== "-" || currentNewAckEtag[13] !== "-" ||
                    currentNewAckEtag[18] !== "-" || currentNewAckEtag[23] !== "-") {
                    errorMessage = "Invalid etag format";
                }
                return errorMessage;
            });
        }
        return Subscription;
    })();
    return Subscription;
});
define("models/filesystem/configurationKey", ["require", "exports", "commands/filesystem/getConfigurationByKeyCommand"], function (require, exports, getConfigurationByKeyCommand) {
    var configurationKey = (function () {
        function configurationKey(fs, key) {
            this.fs = fs;
            this.key = key;
        }
        // Notifies consumers that this configuration key should be the selected one.
        // Called from the UI when a user clicks a configuration key the configuration page.
        configurationKey.prototype.activate = function () {
            ko.postbox.publish("ActivateConfigurationKey", this);
        };
        configurationKey.prototype.getValues = function () {
            return new getConfigurationByKeyCommand(this.fs, this.key).execute();
        };
        return configurationKey;
    })();
    return configurationKey;
});
define("models/filesystem/conflictHistory", ["require", "exports"], function (require, exports) {
    var conflictHistory = (function () {
        function conflictHistory(version, serverId) {
            this.version = version;
            this.serverId = serverId;
        }
        return conflictHistory;
    })();
    return conflictHistory;
});
define("models/filesystem/conflictItem", ["require", "exports", "models/filesystem/conflictHistory"], function (require, exports, conflictHistory) {
    var conflictItem = (function () {
        function conflictItem(fileName, remoteServerUrl) {
            this.fileName = fileName;
            this.remoteServerUrl = remoteServerUrl;
            this.remoteHistory = [];
            this.currentHistory = [];
        }
        conflictItem.fromConflictItemDto = function (dto) {
            var item = new conflictItem(dto.FileName, dto.RemoteServerUrl);
            if (dto.RemoteHistory != null) {
                item.remoteHistory.pushAll(dto.RemoteHistory.map(function (x) { return new conflictHistory(x.Version, x.ServerId); }));
            }
            if (dto.CurrentHistory != null) {
                item.currentHistory.pushAll(dto.CurrentHistory.map(function (x) { return new conflictHistory(x.Version, x.ServerId); }));
            }
            return item;
        };
        conflictItem.fromConflictNotificationDto = function (dto) {
            var item = new conflictItem(dto.FileName, dto.SourceServerUrl);
            return item;
        };
        return conflictItem;
    })();
    return conflictItem;
});
define("models/filesystem/file", ["require", "exports", "models/filesystem/fileMetadata"], function (require, exports, fileMetadata) {
    var file = (function () {
        function file(dto, excludeDirectoryInId) {
            this.id = ko.observable();
            if (dto) {
                if (dto.FullPath && dto.FullPath[0] === "/") {
                    dto.FullPath = dto.FullPath.replace("/", "");
                }
                if (excludeDirectoryInId) {
                    this.id(dto.Name);
                }
                else {
                    this.id(dto.FullPath);
                }
                if (dto.HumaneTotalSize === " Bytes") {
                    dto.HumaneTotalSize = "0 Bytes";
                }
                this.directory = dto.Directory;
                this.fullPath = dto.FullPath;
                this.Size = dto.HumaneTotalSize;
                this.LastModified = dto.Metadata["Last-Modified"];
                this.__metadata = new fileMetadata(dto.Metadata);
            }
        }
        file.prototype.getId = function () {
            return this.id();
        };
        file.prototype.setId = function (newName) {
            this.id(newName);
        };
        file.prototype.getUrl = function () {
            return this.fullPath ? this.fullPath : this.id();
        };
        file.prototype.getDocumentPropertyNames = function () {
            return ["Size", "LastModified"];
        };
        return file;
    })();
    return file;
});
define("models/filesystem/fileMetadata", ["require", "exports"], function (require, exports) {
    var fileMetadata = (function () {
        function fileMetadata(dto) {
            this.standardProps = ["ravenfs-size", "raven-last-modified", "etag", "raven-creation-date"];
            // We don't want to keep certain reserved properties in the metadata text area.    
            this.headerPropsToRemove = ["Origin", "Raven-Server-Build", "Raven-Client-Version", "Non-Authoritative-Information", "Raven-Timer-Request", "Content-Length",
                "Raven-Authenticated-User", "Has-Api-Key", "Access-Control-Allow-Origin", "Access-Control-Max-Age", "Access-Control-Allow-Methods",
                "Access-Control-Request-Headers", "Access-Control-Allow-Headers", "Reverse-Via", "Persistent-Auth", "Allow", "Content-Disposition",
                "Content-Encoding", "Content-Language", "Content-Location", "Content-MD5", "Content-Range", "Content-Type", "Expires", "Keep-Alive",
                "X-Powered-By", "X-AspNet-Version", "X-Requested-With", "X-SourceFiles", "Accept-Charset", "Accept-Encoding", "Accept", "Accept-Language",
                "Authorization", "Cookie", "Expect", "From", "Host", "If-MatTemp-Index-Scorech", "If-Modified-Since", "If-None-Match", "If-Range",
                "If-Unmodified-Since", "Max-Forwards", "Referer", "TE", "User-Agent", "Accept-Ranges", "Age", "Allow", "Location", "Retry-After",
                "Server", "Set-Cookie2", "Set-Cookie", "Vary", "Www-Authenticate", "Cache-Control", "Connection", "Date", "Pragma", "Trailer", "Upgrade",
                "Transfer-Encoding", "Via", "Warning", "X-ARR-LOG-ID", "X-ARR-SSL", "X-Forwarded-For", "X-Original-URL", "Temp-Request-Time", "DNT"];
            if (dto) {
                this.ravenFSSize = dto['RavenFS-Size'];
                this.creationDate = dto['Raven-Creation-Date'];
                this.lastModified = dto['Raven-Last-Modified'];
                this.etag = dto['ETag'];
                if (this.etag == null)
                    this.etag = dto['Etag'];
                if (this.etag.startsWith('"'))
                    this.etag = this.etag.slice(1, this.etag.length - 1);
                // Effectively remove all the headers that are not useful as metadata.
                for (var property in dto) {
                    if (this.headerPropsToRemove.contains(property))
                        delete dto[property];
                }
                for (var property in dto) {
                    if (!this.standardProps.contains(property.toLowerCase())) {
                        this.nonStandardProps = this.nonStandardProps || [];
                        var value = dto[property];
                        this[property] = value;
                        this.nonStandardProps.push(property);
                    }
                }
            }
        }
        fileMetadata.prototype.toDto = function () {
            var _this = this;
            var dto = {
                'Raven-Creation-Date': this.creationDate,
                'Raven-Last-Modified': this.lastModified,
                'ETag': '"' + this.etag + '"',
                'RavenFS-Size': this.ravenFSSize,
            };
            if (this.nonStandardProps) {
                this.nonStandardProps.forEach(function (p) {
                    dto[p] = _this[p];
                });
            }
            return dto;
        };
        return fileMetadata;
    })();
    return fileMetadata;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/filesystem/filesystem", ["require", "exports", "models/resources/resource", "models/auth/license", "models/filesystem/fileSystemStatistics"], function (require, exports, resource, license, fileSystemStatistics) {
    var filesystem = (function (_super) {
        __extends(filesystem, _super);
        function filesystem(name, isAdminCurrentTenant, isDisabled, isLoaded, bundles) {
            var _this = this;
            if (isAdminCurrentTenant === void 0) { isAdminCurrentTenant = true; }
            if (isDisabled === void 0) { isDisabled = false; }
            if (isLoaded === void 0) { isLoaded = false; }
            if (bundles === void 0) { bundles = []; }
            _super.call(this, name, TenantType.FileSystem, isAdminCurrentTenant);
            this.statistics = ko.observable();
            this.files = ko.observableArray();
            this.iconName = "fa fa-fw fa-file-image-o";
            this.fullTypeName = "File System";
            this.disabled(isDisabled);
            this.activeBundles(bundles);
            this.isLoaded(isLoaded);
            this.itemCountText = ko.computed(function () { return !!_this.statistics() ? _this.statistics().fileCountText() : ""; });
            this.isLicensed = ko.computed(function () {
                if (!!license.licenseStatus() && license.licenseStatus().IsCommercial) {
                    var ravenFsValue = license.licenseStatus().Attributes.ravenfs;
                    return /^true$/i.test(ravenFsValue);
                }
                return true;
            });
        }
        filesystem.prototype.activate = function () {
            this.isLoaded(true);
            ko.postbox.publish("ActivateFilesystem", this);
        };
        filesystem.prototype.saveStatistics = function (dto) {
            if (!this.statistics()) {
                this.statistics(new fileSystemStatistics());
            }
            this.statistics().fromDto(dto);
        };
        filesystem.getNameFromUrl = function (url) {
            var index = url.indexOf("filesystems/");
            return (index > 0) ? url.substring(index + 10) : "";
        };
        filesystem.type = "filesystem";
        return filesystem;
    })(resource);
    return filesystem;
});
define("models/filesystem/filesystemCollection", ["require", "exports", "common/pagedList"], function (require, exports, pagedList) {
    var filesystemCollection = (function () {
        function filesystemCollection(name, owner) {
            this.name = name;
            this.owner = owner;
            this.itemsCount = ko.observable(0);
            this.isAllItems = false;
        }
        // Notifies consumers that this collection should be the selected one.
        // Called from the UI when a user clicks a collection the documents page.
        filesystemCollection.prototype.activate = function () {
            ko.postbox.publish("ActivateCollection", this);
        };
        filesystemCollection.prototype.getItems = function () {
            if (!this.itemsList) {
                this.itemsList = this.createPagedList();
            }
            return this.itemsList;
        };
        filesystemCollection.prototype.fetchItems = function (skip, take) {
            throw new Error('This method is abstract');
        };
        filesystemCollection.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchItems(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        return filesystemCollection;
    })();
    return filesystemCollection;
});
define("models/filesystem/fileSystemStatistics", ["require", "exports"], function (require, exports) {
    var fileSystemStatistics = (function () {
        function fileSystemStatistics() {
            this.fileCountText = ko.observable();
            this.requestsCountText = ko.observable();
            this.meanDurationText = ko.observable();
        }
        fileSystemStatistics.prototype.fromDto = function (dto) {
            this.fileCountText(this.getItemCountText(dto.FileCount, "file", "s"));
            this.requestsCountText(dto.Metrics.RequestsPerSecond + " requests per second");
            this.meanDurationText(dto.Metrics.RequestsDuration.Mean.toFixed(1) + "ms mean duration");
        };
        fileSystemStatistics.prototype.getItemCountText = function (itemCount, singularText, suffix) {
            var itemCountText = itemCount.toLocaleString() + " " + singularText;
            if (itemCount !== 1) {
                itemCountText += suffix;
            }
            return itemCountText;
        };
        return fileSystemStatistics;
    })();
    return fileSystemStatistics;
});
define("models/filesystem/folder", ["require", "exports"], function (require, exports) {
    var folder = (function () {
        function folder(path) {
            this.path = path;
            if (path) {
                var lastSlash = path.lastIndexOf("/");
                if (lastSlash > 0) {
                    this.name = path.substring(lastSlash + 1);
                }
                else {
                    this.name = path.substring(1);
                }
            }
        }
        folder.getFolderFromFilePath = function (filePath) {
            var lastSlash = filePath.lastIndexOf("/");
            if (lastSlash == 0) {
                return new folder("/");
            }
            if (lastSlash > 0) {
                return new folder(filePath.substring(0, lastSlash));
            }
            return null;
        };
        folder.prototype.getSubpathsFrom = function (basePath) {
            if (basePath) {
                if (basePath[basePath.length - 1] != "/") {
                    basePath = basePath + "/";
                }
                var restOfPath = this.path.replace(basePath, "");
                var subPaths = [new folder(basePath)];
                var lastIndexOfSlash = restOfPath.indexOf("/");
                for (var i = 0; i < restOfPath.count("/"); i++) {
                    if (lastIndexOfSlash > 0) {
                        subPaths.push(new folder(basePath + restOfPath.substring(0, lastIndexOfSlash)));
                    }
                    lastIndexOfSlash = restOfPath.indexOf("/", lastIndexOfSlash + 1);
                }
                subPaths.push(new folder(this.path));
                return subPaths;
            }
            return null;
        };
        folder.prototype.isFileAtFolderLevel = function (filePath) {
            var fileToFolder = folder.getFolderFromFilePath(filePath);
            return fileToFolder && this.path === fileToFolder.path;
        };
        return folder;
    })();
    return folder;
});
define("models/filesystem/synchronizationConfig", ["require", "exports"], function (require, exports) {
    var synchronizationConfig = (function () {
        function synchronizationConfig(dto) {
            this.fileConflictResolution = ko.observable();
            this.maxNumberOfSynchronizationsPerDestination = ko.observable();
            this.synchronizationLockTimeout = ko.observable();
            this.synchronizationLockTimeoutUnit = ko.observable();
            this.availableIntervalUnits = [synchronizationConfig.TU_MINUTES, synchronizationConfig.TU_HOURS, synchronizationConfig.TU_DAYS];
            this.fileConflictResolution(dto.FileConflictResolution);
            this.maxNumberOfSynchronizationsPerDestination(dto.MaxNumberOfSynchronizationsPerDestination);
            var timeout = this.prepareInterval(dto.SynchronizationLockTimeoutMiliseconds);
            this.synchronizationLockTimeout(timeout[0]);
            this.synchronizationLockTimeoutUnit(timeout[1]);
        }
        synchronizationConfig.empty = function () {
            return new synchronizationConfig({
                FileConflictResolution: "None",
                MaxNumberOfSynchronizationsPerDestination: 5,
                SynchronizationLockTimeoutMiliseconds: 10 * 60 * 1000
            });
        };
        synchronizationConfig.prototype.toDto = function () {
            return {
                FileConflictResolution: this.fileConflictResolution(),
                MaxNumberOfSynchronizationsPerDestination: this.maxNumberOfSynchronizationsPerDestination(),
                SynchronizationLockTimeoutMiliseconds: this.convertToMilliseconds(this.synchronizationLockTimeout(), this.synchronizationLockTimeoutUnit())
            };
        };
        synchronizationConfig.prototype.convertToMilliseconds = function (value, unit) {
            if (value && unit) {
                switch (unit) {
                    case synchronizationConfig.TU_MINUTES:
                        return value * 1000 * 60;
                    case synchronizationConfig.TU_HOURS:
                        return value * 1000 * 60 * 60;
                    case synchronizationConfig.TU_DAYS:
                        return value * 1000 * 60 * 60 * 24;
                }
            }
            return null;
        };
        synchronizationConfig.prototype.prepareInterval = function (milliseconds) {
            if (milliseconds) {
                var seconds = milliseconds / 1000;
                var minutes = seconds / 60;
                var hours = minutes / 60;
                if (this.isValidTimeValue(hours)) {
                    var days = hours / 24;
                    if (this.isValidTimeValue(days)) {
                        return [days, synchronizationConfig.TU_DAYS];
                    }
                    return [hours, synchronizationConfig.TU_HOURS];
                }
                return [minutes, synchronizationConfig.TU_MINUTES];
            }
            return [0, synchronizationConfig.TU_MINUTES];
        };
        synchronizationConfig.prototype.isValidTimeValue = function (value) {
            return value >= 1 && value % 1 === 0;
        };
        synchronizationConfig.TU_MINUTES = "minutes";
        synchronizationConfig.TU_HOURS = "hours";
        synchronizationConfig.TU_DAYS = "days";
        return synchronizationConfig;
    })();
    return synchronizationConfig;
});
define("models/filesystem/synchronizationDestination", ["require", "exports"], function (require, exports) {
    var synchronizationDestination = (function () {
        function synchronizationDestination(dto) {
            var _this = this;
            this.url = ko.observable().extend({ required: true });
            this.username = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.domain = ko.observable().extend({ required: true });
            this.apiKey = ko.observable().extend({ required: true });
            this.filesystem = ko.observable().extend({ required: true });
            this.disabled = ko.observable().extend({ required: true });
            this.name = ko.computed(function () {
                if (_this.url() && _this.filesystem()) {
                    return _this.filesystem() + " on " + _this.url();
                }
                else if (_this.url()) {
                    return _this.url();
                }
                else if (_this.filesystem()) {
                    return _this.filesystem();
                }
                return "[empty]";
            });
            this.isValid = ko.computed(function () { return _this.url() != null && _this.url().length > 0; });
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.url(dto.ServerUrl);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.disabled(!dto.Enabled);
            this.filesystem(dto.FileSystem);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
        }
        synchronizationDestination.prototype.toggleUserCredentials = function () {
            this.isUserCredentials.toggle();
            if (this.isUserCredentials()) {
                this.isApiKeyCredentials(false);
            }
        };
        synchronizationDestination.prototype.toggleApiKeyCredentials = function () {
            this.isApiKeyCredentials.toggle();
            if (this.isApiKeyCredentials()) {
                this.isUserCredentials(false);
            }
        };
        synchronizationDestination.empty = function (fileSystemName) {
            return new synchronizationDestination({
                ServerUrl: location.protocol + "//" + location.host,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null,
                FileSystem: fileSystemName,
                Enabled: false
            });
        };
        synchronizationDestination.prototype.enable = function () {
            this.disabled(false);
        };
        synchronizationDestination.prototype.disable = function () {
            this.disabled(true);
        };
        synchronizationDestination.prototype.toDto = function () {
            return {
                ServerUrl: this.prepareUrl(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                FileSystem: this.filesystem(),
                Enabled: !this.disabled(),
            };
        };
        synchronizationDestination.prototype.prepareUrl = function () {
            var url = this.url();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        return synchronizationDestination;
    })();
    return synchronizationDestination;
});
define("models/filesystem/synchronizationDetail", ["require", "exports"], function (require, exports) {
    var synchronizationDetail = (function () {
        function synchronizationDetail(dto, direction, status) {
            this.FileName = ko.observable();
            this.FileEtag = ko.observable();
            this.DestinationUrl = ko.observable();
            this.Type = ko.observable();
            this.TypeDescription = ko.observable();
            this.Status = ko.observable();
            this.Direction = ko.observable();
            this.AdditionalInfo = ko.observable();
            this.FileName(dto.FileName);
            this.FileEtag(dto.FileETag);
            this.DestinationUrl(dto.DestinationUrl);
            this.Type(synchronizationDetail.getType(dto.Type));
            this.TypeDescription(synchronizationDetail.getTypeDescription(this.Type()));
            this.Status(status);
            this.Direction(direction);
        }
        synchronizationDetail.prototype.getId = function () {
            return this.FileName();
        };
        synchronizationDetail.prototype.getUrl = function () {
            return this.getId();
        };
        synchronizationDetail.prototype.getDocumentPropertyNames = function () {
            return ["Id", "FileEtag", "DestinationUrl", "Type", "Status"];
        };
        synchronizationDetail.getType = function (typeAsString) {
            switch (typeAsString) {
                case "ContentUpdate":
                    return 1 /* ContentUpdate */;
                case "Delete":
                    return 4 /* Delete */;
                case "MetadataUpdate":
                    return 2 /* MetadataUpdate */;
                case "Rename":
                    return 3 /* Rename */;
                default:
                    return 0 /* Unknown */;
            }
        };
        synchronizationDetail.getTypeDescription = function (type) {
            switch (type) {
                case 1 /* ContentUpdate */:
                    return "Content Update";
                case 4 /* Delete */:
                    return "Delete";
                case 2 /* MetadataUpdate */:
                    return "Metadata Update";
                case 3 /* Rename */:
                    return "Rename";
                default:
                    return "Unknown";
            }
        };
        return synchronizationDetail;
    })();
    return synchronizationDetail;
});
define("models/filesystem/synchronizationReplicationSetup", ["require", "exports", "models/filesystem/synchronizationDestination"], function (require, exports, synchronizationDestination) {
    var synchronizationReplicationSetup = (function () {
        function synchronizationReplicationSetup(dto) {
            this.source = ko.observable();
            this.destinations = ko.observableArray().extend({ required: true });
            this.source(dto.Source);
            this.destinations(dto.Destinations.map(function (dest) { return new synchronizationDestination(dest); }));
        }
        synchronizationReplicationSetup.prototype.toDto = function () {
            return {
                Destinations: this.destinations().map(function (dest) { return dest.toDto(); }),
                Source: this.source()
            };
        };
        return synchronizationReplicationSetup;
    })();
    return synchronizationReplicationSetup;
});
define("models/filesystem/uploadItem", ["require", "exports"], function (require, exports) {
    var uploadItem = (function () {
        function uploadItem(id, fileName, status, filesystem) {
            this.id = ko.observable("");
            this.fileName = ko.observable("");
            this.status = ko.observable("");
            this.id(id);
            this.fileName(fileName);
            this.status(status);
            this.filesystem = filesystem;
        }
        return uploadItem;
    })();
    return uploadItem;
});
define("models/filesystem/versioningEntry", ["require", "exports"], function (require, exports) {
    var versioningEntry = (function () {
        function versioningEntry(dto) {
            var _this = this;
            this.maxRevisions = ko.observable();
            this.exclude = ko.observable();
            this.excludeUnlessExplicit = ko.observable();
            this.purgeOnDelete = ko.observable();
            this.resetOnRename = ko.observable();
            if (!dto) {
                // Default settings for new entries
                dto = {
                    Id: dto.Id,
                    MaxRevisions: 214748368,
                    Exclude: false,
                    ExcludeUnlessExplicit: false,
                    PurgeOnDelete: false,
                    ResetOnRename: false
                };
            }
            this.maxRevisions(dto.MaxRevisions);
            this.exclude(dto.Exclude);
            this.excludeUnlessExplicit(dto.ExcludeUnlessExplicit);
            this.purgeOnDelete(dto.PurgeOnDelete);
            this.resetOnRename(dto.ResetOnRename);
            this.disabled = ko.computed(function () { return _this.exclude(); });
        }
        versioningEntry.prototype.toDto = function () {
            var dto = {
                Id: 'Raven/Versioning/DefaultConfiguration',
                MaxRevisions: this.maxRevisions(),
                Exclude: this.exclude(),
                ExcludeUnlessExplicit: this.excludeUnlessExplicit(),
                PurgeOnDelete: this.purgeOnDelete(),
                ResetOnRename: this.resetOnRename()
            };
            return dto;
        };
        return versioningEntry;
    })();
    return versioningEntry;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/resources/database", ["require", "exports", "models/resources/resource", "models/auth/license", "models/resources/databaseStatistics"], function (require, exports, resource, license, databaseStatistics) {
    var database = (function (_super) {
        __extends(database, _super);
        function database(name, isAdminCurrentTenant, isDisabled, bundles, isIndexingDisabled, isRejectClientsMode, isLoaded, clusterWide) {
            var _this = this;
            if (isAdminCurrentTenant === void 0) { isAdminCurrentTenant = true; }
            if (isDisabled === void 0) { isDisabled = false; }
            if (bundles === void 0) { bundles = []; }
            if (isIndexingDisabled === void 0) { isIndexingDisabled = false; }
            if (isRejectClientsMode === void 0) { isRejectClientsMode = false; }
            if (isLoaded === void 0) { isLoaded = false; }
            if (clusterWide === void 0) { clusterWide = false; }
            _super.call(this, name, TenantType.Database, isAdminCurrentTenant);
            this.statistics = ko.observable();
            this.indexingDisabled = ko.observable(false);
            this.rejectClientsMode = ko.observable(false);
            this.clusterWide = ko.observable(false);
            this.fullTypeName = "Database";
            this.disabled(isDisabled);
            this.activeBundles(bundles);
            this.indexingDisabled(isIndexingDisabled);
            this.rejectClientsMode(isRejectClientsMode);
            this.isLoaded(isLoaded);
            this.clusterWide(clusterWide);
            this.iconName = ko.computed(function () { return !_this.clusterWide() ? "fa fa-fw fa-database" : "fa fa-fw fa-cubes"; });
            this.itemCountText = ko.computed(function () { return !!_this.statistics() ? _this.statistics().countOfDocumentsText() : ""; });
            this.isLicensed = ko.pureComputed(function () {
                if (!!license.licenseStatus() && license.licenseStatus().IsCommercial) {
                    var attributes = license.licenseStatus().Attributes;
                    var result = _this.activeBundles()
                        .map(function (bundleName) { return _this.attributeValue(attributes, bundleName === "periodicBackup" ? "periodicExport" : bundleName); })
                        .reduce(function (a, b) { return /^true$/i.test(a) && /^true$/i.test(b); }, true);
                    return result;
                }
                return true;
            });
            this.recentQueriesLocalStorageName = "ravenDB-recentQueries." + name;
            this.recentPatchesLocalStorageName = "ravenDB-recentPatches." + name;
            this.mergedIndexLocalStoragePrefix = "ravenDB-mergedIndex." + name;
        }
        database.prototype.activate = function () {
            this.isLoaded(true);
            ko.postbox.publish("ActivateDatabase", this);
        };
        database.prototype.saveStatistics = function (dto) {
            if (!this.statistics()) {
                this.statistics(new databaseStatistics());
            }
            this.statistics().fromDto(dto);
        };
        database.prototype.attributeValue = function (attributes, bundleName) {
            for (var key in attributes) {
                if (attributes.hasOwnProperty(key) && key.toLowerCase() === bundleName.toLowerCase()) {
                    return attributes[key];
                }
            }
            return "true";
        };
        database.getNameFromUrl = function (url) {
            var index = url.indexOf("databases/");
            return (index > 0) ? url.substring(index + 10) : "";
        };
        database.prototype.isBundleActive = function (bundleName) {
            if (!!bundleName) {
                var bundle = this.activeBundles.first(function (x) { return x.toLowerCase() === bundleName.toLowerCase(); });
                return !!bundle;
            }
            return false;
        };
        database.type = "database";
        return database;
    })(resource);
    return database;
});
define("models/resources/databaseAccess", ["require", "exports", "viewmodels/shell"], function (require, exports, shell) {
    var databaseAccess = (function () {
        function databaseAccess(dto) {
            var _this = this;
            this.admin = ko.observable();
            this.tenantId = ko.observable();
            this.readOnly = ko.observable();
            this.currentAccessType = ko.computed({
                read: function () {
                    if (_this.admin()) {
                        return databaseAccess.adminAccessType;
                    }
                    else if (_this.readOnly()) {
                        return databaseAccess.readOnlyAccessType;
                    }
                    return databaseAccess.readWriteAccessType;
                },
                write: function (value) {
                    switch (value) {
                        case databaseAccess.adminAccessType:
                            _this.admin(true);
                            _this.readOnly(false);
                            break;
                        case databaseAccess.readOnlyAccessType:
                            _this.admin(false);
                            _this.readOnly(true);
                            break;
                        case databaseAccess.readWriteAccessType:
                            _this.admin(false);
                            _this.readOnly(false);
                            break;
                        default:
                    }
                }
            });
            this.admin(dto.Admin);
            this.readOnly(dto.ReadOnly);
            this.tenantId(dto.TenantId != null ? dto.TenantId : '');
            this.searchResults = ko.computed(function () {
                var readOnly = shell.isGlobalAdmin() === false && shell.canReadWriteSettings() === false && shell.canReadSettings();
                if (readOnly)
                    return [];
                var newResourceName = _this.tenantId();
                var resourceNames = shell.resources().map(function (rs) { return rs.name; })
                    .concat("*")
                    .filter(function (x, i, c) { return c.indexOf(x) == i; });
                return resourceNames.filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.tenantCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newTenantId = _this.tenantId();
                var resourceNames = shell.resources().map(function (rs) { return rs.name; })
                    .concat("*")
                    .filter(function (x, i, c) { return c.indexOf(x) == i; });
                var foundResource = resourceNames.first(function (name) { return newTenantId === name; });
                if (!foundResource && newTenantId.length > 0) {
                    errorMessage = "There is no database nor file system with such a name!";
                }
                return errorMessage;
            });
        }
        databaseAccess.prototype.toDto = function () {
            return {
                Admin: this.admin(),
                TenantId: this.tenantId(),
                ReadOnly: this.readOnly()
            };
        };
        databaseAccess.empty = function () {
            return new databaseAccess({
                Admin: false,
                TenantId: "",
                ReadOnly: false
            });
        };
        databaseAccess.prototype.getTypes = function () {
            return databaseAccess.databaseAccessTypes();
        };
        databaseAccess.adminAccessType = 'Admin';
        databaseAccess.readWriteAccessType = 'Read, Write';
        databaseAccess.readOnlyAccessType = 'Read Only';
        databaseAccess.databaseAccessTypes = ko.observableArray([databaseAccess.adminAccessType, databaseAccess.readWriteAccessType, databaseAccess.readOnlyAccessType]);
        return databaseAccess;
    })();
    return databaseAccess;
});
define("models/resources/databaseStatistics", ["require", "exports"], function (require, exports) {
    var databaseStatistics = (function () {
        function databaseStatistics() {
            this.countOfDocuments = ko.observable();
            this.databaseId = ko.observable();
            this.countOfDocumentsText = ko.observable();
            this.countOfIndexesText = ko.observable();
            this.staleFilteredIndexesCountText = ko.observable();
            this.countOfFilteredIndexesText = ko.observable();
            this.staleIndexesCountText = ko.observable();
            this.errorsCountText = ko.observable();
        }
        databaseStatistics.prototype.fromDto = function (dto) {
            this.countOfDocuments(dto.CountOfDocuments);
            this.databaseId(dto.DatabaseId);
            this.countOfFilteredIndexesText(this.getItemCountText(dto.CountOfIndexesExcludingDisabledAndAbandoned, "index", "es"));
            this.staleFilteredIndexesCountText(dto.CountOfStaleIndexesExcludingDisabledAndAbandoned.toLocaleString() + " stale");
            this.errorsCountText(this.getItemCountText(dto.Errors.length, "error", "s"));
            this.countOfDocumentsText(this.getItemCountText(dto.CountOfDocuments, "document", "s"));
            this.countOfIndexesText(this.getItemCountText(dto.CountOfIndexes, "index", "es"));
            this.staleIndexesCountText(dto.StaleIndexes.length.toLocaleString() + " stale");
            this.errorsCountText(this.getItemCountText(dto.Errors.length, "error", "s"));
        };
        databaseStatistics.prototype.getItemCountText = function (itemCount, singularText, suffix) {
            var itemCountText = itemCount.toLocaleString() + " " + singularText;
            if (itemCount !== 1) {
                itemCountText += suffix;
            }
            return itemCountText;
        };
        return databaseStatistics;
    })();
    return databaseStatistics;
});
define("models/resources/environmentColor", ["require", "exports"], function (require, exports) {
    var environmentColor = (function () {
        function environmentColor(name, backgroundColor) {
            this.name = name;
            this.backgroundColor = backgroundColor;
        }
        environmentColor.prototype.toDto = function () {
            return {
                Name: this.name,
                BackgroundColor: this.backgroundColor
            };
        };
        return environmentColor;
    })();
    return environmentColor;
});
define("models/resources/resource", ["require", "exports"], function (require, exports) {
    var resource = (function () {
        function resource(name, type, isAdminCurrentTenant) {
            this.name = name;
            this.type = type;
            this.isAdminCurrentTenant = ko.observable(false);
            this.isSystem = false;
            this.isSelected = ko.observable(false);
            this.isChecked = ko.observable(false);
            this.isVisible = ko.observable(true);
            this.isLoading = ko.observable(false);
            this.isLoaded = ko.observable(false);
            this.disabled = ko.observable(false);
            this.activeBundles = ko.observableArray();
            this.isImporting = ko.observable(false);
            this.importStatus = ko.observable("");
            this.isAdminCurrentTenant(isAdminCurrentTenant);
        }
        resource.prototype.activate = function () {
            throw new Error("Activate must be overridden.");
        };
        resource.prototype.checkboxToggle = function () {
            this.isChecked.toggle();
        };
        resource.prototype.isDatabase = function () {
            return this.type === TenantType.Database;
        };
        resource.prototype.isFileSystem = function () {
            return this.type === TenantType.FileSystem;
        };
        resource.prototype.isCounterStorage = function () {
            return this.type === TenantType.CounterStorage;
        };
        resource.prototype.isTimeSeries = function () {
            return this.type === TenantType.TimeSeries;
        };
        return resource;
    })();
    return resource;
});
define("models/resources/serverSmugglingItem", ["require", "exports"], function (require, exports) {
    var serverSmugglingItem = (function () {
        function serverSmugglingItem(database) {
            var _this = this;
            this.incremental = ko.observable(true);
            this.stripReplicationInformation = ko.observable(false);
            this.shouldDisableVersioningBundle = ko.observable(false);
            this.resource = database;
            this.hasReplicationBundle = ko.computed(function () { return _this.resource.isBundleActive("replication"); });
            this.hasVersioningBundle = ko.computed(function () { return _this.resource.isBundleActive("versioning"); });
        }
        serverSmugglingItem.prototype.toDto = function () {
            return {
                Name: this.resource.name,
                Incremental: this.incremental(),
                StripReplicationInformation: this.stripReplicationInformation(),
                ShouldDisableVersioningBundle: this.shouldDisableVersioningBundle()
            };
        };
        return serverSmugglingItem;
    })();
    return serverSmugglingItem;
});
define("models/timeSeries/pointChange", ["require", "exports"], function (require, exports) {
    var pointChange = (function () {
        function pointChange(point, isNew) {
            var _this = this;
            if (isNew === void 0) { isNew = false; }
            this.type = ko.observable("");
            this.key = ko.observable("");
            this.at = ko.observable();
            this.fields = ko.observableArray();
            this.values = ko.observableArray();
            this.isNew = ko.observable(false);
            this.type(point.type);
            this.key(point.key);
            this.at(point.At);
            this.fields(point.fields);
            this.values(point.values);
            this.isNew(isNew);
            this.typeCustomValidityError = ko.computed(function () {
                var type = _this.type();
                if (!$.trim(type))
                    return "'Type' cannot be empty";
                if (type.length > 255) {
                    return "'Type' length can't exceed 255 characters";
                }
                if (type.contains('\\')) {
                    return "'Type' cannot contain '\\' char";
                }
                return "";
            });
            this.keyCustomValidityError = ko.computed(function () {
                var key = _this.key();
                if (!$.trim(key))
                    return "'key' cannot be empty";
                if (key.length > 255) {
                    return "'key' length can't exceed 255 characters";
                }
                return "";
            });
            this.atCustomValidityError = ko.computed(function () {
                var at = _this.at();
                if (!$.trim(at))
                    return "'At' cannot be empty";
                // Todo: Validate a valid DateTime.ticks value
                return "";
            });
        }
        pointChange.prototype.isNumber = function (num) {
            if (num < 0)
                return true;
            var n1 = Math.abs(num);
            var n2 = parseInt(num, 10);
            return !isNaN(n1) && n2 === n1 && n1.toString() === num;
        };
        return pointChange;
    })();
    return pointChange;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("models/timeSeries/timeSeries", ["require", "exports", "models/resources/resource", "models/auth/license", "models/timeSeries/timeSeriesStatistics"], function (require, exports, resource, license, timeSeriesStatistics) {
    var timeSeries = (function (_super) {
        __extends(timeSeries, _super);
        function timeSeries(name, isAdminCurrentTenant, isDisabled, bundles) {
            var _this = this;
            if (isAdminCurrentTenant === void 0) { isAdminCurrentTenant = true; }
            if (isDisabled === void 0) { isDisabled = false; }
            if (bundles === void 0) { bundles = []; }
            _super.call(this, name, TenantType.TimeSeries, isAdminCurrentTenant);
            this.statistics = ko.observable();
            this.iconName = "fa fa-fw fa-clock-o";
            if (!name) {
                debugger;
            }
            this.fullTypeName = "Time Series";
            this.disabled(isDisabled);
            this.activeBundles(bundles);
            this.itemCountText = ko.computed(function () { return !!_this.statistics() ? _this.statistics().keysCountText() : ""; });
            this.isLicensed = ko.computed(function () {
                if (!!license.licenseStatus() && license.licenseStatus().IsCommercial) {
                    var timeSeriesValue = license.licenseStatus().Attributes.timeSeries;
                    return /^true$/i.test(timeSeriesValue);
                }
                return true;
            });
        }
        timeSeries.prototype.activate = function () {
            this.isLoaded(true);
            ko.postbox.publish("ActivateTimeSeries", this);
        };
        timeSeries.prototype.saveStatistics = function (dto) {
            if (!this.statistics()) {
                this.statistics(new timeSeriesStatistics());
            }
            this.statistics().fromDto(dto);
        };
        timeSeries.getNameFromUrl = function (url) {
            var index = url.indexOf("timeseries/");
            return (index > 0) ? url.substring(index + 10) : "";
        };
        timeSeries.type = "timeSeries";
        return timeSeries;
    })(resource);
    return timeSeries;
});
define("models/timeSeries/timeSeriesKey", ["require", "exports"], function (require, exports) {
    var timeSeriesKey = (function () {
        function timeSeriesKey(dto, ownerTimeSeries) {
            this.ownerTimeSeries = ownerTimeSeries;
            this.Type = dto.Type.Type;
            this.Fields = dto.Type.Fields;
            this.Key = dto.Key;
            this.Points = dto.PointsCount;
        }
        timeSeriesKey.prototype.getEntityName = function () {
            return this.getId();
        };
        timeSeriesKey.prototype.getDocumentPropertyNames = function () {
            return ["Key", "Points"];
        };
        timeSeriesKey.prototype.getId = function () {
            return this.Type + "/" + this.Key;
        };
        timeSeriesKey.prototype.getUrl = function () {
            return this.getId();
        };
        return timeSeriesKey;
    })();
    return timeSeriesKey;
});
define("models/timeSeries/timeSeriesPoint", ["require", "exports"], function (require, exports) {
    var timeSeriesPoint = (function () {
        function timeSeriesPoint(type, fields, key, at, values) {
            this.type = type;
            this.fields = fields;
            this.key = key;
            this.At = at;
            this.values = values;
            for (var i = 0; i < this.fields.length; i++) {
                this[fields[i]] = values[i];
            }
        }
        timeSeriesPoint.prototype.getEntityName = function () {
            return this.getId();
        };
        timeSeriesPoint.prototype.getDocumentPropertyNames = function () {
            var columns = ["At"];
            for (var i = 0; i < this.fields.length; i++) {
                columns.push(this.fields[i]);
            }
            return columns;
        };
        timeSeriesPoint.prototype.getId = function () {
            return this.type + "/" + this.key + "/" + this.At;
        };
        timeSeriesPoint.prototype.getUrl = function () {
            return this.getId();
        };
        return timeSeriesPoint;
    })();
    return timeSeriesPoint;
});
define("models/timeSeries/timeSeriesStatistics", ["require", "exports"], function (require, exports) {
    var timeSeriesStatistics = (function () {
        function timeSeriesStatistics() {
            this.typesCount = ko.observable();
            this.keysCount = ko.observable();
            this.pointsCount = ko.observable();
            this.typesCountText = ko.observable("");
            this.keysCountText = ko.observable("");
            this.pointsCountText = ko.observable("");
            this.requestsPerSecondText = ko.observable("");
            this.timeSeriesSize = ko.observable("");
        }
        timeSeriesStatistics.prototype.getItemCountText = function (itemCount, singularText, suffix) {
            var itemCountText = itemCount.toLocaleString() + " " + singularText;
            if (itemCount !== 1) {
                itemCountText += suffix;
            }
            return itemCountText;
        };
        timeSeriesStatistics.prototype.fromDto = function (dto) {
            this.typesCount(dto.TypesCount);
            this.keysCount(dto.KeysCount);
            this.pointsCount(dto.PointsCount);
            this.typesCountText(this.getItemCountText(dto.TypesCount, "type", "s"));
            this.keysCountText(this.getItemCountText(dto.KeysCount, "key", "s"));
            this.pointsCountText(this.getItemCountText(dto.PointsCount, "point", "s"));
            this.requestsPerSecondText(dto.RequestsPerSecond + " requests per second");
            this.timeSeriesSize(dto.TimeSeriesSize);
        };
        return timeSeriesStatistics;
    })();
    return timeSeriesStatistics;
});
define("models/timeSeries/timeSeriesType", ["require", "exports", "common/pagedList", "common/cssGenerator", "commands/timeSeries/getKeysCommand"], function (require, exports, pagedList, cssGenerator, getKeysCommand) {
    var timeSeriesType = (function () {
        function timeSeriesType(name, fields, keysCount, ownerTimeSeries) {
            var _this = this;
            this.name = name;
            this.fields = fields;
            this.ownerTimeSeries = ownerTimeSeries;
            this.colorClass = "";
            this.keysCount = ko.observable(0);
            this.timeSeriesCountWithThousandsSeparator = ko.computed(function () { return _this.keysCount().toLocaleString(); });
            this.keysCount(keysCount);
            this.colorClass = timeSeriesType.getTypeCssClass(this.name, ownerTimeSeries);
            this["Name"] = name;
            this["Fields"] = fields.join(', ');
            this["Keys"] = keysCount;
        }
        timeSeriesType.prototype.activate = function () {
            ko.postbox.publish("ActivateType", this);
        };
        timeSeriesType.prototype.getKeys = function () {
            if (!this.timeSeriesList) {
                this.timeSeriesList = this.createPagedList();
            }
            return this.timeSeriesList;
        };
        timeSeriesType.fromDto = function (dto, ts) {
            return new timeSeriesType(dto.Type, dto.Fields, dto.KeysCount, ts);
        };
        timeSeriesType.getTypeCssClass = function (type, ts) {
            return cssGenerator.getCssClass(type, timeSeriesType.typeColorMaps, ts);
        };
        timeSeriesType.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchKeys(skip, take); };
            var list = new pagedList(fetcher);
            list.collectionName = this.name;
            return list;
        };
        timeSeriesType.prototype.fetchKeys = function (skip, take) {
            return new getKeysCommand(this.ownerTimeSeries, skip, take, this.name, this.keysCount()).execute();
        };
        timeSeriesType.prototype.getEntityName = function () {
            return this.getId();
        };
        timeSeriesType.prototype.getDocumentPropertyNames = function () {
            return ["Name", "Fields", "Keys"];
        };
        timeSeriesType.prototype.getId = function () {
            return this.name;
        };
        timeSeriesType.prototype.getUrl = function () {
            return this.getId();
        };
        timeSeriesType.typeColorMaps = [];
        return timeSeriesType;
    })();
    return timeSeriesType;
});
define("models/timeSeries/typeChange", ["require", "exports"], function (require, exports) {
    var typeChange = (function () {
        function typeChange(type, isNew) {
            var _this = this;
            if (isNew === void 0) { isNew = false; }
            this.type = ko.observable("");
            this.fields = ko.observableArray();
            this.isNew = ko.observable(false);
            this.type(type.name);
            this.fields(type.fields);
            this.isNew(isNew);
            this.typeCustomValidityError = ko.computed(function () {
                var name = _this.type();
                if (!$.trim(name))
                    return "Type name cannot be empty";
                if (name.length > 255) {
                    return "Type name length can't exceed 255 characters";
                }
                if (name.contains('\\')) {
                    return "Type name cannot contain '\\' char";
                }
                return "";
            });
        }
        typeChange.prototype.addField = function () {
            this.fields.push("");
        };
        typeChange.prototype.removeField = function (field) {
            this.fields.remove(field);
        };
        return typeChange;
    })();
    return typeChange;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/databaseStudioConfig", ["require", "exports", "viewmodels/viewModelBase", "models/database/documents/document", "models/resources/environmentColor", "commands/database/documents/saveDocumentCommand", "viewmodels/shell"], function (require, exports, viewModelBase, documentClass, environmentColor, saveDocumentCommand, shell) {
    var studioConfig = (function (_super) {
        __extends(studioConfig, _super);
        function studioConfig() {
            _super.call(this);
            this.configDocument = ko.observable();
            this.environmentColors = [
                new environmentColor("Default", "#f8f8f8"),
                new environmentColor("Development", "#80FF80"),
                new environmentColor("Staging", "#F5824D"),
                new environmentColor("Production", "#FF8585")
            ];
            this.selectedColor = ko.observable();
            var color = this.environmentColors.filter(function (color) { return color.name === shell.selectedEnvironmentColorStatic().name; });
            var selectedColor = !!color[0] ? color[0] : this.environmentColors[0];
            this.selectedColor(selectedColor);
            var self = this;
            this.selectedColor.subscribe(function (newValue) { return self.setEnvironmentColor(newValue); });
        }
        studioConfig.prototype.canActivate = function (args) {
            var deffered = $.Deferred();
            deffered.resolve({ can: true });
            this.configDocument(documentClass.empty());
            return deffered;
        };
        studioConfig.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            $("select").selectpicker();
            this.pickColor();
            $("#select-color li").each(function (index, element) {
                var color = _this.environmentColors[index];
                $(element).css("backgroundColor", color.backgroundColor);
            });
        };
        studioConfig.prototype.setEnvironmentColor = function (envColor) {
            var _this = this;
            var newDocument = this.configDocument();
            newDocument["EnvironmentColor"] = envColor.toDto();
            var saveTask = this.saveStudioConfig(newDocument);
            saveTask.done(function () {
                _this.pickColor();
                shell.selectedEnvironmentColorStatic(envColor);
            });
        };
        studioConfig.prototype.pickColor = function () {
            $("#select-color button").css("backgroundColor", this.selectedColor().backgroundColor);
        };
        studioConfig.prototype.saveStudioConfig = function (newDocument) {
            var _this = this;
            return new saveDocumentCommand(studioConfig.documentId, newDocument, this.activeDatabase())
                .execute()
                .done(function (saveResult) {
                _this.configDocument(newDocument);
                _this.configDocument().__metadata['@etag'] = saveResult[0].Etag;
            });
        };
        studioConfig.documentId = shell.studioConfigDocumentId;
        return studioConfig;
    })(viewModelBase);
    return studioConfig;
});
define("viewmodels/dialogViewModelBase", ["require", "exports", "plugins/dialog"], function (require, exports, dialog) {
    /*
     * Base view model for view models used in dialogs.
     * Provides some default functionality:
     *    - Optionallay focuses an element when the dialog is closed.
     *    - Listens for ESC to close the dialog.
     */
    var dialogViewModelBase = (function () {
        function dialogViewModelBase(elementToFocusOnDismissal) {
            var _this = this;
            this.elementToFocusOnDismissal = elementToFocusOnDismissal;
            this.dialogSelectorName = "";
            this.width = ko.observable(500);
            this.height = ko.observable(500);
            this.isFocused = ko.observable(false);
            this.left = ko.computed(function () { return -_this.width() * 0.5; });
            this.top = ko.computed(function () { return -_this.height() * 0.5; });
            this.bodyHeight = ko.computed(function () { return _this.height() - 150; });
        }
        dialogViewModelBase.prototype.attached = function () {
            var _this = this;
            jwerty.key("esc", function (e) { return _this.escapeKeyPressed(e); }, this, this.dialogSelectorName === "" ? dialogViewModelBase.dialogSelector : this.dialogSelectorName);
            jwerty.key("enter", function () { return _this.enterKeyPressed(); }, this, dialogViewModelBase.dialogSelector);
            var width = $(".messageBox").width();
            $(".messageBox").width(width + 2);
        };
        dialogViewModelBase.prototype.deactivate = function (args) {
            $(this.dialogSelectorName === "" ? dialogViewModelBase.dialogSelector : this.dialogSelectorName).unbind("keydown.jwerty");
        };
        dialogViewModelBase.prototype.detached = function () {
            if (this.elementToFocusOnDismissal) {
                $(this.elementToFocusOnDismissal).focus();
            }
        };
        dialogViewModelBase.prototype.compositionComplete = function (view, parent) {
            var _this = this;
            setTimeout(function () { return _this.setInitialFocus(); }, 100); // We have to time-delay this, else it never receives focus.
        };
        dialogViewModelBase.prototype.setInitialFocus = function () {
            var autoFocusElement = $(".messageBox [autofocus]");
            if (autoFocusElement.length) {
                autoFocusElement.focus();
                autoFocusElement.select();
            }
            else {
                $(dialogViewModelBase.dialogSelector).focus();
            }
        };
        dialogViewModelBase.prototype.enterKeyPressed = function () {
            var acceptButton = $(".modal-footer:visible .btn-primary")[0];
            if (acceptButton && acceptButton.click) {
                acceptButton.click();
            }
            return true;
        };
        dialogViewModelBase.prototype.escapeKeyPressed = function (e) {
            e.preventDefault();
            dialog.close(this);
        };
        dialogViewModelBase.prototype.unregisterResizing = function (id) {
            $(document).off("mousedown." + id);
            $(document).off("mouseup." + id);
            $(document).off("mousemove." + id);
        };
        dialogViewModelBase.prototype.registerResizing = function (id, resizerSelector) {
            var _this = this;
            if (resizerSelector === void 0) { resizerSelector = ".dialogResizer"; }
            var w = 0;
            var h = 0;
            var startX = 0;
            var startY = 0;
            var resizing = false;
            $(document).on("mousedown." + id, resizerSelector, function (e) {
                w = _this.width();
                h = _this.height();
                startX = e.pageX;
                startY = e.pageY;
                resizing = true;
            });
            $(document).on("mouseup." + id, "", function (e) {
                resizing = false;
            });
            $(document).on("mousemove." + id, "", function (e) {
                if (resizing) {
                    var targetWidth = w + 2 * (e.pageX - startX);
                    var targetHeight = h + 2 * (e.pageY - startY);
                    if (targetWidth < 600)
                        targetWidth = 600;
                    if (targetHeight < 500)
                        targetHeight = 500;
                    _this.width(targetWidth);
                    _this.height(targetHeight);
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        dialogViewModelBase.prototype.alignBoxVertically = function () {
            var messageBoxHeight = parseInt($(".messageBox").css('height'), 10);
            // find element to alter margin-top - it should be outer html element in dialog view
            // we can find this by looking for element with data-view inside modalHost container. 
            $(".modalHost [data-view]").css('margin-top', Math.floor(-1 * messageBoxHeight / 2) + 'px');
        };
        dialogViewModelBase.dialogSelector = ".messageBox";
        return dialogViewModelBase;
    })();
    return dialogViewModelBase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/shell", ["require", "exports", "plugins/router", "durandal/app", "durandal/system", "durandal/viewLocator", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "models/database/documents/collection", "models/auth/license", "models/resources/environmentColor", "common/appUrl", "common/uploadQueueHelper", "common/bindingHelpers/dynamicHeightBindingHandler", "common/bindingHelpers/autoCompleteBindingHandler", "common/bindingHelpers/enableResizeBindingHandler", "common/bindingHelpers/helpBindingHandler", "common/changesApi", "common/changesContext", "common/oauthContext", "common/messagePublisher", "common/apiKeyLocalStorage", "common/extensions", "common/serverBuildReminder", "common/eventSourceSettingStorage", "commands/resources/getDatabasesCommand", "commands/resources/getDatabaseStatsCommand", "commands/resources/getServerBuildVersionCommand", "commands/database/studio/getLatestServerBuildVersionCommand", "commands/database/studio/getClientBuildVersionCommand", "commands/auth/getLicenseStatusCommand", "commands/auth/getSupportCoverageCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "commands/database/documents/getDocumentWithMetadataCommand", "commands/filesystem/getFileSystemsCommand", "commands/filesystem/getFileSystemStatsCommand", "commands/counter/getCounterStoragesCommand", "commands/counter/getCounterStorageStatsCommand", "commands/timeSeries/getTimeSeriesCommand", "commands/timeSeries/getTimeSeriesStatsCommand", "commands/database/documents/getSystemDocumentCommand", "commands/database/studio/getServerConfigsCommand", "commands/database/cluster/getClusterTopologyCommand", "commands/getStudioConfig", "viewmodels/viewModelBase", "viewmodels/common/licensingStatus", "viewmodels/common/recentErrors", "viewmodels/common/enterApiKey", "viewmodels/common/latestBuildReminder", "common/recentQueriesStorage", "commands/licensing/GetHotSpareInformation"], function (require, exports, router, app, sys, viewLocator, database, fileSystem, counterStorage, timeSeries, collection, license, environmentColor, appUrl, uploadQueueHelper, dynamicHeightBindingHandler, autoCompleteBindingHandler, enableResizeBindingHandler, helpBindingHandler, changesApi, changesContext, oauthContext, messagePublisher, apiKeyLocalStorage, extensions, serverBuildReminder, eventSourceSettingStorage, getDatabasesCommand, getDatabaseStatsCommand, getServerBuildVersionCommand, getLatestServerBuildVersionCommand, getClientBuildVersionCommand, getLicenseStatusCommand, getSupportCoverageCommand, getDocumentsMetadataByIDPrefixCommand, getDocumentWithMetadataCommand, getFileSystemsCommand, getFileSystemStatsCommand, getCounterStoragesCommand, getCounterStorageStatsCommand, getTimeSeriesCommand, getTimeSeriesStatsCommand, getSystemDocumentCommand, getServerConfigsCommand, getClusterTopologyCommand, getStudioConfig, viewModelBase, licensingStatus, recentErrors, enterApiKey, latestBuildReminder, recentQueriesStorage, getHotSpareInformation) {
    var shell = (function (_super) {
        __extends(shell, _super);
        function shell() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.selectedColor = shell.selectedEnvironmentColorStatic;
            this.selectedEnvironmentText = ko.computed(function () { return _this.selectedColor().name + " Environment"; });
            this.canShowEnvironmentText = ko.computed(function () { return _this.selectedColor().name !== "Default"; });
            this.showContinueTestButton = ko.computed(function () { return viewModelBase.hasContinueTestOption(); });
            this.isLoadingStatistics = ko.computed(function () { return !!_this.lastActivatedResource() && !_this.lastActivatedResource().statistics(); }).extend({ rateLimit: 100 });
            this.canShowDatabaseNavbar = ko.computed(function () {
                return !!_this.lastActivatedResource()
                    && _this.lastActivatedResource().type === TenantType.Database
                    && (_this.appUrls.isAreaActive("databases")() || _this.appUrls.isAreaActive("resources")());
            });
            this.goToDocumentSearchResults = ko.observableArray();
            this.canShowFileSystemNavbar = ko.computed(function () {
                return !!_this.lastActivatedResource()
                    && _this.lastActivatedResource().type === TenantType.FileSystem
                    && (_this.appUrls.isAreaActive("filesystems")() || _this.appUrls.isAreaActive("resources")());
            });
            this.canShowFileSystemSettings = ko.computed(function () {
                if (!_this.canShowFileSystemNavbar())
                    return false;
                var fs = _this.lastActivatedResource();
                return fs.activeBundles.contains("Versioning");
            });
            this.canShowCountersNavbar = ko.computed(function () {
                return !!_this.lastActivatedResource()
                    && _this.lastActivatedResource().type === TenantType.CounterStorage
                    && (_this.appUrls.isAreaActive("counterstorages")() || _this.appUrls.isAreaActive("resources")());
            });
            this.canShowTimeSeriesNavbar = ko.computed(function () {
                return !!_this.lastActivatedResource()
                    && _this.lastActivatedResource().type === TenantType.TimeSeries
                    && (_this.appUrls.isAreaActive("timeseries")() || _this.appUrls.isAreaActive("resources")());
            });
            this.canShowResourcesNavbar = ko.computed(function () {
                var canDb = _this.canShowDatabaseNavbar();
                var canFs = _this.canShowFileSystemNavbar();
                var canCs = _this.canShowCountersNavbar();
                var canTs = _this.canShowTimeSeriesNavbar();
                return canDb || canFs || canCs || canTs;
            });
            this.currentAlert = ko.observable();
            this.isInCluster = ko.computed(function () { return shell.clusterMode(); });
            this.serverBuildVersion = ko.observable();
            this.clientBuildVersion = ko.observable();
            this.recordedErrors = ko.observableArray();
            this.newIndexUrl = appUrl.forCurrentDatabase().newIndex;
            this.newTransformerUrl = appUrl.forCurrentDatabase().newTransformer;
            this.currentRawUrl = ko.observable("");
            this.rawUrlIsVisible = ko.computed(function () { return _this.currentRawUrl().length > 0; });
            this.activeArea = ko.observable("Databases");
            this.hasReplicationSupport = ko.computed(function () { return !!_this.activeDatabase() && _this.activeDatabase().activeBundles.contains("Replication"); });
            this.showSplash = viewModelBase.showSplash;
            this.licenseStatus = license.licenseCssClass;
            this.supportStatus = license.supportCssClass;
            this.spinnerOptions = {
                lines: 17,
                length: 28,
                width: 14,
                radius: 44,
                scale: 1,
                corners: 1,
                color: ["#d74c0c", "#CC0000"],
                opacity: 0.35,
                rotate: 0,
                direction: 1,
                speed: 0.8,
                trail: 60,
                fps: 20,
                zIndex: 2e9,
                className: "spinner",
                top: "50%",
                left: "50%",
                shadow: false,
                hwaccel: false,
                position: "absolute" // Element positioning
            };
            this.spinner = new Spinner(this.spinnerOptions);
            this.preLoadRecentErrorsView();
            extensions.install();
            this.showLogOutButton = ko.computed(function () {
                var lsApiKey = apiKeyLocalStorage.get();
                var contextApiKey = oauthContext.apiKey();
                return lsApiKey || contextApiKey;
            });
            oauthContext.enterApiKeyTask = this.setupApiKey();
            oauthContext.enterApiKeyTask.done(function () {
                _this.globalChangesApi = new changesApi(appUrl.getSystemDatabase());
                _this.notifications = _this.createNotifications();
            });
            ko.postbox.subscribe("Alert", function (alert) { return _this.showAlert(alert); });
            ko.postbox.subscribe("LoadProgress", function (alertType) { return _this.dataLoadProgress(alertType); });
            ko.postbox.subscribe("ActivateDatabaseWithName", function (databaseName) { return _this.activateDatabaseWithName(databaseName); });
            ko.postbox.subscribe("SetRawJSONUrl", function (jsonUrl) { return _this.currentRawUrl(jsonUrl); });
            ko.postbox.subscribe("SelectNone", function () { return _this.selectNone(); });
            ko.postbox.subscribe("ActivateDatabase", function (db) { return _this.activateDatabase(db); });
            ko.postbox.subscribe("ActivateFilesystem", function (fs) { return _this.activateFileSystem(fs); });
            ko.postbox.subscribe("ActivateCounterStorage", function (cs) { return _this.activateCounterStorage(cs); });
            ko.postbox.subscribe("ActivateTimeSeries", function (ts) { return _this.activateTimeSeries(ts); });
            ko.postbox.subscribe("UploadFileStatusChanged", function (uploadStatus) { return _this.uploadStatusChanged(uploadStatus); });
            ko.postbox.subscribe("ChangesApiReconnected", function (rs) { return _this.reloadDataAfterReconnection(rs); });
            this.currentConnectedResource = appUrl.getSystemDatabase();
            this.goToDocumentSearch = ko.observable();
            this.goToDocumentSearch.throttle(250).subscribe(function (search) { return _this.fetchGoToDocSearchResults(search); });
            dynamicHeightBindingHandler.install();
            autoCompleteBindingHandler.install();
            enableResizeBindingHandler.install();
            helpBindingHandler.install();
            this.isSystemConnected = ko.computed(function () {
                var activeDb = _this.activeDatabase();
                var systemDb = _this.systemDatabase;
                return (!!activeDb && !!systemDb) ? systemDb.name != activeDb.name : false;
            });
            this.isActiveDatabaseDisabled = ko.computed(function () { return _this.isActiveResourceDisabled(_this.activeDatabase()); });
            this.isActiveFileSystemDisabled = ko.computed(function () { return _this.isActiveResourceDisabled(_this.activeFilesystem()); });
            this.isActiveCounterStorageDisabled = ko.computed(function () { return _this.isActiveResourceDisabled(_this.activeCounterStorage()); });
            this.isActiveTimeSeriesDisabled = ko.computed(function () { return _this.isActiveResourceDisabled(_this.activeTimeSeries()); });
            this.listedResources = ko.computed(function () {
                var currentResource = _this.lastActivatedResource();
                if (!!currentResource) {
                    return shell.resources().filter(function (rs) { return (rs.type !== currentResource.type || (rs.type === currentResource.type && rs.name !== currentResource.name)) && rs.name !== "<system>"; });
                }
                return shell.resources();
            });
            this.clientBuildVersion.subscribe(function (v) { return viewModelBase.clientVersion("3.5." + v.BuildVersion); });
        }
        // Override canActivate: we can always load this page, regardless of any system db prompt.
        shell.prototype.canActivate = function (args) {
            return true;
        };
        shell.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args, true);
            oauthContext.enterApiKeyTask.done(function () { return _this.connectToRavenServer(); });
            NProgress.set(.7);
            router.map([
                { route: "admin/settings*details", title: "Admin Settings", moduleId: "viewmodels/manage/adminSettings", nav: true, hash: this.appUrls.adminSettings },
                { route: ["", "resources"], title: "Resources", moduleId: "viewmodels/resources/resources", nav: true, hash: this.appUrls.resourcesManagement },
                { route: "databases/documents", title: "Documents", moduleId: "viewmodels/database/documents/documents", nav: true, hash: this.appUrls.documents },
                { route: "databases/conflicts", title: "Conflicts", moduleId: "viewmodels/database/conflicts/conflicts", nav: true, hash: this.appUrls.conflicts },
                { route: "databases/patch(/:recentPatchHash)", title: "Patch", moduleId: "viewmodels/database/patch/patch", nav: true, hash: this.appUrls.patch },
                { route: "databases/upgrade", title: "Upgrade in progress", moduleId: "viewmodels/common/upgrade", nav: false, hash: this.appUrls.upgrade },
                { route: "databases/indexes*details", title: "Indexes", moduleId: "viewmodels/database/indexes/indexesShell", nav: true, hash: this.appUrls.indexes },
                { route: "databases/transformers*details", title: "Transformers", moduleId: "viewmodels/database/transformers/transformersShell", nav: false, hash: this.appUrls.transformers },
                { route: "databases/query*details", title: "Query", moduleId: "viewmodels/database/query/queryShell", nav: true, hash: this.appUrls.query(null) },
                { route: "databases/tasks*details", title: "Tasks", moduleId: "viewmodels/database/tasks/tasks", nav: true, hash: this.appUrls.tasks, },
                { route: "databases/settings*details", title: "Settings", moduleId: "viewmodels/database/settings/settings", nav: true, hash: this.appUrls.settings },
                { route: "databases/status*details", title: "Status", moduleId: "viewmodels/database/status/status", nav: true, hash: this.appUrls.status },
                { route: "databases/edit", title: "Edit Document", moduleId: "viewmodels/database/documents/editDocument", nav: false },
                { route: "filesystems/files", title: "Files", moduleId: "viewmodels/filesystem/files/filesystemFiles", nav: true, hash: this.appUrls.filesystemFiles },
                { route: "filesystems/search", title: "Search", moduleId: "viewmodels/filesystem/search/search", nav: true, hash: this.appUrls.filesystemSearch },
                { route: "filesystems/synchronization*details", title: "Synchronization", moduleId: "viewmodels/filesystem/synchronization/synchronization", nav: true, hash: this.appUrls.filesystemSynchronization },
                { route: "filesystems/status*details", title: "Status", moduleId: "viewmodels/filesystem/status/status", nav: true, hash: this.appUrls.filesystemStatus },
                { route: "filesystems/tasks*details", title: "Tasks", moduleId: "viewmodels/filesystem/tasks/tasks", nav: true, hash: this.appUrls.filesystemTasks },
                { route: "filesystems/settings*details", title: "Settings", moduleId: "viewmodels/filesystem/settings/settings", nav: true, hash: this.appUrls.filesystemSettings },
                { route: "filesystems/configuration", title: "Configuration", moduleId: "viewmodels/filesystem/configurations/configuration", nav: true, hash: this.appUrls.filesystemConfiguration },
                { route: "filesystems/edit", title: "Edit File", moduleId: "viewmodels/filesystem/files/filesystemEditFile", nav: false },
                { route: "counterstorages/counters", title: "Counters", moduleId: "viewmodels/counter/counters", nav: true, hash: this.appUrls.counterStorageCounters },
                { route: "counterstorages/replication", title: "Replication", moduleId: "viewmodels/counter/counterStorageReplication", nav: true, hash: this.appUrls.counterStorageReplication },
                { route: "counterstorages/tasks*details", title: "Stats", moduleId: "viewmodels/counter/tasks/tasks", nav: true, hash: this.appUrls.counterStorageStats },
                { route: "counterstorages/stats", title: "Stats", moduleId: "viewmodels/counter/counterStorageStats", nav: true, hash: this.appUrls.counterStorageStats },
                { route: "counterstorages/configuration", title: "Configuration", moduleId: "viewmodels/counter/counterStorageConfiguration", nav: true, hash: this.appUrls.counterStorageConfiguration },
                { route: "counterstorages/edit", title: "Edit Counter", moduleId: "viewmodels/counter/editCounter", nav: false },
                { route: "timeseries/types", title: "Types", moduleId: "viewmodels/timeSeries/timeSeriesTypes", nav: true, hash: this.appUrls.timeSeriesType },
                { route: "timeseries/points", title: "Points", moduleId: "viewmodels/timeSeries/timeSeriesPoints", nav: true, hash: this.appUrls.timeSeriesPoints },
                { route: "timeseries/stats", title: "Stats", moduleId: "viewmodels/timeSeries/timeSeriesStats", nav: true, hash: this.appUrls.timeSeriesStats },
                { route: "timeseries/configuration*details", title: "Configuration", moduleId: "viewmodels/timeSeries/configuration/configuration", nav: true, hash: this.appUrls.timeSeriesConfiguration }
            ]).buildNavigationModel();
            // Show progress whenever we navigate.
            router.isNavigating.subscribe(function (isNavigating) { return _this.showNavigationProgress(isNavigating); });
            router.on('router:navigation:cancelled', function () { return _this.showNavigationProgress(false); });
            appUrl.mapUnknownRoutes(router);
            var self = this;
            window.addEventListener("beforeunload", self.destroyChangesApi.bind(self));
            $(window).bind("storage", function (e) {
                if (e.originalEvent.key === eventSourceSettingStorage.localStorageName) {
                    if (!JSON.parse(e.originalEvent.newValue)) {
                        self.destroyChangesApi();
                    }
                    else {
                        // enable changes api
                        _this.globalChangesApi = new changesApi(appUrl.getSystemDatabase());
                        _this.notifications = _this.createNotifications();
                    }
                }
                else if (e.originalEvent.key === apiKeyLocalStorage.localStorageName) {
                    _this.onLogOut();
                }
            });
            this.maxResourceNameWidth = ko.computed(function () {
                if (_this.canShowResourcesNavbar() && !!_this.lastActivatedResource()) {
                    var navigationLinksWidth = 50;
                    if (_this.canShowDatabaseNavbar()) {
                        navigationLinksWidth += 804;
                    }
                    else if (_this.canShowFileSystemNavbar()) {
                        navigationLinksWidth += 600;
                    }
                    else if (_this.canShowCountersNavbar()) {
                        navigationLinksWidth += 600; //todo: calculate
                    }
                    else if (_this.canShowTimeSeriesNavbar()) {
                        navigationLinksWidth += 600; //todo: calculate
                    }
                    var brandWidth = _this.getWidth("brand");
                    var logOutWidth = _this.getWidth("logOut");
                    var continueTestWidth = _this.getWidth("continueTest");
                    var featureNameWidth = _this.getWidth("featureName");
                    var freeSpace = $(window).width() - (navigationLinksWidth + brandWidth + logOutWidth + continueTestWidth + featureNameWidth);
                    var maxWidth = Math.floor(freeSpace);
                    return maxWidth + "px";
                }
                return "1px";
            });
            $(window).resize(function () { return self.lastActivatedResource.valueHasMutated(); });
            return shell.fetchLicenseStatus();
        };
        shell.prototype.isActiveResourceDisabled = function (rs) {
            return !!rs ? rs.disabled() || !rs.isLicensed() : false;
        };
        shell.prototype.getWidth = function (tag) {
            return $("#" + tag).length > 0 ? $("#" + tag).width() : 0;
        };
        shell.prototype.destroyChangesApi = function () {
            this.cleanupNotifications();
            this.globalChangesApi.dispose();
            shell.disconnectFromResourceChangesApi();
        };
        // Called by Durandal when shell.html has been put into the DOM.
        // The view must be attached to the DOM before we can hook up keyboard shortcuts.
        shell.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            var target = document.getElementById("splash-spinner");
            //this.spinner.stop();
            this.showSplash.subscribe(function (show) {
                if (show) {
                    _this.spinner.spin(target);
                }
                else {
                    _this.spinner.stop();
                }
            });
            jwerty.key("ctrl+alt+n", function (e) {
                e.preventDefault();
                _this.newDocument();
            });
            jwerty.key("enter", function (e) {
                e.preventDefault();
                return false;
            }, this, "#goToDocInput");
            router.activeInstruction.subscribe(function (val) {
                if (!!val && val.config.route.split('/').length == 1)
                    _this.activeArea(val.config.title);
            });
            sys.error = function (e) {
                console.error(e);
                messagePublisher.reportError("Failed to load routed module!", e);
            };
        };
        shell.prototype.preLoadRecentErrorsView = function () {
            // preload this view as in case of failure server can't serve it.
            viewLocator.locateView("views/common/recentErrors");
        };
        shell.fetchStudioConfig = function () {
            var _this = this;
            var hotSpareTask = new getHotSpareInformation().execute();
            var configTask = new getDocumentWithMetadataCommand(shell.studioConfigDocumentId, appUrl.getSystemDatabase(), true).execute();
            $.when(hotSpareTask, configTask).done(function (hotSpareResult, doc) {
                var hotSpare = hotSpareResult[0];
                appUrl.warnWhenUsingSystemDatabase = doc && doc["WarnWhenUsingSystemDatabase"];
                if (license.licenseStatus().Attributes.hotSpare === "true") {
                    // override environment colors with hot spare
                    _this.activateHotSpareEnvironment(hotSpare);
                }
                else {
                    var envColor = doc && doc["EnvironmentColor"];
                    if (envColor != null) {
                        var color = new environmentColor(envColor.Name, envColor.BackgroundColor);
                        shell.selectedEnvironmentColorStatic(color);
                        shell.originalEnvironmentColor(color);
                    }
                }
            });
        };
        shell.prototype.fecthStudioConfigForDatabase = function (db) {
            var hotSpareTask = new getHotSpareInformation().execute();
            var configTask = new getStudioConfig(db).execute();
            $.when(hotSpareTask, configTask).done(function (hotSpareResult, docResult) {
                var hotSpare = hotSpareResult[0];
                var doc = docResult[0];
                if (hotSpare.ActivationMode === "Activated") {
                    // override environment colors with hot spare
                    shell.activateHotSpareEnvironment(hotSpare);
                }
                else {
                    var envColor = doc["EnvironmentColor"];
                    if (envColor != null) {
                        shell.selectedEnvironmentColorStatic(new environmentColor(envColor.Name, envColor.BackgroundColor));
                    }
                }
            }).fail(function () { return shell.selectedEnvironmentColorStatic(shell.originalEnvironmentColor()); });
        };
        shell.prototype.activateDatabase = function (db) {
            var _this = this;
            if (db == null) {
                this.disconnectFromCurrentResource();
                return;
            }
            this.fecthStudioConfigForDatabase(db);
            var changeSubscriptionArray = function () { return [
                changesContext.currentResourceChangesApi().watchAllDocs(function () { return _this.fetchDbStats(db); }),
                changesContext.currentResourceChangesApi().watchAllIndexes(function () { return _this.fetchDbStats(db); }),
                changesContext.currentResourceChangesApi().watchBulks(function () { return _this.fetchDbStats(db); })
            ]; };
            var isNotADatabase = this.currentConnectedResource instanceof database === false;
            this.updateChangesApi(db, isNotADatabase, function () { return _this.fetchDbStats(db); }, changeSubscriptionArray);
            shell.resources().forEach(function (r) { return r.isSelected(r instanceof database && r.name === db.name); });
        };
        shell.prototype.fetchDbStats = function (db) {
            if (!!db && !db.disabled() && db.isLicensed()) {
                new getDatabaseStatsCommand(db, true)
                    .execute()
                    .done(function (result) { return db.saveStatistics(result); });
            }
        };
        shell.prototype.activateFileSystem = function (fs) {
            var _this = this;
            if (fs == null) {
                this.disconnectFromCurrentResource();
                return;
            }
            this.fecthStudioConfigForDatabase(new database(fs.name));
            var changesSubscriptionArray = function () { return [
                changesContext.currentResourceChangesApi().watchFsFolders("", function () { return _this.fetchFsStats(fs); })
            ]; };
            var isNotAFileSystem = this.currentConnectedResource instanceof fileSystem === false;
            this.updateChangesApi(fs, isNotAFileSystem, function () { return _this.fetchFsStats(fs); }, changesSubscriptionArray);
            shell.resources().forEach(function (r) { return r.isSelected(r instanceof fileSystem && r.name === fs.name); });
        };
        shell.prototype.fetchFsStats = function (fs) {
            if (!!fs && !fs.disabled() && fs.isLicensed()) {
                new getFileSystemStatsCommand(fs, true)
                    .execute()
                    .done(function (result) { return fs.saveStatistics(result); })
                    .fail(function (response) { return messagePublisher.reportError("Failed to get file system stats", response.responseText, response.statusText); });
            }
        };
        shell.prototype.disconnectFromCurrentResource = function () {
            shell.disconnectFromResourceChangesApi();
            this.lastActivatedResource(null);
            this.currentConnectedResource = appUrl.getSystemDatabase();
        };
        shell.prototype.activateCounterStorage = function (cs) {
            var _this = this;
            var changesSubscriptionArray = function () { return [
                changesContext.currentResourceChangesApi().watchAllCounters(function () { return _this.fetchCsStats(cs); }),
                changesContext.currentResourceChangesApi().watchCounterBulkOperation(function () { return _this.fetchCsStats(cs); })
            ]; };
            var isNotACounterStorage = this.currentConnectedResource instanceof counterStorage === false;
            this.updateChangesApi(cs, isNotACounterStorage, function () { return _this.fetchCsStats(cs); }, changesSubscriptionArray);
            shell.resources().forEach(function (r) { return r.isSelected(r instanceof counterStorage && r.name === cs.name); });
        };
        shell.prototype.fetchCsStats = function (cs) {
            if (!!cs && !cs.disabled() && cs.isLicensed()) {
                new getCounterStorageStatsCommand(cs, true)
                    .execute()
                    .done(function (result) { return cs.saveStatistics(result); });
            }
        };
        shell.prototype.activateTimeSeries = function (ts) {
            var _this = this;
            var changesSubscriptionArray = function () { return [
                changesContext.currentResourceChangesApi().watchAllTimeSeries(function () { return _this.fetchTsStats(ts); }),
                changesContext.currentResourceChangesApi().watchTimeSeriesBulkOperation(function () { return _this.fetchTsStats(ts); })
            ]; };
            var isNotATimeSeries = this.currentConnectedResource instanceof timeSeries === false;
            this.updateChangesApi(ts, isNotATimeSeries, function () { return _this.fetchTsStats(ts); }, changesSubscriptionArray);
            shell.resources().forEach(function (r) { return r.isSelected(r instanceof timeSeries && r.name === ts.name); });
        };
        shell.prototype.fetchTsStats = function (ts) {
            if (!!ts && !ts.disabled() && ts.isLicensed()) {
                new getTimeSeriesStatsCommand(ts, true)
                    .execute()
                    .done(function (result) { return ts.saveStatistics(result); });
            }
        };
        shell.prototype.updateChangesApi = function (rs, isPreviousDifferentKind, fetchStats, subscriptionsArray) {
            fetchStats();
            if (isPreviousDifferentKind || this.currentConnectedResource.name !== rs.name) {
                // disconnect from the current resource changes api and set the current connected resource
                shell.disconnectFromResourceChangesApi();
                this.currentConnectedResource = rs;
            }
            if ((!rs.disabled() && rs.isLicensed()) &&
                (isPreviousDifferentKind || changesContext.currentResourceChangesApi() == null)) {
                // connect to changes api, if it's not disabled and the changes api isn't already connected
                var changes = new changesApi(rs, 5000);
                changes.connectToChangesApiTask.done(function () {
                    fetchStats();
                    changesContext.currentResourceChangesApi(changes);
                    shell.changeSubscriptionArray = subscriptionsArray();
                });
            }
        };
        shell.prototype.setupApiKey = function () {
            var _this = this;
            // try to find api key as studio hash parameter
            var hash = window.location.hash;
            if (hash === "#has-api-key") {
                return this.showApiKeyDialog();
            }
            else if (hash.match(/#api-key/g)) {
                var match = /#api-key=(.*)/.exec(hash);
                if (match && match.length === 2) {
                    oauthContext.apiKey(match[1]);
                    apiKeyLocalStorage.setValue(match[1]);
                }
                var splittedHash = hash.split("&#api-key");
                var url = (splittedHash.length === 1) ? "#resources" : splittedHash[0];
                window.location.href = url;
            }
            else {
                var apiKeyFromStorage = apiKeyLocalStorage.get();
                if (apiKeyFromStorage) {
                    oauthContext.apiKey(apiKeyFromStorage);
                }
            }
            oauthContext.authHeader.subscribe(function (h) {
                if (_this.renewOAuthTokenTimeoutId) {
                    clearTimeout(_this.renewOAuthTokenTimeoutId);
                    _this.renewOAuthTokenTimeoutId = null;
                }
                if (h) {
                    _this.renewOAuthTokenTimeoutId = setTimeout(function () { return _this.renewOAuthToken(); }, 25 * 60 * 1000);
                }
            });
            return $.Deferred().resolve();
        };
        shell.prototype.renewOAuthToken = function () {
            oauthContext.authHeader(null);
            new getDatabaseStatsCommand(this.systemDatabase).execute();
        };
        shell.prototype.showNavigationProgress = function (isNavigating) {
            if (isNavigating) {
                NProgress.start();
                var currentProgress = parseFloat(NProgress.status);
                var newProgress = isNaN(currentProgress) ? 0.5 : currentProgress + (currentProgress / 2);
                NProgress.set(newProgress);
            }
            else {
                NProgress.done();
                this.activeArea(appUrl.checkIsAreaActive("filesystems") ? "File Systems" : "Databases");
            }
        };
        shell.reloadResources = function (getResources, resourceObservableArray) {
            var _this = this;
            return getResources()
                .done(function (results) { return _this.updateResourceObservableArray(resourceObservableArray, results); });
        };
        shell.updateResourceObservableArray = function (resourceObservableArray, recievedResourceArray) {
            var deletedResources = [];
            resourceObservableArray().forEach(function (rs) {
                if (rs.name !== "<system>") {
                    var existingResource = recievedResourceArray.first(function (recievedResource) { return recievedResource.name === rs.name; });
                    if (existingResource == null) {
                        deletedResources.push(rs);
                    }
                }
            });
            resourceObservableArray.removeAll(deletedResources);
            recievedResourceArray.forEach(function (recievedResource) {
                var foundResource = resourceObservableArray().first(function (rs) { return rs.name === recievedResource.name; });
                if (foundResource == null) {
                    resourceObservableArray.push(recievedResource);
                }
                else {
                    foundResource.disabled(recievedResource.disabled());
                }
            });
        };
        shell.prototype.reloadDataAfterReconnection = function (rs) {
            var _this = this;
            if (rs.name === "<system>") {
                shell.fetchStudioConfig();
                this.fetchServerBuildVersion();
                this.fetchClientBuildVersion();
                shell.fetchLicenseStatus();
                this.fetchSupportCoverage();
                this.loadServerConfig();
                var databasesLoadTask = shell.reloadDatabases();
                var fileSystemsLoadTask = shell.reloadFileSystems();
                var counterStoragesLoadTask = shell.reloadCounterStorages();
                var timeSeriesLoadTask = shell.reloadTimeSeries();
                $.when(databasesLoadTask, fileSystemsLoadTask, counterStoragesLoadTask, timeSeriesLoadTask)
                    .done(function () {
                    var connectedResource = _this.currentConnectedResource;
                    var resourceObservableArray = shell.databases;
                    var activeResourceObservable = _this.activeDatabase;
                    var isNotDatabase = !(connectedResource instanceof database);
                    if (isNotDatabase && connectedResource instanceof fileSystem) {
                        resourceObservableArray = shell.fileSystems;
                        activeResourceObservable = _this.activeFilesystem;
                    }
                    else if (isNotDatabase && connectedResource instanceof counterStorage) {
                        resourceObservableArray = shell.counterStorages;
                        activeResourceObservable = _this.activeCounterStorage;
                    }
                    else if (isNotDatabase && connectedResource instanceof timeSeries) {
                        resourceObservableArray = shell.timeSeries;
                        activeResourceObservable = _this.activeTimeSeries;
                    }
                    _this.selectNewActiveResourceIfNeeded(resourceObservableArray, activeResourceObservable);
                });
            }
        };
        shell.prototype.selectNewActiveResourceIfNeeded = function (resourceObservableArray, activeResourceObservable) {
            var activeResource = activeResourceObservable();
            var actualResourceObservableArray = resourceObservableArray().filter(function (rs) { return rs.name !== "<system>"; });
            if (!!activeResource && actualResourceObservableArray.contains(activeResource) === false) {
                if (actualResourceObservableArray.length > 0) {
                    resourceObservableArray().first().activate();
                }
                else {
                    shell.disconnectFromResourceChangesApi();
                    activeResourceObservable(null);
                    this.lastActivatedResource(null);
                }
                this.navigate(appUrl.forResources());
            }
        };
        shell.prototype.createNotifications = function () {
            var _this = this;
            return [
                this.globalChangesApi.watchDocsStartingWith("Raven/Databases/", function (e) { return _this.changesApiFiredForResource(e, shell.databases, _this.activeDatabase, TenantType.Database); }),
                this.globalChangesApi.watchDocsStartingWith("Raven/FileSystems/", function (e) { return _this.changesApiFiredForResource(e, shell.fileSystems, _this.activeFilesystem, TenantType.FileSystem); }),
                this.globalChangesApi.watchDocsStartingWith("Raven/Counters/", function (e) { return _this.changesApiFiredForResource(e, shell.counterStorages, _this.activeCounterStorage, TenantType.CounterStorage); }),
                this.globalChangesApi.watchDocsStartingWith("Raven/TimeSeries/", function (e) { return _this.changesApiFiredForResource(e, shell.timeSeries, _this.activeTimeSeries, TenantType.TimeSeries); }),
                this.globalChangesApi.watchDocsStartingWith(shell.studioConfigDocumentId, function () { return shell.fetchStudioConfig(); }),
                this.globalChangesApi.watchDocsStartingWith("Raven/Alerts", function () { return _this.fetchSystemDatabaseAlerts(); })
            ];
        };
        shell.prototype.changesApiFiredForResource = function (e, resourceObservableArray, activeResourceObservable, resourceType) {
            var _this = this;
            if (!!e.Id && (e.Type === "Delete" || e.Type === "Put")) {
                var receivedResourceName = e.Id.slice(e.Id.lastIndexOf('/') + 1);
                if (e.Type === "Delete") {
                    var resourceToDelete = resourceObservableArray.first(function (rs) { return rs.name == receivedResourceName; });
                    if (!!resourceToDelete) {
                        resourceObservableArray.remove(resourceToDelete);
                        //this.selectNewActiveResourceIfNeeded(resourceObservableArray, activeResourceObservable);
                        if (resourceType == TenantType.Database)
                            recentQueriesStorage.removeRecentQueries(resourceToDelete);
                    }
                }
                else {
                    var getSystemDocumentTask = new getSystemDocumentCommand(e.Id).execute();
                    getSystemDocumentTask.done(function (dto) {
                        var existingResource = resourceObservableArray.first(function (rs) { return rs.name == receivedResourceName; });
                        if (existingResource == null) {
                            existingResource = _this.createNewResource(resourceType, receivedResourceName, dto);
                            resourceObservableArray.unshift(existingResource);
                        }
                        else {
                            if (existingResource.disabled() != dto.Disabled) {
                                existingResource.disabled(dto.Disabled);
                                if (dto.Disabled === false && _this.currentConnectedResource.name === receivedResourceName) {
                                    existingResource.activate();
                                }
                            }
                        }
                        if (resourceType === TenantType.Database) {
                            var bundles = !!dto.Settings["Raven/ActiveBundles"] ? dto.Settings["Raven/ActiveBundles"].split(";") : [];
                            existingResource.activeBundles(bundles);
                            var indexingDisabled = _this.getIndexingDisbaledValue(dto.Settings["Raven/IndexingDisabled"]);
                            existingResource.indexingDisabled(indexingDisabled);
                            var isRejectclientsEnabled = _this.getIndexingDisbaledValue(dto.Settings["Raven/RejectClientsModeEnabled"]);
                            existingResource.rejectClientsMode(isRejectclientsEnabled);
                        }
                    });
                }
            }
        };
        shell.prototype.getIndexingDisbaledValue = function (indexingDisabledString) {
            if (indexingDisabledString === undefined || indexingDisabledString == null)
                return false;
            if (indexingDisabledString.toLowerCase() === "true")
                return true;
            return false;
        };
        shell.prototype.createNewResource = function (resourceType, resourceName, dto) {
            var newResource = null;
            if (resourceType === TenantType.Database) {
                newResource = new database(resourceName, true, dto.Disabled);
            }
            else if (resourceType === TenantType.FileSystem) {
                newResource = new fileSystem(resourceName, true, dto.Disabled);
            }
            else if (resourceType === TenantType.CounterStorage) {
                newResource = new counterStorage(resourceName, true, dto.Disabled);
            }
            else if (resourceType === TenantType.TimeSeries) {
                newResource = new timeSeries(resourceName, true, dto.Disabled);
            }
            return newResource;
        };
        shell.prototype.selectResource = function (rs) {
            rs.activate();
            var locationHash = window.location.hash;
            var isMainPage = locationHash === appUrl.forResources();
            if (isMainPage === false) {
                var updatedUrl = appUrl.forCurrentPage(rs);
                this.navigate(updatedUrl);
            }
        };
        shell.prototype.databasesLoaded = function (databases) {
            // we can't use appUrl.getSystemDatabase() here as it isn't loaded yet!
            this.systemDatabase = new database("<system>");
            this.systemDatabase.isSystem = true;
            this.systemDatabase.isVisible(false);
            shell.databases(databases.concat([this.systemDatabase]));
        };
        shell.prototype.launchDocEditor = function (docId, docsList) {
            var editDocUrl = appUrl.forEditDoc(docId, docsList ? docsList.collectionName : null, docsList ? docsList.currentItemIndex() : null, this.activeDatabase());
            this.navigate(editDocUrl);
        };
        shell.prototype.loadDatabases = function () {
            var _this = this;
            var deferred = $.Deferred();
            this.databasesLoadedTask = new getDatabasesCommand()
                .execute()
                .fail(function (result) { return _this.handleRavenConnectionFailure(result); })
                .done(function (results) {
                _this.databasesLoaded(results);
                shell.fetchStudioConfig();
                _this.fetchClusterTopology();
                _this.fetchServerBuildVersion();
                _this.fetchClientBuildVersion();
                shell.fetchLicenseStatus();
                _this.fetchSupportCoverage();
                _this.fetchSystemDatabaseAlerts();
                router.activate();
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.loadFileSystems = function () {
            var deferred = $.Deferred();
            new getFileSystemsCommand()
                .execute()
                .done(function (results) { return shell.fileSystems(results); })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.loadCounterStorages = function () {
            var deferred = $.Deferred();
            new getCounterStoragesCommand()
                .execute()
                .done(function (results) { return shell.counterStorages(results); })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.loadTimeSeries = function () {
            var deferred = $.Deferred();
            new getTimeSeriesCommand()
                .execute()
                .done(function (results) { return shell.timeSeries(results); })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.loadServerConfig = function () {
            var deferred = $.Deferred();
            new getServerConfigsCommand()
                .execute()
                .done(function (serverConfigs) {
                shell.isGlobalAdmin(serverConfigs.IsGlobalAdmin);
                shell.canReadWriteSettings(serverConfigs.CanReadWriteSettings);
                shell.canReadSettings(serverConfigs.CanReadSettings);
                shell.canExposeConfigOverTheWire(serverConfigs.CanExposeConfigOverTheWire);
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        shell.prototype.connectToRavenServer = function () {
            var _this = this;
            var serverConfigsLoadTask = this.loadServerConfig();
            var databasesLoadTask = this.loadDatabases();
            var fileSystemsLoadTask = this.loadFileSystems();
            var counterStoragesLoadTask = this.loadCounterStorages();
            var timeSeriesLoadTask = this.loadTimeSeries();
            $.when(serverConfigsLoadTask, databasesLoadTask, fileSystemsLoadTask, counterStoragesLoadTask, timeSeriesLoadTask)
                .always(function () {
                var locationHash = window.location.hash;
                if (appUrl.getFileSystem()) {
                    _this.activateResource(appUrl.getFileSystem(), shell.fileSystems, appUrl.forResources);
                }
                else if (appUrl.getCounterStorage()) {
                    _this.activateResource(appUrl.getCounterStorage(), shell.counterStorages, appUrl.forResources);
                }
                else if (appUrl.getTimeSeries()) {
                    _this.activateResource(appUrl.getTimeSeries(), shell.timeSeries, appUrl.forResources);
                }
                else if ((locationHash.indexOf(appUrl.forAdminSettings()) === -1)) {
                    _this.activateResource(appUrl.getDatabase(), shell.databases, appUrl.forResources);
                }
            });
        };
        shell.prototype.activateResource = function (resource, resourceObservableArray, url) {
            if (!!resource) {
                var newResource = resourceObservableArray.first(function (rs) { return rs.name === resource.name; });
                if (newResource != null) {
                    newResource.activate();
                }
                else {
                    messagePublisher.reportError("The resource " + resource.name + " doesn't exist!");
                    this.navigate(url());
                }
            }
        };
        shell.prototype.navigateToResources = function () {
            shell.disconnectFromResourceChangesApi();
            if (!!this.activeDatabase()) {
                shell.databases().length === 1 ? this.activeDatabase(null) : this.activeDatabase().activate();
            }
            else if (!!this.activeFilesystem()) {
                this.activeFilesystem().activate();
            }
            else if (!!this.activeCounterStorage()) {
                this.activeCounterStorage().activate();
            }
            else if (!!this.activeTimeSeries()) {
                this.activeTimeSeries().activate();
            }
            this.navigate(appUrl.forResources());
        };
        shell.activateHotSpareEnvironment = function (hotSpare) {
            var color = new environmentColor(hotSpare.ActivationMode === "Activated" ? "Active Hot Spare" : "Hot Spare", "#FF8585");
            license.hotSpare(hotSpare);
            shell.selectedEnvironmentColorStatic(color);
            shell.originalEnvironmentColor(color);
        };
        shell.prototype.handleRavenConnectionFailure = function (result) {
            var _this = this;
            NProgress.done();
            if (result.status === 401) {
                // Unauthorized might be caused by invalid credentials. 
                // Remove them from both local storage and oauth context.
                apiKeyLocalStorage.clean();
                oauthContext.clean();
            }
            sys.log("Unable to connect to Raven.", result);
            var tryAgain = "Try again";
            var messageBoxResultPromise = this.confirmationMessage(':-(', "Couldn't connect to Raven. Details in the browser console.", [tryAgain]);
            messageBoxResultPromise.done(function () {
                NProgress.start();
                _this.connectToRavenServer();
            });
        };
        shell.prototype.dataLoadProgress = function (splashType) {
            if (!splashType) {
                NProgress.configure({ showSpinner: false });
                NProgress.done();
            }
            else if (splashType === 2 /* warning */) {
                NProgress.configure({ showSpinner: true });
                NProgress.start();
            }
            else {
                NProgress.done();
                NProgress.configure({ showSpinner: false });
                //this.showAlert(new alertArgs(alertType.danger, "Load time is too long", "The server might not be responding."));
                $.blockUI({ message: '<div id="longTimeoutMessage"><span> This is taking longer than usual</span><br/><span>(Waiting for server to respond)</span></div>' });
            }
        };
        shell.prototype.showAlert = function (alert) {
            var _this = this;
            if (alert.displayInRecentErrors && (alert.type === 3 /* danger */ || alert.type === 2 /* warning */)) {
                this.recordedErrors.unshift(alert);
            }
            var currentAlert = this.currentAlert();
            if (currentAlert) {
                this.queuedAlert = alert;
                this.closeAlertAndShowNext(currentAlert);
            }
            else {
                this.currentAlert(alert);
                var fadeTime = 2000; // If there are no pending alerts, show it for 2 seconds before fading out.
                /*            if (alert.title.indexOf("Changes stream was disconnected.") == 0) {
                                fadeTime = 100000000;
                            }*/
                if (alert.type === 3 /* danger */ || alert.type === 2 /* warning */) {
                    fadeTime = 5000; // If there are pending alerts, show the error alert for 4 seconds before fading out.
                }
                setTimeout(function () {
                    _this.closeAlertAndShowNext(alert);
                }, fadeTime);
            }
        };
        shell.prototype.closeAlertAndShowNext = function (alertToClose) {
            var _this = this;
            var alertElement = $('#' + alertToClose.id);
            if (alertElement.length === 0) {
                return;
            }
            // If the mouse is over the alert, keep it around.
            if (alertElement.is(":hover")) {
                setTimeout(function () { return _this.closeAlertAndShowNext(alertToClose); }, 1000);
            }
            else {
                alertElement.alert("close");
            }
        };
        shell.prototype.onAlertHidden = function () {
            this.currentAlert(null);
            var nextAlert = this.queuedAlert;
            if (nextAlert) {
                this.queuedAlert = null;
                this.showAlert(nextAlert);
            }
        };
        shell.prototype.newDocument = function () {
            this.launchDocEditor(null);
        };
        shell.prototype.activateDatabaseWithName = function (databaseName) {
            var _this = this;
            if (this.databasesLoadedTask) {
                this.databasesLoadedTask.done(function () {
                    var matchingDatabase = shell.databases().first(function (db) { return db.name == databaseName; });
                    if (matchingDatabase && _this.activeDatabase() !== matchingDatabase) {
                        ko.postbox.publish("ActivateDatabase", matchingDatabase);
                    }
                });
            }
        };
        shell.disconnectFromResourceChangesApi = function () {
            if (changesContext.currentResourceChangesApi()) {
                shell.changeSubscriptionArray.forEach(function (subscripbtion) { return subscripbtion.off(); });
                shell.changeSubscriptionArray = [];
                changesContext.currentResourceChangesApi().dispose();
                if (changesContext.currentResourceChangesApi().getResourceName() !== "<system>") {
                    viewModelBase.isConfirmedUsingSystemDatabase = false;
                }
                changesContext.currentResourceChangesApi(null);
            }
        };
        shell.prototype.getCurrentActiveFeatureName = function () {
            if (this.appUrls.isAreaActive("admin")()) {
                return "Manage Your Server";
            }
            else {
                return "Resources";
            }
        };
        shell.prototype.getCurrentActiveFeatureHref = function () {
            if (this.appUrls.isAreaActive("admin")()) {
                return this.appUrls.adminSettings();
            }
            else {
                return this.appUrls.resources();
            }
        };
        shell.prototype.goToDoc = function (doc) {
            this.goToDocumentSearch("");
            this.navigate(appUrl.forEditDoc(doc["@metadata"]["@id"], null, null, this.activeDatabase()));
        };
        shell.prototype.getDocCssClass = function (doc) {
            return collection.getCollectionCssClass(doc["@metadata"]["Raven-Entity-Name"], this.activeDatabase());
        };
        shell.prototype.fetchServerBuildVersion = function () {
            var _this = this;
            new getServerBuildVersionCommand()
                .execute()
                .done(function (serverBuildResult) {
                _this.serverBuildVersion(serverBuildResult);
                var currentBuildVersion = serverBuildResult.BuildVersion;
                if (currentBuildVersion !== 13) {
                    shell.serverMainVersion(Math.floor(currentBuildVersion / 10000));
                }
                if (serverBuildReminder.isReminderNeeded() && currentBuildVersion !== 13) {
                    new getLatestServerBuildVersionCommand(true, 35000, 39999) //pass false as a parameter to get the latest unstable
                        .execute()
                        .done(function (latestServerBuildResult) {
                        if (latestServerBuildResult.LatestBuild > currentBuildVersion) {
                            var latestBuildReminderViewModel = new latestBuildReminder(latestServerBuildResult);
                            app.showDialog(latestBuildReminderViewModel);
                        }
                    });
                }
            });
        };
        shell.prototype.fetchClientBuildVersion = function () {
            var _this = this;
            new getClientBuildVersionCommand()
                .execute()
                .done(function (result) { _this.clientBuildVersion(result); });
        };
        shell.prototype.fetchClusterTopology = function () {
            new getClusterTopologyCommand(appUrl.getSystemDatabase())
                .execute()
                .done(function (topology) {
                shell.clusterMode(topology && topology.allNodes().length > 0);
            });
        };
        shell.fetchLicenseStatus = function () {
            return new getLicenseStatusCommand()
                .execute()
                .done(function (result) {
                if (result.Status.contains("AGPL")) {
                    result.Status = "Development Only";
                }
                license.licenseStatus(result);
            });
        };
        shell.prototype.fetchSupportCoverage = function () {
            new getSupportCoverageCommand()
                .execute()
                .done(function (result) {
                license.supportCoverage(result);
            });
        };
        shell.prototype.fetchGoToDocSearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.activeDatabase())
                    .execute()
                    .done(function (results) {
                    if (_this.goToDocumentSearch() === query) {
                        _this.goToDocumentSearchResults(results);
                    }
                });
            }
            else if (query.length == 0) {
                this.goToDocumentSearchResults.removeAll();
            }
        };
        shell.prototype.showApiKeyDialog = function () {
            var dialog = new enterApiKey();
            return app.showDialog(dialog).then(function () { return window.location.href = "#resources"; });
        };
        shell.prototype.showErrorsDialog = function () {
            var errorDetails = new recentErrors(this.recordedErrors);
            app.showDialog(errorDetails);
        };
        shell.prototype.uploadStatusChanged = function (item) {
            var queue = uploadQueueHelper.parseUploadQueue(window.localStorage[uploadQueueHelper.localStorageUploadQueueKey + item.filesystem.name], item.filesystem);
            uploadQueueHelper.updateQueueStatus(item.id(), item.status(), queue);
            uploadQueueHelper.updateLocalStorage(queue, item.filesystem);
        };
        shell.prototype.showLicenseStatusDialog = function () {
            var dialog = new licensingStatus(license.licenseStatus(), license.supportCoverage(), license.hotSpare());
            app.showDialog(dialog);
        };
        shell.prototype.fetchSystemDatabaseAlerts = function () {
            new getDocumentWithMetadataCommand("Raven/Alerts", this.systemDatabase)
                .execute()
                .done(function (doc) {
                //
            });
        };
        shell.prototype.logOut = function () {
            // this call dispatches storage event to all tabs in browser and calls onLogOut inside each.
            apiKeyLocalStorage.clean();
            apiKeyLocalStorage.notifyAboutLogOut();
        };
        shell.prototype.onLogOut = function () {
            window.location.hash = this.appUrls.hasApiKey();
            window.location.reload();
        };
        shell.prototype.navigateToClusterSettings = function () {
            this.navigate(this.appUrls.adminSettingsCluster());
        };
        shell.prototype.selectNone = function () {
            this.activateDatabase(null);
            this.activeFilesystem(null);
        };
        shell.resourcesNamesComputed = function () {
            return ko.computed(function () {
                var resourcesNames = shell.resources().map(function (rs) { return rs.name; });
                return resourcesNames.distinct();
            });
        };
        shell.studioConfigDocumentId = "Raven/StudioConfig";
        shell.selectedEnvironmentColorStatic = ko.observable(new environmentColor("Default", "#f8f8f8"));
        shell.originalEnvironmentColor = ko.observable(shell.selectedEnvironmentColorStatic());
        shell.isGlobalAdmin = ko.observable(false);
        shell.canReadWriteSettings = ko.observable(false);
        shell.canReadSettings = ko.observable(false);
        shell.canExposeConfigOverTheWire = ko.observable(false);
        shell.databases = ko.observableArray();
        shell.fileSystems = ko.observableArray();
        shell.counterStorages = ko.observableArray();
        shell.timeSeries = ko.observableArray();
        shell.resources = ko.computed(function () {
            var databases = shell.databases();
            var fileSystems = shell.fileSystems();
            var counterStorages = shell.counterStorages();
            var timeSeries = shell.timeSeries();
            var result = databases.concat(counterStorages, timeSeries, fileSystems);
            return result.sort(function (a, b) {
                if (a.name === "<system>")
                    return 1;
                if (b.name === "<system>")
                    return -1;
                return a.name.toLowerCase() > b.name.toLowerCase() ? 1 : -1;
            });
        });
        shell.clusterMode = ko.observable(false);
        shell.serverMainVersion = ko.observable(3);
        shell.has40Features = ko.computed(function () { return shell.serverMainVersion() >= 4; });
        shell.reloadDatabases = function () { return shell.reloadResources(function () { return new getDatabasesCommand().execute(); }, shell.databases); };
        shell.reloadFileSystems = function () { return shell.reloadResources(function () { return new getFileSystemsCommand().execute(); }, shell.fileSystems); };
        shell.reloadCounterStorages = function () { return shell.reloadResources(function () { return new getCounterStoragesCommand().execute(); }, shell.counterStorages); };
        shell.reloadTimeSeries = function () { return shell.reloadResources(function () { return new getTimeSeriesCommand().execute(); }, shell.timeSeries); };
        return shell;
    })(viewModelBase);
    return shell;
});
define("viewmodels/viewModelBase", ["require", "exports", "common/appUrl", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "plugins/router", "durandal/app", "viewmodels/common/viewSystemDatabaseConfirm", "common/oauthContext", "common/changesContext", "common/messagePublisher", "viewmodels/common/confirmationDialog", "commands/database/documents/saveDocumentCommand", "models/database/documents/document", "common/downloader"], function (require, exports, appUrl, filesystem, counterStorage, timeSeries, router, app, viewSystemDatabaseConfirm, oauthContext, changesContext, messagePublisher, confirmationDialog, saveDocumentCommand, document, downloader) {
    /*
     * Base view model class that provides basic view model services, such as tracking the active database and providing a means to add keyboard shortcuts.
    */
    var viewModelBase = (function () {
        function viewModelBase() {
            var _this = this;
            this.activeDatabase = ko.observable().subscribeTo("ActivateDatabase", true);
            this.activeFilesystem = ko.observable().subscribeTo("ActivateFilesystem", true);
            this.activeCounterStorage = ko.observable().subscribeTo("ActivateCounterStorage", true);
            this.activeTimeSeries = ko.observable().subscribeTo("ActivateTimeSeries", true);
            this.lastActivatedResource = ko.observable()
                .subscribeTo("ActivateDatabase", true)
                .subscribeTo("ActivateFilesystem", true)
                .subscribeTo("ActivateCounterStorage", true)
                .subscribeTo("ActivateTimeSeries", true);
            this.downloader = new downloader();
            this.keyboardShortcutDomContainers = [];
            this.notifications = [];
            this.postboxSubscriptions = [];
            this.isAttached = false;
            this.dirtyFlag = new ko.DirtyFlag([]);
            this.currentHelpLink = ko.observable().subscribeTo('globalHelpLink', true);
            this.beforeUnloadListener = function (e) {
                var isDirty = _this.dirtyFlag().isDirty();
                if (isDirty) {
                    var message = "You have unsaved data.";
                    e = e || window.event;
                    // For IE and Firefox
                    if (e) {
                        e.returnValue = message;
                    }
                    // For Safari
                    return message;
                }
            };
            this.appUrls = appUrl.forCurrentDatabase();
        }
        /*
         * Called by Durandal when checking whether this navigation is allowed.
         * Possible return values: boolean, promise<boolean>, {redirect: 'some/other/route'}, promise<{redirect: 'some/other/route'}>
         *
         * We use this to determine whether we should allow navigation to a system DB page.
         * p.s. from Judah: a big scary prompt when loading the system DB is a bit heavy-handed, no?
         */
        viewModelBase.prototype.canActivate = function (args) {
            var self = this;
            setTimeout(function () { return viewModelBase.showSplash(self.isAttached === false); }, 700);
            this.downloader.reset();
            var resource = appUrl.getResource();
            if (resource instanceof filesystem) {
                var fs = this.activeFilesystem();
                if (!!fs && fs.disabled()) {
                    messagePublisher.reportError("File system '" + fs.name + "' is disabled!", "You can't access any section of the file system while it's disabled.");
                    return { redirect: appUrl.forResources() };
                }
            }
            else if (resource instanceof counterStorage) {
                var cs = this.activeCounterStorage();
                if (!!cs && cs.disabled()) {
                    messagePublisher.reportError("Counter Storage '" + cs.name + "' is disabled!", "You can't access any section of the counter storage while it's disabled.");
                    return { redirect: appUrl.forResources() };
                }
            }
            else if (resource instanceof timeSeries) {
                var ts = this.activeTimeSeries();
                if (!!ts && ts.disabled()) {
                    messagePublisher.reportError("Time Series '" + ts.name + "' is disabled!", "You can't access any section of the time series while it's disabled.");
                    return { redirect: appUrl.forResources() };
                }
            }
            else {
                var db = this.activeDatabase();
                // we only want to prompt warning to system db if we are in the databases section
                if (!!db && db.isSystem) {
                    return this.promptNavSystemDb();
                }
                else if (!!db && db.disabled()) {
                    messagePublisher.reportError("Database '" + db.name + "' is disabled!", "You can't access any section of the database while it's disabled.");
                    return { redirect: appUrl.forResources() };
                }
                viewModelBase.isConfirmedUsingSystemDatabase = false;
            }
            return true;
        };
        /*
         * Called by Durandal when the view model is loaded and before the view is inserted into the DOM.
         */
        viewModelBase.prototype.activate = function (args, isShell) {
            var _this = this;
            if (isShell === void 0) { isShell = false; }
            var db = appUrl.getDatabase();
            var currentDb = this.activeDatabase();
            if (!!db && (!currentDb || currentDb.name !== db.name)) {
                ko.postbox.publish("ActivateDatabaseWithName", db.name);
            }
            // create this ko.computed once to avoid creation and subscribing every 50 ms - thus creating memory leak.
            var adminArea = this.appUrls.isAreaActive("admin");
            oauthContext.enterApiKeyTask.done(function () {
                // we have to wait for changes api to connect as well
                // as obtaining changes api connection might take a while, we have to spin until connection is read
                var createNotifySpinFunction = function () {
                    if (isShell || adminArea())
                        return;
                    if (changesContext.currentResourceChangesApi && changesContext.currentResourceChangesApi()) {
                        _this.notifications = _this.createNotifications();
                    }
                    else {
                        setTimeout(createNotifySpinFunction, 50);
                    }
                };
                createNotifySpinFunction();
            });
            this.postboxSubscriptions = this.createPostboxSubscriptions();
            this.modelPollingStart();
            ko.postbox.publish("SetRawJSONUrl", "");
            this.updateHelpLink(null); // clean link
        };
        viewModelBase.prototype.attached = function () {
            window.addEventListener("beforeunload", this.beforeUnloadListener, false);
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        /*
         * Called by Durandal when the view model is loaded and after the view is inserted into the DOM.
         */
        viewModelBase.prototype.compositionComplete = function () {
            this.dirtyFlag().reset(); //Resync Changes
        };
        /*
         * Called by Durandal before deactivate in order to determine whether removing from the DOM is necessary.
         */
        viewModelBase.prototype.canDeactivate = function (isClose) {
            var _this = this;
            var isDirty = this.dirtyFlag().isDirty();
            if (isDirty) {
                var discard = "Discard changes";
                var stay = "Stay on this page";
                var discardStayResult = $.Deferred();
                var confirmation = this.confirmationMessage("Unsaved changes", "You have unsaved changes. How do you want to proceed?", [discard, stay], true);
                confirmation.done(function (result) {
                    if (!result.can) {
                        _this.dirtyFlag().reset();
                    }
                    result.can = !result.can;
                    discardStayResult.resolve(result);
                });
                return discardStayResult;
            }
            return true;
        };
        /*
         * Called by Durandal when the view model is unloaded and after the view is removed from the DOM.
         */
        viewModelBase.prototype.detached = function () {
            this.activeDatabase.unsubscribeFrom("ActivateDatabase");
            this.activeFilesystem.unsubscribeFrom("ActivateFilesystem");
            this.activeCounterStorage.unsubscribeFrom("ActivateCounterStorage");
            this.activeTimeSeries.unsubscribeFrom("ActivateTimeSeries");
            this.lastActivatedResource.unsubscribeFrom("ActivateDatabase");
            this.lastActivatedResource.unsubscribeFrom("ActivateFilesystem");
            this.lastActivatedResource.unsubscribeFrom("ActivateCounterStorage");
            this.lastActivatedResource.unsubscribeFrom("ActivateTimeSeries");
            this.currentHelpLink.unsubscribeFrom("currentHelpLink");
            this.cleanupNotifications();
            this.cleanupPostboxSubscriptions();
            window.removeEventListener("beforeunload", this.beforeUnloadListener, false);
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        /*
         * Called by Durandal when the view model is unloading and the view is about to be removed from the DOM.
         */
        viewModelBase.prototype.deactivate = function () {
            var _this = this;
            this.keyboardShortcutDomContainers.forEach(function (el) { return _this.removeKeyboardShortcuts(el); });
            this.modelPollingStop();
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        viewModelBase.prototype.createNotifications = function () {
            return [];
        };
        viewModelBase.prototype.cleanupNotifications = function () {
            this.notifications.forEach(function (notification) { return notification.off(); });
            this.notifications = [];
        };
        viewModelBase.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        viewModelBase.prototype.cleanupPostboxSubscriptions = function () {
            this.postboxSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.postboxSubscriptions = [];
        };
        /*
         * Creates a keyboard shortcut local to the specified element and its children.
         * The shortcut will be removed as soon as the view model is deactivated.
         * Also defines shortcut for ace editor, if ace editor was received
         */
        viewModelBase.prototype.createKeyboardShortcut = function (keys, handler, elementSelector) {
            jwerty.key(keys, function (e) {
                e.preventDefault();
                handler();
            }, this, elementSelector);
            if (!this.keyboardShortcutDomContainers.contains(elementSelector)) {
                this.keyboardShortcutDomContainers.push(elementSelector);
            }
        };
        viewModelBase.prototype.removeKeyboardShortcuts = function (elementSelector) {
            $(elementSelector).unbind('keydown.jwerty');
        };
        /*
         * Navigates to the specified URL, recording a navigation event in the browser's history.
         */
        viewModelBase.prototype.navigate = function (url) {
            router.navigate(url);
        };
        /*
         * Navigates by replacing the current URL. It does not record a new entry in the browser's navigation history.
         */
        viewModelBase.prototype.updateUrl = function (url) {
            var options = {
                replace: true,
                trigger: false
            };
            router.navigate(url, options);
        };
        viewModelBase.prototype.modelPolling = function () {
            return null;
        };
        viewModelBase.prototype.pollingWithContinuation = function () {
            var _this = this;
            var poolPromise = this.modelPolling();
            if (poolPromise) {
                poolPromise.always(function () {
                    viewModelBase.modelPollingHandle = setTimeout(function () {
                        viewModelBase.modelPollingHandle = null;
                        _this.pollingWithContinuation();
                    }, 5000);
                });
            }
        };
        viewModelBase.prototype.modelPollingStart = function () {
            // clear previous pooling handle (if any)
            if (viewModelBase.modelPollingHandle) {
                this.modelPollingStop();
            }
            this.pollingWithContinuation();
        };
        viewModelBase.prototype.modelPollingStop = function () {
            clearTimeout(viewModelBase.modelPollingHandle);
            viewModelBase.modelPollingHandle = null;
        };
        viewModelBase.prototype.confirmationMessage = function (title, confirmationMessage, options, forceRejectWithResolve) {
            if (options === void 0) { options = ["No", "Yes"]; }
            if (forceRejectWithResolve === void 0) { forceRejectWithResolve = false; }
            var viewTask = $.Deferred();
            var confirmTask = app.showDialog(new confirmationDialog(confirmationMessage, title, options));
            confirmTask.done(function (answer) {
                var isConfirmed = answer === options.last();
                if (isConfirmed) {
                    viewTask.resolve({ can: true });
                }
                else if (!forceRejectWithResolve) {
                    viewTask.reject();
                }
                else {
                    viewTask.resolve({ can: false });
                }
            });
            return viewTask;
        };
        viewModelBase.prototype.canContinueIfNotDirty = function (title, confirmationMessage) {
            var deferred = $.Deferred();
            var isDirty = this.dirtyFlag().isDirty();
            if (isDirty) {
                var confirmationMessageViewModel = this.confirmationMessage(title, confirmationMessage);
                confirmationMessageViewModel.done(function () { return deferred.resolve(); });
            }
            else {
                deferred.resolve();
            }
            return deferred;
        };
        viewModelBase.prototype.promptNavSystemDb = function (forceRejectWithResolve) {
            if (forceRejectWithResolve === void 0) { forceRejectWithResolve = false; }
            var canNavTask = $.Deferred();
            if (!appUrl.warnWhenUsingSystemDatabase || viewModelBase.isConfirmedUsingSystemDatabase) {
                canNavTask.resolve({ can: true });
            }
            else {
                var systemDbConfirm = new viewSystemDatabaseConfirm("Meddling with the system database could cause irreversible damage");
                systemDbConfirm.viewTask
                    .fail(function () { return forceRejectWithResolve == false ? canNavTask.resolve({ redirect: appUrl.forResources() }) : canNavTask.reject(); })
                    .done(function () {
                    viewModelBase.isConfirmedUsingSystemDatabase = true;
                    canNavTask.resolve({ can: true });
                });
                app.showDialog(systemDbConfirm);
            }
            return canNavTask;
        };
        viewModelBase.prototype.AddNotification = function (subscription) {
            this.notifications.push(subscription);
        };
        viewModelBase.prototype.RemoveNotification = function (subscription) {
            this.notifications.remove(subscription);
        };
        viewModelBase.prototype.continueTest = function () {
            var doc = document.empty();
            new saveDocumentCommand("Debug/Done", doc, this.activeDatabase(), false)
                .execute()
                .done(function () { return viewModelBase.hasContinueTestOption(false); });
        };
        viewModelBase.prototype.updateHelpLink = function (hash) {
            if (hash === void 0) { hash = null; }
            if (hash) {
                var version = viewModelBase.clientVersion();
                if (version) {
                    var href = "http://ravendb.net/l/" + hash + "/" + version + "/";
                    ko.postbox.publish('globalHelpLink', href);
                    return;
                }
                var subscribtion = viewModelBase.clientVersion.subscribe(function (v) {
                    var href = "http://ravendb.net/l/" + hash + "/" + v + "/";
                    ko.postbox.publish('globalHelpLink', href);
                    if (subscribtion) {
                        subscribtion.dispose();
                    }
                });
            }
            else {
                ko.postbox.publish('globalHelpLink', null);
            }
        };
        viewModelBase.isConfirmedUsingSystemDatabase = false;
        viewModelBase.showSplash = ko.observable(false);
        //holds full studio version eg. 3.0.3528
        viewModelBase.clientVersion = ko.observable();
        viewModelBase.hasContinueTestOption = ko.observable(false);
        return viewModelBase;
    })();
    return viewModelBase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/confirmationDialog", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog"], function (require, exports, dialogViewModelBase, dialog) {
    /*
     * Generic confirmation dialog. We created this because Durandal's built-in
     * message box doesn't handle keyboard shortcuts like ESC and Enter.
     */
    var confirmationDialog = (function (_super) {
        __extends(confirmationDialog, _super);
        function confirmationDialog(confirmationMessage, title, options) {
            _super.call(this);
            this.confirmationMessage = confirmationMessage;
            this.title = title;
            this.options = options;
        }
        confirmationDialog.prototype.onOptionClicked = function (option) {
            dialog.close(this, option);
        };
        confirmationDialog.prototype.close = function () {
            dialog.close(this, null);
        };
        return confirmationDialog;
    })(dialogViewModelBase);
    return confirmationDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/deleteDocuments", ["require", "exports", "plugins/dialog", "commands/database/documents/deleteDocumentsCommand", "common/appUrl", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteDocumentsCommand, appUrl, dialogViewModelBase) {
    var deleteDocuments = (function (_super) {
        __extends(deleteDocuments, _super);
        function deleteDocuments(documents, elementToFocusOnDismissal) {
            _super.call(this, elementToFocusOnDismissal);
            this.documents = ko.observableArray();
            this.deletionStarted = false;
            this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
            if (documents.length === 0) {
                throw new Error("Must have at least one document to delete.");
            }
            this.documents(documents);
        }
        deleteDocuments.prototype.deleteDocs = function () {
            var _this = this;
            var deletedDocIds = this.documents().map(function (i) { return i.getId(); });
            var deleteCommand = new deleteDocumentsCommand(deletedDocIds, appUrl.getDatabase());
            var deleteCommandTask = deleteCommand.execute();
            deleteCommandTask.done(function () { return _this.deletionTask.resolve(_this.documents()); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteDocuments.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteDocuments.prototype.deactivate = function (args) {
            _super.prototype.deactivate.call(this, args);
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteDocuments;
    })(dialogViewModelBase);
    return deleteDocuments;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/deleteItems", ["require", "exports", "models/database/documents/document", "models/filesystem/file", "models/counter/counterSummary", "models/timeSeries/timeSeriesPoint", "models/timeSeries/timeSeriesKey", "models/timeSeries/timeSeriesType", "plugins/dialog", "commands/database/documents/deleteDocumentsCommand", "commands/filesystem/deleteFilesCommand", "commands/counter/deleteCountersCommand", "commands/timeSeries/deletePointsCommand", "commands/timeSeries/deleteTypesCommand", "common/appUrl", "viewmodels/dialogViewModelBase"], function (require, exports, document, file, counterSummary, timeSeriesPoint, timeSeriesKey, timeSeriesType, dialog, deleteDocumentsCommand, deleteFilesCommand, deleteCountersCommand, deletePointsCommand, deleteTypesCommand, appUrl, dialogViewModelBase) {
    var deleteItems = (function (_super) {
        __extends(deleteItems, _super);
        function deleteItems(items, elementToFocusOnDismissal) {
            _super.call(this, elementToFocusOnDismissal);
            this.items = ko.observableArray();
            this.deletionStarted = false;
            this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
            if (items.length === 0) {
                throw new Error("Must have at least one document to delete.");
            }
            this.items(items);
        }
        deleteItems.prototype.deleteItems = function () {
            var _this = this;
            var deleteItemsIds = this.items().map(function (i) { return i.getUrl(); });
            var deleteCommand;
            var firstItem = this.items()[0];
            if (firstItem instanceof document) {
                deleteCommand = new deleteDocumentsCommand(deleteItemsIds, appUrl.getDatabase());
            }
            else if (firstItem instanceof file) {
                deleteCommand = new deleteFilesCommand(deleteItemsIds, appUrl.getFileSystem());
            }
            else if (firstItem instanceof counterSummary) {
                var counters = this.items();
                var groupAndNames = counters.map(function (x) {
                    return {
                        groupName: x.getEntityName(),
                        counterName: x.getId()
                    };
                });
                deleteCommand = new deleteCountersCommand(groupAndNames, appUrl.getCounterStorage());
            }
            else if (firstItem instanceof timeSeriesPoint) {
                var points = this.items().map(function (x) {
                    return {
                        Type: x.type,
                        Key: x.key,
                        At: x.At
                    };
                });
                deleteCommand = new deletePointsCommand(points, appUrl.getTimeSeries());
            }
            else if (firstItem instanceof timeSeriesKey) {
                debugger;
            }
            else if (firstItem instanceof timeSeriesType) {
                var types = this.items().map(function (type) {
                    return type.name;
                });
                deleteCommand = new deleteTypesCommand(types, appUrl.getTimeSeries());
            }
            else {
                debugger;
            }
            var deleteCommandTask = deleteCommand.execute();
            deleteCommandTask.done(function () { return _this.deletionTask.resolve(_this.items()); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteItems.prototype.getDeletedItemName = function () {
            var item = this.items()[0];
            if (item instanceof counterSummary) {
                var summary = item;
                return " counter name: " + summary.getId() + ", group name: " + summary.getEntityName();
            }
            return item.getId();
        };
        deleteItems.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteItems.prototype.deactivate = function (args) {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteItems;
    })(dialogViewModelBase);
    return deleteItems;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/enterApiKey", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/oauthContext", "common/apiKeyLocalStorage"], function (require, exports, dialog, dialogViewModelBase, oauthContext, apiKeyLocalStorage) {
    var enterApiKey = (function (_super) {
        __extends(enterApiKey, _super);
        function enterApiKey() {
            _super.apply(this, arguments);
            this.apiKey = ko.observable("");
            this.saveInLocalStorage = ko.observable(false);
        }
        enterApiKey.prototype.cancel = function () {
            dialog.close(this);
        };
        enterApiKey.prototype.ok = function () {
            oauthContext.apiKey(this.apiKey());
            if (this.saveInLocalStorage()) {
                apiKeyLocalStorage.setValue(this.apiKey());
            }
            else {
                apiKeyLocalStorage.clean();
            }
            dialog.close(this);
        };
        return enterApiKey;
    })(dialogViewModelBase);
    return enterApiKey;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/latestBuildReminder", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/serverBuildReminder"], function (require, exports, dialog, dialogViewModelBase, serverBuildReminder) {
    var latestBuildReminder = (function (_super) {
        __extends(latestBuildReminder, _super);
        function latestBuildReminder(latestServerBuildResult, elementToFocusOnDismissal) {
            var _this = this;
            _super.call(this, elementToFocusOnDismissal);
            this.latestServerBuildResult = latestServerBuildResult;
            this.dialogTask = $.Deferred();
            this.mute = ko.observable(false);
            this.mute.subscribe(function () {
                serverBuildReminder.mute(_this.mute());
            });
        }
        latestBuildReminder.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.dialogTask.resolve();
        };
        latestBuildReminder.prototype.close = function () {
            dialog.close(this);
        };
        return latestBuildReminder;
    })(dialogViewModelBase);
    return latestBuildReminder;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/licensingStatus", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "moment", "common/generalUtils"], function (require, exports, dialog, dialogViewModelBase, moment, genUtils) {
    var licensingStatus = (function (_super) {
        __extends(licensingStatus, _super);
        function licensingStatus(licenseStatus, supportCoverage, hotSpare) {
            var _this = this;
            _super.call(this);
            this.licenseStatus = licenseStatus;
            this.isHotSpare = false;
            this.isDevelopmentOnly = false;
            this.isNonExpiredCommercial = false;
            this.isExpired = false;
            if (licenseStatus.Attributes.hotSpare === "true") {
                this.isHotSpare = true;
                if (hotSpare.ActivationMode === "Activated") {
                    this.licenseStatusText = "Hot Spare: Activated";
                    this.licenseExpiresAt = moment(hotSpare.ActivationTime).add("days", 4).format("YYYY-MMM-DD");
                }
                else {
                    this.licenseStatusText = "Hot Spare: Not Activated";
                    this.licenseExpiresAt = null;
                }
            }
            else {
                this.isDevelopmentOnly = !licenseStatus.IsCommercial;
                this.isNonExpiredCommercial = licenseStatus.IsCommercial && !licenseStatus.Status.contains("Expired");
                this.isExpired = licenseStatus.IsCommercial && licenseStatus.Status.contains("Expired");
                this.licenseStatusText = licenseStatus.Status;
                this.licenseExpiresAt = licenseStatus.Attributes.updatesExpiration;
            }
            this.supportStatus = supportCoverage.Status;
            this.isProfessionalSupport = supportCoverage.Status === 'ProfessionalSupport';
            this.isProductionSupport = supportCoverage.Status === 'ProductionSupport';
            this.isPartialSupport = supportCoverage.Status === 'PartialSupport';
            this.isCommunitySupport = !this.isProfessionalSupport && !this.isProductionSupport && !this.isPartialSupport;
            var rawFormatted = licenseStatus.Attributes.maxRamUtilization === "unlimited" ? "unlimited" : genUtils.formatBytesToSize(parseInt(licenseStatus.Attributes.maxRamUtilization));
            this.attrRam = this.prepareHtmlForAttribute(rawFormatted);
            this.attrCpus = this.prepareHtmlForAttribute(licenseStatus.Attributes.maxParallelism);
            this.supportExpiresAt = supportCoverage.EndsAt ? moment(supportCoverage.EndsAt).format("YYYY-MMM-DD") : null;
            this.licensePath = licenseStatus.LicensePath;
            this.attributes = [
                { displayName: "Databases", value: licenseStatus.Attributes.numberOfDatabases },
                { displayName: "Database size", value: licenseStatus.Attributes.maxSizeInMb },
                { displayName: "RavenFS", value: licenseStatus.Attributes.ravenfs },
                { displayName: "Periodic Backup", value: licenseStatus.Attributes.periodicBackup },
                { displayName: "Replication", value: licenseStatus.Attributes.replication },
                { displayName: "Encryption", value: licenseStatus.Attributes.encryption },
                { displayName: "Compression", value: licenseStatus.Attributes.compression },
                { displayName: "FIPS Compliance", value: licenseStatus.Attributes.fips },
                { displayName: "Quotas", value: licenseStatus.Attributes.quotas },
                { displayName: "Global Configuration", value: licenseStatus.Attributes.globalConfigurations },
                { displayName: "Authorization", value: licenseStatus.Attributes.authorization },
                { displayName: "Document Expiration", value: licenseStatus.Attributes.documentExpiration },
                { displayName: "Versioning", value: licenseStatus.Attributes.versioning },
                { displayName: "Cluster", value: licenseStatus.Attributes.clustering },
                { displayName: "Monitoring", value: licenseStatus.Attributes.monitoring },
                { displayName: "Hot Spare", value: licenseStatus.Attributes.hotSpare },
                { displayName: "Allow Windows Clustering", value: licenseStatus.Attributes.allowWindowsClustering }
            ];
            this.attributes.forEach(function (attr) { return attr.value = _this.prepareHtmlForAttribute(attr.value); });
        }
        licensingStatus.prototype.prepareHtmlForAttribute = function (input) {
            if (input === 'unlimited') {
                return '<i class="fa fa-infinity"></i>';
            }
            else if (input === 'true') {
                return '<i class="fa fa-check"></i>';
            }
            else if (input === 'false' || !input) {
                return '<i class="fa fa-times"></i>';
            }
            return input;
        };
        licensingStatus.prototype.cancel = function () {
            dialog.close(this);
        };
        licensingStatus.prototype.ok = function () {
            dialog.close(this);
        };
        return licensingStatus;
    })(dialogViewModelBase);
    return licensingStatus;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/recentErrors", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var recentErrors = (function (_super) {
        __extends(recentErrors, _super);
        function recentErrors(errors) {
            _super.call(this);
            this.errors = errors;
        }
        recentErrors.prototype.attached = function () {
            _super.prototype.attached.call(this);
            // Expand the first error.
            if (this.errors().length > 0) {
                $("#errorDetailsCollapse0").collapse("show");
            }
            this.registerResizing("recentErrorsResize");
        };
        recentErrors.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.unregisterResizing("recentErrorsResize");
        };
        recentErrors.prototype.clear = function () {
            this.errors.removeAll();
            dialog.close(this);
        };
        recentErrors.prototype.close = function () {
            dialog.close(this);
        };
        recentErrors.prototype.getErrorDetails = function (alert) {
            var error = alert.errorInfo;
            if (error != null && error.stackTrace) {
                return error.stackTrace.replace("\r\n", "\n");
            }
            return alert.details;
        };
        recentErrors.prototype.getDangerAlertType = function () {
            return 3 /* danger */;
        };
        recentErrors.prototype.getWarningAlertType = function () {
            return 2 /* warning */;
        };
        return recentErrors;
    })(dialogViewModelBase);
    return recentErrors;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/selectColumns", ["require", "exports", "models/database/documents/document", "plugins/dialog", "viewmodels/dialogViewModelBase", "models/database/documents/customColumnParams", "commands/database/documents/saveDocumentCommand", "commands/database/documents/deleteDocumentCommand", "common/autoCompleterSupport", "common/messagePublisher"], function (require, exports, document, dialog, dialogViewModelBase, customColumnParams, saveDocumentCommand, deleteDocumentCommand, autoCompleterSupport, messagePublisher) {
    var selectColumns = (function (_super) {
        __extends(selectColumns, _super);
        function selectColumns(customColumns, customFunctions, context, database, names) {
            var _this = this;
            _super.call(this);
            this.customColumns = customColumns;
            this.customFunctions = customFunctions;
            this.context = context;
            this.database = database;
            this.nextTask = $.Deferred();
            this.nextTaskStarted = false;
            this.autoCompleteBase = ko.observableArray([]);
            this.autoCompleteResults = ko.observableArray([]);
            this.completionSearchSubscriptions = [];
            this.columnsNames = ko.observableArray([]);
            this.maxTableHeight = ko.observable();
            this.lineHeight = 51;
            this.columnsNames(names);
            this.generateCompletionBase();
            this.regenerateBindingSubscriptions();
            this.monitorForNewRows();
            this.autoCompleterSupport = new autoCompleterSupport(this.autoCompleteBase, this.autoCompleteResults, true);
            this.maxTableHeight(Math.floor($(window).height() * 0.43));
            $(window).resize(function () {
                _this.maxTableHeight(Math.floor($(window).height() * 0.43));
                _this.alignBoxVertically();
            });
            this.isScrollNeeded = ko.computed(function () {
                var currentColumnsCount = _this.customColumns.columns().length;
                var currentColumnHeight = currentColumnsCount * _this.lineHeight;
                if (currentColumnHeight > _this.maxTableHeight()) {
                    return true;
                }
                return false;
            });
        }
        selectColumns.prototype.generateCompletionBase = function () {
            this.autoCompleteBase([]);
            this.updateColumnsNames();
            var moduleSource = "var exports = {}; " + this.customFunctions.functions + "; return exports;";
            var exports = new Function(moduleSource)();
            for (var funcName in exports) {
                this.autoCompleteBase().push(ko.observable(funcName + "()"));
            }
        };
        selectColumns.prototype.regenerateBindingSubscriptions = function () {
            var _this = this;
            this.completionSearchSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.completionSearchSubscriptions = [];
            this.customColumns.columns().forEach(function (column, index) {
                return _this.completionSearchSubscriptions.push(column.binding.subscribe(_this.searchForCompletions.bind(_this)));
            });
        };
        selectColumns.prototype.monitorForNewRows = function () {
            var _this = this;
            this.customColumns.columns().forEach(function (column) { return column.binding.subscribe(function () {
                _this.updateColumnsNames();
            }); });
            this.customColumns.columns.subscribe(function (changes) {
                _this.updateColumnsNames();
                var somethingRemoved = false;
                changes.forEach(function (change) {
                    if (change.status === "added") {
                        _this.completionSearchSubscriptions.push(change.value.binding.subscribe(_this.searchForCompletions.bind(_this)));
                    }
                    else if (change.status === "deleted") {
                        _this.completionSearchSubscriptions.push(change.value.binding.subscribe(_this.searchForCompletions.bind(_this)));
                        somethingRemoved = true;
                    }
                    if (somethingRemoved) {
                        _this.regenerateBindingSubscriptions();
                    }
                });
            }, null, "arrayChange");
        };
        selectColumns.prototype.updateColumnsNames = function () {
            var customColumnsArray = this.customColumns.columns();
            var customColumnsNamesArray = customColumnsArray.map(function (column) { return column.binding(); });
            var allColumns = this.columnsNames();
            var arrayWithoutCustomNames = allColumns.filter(function (name) {
                var filteredNames = customColumnsNamesArray.filter(function (customName) { return customName.localeCompare(name) === 0; });
                return filteredNames.length === 0;
            });
            var array = arrayWithoutCustomNames.map(function (name) { return ko.observable(name); });
            this.autoCompleteBase(array);
        };
        selectColumns.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.form = $("#selectColumnsForm");
        };
        selectColumns.prototype.cancel = function () {
            dialog.close(this);
        };
        selectColumns.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        selectColumns.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        selectColumns.prototype.changeCurrentColumns = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.customColumns);
            dialog.close(this);
        };
        selectColumns.prototype.insertNewRow = function () {
            this.customColumns.columns.push(customColumnParams.empty());
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        selectColumns.prototype.deleteRow = function (row) {
            this.customColumns.columns.remove(row);
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        selectColumns.prototype.moveUp = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i - 1, 2, array[i], array[i - 1]);
            }
        };
        selectColumns.prototype.moveDown = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 0 && i < this.customColumns.columns().length - 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i, 2, array[i + 1], array[i]);
            }
        };
        selectColumns.prototype.customScheme = function (val) {
            if (this.customColumns.customMode() != val) {
                this.customColumns.customMode(val);
                this.alignBoxVertically();
            }
        };
        selectColumns.prototype.saveAsDefault = function () {
            var _this = this;
            if (this.form[0].checkValidity() === true) {
                if (this.customColumns.customMode()) {
                    var configurationDocument = new document(this.customColumns.toDto());
                    new saveDocumentCommand(this.context, configurationDocument, this.database, false).execute()
                        .done(function () { return _this.onConfigSaved(); })
                        .fail(function () { return messagePublisher.reportError("Unable to save configuration!"); });
                }
                else {
                    new deleteDocumentCommand(this.context, this.database).execute().done(function () { return _this.onConfigSaved(); })
                        .fail(function () { return messagePublisher.reportError("Unable to save configuration!"); });
                }
            }
            else {
                messagePublisher.reportWarning('Configuration contains errors. Not saving it.');
            }
        };
        selectColumns.prototype.onConfigSaved = function () {
            messagePublisher.reportSuccess('Configuration saved!');
        };
        selectColumns.prototype.generateBindingInputId = function (index) {
            return 'binding-' + index;
        };
        selectColumns.prototype.enterKeyPressed = function () {
            var focusedBindingInput = $("[id ^= 'binding-']:focus");
            if (focusedBindingInput.length) {
                // insert first completion
                if (this.autoCompleteResults().length > 0) {
                    this.completeTheWord(this.autoCompleteResults()[0]());
                }
                // prevent submitting the form and closing dialog when accepting completion
                return false;
            }
            return _super.prototype.enterKeyPressed.call(this);
        };
        selectColumns.prototype.consumeUpDownArrowKeys = function (columnParams, event) {
            if (event.keyCode === 38 || event.keyCode === 40) {
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.consumeClick = function (columnParams, event) {
            if (columnParams.binding().length === 0) {
                columnParams.binding.valueHasMutated();
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.searchForCompletions = function () {
            this.activeInput = $("[id ^= 'binding-']:focus");
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.searchForCompletions(this.activeInput);
            }
        };
        selectColumns.prototype.completeTheWord = function (selectedCompletion) {
            var _this = this;
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.completeTheWord(this.activeInput, selectedCompletion, function (newValue) {
                    var columnParams = ko.dataFor(_this.activeInput[0]);
                    columnParams.binding(newValue);
                });
            }
        };
        return selectColumns;
    })(dialogViewModelBase);
    return selectColumns;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/selectCsvColumns", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/autoCompleterSupport"], function (require, exports, dialog, dialogViewModelBase, autoCompleterSupport) {
    var csvColumn = (function () {
        function csvColumn(value) {
            if (value === void 0) { value = ""; }
            this.name = ko.observable(value);
        }
        return csvColumn;
    })();
    var selectColumns = (function (_super) {
        __extends(selectColumns, _super);
        function selectColumns(columnNames) {
            var _this = this;
            _super.call(this);
            this.columnNames = columnNames;
            this.nextTask = $.Deferred();
            this.nextTaskStarted = false;
            this.lineHeight = 51;
            this.maxTableHeight = ko.observable();
            this.csvColumns = ko.observableArray([]);
            this.autoCompleteBase = ko.observableArray([]);
            this.autoCompleteResults = ko.observableArray([]);
            this.completionSearchSubscriptions = [];
            this.csvColumns(columnNames.map(function (x) { return new csvColumn(x); }));
            this.generateCompletionBase();
            this.regenerateBindingSubscriptions();
            this.monitorForNewRows();
            this.autoCompleterSupport = new autoCompleterSupport(this.autoCompleteBase, this.autoCompleteResults, true);
            this.maxTableHeight(Math.floor($(window).height() * 0.43));
            $(window).resize(function () {
                _this.maxTableHeight(Math.floor($(window).height() * 0.43));
                _this.alignBoxVertically();
            });
            this.isScrollNeeded = ko.computed(function () {
                var currentColumnsCount = _this.csvColumns().length;
                var currentColumnHeight = currentColumnsCount * _this.lineHeight;
                return currentColumnHeight > _this.maxTableHeight();
            });
        }
        selectColumns.prototype.generateCompletionBase = function () {
            this.autoCompleteBase([]);
            this.prepareAutocompleteOptions();
        };
        selectColumns.prototype.regenerateBindingSubscriptions = function () {
            var _this = this;
            this.completionSearchSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.completionSearchSubscriptions = [];
            this.csvColumns().forEach(function (column) {
                return _this.completionSearchSubscriptions.push(column.name.subscribe(_this.searchForCompletions.bind(_this)));
            });
        };
        selectColumns.prototype.monitorForNewRows = function () {
            var _this = this;
            this.csvColumns().forEach(function (column) { return column.name.subscribe(function () {
                _this.prepareAutocompleteOptions();
            }); });
            this.csvColumns.subscribe(function (changes) {
                _this.prepareAutocompleteOptions();
                var somethingRemoved = false;
                changes.forEach(function (change) {
                    if (change.status === "added" || change.status === "deleted") {
                        _this.completionSearchSubscriptions.push(change.value.name.subscribe(_this.searchForCompletions.bind(_this)));
                    }
                    if (change.status === "deleted") {
                        somethingRemoved = true;
                    }
                });
                if (somethingRemoved) {
                    _this.regenerateBindingSubscriptions();
                }
            }, null, "arrayChange");
        };
        /**
         * Prepare auto complate options by filtering out already used columns
         */
        selectColumns.prototype.prepareAutocompleteOptions = function () {
            var alreadyUsedColumns = this.csvColumns().map(function (x) { return x.name(); }).filter(function (x) { return !!x; });
            var allAvailableColumns = this.columnNames;
            var notAlreadyUsedColumns = allAvailableColumns.filter(function (name) {
                var nameLower = name.toLocaleLowerCase();
                return !alreadyUsedColumns.first(function (x) { return x.toLocaleLowerCase() === nameLower; });
            });
            this.autoCompleteBase(notAlreadyUsedColumns.map(function (x) { return ko.observable(x); }));
        };
        selectColumns.prototype.cancel = function () {
            dialog.close(this);
        };
        selectColumns.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        selectColumns.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        selectColumns.prototype.exportCsv = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.csvColumns().map(function (x) { return x.name(); }));
            dialog.close(this);
        };
        selectColumns.prototype.insertNewRow = function () {
            this.csvColumns.push(new csvColumn());
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        selectColumns.prototype.deleteRow = function (row) {
            this.csvColumns.remove(row);
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        selectColumns.prototype.moveUp = function (row) {
            var i = this.csvColumns.indexOf(row);
            if (i >= 1) {
                var array = this.csvColumns();
                this.csvColumns.splice(i - 1, 2, array[i], array[i - 1]);
            }
        };
        selectColumns.prototype.moveDown = function (row) {
            var i = this.csvColumns.indexOf(row);
            if (i >= 0 && i < this.csvColumns().length - 1) {
                var array = this.csvColumns();
                this.csvColumns.splice(i, 2, array[i + 1], array[i]);
            }
        };
        selectColumns.prototype.generateBindingInputId = function (index) {
            return 'binding-' + index;
        };
        selectColumns.prototype.enterKeyPressed = function () {
            var focusedBindingInput = $("[id ^= 'binding-']:focus");
            if (focusedBindingInput.length) {
                // insert first completion
                if (this.autoCompleteResults().length > 0) {
                    this.completeTheWord(this.autoCompleteResults()[0]());
                }
                // prevent submitting the form and closing dialog when accepting completion
                return false;
            }
            return _super.prototype.enterKeyPressed.call(this);
        };
        selectColumns.prototype.consumeUpDownArrowKeys = function (columnParams, event) {
            if (event.keyCode === 38 || event.keyCode === 40) {
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.consumeClick = function (columnParams, event) {
            if (columnParams.name().length === 0) {
                columnParams.name.valueHasMutated();
                event.preventDefault();
                return false;
            }
            return true;
        };
        selectColumns.prototype.searchForCompletions = function () {
            this.activeInput = $("[id ^= 'binding-']:focus");
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.searchForCompletions(this.activeInput);
            }
        };
        selectColumns.prototype.completeTheWord = function (selectedCompletion) {
            var _this = this;
            if (this.activeInput.length > 0) {
                this.autoCompleterSupport.completeTheWord(this.activeInput, selectedCompletion, function (newValue) {
                    var columnParams = ko.dataFor(_this.activeInput[0]);
                    columnParams.name(newValue);
                });
            }
        };
        return selectColumns;
    })(dialogViewModelBase);
    return selectColumns;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/showDataDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var showDataDialog = (function (_super) {
        __extends(showDataDialog, _super);
        function showDataDialog(title, inputData, elementToFocusOnDismissal) {
            _super.call(this, elementToFocusOnDismissal);
            this.title = title;
            this.inputData = inputData;
        }
        showDataDialog.prototype.canActivate = function (args) {
            return true;
        };
        showDataDialog.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.registerResizing("documentsResize");
            this.selectText();
            jwerty.key("CTRL+C, enter", function (e) {
                e.preventDefault();
                _this.close();
            }, this, "#documentsText");
        };
        showDataDialog.prototype.deactivate = function () {
            $("#inputData").unbind('keydown.jwerty');
        };
        showDataDialog.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.unregisterResizing("documentsResize");
        };
        showDataDialog.prototype.selectText = function () {
            $("#inputData").select();
        };
        showDataDialog.prototype.close = function () {
            dialog.close(this);
        };
        return showDataDialog;
    })(dialogViewModelBase);
    return showDataDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/upgrade", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "plugins/router", "common/appUrl"], function (require, exports, viewModelBase, getDatabaseStatsCommand, router, appUrl) {
    var upgrade = (function (_super) {
        __extends(upgrade, _super);
        function upgrade() {
            _super.apply(this, arguments);
            this.inProgress = ko.observable(true);
        }
        upgrade.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.poolStats();
            ko.postbox.subscribe("ActivateDatabase", function (db) { return _this.dbChanged(db); });
        };
        upgrade.prototype.dbChanged = function (db) {
            router.navigate(appUrl.forDocuments(null, db));
        };
        upgrade.prototype.poolStats = function () {
            var _this = this;
            this.fetchStats()
                .done(function () {
                _this.inProgress(false);
            }).fail(function () {
                _this.timeoutHandle = setTimeout(_this.poolStats(), 500);
            });
        };
        upgrade.prototype.detached = function () {
            clearTimeout(this.timeoutHandle);
        };
        upgrade.prototype.fetchStats = function () {
            var db = this.activeDatabase();
            if (db) {
                return new getDatabaseStatsCommand(db)
                    .execute();
            }
            return null;
        };
        return upgrade;
    })(viewModelBase);
    return upgrade;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/common/viewSystemDatabaseConfirm", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var viewSystemDatabaseConfirm = (function (_super) {
        __extends(viewSystemDatabaseConfirm, _super);
        function viewSystemDatabaseConfirm(confirmationMessage, previousDb) {
            if (previousDb === void 0) { previousDb = null; }
            _super.call(this);
            this.confirmationMessage = confirmationMessage;
            this.previousDb = previousDb;
            this.viewTask = $.Deferred();
            this.wasConfirmed = false;
        }
        viewSystemDatabaseConfirm.prototype.viewSystemDatabase = function () {
            this.viewTask.resolve();
            this.wasConfirmed = true;
            dialog.close(this);
        };
        viewSystemDatabaseConfirm.prototype.cancel = function () {
            this.viewTask.reject(this.previousDb);
            this.wasConfirmed = false;
            dialog.close(this);
        };
        viewSystemDatabaseConfirm.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (!this.wasConfirmed) {
                this.viewTask.reject(this.previousDb);
            }
        };
        return viewSystemDatabaseConfirm;
    })(dialogViewModelBase);
    return viewSystemDatabaseConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/counters", ["require", "exports", "durandal/app", "plugins/router", "common/changesContext", "common/appUrl", "models/counter/counterChange", "models/counter/counterGroup", "commands/counter/getCounterGroupsCommand", "commands/counter/updateCounterCommand", "commands/counter/resetCounterCommand", "viewmodels/viewModelBase", "viewmodels/counter/editCounterDialog", "viewmodels/counter/deleteGroup"], function (require, exports, app, router, changesContext, appUrl, counterChange, counterGroup, getCounterGroupsCommand, updateCounterCommand, resetCounterCommand, viewModelBase, editCounterDialog, deleteGroup) {
    var counters = (function (_super) {
        __extends(counters, _super);
        function counters() {
            var _this = this;
            _super.call(this);
            this.viewType = 1 /* Counters */;
            this.groups = ko.observableArray();
            this.selectedGroup = ko.observable().subscribeTo("ActivateGroup").distinctUntilChanged();
            this.currentGroup = ko.observable();
            this.currentGroupPagedItems = ko.observable();
            this.selectedCounterIndices = ko.observableArray();
            this.isAnyCountersAutoSelected = ko.observable(false);
            this.isAllCountersAutoSelected = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.isInitialized = counters.isInitialized;
            this.selectedGroup.subscribe(function (c) { return _this.selectedGroupChanged(c); });
            this.isAllGroupsGroup = ko.computed(function () { return !!_this.selectedGroup() ? _this.selectedGroup().isAllGroupsGroup : true; });
            this.hasCounters = ko.computed(function () {
                var selectedGroup = _this.selectedGroup();
                if (!!selectedGroup) {
                    if (selectedGroup.name === counterGroup.allGroupsGroupName) {
                        var cs = _this.activeCounterStorage();
                        return !!cs.statistics() ? cs.statistics().countersCount() > 0 : false;
                    }
                    return _this.selectedGroup().countersCount() > 0;
                }
                return false;
            });
            this.hasAnyCountersSelected = ko.computed(function () { return _this.selectedCounterIndices().length > 0; });
            this.hasAllCountersSelected = ko.computed(function () {
                var numOfSelectedCounters = _this.selectedCounterIndices().length;
                if (!!_this.selectedGroup() && numOfSelectedCounters !== 0) {
                    return numOfSelectedCounters === _this.selectedGroup().countersCount();
                }
                return false;
            });
            this.selectedCountersText = ko.computed(function () {
                if (!!_this.selectedCounterIndices()) {
                    var documentsText = "counter";
                    if (_this.selectedCounterIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            this.countersSelection = ko.computed(function () {
                if (_this.hasAllCountersSelected())
                    return checkbox.Checked;
                if (_this.hasAnyCountersSelected())
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
        }
        counters.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a group name to view's URL, e.g. #counterstorages/counters?group=Foo&counterstorage=test
            this.groupToSelectName = args ? args.group : null;
            var cs = this.activeCounterStorage();
            //initial groups fetch
            this.fetchGroups(cs, 0, Number.MAX_VALUE).done(function (results) {
                _this.groupsLoaded(results, cs);
                counters.isInitialized(true);
            });
        };
        counters.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedCounter(), counters.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        counters.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            counters.isInitialized(false);
        };
        counters.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllCounters(function (e) { return _this.refreshGroups(); }),
                changesContext.currentResourceChangesApi().watchCounterBulkOperation(function () { return _this.refreshGroups(); })
            ];
        };
        counters.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("ChangeCounterValue", function () { return _this.change(); }),
                ko.postbox.subscribe("ResetCounter", function () { return _this.reset(); }),
                ko.postbox.subscribe("ChangesApiReconnected", function (cs) { return _this.reloadCountersData(cs); }),
                ko.postbox.subscribe("SortGroups", function () { return _this.sortGroups(); }),
                ko.postbox.subscribe("SelectGroup", function (groupName) {
                    var groupToSelect = _this.groups.first(function (g) { return g.name === groupName; });
                    if (!!groupToSelect) {
                        _this.selectGroupInternal(groupToSelect);
                    }
                })
            ];
        };
        counters.prototype.fetchGroups = function (cs, skip, pageSize) {
            var deferred = $.Deferred();
            var getGroupsCommand = new getCounterGroupsCommand(cs, skip, pageSize);
            getGroupsCommand.execute().done(function (results) { return deferred.resolve(results); });
            return deferred;
        };
        counters.prototype.groupsLoaded = function (groups, cs) {
            var _this = this;
            // Create the "All Groups" pseudo collection.
            this.allGroupsGroup = counterGroup.createAllGroupsCollection(cs);
            this.allGroupsGroup.countersCount = ko.computed(function () { return !!cs.statistics() ? cs.statistics().countersCount() : 0; });
            // All systems a-go. Load them into the UI and select the first one.
            var allGroups = [this.allGroupsGroup].concat(groups);
            this.groups(allGroups);
            var groupToSelect = this.groups.first(function (g) { return g.name === _this.groupToSelectName; }) || this.allGroupsGroup;
            groupToSelect.activate();
        };
        counters.prototype.newCounter = function () {
            var _this = this;
            var counterChangeVm;
            var currentGroupName = this.selectedGroup().name;
            if (currentGroupName === counterGroup.allGroupsGroupName) {
                counterChangeVm = new editCounterDialog();
            }
            else {
                var dto = {
                    CurrentValue: 0,
                    Group: currentGroupName,
                    CounterName: "",
                    Delta: 0
                };
                var change = new counterChange(dto, true);
                counterChangeVm = new editCounterDialog(change);
            }
            counterChangeVm.updateTask.done(function (change) {
                var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), change.isNew());
                var execute = counterCommand.execute();
                execute.done(function () { return _this.refreshGridAndGroup(change.group()); });
            });
            app.showDialog(counterChangeVm);
        };
        counters.prototype.refresh = function () {
            var selectedGroupName = this.selectedGroup().name;
            this.refreshGridAndGroup(selectedGroupName);
        };
        counters.prototype.edit = function () {
            var grid = this.getCountersGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        counters.prototype.change = function () {
            var _this = this;
            var grid = this.getCountersGrid();
            if (grid) {
                var counterData = grid.getSelectedItems(1).first();
                var dto = {
                    CurrentValue: counterData.Total,
                    Group: counterData["Group Name"],
                    CounterName: counterData["Counter Name"],
                    Delta: 0
                };
                var change = new counterChange(dto);
                var counterChangeVm = new editCounterDialog(change);
                counterChangeVm.updateTask.done(function (change, isNew) {
                    var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), isNew);
                    var execute = counterCommand.execute();
                    execute.done(function () { return _this.refreshGridAndGroup(counterData.GroupName); });
                });
                app.showDialog(counterChangeVm);
            }
        };
        counters.prototype.reset = function () {
            var _this = this;
            var grid = this.getCountersGrid();
            if (grid) {
                var counterData = grid.getSelectedItems(1).first();
                var confirmation = this.confirmationMessage("Reset Counter", "Are you sure that you want to reset the counter?");
                confirmation.done(function () {
                    var resetCommand = new resetCounterCommand(_this.activeCounterStorage(), counterData["Group Name"], counterData["Counter Name"]);
                    var execute = resetCommand.execute();
                    execute.done(function () { return _this.refreshGridAndGroup(counterData.GroupName); });
                });
            }
        };
        counters.prototype.refreshGridAndGroup = function (changedGroupName) {
            var group = this.selectedGroup();
            if (group.name === changedGroupName || group.name === counterGroup.allGroupsGroupName) {
                this.getCountersGrid().refreshCollectionData();
            }
            group.invalidateCache();
            this.selectNone();
        };
        counters.prototype.selectedGroupChanged = function (selected) {
            if (!!selected) {
                var pagedList = selected.getCounters();
                this.currentGroupPagedItems(pagedList);
                this.currentGroup(selected);
            }
        };
        counters.prototype.toggleSelectAll = function () {
            var countersGrid = this.getCountersGrid();
            if (!!countersGrid) {
                if (this.hasAnyCountersSelected()) {
                    countersGrid.selectNone();
                }
                else {
                    countersGrid.selectSome();
                    this.isAnyCountersAutoSelected(this.hasAllCountersSelected() === false);
                }
            }
        };
        counters.prototype.selectAll = function () {
            var countersGrid = this.getCountersGrid();
            var group = this.selectedGroup();
            if (!!countersGrid && !!group) {
                countersGrid.selectAll(group.countersCount());
            }
        };
        counters.prototype.selectNone = function () {
            var countersGrid = this.getCountersGrid();
            if (!!countersGrid) {
                countersGrid.selectNone();
            }
        };
        counters.prototype.deleteSelectedCounters = function () {
            if (this.hasAllCountersSelected()) {
                this.deleteGroupInternal(this.selectedGroup());
            }
            else {
                var grid = this.getCountersGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        counters.prototype.deleteGroupInternal = function (group) {
            var _this = this;
            var deleteGroupVm = new deleteGroup(group, this.activeCounterStorage());
            deleteGroupVm.deletionTask.done(function () {
                if (!group.isAllGroupsGroup) {
                    _this.groups.remove(group);
                    var selectedCollection = _this.selectedGroup();
                    if (group.name === selectedCollection.name) {
                        _this.selectedGroup(_this.allGroupsGroup);
                    }
                }
                else {
                    _this.refreshGridAndGroup(group.name);
                }
            });
            app.showDialog(deleteGroupVm);
        };
        counters.prototype.updateGroups = function (receivedGroups) {
            var _this = this;
            var deletedGroups = [];
            this.groups().forEach(function (group) {
                if (!receivedGroups.first(function (receivedGroup) { return group.name === receivedGroup.name; }) && group.name !== "All Groups") {
                    deletedGroups.push(group);
                }
            });
            this.groups.removeAll(deletedGroups);
            receivedGroups.forEach(function (receivedGroup) {
                var foundGroup = _this.groups().first(function (group) { return group.name === receivedGroup.name; });
                if (!foundGroup) {
                    _this.groups.push(receivedGroup);
                }
                else {
                    foundGroup.countersCount(receivedGroup.countersCount());
                }
            });
            //if the group is deleted, go to the all groups group
            var currentGroup = this.groups().first(function (g) { return g.name === _this.selectedGroup().name; });
            if (!currentGroup || currentGroup.countersCount() === 0) {
                this.selectedGroup(this.allGroupsGroup);
            }
        };
        counters.prototype.refreshGroupsData = function () {
            var _this = this;
            var selectedGroup = this.selectedGroup();
            this.groups().forEach(function (group) {
                if (group.name === selectedGroup.name) {
                    var docsGrid = _this.getCountersGrid();
                    if (!!docsGrid) {
                        docsGrid.refreshCollectionData();
                    }
                }
                else {
                    var pagedList = group.getCounters();
                    pagedList.invalidateCache();
                }
            });
        };
        counters.prototype.refreshGroups = function () {
            var _this = this;
            var deferred = $.Deferred();
            var cs = this.activeCounterStorage();
            //TODO: add proper paging support here too
            this.fetchGroups(cs, 0, Number.MAX_VALUE).done(function (results) {
                _this.updateGroups(results);
                _this.refreshGroupsData();
                //TODO: add a button to refresh the counters and than use this.refreshCollectionsData();
                deferred.resolve();
            });
            return deferred;
        };
        counters.prototype.reloadCountersData = function (cs) {
            var _this = this;
            if (cs.name === this.activeCounterStorage().name) {
                this.refreshGroups().done(function () { return _this.refreshGroupsData(); });
            }
        };
        counters.prototype.selectGroup = function (group, event) {
            if (!event || event.which !== 3) {
                this.selectGroupInternal(group);
            }
        };
        counters.prototype.selectGroupInternal = function (group) {
            group.activate();
            var countersWithGroupUrl = appUrl.forCounterStorageCounters(group.name, this.activeCounterStorage());
            router.navigate(countersWithGroupUrl, false);
        };
        counters.prototype.getCountersGrid = function () {
            var gridContents = $(counters.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        counters.prototype.sortGroups = function () {
            this.groups.sort(function (c1, c2) {
                if (c1.isAllGroupsGroup)
                    return -1;
                if (c2.isAllGroupsGroup)
                    return 1;
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the groups list
        counters.prototype.showGroupElement = function (element) {
            if (element.nodeType === 1 && counters.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortGroups");
                    $(element).highlight();
                });
            }
        };
        counters.prototype.hideGroupElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        counters.gridSelector = "#countersGrid";
        counters.isInitialized = ko.observable(false);
        return counters;
    })(viewModelBase);
    return counters;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/counterStorageConfiguration", ["require", "exports", "viewmodels/viewModelBase"], function (require, exports, viewModelBase) {
    var counterStorageConfiguration = (function (_super) {
        __extends(counterStorageConfiguration, _super);
        function counterStorageConfiguration() {
            _super.apply(this, arguments);
        }
        counterStorageConfiguration.prototype.canActivate = function (args) {
            return true;
        };
        return counterStorageConfiguration;
    })(viewModelBase);
    return counterStorageConfiguration;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/counterStorageReplication", ["require", "exports", "common/appUrl", "viewmodels/viewModelBase", "models/counter/counterStorageReplicationSetup", "models/counter/counterStorageReplicationDestination", "commands/counter/getCounterStorageReplicationCommand", "commands/counter/saveCounterStorageReplicationCommand"], function (require, exports, appUrl, viewModelBase, counterStorageReplicationSetup, counterStorageReplicationDestination, getCounterStorageReplicationCommand, saveCounterStorageReplicationCommand) {
    var counterStorageReplication = (function (_super) {
        __extends(counterStorageReplication, _super);
        function counterStorageReplication() {
            _super.call(this);
            this.replicationsSetup = ko.observable().extend({ required: true });
            this.appUrls = appUrl.forCurrentCounterStorage();
        }
        counterStorageReplication.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            var counterStorage = this.activeCounterStorage();
            if (counterStorage) {
                this.fetchCountersDestinations(counterStorage)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forCounterStorage(counterStorage) }); });
            }
            return deferred;
        };
        counterStorageReplication.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.replicationsSetup]);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        };
        counterStorageReplication.prototype.fetchCountersDestinations = function (counterStorage, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            var deferred = $.Deferred();
            if (counterStorage) {
                new getCounterStorageReplicationCommand(counterStorage, reportFetchProgress)
                    .execute()
                    .done(function (data) { return _this.replicationsSetup(new counterStorageReplicationSetup({ Destinations: data.Destinations })); })
                    .fail(function () { return _this.replicationsSetup(new counterStorageReplicationSetup({ Destinations: [] })); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        counterStorageReplication.prototype.saveChanges = function () {
            var _this = this;
            var counter = this.activeCounterStorage();
            if (counter) {
                new saveCounterStorageReplicationCommand(this.replicationsSetup().toDto(), counter)
                    .execute()
                    .done(function () { return _this.dirtyFlag().reset(); });
            }
        };
        counterStorageReplication.prototype.createNewDestination = function () {
            var cs = this.activeCounterStorage();
            this.replicationsSetup().destinations.unshift(counterStorageReplicationDestination.empty(cs.name));
        };
        counterStorageReplication.prototype.removeDestination = function (resplicationDestination) {
            this.replicationsSetup().destinations.remove(resplicationDestination);
        };
        counterStorageReplication.prototype.refreshFromServer = function () {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to refresh the data from the server?');
            canContinue.done(function () {
                _this.fetchCountersDestinations(_this.activeCounterStorage(), true)
                    .done(function () { return _this.dirtyFlag().reset(); });
            });
        };
        return counterStorageReplication;
    })(viewModelBase);
    return counterStorageReplication;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/counterStorageStats", ["require", "exports", "viewmodels/viewModelBase"], function (require, exports, viewModelBase) {
    var counterStorageStats = (function (_super) {
        __extends(counterStorageStats, _super);
        function counterStorageStats() {
            _super.apply(this, arguments);
        }
        counterStorageStats.prototype.canActivate = function (args) {
            return true;
        };
        return counterStorageStats;
    })(viewModelBase);
    return counterStorageStats;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/deleteCounterStorageConfirm", ["require", "exports", "commands/counter/deleteCounterStorageCommand", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, deleteCounterStorageCommand, dialog, dialogViewModelBase) {
    var deleteCounterStorageConfirm = (function (_super) {
        __extends(deleteCounterStorageConfirm, _super);
        function deleteCounterStorageConfirm(storageToDelete) {
            _super.call(this);
            this.storageToDelete = storageToDelete;
            this.isKeepingFiles = ko.observable(true);
            this.deleteTask = $.Deferred();
            if (!storageToDelete) {
                throw new Error("Must specified counter storage to delete.");
            }
        }
        deleteCounterStorageConfirm.prototype.keepFiles = function () {
            this.isKeepingFiles(true);
        };
        deleteCounterStorageConfirm.prototype.deleteEverything = function () {
            this.isKeepingFiles(false);
        };
        deleteCounterStorageConfirm.prototype.deleteCounterStorage = function () {
            var _this = this;
            new deleteCounterStorageCommand(this.storageToDelete.name, this.isKeepingFiles() === false)
                .execute()
                .done(function (results) { return _this.deleteTask.resolve(results); })
                .fail(function (details) { return _this.deleteTask.reject(details); });
            dialog.close(this);
        };
        deleteCounterStorageConfirm.prototype.cancel = function () {
            this.deleteTask.reject();
            dialog.close(this);
        };
        return deleteCounterStorageConfirm;
    })(dialogViewModelBase);
    return deleteCounterStorageConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/deleteGroup", ["require", "exports", "plugins/dialog", "commands/counter/deleteCountersByGroupCommand", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteCountersByGroupCommand, dialogViewModelBase) {
    var deleteGroup = (function (_super) {
        __extends(deleteGroup, _super);
        function deleteGroup(group, cs) {
            _super.call(this);
            this.group = group;
            this.cs = cs;
            this.deletionTask = $.Deferred();
            this.deletionStarted = false;
        }
        deleteGroup.prototype.deleteGroup = function () {
            var _this = this;
            var deleteCommand = new deleteCountersByGroupCommand(this.group, this.cs);
            var deleteCommandTask = deleteCommand.execute();
            deleteCommandTask.done(function (result) { return _this.deletionTask.resolve(result); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteGroup.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteGroup.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteGroup;
    })(dialogViewModelBase);
    return deleteGroup;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/editCounter", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "common/messagePublisher", "models/counter/counterChange", "models/counter/counterSummary", "viewmodels/counter/editCounterDialog", "commands/counter/getCounterCommand", "commands/counter/updateCounterCommand", "commands/counter/resetCounterCommand", "viewmodels/viewModelBase", "viewmodels/common/deleteItems"], function (require, exports, app, router, appUrl, messagePublisher, counterChange, counterSummary, editCounterDialog, getCounterCommand, updateCounterCommand, resetCounterCommand, viewModelBase, deleteItems) {
    var editCounter = (function (_super) {
        __extends(editCounter, _super);
        function editCounter() {
            var _this = this;
            _super.call(this);
            this.groupName = ko.observable();
            this.counterName = ko.observable();
            this.groupLink = ko.observable();
            this.counter = ko.observable();
            this.isLoading = ko.observable();
            this.topRecentCounters = ko.computed(function () { return _this.getTopRecentCounters(); });
            this.isBusy = ko.observable(false);
        }
        editCounter.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deffered = $.Deferred();
            if (!args.groupName || !args.counterName) {
                messagePublisher.reportError("Can't find group name or counter name in query string!");
                deffered.resolve({ redirect: appUrl.forCounterStorageCounters(null, this.activeCounterStorage()) });
            }
            var cs = this.activeCounterStorage();
            this.load(args.groupName, args.counterName)
                .done(function () {
                _this.groupName(args.groupName);
                _this.counterName(args.counterName);
                _this.groupLink(appUrl.forCounterStorageCounters(args.groupName, cs));
                _this.appendRecentCounter(args.groupName, args.counterName);
                deffered.resolve({ can: true });
            })
                .fail(function () {
                messagePublisher.reportError("Can't find counter!");
                _this.removeFromTopRecentCounters(args.groupName, args.counterName);
                deffered.resolve({ redirect: appUrl.forCounterStorageCounters(null, cs) });
            });
            return deffered;
        };
        editCounter.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
        };
        editCounter.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteCounter(); }, editCounter.container);
        };
        editCounter.prototype.load = function (groupName, counterName) {
            var _this = this;
            this.isLoading(true);
            return new getCounterCommand(this.activeCounterStorage(), groupName, counterName)
                .execute()
                .done(function (result) {
                _this.counter(result);
                _this.isLoading(false);
            });
        };
        editCounter.prototype.refresh = function () {
            this.load(this.groupName(), this.counterName());
        };
        editCounter.prototype.change = function () {
            var _this = this;
            var dto = {
                CurrentValue: this.counter().total(),
                Group: this.groupName(),
                CounterName: this.counterName(),
                Delta: 0
            };
            var change = new counterChange(dto);
            var counterChangeVm = new editCounterDialog(change);
            counterChangeVm.updateTask.done(function (change, isNew) {
                var counterCommand = new updateCounterCommand(_this.activeCounterStorage(), change.group(), change.counterName(), change.delta(), isNew);
                var execute = counterCommand.execute();
                execute.done(function () { return _this.refresh(); });
            });
            app.showDialog(counterChangeVm);
        };
        editCounter.prototype.reset = function () {
            var _this = this;
            var confirmation = this.confirmationMessage("Reset Counter", "Are you sure that you want to reset the counter?");
            confirmation.done(function () {
                var resetCommand = new resetCounterCommand(_this.activeCounterStorage(), _this.groupName(), _this.counterName());
                var execute = resetCommand.execute();
                execute.done(function () { return _this.refresh(); });
            });
        };
        editCounter.prototype.deleteCounter = function () {
            var _this = this;
            var summary = new counterSummary({
                GroupName: this.groupName(),
                CounterName: this.counterName(),
                Total: this.counter().total()
            });
            var viewModel = new deleteItems([summary]);
            viewModel.deletionTask.done(function () {
                var countersUrl = appUrl.forCounterStorageCounters(null, _this.activeCounterStorage());
                router.navigate(countersUrl);
            });
            app.showDialog(viewModel, editCounter.container);
        };
        editCounter.prototype.removeFromTopRecentCounters = function (groupName, counterName) {
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = editCounter.recentCountersInCounterStorage().first(function (x) { return x.counterStorageName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                var counter = {
                    groupName: groupName,
                    counterName: counterName
                };
                recentFilesForCurFilesystem.recentCounters.remove(counter);
            }
        };
        editCounter.prototype.getTopRecentCounters = function () {
            var _this = this;
            var cs = this.activeCounterStorage();
            var recentFilesForCurFilesystem = editCounter.recentCountersInCounterStorage().first(function (x) { return x.counterStorageName === cs.name; });
            if (recentFilesForCurFilesystem) {
                var value = recentFilesForCurFilesystem
                    .recentCounters()
                    .filter(function (x) {
                    return x.groupName !== _this.groupName() && x.counterName !== _this.counterName();
                })
                    .slice(0, 5)
                    .map(function (groupAndCounterName) {
                    var groupName = groupAndCounterName.groupName;
                    var counterName = groupAndCounterName.counterName;
                    return {
                        groupName: groupName,
                        counterName: counterName,
                        counterUrl: appUrl.forEditCounter(cs, groupName, counterName)
                    };
                });
                return value;
            }
            else {
                return [];
            }
        };
        editCounter.prototype.appendRecentCounter = function (groupName, counterName) {
            var csName = this.activeCounterStorage().name;
            var existingRecentCounters = editCounter.recentCountersInCounterStorage.first(function (x) { return x.counterStorageName === csName; });
            if (existingRecentCounters) {
                var existingCounter = existingRecentCounters.recentCounters.first(function (x) { return x.groupName === groupName && x.counterName === counterName; });
                if (!existingCounter) {
                    if (existingRecentCounters.recentCounters().length === 5) {
                        existingRecentCounters.recentCounters.pop();
                    }
                    existingRecentCounters.recentCounters.unshift({ groupName: groupName, counterName: counterName });
                }
            }
            else {
                editCounter.recentCountersInCounterStorage.push({ counterStorageName: csName, recentCounters: ko.observableArray([{ groupName: groupName, counterName: counterName }]) });
            }
        };
        editCounter.container = "#editCounterContainer";
        editCounter.recentCountersInCounterStorage = ko.observableArray();
        return editCounter;
    })(viewModelBase);
    return editCounter;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/counter/editCounterDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "models/counter/counterChange"], function (require, exports, dialog, dialogViewModelBase, counterChange) {
    var editCounterDialog = (function (_super) {
        __extends(editCounterDialog, _super);
        function editCounterDialog(editedCounter) {
            var _this = this;
            _super.call(this);
            this.updateTask = $.Deferred();
            this.updateTaskStarted = false;
            this.editedCounter = ko.observable();
            this.editedCounter(!editedCounter ? counterChange.empty() : editedCounter);
            this.isNew = ko.computed(function () { return !!_this.editedCounter() && _this.editedCounter().isNew(); });
        }
        editCounterDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        editCounterDialog.prototype.nextOrCreate = function () {
            this.updateTaskStarted = true;
            this.updateTask.resolve(this.editedCounter());
            dialog.close(this);
        };
        editCounterDialog.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.updateTaskStarted) {
                this.updateTask.reject();
            }
        };
        return editCounterDialog;
    })(dialogViewModelBase);
    return editCounterDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/conflicts/conflicts", ["require", "exports", "plugins/router", "common/appUrl", "common/pagedList", "models/database/documents/customColumns", 'models/database/documents/customColumnParams', "common/changesContext", "commands/database/replication/getConflictsCommand", "commands/database/replication/getReplicationSourcesCommand", "commands/database/index/getIndexDefinitionCommand", "commands/database/transformers/getSingleTransformerCommand", "commands/database/replication/conflictsResolveCommand", "commands/database/globalConfig/getEffectiveConflictResolutionCommand", "viewmodels/viewModelBase"], function (require, exports, router, appUrl, pagedList, customColumns, customColumnParams, changesContext, getConflictsCommand, getReplicationSourcesCommand, getIndexDefinitionCommand, getSingleTransformerCommand, conflictsResolveCommand, getEffectiveConflictResolutionCommand, viewModelBase) {
    var conflicts = (function (_super) {
        __extends(conflicts, _super);
        function conflicts() {
            _super.apply(this, arguments);
            this.displayName = "conflicts";
            this.sourcesLookup = {};
            this.refreshConflictsObservable = ko.observable();
            this.currentColumns = ko.observable(customColumns.empty());
            this.currentConflictsPagedItems = ko.observable();
            this.selectedDocumentIndices = ko.observableArray();
            this.serverConflictResolution = ko.observable();
        }
        conflicts.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllReplicationConflicts(function (e) { return _this.refreshConflictsObservable(new Date().getTime()); })
            ];
        };
        conflicts.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.conflictsSubscription = this.refreshConflictsObservable.throttle(3000).subscribe(function (e) { return _this.fetchConflicts(_this.activeDatabase()); });
        };
        conflicts.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (this.conflictsSubscription != null) {
                this.conflictsSubscription.dispose();
            }
        };
        conflicts.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function (db) { return _this.databaseChanged(db); });
            this.hasAnyConflict = ko.computed(function () {
                var pagedItems = _this.currentConflictsPagedItems();
                if (pagedItems) {
                    return pagedItems.totalResultCount() > 0;
                }
                return false;
            });
            this.currentColumns().columns([
                new customColumnParams({ Header: "Detected At (UTC)", Binding: "conflictDetectedAt", DefaultWidth: 300 }),
                new customColumnParams({ Header: "Versions", Binding: "versions", DefaultWidth: 400, Template: 'versions-template' })
            ]);
            this.currentColumns().customMode(true);
            return this.performIndexCheck(this.activeDatabase()).then(function () {
                return _this.loadReplicationSources(_this.activeDatabase());
            }).done(function () {
                _this.fetchAutomaticConflictResolution(_this.activeDatabase());
                _this.fetchConflicts(appUrl.getDatabase());
            });
        };
        conflicts.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            return new getEffectiveConflictResolutionCommand(db)
                .execute()
                .done(function (repConfig) {
                _this.serverConflictResolution(repConfig.MergedDocument.DocumentConflictResolution);
            });
        };
        conflicts.prototype.fetchConflicts = function (database) {
            this.currentConflictsPagedItems(this.createPagedList(database));
        };
        conflicts.prototype.loadReplicationSources = function (db) {
            var _this = this;
            return new getReplicationSourcesCommand(db)
                .execute()
                .done(function (results) { return _this.replicationSourcesLoaded(results, db); });
        };
        conflicts.prototype.performIndexCheck = function (db) {
            // first look in cache
            if (conflicts.performedIndexChecks.contains(db.name)) {
                return $.Deferred().resolve();
            }
            var performCheckTask = $.Deferred();
            // perform index check against DB
            $.when(new getIndexDefinitionCommand(conflicts.conflictsIndexName, db).execute(), new getSingleTransformerCommand(conflicts.conflictsTransformerName, db).execute())
                .done(function () {
                conflicts.performedIndexChecks.push(db.name);
                performCheckTask.resolve();
            })
                .fail(function () {
                conflicts.performedIndexChecks.push(db.name);
            });
            return performCheckTask;
        };
        conflicts.prototype.replicationSourcesLoaded = function (sources, db) {
            this.sourcesLookup = sources;
        };
        conflicts.prototype.databaseChanged = function (db) {
            var _this = this;
            var conflictsUrl = appUrl.forConflicts(db);
            router.navigate(conflictsUrl, false);
            this.performIndexCheck(db).then(function () {
                return _this.loadReplicationSources(db);
            }).done(function () {
                _this.fetchConflicts(db);
            });
        };
        conflicts.prototype.createPagedList = function (database) {
            var fetcher = function (skip, take) { return new getConflictsCommand(database, skip, take).execute(); };
            return new pagedList(fetcher);
        };
        conflicts.prototype.getUrlForConflict = function (conflictVersion) {
            return appUrl.forEditDoc(conflictVersion.id, null, 0, this.activeDatabase());
        };
        conflicts.prototype.getTextForVersion = function (conflictVersion) {
            var replicationSource = this.sourcesLookup[conflictVersion.sourceId];
            var text = "";
            if (replicationSource) {
                text = " (" + replicationSource + ")";
            }
            return text;
        };
        conflicts.prototype.getServerUrlForVersion = function (conflictVersion) {
            return this.sourcesLookup[conflictVersion.sourceId] || "";
        };
        conflicts.prototype.resolveToLocal = function () {
            var _this = this;
            this.confirmationMessage("Sure?", "You're resolving all conflicts to local.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToLocal");
            });
        };
        conflicts.prototype.resolveToNewestRemote = function () {
            var _this = this;
            this.confirmationMessage("Sure?", "You're resolving all conflicts to newest remote.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToRemote");
            });
        };
        conflicts.prototype.resolveToLatest = function () {
            var _this = this;
            this.confirmationMessage("Sure?", "You're resolving all conflicts to latest.", ["No", "Yes"])
                .done(function () {
                _this.performResolve("ResolveToLatest");
            });
        };
        conflicts.prototype.performResolve = function (resolution) {
            var _this = this;
            new conflictsResolveCommand(this.activeDatabase(), resolution)
                .execute()
                .done(function () {
                _this.fetchConflicts(_this.activeDatabase());
            });
        };
        conflicts.performedIndexChecks = [];
        conflicts.conflictsIndexName = "Raven/ConflictDocuments";
        conflicts.conflictsTransformerName = "Raven/ConflictDocumentsTransformer";
        conflicts.gridSelector = "#conflictsGrid";
        return conflicts;
    })(viewModelBase);
    return conflicts;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/documents/copyDocuments", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var copyDocuments = (function (_super) {
        __extends(copyDocuments, _super);
        function copyDocuments(documents, elementToFocusOnDismissal) {
            var _this = this;
            _super.call(this, elementToFocusOnDismissal);
            this.isCopyingDocs = ko.observable(true);
            this.documentsOrIdsText = ko.computed(function () {
                var prettifySpacing = 4;
                if (_this.isCopyingDocs()) {
                    return documents.map(function (d) { return d.getId() + "\r\n" + JSON.stringify(d.toDto(false), null, prettifySpacing); }).join("\r\n\r\n");
                }
                else {
                    return documents.map(function (d) { return d.getId(); }).join(", ");
                }
            });
        }
        copyDocuments.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.registerResizing("copyDocumentsResize");
            this.selectText();
            jwerty.key("CTRL+C, enter", function (e) {
                e.preventDefault();
                _this.close();
            }, this, "#documentsText");
        };
        copyDocuments.prototype.deactivate = function () {
            $("#documentsText").unbind('keydown.jwerty');
        };
        copyDocuments.prototype.selectText = function () {
            $("#documentsText").select();
        };
        copyDocuments.prototype.close = function () {
            dialog.close(this);
        };
        copyDocuments.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.unregisterResizing("copyDocumentsResize");
        };
        copyDocuments.prototype.activateDocs = function () {
            this.isCopyingDocs(true);
            this.selectText();
        };
        copyDocuments.prototype.activateIds = function () {
            this.isCopyingDocs(false);
            this.selectText();
        };
        return copyDocuments;
    })(dialogViewModelBase);
    return copyDocuments;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/documents/deleteCollection", ["require", "exports", "plugins/dialog", "commands/database/documents/deleteCollectionCommand", "common/appUrl", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteCollectionCommand, appUrl, dialogViewModelBase) {
    var deleteCollection = (function (_super) {
        __extends(deleteCollection, _super);
        function deleteCollection(collection) {
            _super.call(this);
            this.collection = collection;
            this.deletionTask = $.Deferred();
            this.deletionStarted = false;
            this.isAllDocuments = collection.isAllDocuments;
        }
        deleteCollection.prototype.deleteCollection = function () {
            var _this = this;
            var collectionName = this.isAllDocuments ? "*" : this.collection.name;
            var deleteCommand = new deleteCollectionCommand(collectionName, appUrl.getDatabase());
            var deleteCommandTask = deleteCommand.execute();
            deleteCommandTask.done(function (result) { return _this.deletionTask.resolve(result); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteCollection.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteCollection.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteCollection;
    })(dialogViewModelBase);
    return deleteCollection;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/documents/documents", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "common/bindingHelpers/dynamicHeightBindingHandler", "common/changesContext", "viewmodels/viewModelBase", "viewmodels/database/documents/deleteCollection", "viewmodels/common/selectColumns", "viewmodels/common/selectCsvColumns", "viewmodels/common/showDataDialog", "models/database/documents/collection", "models/database/documents/customFunctions", "models/database/documents/customColumns", "models/database/documents/customColumnParams", "commands/database/documents/getCollectionsCommand", "commands/database/documents/getCustomColumnsCommand", "commands/database/globalConfig/getEffectiveCustomFunctionsCommand", "commands/operations/getOperationStatusCommand", "commands/operations/getOperationAlertsCommand", "commands/operations/dismissAlertCommand", "commands/database/documents/generateClassCommand"], function (require, exports, app, router, appUrl, dynamicHeightBindingHandler, changesContext, viewModelBase, deleteCollection, selectColumns, selectCsvColumnsDialog, showDataDialog, collection, customFunctions, customColumns, customColumnParams, getCollectionsCommand, getCustomColumnsCommand, getEffectiveCustomFunctionsCommand, getOperationStatusCommand, getOperationAlertsCommand, dismissAlertCommand, generateClassCommand) {
    var documents = (function (_super) {
        __extends(documents, _super);
        function documents() {
            var _this = this;
            _super.call(this);
            this.displayName = "documents";
            this.collections = ko.observableArray();
            this.selectedCollection = ko.observable().subscribeTo("ActivateCollection").distinctUntilChanged();
            this.allDocumentsCollection = ko.observable();
            this.currentCollectionPagedItems = ko.observable();
            this.currentColumnsParams = ko.observable(customColumns.empty());
            this.currentCustomFunctions = ko.observable(customFunctions.empty());
            this.selectedDocumentIndices = ko.observableArray();
            this.contextName = ko.observable('');
            this.currentCollection = ko.observable();
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.isAnyDocumentsAutoSelected = ko.observable(false);
            this.isAllDocumentsAutoSelected = ko.observable(false);
            this.lastCollectionCountUpdate = ko.observable();
            this.alerts = ko.observable([]);
            this.isInitialized = documents.isInitialized;
            this.selectedCollection.subscribe(function (c) { return _this.selectedCollectionChanged(c); });
            this.hasDocuments = ko.computed(function () {
                var selectedCollection = _this.selectedCollection();
                if (!!selectedCollection) {
                    if (selectedCollection.name === collection.allDocsCollectionName) {
                        var db = _this.activeDatabase();
                        return !!db.statistics() ? db.statistics().countOfDocuments() > 0 : false;
                    }
                    return _this.selectedCollection().documentCount() > 0;
                }
                return false;
            });
            this.hasAnyDocumentsSelected = ko.computed(function () { return _this.selectedDocumentIndices().length > 0; });
            this.hasAllDocumentsSelected = ko.computed(function () {
                var numOfSelectedDocuments = _this.selectedDocumentIndices().length;
                if (!!_this.selectedCollection() && numOfSelectedDocuments !== 0) {
                    return numOfSelectedDocuments === _this.selectedCollection().documentCount();
                }
                return false;
            });
            this.documentsSelection = ko.computed(function () {
                var selected = _this.selectedDocumentIndices();
                if (_this.hasAllDocumentsSelected()) {
                    return checkbox.Checked;
                }
                if (selected.length > 0) {
                    return checkbox.SomeChecked;
                }
                return checkbox.UnChecked;
            });
            this.canCopyAllSelected = ko.computed(function () {
                _this.showLoadingIndicator(); //triggers computing the new cached selected items
                var numOfSelectedDocuments = _this.selectedDocumentIndices().length;
                var docsGrid = _this.getDocumentsGrid();
                if (!!docsGrid) {
                    var cachedItems = docsGrid.getNumberOfCachedItems();
                    return cachedItems >= numOfSelectedDocuments;
                }
                return false;
            });
            this.isSystemDocumentsCollection = ko.computed(function () {
                var collection = _this.selectedCollection();
                return !!collection && collection.isSystemDocuments;
            });
            this.isRegularCollection = ko.computed(function () {
                var collection = _this.selectedCollection();
                return !!collection && !collection.isAllDocuments && !collection.isSystemDocuments;
            });
            this.selectedDocumentsText = ko.computed(function () {
                if (!!_this.selectedDocumentIndices()) {
                    var documentsText = "document";
                    if (_this.selectedDocumentIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            this.collectionsExceptAllDocs = ko.computed(function () {
                var allDocs = _this.allDocumentsCollection();
                if (!allDocs) {
                    return [];
                }
                var collections = _this.collections();
                return collections.filter(function (x) { return x !== allDocs; });
            });
        }
        documents.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (args.withStop) {
                viewModelBase.hasContinueTestOption(true);
            }
            this.fetchCustomFunctions();
            this.updateHelpLink("G8CDCP");
            // We can optionally pass in a collection name to view's URL, e.g. #/documents?collection=Foo&database="blahDb"
            this.collectionToSelectName = args ? args.collection : null;
            var db = this.activeDatabase();
            this.fetchAlerts();
            this.fetchCollections(db).done(function (results) {
                _this.collectionsLoaded(results, db);
                documents.isInitialized(true);
            });
        };
        documents.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            _super.prototype.createKeyboardShortcut.call(this, "F2", function () { return _this.editSelectedDoc(); }, "#documentsGrid");
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", function () { return _this.getDocumentsGrid().deleteSelectedItems(); }, docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", function () { return _this.copySelectedDocs(); }, docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", function () { return _this.copySelectedDocIds(); }, docsPageSelector);
            this.registerCollectionsResize();
        };
        documents.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            documents.isInitialized(false);
            this.unregisterCollectionsResizing();
        };
        documents.prototype.registerCollectionsResize = function () {
            var resizingColumn = false;
            var startX = 0;
            var startingLeftWidth = 0;
            var startingRightWidth = 0;
            var totalStartingWidth = 0;
            $(document).on("mousedown.collectionsResize", ".collection-resize", function (e) {
                startX = e.pageX;
                resizingColumn = true;
                startingLeftWidth = $("#documents-page-container").innerWidth();
                startingRightWidth = $("#documents-page-right-container").innerWidth();
                totalStartingWidth = startingLeftWidth + startingRightWidth;
            });
            $(document).on("mouseup.collectionsResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.collectionsResize", "", function (e) {
                if (resizingColumn) {
                    // compute new percentage values
                    var diff = e.pageX - startX;
                    $("#documents-page-container").innerWidth(((startingLeftWidth + diff) * 100.0 / totalStartingWidth) + "%");
                    $("#documents-page-right-container").innerWidth(((startingRightWidth - diff) * 100.0 / totalStartingWidth) + "%");
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        documents.prototype.unregisterCollectionsResizing = function () {
            $(document).off("mousedown.collectionsResize");
            $(document).off("mouseup.collectionsResize");
            $(document).off("mousemove.collectionsResize");
        };
        documents.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("EditItem", function () { return _this.editSelectedDoc(); }),
                ko.postbox.subscribe("ChangesApiReconnected", function (db) { return _this.reloadDocumentsData(db); }),
                ko.postbox.subscribe("SortCollections", function () { return _this.sortCollections(); })
            ];
        };
        documents.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllIndexes(function () { return _this.refreshCollections(); }),
                changesContext.currentResourceChangesApi().watchAllDocs(function () { return _this.refreshCollections(); }),
                changesContext.currentResourceChangesApi().watchBulks(function () { return _this.refreshCollections(); })
            ];
        };
        documents.prototype.exportCsv = function () {
            this.exportCsvInternal();
        };
        documents.prototype.exportCsvInternal = function (customColumns) {
            if (this.isRegularCollection()) {
                var collection = this.selectedCollection();
                var db = this.activeDatabase();
                var url = appUrl.forExportCollectionCsv(collection, collection.ownerDatabase, customColumns);
                this.downloader.download(db, url);
            }
        };
        documents.prototype.fetchAlerts = function () {
            var _this = this;
            new getOperationAlertsCommand(this.activeDatabase())
                .execute()
                .then(function (result) {
                _this.alerts(result);
            });
        };
        documents.prototype.dismissAlert = function (uniqueKey) {
            new dismissAlertCommand(this.activeDatabase(), uniqueKey).execute();
            setTimeout(function () { return dynamicHeightBindingHandler.stickToTarget($(".ko-grid-viewport-container")[0], 'footer', 0); }, 25);
        };
        documents.prototype.fetchCollections = function (db) {
            return new getCollectionsCommand(db, this.collections(), this.lastCollectionCountUpdate).execute();
        };
        documents.prototype.refreshCollections = function () {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            this.fetchCollections(db).done(function (results) {
                _this.updateCollections(results);
                _this.refreshCollectionsData();
                //TODO: add a button to refresh the documents and than use this.refreshCollectionsData();
                deferred.resolve();
            });
            return deferred;
        };
        documents.prototype.collectionsLoaded = function (collections, db) {
            var _this = this;
            // Create the "All Documents" pseudo collection.
            this.allDocumentsCollection(collection.createAllDocsCollection(db));
            this.allDocumentsCollection().documentCount = ko.computed(function () { return !!db.statistics() ? db.statistics().countOfDocuments() : 0; });
            // Create the "System Documents" pseudo collection.
            var systemDocumentsCollection = collection.createSystemDocsCollection(db);
            systemDocumentsCollection.documentCount = ko.computed(function () {
                var regularCollections = _this.collections().filter(function (c) { return c.isAllDocuments === false && c.isSystemDocuments === false; });
                if (regularCollections.length === 0)
                    return 0;
                var sum = regularCollections.map(function (c) { return c.documentCount(); }).reduce(function (a, b) { return a + b; });
                return _this.allDocumentsCollection().documentCount() - sum;
            });
            // All systems a-go. Load them into the UI and select the first one.
            var collectionsWithSysCollection = [systemDocumentsCollection].concat(collections);
            var allCollections = [this.allDocumentsCollection()].concat(collectionsWithSysCollection);
            this.collections(allCollections);
            var collectionToSelect = allCollections.first(function (c) { return c.name === _this.collectionToSelectName; }) || this.allDocumentsCollection();
            collectionToSelect.activate();
        };
        documents.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var customFunctionsCommand = new getEffectiveCustomFunctionsCommand(this.activeDatabase()).execute();
            customFunctionsCommand.done(function (cf) {
                _this.currentCustomFunctions(new customFunctions(cf.MergedDocument));
            });
        };
        //TODO: this binding has notification leak!
        documents.prototype.selectedCollectionChanged = function (selected) {
            var _this = this;
            if (!!selected) {
                var customColumnsCommand = selected.isAllDocuments ?
                    getCustomColumnsCommand.forAllDocuments(this.activeDatabase()) : getCustomColumnsCommand.forCollection(selected.name, this.activeDatabase());
                this.contextName(customColumnsCommand.docName);
                customColumnsCommand.execute().done(function (dto) {
                    if (dto) {
                        _this.currentColumnsParams().columns($.map(dto.Columns, function (c) { return new customColumnParams(c); }));
                        _this.currentColumnsParams().customMode(true);
                        selected.bindings(_this.currentColumnsParams().getBindings());
                    }
                    else {
                        // use default values!
                        selected.bindings(undefined);
                        _this.currentColumnsParams().columns.removeAll();
                        _this.currentColumnsParams().customMode(false);
                    }
                    var pagedList = selected.getDocuments();
                    _this.currentCollectionPagedItems(pagedList);
                    _this.currentCollection(selected);
                });
            }
        };
        documents.prototype.deleteCollection = function (collection) {
            var _this = this;
            if (collection) {
                var viewModel = new deleteCollection(collection);
                viewModel.deletionTask.done(function (result) {
                    if (!collection.isAllDocuments) {
                        _this.collections.remove(collection);
                        var selectedCollection = _this.selectedCollection();
                        if (collection.name === selectedCollection.name) {
                            _this.selectCollection(_this.allDocumentsCollection());
                        }
                    }
                    else {
                        _this.selectNone();
                    }
                    _this.updateGridAfterOperationComplete(collection, result.OperationId);
                });
                app.showDialog(viewModel);
            }
        };
        documents.prototype.updateGridAfterOperationComplete = function (collection, operationId) {
            var _this = this;
            var getOperationStatusTask = new getOperationStatusCommand(collection.ownerDatabase, operationId);
            getOperationStatusTask.execute()
                .done(function (result) {
                if (result.Completed) {
                    var selectedCollection = _this.selectedCollection();
                    if (selectedCollection.isAllDocuments) {
                        var docsGrid = _this.getDocumentsGrid();
                        docsGrid.refreshCollectionData();
                    }
                    else {
                        var allDocumentsPagedList = _this.allDocumentsCollection().getDocuments();
                        allDocumentsPagedList.invalidateCache();
                    }
                }
                else {
                    setTimeout(function () { return _this.updateGridAfterOperationComplete(collection, operationId); }, 500);
                }
            });
        };
        documents.prototype.updateCollections = function (receivedCollections) {
            var _this = this;
            var deletedCollections = [];
            this.collections().forEach(function (col) {
                if (!receivedCollections.first(function (receivedCol) { return col.name === receivedCol.name; }) && col.name !== "System Documents" && col.name !== "All Documents") {
                    deletedCollections.push(col);
                }
            });
            this.collections.removeAll(deletedCollections);
            receivedCollections.forEach(function (receivedCol) {
                var foundCollection = _this.collections().first(function (col) { return col.name === receivedCol.name; });
                if (!foundCollection) {
                    _this.collections.push(receivedCol);
                }
                else {
                    foundCollection.documentCount(receivedCol.documentCount());
                }
            });
            //if the collection is deleted, go to the all documents collection
            var currentCollection = this.collections().first(function (c) { return c.name === _this.selectedCollection().name; });
            if (!currentCollection || currentCollection.documentCount() === 0) {
                this.selectCollection(this.allDocumentsCollection());
            }
        };
        documents.prototype.refreshCollectionsData = function () {
            var _this = this;
            var selectedCollection = this.selectedCollection();
            this.collections().forEach(function (collection) {
                if (collection.name === selectedCollection.name) {
                    var docsGrid = _this.getDocumentsGrid();
                    if (!!docsGrid) {
                        docsGrid.refreshCollectionData();
                    }
                }
                else {
                    var pagedList = collection.getDocuments();
                    pagedList.invalidateCache();
                }
            });
        };
        documents.prototype.reloadDocumentsData = function (db) {
            var _this = this;
            if (db.name === this.activeDatabase().name) {
                this.refreshCollections().done(function () {
                    _this.refreshCollectionsData();
                });
            }
        };
        documents.prototype.selectCollection = function (collection, event) {
            if (!event || event.which !== 3) {
                collection.activate();
                var documentsWithCollectionUrl = appUrl.forDocuments(collection.name, this.activeDatabase());
                router.navigate(documentsWithCollectionUrl, false);
            }
        };
        documents.prototype.selectCsvColumns = function () {
            var _this = this;
            var dialog = new selectCsvColumnsDialog(this.getDocumentsGrid().getColumnsNames());
            app.showDialog(dialog);
            dialog.onExit().done(function (cols) {
                _this.exportCsvInternal(cols);
            });
        };
        documents.prototype.selectColumns = function () {
            var _this = this;
            // Fetch column widths from virtual table
            var virtualTable = this.getDocumentsGrid();
            var columnsNames = virtualTable.getColumnsNames();
            var vtColumns = virtualTable.columns();
            this.currentColumnsParams().columns().forEach(function (column) {
                for (var i = 0; i < vtColumns.length; i++) {
                    if (column.binding() === vtColumns[i].binding) {
                        column.width(vtColumns[i].width() | 0);
                        break;
                    }
                }
            });
            var selectColumnsViewModel = new selectColumns(this.currentColumnsParams().clone(), this.currentCustomFunctions(), this.contextName(), this.activeDatabase(), columnsNames);
            app.showDialog(selectColumnsViewModel);
            selectColumnsViewModel.onExit().done(function (cols) {
                _this.currentColumnsParams(cols);
                _this.currentCollection().bindings(_this.currentColumnsParams().getBindings());
                var pagedList = _this.currentCollection().getDocuments();
                pagedList.invalidateCache();
                _this.currentCollectionPagedItems(pagedList);
            });
        };
        documents.prototype.newDocument = function () {
            router.navigate(appUrl.forNewDoc(this.activeDatabase()));
        };
        documents.prototype.refresh = function () {
            this.getDocumentsGrid().refreshCollectionData();
            var selectedCollection = this.selectedCollection();
            selectedCollection.invalidateCache();
            this.selectNone();
        };
        documents.prototype.toggleSelectAll = function () {
            var docsGrid = this.getDocumentsGrid();
            if (!!docsGrid) {
                if (this.hasAnyDocumentsSelected()) {
                    docsGrid.selectNone();
                }
                else {
                    docsGrid.selectSome();
                    this.isAnyDocumentsAutoSelected(this.hasAllDocumentsSelected() == false);
                }
            }
        };
        documents.prototype.selectAll = function () {
            var docsGrid = this.getDocumentsGrid();
            var c = this.selectedCollection();
            if (!!docsGrid && !!c) {
                docsGrid.selectAll(c.documentCount());
            }
        };
        documents.prototype.selectNone = function () {
            var docsGrid = this.getDocumentsGrid();
            if (!!docsGrid) {
                docsGrid.selectNone();
            }
        };
        documents.prototype.editSelectedDoc = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        documents.prototype.deleteSelectedDocs = function () {
            if (this.selectedCollection().isSystemDocuments === false && this.hasAllDocumentsSelected()) {
                this.deleteCollection(this.selectedCollection());
            }
            else {
                var grid = this.getDocumentsGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        documents.prototype.copySelectedDocs = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocs();
            }
        };
        documents.prototype.copySelectedDocIds = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocIds();
            }
        };
        documents.prototype.generateDocCode = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1).first();
                var metadata = selectedItem["__metadata"];
                var id = metadata["id"];
                var generate = new generateClassCommand(this.activeDatabase(), id, "csharp");
                var deffered = generate.execute();
                deffered.done(function (code) {
                    app.showDialog(new showDataDialog("Generated Class", code["Code"]));
                });
            }
        };
        documents.prototype.getDocumentsGrid = function () {
            var gridContents = $(documents.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        documents.prototype.urlForAlert = function (alert) {
            var index = this.alerts().indexOf(alert);
            return appUrl.forAlerts(this.activeDatabase()) + "&item=" + index;
        };
        documents.prototype.sortCollections = function () {
            this.collections.sort(function (c1, c2) {
                if (c1.isAllDocuments)
                    return -1;
                if (c2.isAllDocuments)
                    return 1;
                if (c1.isSystemDocuments)
                    return -1;
                if (c2.isSystemDocuments)
                    return 1;
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the groups list
        documents.prototype.showCollectionElement = function (element) {
            if (element.nodeType === 1 && documents.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortCollections");
                    $(element).highlight();
                });
            }
        };
        documents.prototype.hideCollectionElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        documents.gridSelector = "#documentsGrid";
        documents.isInitialized = ko.observable(false);
        return documents;
    })(viewModelBase);
    return documents;
});
/// <reference path="../../../../Scripts/typings/ace/ace.amd.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/documents/editDocument", ["require", "exports", "durandal/app", "plugins/router", "models/database/documents/document", "models/database/documents/documentMetadata", "models/database/documents/collection", "models/database/query/querySort", "commands/database/documents/saveDocumentCommand", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/documents/verifyDocumentsIDsCommand", "commands/database/query/queryIndexCommand", "commands/database/studio/resolveMergeCommand", "common/pagedList", "common/appUrl", "common/jsonUtil", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler", "common/generalUtils", "viewmodels/common/deleteDocuments", "viewmodels/viewModelBase", "commands/database/documents/generateClassCommand", "viewmodels/common/showDataDialog"], function (require, exports, app, router, document, documentMetadata, collection, querySort, saveDocumentCommand, getDocumentWithMetadataCommand, verifyDocumentsIDsCommand, queryIndexCommand, resolveMergeCommand, pagedList, appUrl, jsonUtil, messagePublisher, aceEditorBindingHandler, genUtils, deleteDocuments, viewModelBase, generateClassCommand, showDataDialog) {
    var editDocument = (function (_super) {
        __extends(editDocument, _super);
        function editDocument() {
            var _this = this;
            _super.call(this);
            this.isConflictDocument = ko.observable();
            this.document = ko.observable();
            this.documentText = ko.observable('').extend({ required: true });
            this.metadataText = ko.observable('').extend({ required: true });
            this.isEditingMetadata = ko.observable(false);
            this.isBusy = ko.observable(false);
            this.metaPropsToRestoreOnSave = [];
            this.userSpecifiedId = ko.observable('').extend({ required: true });
            this.isCreatingNewDocument = ko.observable(false);
            this.docsList = ko.observable();
            this.queryResultList = ko.observable();
            this.topRecentDocuments = ko.computed(function () { return _this.getTopRecentDocuments(); });
            this.relatedDocumentHrefs = ko.observableArray();
            this.docEditroHasFocus = ko.observable(true);
            this.documentMatchRegexp = /\w+\/\w+/ig;
            this.loadedDocumentName = ko.observable('');
            this.isInDocMode = ko.observable(true);
            this.queryIndex = ko.observable();
            this.isNewLineFriendlyMode = ko.observable(false);
            this.autoCollapseMode = ko.observable(false);
            this.newLineToggle = '\\n';
            this.isSystemDocumentByDocTitle = ko.observable(false);
            aceEditorBindingHandler.install();
            this.metadata = ko.computed(function () { return _this.document() ? _this.document().__metadata : null; });
            this.isConflictDocument = ko.computed(function () {
                var metadata = _this.metadata();
                return metadata != null && !!metadata["Raven-Replication-Conflict"] && !metadata.id.contains("/conflicts/");
            });
            this.document.subscribe(function (doc) {
                if (doc) {
                    if (_this.isConflictDocument()) {
                        _this.resolveConflicts();
                    }
                    else {
                        var docText = _this.stringify(doc.toDto());
                        _this.documentText(docText);
                    }
                }
            });
            this.documentSize = ko.computed(function () {
                try {
                    var size = ((_this.documentText().getSizeInBytesAsUTF8() + _this.metadataText().getSizeInBytesAsUTF8()) / 1024);
                    return genUtils.formatAsCommaSeperatedString(size, 2);
                }
                catch (e) {
                    return "cannot compute";
                }
            });
            this.metadata.subscribe(function (meta) { return _this.metadataChanged(meta); });
            this.editedDocId = ko.computed(function () { return _this.metadata() ? _this.metadata().id : ''; });
            this.editedDocId.subscribe(function (docId) {
                return ko.postbox.publish("SetRawJSONUrl", docId ? appUrl.forDocumentRawData(_this.activeDatabase(), docId) : "");
            });
            // When we programmatically change the document text or meta text, push it into the editor.
            this.isEditingMetadata.subscribe(function () {
                if (_this.docEditor) {
                    var text = _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                    _this.docEditor.getSession().setValue(text);
                }
            });
            this.text = ko.computed({
                read: function () {
                    return _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                },
                write: function (text) {
                    var currentObservable = _this.isEditingMetadata() ? _this.metadataText : _this.documentText;
                    currentObservable(text);
                },
                owner: this
            });
            this.docTitle = ko.computed(function () {
                if (_this.isInDocMode()) {
                    if (_this.isCreatingNewDocument()) {
                        _this.isSystemDocumentByDocTitle(false);
                        return 'New Document';
                    }
                    else {
                        var editedDocId = _this.editedDocId();
                        if (editedDocId.indexOf("Raven/") === 0)
                            _this.isSystemDocumentByDocTitle(true);
                        else
                            _this.isSystemDocumentByDocTitle(false);
                        if (!!editedDocId) {
                            var lastIndexInEditedDocId = editedDocId.lastIndexOf('/') + 1;
                            if (lastIndexInEditedDocId > 0) {
                                editedDocId = editedDocId.slice(lastIndexInEditedDocId);
                            }
                        }
                        return editedDocId;
                    }
                }
                else {
                    return 'Projection';
                }
            });
            this.isFirstDocumenNavtDisabled = ko.computed(function () {
                var list = _this.docsList();
                if (list) {
                    var currentDocumentIndex = list.currentItemIndex();
                    if (currentDocumentIndex === 0) {
                        return true;
                    }
                }
                return false;
            });
            this.isLastDocumentNavDisabled = ko.computed(function () {
                var list = _this.docsList();
                if (list) {
                    var currentDocumentIndex = list.currentItemIndex();
                    var totalDocuments = list.totalResultCount();
                    if (currentDocumentIndex === totalDocuments - 1) {
                        return true;
                    }
                }
                return false;
            });
        }
        editDocument.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var canActivateResult = $.Deferred();
            if (args && args.id) {
                this.databaseForEditedDoc = appUrl.getDatabase();
                this.loadDocument(args.id)
                    .done(function () {
                    canActivateResult.resolve({ can: true });
                })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + args.id + " document");
                    canActivateResult.resolve({ redirect: appUrl.forDocuments(collection.allDocsCollectionName, _this.activeDatabase()) });
                });
                return canActivateResult;
            }
            else if (args && args.item && args.list) {
                return $.Deferred().resolve({ can: true }); //todo: maybe treat case when there is collection and item number but no id
            }
            else if (args && args.index) {
                this.isInDocMode(false);
                var indexName = args.index;
                var queryText = args.query;
                var sorts;
                if (args.sorts) {
                    sorts = args.sorts.split(',').map(function (curSort) { return querySort.fromQuerySortString(curSort.trim()); });
                }
                else {
                    sorts = [];
                }
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(indexName, _this.activeDatabase(), skip, take, queryText, sorts);
                    return command
                        .execute();
                };
                var list = new pagedList(resultsFetcher);
                var item = !!args.item && !isNaN(args.item) ? args.item : 0;
                list.getNthItem(item)
                    .done(function (doc) {
                    _this.document(doc);
                    _this.loadedDocumentName("");
                    canActivateResult.resolve({ can: true });
                })
                    .fail(function () {
                    messagePublisher.reportError("Could not find query result");
                    canActivateResult.resolve({ redirect: appUrl.forDocuments(collection.allDocsCollectionName, _this.activeDatabase()) });
                });
                this.currentQueriedItemIndex = item;
                this.queryResultList(list);
                this.queryIndex(indexName);
                return canActivateResult;
            }
            else {
                return $.Deferred().resolve({ can: true });
            }
        };
        editDocument.prototype.activate = function (navigationArgs) {
            var _this = this;
            _super.prototype.activate.call(this, navigationArgs);
            this.updateHelpLink('M72H1R');
            this.loadedDocumentName(this.userSpecifiedId());
            this.dirtyFlag = new ko.DirtyFlag([this.documentText, this.metadataText, this.userSpecifiedId], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () {
                return (_this.dirtyFlag().isDirty() || _this.loadedDocumentName() == ""); // && !!self.userSpecifiedId(); || 
            }, this);
            // Find the database and collection we're supposed to load.
            // Used for paging through items.
            this.databaseForEditedDoc = this.activeDatabase();
            if (navigationArgs && navigationArgs.database) {
                ko.postbox.publish("ActivateDatabaseWithName", navigationArgs.database);
            }
            if (navigationArgs && navigationArgs.list && navigationArgs.item) {
                var itemIndex = parseInt(navigationArgs.item, 10);
                if (!isNaN(itemIndex)) {
                    var newCollection = new collection(navigationArgs.list, appUrl.getDatabase());
                    var fetcher = function (skip, take) { return newCollection.fetchDocuments(skip, take); };
                    var list = new pagedList(fetcher);
                    list.collectionName = navigationArgs.list;
                    list.currentItemIndex(itemIndex);
                    list.getNthItem(0); // Force us to get the total items count.
                    this.docsList(list);
                }
            }
            if (navigationArgs && navigationArgs.id) {
                this.appendRecentDocument(navigationArgs.id);
                ko.postbox.publish("SetRawJSONUrl", appUrl.forDocumentRawData(this.activeDatabase(), navigationArgs.id));
            }
            else if (navigationArgs && navigationArgs.index) {
            }
            else {
                this.editNewDocument();
            }
        };
        editDocument.prototype.updateNewlineLayoutInDocument = function (unescapeNewline) {
            var dirtyFlagValue = this.dirtyFlag().isDirty();
            if (unescapeNewline == true) {
                this.documentText(this.unescapeNewlinesAndTabsInTextFields(this.documentText()));
                this.docEditor.getSession().setMode('ace/mode/json_newline_friendly');
            }
            else {
                this.documentText(this.escapeNewlinesAndTabsInTextFields(this.documentText()));
                this.docEditor.getSession().setMode('ace/mode/json');
                this.formatDocument();
            }
            if (dirtyFlagValue == false) {
                this.dirtyFlag().reset();
            }
        };
        // Called when the view is attached to the DOM.
        editDocument.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
            $("#docEditor").resize();
            this.isNewLineFriendlyMode.subscribe(function (val) {
                _this.updateNewlineLayoutInDocument(val);
            });
        };
        editDocument.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#docEditor").off('DynamicHeightSet');
        };
        editDocument.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.documentNameElement = $("#documentName");
            var editorElement = $("#docEditor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#docEditor").on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
            $("#docEditor").bind("paste", function () { this.text.valueHasMutated(); });
            this.focusOnEditor();
        };
        editDocument.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+d", function () { return _this.focusOnDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+m", function () { return _this.focusOnMetadata(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+r", function () { return _this.refreshDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+c", function () { return _this.focusOnEditor(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+home", function () { return _this.firstDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+end", function () { return _this.lastDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+page-up", function () { return _this.previousDocumentOrLast(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+page-down", function () { return _this.nextDocumentOrFirst(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteDocument(); }, editDocument.editDocSelector);
            this.createKeyboardShortcut("alt+s", function () { return _this.saveDocument(); }, editDocument.editDocSelector); // Q. Why do we have to setup ALT+S, when we could just use HTML's accesskey attribute? A. Because the accesskey attribute causes the save button to take focus, thus stealing the focus from the user's editing spot in the doc editor, disrupting his workflow.
            //this.createKeyboardShortcut("/", () => this.docsList(), editDocument.editDocSelector);
        };
        editDocument.prototype.focusOnMetadata = function () {
            this.isEditingMetadata(true);
            this.focusOnEditor();
        };
        editDocument.prototype.focusOnDocument = function () {
            this.isEditingMetadata(false);
            this.focusOnEditor();
        };
        editDocument.prototype.focusOnEditor = function () {
            this.docEditor.focus();
        };
        editDocument.prototype.editNewDocument = function () {
            this.isCreatingNewDocument(true);
            var newDocument = document.empty();
            newDocument["Name"] = "...";
            this.document(newDocument);
        };
        editDocument.prototype.failedToLoadDoc = function (docId, errorResponse) {
            messagePublisher.reportError("Could not find " + docId + " document");
        };
        editDocument.prototype.escapeNewlinesAndTabsInTextFields = function (str) {
            var AceDocumentClass = require("ace/document").Document;
            var AceEditSessionClass = require("ace/edit_session").EditSession;
            var AceJSONMode = require("ace/mode/json_newline_friendly").Mode;
            var documentTextAceDocument = new AceDocumentClass(str);
            var jsonMode = new AceJSONMode();
            var documentTextAceEditSession = new AceEditSessionClass(documentTextAceDocument, jsonMode);
            var previousLine = 0;
            var TokenIterator = require("ace/token_iterator").TokenIterator;
            var iterator = new TokenIterator(documentTextAceEditSession, 0, 0);
            var curToken = iterator.getCurrentToken();
            var text = "";
            while (curToken) {
                if (iterator.$row - previousLine > 1) {
                    var rowsGap = iterator.$row - previousLine;
                    for (var i = 0; i < rowsGap - 1; i++) {
                        text += "\\r\\n";
                    }
                }
                if (curToken.type === "string" || curToken.type == "constant.language.escape") {
                    if (previousLine < iterator.$row) {
                        text += "\\r\\n";
                    }
                    var newTokenValue = curToken.value
                        .replace(/(\r\n)/g, '\\r\\n')
                        .replace(/(\n)/g, '\\n')
                        .replace(/(\t)/g, '\\t');
                    text += newTokenValue;
                }
                else {
                    text += curToken.value;
                }
                previousLine = iterator.$row;
                curToken = iterator.stepForward();
            }
            return text;
        };
        editDocument.prototype.toggleNewlineMode = function () {
            var _this = this;
            if (this.isNewLineFriendlyMode() === false && parseInt(this.documentSize().replace(",", "")) > 150) {
                app.showMessage("This operation might take long time with big documents, are you sure you want to continue?", "Toggle newline mode", ["Cancel", "Continue"])
                    .then(function (dialogResult) {
                    if (dialogResult === "Continue") {
                        _this.isNewLineFriendlyMode.toggle();
                    }
                });
            }
            else {
                this.isNewLineFriendlyMode.toggle();
            }
        };
        editDocument.prototype.toggleAutoCollapse = function () {
            this.autoCollapseMode.toggle();
            if (this.autoCollapseMode()) {
                this.foldAll();
            }
            else {
                this.docEditor.getSession().unfold(null, true);
            }
        };
        editDocument.prototype.foldAll = function () {
            var _this = this;
            var AceRange = require("ace/range").Range;
            this.docEditor.getSession().foldAll();
            var folds = this.docEditor.getSession().getFoldsInRange(new AceRange(0, 0, this.docEditor.getSession().getLength(), 0));
            folds.map(function (f) { return _this.docEditor.getSession().expandFold(f); });
        };
        editDocument.prototype.unescapeNewlinesAndTabsInTextFields = function (str) {
            var AceDocumentClass = require("ace/document").Document;
            var AceEditSessionClass = require("ace/edit_session").EditSession;
            var AceJSONMode = require("ace/mode/json").Mode;
            var documentTextAceDocument = new AceDocumentClass(str);
            var jsonMode = new AceJSONMode();
            var documentTextAceEditSession = new AceEditSessionClass(documentTextAceDocument, jsonMode);
            var TokenIterator = require("ace/token_iterator").TokenIterator;
            var iterator = new TokenIterator(documentTextAceEditSession, 0, 0);
            var curToken = iterator.getCurrentToken();
            // first, calculate newline indexes
            var rowsIndexes = str.split("").map(function (x, index) { return { char: x, index: index }; }).filter(function (x) { return x.char == "\n"; }).map(function (x) { return x.index; });
            // start iteration from the end of the document
            while (curToken) {
                curToken = iterator.stepForward();
            }
            curToken = iterator.stepBackward();
            var lastTextSectionPosEnd = null;
            while (curToken) {
                if (curToken.type === "string" || curToken.type == "constant.language.escape") {
                    if (lastTextSectionPosEnd == null) {
                        curToken = iterator.stepForward();
                        lastTextSectionPosEnd = { row: iterator.getCurrentTokenRow(), column: iterator.getCurrentTokenColumn() + 1 };
                        curToken = iterator.stepBackward();
                    }
                }
                else {
                    if (lastTextSectionPosEnd != null) {
                        curToken = iterator.stepForward();
                        var lastTextSectionPosStart = { row: iterator.getCurrentTokenRow(), column: iterator.getCurrentTokenColumn() + 1 };
                        var stringTokenStartIndexInSourceText = (lastTextSectionPosStart.row > 0 ? rowsIndexes[lastTextSectionPosStart.row - 1] : 0) + lastTextSectionPosStart.column;
                        var stringTokenEndIndexInSourceText = (lastTextSectionPosEnd.row > 0 ? rowsIndexes[lastTextSectionPosEnd.row - 1] : 0) + lastTextSectionPosEnd.column;
                        var newTextPrefix = str.substring(0, stringTokenStartIndexInSourceText);
                        var newTextSuffix = str.substring(stringTokenEndIndexInSourceText, str.length);
                        var newStringTokenValue = str.substring(stringTokenStartIndexInSourceText, stringTokenEndIndexInSourceText)
                            .replace(/(\\\\n|\\\\r\\\\n|\\n|\\r\\n|\\t|\\\\t)/g, function (x) {
                            if (x == "\\\\n" || x == "\\\\r\\\\n") {
                                return "\\r\\n";
                            }
                            else if (x == "\\n" || x == "\\r\\n") {
                                return "\r\n";
                            }
                            else if (x == "\\t") {
                                return "\t";
                            }
                            else if (x == "\\\\t") {
                                return "\\t";
                            }
                            else {
                                return "\r\n";
                            }
                        });
                        str = newTextPrefix + newStringTokenValue + newTextSuffix;
                        curToken = iterator.stepBackward();
                    }
                    lastTextSectionPosEnd = null;
                }
                curToken = iterator.stepBackward();
            }
            return str;
        };
        editDocument.prototype.saveDocument = function () {
            var _this = this;
            this.isInDocMode(true);
            var currentDocumentId = this.userSpecifiedId();
            if ((currentDocumentId == "") || (this.loadedDocumentName() != currentDocumentId)) {
                //the name of the document was changed and we have to save it as a new one
                this.isCreatingNewDocument(true);
            }
            var message = "";
            if (currentDocumentId.indexOf("\\") != -1) {
                message = "Document name cannot contain '\\'";
                this.documentNameElement.focus();
            }
            else {
                try {
                    var updatedDto;
                    if (this.isNewLineFriendlyMode()) {
                        updatedDto = JSON.parse(this.escapeNewlinesAndTabsInTextFields(this.documentText()));
                    }
                    else {
                        updatedDto = JSON.parse(this.documentText());
                    }
                    var meta = JSON.parse(this.metadataText());
                }
                catch (e) {
                    if (updatedDto == undefined) {
                        message = "The document data isn't a legal JSON expression!";
                        this.isEditingMetadata(false);
                    }
                    else if (meta == undefined) {
                        message = "The document metadata isn't a legal JSON expression!";
                        this.isEditingMetadata(true);
                    }
                    this.focusOnEditor();
                }
            }
            if (message != "") {
                messagePublisher.reportError(message, undefined, undefined, false);
                return;
            }
            updatedDto['@metadata'] = meta;
            // Fix up the metadata: if we're a new doc, attach the expected reserved properties like ID, ETag, and RavenEntityName.
            // AFAICT, Raven requires these reserved meta properties in order for the doc to be seen as a member of a collection.
            if (this.isCreatingNewDocument()) {
                this.attachReservedMetaProperties(currentDocumentId, meta);
            }
            else {
                // If we're editing a document, we hide some reserved properties from the user.
                // Restore these before we save.
                this.metaPropsToRestoreOnSave.forEach(function (p) {
                    if (p.name !== "Origin") {
                        meta[p.name] = p.value;
                    }
                });
            }
            // skip some not necessary meta in headers
            var metaToSkipInHeaders = ['Raven-Replication-History'];
            for (var i in metaToSkipInHeaders) {
                var skippedHeader = metaToSkipInHeaders[i];
                delete meta[skippedHeader];
            }
            if (!!this.docsList()) {
                this.docsList().invalidateCache();
            }
            var newDoc = new document(updatedDto);
            var saveCommand = new saveDocumentCommand(currentDocumentId, newDoc, this.activeDatabase());
            var saveTask = saveCommand.execute();
            saveTask.done(function (saveResult) {
                var savedDocumentDto = saveResult[0];
                var currentSelection = _this.docEditor.getSelectionRange();
                _this.loadDocument(savedDocumentDto.Key)
                    .always(function () {
                    _this.updateNewlineLayoutInDocument(_this.isNewLineFriendlyMode());
                    // Try to restore the selection.
                    _this.docEditor.selection.setRange(currentSelection, false);
                });
                _this.updateUrl(savedDocumentDto.Key);
                _this.dirtyFlag().reset(); //Resync Changes
                // add the new document to the paged list
                var list = _this.docsList();
                if (!!list) {
                    if (_this.isCreatingNewDocument()) {
                        var newTotalResultCount = list.totalResultCount() + 1;
                        list.totalResultCount(newTotalResultCount);
                        list.currentItemIndex(newTotalResultCount - 1);
                    }
                    else {
                        list.currentItemIndex(list.totalResultCount() - 1);
                    }
                    _this.updateUrl(currentDocumentId);
                }
                _this.isCreatingNewDocument(false);
            });
        };
        editDocument.prototype.attachReservedMetaProperties = function (id, target) {
            target['@etag'] = '00000000-0000-0000-0000-000000000000';
            target['Raven-Entity-Name'] = !target['Raven-Entity-Name'] ? document.getEntityNameFromId(id) : target['Raven-Entity-Name'];
            target['@id'] = id;
        };
        editDocument.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        editDocument.prototype.activateMeta = function () {
            this.isEditingMetadata(true);
            this.docEditor.getSession().setMode('ace/mode/json');
        };
        editDocument.prototype.activateDoc = function () {
            this.isEditingMetadata(false);
            if (this.isNewLineFriendlyMode() == true) {
                this.docEditor.getSession().setMode('ace/mode/json_newline_friendly');
            }
        };
        editDocument.prototype.findRelatedDocumentsCandidates = function (doc) {
            var results = [];
            var initialDocumentFields = doc.getDocumentPropertyNames();
            var documentNodesFlattenedList = [];
            // get initial nodes list to work with
            initialDocumentFields.forEach(function (curField) {
                documentNodesFlattenedList.push(doc[curField]);
            });
            for (var documentNodesCursor = 0; documentNodesCursor < documentNodesFlattenedList.length; documentNodesCursor++) {
                var curField = documentNodesFlattenedList[documentNodesCursor];
                if (typeof curField === "string" && /\w+\/\w+/ig.test(curField)) {
                    if (!results.first(function (x) { return x === curField.toString(); })) {
                        results.push(curField.toString());
                    }
                }
                else if (typeof curField == "object" && !!curField) {
                    for (var curInnerField in curField) {
                        documentNodesFlattenedList.push(curField[curInnerField]);
                    }
                }
            }
            return results;
        };
        editDocument.prototype.loadDocument = function (id) {
            var _this = this;
            var loadDocTask = new getDocumentWithMetadataCommand(id, this.databaseForEditedDoc).execute();
            loadDocTask.done(function (document) {
                _this.document(document);
                _this.loadedDocumentName(_this.userSpecifiedId());
                _this.dirtyFlag().reset(); //Resync Changes
                _this.loadRelatedDocumentsList(document);
                _this.appendRecentDocument(id);
                if (_this.autoCollapseMode()) {
                    _this.foldAll();
                }
            });
            loadDocTask.fail(function (response) { return _this.failedToLoadDoc(id, response); });
            loadDocTask.always(function () { return _this.isBusy(false); });
            this.isBusy(true);
            return loadDocTask;
        };
        editDocument.prototype.refreshDocument = function () {
            var _this = this;
            if (this.isInDocMode()) {
                if (!this.isCreatingNewDocument()) {
                    var docId = this.editedDocId();
                    this.document(null);
                    this.documentText(null);
                    this.metadataText(null);
                    this.userSpecifiedId('');
                    this.loadDocument(docId);
                }
                else {
                    this.editNewDocument();
                }
            }
            else {
                this.queryResultList().getNthItem(this.currentQueriedItemIndex).done(function (doc) { return _this.document(doc); });
                this.loadedDocumentName("");
            }
        };
        editDocument.prototype.deleteDocument = function () {
            var _this = this;
            var doc = this.document();
            if (doc) {
                var viewModel = new deleteDocuments([doc]);
                viewModel.deletionTask.done(function () {
                    _this.dirtyFlag().reset(); //Resync Changes
                    var list = _this.docsList();
                    if (!!list) {
                        _this.docsList().invalidateCache();
                        var newTotalResultCount = list.totalResultCount() - 1;
                        list.totalResultCount(newTotalResultCount);
                        var nextIndex = list.currentItemIndex();
                        if (nextIndex >= newTotalResultCount) {
                            nextIndex = 0;
                        }
                        if (newTotalResultCount > 0) {
                            _this.pageToItem(nextIndex, newTotalResultCount);
                        }
                        else {
                            router.navigate(appUrl.forDocuments(null, _this.activeDatabase()));
                        }
                    }
                });
                app.showDialog(viewModel, editDocument.editDocSelector);
            }
        };
        editDocument.prototype.formatDocument = function () {
            try {
                var docEditorText = this.docEditor.getSession().getValue();
                var observableToUpdate = this.isEditingMetadata() ? this.metadataText : this.documentText;
                var tempDoc = JSON.parse(docEditorText);
                var formatted = this.stringify(tempDoc);
                observableToUpdate(formatted);
            }
            catch (e) {
                messagePublisher.reportError("Could not format json", undefined, undefined, false);
            }
        };
        editDocument.prototype.nextDocumentOrFirst = function () {
            var list = this.docsList();
            if (list) {
                var nextIndex = list.currentItemIndex() + 1;
                if (nextIndex >= list.totalResultCount()) {
                    nextIndex = 0;
                }
                this.pageToItem(nextIndex);
            }
            else {
                this.navigateToDocuments();
            }
        };
        editDocument.prototype.previousDocumentOrLast = function () {
            var list = this.docsList();
            if (list) {
                var previousIndex = list.currentItemIndex() - 1;
                if (previousIndex < 0) {
                    previousIndex = list.totalResultCount() - 1;
                }
                this.pageToItem(previousIndex);
            }
        };
        editDocument.prototype.lastDocument = function () {
            var list = this.docsList();
            if (list) {
                this.pageToItem(list.totalResultCount() - 1);
            }
        };
        editDocument.prototype.firstDocument = function () {
            this.pageToItem(0);
        };
        editDocument.prototype.pageToItem = function (index, newTotalResultCount) {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to continue?');
            canContinue.done(function () {
                var list = _this.docsList();
                if (list) {
                    list.getNthItem(index)
                        .done(function (doc) {
                        if (_this.isInDocMode()) {
                            _this.loadDocument(doc.getId());
                            list.currentItemIndex(index);
                            _this.updateUrl(doc.getId());
                        }
                        else {
                            _this.document(doc);
                            _this.loadedDocumentName("");
                            _this.dirtyFlag().reset(); //Resync Changes
                        }
                        if (!!newTotalResultCount) {
                            list.totalResultCount(newTotalResultCount);
                        }
                    });
                }
            });
        };
        editDocument.prototype.navigateToCollection = function (collectionName) {
            var collectionUrl = appUrl.forDocuments(collectionName, this.activeDatabase());
            router.navigate(collectionUrl);
        };
        editDocument.prototype.navigateToDocuments = function () {
            this.navigateToCollection(null);
        };
        editDocument.prototype.updateUrl = function (docId) {
            var collectionName = this.docsList() ? this.docsList().collectionName : null;
            var currentItemIndex = this.docsList() ? this.docsList().currentItemIndex() : null;
            var editDocUrl = appUrl.forEditDoc(docId, collectionName, currentItemIndex, this.activeDatabase());
            router.navigate(editDocUrl, false);
        };
        editDocument.prototype.getTopRecentDocuments = function () {
            var _this = this;
            var currentDbName = this.activeDatabase().name;
            var recentDocumentsForCurDb = editDocument.recentDocumentsInDatabases().first(function (x) { return x.databaseName === currentDbName; });
            if (recentDocumentsForCurDb) {
                var value = recentDocumentsForCurDb
                    .recentDocuments()
                    .filter(function (x) {
                    return x !== _this.userSpecifiedId();
                })
                    .slice(0, 5)
                    .map(function (docId) {
                    return {
                        docId: (docId.length > 35) ? docId.substr(0, 35) + '...' : docId,
                        docUrl: appUrl.forEditDoc(docId, null, null, _this.activeDatabase()),
                        fullDocId: docId
                    };
                });
                return value;
            }
            else {
                return [];
            }
        };
        editDocument.prototype.metadataChanged = function (meta) {
            if (meta) {
                this.metaPropsToRestoreOnSave.length = 0;
                var metaDto = this.metadata().toDto();
                documentMetadata.filterMetadata(metaDto, this.metaPropsToRestoreOnSave);
                var metaString = this.stringify(metaDto);
                this.metadataText(metaString);
                if (meta.id != undefined) {
                    this.userSpecifiedId(meta.id);
                }
            }
        };
        editDocument.prototype.loadRelatedDocumentsList = function (document) {
            var _this = this;
            var relatedDocumentsCandidates = this.findRelatedDocumentsCandidates(document);
            var docIDsVerifyCommand = new verifyDocumentsIDsCommand(relatedDocumentsCandidates, this.activeDatabase(), true, true);
            var response = docIDsVerifyCommand.execute();
            if (response.then) {
                response.done(function (verifiedIDs) {
                    _this.relatedDocumentHrefs(verifiedIDs.map(function (verified) {
                        return {
                            id: verified.toString(),
                            href: appUrl.forEditDoc(verified.toString(), null, null, _this.activeDatabase())
                        };
                    }));
                });
            }
            else {
                this.relatedDocumentHrefs(response.map(function (verified) {
                    return {
                        id: verified.toString(),
                        href: appUrl.forEditDoc(verified.toString(), null, null, _this.activeDatabase())
                    };
                }));
            }
        };
        editDocument.prototype.appendRecentDocument = function (docId) {
            var _this = this;
            var existingRecentDocumentsStore = editDocument.recentDocumentsInDatabases.first(function (x) { return x.databaseName == _this.databaseForEditedDoc.name; });
            if (existingRecentDocumentsStore) {
                var existingDocumentInStore = existingRecentDocumentsStore.recentDocuments.first(function (x) { return x === docId; });
                if (!existingDocumentInStore) {
                    if (existingRecentDocumentsStore.recentDocuments().length == 5) {
                        existingRecentDocumentsStore.recentDocuments.pop();
                    }
                    existingRecentDocumentsStore.recentDocuments.unshift(docId);
                }
            }
            else {
                editDocument.recentDocumentsInDatabases.push({ databaseName: this.databaseForEditedDoc.name, recentDocuments: ko.observableArray([docId]) });
            }
        };
        editDocument.prototype.resolveConflicts = function () {
            var _this = this;
            var task = new resolveMergeCommand(this.activeDatabase(), this.editedDocId()).execute();
            task.done(function (response) {
                _this.documentText(response.Document);
                _this.metadataText(response.Metadata);
            });
        };
        editDocument.prototype.getColorClass = function (documentId) {
            var entityName = document.getEntityNameFromId(documentId);
            if (entityName) {
                return collection.getCollectionCssClass(entityName, this.activeDatabase());
            }
            return "";
        };
        editDocument.prototype.prettyLabel = function (text) {
            return text ? text.replace(/__/g, '/') : text;
        };
        editDocument.prototype.generateCollectionName = function (ravenEntityName, withPrettyLabel) {
            if (withPrettyLabel === void 0) { withPrettyLabel = false; }
            if (withPrettyLabel) {
                return ravenEntityName
                    ? this.prettyLabel(ravenEntityName)
                    : (this.isSystemDocumentByDocTitle() ? 'System Documents' : 'No Collection');
            }
            else {
                return ravenEntityName || (this.isSystemDocumentByDocTitle() ? 'System Documents' : 'No Collection');
            }
        };
        editDocument.prototype.generateCode = function () {
            var doc = this.document();
            var generate = new generateClassCommand(this.activeDatabase(), doc.getId(), "csharp");
            var deffered = generate.execute();
            deffered.done(function (code) {
                app.showDialog(new showDataDialog("Generated Class", code["Code"]));
            });
        };
        editDocument.editDocSelector = "#editDocumentContainer";
        editDocument.recentDocumentsInDatabases = ko.observableArray();
        return editDocument;
    })(viewModelBase);
    return editDocument;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/exploration/exploration", ["require", "exports", "viewmodels/viewModelBase", "commands/database/index/getIndexTermsCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/appUrl", "models/database/query/dataExplorationRequest", "commands/database/query/dataExplorationCommand", "common/pagedResultSet", "common/pagedList", "models/database/documents/document", "common/messagePublisher"], function (require, exports, viewModelBase, getIndexTermsCommand, aceEditorBindingHandler, appUrl, dataExplorationRequest, dataExplorationCommand, pagedResultSet, pagedList, document, messagePublisher) {
    var exploration = (function (_super) {
        __extends(exploration, _super);
        function exploration() {
            var _this = this;
            _super.call(this);
            this.collections = ko.observableArray([]);
            this.isBusy = ko.observable(false);
            this.explorationRequest = dataExplorationRequest.empty();
            this.queryResults = ko.observable();
            this.isLoading = ko.observable(false);
            this.dataLoadingXhr = ko.observable();
            this.selectedCollectionLabel = ko.computed(function () { return _this.explorationRequest.collection() || "Select a collection"; });
            this.runEnabled = ko.computed(function () { return !!_this.explorationRequest.collection(); });
            this.appUrls = appUrl.forCurrentDatabase();
            aceEditorBindingHandler.install();
        }
        exploration.prototype.canActivate = function (args) {
            var _this = this;
            var deffered = $.Deferred();
            new getIndexTermsCommand("Raven/DocumentsByEntityName", "Tag", this.activeDatabase())
                .execute()
                .done(function (terms) {
                _this.collections(terms);
            })
                .always(function () { return deffered.resolve({ can: true }); });
            return deffered;
        };
        exploration.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
        };
        exploration.prototype.exportCsv = function () {
            var db = this.activeDatabase();
            var url = new dataExplorationCommand(this.explorationRequest.toDto(), db).getCsvUrl();
            this.downloader.download(db, url);
        };
        exploration.prototype.runExploration = function () {
            var _this = this;
            this.isBusy(true);
            var requestDto = this.explorationRequest.toDto();
            var command = new dataExplorationCommand(requestDto, this.activeDatabase());
            command.execute()
                .done(function (results) {
                if (results.Error) {
                    messagePublisher.reportError("Unable to execute query", results.Error);
                }
                else {
                    var mainSelector = new pagedResultSet(results.Results.map(function (d) { return new document(d); }), results.Results.length, results);
                    var resultsFetcher = function (skip, take) {
                        var slicedResult = new pagedResultSet(mainSelector.items.slice(skip, Math.min(skip + take, mainSelector.totalResultCount)), mainSelector.totalResultCount);
                        return $.Deferred().resolve(slicedResult).promise();
                    };
                    var resultsList = new pagedList(resultsFetcher);
                    _this.queryResults(resultsList);
                }
            })
                .always(function () {
                _this.isBusy(false);
            });
            this.dataLoadingXhr(command.xhr);
        };
        exploration.prototype.killTask = function () {
            var xhr = this.dataLoadingXhr();
            if (xhr) {
                xhr.abort();
            }
            this.isBusy(false);
            this.queryResults(null);
        };
        return exploration;
    })(viewModelBase);
    return exploration;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/cancelSideBySizeConfirm", ["require", "exports", "commands/database/index/deleteIndexCommand", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/messagePublisher"], function (require, exports, deleteIndexCommand, dialog, dialogViewModelBase, messagePublisher) {
    var cancelSideBySizeConfirm = (function (_super) {
        __extends(cancelSideBySizeConfirm, _super);
        function cancelSideBySizeConfirm(indexNames, db, title) {
            _super.call(this);
            this.indexNames = indexNames;
            this.db = db;
            this.cancelTask = $.Deferred();
            if (!indexNames || indexNames.length === 0) {
                throw new Error("Indexes must not be null or empty.");
            }
            this.title = !!title ? title : indexNames.length == 1 ? 'Cancel side-by-side index?' : 'Cancel side-by-side indexes?';
        }
        cancelSideBySizeConfirm.prototype.cancelIndexes = function () {
            var _this = this;
            var cancelTasks = this.indexNames.map(function (name) { return new deleteIndexCommand(name, _this.db).execute(); });
            var myDeleteTask = this.cancelTask;
            $.when.apply($, cancelTasks)
                .done(function () {
                if (_this.indexNames.length > 1) {
                    messagePublisher.reportSuccess("Successfully cancelled " + _this.indexNames.length + " index replacements!");
                }
                myDeleteTask.resolve(false);
            })
                .fail(function () {
                myDeleteTask.reject();
            });
            dialog.close(this);
        };
        cancelSideBySizeConfirm.prototype.cancel = function () {
            this.cancelTask.reject();
            dialog.close(this);
        };
        return cancelSideBySizeConfirm;
    })(dialogViewModelBase);
    return cancelSideBySizeConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/copyIndexDialog", ["require", "exports", "models/database/index/indexDefinition", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/database/index/getIndexDefinitionCommand", "commands/database/index/saveIndexDefinitionCommand", "plugins/router", "common/appUrl", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, indexDefinition, dialog, dialogViewModelBase, getIndexDefinitionCommand, saveIndexDefinitionCommand, router, appUrl, messagePublisher, aceEditorBindingHandler) {
    var copyIndexDialog = (function (_super) {
        __extends(copyIndexDialog, _super);
        function copyIndexDialog(indexName, db, isPaste, elementToFocusOnDismissal) {
            if (isPaste === void 0) { isPaste = false; }
            _super.call(this, elementToFocusOnDismissal);
            this.indexName = indexName;
            this.db = db;
            this.isPaste = isPaste;
            this.indexJSON = ko.observable("");
            aceEditorBindingHandler.install();
        }
        copyIndexDialog.prototype.canActivate = function (args) {
            var _this = this;
            if (this.isPaste) {
                return true;
            }
            else {
                var canActivateResult = $.Deferred();
                new getIndexDefinitionCommand(this.indexName, this.db)
                    .execute()
                    .done(function (results) {
                    var prettifySpacing = 4;
                    var jsonString = JSON.stringify(new indexDefinition(results.Index).toDto(), null, prettifySpacing);
                    _this.indexJSON(jsonString);
                    canActivateResult.resolve({ can: true });
                })
                    .fail(function () { return canActivateResult.reject(); });
                return canActivateResult;
            }
        };
        copyIndexDialog.prototype.setInitialFocus = function () {
            // Overrides the base class' setInitialFocus and does nothing.
            // Doing nothing because we will focus the Ace Editor when it's initialized.
        };
        copyIndexDialog.prototype.enterKeyPressed = function () {
            // Overrides the base class' enterKeyPressed. Because the user might
            // edit the JSON, or even type some in manually, enter might really mean new line, not Save changes.
            if (!this.isPaste) {
                return _super.prototype.enterKeyPressed.call(this);
            }
            else {
                this.saveIndex();
            }
            return true;
        };
        copyIndexDialog.prototype.saveIndex = function () {
            var _this = this;
            if (this.isPaste && this.indexJSON()) {
                var indexDto;
                try {
                    indexDto = JSON.parse(this.indexJSON());
                }
                catch (e) {
                    indexDto = null;
                    messagePublisher.reportError("Index paste failed, invalid json string", e);
                }
                if (indexDto) {
                    new saveIndexDefinitionCommand(indexDto, this.db)
                        .execute()
                        .done(function () {
                        router.navigate(appUrl.forEditIndex(indexDto.Name, _this.db));
                        _this.close();
                    });
                }
            }
            else {
                this.close();
            }
        };
        copyIndexDialog.prototype.close = function () {
            dialog.close(this);
        };
        return copyIndexDialog;
    })(dialogViewModelBase);
    return copyIndexDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/deleteIndexesConfirm", ["require", "exports", "commands/database/index/deleteIndexCommand", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/messagePublisher"], function (require, exports, deleteIndexCommand, dialog, dialogViewModelBase, messagePublisher) {
    var deleteIndexesConfirm = (function (_super) {
        __extends(deleteIndexesConfirm, _super);
        function deleteIndexesConfirm(indexNames, db, title) {
            _super.call(this);
            this.indexNames = indexNames;
            this.db = db;
            this.deleteTask = $.Deferred();
            if (!indexNames || indexNames.length === 0) {
                throw new Error("Indexes must not be null or empty.");
            }
            this.title = !!title ? title : indexNames.length == 1 ? 'Delete index?' : 'Delete indexes?';
        }
        deleteIndexesConfirm.prototype.deleteIndexes = function () {
            var _this = this;
            var deleteTasks = this.indexNames.map(function (name) { return new deleteIndexCommand(name, _this.db).execute(); });
            var myDeleteTask = this.deleteTask;
            $.when.apply($, deleteTasks)
                .done(function () {
                if (_this.indexNames.length > 1) {
                    messagePublisher.reportSuccess("Successfully deleted " + _this.indexNames.length + " indexes!");
                }
                myDeleteTask.resolve(false);
            })
                .fail(function () {
                myDeleteTask.reject();
            });
            dialog.close(this);
        };
        deleteIndexesConfirm.prototype.cancel = function () {
            this.deleteTask.reject();
            dialog.close(this);
        };
        return deleteIndexesConfirm;
    })(dialogViewModelBase);
    return deleteIndexesConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/editIndex", ["require", "exports", "plugins/router", "viewmodels/viewModelBase", "models/database/index/index", "models/database/index/indexDefinition", "models/database/index/luceneField", "models/database/index/spatialIndexField", "commands/database/index/getIndexDefinitionCommand", "common/appUrl", "plugins/dialog", "common/jsonUtil", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher", "common/bindingHelpers/autoCompleteBindingHandler", "durandal/app", "models/database/index/indexAceAutoCompleteProvider", "commands/database/index/getScriptedIndexesCommand", "models/database/index/scriptedIndex", "common/autoCompleterSupport", "common/mergedIndexesStorage", "viewmodels/database/indexes/deleteIndexesConfirm", "viewmodels/database/indexes/replaceIndexDialog", "commands/database/documents/saveDocumentCommand", "models/database/index/indexReplaceDocument", "commands/database/index/saveIndexDefinitionCommand", "commands/database/documents/saveScriptedIndexesCommand", "commands/database/index/deleteIndexCommand", "viewmodels/database/indexes/cancelSideBySizeConfirm", "viewmodels/database/indexes/copyIndexDialog", "commands/database/index/getCSharpIndexDefinitionCommand", "viewmodels/common/showDataDialog", "commands/database/index/formatIndexCommand"], function (require, exports, router, viewModelBase, index, indexDefinition, luceneField, spatialIndexField, getIndexDefinitionCommand, appUrl, dialog, jsonUtil, aceEditorBindingHandler, messagePublisher, autoCompleteBindingHandler, app, indexAceAutoCompleteProvider, getScriptedIndexesCommand, scriptedIndexModel, autoCompleterSupport, mergedIndexesStorage, deleteIndexesConfirm, replaceIndexDialog, saveDocumentCommand, indexReplaceDocument, saveIndexDefinitionCommand, saveScriptedIndexesCommand, deleteIndexCommand, cancelSideBySizeConfirm, copyIndexDialog, getCSharpIndexDefinitionCommand, showDataDialog, formatIndexCommand) {
    var editIndex = (function (_super) {
        __extends(editIndex, _super);
        function editIndex() {
            var _this = this;
            _super.call(this);
            this.isEditingExistingIndex = ko.observable(false);
            this.mergeSuggestion = ko.observable(null);
            this.editedIndex = ko.observable();
            this.termsUrl = ko.observable();
            this.queryUrl = ko.observable();
            this.editMaxIndexOutputsPerDocument = ko.observable(false);
            this.indexErrorsList = ko.observableArray();
            this.loadedIndexName = ko.observable();
            // Scripted Index Part
            this.isScriptedIndexBundleActive = ko.observable(false);
            this.scriptedIndex = ko.observable(null);
            this.indexScript = ko.observable("");
            this.deleteScript = ko.observable("");
            aceEditorBindingHandler.install();
            autoCompleteBindingHandler.install();
            this.appUrls = appUrl.forCurrentDatabase();
            this.hasExistingReduce = ko.computed(function () { return _this.editedIndex() && _this.editedIndex().reduce(); });
            this.hasMultipleMaps = ko.computed(function () { return _this.editedIndex() && _this.editedIndex().maps().length > 1; });
            this.indexName = ko.computed(function () { return (!!_this.editedIndex() && _this.isEditingExistingIndex()) ? _this.editedIndex().name() : "New Index"; });
            this.currentIndexName = ko.computed(function () { return _this.isEditingExistingIndex() ? _this.editedIndex().name() : (_this.mergeSuggestion() != null) ? "Merged Index" : "New Index"; });
            this.isScriptedIndexBundleActive.subscribe(function (active) {
                if (active) {
                    _this.fetchOrCreateScriptedIndex();
                }
            });
            this.indexName.subscribe(function (name) {
                if (_this.scriptedIndex() !== null) {
                    _this.scriptedIndex().indexName(name);
                }
            });
            this.scriptedIndex.subscribe(function (scriptedIndex) {
                _this.indexScript = scriptedIndex.indexScript;
                _this.deleteScript = scriptedIndex.deleteScript;
                _this.initializeDirtyFlag();
                _this.editedIndex().name.valueHasMutated();
            });
            this.editedIndex(this.createNewIndexDefinition());
        }
        editIndex.prototype.canActivate = function (indexToEditName) {
            _super.prototype.canActivate.call(this, indexToEditName);
            var db = this.activeDatabase();
            var mergeSuggestion = mergedIndexesStorage.getMergedIndex(db, indexToEditName);
            if (mergeSuggestion != null) {
                this.mergeSuggestion(mergeSuggestion);
                this.editedIndex(mergeSuggestion.mergedIndexDefinition);
            }
            else if (indexToEditName) {
                this.isEditingExistingIndex(true);
                var canActivateResult = $.Deferred();
                this.fetchIndexData(indexToEditName)
                    .done(function () { return canActivateResult.resolve({ can: true }); })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + decodeURIComponent(indexToEditName) + " index");
                    canActivateResult.resolve({ redirect: appUrl.forIndexes(db) });
                });
                return canActivateResult;
            }
            return $.Deferred().resolve({ can: true });
        };
        editIndex.prototype.activate = function (indexToEditName) {
            _super.prototype.activate.call(this, indexToEditName);
            if (this.isEditingExistingIndex()) {
                this.editExistingIndex(indexToEditName);
            }
            this.updateHelpLink('CQ5AYO');
            this.initializeDirtyFlag();
            this.indexAutoCompleter = new indexAceAutoCompleteProvider(this.activeDatabase(), this.editedIndex);
            this.checkIfScriptedIndexBundleIsActive();
        };
        editIndex.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.addMapHelpPopover();
            this.addReduceHelpPopover();
            this.addScriptsLabelPopover();
        };
        editIndex.prototype.initializeDirtyFlag = function () {
            var _this = this;
            var indexDef = this.editedIndex();
            var checkedFieldsArray = [indexDef.storeAllFields, indexDef.name, indexDef.map, indexDef.maps, indexDef.reduce, indexDef.numOfLuceneFields, indexDef.numOfSpatialFields, indexDef.maxIndexOutputsPerDocument];
            indexDef.luceneFields().forEach(function (lf) {
                checkedFieldsArray.push(lf.name);
                checkedFieldsArray.push(lf.stores);
                checkedFieldsArray.push(lf.sort);
                checkedFieldsArray.push(lf.termVector);
                checkedFieldsArray.push(lf.indexing);
                checkedFieldsArray.push(lf.analyzer);
                checkedFieldsArray.push(lf.suggestionEnabled);
            });
            indexDef.spatialFields().forEach(function (sf) {
                checkedFieldsArray.push(sf.name);
                checkedFieldsArray.push(sf.type);
                checkedFieldsArray.push(sf.strategy);
                checkedFieldsArray.push(sf.minX);
                checkedFieldsArray.push(sf.maxX);
                checkedFieldsArray.push(sf.circleRadiusUnits);
                checkedFieldsArray.push(sf.maxTreeLevel);
                checkedFieldsArray.push(sf.minY);
                checkedFieldsArray.push(sf.maxY);
            });
            checkedFieldsArray.push(this.indexScript);
            checkedFieldsArray.push(this.deleteScript);
            this.dirtyFlag = new ko.DirtyFlag(checkedFieldsArray, false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return !!_this.editedIndex().name() && _this.dirtyFlag().isDirty(); });
        };
        editIndex.prototype.editExistingIndex = function (unescapedIndexName) {
            var indexName = decodeURIComponent(unescapedIndexName);
            this.loadedIndexName(indexName);
            this.termsUrl(appUrl.forTerms(unescapedIndexName, this.activeDatabase()));
            this.queryUrl(appUrl.forQuery(this.activeDatabase(), indexName));
        };
        editIndex.prototype.addMapHelpPopover = function () {
            $("#indexMapsLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'Maps project the fields to search on or to group by. It uses LINQ query syntax.<br/><br/>Example:</br><pre><span class="code-keyword">from</span> order <span class="code-keyword">in</span> docs.Orders<br/><span class="code-keyword">where</span> order.IsShipped<br/><span class="code-keyword">select new</span><br/>{</br>   order.Date, <br/>   order.Amount,<br/>   RegionId = order.Region.Id <br />}</pre>Each map function should project the same set of fields.',
            });
        };
        editIndex.prototype.addReduceHelpPopover = function () {
            $("#indexReduceLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'The Reduce function consolidates documents from the Maps stage into a smaller set of documents. It uses LINQ query syntax.<br/><br/>Example:</br><pre><span class="code-keyword">from</span> result <span class="code-keyword">in</span> results<br/><span class="code-keyword">group</span> result <span class="code-keyword">by new</span> { result.RegionId, result.Date } into g<br/><span class="code-keyword">select new</span><br/>{<br/>  Date = g.Key.Date,<br/>  RegionId = g.Key.RegionId,<br/>  Amount = g.Sum(x => x.Amount)<br/>}</pre>The objects produced by the Reduce function should have the same fields as the inputs.',
            });
        };
        editIndex.prototype.fetchIndexData = function (unescapedIndexName) {
            var indexName = decodeURIComponent(unescapedIndexName);
            return $.when(this.fetchIndexToEdit(indexName));
        };
        editIndex.prototype.fetchIndexToEdit = function (indexName) {
            var _this = this;
            var deferred = $.Deferred();
            new getIndexDefinitionCommand(indexName, this.activeDatabase())
                .execute()
                .done(function (results) {
                _this.editedIndex(new indexDefinition(results.Index));
                _this.originalIndexName = _this.editedIndex().name();
                _this.editMaxIndexOutputsPerDocument(results.Index.MaxIndexOutputsPerDocument ? results.Index.MaxIndexOutputsPerDocument > 0 ? true : false : false);
                deferred.resolve();
            })
                .fail(function () { return deferred.reject(); });
            return deferred;
        };
        editIndex.prototype.createNewIndexDefinition = function () {
            return indexDefinition.empty();
        };
        editIndex.prototype.save = function () {
            var editedIndex = this.editedIndex();
            //if index name has changed it isn't the same index
            if (this.originalIndexName === this.indexName() && editedIndex.lockMode === "LockedIgnore") {
                messagePublisher.reportWarning("Can not overwrite locked index: " + editedIndex.name() + ". " +
                    "Any changes to the index will be ignored.");
                return;
            }
            if (editedIndex.name()) {
                var index = editedIndex.toDto();
                this.saveIndex(index);
            }
        };
        editIndex.prototype.deleteMergedIndexes = function (indexesToDelete) {
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(indexesToDelete, db, "Delete Merged Indexes?");
            dialog.show(deleteViewModel);
        };
        editIndex.prototype.updateUrl = function (indexName, isSavingMergedIndex) {
            if (isSavingMergedIndex === void 0) { isSavingMergedIndex = false; }
            var url = appUrl.forEditIndex(indexName, this.activeDatabase());
            if (this.loadedIndexName() !== indexName) {
                _super.prototype.navigate.call(this, url);
            }
            else if (isSavingMergedIndex) {
                _super.prototype.updateUrl.call(this, url);
            }
        };
        editIndex.prototype.refreshIndex = function () {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to refresh the index from the server?');
            canContinue.done(function () {
                _this.fetchIndexData(_this.loadedIndexName())
                    .done(function () {
                    _this.initializeDirtyFlag();
                    _this.editedIndex().name.valueHasMutated();
                });
            });
        };
        editIndex.prototype.deleteIndex = function () {
            var _this = this;
            var indexName = this.loadedIndexName();
            if (indexName) {
                var db = this.activeDatabase();
                var deleteViewModel = new deleteIndexesConfirm([indexName], db);
                deleteViewModel.deleteTask.done(function () {
                    //prevent asking for unsaved changes
                    _this.dirtyFlag().reset(); // Resync Changes
                    router.navigate(appUrl.forIndexes(db));
                });
                dialog.show(deleteViewModel);
            }
        };
        editIndex.prototype.cancelSideBySideIndex = function () {
            var _this = this;
            var indexName = this.loadedIndexName();
            if (indexName) {
                var db = this.activeDatabase();
                var cancelSideBySideIndexViewModel = new cancelSideBySizeConfirm([indexName], db);
                cancelSideBySideIndexViewModel.cancelTask.done(function () {
                    //prevent asking for unsaved changes
                    _this.dirtyFlag().reset(); // Resync Changes
                    router.navigate(appUrl.forIndexes(db));
                });
                dialog.show(cancelSideBySideIndexViewModel);
            }
        };
        editIndex.prototype.addMap = function () {
            this.editedIndex().maps.push(ko.observable());
        };
        editIndex.prototype.addReduce = function () {
            if (!this.hasExistingReduce()) {
                this.editedIndex().reduce(" ");
                this.addReduceHelpPopover();
            }
        };
        editIndex.prototype.addField = function () {
            var field = new luceneField("");
            field.indexFieldNames = this.editedIndex().fields();
            field.calculateFieldNamesAutocomplete();
            this.editedIndex().luceneFields.push(field);
        };
        editIndex.prototype.removeStoreAllFields = function () {
            this.editedIndex().setOrRemoveStoreAllFields(false);
        };
        editIndex.prototype.removeMaxIndexOutputs = function () {
            this.editedIndex().maxIndexOutputsPerDocument(0);
            this.editMaxIndexOutputsPerDocument(false);
        };
        editIndex.prototype.addSpatialField = function () {
            var field = spatialIndexField.empty();
            this.editedIndex().spatialFields.push(field);
        };
        editIndex.prototype.removeMap = function (mapIndex) {
            this.editedIndex().maps.splice(mapIndex, 1);
        };
        editIndex.prototype.removeReduce = function () {
            this.editedIndex().reduce(null);
        };
        editIndex.prototype.removeLuceneField = function (fieldIndex) {
            this.editedIndex().luceneFields.splice(fieldIndex, 1);
        };
        editIndex.prototype.removeSpatialField = function (fieldIndex) {
            this.editedIndex().spatialFields.splice(fieldIndex, 1);
        };
        editIndex.prototype.copyIndex = function () {
            app.showDialog(new copyIndexDialog(this.editedIndex().name(), this.activeDatabase(), false));
        };
        editIndex.prototype.createCSharpCode = function () {
            new getCSharpIndexDefinitionCommand(this.editedIndex().name(), this.activeDatabase())
                .execute()
                .done(function (data) {
                app.showDialog(new showDataDialog("C# Index Definition", data));
            });
        };
        editIndex.prototype.formatIndex = function () {
            var index = this.editedIndex();
            var mapReduceObservableArray = new Array();
            mapReduceObservableArray.pushAll(index.maps());
            if (!!index.reduce()) {
                mapReduceObservableArray.push(index.reduce);
            }
            var mapReduceArray = mapReduceObservableArray.map(function (observable) { return observable(); });
            new formatIndexCommand(this.activeDatabase(), mapReduceArray)
                .execute()
                .done(function (formatedMapReduceArray) {
                formatedMapReduceArray.forEach(function (element, i) {
                    if (element.indexOf("Could not format:") == -1) {
                        mapReduceObservableArray[i](element);
                    }
                    else {
                        var isReduce = !!index.reduce() && i == formatedMapReduceArray.length - 1;
                        var errorMessage = isReduce ? "Failed to format reduce!" : "Failed to format map '" + i + "'!";
                        messagePublisher.reportError(errorMessage, element);
                    }
                });
            });
        };
        editIndex.prototype.checkIfScriptedIndexBundleIsActive = function () {
            var db = this.activeDatabase();
            var activeBundles = db.activeBundles();
            this.isScriptedIndexBundleActive(activeBundles.indexOf("ScriptedIndexResults") != -1);
        };
        editIndex.prototype.fetchOrCreateScriptedIndex = function () {
            var _this = this;
            var self = this;
            new getScriptedIndexesCommand(this.activeDatabase(), this.indexName())
                .execute()
                .done(function (scriptedIndexes) {
                if (scriptedIndexes.length > 0) {
                    self.scriptedIndex(scriptedIndexes[0]);
                }
                else {
                    self.scriptedIndex(scriptedIndexModel.emptyForIndex(self.indexName()));
                }
                _this.initializeDirtyFlag();
            });
        };
        editIndex.prototype.addScriptsLabelPopover = function () {
            var indexScriptpopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span>(company == null) <span class="code-keyword">return</span>;<br/>company.Orders = { Count: <span class="code-keyword">this</span>.Count, Total: <span class="code-keyword">this</span>.Total };<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.index-script-label',
            };
            $('#indexScriptPopover').popover(indexScriptpopOverSettings);
            var deleteScriptPopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span> (company == null) <span class="code-keyword">return</span>;<br/><span class="code-keyword">delete</span> company.Orders;<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.delete-script-label',
            };
            $('#deleteScriptPopover').popover(deleteScriptPopOverSettings);
        };
        editIndex.prototype.scriptedIndexCompleter = function (editor, session, pos, prefix, callback) {
            var completions = [
                { name: "LoadDocument", args: "id" },
                { name: "PutDocument", args: "id, doc" },
                { name: "DeleteDocument", args: "id" }
            ];
            var result = completions
                .filter(function (entry) { return autoCompleterSupport.wordMatches(prefix, entry.name); })
                .map(function (entry) { return { name: entry.name, value: entry.name, score: 100, meta: entry.args }; });
            callback(null, result);
        };
        editIndex.prototype.makePermanent = function () {
            var _this = this;
            if (this.editedIndex().name() && this.editedIndex().isTestIndex()) {
                this.editedIndex().isTestIndex(false);
                // trim Test prefix
                var indexToDelete = this.editedIndex().name();
                this.editedIndex().name(this.editedIndex().name().substr(index.TestIndexPrefix.length));
                var indexDef = this.editedIndex().toDto();
                if (this.scriptedIndex() !== null) {
                    // reset etag as we save different document
                    delete this.scriptedIndex().__metadata.etag;
                }
                this.saveIndex(indexDef)
                    .done(function () {
                    new deleteIndexCommand(indexToDelete, _this.activeDatabase()).execute();
                });
            }
        };
        editIndex.prototype.tryIndex = function () {
            if (this.editedIndex().name()) {
                if (!this.editedIndex().isTestIndex()) {
                    this.editedIndex().isTestIndex(true);
                    this.editedIndex().name(index.TestIndexPrefix + this.editedIndex().name());
                }
                var indexDef = this.editedIndex().toDto();
                if (this.scriptedIndex() !== null) {
                    // reset etag as we save different document
                    delete this.scriptedIndex().__metadata.etag;
                }
                this.saveIndex(indexDef);
            }
        };
        editIndex.prototype.saveIndex = function (index) {
            var _this = this;
            var commands = [];
            commands.push(new saveIndexDefinitionCommand(index, this.activeDatabase()).execute());
            if (this.scriptedIndex() !== null) {
                commands.push(new saveScriptedIndexesCommand([this.scriptedIndex()], this.activeDatabase()).execute());
            }
            return $.when.apply($, commands).done(function () {
                _this.initializeDirtyFlag();
                _this.editedIndex().name.valueHasMutated();
                var isSavingMergedIndex = _this.mergeSuggestion() != null;
                if (!_this.isEditingExistingIndex()) {
                    _this.isEditingExistingIndex(true);
                    _this.editExistingIndex(index.Name);
                }
                if (isSavingMergedIndex) {
                    var indexesToDelete = _this.mergeSuggestion().canMerge.filter(function (indexName) { return indexName != _this.editedIndex().name(); });
                    _this.deleteMergedIndexes(indexesToDelete);
                    _this.mergeSuggestion(null);
                }
                _this.updateUrl(index.Name, isSavingMergedIndex);
            });
        };
        editIndex.prototype.replaceIndex = function () {
            var _this = this;
            var indexToReplaceName = this.editedIndex().name();
            var replaceDialog = new replaceIndexDialog(indexToReplaceName, this.activeDatabase());
            replaceDialog.replaceSettingsTask.done(function (replaceDocument) {
                if (!_this.editedIndex().isSideBySideIndex()) {
                    _this.editedIndex().name(index.SideBySideIndexPrefix + _this.editedIndex().name());
                }
                var indexDef = _this.editedIndex().toDto();
                var replaceDocumentKey = indexReplaceDocument.replaceDocumentPrefix + _this.editedIndex().name();
                if (_this.scriptedIndex() !== null) {
                    // reset etag as we save different document
                    delete _this.scriptedIndex().__metadata.etag;
                }
                _this.saveIndex(indexDef)
                    .fail(function (response) { return messagePublisher.reportError("Failed to save replace index.", response.responseText, response.statusText); })
                    .done(function () {
                    new saveDocumentCommand(replaceDocumentKey, replaceDocument, _this.activeDatabase(), false)
                        .execute()
                        .fail(function (response) { return messagePublisher.reportError("Failed to save replace index document.", response.responseText, response.statusText); })
                        .done(function () { return messagePublisher.reportSuccess("Successfully saved side-by-side index"); });
                })
                    .always(function () { return dialog.close(replaceDialog); });
            });
            app.showDialog(replaceDialog);
        };
        return editIndex;
    })(viewModelBase);
    return editIndex;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexes", ["require", "exports", "commands/resources/getDatabaseStatsCommand", "viewmodels/viewModelBase", "models/database/index/index", "common/appUrl", "commands/database/index/saveIndexLockModeCommand", "commands/database/index/saveIndexAsPersistentCommand", "models/database/query/querySort", "durandal/app", "viewmodels/database/indexes/resetIndexConfirm", "common/recentQueriesStorage", "common/changesContext", "viewmodels/database/indexes/copyIndexDialog", "viewmodels/database/indexes/indexesAndTransformersClipboardDialog", "models/database/index/indexReplaceDocument", "commands/database/index/getPendingIndexReplacementsCommand", 'd3/d3', "viewmodels/database/indexes/cancelSideBySizeConfirm", "viewmodels/database/indexes/deleteIndexesConfirm", "commands/database/index/forceIndexReplace", "commands/database/index/saveIndexPriorityCommand", "commands/database/index/tryRecoverCorruptedIndexes", "viewmodels/database/indexes/indexLockAllConfirm"], function (require, exports, getDatabaseStatsCommand, viewModelBase, index, appUrl, saveIndexLockModeCommand, saveIndexAsPersistentCommand, querySort, app, resetIndexConfirm, recentQueriesStorage, changesContext, copyIndexDialog, indexesAndTransformersClipboardDialog, indexReplaceDocument, getPendingIndexReplacementsCommand, d3, cancelSideBySizeConfirm, deleteIndexesConfirm, forceIndexReplace, saveIndexPriorityCommand, tryRecoverCorruptedIndexes, indexLockAllConfirm) {
    var indexes = (function (_super) {
        __extends(indexes, _super);
        function indexes() {
            var _this = this;
            _super.call(this);
            this.resetsInProgress = d3.set();
            this.indexGroups = ko.observableArray();
            this.queryUrl = ko.observable();
            this.newIndexUrl = appUrl.forCurrentDatabase().newIndex;
            this.containerSelector = "#indexesContainer";
            this.recentQueries = ko.observableArray();
            this.indexMutex = true;
            this.btnState = ko.observable(false);
            this.btnStateTooltip = ko.observable("ExpandAll");
            this.btnTitle = ko.computed(function () { return _this.btnState() ? "Expand all" : "Collapse all"; });
            this.searchText = ko.observable();
            this.searchText.extend({ throttle: 200 }).subscribe(function () { return _this.filterIndexes(); });
            this.sortedGroups = ko.computed(function () {
                var groups = _this.indexGroups().slice(0).sort(function (l, r) { return l.entityName.toLowerCase() > r.entityName.toLowerCase() ? 1 : -1; });
                groups.forEach(function (group) {
                    group.indexes(group.indexes().slice(0).sort(function (l, r) { return l.name.toLowerCase() > r.name.toLowerCase() ? 1 : -1; }));
                });
                return groups;
            });
            this.corruptedIndexes = ko.computed(function () {
                var corrupted = [];
                _this.indexGroups().forEach(function (g) { return corrupted.pushAll(g.indexes().filter(function (i) { return i.priority && i.priority.indexOf(index.priorityErrored) !== -1; })); });
                return corrupted.distinct();
            });
            this.lockModeCommon = ko.computed(function () {
                var allIndexes = _this.getAllIndexes();
                if (allIndexes.length === 0)
                    return "None";
                var firstLockMode = allIndexes[0].lockMode();
                for (var i = 1; i < allIndexes.length; i++) {
                    if (allIndexes[i].lockMode() !== firstLockMode) {
                        return "Mixed";
                    }
                }
                return firstLockMode;
            });
        }
        indexes.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.fetchRecentQueries();
            $.when(this.fetchIndexes())
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ can: false }); });
            return deferred;
        };
        indexes.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('AIHAR1');
            this.queryUrl(appUrl.forQuery(this.activeDatabase(), null));
        };
        indexes.prototype.attached = function () {
            _super.prototype.attached.call(this);
            // Alt+Minus and Alt+Plus are already setup. Since laptops don't have a dedicated key for plus, we'll also use the equal sign key (co-opted for plus).
            //this.createKeyboardShortcut("Alt+=", () => this.toggleExpandAll(), this.containerSelector);
            ko.postbox.publish("SetRawJSONUrl", appUrl.forIndexesRawData(this.activeDatabase()));
            var self = this;
            $(window).bind('storage', function () {
                self.fetchRecentQueries();
            });
        };
        indexes.prototype.idlePriority = function (idx) {
            this.setIndexPriority(idx, 4 /* idleForced */);
        };
        indexes.prototype.disabledPriority = function (idx) {
            this.setIndexPriority(idx, 5 /* disabledForced */);
        };
        indexes.prototype.abandonedPriority = function (idx) {
            this.setIndexPriority(idx, 6 /* abandonedForced */);
        };
        indexes.prototype.normalPriority = function (idx) {
            this.setIndexPriority(idx, 0 /* normal */);
        };
        indexes.prototype.setIndexPriority = function (idx, newPriority) {
            var _this = this;
            new saveIndexPriorityCommand(idx.name, newPriority, this.activeDatabase())
                .execute()
                .done(function () {
                _this.fetchIndexes();
            });
        };
        indexes.prototype.fetchIndexes = function () {
            var _this = this;
            var deferred = $.Deferred();
            var statsTask = new getDatabaseStatsCommand(this.activeDatabase())
                .execute();
            var replacementTask = new getPendingIndexReplacementsCommand(this.activeDatabase()).execute();
            $.when(statsTask, replacementTask)
                .done(function (statsTaskResult, replacements) {
                var stats = statsTaskResult[0];
                _this.processData(stats, replacements);
                deferred.resolve(stats);
            })
                .fail(function (xhr) { return deferred.reject(xhr); });
            return deferred.promise();
        };
        indexes.prototype.fetchRecentQueries = function () {
            this.recentQueries(recentQueriesStorage.getRecentQueries(this.activeDatabase()));
        };
        indexes.prototype.filterIndexes = function () {
            var filterLower = this.searchText().toLowerCase();
            this.indexGroups().forEach(function (indexGroup) {
                var hasAnyInGroup = false;
                indexGroup.indexes().forEach(function (index) {
                    var match = index.name.toLowerCase().indexOf(filterLower) >= 0;
                    index.filteredOut(!match);
                    if (match) {
                        hasAnyInGroup = true;
                    }
                });
                indexGroup.groupHidden(!hasAnyInGroup);
            });
        };
        indexes.prototype.getRecentQueryUrl = function (query) {
            return appUrl.forQuery(this.activeDatabase(), query.Hash);
        };
        indexes.prototype.getRecentQuerySortText = function (sorts) {
            if (sorts.length > 0) {
                return sorts
                    .map(function (s) { return querySort.fromQuerySortString(s); })
                    .map(function (s) { return s.toHumanizedString(); })
                    .reduce(function (first, second) { return first + ", " + second; });
            }
            return "";
        };
        indexes.prototype.getStoredQueryTransformerParameters = function (queryParams) {
            if (queryParams.length > 0) {
                return "(" +
                    queryParams
                        .map(function (param) { return param.name + "=" + param.value; })
                        .join(", ") + ")";
            }
            return "";
        };
        indexes.prototype.processData = function (stats, replacements) {
            var _this = this;
            var willReplaceMap = d3.map([]);
            var willBeReplacedMap = d3.map([]);
            replacements.forEach(function (r) {
                willBeReplacedMap.set(r.indexToReplace, r.extractReplaceWithIndexName());
                willReplaceMap.set(r.extractReplaceWithIndexName(), r.indexToReplace);
            });
            stats.Indexes
                .map(function (i) {
                var idx = new index(i);
                if (willBeReplacedMap.has(idx.name)) {
                    idx.willBeReplacedByIndex(willBeReplacedMap.get(idx.name));
                }
                if (willReplaceMap.has(idx.name)) {
                    idx.willReplaceIndex(willReplaceMap.get(idx.name));
                }
                return idx;
            })
                .forEach(function (i) { return _this.putIndexIntoGroups(i); });
        };
        indexes.prototype.putIndexIntoGroups = function (i) {
            if (!i.forEntityName || i.forEntityName.length === 0) {
                this.putIndexIntoGroupNamed(i, "Other");
            }
            else {
                this.putIndexIntoGroupNamed(i, this.getGroupName(i));
            }
        };
        indexes.prototype.getGroupName = function (i) {
            return i.forEntityName.sort(function (l, r) { return l.toLowerCase() > r.toLowerCase() ? 1 : -1; }).join(", ");
        };
        indexes.prototype.putIndexIntoGroupNamed = function (i, groupName) {
            var group = this.indexGroups.first(function (g) { return g.entityName === groupName; });
            var oldIndex;
            if (group) {
                oldIndex = group.indexes.first(function (cur) { return cur.name === i.name; });
                if (!!oldIndex) {
                    group.indexes.replace(oldIndex, i);
                }
                else {
                    group.indexes.push(i);
                }
            }
            else {
                this.indexGroups.push({ entityName: groupName, indexes: ko.observableArray([i]), groupHidden: ko.observable(false) });
            }
        };
        indexes.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.processIndexEvent(e); }),
                changesContext.currentResourceChangesApi().watchDocsStartingWith(indexReplaceDocument.replaceDocumentPrefix, function () { return _this.processReplaceEvent(); })
            ];
        };
        indexes.prototype.processReplaceEvent = function () {
            var _this = this;
            if (this.indexMutex) {
                this.indexMutex = false;
                setTimeout(function () {
                    _this.fetchIndexes().always(function () { return _this.indexMutex = true; });
                }, 10);
            }
        };
        indexes.prototype.processIndexEvent = function (e) {
            var _this = this;
            if (e.Type === "IndexRemoved") {
                if (!this.resetsInProgress.has(e.Name)) {
                    this.removeIndexesFromAllGroups(this.findIndexesByName(e.Name));
                }
            }
            else {
                if (this.indexMutex) {
                    this.indexMutex = false;
                    setTimeout(function () {
                        _this.fetchIndexes().always(function () { return _this.indexMutex = true; });
                    }, 5000);
                }
            }
        };
        indexes.prototype.findIndexesByName = function (indexName) {
            var result = new Array();
            this.indexGroups().forEach(function (g) {
                g.indexes().forEach(function (i) {
                    if (i.name === indexName) {
                        result.push(i);
                    }
                });
            });
            return result;
        };
        indexes.prototype.copyIndex = function (i) {
            app.showDialog(new copyIndexDialog(i.name, this.activeDatabase(), false));
        };
        indexes.prototype.pasteIndex = function () {
            app.showDialog(new copyIndexDialog('', this.activeDatabase(), true));
        };
        indexes.prototype.copyIndexesAndTransformers = function () {
            app.showDialog(new indexesAndTransformersClipboardDialog(this.activeDatabase(), false));
        };
        indexes.prototype.pasteIndexesAndTransformers = function () {
            var _this = this;
            var dialog = new indexesAndTransformersClipboardDialog(this.activeDatabase(), true);
            app.showDialog(dialog);
            dialog.pasteDeferred.done(function (summary) {
                _this.confirmationMessage("Indexes And Transformers Paste Summary", summary, ['Ok']);
            });
        };
        indexes.prototype.toggleExpandAll = function () {
            if (this.btnState()) {
                $(".index-group-content").collapse('show');
            }
            else {
                $(".index-group-content").collapse('hide');
            }
            this.btnState.toggle();
        };
        indexes.prototype.deleteIdleIndexes = function () {
            var idleIndexes = this.getAllIndexes().filter(function (i) { return i.priority && i.priority.indexOf("Idle") !== -1; });
            this.promptDeleteIndexes(idleIndexes);
        };
        indexes.prototype.deleteDisabledIndexes = function () {
            var abandonedIndexes = this.getAllIndexes().filter(function (i) { return i.priority && i.priority.indexOf("Disabled") !== -1; });
            this.promptDeleteIndexes(abandonedIndexes);
        };
        indexes.prototype.deleteAbandonedIndexes = function () {
            var abandonedIndexes = this.getAllIndexes().filter(function (i) { return i.priority && i.priority.indexOf("Abandoned") !== -1; });
            this.promptDeleteIndexes(abandonedIndexes);
        };
        indexes.prototype.deleteAllIndexes = function () {
            this.promptDeleteIndexes(this.getAllIndexes().filter(function (i) { return i.name !== "Raven/DocumentsByEntityName"; }));
        };
        indexes.prototype.deleteIndex = function (i) {
            this.promptDeleteIndexes([i]);
            this.resetsInProgress.remove(i.name);
        };
        indexes.prototype.deleteIndexGroup = function (i) {
            this.promptDeleteIndexes(i.indexes());
        };
        indexes.prototype.cancelIndex = function (i) {
            var _this = this;
            var cancelSideBySideIndexViewModel = new cancelSideBySizeConfirm([i.name], this.activeDatabase());
            app.showDialog(cancelSideBySideIndexViewModel);
            cancelSideBySideIndexViewModel.cancelTask
                .done(function (closedWithoutDeletion) {
                if (!closedWithoutDeletion) {
                    _this.removeIndexesFromAllGroups([i]);
                }
            })
                .fail(function () {
                _this.removeIndexesFromAllGroups([i]);
                _this.fetchIndexes();
            });
        };
        indexes.prototype.promptDeleteIndexes = function (indexes) {
            var _this = this;
            if (indexes.length > 0) {
                var deleteIndexesVm = new deleteIndexesConfirm(indexes.map(function (i) { return i.name; }), this.activeDatabase());
                app.showDialog(deleteIndexesVm);
                deleteIndexesVm.deleteTask
                    .done(function (closedWithoutDeletion) {
                    if (!closedWithoutDeletion) {
                        _this.removeIndexesFromAllGroups(indexes);
                    }
                })
                    .fail(function () {
                    _this.removeIndexesFromAllGroups(indexes);
                    _this.fetchIndexes();
                });
            }
        };
        indexes.prototype.resetIndex = function (indexToReset) {
            var _this = this;
            var resetIndexVm = new resetIndexConfirm(indexToReset.name, this.activeDatabase());
            // reset index is implemented as delete and insert, so we receive notification about deleted index via changes API
            // let's issue marker to ignore index delete information for next few seconds because it might be caused by reset.
            // Unfortunettely we can't use resetIndexVm.resetTask.done, because we receive event via changes api before resetTask promise 
            // if resolved. 
            this.resetsInProgress.add(indexToReset.name);
            setTimeout(function () {
                _this.resetsInProgress.remove(indexToReset.name);
            }, 30000);
            app.showDialog(resetIndexVm);
        };
        indexes.prototype.removeIndexesFromAllGroups = function (indexes) {
            this.indexGroups().forEach(function (g) {
                g.indexes.removeAll(indexes);
            });
            // Remove any empty groups.
            this.indexGroups.remove(function (item) { return item.indexes().length === 0; });
        };
        indexes.prototype.unlockIndex = function (i) {
            this.updateIndexLockMode(i, "Unlock");
        };
        indexes.prototype.lockIndex = function (i) {
            this.updateIndexLockMode(i, "LockedIgnore");
        };
        indexes.prototype.lockErrorIndex = function (i) {
            this.updateIndexLockMode(i, "LockedError");
        };
        indexes.prototype.lockSideBySide = function (i) {
            this.updateIndexLockMode(i, 'SideBySide');
        };
        indexes.prototype.updateIndexLockMode = function (i, newLockMode) {
            // The old Studio would prompt if you were sure.
            // However, changing the lock status is easily reversible, so we're skipping the prompt.
            var originalLockMode = i.lockMode();
            if (originalLockMode !== newLockMode) {
                i.lockMode(newLockMode);
                new saveIndexLockModeCommand(i, newLockMode, this.activeDatabase())
                    .execute()
                    .fail(function () { return i.lockMode(originalLockMode); });
            }
        };
        indexes.prototype.getAllIndexes = function () {
            var all = [];
            this.indexGroups().forEach(function (g) { return all.pushAll(g.indexes()); });
            return all.distinct();
        };
        indexes.prototype.makeIndexPersistent = function (index) {
            new saveIndexAsPersistentCommand(index, this.activeDatabase()).execute();
        };
        indexes.prototype.forceSideBySide = function (idx) {
            new forceIndexReplace(idx.name, this.activeDatabase()).execute();
        };
        indexes.prototype.tryRecoverCorruptedIndexes = function () {
            new tryRecoverCorruptedIndexes(this.activeDatabase()).execute();
        };
        indexes.prototype.setLockModeAllIndexes = function (lockModeString, lockModeStrForTitle) {
            if (this.lockModeCommon() === lockModeString)
                return;
            var lockModeTitle = "Do you want to " + lockModeStrForTitle + " ALL Indexes?";
            var indexLockAllVm = new indexLockAllConfirm(lockModeString, this.activeDatabase(), this.getAllIndexes(), lockModeTitle);
            app.showDialog(indexLockAllVm);
        };
        return indexes;
    })(viewModelBase);
    return indexes;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexesAndTransformersClipboardDialog", ["require", "exports", "models/database/index/transformer", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/database/index/getIndexesDefinitionsCommand", "commands/database/transformers/getTransformersCommand", "commands/database/index/saveIndexDefinitionCommand", "commands/database/transformers/saveTransformerCommand", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, transformerDefinition, dialog, dialogViewModelBase, getIndexesDefinitionsCommand, getTransformersCommand, saveIndexDefinitionCommand, saveTransformerCommand, messagePublisher, aceEditorBindingHandler) {
    var indexesAndTransformersClipboardDialog = (function (_super) {
        __extends(indexesAndTransformersClipboardDialog, _super);
        function indexesAndTransformersClipboardDialog(db, isPaste, elementToFocusOnDismissal) {
            if (isPaste === void 0) { isPaste = false; }
            _super.call(this, elementToFocusOnDismissal);
            this.db = db;
            this.isPaste = isPaste;
            this.json = ko.observable("");
            this.indexes = ko.observableArray([]);
            this.transformers = ko.observableArray([]);
            this.pasteDeferred = $.Deferred();
            aceEditorBindingHandler.install();
        }
        indexesAndTransformersClipboardDialog.prototype.canActivate = function (args) {
            var _this = this;
            if (this.isPaste) {
                return true;
            }
            else {
                var canActivateResult = $.Deferred();
                var getIndexDefinitionsPromise = new getIndexesDefinitionsCommand(this.db)
                    .execute()
                    .done(function (results) {
                    _this.indexes(results);
                });
                var getTransformersPromise = new getTransformersCommand(this.db)
                    .execute()
                    .done(function (results) {
                    _this.transformers(results);
                });
                $.when(getTransformersPromise, getIndexDefinitionsPromise)
                    .then(function () {
                    canActivateResult.resolve({ can: true });
                    var prettifySpacing = 4;
                    _this.json(JSON.stringify({
                        Indexes: _this.indexes(),
                        Transformers: _this.transformers()
                    }, null, prettifySpacing));
                }, function () {
                    canActivateResult.reject();
                });
                return canActivateResult;
            }
        };
        indexesAndTransformersClipboardDialog.prototype.setInitialFocus = function () {
            // Overrides the base class' setInitialFocus and does nothing.
            // Doing nothing because we will focus the Ace Editor when it's initialized.
        };
        indexesAndTransformersClipboardDialog.prototype.enterKeyPressed = function () {
            // Overrides the base class' enterKeyPressed. Because the user might
            // edit the JSON, or even type some in manually, enter might really mean new line, not Save changes.
            if (!this.isPaste) {
                return _super.prototype.enterKeyPressed.call(this);
            }
            else {
                this.saveAll();
            }
            return true;
        };
        indexesAndTransformersClipboardDialog.prototype.saveAll = function () {
            var _this = this;
            if (this.isPaste && this.json()) {
                var indexesAndTransformers;
                var indexesDefinitions = [];
                var transformersDefinitions = [];
                try {
                    indexesAndTransformers = JSON.parse(this.json());
                    if (indexesAndTransformers.Indexes && indexesAndTransformers.Indexes.length > 0) {
                        indexesDefinitions.pushAll(indexesAndTransformers.Indexes.map(function (index) {
                            return index.definition;
                        }));
                    }
                    if (indexesAndTransformers.Transformers && indexesAndTransformers.Transformers.length > 0) {
                        transformersDefinitions.pushAll(indexesAndTransformers.Transformers.map(function (transformer) {
                            return {
                                Transformer: {
                                    Name: transformer.name,
                                    TransformResults: transformer.definition.TransformResults,
                                    LockMode: transformer.definition.LockMode
                                }
                            };
                        }));
                    }
                    if (indexesDefinitions.length === 0 && transformersDefinitions.length === 0) {
                        throw "No indexes or transformers found in json string";
                    }
                }
                catch (e) {
                    this.pasteDeferred.reject();
                    messagePublisher.reportError("Index paste failed, invalid json string", e);
                }
                var allOperationsPromises = [];
                var succeededIndexes = [];
                var failedIndexes = [];
                var succeededTransformers = [];
                var failedTransformers = [];
                if (indexesDefinitions.length > 0) {
                    indexesDefinitions.forEach(function (index) {
                        var curDeferred = $.Deferred();
                        allOperationsPromises.push(curDeferred);
                        new saveIndexDefinitionCommand(index, _this.db)
                            .execute()
                            .done(function () { return succeededIndexes.push(index.Name); })
                            .fail(function () { return failedIndexes.push(index.Name); })
                            .always(function () { return curDeferred.resolve(); });
                    });
                }
                if (transformersDefinitions.length > 0) {
                    transformersDefinitions.forEach(function (transformer) {
                        var curDeferred = $.Deferred();
                        allOperationsPromises.push(curDeferred);
                        new saveTransformerCommand(new transformerDefinition().initFromSave(transformer), _this.db)
                            .execute()
                            .done(function () {
                            succeededTransformers.push(transformer.Transformer.Name);
                        })
                            .fail(function () {
                            failedTransformers.push(transformer.Transformer.Name);
                        })
                            .always(function () { return curDeferred.resolve(); });
                    });
                }
                $.when.apply($, allOperationsPromises)
                    .always(function () { return _this.summarize(succeededIndexes, failedIndexes, succeededTransformers, failedTransformers); });
            }
            else {
                this.close();
            }
        };
        indexesAndTransformersClipboardDialog.prototype.summarize = function (succeededIndexes, failedIndexes, succeededTransformers, failedTransformers) {
            var summaryText = "";
            if (succeededIndexes.length > 0) {
                summaryText += "Succeeded Indexes: " + succeededIndexes.length + "\n";
            }
            if (failedIndexes.length > 0) {
                summaryText += "Failed Indexes: \n" + failedIndexes.join("\n") + "\n";
            }
            if (succeededTransformers.length > 0) {
                summaryText += "Succeeded Transformers: " + succeededTransformers.length + "\n";
            }
            if (failedTransformers.length > 0) {
                summaryText += "Failed Transformers: \n" + failedTransformers.join("\n") + "\n";
            }
            this.pasteDeferred.resolve(summaryText);
            this.close();
        };
        indexesAndTransformersClipboardDialog.prototype.close = function () {
            dialog.close(this);
        };
        return indexesAndTransformersClipboardDialog;
    })(dialogViewModelBase);
    return indexesAndTransformersClipboardDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexesShell", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var indexesShell = (function (_super) {
        __extends(indexesShell, _super);
        function indexesShell() {
            _super.call(this);
            this.router = durandalRouter.createChildRouter()
                .map([
                { route: 'databases/indexes', moduleId: 'viewmodels/database/indexes/indexes', title: 'Indexes', nav: true },
                { route: 'databases/indexes/mergeSuggestions', moduleId: 'viewmodels/database/indexes/indexMergeSuggestions', title: 'Index Merge Suggestions', nav: true },
                { route: 'databases/indexes/edit(/:indexName)', moduleId: 'viewmodels/database/indexes/editIndex', title: 'Edit Index', nav: true },
                { route: 'databases/indexes/terms/(:indexName)', moduleId: 'viewmodels/database/indexes/indexTerms', title: 'Terms', nav: true }
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
        }
        return indexesShell;
    })(viewModelBase);
    return indexesShell;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexLockAllConfirm", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/database/index/saveIndexLockModeCommand"], function (require, exports, dialog, dialogViewModelBase, saveIndexLockModeCommand) {
    var indexLockAllConfirm = (function (_super) {
        __extends(indexLockAllConfirm, _super);
        function indexLockAllConfirm(setLockOperation, db, allIndexes, message) {
            _super.call(this);
            this.setLockOperation = setLockOperation;
            this.db = db;
            this.allIndexes = allIndexes;
            this.message = message;
        }
        indexLockAllConfirm.prototype.indexLockAllIndex = function () {
            var _this = this;
            this.allIndexes.forEach(function (i) {
                var originalLockMode = i.lockMode();
                if (originalLockMode !== _this.setLockOperation) {
                    i.lockMode(_this.setLockOperation);
                    new saveIndexLockModeCommand(i, _this.setLockOperation, _this.db)
                        .execute()
                        .fail(function () { return i.lockMode(originalLockMode); });
                }
            });
            dialog.close(this);
        };
        indexLockAllConfirm.prototype.cancel = function () {
            dialog.close(this);
        };
        return indexLockAllConfirm;
    })(dialogViewModelBase);
    return indexLockAllConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexMergeSuggestions", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getIndexMergeSuggestionsCommand", "common/appUrl", "common/mergedIndexesStorage", "models/database/index/indexMergeSuggestion", "commands/resources/getDatabaseStatsCommand", "moment", "plugins/dialog", "common/changesContext", "common/optional", "viewmodels/database/indexes/deleteIndexesConfirm"], function (require, exports, viewModelBase, getIndexMergeSuggestionsCommand, appUrl, mergedIndexesStorage, indexMergeSuggestion, getDatabaseStatsCommand, moment, dialog, changesContext, optional, deleteIndexesConfirm) {
    var indexMergeSuggestions = (function (_super) {
        __extends(indexMergeSuggestions, _super);
        function indexMergeSuggestions() {
            _super.call(this);
            this.suggestions = ko.observableArray();
            this.unmergables = ko.observableArray();
            this.idleOrAbandonedIndexes = ko.observableArray();
            this.notUsedForLastWeek = ko.observableArray();
            this.appUrls = appUrl.forCurrentDatabase();
        }
        indexMergeSuggestions.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            this.reload()
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) }); });
            return deferred;
        };
        indexMergeSuggestions.prototype.reload = function () {
            var fetchIndexMergeSuggestionsTask = this.fetchIndexMergeSuggestions();
            var fetchStatsTask = this.fetchStats();
            return $.when(fetchIndexMergeSuggestionsTask, fetchStatsTask);
        };
        indexMergeSuggestions.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchAllIndexes(function () { return _this.fetchIndexMergeSuggestions(); })];
        };
        indexMergeSuggestions.prototype.fetchStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getDatabaseStatsCommand(db)
                    .execute()
                    .done(function (result) { return _this.processStatsResults(result); });
            }
            return null;
        };
        indexMergeSuggestions.prototype.processStatsResults = function (stats) {
            var _this = this;
            this.idleOrAbandonedIndexes([]);
            this.notUsedForLastWeek([]);
            var now = moment();
            var miliSecondsInWeek = 1000 * 3600 * 24 * 7;
            stats.Indexes.forEach(function (indexDto) {
                // we are using contains not equals as priority may contains 
                if (indexDto.Priority.contains("Idle") || indexDto.Priority.contains("Abandoned")) {
                    _this.idleOrAbandonedIndexes.push(indexDto);
                }
                if (indexDto.LastQueryTimestamp) {
                    var lastQueryDate = moment(indexDto.LastQueryTimestamp);
                    if (lastQueryDate.isValid()) {
                        var agoInMs = now.diff(lastQueryDate);
                        if (agoInMs > miliSecondsInWeek) {
                            indexDto["LastQueryTimestampText"] = optional.val(indexDto.LastQueryTimestamp).bind(function (v) { return v.toHumanizedDate(); });
                            _this.notUsedForLastWeek.push(indexDto);
                        }
                    }
                }
            });
        };
        indexMergeSuggestions.prototype.fetchIndexMergeSuggestions = function () {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            new getIndexMergeSuggestionsCommand(db)
                .execute()
                .done(function (results) {
                var suggestions = results.Suggestions.map(function (suggestion) { return new indexMergeSuggestion(suggestion); });
                _this.suggestions(suggestions);
                var unmergables = Object.keys(results.Unmergables).map(function (value, index) {
                    return { indexName: value, reason: results.Unmergables[value] };
                });
                _this.unmergables(unmergables);
                deferred.resolve();
            })
                .fail(function () { return deferred.reject(); });
            return deferred;
        };
        indexMergeSuggestions.prototype.mergeSuggestionIndex = function (index) {
            return parseInt(index) + 1;
        };
        indexMergeSuggestions.prototype.mergedIndexUrl = function (id) {
            var db = this.activeDatabase();
            var mergedIndexName = mergedIndexesStorage.getMergedIndexName(db, id);
            return this.appUrls.editIndex(mergedIndexName);
        };
        indexMergeSuggestions.prototype.saveMergedIndex = function (id, suggestion) {
            var db = this.activeDatabase();
            mergedIndexesStorage.saveMergedIndex(db, id, suggestion);
            return true;
        };
        indexMergeSuggestions.prototype.deleteIndexes = function (index) {
            var _this = this;
            var mergeSuggestion = this.suggestions()[index];
            var indexesToDelete = mergeSuggestion.canDelete;
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(indexesToDelete, db);
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteIndex = function (name) {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm([name], db);
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteAllIdleOrAbandoned = function () {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(this.idleOrAbandonedIndexes().map(function (index) { return index.Name; }), db, "Delete all idle or abandoned indexes?");
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        indexMergeSuggestions.prototype.deleteAllNotUsedForWeek = function () {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewModel = new deleteIndexesConfirm(this.notUsedForLastWeek().map(function (index) { return index.Name; }), db, "Delete all indexes not used within last week?");
            deleteViewModel.deleteTask.always(function () { return _this.reload(); });
            dialog.show(deleteViewModel);
        };
        return indexMergeSuggestions;
    })(viewModelBase);
    return indexMergeSuggestions;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/indexTerms", ["require", "exports", "viewmodels/viewModelBase", "commands/database/index/getIndexTermsCommand", "commands/database/index/getIndexDefinitionCommand", "common/appUrl"], function (require, exports, viewModelBase, getIndexTermsCommand, getIndexDefinitionCommand, appUrl) {
    var indexTerms = (function (_super) {
        __extends(indexTerms, _super);
        function indexTerms() {
            _super.call(this);
            this.fields = ko.observableArray();
            this.termsPageLimit = 1024;
            this.appUrls = appUrl.forCurrentDatabase();
        }
        indexTerms.prototype.activate = function (indexName) {
            _super.prototype.activate.call(this, indexName);
            this.indexName = indexName;
            this.fetchIndexDefinition(indexName);
        };
        indexTerms.prototype.fetchIndexDefinition = function (indexName) {
            var _this = this;
            new getIndexDefinitionCommand(indexName, this.activeDatabase())
                .execute()
                .done(function (results) { return _this.processIndex(results); });
        };
        indexTerms.prototype.processIndex = function (indexContainer) {
            var _this = this;
            var fields = indexContainer.Index.Fields.map(function (fieldName) {
                return { name: fieldName, terms: ko.observableArray(), hasMoreTerms: ko.observable(false) };
            });
            this.fields(fields);
            this.fields()
                .forEach(function (field) {
                new getIndexTermsCommand(indexContainer.Index.Name, field.name, _this.activeDatabase())
                    .execute()
                    .done(function (terms) {
                    if (terms.length >= _this.termsPageLimit) {
                        field.hasMoreTerms(true);
                        terms = terms.slice(0, _this.termsPageLimit - 1);
                    }
                    field.terms(terms);
                });
            });
        };
        return indexTerms;
    })(viewModelBase);
    return indexTerms;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/replaceIndexDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/resources/getDatabaseStatsCommand", "common/bindingHelpers/datePickerBindingHandler", "moment", "models/database/index/indexReplaceDocument"], function (require, exports, dialog, dialogViewModelBase, getDatabaseStatsCommand, datePickerBindingHandler, moment, indexReplaceDocument) {
    var replaceIndexDialog = (function (_super) {
        __extends(replaceIndexDialog, _super);
        function replaceIndexDialog(indexName, db, elementToFocusOnDismissal) {
            var _this = this;
            _super.call(this, elementToFocusOnDismissal);
            this.indexName = indexName;
            this.db = db;
            this.replaceSettingsTask = $.Deferred();
            this.etagMode = ko.observable(false);
            this.dateMode = ko.observable(false);
            this.lastIndexedEtag = ko.observable();
            this.replaceDate = ko.observable(moment(new Date()));
            this.replaceDateText = ko.computed(function () {
                return _this.replaceDate() != null ? _this.replaceDate().format("YYYY/MM/DD H:mm:ss") : "";
            });
            datePickerBindingHandler.install();
        }
        replaceIndexDialog.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            this.fetchIndexes()
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ can: false }); });
            return deferred.promise();
        };
        replaceIndexDialog.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.db)
                .execute()
                .done(function (stats) { return _this.processDbStats(stats); });
        };
        replaceIndexDialog.prototype.processDbStats = function (stats) {
            var _this = this;
            var oldIndex = stats.Indexes.first(function (i) { return i.Name == _this.indexName; });
            this.lastIndexedEtag(oldIndex.LastIndexedEtag);
        };
        replaceIndexDialog.prototype.saveReplace = function () {
            var replaceDocument = new indexReplaceDocument({ IndexToReplace: this.indexName });
            if (this.etagMode()) {
                replaceDocument.minimumEtagBeforeReplace = this.lastIndexedEtag();
            }
            if (this.dateMode()) {
                replaceDocument.replaceTimeUtc = this.replaceDate().toISOString();
            }
            this.replaceSettingsTask.resolve(replaceDocument);
        };
        replaceIndexDialog.prototype.close = function () {
            this.replaceSettingsTask.reject();
            dialog.close(this);
        };
        replaceIndexDialog.prototype.toggleEtagMode = function () {
            this.etagMode(!this.etagMode());
        };
        replaceIndexDialog.prototype.toggleDateMode = function () {
            this.dateMode(!this.dateMode());
        };
        return replaceIndexDialog;
    })(dialogViewModelBase);
    return replaceIndexDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/resetIndexConfirm", ["require", "exports", "commands/database/index/resetIndexCommand", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, resetIndexCommand, dialog, dialogViewModelBase) {
    var resetIndexConfirm = (function (_super) {
        __extends(resetIndexConfirm, _super);
        function resetIndexConfirm(indexName, db) {
            _super.call(this);
            this.indexName = indexName;
            this.db = db;
            this.resetTask = $.Deferred();
        }
        resetIndexConfirm.prototype.resetIndex = function () {
            var _this = this;
            new resetIndexCommand(this.indexName, this.db).execute().done(function () { return _this.resetTask.resolve(); }).fail(function () { return _this.resetTask.reject(); });
            dialog.close(this);
        };
        resetIndexConfirm.prototype.cancel = function () {
            this.resetTask.reject();
            dialog.close(this);
        };
        return resetIndexConfirm;
    })(dialogViewModelBase);
    return resetIndexConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/indexes/scriptedIndexes", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "common/bindingHelpers/aceEditorBindingHandler", "models/database/index/scriptedIndex", "commands/database/index/getScriptedIndexesCommand", "commands/database/documents/saveScriptedIndexesCommand", "common/appUrl"], function (require, exports, viewModelBase, getDatabaseStatsCommand, aceEditorBindingHandler, scriptedIndex, getScriptedIndexesCommand, saveScriptedIndexesCommand, appUrl) {
    var scriptedIndexes = (function (_super) {
        __extends(scriptedIndexes, _super);
        function scriptedIndexes() {
            var _this = this;
            _super.call(this);
            this.allScriptedIndexes = ko.observableArray();
            this.activeScriptedIndexes = ko.observableArray().extend({ required: true });
            this.indexNames = ko.observableArray();
            this.isFirstLoad = ko.observable(true);
            aceEditorBindingHandler.install();
            this.inactiveIndexNames = ko.computed(function () {
                var activeIndexNames = _this.allScriptedIndexes()
                    .filter(function (index) { return !index.isMarkedToDelete(); })
                    .map(function (index) { return index.indexName(); });
                return _this.indexNames().filter(function (indexName) { return activeIndexNames.indexOf(indexName) < 0; });
            }, this);
            this.firstIndex = ko.computed(function () {
                return !_this.isFirstLoad() ? 0 : -1;
            }, this);
        }
        scriptedIndexes.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchAllIndexes(db)
                    .done(function () {
                    _this.fetchAllScriptedIndexes(db)
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
                });
            }
            return deferred;
        };
        scriptedIndexes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.activeScriptedIndexes]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        scriptedIndexes.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.addScriptsLabelPopover();
            this.initializeCollapsedInvalidElements();
            $('pre').each(function (index, currentPreElement) {
                if (currentPreElement) {
                    var editor = ko.utils.domData.get(currentPreElement, "aceEditor");
                    var editorValue = editor.getSession().getValue();
                    _this.initializeAceValidity(currentPreElement, editorValue);
                }
            });
        };
        scriptedIndexes.prototype.createScriptedIndex = function (indexName) {
            var _this = this;
            if (this.isFirstLoad()) {
                this.isFirstLoad(false);
            }
            var results = this.allScriptedIndexes().filter(function (index) { return index.indexName() == indexName; });
            var activeScriptedIndex = results[0];
            if (activeScriptedIndex) {
                activeScriptedIndex.cancelDeletion();
            }
            else {
                activeScriptedIndex = scriptedIndex.emptyForIndex(indexName);
                this.allScriptedIndexes.unshift(activeScriptedIndex);
            }
            this.activeScriptedIndexes.unshift(activeScriptedIndex);
            var twoCreatedPreElements = $('.in pre');
            var editor = ko.utils.domData.get(twoCreatedPreElements[0], "aceEditor");
            editor.focus();
            twoCreatedPreElements.each(function (index, preElement) {
                _this.initializeAceValidity(preElement, "");
            });
            this.initializeCollapsedInvalidElements();
        };
        scriptedIndexes.prototype.removeScriptedIndex = function (scriptedIndexToDelete) {
            this.activeScriptedIndexes.remove(scriptedIndexToDelete);
            scriptedIndexToDelete.markToDelete();
        };
        scriptedIndexes.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            new saveScriptedIndexesCommand(this.allScriptedIndexes(), db)
                .execute()
                .done(function (result) {
                _this.updateIndexes(result);
                _this.dirtyFlag().reset(); //Resync Changes
            });
        };
        scriptedIndexes.prototype.fetchAllIndexes = function (db) {
            var _this = this;
            return new getDatabaseStatsCommand(db)
                .execute()
                .done(function (results) { return _this.performAllIndexesResult(results); });
        };
        scriptedIndexes.prototype.performAllIndexesResult = function (results) {
            this.indexNames(results.Indexes.map(function (i) { return i.Name; }));
        };
        scriptedIndexes.prototype.fetchAllScriptedIndexes = function (db) {
            var _this = this;
            return new getScriptedIndexesCommand(db)
                .execute()
                .done(function (indexes) {
                _this.allScriptedIndexes.pushAll(indexes);
                _this.activeScriptedIndexes.pushAll(indexes);
            });
        };
        scriptedIndexes.prototype.addScriptsLabelPopover = function () {
            var indexScriptpopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span>(company == null) <span class="code-keyword">return</span>;<br/>company.Orders = { Count: <span class="code-keyword">this</span>.Count, Total: <span class="code-keyword">this</span>.Total };<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.index-script-label'
            };
            $('#accordion').popover(indexScriptpopOverSettings);
            var deleteScriptPopOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Index Scripts are written in JavaScript.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> company = LoadDocument(<span class="code-keyword">this</span>.Company);<br/><span class="code-keyword">if</span> (company == null) <span class="code-keyword">return</span>;<br/><span class="code-keyword">delete</span> company.Orders;<br/>PutDocument(<span class="code-keyword">this</span>.Company, company);</pre>',
                selector: '.delete-script-label'
            };
            $('#scriptedIndexesForm').popover(deleteScriptPopOverSettings);
        };
        //when pressing the save button, show all elements which are collapsed and at least one of its' fields isn't valid.
        scriptedIndexes.prototype.initializeCollapsedInvalidElements = function () {
            $('textarea').bind('invalid', function (e) {
                var element = e.target;
                if (!element.validity.valid) {
                    var parentElement = $(this).parents('.panel-default');
                    parentElement.children('.collapse').collapse('show');
                }
            });
        };
        scriptedIndexes.prototype.initializeAceValidity = function (element, editorValue) {
            if (editorValue === "") {
                var textarea = $(element).find('textarea')[0];
                textarea.setCustomValidity("Please fill out this field.");
            }
        };
        scriptedIndexes.prototype.updateIndexes = function (serverIndexes) {
            for (var i = 0; i < this.allScriptedIndexes().length; i++) {
                var index = this.allScriptedIndexes()[i];
                var serverIndex = serverIndexes.first(function (k) { return k.Key === index.getId(); });
                if (serverIndex && !serverIndex.Deleted) {
                    index.__metadata.etag = serverIndex.Etag;
                    index.__metadata.lastModified = serverIndex.Metadata['Last-Modified'];
                }
                else if (serverIndex && serverIndex.Deleted) {
                    this.allScriptedIndexes().splice(i--, 1);
                }
            }
        };
        return scriptedIndexes;
    })(viewModelBase);
    return scriptedIndexes;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/patch/executePatchConfirm", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var executePatchConfirm = (function (_super) {
        __extends(executePatchConfirm, _super);
        function executePatchConfirm() {
            _super.apply(this, arguments);
            this.viewTask = $.Deferred();
            this.wasConfirmed = false;
        }
        executePatchConfirm.prototype.executePatch = function () {
            this.viewTask.resolve();
            this.wasConfirmed = true;
            dialog.close(this);
        };
        executePatchConfirm.prototype.cancel = function () {
            this.viewTask.reject();
            this.wasConfirmed = false;
            dialog.close(this);
        };
        executePatchConfirm.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (!this.wasConfirmed) {
                this.viewTask.reject();
            }
        };
        return executePatchConfirm;
    })(dialogViewModelBase);
    return executePatchConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/patch/patch", ["require", "exports", "durandal/app", "viewmodels/viewModelBase", "models/database/patch/patchDocument", "common/bindingHelpers/aceEditorBindingHandler", "commands/resources/getDatabaseStatsCommand", "commands/database/documents/getCollectionsCommand", "models/database/documents/collection", "models/database/documents/document", "common/pagedList", "common/jsonUtil", "common/appUrl", "commands/database/query/queryIndexCommand", "commands/database/documents/getDocumentWithMetadataCommand", 'viewmodels/database/patch/savePatch', 'viewmodels/database/patch/executePatchConfirm', 'commands/database/patch/savePatchCommand', "commands/database/patch/executePatchCommand", "commands/database/patch/evalByQueryCommand", "models/database/documents/documentMetadata", "commands/database/documents/getDocumentsByEntityNameCommand", "commands/database/index/getIndexDefinitionCommand", "common/queryUtil", "common/recentPatchesStorage", 'commands/database/patch/getPatchesCommand'], function (require, exports, app, viewModelBase, patchDocument, aceEditorBindingHandler, getDatabaseStatsCommand, getCollectionsCommand, collection, document, pagedList, jsonUtil, appUrl, queryIndexCommand, getDocumentWithMetadataCommand, savePatch, executePatchConfirm, savePatchCommand, executePatchCommand, evalByQueryCommand, documentMetadata, getDocumentsByEntityNameCommand, getIndexDefinitionCommand, queryUtil, recentPatchesStorage, getPatchesCommand) {
    var patch = (function (_super) {
        __extends(patch, _super);
        function patch() {
            var _this = this;
            _super.call(this);
            this.displayName = "patch";
            this.indices = ko.observableArray([]);
            this.collections = ko.observableArray([]);
            this.recentPatches = ko.observableArray();
            this.savedPatches = ko.observableArray();
            this.currentCollectionPagedItems = ko.observable();
            this.selectedDocumentIndices = ko.observableArray();
            this.patchDocument = ko.observable();
            this.beforePatchDoc = ko.observable();
            this.beforePatchMeta = ko.observable();
            this.beforePatchDocMode = ko.observable(true);
            this.afterPatch = ko.observable();
            this.afterPatchDoc = ko.observable();
            this.afterPatchMeta = ko.observable();
            this.afterPatchDocMode = ko.observable(true);
            this.loadedDocuments = ko.observableArray();
            this.putDocuments = ko.observableArray();
            this.outputLog = ko.observableArray();
            this.documentKey = ko.observable();
            this.isPatchingInProgress = ko.observable(false);
            this.showPatchingProgress = ko.observable(false);
            this.patchingProgress = ko.observable(0);
            this.patchingProgressText = ko.observable();
            this.indexFields = ko.observableArray();
            this.selectedIndex = ko.observable();
            this.dynamicPrefix = "dynamic/";
            this.isTestIndex = ko.observable(false);
            this.queryText = ko.observable("");
            aceEditorBindingHandler.install();
            // When we programmatically change the document text or meta text, push it into the editor.
            this.beforePatchDocMode.subscribe(function () {
                if (_this.beforePatchEditor) {
                    var text = _this.beforePatchDocMode() ? _this.beforePatchDoc() : _this.beforePatchMeta();
                    _this.beforePatchEditor.getSession().setValue(text);
                }
            });
            this.beforePatch = ko.computed({
                read: function () {
                    return _this.beforePatchDocMode() ? _this.beforePatchDoc() : _this.beforePatchMeta();
                },
                write: function (text) {
                    var currentObservable = _this.beforePatchDocMode() ? _this.beforePatchDoc : _this.beforePatchMeta;
                    currentObservable(text);
                },
                owner: this
            });
            this.afterPatchDocMode.subscribe(function () {
                if (_this.afterPatchEditor) {
                    var text = _this.afterPatchDocMode() ? _this.afterPatchDoc() : _this.afterPatchMeta();
                    _this.afterPatchEditor.getSession().setValue(text);
                }
            });
            this.afterPatch = ko.computed({
                read: function () {
                    return _this.afterPatchDocMode() ? _this.afterPatchDoc() : _this.afterPatchMeta();
                },
                write: function (text) {
                    var currentObservable = _this.afterPatchDocMode() ? _this.afterPatchDoc : _this.afterPatchMeta;
                    currentObservable(text);
                },
                owner: this
            });
            // Refetch the index fields whenever the selected index name changes.
            this.selectedIndex
                .where(function (indexName) { return indexName != null; })
                .subscribe(function (indexName) { return _this.fetchIndexFields(indexName); });
            this.indicesToSelect = ko.computed(function () {
                var indicies = _this.indices();
                var patchDocument = _this.patchDocument();
                if (indicies.length === 0 || !patchDocument)
                    return [];
                return indicies.filter(function (x) { return x.name !== patchDocument.selectedItem(); });
            });
            this.collectionToSelect = ko.computed(function () {
                var collections = _this.collections();
                var patchDocument = _this.patchDocument();
                if (collections.length === 0 || !patchDocument)
                    return [];
                return collections.filter(function (x) { return x.name !== patchDocument.selectedItem(); });
            });
            this.showDocumentsPreview = ko.computed(function () {
                if (!_this.patchDocument()) {
                    return false;
                }
                var indexPath = _this.patchDocument().isIndexPatch();
                var collectionPath = _this.patchDocument().isCollectionPatch();
                return indexPath || collectionPath;
            });
        }
        patch.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            var beforePatchEditorElement = $("#beforePatchEditor");
            if (beforePatchEditorElement.length > 0) {
                this.beforePatchEditor = ko.utils.domData.get(beforePatchEditorElement[0], "aceEditor");
            }
            var afterPatchEditorElement = $("#afterPatchEditor");
            if (afterPatchEditorElement.length > 0) {
                this.afterPatchEditor = ko.utils.domData.get(afterPatchEditorElement[0], "aceEditor");
            }
        };
        patch.prototype.activate = function (recentPatchHash) {
            var _this = this;
            _super.prototype.activate.call(this, recentPatchHash);
            this.updateHelpLink("QGGJR5");
            this.patchDocument(patchDocument.empty());
            this.queryText.throttle(1000).subscribe(function () {
                _this.runQuery();
            });
            this.isExecuteAllowed = ko.computed(function () { return !!_this.patchDocument().script() && !!_this.beforePatchDoc(); });
            this.isMapReduceIndexSelected = ko.computed(function () {
                if (_this.patchDocument().patchOnOption() !== "Index") {
                    return false;
                }
                var indexName = _this.selectedIndex();
                var usedIndex = _this.indices().first(function (x) { return x.name === indexName; });
                if (usedIndex) {
                    return usedIndex.isMapReduce;
                }
                return false;
            });
            this.keyOfTestedDocument = ko.computed(function () {
                switch (_this.patchDocument().patchOnOption()) {
                    case "Collection":
                    case "Index":
                        return _this.documentKey();
                    case "Document":
                        return _this.patchDocument().selectedItem();
                }
            });
            this.selectedDocumentIndices.subscribe(function (list) {
                var firstCheckedOnList = list.last();
                if (firstCheckedOnList != null) {
                    _this.currentCollectionPagedItems().getNthItem(firstCheckedOnList)
                        .done(function (document) {
                        // load document directly from server as documents on list are loaded using doc-preview endpoint, which doesn't display entire document
                        _this.loadDocumentToTest(document.__metadata.id);
                        _this.documentKey(document.__metadata.id);
                    });
                }
                else {
                    _this.clearDocumentPreview();
                }
            });
            this.patchingProgressPercentage = ko.computed(function () { return _this.patchingProgress() + "%"; });
            var db = this.activeDatabase();
            if (!!db) {
                this.fetchRecentPatches();
                this.fetchAllPatches();
            }
            if (recentPatchHash) {
                this.selectInitialPatch(recentPatchHash);
            }
        };
        patch.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#indexQueryLabel").popover({
                html: true,
                trigger: "hover",
                container: '.form-horizontal',
                content: '<p>Queries use Lucene syntax. Examples:</p><pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>'
            });
            $("#patchScriptsLabel").popover({
                html: true,
                trigger: "hover",
                container: ".form-horizontal",
                content: '<p>Patch Scripts are written in JavaScript. Examples:</p><pre><span class="code-keyword">this</span>.NewProperty = <span class="code-keyword">this</span>.OldProperty + myParameter;<br/><span class="code-keyword">delete this</span>.UnwantedProperty;<br/><span class="code-keyword">this</span>.Comments.RemoveWhere(<span class="code-keyword">function</span>(comment){<br/>  <span class="code-keyword">return</span> comment.Spam;<br/>});</pre>'
            });
            var rowCreatedEvent = app.on(patch.gridSelector + 'RowsCreated').then(function () {
                rowCreatedEvent.off();
            });
            var self = this;
            $(window).bind('storage', function () {
                self.fetchRecentPatches();
            });
        };
        patch.prototype.fetchAllPatches = function () {
            var _this = this;
            new getPatchesCommand(this.activeDatabase())
                .execute()
                .done(function (patches) { return _this.savedPatches(patches); });
        };
        patch.prototype.fetchRecentPatches = function () {
            this.recentPatches(recentPatchesStorage.getRecentPatches(this.activeDatabase()));
        };
        patch.prototype.detached = function () {
            _super.prototype.detached.call(this);
            aceEditorBindingHandler.detached();
        };
        patch.prototype.selectInitialPatch = function (recentPatchHash) {
            if (recentPatchHash.indexOf("recentpatch-") === 0) {
                var hash = parseInt(recentPatchHash.substr("recentpatch-".length), 10);
                var matchingPatch = this.recentPatches.first(function (q) { return q.Hash === hash; });
                if (matchingPatch) {
                    this.useRecentPatch(matchingPatch);
                }
                else {
                    this.navigate(appUrl.forPatch(this.activeDatabase()));
                }
            }
        };
        patch.prototype.loadDocumentToTest = function (selectedItem) {
            var _this = this;
            if (selectedItem) {
                var loadDocTask = new getDocumentWithMetadataCommand(selectedItem, this.activeDatabase()).execute();
                loadDocTask.done(function (document) {
                    _this.beforePatchDoc(JSON.stringify(document.toDto(), null, 4));
                    _this.beforePatchMeta(JSON.stringify(documentMetadata.filterMetadata(document.__metadata.toDto()), null, 4));
                }).fail(this.clearDocumentPreview());
            }
            else {
                this.clearDocumentPreview();
            }
        };
        patch.prototype.clearDocumentPreview = function () {
            this.beforePatchDoc("");
            this.beforePatchMeta("");
            this.afterPatchDoc("");
            this.afterPatchMeta("");
            this.putDocuments([]);
            this.loadedDocuments([]);
            this.outputLog([]);
        };
        patch.prototype.setSelectedPatchOnOption = function (patchOnOption) {
            var _this = this;
            this.resetProgressBar();
            this.patchDocument().patchOnOption(patchOnOption);
            this.patchDocument().selectedItem('');
            this.clearDocumentPreview();
            switch (patchOnOption) {
                case "Collection":
                    this.fetchAllCollections();
                    break;
                case "Index":
                    this.fetchAllIndexes()
                        .done(function () { return _this.runQuery(); });
                    $("#matchingDocumentsGrid").resize();
                    break;
                default:
                    this.currentCollectionPagedItems(null);
                    break;
            }
        };
        patch.prototype.fetchAllCollections = function () {
            var _this = this;
            return new getCollectionsCommand(this.activeDatabase())
                .execute()
                .always(function () { return NProgress.done(); })
                .done(function (colls) {
                var currentlySelectedCollection = null;
                if (_this.patchDocument().selectedItem()) {
                    var selected = _this.patchDocument().selectedItem();
                    currentlySelectedCollection = _this.collections.first(function (c) { return c.name === selected; });
                }
                _this.collections(colls);
                if (_this.collections().length > 0) {
                    _this.setSelectedCollection(currentlySelectedCollection || _this.collections().first());
                }
            });
        };
        patch.prototype.setSelectedCollection = function (coll) {
            this.resetProgressBar();
            this.patchDocument().selectedItem(coll.name);
            var list = coll.getDocuments();
            this.currentCollectionPagedItems(list);
            list.fetch(0, 20).always(function () { return $("#matchingDocumentsGrid").resize(); });
        };
        patch.prototype.fetchAllIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (results) {
                _this.indices(results.Indexes.map(function (i) {
                    return {
                        name: i.Name,
                        isMapReduce: i.IsMapReduce
                    };
                }));
                if (_this.indices().length > 0) {
                    _this.setSelectedIndex(_this.indices().first().name);
                }
            });
        };
        patch.prototype.setSelectedIndex = function (indexName) {
            this.resetProgressBar();
            this.selectedIndex(indexName);
            this.patchDocument().selectedItem(indexName);
        };
        patch.prototype.useIndex = function (indexName) {
            this.setSelectedIndex(indexName);
            this.runQuery();
        };
        patch.prototype.runQuery = function () {
            var selectedIndex = this.patchDocument().selectedItem();
            if (selectedIndex) {
                var queryText = this.queryText();
                this.patchDocument().query(queryText);
                var database = this.activeDatabase();
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(selectedIndex, database, skip, take, queryText, []);
                    return command.execute()
                        .fail(function () { return recentPatchesStorage.removeIndexFromRecentPatches(database, selectedIndex); });
                };
                var resultsList = new pagedList(resultsFetcher);
                this.currentCollectionPagedItems(resultsList);
                return resultsList;
            }
            return null;
        };
        patch.prototype.savePatch = function () {
            var _this = this;
            var savePatchViewModel = new savePatch();
            app.showDialog(savePatchViewModel);
            savePatchViewModel.onExit().done(function (patchName) {
                new savePatchCommand(patchName, _this.patchDocument(), _this.activeDatabase())
                    .execute()
                    .done(function () { return _this.fetchAllPatches(); });
            });
        };
        patch.prototype.usePatch = function (patch) {
            var _this = this;
            this.clearDocumentPreview();
            var selectedItem = patch.selectedItem();
            patch = patch.clone();
            patch.resetMetadata();
            this.patchDocument(patch);
            switch (this.patchDocument().patchOnOption()) {
                case "Collection":
                    this.fetchAllCollections().then(function () {
                        _this.setSelectedCollection(_this.collections().filter(function (coll) { return (coll.name === selectedItem); }).first());
                    });
                    break;
                case "Index":
                    this.fetchAllIndexes().then(function () {
                        _this.setSelectedIndex(selectedItem);
                        _this.queryText(patch.query());
                        _this.runQuery();
                    });
                    break;
                case "Document":
                    this.loadDocumentToTest(patch.selectedItem());
                    break;
            }
        };
        patch.prototype.testPatch = function () {
            var _this = this;
            var values = {};
            this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var bulkDocs = [];
            bulkDocs.push({
                Key: this.keyOfTestedDocument(),
                Method: 'EVAL',
                DebugMode: true,
                Patch: {
                    Script: this.patchDocument().script(),
                    Values: values
                }
            });
            new executePatchCommand(bulkDocs, this.activeDatabase(), true)
                .execute()
                .done(function (result) {
                var testResult = new document(result[0].AdditionalData['Document']);
                _this.afterPatchDoc(JSON.stringify(testResult.toDto(), null, 4));
                _this.afterPatchMeta(JSON.stringify(documentMetadata.filterMetadata(testResult.__metadata.toDto()), null, 4));
                _this.updateActions(result[0].AdditionalData['Actions']);
                _this.outputLog(result[0].AdditionalData["Debug"]);
            })
                .fail(function (result) { return console.log(result.responseText); });
            this.recordPatchRun();
        };
        patch.prototype.updatePageUrl = function (hash) {
            // Put the patch into the URL, so that if the user refreshes the page, he's still got this patch loaded.
            var queryUrl = appUrl.forPatch(this.activeDatabase(), hash);
            this.updateUrl(queryUrl);
        };
        patch.prototype.recordPatchRun = function () {
            var patchDocument = this.patchDocument();
            var newPatch = patchDocument.toDto();
            delete newPatch["@metadata"];
            newPatch.Hash = 0;
            var stringForHash = newPatch.PatchOnOption + newPatch.SelectedItem + newPatch.Script + newPatch.Values;
            if (patchDocument.patchOnOption() === "Index") {
                newPatch.Query = this.queryText();
                stringForHash += newPatch.Query;
            }
            newPatch.Hash = stringForHash.hashCode();
            this.updatePageUrl(newPatch.Hash);
            // Add this query to our recent patches list in the UI, or move it to the top of the list if it's already there.
            var existing = this.recentPatches.first(function (q) { return q.Hash === newPatch.Hash; });
            if (existing) {
                this.recentPatches.remove(existing);
                this.recentPatches.unshift(existing);
            }
            else {
                this.recentPatches.unshift(newPatch);
            }
            // Limit us to 15 recent patchs
            if (this.recentPatches().length > 15) {
                this.recentPatches.remove(this.recentPatches()[15]);
            }
            //save the recent queries to local storage
            recentPatchesStorage.saveRecentPatches(this.activeDatabase(), this.recentPatches());
        };
        patch.prototype.useRecentPatch = function (patchToUse) {
            var patchDoc = new patchDocument(patchToUse);
            this.usePatch(patchDoc);
        };
        patch.prototype.updateActions = function (actions) {
            this.loadedDocuments(actions.LoadDocument || []);
            this.putDocuments((actions.PutDocument || []).map(function (doc) { return jsonUtil.syntaxHighlight(doc); }));
        };
        patch.prototype.executePatchOnSingle = function () {
            var keys = [];
            keys.push(this.patchDocument().selectedItem());
            this.confirmAndExecutePatch(keys);
        };
        patch.prototype.executePatchOnSelected = function () {
            this.confirmAndExecutePatch(this.getDocumentsGrid().getSelectedItems().map(function (doc) { return doc.__metadata.id; }));
        };
        patch.prototype.executePatchOnAll = function () {
            var _this = this;
            var confirmExec = new executePatchConfirm();
            confirmExec.viewTask.done(function () { return _this.executePatchByIndex(); });
            app.showDialog(confirmExec);
        };
        patch.prototype.executePatchByIndex = function () {
            var _this = this;
            var index = null;
            var query = null;
            switch (this.patchDocument().patchOnOption()) {
                case "Collection":
                    index = "Raven/DocumentsByEntityName";
                    query = "Tag:" + queryUtil.escapeTerm(this.patchDocument().selectedItem());
                    break;
                case "Index":
                    index = this.patchDocument().selectedItem();
                    query = this.patchDocument().query();
                    break;
            }
            var values = {};
            this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var patch = {
                Script: this.patchDocument().script(),
                Values: values
            };
            var patchByQueryCommand = new evalByQueryCommand(index, query, JSON.stringify(patch), this.activeDatabase(), function (status) { return _this.updateProgress(status); });
            patchByQueryCommand.execute()
                .done(function () {
                _this.resetProgressBar();
                _this.isPatchingInProgress(true);
                _this.showPatchingProgress(true);
            });
            this.recordPatchRun();
        };
        patch.prototype.resetProgressBar = function () {
            this.showPatchingProgress(false);
            this.patchingProgress(0);
            this.patchingProgressText("");
        };
        patch.prototype.updateProgress = function (status) {
            var _this = this;
            if (status.OperationProgress != null) {
                var progressValue = Math.round(100 * (status.OperationProgress.ProcessedEntries / status.OperationProgress.TotalEntries));
                this.patchingProgress(progressValue);
                this.patchingProgressText(status.OperationProgress.ProcessedEntries.toLocaleString() + " / " + status.OperationProgress.TotalEntries.toLocaleString() + " (" + progressValue + "%)");
            }
            if (status.Completed) {
                this.isPatchingInProgress(false);
                setTimeout(function () { return _this.showPatchingProgress(false); }, 2000);
            }
        };
        patch.prototype.confirmAndExecutePatch = function (keys) {
            var _this = this;
            var confirmExec = new executePatchConfirm();
            confirmExec.viewTask.done(function () { return _this.executePatch(keys); });
            app.showDialog(confirmExec);
        };
        patch.prototype.executePatch = function (keys) {
            var _this = this;
            var values = {};
            var patchDtos = this.patchDocument().parameters().map(function (param) {
                var dto = param.toDto();
                values[dto.Key] = dto.Value;
            });
            var bulkDocs = [];
            keys.forEach(function (key) { return bulkDocs.push({
                Key: key,
                Method: 'EVAL',
                DebugMode: false,
                Patch: {
                    Script: _this.patchDocument().script(),
                    Values: values
                }
            }); });
            new executePatchCommand(bulkDocs, this.activeDatabase(), false)
                .execute()
                .done(function (result) {
                _this.afterPatchDoc("");
                _this.afterPatchMeta("");
                if (_this.patchDocument().patchOnOption() === 'Document') {
                    _this.loadDocumentToTest(_this.patchDocument().selectedItem());
                }
                _this.updateDocumentsList();
            })
                .fail(function (result) { return console.log(result.responseText); });
            this.recordPatchRun();
        };
        patch.prototype.updateDocumentsList = function () {
            var _this = this;
            switch (this.patchDocument().patchOnOption()) {
                case "Collection":
                    this.fetchAllCollections().then(function () {
                        _this.setSelectedCollection(_this.collections().filter(function (coll) { return (coll.name === _this.patchDocument().selectedItem()); }).first());
                    });
                    break;
                case "Index":
                    this.useIndex(this.patchDocument().selectedItem());
                    break;
            }
        };
        patch.prototype.getDocumentsGrid = function () {
            var gridContents = $(patch.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        patch.prototype.activateBeforeDoc = function () {
            this.beforePatchDocMode(true);
        };
        patch.prototype.activateBeforeMeta = function () {
            this.beforePatchDocMode(false);
        };
        patch.prototype.activateAfterDoc = function () {
            this.afterPatchDocMode(true);
        };
        patch.prototype.activateAfterMeta = function () {
            this.afterPatchDocMode(false);
        };
        patch.prototype.queryCompleter = function (editor, session, pos, prefix, callback) {
            queryUtil.queryCompleter(this.indexFields, this.selectedIndex, this.dynamicPrefix, this.activeDatabase, editor, session, pos, prefix, callback);
        };
        patch.prototype.fetchIndexFields = function (indexName) {
            var _this = this;
            // Fetch the index definition so that we get an updated list of fields to be used as sort by options.
            // Fields don't show for All Documents.
            var self = this;
            var isAllDocumentsDynamicQuery = indexName === "All Documents";
            if (!isAllDocumentsDynamicQuery) {
                //if index is dynamic, get columns using index definition, else get it using first index result
                if (indexName.indexOf(this.dynamicPrefix) === 0) {
                    var collectionName = indexName.substring(8);
                    new getDocumentsByEntityNameCommand(new collection(collectionName, this.activeDatabase()), 0, 1)
                        .execute()
                        .done(function (result) {
                        if (!!result && result.totalResultCount > 0 && result.items.length > 0) {
                            var dynamicIndexPattern = new document(result.items[0]);
                            if (!!dynamicIndexPattern) {
                                _this.indexFields(dynamicIndexPattern.getDocumentPropertyNames());
                            }
                        }
                    });
                }
                else {
                    new getIndexDefinitionCommand(indexName, this.activeDatabase())
                        .execute()
                        .done(function (result) {
                        self.isTestIndex(result.Index.IsTestIndex);
                        self.indexFields(result.Index.Fields);
                    })
                        .fail(function () {
                        recentPatchesStorage.removeIndexFromRecentPatches(_this.activeDatabase(), indexName);
                    });
                }
            }
        };
        patch.gridSelector = "#matchingDocumentsGrid";
        return patch;
    })(viewModelBase);
    return patch;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/patch/savePatch", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var savePatch = (function (_super) {
        __extends(savePatch, _super);
        function savePatch() {
            _super.apply(this, arguments);
            this.nextTask = $.Deferred();
            this.nextTaskStarted = false;
            this.patchName = ko.observable();
        }
        savePatch.prototype.cancel = function () {
            dialog.close(this);
        };
        savePatch.prototype.saveThePatch = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.patchName());
            dialog.close(this);
        };
        savePatch.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        savePatch.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        return savePatch;
    })(dialogViewModelBase);
    return savePatch;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/query/deleteDocumentsMatchingQueryConfirm", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/database/documents/deleteDocsMatchingQueryCommand"], function (require, exports, dialog, dialogViewModelBase, deleteDocsMatchingQueryCommand) {
    var deleteDocumentsMatchingQueryConfirm = (function (_super) {
        __extends(deleteDocumentsMatchingQueryConfirm, _super);
        function deleteDocumentsMatchingQueryConfirm(indexName, queryText, totalDocCount, db) {
            _super.call(this);
            this.indexName = indexName;
            this.queryText = queryText;
            this.totalDocCount = totalDocCount;
            this.db = db;
        }
        deleteDocumentsMatchingQueryConfirm.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteDocumentsMatchingQueryConfirm.prototype.deleteDocs = function () {
            new deleteDocsMatchingQueryCommand(this.indexName, this.queryText, this.db).execute();
            dialog.close(this);
        };
        return deleteDocumentsMatchingQueryConfirm;
    })(dialogViewModelBase);
    return deleteDocumentsMatchingQueryConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/query/query", ["require", "exports", "viewmodels/filesystem/search/inMethodFilter", "viewmodels/filesystem/search/fieldStringFilter", "viewmodels/filesystem/search/fieldRangeFilter", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "commands/database/documents/getCollectionsCommand", "commands/database/index/getIndexDefinitionCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/pagedList", "commands/database/query/queryIndexCommand", "models/database/query/querySort", "models/database/documents/collection", "commands/database/transformers/getTransformersCommand", "commands/database/globalConfig/getEffectiveCustomFunctionsCommand", "viewmodels/database/query/deleteDocumentsMatchingQueryConfirm", "models/database/documents/document", "models/database/documents/customColumnParams", "models/database/documents/customColumns", "viewmodels/common/selectColumns", "commands/database/documents/getCustomColumnsCommand", "commands/database/documents/getDocumentsByEntityNameCommand", "viewmodels/database/query/queryStatsDialog", "models/database/documents/customFunctions", "models/database/index/transformer", "models/database/index/transformerQuery", "commands/database/index/getIndexSuggestionsCommand", "common/recentQueriesStorage", "common/queryUtil"], function (require, exports, InMethodFilter, fieldStringFilter, fieldRangeFilter, app, router, appUrl, viewModelBase, getDatabaseStatsCommand, getCollectionsCommand, getIndexDefinitionCommand, aceEditorBindingHandler, pagedList, queryIndexCommand, querySort, collection, getTransformersCommand, getEffectiveCustomFunctionsCommand, deleteDocumentsMatchingQueryConfirm, document, customColumnParams, customColumns, selectColumns, getCustomColumnsCommand, getDocumentsByEntityNameCommand, queryStatsDialog, customFunctions, transformerType, transformerQueryType, getIndexSuggestionsCommand, recentQueriesStorage, queryUtil) {
    var query = (function (_super) {
        __extends(query, _super);
        function query() {
            var _this = this;
            _super.call(this);
            this.isTestIndex = ko.observable(false);
            this.selectedIndex = ko.observable();
            this.indexes = ko.observableArray();
            this.queryText = ko.observable("");
            this.queryResults = ko.observable();
            this.selectedResultIndices = ko.observableArray();
            this.queryStats = ko.observable();
            this.sortBys = ko.observableArray();
            this.indexFields = ko.observableArray();
            this.transformer = ko.observable();
            this.allTransformers = ko.observableArray();
            this.isDefaultOperatorOr = ko.observable(true);
            this.showFields = ko.observable(false);
            this.indexEntries = ko.observable(false);
            this.recentQueries = ko.observableArray();
            this.rawJsonUrl = ko.observable();
            this.collections = ko.observableArray([]);
            this.collectionNames = ko.observableArray();
            this.isLoading = ko.observable(false);
            this.isCacheDisable = ko.observable(false);
            this.warningText = ko.observable();
            this.isWarning = ko.observable(false);
            this.searchField = ko.observable("Select field");
            this.contextName = ko.observable();
            this.dynamicPrefix = "dynamic/";
            this.currentColumnsParams = ko.observable(customColumns.empty());
            this.currentCustomFunctions = ko.observable(customFunctions.empty());
            this.indexSuggestions = ko.observableArray([]);
            this.csvUrl = ko.observable();
            this.appUrls = appUrl.forCurrentDatabase();
            this.editIndexUrl = ko.computed(function () { return _this.selectedIndex() ? appUrl.forEditIndex(_this.selectedIndex(), _this.activeDatabase()) : null; });
            this.visualizerUrl = ko.computed(function () { return _this.selectedIndex() ? appUrl.forVisualizer(_this.activeDatabase(), _this.selectedIndex()) : null; });
            this.indexPerfStatsUrl = ko.computed(function () { return _this.selectedIndex() ? appUrl.forIndexPerformance(_this.activeDatabase()) : null; });
            this.termsUrl = ko.computed(function () { return _this.selectedIndex() ? appUrl.forTerms(_this.selectedIndex(), _this.activeDatabase()) : null; });
            this.statsUrl = ko.computed(function () { return appUrl.forStatus(_this.activeDatabase()); });
            this.hasSelectedIndex = ko.computed(function () { return _this.selectedIndex() != null; });
            this.hasEditableIndex = ko.computed(function () { return _this.selectedIndex() != null && _this.selectedIndex().indexOf("dynamic/") !== 0; });
            this.rawJsonUrl.subscribe(function (value) { return ko.postbox.publish("SetRawJSONUrl", value); });
            this.selectedIndexLabel = ko.computed(function () { return _this.selectedIndex() === "dynamic" ? "All Documents" : _this.selectedIndex(); });
            this.containsAsterixQuery = ko.computed(function () { return _this.queryText().contains("*.*"); });
            this.isStaticIndexSelected = ko.computed(function () { return _this.selectedIndex() == null || _this.selectedIndex().indexOf(_this.dynamicPrefix) == -1; });
            this.selectedIndexEditUrl = ko.computed(function () {
                if (_this.queryStats()) {
                    var index = _this.queryStats().IndexName;
                    if (!!index && index.indexOf(_this.dynamicPrefix) !== 0) {
                        return appUrl.forEditIndex(index, _this.activeDatabase());
                    }
                }
                return "";
            });
            this.indexesExceptCurrent = ko.computed(function () {
                var allIndexes = _this.indexes();
                var selectedIndex = _this.selectedIndex();
                var allIndexesCopy;
                if (!!selectedIndex && selectedIndex.indexOf(_this.dynamicPrefix) === -1) {
                    allIndexesCopy = allIndexes.filter(function (indexDto) { return indexDto.name !== selectedIndex; });
                }
                else {
                    allIndexesCopy = allIndexes.slice(0); // make copy as sort works in situ
                }
                allIndexesCopy.sort(function (l, r) { return l.name.toLowerCase() > r.name.toLowerCase() ? 1 : -1; });
                return allIndexesCopy;
            });
            this.collectionNamesExceptCurrent = ko.computed(function () {
                var allCollectionNames = _this.collectionNames();
                var selectedIndex = _this.selectedIndex();
                if (!!selectedIndex && selectedIndex.indexOf(_this.dynamicPrefix) == 0) {
                    return allCollectionNames.filter(function (collectionName) { return _this.dynamicPrefix + collectionName != selectedIndex; });
                }
                return allCollectionNames;
            });
            this.showSuggestions = ko.computed(function () {
                return _this.indexSuggestions().length > 0;
            });
            this.didDynamicChangeIndex = ko.computed(function () {
                if (_this.queryStats()) {
                    var recievedIndex = _this.queryStats().IndexName;
                    var selectedIndex = _this.selectedIndex();
                    return selectedIndex.indexOf(_this.dynamicPrefix) === 0 && _this.indexes()[0].name !== recievedIndex;
                }
                else {
                    return false;
                }
            });
            this.isIndexMapReduce = ko.computed(function () {
                var currentIndex = _this.indexes.first(function (i) { return i.name == _this.selectedIndex(); });
                return !!currentIndex && currentIndex.hasReduce;
            });
            this.isDynamicIndex = ko.computed(function () {
                var currentIndex = _this.indexes.first(function (i) { return i.name == _this.selectedIndex(); });
                if (currentIndex) {
                    console.log(currentIndex.name);
                }
                return !!currentIndex && currentIndex.name.startsWith("Auto/");
            });
            this.enableDeleteButton = ko.computed(function () {
                var currentIndex = _this.indexes.first(function (i) { return i.name == _this.selectedIndex(); });
                var isMapReduce = _this.isIndexMapReduce();
                var isDynamic = _this.isDynamicIndex();
                return !!currentIndex && !isMapReduce && !isDynamic;
            });
            aceEditorBindingHandler.install();
            // Refetch the index fields whenever the selected index name changes.
            this.selectedIndex
                .where(function (indexName) { return indexName != null; })
                .subscribe(function (indexName) { return _this.fetchIndexFields(indexName); });
        }
        query.prototype.openQueryStats = function () {
            var viewModel = new queryStatsDialog(this.queryStats(), this.selectedIndexEditUrl(), this.didDynamicChangeIndex(), this.rawJsonUrl());
            app.showDialog(viewModel);
        };
        query.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (!!db) {
                this.fetchRecentQueries();
                $.when(this.fetchCustomFunctions(db), this.fetchAllTransformers(db))
                    .done(function () { return deferred.resolve({ can: true }); });
            }
            else {
                deferred.resolve({ redirect: "#resources" });
            }
            return deferred;
        };
        query.prototype.activate = function (indexNameOrRecentQueryHash) {
            var _this = this;
            _super.prototype.activate.call(this, indexNameOrRecentQueryHash);
            this.updateHelpLink('KCIMJK');
            this.selectedIndex.subscribe(function (index) { return _this.onIndexChanged(index); });
            var db = this.activeDatabase();
            return $.when(this.fetchAllCollections(db), this.fetchAllIndexes(db))
                .done(function () { return _this.selectInitialQuery(indexNameOrRecentQueryHash); });
        };
        query.prototype.detached = function () {
            _super.prototype.detached.call(this);
            aceEditorBindingHandler.detached();
        };
        query.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.createKeyboardShortcut("F2", function () { return _this.editSelectedIndex(); }, query.containerSelector);
            this.createKeyboardShortcut("ctrl+enter", function () { return _this.runQuery(); }, query.containerSelector);
            this.createKeyboardShortcut("alt+c", function () { return _this.focusOnQuery(); }, query.containerSelector);
            this.createKeyboardShortcut("alt+r", function () { return _this.runQuery(); }, query.containerSelector); // Using keyboard shortcut here, rather than HTML's accesskey, so that we don't steal focus from the editor.
            $("#indexQueryLabel").popover({
                html: true,
                trigger: "hover",
                container: ".form-horizontal",
                content: '<p>Queries use Lucene syntax. Examples:</p><pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>',
            });
            ko.postbox.publish("SetRawJSONUrl", appUrl.forIndexQueryRawData(this.activeDatabase(), this.selectedIndex()));
            this.focusOnQuery();
            var self = this;
            $(window).bind('storage', function () {
                self.fetchRecentQueries();
            });
            this.isLoading.extend({ rateLimit: 100 });
        };
        query.prototype.fetchRecentQueries = function () {
            this.recentQueries(recentQueriesStorage.getRecentQueries(this.activeDatabase()));
        };
        query.prototype.fetchCustomFunctions = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            var customFunctionsCommand = new getEffectiveCustomFunctionsCommand(db).execute();
            customFunctionsCommand.done(function (cf) {
                _this.currentCustomFunctions(new customFunctions(cf.MergedDocument));
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        query.prototype.fetchAllTransformers = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getTransformersCommand(db)
                .execute()
                .done(function (results) {
                _this.allTransformers(results);
                deferred.resolve();
            });
            return deferred;
        };
        query.prototype.fetchAllCollections = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getCollectionsCommand(db)
                .execute()
                .done(function (results) {
                _this.collections(results);
                _this.collectionNames(results.map(function (c) { return c.name; }));
                deferred.resolve();
            });
            return deferred;
        };
        query.prototype.fetchAllIndexes = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getDatabaseStatsCommand(db)
                .execute()
                .done(function (results) {
                _this.indexes(results.Indexes.map(function (i) {
                    return {
                        name: i.Name,
                        hasReduce: !!i.LastReducedTimestamp
                    };
                }));
                deferred.resolve();
            });
            return deferred;
        };
        query.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("EditItem", function (itemNumber) {
                    //(itemNumber: number, res: resource, index: string, query?: string, sort?:string)
                    var queriess = _this.recentQueries();
                    var recentq = _this.recentQueries()[0];
                    var sorts = recentq.Sorts
                        .join(',');
                    //alert(appUrl.forEditQueryItem(itemNumber, this.activeDatabase(), recentq.IndexName,recentq.QueryText,sorts));
                    router.navigate(appUrl.forEditQueryItem(itemNumber, _this.activeDatabase(), recentq.IndexName, recentq.QueryText, sorts), true);
                })
            ];
        };
        query.prototype.onIndexChanged = function (newIndexName) {
            var _this = this;
            var command = getCustomColumnsCommand.forIndex(newIndexName, this.activeDatabase());
            this.contextName(command.docName);
            command.execute().done(function (dto) {
                if (dto) {
                    _this.currentColumnsParams().columns($.map(dto.Columns, function (c) { return new customColumnParams(c); }));
                    _this.currentColumnsParams().customMode(true);
                }
                else {
                    // use default values!
                    _this.currentColumnsParams().columns.removeAll();
                    _this.currentColumnsParams().customMode(false);
                }
            });
        };
        query.prototype.selectInitialQuery = function (indexNameOrRecentQueryHash) {
            if (!indexNameOrRecentQueryHash && this.indexes().length > 0) {
                var firstIndexName = this.indexes.first().name;
                this.setSelectedIndex(firstIndexName);
            }
            else if (this.indexes.first(function (i) { return i.name == indexNameOrRecentQueryHash; }) || indexNameOrRecentQueryHash.indexOf(this.dynamicPrefix) === 0 || indexNameOrRecentQueryHash === "dynamic") {
                this.setSelectedIndex(indexNameOrRecentQueryHash);
            }
            else if (indexNameOrRecentQueryHash.indexOf("recentquery-") === 0) {
                var hash = parseInt(indexNameOrRecentQueryHash.substr("recentquery-".length), 10);
                var matchingQuery = this.recentQueries.first(function (q) { return q.Hash === hash; });
                if (matchingQuery) {
                    this.runRecentQuery(matchingQuery);
                }
                else {
                    this.navigate(appUrl.forQuery(this.activeDatabase()));
                }
            }
        };
        query.prototype.focusOnQuery = function () {
            var editorElement = $("#queryEditor").length == 1 ? $("#queryEditor")[0] : null;
            if (editorElement) {
                var docEditor = ko.utils.domData.get(editorElement, "aceEditor");
                if (docEditor) {
                    docEditor.focus();
                }
            }
        };
        query.prototype.editSelectedIndex = function () {
            this.navigate(this.editIndexUrl());
        };
        query.prototype.runRecentQuery = function (query) {
            this.selectedIndex(query.IndexName);
            this.queryText(query.QueryText);
            this.showFields(query.ShowFields);
            this.indexEntries(query.IndexEntries);
            this.isDefaultOperatorOr(query.UseAndOperator === false);
            this.sortBys(query.Sorts.map(function (s) { return querySort.fromQuerySortString(s); }));
            this.selectTransformer(this.findTransformerByName(this.getStoredQueryTransformerName(query)));
            this.applyTransformerParameters(query);
            this.runQuery();
        };
        query.prototype.toggleCacheEnable = function () {
            this.isCacheDisable(!this.isCacheDisable());
        };
        query.prototype.runQuery = function () {
            var _this = this;
            var selectedIndex = this.selectedIndex();
            if (selectedIndex) {
                this.isWarning(false);
                this.isLoading(true);
                this.focusOnQuery();
                var queryText = this.queryText();
                var sorts = this.sortBys().filter(function (s) { return s.fieldName() != null; });
                var database = this.activeDatabase();
                var showFields = this.showFields();
                var indexEntries = this.indexEntries();
                var transformer = null;
                if (this.transformer()) {
                    transformer = new transformerQueryType({
                        transformerName: this.transformer().name(),
                        queryParams: []
                    });
                    var canRunQuery = true;
                    $("#transformerParams .transformer_param_flag").each(function (index, element) {
                        $(element).parent().removeClass("has-error");
                        if (element.value) {
                            transformer.addParamByNameAndValue(element.name, element.value);
                        }
                        else if (element.required) {
                            canRunQuery = false;
                            $(element).parent().addClass("has-error");
                        }
                    });
                    if (!canRunQuery) {
                        return; // Cannot run query without required parameters
                    }
                }
                this.currentColumnsParams().enabled(this.showFields() === false && this.indexEntries() === false);
                var useAndOperator = this.isDefaultOperatorOr() === false;
                var queryCommand = new queryIndexCommand(selectedIndex, database, 0, 25, queryText, sorts, transformer, showFields, indexEntries, useAndOperator, this.isCacheDisable());
                this.rawJsonUrl(appUrl.forResourceQuery(database) + queryCommand.getUrl());
                this.csvUrl(queryCommand.getCsvUrl());
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(selectedIndex, database, skip, take, queryText, sorts, transformer, showFields, indexEntries, useAndOperator, _this.isCacheDisable());
                    return command.execute()
                        .always(function () {
                        _this.isLoading(false);
                        _this.focusOnQuery();
                    })
                        .done(function (queryResults) {
                        _this.queryStats(queryResults.additionalResultInfo);
                        _this.indexSuggestions([]);
                        if (queryResults.totalResultCount == 0) {
                            var queryFields = _this.extractQueryFields();
                            var alreadyLookedForNull = false;
                            if (_this.selectedIndex().indexOf(_this.dynamicPrefix) !== 0) {
                                alreadyLookedForNull = true;
                                for (var i = 0; i < queryFields.length; i++) {
                                    _this.getIndexSuggestions(selectedIndex, queryFields[i]);
                                    if (queryFields[i].FieldValue == 'null') {
                                        _this.isWarning(true);
                                        _this.warningText(("The Query contains '" + queryFields[i].FieldName + ": null', this will check if the field contains the string 'null', is this what you meant?"));
                                    }
                                }
                            }
                            if (!alreadyLookedForNull) {
                                for (var i = 0; i < queryFields.length; i++) {
                                    ;
                                    if (queryFields[i].FieldValue == 'null') {
                                        _this.isWarning(true);
                                        _this.warningText(("The Query contains '" + queryFields[i].FieldName + ": null', this will check if the field contains the string 'null', is this what you meant?"));
                                    }
                                }
                            }
                        }
                    })
                        .fail(function () {
                        recentQueriesStorage.removeIndexFromRecentQueries(database, selectedIndex);
                    });
                };
                var resultsList = new pagedList(resultsFetcher);
                this.queryResults(resultsList);
                this.recordQueryRun(selectedIndex, queryText, sorts.map(function (s) { return s.toQuerySortString(); }), transformer, showFields, indexEntries, useAndOperator);
                return resultsList;
            }
            return null;
        };
        query.prototype.queryCompleter = function (editor, session, pos, prefix, callback) {
            queryUtil.queryCompleter(this.indexFields, this.selectedIndex, this.dynamicPrefix, this.activeDatabase, editor, session, pos, prefix, callback);
        };
        query.prototype.recordQueryRun = function (indexName, queryText, sorts, transformerQuery, showFields, indexEntries, useAndOperator) {
            var newQuery = {
                IndexEntries: indexEntries,
                IndexName: indexName,
                IsPinned: false,
                QueryText: queryText,
                ShowFields: showFields,
                Sorts: sorts,
                TransformerQuery: transformerQuery,
                UseAndOperator: useAndOperator,
                Hash: (indexName + (queryText || "") +
                    sorts.reduce(function (a, b) { return a + b; }, "") +
                    (transformerQuery ? transformerQuery.toUrl() : "") +
                    showFields +
                    indexEntries +
                    useAndOperator).hashCode()
            };
            // Put the query into the URL, so that if the user refreshes the page, he's still got this query loaded.
            var queryUrl = appUrl.forQuery(this.activeDatabase(), newQuery.Hash);
            this.updateUrl(queryUrl);
            // Add this query to our recent queries list in the UI, or move it to the top of the list if it's already there.
            var existing = this.recentQueries.first(function (q) { return q.Hash === newQuery.Hash; });
            if (existing) {
                this.recentQueries.remove(existing);
                this.recentQueries.unshift(existing);
            }
            else {
                this.recentQueries.unshift(newQuery);
            }
            // Limit us to 15 query recent runs.
            if (this.recentQueries().length > 15) {
                this.recentQueries.remove(this.recentQueries()[15]);
            }
            //save the recent queries to local storage
            recentQueriesStorage.saveRecentQueries(this.activeDatabase(), this.recentQueries());
        };
        query.prototype.getRecentQuerySortText = function (sorts) {
            if (sorts.length > 0) {
                return sorts
                    .map(function (s) { return querySort.fromQuerySortString(s); })
                    .map(function (s) { return s.toHumanizedString(); })
                    .reduce(function (first, second) { return first + ", " + second; });
            }
            return "";
        };
        query.prototype.setSelectedIndex = function (indexName) {
            this.sortBys.removeAll();
            this.selectedIndex(indexName);
            this.runQuery();
            // Reflect the new index in the address bar.
            var indexQuery = query.getIndexUrlPartFromIndexName(indexName);
            var url = appUrl.forQuery(this.activeDatabase(), indexQuery);
            this.updateUrl(url);
            NProgress.done();
        };
        query.getIndexUrlPartFromIndexName = function (indexNameOrCollectionName) {
            if (indexNameOrCollectionName === "All Documents") {
                return "dynamic";
            }
            return indexNameOrCollectionName;
        };
        query.prototype.addSortBy = function () {
            var _this = this;
            var sort = new querySort();
            sort.fieldName.subscribe(function () { return _this.runQuery(); });
            sort.isAscending.subscribe(function () { return _this.runQuery(); });
            sort.isRange.subscribe(function () { return _this.runQuery(); });
            this.sortBys.push(sort);
        };
        query.prototype.removeSortBy = function (sortBy) {
            this.sortBys.remove(sortBy);
            this.runQuery();
        };
        query.prototype.addTransformer = function () {
            this.transformer(new transformerType());
        };
        query.prototype.selectTransformer = function (dto) {
            if (!!dto) {
                var t = new transformerType();
                t.initFromLoad(dto);
                this.transformer(t);
                this.runQuery();
            }
            else {
                this.transformer(null);
            }
        };
        query.prototype.removeTransformer = function () {
            this.transformer(null);
            this.runQuery();
        };
        query.prototype.setOperatorOr = function () {
            this.isDefaultOperatorOr(true);
            this.runQuery();
        };
        query.prototype.setOperatorAnd = function () {
            this.isDefaultOperatorOr(false);
            this.runQuery();
        };
        query.prototype.toggleShowFields = function () {
            this.showFields(!this.showFields());
            this.runQuery();
        };
        query.prototype.toggleIndexEntries = function () {
            this.indexEntries(!this.indexEntries());
            this.runQuery();
        };
        query.prototype.deleteDocsMatchingQuery = function () {
            var _this = this;
            // Run the query so that we have an idea of what we'll be deleting.
            var queryResult = this.runQuery();
            queryResult
                .fetch(0, 1)
                .done(function (results) {
                if (results.totalResultCount === 0) {
                    app.showMessage("There are no documents matching your query.", "Nothing to do");
                }
                else {
                    _this.promptDeleteDocsMatchingQuery(results.totalResultCount);
                }
            });
        };
        query.prototype.promptDeleteDocsMatchingQuery = function (resultCount) {
            var _this = this;
            var viewModel = new deleteDocumentsMatchingQueryConfirm(this.selectedIndex(), this.queryText(), resultCount, this.activeDatabase());
            app
                .showDialog(viewModel)
                .done(function () { return _this.runQuery(); });
        };
        query.prototype.getQueryGrid = function () {
            var gridContents = $(query.queryGridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        query.prototype.selectColumns = function () {
            var _this = this;
            var selectColumnsViewModel = new selectColumns(this.currentColumnsParams().clone(), this.currentCustomFunctions().clone(), this.contextName(), this.activeDatabase(), this.getQueryGrid().getColumnsNames());
            app.showDialog(selectColumnsViewModel);
            selectColumnsViewModel.onExit().done(function (cols) {
                _this.currentColumnsParams(cols);
                _this.runQuery();
            });
        };
        query.prototype.fetchIndexFields = function (indexName) {
            var _this = this;
            // Fetch the index definition so that we get an updated list of fields to be used as sort by options.
            // Fields don't show for All Documents.
            var self = this;
            var isAllDocumentsDynamicQuery = indexName === "All Documents";
            if (!isAllDocumentsDynamicQuery) {
                //if index is dynamic, get columns using index definition, else get it using first index result
                if (indexName.indexOf(this.dynamicPrefix) === 0) {
                    var collectionName = indexName.substring(8);
                    new getDocumentsByEntityNameCommand(new collection(collectionName, this.activeDatabase()), 0, 1)
                        .execute()
                        .done(function (result) {
                        if (!!result && result.totalResultCount > 0 && result.items.length > 0) {
                            var dynamicIndexPattern = new document(result.items[0]);
                            if (!!dynamicIndexPattern) {
                                _this.indexFields(dynamicIndexPattern.getDocumentPropertyNames());
                            }
                        }
                    });
                }
                else {
                    new getIndexDefinitionCommand(indexName, this.activeDatabase())
                        .execute()
                        .done(function (result) {
                        self.isTestIndex(result.Index.IsTestIndex);
                        self.indexFields(result.Index.Fields);
                    });
                }
            }
        };
        query.prototype.findTransformerByName = function (transformerName) {
            try {
                return this.allTransformers().filter(function (dto) { return transformerName === dto.name; })[0];
            }
            catch (e) {
                return null;
            }
        };
        query.prototype.getStoredQueryTransformerName = function (query) {
            if (query.TransformerQuery) {
                return query.TransformerQuery.transformerName;
            }
            return "";
        };
        query.prototype.getStoredQueryTransformerParameters = function (queryParams) {
            if (queryParams.length > 0) {
                return "(" +
                    queryParams
                        .map(function (param) { return param.name + "=" + param.value; })
                        .join(", ") + ")";
            }
            return "";
        };
        query.prototype.applyTransformerParameters = function (query) {
            if (query.TransformerQuery && query.TransformerQuery.queryParams) {
                query.TransformerQuery.queryParams.forEach(function (param) {
                    $("#transformerParams input[name=" + param.name + "]").val(param.value);
                });
            }
        };
        query.prototype.extractQueryFields = function () {
            var query = this.queryText();
            var luceneSimpleFieldRegex = /(\w+):\s*("((?:[^"\\]|\\.)*)"|'((?:[^'\\]|\\.)*)'|(\w+))/g;
            var queryFields = [];
            var match = null;
            while ((match = luceneSimpleFieldRegex.exec(query))) {
                var value = match[3] || match[4] || match[5];
                queryFields.push({
                    FieldName: match[1],
                    FieldValue: value,
                    Index: match.index
                });
            }
            return queryFields;
        };
        query.prototype.getIndexSuggestions = function (indexName, info) {
            var _this = this;
            if (this.indexFields().contains(info.FieldName)) {
                var task = new getIndexSuggestionsCommand(this.activeDatabase(), indexName, info.FieldName, info.FieldValue).execute();
                task.done(function (result) {
                    for (var index = 0; index < result.Suggestions.length; index++) {
                        _this.indexSuggestions.push({
                            Index: info.Index,
                            FieldName: info.FieldName,
                            FieldValue: info.FieldValue,
                            Suggestion: result.Suggestions[index]
                        });
                    }
                });
            }
        };
        query.prototype.applySuggestion = function (suggestion) {
            var value = this.queryText();
            var startIndex = value.indexOf(suggestion.FieldValue, suggestion.Index);
            this.queryText(value.substring(0, startIndex) + suggestion.Suggestion + value.substring(startIndex + suggestion.FieldValue.length));
            this.indexSuggestions([]);
            this.runQuery();
        };
        query.prototype.exportCsv = function () {
            var db = this.activeDatabase();
            var url = appUrl.forResourceQuery(db) + this.csvUrl();
            this.downloader.download(db, url);
        };
        query.prototype.fieldNameStartsWith = function () {
            var _this = this;
            var fieldStartsWithViewModel = new fieldStringFilter("Field sub text");
            fieldStartsWithViewModel
                .applyFilterTask
                .done(function (input, option) {
                if (_this.queryText().length !== 0) {
                    _this.queryText(_this.queryText() + " AND ");
                }
                if (option === "Starts with") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":" + queryUtil.escapeTerm(input) + "*");
                }
                else if (option === "Ends with") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":" + "*" + queryUtil.escapeTerm(input));
                }
                else if (option === "Contains") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":" + "*" + queryUtil.escapeTerm(input) + "*");
                }
                else if (option === "Exact") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":" + queryUtil.escapeTerm(input));
                }
            });
            app.showDialog(fieldStartsWithViewModel);
        };
        query.prototype.fieldValueRange = function () {
            var _this = this;
            var fieldRangeFilterViewModel = new fieldRangeFilter("Field range filter");
            fieldRangeFilterViewModel
                .applyFilterTask
                .done(function (fromStr, toStr, option) {
                var from = !fromStr || fromStr.length === 0 ? "*" : fromStr;
                var to = !toStr || toStr.length === 0 ? "*" : toStr;
                var fromPrefix = "";
                var toPrefix = "";
                if (_this.queryText().length !== 0) {
                    _this.queryText(_this.queryText() + " AND ");
                }
                if (option === "Numeric Double") {
                    if (from !== "*")
                        fromPrefix = "Dx";
                    if (to !== "*")
                        toPrefix = "Dx";
                    _this.queryText(_this.queryText() + _this.searchField() + "_Range:[" + fromPrefix + from + " TO " + toPrefix + to + "]");
                }
                else if (option === "Numeric Int") {
                    if (from !== "*")
                        fromPrefix = "Ix";
                    if (to !== "*")
                        toPrefix = "Ix";
                    _this.queryText(_this.queryText() + _this.searchField() + "_Range:[" + fromPrefix + from + " TO " + toPrefix + to + "]");
                }
                else if (option === "Alphabetical") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":[" + from + " TO " + to + "]");
                }
                else if (option === "Datetime") {
                    _this.queryText(_this.queryText() + _this.searchField() + ":[" + from + " TO " + to + "]");
                }
            });
            app.showDialog(fieldRangeFilterViewModel);
        };
        query.prototype.fieldValueInMethod = function () {
            var _this = this;
            var inMethodFilterViewModel = new InMethodFilter("Field in method filter");
            inMethodFilterViewModel
                .applyFilterTask
                .done(function (inputs) {
                if (_this.queryText().length !== 0) {
                    _this.queryText(_this.queryText() + " AND ");
                }
                var escapedStrings = inputs.map(function (s) { return queryUtil.escapeTerm(s); }).join();
                _this.queryText(_this.queryText() + "@in<" + _this.searchField() + ">:(" + escapedStrings + ")");
            });
            app.showDialog(inMethodFilterViewModel);
        };
        query.containerSelector = "#queryContainer";
        query.queryGridSelector = "#queryResultsGrid";
        return query;
    })(viewModelBase);
    return query;
});
define("viewmodels/database/query/queryShell", ["require", "exports", "plugins/router", "common/appUrl"], function (require, exports, durandalRouter, appUrl) {
    var queryShell = (function () {
        function queryShell() {
            this.router = durandalRouter.createChildRouter()
                .map([
                { route: ['', 'databases/query/index(/:indexNameOrRecentQueryIndex)'], moduleId: 'viewmodels/database/query/query', title: 'Query', nav: true },
                { route: 'databases/query/reporting(/:indexName)', moduleId: 'viewmodels/database/reporting/reporting', title: 'Reporting', nav: true },
                { route: 'databases/query/exploration', moduleId: 'viewmodels/database/exploration/exploration', title: "Data exploration", nav: true }
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
        }
        return queryShell;
    })();
    return queryShell;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/query/queryStatsDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "plugins/router"], function (require, exports, dialog, dialogViewModelBase, router) {
    var queryStatsDialog = (function (_super) {
        __extends(queryStatsDialog, _super);
        function queryStatsDialog(queryStats, selectedIndexEditUrl, didDynamicChangeIndex, rawJsonUrl) {
            _super.call(this);
            this.queryStats = queryStats;
            this.selectedIndexEditUrl = selectedIndexEditUrl;
            this.didDynamicChangeIndex = didDynamicChangeIndex;
            this.rawJsonUrl = rawJsonUrl;
        }
        queryStatsDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        queryStatsDialog.prototype.goToIndex = function () {
            dialog.close(this);
            router.navigate(this.selectedIndexEditUrl, true);
        };
        return queryStatsDialog;
    })(dialogViewModelBase);
    return queryStatsDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/reporting/reporting", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/resources/getDatabaseStatsCommand", "commands/database/index/getIndexDefinitionCommand", "models/database/query/facet", "commands/database/query/queryFacetsCommand", "common/bindingHelpers/aceEditorBindingHandler", "common/pagedList"], function (require, exports, viewModelBase, appUrl, getDatabaseStatsCommand, getIndexDefinitionCommand, facet, queryFacetsCommand, aceEditorBindingHandler, pagedList) {
    var reporting = (function (_super) {
        __extends(reporting, _super);
        function reporting() {
            var _this = this;
            _super.call(this);
            this.selectedIndexName = ko.observable();
            this.selectedIndexLabel = ko.computed(function () { return _this.selectedIndexName() ? _this.selectedIndexName() : "[Select an index]"; });
            this.indexNames = ko.observableArray();
            this.hasSelectedIndex = ko.computed(function () { return _this.selectedIndexName() && _this.selectedIndexName().length > 0; });
            this.editSelectedIndexUrl = ko.computed(function () { return _this.hasSelectedIndex() ? appUrl.forEditIndex(_this.selectedIndexName(), _this.activeDatabase()) : null; });
            this.availableFields = ko.observableArray();
            this.sortOptions = ko.observableArray();
            this.selectedField = ko.observable();
            this.selectedFieldLabel = ko.computed(function () { return _this.selectedField() ? _this.selectedField() : "Select a field"; });
            this.addedValues = ko.observableArray();
            this.filter = ko.observable();
            this.hasFilter = ko.observable(false);
            this.reportResults = ko.observable();
            this.totalQueryResults = ko.computed(function () { return _this.reportResults() ? _this.reportResults().totalResultCount() : null; });
            this.queryDuration = ko.observable();
            this.isCacheDisable = ko.observable(false);
            this.isExportEnabled = ko.computed(function () { return _this.reportResults() ? _this.reportResults().totalResultCount() > 0 : false; });
            this.appUrls = appUrl.forCurrentDatabase();
        }
        reporting.prototype.exportCsv = function () {
            if (this.isExportEnabled() === false)
                return false;
            var objArray = JSON.stringify(this.reportResults().getAllCachedItems());
            var array = typeof objArray != 'object' ? JSON.parse(objArray) : objArray;
            if (array[0] === undefined)
                return false;
            var str = '';
            var line = '';
            for (var index in array[0]) {
                if (index === "__metadata")
                    continue;
                if (line != '')
                    line += ',';
                line += index;
            }
            str += line + '\r\n';
            for (var i = 0; i < array.length; i++) {
                line = '';
                for (var index in array[i]) {
                    if (index === "__metadata")
                        continue;
                    if (line != '')
                        line += ',';
                    line += array[i][index];
                }
                str += line + '\r\n';
            }
            var uriContent = encodeURIComponent(str);
            var link = document.createElement('a');
            link["download"] = this.selectedIndexName() ? "Reporting_" + this.selectedIndexName() + ".csv" : "reporting.csv";
            link.href = 'data:,' + uriContent;
            link.click();
            return true;
        };
        reporting.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#filterQueryLabel").popover({
                html: true,
                trigger: "hover",
                container: ".form-horizontal",
                content: 'Queries use Lucene syntax. Examples:<pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>',
            });
        };
        reporting.prototype.activate = function (indexToActivateOrNull) {
            var _this = this;
            _super.prototype.activate.call(this, indexToActivateOrNull);
            this.updateHelpLink('O3EA1R');
            this.fetchIndexes().done(function () { return _this.selectInitialIndex(indexToActivateOrNull); });
            this.selectedIndexName.subscribe(function () { return _this.resetSelections(); });
            aceEditorBindingHandler.install();
        };
        reporting.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (results) { return _this.indexNames(results.Indexes.map(function (i) { return i.Name; })); });
        };
        reporting.prototype.fetchIndexDefinition = function (indexName) {
            var _this = this;
            new getIndexDefinitionCommand(indexName, this.activeDatabase())
                .execute()
                .done(function (dto) {
                _this.sortOptions(dto.Index.SortOptions);
                _this.availableFields(dto.Index.Fields);
            });
        };
        reporting.prototype.selectInitialIndex = function (indexToActivateOrNull) {
            if (indexToActivateOrNull && this.indexNames.contains(indexToActivateOrNull)) {
                this.setSelectedIndex(indexToActivateOrNull);
            }
            else if (this.indexNames().length > 0) {
                this.setSelectedIndex(this.indexNames.first());
            }
        };
        reporting.prototype.setSelectedIndex = function (indexName) {
            this.selectedIndexName(indexName);
            this.updateUrl(appUrl.forReporting(this.activeDatabase(), indexName));
            this.fetchIndexDefinition(indexName);
        };
        reporting.prototype.setSelectedField = function (fieldName) {
            this.selectedField(fieldName);
            // Update all facets to use that too.
            this.addedValues().forEach(function (v) { return v.name = fieldName; });
        };
        reporting.prototype.resetSelections = function () {
            this.selectedField(null);
            this.addedValues([]);
            this.availableFields([]);
            if (!!this.reportResults()) {
                this.reportResults(null);
            }
        };
        reporting.prototype.mapSortToType = function (sort) {
            switch (sort) {
                case 'Int':
                    return "System.Int32";
                case 'Float':
                    return "System.Single";
                case 'Long':
                    return 'System.Int64';
                case 'Double':
                    return "System.Double";
                case 'Short':
                    return 'System.Int16';
                case 'Byte':
                    return 'System.Byte';
                default:
                    return 'System.String';
            }
        };
        reporting.prototype.addValue = function (fieldName) {
            var sortOps = this.sortOptions();
            var sortOption = (fieldName in sortOps) ? sortOps[fieldName] : "String";
            var val = facet.fromNameAndAggregation(this.selectedField(), fieldName, this.mapSortToType(sortOption));
            this.addedValues.push(val);
        };
        reporting.prototype.removeValue = function (val) {
            this.addedValues.remove(val);
        };
        reporting.prototype.runReport = function () {
            var _this = this;
            var selectedIndex = this.selectedIndexName();
            var filterQuery = this.hasFilter() ? this.filter() : null;
            var facets = this.addedValues().map(function (v) { return v.toDto(); });
            var groupedFacets = [];
            facets.forEach(function (curFacet) {
                var foundFacet = groupedFacets.first(function (x) { return x.AggregationField == curFacet.AggregationField; });
                if (foundFacet) {
                    foundFacet.Aggregation += curFacet.Aggregation;
                }
                else {
                    groupedFacets.push(curFacet);
                }
            });
            var db = this.activeDatabase();
            var resultsFetcher = function (skip, take) {
                var command = new queryFacetsCommand(selectedIndex, filterQuery, skip, take, groupedFacets, db, _this.isCacheDisable());
                ko.postbox.publish("SetRawJSONUrl", appUrl.forReportingRawData(_this.activeDatabase(), _this.selectedIndexName()) + command.argsUrl);
                return command.execute()
                    .done(function (resultSet) { return _this.queryDuration(resultSet.additionalResultInfo); });
            };
            this.reportResults(new pagedList(resultsFetcher));
        };
        reporting.prototype.toggleCacheEnable = function () {
            this.isCacheDisable(!this.isCacheDisable());
        };
        return reporting;
    })(viewModelBase);
    return reporting;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/customFunctionsEditor", ["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "commands/database/documents/getCustomFunctionsCommand", "commands/database/documents/saveCustomFunctionsCommand", "commands/database/globalConfig/getEffectiveCustomFunctionsCommand", "models/database/documents/customFunctions", "common/jsonUtil", "common/messagePublisher"], function (require, exports, aceEditorBindingHandler, viewModelBase, getCustomFunctionsCommand, saveCustomFunctionsCommand, getEffectiveCustomFunctionsCommand, customFunctions, jsonUtil, messagePublisher) {
    var customFunctionsEditor = (function (_super) {
        __extends(customFunctionsEditor, _super);
        function customFunctionsEditor() {
            var _this = this;
            _super.call(this);
            this.hasGlobal = ko.observable(false);
            aceEditorBindingHandler.install();
            this.documentText = ko.observable("");
            this.globalDocumentText = ko.observable("");
            this.dirtyFlag = new ko.DirtyFlag([this.documentText], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        }
        customFunctionsEditor.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('XLDBRW');
        };
        customFunctionsEditor.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#customFunctionsExample").popover({
                html: true,
                trigger: "hover",
                content: "<p>Examples:</p><pre>exports.greet = <span class=\"code-keyword\">function</span>(name) {<br/>    <span class=\"code-keyword\">return</span> <span class=\"code-string\">\"Hello \" + name + \"!\"</span>;<br/>}</pre>"
            });
        };
        customFunctionsEditor.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#customFunctionsEditor.editor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#customFunctionsEditor").on("DynamicHeightSet", function () { return _this.docEditor.resize(); });
            this.fetchCustomFunctions();
        };
        customFunctionsEditor.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#customFunctionsEditor").off("DynamicHeightSet");
        };
        customFunctionsEditor.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var fetchTask = new getCustomFunctionsCommand(this.activeDatabase()).execute();
            fetchTask.done(function (cf) {
                _this.documentText(cf.functions);
                _this.dirtyFlag().reset();
            });
            var globalFetchTask = new getEffectiveCustomFunctionsCommand(this.activeDatabase()).execute();
            globalFetchTask.done(function (result) {
                _this.hasGlobal(result.GlobalExists);
                if (result.GlobalExists) {
                    _this.globalDocumentText(result.GlobalDocument.Functions);
                }
            });
        };
        customFunctionsEditor.prototype.saveChanges = function () {
            var _this = this;
            var annotations = this.docEditor.getSession().getAnnotations();
            var hasErrors = false;
            for (var i = 0; i < annotations.length; i++) {
                if (annotations[i].type === "error") {
                    hasErrors = true;
                    break;
                }
            }
            if (!hasErrors) {
                var cf = new customFunctions({
                    Functions: this.documentText()
                });
                var saveTask = new saveCustomFunctionsCommand(this.activeDatabase(), cf).execute();
                saveTask.done(function () { return _this.dirtyFlag().reset(); });
            }
            else {
                messagePublisher.reportError("Errors in the functions file", "Please correct the errors in the file in order to save it.");
            }
        };
        return customFunctionsEditor;
    })(viewModelBase);
    return customFunctionsEditor;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/databaseSettings", ["require", "exports", "durandal/app", "common/appUrl", "common/bindingHelpers/aceEditorBindingHandler", "commands/resources/getDatabaseSettingsCommand", "commands/resources/saveDatabaseSettingsCommand", "models/database/documents/document", "common/jsonUtil", "viewmodels/viewModelBase", "viewmodels/common/viewSystemDatabaseConfirm", "common/messagePublisher", 'viewmodels/shell'], function (require, exports, app, appUrl, aceEditorBindingHandler, getDatabaseSettingsCommand, saveDatabaseSettingsCommand, document, jsonUtil, viewModelBase, viewSystemDatabaseConfirm, messagePublisher, shell) {
    var databaseSettings = (function (_super) {
        __extends(databaseSettings, _super);
        function databaseSettings() {
            var _this = this;
            _super.call(this);
            this.document = ko.observable();
            this.documentText = ko.observable().extend({ required: true });
            this.metadataText = ko.observable("{}").extend({ required: true });
            this.isEditingEnabled = ko.observable(false);
            this.isEditingMetadata = ko.observable(false);
            this.isForbidden = ko.observable(false);
            aceEditorBindingHandler.install();
            this.document.subscribe(function (doc) {
                if (doc) {
                    var docDto = doc.toDto();
                    _this.securedSettings = ko.toJSON(docDto.SecuredSettings);
                    var docText = _this.stringify(doc.toDto());
                    _this.documentText(docText);
                }
            });
            this.isEditingMetadata.subscribe(function () {
                if (_this.docEditor) {
                    var text = _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                    _this.docEditor.getSession().setValue(text);
                }
            });
            this.text = ko.computed({
                read: function () {
                    return _this.isEditingMetadata() ? _this.metadataText() : _this.documentText();
                },
                write: function (text) {
                    var currentObservable = _this.isEditingMetadata() ? _this.metadataText : _this.documentText;
                    currentObservable(text);
                },
                owner: this
            });
        }
        databaseSettings.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.isForbidden(!shell.isGlobalAdmin());
            if (this.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                var db = this.activeDatabase();
                this.fetchDatabaseSettings(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function (response) {
                    messagePublisher.reportError("Error fetching database document!", response.responseText, response.statusText);
                    deferred.resolve({ redirect: appUrl.forStatus(db) });
                });
            }
            return deferred;
        };
        databaseSettings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('3QMLGH');
            this.dirtyFlag = new ko.DirtyFlag([this.documentText, this.metadataText], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        databaseSettings.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#dbDocEditor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#dbDocEditor").on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
        };
        databaseSettings.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#dbDocEditor").off('DynamicHeightSet');
        };
        databaseSettings.prototype.editDatabaseSettings = function () {
            var _this = this;
            var editDbConfirm = new viewSystemDatabaseConfirm("Meddling with the database settings document could cause irreversible damage!");
            editDbConfirm
                .viewTask
                .done(function () {
                _this.isEditingEnabled(true);
            });
            app.showDialog(editDbConfirm);
        };
        databaseSettings.prototype.refreshFromServer = function () {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty('Unsaved Data', 'You have unsaved data. Are you sure you want to refresh the data from the server?');
            canContinue.done(function () {
                _this.fetchDatabaseSettings(_this.activeDatabase(), true)
                    .done(function () {
                    _this.metadataText("{}");
                    _this.isEditingEnabled(false);
                    _this.activateDoc();
                    _this.dirtyFlag().reset(); //Resync Changes
                });
            });
        };
        databaseSettings.prototype.formatDocument = function () {
            var text = this.isEditingMetadata() ? this.metadataText() : this.documentText();
            var observableToUpdate = this.isEditingMetadata() ? this.metadataText : this.documentText;
            try {
                var tempDoc = JSON.parse(text);
                var formatted = this.stringify(tempDoc);
                observableToUpdate(formatted);
            }
            catch (e) { }
        };
        databaseSettings.prototype.activateMeta = function () {
            this.isEditingMetadata(true);
        };
        databaseSettings.prototype.activateDoc = function () {
            this.isEditingMetadata(false);
        };
        databaseSettings.prototype.saveChanges = function () {
            var _this = this;
            var editDbConfirm = new viewSystemDatabaseConfirm("Meddling with the database settings document could cause irreversible damage!");
            editDbConfirm
                .viewTask
                .done(function () {
                try {
                    var updatedDto = JSON.parse(_this.documentText());
                    var meta = JSON.parse(_this.metadataText());
                    updatedDto['@metadata'] = meta;
                    updatedDto['@metadata']['@etag'] = _this.document().__metadata['@etag'];
                    var newDoc = new document(updatedDto);
                    var saveCommand = new saveDatabaseSettingsCommand(appUrl.getDatabase(), newDoc);
                    var saveTask = saveCommand.execute();
                    saveTask.done(function (saveResult) {
                        _this.document().__metadata['@etag'] = saveResult.ETag;
                        _this.metadataText("{}");
                        _this.isEditingEnabled(false);
                        _this.formatDocument();
                        _this.dirtyFlag().reset(); //Resync Changes
                    });
                }
                catch (e) {
                    var message = "";
                    if (updatedDto == undefined) {
                        message = "The data isn't a legal JSON expression!";
                        _this.isEditingMetadata(false);
                    }
                    else if (meta == undefined) {
                        message = "The metadata isn't a legal JSON expression!";
                        _this.isEditingMetadata(true);
                    }
                    _this.docEditor.focus();
                    messagePublisher.reportError(message);
                }
            });
            app.showDialog(editDbConfirm);
        };
        databaseSettings.prototype.fetchDatabaseSettings = function (db, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            return new getDatabaseSettingsCommand(db, reportFetchProgress)
                .execute()
                .done(function (document) { return _this.document(document); });
        };
        databaseSettings.prototype.getDatabaseSettingsDocumentId = function (db) {
            return "Raven/Databases/" + db.name;
        };
        databaseSettings.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        databaseSettings.containerId = "#databaseSettingsContainer";
        return databaseSettings;
    })(viewModelBase);
    return databaseSettings;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/editSqlReplication", ["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl", "common/jsonUtil", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher", "durandal/app", "models/database/sqlReplication/sqlReplication", "commands/database/documents/getCollectionsCommand", "viewmodels/database/status/sqlReplicationStatsDialog", "models/database/documents/document", "commands/database/documents/saveDocumentCommand", "viewmodels/common/deleteDocuments", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "models/database/documents/documentMetadata", "commands/database/sqlReplication/resetSqlReplicationCommand", "viewmodels/database/status/sqlReplicationSimulationDialog", "models/database/sqlReplication/sqlReplicationConnections", "commands/database/globalConfig/getEffectiveSqlReplicationConnectionStringsCommand"], function (require, exports, router, viewModelBase, appUrl, jsonUtil, aceEditorBindingHandler, messagePublisher, app, sqlReplication, getCollectionsCommand, sqlReplicationStatsDialog, document, saveDocumentCommand, deleteDocuments, getDocumentWithMetadataCommand, getDocumentsMetadataByIDPrefixCommand, documentMetadata, resetSqlReplicationCommand, sqlReplicationSimulationDialog, sqlReplicationConnections, getEffectiveSqlReplicationConnectionStringsCommand) {
    var editSqlReplication = (function (_super) {
        __extends(editSqlReplication, _super);
        function editSqlReplication() {
            var _this = this;
            _super.call(this);
            this.editedReplication = ko.observable();
            this.collections = ko.observableArray();
            this.loadedSqlReplications = [];
            this.isEditingNewReplication = ko.observable(false);
            this.isBasicView = ko.observable(true);
            this.availableConnectionStrings = ko.observableArray();
            this.sqlReplicationStatsAndMetricsHref = appUrl.forCurrentDatabase().statusDebugSqlReplication;
            this.script = ko.computed({
                read: function () {
                    var r = _this.editedReplication();
                    return r ? r.script() : "";
                },
                write: function (v) { return _this.editedReplication().script(v); }
            });
            this.simulationDocumentId = ko.observable();
            this.isBusy = ko.observable(false);
            this.initialReplicationId = '';
            aceEditorBindingHandler.install();
            this.appUrls = appUrl.forCurrentDatabase();
            this.sqlReplicationName = ko.computed(function () { return (!!_this.editedReplication() && !_this.isEditingNewReplication()) ? _this.editedReplication().name() : null; });
        }
        editSqlReplication.prototype.toggleBasicMode = function () {
            this.isBasicView(!this.isBasicView());
        };
        editSqlReplication.prototype.addScriptLabelPopover = function () {
            var popOverSettings = {
                html: true,
                trigger: 'hover',
                content: 'Replication scripts use JavaScript.<br/><br/>The script will be called once for each document in the source document collection, with <span class="code-keyword">this</span> representing the document, and the document id available as <i>documentId</i>.<br/><br/>Call <i>replicateToTableName</i> for each row you want to write to the database.<br/><br/>Example:</br><pre><span class="code-keyword">var</span> orderData = {<br/>   Id: documentId,<br/>   OrderLinesCount: <span class="code-keyword">this</span>.Lines.length,<br/>   TotalCost: 0<br/>};<br/><br/>for (<span class="code-keyword">var</span> i = 0; i &lt; <span class="code-keyword">this</span>.Lines.length; i++) {<br/>   <span class="code-keyword">var</span> line = <span class="code-keyword">this</span>.Lines[i];<br/>   <span class="code-keyword">var</span> lineCost = ((line.Quantity * line.PricePerUnit) <br />                     * (1 - line.Discount));<br/>   orderData.TotalCost += lineCost;<br/><br/>   replicateToOrderLines({<br/>      OrderId: documentId,<br/>      Qty: line.Quantity,<br/>      Product: line.Product,<br/>      Cost: lineCost<br/>   });<br/>}<br/><br/>replicateToOrders(orderData);</pre>',
                selector: '.script-label',
                placement: "right"
            };
            $('body').popover(popOverSettings);
            $('form :input[name="ravenEntityName"]').on("keypress", function (e) { return e.which != 13; });
        };
        editSqlReplication.prototype.loadSqlReplicationConnections = function () {
            var _this = this;
            return new getEffectiveSqlReplicationConnectionStringsCommand(this.activeDatabase())
                .execute()
                .done(function (dto) {
                var connections = new sqlReplicationConnections(dto);
                if (connections.predefinedConnections().length > 0) {
                    connections.predefinedConnections().forEach(function (x) { return _this.availableConnectionStrings.push(x); });
                }
            });
        };
        editSqlReplication.prototype.canActivate = function (replicationToEditName) {
            var _this = this;
            var canActivateResult = $.Deferred();
            this.loadSqlReplicationConnections().always(function () {
                if (replicationToEditName) {
                    _this.loadSqlReplication(replicationToEditName)
                        .done(function () { return canActivateResult.resolve({ can: true }); })
                        .fail(function () {
                        messagePublisher.reportError("Could not find " + decodeURIComponent(replicationToEditName) + " replication");
                        canActivateResult.resolve({ redirect: appUrl.forSqlReplications(_this.activeDatabase()) });
                    });
                }
                else {
                    _this.isEditingNewReplication(true);
                    _this.editedReplication(_this.createSqlReplication());
                    _this.fetchCollections(_this.activeDatabase()).always(function () { return canActivateResult.resolve({ can: true }); });
                }
            });
            return canActivateResult;
        };
        editSqlReplication.prototype.activate = function (replicationToEditName) {
            var _this = this;
            _super.prototype.activate.call(this, replicationToEditName);
            this.dirtyFlag = new ko.DirtyFlag([this.editedReplication], false, jsonUtil.newLineNormalizingHashFunctionWithIgnoredFields(["__metadata", "metadata"]));
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        editSqlReplication.prototype.providerChanged = function (obj, event) {
            if (event.originalEvent && obj.connectionStringType() == obj.CONNECTION_STRING) {
                var curConnectionString = !!this.editedReplication().connectionStringValue() ? this.editedReplication().connectionStringValue().trim() : "";
                if (curConnectionString === "" ||
                    sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ConnectionString == curConnectionString; })) {
                    var matchingConnectionStringPair = sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ProviderName == event.originalEvent.srcElement.selectedOptions[0].value; });
                    if (!!matchingConnectionStringPair) {
                        var matchingConnectionStringValue = matchingConnectionStringPair.ConnectionString;
                        this.editedReplication().connectionStringValue(matchingConnectionStringValue);
                    }
                }
            }
        };
        editSqlReplication.prototype.loadSqlReplication = function (replicationToLoadName) {
            var _this = this;
            var loadDeferred = $.Deferred();
            $.when(this.fetchSqlReplicationToEdit(replicationToLoadName), this.fetchCollections(this.activeDatabase()))
                .done(function () {
                _this.editedReplication().collections = _this.collections;
                new getDocumentsMetadataByIDPrefixCommand(editSqlReplication.sqlReplicationDocumentPrefix, 256, _this.activeDatabase())
                    .execute()
                    .done(function (results) {
                    _this.loadedSqlReplications = results;
                    loadDeferred.resolve();
                }).
                    fail(function () { return loadDeferred.reject(); });
            })
                .fail(function () {
                loadDeferred.reject();
            });
            return loadDeferred;
        };
        editSqlReplication.prototype.fetchSqlReplicationToEdit = function (sqlReplicationName) {
            var _this = this;
            var loadDocTask = new getDocumentWithMetadataCommand(editSqlReplication.sqlReplicationDocumentPrefix + sqlReplicationName, this.activeDatabase()).execute();
            loadDocTask.done(function (document) {
                var sqlReplicationDto = document.toDto(true);
                _this.editedReplication(new sqlReplication(sqlReplicationDto));
                _this.initialReplicationId = _this.editedReplication().name();
                _this.dirtyFlag().reset(); //Resync Changes
            });
            loadDocTask.always(function () { return _this.isBusy(false); });
            this.isBusy(true);
            return loadDocTask;
        };
        editSqlReplication.prototype.fetchCollections = function (db) {
            var _this = this;
            return new getCollectionsCommand(db)
                .execute()
                .done(function (collections) {
                _this.collections(collections.map(function (collection) { return collection.name; }));
            });
        };
        editSqlReplication.prototype.showStats = function () {
            var viewModel = new sqlReplicationStatsDialog(this.activeDatabase(), this.editedReplication().name());
            app.showDialog(viewModel);
        };
        editSqlReplication.prototype.refreshSqlReplication = function () {
            if (this.isEditingNewReplication() === false) {
                var docId = this.initialReplicationId;
                this.loadSqlReplication(docId);
            }
            else {
                this.editedReplication(this.createSqlReplication());
            }
        };
        editSqlReplication.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.addScriptLabelPopover();
            $('pre').each(function (index, currentPreElement) {
                _this.initializeAceValidity(currentPreElement);
            });
            var editorElement = $("#sqlReplicationEditor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#sqlReplicationEditor").on('DynamicHeightSet', function () { return _this.docEditor.resize(); });
        };
        editSqlReplication.prototype.createSqlReplication = function () {
            var newSqlReplication = sqlReplication.empty();
            newSqlReplication.collections(this.collections());
            this.collections.subscribe(function (value) { return newSqlReplication.collections(value); });
            this.subscribeToSqlReplicationName(newSqlReplication);
            return newSqlReplication;
        };
        editSqlReplication.prototype.subscribeToSqlReplicationName = function (sqlReplicationElement) {
            var _this = this;
            sqlReplicationElement.name.subscribe(function (previousName) {
                //Get the previous value of 'name' here before it's set to newValue
                var nameInputArray = $('input[name="name"]').filter(function () { return this.value === previousName; });
                if (nameInputArray.length === 1) {
                    var inputField = nameInputArray[0];
                    inputField.setCustomValidity("");
                }
            }, this, "beforeChange");
            sqlReplicationElement.name.subscribe(function (newName) {
                var message = "";
                if (newName === "") {
                    message = "Please fill out this field.";
                }
                else if (_this.isSqlReplicationNameExists(newName)) {
                    message = "SQL Replication name already exists.";
                }
                $('input[name="name"]')
                    .filter(function () { return this.value === newName; })
                    .each(function (index, element) {
                    element.setCustomValidity(message);
                });
            });
        };
        editSqlReplication.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#sqlReplicationEditor").off('DynamicHeightSet');
        };
        editSqlReplication.prototype.isSqlReplicationNameExists = function (name) {
            var count = 0;
            return !!this.loadedSqlReplications.first(function (x) { return x == name; });
        };
        editSqlReplication.prototype.initializeAceValidity = function (element) {
            var editorElement = $("#aceEditor");
            if (editorElement.length > 0) {
                var editor = ko.utils.domData.get(editorElement[0], "aceEditor");
                var editorValue = editor.getSession().getValue();
                if (editorValue === "") {
                    var textarea = $(element).find('textarea')[0];
                    textarea.setCustomValidity("Please fill out this field.");
                }
            }
        };
        editSqlReplication.prototype.save = function () {
            var _this = this;
            var currentDocumentId = this.editedReplication().name();
            this.editedReplication().script(this.script());
            if (this.initialReplicationId !== currentDocumentId) {
                delete this.editedReplication().__metadata.etag;
                delete this.editedReplication().__metadata.lastModified;
            }
            var newDoc = new document(this.editedReplication().toDto());
            newDoc.__metadata = new documentMetadata();
            this.attachReservedMetaProperties(editSqlReplication.sqlReplicationDocumentPrefix + currentDocumentId, newDoc.__metadata);
            var saveCommand = new saveDocumentCommand(editSqlReplication.sqlReplicationDocumentPrefix + currentDocumentId, newDoc, this.activeDatabase());
            var saveTask = saveCommand.execute();
            saveTask.done(function (saveResult) {
                var savedDocumentDto = saveResult[0];
                var sqlReplicationKey = savedDocumentDto.Key.substring(editSqlReplication.sqlReplicationDocumentPrefix.length);
                _this.loadSqlReplication(sqlReplicationKey)
                    .done(function () { return _this.dirtyFlag().reset(); });
                _this.updateUrl(sqlReplicationKey);
                _this.isEditingNewReplication(false);
                _this.initialReplicationId = currentDocumentId;
            });
        };
        editSqlReplication.prototype.updateUrl = function (docId) {
            var url = appUrl.forEditSqlReplication(docId, this.activeDatabase());
            router.navigate(url, false);
        };
        editSqlReplication.prototype.attachReservedMetaProperties = function (id, target) {
            target.etag = '';
            target.ravenEntityName = !target.ravenEntityName ? document.getEntityNameFromId(id) : target.ravenEntityName;
            target.id = id;
        };
        editSqlReplication.prototype.deleteSqlReplication = function () {
            var _this = this;
            var newDoc = new document(this.editedReplication().toDto());
            if (newDoc) {
                var viewModel = new deleteDocuments([newDoc]);
                viewModel.deletionTask.done(function () {
                    _this.dirtyFlag().reset(); //Resync Changes
                    router.navigate(appUrl.forCurrentDatabase().sqlReplications());
                });
                app.showDialog(viewModel, editSqlReplication.editSqlReplicationSelector);
            }
        };
        editSqlReplication.prototype.resetSqlReplication = function () {
            var _this = this;
            app.showMessage("You are about to reset this SQL Replication, forcing replication of all collection items", "SQL Replication Reset", ["Cancel", "Reset"])
                .then(function (dialogResult) {
                if (dialogResult === "Reset") {
                    var replicationId = _this.initialReplicationId;
                    new resetSqlReplicationCommand(_this.activeDatabase(), replicationId).execute()
                        .done(function () { return messagePublisher.reportSuccess("SQL replication " + replicationId + " was reset successfully!"); })
                        .fail(function () { return messagePublisher.reportError("SQL replication " + replicationId + " failed to reset!"); });
                }
            });
        };
        editSqlReplication.prototype.simulateSqlReplication = function () {
            this.editedReplication().script(this.script());
            var viewModel = new sqlReplicationSimulationDialog(this.activeDatabase(), this.editedReplication(), this.simulationDocumentId);
            app.showDialog(viewModel);
        };
        editSqlReplication.prototype.getSqlReplicationConnectionStringsUrl = function (sqlReplicationName) {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        editSqlReplication.editSqlReplicationSelector = "#editSQLReplication";
        editSqlReplication.sqlReplicationDocumentPrefix = "Raven/SqlReplication/Configuration/";
        return editSqlReplication;
    })(viewModelBase);
    return editSqlReplication;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/periodicExport", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseSettingsCommand", "commands/maintenance/savePeriodicExportSetupCommand", "models/database/documents/periodicExportSetup", "commands/database/globalConfig/getEffectivePeriodicExportCommand", "common/appUrl", "commands/database/globalConfig/getConfigurationSettingsCommand", "commands/database/globalConfig/deleteLocalPeriodicExportSetupCommand"], function (require, exports, viewModelBase, getDatabaseSettingsCommand, savePeriodicExportSetupCommand, periodicExportSetup, getEffectivePeriodicExportCommand, appUrl, getConfigurationSettingsCommand, deleteLocalPeriodicExportSetupCommand) {
    var periodicExport = (function (_super) {
        __extends(periodicExport, _super);
        function periodicExport() {
            var _this = this;
            _super.call(this);
            this.backupSetup = ko.observable().extend({ required: true });
            this.globalBackupSetup = ko.observable();
            this.exportDisabled = ko.observable(false);
            this.usingGlobal = ko.observable(false);
            this.hasGlobalValues = ko.observable(false);
            this.isForbidden = ko.observable(false);
            this.activeDatabase.subscribe(function (db) { return _this.isForbidden(!db.isAdminCurrentTenant()); });
            this.showOnDiskExportRow = ko.computed(function () {
                var localSetting = _this.backupSetup() && _this.backupSetup().onDiskExportEnabled();
                var globalSetting = _this.hasGlobalValues() && _this.globalBackupSetup().onDiskExportEnabled();
                return localSetting || globalSetting;
            });
        }
        periodicExport.prototype.attached = function () {
            _super.prototype.attached.call(this);
            var content = "Could not decrypt the access settings, if you are running on IIS, make sure that load user profile is set to true. " +
                "Alternatively this can happen when the server was started using different account than when the settings were created.<br />" +
                "Reenter your settings and click save.";
            $("#awsDecryptFailureSpan").popover({
                html: true,
                trigger: "hover",
                container: $("body"),
                content: content
            });
            $("#azureDecryptFailureSpan").popover({
                html: true,
                trigger: "hover",
                container: $("body"),
                content: content
            });
        };
        periodicExport.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            this.backupSetup(new periodicExportSetup);
            this.globalBackupSetup(new periodicExportSetup);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            this.isForbidden(!db.isAdminCurrentTenant());
            if (db.isAdminCurrentTenant()) {
                $.when(this.fetchPeriodicExportSetup(db), this.fetchPeriodicExportAccountsSettings(db))
                    .done(function () {
                    _this.updateExportDisabledFlag();
                    deferred.resolve({ can: true });
                })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        periodicExport.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("OU78CB");
            this.dirtyFlag = new ko.DirtyFlag([this.backupSetup, this.usingGlobal]);
            var self = this;
            this.isSaveEnabled = ko.computed(function () {
                var onDisk = self.backupSetup().onDiskExportEnabled();
                var remote = self.backupSetup().remoteUploadEnabled();
                var hasAnyOption = onDisk || remote;
                var isDirty = _this.dirtyFlag().isDirty();
                return hasAnyOption && isDirty;
            });
        };
        periodicExport.prototype.fetchPeriodicExportSetup = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getEffectivePeriodicExportCommand(db)
                .execute()
                .done(function (result) {
                _this.backupSetup().fromDto(result.MergedDocument);
                _this.hasGlobalValues(result.GlobalExists);
                _this.usingGlobal(result.GlobalExists && !result.LocalExists);
                if (_this.hasGlobalValues()) {
                    _this.globalBackupSetup().fromDto(result.GlobalDocument);
                }
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        periodicExport.prototype.fetchPeriodicExportAccountsSettings = function (db) {
            var _this = this;
            var task = $.Deferred();
            var dbSettingsTask = new getDatabaseSettingsCommand(db)
                .execute()
                .done(function (document) { _this.backupSetup().fromDatabaseSettingsDto(document.toDto(true)); });
            dbSettingsTask.then(function () {
                new getConfigurationSettingsCommand(db, ["Raven/AWSAccessKey", "Raven/AWSSecretKey", "Raven/AzureStorageAccount", "Raven/AzureStorageKey"])
                    .execute()
                    .done(function (result) {
                    var awsAccess = result.results["Raven/AWSAccessKey"];
                    var awsSecret = result.results["Raven/AWSSecretKey"];
                    var azureAccess = result.results["Raven/AzureStorageAccount"];
                    var azureSecret = result.results["Raven/AzureStorageKey"];
                    _this.globalBackupSetup().awsAccessKey(awsAccess.globalValue());
                    _this.globalBackupSetup().awsSecretKey(awsSecret.globalValue());
                    _this.globalBackupSetup().azureStorageAccount(azureAccess.globalValue());
                    _this.globalBackupSetup().azureStorageKey(azureSecret.globalValue());
                    _this.backupSetup().awsAccessKey(awsAccess.effectiveValue());
                    _this.backupSetup().awsSecretKey(awsSecret.effectiveValue());
                    _this.backupSetup().azureStorageAccount(azureAccess.effectiveValue());
                    _this.backupSetup().azureStorageKey(azureSecret.effectiveValue());
                    task.resolve();
                });
            });
            return task;
        };
        periodicExport.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var task;
                if (this.usingGlobal()) {
                    task = new deleteLocalPeriodicExportSetupCommand(this.backupSetup(), db)
                        .execute();
                }
                else {
                    task = new savePeriodicExportSetupCommand(this.backupSetup(), db).execute();
                }
                task.done(function (resultArray) {
                    var newEtag = resultArray[0].ETag;
                    _this.backupSetup().setEtag(newEtag);
                    _this.backupSetup().resetDecryptionFailures();
                    _this.dirtyFlag().reset(); // Resync changes
                    _this.updateExportDisabledFlag();
                });
            }
        };
        periodicExport.prototype.useLocal = function () {
            this.usingGlobal(false);
        };
        periodicExport.prototype.useGlobal = function () {
            this.usingGlobal(true);
            this.backupSetup().copyFrom(this.globalBackupSetup());
        };
        periodicExport.prototype.updateExportDisabledFlag = function () {
            if (this.usingGlobal()) {
                this.exportDisabled(this.globalBackupSetup().disabled());
            }
            else {
                this.exportDisabled(this.backupSetup().disabled());
            }
        };
        return periodicExport;
    })(viewModelBase);
    return periodicExport;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/quotas", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/saveDatabaseSettingsCommand", "commands/database/globalConfig/getConfigurationSettingsCommand", "models/database/documents/document", "common/appUrl", "commands/resources/getDatabaseSettingsCommand", 'viewmodels/shell'], function (require, exports, viewModelBase, saveDatabaseSettingsCommand, getConfigurationSettingsCommand, document, appUrl, getDatabaseSettingsCommand, shell) {
    var quotas = (function (_super) {
        __extends(quotas, _super);
        function quotas() {
            var _this = this;
            _super.call(this);
            this.settingsDocument = ko.observable();
            this.usingGlobal = ko.observable(false);
            this.hasGlobalValues = ko.observable(false);
            this.isForbidden = ko.observable(false);
            this.activeDatabase.subscribe(function (db) { return _this.isForbidden(!db.isAdminCurrentTenant()); });
        }
        quotas.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.isForbidden(shell.isGlobalAdmin() == false);
            if (this.isForbidden() == false) {
                var db = this.activeDatabase();
                // fetch current quotas from the database
                this.fetchQuotas(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        quotas.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('594W7T');
            this.initializeDirtyFlag();
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty() === true; });
        };
        quotas.prototype.fetchQuotas = function (db, reportFetchProgress) {
            var _this = this;
            if (reportFetchProgress === void 0) { reportFetchProgress = false; }
            var dbSettingsTask = new getDatabaseSettingsCommand(db, reportFetchProgress)
                .execute()
                .done(function (doc) { return _this.settingsDocument(doc); });
            var configTask = new getConfigurationSettingsCommand(db, ["Raven/Quotas/Size/HardLimitInKB", "Raven/Quotas/Size/SoftMarginInKB", "Raven/Quotas/Documents/HardLimit", "Raven/Quotas/Documents/SoftLimit"])
                .execute()
                .done(function (result) {
                _this.maximumSize = result.results["Raven/Quotas/Size/HardLimitInKB"];
                _this.warningLimitThreshold = result.results["Raven/Quotas/Size/SoftMarginInKB"];
                _this.maxNumberOfDocs = result.results["Raven/Quotas/Documents/HardLimit"];
                _this.warningThresholdForDocs = result.results["Raven/Quotas/Documents/SoftLimit"];
                _this.usingGlobal(_this.maximumSize.isUsingGlobal());
                _this.hasGlobalValues(_this.maximumSize.globalExists());
                var divideBy1024 = function (x) {
                    if (x()) {
                        x(x() / 1024);
                    }
                };
                divideBy1024(_this.maximumSize.effectiveValue);
                divideBy1024(_this.maximumSize.globalValue);
                divideBy1024(_this.warningLimitThreshold.effectiveValue);
                divideBy1024(_this.warningLimitThreshold.globalValue);
            });
            return $.when(dbSettingsTask, configTask);
        };
        quotas.prototype.initializeDirtyFlag = function () {
            this.dirtyFlag = new ko.DirtyFlag([
                this.maximumSize.effectiveValue, this.maximumSize.localExists,
                this.warningLimitThreshold.effectiveValue, this.warningLimitThreshold.localExists,
                this.maxNumberOfDocs.effectiveValue, this.maxNumberOfDocs.localExists,
                this.warningThresholdForDocs.effectiveValue, this.warningThresholdForDocs.localExists,
                this.usingGlobal
            ]);
        };
        quotas.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var settingsDocument = this.settingsDocument();
                settingsDocument["@metadata"] = this.settingsDocument().__metadata;
                settingsDocument["@metadata"]["@etag"] = this.settingsDocument().__metadata["@etag"];
                var doc = new document(settingsDocument.toDto(true));
                if (this.usingGlobal()) {
                    delete doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                    delete doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"];
                    delete doc["Settings"]["Raven/Quotas/Documents/HardLimit"];
                    delete doc["Settings"]["Raven/Quotas/Documents/SoftLimit"];
                }
                else {
                    doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = this.maximumSize.effectiveValue() * 1024;
                    doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = this.warningLimitThreshold.effectiveValue() * 1024;
                    doc["Settings"]["Raven/Quotas/Documents/HardLimit"] = this.maxNumberOfDocs.effectiveValue();
                    doc["Settings"]["Raven/Quotas/Documents/SoftLimit"] = this.warningThresholdForDocs.effectiveValue();
                }
                var saveTask = new saveDatabaseSettingsCommand(db, doc).execute();
                saveTask.done(function (saveResult) {
                    _this.settingsDocument().__metadata["@etag"] = saveResult.ETag;
                    _this.dirtyFlag().reset(); //Resync Changes
                });
            }
        };
        quotas.prototype.useLocal = function () {
            this.usingGlobal(false);
        };
        quotas.prototype.useGlobal = function () {
            this.usingGlobal(true);
            this.maximumSize.copyFromGlobal();
            this.warningLimitThreshold.copyFromGlobal();
            this.maxNumberOfDocs.copyFromGlobal();
            this.warningThresholdForDocs.copyFromGlobal();
        };
        return quotas;
    })(viewModelBase);
    return quotas;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/replications", ["require", "exports", "viewmodels/viewModelBase", "models/database/replication/replicationsSetup", "models/database/replication/replicationConfig", "models/database/replication/replicationDestination", "commands/resources/getDatabaseStatsCommand", "commands/database/replication/getReplicationsCommand", "commands/database/documents/updateServerPrefixHiLoCommand", "commands/database/replication/saveReplicationDocumentCommand", "commands/database/replication/saveAutomaticConflictResolutionDocument", "commands/database/documents/getServerPrefixForHiLoCommand", "commands/database/replication/replicateAllIndexesCommand", "common/bindingHelpers/aceEditorBindingHandler", "commands/database/replication/replicateAllTransformersCommand", "commands/database/globalConfig/deleteLocalReplicationsSetupCommand", "commands/database/replication/replicateIndexesCommand", "commands/database/replication/replicateTransformersCommand", "commands/database/globalConfig/getEffectiveConflictResolutionCommand", "commands/database/documents/getCollectionsCommand", "common/appUrl", "commands/database/replication/enableReplicationCommand", "models/database/replication/replicationPatchScript", "commands/database/replication/resolveAllConflictsCommand"], function (require, exports, viewModelBase, replicationsSetup, replicationConfig, replicationDestination, getDatabaseStatsCommand, getReplicationsCommand, updateServerPrefixHiLoCommand, saveReplicationDocumentCommand, saveAutomaticConflictResolutionDocument, getServerPrefixForHiLoCommand, replicateAllIndexesCommand, aceEditorBindingHandler, replicateAllTransformersCommand, deleteLocalReplicationsSetupCommand, replicateIndexesCommand, replicateTransformersCommand, getEffectiveConflictResolutionCommand, getCollectionsCommand, appUrl, enableReplicationCommand, replicationPatchScript, ResolveAllConflictsCommand) {
    var replications = (function (_super) {
        __extends(replications, _super);
        function replications() {
            var _this = this;
            _super.call(this);
            this.replicationEnabled = ko.observable(false);
            this.prefixForHilo = ko.observable("");
            this.replicationConfig = ko.observable(new replicationConfig({ DocumentConflictResolution: "None", AttachmentConflictResolution: "None" }));
            this.replicationsSetup = ko.observable(new replicationsSetup({ MergedDocument: { Destinations: [], Source: null } }));
            this.globalClientFailoverBehaviour = ko.observable(null);
            this.globalClientRequestTimeSlaThreshold = ko.observable();
            this.globalReplicationConfig = ko.observable();
            this.collections = ko.observableArray();
            this.serverPrefixForHiLoDirtyFlag = new ko.DirtyFlag([]);
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([]);
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag([]);
            this.skipIndexReplicationForAllDestinationsStatus = ko.observable();
            this.skipIndexReplicationForAll = ko.observable();
            this.usingGlobal = ko.observable(false);
            this.hasGlobalValues = ko.observable(false);
            this.globalReadFromAllAllowWriteToSecondaries = ko.computed(function () {
                var behaviour = _this.globalClientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return tokens.contains("ReadFromAllServers") && tokens.contains("AllowReadsFromSecondariesAndWritesToSecondaries");
            });
            this.globalReadFromAllButSwitchWhenRequestTimeSlaThresholdIsReached = ko.computed(function () {
                var behaviour = _this.globalClientFailoverBehaviour();
                if (behaviour == null) {
                    return false;
                }
                var tokens = behaviour.split(",").map(function (x) { return x.trim(); });
                return tokens.contains("ReadFromAllServers") && tokens.contains("AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
            });
            aceEditorBindingHandler.install();
            this.showRequestTimeoutRow = ko.computed(function () {
                var localSetting = _this.replicationsSetup().showRequestTimeSlaThreshold();
                var globalSetting = _this.hasGlobalValues() && _this.globalClientFailoverBehaviour() &&
                    _this.globalClientFailoverBehaviour().contains("AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached");
                return localSetting || globalSetting;
            });
        }
        replications.prototype.refereshSkipIndexReplicationForAllDestinations = function () {
            var _this = this;
            if (this.skipIndexReplicationForAllSubscription != null)
                this.skipIndexReplicationForAllSubscription.dispose();
            var newStatus = this.getIndexReplicationStatusForAllDestinations();
            this.skipIndexReplicationForAll(newStatus === 'all');
            this.skipIndexReplicationForAllSubscription = this.skipIndexReplicationForAll.subscribe(function (newValue) { return _this.toggleIndexReplication(newValue); });
        };
        replications.prototype.getIndexReplicationStatusForAllDestinations = function () {
            var countOfSkipIndexReplication = 0;
            ko.utils.arrayForEach(this.replicationsSetup().destinations(), function (dest) {
                if (dest.skipIndexReplication()) {
                    countOfSkipIndexReplication++;
                }
            });
            // ReSharper disable once ConditionIsAlwaysConst
            if (countOfSkipIndexReplication === 0)
                return 'none';
            if (countOfSkipIndexReplication === this.replicationsSetup().destinations().length)
                return 'all';
            return 'mixed';
        };
        replications.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                if (db.activeBundles.contains("Replication")) {
                    this.replicationEnabled(true);
                    $.when(this.fetchServerPrefixForHiLoCommand(db), this.fetchAutomaticConflictResolution(db), this.fetchReplications(db))
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(db) }); });
                }
                else {
                    this.replicationEnabled(false);
                    deferred.resolve({ can: true });
                }
            }
            return deferred;
        };
        replications.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("7K1KES");
            this.serverPrefixForHiLoDirtyFlag = new ko.DirtyFlag([this.prefixForHilo]);
            this.isServerPrefixForHiLoSaveEnabled = ko.computed(function () { return _this.serverPrefixForHiLoDirtyFlag().isDirty(); });
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([this.replicationConfig]);
            this.isConfigSaveEnabled = ko.computed(function () { return _this.replicationConfigDirtyFlag().isDirty(); });
            var replicationSetupDirtyFlagItems = [this.replicationsSetup, this.replicationsSetup().destinations(), this.skipIndexReplicationForAll, this.replicationConfig, this.replicationsSetup().clientFailoverBehaviour, this.usingGlobal];
            $.each(this.replicationsSetup().destinations(), function (i, dest) {
                replicationSetupDirtyFlagItems.push(dest.specifiedCollections);
                dest.specifiedCollections.subscribe(function (array) {
                    dest.ignoredClient(dest.specifiedCollections.length > 0);
                });
            });
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag(replicationSetupDirtyFlagItems);
            this.isSetupSaveEnabled = ko.computed(function () { return _this.replicationsSetupDirtyFlag().isDirty(); });
            var combinedFlag = ko.computed(function () {
                var rc = _this.replicationConfigDirtyFlag().isDirty();
                var rs = _this.replicationsSetupDirtyFlag().isDirty();
                var sp = _this.serverPrefixForHiLoDirtyFlag().isDirty();
                return rc || rs || sp;
            });
            this.dirtyFlag = new ko.DirtyFlag([combinedFlag, this.usingGlobal]);
            var db = this.activeDatabase();
            this.fetchCollections(db).done(function (results) {
                _this.collections(results);
            });
        };
        replications.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $.each(this.replicationsSetup().destinations(), this.addScriptHelpPopover);
        };
        replications.prototype.fetchServerPrefixForHiLoCommand = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getServerPrefixForHiLoCommand(db)
                .execute()
                .done(function (result) { return _this.prefixForHilo(result); })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getEffectiveConflictResolutionCommand(db)
                .execute()
                .done(function (repConfig) {
                _this.replicationConfig(new replicationConfig(repConfig.MergedDocument));
                if (repConfig.GlobalDocument) {
                    _this.globalReplicationConfig(new replicationConfig(repConfig.GlobalDocument));
                }
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.fetchReplications = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            ko.postbox.subscribe('skip-index-replication', function () { return _this.refereshSkipIndexReplicationForAllDestinations(); });
            new getReplicationsCommand(db)
                .execute()
                .done(function (repSetup) {
                _this.replicationsSetup(new replicationsSetup(repSetup));
                _this.usingGlobal(repSetup.GlobalExists && !repSetup.LocalExists);
                _this.hasGlobalValues(repSetup.GlobalExists);
                if (repSetup.GlobalDocument && repSetup.GlobalDocument.ClientConfiguration) {
                    _this.globalClientFailoverBehaviour(repSetup.GlobalDocument.ClientConfiguration.FailoverBehavior);
                    _this.globalClientRequestTimeSlaThreshold(repSetup.GlobalDocument.ClientConfiguration.RequestTimeSlaThresholdInMilliseconds);
                }
                var status = _this.getIndexReplicationStatusForAllDestinations();
                if (status === 'all')
                    _this.skipIndexReplicationForAll(true);
                else
                    _this.skipIndexReplicationForAll(false);
                _this.skipIndexReplicationForAllSubscription = _this.skipIndexReplicationForAll.subscribe(function (newValue) { return _this.toggleIndexReplication(newValue); });
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        replications.prototype.addScriptHelpPopover = function () {
            $(".scriptPopover").popover({
                html: true,
                trigger: 'hover',
                container: ".form-horizontal",
                content: '<p>Return <code>null</code> in transform script to skip document from replication. </p>' +
                    '<p>Example: </p>' +
                    '<pre><span class="code-keyword">if</span> (<span class="code-keyword">this</span>.Region !== <span class="code-string">"Europe"</span>) { <br />   <span class="code-keyword">return null</span>; <br />}<br/><span class="code-keyword">this</span>.Currency = <span class="code-string">"EUR"</span>; </pre>'
            });
        };
        replications.prototype.onTransformCollectionClick = function (destination, collectionName) {
            var collections = destination.specifiedCollections();
            var item = collections.first(function (c) { return c.collection() === collectionName; });
            if (typeof (item.script()) === "undefined") {
                item.script("");
            }
            else {
                item.script(undefined);
            }
            destination.specifiedCollections.notifySubscribers();
        };
        replications.prototype.onReplicateToCollectionClick = function (destination, collectionName) {
            var collections = destination.specifiedCollections();
            var item = collections.first(function (c) { return c.collection() === collectionName; });
            if (item) {
                collections.remove(item);
            }
            else {
                var patchScript = replicationPatchScript.empty();
                patchScript.collection(collectionName);
                collections.push(patchScript);
            }
            destination.specifiedCollections.notifySubscribers();
        };
        replications.prototype.toggleSkipIndexReplicationForAll = function () {
            this.skipIndexReplicationForAll.toggle();
        };
        replications.prototype.createNewDestination = function () {
            var db = this.activeDatabase();
            this.replicationsSetup().destinations.unshift(replicationDestination.empty(db.name));
            this.refereshSkipIndexReplicationForAllDestinations();
            this.addScriptHelpPopover();
        };
        replications.prototype.removeDestination = function (repl) {
            this.replicationsSetup().destinations.remove(repl);
        };
        replications.prototype.saveChanges = function () {
            var _this = this;
            if (this.usingGlobal()) {
                new deleteLocalReplicationsSetupCommand(this.activeDatabase())
                    .execute();
            }
            else {
                if (this.isConfigSaveEnabled())
                    this.saveAutomaticConflictResolutionSettings();
                if (this.isSetupSaveEnabled()) {
                    if (this.replicationsSetup().source()) {
                        this.saveReplicationSetup();
                    }
                    else {
                        var db = this.activeDatabase();
                        if (db) {
                            new getDatabaseStatsCommand(db)
                                .execute()
                                .done(function (result) {
                                _this.prepareAndSaveReplicationSetup(result.DatabaseId);
                            });
                        }
                    }
                }
            }
        };
        replications.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };
        replications.prototype.saveReplicationSetup = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new saveReplicationDocumentCommand(this.replicationsSetup().toDto(), db)
                    .execute()
                    .done(function () {
                    _this.replicationsSetupDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.fetchCollections = function (db) {
            return new getCollectionsCommand(db, this.collections()).execute();
        };
        replications.prototype.toggleIndexReplication = function (skipReplicationValue) {
            this.replicationsSetup().destinations().forEach(function (dest) {
                dest.skipIndexReplication(skipReplicationValue);
            });
        };
        replications.prototype.sendReplicateCommand = function (destination, parentClass) {
            var db = parentClass.activeDatabase();
            if (db) {
                new replicateIndexesCommand(db, destination).execute();
                new replicateTransformersCommand(db, destination).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.sendReplicateAllCommand = function () {
            var db = this.activeDatabase();
            if (db) {
                new replicateAllIndexesCommand(db).execute();
                new replicateAllTransformersCommand(db).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.sendResolveAllConflictsCommand = function () {
            var db = this.activeDatabase();
            if (db) {
                new ResolveAllConflictsCommand(db).execute();
            }
            else {
                alert("No database selected! This error should not be seen."); //precaution to ease debugging - in case something bad happens
            }
        };
        replications.prototype.saveServerPrefixForHiLo = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new updateServerPrefixHiLoCommand(this.prefixForHilo(), db)
                    .execute()
                    .done(function () {
                    _this.serverPrefixForHiLoDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.saveAutomaticConflictResolutionSettings = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new saveAutomaticConflictResolutionDocument(this.replicationConfig().toDto(), db)
                    .execute()
                    .done(function () {
                    _this.replicationConfigDirtyFlag().reset();
                    _this.dirtyFlag().reset();
                });
            }
        };
        replications.prototype.override = function (value, destination) {
            destination.hasLocal(value);
            if (!destination.hasLocal()) {
                destination.copyFromGlobal();
            }
        };
        replications.prototype.useLocal = function () {
            this.usingGlobal(false);
        };
        replications.prototype.useGlobal = function () {
            this.usingGlobal(true);
            if (this.globalReplicationConfig()) {
                this.replicationConfig().attachmentConflictResolution(this.globalReplicationConfig().attachmentConflictResolution());
                this.replicationConfig().documentConflictResolution(this.globalReplicationConfig().documentConflictResolution());
            }
            this.replicationsSetup().copyFromParent(this.globalClientFailoverBehaviour(), this.globalClientRequestTimeSlaThreshold());
        };
        replications.prototype.enableReplication = function () {
            var _this = this;
            new enableReplicationCommand(this.activeDatabase())
                .execute()
                .done(function (bundles) {
                var db = _this.activeDatabase();
                db.activeBundles(bundles);
                _this.replicationEnabled(true);
                _this.fetchServerPrefixForHiLoCommand(db);
                _this.fetchAutomaticConflictResolution(db);
                _this.fetchReplications(db);
                _this.fetchCollections(db).done(function (results) {
                    _this.collections(results);
                });
            });
        };
        return replications;
    })(viewModelBase);
    return replications;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/settings", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var settings = (function (_super) {
        __extends(settings, _super);
        function settings() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.bundleMap = { quotas: "Quotas", replication: "Replication", sqlreplication: "SQL Replication", versioning: "Versioning", periodicexport: "Periodic Export", scriptedindexresults: "Scripted Index", periodicbackup: "Periodic Export" };
            this.sqlSubBundles = ["sqlreplicationconnectionstringsmanagement", "editsqlreplication"];
            this.userDatabasePages = ko.observableArray(["Database Settings", "Custom Functions", "Studio Config"]);
            this.appUrls = appUrl.forCurrentDatabase();
            var databaseSettingsRoute = { route: ['databases/settings', 'databases/settings/databaseSettings'], moduleId: 'viewmodels/database/settings/databaseSettings', title: 'Database Settings', nav: true, hash: appUrl.forCurrentDatabase().databaseSettings };
            var quotasRoute = { route: 'databases/settings/quotas', moduleId: 'viewmodels/database/settings/quotas', title: 'Quotas', nav: true, hash: appUrl.forCurrentDatabase().quotas };
            var replicationsRoute = { route: 'databases/settings/replication', moduleId: 'viewmodels/database/settings/replications', title: 'Replication', nav: true, hash: appUrl.forCurrentDatabase().replications };
            var sqlReplicationsRoute = { route: 'databases/settings/sqlReplication', moduleId: 'viewmodels/database/settings/sqlReplications', title: 'SQL Replication', nav: true, hash: appUrl.forCurrentDatabase().sqlReplications };
            var editsqlReplicationsRoute = { route: 'databases/settings/editSqlReplication(/:sqlReplicationName)', moduleId: 'viewmodels/database/settings/editSqlReplication', title: 'Edit SQL Replication', nav: true, hash: appUrl.forCurrentDatabase().editSqlReplication };
            var sqlReplicationsConnectionsRoute = { route: 'databases/settings/sqlReplicationConnectionStringsManagement', moduleId: 'viewmodels/database/settings/sqlReplicationConnectionStringsManagement', title: 'SQL Replication Connection Strings', nav: true, hash: appUrl.forCurrentDatabase().sqlReplicationsConnections };
            var versioningRoute = { route: 'databases/settings/versioning', moduleId: 'viewmodels/database/settings/versioning', title: 'Versioning', nav: true, hash: appUrl.forCurrentDatabase().versioning };
            var periodicExportRoute = { route: 'databases/settings/periodicExport', moduleId: 'viewmodels/database/settings/periodicExport', title: 'Periodic Export', nav: true, hash: appUrl.forCurrentDatabase().periodicExport };
            var customFunctionsEditorRoute = { route: 'databases/settings/customFunctionsEditor', moduleId: 'viewmodels/database/settings/customFunctionsEditor', title: 'Custom Functions', nav: true, hash: appUrl.forCurrentDatabase().customFunctionsEditor };
            var databaseStudioConfig = { route: 'databases/settings/databaseStudioConfig', moduleId: 'viewmodels/databaseStudioConfig', title: 'Studio Config', nav: true, hash: appUrl.forCurrentDatabase().databaseStudioConfig };
            this.router = durandalRouter.createChildRouter()
                .map([
                databaseSettingsRoute,
                quotasRoute,
                replicationsRoute,
                sqlReplicationsRoute,
                sqlReplicationsConnectionsRoute,
                editsqlReplicationsRoute,
                versioningRoute,
                periodicExportRoute,
                //scriptedIndexesRoute,
                customFunctionsEditorRoute,
                databaseStudioConfig
            ])
                .buildNavigationModel();
            this.router.guardRoute = function (instance, instruction) { return _this.getValidRoute(instance, instruction); };
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        /**
        * Checks whether the route can be navigated to. Returns true if it can be navigated to, or a redirect URI if it can't be navigated to.
        * This is used for preventing a navigating to system-only pages when the current databagse is non-system, and vice-versa.
        */
        settings.prototype.getValidRoute = function (instance, instruction) {
            var db = this.activeDatabase();
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) > -1;
            var isSqlSubBundle = this.sqlSubBundles.indexOf(bundelName) > -1;
            var isSqlBundleExists = this.userDatabasePages.indexOf("SQL Replication") > -1;
            if (db.isSystem) {
                return appUrl.forDocuments(null, db);
            }
            else if ((isLegalBundelName && isBundleExists == false) || (isSqlSubBundle && isSqlBundleExists == false)) {
                return appUrl.forCurrentDatabase().databaseSettings();
            }
            return true;
        };
        settings.prototype.isEditingSqlReplication = function (navigationalModel, curNavHash) {
            var activeRoute = navigationalModel.first(function (r) { return r.isActive(); });
            if (!!activeRoute && !!curNavHash && !!activeRoute.hash) {
                return curNavHash.indexOf('databases/settings/sqlReplication') >= 0 &&
                    (activeRoute.route.indexOf('databases/settings/editSqlReplication') >= 0 ||
                        activeRoute.route.indexOf('databases/settings/sqlReplicationConnectionStringsManagement') >= 0);
            }
            return false;
        };
        settings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.userDatabasePages(["Database Settings", "Custom Functions", "Studio Config"]);
            var db = this.activeDatabase();
            var bundles = db.activeBundles();
            bundles.forEach(function (bundle) {
                var bundleName = _this.bundleMap[bundle.toLowerCase()];
                if (bundleName != undefined) {
                    _this.userDatabasePages.push(bundleName);
                }
            });
            // RavenDB-3640 Allow to enable replication to existing db in studio
            // even if replication isn't enabled we display button to enable it
            this.userDatabasePages.push("Replication");
        };
        settings.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;
            if (this.userDatabasePages.indexOf(bundleTitle) !== -1) {
                // Database Settings, Quotas, Replication, SQL Replication, Versioning, Periodic Export and Scripted Index are visible only when we're on a user database.
                return true;
            }
            return false;
        };
        return settings;
    })(viewModelBase);
    return settings;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/sqlReplicationConnectionStringsManagement", ["require", "exports", "viewmodels/viewModelBase", "models/database/sqlReplication/sqlReplicationConnections", "models/database/sqlReplication/predefinedSqlConnection", "models/database/documents/document", "commands/database/documents/saveDocumentCommand", "common/appUrl", "commands/database/globalConfig/getEffectiveSqlReplicationConnectionStringsCommand", "common/messagePublisher", "commands/database/documents/deleteDocumentCommand"], function (require, exports, viewModelBase, sqlReplicationConnections, predefinedSqlConnection, document, saveDocumentCommand, appUrl, getEffectiveSqlReplicationConnectionStringsCommand, messagePublisher, deleteDocumentCommand) {
    var sqlReplicationConnectionStringsManagement = (function (_super) {
        __extends(sqlReplicationConnectionStringsManagement, _super);
        function sqlReplicationConnectionStringsManagement() {
            _super.call(this);
            this.htmlSelector = "#sqlReplicationConnectionsManagement";
            this.connections = ko.observable();
            this.usingGlobal = ko.observable(false);
            this.hasGlobalValues = ko.observable(false);
        }
        sqlReplicationConnectionStringsManagement.prototype.loadConnections = function () {
            var _this = this;
            return new getEffectiveSqlReplicationConnectionStringsCommand(this.activeDatabase())
                .execute()
                .done(function (repSetup) {
                _this.usingGlobal(repSetup.GlobalExists && !repSetup.LocalExists);
                _this.hasGlobalValues(repSetup.GlobalExists);
                _this.connections(new sqlReplicationConnections(repSetup));
                if (_this.connections().predefinedConnections().length > 0) {
                    _this.connections().predefinedConnections().forEach(function (x) { return _this.subscribeToSqlReplicationConnectionName(x); });
                }
            })
                .fail(function () {
                _this.connections(sqlReplicationConnections.empty());
            });
        };
        sqlReplicationConnectionStringsManagement.prototype.canActivate = function () {
            var def = $.Deferred();
            this.loadConnections()
                .always(function () { return def.resolve({ can: true }); });
            return def;
        };
        sqlReplicationConnectionStringsManagement.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.connections]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        sqlReplicationConnectionStringsManagement.prototype.save = function () {
            var _this = this;
            if (this.usingGlobal()) {
                new deleteDocumentCommand("Raven/SqlReplication/Connections", this.activeDatabase())
                    .execute()
                    .done(function () {
                    messagePublisher.reportSuccess("Settings were successfully saved!");
                    _this.dirtyFlag().reset();
                })
                    .fail(function (response) { return messagePublisher.reportError("Failed to save settings!", response.responseText, response.statusText); });
            }
            else {
                var newDoc = new document(this.connections().toDto());
                this.attachReservedMetaProperties("Raven/SqlReplication/Connections", newDoc.__metadata);
                var saveCommand = new saveDocumentCommand("Raven/SqlReplication/Connections", newDoc, this.activeDatabase());
                var saveTask = saveCommand.execute();
                saveTask.done(function () { return _this.dirtyFlag().reset(); });
            }
        };
        sqlReplicationConnectionStringsManagement.prototype.attachReservedMetaProperties = function (id, target) {
            target.etag = '';
            target.ravenEntityName = !target.ravenEntityName ? document.getEntityNameFromId(id) : target.ravenEntityName;
            target.id = id;
        };
        sqlReplicationConnectionStringsManagement.prototype.getSqlReplicationConnectionsUrl = function () {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        sqlReplicationConnectionStringsManagement.prototype.addSqlReplicationConnection = function () {
            var newPredefinedConnection = predefinedSqlConnection.empty();
            this.connections().predefinedConnections.splice(0, 0, newPredefinedConnection);
            this.subscribeToSqlReplicationConnectionName(newPredefinedConnection);
            newPredefinedConnection.name("New");
        };
        sqlReplicationConnectionStringsManagement.prototype.removeSqlReplicationConnection = function (connection) {
            this.connections().predefinedConnections.remove(connection);
        };
        sqlReplicationConnectionStringsManagement.prototype.subscribeToSqlReplicationConnectionName = function (con) {
            var _this = this;
            con.name.subscribe(function (previousName) {
                //Get the previous value of 'name' here before it's set to newValue
                var nameInputArray = $('input[name="name"]')
                    .each(function (index, inputField) {
                    inputField.setCustomValidity("");
                });
            }, this, "beforeChange");
            con.name.subscribe(function (newName) {
                var message = "";
                if (newName === "") {
                    message = "Please fill out this field.";
                }
                else if (_this.isSqlPredefinedConnectionNameExists(newName)) {
                    message = "SQL Replication Connection name already exists.";
                }
                $('input[name="name"]')
                    .filter(function () { return this.value === newName; })
                    .each(function (index, element) {
                    element.setCustomValidity(message);
                });
            });
        };
        sqlReplicationConnectionStringsManagement.prototype.isSqlPredefinedConnectionNameExists = function (connectionName) {
            if (this.connections().predefinedConnections().count(function (x) { return x.name() == connectionName; }) > 1) {
                return true;
            }
            return false;
        };
        sqlReplicationConnectionStringsManagement.prototype.providerChanged = function (obj, event) {
            if (event.originalEvent) {
                var curConnectionString = !!obj.connectionString() ? obj.connectionString().trim() : "";
                if (curConnectionString === "" ||
                    sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ConnectionString == curConnectionString; })) {
                    var matchingConnectionStringPair = sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ProviderName == event.originalEvent.srcElement.selectedOptions[0].value; });
                    if (!!matchingConnectionStringPair) {
                        var matchingConnectionStringValue = matchingConnectionStringPair.ConnectionString;
                        obj.connectionString(matchingConnectionStringValue);
                    }
                }
            }
        };
        sqlReplicationConnectionStringsManagement.prototype.override = function (value, config) {
            config.hasLocal(value);
            if (!config.hasLocal()) {
                config.copyFromGlobal();
            }
        };
        sqlReplicationConnectionStringsManagement.prototype.useLocal = function () {
            this.usingGlobal(false);
        };
        sqlReplicationConnectionStringsManagement.prototype.useGlobal = function () {
            this.usingGlobal(true);
            this.connections().copyFromParent();
        };
        return sqlReplicationConnectionStringsManagement;
    })(viewModelBase);
    return sqlReplicationConnectionStringsManagement;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/sqlReplications", ["require", "exports", "viewmodels/viewModelBase", "commands/database/sqlReplication/getSqlReplicationsCommand", "common/appUrl", "viewmodels/database/status/sqlReplicationStatsDialog", "durandal/app", "models/database/documents/document", "viewmodels/common/deleteDocuments", "common/messagePublisher", "commands/database/sqlReplication/resetSqlReplicationCommand"], function (require, exports, viewModelBase, getSqlReplicationsCommand, appUrl, sqlReplicationStatsDialog, app, document, deleteDocuments, messagePublisher, resetSqlReplicationCommand) {
    var sqlReplications = (function (_super) {
        __extends(sqlReplications, _super);
        function sqlReplications() {
            _super.call(this);
            this.replications = ko.observableArray();
            this.itemNumber = function (index) {
                return index + 1;
            };
        }
        sqlReplications.prototype.showStats = function (replicationName) {
            var viewModel = new sqlReplicationStatsDialog(this.activeDatabase(), replicationName);
            app.showDialog(viewModel);
        };
        sqlReplications.prototype.getSqlReplicationUrl = function (sqlReplicationName) {
            return appUrl.forEditSqlReplication(sqlReplicationName, this.activeDatabase());
        };
        sqlReplications.prototype.getSqlReplicationConnectionStringsUrl = function (sqlReplicationName) {
            return appUrl.forSqlReplicationConnections(this.activeDatabase());
        };
        sqlReplications.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchSqlReplications(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(_this.activeDatabase()) }); });
            }
            return deferred;
        };
        sqlReplications.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('J9VD59');
        };
        sqlReplications.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
        };
        sqlReplications.prototype.removeSqlReplication = function (sr) {
            var _this = this;
            var newDoc = new document(sr);
            if (newDoc) {
                var viewModel = new deleteDocuments([newDoc]);
                viewModel.deletionTask.done(function () {
                    _this.fetchSqlReplications(_this.activeDatabase());
                });
                app.showDialog(viewModel, sqlReplications.sqlReplicationsSelector);
            }
        };
        sqlReplications.prototype.resetSqlReplication = function (replicationId) {
            var _this = this;
            app.showMessage("You are about to reset this SQL Replication, forcing replication of all collection items", "SQL Replication Reset", ["Cancel", "Reset"])
                .then(function (dialogResult) {
                if (dialogResult === "Reset") {
                    new resetSqlReplicationCommand(_this.activeDatabase(), replicationId).execute()
                        .done(function () { return messagePublisher.reportSuccess("SQL replication " + replicationId + " was reset successfully!"); })
                        .fail(function () { return messagePublisher.reportError("SQL replication " + replicationId + " failed to reset!"); });
                }
            });
        };
        sqlReplications.prototype.fetchSqlReplications = function (db) {
            var _this = this;
            return new getSqlReplicationsCommand(db)
                .execute()
                .done(function (results) {
                _this.replications(results);
            });
        };
        sqlReplications.sqlReplicationsSelector = "#sqlReplications";
        return sqlReplications;
    })(viewModelBase);
    return sqlReplications;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/settings/versioning", ["require", "exports", "viewmodels/viewModelBase", "models/database/documents/versioningEntry", "common/appUrl", "commands/database/documents/saveVersioningCommand", "commands/database/globalConfig/getEffectiveVersioningsCommand", "models/database/globalConfig/configurationDocument"], function (require, exports, viewModelBase, versioningEntry, appUrl, saveVersioningCommand, getEffectiveVersioningsCommand, configurationDocument) {
    var versioning = (function (_super) {
        __extends(versioning, _super);
        function versioning() {
            _super.call(this);
            this.versionings = ko.observableArray().extend({ required: true });
            this.usingGlobal = ko.observable(false);
            this.hasGlobalValues = ko.observable(false);
            this.versionings = ko.observableArray();
        }
        versioning.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchVersioningEntries(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
            }
            return deferred;
        };
        versioning.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("1UZ5WL");
            this.toRemove = [];
            this.dirtyFlag = new ko.DirtyFlag([this.versionings, this.usingGlobal]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        versioning.prototype.fetchVersioningEntries = function (db) {
            var _this = this;
            var task = new getEffectiveVersioningsCommand(db).execute();
            task.done(function (versionings) { return _this.versioningsLoaded(versionings); });
            return task;
        };
        versioning.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var saveTask = new saveVersioningCommand(db, this.versionings().filter(function (v) { return v.localExists(); }).map(function (v) { return v.mergedDocument().toDto(true); }), this.toRemove.map(function (v) { return v.mergedDocument().toDto(true); })).execute();
                saveTask.done(function (saveResult) {
                    _this.versioningsSaved(saveResult);
                });
            }
        };
        versioning.prototype.createNewVersioning = function () {
            this.versionings.push(new configurationDocument({
                GlobalExists: false,
                GlobalDocument: null,
                LocalExists: true,
                MergedDocument: new versioningEntry(),
                Etag: null,
                Metadata: null
            }));
        };
        versioning.prototype.removeVersioning = function (entry) {
            if (entry.mergedDocument().fromDatabase()) {
                // If this entry is in database schedule the removal
                this.toRemove.push(entry);
            }
            this.versionings.remove(entry);
        };
        versioning.prototype.versioningsLoaded = function (data) {
            this.versionings(data);
            this.hasGlobalValues(!!data.first(function (config) { return config.globalExists(); }));
            this.usingGlobal(this.hasGlobalValues() && !data.first(function (config) { return config.localExists(); }));
            this.dirtyFlag().reset();
        };
        versioning.prototype.versioningsSaved = function (saveResult) {
            var locals = this.versionings().filter(function (v) { return v.localExists(); });
            for (var i = 0; i < locals.length; i++) {
                locals[i].mergedDocument().__metadata.etag = saveResult[i].Etag;
            }
            // After save the collection names are not editable
            this.versionings().forEach(function (c) {
                c.mergedDocument().fromDatabase(true);
            });
            this.dirtyFlag().reset();
            this.toRemove = [];
        };
        versioning.prototype.override = function (value, config) {
            config.localExists(value);
            if (!config.localExists()) {
                this.toRemove.push(config);
                config.copyFromGlobal();
            }
            else {
                this.toRemove.remove(config);
            }
        };
        versioning.prototype.useLocal = function () {
            this.usingGlobal(false);
        };
        versioning.prototype.useGlobal = function () {
            this.usingGlobal(true);
            this.toRemove.pushAll(this.versionings().filter(function (c) { return c.localExists(); }));
            var newVersionsings = this.versionings().filter(function (c) { return c.globalExists(); });
            newVersionsings.forEach(function (v) { return v.copyFromGlobal(); });
            this.versionings(newVersionsings);
        };
        return versioning;
    })(viewModelBase);
    return versioning;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/alerts", ["require", "exports", "plugins/router", "commands/operations/getAlertsCommand", "viewmodels/viewModelBase", "moment", "models/database/debug/alert", "common/appUrl", "commands/operations/saveAlertsCommand", "common/bindingHelpers/autoRefreshBindingHandler"], function (require, exports, router, getAlertsCommand, viewModelBase, moment, alert, appUrl, saveAlertsCommand, autoRefreshBindingHandler) {
    var alerts = (function (_super) {
        __extends(alerts, _super);
        function alerts() {
            var _this = this;
            _super.call(this);
            this.alertDoc = ko.observable();
            this.allAlerts = ko.observableArray();
            this.filterLevel = ko.observable("All");
            this.selectedAlert = ko.observable();
            this.selectedAlertIndex = ko.observable();
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.autoRefreshEnabled = ko.observable(true);
            autoRefreshBindingHandler.install();
            this.unreadAlertCount = ko.computed(function () { return _this.allAlerts().count(function (a) { return !a.observed(); }); });
            this.readAlertCount = ko.computed(function () { return _this.allAlerts().count(function (a) { return a.observed(); }); });
            this.updateCurrentNowTime();
            this.activeDatabase.subscribe(function () { return _this.fetchAlerts(); });
        }
        alerts.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.fetchAlerts();
            var item = !!args.item && !isNaN(args.item) ? args.item : 0;
            this.updateHelpLink('HL46QE');
            this.selectedAlertIndex(item);
        };
        alerts.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
        };
        alerts.prototype.fetchAlerts = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getAlertsCommand(db)
                    .execute()
                    .done(function (result) { return _this.processAlertsResults(result); });
            }
            return null;
        };
        alerts.prototype.processAlertsResults = function (result) {
            var _this = this;
            var now = moment();
            var alerts = result.Alerts.map(function (a) { return new alert(a); });
            alerts.forEach(function (r) {
                r.createdAtHumanized = _this.createHumanReadableTime(r.createdAt),
                    r.isVisible = ko.computed(function () { return _this.matchesFilter(r); });
            });
            this.alertDoc(result);
            this.allAlerts(alerts);
            if (alerts.length > 0) {
                this.selectAlert(alerts[this.selectedAlertIndex()]);
            }
        };
        alerts.prototype.matchesFilter = function (a) {
            if (this.filterLevel() === "All") {
                return true;
            }
            var unreadFilterWithUnreadAlert = this.filterLevel() === "Unread" && !a.observed();
            var readFilterWithReadAlert = this.filterLevel() === "Read" && a.observed();
            return unreadFilterWithUnreadAlert || readFilterWithReadAlert;
        };
        alerts.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                // Return a computed that returns a humanized string based off the current time, e.g. "7 minutes ago".
                // It's a computed so that it updates whenever we update this.now (scheduled to occur every minute.)
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        alerts.prototype.selectAlert = function (selection) {
            var index = this.allAlerts.indexOf(selection);
            this.selectedAlertIndex(index);
            this.selectedAlert(selection);
            var alertUrl = appUrl.forAlerts(this.activeDatabase()) + "&item=" + this.selectedAlertIndex();
            router.navigate(alertUrl, false);
        };
        alerts.prototype.tableKeyDown = function (sender, e) {
            var isKeyUp = e.keyCode === 38;
            var isKeyDown = e.keyCode === 40;
            if (isKeyUp || isKeyDown) {
                e.preventDefault();
                var oldSelection = this.selectedAlert();
                if (oldSelection) {
                    var oldSelectionIndex = this.allAlerts.indexOf(oldSelection);
                    var newSelectionIndex = oldSelectionIndex;
                    if (isKeyUp && oldSelectionIndex > 0) {
                        newSelectionIndex--;
                    }
                    else if (isKeyDown && oldSelectionIndex < this.allAlerts().length - 1) {
                        newSelectionIndex++;
                    }
                    this.selectAlert(this.allAlerts()[newSelectionIndex]);
                    var newSelectedRow = $("#alertsContainer table tbody tr:nth-child(" + (newSelectionIndex + 1) + ")");
                    if (newSelectedRow) {
                        this.ensureRowVisible(newSelectedRow);
                    }
                }
            }
        };
        alerts.prototype.ensureRowVisible = function (row) {
            var table = $("#alertTableContainer");
            var scrollTop = table.scrollTop();
            var scrollBottom = scrollTop + table.height();
            var scrollHeight = scrollBottom - scrollTop;
            var rowPosition = row.position();
            var rowTop = rowPosition.top;
            var rowBottom = rowTop + row.height();
            if (rowTop < 0) {
                table.scrollTop(scrollTop + rowTop);
            }
            else if (rowBottom > scrollHeight) {
                table.scrollTop(scrollTop + (rowBottom - scrollHeight));
            }
        };
        alerts.prototype.setFilterAll = function () {
            this.filterLevel("All");
        };
        alerts.prototype.setFilterUnread = function () {
            this.filterLevel("Unread");
        };
        alerts.prototype.setFilterRead = function () {
            this.filterLevel("Read");
        };
        alerts.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        alerts.prototype.toggleSelectedReadState = function () {
            this.disableAutoRefresh();
            var alert = this.selectedAlert();
            if (alert) {
                if (!alert.observed()) {
                    alert.lastDismissedAt = this.now().toISOString();
                }
                alert.observed(!alert.observed());
            }
        };
        alerts.prototype.disableAutoRefresh = function () {
            this.autoRefreshEnabled(false);
        };
        alerts.prototype.deleteSelectedAlert = function () {
            this.disableAutoRefresh();
            var alert = this.selectedAlert();
            if (alert) {
                this.allAlerts.remove(alert);
            }
        };
        alerts.prototype.deleteReadAlerts = function () {
            this.disableAutoRefresh();
            this.allAlerts.remove(function (a) { return a.observed(); });
        };
        alerts.prototype.deleteAllAlerts = function () {
            this.disableAutoRefresh();
            this.allAlerts.removeAll();
        };
        alerts.prototype.saveAlerts = function () {
            var alertDoc = this.alertDoc();
            var db = this.activeDatabase();
            if (alertDoc && db) {
                alertDoc.Alerts = this.allAlerts().map(function (a) { return a.toDto(); });
                new saveAlertsCommand(alertDoc, db).execute();
            }
        };
        return alerts;
    })(viewModelBase);
    return alerts;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexErrors", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "common/changesContext", "moment"], function (require, exports, viewModelBase, getDatabaseStatsCommand, changesContext, moment) {
    var indexErrors = (function (_super) {
        __extends(indexErrors, _super);
        function indexErrors() {
            _super.call(this);
            this.allIndexErrors = ko.observableArray();
            this.hasFetchedErrors = ko.observable(false);
            this.selectedIndexError = ko.observable();
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.updateCurrentNowTime();
        }
        indexErrors.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.fetchIndexErrors(); })
            ];
        };
        indexErrors.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('ABUXGF');
            this.fetchIndexErrors();
        };
        indexErrors.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
        };
        indexErrors.prototype.fetchIndexErrors = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                // Index errors are actually the .ServerErrors returned from the database statistics query.
                return new getDatabaseStatsCommand(db)
                    .execute()
                    .done(function (stats) {
                    stats.Errors.forEach(function (e) { return e['TimestampHumanized'] = _this.createHumanReadableTime(e.Timestamp); });
                    _this.allIndexErrors(stats.Errors);
                    _this.hasFetchedErrors(true);
                });
            }
            return null;
        };
        indexErrors.prototype.tableKeyDown = function () {
        };
        indexErrors.prototype.selectIndexError = function (indexError) {
            this.selectedIndexError(indexError);
        };
        indexErrors.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                // Return a computed that returns a humanized string based off the current time, e.g. "7 minutes ago".
                // It's a computed so that it updates whenever we update this.now field.
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        indexErrors.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        return indexErrors;
    })(viewModelBase);
    return indexErrors;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/logs", ["require", "exports", "commands/database/debug/getLogsCommand", "viewmodels/viewModelBase", "moment", "models/database/documents/document", "common/bindingHelpers/autoRefreshBindingHandler", "models/database/debug/logEntry"], function (require, exports, getLogsCommand, viewModelBase, moment, document, autoRefreshBindingHandler, logEntry) {
    var logs = (function (_super) {
        __extends(logs, _super);
        function logs() {
            var _this = this;
            _super.call(this);
            this.allLogs = ko.observableArray();
            this.filterLevel = ko.observable("All");
            this.selectedLog = ko.observable();
            this.searchText = ko.observable("");
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.filteredLoggers = ko.observableArray();
            this.sortColumn = ko.observable("timeStamp");
            this.sortAsc = ko.observable(true);
            this.showLogDetails = ko.observable(false);
            autoRefreshBindingHandler.install();
            this.debugLogCount = ko.pureComputed(function () { return _this.allLogs().count(function (l) { return l.level() === "Debug"; }); });
            this.infoLogCount = ko.pureComputed(function () { return _this.allLogs().count(function (l) { return l.level() === "Info"; }); });
            this.warningLogCount = ko.pureComputed(function () { return _this.allLogs().count(function (l) { return l.level() === "Warn"; }); });
            this.errorLogCount = ko.pureComputed(function () { return _this.allLogs().count(function (l) { return l.level() === "Error"; }); });
            this.fatalLogCount = ko.pureComputed(function () { return _this.allLogs().count(function (l) { return l.level() === "Fatal"; }); });
            this.searchTextThrottled = this.searchText.throttle(400);
            this.activeDatabase.subscribe(function () { return _this.fetchLogs(); });
            this.updateCurrentNowTime();
            this.sortColumn.subscribe(function () { return _this.sortInPlace(); });
            this.sortAsc.subscribe(function () { return _this.sortInPlace(); });
        }
        logs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.columnWidths = [
                ko.observable(100),
                ko.observable(265),
                ko.observable(300),
                ko.observable(200),
                ko.observable(360)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('3Z9IJS');
            return this.fetchLogs();
        };
        logs.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.showLogDetails.subscribe(function () {
                $(".logRecords").toggleClass("logRecords-small");
            });
            $("#logRecordsContainer").width();
            var widthUnit = 0.08;
            this.columnWidths[0](100 * widthUnit);
            this.columnWidths[1](100 * widthUnit);
            this.columnWidths[2](100 * widthUnit * 6);
            this.columnWidths[3](100 * widthUnit * 2);
            this.columnWidths[4](100 * widthUnit * 2);
        };
        logs.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
            this.unregisterColumnResizing();
        };
        logs.prototype.fetchLogs = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var deferred = $.Deferred();
                new getLogsCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.processLogResults(results);
                    deferred.resolve(results);
                });
                return deferred;
            }
            return null;
        };
        logs.prototype.processLogResults = function (results) {
            var _this = this;
            var mappedResults = results.map(function (r) {
                var mapped = new logEntry(r, _this.now);
                mapped['isVisible'] = ko.pureComputed(function () {
                    var matchesSearch = _this.matchesFilterAndSearch(mapped);
                    var matchesFilters = _this.filteredLoggers().contains(mapped.loggerName());
                    return matchesSearch && !matchesFilters;
                });
                return mapped;
            });
            var sortedResults = this.sortResults(mappedResults.reverse());
            var allLogsRaw = this.allLogs();
            if (sortedResults.length === allLogsRaw.length) {
                for (var i = 0; i < sortedResults.length; i++) {
                    allLogsRaw[i].copyFrom(sortedResults[i]);
                }
            }
            else {
                this.allLogs(sortedResults);
            }
        };
        logs.prototype.sortInPlace = function () {
            var dataToSort = this.sortResults(this.allLogs());
            this.allLogs(dataToSort);
        };
        logs.prototype.sortResults = function (dataToSort) {
            var column = this.sortColumn();
            var asc = this.sortAsc();
            var test = asc ? (function (l, r) { return l < r; }) : (function (l, r) { return l > r; });
            var sortFunc = function (left, right) {
                if (left[column]() === right[column]()) {
                    return 0;
                }
                return test(left[column](), right[column]()) ? 1 : -1;
            };
            return dataToSort.sort(sortFunc);
        };
        logs.prototype.matchesFilterAndSearch = function (log) {
            var searchTextThrottled = this.searchTextThrottled().toLowerCase();
            var filterLevel = this.filterLevel();
            var matchesLogLevel = filterLevel === "All" || log.level() === filterLevel;
            var matchesSearchText = !searchTextThrottled ||
                (log.message() && log.message().toLowerCase().indexOf(searchTextThrottled) >= 0) ||
                (log.exception() && log.exception().toLowerCase().indexOf(searchTextThrottled) >= 0);
            return matchesLogLevel && matchesSearchText;
        };
        logs.prototype.selectLog = function (log) {
            this.selectedLog(log);
            this.showLogDetails(true);
            $(".logRecords").addClass("logRecords-small");
        };
        logs.prototype.unSelectLog = function (log) {
            this.selectedLog(null);
            this.showLogDetails(false);
        };
        logs.prototype.tableKeyDown = function (sender, e) {
            var isKeyUp = e.keyCode === 38;
            var isKeyDown = e.keyCode === 40;
            if (isKeyUp || isKeyDown) {
                e.preventDefault();
                var oldSelection = this.selectedLog();
                if (oldSelection) {
                    var oldSelectionIndex = this.allLogs.indexOf(oldSelection);
                    var newSelectionIndex = oldSelectionIndex;
                    if (isKeyUp && oldSelectionIndex > 0) {
                        newSelectionIndex--;
                    }
                    else if (isKeyDown && oldSelectionIndex < this.allLogs().length - 1) {
                        newSelectionIndex++;
                    }
                    this.selectedLog(this.allLogs()[newSelectionIndex]);
                    var newSelectedRow = $("#logRecords > div:nth-child(" + (newSelectionIndex + 1) + ")");
                    if (newSelectedRow) {
                        this.ensureRowVisible(newSelectedRow);
                    }
                }
            }
            return true;
        };
        logs.prototype.ensureRowVisible = function (row) {
            var table = $("#logRecords");
            var scrollTop = table.scrollTop();
            var scrollBottom = scrollTop + table.height();
            var scrollHeight = scrollBottom - scrollTop;
            var rowPosition = row.position();
            var rowTop = rowPosition.top;
            var rowBottom = rowTop + row.height();
            if (rowTop < 0) {
                table.scrollTop(scrollTop + rowTop);
            }
            else if (rowBottom > scrollHeight) {
                table.scrollTop(scrollTop + (rowBottom - scrollHeight));
            }
        };
        logs.prototype.setFilterAll = function () {
            this.filterLevel("All");
        };
        logs.prototype.setFilterDebug = function () {
            this.filterLevel("Debug");
        };
        logs.prototype.setFilterInfo = function () {
            this.filterLevel("Info");
        };
        logs.prototype.setFilterWarning = function () {
            this.filterLevel("Warn");
        };
        logs.prototype.setFilterError = function () {
            this.filterLevel("Error");
        };
        logs.prototype.setFilterFatal = function () {
            this.filterLevel("Fatal");
        };
        logs.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        logs.prototype.hideLogType = function (log) {
            if (!this.filteredLoggers.contains(log.loggerName())) {
                this.filteredLoggers.push(log.loggerName());
            }
        };
        logs.prototype.unHidelogType = function (loggerName) {
            if (this.filteredLoggers.contains(loggerName)) {
                this.filteredLoggers.remove(loggerName);
            }
        };
        logs.prototype.sortBy = function (columnName, logs, event) {
            if (this.sortColumn() === columnName) {
                this.sortAsc(!this.sortAsc());
            }
            else {
                this.sortColumn(columnName);
                this.sortAsc(true);
            }
        };
        logs.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var logsRecordsContainerWidth = $("#logRecordsContainer").width();
                    var targetColumnSize = startingWidth + 100 * (e.pageX - startX) / logsRecordsContainerWidth;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        logs.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.logTableColumnResize");
            $(document).off("mouseup.logTableColumnResize");
            $(document).off("mousemove.logTableColumnResize");
        };
        return logs;
    })(viewModelBase);
    return logs;
});
/// <reference path="../../../../Scripts/typings/d3/dagre.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/replicationStats", ["require", "exports", "viewmodels/viewModelBase", "commands/database/replication/getReplicationStatsCommand", "moment", "common/generalUtils", "common/svgDownloader", "common/fileDownloader", "commands/resources/getDatabaseSettingsCommand", "commands/database/replication/getReplicationTopology", "commands/database/debug/getReplicationPerfStatsCommand", 'd3/d3', 'nvd3', 'dagre'], function (require, exports, viewModelBase, getReplicationStatsCommand, moment, generalUtils, svgDownloader, fileDownloader, getDatabaseSettingsCommand, getReplicationTopology, getReplicationPerfStatsCommand, d3, nv, dagre) {
    var replicationStats = (function (_super) {
        __extends(replicationStats, _super);
        function replicationStats() {
            _super.call(this);
            this.panelBodyPadding = 20;
            this.topology = ko.observable(null);
            this.currentLink = ko.observable(null);
            this.hasReplicationEnabled = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.replStatsDoc = ko.observable();
            this.hasNoReplStatsAvailable = ko.observable(false);
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.colors = d3.scale.category10();
            this.line = d3.svg.line().x(function (d) { return d.x; }).y(function (d) { return d.y; });
            // perf stats related variables start
            this.jsonData = [];
            this.rawJsonData = [];
            this.hiddenNames = d3.set([]);
            this.destinations = [];
            this.margin = { top: 40, right: 20, bottom: 20, left: 40 };
            this.barWidth = 30;
            this.barPadding = 15;
            this.legendWidth = 0;
            this.isoFormat = d3.time.format.iso;
            this.xTickFormat = d3.time.format("%H:%M:%S");
            this.color = d3.scale.category20();
            // perf stats related variables end
            this.hasSaveAsPngSupport = ko.computed(function () {
                return !(navigator && navigator.msSaveBlob);
            });
            this.updateCurrentNowTime();
        }
        replicationStats.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () {
                _this.fetchReplStats();
                _this.checkIfHasReplicationEnabled();
            });
            this.updateHelpLink('ES8PCB');
            this.fetchReplStats();
        };
        replicationStats.prototype.checkIfHasReplicationEnabled = function () {
            var _this = this;
            new getDatabaseSettingsCommand(this.activeDatabase())
                .execute()
                .done(function (document) {
                var documentSettings = document.Settings["Raven/ActiveBundles"];
                _this.hasReplicationEnabled(documentSettings.indexOf("Replication") !== -1);
            });
        };
        replicationStats.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            d3.select(window).on("resize", this.resize.bind(this));
            $("#replicationTopologySection").scroll(function () { return _this.graphScrolled(); });
            this.checkIfHasReplicationEnabled();
            $("#replicationStatsContainer").scroll(function () { return _this.graphScrolled(); });
            this.refresh();
        };
        replicationStats.prototype.compositionComplete = function () {
            this.resize();
        };
        replicationStats.prototype.resize = function () {
            this.width = $("#replicationTopologySection").width() * 0.6;
            this.onWindowHeightChanged();
        };
        replicationStats.prototype.fetchReplStats = function () {
            var _this = this;
            this.replStatsDoc(null);
            new getReplicationStatsCommand(this.activeDatabase())
                .execute()
                .fail(function () { return _this.hasNoReplStatsAvailable(true); }) // If replication is not setup, the fetch will fail with 404.
                .done(function (result) {
                _this.hasNoReplStatsAvailable(result.Stats.length === 0);
                _this.processResults(result);
            });
        };
        replicationStats.prototype.processResults = function (results) {
            var _this = this;
            if (results) {
                results.Stats.forEach(function (s) {
                    s['LastReplicatedLastModifiedHumanized'] = _this.createHumanReadableTime(s.LastReplicatedLastModified);
                    s['LastFailureTimestampHumanized'] = _this.createHumanReadableTime(s.LastFailureTimestamp);
                    s['LastHeartbeatReceivedHumanized'] = _this.createHumanReadableTime(s.LastHeartbeatReceived);
                    s['LastSuccessTimestampHumanized'] = _this.createHumanReadableTime(s.LastSuccessTimestamp);
                    s['isHotFailure'] = _this.isFailEarlierThanSuccess(s.LastFailureTimestamp, s.LastSuccessTimestamp);
                });
            }
            this.replStatsDoc(results);
        };
        replicationStats.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                // Return a computed that returns a humanized string based off the current time, e.g. "7 minutes ago".
                // It's a computed so that it updates whenever we update this.now (scheduled to occur every minute.)
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }
            return ko.computed(function () { return time; });
        };
        replicationStats.prototype.isFailEarlierThanSuccess = function (lastFailureTime, lastSuccessTime) {
            if (!!lastFailureTime) {
                if (!!lastSuccessTime) {
                    return lastFailureTime >= lastSuccessTime;
                }
                else {
                    return true;
                }
            }
            return false;
        };
        replicationStats.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        replicationStats.prototype.createReplicationTopology = function () {
            var self = this;
            this.height = 600;
            this.svg = d3.select("#replicationTopology")
                .style({ height: self.height + 'px' })
                .style({ width: self.width + 'px' })
                .attr("viewBox", "0 0 " + self.width + " " + self.height);
            var nodes = this.topology().Servers;
            var topologyGraph = new dagre.graphlib.Graph();
            topologyGraph.setGraph({});
            topologyGraph.setDefaultEdgeLabel(function () { return ""; });
            this.topology().Servers.forEach(function (s) {
                topologyGraph.setNode(s, {
                    id: s,
                    width: 200,
                    height: 40
                });
            });
            this.topology().Connections.forEach(function (c) {
                topologyGraph.setEdge(c.Source, c.Destination, c);
            });
            dagre.layout(topologyGraph);
            this.renderTopology(topologyGraph);
        };
        replicationStats.prototype.linkWithArrow = function (d) {
            var self = this;
            var pointCount = d.points.length;
            var d90 = Math.PI / 2, sourceX = d.points[pointCount - 2].x, sourceY = d.points[pointCount - 2].y, targetX = d.points[pointCount - 1].x, targetY = d.points[pointCount - 1].y;
            var theta = Math.atan2(targetY - sourceY, targetX - sourceX);
            return self.line(d.points) +
                "M" + targetX + "," + targetY +
                "l" + (3.5 * Math.cos(d90 - theta) - 10 * Math.cos(theta)) + "," + (-3.5 * Math.sin(d90 - theta) - 10 * Math.sin(theta)) +
                "L" + (targetX - 3.5 * Math.cos(d90 - theta) - 10 * Math.cos(theta)) + "," + (targetY + 3.5 * Math.sin(d90 - theta) - 10 * Math.sin(theta)) + "z";
        };
        replicationStats.prototype.linkHasError = function (d) {
            return d.SourceToDestinationState !== "Online";
        };
        replicationStats.prototype.renderTopology = function (topologyGraph) {
            var self = this;
            var enteringGraph = this.svg.selectAll('.graph').data([null]).enter().append('g').attr('class', 'graph');
            enteringGraph.append('g').attr('class', 'nodes');
            enteringGraph.append('g').attr('class', 'edges');
            enteringGraph.attr('transform', 'translate(' + ((self.width - topologyGraph.graph().width) / 2) + ',10)');
            var mappedNodes = topologyGraph.nodes().map(function (n) { return topologyGraph.node(n); });
            var nodesDom = this.svg.select('.nodes').selectAll('.node').data(mappedNodes, function (d) { return d.id; });
            var nGroup = nodesDom.enter()
                .append('g')
                .attr('class', 'node')
                .attr('transform', function (d) { return 'translate(' + d.x + ',' + d.y + ')'; });
            nGroup.append('svg:rect')
                .attr('class', 'node')
                .attr('rx', 5)
                .attr("x", -100)
                .attr("y", -20)
                .attr('width', 200)
                .attr('height', 40);
            nGroup.append('svg:text')
                .attr('x', 0)
                .attr('y', -4)
                .attr('class', 'id')
                .text(function (d) { return d.id.split("/databases/")[0]; });
            nGroup.append('svg:text')
                .attr('x', 0)
                .attr('y', 14)
                .attr('class', 'id')
                .text(function (d) { return d.id.split("/databases/")[1]; });
            var mappedEdges = topologyGraph.edges().map(function (e) { return topologyGraph.edge(e); });
            var edgesDom = this.svg.select('.edges').selectAll('.edge').data(mappedEdges, function (d) { return d.Source + "_" + d.Destination; });
            edgesDom.enter()
                .append('path')
                .attr('class', 'link')
                .classed('error', self.linkHasError)
                .attr('d', function (d) { return self.linkWithArrow(d); })
                .on("click", function (d) {
                var currentSelection = d3.select(".selected").node();
                d3.selectAll(".selected").classed("selected", false);
                d3.select(this).classed('selected', currentSelection != this);
                self.currentLink(currentSelection != this ? d : null);
            });
        };
        replicationStats.prototype.fetchTopology = function () {
            var _this = this;
            this.showLoadingIndicator(true);
            new getReplicationTopology(this.activeDatabase())
                .execute()
                .done(function (topo) {
                _this.topology(topo);
                _this.createReplicationTopology();
                $("#replicationSetupCollapse").addClass("in"); // Force the panel to expand. Fixes a bug where the panel collapses when we fill it with content.
            })
                .always(function () { return _this.showLoadingIndicator(false); });
        };
        replicationStats.prototype.saveAsPng = function () {
            svgDownloader.downloadPng(d3.select('#replicationTopology').node(), 'replicationTopology.png', function () { return replicationStats.inlineCss; });
        };
        replicationStats.prototype.saveAsSvg = function () {
            svgDownloader.downloadSvg(d3.select('#replicationTopology').node(), 'replicationTopology.svg', function () { return replicationStats.inlineCss; });
        };
        replicationStats.prototype.saveAsJson = function () {
            fileDownloader.downloadAsJson(this.topology(), "topology.json");
        };
        replicationStats.prototype.fetchJsonData = function () {
            return new getReplicationPerfStatsCommand(this.activeDatabase()).execute();
        };
        replicationStats.prototype.modelPolling = function () {
            // don't pool if unable to locate element
            var container = $("#replicationStatsContainer");
            if (container.length > 0) {
                return this.refresh();
            }
            return $.Deferred().resolve();
        };
        replicationStats.prototype.filterJsonData = function () {
            var _this = this;
            this.jsonData = [];
            this.rawJsonData.forEach(function (v) {
                var filteredStats = v.Stats.filter(function (s) { return !_this.hiddenNames.has(s.Destination); });
                if (filteredStats.length > 0) {
                    _this.jsonData.push({
                        'Started': v.Started,
                        'Stats': filteredStats
                    });
                }
            });
        };
        replicationStats.prototype.refresh = function () {
            var _this = this;
            return this.fetchJsonData().done(function (data) {
                _this.rawJsonData = _this.mergeJsonData(_this.rawJsonData, data);
                _this.destinations = _this.findDestinations(_this.rawJsonData);
                _this.filterJsonData();
                _this.redrawGraph();
            });
        };
        replicationStats.prototype.mergeJsonData = function (currentData, incomingData) {
            // create lookup map to avoid O(n^2) 
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.Started, i);
            });
            incomingData.forEach(function (d) {
                if (dateLookup.has(d.Started)) {
                    var index = dateLookup.get(d.Started);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        replicationStats.prototype.computeBarWidths = function (data) {
            var _this = this;
            var cumulative = 10;
            var result = data.map(function (perfData) {
                var prevValue = cumulative;
                perfData.sectionWidth = perfData.Stats.length * _this.barWidth + _this.barPadding * 2;
                cumulative += perfData.sectionWidth;
                return prevValue;
            });
            result.push(cumulative);
            return result;
        };
        replicationStats.prototype.graphScrolled = function () {
            var leftScroll = $("#replicationStatsContainer").scrollLeft();
            var self = this;
            this.perfSvg.select('.y.axis')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.perfSvg.select('#dataClip rect')
                .attr('x', leftScroll);
            this.perfSvg.select('.legend_bg_group')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.perfSvg.select('.controlls')
                .selectAll(".legend")
                .attr("transform", function (d, i) { return "translate(" + leftScroll + "," + i * 20 + ")"; });
            nv.tooltip.cleanup();
        };
        replicationStats.prototype.toggleGroupVisible = function (groupName) {
            nv.tooltip.cleanup();
            var alreadyHidden = this.hiddenNames.has(groupName);
            if (alreadyHidden) {
                this.hiddenNames.remove(groupName);
            }
            else {
                this.hiddenNames.add(groupName);
            }
            d3.select('.rect-legend-' + generalUtils.escape(groupName)).classed('legendHidden', !alreadyHidden);
            this.filterJsonData();
            this.redrawGraph();
            // we have to manually trigger on scroll even to fix firefox issue (missing event call)
            this.graphScrolled();
        };
        replicationStats.prototype.redrawGraph = function () {
            var self = this;
            this.perfWidth = $("#replicationTopologySection").width() - this.panelBodyPadding * 2 - this.margin.left - this.margin.right;
            this.perfHeight = $("#replicationStatsContainer").height() - this.margin.top - this.margin.bottom - 20; // substract scroll width
            var cumulativeWidths = this.computeBarWidths(this.jsonData);
            this.x0Scale = d3.scale.ordinal().range(cumulativeWidths);
            this.yScale = d3.scale.linear().range([self.perfHeight, 0]);
            this.xAxis = d3.svg.axis()
                .scale(self.x0Scale)
                .orient("bottom")
                .tickFormat(function (d) { return ""; })
                .tickPadding(20);
            this.yAxis = d3.svg.axis()
                .scale(self.yScale)
                .orient("left")
                .tickFormat(d3.format(".2s"));
            var totalHeight = self.perfHeight + self.margin.top + self.margin.bottom;
            // get higer value from total (visiable and not visible graph width) and viewbox width.
            var totalWidth = Math.max(cumulativeWidths[cumulativeWidths.length - 1], this.perfWidth) + this.margin.left + this.margin.right;
            $("#replicationStatsContainer").css('overflow-x', cumulativeWidths[cumulativeWidths.length - 1] > this.perfWidth ? 'scroll' : 'hidden');
            this.perfSvg = d3.select("#replicationStatsGraph")
                .attr("width", totalWidth)
                .attr("height", totalHeight)
                .style({ height: totalHeight + 'px' })
                .style({ width: totalWidth + 'px' })
                .attr("viewBox", "0 0 " + totalWidth + " " + totalHeight);
            this.perfSvg.selectAll('.main_group')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            this.perfSvg
                .selectAll('defs')
                .data([this.jsonData])
                .enter()
                .append('defs')
                .append('clipPath')
                .attr('id', 'dataClip')
                .append('rect')
                .attr('x', 0)
                .attr('y', 0)
                .attr('width', 1200000)
                .attr('height', 50000);
            var svgEnter = this.perfSvg
                .selectAll(".main_group")
                .data([this.jsonData]).enter();
            svgEnter.append('g')
                .attr('class', 'main_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            var controllsEnter = this.perfSvg
                .selectAll(".controlls")
                .data([this.jsonData]).enter()
                .append("g")
                .attr('class', 'controlls')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            controllsEnter.append("g")
                .attr("class", "x axis");
            controllsEnter.append('g')
                .attr('class', 'y axis');
            controllsEnter.append('g')
                .attr('class', 'legend_bg_group')
                .append('rect')
                .attr('class', 'legend_bg')
                .attr('x', self.perfWidth)
                .attr('y', 0)
                .attr('width', 0)
                .attr('height', 0);
            controllsEnter.select('.y.axis')
                .append("text")
                .attr("transform", "rotate(-90)")
                .attr("y", 6)
                .attr("dy", ".71em")
                .style("text-anchor", "end")
                .text("Batch size");
            this.x0Scale.domain(d3.nest()
                .key(function (d) { return d.Started; })
                .sortKeys(d3.ascending)
                .entries(self.jsonData)
                .map(function (d) { return d.key; }));
            this.yScale.domain([0, d3.max(this.jsonData, function (d) { return d3.max(d.Stats, function (dd) { return dd.BatchSize; }); })]);
            this.perfSvg.select(".x.axis")
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(0," + self.perfHeight + ")")
                .transition()
                .call(self.xAxis);
            this.perfSvg.select('.y.axis')
                .transition()
                .call(self.yAxis);
            var frame = this.perfSvg.select('.main_group').selectAll(".frame")
                .data(self.jsonData, function (d) { return d.Started; });
            frame.exit().remove();
            frame
                .transition()
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frame
                .select('.date_tick')
                .transition()
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.perfHeight + 16);
            var frameEnter = frame.enter()
                .append("g")
                .attr("class", "frame")
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frameEnter.append("text")
                .attr('class', 'date_tick')
                .attr('text-anchor', 'middle')
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.perfHeight + 16)
                .text(function (d) { return self.xTickFormat(self.isoFormat.parse(d.Started)); });
            frameEnter.append("g")
                .attr('class', 'inputs');
            var inputCounts = frame.select('.inputs').selectAll(".inputCounts")
                .data(function (d) { return d.Stats; }, function (d) { return d.Destination; });
            inputCounts.exit().remove();
            inputCounts
                .transition()
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPadding; })
                .attr("y", function (d) { return self.yScale(d.BatchSize); })
                .attr("height", function (d) { return self.perfHeight - self.yScale(d.BatchSize); })
                .style("fill", function (d) { return self.color(d.Destination); });
            inputCounts.enter().append("rect")
                .attr("class", "inputCounts")
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPadding; })
                .attr("y", function (d) { return self.perfHeight; })
                .attr("height", 0)
                .style("fill", function (d) { return self.color(d.Destination); })
                .on('click', function (d) {
                nv.tooltip.cleanup();
                var offset = $(this).offset();
                var leftScroll = $("#replicationStatsContainer").scrollLeft();
                var containerOffset = $("#replicationTopologySection").offset();
                nv.tooltip.show([offset.left - containerOffset.left + leftScroll + self.barWidth, offset.top - containerOffset.top], self.getTooltip(d), 's', 5, document.getElementById("replicationStatsContainer"), "selectable-tooltip");
            })
                .transition()
                .attr("height", function (d) { return self.perfHeight - self.yScale(d.BatchSize); })
                .attr("y", function (d) { return self.yScale(d.BatchSize); });
            this.legend = this.perfSvg.select('.controlls').selectAll(".legend")
                .data(this.destinations, function (d) { return d; });
            this.legend.selectAll("rect").transition()
                .attr("x", this.perfWidth - 18);
            this.legend.selectAll("text").transition()
                .attr("x", this.perfWidth - 24)
                .text(function (d) { return d; });
            var legendEnter = this.legend
                .enter().append("g")
                .attr("class", "legend")
                .attr("transform", function (d, i) { return "translate(0," + i * 20 + ")"; });
            legendEnter.append("rect")
                .attr("x", this.perfWidth - 18)
                .attr('class', function (d) { return 'rect-legend-' + generalUtils.escape(d); })
                .attr("width", 18)
                .attr("height", 18)
                .style("fill", self.color)
                .style("stroke", self.color)
                .on('click', function (d) { return self.toggleGroupVisible(d); });
            legendEnter.append("text")
                .attr("x", this.perfWidth - 24)
                .attr("y", 9)
                .attr("dy", ".35em")
                .style("text-anchor", "end")
                .text(function (d) { return d; });
            this.legendWidth = d3.max($(".legend text"), function (d) { return d.getBBox().width; }) + 40 || 0;
            this.perfSvg.select('.legend_bg')
                .attr('y', -6)
                .attr('height', this.destinations.length * 20 + 10)
                .attr('width', this.legendWidth)
                .attr('x', this.perfWidth - this.legendWidth + 10);
        };
        replicationStats.prototype.onWindowHeightChanged = function () {
            nv.tooltip.cleanup();
            this.perfWidth = $("#replicationTopologySection").width() - this.panelBodyPadding * 2;
            this.perfHeight = $("#replicationStatsContainer").height();
            this.redrawGraph();
        };
        replicationStats.prototype.getTooltip = function (d) {
            return "<table>"
                + "<tr><td><strong>Destination:</strong></td><td>" + d.Destination + "</td></tr>"
                + "<tr><td><strong>Duration milliseconds:</strong></td><td>" + d.DurationMilliseconds + "</td></tr>"
                + "<tr><td><strong>Batch size:</strong></td><td>" + d.BatchSize + "</td></tr>"
                + "</table>";
        };
        replicationStats.prototype.findDestinations = function (jsonData) {
            var statsInline = d3.merge(jsonData.map(function (d) { return d.Stats; }));
            var byKey = d3
                .nest()
                .key(function (d) { return d.Destination; })
                .sortKeys(d3.ascending)
                .rollup(function (l) { return l.length; })
                .entries(statsInline);
            return byKey.map(function (d) { return d.key; });
        };
        replicationStats.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#visualizerContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
        };
        replicationStats.prototype.replicationStatToggle = function () {
            var _this = this;
            setTimeout(function () { return _this.redrawGraph(); }, 1);
        };
        replicationStats.inlineCss = " svg { background-color: white; } " +
            " path.link { fill: none; stroke: #38b44a; stroke-width: 5px; cursor: default; } " +
            " path.link.error {  stroke: #df382c; } " +
            " svg:not(.active):not(.ctrl) path.link { cursor: pointer; } " +
            " path.link.hidden {  stroke-width: 0; } " +
            " rect.node {  stroke-width: 1.5px;  fill: rgba(243, 101, 35, 0.15); stroke: #d74c0c;  } " +
            " text { font: 12px sans-serif;  pointer-events: none;  } " +
            " text.id { text-anchor: middle;  font-weight: bold;  }";
        return replicationStats;
    })(viewModelBase);
    return replicationStats;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/runningTasks", ["require", "exports", "viewmodels/viewModelBase", "commands/operations/killRunningTaskCommand", "commands/operations/getRunningTasksCommand", "moment", "models/database/documents/document", "models/database/debug/runningTask", "common/bindingHelpers/autoRefreshBindingHandler", "common/messagePublisher"], function (require, exports, viewModelBase, killRunningTaskCommand, getRunningTasksCommand, moment, document, runningTask, autoRefreshBindingHandler, messagePublisher) {
    var runningTasks = (function (_super) {
        __extends(runningTasks, _super);
        function runningTasks() {
            var _this = this;
            _super.call(this);
            this.allTasks = ko.observableArray();
            this.filterType = ko.observable(null);
            this.selectedTask = ko.observable();
            this.noExceptionText = ko.computed(function () { return !!_this.selectedTask() && !!_this.selectedTask().exceptionText; });
            this.taskTypes = ko.observableArray([]);
            this.searchText = ko.observable("");
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.sortColumn = ko.observable("logged");
            this.sortAsc = ko.observable(true);
            autoRefreshBindingHandler.install();
            this.searchTextThrottled = this.searchText.throttle(200);
            this.activeDatabase.subscribe(function () { return _this.fetchTasks(); });
            this.updateCurrentNowTime();
            this.filteredAndSortedTasks = ko.computed(function () {
                var tasks = _this.allTasks().filter(function (t) { return _this.matchesFilterAndSearch(t); });
                var column = _this.sortColumn();
                var asc = _this.sortAsc();
                var sortFunc = function (left, right) {
                    if (left[column] === right[column]) {
                        return 0;
                    }
                    var test = asc ? (function (l, r) { return l < r; }) : (function (l, r) { return l > r; });
                    return test(left[column], right[column]) ? 1 : -1;
                };
                return tasks.sort(sortFunc);
            });
        }
        runningTasks.prototype.recalculateTaskTypes = function () {
            var types = [];
            var allTasks = this.allTasks();
            allTasks.forEach(function (task) {
                var type = task.taskType;
                var existingItem = types.first(function (t) { return t.name === type; });
                if (existingItem) {
                    existingItem.count++;
                }
                else {
                    types.push({ name: type, count: 1 });
                }
            });
            this.taskTypes(types);
        };
        runningTasks.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.columnWidths = [
                ko.observable(100),
                ko.observable(300),
                ko.observable(460),
                ko.observable(265),
                ko.observable(100)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('2KH22A');
            return this.fetchTasks();
        };
        runningTasks.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
            this.unregisterColumnResizing();
        };
        runningTasks.prototype.fetchTasks = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var deferred = $.Deferred();
                new getRunningTasksCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.processRunningTasksResults(results);
                    deferred.resolve(results);
                });
                return deferred;
            }
            return null;
        };
        runningTasks.prototype.processRunningTasksResults = function (results) {
            this.allTasks(results.reverse().map(function (r) { return new runningTask(r); }));
            this.recalculateTaskTypes();
        };
        runningTasks.prototype.matchesFilterAndSearch = function (task) {
            var searchTextThrottled = this.searchTextThrottled().toLowerCase();
            var filterType = this.filterType();
            var matchesLogLevel = filterType === null || task.taskType === filterType;
            var matchesSearchText = !searchTextThrottled ||
                (task.description && task.description.toLowerCase().indexOf(searchTextThrottled) >= 0) ||
                (task.exceptionText && task.exceptionText.toLowerCase().indexOf(searchTextThrottled) >= 0);
            return matchesLogLevel && matchesSearchText;
        };
        runningTasks.prototype.selectTask = function (task) {
            this.selectedTask(task);
        };
        runningTasks.prototype.taskKill = function (task) {
            var _this = this;
            new killRunningTaskCommand(this.activeDatabase(), task.id).execute()
                .done(function () {
                messagePublisher.reportSuccess("Send kill task request");
            })
                .always(function () { return setTimeout(function () {
                _this.selectedTask(null);
                _this.fetchTasks();
            }, 1000); });
        };
        runningTasks.prototype.tableKeyDown = function (sender, e) {
            var isKeyUp = e.keyCode === 38;
            var isKeyDown = e.keyCode === 40;
            if (isKeyUp || isKeyDown) {
                e.preventDefault();
                var oldSelection = this.selectedTask();
                if (oldSelection) {
                    var oldSelectionIndex = this.filteredAndSortedTasks().indexOf(oldSelection);
                    var newSelectionIndex = oldSelectionIndex;
                    if (isKeyUp && oldSelectionIndex > 0) {
                        newSelectionIndex--;
                    }
                    else if (isKeyDown && oldSelectionIndex < this.filteredAndSortedTasks().length - 1) {
                        newSelectionIndex++;
                    }
                    this.selectedTask(this.filteredAndSortedTasks()[newSelectionIndex]);
                    var newSelectedRow = $("#runningTasksItemsContainer > div:nth-child(" + (newSelectionIndex + 1) + ")");
                    if (newSelectedRow) {
                        this.ensureRowVisible(newSelectedRow);
                    }
                }
            }
        };
        runningTasks.prototype.ensureRowVisible = function (row) {
            var table = $("#runningTasksTableContainer");
            var scrollTop = table.scrollTop();
            var scrollBottom = scrollTop + table.height();
            var scrollHeight = scrollBottom - scrollTop;
            var rowPosition = row.position();
            var rowTop = rowPosition.top;
            var rowBottom = rowTop + row.height();
            if (rowTop < 0) {
                table.scrollTop(scrollTop + rowTop);
            }
            else if (rowBottom > scrollHeight) {
                table.scrollTop(scrollTop + (rowBottom - scrollHeight));
            }
        };
        runningTasks.prototype.setFilterTypeAll = function () {
            this.filterType(null);
        };
        runningTasks.prototype.setFilterType = function (value) {
            this.filterType(value);
        };
        runningTasks.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 60000);
        };
        runningTasks.prototype.sortBy = function (columnName, logs, event) {
            if (this.sortColumn() === columnName) {
                this.sortAsc(!this.sortAsc());
            }
            else {
                this.sortColumn(columnName);
                this.sortAsc(true);
            }
        };
        runningTasks.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.tasksTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.tasksTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.tasksTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var targetColumnSize = startingWidth + e.pageX - startX;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        runningTasks.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.tasksTableColumnResize");
            $(document).off("mouseup.tasksTableColumnResize");
            $(document).off("mousemove.tasksTableColumnResize");
        };
        return runningTasks;
    })(viewModelBase);
    return runningTasks;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/sqlReplicationPerfStats", ["require", "exports", "viewmodels/viewModelBase", "common/generalUtils", "common/changesContext", "commands/database/debug/getSqlReplicationPerfStatsCommand", "d3/d3", 'nvd3', "commands/resources/getDatabaseSettingsCommand"], function (require, exports, viewModelBase, generalUtils, changesContext, getSqlReplicationPerfStatsCommand, d3, nv, getDatabaseSettingsCommand) {
    var sqlReplicationPerfStats = (function (_super) {
        __extends(sqlReplicationPerfStats, _super);
        function sqlReplicationPerfStats() {
            _super.apply(this, arguments);
            this.hasReplicationEnabled = ko.observable(false);
            this.jsonData = [];
            this.rawJsonData = [];
            this.hiddenNames = d3.set([]);
            this.replicationNames = [];
            this.refreshGraphObservable = ko.observable();
            this.margin = { top: 40, right: 20, bottom: 40, left: 40 };
            this.barWidth = 30;
            this.barPadding = 15;
            this.legendWidth = 0;
            this.isoFormat = d3.time.format.iso;
            this.xTickFormat = d3.time.format("%H:%M:%S");
            this.color = d3.scale.category20();
        }
        sqlReplicationPerfStats.prototype.fetchJsonData = function () {
            return new getSqlReplicationPerfStatsCommand(this.activeDatabase()).execute();
        };
        sqlReplicationPerfStats.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () {
                _this.checkIfHasReplicationEnabled();
            });
            this.checkIfHasReplicationEnabled();
        };
        sqlReplicationPerfStats.prototype.checkIfHasReplicationEnabled = function () {
            var _this = this;
            new getDatabaseSettingsCommand(this.activeDatabase())
                .execute()
                .done(function (document) {
                var documentSettings = document.Settings["Raven/ActiveBundles"];
                _this.hasReplicationEnabled(documentSettings.toLowerCase().indexOf("sqlreplication") !== -1);
            });
        };
        sqlReplicationPerfStats.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            $("#replicationStatsContainer").resize().on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            $("#replicationStatsContainer").scroll(function () { return _this.graphScrolled(); });
            this.refresh();
            this.refreshSubscription = this.refreshGraphObservable.throttle(5000).subscribe(function (e) { return _this.refresh(); });
        };
        sqlReplicationPerfStats.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchDocsStartingWith("Raven/SqlReplication/Status", function (e) { return _this.processUpdate(e); })];
        };
        sqlReplicationPerfStats.prototype.processUpdate = function (e) {
            this.refreshGraphObservable(new Date().getTime());
        };
        sqlReplicationPerfStats.prototype.filterJsonData = function () {
            var _this = this;
            this.jsonData = [];
            this.rawJsonData.forEach(function (v) {
                var filteredStats = v.Stats.filter(function (s) { return !_this.hiddenNames.has(s.ReplicationName); });
                if (filteredStats.length > 0) {
                    _this.jsonData.push({
                        'Started': v.Started,
                        'Stats': filteredStats
                    });
                }
            });
        };
        sqlReplicationPerfStats.prototype.refresh = function () {
            var _this = this;
            return this.fetchJsonData().done(function (data) {
                _this.rawJsonData = _this.mergeJsonData(_this.rawJsonData, data);
                _this.replicationNames = _this.findReplicationNames(_this.rawJsonData);
                _this.filterJsonData();
                _this.redrawGraph();
            });
        };
        sqlReplicationPerfStats.prototype.mergeJsonData = function (currentData, incomingData) {
            // create lookup map to avoid O(n^2) 
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.Started, i);
            });
            incomingData.forEach(function (d) {
                if (dateLookup.has(d.Started)) {
                    var index = dateLookup.get(d.Started);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        sqlReplicationPerfStats.prototype.computeBarWidths = function (data) {
            var _this = this;
            var cumulative = 10;
            var result = data.map(function (perfData) {
                var prevValue = cumulative;
                perfData.sectionWidth = perfData.Stats.length * _this.barWidth + _this.barPadding * 2;
                cumulative += perfData.sectionWidth;
                return prevValue;
            });
            result.push(cumulative);
            return result;
        };
        sqlReplicationPerfStats.prototype.graphScrolled = function () {
            var leftScroll = $("#replicationStatsContainer").scrollLeft();
            var self = this;
            this.svg.select('.y.axis')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('#dataClip rect')
                .attr('x', leftScroll);
            this.svg.select('.legend_bg_group')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('.controlls')
                .selectAll(".legend")
                .attr("transform", function (d, i) { return "translate(" + leftScroll + "," + i * 20 + ")"; });
            nv.tooltip.cleanup();
        };
        sqlReplicationPerfStats.prototype.toggleGroupVisible = function (groupName) {
            nv.tooltip.cleanup();
            var alreadyHidden = this.hiddenNames.has(groupName);
            if (alreadyHidden) {
                this.hiddenNames.remove(groupName);
            }
            else {
                this.hiddenNames.add(groupName);
            }
            d3.select('.rect-legend-' + generalUtils.escape(groupName)).classed('legendHidden', !alreadyHidden);
            this.filterJsonData();
            this.redrawGraph();
            // we have to manually trigger on scroll even to fix firefox issue (missing event call)
            this.graphScrolled();
        };
        sqlReplicationPerfStats.prototype.redrawGraph = function () {
            var self = this;
            this.width = $("#replicationStatsContainer").width() - this.margin.left - this.margin.right;
            this.height = $("#replicationStatsContainer").height() - this.margin.top - this.margin.bottom - 20; // substract scroll width
            var cumulativeWidths = this.computeBarWidths(this.jsonData);
            this.x0Scale = d3.scale.ordinal().range(cumulativeWidths);
            this.yScale = d3.scale.linear().range([self.height, 0]);
            this.xAxis = d3.svg.axis()
                .scale(self.x0Scale)
                .orient("bottom")
                .tickFormat(function (d) { return ""; })
                .tickPadding(20);
            this.yAxis = d3.svg.axis()
                .scale(self.yScale)
                .orient("left")
                .tickFormat(d3.format(".2s"));
            var totalHeight = self.height + self.margin.top + self.margin.bottom;
            // get higer value from total (visiable and not visible graph width) and viewbox width.
            var totalWidth = Math.max(cumulativeWidths[cumulativeWidths.length - 1], this.width) + this.margin.left + this.margin.right;
            $("#replicationStatsContainer").css('overflow-x', cumulativeWidths[cumulativeWidths.length - 1] > this.width ? 'scroll' : 'hidden');
            this.svg = d3.select("#replicationStatsGraph")
                .attr("width", totalWidth)
                .attr("height", totalHeight)
                .style({ height: totalHeight + 'px' })
                .style({ width: totalWidth + 'px' })
                .attr("viewBox", "0 0 " + totalWidth + " " + totalHeight);
            this.svg.selectAll('.main_group')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            this.svg
                .selectAll('defs')
                .data([this.jsonData])
                .enter()
                .append('defs')
                .append('clipPath')
                .attr('id', 'dataClip')
                .append('rect')
                .attr('x', 0)
                .attr('y', 0)
                .attr('width', 1200000)
                .attr('height', 50000);
            var svgEnter = this.svg
                .selectAll(".main_group")
                .data([this.jsonData]).enter();
            svgEnter.append('g')
                .attr('class', 'main_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            var controllsEnter = this.svg
                .selectAll(".controlls")
                .data([this.jsonData]).enter()
                .append("g")
                .attr('class', 'controlls')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            controllsEnter.append("g")
                .attr("class", "x axis");
            controllsEnter.append('g')
                .attr('class', 'y axis');
            controllsEnter.append('g')
                .attr('class', 'legend_bg_group')
                .append('rect')
                .attr('class', 'legend_bg')
                .attr('x', self.width)
                .attr('y', 0)
                .attr('width', 0)
                .attr('height', 0);
            controllsEnter.select('.y.axis')
                .append("text")
                .attr("transform", "rotate(-90)")
                .attr("y", 6)
                .attr("dy", ".71em")
                .style("text-anchor", "end")
                .text("Batch size");
            this.x0Scale.domain(d3.nest()
                .key(function (d) { return d.Started; })
                .sortKeys(d3.ascending)
                .entries(self.jsonData)
                .map(function (d) { return d.key; }));
            this.yScale.domain([0, d3.max(this.jsonData, function (d) { return d3.max(d.Stats, function (dd) { return dd.BatchSize; }); })]);
            this.svg.select(".x.axis")
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(0," + self.height + ")")
                .transition()
                .call(self.xAxis);
            this.svg.select('.y.axis')
                .transition()
                .call(self.yAxis);
            var frame = this.svg.select('.main_group').selectAll(".frame")
                .data(self.jsonData, function (d) { return d.Started; });
            frame.exit().remove();
            frame
                .transition()
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frame
                .select('.date_tick')
                .transition()
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.height + 16);
            var frameEnter = frame.enter()
                .append("g")
                .attr("class", "frame")
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frameEnter.append("text")
                .attr('class', 'date_tick')
                .attr('text-anchor', 'middle')
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.height + 16)
                .text(function (d) { return self.xTickFormat(self.isoFormat.parse(d.Started)); });
            frameEnter.append("g")
                .attr('class', 'inputs');
            var inputCounts = frame.select('.inputs').selectAll(".inputCounts")
                .data(function (d) { return d.Stats; }, function (d) { return d.ReplicationName; });
            inputCounts.exit().remove();
            inputCounts
                .transition()
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPadding; })
                .attr("y", function (d) { return self.yScale(d.BatchSize); })
                .attr("height", function (d) { return self.height - self.yScale(d.BatchSize); })
                .style("fill", function (d) { return self.color(d.ReplicationName); });
            inputCounts.enter().append("rect")
                .attr("class", "inputCounts")
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPadding; })
                .attr("y", function (d) { return self.height; })
                .attr("height", 0)
                .style("fill", function (d) { return self.color(d.ReplicationName); })
                .on('click', function (d) {
                nv.tooltip.cleanup();
                var offset = $(this).offset();
                var leftScroll = $("#replicationStatsContainer").scrollLeft();
                var containerOffset = $("#replicationStatsContainer").offset();
                nv.tooltip.show([offset.left - containerOffset.left + leftScroll + self.barWidth, offset.top - containerOffset.top], self.getTooltip(d), 's', 5, document.getElementById("replicationStatsContainer"), "selectable-tooltip");
            })
                .transition()
                .attr("height", function (d) { return self.height - self.yScale(d.BatchSize); })
                .attr("y", function (d) { return self.yScale(d.BatchSize); });
            this.legend = this.svg.select('.controlls').selectAll(".legend")
                .data(this.replicationNames, function (d) { return d; });
            this.legend.selectAll("rect").transition()
                .attr("x", this.width - 18);
            this.legend.selectAll("text").transition()
                .attr("x", this.width - 24)
                .text(function (d) { return d; });
            var legendEnter = this.legend
                .enter().append("g")
                .attr("class", "legend")
                .attr("transform", function (d, i) { return "translate(0," + i * 20 + ")"; });
            legendEnter.append("rect")
                .attr("x", this.width - 18)
                .attr('class', function (d) { return 'rect-legend-' + generalUtils.escape(d); })
                .attr("width", 18)
                .attr("height", 18)
                .style("fill", self.color)
                .style("stroke", self.color)
                .on('click', function (d) { return self.toggleGroupVisible(d); });
            legendEnter.append("text")
                .attr("x", this.width - 24)
                .attr("y", 9)
                .attr("dy", ".35em")
                .style("text-anchor", "end")
                .text(function (d) { return d; });
            this.legendWidth = d3.max($(".legend text"), function (d) { return d.getBBox().width; }) + 40 || 0;
            this.svg.select('.legend_bg')
                .attr('y', -6)
                .attr('height', this.replicationNames.length * 20 + 10)
                .attr('width', this.legendWidth)
                .attr('x', this.width - this.legendWidth + 10);
        };
        sqlReplicationPerfStats.prototype.onWindowHeightChanged = function () {
            nv.tooltip.cleanup();
            this.width = $("#replicationStatsContainer").width();
            this.height = $("#replicationStatsContainer").height();
            this.redrawGraph();
        };
        sqlReplicationPerfStats.prototype.getTooltip = function (d) {
            return "<strong>Replication Name:</strong> <span>" + d.ReplicationName + "</span><br />"
                + "<strong>Duration milliseconds:</strong> <span>" + d.DurationMilliseconds + "</span><br />"
                + "<strong>Batch size:</strong> <span>" + d.BatchSize + "</span><br />";
        };
        sqlReplicationPerfStats.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#visualizerContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
            if (this.refreshSubscription != null) {
                this.refreshSubscription.dispose();
            }
        };
        sqlReplicationPerfStats.prototype.findReplicationNames = function (jsonData) {
            var statsInline = d3.merge(jsonData.map(function (d) { return d.Stats; }));
            var byKey = d3
                .nest()
                .key(function (d) { return d.ReplicationName; })
                .sortKeys(d3.ascending)
                .rollup(function (l) { return l.length; })
                .entries(statsInline);
            return byKey.map(function (d) { return d.key; });
        };
        return sqlReplicationPerfStats;
    })(viewModelBase);
    return sqlReplicationPerfStats;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/sqlReplicationSimulationDialog", ["require", "exports", "viewmodels/dialogViewModelBase", "commands/database/studio/simulateSqlReplicationCommand", "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "plugins/dialog", "models/database/documents/collection", "models/database/sqlReplication/sqlReplicationSimulatedCommand"], function (require, exports, dialogViewModelBase, simulateSqlReplicationCommand, getDocumentsMetadataByIDPrefixCommand, dialog, collection, sqlReplicationSimulatedCommand) {
    var sqlReplicationSimulationDialog = (function (_super) {
        __extends(sqlReplicationSimulationDialog, _super);
        function sqlReplicationSimulationDialog(db, simulatedSqlReplication, documentId) {
            var _this = this;
            _super.call(this);
            this.db = db;
            this.simulatedSqlReplication = simulatedSqlReplication;
            this.documentId = documentId;
            this.simulationResults = ko.observableArray([]);
            this.rolledBackTransactionPassed = ko.observable(false);
            this.documentAutocompletes = ko.observableArray();
            this.lastSearchedDocumentID = ko.observable("");
            this.rolledbackTransactionPerformed = ko.observable(false);
            this.lastAlert = ko.observable("");
            this.waitingToAnswer = ko.observable(false);
            this.documentId.throttle(250).subscribe(function (search) { return _this.fetchDocumentIdAutocompletes(search); });
            this.isAutoCompleteVisible = ko.computed(function () {
                return _this.lastSearchedDocumentID() !== _this.documentId() &&
                    (_this.documentAutocompletes().length > 1 || _this.documentAutocompletes().length == 1 && _this.documentId() !== _this.documentAutocompletes()[0]);
            });
        }
        sqlReplicationSimulationDialog.prototype.toggleCommandParamView = function (command) {
            command.showParamsValues.toggle();
        };
        sqlReplicationSimulationDialog.prototype.getResults = function (performRolledbackTransaction) {
            var _this = this;
            this.lastSearchedDocumentID(this.documentId());
            this.waitingToAnswer(true);
            new simulateSqlReplicationCommand(this.db, this.simulatedSqlReplication, this.documentId(), performRolledbackTransaction)
                .execute()
                .done(function (result) {
                if (!!result.Results) {
                    _this.simulationResults(result.Results.map(function (x) { return x.Commands; }).reduce(function (x, y) { return x.concat(y); }).map(function (x) { return new sqlReplicationSimulatedCommand(!performRolledbackTransaction, x); }));
                }
                else {
                    _this.simulationResults([]);
                }
                _this.rolledBackTransactionPassed(!result.LastAlert);
                if (!!result.LastAlert) {
                    _this.lastAlert(result.LastAlert.Exception);
                }
                else {
                    _this.lastAlert("");
                }
                _this.rolledbackTransactionPerformed(performRolledbackTransaction);
            })
                .fail(function () {
                _this.simulationResults([]);
                _this.rolledBackTransactionPassed(false);
            })
                .always(function () { return _this.waitingToAnswer(false); });
        };
        // overrid dialogViewModelBase shortcuts behavior
        sqlReplicationSimulationDialog.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#docIdInput").focus();
            var that = this;
            jwerty.key("esc", function (e) {
                e.preventDefault();
                dialog.close(that);
            }, this, this.dialogSelectorName == "" ? dialogViewModelBase.dialogSelector : this.dialogSelectorName);
        };
        sqlReplicationSimulationDialog.prototype.fetchDocumentIdAutocompletes = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.db)
                    .execute()
                    .done(function (results) {
                    if (_this.documentId() === query) {
                        if (results.length == 1 && _this.documentId() == results[0]) {
                            _this.documentAutocompletes.removeAll();
                            return;
                        }
                        _this.documentAutocompletes(results);
                    }
                });
            }
            else if (query.length == 0) {
                this.documentAutocompletes.removeAll();
            }
        };
        sqlReplicationSimulationDialog.prototype.documentIdSubmitted = function (submittedDocumentId) {
            this.documentId(submittedDocumentId);
            $('#docIdInput').focus();
            this.getResults(false);
        };
        sqlReplicationSimulationDialog.prototype.getDocCssClass = function (doc) {
            return collection.getCollectionCssClass(doc['@metadata']['Raven-Entity-Name'], this.db);
        };
        sqlReplicationSimulationDialog.prototype.keyPressedOnDocumentAutocomplete = function (doc, event) {
            if (event.keyCode == 13 && !!doc) {
                var docId = !!doc['@metadata'] ? doc['@metadata']['@id'] : null;
                if (!!docId) {
                    this.documentId(docId);
                }
            }
        };
        sqlReplicationSimulationDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        return sqlReplicationSimulationDialog;
    })(dialogViewModelBase);
    return sqlReplicationSimulationDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/sqlReplicationStatsDialog", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog", "commands/database/studio/getSqlReplicationStatsCommand"], function (require, exports, dialogViewModelBase, dialog, getSqlReplicationStatsCommand) {
    var sqlReplicationStatsDialog = (function (_super) {
        __extends(sqlReplicationStatsDialog, _super);
        function sqlReplicationStatsDialog(activeDatabase, sqlReplicationName) {
            _super.call(this);
            this.activeDatabase = activeDatabase;
            this.sqlReplicationName = sqlReplicationName;
            this.repData = ko.observable();
        }
        sqlReplicationStatsDialog.prototype.activate = function (args) {
            this.fetchSqlReplicationStats();
        };
        sqlReplicationStatsDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        sqlReplicationStatsDialog.prototype.fetchSqlReplicationStats = function () {
            var _this = this;
            return new getSqlReplicationStatsCommand(this.activeDatabase, this.sqlReplicationName)
                .execute()
                .done(function (result) {
                _this.repData(result);
            }).fail(function () {
                _this.repData(null);
            });
        };
        return sqlReplicationStatsDialog;
    })(dialogViewModelBase);
    return sqlReplicationStatsDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/statistics", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "common/changesContext", "common/optional"], function (require, exports, viewModelBase, getDatabaseStatsCommand, changesContext, optional) {
    var statistics = (function (_super) {
        __extends(statistics, _super);
        function statistics() {
            var _this = this;
            _super.apply(this, arguments);
            this.stats = ko.observable();
            this.indexes = ko.observableArray();
            this.noStaleIndexes = ko.computed(function () { return !!_this.stats() && _this.stats().CountOfStaleIndexesExcludingDisabledAndAbandoned == 0; });
            this.disabledIndexes = ko.computed(function () {
                if (_this.stats()) {
                    var stats = _this.stats();
                    return stats.Indexes.filter(function (idx) { return idx.Priority.indexOf("Disabled") >= 0; }).map(function (idx) { return idx.Name; });
                }
            });
            this.refreshStatsObservable = ko.observable();
        }
        statistics.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.statsSubscription = this.refreshStatsObservable.throttle(3000).subscribe(function (e) { return _this.fetchStats(); });
            this.fetchStats();
            this.updateHelpLink('H6GYYL');
        };
        statistics.prototype.detached = function () {
            _super.prototype.detached.call(this);
            if (this.statsSubscription != null) {
                this.statsSubscription.dispose();
            }
        };
        statistics.prototype.fetchStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getDatabaseStatsCommand(db)
                    .execute()
                    .done(function (result) { return _this.processStatsResults(result); });
            }
            return null;
        };
        statistics.prototype.createNotifications = function () {
            var _this = this;
            return [
                changesContext.currentResourceChangesApi().watchAllDocs(function (e) { return _this.refreshStatsObservable(new Date().getTime()); }),
                changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.refreshStatsObservable(new Date().getTime()); })];
        };
        statistics.prototype.processStatsResults = function (results) {
            var _this = this;
            // Attach some human readable dates to the indexes.
            // Attach string versions numbers with thousands separator to the indexes.
            results['CountOfDocumentsLocale'] = optional.val(results.CountOfDocuments).bind(function (v) { return v; }).bind(function (v) { return v.toLocaleString(); });
            results['CurrentNumberOfItemsToIndexInSingleBatchLocale'] = optional.val(results.CurrentNumberOfItemsToIndexInSingleBatch).bind(function (v) { return v.toLocaleString(); });
            results['CurrentNumberOfItemsToReduceInSingleBatchLocale'] = optional.val(results.CurrentNumberOfItemsToReduceInSingleBatch).bind(function (v) { return v.toLocaleString(); });
            results['LastIndexingDateTime'] = String(optional.val(results.Indexes.map(function (x) { return x.LastIndexedTimestamp; }).reduce(function (prev, curr) { return moment(prev).isAfter(moment(curr)) ? prev : curr; })).bind(function (v) { return v.toHumanizedDate(); }));
            results.Indexes.forEach(function (i) {
                i['CreatedTimestampText'] = optional.val(i.CreatedTimestamp).bind(function (v) { return v.toHumanizedDate(); });
                i['LastIndexedTimestampText'] = optional.val(i.LastIndexedTimestamp).bind(function (v) { return v.toHumanizedDate(); });
                i['LastQueryTimestampText'] = optional.val(i.LastQueryTimestamp).bind(function (v) { return v.toHumanizedDate(); });
                i['LastIndexingTimeText'] = optional.val(i.LastIndexingTime).bind(function (v) { return v.toHumanizedDate(); });
                i['LastReducedTimestampText'] = optional.val(i.LastReducedTimestamp).bind(function (v) { return v.toHumanizedDate(); });
                i['DocsCountLocale'] = optional.val(i.DocsCount).bind(function (v) { return v.toLocaleString(); });
                i['ReduceIndexingAttemptsLocale'] = optional.val(i.ReduceIndexingAttempts).bind(function (v) { return v.toLocaleString(); });
                i['ReduceIndexingErrorsLocale'] = optional.val(i.ReduceIndexingErrors).bind(function (v) { return v.toLocaleString(); });
                i['ReduceIndexingSuccessesLocale'] = optional.val(i.ReduceIndexingSuccesses).bind(function (v) { return v.toLocaleString(); });
                i['IndexingAttemptsLocale'] = optional.val(i.IndexingAttempts).bind(function (v) { return v.toLocaleString(); });
                i['IndexingErrorsLocale'] = optional.val(i.IndexingErrors).bind(function (v) { return v.toLocaleString(); });
                i['IndexingSuccessesLocale'] = optional.val(i.IndexingSuccesses).bind(function (v) { return v.toLocaleString(); });
            });
            results.Indexes.sort(function (a, b) { return a.Name < b.Name ? -1 : a.Name > b.Name ? 1 : 0; });
            this.stats(results);
            var existingIndexes = this.indexes().map(function (i) { return i().Name; });
            var newIndexes = results.Indexes.map(function (i) { return i.Name; });
            var enteringIndexes = newIndexes.filter(function (i) { return !existingIndexes.contains(i); });
            var exitIndexes = existingIndexes.filter(function (i) { return !newIndexes.contains(i); });
            var sameIndexes = newIndexes.filter(function (i) { return existingIndexes.contains(i); });
            this.indexes.pushAll(enteringIndexes.map(function (idx) { return ko.observable(results.Indexes.first(function (item) { return item.Name == idx; })); }));
            this.indexes.removeAll(exitIndexes.map(function (idx) { return _this.indexes().first(function (item) { return item().Name == idx; }); }));
            sameIndexes.forEach(function (idx) {
                var newData = results.Indexes.first(function (item) { return item.Name == idx; });
                _this.indexes().first(function (item) { return item().Name == idx; })(newData);
            });
        };
        return statistics;
    })(viewModelBase);
    return statistics;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/status", ["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl", 'viewmodels/shell'], function (require, exports, durandalRouter, viewModelBase, appUrl, shell) {
    var status = (function (_super) {
        __extends(status, _super);
        function status() {
            var _this = this;
            _super.call(this);
            this.bundleMap = { replication: "Replication Stats", sqlreplication: "SQL Replication Stats" };
            this.userDatabasePages = ko.observableArray(status.initialVisibleViews.slice());
            this.appUrls = appUrl.forCurrentDatabase();
            this.router = durandalRouter.createChildRouter()
                .map([
                { route: 'databases/status', moduleId: 'viewmodels/database/status/statistics', title: 'Stats', nav: true, hash: appUrl.forCurrentDatabase().status },
                { route: 'databases/status/indexing*details', moduleId: 'viewmodels/database/status/indexing/indexing', title: 'Indexing', nav: true, hash: appUrl.forCurrentDatabase().indexPerformance },
                { route: 'databases/status/requests*details', moduleId: 'viewmodels/database/status/requests/requests', title: 'Request Tracking', nav: true, hash: appUrl.forCurrentDatabase().requestsCount },
                { route: 'databases/status/logs', moduleId: 'viewmodels/database/status/logs', title: 'Logs', nav: true, hash: appUrl.forCurrentDatabase().logs },
                { route: 'databases/status/runningTasks', moduleId: 'viewmodels/database/status/runningTasks', title: 'Running Tasks', nav: true, hash: appUrl.forCurrentDatabase().runningTasks },
                { route: 'databases/status/alerts', moduleId: 'viewmodels/database/status/alerts', title: 'Alerts', nav: true, hash: appUrl.forCurrentDatabase().alerts },
                { route: 'databases/status/indexErrors', moduleId: 'viewmodels/database/status/indexErrors', title: 'Index Errors', nav: true, hash: appUrl.forCurrentDatabase().indexErrors },
                { route: 'databases/status/replicationStats', moduleId: 'viewmodels/database/status/replicationStats', title: 'Replication Stats', nav: true, hash: appUrl.forCurrentDatabase().replicationStats },
                { route: 'databases/status/sqlReplicationPerfStats', moduleId: 'viewmodels/database/status/sqlReplicationPerfStats', title: 'SQL Replication Stats', nav: true, hash: appUrl.forCurrentDatabase().sqlReplicationPerfStats },
                { route: 'databases/status/userInfo', moduleId: 'viewmodels/database/status/userInfo', title: 'User Info', nav: true, hash: appUrl.forCurrentDatabase().userInfo },
                { route: 'databases/status/visualizer', moduleId: 'viewmodels/database/status/visualizer', title: 'Map/Reduce Visualizer', nav: true, hash: appUrl.forCurrentDatabase().visualizer },
                { route: 'databases/status/debug*details', moduleId: 'viewmodels/database/status/debug/statusDebug', title: 'Debug', nav: true, hash: appUrl.forCurrentDatabase().statusDebug },
                { route: 'databases/status/storage*details', moduleId: 'viewmodels/database/status/storage/statusStorage', title: 'Storage', nav: true, hash: appUrl.forCurrentDatabase().statusStorageOnDisk },
                { route: 'databases/status/infoPackage', moduleId: 'viewmodels/manage/infoPackage', title: 'Gather Debug Info', nav: shell.canExposeConfigOverTheWire(), hash: appUrl.forCurrentDatabase().infoPackage }
            ])
                .buildNavigationModel();
            status.statusRouter = this.router;
            this.router.guardRoute = function (instance, instruction) { return _this.getValidRoute(instance, instruction); };
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        /**
        * Checks whether the route can be navigated to. Returns true if it can be navigated to, or a redirect URI if it can't be navigated to.
        * This is used for preventing a navigating to system-only pages when the current databagse is non-system, and vice-versa.
        */
        status.prototype.getValidRoute = function (instance, instruction) {
            var db = this.activeDatabase();
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) > -1;
            if (isLegalBundelName && isBundleExists == false) {
                return appUrl.forCurrentDatabase().databaseSettings();
            }
            return true;
        };
        status.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.userDatabasePages(status.initialVisibleViews.slice());
            var db = this.activeDatabase();
            var bundles = db.activeBundles();
            bundles.forEach(function (bundle) {
                var bundleName = _this.bundleMap[bundle.toLowerCase()];
                if (bundleName != undefined) {
                    _this.userDatabasePages.push(bundleName);
                }
            });
        };
        status.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;
            if (this.userDatabasePages.indexOf(bundleTitle) !== -1) {
                // Replication stats and Sql Replication stats are visible only when we're on a user database.
                return true;
            }
            return false;
        };
        status.initialVisibleViews = ["Stats", "Indexing", 'Request Tracking', 'Logs', 'Running Tasks', 'Alerts', 'Index Errors', 'User Info', 'Map/Reduce Visualizer', 'Debug', 'Storage', 'Gather Debug Info'];
        return status;
    })(viewModelBase);
    return status;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/userInfo", ["require", "exports", "commands/database/debug/getUserInfoCommand", "viewmodels/viewModelBase"], function (require, exports, getUserInfoCommand, viewModelBase) {
    var userInfo = (function (_super) {
        __extends(userInfo, _super);
        function userInfo() {
            _super.apply(this, arguments);
            this.data = ko.observable();
        }
        userInfo.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.fetchUserInfo(); });
            this.updateHelpLink('JSVY4P');
            return this.fetchUserInfo();
        };
        userInfo.prototype.fetchUserInfo = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getUserInfoCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return userInfo;
    })(viewModelBase);
    return userInfo;
});
/// <reference path="../../../../Scripts/typings/jquery.fullscreen/jquery.fullscreen.d.ts"/>
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/visualizer", ["require", "exports", "durandal/app", "common/appUrl", "common/jsonUtil", "common/generalUtils", "common/svgDownloader", "common/fileDownloader", "common/messagePublisher", "common/chunkFetcher", "plugins/router", "viewmodels/database/status/visualizerKeys", "viewmodels/database/status/visualizerImport", "viewmodels/viewModelBase", "commands/database/debug/queryIndexDebugDocsCommand", "commands/database/debug/queryIndexDebugMapCommand", "commands/database/debug/queryIndexDebugReduceCommand", "commands/database/debug/queryIndexDebugAfterReduceCommand", "commands/resources/getDatabaseStatsCommand", 'd3/d3', 'nvd3'], function (require, exports, app, appUrl, jsonUtil, generalUtils, svgDownloader, fileDownloader, messagePublisher, chunkFetcher, router, visualizerKeys, visualizerImport, viewModelBase, queryIndexDebugDocsCommand, queryIndexDebugMapCommand, queryIndexDebugReduceCommand, queryIndexDebugAfterReduceCommand, getDatabaseStatsCommand, d3, nv) {
    var visualizer = (function (_super) {
        __extends(visualizer, _super);
        function visualizer() {
            var _this = this;
            _super.apply(this, arguments);
            this.indexes = ko.observableArray();
            this.indexName = ko.observable(visualizer.chooseIndexText);
            this.onlineMode = ko.observable(true);
            this.showLoadingIndicator = ko.observable(false);
            this.docKey = ko.observable("");
            this.docKeys = ko.observableArray();
            this.docKeysSearchResults = ko.observableArray();
            this.reduceKey = ko.observable("");
            this.reduceKeys = ko.observableArray();
            this.reduceKeysSearchResults = ko.observableArray();
            this.hasIndexSelected = ko.computed(function () { return _this.indexName() !== visualizer.chooseIndexText; });
            this.docKeysOrReduceKeysSelected = ko.computed(function () { return _this.docKeys().length > 0 || _this.reduceKeys().length > 0; });
            this.colors = d3.scale.category10();
            this.colorMap = {};
            this.selectedDocs = ko.observableArray();
            this.currentlySelectedNodes = d3.set([]);
            this.currentlySelectedLinks = d3.set([]);
            this.tree = null;
            this.nodes = []; // nodes data
            this.links = []; // links data
            this.margin = {
                left: 10,
                right: 10,
                bottom: 10,
                top: 10
            };
            this.boxSpacing = 30;
            this.node = null; // nodes selection
            this.link = null; // links selection
            this.hasSaveAsPngSupport = ko.computed(function () {
                return !(navigator && navigator.msSaveBlob);
            });
        }
        visualizer.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (args && args.index) {
                this.indexName(args.index);
            }
            this.updateHelpLink('RNXYCB');
            this.editIndexUrl = ko.computed(function () {
                return appUrl.forEditIndex(_this.indexName(), _this.activeDatabase());
            });
            this.runQueryUrl = ko.computed(function () {
                return appUrl.forQuery(_this.activeDatabase(), _this.indexName());
            });
            this.reduceKey.throttle(250).subscribe(function (search) { return _this.fetchReduceKeySearchResults(search); });
            this.docKey.throttle(250).subscribe(function (search) { return _this.fetchDocKeySearchResults(search); });
            this.selectedDocs.subscribe(function () { return _this.repaintSelectedNodes(); });
            $(document).bind("fullscreenchange", function () {
                if ($(document).fullScreen()) {
                    $("#fullScreenButton i").removeClass("fa-expand").addClass("fa-compress");
                }
                else {
                    $("#fullScreenButton i").removeClass("fa-compress").addClass("fa-expand");
                }
            });
            return this.fetchAllIndexes();
        };
        visualizer.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            var that = this;
            this.createKeyboardShortcut("esc", function () { return that.tooltipClose(); }, "body");
            this.resetChart();
            var svg = d3.select("#visualizer");
            this.diagonal = d3.svg.diagonal()
                .projection(function (d) { return [d.y, d.x]; });
            this.node = svg.selectAll(".node");
            this.link = svg.selectAll(".link");
            $("#visualizerContainer").resize().on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            this.width = $("#visualizerContainer").width();
            this.height = $("#visualizerContainer").height();
            this.updateScale();
            this.drawHeader();
        };
        visualizer.prototype.resetChart = function () {
            this.tooltipClose();
            this.reduceKey("");
            this.reduceKeys([]);
            this.docKey("");
            this.docKeys([]);
            this.tree = {
                level: 5,
                name: 'root',
                children: []
            };
            this.colorMap = {};
            this.currentlySelectedNodes = d3.set([]);
            this.currentlySelectedLinks = d3.set([]);
        };
        visualizer.prototype.fetchReduceKeySearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new queryIndexDebugMapCommand(this.indexName(), this.activeDatabase(), { startsWith: query }, 0, 10)
                    .execute()
                    .done(function (results) {
                    if (_this.reduceKey() === query) {
                        _this.reduceKeysSearchResults(results.sort());
                    }
                });
            }
            else if (query.length == 0) {
                this.reduceKeysSearchResults.removeAll();
            }
        };
        visualizer.prototype.fetchDocKeySearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new queryIndexDebugDocsCommand(this.indexName(), this.activeDatabase(), query, 0, 10)
                    .execute()
                    .done(function (results) {
                    if (_this.docKey() === query) {
                        _this.docKeysSearchResults(results.sort());
                    }
                });
            }
            else if (query.length == 0) {
                this.docKeysSearchResults.removeAll();
            }
        };
        visualizer.prototype.updateScale = function () {
            this.xScale = d3.scale.linear().domain([0, 5]).range([this.margin.left, this.width - this.margin.left - this.margin.right]);
            this.boxWidth = this.width / 7;
        };
        visualizer.prototype.drawHeader = function () {
            var _this = this;
            var header = d3.select("#visHeader");
            var headerData = ["Input", "Map", "Reduce 0", "Reduce 1", "Reduce 2"];
            var texts = header.selectAll("text").data(headerData);
            texts.attr('x', function (d, i) { return _this.xScale(i) + _this.boxWidth / 2; });
            texts
                .enter()
                .append("text")
                .attr('y', 20)
                .attr('x', function (d, i) { return _this.xScale(i) + _this.boxWidth / 2; })
                .text(function (d) { return d; })
                .attr("text-anchor", "middle");
        };
        visualizer.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#visualizerContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
        };
        visualizer.prototype.onWindowHeightChanged = function () {
            this.width = $("#visualizerContainer").width();
            this.updateScale();
            this.drawHeader();
            this.updateGraph();
        };
        visualizer.prototype.addDocKey = function (key) {
            var _this = this;
            if (key && this.docKeys.contains(key))
                return;
            this.showLoadingIndicator(true);
            new queryIndexDebugDocsCommand(this.indexName(), this.activeDatabase(), key, 0, 1)
                .execute()
                .done(function (results) {
                if (results && results.length === 1 && results[0] == key) {
                    if (key && !_this.docKeys.contains(key)) {
                        _this.docKeys.push(key);
                        new queryIndexDebugMapCommand(_this.indexName(), _this.activeDatabase(), { sourceId: key }, 0, 1024)
                            .execute()
                            .then(function (results) {
                            var addReduceKeysTasks = [];
                            results.forEach(function (r) { return addReduceKeysTasks.push(_this.addReduceKey(r, false)); });
                            $.when.apply($, addReduceKeysTasks).done(function () {
                                _this.showLoadingIndicator(false);
                            });
                        });
                    }
                }
                else {
                    messagePublisher.reportWarning("The document, " + key + " , is not contained in the documents for this index.");
                    _this.showLoadingIndicator(false);
                }
            });
        };
        visualizer.prototype.addReduceKey = function (key, needToChangeLoadingIndicator) {
            var _this = this;
            if (needToChangeLoadingIndicator === void 0) { needToChangeLoadingIndicator = true; }
            var deferred = $.Deferred();
            if (key && this.reduceKeys().contains(key))
                return deferred.resolve();
            if (needToChangeLoadingIndicator)
                this.showLoadingIndicator(true);
            var self = this;
            var getTask = new queryIndexDebugMapCommand(this.indexName(), this.activeDatabase(), { startsWith: key }, 0, 1).execute();
            getTask.done(function (x) {
                if (x) {
                    if (key && !_this.reduceKeys().contains(key)) {
                        _this.reduceKeys.push(key);
                        _this.colorMap[key] = _this.colors(Object.keys(_this.colorMap).length);
                        _this.reduceKey("");
                        _this.fetchDataFor(key).then(function (subTree) {
                            if (self.tree.children === undefined) {
                                self.tree.children = [];
                            }
                            if (subTree.children.length > 0) {
                                self.tree.children.push(subTree);
                                self.updateGraph();
                            }
                        }).always(function () {
                            if (needToChangeLoadingIndicator)
                                _this.showLoadingIndicator(false);
                            deferred.resolve();
                        });
                    }
                    else {
                        if (needToChangeLoadingIndicator)
                            _this.showLoadingIndicator(false);
                        deferred.resolve();
                    }
                }
                else {
                    messagePublisher.reportWarning("The key, " + key + " , is not contained in the reduce keys for this index.");
                    deferred.resolve();
                }
            }).always(function () {
                if (needToChangeLoadingIndicator)
                    _this.showLoadingIndicator(false);
                deferred.resolve();
            });
            return deferred;
        };
        visualizer.prototype.setSelectedIndex = function (indexName) {
            this.indexName(indexName);
            this.resetChart();
            this.updateGraph();
        };
        visualizer.prototype.clearChart = function () {
            this.indexName(visualizer.chooseIndexText);
            this.onlineMode(true);
            this.resetChart();
            this.updateGraph();
        };
        visualizer.prototype.fetchAllIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (results) { return _this.indexes(results.Indexes.map(function (i) {
                return {
                    name: i.Name,
                    hasReduce: !!i.LastReducedTimestamp
                };
            }).filter(function (i) { return i.hasReduce; })); });
        };
        visualizer.makeLinkId = function (link) {
            if ("cachedId" in link) {
                return link.cachedId;
            }
            else {
                var result = "link-" + visualizer.makeNodeId(link.source.origin) + "-" + visualizer.makeNodeId(link.target.origin);
                link.cachedId = result;
                return result;
            }
        };
        visualizer.makeNodeId = function (data) {
            if ("cachedId" in data) {
                return data.cachedId;
            }
            else {
                var nodeId = null;
                if (data.level == 4) {
                    nodeId = "node-" + data.level + "-" + data.payload.Data["__reduce_key"];
                }
                else if (data.payload) {
                    nodeId = "node-" + data.level + "-" + data.payload.ReduceKey + "-" + data.payload.Source + "-" + data.payload.Bucket;
                }
                else {
                    nodeId = "node-" + data.level + "-" + data.name;
                }
                nodeId = generalUtils.escape(nodeId);
                data.cachedId = nodeId;
                return nodeId;
            }
        };
        visualizer.prototype.estimateHeight = function () {
            var level1Nodes = 0;
            var nodes = [this.tree];
            var node = null;
            while ((node = nodes.pop()) != null) {
                if (node.level == 1)
                    level1Nodes++;
                if ((children = node.children) && (n = children.length)) {
                    var n, children;
                    while (--n >= 0)
                        nodes.push(children[n]);
                }
            }
            return this.boxSpacing * level1Nodes + this.margin.top + this.margin.bottom;
        };
        visualizer.prototype.goToDoc = function (docName) {
            router.navigate(appUrl.forEditDoc(docName, null, null, this.activeDatabase()));
        };
        visualizer.prototype.getTooltip = function (data) {
            var content = "";
            if (data.level == 0) {
                content += '<button data-bind="click: goToDoc.bind($root, \'' + data.name + '\')" class="btn" type="button">Go to document</button>';
            }
            else {
                var dataFormatted = JSON.stringify(data.payload.Data, undefined, 2);
                content += '<button type="button" class="close" data-bind="click: tooltipClose" aria-hidden="true"><i class="fa fa-times"></i></button>' +
                    "<table> ";
                if (data.level < 4) {
                    content += "<tr><td><strong>Reduce Key</strong></td><td>" + data.payload.ReduceKey + "</td></tr>" +
                        "<tr><td><strong>Timestamp</strong></td><td>" + data.payload.Timestamp + "</td></tr>" +
                        "<tr><td><strong>Etag</strong></td><td>" + data.payload.Etag + "</td></tr>" +
                        "<tr><td><strong>Bucket</strong></td><td>" + data.payload.Bucket + "</td></tr>" +
                        "<tr><td><strong>Source</strong></td><td>" + data.payload.Source + "</td></tr>";
                }
                content += "<tr><td><strong>Data</strong></td><td><pre>" + jsonUtil.syntaxHighlight(dataFormatted) + "</pre></td></tr>" +
                    "</table>";
            }
            return content;
        };
        visualizer.prototype.updateGraph = function () {
            var _this = this;
            this.height = this.estimateHeight();
            var self = this;
            d3.select("#visualizer")
                .style({ height: self.height + 'px' })
                .attr("viewBox", "0 0 " + this.width + " " + this.height);
            this.graph = d3.layout.cluster()
                .size([this.height - this.margin.top - this.margin.bottom, this.width - this.margin.left - this.margin.right]);
            this.nodes = this.graph.nodes(this.tree);
            this.links = this.graph.links(this.nodes);
            this.remapNodesAndLinks();
            this.links = this.links
                .filter(function (l) { return l.target.level < 4; })
                .map(function (l) {
                return {
                    source: {
                        y: self.xScale(l.source.level),
                        x: l.source.x + self.margin.top,
                        origin: l.source
                    },
                    target: {
                        y: self.xScale(l.target.level) + self.boxWidth,
                        x: l.target.x + self.margin.top,
                        origin: l.target
                    }
                };
            });
            this.node = this.node.data(this.nodes);
            this.link = this.link.data(this.links);
            var existingNodes = this.node
                .attr("transform", function (d) { return "translate(" + self.xScale(d.level) + "," + (d.x + _this.margin.top) + ")"; });
            existingNodes.select("rect")
                .attr('width', self.boxWidth);
            existingNodes.select('text')
                .attr('x', self.boxWidth / 2);
            this.link
                .attr("d", this.diagonal);
            this.link
                .enter()
                .append("path")
                .attr("class", "link")
                .attr("id", visualizer.makeLinkId)
                .attr("d", this.diagonal);
            var enteringNodes = this.node
                .enter()
                .append("g")
                .attr("id", visualizer.makeNodeId)
                .attr("transform", function (d) { return "translate(" + self.xScale(d.level) + "," + (d.x + _this.margin.top) + ")"; });
            enteringNodes
                .filter(function (d) { return d.level > 4; })
                .classed("hidden", true);
            enteringNodes
                .append("rect")
                .attr('class', 'nodeRect')
                .attr('x', 0)
                .attr('y', -10)
                .attr("fill", function (d) { return d.level > 0 ? self.colorMap[d.name] : 'white'; })
                .attr('width', self.boxWidth)
                .attr('height', 20)
                .attr('rx', 5)
                .on("click", function (d) {
                nv.tooltip.cleanup();
                var offset = $(this).offset();
                var containerOffset = $("#visualizerSection").offset();
                nv.tooltip.show([offset.left - containerOffset.left + self.boxWidth / 2, offset.top - containerOffset.top], self.getTooltip(d), 'n', 25, document.getElementById("visualizerSection"), "selectable-tooltip");
                $(".nvtooltip").each(function (i, elem) {
                    ko.applyBindings(self, elem);
                });
            });
            enteringNodes
                .filter(function (d) { return d.level <= 3; })
                .append("rect")
                .attr('class', 'nodeCheck')
                .attr('x', 3)
                .attr('y', -7)
                .attr('width', 14)
                .attr('height', 14)
                .attr('rx', 2)
                .on("click", function (d) {
                nv.tooltip.cleanup();
                var selected = this.classList.contains("selected");
                d3.select(this).classed("selected", !selected);
                if (selected) {
                    self.selectedDocs.remove(d);
                }
                else {
                    if (!self.selectedDocs.contains(d)) {
                        self.selectedDocs.push(d);
                    }
                }
            });
            enteringNodes.append("text")
                .attr("x", self.boxWidth / 2)
                .attr("y", 4.5)
                .text(function (d) { return self.trimTextIfNeeded(d.name); });
            this.node.exit().remove();
            this.link.exit().remove();
        };
        visualizer.prototype.trimTextIfNeeded = function (input) {
            return input.length > 25 ? input.substring(0, 25) + "..." : input;
        };
        visualizer.prototype.fetchDataFor = function (key) {
            var _this = this;
            var allDataFetched = $.Deferred();
            var mapFetcherTask = new chunkFetcher(function (skip, take) { return new queryIndexDebugMapCommand(_this.indexName(), _this.activeDatabase(), { key: key }, skip, take).execute(); }).execute();
            var reduce1FetcherTask = new chunkFetcher(function (skip, take) { return new queryIndexDebugReduceCommand(_this.indexName(), _this.activeDatabase(), 1, key, skip, take).execute(); }).execute();
            var reduce2FetcherTask = new chunkFetcher(function (skip, take) { return new queryIndexDebugReduceCommand(_this.indexName(), _this.activeDatabase(), 2, key, skip, take).execute(); }).execute();
            var indexEntryTask = new queryIndexDebugAfterReduceCommand(this.indexName(), this.activeDatabase(), [key]).execute();
            $.when(mapFetcherTask, reduce1FetcherTask, reduce2FetcherTask, indexEntryTask)
                .then(function (map, reduce1, reduce2, indexEntries) {
                if (map.length == 0 && reduce1.length == 0 && reduce2.length == 0) {
                    allDataFetched.resolve({
                        level: 4,
                        name: key,
                        children: []
                    });
                    return;
                }
                var mapGroupedByBucket = d3
                    .nest()
                    .key(function (k) { return String(k.Bucket); })
                    .map(map, d3.map);
                var reduce1GropedByBucket = d3
                    .nest()
                    .key(function (d) { return String(d.Bucket); })
                    .map(reduce1, d3.map);
                var indexEntry = indexEntries[0];
                if (reduce2.length == 0 && reduce1.length == 0) {
                    var subTree = map.map(function (m) {
                        return {
                            name: m.ReduceKey,
                            payload: m,
                            level: 1,
                            children: [
                                {
                                    name: m.Source,
                                    level: 0,
                                    children: []
                                }
                            ]
                        };
                    });
                    allDataFetched.resolve({
                        level: 4,
                        name: key,
                        payload: { Data: indexEntry },
                        children: subTree
                    });
                }
                if (reduce2.length > 0 && reduce1.length > 0) {
                    var subTree = reduce2.map(function (r2) {
                        return {
                            name: r2.ReduceKey,
                            payload: r2,
                            level: 3,
                            children: reduce1GropedByBucket.get(r2.Source).map(function (r1) {
                                return {
                                    name: r1.ReduceKey,
                                    payload: r1,
                                    level: 2,
                                    children: mapGroupedByBucket.get(r1.Source).map(function (m) {
                                        return {
                                            name: m.ReduceKey,
                                            payload: m,
                                            level: 1,
                                            children: [
                                                {
                                                    name: m.Source,
                                                    level: 0,
                                                    children: []
                                                }
                                            ]
                                        };
                                    })
                                };
                            })
                        };
                    });
                    allDataFetched.resolve({
                        level: 4,
                        name: key,
                        payload: { Data: indexEntry },
                        children: subTree
                    });
                }
            }, function () {
                allDataFetched.reject();
            });
            return allDataFetched;
        };
        visualizer.prototype.tooltipClose = function () {
            nv.tooltip.cleanup();
        };
        visualizer.prototype.selectReduceKey = function (value) {
            this.addReduceKey(value);
            this.reduceKey("");
        };
        visualizer.prototype.selectDocKey = function (value) {
            this.addDocKey(value);
            this.docKey("");
        };
        visualizer.prototype.transiviteClosure = function () {
            var result = d3.set([]);
            var nodes = this.selectedDocs();
            var queue = [];
            queue.pushAll(nodes);
            this.treeTranverse(queue, function (node) { return result.add(visualizer.makeNodeId(node)); });
            return result;
        };
        visualizer.prototype.treeTranverse = function (queue, callback, refPropName) {
            if (refPropName === void 0) { refPropName = "connections"; }
            var everQueued = d3.set([]);
            while (queue.length > 0) {
                var node = queue.shift();
                callback(node);
                if (node[refPropName]) {
                    node[refPropName].forEach(function (c) {
                        var nodeId = visualizer.makeNodeId(c);
                        if (!everQueued.has(nodeId)) {
                            queue.push(c);
                            everQueued.add(nodeId);
                        }
                    });
                }
            }
        };
        visualizer.prototype.exportTree = function (root) {
            var _this = this;
            return {
                level: root.level,
                name: root.name,
                payload: root.payload,
                children: root.children ? $.map(root.children, function (v, i) { return _this.exportTree(v); }) : undefined
            };
        };
        visualizer.prototype.computeLinks = function (nodes) {
            var output = d3.set([]);
            this.links.forEach(function (link) {
                var node1 = visualizer.makeNodeId(link.source.origin);
                var node2 = visualizer.makeNodeId(link.target.origin);
                if (nodes.has(node1) && nodes.has(node2)) {
                    output.add(visualizer.makeLinkId(link));
                }
            });
            return output;
        };
        visualizer.prototype.repaintSelectedNodes = function () {
            var newClosure = this.transiviteClosure();
            var currentClosure = this.currentlySelectedNodes;
            var currentLinks = this.currentlySelectedLinks;
            var newLinks = this.computeLinks(newClosure);
            // remove all works in situ
            var incoming = newClosure.values();
            incoming.removeAll(currentClosure.values());
            var outgoing = currentClosure.values();
            outgoing.removeAll(newClosure.values());
            var incomingLinks = newLinks.values();
            incomingLinks.removeAll(currentLinks.values());
            var outgoingLinks = currentLinks.values();
            outgoingLinks.removeAll(newLinks.values());
            incoming.forEach(function (name) { return d3.select("#" + name).select('rect').classed("highlight", true); });
            outgoing.forEach(function (name) { return d3.select("#" + name).select('rect').classed("highlight", false); });
            incomingLinks.forEach(function (name) { return d3.select("#" + name).classed("selected", true); });
            outgoingLinks.forEach(function (name) { return d3.select("#" + name).classed("selected", false); });
            this.currentlySelectedNodes = newClosure;
            this.currentlySelectedLinks = newLinks;
        };
        visualizer.prototype.remapNodesAndLinks = function () {
            var seenNames = {};
            var nodesToDelete = [];
            // process nodes
            this.nodes.forEach(function (node) {
                if (node.level == 0) {
                    if (node.name in seenNames) {
                        nodesToDelete.push(node);
                    }
                    else {
                        seenNames[node.name] = node;
                    }
                }
            });
            this.nodes.removeAll(nodesToDelete);
            // process links
            this.links = this.links.map(function (link) {
                if (!("connections" in link.target)) {
                    link.target.connections = [];
                }
                if (link.target.level == 0) {
                    var newTarget = seenNames[link.target.name];
                    link.target = newTarget;
                    if (!link.target.connections.contains(link.source)) {
                        link.target.connections.push(link.source);
                    }
                    return link;
                }
                else {
                    if (!link.target.connections.contains(link.source)) {
                        link.target.connections.push(link.source);
                    }
                    return link;
                }
            });
        };
        visualizer.prototype.toggleFullscreen = function (e) {
            if ($(document).fullScreen()) {
                $("#visualizerSection").width('').height('');
                $("#keysDialogBtn").removeAttr('disabled');
            }
            else {
                $("#visualizerSection").width("100%").height("100%");
                $("#keysDialogBtn").attr('disabled', 'disabled');
            }
            $("#visualizerSection").toggleFullScreen();
        };
        visualizer.prototype.displayKeyInfo = function () {
            app.showDialog(new visualizerKeys(this));
        };
        visualizer.prototype.saveAsSvg = function () {
            svgDownloader.downloadSvg(d3.select('#visualizer').node(), 'visualization.svg', function (e) { return visualizer.visualizationCss; });
        };
        visualizer.prototype.saveAsPng = function () {
            svgDownloader.downloadPng(d3.select('#visualizer').node(), 'visualization.png', function (e) { return visualizer.visualizationCss; });
        };
        visualizer.prototype.saveAsJson = function () {
            var model = {
                indexName: this.indexName(),
                docKeys: this.docKeys(),
                reduceKeys: this.reduceKeys(),
                tree: this.exportTree(this.tree)
            };
            fileDownloader.downloadAsJson(model, "visualizer.json");
        };
        visualizer.prototype.chooseImportFile = function () {
            var _this = this;
            var dialog = new visualizerImport();
            dialog.task().
                done(function (importedData) {
                _this.clearChart();
                _this.onlineMode(false);
                _this.docKeys(importedData.docKeys);
                _this.reduceKeys(importedData.reduceKeys);
                importedData.reduceKeys.forEach(function (reduceKey) {
                    _this.colorMap[reduceKey] = _this.colors(Object.keys(_this.colorMap).length);
                });
                _this.tree = importedData.tree;
                _this.indexName(importedData.indexName);
                _this.updateGraph();
            });
            app.showDialog(dialog);
        };
        visualizer.chooseIndexText = "Select an index";
        visualizer.visualizationCss = 'svg { background-color: white; }\n' +
            '* { box-sizing: border-box; }\n' +
            '.hidden { display: none !important; visibility: hidden !important; }\n' +
            'svg text { font-style: normal; font-variant: normal; font-weight: normal; font-size: 12px; line-height: normal; font-family: Arial; }\n' +
            '.nodeRect { stroke: rgb(119, 119, 119); stroke-width: 1.5px; fill-opacity: 0.4 !important; }\n' +
            '.nodeCheck { stroke-width: 2px; stroke: rgb(0, 0, 0); fill: rgb(255, 255, 255); }\n' +
            '.hidden { display: none; }\n' +
            'g { font-style: normal; font-variant: normal; font-weight: normal; font-size: 10px; line-height: normal; font-family: sans - serif; cursor: pointer; }\n' +
            '.link { fill: none; stroke: rgb(204, 204, 204); stroke-width: 1.5px; }\n' +
            'text { pointer-events: none; text-anchor: middle; }\n' +
            '.link.selected { fill: none; stroke: black; stroke-width: 2.5px; } \n';
        return visualizer;
    })(viewModelBase);
    return visualizer;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/visualizerImport", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var visualizerImport = (function (_super) {
        __extends(visualizerImport, _super);
        function visualizerImport() {
            _super.apply(this, arguments);
            this.importTask = $.Deferred();
            this.hasFileSelected = ko.observable(false);
            this.isImporting = ko.observable(false);
        }
        visualizerImport.prototype.fileSelected = function (args) {
            this.hasFileSelected(true);
        };
        visualizerImport.prototype.doImport = function () {
            var _this = this;
            var fileInput = document.querySelector("#importFilePicker");
            var self = this;
            var file = fileInput.files[0];
            var reader = new FileReader();
            reader.onload = function () {
                self.dataImported(this.result);
            };
            reader.onerror = function (error) { return _this.importTask.reject(error); };
            reader.readAsText(file);
        };
        visualizerImport.prototype.dataImported = function (result) {
            var json = JSON.parse(result);
            this.importTask.resolve(json);
            dialog.close(this);
        };
        visualizerImport.prototype.cancel = function () {
            dialog.close(this);
        };
        visualizerImport.prototype.task = function () {
            return this.importTask.promise();
        };
        return visualizerImport;
    })(dialogViewModelBase);
    return visualizerImport;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/visualizerKeys", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var visualizerKeys = (function (_super) {
        __extends(visualizerKeys, _super);
        function visualizerKeys(visualizer) {
            _super.call(this);
            this.visualizer = visualizer;
        }
        visualizerKeys.prototype.cancel = function () {
            dialog.close(this);
        };
        return visualizerKeys;
    })(dialogViewModelBase);
    return visualizerKeys;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebug", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/database/status/status", 'viewmodels/shell'], function (require, exports, viewModelBase, appUrl, status, shell) {
    var statusDebug = (function (_super) {
        __extends(statusDebug, _super);
        function statusDebug() {
            var _this = this;
            _super.call(this);
            var db = this.activeDatabase();
            var durandalConfigurationArray = [
                { route: 'databases/status/debug', moduleId: 'viewmodels/database/status/debug/statusDebugChanges', title: 'Changes', tooltip: 'Shows information about active changes API subscriptions', nav: true, hash: appUrl.forCurrentDatabase().statusDebugChanges },
                { route: 'databases/status/debug/dataSubscriptions', moduleId: 'viewmodels/database/status/debug/statusDebugDataSubscriptions', title: 'Data subscriptions', tooltip: "Shows information about data subscriptions", nav: true, hash: appUrl.forCurrentDatabase().dataSubscriptions },
                { route: 'databases/status/debug/metrics', moduleId: 'viewmodels/database/status/debug/statusDebugMetrics', title: 'Metrics', tooltip: "Shows database metrics", nav: true, hash: appUrl.forCurrentDatabase().statusDebugMetrics },
                { route: 'databases/status/debug/config', moduleId: 'viewmodels/database/status/debug/statusDebugConfig', title: 'Config', tooltip: "Displays server configuration", nav: true, hash: appUrl.forCurrentDatabase().statusDebugConfig },
                { route: 'databases/status/debug/docrefs', moduleId: 'viewmodels/database/status/debug/statusDebugDocrefs', title: 'Doc refs', tooltip: "Allows to find documents referenced by given document id", nav: true, hash: appUrl.forCurrentDatabase().statusDebugDocrefs },
                { route: 'databases/status/debug/currentlyIndexing', moduleId: 'viewmodels/database/status/debug/statusDebugCurrentlyIndexing', title: 'Currently indexing', tooltip: "Displays currently performed indexing work", nav: true, hash: appUrl.forCurrentDatabase().statusDebugCurrentlyIndexing },
                { route: 'databases/status/debug/queries', moduleId: 'viewmodels/database/status/debug/statusDebugQueries', title: 'Queries', tooltip: "Displays currently running queries", nav: true, hash: appUrl.forCurrentDatabase().statusDebugQueries },
                { route: 'databases/status/debug/tasks', moduleId: 'viewmodels/database/status/debug/statusDebugTasks', title: 'Tasks', tooltip: "Displays currently running index tasks", nav: true, hash: appUrl.forCurrentDatabase().statusDebugTasks },
                { route: 'databases/status/debug/routes', moduleId: 'viewmodels/database/status/debug/statusDebugRoutes', title: 'Routes', tooltip: "Displays all available routes", nav: shell.isGlobalAdmin(), hash: appUrl.forCurrentDatabase().statusDebugRoutes },
                { route: 'databases/status/debug/sqlReplication', moduleId: 'viewmodels/database/status/debug/statusDebugSqlReplication', title: 'SQL Replication', tooltip: "Shows information about SQL replication", nav: db.isBundleActive("SqlReplication"), hash: appUrl.forCurrentDatabase().statusDebugSqlReplication },
                { route: 'databases/status/debug/indexFields', moduleId: 'viewmodels/database/status/debug/statusDebugIndexFields', title: 'Index fields', tooltip: "Shows names of indexed fields based on entered index definition", nav: true, hash: appUrl.forCurrentDatabase().statusDebugIndexFields },
                { route: 'databases/status/debug/identities', moduleId: 'viewmodels/database/status/debug/statusDebugIdentities', title: 'Identities', tooltip: "Shows identities values for collections", nav: true, hash: appUrl.forCurrentDatabase().statusDebugIdentities },
                { route: 'databases/status/debug/websocket', moduleId: 'viewmodels/database/status/debug/statusDebugWebSocket', title: 'Web Socket', tooltip: "Allows to debug websockets connection", nav: true, hash: appUrl.forCurrentDatabase().statusDebugWebSocket },
                { route: 'databases/status/debug/persist', moduleId: 'viewmodels/database/status/debug/statusDebugPersistAutoIndex', title: 'Persist auto index', tooltip: "Persists auto index", nav: true, hash: appUrl.forCurrentDatabase().statusDebugPersistAutoIndex },
                { route: 'databases/status/debug/explainReplication', moduleId: 'viewmodels/database/status/debug/statusDebugExplainReplication', title: 'Explain replication', tooltip: "Shows information about replication of given document to given replication destination", nav: db.isBundleActive("Replication"), hash: appUrl.forCurrentDatabase().statusDebugExplainReplication },
            ];
            this.router = status.statusRouter.createChildRouter()
                .map(durandalConfigurationArray)
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return statusDebug;
    })(viewModelBase);
    return statusDebug;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugChanges", ["require", "exports", "commands/database/debug/getStatusDebugChangesCommand", "viewmodels/viewModelBase"], function (require, exports, getStatusDebugChangesCommand, viewModelBase) {
    var statusDebugChanges = (function (_super) {
        __extends(statusDebugChanges, _super);
        function statusDebugChanges() {
            _super.apply(this, arguments);
            this.data = ko.observable();
        }
        statusDebugChanges.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.fetchStatusDebugChanges(); });
            this.updateHelpLink('JHZ574');
            return this.fetchStatusDebugChanges();
        };
        statusDebugChanges.prototype.fetchStatusDebugChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugChangesCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugChanges;
    })(viewModelBase);
    return statusDebugChanges;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugConfig", ["require", "exports", "commands/database/debug/getStatusDebugConfigCommand", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, getStatusDebugConfigCommand, viewModelBase, aceEditorBindingHandler) {
    var statusDebugConfig = (function (_super) {
        __extends(statusDebugConfig, _super);
        function statusDebugConfig() {
            var _this = this;
            _super.call(this);
            this.data = ko.observable("");
            aceEditorBindingHandler.install();
            this.isForbidden = ko.computed(function () { return !_this.data(); });
        }
        statusDebugConfig.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deffered = $.Deferred();
            this.fetchStatusDebugConfig().always(function () { return deffered.resolve({ can: true }); });
            return deffered;
        };
        statusDebugConfig.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchStatusDebugConfig(); });
        };
        statusDebugConfig.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#statusDebugConfigEditor");
            if (editorElement.length > 0) {
                this.editor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#statusDebugConfigEditor").on('DynamicHeightSet', function () { return _this.editor.resize(); });
        };
        statusDebugConfig.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#statusDebugConfigEditor").off('DynamicHeightSet');
        };
        statusDebugConfig.prototype.fetchStatusDebugConfig = function () {
            var _this = this;
            var db = this.activeDatabase();
            return new getStatusDebugConfigCommand(db)
                .execute()
                .done(function (results) { return _this.data(JSON.stringify(results, null, 4)); });
        };
        return statusDebugConfig;
    })(viewModelBase);
    return statusDebugConfig;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugCurrentlyIndexing", ["require", "exports", "commands/database/debug/getStatusDebugCurrentlyIndexingCommand", "viewmodels/viewModelBase", "common/bindingHelpers/autoRefreshBindingHandler"], function (require, exports, getStatusDebugCurrentlyIndexingCommand, viewModelBase, autoRefreshBindingHandler) {
    var statusDebugCurrentlyIndexing = (function (_super) {
        __extends(statusDebugCurrentlyIndexing, _super);
        function statusDebugCurrentlyIndexing() {
            _super.call(this);
            this.data = ko.observable();
            autoRefreshBindingHandler.install();
        }
        statusDebugCurrentlyIndexing.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchCurrentlyIndexing(); });
            return this.fetchCurrentlyIndexing();
        };
        statusDebugCurrentlyIndexing.prototype.fetchCurrentlyIndexing = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugCurrentlyIndexingCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugCurrentlyIndexing;
    })(viewModelBase);
    return statusDebugCurrentlyIndexing;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugDataSubscriptions", ["require", "exports", "commands/database/debug/getStatusDebugDataSubscriptionsCommand", "viewmodels/viewModelBase"], function (require, exports, getStatusDebugDataSubscriptionsCommand, viewModelBase) {
    var statusDebugDataSubscriptions = (function (_super) {
        __extends(statusDebugDataSubscriptions, _super);
        function statusDebugDataSubscriptions() {
            _super.apply(this, arguments);
            this.data = ko.observable();
        }
        statusDebugDataSubscriptions.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.fetchStatusDebugDataSubscriptions(); });
            return this.fetchStatusDebugDataSubscriptions();
        };
        statusDebugDataSubscriptions.prototype.fetchStatusDebugDataSubscriptions = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugDataSubscriptionsCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugDataSubscriptions;
    })(viewModelBase);
    return statusDebugDataSubscriptions;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugDocrefs", ["require", "exports", "viewmodels/viewModelBase", "models/database/documents/customColumns", 'models/database/documents/customColumnParams', "common/pagedList", "commands/database/debug/getDocRefsCommand"], function (require, exports, viewModelBase, customColumns, customColumnParams, pagedList, getDocRefsCommand) {
    var statusDebugDocrefs = (function (_super) {
        __extends(statusDebugDocrefs, _super);
        function statusDebugDocrefs() {
            var _this = this;
            _super.apply(this, arguments);
            this.currentDocRefs = ko.observable();
            this.columns = ko.observable(customColumns.empty());
            this.docId = ko.observable("");
            this.canSearch = ko.computed(function () { return _this.docId().length > 0; });
            this.resultsCount = ko.computed(function () {
                if (!_this.currentDocRefs())
                    return -1;
                return _this.currentDocRefs().totalResultCount();
            });
        }
        statusDebugDocrefs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.columns().columns([
                new customColumnParams({ Header: "Id", Binding: "Id", DefaultWidth: 300 }),
            ]);
        };
        statusDebugDocrefs.prototype.fetchDocRefs = function () {
            this.currentDocRefs(this.createPagedList(this.activeDatabase()));
        };
        statusDebugDocrefs.prototype.createPagedList = function (db) {
            var _this = this;
            var fetcher = function (skip, take) {
                return new getDocRefsCommand(db, _this.docId(), skip, take).execute();
            };
            return new pagedList(fetcher);
        };
        return statusDebugDocrefs;
    })(viewModelBase);
    return statusDebugDocrefs;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugExplainReplication", ["require", "exports", "viewmodels/viewModelBase", "common/extensions", 'commands/database/replication/getReplicationsCommand', "commands/database/documents/getDocumentsMetadataByIDPrefixCommand", "commands/database/replication/explainReplicationCommand"], function (require, exports, viewModelBase, extensions, getReplicationsCommand, getDocumentsMetadataByIDPrefixCommand, explainReplicationCommand) {
    var statusDebugExplainReplication = (function (_super) {
        __extends(statusDebugExplainReplication, _super);
        function statusDebugExplainReplication() {
            var _this = this;
            _super.call(this);
            this.destinations = ko.observable([]);
            this.selectedDestination = ko.observable();
            this.documentId = ko.observable();
            this.documentIdSearchResults = ko.observableArray();
            this.explanation = ko.observable();
            this.buttonEnabled = ko.computed(function () {
                var destionationSelected = _this.selectedDestination() != null;
                var documentSelected = _this.documentId();
                return destionationSelected && (documentSelected != null && documentSelected.length > 0);
            });
            this.selectedDestionationText = ko.computed(function () {
                var dest = _this.selectedDestination();
                if (dest) {
                    return dest.Database + ' on ' + dest.Url;
                }
                else {
                    return 'select destination';
                }
            });
            extensions.install();
            this.documentId.throttle(250).subscribe(function (search) { return _this.fetchDocSearchResults(search); });
        }
        statusDebugExplainReplication.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            $.when(this.fetchReplicationDestinations())
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        statusDebugExplainReplication.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
        };
        statusDebugExplainReplication.prototype.fetchReplicationDestinations = function () {
            var _this = this;
            return new getReplicationsCommand(this.activeDatabase())
                .execute()
                .done(function (destinations) {
                _this.destinations(destinations.MergedDocument.Destinations);
            });
        };
        statusDebugExplainReplication.prototype.setSelectedDestination = function (destination) {
            this.selectedDestination(destination);
        };
        statusDebugExplainReplication.prototype.selectDocument = function (data) {
            this.documentId(data['@metadata']['@id']);
        };
        statusDebugExplainReplication.prototype.fetchDocSearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.activeDatabase())
                    .execute()
                    .done(function (results) {
                    if (_this.documentId() === query) {
                        _this.documentIdSearchResults(results);
                    }
                });
            }
            else if (query.length == 0) {
                this.documentIdSearchResults.removeAll();
            }
        };
        statusDebugExplainReplication.prototype.explain = function () {
            var _this = this;
            new explainReplicationCommand(this.activeDatabase(), this.documentId(), this.selectedDestination().Url, this.selectedDestination().Database)
                .execute()
                .done(function (result) { return _this.explanation(result); });
        };
        return statusDebugExplainReplication;
    })(viewModelBase);
    return statusDebugExplainReplication;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugIdentities", ["require", "exports", "viewmodels/viewModelBase", "common/pagedList", "models/database/documents/customColumns", 'models/database/documents/customColumnParams', "commands/database/debug/getIdentitiesCommand"], function (require, exports, viewModelBase, pagedList, customColumns, customColumnParams, getIdentitiesCommand) {
    var statusDebugIdentities = (function (_super) {
        __extends(statusDebugIdentities, _super);
        function statusDebugIdentities() {
            var _this = this;
            _super.apply(this, arguments);
            this.currentIdentities = ko.observable();
            this.columns = ko.observable(customColumns.empty());
            this.resultsCount = ko.computed(function () {
                if (!_this.currentIdentities())
                    return -1;
                return _this.currentIdentities().totalResultCount();
            });
        }
        statusDebugIdentities.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.columns().columns([
                new customColumnParams({ Header: "Key", Binding: "Key", DefaultWidth: 300 }),
                new customColumnParams({ Header: "Value", Binding: "Value", DefaultWidth: 300 }),
            ]);
            this.updateHelpLink('JHZ574');
            this.fetchIdentities();
        };
        statusDebugIdentities.prototype.fetchIdentities = function () {
            this.currentIdentities(this.createPagedList(this.activeDatabase()));
        };
        statusDebugIdentities.prototype.createPagedList = function (db) {
            var fetcher = function (skip, take) {
                return new getIdentitiesCommand(db, skip, take).execute();
            };
            return new pagedList(fetcher);
        };
        return statusDebugIdentities;
    })(viewModelBase);
    return statusDebugIdentities;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugIndexFields", ["require", "exports", "commands/database/debug/getStatusDebugIndexFieldsCommand", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher"], function (require, exports, getStatusDebugIndexFieldsCommand, viewModelBase, aceEditorBindingHandler, messagePublisher) {
    var statusDebugIndexFields = (function (_super) {
        __extends(statusDebugIndexFields, _super);
        function statusDebugIndexFields() {
            _super.call(this);
            this.result = ko.observable();
            this.indexStr = ko.observable("");
            aceEditorBindingHandler.install();
        }
        statusDebugIndexFields.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
        };
        statusDebugIndexFields.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#indexDefLabel").popover({
                html: true,
                trigger: 'hover',
                container: '.form-horizontal',
                content: 'Enter index definition and click <kbd>Show index fields</kbd>.<br /> Example:<pre><span class="code-keyword">from</span> doc <span class="code-keyword">in</span> docs <span class="code-keyword">select new</span> { Id = doc.Id }</pre>',
            });
        };
        statusDebugIndexFields.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#statusDebugIndexFieldsEditor");
            if (editorElement.length > 0) {
                this.editor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#statusDebugIndexFieldsEditor").on('DynamicHeightSet', function () { return _this.editor.resize(); });
            this.focusOnEditor();
            this.indexStr.subscribe(function (newValue) {
                var message = "";
                var currentEditor = aceEditorBindingHandler.currentEditor;
                var textarea = $(currentEditor.container).find("textarea")[0];
                if (newValue === "") {
                    message = "Please fill out this field.";
                }
                textarea.setCustomValidity(message);
                /*setTimeout(() => {
                    var annotations = currentEditor.getSession().getAnnotations();
                    var isErrorExists = false;
                    for (var i = 0; i < annotations.length; i++) {
                        var annotationType = annotations[i].type;
                        if (annotationType === "error" || annotationType === "warning") {
                            isErrorExists = true;
                            break;
                        }
                    }
                    if (isErrorExists) {
                        message = "The script isn't a javascript legal expression!";
                        textarea.setCustomValidity(message);
                    }
                }, 700);*/
            });
            this.indexStr.valueHasMutated();
        };
        statusDebugIndexFields.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#statusDebugIndexFieldsEditor").off('DynamicHeightSet');
        };
        statusDebugIndexFields.prototype.focusOnEditor = function () {
            this.editor.focus();
        };
        statusDebugIndexFields.prototype.fetchIndexFields = function () {
            var _this = this;
            this.result(null);
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugIndexFieldsCommand(db, this.indexStr())
                    .execute()
                    .done(function (results) { return _this.result(results); })
                    .fail(function (response) {
                    _this.focusOnEditor();
                    messagePublisher.reportError("Failed to compute index fields!", response.responseText, response.statusText);
                });
            }
            return null;
        };
        return statusDebugIndexFields;
    })(viewModelBase);
    return statusDebugIndexFields;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugMetrics", ["require", "exports", "commands/database/debug/getDebugMetricsCommand", "viewmodels/viewModelBase"], function (require, exports, getDebugMetricsCommand, viewModelBase) {
    var statusDebugMetrics = (function (_super) {
        __extends(statusDebugMetrics, _super);
        function statusDebugMetrics() {
            var _this = this;
            _super.apply(this, arguments);
            this.data = ko.observable();
            this.requestPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().RequestsDuration.Percentiles);
                }
                return null;
            });
            this.staleIndexMapsPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().StaleIndexMaps.Percentiles);
                }
                return null;
            });
            this.staleIndexReducesPercentiles = ko.computed(function () {
                if (_this.data()) {
                    return _this.extractPercentiles(_this.data().StaleIndexReduces.Percentiles);
                }
                return null;
            });
            this.gauges = ko.computed(function () {
                if (_this.data()) {
                    return $.map(_this.data().Gauges, function (v, k) {
                        return {
                            key: k,
                            values: $.map(v, function (innerValue, innerKey) {
                                return {
                                    key: innerKey,
                                    value: innerValue
                                };
                            })
                        };
                    });
                }
                return null;
            });
            this.replicationDestinations = ko.computed(function () {
                if (_this.data()) {
                    // sample destinations using ReplicationDurationHistogram
                    return $.map(_this.data().ReplicationBatchSizeHistogram, function (v, key) { return key; });
                }
                return null;
            });
        }
        statusDebugMetrics.prototype.extractPercentiles = function (input) {
            var result = [];
            for (var prop in input) {
                var v = input[prop];
                result.push({ "key": prop, "value": v });
            }
            return result;
        };
        statusDebugMetrics.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchStatusDebugMetrics(); });
            return this.fetchStatusDebugMetrics();
        };
        statusDebugMetrics.prototype.fetchStatusDebugMetrics = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getDebugMetricsCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugMetrics;
    })(viewModelBase);
    return statusDebugMetrics;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugPersistAutoIndex", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "models/database/index/index", "commands/database/index/persistIndexCommand"], function (require, exports, viewModelBase, getDatabaseStatsCommand, index, persistIndexCommand) {
    var statusDebugPersistAutoIndex = (function (_super) {
        __extends(statusDebugPersistAutoIndex, _super);
        function statusDebugPersistAutoIndex() {
            var _this = this;
            _super.apply(this, arguments);
            this.onRamIndexes = ko.observableArray([]);
            this.indexName = ko.observable(statusDebugPersistAutoIndex.selectIndexText);
            this.buttonEnabled = ko.computed(function () {
                return _this.indexName() != statusDebugPersistAutoIndex.selectIndexText;
            });
        }
        statusDebugPersistAutoIndex.prototype.persistIndex = function () {
            var _this = this;
            var indexName = this.indexName();
            new persistIndexCommand(indexName, this.activeDatabase())
                .execute()
                .done(function () {
                _this.indexName(statusDebugPersistAutoIndex.selectIndexText);
                var index = _this.onRamIndexes.first(function (i) { return i.name == indexName; });
                if (index) {
                    _this.onRamIndexes.remove(index);
                }
            });
        };
        statusDebugPersistAutoIndex.prototype.setSelectedIndex = function (indexName) {
            this.indexName(indexName);
        };
        statusDebugPersistAutoIndex.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
        };
        statusDebugPersistAutoIndex.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            $.when(this.fetchIndexes())
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ can: false }); });
            return deferred;
        };
        statusDebugPersistAutoIndex.prototype.fetchIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase()).execute()
                .done(function (result) {
                _this.onRamIndexes(result.Indexes.map(function (i) { return new index(i); }).filter(function (i) { return i.isOnRam() != "false"; }));
            });
        };
        statusDebugPersistAutoIndex.selectIndexText = "Select index";
        return statusDebugPersistAutoIndex;
    })(viewModelBase);
    return statusDebugPersistAutoIndex;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugQueries", ["require", "exports", "commands/database/debug/getStatusDebugQueriesCommand", "commands/database/query/getKillQueryCommand", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "models/database/debug/statusDebugQueriesGroup", "models/database/debug/statusDebugQueriesQuery", "common/bindingHelpers/autoRefreshBindingHandler"], function (require, exports, getStatusDebugQueriesCommand, getKillQueryCommand, viewModelBase, aceEditorBindingHandler, statusDebugQueriesGroup, statusDebugQueriesQuery, autoRefreshBindingHandler) {
    var statusDebugQueries = (function (_super) {
        __extends(statusDebugQueries, _super);
        function statusDebugQueries() {
            _super.call(this);
            this.data = ko.observableArray();
            autoRefreshBindingHandler.install();
            aceEditorBindingHandler.install();
        }
        statusDebugQueries.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchCurrentQueries(); });
            return this.fetchCurrentQueries();
        };
        statusDebugQueries.prototype.fetchCurrentQueries = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugQueriesCommand(db)
                    .execute()
                    .done(function (results) { return _this.onResultsLoaded(results); });
            }
            return null;
        };
        statusDebugQueries.prototype.onResultsLoaded = function (results) {
            var _this = this;
            var currentGroups = $.map(this.data(), function (group) { return group.indexName; });
            $.map(results, function (dtoGroup) {
                if (dtoGroup.Queries.length > 0) {
                    var foundGroup = _this.data().first(function (item) { return item.indexName === dtoGroup.IndexName; });
                    if (foundGroup) {
                        currentGroups.remove(dtoGroup.IndexName);
                    }
                    else {
                        foundGroup = new statusDebugQueriesGroup(dtoGroup);
                        _this.data.push(foundGroup);
                    }
                    _this.updateGroup(foundGroup, dtoGroup);
                }
            });
            // remove empty and unused groups
            currentGroups.forEach(function (group) {
                var foundGroup = _this.data.first(function (item) { return item.indexName === group; });
                if (foundGroup) {
                    _this.data.remove(foundGroup);
                }
            });
        };
        statusDebugQueries.prototype.updateGroup = function (group, dtoGroup) {
            var currentQueryIds = $.map(group.queries(), function (query) { return query.queryId; });
            $.map(dtoGroup.Queries, function (dtoQuery) {
                var foundQuery = group.queries.first(function (item) { return item.queryId == dtoQuery.QueryId; });
                if (foundQuery) {
                    currentQueryIds.remove(foundQuery.queryId);
                    foundQuery.duration(dtoQuery.Duration);
                }
                else {
                    group.queries.push(new statusDebugQueriesQuery(dtoQuery));
                }
            });
            // remove unused queries
            currentQueryIds.forEach(function (query) {
                var foundQuery = group.queries.first(function (item) { return item.queryId == query; });
                if (foundQuery) {
                    group.queries.remove(foundQuery);
                }
            });
        };
        statusDebugQueries.prototype.killQuery = function (queryId) {
            var _this = this;
            new getKillQueryCommand(this.activeDatabase(), queryId)
                .execute()
                .done(function () {
                // find and delete query from model
                _this.data().forEach(function (group) {
                    var foundQuery = group.queries.first(function (q) { return q.queryId == queryId; });
                    if (foundQuery) {
                        group.queries.remove(foundQuery);
                    }
                });
            });
        };
        return statusDebugQueries;
    })(viewModelBase);
    return statusDebugQueries;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugRoutes", ["require", "exports", "commands/database/debug/getStatusDebugRoutesCommand", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", 'viewmodels/shell'], function (require, exports, getStatusDebugRoutesCommand, viewModelBase, aceEditorBindingHandler, shell) {
    var statusDebugRoutes = (function (_super) {
        __extends(statusDebugRoutes, _super);
        function statusDebugRoutes() {
            _super.call(this);
            this.data = ko.observable();
            this.isGlobalAdmin = shell.isGlobalAdmin;
            aceEditorBindingHandler.install();
        }
        statusDebugRoutes.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#statusDebugRoutesEditor");
            if (editorElement.length > 0) {
                this.editor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#statusDebugRoutesEditor").on('DynamicHeightSet', function () { return _this.editor.resize(); });
        };
        statusDebugRoutes.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#statusDebugRoutesEditor").off('DynamicHeightSet');
        };
        statusDebugRoutes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            if (this.isGlobalAdmin()) {
                this.fetchRoutes();
                this.activeDatabase.subscribe(function () { return _this.fetchRoutes(); });
            }
        };
        statusDebugRoutes.prototype.fetchRoutes = function () {
            var _this = this;
            return new getStatusDebugRoutesCommand()
                .execute()
                .done(function (results) {
                return _this.data(JSON.stringify(results, null, 4));
            });
        };
        return statusDebugRoutes;
    })(viewModelBase);
    return statusDebugRoutes;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugSqlReplication", ["require", "exports", "commands/database/debug/getStatusDebugSqlReplicationCommand", "viewmodels/viewModelBase", "common/sqlReplicationStats"], function (require, exports, getStatusDebugSqlReplicationCommand, viewModelBase, sqlReplicationStats) {
    var statusDebugSqlReplication = (function (_super) {
        __extends(statusDebugSqlReplication, _super);
        function statusDebugSqlReplication() {
            _super.call(this);
            this.data = ko.observable();
        }
        statusDebugSqlReplication.prototype.activate = function (args) {
            var _this = this;
            var widthUnit = 8;
            this.columnWidths = [
                ko.observable(widthUnit),
                ko.observable(12),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(widthUnit),
                ko.observable(10),
                ko.observable(6),
                ko.observable(18),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(4 * 6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6),
                ko.observable(6)
            ];
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchSqlReplicationStats(); });
            this.registerColumnResizing();
            return this.fetchSqlReplicationStats();
        };
        statusDebugSqlReplication.prototype.deactivate = function () {
            this.unregisterColumnResizing();
        };
        statusDebugSqlReplication.prototype.fetchSqlReplicationStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugSqlReplicationCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.data(results.map(function (x) { return new sqlReplicationStats(x); }));
                });
            }
            return null;
        };
        statusDebugSqlReplication.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var elemFontSize = parseInt(window.getComputedStyle(e.toElement, null).fontSize, 10);
                    var targetColumnSize = startingWidth + (e.pageX - startX) / elemFontSize;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        statusDebugSqlReplication.prototype.unregisterColumnResizing = function () {
            $(document).off("mousedown.logTableColumnResize");
            $(document).off("mouseup.logTableColumnResize");
            $(document).off("mousemove.logTableColumnResize");
        };
        return statusDebugSqlReplication;
    })(viewModelBase);
    return statusDebugSqlReplication;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugTasks", ["require", "exports", "commands/getStatusDebugTasksSummaryCommand", "viewmodels/viewModelBase", "common/appUrl"], function (require, exports, getStatusDebugTasksSummaryCommand, viewModelBase, appUrl) {
    var statusDebugTasks = (function (_super) {
        __extends(statusDebugTasks, _super);
        function statusDebugTasks() {
            var _this = this;
            _super.call(this);
            this.data = ko.observable();
            this.detailsUrl = ko.computed(function () { return appUrl.forResourceQuery(_this.activeDatabase()) + "/debug/tasks"; });
        }
        statusDebugTasks.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
            this.activeDatabase.subscribe(function () { return _this.fetchTasks(); });
            return this.fetchTasks();
        };
        statusDebugTasks.prototype.fetchTasks = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getStatusDebugTasksSummaryCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        return statusDebugTasks;
    })(viewModelBase);
    return statusDebugTasks;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/debug/statusDebugWebSocket", ["require", "exports", "common/appUrl", "viewmodels/viewModelBase", "commands/auth/getSingleAuthTokenCommand"], function (require, exports, appUrl, viewModelBase, getSingleAuthTokenCommand) {
    var statusDebugWebSocket = (function (_super) {
        __extends(statusDebugWebSocket, _super);
        function statusDebugWebSocket() {
            _super.apply(this, arguments);
            this.results = ko.observable("");
        }
        statusDebugWebSocket.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('JHZ574');
        };
        statusDebugWebSocket.prototype.test = function () {
            var _this = this;
            if ("WebSocket" in window) {
                var getTokenTask = new getSingleAuthTokenCommand(this.activeDatabase()).execute();
                getTokenTask
                    .done(function (tokenObject) {
                    var token = tokenObject.Token;
                    var connectionString = 'singleUseAuthToken=' + token + '&id=test&coolDownWithDataLoss=1000&isMultyTenantTransport=false';
                    _this.connectWebSocket(connectionString);
                })
                    .fail(function (e) {
                    if (e.status == 0) {
                        _this.appendLog("Connection has closed (during getToken)");
                    }
                    else {
                        _this.appendLog(e.responseJSON.Error);
                    }
                });
            }
            else {
                this.appendLog("Looks like your browser doesn't support web sockets");
            }
        };
        statusDebugWebSocket.prototype.connectWebSocket = function (connectionString) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === 'https:' ? 'wss://' : 'ws://';
            var url = wsProtocol + window.location.host + appUrl.forResourceQuery(this.activeDatabase()) + '/websocket/validate?' + connectionString;
            this.appendLog("Connecting to web socket using url: " + url);
            var webSocket = new WebSocket(url);
            webSocket.onerror = function (e) {
                if (connectionOpened == false) {
                    _this.appendLog("Server doesn't support web sockets protocol");
                }
                else {
                    _this.appendLog("WebSocket Error" + JSON.stringify(e));
                }
            };
            webSocket.onmessage = function (e) {
                var data = JSON.parse(e.data);
                _this.appendLog("Got message from websocket. Status Code = " + data.StatusCode + ", message = " + data.StatusMessage);
                setTimeout(function () { return webSocket.close(1000, "CLOSE_NORMAL"); }, 50);
            };
            webSocket.onclose = function (e) {
                if (e.wasClean == false) {
                    _this.appendLog("WebSocket disconnected in unclean way");
                }
                else {
                    _this.appendLog("Closed WebSocket connection");
                }
            };
            webSocket.onopen = function () {
                _this.appendLog("Connected to WebSocket");
                connectionOpened = true;
            };
        };
        statusDebugWebSocket.prototype.appendLog = function (msg) {
            this.results(this.results() + msg + "\r\n");
        };
        return statusDebugWebSocket;
    })(viewModelBase);
    return statusDebugWebSocket;
});
/// <reference path="../../../../../Scripts/typings/d3/nvd3.d.ts" />
/// <reference path="../../../../../Scripts/typings/d3/d3.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/indexBatchSize", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getIndexingBatchStatsCommand", 'd3/d3', 'nvd3'], function (require, exports, viewModelBase, getIndexingBatchStatsCommand, d3, nv) {
    var metricsIndexBatchSize = (function (_super) {
        __extends(metricsIndexBatchSize, _super);
        function metricsIndexBatchSize() {
            _super.apply(this, arguments);
            this.currentStats = ko.observable(null);
            this.indexBatchSizeQueryUrl = ko.observable("");
            this.batchSizeChart = null;
            this.batchSizeChartData = [
                {
                    key: 'Batch size',
                    values: []
                }
            ];
        }
        metricsIndexBatchSize.prototype.compositionComplete = function () {
            this.modelPolling();
        };
        metricsIndexBatchSize.prototype.detached = function () {
            _super.prototype.detached.call(this);
            window.onresize = null; // FIX nvd3 event attached globally
        };
        metricsIndexBatchSize.prototype.modelPolling = function () {
            var _this = this;
            return this.fetchStats().then(function () {
                _this.appendData();
                _this.updateGraph();
            });
        };
        metricsIndexBatchSize.prototype.appendData = function () {
            var batchInfos = this.currentStats();
            var values = this.batchSizeChartData[0].values;
            for (var i = 0; i < batchInfos.length; i++) {
                var item = {
                    x: new Date(batchInfos[i].StartedAt),
                    y: batchInfos[i].TotalDocumentSize,
                    size: batchInfos[i].TotalDocumentSize
                };
                var match = values.first(function (e) { return e.x.getTime() === item.x.getTime() && e.y === item.y; });
                if (!match) {
                    values.push(item);
                }
            }
        };
        metricsIndexBatchSize.prototype.updateGraph = function () {
            var _this = this;
            if (this.batchSizeChart === null) {
                nv.addGraph(function () {
                    var chart = nv.models.scatterChart()
                        .margin({ left: 130 })
                        .transitionDuration(350)
                        .showLegend(true)
                        .showYAxis(true)
                        .showXAxis(true)
                        .forceY([0]);
                    ;
                    chart.xAxis
                        .axisLabel('Time')
                        .tickFormat(function (d) { return d3.time.format('%H:%M:%S')(new Date(d)); });
                    chart.yAxis
                        .axisLabel('batch size [bytes]')
                        .tickFormat(d3.format(',f'));
                    nv.utils.windowResize(function () { chart.update(); });
                    return chart;
                }, function (chart) {
                    _this.batchSizeChart = chart;
                    d3.select('#actualIndexingBatchSizeContainer svg')
                        .datum(_this.batchSizeChartData)
                        .call(_this.batchSizeChart);
                });
            }
            else {
                // just update data
                d3.select('#actualIndexingBatchSizeContainer svg')
                    .datum(this.batchSizeChartData)
                    .call(this.batchSizeChart);
            }
        };
        metricsIndexBatchSize.prototype.fetchStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var command = new getIndexingBatchStatsCommand(db);
                this.indexBatchSizeQueryUrl(command.getQueryUrl());
                return command
                    .execute()
                    .done(function (s) { return _this.currentStats(s); });
            }
            return null;
        };
        return metricsIndexBatchSize;
    })(viewModelBase);
    return metricsIndexBatchSize;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/indexing", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/database/status/status"], function (require, exports, viewModelBase, appUrl, status) {
    var requests = (function (_super) {
        __extends(requests, _super);
        function requests() {
            var _this = this;
            _super.call(this);
            this.router = status.statusRouter.createChildRouter()
                .map([
                { route: 'databases/status/indexing', moduleId: 'viewmodels/database/status/indexing/indexPerformance', title: 'Indexing performance', tooltip: "Shows details about indexing peformance", nav: true, hash: appUrl.forCurrentDatabase().indexPerformance },
                { route: 'databases/status/indexing/stats', moduleId: 'viewmodels/database/status/indexing/indexStats', title: 'Index stats', tooltip: "Show details about indexing in/out counts", nav: true, hash: appUrl.forCurrentDatabase().indexStats },
                { route: 'databases/status/indexing/batchSize', moduleId: 'viewmodels/database/status/indexing/indexBatchSize', title: 'Index batch size', tooltip: "Index batch sizes", nav: true, hash: appUrl.forCurrentDatabase().indexBatchSize },
                { route: 'databases/status/indexing/prefetches', moduleId: 'viewmodels/database/status/indexing/indexPrefetches', title: 'Prefetches', tooltip: "Prefetches", nav: true, hash: appUrl.forCurrentDatabase().indexPrefetches },
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return requests;
    })(viewModelBase);
    return requests;
});
/// <reference path="../../../../../Scripts/typings/bootstrap.multiselect/bootstrap.multiselect.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/indexPerformance", ["require", "exports", "viewmodels/viewModelBase", "common/changesContext", "commands/database/debug/getIndexingBatchStatsCommand", "commands/database/debug/getReducingBatchStatsCommand", "commands/resources/getDatabaseStatsCommand", "commands/database/debug/getFilteredOutIndexStatsCommand", 'd3/d3', 'nvd3', "durandal/app", "viewmodels/database/status/indexing/perfDialogs/mapBatchInfo", "viewmodels/database/status/indexing/perfDialogs/reduceBatchInfo", "viewmodels/database/status/indexing/perfDialogs/mapStatsInfo", "viewmodels/database/status/indexing/perfDialogs/reduceStatsInfo", "viewmodels/database/status/indexing/perfDialogs/prefetchInfo", "viewmodels/database/status/indexing/perfDialogs/filteredOutIndexInfo", "common/awesomeMultiselect"], function (require, exports, viewModelBase, changesContext, getIndexingBatchStatsCommand, getReducingBatchStatsCommand, getDatabaseStatsCommand, getFilteredOutIndexStatsCommand, d3, nv, app, mapBatchInfo, reduceBatchInfo, mapStatsInfo, reduceStatsInfo, prefetchInfo, filteredOutIndexInfo, awesomeMultiselect) {
    var dateRange = (function () {
        function dateRange(start, end) {
            this.start = start;
            this.end = end;
        }
        return dateRange;
    })();
    var timeGap = (function () {
        function timeGap(position, timespan) {
            this.position = position;
            this.timespan = timespan;
        }
        return timeGap;
    })();
    var gapFinder = (function () {
        function gapFinder(dateRanges, pixelsPerSecond, minGapTime) {
            this.dateRanges = dateRanges;
            this.pixelsPerSecond = pixelsPerSecond;
            this.minGapTime = minGapTime;
            this.paddingBetweenGaps = 30;
            this.domain = [];
            this.range = [];
            this.gapsPositions = [];
            this.totalWidth = 0;
            this.computeGaps(minGapTime);
            if (this.gapsPositions.length > 0) {
                this.totalWidth -= this.paddingBetweenGaps;
            }
        }
        gapFinder.prototype.constructScale = function () {
            if (this.domain.length == 0) {
                return d3.time.scale();
            }
            else {
                return d3.time.scale()
                    .range(this.range)
                    .domain(this.domain);
            }
        };
        gapFinder.prototype.pushRegion = function (region) {
            this.domain.push(region.start);
            this.domain.push(region.end);
            if (this.totalWidth > 0) {
                var lastEnd = this.domain[this.domain.length - 3].getTime();
                var gap = new timeGap(this.totalWidth - this.paddingBetweenGaps / 2, region.start.getTime() - lastEnd);
                this.gapsPositions.push(gap);
            }
            this.range.push(this.totalWidth);
            var regionWidth = (region.end.getTime() - region.start.getTime()) / 1000 * this.pixelsPerSecond;
            this.range.push(this.totalWidth + regionWidth);
            this.totalWidth += regionWidth + this.paddingBetweenGaps;
        };
        gapFinder.prototype.computeGaps = function (minGapTime) {
            if (this.dateRanges.length > 0) {
                var s = this.dateRanges[0].start;
                var e = this.dateRanges[0].end;
                this.minTime = s;
                this.maxTime = e;
                for (var i = 1; i < this.dateRanges.length; i++) {
                    var newRange = this.dateRanges[i];
                    if (this.minTime > newRange.start) {
                        this.minTime = newRange.start;
                    }
                    if (this.maxTime < newRange.end) {
                        this.maxTime = newRange.end;
                    }
                    if (newRange.start.getTime() > e.getTime() + minGapTime) {
                        this.pushRegion(new dateRange(s, e));
                        s = newRange.start;
                    }
                    if (newRange.end.getTime() > e.getTime()) {
                        e = newRange.end;
                    }
                }
                this.pushRegion(new dateRange(s, e));
            }
        };
        return gapFinder;
    })();
    var metrics = (function (_super) {
        __extends(metrics, _super);
        function metrics() {
            var _this = this;
            _super.call(this);
            this.yBarHeight = 40;
            this.yBarMargin = 10;
            this.mapGroupOffset = 0;
            this.reduceGroupOffset = 0;
            this.mapJsonData = [];
            this.rawMapJsonData = [];
            this.reduceJsonData = [];
            this.rawReduceJsonData = [];
            this.prefetchJsonData = [];
            this.rawPrefetchJsonData = [];
            this.filteredOutIndexesJsonData = [];
            this.rawFilteredOutIndexesJsonData = [];
            this.mapAllIndexNames = ko.observableArray();
            this.reduceAllIndexNames = ko.observableArray();
            this.allIndexNames = ko.computed(function () { return d3.set(_this.mapAllIndexNames().concat(_this.reduceAllIndexNames())).values(); });
            this.selectedIndexNames = ko.observableArray();
            this.selectedMapIndexNames = ko.computed(function () { return _this.selectedIndexNames().filter(function (x) { return _this.mapAllIndexNames().contains(x); }); });
            this.selectedReduceIndexes = ko.computed(function () { return _this.selectedIndexNames().filter(function (x) { return _this.reduceAllIndexNames().contains(x); }); });
            this.color = d3.scale.ordinal().range(metrics.batchesColors);
            this.margin = { top: 40, right: 20, bottom: 40, left: 200, between: 10 };
            this.prefetchingLineHeight = this.yBarHeight + this.yBarMargin * 2 + this.margin.between;
            this.pixelsPerSecond = ko.observable(100);
            this.isoFormat = d3.time.format.iso;
            this.xTickFormat = d3.time.format("%H:%M:%S");
            this.lastIndexingId = 0;
            this.lastReducingId = 0;
            this.refreshGraphObservable = ko.observable();
            this.zoomingIn = false;
            this.zoomingOut = false;
            this.pixelsPerSecond.throttle(100).subscribe(function (value) {
                nv.tooltip.cleanup();
                _this.redrawGraph();
            });
        }
        metrics.prototype.fetchMapJsonData = function () {
            return new getIndexingBatchStatsCommand(this.activeDatabase(), this.lastIndexingId).execute();
        };
        metrics.prototype.fetchReduceJsonData = function () {
            return new getReducingBatchStatsCommand(this.activeDatabase(), this.lastReducingId).execute();
        };
        metrics.prototype.fetchPrefetcherJsonData = function () {
            var promise = $.Deferred();
            new getDatabaseStatsCommand(this.activeDatabase())
                .execute()
                .done(function (result) { return promise.resolve(result.Prefetches); });
            return promise;
        };
        metrics.prototype.fetchFilteredOutJsonData = function () {
            return new getFilteredOutIndexStatsCommand(this.activeDatabase())
                .execute();
        };
        metrics.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            $(document).bind("fullscreenchange", function () {
                if ($(document).fullScreen()) {
                    $("#fullScreenButton i").removeClass("fa-expand").addClass("fa-compress");
                }
                else {
                    $("#fullScreenButton i").removeClass("fa-compress").addClass("fa-expand");
                }
            });
        };
        metrics.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.createKeyboardShortcut("esc", nv.tooltip.cleanup, "body");
            this.updateHelpLink('QCVU81');
            $("#metricsContainer").resize().on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            $("#metricsContainer").scroll(function () { return _this.graphScrolled(); });
            this.refreshSubscription = this.refreshGraphObservable.throttle(5000).subscribe(function (e) { return _this.refresh(); });
            awesomeMultiselect.build($("#visibleIndexesSelector"));
            this.svg = d3.select("#indexPerformanceGraph");
        };
        metrics.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            this.selectedIndexNames.subscribe(function (v) {
                _this.filterJsonData();
                nv.tooltip.cleanup();
                _this.redrawGraph();
            });
            this.refresh();
        };
        metrics.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.processIndexEvent(e); })];
        };
        metrics.prototype.processIndexEvent = function (e) {
            if (e.Type === "MapCompleted" || e.Type === "ReduceCompleted") {
                this.refreshGraphObservable(new Date().getTime());
            }
        };
        metrics.prototype.filterJsonData = function () {
            var _this = this;
            this.mapJsonData = [];
            this.reduceJsonData = [];
            this.prefetchJsonData = [];
            this.filteredOutIndexesJsonData = [];
            var selectedIndexes = this.selectedIndexNames();
            this.rawMapJsonData.forEach(function (rawData) {
                var filteredStats = rawData.PerfStats.filter(function (p) { return selectedIndexes.contains(p.indexName); });
                if (filteredStats.length > 0) {
                    var rawCopy = $.extend(false, {}, rawData);
                    rawCopy.PerfStats = filteredStats;
                    _this.mapJsonData.push(rawCopy);
                }
            });
            this.rawReduceJsonData.forEach(function (rawData) {
                var filteredStats = rawData.PerfStats.filter(function (p) { return selectedIndexes.contains(p.indexName); });
                if (filteredStats.length > 0) {
                    var rawCopy = $.extend(false, {}, rawData);
                    rawCopy.PerfStats = filteredStats;
                    _this.reduceJsonData.push(rawCopy);
                }
            });
            this.rawPrefetchJsonData.forEach(function (rawData) {
                _this.prefetchJsonData.push(rawData);
            });
            this.rawFilteredOutIndexesJsonData.forEach(function (rawData) {
                if (selectedIndexes.contains(rawData.IndexName)) {
                    _this.filteredOutIndexesJsonData.push(rawData);
                }
            });
        };
        metrics.prototype.showHideNoData = function (show) {
            if (show) {
                var metricsContainer = $("#metricsContainer");
                this.svg.append('text')
                    .attr('class', 'no_data')
                    .text('No data available')
                    .attr('fill', 'black')
                    .attr('text-anchor', 'middle')
                    .attr('x', metricsContainer.width() / 2)
                    .attr('y', 50);
                this.svg.select('.controlls').style('display', 'none');
            }
            else {
                this.svg.select('.no_data').remove();
                this.svg.select('.controlls').style('display', null);
            }
        };
        metrics.prototype.refresh = function () {
            var _this = this;
            var mapTask = this.fetchMapJsonData();
            var reduceTask = this.fetchReduceJsonData();
            var prefetchTask = this.fetchPrefetcherJsonData();
            var filterTask = this.fetchFilteredOutJsonData();
            $.when(mapTask, reduceTask, prefetchTask, filterTask)
                .done(function (mapResult, reduceResult, prefetches, filteredOutIndexes) {
                var oldAllIndexes = _this.allIndexNames();
                if (mapResult.length > 0) {
                    _this.lastIndexingId = d3.max(mapResult, function (x) { return x.Id; });
                }
                if (reduceResult.length > 0) {
                    _this.lastReducingId = d3.max(reduceResult, function (x) { return x.Id; });
                }
                if (oldAllIndexes.length === 0) {
                    if (mapResult.length > 0) {
                        _this.edgeDate = d3.min(mapResult, function (r) { return r.StartedAtDate; });
                    }
                    else if (reduceResult.length > 0) {
                        _this.edgeDate = d3.min(reduceResult, function (r) { return r.StartedAtDate; });
                    }
                }
                mapResult = mapResult.filter(function (r) { return r.StartedAtDate >= _this.edgeDate; });
                reduceResult = reduceResult.filter(function (r) { return r.StartedAtDate >= _this.edgeDate; });
                filteredOutIndexes = filteredOutIndexes.filter(function (r) { return r.TimestampParsed >= _this.edgeDate; });
                _this.rawFilteredOutIndexesJsonData = _this.mergeFilteredOutIndexesJsonData(_this.rawFilteredOutIndexesJsonData, filteredOutIndexes);
                _this.rawPrefetchJsonData = _this.mergePrefetchJsonData(_this.rawPrefetchJsonData, prefetches);
                _this.rawMapJsonData = _this.mergeMapJsonData(_this.rawMapJsonData, mapResult);
                var mapIndexes = _this.findIndexNames(_this.rawMapJsonData);
                var filteredOutResourcesIndexNames = _this.rawFilteredOutIndexesJsonData.map(function (x) { return x.IndexName; });
                // merge filtered out resources with map index names as we display them in map section
                mapIndexes = d3.set(mapIndexes.concat(filteredOutResourcesIndexNames)).values();
                var oldMapAllIndexes = _this.mapAllIndexNames();
                var newMapIndexes = mapIndexes.filter(function (i) { return !oldMapAllIndexes.contains(i); });
                _this.mapAllIndexNames.pushAll(newMapIndexes);
                _this.rawReduceJsonData = _this.mergeReduceJsonData(_this.rawReduceJsonData, reduceResult);
                var reduceIndexes = _this.findIndexNames(_this.rawReduceJsonData);
                var oldReduceAllIndexes = _this.reduceAllIndexNames();
                var newReduceIndexes = reduceIndexes.filter(function (i) { return !oldReduceAllIndexes.contains(i); });
                _this.reduceAllIndexNames.pushAll(newReduceIndexes);
                var newIndexes = d3.set(mapIndexes.concat(reduceIndexes)).values().filter(function (i) { return !oldAllIndexes.contains(i); });
                // this will filterJsonData and redrawGraph
                _this.selectedIndexNames(_this.selectedIndexNames().concat(newIndexes));
                // refresh multiselect widget:
                awesomeMultiselect.rebuild($("#visibleIndexesSelector"));
            });
        };
        metrics.prototype.createFilteredOutCacheKey = function (input) {
            return input.Timestamp + "$" + input.IndexName;
        };
        metrics.prototype.mergeFilteredOutIndexesJsonData = function (currentData, incomingData) {
            var _this = this;
            // create lookup map to avoid O(n^2)
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(_this.createFilteredOutCacheKey(d), i);
            });
            incomingData.forEach(function (d) {
                var cacheKey = _this.createFilteredOutCacheKey(d);
                if (dateLookup.has(cacheKey)) {
                    var index = dateLookup.get(cacheKey);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        metrics.prototype.mergePrefetchJsonData = function (currentData, incomingData) {
            var _this = this;
            // create lookup map to avoid O(n^2)
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.Timestamp, i);
            });
            incomingData.forEach(function (d) {
                // we have to filter out future batches that are not completed
                if (!d.Duration) {
                    return;
                }
                _this.computePrefetchCache(d);
                if (dateLookup.has(d.Timestamp)) {
                    var index = dateLookup.get(d.Timestamp);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        metrics.prototype.mergeMapJsonData = function (currentData, incomingData) {
            // create lookup map to avoid O(n^2) 
            var self = this;
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.StartedAt, i);
            });
            incomingData.forEach(function (d) {
                d.PerfStats.forEach(self.computeMapCache.bind(self));
                if (dateLookup.has(d.StartedAt)) {
                    var index = dateLookup.get(d.StartedAt);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        metrics.prototype.mergeReduceJsonData = function (currentData, incomingData) {
            // create lookup map to avoid O(n^2) 
            var self = this;
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.StartedAt, i);
            });
            incomingData.forEach(function (d) {
                d.PerfStats.forEach(function (s) { return s.stats.LevelStats.forEach(self.computeReduceCache.bind(self)); });
                if (dateLookup.has(d.StartedAt)) {
                    var index = dateLookup.get(d.StartedAt);
                    currentData[index] = d;
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        metrics.prototype.computeMapCache = function (input) {
            var currentOffset = 0;
            var self = this;
            var timings = input.stats.Operations.map(function (o) { return o.DurationMs; });
            for (var i = 0; i < timings.length; i++) {
                var currentWidth = Math.max(timings[i], 0);
                var op = input.stats.Operations[i];
                op.CacheWidth = currentWidth;
                op.CacheCumulativeSum = currentOffset;
                op.CacheIsSingleThread = "Name" in op;
                if (!op.CacheIsSingleThread) {
                    var mtGroup = op;
                    self.computeParallelOpsCache(mtGroup);
                }
                currentOffset += currentWidth;
            }
        };
        metrics.prototype.computeReduceCache = function (input) {
            var currentOffset = 0;
            var self = this;
            var timings = input.Operations.map(function (o) { return o.DurationMs; });
            for (var i = 0; i < timings.length; i++) {
                var currentWidth = Math.max(timings[i], 0);
                var op = input.Operations[i];
                op.CacheWidth = currentWidth;
                op.CacheCumulativeSum = currentOffset;
                op.CacheIsSingleThread = "Name" in op;
                if (!op.CacheIsSingleThread) {
                    var mtGroup = op;
                    self.computeParallelOpsCache(mtGroup);
                }
                currentOffset += currentWidth;
            }
        };
        metrics.prototype.computePrefetchCache = function (input) {
            var inputAny = input;
            inputAny.CacheTimestamp = this.isoFormat.parse(input.Timestamp);
            inputAny.DurationMs = this.parseTimespan(input.Duration);
        };
        metrics.prototype.computeParallelOpsCache = function (mtGroup) {
            for (var thread = 0; thread < mtGroup.BatchedOperations.length; thread++) {
                var perThreadInfo = mtGroup.BatchedOperations[thread];
                var currentPerThreadOffset = perThreadInfo.StartDelay;
                for (var j = 0; j < perThreadInfo.Operations.length; j++) {
                    var innerOp = perThreadInfo.Operations[j];
                    var w = Math.max(innerOp.DurationMs, 0);
                    innerOp.CacheWidth = w;
                    innerOp.CacheCumulativeSum = currentPerThreadOffset;
                    innerOp.CacheIsSingleThread = true;
                    currentPerThreadOffset += w;
                }
            }
        };
        metrics.prototype.graphScrolled = function () {
            var leftScroll = $("#metricsContainer").scrollLeft();
            this.svg.select('.y.axis.map')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('.y.axis.reduce')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('.y.axis.prefetch')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('#dataClip rect')
                .attr('x', leftScroll);
            this.svg.select('.left_texts')
                .attr("transform", "translate(" + leftScroll + ",0)");
            nv.tooltip.cleanup();
        };
        metrics.prototype.redrawGraph = function () {
            var self = this;
            this.width = $("#metricsContainer").width();
            // compute dates extents
            var mapDateRange = this.mapJsonData.map(function (j) { return new dateRange(j.StartedAtDate, new Date(j.StartedAtDate.getTime() + j.TotalDurationMs)); });
            var reduceDateRange = this.reduceJsonData.map(function (j) { return new dateRange(j.StartedAtDate, new Date(j.StartedAtDate.getTime() + j.TotalDurationMs)); });
            var filteredOutDateRange = this.filteredOutIndexesJsonData.map(function (j) { return new dateRange(j.TimestampParsed, new Date(j.TimestampParsed.getTime() + 10)); });
            var mergedDateRange = mapDateRange.concat(reduceDateRange).concat(filteredOutDateRange).sort(function (a, b) { return a.start.getTime() - b.start.getTime(); });
            var gapsFinder = new gapFinder(mergedDateRange, self.pixelsPerSecond(), metrics.minGapTime);
            self.xScale = gapsFinder.constructScale();
            var totalWidthWithMargins = gapsFinder.totalWidth
                + self.margin.left
                + self.margin.right
                + 10; // add few more extra pixels
            totalWidthWithMargins = Math.max(totalWidthWithMargins, this.width);
            var totalHeight = self.prefetchingLineHeight
                + self.selectedMapIndexNames().length * (self.yBarHeight + self.yBarMargin * 2)
                + self.margin.between
                + self.selectedReduceIndexes().length * (self.yBarHeight + self.yBarMargin * 2)
                + self.margin.top
                + self.margin.bottom;
            this.svg = d3.select("#indexPerformanceGraph");
            (this.svg.select('defs').node() !== null ? this.svg.transition() : this.svg)
                .attr("width", totalWidthWithMargins)
                .attr("height", totalHeight)
                .style('height', totalHeight + 'px')
                .style('width', totalWidthWithMargins + 'px')
                .attr("viewBox", "0 0 " + totalWidthWithMargins + " " + totalHeight);
            self.mapGroupOffset = self.prefetchingLineHeight;
            self.reduceGroupOffset = self.mapGroupOffset + self.selectedMapIndexNames().length * (self.yBarHeight + self.yBarMargin * 2) + self.margin.between;
            this.svg.selectAll('.map_group')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            this.svg.selectAll('.reduce_group')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            $("#metricsContainer").css('overflow-x', totalWidthWithMargins > this.width ? 'scroll' : 'hidden');
            var defs = this.svg
                .selectAll('defs')
                .data([null])
                .enter()
                .append('defs');
            defs.append('clipPath')
                .attr('id', 'dataClip')
                .append('rect')
                .attr('x', 0)
                .attr('y', -30)
                .attr('width', 1200000)
                .attr('height', 50000);
            defs.append('pattern')
                .attr('id', 'hash')
                .attr('x', 0)
                .attr('y', 0)
                .attr('width', 6)
                .attr('height', 6)
                .attr('patternUnits', 'userSpaceOnUse')
                .append('path')
                .attr('d', 'M 0 0 L 6 6 M 0 6 L 6 0')
                .attr('stroke', '#333')
                .attr('fill', 'transparent')
                .attr('stroke-width', 0.25)
                .attr('stroke-linecap', 'square')
                .attr('stroke-linejoin', 'miter');
            defs.append('pattern')
                .attr('id', 'lines')
                .attr('x', 0)
                .attr('y', 0)
                .attr('width', 5)
                .attr('height', 5)
                .attr('patternUnits', 'userSpaceOnUse')
                .append('path')
                .attr('d', 'M 0 5 L 5 0')
                .attr('stroke', '#333')
                .attr('fill', 'transparent')
                .attr('stroke-width', 0.25)
                .attr('stroke-linecap', 'square')
                .attr('stroke-linejoin', 'miter');
            var svgEnter = this.svg
                .selectAll(".map_group")
                .data([null]).enter();
            svgEnter.append('g')
                .attr('class', 'gaps')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            var mapGroup = svgEnter.append('g')
                .attr('class', 'map_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            mapGroup.append('g')
                .attr('class', 'batches');
            mapGroup.append('g')
                .attr('class', 'ops');
            var reduceGroup = svgEnter.append('g')
                .attr('class', 'reduce_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            reduceGroup.append('g')
                .attr('class', 'batches');
            reduceGroup.append('g')
                .attr('class', 'ops');
            svgEnter.append('g')
                .attr('class', 'prefetch_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            svgEnter.append('g')
                .attr('class', 'filtered_out_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            var controllsEnter = this.svg
                .selectAll(".controlls")
                .data([null]).enter()
                .append("g")
                .attr('class', 'controlls')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            this.updateGroupNames(controllsEnter, gapsFinder.totalWidth);
            this.updateXAxis(controllsEnter, gapsFinder.totalWidth);
            this.updateYMapAxis(controllsEnter);
            this.updateYReduceAxis(controllsEnter);
            this.updateYPrefetchAxis(controllsEnter);
            this.updateMapBatchesRanges();
            this.updateReduceBatchesRanges();
            this.updateMapOperations();
            this.updateReduceOperations();
            this.updatePrefetchOperations();
            this.updateFilteredOutIndexesOperations();
            this.updateGaps(gapsFinder.gapsPositions);
            this.showHideNoData(this.rawMapJsonData.length == 0 && this.rawReduceJsonData.length == 0);
        };
        metrics.prototype.updateGroupNames = function (controllsEnter, totalWidth) {
            var self = this;
            var mapHeight = self.selectedMapIndexNames().length * (self.yBarHeight + self.yBarMargin * 2);
            var reduceHeight = self.selectedReduceIndexes().length * (self.yBarHeight + self.yBarMargin * 2);
            self.svg.select('.map_text')
                .transition()
                .attr('x', -self.mapGroupOffset - mapHeight / 2)
                .style('opacity', mapHeight > 0 ? 1 : 0);
            self.svg.select('.reduce_text')
                .transition()
                .attr('x', -self.reduceGroupOffset - reduceHeight / 2)
                .style('opacity', reduceHeight > 0 ? 1 : 0);
            var textsEnter = controllsEnter.append('g')
                .attr('class', 'left_texts');
            textsEnter.append('text')
                .attr('class', 'map_text')
                .attr("transform", "rotate(-90)")
                .attr("dy", ".71em")
                .attr('x', -self.mapGroupOffset - mapHeight / 2)
                .attr('y', -self.margin.left)
                .style("text-anchor", "middle")
                .text('Map')
                .style('opacity', mapHeight > 0 ? 1 : 0);
            textsEnter.append('text')
                .attr('class', 'reduce_text')
                .attr("transform", "rotate(-90)")
                .attr("dy", ".71em")
                .attr('x', -self.reduceGroupOffset - reduceHeight / 2)
                .attr('y', -self.margin.left)
                .style("text-anchor", "middle")
                .text('Reduce')
                .style('opacity', reduceHeight > 0 ? 1 : 0);
            textsEnter.append('text')
                .attr('class', 'prefetch_text')
                .attr("transform", "rotate(-90)")
                .attr("dy", ".71em")
                .attr('x', -self.margin.top)
                .attr('y', -self.margin.left)
                .style("text-anchor", "middle")
                .text('Prefetch')
                .style('opacity', self.prefetchJsonData.length > 0 ? 1 : 0);
        };
        metrics.prototype.updateXAxis = function (controllsEnter, totalWidth) {
            var self = this;
            controllsEnter.append("g")
                .attr("class", "x axis")
                .attr('clip-path', "url(#dataClip)");
            var t = d3.scale.linear()
                .domain([0, totalWidth])
                .ticks(Math.ceil(totalWidth / 100)).map(function (y) { return self.xScale.invert(y); });
            self.xAxis = d3.svg.axis()
                .scale(self.xScale)
                .orient("top")
                .tickValues(t)
                .tickSize(10)
                .tickFormat(self.xTickFormat);
            this.svg.select(".x.axis")
                .attr("transform", "translate(0,0)")
                .transition()
                .call(self.xAxis);
        };
        metrics.prototype.updateYMapAxis = function (controllsEnter) {
            var self = this;
            controllsEnter.append('g')
                .attr('class', 'y axis map')
                .attr("transform", "translate(0,0)");
            var indexCount = self.selectedMapIndexNames().length;
            self.yMapScale = d3.scale.ordinal()
                .domain(self.selectedMapIndexNames())
                .rangeBands([self.mapGroupOffset, self.mapGroupOffset + indexCount * (self.yBarHeight + self.yBarMargin * 2)]);
            self.yMapAxis = d3.svg.axis()
                .scale(self.yMapScale)
                .orient("left");
            self.svg.select(".y.axis.map")
                .transition()
                .call(self.yMapAxis);
        };
        metrics.prototype.updateYReduceAxis = function (controllsEnter) {
            var self = this;
            controllsEnter.append('g')
                .attr('class', 'y axis reduce')
                .attr("transform", "translate(0,0)");
            var indexCount = self.selectedReduceIndexes().length;
            self.yReduceScale = d3.scale.ordinal()
                .domain(self.selectedReduceIndexes())
                .rangeBands([self.reduceGroupOffset, self.reduceGroupOffset + indexCount * (self.yBarHeight + self.yBarMargin * 2)]);
            self.yReduceAxis = d3.svg.axis()
                .scale(self.yReduceScale)
                .orient("left");
            self.svg.select(".y.axis.reduce")
                .transition()
                .call(self.yReduceAxis);
        };
        metrics.prototype.updateYPrefetchAxis = function (controllsEnter) {
            var self = this;
            controllsEnter.append('g')
                .attr('class', 'y axis prefetch')
                .attr("transform", "translate(0,0)");
            self.yPrefetchScale = d3.scale.ordinal()
                .domain(["Prefetching "])
                .rangeBands([0, (self.yBarHeight + self.yBarMargin * 2)]);
            self.yPrefetchAxis = d3.svg.axis()
                .scale(self.yPrefetchScale)
                .orient("left");
            self.svg.select(".y.axis.prefetch")
                .transition()
                .call(self.yPrefetchAxis);
        };
        // uses optimalization which assumes that both start and start + size falls into continuous region
        metrics.prototype.xScaleExtent = function (size) {
            var self = this;
            return size / 1000 * self.pixelsPerSecond();
        };
        metrics.prototype.updateMapBatchesRanges = function () {
            var self = this;
            var batches = self.svg.select(".map_group").select(".batches")
                .selectAll(".batchRange")
                .data(self.mapJsonData, function (d) { return d.StartedAt; });
            batches.exit().remove();
            batches.select('rect')
                .transition()
                .attr('x', function (d) { return self.xScale(d.StartedAtDate); })
                .attr('y', function (d) { return d3.min(d.PerfStats, function (v) { return self.yMapScale(v.indexName); }); })
                .attr('width', function (d) { return self.xScaleExtent(d.TotalDurationMs); })
                .attr('height', function (d) {
                var extent = d3.extent(d.PerfStats, function (v) { return self.yMapScale(v.indexName); });
                return extent[1] - extent[0] + self.yBarHeight + self.yBarMargin * 2;
            });
            batches.enter()
                .append('g')
                .attr('class', 'batchRange')
                .on('click', self.mapBatchInfoClicked.bind(self))
                .append('rect')
                .attr('x', function (d) { return self.xScale(d.StartedAtDate); })
                .attr('y', function (d) { return d3.min(d.PerfStats, function (v) { return self.yMapScale(v.indexName); }); })
                .attr('width', 0)
                .attr('height', function (d) {
                var extent = d3.extent(d.PerfStats, function (v) { return self.yMapScale(v.indexName); });
                return extent[1] - extent[0] + self.yBarHeight + self.yBarMargin * 2;
            })
                .style("fill", function (d) { return self.color(d.StartedAt); })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.TotalDurationMs); });
        };
        metrics.prototype.updateReduceBatchesRanges = function () {
            var self = this;
            var batches = self.svg.select('.reduce_group').select(".batches")
                .selectAll(".batchRange")
                .data(self.reduceJsonData, function (d) { return d.StartedAt; });
            batches.exit().remove();
            batches.select('rect')
                .transition()
                .attr('x', function (d) { return self.xScale(d.StartedAtDate); })
                .attr('y', function (d) { return d3.min(d.PerfStats, function (v) { return self.yReduceScale(v.indexName); }); })
                .attr('width', function (d) { return self.xScaleExtent(d.TotalDurationMs); })
                .attr('height', function (d) {
                var extent = d3.extent(d.PerfStats, function (v) { return self.yReduceScale(v.indexName); });
                return extent[1] - extent[0] + self.yBarHeight + self.yBarMargin * 2;
            });
            batches.enter()
                .append('g')
                .attr('class', 'batchRange')
                .on('click', self.reduceBatchInfoClicked.bind(self))
                .append('rect')
                .attr('x', function (d) { return self.xScale(d.StartedAtDate); })
                .attr('y', function (d) { return d3.min(d.PerfStats, function (v) { return self.yReduceScale(v.indexName); }); })
                .attr('width', 0)
                .attr('height', function (d) {
                var extent = d3.extent(d.PerfStats, function (v) { return self.yReduceScale(v.indexName); });
                return extent[1] - extent[0] + self.yBarHeight + self.yBarMargin * 2;
            })
                .style("fill", function (d) { return self.color(d.StartedAt); })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.TotalDurationMs); });
        };
        metrics.prototype.updateMapOperations = function () {
            var self = this;
            var batches = self.svg.select('.map_group').select(".ops").selectAll(".opGroup")
                .data(self.mapJsonData, function (d) { return d.StartedAt; });
            batches.exit().remove();
            var enteringOpsGroups = batches
                .enter()
                .append('g')
                .attr('class', 'opGroup');
            var op = batches.selectAll('.op')
                .data(function (d) { return d.PerfStats; }, function (d) { return d.indexName; });
            op.exit().remove();
            var opTransition = op.
                transition()
                .attr("transform", function (d) {
                return "translate(" + self.xScale(self.isoFormat.parse(d.stats.Started)) + "," + self.yMapScale(d.indexName) + ")";
            });
            opTransition.select('.main_bar')
                .attr('width', function (d) { return self.xScaleExtent(d.stats.DurationMilliseconds); });
            opTransition.selectAll('.mto_hatch')
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            opTransition.selectAll('.sto_item')
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            opTransition.selectAll('.mto_items')
                .attr("transform", function (d) { return "translate(" + self.xScaleExtent(d.CacheCumulativeSum) + ",0)"; });
            opTransition.selectAll('.mto_item')
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var enteringOps = op.enter()
                .append('g')
                .attr('class', 'op')
                .on('click', self.indexStatClicked.bind(self))
                .attr("transform", function (d) {
                return "translate(" + self.xScale(self.isoFormat.parse(d.stats.Started)) + "," + self.yMapScale(d.indexName) + ")";
            });
            enteringOps.append('rect')
                .attr('class', 'main_bar')
                .attr('x', 0)
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .style('fill', function () {
                var d = d3.select(this.parentNode.parentNode).datum().StartedAt;
                return self.color(d);
            })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.stats.DurationMilliseconds); });
            enteringOps.selectAll('.sto_item')
                .data(function (d) { return d.stats.Operations.filter(function (o) { return o.CacheIsSingleThread; }); })
                .enter()
                .append('rect')
                .attr('class', function (d) { return 'sto_item ' + d.Name; })
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var mtoItems = enteringOps.selectAll('.mto_items')
                .data(function (d) { return d.stats.Operations.filter(function (o) { return !o.CacheIsSingleThread; }); })
                .enter()
                .append('g')
                .attr('class', 'mto_items')
                .attr("transform", function (d) {
                return "translate(" + self.xScaleExtent(d.CacheCumulativeSum) + ",0)";
            });
            mtoItems
                .append('rect')
                .attr('class', 'mto_hatch')
                .attr('x', 0)
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .style('fill', 'url(#lines)')
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var mtoThreadItems = mtoItems.selectAll('.mto_thread_item')
                .data(function (d) { return d.BatchedOperations; })
                .enter()
                .append('g')
                .attr('class', 'mto_thread_item')
                .attr("transform", function (d, i) {
                return "translate(0," + (i * self.yBarHeight / d.Parent.NumberOfThreads + self.yBarMargin) + ")";
            });
            mtoThreadItems.selectAll('.mto_item')
                .data(function (d) { return d.Operations; })
                .enter()
                .append('rect')
                .attr('class', function (d) { return 'mto_item ' + d.Name; })
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('y', 0)
                .attr('width', 0)
                .attr('height', function (d) { return self.yBarHeight / d.ParallelParent.Parent.NumberOfThreads; })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
        };
        metrics.prototype.updateReduceOperations = function () {
            var self = this;
            var batches = self.svg.select('.reduce_group').select(".ops").selectAll(".opGroup")
                .data(self.reduceJsonData, function (d) { return d.StartedAt; });
            batches.exit().remove();
            batches
                .enter()
                .append('g')
                .attr('class', 'opGroup');
            var op = batches.selectAll('.op_g')
                .data(function (d) { return d.PerfStats; }, function (d) { return d.indexName + "/" + d.stats.ReduceType; });
            op.exit().remove();
            var opTransition = op
                .selectAll('.op')
                .transition()
                .attr("transform", function (d) { return "translate(" + self.xScale(self.isoFormat.parse(d.Started)) + "," + self.yReduceScale(d.parent.indexName) + ")"; });
            opTransition.select('.main_bar')
                .attr('width', function (d) { return self.xScaleExtent(d.DurationMs); });
            opTransition.selectAll('.mto_hatch')
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            opTransition.selectAll('.sto_item')
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            opTransition.selectAll('.mto_items')
                .attr("transform", function (d) { return "translate(" + self.xScaleExtent(d.CacheCumulativeSum) + ",0)"; });
            opTransition.selectAll('.mto_item')
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var enteringOps = op.enter()
                .append('g')
                .attr('class', 'op_g')
                .selectAll('op')
                .data(function (d) { return d.stats.LevelStats; })
                .enter()
                .append('g')
                .attr('class', 'op')
                .on('click', self.reduceStatClicked.bind(self))
                .attr("transform", function (d) {
                return "translate(" + self.xScale(self.isoFormat.parse(d.Started)) + "," + self.yReduceScale(d.parent.indexName) + ")";
            });
            enteringOps
                .append('line')
                .attr('class', 'reduce_split_line')
                .attr('x1', 0)
                .attr('x2', 0)
                .attr('y1', 0)
                .attr('y2', self.yBarHeight + self.yBarMargin * 2);
            enteringOps.append('rect')
                .attr('class', 'main_bar')
                .attr('x', 0)
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .style('fill', function (d) { return self.color(d.parent.parent.StartedAt); })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.DurationMs); });
            enteringOps.selectAll('.sto_item')
                .data(function (d) { return d.Operations.filter(function (o) { return o.CacheIsSingleThread; }); })
                .enter()
                .append('rect')
                .attr('class', function (d) { return 'sto_item ' + d.Name; })
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var mtoItems = enteringOps.selectAll('.mto_items')
                .data(function (d) { return d.Operations.filter(function (o) { return !o.CacheIsSingleThread; }); })
                .enter()
                .append('g')
                .attr('class', 'mto_items')
                .attr("transform", function (d) {
                return "translate(" + self.xScaleExtent(d.CacheCumulativeSum) + ",0)";
            });
            mtoItems
                .append('rect')
                .attr('class', 'mto_hatch')
                .attr('x', 0)
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .style('fill', 'url(#lines)')
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
            var mtoThreadItems = mtoItems.selectAll('.mto_thread_item')
                .data(function (d) { return d.BatchedOperations; })
                .enter()
                .append('g')
                .attr('class', 'mto_thread_item')
                .attr("transform", function (d, i) {
                return "translate(0," + (i * self.yBarHeight / d.Parent.NumberOfThreads + self.yBarMargin) + ")";
            });
            mtoThreadItems.selectAll('.mto_item')
                .data(function (d) { return d.Operations; })
                .enter()
                .append('rect')
                .attr('class', function (d) { return 'mto_item ' + d.Name; })
                .attr('x', function (d) { return self.xScaleExtent(d.CacheCumulativeSum); })
                .attr('y', 0)
                .attr('width', 0)
                .attr('height', function (d) { return self.yBarHeight / d.ParallelParent.Parent.NumberOfThreads; })
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.CacheWidth); });
        };
        metrics.prototype.updatePrefetchOperations = function () {
            var self = this;
            var batches = self.svg.select('.prefetch_group')
                .selectAll(".prefetch_op")
                .data(self.prefetchJsonData, function (d) { return d.CacheTimestamp; });
            batches.exit().remove();
            batches
                .select('rect')
                .transition()
                .attr('x', function (d) { return self.xScale(d.CacheTimestamp); })
                .attr('width', function (d) { return self.xScaleExtent(d.DurationMs); });
            var enteringOps = batches
                .enter()
                .append('g')
                .attr('class', 'prefetch_op');
            enteringOps
                .append('rect')
                .attr('class', 'prefetch_box')
                .attr('x', function (d) { return self.xScale(d.CacheTimestamp); })
                .attr('y', self.yBarMargin)
                .attr('width', 0)
                .attr('height', self.yBarHeight)
                .on('click', self.prefetchStatClicked.bind(self))
                .transition()
                .attr('width', function (d) { return self.xScaleExtent(d.DurationMs); });
        };
        metrics.prototype.updateFilteredOutIndexesOperations = function () {
            var _this = this;
            var self = this;
            var batches = self.svg.select('.filtered_out_group')
                .selectAll('.filter_op')
                .data(self.filteredOutIndexesJsonData, function (d) { return _this.createFilteredOutCacheKey(d); });
            batches.exit().remove();
            var radius = 5;
            batches
                .transition()
                .attr('cx', function (d) { return self.xScale(d.TimestampParsed); })
                .attr('cy', function (d) { return self.yMapScale(d.IndexName) + self.yBarHeight / 2 + radius / 2 + self.yBarMargin; });
            batches
                .enter()
                .append('circle')
                .attr('class', 'filter_op')
                .attr('r', 0)
                .attr('cx', function (d) { return self.xScale(d.TimestampParsed); })
                .attr('cy', function (d) { return self.yMapScale(d.IndexName) + self.yBarHeight / 2 + radius / 2 + self.yBarMargin; })
                .style('stroke', 'black')
                .on('click', self.filterOutIndexClicked.bind(self))
                .transition()
                .attr('r', radius);
        };
        metrics.prototype.updateGaps = function (gapsPositions) {
            var self = this;
            var gaps = self.svg.select('.gaps').selectAll('.gap')
                .data(gapsPositions);
            gaps.exit().remove();
            var patternWidth = 20;
            var gapHeight = (self.selectedMapIndexNames().length + self.selectedReduceIndexes().length + 1 /* prefetch */) * (self.yBarHeight + self.yBarMargin * 2) + self.margin.between * 2;
            gaps.select('text')
                .transition()
                .attr("y", function (d) { return d.position; })
                .attr('x', -gapHeight / 2);
            gaps.select('rect')
                .transition()
                .attr('x', function (d) { return d.position - patternWidth / 2; })
                .attr('height', gapHeight);
            var enteringGaps = gaps.enter()
                .append('g')
                .attr('class', 'gap');
            enteringGaps.append('rect')
                .attr('x', function (d) { return d.position - patternWidth / 2; })
                .attr('y', 0)
                .attr('width', patternWidth)
                .attr('height', 0)
                .style('fill', 'url(#hash)')
                .transition()
                .attr('height', gapHeight);
            enteringGaps.append('text')
                .attr("transform", "rotate(-90)")
                .attr("y", function (d) { return d.position; })
                .attr('x', -gapHeight / 2)
                .attr("dy", 5)
                .style("text-anchor", "middle")
                .text(function (d) { return self.timeSpanFormat(d.timespan); });
        };
        metrics.prototype.parseTimespan = function (span) {
            var dotSplit = span.split(".");
            var milis = 0;
            if (dotSplit.length === 2) {
                milis = parseInt(dotSplit[1].substr(0, 3).replace("/^(0+)", ""));
                span = dotSplit[0];
            }
            var tokens = span.split(":").reverse();
            if (tokens.length > 0) {
                milis += parseInt(tokens[0]) * 1000;
            }
            if (tokens.length > 1) {
                milis += parseInt(tokens[1]) * 1000 * 60;
            }
            if (tokens.length > 2) {
                milis += parseInt(tokens[2]) * 1000 * 60 * 60;
            }
            return milis;
        };
        metrics.prototype.timeSpanFormat = function (milis) {
            var t = milis;
            t -= t % 1000;
            t /= 1000;
            var s = t % 60;
            t -= s;
            t /= 60;
            var m = t % 60;
            t -= m;
            t /= 60;
            var h = t;
            var f = d3.format("02d");
            return f(h) + ":" + f(m) + ":" + f(s);
        };
        metrics.prototype.findMaxThreadCountInMap = function (data) {
            var parallelOps = data.stats.Operations.filter(function (x) { return "NumberOfThreads" in x; });
            return d3.max(parallelOps.map(function (p) { return p.NumberOfThreads; }));
        };
        metrics.prototype.findMaxThreadCountInReduce = function (data) {
            var parallelOps = data.Operations.filter(function (x) { return "NumberOfThreads" in x; });
            return d3.max(parallelOps.map(function (p) { return p.NumberOfThreads; }));
        };
        metrics.prototype.showTooltip = function (data, templateName) {
            var self = this;
            var container = document.getElementById("indexingPerformance");
            nv.tooltip.cleanup();
            var html = '<div data-bind="template: { name : \'' + templateName + '\' }"></div>';
            var clickLocation = d3.mouse(container);
            nv.tooltip.show([clickLocation[0], clickLocation[1]], html, 'n', 0, container, "selectable-tooltip");
            var tool = $(".nvtooltip");
            var node = tool[0];
            ko.applyBindings({ data: data, tooltipClose: nv.tooltip.cleanup }, node);
            var drag = d3.behavior.drag()
                .on('drag', function () {
                var o = tool.offset();
                tool.offset({ top: o.top + d3.event.dy, left: o.left + d3.event.dx });
            });
            d3.select('.nvtooltip').call(drag);
        };
        metrics.prototype.filterOutIndexClicked = function (data) {
            app.showDialog(new filteredOutIndexInfo(data));
        };
        metrics.prototype.indexStatClicked = function (data) {
            data.CacheThreadCount = this.findMaxThreadCountInMap(data);
            app.showDialog(new mapStatsInfo(data));
        };
        metrics.prototype.reduceStatClicked = function (data) {
            data.CacheThreadCount = this.findMaxThreadCountInReduce(data);
            app.showDialog(new reduceStatsInfo(data));
        };
        metrics.prototype.mapBatchInfoClicked = function (data) {
            app.showDialog(new mapBatchInfo(data));
        };
        metrics.prototype.reduceBatchInfoClicked = function (data) {
            app.showDialog(new reduceBatchInfo(data));
        };
        metrics.prototype.prefetchStatClicked = function (data) {
            app.showDialog(new prefetchInfo(data));
        };
        metrics.prototype.onWindowHeightChanged = function () {
            nv.tooltip.cleanup();
            this.width = $("#metricsContainer").width();
            $("#metricsContainer").css('overflow-x', $("#metricsContainer svg").width() > this.width ? 'scroll' : 'hidden');
        };
        metrics.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#metricsContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
            if (this.refreshSubscription != null) {
                this.refreshSubscription.dispose();
            }
        };
        metrics.prototype.findIndexNames = function (jsonData) {
            var statsInline = d3.merge(jsonData.map(function (d) { return d.PerfStats; }));
            var byKey = d3
                .nest()
                .key(function (d) { return d.indexName; })
                .sortKeys(d3.ascending)
                .rollup(function (l) { return l.length; })
                .entries(statsInline);
            return byKey.map(function (d) { return d.key; });
        };
        metrics.prototype.startZoomIn = function () {
            this.zoomingIn = true;
            this.zoomIn();
        };
        metrics.prototype.startZoomOut = function () {
            this.zoomingOut = true;
            this.zoomOut();
        };
        metrics.prototype.stopZoomIn = function () {
            this.zoomingIn = false;
        };
        metrics.prototype.stopZoomOut = function () {
            this.zoomingOut = false;
        };
        metrics.prototype.zoomIn = function () {
            var _this = this;
            if (this.zoomingIn) {
                this.pixelsPerSecond(this.pixelsPerSecond() + 20);
                setTimeout(function () { return _this.zoomIn(); }, 100);
            }
        };
        metrics.prototype.zoomOut = function () {
            var _this = this;
            if (this.zoomingOut) {
                if (this.pixelsPerSecond() > 20) {
                    this.pixelsPerSecond(this.pixelsPerSecond() - 20);
                }
                setTimeout(function () { return _this.zoomOut(); }, 100);
            }
        };
        metrics.prototype.toggleFullscreen = function () {
            if ($(document).fullScreen()) {
                $("#indexingPerformance").width('').height('');
            }
            else {
                $("#indexingPerformance").width("100%").height("100%");
            }
            $("#indexingPerformance").toggleFullScreen();
        };
        metrics.reduce_bar_names = ['linq_map_bar', 'linq_reduce_bar',
            'l_delete_bar', 'l_convert_bar', 'l_add_bar', 'l_flush_bar', 'l_recreate_bar',
            'red_get_items_bar', 'red_delete_bar', 'red_schedule_bar', 'red_get_mapped_bar', 'red_remove_bar', 'red_commit_bar'];
        metrics.reduce_bar_colors = [
            '#1f77b4', '#ff7f0e',
            '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2',
            '#c5b0d5', '#c49c94', '#f7b6d2', '#bcbd22', '#dbdb8d', '#9edae5'];
        metrics.minGapTime = 1000 * 10;
        metrics.batchesColors = ['#ececec', '#c2c2c2', '#959595'];
        return metrics;
    })(viewModelBase);
    return metrics;
});
/// <reference path="../../../../../Scripts/typings/d3/nvd3.d.ts" />
/// <reference path="../../../../../Scripts/typings/d3/d3.d.ts" />
/// <reference path="../../../../../Scripts/typings/d3/timelinesChart.d.ts" />
/// <reference path="../../../../../Scripts/typings/d3/timelines.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/indexPrefetches", ["require", "exports", "viewmodels/viewModelBase", "commands/resources/getDatabaseStatsCommand", "moment", 'd3/d3', 'nvd3'], function (require, exports, viewModelBase, getDatabaseStatsCommand, moment, d3, nv) {
    var indexPrefetches = (function (_super) {
        __extends(indexPrefetches, _super);
        function indexPrefetches() {
            _super.apply(this, arguments);
            this.currentStats = ko.observable(null);
            this.prefetchesChart = null;
            this.prefetchesChartData = [{
                    key: 'Prefetches',
                    values: []
                }];
        }
        indexPrefetches.prototype.attached = function () {
            _super.prototype.attached.call(this);
            indexPrefetches.prefetchesAllowZoom = false;
        };
        indexPrefetches.prototype.compositionComplete = function () {
            this.modelPolling();
        };
        indexPrefetches.prototype.canDeactivate = function () {
            $(window).off('resize.indexPrefetches');
            return true;
        };
        indexPrefetches.prototype.modelPolling = function () {
            var _this = this;
            return this.fetchStats().then(function () {
                _this.appendData();
                _this.updateChart();
            });
        };
        indexPrefetches.prototype.appendData = function () {
            var stats = this.currentStats();
            var prefeches = stats.Prefetches;
            var values = this.prefetchesChartData[0].values;
            for (var i = 0; i < prefeches.length; i++) {
                var item = {
                    x: new Date(prefeches[i].Timestamp).getTime(),
                    size: moment.duration(prefeches[i].Duration).asMilliseconds(),
                    y: prefeches[i].Size,
                    payload: prefeches[i]
                };
                var match = values.first(function (e) { return e.x === item.x && e.y === item.y; });
                if (!match) {
                    values.push(item);
                }
            }
        };
        indexPrefetches.prototype.updateChart = function () {
            var _this = this;
            if (!this.prefetchesChart) {
                nv.addGraph(function () {
                    var chart = nv.models.timelinesChart()
                        .showDistX(true)
                        .showDistY(true)
                        .showControls(true)
                        .color(d3.scale.category10().range())
                        .transitionDuration(250);
                    chart.yAxis.showMaxMin(false).axisLabel('size').tickFormat(d3.format(',f'));
                    chart.forceY([0]);
                    chart.y2Axis.showMaxMin(false);
                    chart.xAxis.showMaxMin(false);
                    chart.x2Axis.showMaxMin(false);
                    chart.xAxis.tickFormat(function (_) { return d3.time.format("%H:%M:%S")(new Date(_)); });
                    chart.x2Axis.tickFormat(function (_) { return d3.time.format("%H:%M:%S")(new Date(_)); });
                    chart.tooltipContent(function (key, x, y, data) {
                        var ff = d3.format(",f");
                        return '<h4>' + key + '</h4>'
                            + '<strong>Timestamp:</strong> ' + data.point.payload.Timestamp + '<br />'
                            + '<strong>Duration:</strong> ' + data.point.payload.Duration + '<br />'
                            + '<strong>Size:</strong> ' + ff(data.point.payload.Size) + '<br />'
                            + '<strong>Retries:</strong> ' + ff(data.point.payload.Retries);
                    });
                    chart.dispatch.on('controlsChange', function (e) {
                        indexPrefetches.prefetchesAllowZoom = !!e.disabled;
                    });
                    $(window).on('resize.indexPrefetches', function (e) { return chart(e); });
                    return chart;
                }, function (chart) {
                    _this.prefetchesChart = chart;
                    d3.select('#prefetchesContainer svg')
                        .datum(_this.prefetchesChartData)
                        .call(_this.prefetchesChart);
                });
            }
            else {
                if (!indexPrefetches.prefetchesAllowZoom) {
                    d3.select('#prefetchesContainer svg')
                        .datum(this.prefetchesChartData)
                        .call(this.prefetchesChart);
                }
            }
        };
        indexPrefetches.prototype.fetchStats = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getDatabaseStatsCommand(db)
                    .execute().done(function (s) { return _this.currentStats(s); });
            }
            return null;
        };
        indexPrefetches.prefetchesAllowZoom = false;
        return indexPrefetches;
    })(viewModelBase);
    return indexPrefetches;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/indexStats", ["require", "exports", "viewmodels/viewModelBase", "common/generalUtils", "commands/database/debug/getIndexingPerfStatsCommand", "d3/d3", 'nvd3', "common/changesContext"], function (require, exports, viewModelBase, generalUtils, getIndexingPerfStatsCommand, d3, nv, changesContext) {
    var indexStats = (function (_super) {
        __extends(indexStats, _super);
        function indexStats() {
            _super.apply(this, arguments);
            this.jsonData = [];
            this.rawJsonData = [];
            this.hiddenIndexes = d3.set([]);
            this.indexNames = [];
            this.refreshGraphObservable = ko.observable();
            this.margin = { top: 40, right: 20, bottom: 40, left: 40 };
            this.barWidth = 30;
            this.barPaddingInner = 5;
            this.barPaddingOuter = 10;
            this.legendWidth = 0;
            this.isoFormat = d3.time.format.iso;
            this.xTickFormat = d3.time.format("%H:%M:%S");
            this.color = d3.scale.category20();
        }
        indexStats.prototype.fetchJsonData = function () {
            return new getIndexingPerfStatsCommand(this.activeDatabase()).execute();
        };
        indexStats.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            $("#indexStatsContainer").resize().on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            $("#indexStatsContainer").scroll(function () { return _this.graphScrolled(); });
            this.refreshSubscription = this.refreshGraphObservable.throttle(5000).subscribe(function (e) { return _this.refresh(); });
        };
        indexStats.prototype.compositionComplete = function () {
            this.refresh();
        };
        indexStats.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchAllIndexes(function (e) { return _this.processIndexEvent(e); })];
        };
        indexStats.prototype.processIndexEvent = function (e) {
            if (e.Type == "MapCompleted" || e.Type == "ReduceCompleted") {
                this.refreshGraphObservable(new Date().getTime());
            }
        };
        indexStats.prototype.filterJsonData = function () {
            var _this = this;
            this.jsonData = [];
            this.rawJsonData.forEach(function (v) {
                var filteredStats = v.Stats.filter(function (s) { return !_this.hiddenIndexes.has(s.Index); });
                if (filteredStats.length > 0) {
                    _this.jsonData.push({
                        'Started': v.Started,
                        'Stats': filteredStats
                    });
                }
            });
        };
        indexStats.prototype.refresh = function () {
            var _this = this;
            return this.fetchJsonData().done(function (data) {
                _this.rawJsonData = _this.mergeJsonData(_this.rawJsonData, data);
                _this.indexNames = _this.findIndexNames(_this.rawJsonData);
                _this.filterJsonData();
                _this.redrawGraph();
            });
        };
        indexStats.prototype.mergeJsonData = function (currentData, incomingData) {
            // create lookup map to avoid O(n^2) 
            var dateLookup = d3.map();
            currentData.forEach(function (d, i) {
                dateLookup.set(d.Started, i);
            });
            incomingData.forEach(function (d) {
                if (dateLookup.has(d.Started)) {
                    var index = dateLookup.get(d.Started);
                    var exitingData = currentData[index];
                    var newData = d;
                    // we merge into existing data
                    var indexLookup = d3.map();
                    exitingData.Stats.forEach(function (d, i) {
                        indexLookup.set(d.Index, i);
                    });
                    newData.Stats.forEach(function (s) {
                        if (indexLookup.has(s.Index)) {
                            var index = indexLookup.get(s.Index);
                            var dest = exitingData.Stats[index];
                            dest.DurationMilliseconds = Math.max(dest.DurationMilliseconds, s.DurationMilliseconds);
                            dest.InputCount = Math.max(dest.InputCount, s.InputCount);
                            dest.OutputCount = Math.max(dest.OutputCount, s.OutputCount);
                            dest.ItemsCount = Math.max(dest.ItemsCount, s.ItemsCount);
                        }
                        else {
                            exitingData.Stats.push(s);
                        }
                    });
                }
                else {
                    currentData.push(d);
                }
            });
            return currentData;
        };
        indexStats.prototype.computeBarWidths = function (data) {
            var _this = this;
            var cumulative = 10;
            var result = data.map(function (perfData) {
                var prevValue = cumulative;
                perfData.sectionWidth = perfData.Stats.length * _this.barWidth * 2 + _this.barPaddingInner * 2 + _this.barPaddingOuter;
                cumulative += perfData.sectionWidth;
                return prevValue;
            });
            result.push(cumulative);
            return result;
        };
        indexStats.prototype.graphScrolled = function () {
            var leftScroll = $("#indexStatsContainer").scrollLeft();
            var self = this;
            this.svg.select('.y.axis')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('#dataClip rect')
                .attr('x', leftScroll);
            this.svg.select('.legend_bg_group')
                .attr("transform", "translate(" + leftScroll + ",0)");
            this.svg.select('.controlls')
                .selectAll(".legend")
                .attr("transform", function (d, i) { return "translate(" + leftScroll + "," + i * 20 + ")"; });
            nv.tooltip.cleanup();
        };
        indexStats.prototype.toggleIndexVisible = function (indexName) {
            nv.tooltip.cleanup();
            var alreadyHidden = this.hiddenIndexes.has(indexName);
            if (alreadyHidden) {
                this.hiddenIndexes.remove(indexName);
            }
            else {
                this.hiddenIndexes.add(indexName);
            }
            d3.select('.rect-legend-' + generalUtils.escape(indexName)).classed('legendHidden', !alreadyHidden);
            this.filterJsonData();
            this.redrawGraph();
            // we have to manually trigger on scroll even to fix firefox issue (missing event call)
            this.graphScrolled();
        };
        indexStats.prototype.redrawGraph = function () {
            var self = this;
            this.width = $("#indexStatsContainer").width() - this.margin.left - this.margin.right;
            this.height = $("#indexStatsContainer").height() - this.margin.top - this.margin.bottom - 20; // substract scroll width
            var cumulativeWidths = this.computeBarWidths(this.jsonData);
            this.x0Scale = d3.scale.ordinal().range(cumulativeWidths);
            this.yScale = d3.scale.linear().range([self.height, 0]);
            this.xAxis = d3.svg.axis()
                .scale(self.x0Scale)
                .orient("bottom")
                .tickFormat(function (d) { return ""; })
                .tickPadding(20);
            this.yAxis = d3.svg.axis()
                .scale(self.yScale)
                .orient("left")
                .tickFormat(d3.format(".2s"));
            var totalHeight = self.height + self.margin.top + self.margin.bottom;
            // get higer value from total (visiable and not visible graph width) and viewbox width.
            var totalWidth = Math.max(cumulativeWidths[cumulativeWidths.length - 1], this.width) + this.margin.left + this.margin.right;
            $("#indexStatsContainer").css('overflow-x', cumulativeWidths[cumulativeWidths.length - 1] > this.width ? 'scroll' : 'hidden');
            this.svg = d3.select("#indexStatsGraph")
                .attr("width", totalWidth)
                .attr("height", totalHeight)
                .style({ height: totalHeight + 'px' })
                .style({ width: totalWidth + 'px' })
                .attr("viewBox", "0 0 " + totalWidth + " " + totalHeight);
            this.svg.selectAll('.main_group')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            this.svg
                .selectAll('defs')
                .data([this.jsonData])
                .enter()
                .append('defs')
                .append('clipPath')
                .attr('id', 'dataClip')
                .append('rect')
                .attr('x', 0)
                .attr('y', 0)
                .attr('width', 1200000)
                .attr('height', 50000);
            var svgEnter = this.svg
                .selectAll(".main_group")
                .data([this.jsonData]).enter();
            svgEnter.append('g')
                .attr('class', 'main_group')
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            var controllsEnter = this.svg
                .selectAll(".controlls")
                .data([this.jsonData]).enter()
                .append("g")
                .attr('class', 'controlls')
                .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");
            controllsEnter.append("g")
                .attr("class", "x axis");
            controllsEnter.append('g')
                .attr('class', 'y axis');
            controllsEnter.append('g')
                .attr('class', 'legend_bg_group')
                .append('rect')
                .attr('class', 'legend_bg')
                .attr('x', self.width)
                .attr('y', 0)
                .attr('width', 0)
                .attr('height', 0);
            controllsEnter.select('.y.axis')
                .append("text")
                .attr("transform", "rotate(-90)")
                .attr("y", 6)
                .attr("dy", ".71em")
                .style("text-anchor", "end")
                .text("Doc count");
            this.x0Scale.domain(d3.nest()
                .key(function (d) { return d.Started; })
                .sortKeys(d3.ascending)
                .entries(self.jsonData)
                .map(function (d) { return d.key; }));
            this.yScale.domain([0, d3.max(this.jsonData, function (d) { return d3.max(d.Stats, function (dd) { return d3.max([dd.InputCount, dd.OutputCount]); }); })]);
            this.svg.select(".x.axis")
                .attr('clip-path', "url(#dataClip)")
                .attr("transform", "translate(0," + self.height + ")")
                .transition()
                .call(self.xAxis);
            this.svg.select('.y.axis')
                .transition()
                .call(self.yAxis);
            var frame = this.svg.select('.main_group').selectAll(".frame")
                .data(self.jsonData, function (d) { return d.Started; });
            frame.exit().remove();
            frame
                .transition()
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frame
                .select('.date_tick')
                .transition()
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.height + 36);
            frame
                .select('.input_text')
                .transition()
                .attr("x", function (d) { return d.sectionWidth * 0.25; })
                .attr('y', self.height + 18);
            frame
                .select('.output_text')
                .transition()
                .attr("x", function (d) { return d.sectionWidth * 0.75; })
                .attr('y', self.height + 18);
            frame
                .select('.outputs')
                .transition()
                .attr("transform", function (d) { return "translate(" + (d.sectionWidth / 2) + ",0)"; });
            var frameEnter = frame.enter()
                .append("g")
                .attr("class", "frame")
                .attr("transform", function (d) { return "translate(" + self.x0Scale(d.Started) + ",0)"; });
            frameEnter.append("text")
                .attr('class', 'input_text')
                .attr('text-anchor', 'middle')
                .attr("x", function (d) { return d.sectionWidth * 0.25; })
                .attr("y", self.height + 18)
                .text("In");
            frameEnter.append("text")
                .attr('class', 'output_text')
                .attr('text-anchor', 'middle')
                .attr("x", function (d) { return d.sectionWidth * 0.75; })
                .attr("y", self.height + 18)
                .text("Out");
            frameEnter.append("text")
                .attr('class', 'date_tick')
                .attr('text-anchor', 'middle')
                .attr('x', function (d) { return d.sectionWidth / 2; })
                .attr('y', self.height + 36)
                .text(function (d) { return self.xTickFormat(self.isoFormat.parse(d.Started)); });
            frameEnter.append("g")
                .attr('class', 'inputs');
            frameEnter.append('g')
                .attr('class', 'outputs')
                .attr("transform", function (d) { return "translate(" + (d.sectionWidth / 2) + ",0)"; });
            var inputCounts = frame.select('.inputs').selectAll(".inputCounts")
                .data(function (d) { return d.Stats; }, function (d) { return d.Index; });
            inputCounts.exit().remove();
            inputCounts
                .transition()
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth - self.barPaddingInner + self.barPaddingOuter; })
                .attr("y", function (d) { return self.yScale(d.InputCount); })
                .attr("height", function (d) { return self.height - self.yScale(d.InputCount); })
                .style("fill", function (d) { return self.color(d.Index); });
            inputCounts.enter().append("rect")
                .attr("class", "inputCounts")
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth - self.barPaddingInner + self.barPaddingOuter; })
                .attr("y", function (d) { return self.height; })
                .attr("height", 0)
                .style("fill", function (d) { return self.color(d.Index); })
                .on('click', function (d) {
                nv.tooltip.cleanup();
                var offset = $(this).offset();
                var leftScroll = $("#indexStatsContainer").scrollLeft();
                var containerOffset = $("#indexStatsContainer").offset();
                nv.tooltip.show([offset.left - containerOffset.left + leftScroll + self.barWidth, offset.top - containerOffset.top], self.getTooltip(d), 's', 5, document.getElementById("indexStatsContainer"), "selectable-tooltip");
            })
                .transition()
                .attr("height", function (d) { return self.height - self.yScale(d.InputCount); })
                .attr("y", function (d) { return self.yScale(d.InputCount); });
            var outputCounts = frame.select('.outputs').selectAll(".outputCounts")
                .data(function (d) { return d.Stats; }, function (d) { return d.Index; });
            outputCounts.exit().remove();
            outputCounts
                .transition()
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPaddingInner; })
                .attr("y", function (d) { return self.yScale(d.OutputCount); })
                .attr("height", function (d) { return self.height - self.yScale(d.OutputCount); })
                .style("fill", function (d) { return self.color(d.Index); });
            outputCounts.enter().append("rect")
                .attr("class", "outputCounts")
                .attr("width", self.barWidth)
                .attr("x", function (d, i) { return i * self.barWidth + self.barPaddingInner; })
                .attr("y", function (d) { return self.height; })
                .attr("height", 0)
                .style("fill", function (d) { return self.color(d.Index); })
                .on('click', function (d) {
                nv.tooltip.cleanup();
                var offset = $(this).offset();
                var leftScroll = $("#indexStatsContainer").scrollLeft();
                var containerOffset = $("#indexStatsContainer").offset();
                nv.tooltip.show([offset.left - containerOffset.left + leftScroll + self.barWidth, offset.top - containerOffset.top], self.getTooltip(d), 's', 5, document.getElementById("indexStatsContainer"), "selectable-tooltip");
            })
                .transition()
                .attr("y", function (d) { return self.yScale(d.OutputCount); })
                .attr("height", function (d) { return self.height - self.yScale(d.OutputCount); });
            this.legend = this.svg.select('.controlls').selectAll(".legend")
                .data(this.indexNames, function (d) { return d; });
            this.legend.selectAll("rect").transition()
                .attr("x", this.width - 18);
            this.legend.selectAll("text").transition()
                .attr("x", this.width - 24)
                .text(function (d) { return d; });
            var legendEnter = this.legend
                .enter().append("g")
                .attr("class", "legend")
                .attr("transform", function (d, i) { return "translate(0," + i * 20 + ")"; });
            legendEnter.append("rect")
                .attr("x", this.width - 18)
                .attr('class', function (d) { return 'rect-legend-' + generalUtils.escape(d); })
                .attr("width", 18)
                .attr("height", 18)
                .style("fill", self.color)
                .style("stroke", self.color)
                .on('click', function (d) { return self.toggleIndexVisible(d); });
            legendEnter.append("text")
                .attr("x", this.width - 24)
                .attr("y", 9)
                .attr("dy", ".35em")
                .style("text-anchor", "end")
                .text(function (d) { return d; });
            // Bug fix: default to zero if we don't have any .legend text objects. This can happen when 
            // getIndexPerfStatsCommand returns an empty array. See http://issues.hibernatingrhinos.com/issue/RavenDB-2929
            this.legendWidth = (d3.max($(".legend text"), function (d) { return d.getBBox().width; }) + 40)
                || 0;
            this.svg.select('.legend_bg')
                .attr('y', -6)
                .attr('height', this.indexNames.length * 20 + 10)
                .attr('width', this.legendWidth)
                .attr('x', this.width - this.legendWidth + 10);
        };
        indexStats.prototype.onWindowHeightChanged = function () {
            nv.tooltip.cleanup();
            this.width = $("#indexStatsContainer").width();
            this.height = $("#indexStatsContainer").height();
            this.redrawGraph();
        };
        indexStats.prototype.getTooltip = function (d) {
            return "<strong>Index:</strong> <span>" + d.Index + "</span><br />"
                + "<strong>Duration milliseconds:</strong> <span>" + d.DurationMilliseconds + "</span><br />"
                + "<strong>Input count:</strong> <span>" + d.InputCount + "</span><br />"
                + "<strong>Output count:</strong> <span>" + d.OutputCount + "</span><br />"
                + "<strong>Items count:</strong> <span>" + d.ItemsCount + "</span><br />";
        };
        indexStats.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#visualizerContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
            if (this.refreshSubscription != null) {
                this.refreshSubscription.dispose();
            }
        };
        indexStats.prototype.findIndexNames = function (jsonData) {
            var statsInline = d3.merge(jsonData.map(function (d) { return d.Stats; }));
            var byKey = d3
                .nest()
                .key(function (d) { return d.Index; })
                .sortKeys(d3.ascending)
                .rollup(function (l) { return l.length; })
                .entries(statsInline);
            return byKey.map(function (d) { return d.key; });
        };
        return indexStats;
    })(viewModelBase);
    return indexStats;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/filteredOutIndexInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var filteredOutIndexInfo = (function (_super) {
        __extends(filteredOutIndexInfo, _super);
        function filteredOutIndexInfo(data) {
            _super.call(this);
            this.data = data;
        }
        filteredOutIndexInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return filteredOutIndexInfo;
    })(dialogViewModelBase);
    return filteredOutIndexInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/mapBatchInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var mapBatchInfo = (function (_super) {
        __extends(mapBatchInfo, _super);
        function mapBatchInfo(indexingBatchInfo) {
            _super.call(this);
            this.indexingBatchInfo = indexingBatchInfo;
        }
        mapBatchInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return mapBatchInfo;
    })(dialogViewModelBase);
    return mapBatchInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/mapStatsInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var mapStatsInfo = (function (_super) {
        __extends(mapStatsInfo, _super);
        function mapStatsInfo(perfStats) {
            _super.call(this);
            this.perfStats = perfStats;
        }
        mapStatsInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return mapStatsInfo;
    })(dialogViewModelBase);
    return mapStatsInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/prefetchInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var prefetchInfo = (function (_super) {
        __extends(prefetchInfo, _super);
        function prefetchInfo(data) {
            _super.call(this);
            this.data = data;
        }
        prefetchInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return prefetchInfo;
    })(dialogViewModelBase);
    return prefetchInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/reduceBatchInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var reduceBatchInfo = (function (_super) {
        __extends(reduceBatchInfo, _super);
        function reduceBatchInfo(reducingBatchInfo) {
            _super.call(this);
            this.reducingBatchInfo = reducingBatchInfo;
        }
        reduceBatchInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return reduceBatchInfo;
    })(dialogViewModelBase);
    return reduceBatchInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/indexing/perfDialogs/reduceStatsInfo", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var reduceStatsInfo = (function (_super) {
        __extends(reduceStatsInfo, _super);
        function reduceStatsInfo(perfStats) {
            _super.call(this);
            this.perfStats = perfStats;
        }
        reduceStatsInfo.prototype.cancel = function () {
            dialog.close(this);
        };
        return reduceStatsInfo;
    })(dialogViewModelBase);
    return reduceStatsInfo;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/requests/requests", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/database/status/status", 'viewmodels/shell'], function (require, exports, viewModelBase, appUrl, status, shell) {
    var requests = (function (_super) {
        __extends(requests, _super);
        function requests() {
            var _this = this;
            _super.call(this);
            this.router = status.statusRouter.createChildRouter()
                .map([
                { route: 'databases/status/requests', moduleId: 'viewmodels/database/status/requests/requestsCount', title: 'Requests count', tooltip: "Displays requests counts over time", nav: true, hash: appUrl.forCurrentDatabase().requestsCount },
                { route: 'databases/status/requests/tracing', moduleId: 'viewmodels/database/status/requests/requestTracing', title: 'Request tracing', tooltip: "Displays recent requests with their status and execution times", nav: shell.canExposeConfigOverTheWire(), hash: appUrl.forCurrentDatabase().requestsTracing },
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return requests;
    })(viewModelBase);
    return requests;
});
/// <reference path="../../../../../Scripts/typings/d3/nvd3.d.ts" />
/// <reference path="../../../../../Scripts/typings/d3/d3.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/requests/requestsCount", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getDebugMetricsCommand", "d3/d3", "nvd3"], function (require, exports, viewModelBase, getDebugMetricsCommand, d3, nv) {
    var requestsCount = (function (_super) {
        __extends(requestsCount, _super);
        function requestsCount() {
            _super.apply(this, arguments);
            this.currentMetrics = ko.observable(null);
            this.requestsMetricsUrl = ko.observable("");
            this.requestCounterChart = null;
            this.requestCounterChartData = [
                {
                    key: "Mean",
                    values: []
                },
                {
                    key: "1 min",
                    values: []
                },
                {
                    key: "5 min",
                    values: []
                },
                {
                    key: "15 min",
                    values: []
                }
            ];
        }
        requestsCount.prototype.modelPolling = function () {
            var _this = this;
            var deferred = $.Deferred();
            this.fetchMetrics().then(function () {
                _this.appendData();
                _this.updateGraph();
                deferred.resolve();
            });
            return deferred;
        };
        requestsCount.prototype.compositionComplete = function () {
            this.modelPolling();
        };
        requestsCount.prototype.canDeactivate = function () {
            window.onresize = null; // FIX nvd3 event attached globally
            return true;
        };
        requestsCount.prototype.appendData = function () {
            var metric = this.currentMetrics();
            var now = new Date();
            this.requestCounterChartData[0].values.push({
                x: now,
                y: metric.Requests.MeanRate
            });
            this.requestCounterChartData[1].values.push({
                x: now,
                y: metric.Requests.OneMinuteRate
            });
            this.requestCounterChartData[2].values.push({
                x: now,
                y: metric.Requests.FiveMinuteRate
            });
            this.requestCounterChartData[3].values.push({
                x: now,
                y: metric.Requests.FifteenMinuteRate
            });
        };
        requestsCount.prototype.updateGraph = function () {
            var _this = this;
            if (this.requestCounterChart === null) {
                nv.addGraph(function () {
                    var chart = nv.models.lineChart()
                        .margin({ left: 130 })
                        .useInteractiveGuideline(true)
                        .transitionDuration(350)
                        .showLegend(true)
                        .showYAxis(true)
                        .showXAxis(true)
                        .forceY([0]);
                    ;
                    chart.xAxis
                        .axisLabel('Time')
                        .tickFormat(function (d) { return d3.time.format('%H:%M:%S')(new Date(d)); });
                    chart.xScale(d3.time.scale());
                    chart.yAxis
                        .axisLabel('request/sec')
                        .tickFormat(d3.format('.01f'));
                    nv.utils.windowResize(function () { chart.update(); });
                    return chart;
                }, function (chart) {
                    _this.requestCounterChart = chart;
                    d3.select('#requestCounterContainer svg')
                        .datum(_this.requestCounterChartData)
                        .call(_this.requestCounterChart);
                });
            }
            else {
                // just update data
                d3.select('#requestCounterContainer svg')
                    .datum(this.requestCounterChartData)
                    .call(this.requestCounterChart);
            }
        };
        requestsCount.prototype.fetchMetrics = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var command = new getDebugMetricsCommand(db);
                this.requestsMetricsUrl(command.getQueryUrl());
                return command
                    .execute()
                    .done(function (m) { return _this.currentMetrics(m); });
            }
            return null;
        };
        return requestsCount;
    })(viewModelBase);
    return requestsCount;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/requests/requestTracing", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getRequestTracingCommand", "common/bindingHelpers/autoRefreshBindingHandler"], function (require, exports, viewModelBase, getRequestTracingCommand, autoRefreshBindingHandler) {
    var requestTracing = (function (_super) {
        __extends(requestTracing, _super);
        function requestTracing() {
            var _this = this;
            _super.call(this);
            this.allEntries = ko.observableArray();
            this.statusFilter = ko.observable("All");
            this.selectedEntry = ko.observable();
            autoRefreshBindingHandler.install();
            this.failedCount = ko.computed(function () { return _this.allEntries().count(function (l) { return l.StatusCode >= 400; }); });
            this.successCount = ko.computed(function () { return _this.allEntries().count(function (l) { return l.StatusCode < 400; }); });
            this.activeDatabase.subscribe(function () { return _this.fetchRequestTracing(); });
        }
        requestTracing.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            return this.fetchRequestTracing();
        };
        requestTracing.prototype.fetchRequestTracing = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getRequestTracingCommand(db)
                    .execute()
                    .done(function (results) { return _this.processResults(results); });
            }
            return null;
        };
        requestTracing.prototype.processResults = function (results) {
            var _this = this;
            results.forEach(function (r) {
                r['IsVisible'] = ko.computed(function () { return _this.matchesFilter(r); });
            });
            this.allEntries(results.reverse());
        };
        requestTracing.prototype.matchesFilter = function (entry) {
            var statusFilter = this.statusFilter();
            var matchesStatusFilter = true;
            if (statusFilter === "Success" && entry.StatusCode >= 400) {
                matchesStatusFilter = false;
            }
            else if (statusFilter === "Failed" && entry.StatusCode < 400) {
                matchesStatusFilter = false;
            }
            return matchesStatusFilter;
        };
        requestTracing.prototype.selectEntry = function (entry) {
            this.selectedEntry(entry);
        };
        requestTracing.prototype.tableKeyDown = function (sender, e) {
            var isKeyUp = e.keyCode === 38;
            var isKeyDown = e.keyCode === 40;
            if (isKeyUp || isKeyDown) {
                e.preventDefault();
                var oldSelection = this.selectedEntry();
                if (oldSelection) {
                    var oldSelectionIndex = this.allEntries.indexOf(oldSelection);
                    var newSelectionIndex = oldSelectionIndex;
                    if (isKeyUp && oldSelectionIndex > 0) {
                        newSelectionIndex--;
                    }
                    else if (isKeyDown && oldSelectionIndex < this.allEntries().length - 1) {
                        newSelectionIndex++;
                    }
                    this.selectedEntry(this.allEntries()[newSelectionIndex]);
                    var newSelectedRow = $("#requestTracingTableContainer table tbody tr:nth-child(" + (newSelectionIndex + 1) + ")");
                    if (newSelectedRow) {
                        this.ensureRowVisible(newSelectedRow);
                    }
                }
            }
        };
        requestTracing.prototype.showContextMenu = function () {
            //alert("this");
        };
        requestTracing.prototype.ensureRowVisible = function (row) {
            var table = $("#requestTracingTableContainer");
            var scrollTop = table.scrollTop();
            var scrollBottom = scrollTop + table.height();
            var scrollHeight = scrollBottom - scrollTop;
            var rowPosition = row.position();
            var rowTop = rowPosition.top;
            var rowBottom = rowTop + row.height();
            if (rowTop < 0) {
                table.scrollTop(scrollTop + rowTop);
            }
            else if (rowBottom > scrollHeight) {
                table.scrollTop(scrollTop + (rowBottom - scrollHeight));
            }
        };
        requestTracing.prototype.setStatusAll = function () {
            this.statusFilter("All");
        };
        requestTracing.prototype.setStatusSuccess = function () {
            this.statusFilter("Success");
        };
        requestTracing.prototype.setStatusFailed = function () {
            this.statusFilter("Failed");
        };
        return requestTracing;
    })(viewModelBase);
    return requestTracing;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/storage/statusStorage", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/database/status/status", 'viewmodels/shell'], function (require, exports, viewModelBase, appUrl, status, shell) {
    var statusDebug = (function (_super) {
        __extends(statusDebug, _super);
        function statusDebug() {
            var _this = this;
            _super.call(this);
            this.router = status.statusRouter.createChildRouter()
                .map([
                { route: 'databases/status/storage', moduleId: 'viewmodels/database/status/storage/statusStorageOnDisk', title: 'On disk', tooltip: "Shows disk usage for active resource", nav: shell.isGlobalAdmin(), hash: appUrl.forCurrentDatabase().statusStorageOnDisk },
                { route: 'databases/status/storage/storageBreakdown', moduleId: 'viewmodels/database/status/storage/statusStorageBreakdown', title: 'Internal storage Breakdown', tooltip: "Shows detailed information about internal storage breakdown", nav: shell.isGlobalAdmin(), hash: appUrl.forCurrentDatabase().statusStorageBreakdown },
                { route: 'databases/status/storage/collections', moduleId: 'viewmodels/database/status/storage/statusStorageCollections', title: 'Collections storage', tooltip: "Shows document counts (VERY SLOW)", nav: true, hash: appUrl.forCurrentDatabase().statusStorageCollections },
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return statusDebug;
    })(viewModelBase);
    return statusDebug;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/storage/statusStorageBreakdown", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getStatusStorageBreakdownCommand", 'viewmodels/shell'], function (require, exports, viewModelBase, getStatusStorageBreakdownCommand, shell) {
    var statusStorageOnDisk = (function (_super) {
        __extends(statusStorageOnDisk, _super);
        function statusStorageOnDisk() {
            _super.apply(this, arguments);
            this.data = ko.observable();
            this.isGlobalAdmin = shell.isGlobalAdmin;
        }
        statusStorageOnDisk.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (this.isGlobalAdmin()) {
                this.activeDatabase.subscribe(function () { return _this.fetchData(); });
                return this.fetchData();
            }
        };
        statusStorageOnDisk.prototype.formatToPreTag = function (input) {
            return input.replaceAll('\r\n', '<br />').replaceAll("\t", '&nbsp;&nbsp;&nbsp;&nbsp;');
        };
        statusStorageOnDisk.prototype.fetchData = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (!!db) {
                return new getStatusStorageBreakdownCommand(db)
                    .execute()
                    .done(function (result) { return _this.data(result.map(_this.formatToPreTag)); });
            }
            return null;
        };
        return statusStorageOnDisk;
    })(viewModelBase);
    return statusStorageOnDisk;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/storage/statusStorageCollections", ["require", "exports", "commands/database/debug/getSlowDocCountsCommand", "viewmodels/viewModelBase", "common/generalUtils"], function (require, exports, getSlowDocCountsCommand, viewModelBase, genUtils) {
    var statusStorageCollections = (function (_super) {
        __extends(statusStorageCollections, _super);
        function statusStorageCollections() {
            _super.apply(this, arguments);
            this.data = ko.observable();
            this.canSearch = ko.observable(true);
            this.formatTimeSpan = genUtils.formatTimeSpan;
            this.formatBytesToSize = genUtils.formatBytesToSize;
        }
        statusStorageCollections.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.resetView(); });
            return this.resetView();
        };
        statusStorageCollections.prototype.resetView = function () {
            this.data(null);
            this.canSearch(true);
        };
        statusStorageCollections.prototype.fetchDocCounts = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                this.canSearch(false);
                return new getSlowDocCountsCommand(db)
                    .execute()
                    .done(function (results) {
                    _this.data(results);
                })
                    .always(function () { return _this.canSearch(true); });
            }
            return null;
        };
        return statusStorageCollections;
    })(viewModelBase);
    return statusStorageCollections;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/status/storage/statusStorageOnDisk", ["require", "exports", "viewmodels/viewModelBase", "commands/database/debug/getStatusDebugConfigCommand", "commands/database/debug/getStatusStorageOnDiskCommand"], function (require, exports, viewModelBase, getStatusDebugConfigCommand, getStatusStorageOnDiskCommand) {
    var statusStorageOnDisk = (function (_super) {
        __extends(statusStorageOnDisk, _super);
        function statusStorageOnDisk() {
            var _this = this;
            _super.call(this);
            this.onDiskStats = ko.observable();
            this.config = ko.observable();
            this.isConfigForbidden = ko.computed(function () { return !_this.config(); });
            this.journalsStoragePath = ko.computed(function () {
                if (!!_this.config() && !!_this.config().JournalsStoragePath) {
                    return _this.config().JournalsStoragePath;
                }
                return "-";
            });
        }
        statusStorageOnDisk.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.fetchData(); });
            return this.fetchData();
        };
        statusStorageOnDisk.prototype.fetchData = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var onDiskTask = new getStatusStorageOnDiskCommand(db)
                    .execute();
                var configTask = new getStatusDebugConfigCommand(db)
                    .execute();
                var combinedTask = $.when(onDiskTask, configTask);
                combinedTask.done(function (onDisk, config) {
                    _this.onDiskStats(onDisk[0]);
                    _this.config(config[0]);
                });
                return combinedTask;
            }
            return null;
        };
        return statusStorageOnDisk;
    })(viewModelBase);
    return statusStorageOnDisk;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/createSampleData", ["require", "exports", "viewmodels/viewModelBase", "durandal/app", "commands/database/studio/createSampleDataCommand", "commands/database/studio/createSampleDataClassCommand", "viewmodels/common/showDataDialog"], function (require, exports, viewModelBase, app, createSampleDataCommand, createSampleDataClassCommand, showDataDialog) {
    var createSampleData = (function (_super) {
        __extends(createSampleData, _super);
        function createSampleData() {
            _super.apply(this, arguments);
            this.isBusy = ko.observable(false);
            this.isEnable = ko.observable(true);
            this.isVisible = ko.observable(false);
            this.classData = ko.observable();
        }
        createSampleData.prototype.generateSampleData = function () {
            var _this = this;
            this.isBusy(true);
            new createSampleDataCommand(this.activeDatabase())
                .execute()
                .always(function () { return _this.isBusy(false); });
        };
        createSampleData.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('OGRN53');
        };
        createSampleData.prototype.showSampleDataClass = function () {
            var _this = this;
            new createSampleDataClassCommand(this.activeDatabase())
                .execute()
                .done(function (results) {
                _this.isVisible(true);
                var data = results.replace("\r\n", "");
                app.showDialog(new showDataDialog("Sample Data Classes", data));
            })
                .always(function () { return _this.isBusy(false); });
        };
        return createSampleData;
    })(viewModelBase);
    return createSampleData;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/csvImport", ["require", "exports", "viewmodels/viewModelBase", "commands/database/studio/saveCsvFileCommand"], function (require, exports, viewmodelBase, saveCsvFileCommand) {
    var csvImport = (function (_super) {
        __extends(csvImport, _super);
        function csvImport() {
            _super.apply(this, arguments);
            this.hasFileSelected = ko.observable(false);
            this.isImporting = ko.observable(false);
        }
        csvImport.prototype.fileSelected = function (args) {
            this.hasFileSelected(true);
        };
        csvImport.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('CX96R3');
        };
        csvImport.prototype.importCsv = function () {
            var _this = this;
            if (!this.isImporting()) {
                this.isImporting(true);
                var formData = new FormData();
                var fileInput = document.querySelector("#csvFilePicker");
                formData.append("file", fileInput.files[0]);
                new saveCsvFileCommand(formData, fileInput.files[0].name, this.activeDatabase())
                    .execute()
                    .always(function () { return _this.isImporting(false); });
            }
        };
        return csvImport;
    })(viewmodelBase);
    return csvImport;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/exportDatabase", ["require", "exports", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "commands/database/documents/getCollectionsCommand", "commands/database/studio/validateExportDatabaseOptionsCommand", "common/appUrl", "commands/auth/getSingleAuthTokenCommand", 'common/messagePublisher'], function (require, exports, viewModelBase, aceEditorBindingHandler, getCollectionsCommand, validateExportDatabaseOptionsCommand, appUrl, getSingleAuthTokenCommand, messagePublisher) {
    var filterSetting = (function () {
        function filterSetting() {
            this.path = ko.observable("");
            this.value = ko.observable("");
            this.shouldMatch = ko.observable(false);
        }
        filterSetting.empty = function () {
            return new filterSetting();
        };
        return filterSetting;
    })();
    var exportDatabase = (function (_super) {
        __extends(exportDatabase, _super);
        function exportDatabase() {
            _super.call(this);
            this.includeDocuments = ko.observable(true);
            this.includeIndexes = ko.observable(true);
            this.includeTransformers = ko.observable(true);
            this.includeAttachments = ko.observable(false);
            this.includeExpiredDocuments = ko.observable(false);
            this.includeAllCollections = ko.observable(true);
            this.removeAnalyzers = ko.observable(false);
            this.showAdvancedOptions = ko.observable(false);
            this.batchSize = ko.observable(1024);
            this.includedCollections = ko.observableArray();
            this.filters = ko.observableArray();
            this.transformScript = ko.observable();
            this.noneDefualtFileName = ko.observable("");
            this.chooseDifferntFileName = ko.observable(false);
            this.authToken = ko.observable();
            aceEditorBindingHandler.install();
        }
        exportDatabase.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('YD9M1R');
            new getCollectionsCommand(this.activeDatabase())
                .execute()
                .done(function (collections) {
                _this.includedCollections(collections.map(function (c) {
                    return {
                        collection: c.name,
                        isIncluded: ko.observable(false)
                    };
                }));
            });
            this.exportActionUrl = ko.computed(function () {
                var token = _this.authToken();
                return appUrl.forResourceQuery(_this.activeDatabase()) + "/studio-tasks/exportDatabase" + (token ? '?singleUseAuthToken=' + token : '');
            });
            this.exportCommand = ko.computed(function () {
                var targetServer = appUrl.forServer();
                var outputFilename = _this.chooseDifferntFileName() ? exportDatabase.escapeForShell(_this.noneDefualtFileName()) : "raven.dump";
                var commandTokens = ["Raven.Smuggler", "out", targetServer, outputFilename];
                var types = [];
                if (_this.includeDocuments()) {
                    types.push("Documents");
                }
                if (_this.includeIndexes()) {
                    types.push("Indexes");
                }
                if (_this.includeAttachments()) {
                    types.push("Attachments");
                }
                if (_this.includeTransformers()) {
                    types.push("Transformers");
                }
                if (_this.removeAnalyzers()) {
                    types.push("RemoveAnalyzers");
                }
                if (types.length > 0) {
                    commandTokens.push("--operate-on-types=" + types.join(","));
                }
                var databaseName = _this.activeDatabase().name;
                commandTokens.push("--database=" + exportDatabase.escapeForShell(databaseName));
                var batchSize = _this.batchSize();
                commandTokens.push("--batch-size=" + batchSize);
                if (!_this.includeExpiredDocuments()) {
                    commandTokens.push("--excludeexpired");
                }
                if (!_this.includeAllCollections()) {
                    var collections = _this.includedCollections().filter(function (collection) { return collection.isIncluded(); }).map(function (collection) { return collection.collection; });
                    commandTokens.push("--metadata-filter=Raven-Entity-Name=" + exportDatabase.escapeForShell(collections.toString()));
                }
                var filters = exportDatabase.convertFiltersToDto(_this.filters());
                for (var i = 0; i < filters.length; i++) {
                    var filter = filters[i];
                    var parameterName = filter.ShouldMatch ? "--filter=" : "--negative-filter=";
                    commandTokens.push(parameterName + filter.Path + "=" + exportDatabase.escapeForShell(filter.Values.toString()));
                }
                if (_this.transformScript()) {
                    commandTokens.push("--transform=" + exportDatabase.escapeForShell(_this.transformScript()));
                }
                return commandTokens.join(" ");
            });
        };
        /**
         * Groups filters by key
         */
        exportDatabase.convertFiltersToDto = function (filters) {
            var output = [];
            for (var i = 0; i < filters.length; i++) {
                var filter = filters[i];
                var existingOutput = output.first(function (x) { return filter.shouldMatch() === x.ShouldMatch && filter.path() === x.Path; });
                if (existingOutput) {
                    existingOutput.Values.push(filter.value());
                }
                else {
                    output.push({
                        ShouldMatch: filter.shouldMatch(),
                        Path: filter.path(),
                        Values: [filter.value()]
                    });
                }
            }
            return output;
        };
        exportDatabase.escapeForShell = function (input) {
            return '"' + input.replace(/[\r\n]/g, "").replace(/(["\\])/g, '\\$1') + '"';
        };
        exportDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#transformScriptHelp").popover({
                html: true,
                trigger: 'hover',
                content: "Transform scripts are written in JavaScript. <br /><br/>Example:<pre><span class=\"code-keyword\">function</span>(doc) {<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">var</span> id = doc['@metadata']['@id'];<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">if</span> (id === 'orders/999')<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return null</span>;<br /><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return</span> doc;<br />}</pre>"
            });
        };
        exportDatabase.prototype.startExport = function () {
            var _this = this;
            var operateOnTypes = 0;
            if (this.includeDocuments()) {
                operateOnTypes += 1;
            }
            if (this.includeIndexes()) {
                operateOnTypes += 2;
            }
            if (this.includeAttachments()) {
                operateOnTypes += 4;
            }
            if (this.includeTransformers()) {
                operateOnTypes += 8;
            }
            if (this.removeAnalyzers()) {
                operateOnTypes += 8000;
            }
            var filtersToSend = exportDatabase.convertFiltersToDto(this.filters());
            if (!this.includeAllCollections()) {
                filtersToSend.push({
                    ShouldMatch: true,
                    Path: "@metadata.Raven-Entity-Name",
                    Values: this.includedCollections().filter(function (curCol) { return curCol.isIncluded(); }).map(function (curCol) { return curCol.collection; })
                });
            }
            var smugglerOptions = {
                OperateOnTypes: operateOnTypes,
                BatchSize: this.batchSize(),
                ShouldExcludeExpired: !this.includeExpiredDocuments(),
                Filters: filtersToSend,
                TransformScript: this.transformScript(),
                NoneDefualtFileName: this.noneDefualtFileName()
            };
            $("#SmugglerOptions").val(JSON.stringify(smugglerOptions));
            new validateExportDatabaseOptionsCommand(smugglerOptions, this.activeDatabase()).execute()
                .done(function () {
                new getSingleAuthTokenCommand(_this.activeDatabase()).execute().done(function (token) {
                    _this.authToken(token.Token);
                    $("#dbExportDownloadForm").submit();
                }).fail(function (qXHR, textStatus, errorThrown) { return messagePublisher.reportError("Could not get Single Auth Token for export.", errorThrown); });
            })
                .fail(function (response) {
                messagePublisher.reportError("Invalid export options", response.responseText, response.statusText);
            });
        };
        exportDatabase.prototype.selectOptions = function () {
            this.showAdvancedOptions(false);
        };
        exportDatabase.prototype.selectAdvancedOptions = function () {
            this.showAdvancedOptions(true);
        };
        exportDatabase.prototype.removeFilter = function (filter) {
            this.filters.remove(filter);
        };
        exportDatabase.prototype.addFilter = function () {
            var setting = filterSetting.empty();
            setting.shouldMatch(true);
            this.filters.unshift(setting);
        };
        return exportDatabase;
    })(viewModelBase);
    return exportDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/importDatabase", ["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "commands/operations/getOperationStatusCommand", "common/messagePublisher", "commands/database/studio/importDatabaseCommand", "commands/database/studio/checksufficientdiskspaceCommand"], function (require, exports, aceEditorBindingHandler, viewModelBase, getOperationStatusCommand, messagePublisher, importDatabaseCommand, checksufficientdiskspaceCommand) {
    var importDatabase = (function (_super) {
        __extends(importDatabase, _super);
        function importDatabase() {
            _super.call(this);
            this.showAdvancedOptions = ko.observable(false);
            this.filters = ko.observableArray();
            this.batchSize = ko.observable(1024);
            this.includeExpiredDocuments = ko.observable(true);
            this.stripReplicationInformation = ko.observable(false);
            this.shouldDisableVersioningBundle = ko.observable(false);
            this.transformScript = ko.observable();
            this.includeDocuments = ko.observable(true);
            this.includeIndexes = ko.observable(true);
            this.includeAttachments = ko.observable(false);
            this.includeTransformers = ko.observable(true);
            this.removeAnalyzers = ko.observable(false);
            this.hasFileSelected = ko.observable(false);
            this.importedFileName = ko.observable();
            this.isUploading = false;
            this.filePickerTag = "#importDatabaseFilePicker";
            aceEditorBindingHandler.install();
        }
        importDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#transformScriptHelp").popover({
                html: true,
                trigger: "hover",
                content: "Transform scripts are written in JavaScript. <br /><br/>Example:<pre><span class=\"code-keyword\">function</span>(doc) {<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">var</span> id = doc['@metadata']['@id'];<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">if</span> (id === 'orders/999')<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return null</span>;<br /><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class=\"code-keyword\">return</span> doc;<br />}</pre>"
            });
            this.updateHelpLink("YD9M1R");
            var db = this.activeDatabase();
            var importStatus = db.importStatus();
            if (importStatus === "Uploading 100%") {
                db.importStatus("");
            }
        };
        importDatabase.prototype.canDeactivate = function (isClose) {
            _super.prototype.canDeactivate.call(this, isClose);
            if (this.isUploading) {
                this.confirmationMessage("Upload is in progress", "Please wait until uploading is complete.", ["OK"]);
                return false;
            }
            return true;
        };
        importDatabase.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("UploadProgress", function (percentComplete) { return _this.activeDatabase().importStatus("Uploading " + percentComplete.toFixed(2).replace(/\.0*$/, "") + "%"); }),
                ko.postbox.subscribe("ChangesApiReconnected", function (db) {
                    db.importStatus("");
                    db.isImporting(false);
                    _this.isUploading = false;
                })
            ];
        };
        importDatabase.prototype.selectOptions = function () {
            this.showAdvancedOptions(false);
        };
        importDatabase.prototype.selectAdvancedOptions = function () {
            this.showAdvancedOptions(true);
        };
        importDatabase.prototype.removeFilter = function (filter) {
            this.filters.remove(filter);
        };
        importDatabase.prototype.addFilter = function () {
            var filter = {
                Path: "",
                ShouldMatch: false,
                ShouldMatchObservable: ko.observable(false),
                Values: []
            };
            filter.ShouldMatchObservable.subscribe(function (val) { return filter.ShouldMatch = val; });
            this.filters.splice(0, 0, filter);
        };
        importDatabase.prototype.fileSelected = function (fileName) {
            var _this = this;
            var db = this.activeDatabase();
            var isFileSelected = !!$.trim(fileName);
            var importFileName = $(this.filePickerTag).val().split(/(\\|\/)/g).pop();
            if (isFileSelected) {
                var fileInput = document.querySelector(this.filePickerTag);
                new checksufficientdiskspaceCommand(fileInput.files[0].size, this.activeDatabase())
                    .execute()
                    .done(function () {
                    _this.hasFileSelected(isFileSelected);
                    _this.importedFileName(importFileName);
                    db.importStatus("");
                })
                    .fail(function () {
                    db.importStatus("No sufficient diskspace for import, consider using Raven.Smuggler.exe directly.");
                    _this.hasFileSelected(false);
                    _this.importedFileName("");
                });
            }
        };
        importDatabase.prototype.importDb = function () {
            var _this = this;
            var db = this.activeDatabase();
            db.isImporting(true);
            this.isUploading = true;
            var formData = new FormData();
            var fileInput = document.querySelector(this.filePickerTag);
            formData.append("file", fileInput.files[0]);
            db.importStatus("Uploading 0%");
            var importItemTypes = [];
            if (this.includeDocuments()) {
                importItemTypes.push(1 /* Documents */);
            }
            if (this.includeIndexes()) {
                importItemTypes.push(2 /* Indexes */);
            }
            if (this.includeAttachments()) {
                importItemTypes.push(4 /* Attachments */);
            }
            if (this.includeTransformers()) {
                importItemTypes.push(8 /* Transformers */);
            }
            if (this.removeAnalyzers()) {
                importItemTypes.push(32768 /* RemoveAnalyzers */);
            }
            new importDatabaseCommand(formData, this.batchSize(), this.includeExpiredDocuments(), this.stripReplicationInformation(), this.shouldDisableVersioningBundle(), importItemTypes, this.filters(), this.transformScript(), this.activeDatabase())
                .execute()
                .done(function (result) {
                var operationId = result.OperationId;
                _this.waitForOperationToComplete(db, operationId);
                db.importStatus("Processing uploaded file");
            })
                .fail(function () { return db.importStatus(""); })
                .always(function () { return _this.isUploading = false; });
        };
        importDatabase.prototype.waitForOperationToComplete = function (db, operationId) {
            var _this = this;
            new getOperationStatusCommand(db, operationId)
                .execute()
                .done(function (result) { return _this.importStatusRetrieved(db, operationId, result); });
        };
        importDatabase.prototype.importStatusRetrieved = function (db, operationId, result) {
            var _this = this;
            if (result.Completed) {
                if (result.ExceptionDetails == null && result.State != null && result.State.Progress != null) {
                    this.hasFileSelected(false);
                    $(this.filePickerTag).val('');
                    db.importStatus("Last import was from '" + this.importedFileName() + "', " + result.State.Progress.toLocaleLowerCase());
                    messagePublisher.reportSuccess("Successfully imported data to " + db.name);
                }
                else {
                    db.importStatus("");
                    messagePublisher.reportError("Failed to import data!", result.ExceptionDetails);
                }
                db.isImporting(false);
            }
            else {
                if (!!result.State && result.State.Progress) {
                    db.importStatus("Processing uploaded file, " + result.State.Progress.toLocaleLowerCase());
                }
                setTimeout(function () { return _this.waitForOperationToComplete(db, operationId); }, 1000);
            }
        };
        return importDatabase;
    })(viewModelBase);
    return importDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/subscriptionsTask", ["require", "exports", "viewmodels/viewModelBase", "commands/database/subscriptions/getSubscriptionCommand", "commands/database/subscriptions/setSubscriptionAckEtagCommand"], function (require, exports, viewModelBase, getSubscriptionCommand, setSubscriptionCommand) {
    var SubscriptionsTask = (function (_super) {
        __extends(SubscriptionsTask, _super);
        function SubscriptionsTask() {
            _super.apply(this, arguments);
            this.data = ko.observableArray();
        }
        SubscriptionsTask.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeDatabase.subscribe(function () { return _this.fetchSubscriptions(); });
            return this.fetchSubscriptions();
        };
        SubscriptionsTask.prototype.fetchSubscriptions = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getSubscriptionCommand(db)
                    .execute()
                    .done(function (results) { return _this.data(results); });
            }
            return null;
        };
        SubscriptionsTask.prototype.setSubscriptionEtag = function (subscription) {
            var db = this.activeDatabase();
            if (db) {
                subscription.isChangeInProgress(true);
                var newAckEtag = subscription.newAckEtag();
                return new setSubscriptionCommand(db, subscription.subscriptionId, newAckEtag)
                    .execute()
                    .done(function () { return subscription.ackEtag(newAckEtag); })
                    .always(function () { return subscription.isChangeInProgress(false); });
            }
            return null;
        };
        return SubscriptionsTask;
    })(viewModelBase);
    return SubscriptionsTask;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/tasks", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var tasks = (function (_super) {
        __extends(tasks, _super);
        function tasks() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.isOnSystemDatabase = ko.computed(function () { return !!_this.activeDatabase() && _this.activeDatabase().isSystem; });
            this.isOnUserDatabase = ko.computed(function () { return !!_this.activeDatabase() && !_this.isOnSystemDatabase(); });
            this.appUrls = appUrl.forCurrentDatabase();
            this.router = durandalRouter.createChildRouter();
        }
        tasks.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            var importDatabaseUrl = ko.computed(function () { return appUrl.forImportDatabase(_this.activeDatabase()); });
            var exportDatabaseUrl = ko.computed(function () { return appUrl.forExportDatabase(_this.activeDatabase()); });
            var toggleIndexingUrl = ko.computed(function () { return appUrl.forToggleIndexing(_this.activeDatabase()); });
            var setAcknowledgedEtagUrl = ko.computed(function () { return appUrl.forSetAcknowledgedEtag(_this.activeDatabase()); });
            var sampleDataUrl = ko.computed(function () { return appUrl.forSampleData(_this.activeDatabase()); });
            var csvImportUrl = ko.computed(function () { return appUrl.forCsvImport(_this.activeDatabase()); });
            var routeArray = [
                { route: ['databases/tasks', 'databases/tasks/importDatabase'], moduleId: 'viewmodels/database/tasks/importDatabase', title: 'Import Database', nav: true, hash: importDatabaseUrl },
                { route: 'databases/tasks/exportDatabase', moduleId: 'viewmodels/database/tasks/exportDatabase', title: 'Export Database', nav: true, hash: exportDatabaseUrl },
                { route: 'databases/tasks/toggleIndexing', moduleId: 'viewmodels/database/tasks/toggleIndexing', title: 'Toggle Indexing', nav: this.activeDatabase() && this.activeDatabase().isAdminCurrentTenant(), hash: toggleIndexingUrl },
                { route: 'databases/tasks/subscriptionsTask', moduleId: 'viewmodels/database/tasks/subscriptionsTask', title: 'Subscriptions', nav: this.activeDatabase() && this.activeDatabase().isAdminCurrentTenant(), hash: setAcknowledgedEtagUrl },
                { route: 'databases/tasks/sampleData', moduleId: 'viewmodels/database/tasks/createSampleData', title: 'Create Sample Data', nav: true, hash: sampleDataUrl },
                { route: 'databases/tasks/csvImport', moduleId: 'viewmodels/database/tasks/csvImport', title: 'CSV Import', nav: true, hash: csvImportUrl }
            ];
            this.router
                .reset()
                .map(routeArray)
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        };
        return tasks;
    })(viewModelBase);
    return tasks;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/tasks/toggleIndexing", ["require", "exports", "commands/database/index/stopIndexingCommand", "commands/database/index/startIndexingCommand", "commands/database/index/stopReducingCommand", "commands/database/index/startReducingCommand", "commands/database/index/getIndexingStatusCommand", "viewmodels/viewModelBase", "common/appUrl"], function (require, exports, stopIndexingCommand, startIndexingCommand, stopReducingCommand, startReducingCommand, getIndexingStatusCommand, viewModelBase, appUrl) {
    var toggleIndexing = (function (_super) {
        __extends(toggleIndexing, _super);
        function toggleIndexing() {
            var _this = this;
            _super.call(this);
            this.indexingStatus = ko.observable();
            this.text = ko.observable();
            this.isMappingEnabled = ko.computed(function () {
                var status = _this.indexingStatus();
                if (!status) {
                    return false;
                }
                return status.MappingStatus === "Mapping";
            });
            this.isReducingEnabled = ko.computed(function () {
                var status = _this.indexingStatus();
                if (!status) {
                    return false;
                }
                return status.ReducingStatus === "Reducing";
            });
            this.isIndexingEnabled = ko.computed(function () {
                var status = _this.indexingStatus();
                var isMappingEnabled = _this.isMappingEnabled();
                var isReducingEnabled = _this.isReducingEnabled();
                if (!status) {
                    return false;
                }
                return isMappingEnabled && isReducingEnabled;
            });
            this.canDisableIndexing = ko.computed(function () {
                var status = _this.indexingStatus();
                var isMappingEnabled = _this.isMappingEnabled();
                var isReducingEnabled = _this.isReducingEnabled();
                if (!status) {
                    return false;
                }
                return isMappingEnabled || isReducingEnabled;
            });
            this.indexingStatusText = ko.computed(function () {
                var status = _this.indexingStatus();
                var isMappingEnabled = _this.isMappingEnabled();
                var isReducingEnabled = _this.isReducingEnabled();
                if (!status) {
                    return "None";
                }
                if (isMappingEnabled && isReducingEnabled) {
                    return "Mapping & Reducing";
                }
                if (isMappingEnabled) {
                    return "Mapping only";
                }
                if (isReducingEnabled) {
                    return "Reducing only";
                }
                if (status.MappingStatus === "Disabled" && status.ReducingStatus === "Disabled") {
                    return "Disabled";
                }
                return "Paused";
            });
        }
        toggleIndexing.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            this.getIndexStatus()
                .done(function () { return deferred.resolve({ can: true }); })
                .fail(function () { return deferred.resolve({ redirect: appUrl.forTasks(_this.activeDatabase()) }); });
            return deferred;
        };
        toggleIndexing.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("VXOPAN");
        };
        toggleIndexing.prototype.disableIndexing = function () {
            var _this = this;
            new stopIndexingCommand(this.activeDatabase())
                .execute()
                .done(function () { return _this.getIndexStatus(); });
        };
        toggleIndexing.prototype.enableIndexing = function () {
            var _this = this;
            new startIndexingCommand(this.activeDatabase())
                .execute()
                .done(function () { return _this.getIndexStatus(); });
        };
        toggleIndexing.prototype.disableReducing = function () {
            var _this = this;
            new stopReducingCommand(this.activeDatabase())
                .execute()
                .done(function () { return _this.getIndexStatus(); });
        };
        toggleIndexing.prototype.enableReducing = function () {
            var _this = this;
            new startReducingCommand(this.activeDatabase())
                .execute()
                .done(function () { return _this.getIndexStatus(); });
        };
        toggleIndexing.prototype.getIndexStatus = function () {
            var _this = this;
            var deferred = $.Deferred();
            new getIndexingStatusCommand(this.activeDatabase())
                .execute()
                .done(function (result) {
                _this.indexingStatus(result);
                deferred.resolve();
            })
                .fail(function () { return deferred.reject(); });
            return deferred;
        };
        return toggleIndexing;
    })(viewModelBase);
    return toggleIndexing;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/copyTransformerDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "models/database/index/transformer", "commands/database/transformers/getSingleTransformerCommand", "commands/database/transformers/saveTransformerCommand", "plugins/router", "common/appUrl", "common/messagePublisher", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, dialog, dialogViewModelBase, transformer, getSingleTransformerCommand, saveTransformerCommand, router, appUrl, messagePublisher, aceEditorBindingHandler) {
    var copyTransformerDialog = (function (_super) {
        __extends(copyTransformerDialog, _super);
        function copyTransformerDialog(transformerName, db, isPaste, elementToFocusOnDismissal) {
            if (isPaste === void 0) { isPaste = false; }
            _super.call(this, elementToFocusOnDismissal);
            this.transformerName = transformerName;
            this.db = db;
            this.isPaste = isPaste;
            this.transformerJSON = ko.observable("");
            aceEditorBindingHandler.install();
        }
        copyTransformerDialog.prototype.canActivate = function (args) {
            var _this = this;
            if (this.isPaste) {
                return true;
            }
            else {
                var canActivateResult = $.Deferred();
                new getSingleTransformerCommand(this.transformerName, this.db)
                    .execute()
                    .done(function (results) {
                    var prettifySpacing = 4;
                    var jsonString = JSON.stringify(results, null, prettifySpacing);
                    _this.transformerJSON(jsonString);
                    canActivateResult.resolve({ can: true });
                })
                    .fail(function () { return canActivateResult.reject(); });
                canActivateResult.resolve({ can: true });
                return canActivateResult;
            }
        };
        copyTransformerDialog.prototype.setInitialFocus = function () {
            // Overrides the base class' setInitialFocus and does nothing.
            // Doing nothing because we will focus the Ace Editor when it's initialized.
        };
        copyTransformerDialog.prototype.enterKeyPressed = function () {
            // Overrides the base class' enterKeyPressed. Because the user might
            // edit the JSON, or even type some in manually, enter might really mean new line, not Save changes.
            if (!this.isPaste) {
                return _super.prototype.enterKeyPressed.call(this);
            }
            else {
                this.saveTransformer();
            }
            return true;
        };
        copyTransformerDialog.prototype.saveTransformer = function () {
            var _this = this;
            var transformerJson = this.transformerJSON();
            if (!this.isPaste) {
                this.close();
                return;
            }
            if (transformerJson) {
                var transformerDto;
                var transformerObj;
                try {
                    transformerDto = JSON.parse(transformerJson);
                    transformerObj = new transformer().initFromSave(transformerDto);
                }
                catch (e) {
                    transformerDto = null;
                    transformerObj = null;
                    messagePublisher.reportError("Transformer paste failed, invalid JSON.", e);
                }
                if (transformerDto) {
                    // Verify there's not a transformer with this name.
                    new getSingleTransformerCommand(transformerDto.Transformer.Name, this.db)
                        .execute()
                        .done(function () { return messagePublisher
                        .reportError("Duplicate transformer name. Change the name and try again."); })
                        .fail(function (xhr, status, error) {
                        if (xhr.status === 404 /* NotFound */) {
                            // Good. No existing transformer with this name. We can proceed saving it.
                            new saveTransformerCommand(transformerObj, _this.db)
                                .execute()
                                .done(function () {
                                router.navigate(appUrl.forEditTransformer(transformerObj.name(), _this.db));
                                _this.close();
                            });
                        }
                        else {
                            // Some other error occurred while checking for duplicate transformer. Error out.
                            messagePublisher.reportError("Cannot paste transformer, error occured.", error);
                        }
                    });
                }
            }
            else {
                this.close();
            }
        };
        copyTransformerDialog.prototype.close = function () {
            dialog.close(this);
        };
        return copyTransformerDialog;
    })(dialogViewModelBase);
    return copyTransformerDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/deleteTransformerConfirm", ["require", "exports", "commands/database/transformers/deleteTransformerCommand", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, DeleteTransformerCommand, dialog, dialogViewModelBase) {
    var deleteTransformerConfirm = (function (_super) {
        __extends(deleteTransformerConfirm, _super);
        function deleteTransformerConfirm(transformersNames, db) {
            _super.call(this);
            this.transformersNames = transformersNames;
            this.db = db;
            this.deleteTask = $.Deferred();
            if (!transformersNames || transformersNames.length === 0) {
                throw new Error("Transformers must not be null or empty.");
            }
            //this.message = transformersNames.length === 1 ? "You're deleting  "Really Delete '" + transformersNames[0] + "' transformer?" : 'Really delete all transformers?';
        }
        deleteTransformerConfirm.prototype.deleteTransformers = function () {
            var _this = this;
            var deleteTasks = this.transformersNames
                .map(function (name) { return new DeleteTransformerCommand(name, _this.db).execute(); });
            $.when.apply($, deleteTasks).done(function () { return _this.deleteTask.resolve(); });
            dialog.close(this);
        };
        deleteTransformerConfirm.prototype.cancel = function () {
            this.deleteTask.reject();
            dialog.close(this);
        };
        return deleteTransformerConfirm;
    })(dialogViewModelBase);
    return deleteTransformerConfirm;
});
/// <reference path="../../../models/dto.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/editTransformer", ["require", "exports", "viewmodels/viewModelBase", "models/database/index/transformer", "commands/database/transformers/saveTransformerCommand", "commands/database/transformers/getSingleTransformerCommand", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/database/transformers/deleteTransformerConfirm", "viewmodels/database/transformers/saveTransformerWithNewNameConfirm", "plugins/dialog", "common/appUrl", "common/jsonUtil", "plugins/router", "common/messagePublisher", "commands/database/index/formatIndexCommand"], function (require, exports, viewModelBase, transformer, saveTransformerCommand, getSingleTransformerCommand, aceEditorBindingHandler, deleteTransformerConfirm, saveTransformerWithNewNameConfirm, dialog, appUrl, jsonUtil, router, messagePublisher, formatIndexCommand) {
    var editTransformer = (function (_super) {
        __extends(editTransformer, _super);
        function editTransformer() {
            var _this = this;
            _super.call(this);
            this.editedTransformer = ko.observable();
            this.isEditingExistingTransformer = ko.observable(false);
            this.popoverOptions = ko.observable();
            this.editorCollection = ko.observableArray();
            this.loadedTransformerName = ko.observable();
            aceEditorBindingHandler.install();
            this.appUrls = appUrl.forCurrentDatabase();
            this.transformerName = ko.computed(function () { return (!!_this.editedTransformer() && _this.isEditingExistingTransformer()) ? _this.editedTransformer().name() : null; });
        }
        editTransformer.prototype.canActivate = function (transformerToEditName) {
            var _this = this;
            if (transformerToEditName) {
                var canActivateResult = $.Deferred();
                this.editExistingTransformer(transformerToEditName)
                    .done(function () { return canActivateResult.resolve({ can: true }); })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + transformerToEditName + " transformer");
                    canActivateResult.resolve({ redirect: appUrl.forTransformers(_this.activeDatabase()) });
                });
                return canActivateResult;
            }
            else {
                return $.Deferred().resolve({ can: true });
            }
        };
        editTransformer.prototype.activate = function (transformerToEditName) {
            var _this = this;
            _super.prototype.activate.call(this, transformerToEditName);
            this.updateHelpLink('S467UO');
            if (transformerToEditName) {
                this.isEditingExistingTransformer(true);
            }
            else {
                this.editedTransformer(transformer.empty());
            }
            this.dirtyFlag = new ko.DirtyFlag([this.editedTransformer().name, this.editedTransformer().transformResults], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return !!_this.editedTransformer().name() && _this.dirtyFlag().isDirty(); });
        };
        editTransformer.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.addTransformerHelpPopover();
            this.createKeyboardShortcut("alt+c", function () { return _this.focusOnEditor(); }, editTransformer.containerSelector);
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteTransformer(); }, editTransformer.containerSelector);
        };
        editTransformer.prototype.addTransformerHelpPopover = function () {
            $("#transformerResultsLabel").popover({
                html: true,
                trigger: "hover",
                content: 'The Transform function allows you to change the shape of individual result documents before the server returns them. It uses C# LINQ query syntax <br/> <br/> Example: <pre> <br/> <span class="code-keyword">from</span> result <span class="code-keyword">in</span> results <br/> <span class="code-keyword">let</span> category = LoadDocument(result.Category) <br/> <span class="code-keyword">select new</span> { <br/>    result.Name, <br/>    result.PricePerUnit, <br/>    Category = category.Name, <br/>    CategoryDescription = category.Description <br/>}</pre>',
            });
        };
        editTransformer.prototype.focusOnEditor = function (elements, data) {
            if (elements === void 0) { elements = null; }
            if (data === void 0) { data = null; }
            var editorElement = $("#transformerAceEditor").length == 1 ? $("#transformerAceEditor")[0] : null;
            if (editorElement) {
                var docEditor = ko.utils.domData.get($("#transformerAceEditor")[0], "aceEditor");
                if (docEditor) {
                    docEditor.focus();
                }
            }
        };
        editTransformer.prototype.editExistingTransformer = function (unescapedTransformerName) {
            var _this = this;
            var transformerName = decodeURIComponent(unescapedTransformerName);
            this.loadedTransformerName(transformerName);
            return this.fetchTransformerToEdit(transformerName)
                .done(function (trans) { return _this.editedTransformer(new transformer().initFromSave(trans)); });
        };
        editTransformer.prototype.fetchTransformerToEdit = function (transformerName) {
            return new getSingleTransformerCommand(transformerName, this.activeDatabase()).execute();
        };
        editTransformer.prototype.saveTransformer = function () {
            var _this = this;
            if (this.isEditingExistingTransformer() && this.editedTransformer().wasNameChanged()) {
                var db = this.activeDatabase();
                var saveTransformerWithNewNameViewModel = new saveTransformerWithNewNameConfirm(this.editedTransformer(), db);
                saveTransformerWithNewNameViewModel.saveTask.done(function (trans) {
                    _this.dirtyFlag().reset(); // Resync Changes
                    _this.updateUrl(_this.editedTransformer().name());
                });
                dialog.show(saveTransformerWithNewNameViewModel);
            }
            else {
                this.editedTransformer().name(this.editedTransformer().name().trim());
                new saveTransformerCommand(this.editedTransformer(), this.activeDatabase())
                    .execute()
                    .done(function () {
                    _this.dirtyFlag().reset();
                    if (!_this.isEditingExistingTransformer()) {
                        _this.isEditingExistingTransformer(true);
                        _this.updateUrl(_this.editedTransformer().name());
                    }
                });
            }
        };
        editTransformer.prototype.updateUrl = function (transformerName) {
            router.navigate(appUrl.forEditTransformer(transformerName, this.activeDatabase()));
        };
        editTransformer.prototype.refreshTransformer = function () {
            var _this = this;
            var canContinue = this.canContinueIfNotDirty("Unsaved Data", "You have unsaved data. Are you sure you want to refresh the transformer from the server?");
            canContinue
                .done(function () {
                var transformerName = _this.loadedTransformerName();
                _this.fetchTransformerToEdit(transformerName)
                    .always(function () { return _this.dirtyFlag().reset(); })
                    .done(function (trans) { return _this.editedTransformer().initFromSave(trans); })
                    .fail(function () {
                    messagePublisher.reportError("Could not find " + transformerName + " transformer");
                    _this.navigate(appUrl.forTransformers(_this.activeDatabase()));
                });
            });
        };
        editTransformer.prototype.formatTransformer = function () {
            var editedTransformer = this.editedTransformer();
            new formatIndexCommand(this.activeDatabase(), [editedTransformer.transformResults()])
                .execute()
                .done(function (result) {
                var formatedTransformer = result[0];
                if (formatedTransformer.indexOf("Could not format:") == -1) {
                    editedTransformer.transformResults(formatedTransformer);
                }
                else {
                    messagePublisher.reportError("Failed to format transformer!", formatedTransformer);
                }
            });
        };
        editTransformer.prototype.deleteTransformer = function () {
            var transformer = this.editedTransformer();
            if (transformer) {
                var db = this.activeDatabase();
                var deleteViewmodel = new deleteTransformerConfirm([transformer.name()], db);
                deleteViewmodel.deleteTask.done(function () {
                    router.navigate(appUrl.forTransformers(db));
                });
                dialog.show(deleteViewmodel);
            }
        };
        editTransformer.containerSelector = "#editTransformerContainer";
        return editTransformer;
    })(viewModelBase);
    return editTransformer;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/saveTransformerWithNewNameConfirm", ["require", "exports", "commands/database/transformers/saveTransformerCommand", "plugins/dialog", "viewmodels/dialogViewModelBase", "common/messagePublisher"], function (require, exports, saveTransformerCommand, dialog, dialogViewModelBase, messagePublisher) {
    var saveTransformerWithNewNameConfirm = (function (_super) {
        __extends(saveTransformerWithNewNameConfirm, _super);
        function saveTransformerWithNewNameConfirm(savedTransformer, db) {
            _super.call(this);
            this.savedTransformer = savedTransformer;
            this.db = db;
            this.saveTask = $.Deferred();
            if (!savedTransformer) {
                throw new Error("Transformer must not be null");
            }
            this.message = "If you wish to save a new transformer with this new name press OK, to cancel the save command press Cancel";
        }
        saveTransformerWithNewNameConfirm.prototype.saveTransformer = function () {
            var _this = this;
            new saveTransformerCommand(this.savedTransformer, this.db).execute().done(function (trans) { return _this.saveTask.resolve(trans); });
            dialog.close(this);
        };
        saveTransformerWithNewNameConfirm.prototype.cancel = function () {
            this.saveTask.reject();
            messagePublisher.reportInfo("Transformer Not Saved!");
            dialog.close(this);
        };
        return saveTransformerWithNewNameConfirm;
    })(dialogViewModelBase);
    return saveTransformerWithNewNameConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/transformers", ["require", "exports", "viewmodels/viewModelBase", "models/database/index/transformer", "commands/database/transformers/getTransformersCommand", "commands/saveTransformerLockModeCommand", "common/appUrl", "viewmodels/database/transformers/deleteTransformerConfirm", "durandal/app", "common/changesContext", "viewmodels/database/transformers/copyTransformerDialog"], function (require, exports, viewModelBase, transformer, getTransformersCommand, saveTransformerLockModeCommand, appUrl, deleteTransformerConfirm, app, changesContext, copyTransformerDialog) {
    var transformers = (function (_super) {
        __extends(transformers, _super);
        function transformers() {
            var _this = this;
            _super.call(this);
            this.newTransformerUrl = appUrl.forCurrentDatabase().newTransformer;
            this.transformersGroups = ko.observableArray();
            this.containerSelector = "#transformersContainer";
            this.transformersMutex = true;
            this.allTransformersExpanded = ko.observable(true);
            this.expandCollapseTitle = ko.computed(function () { return _this.allTransformersExpanded() ? "Collapse all" : "Expand all"; });
            this.appUrls = appUrl.forCurrentDatabase();
        }
        transformers.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                this.fetchTransformers(db).done(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        transformers.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.updateHelpLink('OWRJLV');
            this.createKeyboardShortcut("Alt+N", function () { return _this.navigate(_this.newTransformerUrl()); }, this.containerSelector);
            ko.postbox.publish("SetRawJSONUrl", appUrl.forTransformersRawData(this.activeDatabase()));
        };
        transformers.prototype.fetchTransformers = function (db) {
            var _this = this;
            return new getTransformersCommand(db)
                .execute()
                .done(function (transformers) {
                transformers
                    .map(function (curTransformer) { return new transformer().initFromLoad(curTransformer); })
                    .forEach(function (i) { return _this.putTransformerIntoGroups(i); });
            });
        };
        transformers.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchAllTransformers(function (e) { return _this.processTransformerEvent(e); })];
        };
        transformers.prototype.processTransformerEvent = function (e) {
            var _this = this;
            if (e.Type === "TransformerRemoved") {
                this.removeTransformersFromAllGroups(this.findTransformersByName(e.Name));
            }
            else {
                if (this.transformersMutex) {
                    this.transformersMutex = false;
                    setTimeout(function () {
                        _this.fetchTransformers(_this.activeDatabase()).always(function () { return _this.transformersMutex = true; });
                    }, 5000);
                }
            }
        };
        transformers.prototype.findTransformersByName = function (transformerName) {
            var result = new Array();
            this.transformersGroups().forEach(function (g) {
                g.transformers().forEach(function (i) {
                    if (i.name() === transformerName) {
                        result.push(i);
                    }
                });
            });
            return result;
        };
        transformers.prototype.putTransformerIntoGroups = function (trans) {
            var groupName = trans.name().split("/")[0];
            var group = this.transformersGroups.first(function (g) { return g.entityName === groupName; });
            if (group) {
                var existingTrans = group.transformers.first(function (cur) { return cur.name() === trans.name(); });
                if (!existingTrans) {
                    group.transformers.push(trans);
                }
            }
            else {
                this.transformersGroups.push({ entityName: groupName, transformers: ko.observableArray([trans]) });
            }
        };
        transformers.prototype.toggleExpandAll = function () {
            $(".index-group-content").collapse("toggle");
            this.allTransformersExpanded.toggle();
        };
        transformers.prototype.deleteAllTransformers = function () {
            var allTransformers;
            allTransformers = this.getAllTransformers();
            this.promptDeleteTransformers(allTransformers);
        };
        transformers.prototype.getAllTransformers = function () {
            var all = [];
            this.transformersGroups().forEach(function (g) { return all.pushAll(g.transformers()); });
            return all.distinct();
        };
        transformers.prototype.deleteTransformer = function (transformerToDelete) {
            this.promptDeleteTransformers([transformerToDelete]);
        };
        transformers.prototype.pasteTransformer = function () {
            app.showDialog(new copyTransformerDialog('', this.activeDatabase(), true));
        };
        transformers.prototype.copyTransformer = function (t) {
            app.showDialog(new copyTransformerDialog(t.name(), this.activeDatabase(), false));
        };
        transformers.prototype.promptDeleteTransformers = function (transformers) {
            var _this = this;
            var db = this.activeDatabase();
            var deleteViewmodel = new deleteTransformerConfirm(transformers.map(function (i) { return i.name(); }), db);
            deleteViewmodel.deleteTask.done(function () { return _this.removeTransformersFromAllGroups(transformers); });
            app.showDialog(deleteViewmodel);
        };
        transformers.prototype.removeTransformersFromAllGroups = function (transformers) {
            this.transformersGroups().forEach(function (transGroup) { return transGroup.transformers.removeAll(transformers); });
            this.transformersGroups.remove(function (item) { return item.transformers().length === 0; });
        };
        transformers.prototype.updateTransformerLockMode = function (t) {
            var originalLockMode = t.lockMode();
            var newLockMode = t.isLocked() ? 'Unlock' : 'LockedIgnore';
            if (originalLockMode !== newLockMode) {
                t.lockMode(newLockMode);
                new saveTransformerLockModeCommand(t.name(), newLockMode, this.activeDatabase())
                    .execute()
                    .fail(function () { return t.lockMode(originalLockMode); });
            }
        };
        return transformers;
    })(viewModelBase);
    return transformers;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/database/transformers/transformersShell", ["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl"], function (require, exports, durandalRouter, viewModelBase, appUrl) {
    var transformerShell = (function (_super) {
        __extends(transformerShell, _super);
        function transformerShell() {
            var _this = this;
            _super.call(this);
            this.indexesUrl = appUrl.forCurrentDatabase().indexes;
            this.appUrls = appUrl.forCurrentDatabase();
            this.router = durandalRouter.createChildRouter()
                .map([
                { route: 'databases/transformers', moduleId: 'viewmodels/database/transformers/transformers', title: 'Transformers', nav: true },
                { route: 'databases/transformers/edit(/:transformerName)', moduleId: 'viewmodels/database/transformers/editTransformer', title: 'Edit Transformer', nav: true }
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentBreadcrumbTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                if (activeRoute && activeRoute.title === "Transformers") {
                    return "All";
                }
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return transformerShell;
    })(viewModelBase);
    return transformerShell;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/deleteFilesMatchingQueryConfirm", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/filesystem/deleteFilesMatchingQueryCommand"], function (require, exports, dialog, dialogViewModelBase, deleteFilesMatchingQueryCommand) {
    var deleteFilesMatchingQueryConfirm = (function (_super) {
        __extends(deleteFilesMatchingQueryConfirm, _super);
        function deleteFilesMatchingQueryConfirm(queryText, totalFilesCount, fs) {
            _super.call(this);
            this.queryText = queryText;
            this.totalFilesCount = totalFilesCount;
            this.fs = fs;
            this.deletionStarted = false;
            this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
        }
        deleteFilesMatchingQueryConfirm.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteFilesMatchingQueryConfirm.prototype.deleteFiles = function () {
            var _this = this;
            var deleteCommandTask = new deleteFilesMatchingQueryCommand(this.queryText, this.fs).execute();
            deleteCommandTask.done(function () { return _this.deletionTask.resolve(); });
            deleteCommandTask.fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteFilesMatchingQueryConfirm.prototype.deactivate = function (args) {
            _super.prototype.deactivate.call(this, args);
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteFilesMatchingQueryConfirm;
    })(dialogViewModelBase);
    return deleteFilesMatchingQueryConfirm;
});
/// <reference path="../../../../Scripts/typings/ace/ace.amd.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/configurations/configuration", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "common/changesContext", "ace/ace", "viewmodels/viewModelBase", "commands/filesystem/getConfigurationCommand", "models/filesystem/configurationKey", "common/bindingHelpers/aceEditorBindingHandler", "common/messagePublisher", "commands/filesystem/saveConfigurationCommand", "viewmodels/filesystem/configurations/deleteConfigurationKeys", "viewmodels/filesystem/configurations/createConfigurationKey"], function (require, exports, app, router, appUrl, changesContext, ace, viewModelBase, getConfigurationCommand, configurationKey, aceEditorBindingHandler, messagePublisher, saveConfigurationCommand, deleteConfigurationKeys, createConfigurationKey) {
    var configuration = (function (_super) {
        __extends(configuration, _super);
        function configuration() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.keys = ko.observableArray();
            this.selectedKeyValue = ko.observable();
            this.selectedKey = ko.observable().subscribeTo("ActivateConfigurationKey").distinctUntilChanged();
            this.currentKey = ko.observable();
            this.configurationKeyText = ko.observable('').extend({ required: true });
            this.isBusy = ko.observable(false);
            this.enabled = true;
            aceEditorBindingHandler.install();
            this.selectedKey.subscribe(function (k) { return _this.selectedKeyChanged(k); });
            // When we programmatically change a configuration doc, push it into the editor.
            //this.subscription = this.configurationKeyText.subscribe(() => this.updateConfigurationText());
            this.text = ko.computed({
                read: function () {
                    return _this.configurationKeyText();
                },
                write: function (text) {
                    _this.configurationKeyText(text);
                },
                owner: this
            });
        }
        configuration.prototype.activate = function (navigationArgs) {
            var _this = this;
            _super.prototype.activate.call(this, navigationArgs);
            this.appUrls = appUrl.forCurrentFilesystem();
            this.dirtyFlag = new ko.DirtyFlag([this.configurationKeyText]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
            return this.loadKeys(this.activeFilesystem());
        };
        configuration.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.activeFilesystem.subscribe(function (x) {
                _this.loadKeys(x);
            });
            $('.keys-collection').contextmenu({
                target: '#keys-context-menu'
            });
            this.initializeDocEditor();
            this.configurationEditor.focus();
            this.setupKeyboardShortcuts();
        };
        configuration.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#configurationEditor");
            if (editorElement.length > 0) {
                this.editor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#configurationEditor").on('DynamicHeightSet', function () { return _this.editor.resize(); });
            this.focusOnEditor();
        };
        configuration.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.selectedKey.unsubscribeFrom("ActivateConfigurationKey");
            $("#configurationEditor").off('DynamicHeightSet');
        };
        configuration.prototype.focusOnEditor = function () {
            if (!!this.editor) {
                this.editor.focus();
            }
        };
        configuration.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchFsConfig(function (e) { return _this.processFsConfigNotification(e); })];
        };
        configuration.prototype.processFsConfigNotification = function (e) {
            switch (e.Action) {
                case 0 /* Set */:
                    this.addKey(e.Name);
                    break;
                case 1 /* Delete */:
                    this.removeKey(e.Name);
                    break;
                default:
                    console.error("Unknown notification action.");
            }
        };
        configuration.prototype.selectKeyValue = function (selection) {
            this.selectedKeyValue(selection);
        };
        configuration.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteConfiguration(); }, configuration.configSelector);
        };
        configuration.prototype.initializeDocEditor = function () {
            // Startup the Ace editor with JSON syntax highlighting.
            // TODO: Just use the simple binding handler instead.
            this.configurationEditor = ace.edit("configurationEditor");
            this.configurationEditor.setTheme("ace/theme/xcode");
            this.configurationEditor.setFontSize("16px");
            this.configurationEditor.getSession().setMode("ace/mode/json");
        };
        configuration.prototype.updateConfigurationText = function () {
            if (this.configurationEditor) {
                this.configurationEditor.getSession().setValue(this.configurationKeyText());
            }
        };
        configuration.prototype.loadKeys = function (fs) {
            var _this = this;
            if (this.enabled) {
                return new getConfigurationCommand(fs)
                    .execute()
                    .done(function (x) {
                    _this.keys(x);
                    if (x.length > 0) {
                        _this.selectKey(x[0]);
                    }
                    else {
                        _this.enableEditor(false);
                    }
                });
            }
            return $.Deferred().resolve();
        };
        configuration.prototype.selectKey = function (key) {
            key.activate();
        };
        configuration.prototype.enableEditor = function (enable) {
            if (this.configurationEditor) {
                this.configurationEditor.setReadOnly(!enable);
                this.configurationEditor.getSession().setUseWorker(enable);
                if (!enable) {
                    this.configurationKeyText("");
                    this.dirtyFlag().reset();
                }
            }
            this.enabled = enable;
        };
        configuration.prototype.selectedKeyChanged = function (selected) {
            var _this = this;
            if (selected) {
                this.isBusy(true);
                selected.getValues().done(function (data) {
                    _this.configurationKeyText(data);
                }).always(function () {
                    _this.dirtyFlag().reset();
                    _this.isBusy(false);
                });
                this.currentKey(selected);
                this.focusOnEditor();
            }
        };
        configuration.prototype.save = function () {
            var _this = this;
            var message = "";
            try {
                var jsonConfigDoc = JSON.parse(this.configurationKeyText());
            }
            catch (e) {
                if (jsonConfigDoc == undefined) {
                    message = "The configuration key data isn't a legal JSON expression!";
                }
                this.focusOnEditor();
            }
            if (message != "") {
                messagePublisher.reportError(message, undefined, undefined, false);
                return;
            }
            var saveCommand = new saveConfigurationCommand(this.activeFilesystem(), this.currentKey(), jsonConfigDoc);
            var saveTask = saveCommand.execute();
            saveTask.done(function () { return _this.dirtyFlag().reset(); });
        };
        configuration.prototype.refreshConfig = function () {
            this.selectedKeyChanged(this.currentKey());
        };
        configuration.prototype.deleteConfiguration = function () {
            var _this = this;
            var deleteConfigurationKeyViewModel = new deleteConfigurationKeys(this.activeFilesystem(), [this.currentKey()]);
            deleteConfigurationKeyViewModel
                .deletionTask
                .done(function () {
                _this.removeKey(_this.currentKey().key);
            });
            app.showDialog(deleteConfigurationKeyViewModel);
        };
        configuration.prototype.removeKey = function (key) {
            var foundKey = this.keys().filter(function (configKey) { return configKey.key == key; });
            if (foundKey.length > 0) {
                var currentIndex = this.keys.indexOf(this.currentKey());
                var foundIndex = this.keys.indexOf(foundKey[0]);
                var newIndex = currentIndex;
                if (currentIndex + 1 == this.keys().length) {
                    newIndex = currentIndex - 1;
                }
                this.keys.remove(foundKey[0]);
                if (this.keys()[newIndex] && currentIndex == foundIndex) {
                    this.selectKey(this.keys()[newIndex]);
                }
                else {
                    this.enableEditor(false);
                }
            }
        };
        configuration.prototype.addKey = function (key) {
            var foundKey = this.keys.first(function (configKey) { return configKey.key == key; });
            if (!foundKey) {
                var newKey = new configurationKey(this.activeFilesystem(), key);
                this.keys.push(newKey);
                if (this.keys().length > 0 && !this.enabled) {
                    this.enableEditor(true);
                }
                return newKey;
            }
            return foundKey;
        };
        configuration.prototype.newConfigurationKey = function () {
            var _this = this;
            var createConfigurationKeyViewModel = new createConfigurationKey(this.keys());
            createConfigurationKeyViewModel
                .creationTask
                .done(function (key) {
                new saveConfigurationCommand(_this.activeFilesystem(), new configurationKey(_this.activeFilesystem(), key), JSON.parse("{}")).execute()
                    .done(function () {
                    var newKey = _this.addKey(key);
                    _this.selectKey(newKey);
                })
                    .fail(function (qXHR, textStatus, errorThrown) { return messagePublisher.reportError("Could not create Configuration Key!", errorThrown); });
            });
            app.showDialog(createConfigurationKeyViewModel);
        };
        configuration.configSelector = "#settingsContainer";
        return configuration;
    })(viewModelBase);
    return configuration;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/configurations/createConfigurationKey", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog", "common/messagePublisher"], function (require, exports, dialogViewModelBase, dialog, messagePublisher) {
    var createConfigurationKey = (function (_super) {
        __extends(createConfigurationKey, _super);
        function createConfigurationKey(keys) {
            _super.call(this);
            this.creationTask = $.Deferred();
            this.creationTaskStarted = false;
            this.configurationKeyName = ko.observable('');
            this.configurationKeyNameFocus = ko.observable(true);
            this.keys = keys;
        }
        createConfigurationKey.prototype.cancel = function () {
            dialog.close(this);
        };
        createConfigurationKey.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.configurationKeyNameFocus(true);
        };
        createConfigurationKey.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.creationTaskStarted) {
                this.creationTask.reject();
            }
        };
        createConfigurationKey.prototype.create = function () {
            // For now we're just creating the filesystem.
            var configKeyName = this.configurationKeyName();
            if (this.isClientSideInputOk(configKeyName)) {
                this.creationTaskStarted = true;
                this.creationTask.resolve(configKeyName);
                dialog.close(this);
            }
        };
        createConfigurationKey.prototype.isClientSideInputOk = function (keyName) {
            var errorMessage = "";
            if (keyName == null) {
                errorMessage = "Please fill out the Configuration Key name field";
            }
            else if (this.isConfigurationKeyExists(keyName, this.keys)) {
                errorMessage = "Configuration Key Already Exists!";
            }
            else if ((errorMessage = this.checkInput(keyName)) != null) { }
            if (errorMessage != null) {
                messagePublisher.reportError(errorMessage);
                this.configurationKeyNameFocus(true);
                return false;
            }
            return true;
        };
        createConfigurationKey.prototype.checkInput = function (name) {
            var message = null;
            //not implemented for the moment
            return message;
        };
        createConfigurationKey.prototype.isConfigurationKeyExists = function (keyName, configurationKeys) {
            for (var i = 0; i < configurationKeys.length; i++) {
                if (keyName == configurationKeys[i].key) {
                    return true;
                }
            }
            return false;
        };
        return createConfigurationKey;
    })(dialogViewModelBase);
    return createConfigurationKey;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/configurations/deleteConfigurationKeys", ["require", "exports", "plugins/dialog", "commands/filesystem/deleteConfigurationKeyCommand", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteConfigurationKeyCommand, dialogViewModelBase) {
    var deleteConfigurationKeys = (function (_super) {
        __extends(deleteConfigurationKeys, _super);
        function deleteConfigurationKeys(fs, keys, elementToFocusOnDismissal) {
            _super.call(this, elementToFocusOnDismissal);
            this.fs = fs;
            this.keys = ko.observableArray();
            this.deletionStarted = false;
            this.deletionTask = $.Deferred(); // Gives consumers a way to know when the async delete operation completes.
            if (keys.length === 0) {
                throw new Error("Must have at least one key to delete.");
            }
            this.keys(keys);
        }
        deleteConfigurationKeys.prototype.deleteKeys = function () {
            var _this = this;
            this.deletionStarted = true;
            var deleteItemsIds = this.keys().map(function (i) { return i.key; });
            var deletionTasks = [];
            for (var i = 0; i < deleteItemsIds.length; i++) {
                deletionTasks.push(new deleteConfigurationKeyCommand(this.fs, deleteItemsIds[i]).execute());
            }
            var combinedTask = $.when.apply($, deletionTasks);
            combinedTask
                .done(function () { return _this.deletionTask.resolve(_this.keys()); })
                .fail(function (response) { return _this.deletionTask.reject(response); });
            dialog.close(this);
        };
        deleteConfigurationKeys.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteConfigurationKeys.prototype.deactivate = function (args) {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteConfigurationKeys;
    })(dialogViewModelBase);
    return deleteConfigurationKeys;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/createFolderInFilesystem", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog", "common/messagePublisher"], function (require, exports, dialogViewModelBase, dialog, messagePublisher) {
    var createFolderInFilesystem = (function (_super) {
        __extends(createFolderInFilesystem, _super);
        function createFolderInFilesystem(folders) {
            _super.call(this);
            this.creationTask = $.Deferred();
            this.creationTaskStarted = false;
            this.folderName = ko.observable('');
            this.folders = folders;
        }
        createFolderInFilesystem.prototype.cancel = function () {
            dialog.close(this);
        };
        createFolderInFilesystem.prototype.attached = function () {
            _super.prototype.attached.call(this);
            //this.folderName(true);
        };
        createFolderInFilesystem.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.creationTaskStarted) {
                this.creationTask.reject();
            }
        };
        createFolderInFilesystem.prototype.create = function () {
            var folderName = this.folderName();
            if (this.isClientSideInputOK(folderName)) {
                this.creationTaskStarted = true;
                this.creationTask.resolve(folderName.toLowerCase());
                dialog.close(this);
            }
        };
        createFolderInFilesystem.prototype.isClientSideInputOK = function (folderName) {
            var errorMessage;
            if (folderName == null) {
                errorMessage = "Please fill in the folder Name";
            }
            else if (this.folderExists(folderName, this.folders) === true) {
                errorMessage = "Folder already exists!";
            }
            if (errorMessage != null) {
                messagePublisher.reportError(errorMessage);
                return false;
            }
            return true;
        };
        createFolderInFilesystem.prototype.folderExists = function (folderName, folders) {
            for (var i = 0; i < folders.length; i++) {
                if (folderName.toLowerCase() == folders[i]) {
                    return true;
                }
            }
            return false;
        };
        createFolderInFilesystem.prototype.enterKeyPressed = function () {
            var submit = this.folderName != null;
            if (this.folderName())
                submit = false;
            submit = this.folderName() && this.folderName().trim() != "";
            if (submit) {
                _super.prototype.enterKeyPressed.call(this);
            }
            return submit;
        };
        return createFolderInFilesystem;
    })(dialogViewModelBase);
    return createFolderInFilesystem;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/fileRenameDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/filesystem/renameFileCommand"], function (require, exports, dialog, dialogViewModelBase, renameFileCommand) {
    var fileRenameDialog = (function (_super) {
        __extends(fileRenameDialog, _super);
        function fileRenameDialog(oldFilename, fs) {
            _super.call(this);
            this.oldFilename = oldFilename;
            this.fs = fs;
            this.newName = ko.observable();
            this.nextTask = $.Deferred();
            this.nextTaskStarted = false;
            this.newName(oldFilename);
        }
        fileRenameDialog.prototype.cancel = function () {
            this.nextTask.reject();
            dialog.close(this);
        };
        fileRenameDialog.prototype.rename = function () {
            var _this = this;
            this.nextTaskStarted = true;
            new renameFileCommand(this.fs, this.oldFilename, this.newName()).execute()
                .done(function () { return _this.nextTask.resolve(_this.newName()); })
                .fail(function () { return _this.nextTask.reject(); });
            dialog.close(this);
        };
        fileRenameDialog.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };
        fileRenameDialog.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        return fileRenameDialog;
    })(dialogViewModelBase);
    return fileRenameDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/filesystemEditFile", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "commands/filesystem/getFileCommand", "commands/filesystem/updateFileMetadataCommand", "viewmodels/viewModelBase", "viewmodels/common/deleteItems", "viewmodels/filesystem/files/fileRenameDialog", "common/bindingHelpers/aceEditorBindingHandler"], function (require, exports, app, router, appUrl, getFileCommand, updateFileMetadataCommand, viewModelBase, deleteItems, fileRenameDialog, aceEditorBindingHandler) {
    var filesystemEditFile = (function (_super) {
        __extends(filesystemEditFile, _super);
        function filesystemEditFile() {
            var _this = this;
            _super.call(this);
            this.fileName = ko.observable();
            this.file = ko.observable();
            this.topRecentFiles = ko.computed(function () { return _this.getTopRecentFiles(); });
            this.fileMetadataText = ko.observable();
            this.isBusy = ko.observable(false);
            this.metaPropsToRestoreOnSave = [];
            aceEditorBindingHandler.install();
            this.fileName.subscribe(function (x) { return _this.loadFile(x); });
        }
        filesystemEditFile.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.metadata = ko.computed(function () { return _this.file() ? _this.file().__metadata : null; });
            this.filesystemForEditedFile = appUrl.getFileSystem();
            if (args.id != null) {
                this.appendRecentFile(args.id);
                this.fileName(args.id);
            }
            this.metadata.subscribe(function (meta) { return _this.metadataChanged(meta); });
        };
        filesystemEditFile.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.setupKeyboardShortcuts();
        };
        filesystemEditFile.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            var editorElement = $("#fileMetadataEditor");
            if (editorElement.length > 0) {
                this.metadataEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            this.focusOnEditor();
        };
        filesystemEditFile.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.createKeyboardShortcut("alt+shift+del", function () { return _this.deleteFile(); }, filesystemEditFile.editFileSelector);
        };
        filesystemEditFile.prototype.focusOnEditor = function () {
            this.metadataEditor.focus();
        };
        filesystemEditFile.prototype.loadFile = function (fileName) {
            var _this = this;
            new getFileCommand(this.activeFilesystem(), fileName)
                .execute()
                .done(function (result) { return _this.file(result); });
        };
        filesystemEditFile.prototype.navigateToFiles = function () {
            var filesUrl = appUrl.forFilesystemFiles(this.activeFilesystem());
            router.navigate(filesUrl);
        };
        filesystemEditFile.prototype.saveFileMetadata = function () {
            var _this = this;
            //the name of the document was changed and we have to save it as a new one
            var meta = JSON.parse(this.fileMetadataText());
            var currentDocumentId = this.fileName();
            this.metaPropsToRestoreOnSave.forEach(function (p) { return meta[p.name] = p.value; });
            var saveCommand = new updateFileMetadataCommand(this.fileName(), meta, this.activeFilesystem(), true);
            var saveTask = saveCommand.execute();
            saveTask.done(function () {
                _this.dirtyFlag().reset(); // Resync Changes
                _this.loadFile(_this.fileName());
            });
        };
        filesystemEditFile.prototype.downloadFile = function () {
            var fs = this.activeFilesystem();
            var fileName = this.fileName();
            var url = appUrl.forResourceQuery(fs) + "/files/" + encodeURIComponent(fileName);
            this.downloader.download(fs, url);
        };
        filesystemEditFile.prototype.refreshFile = function () {
            this.loadFile(this.fileName());
        };
        filesystemEditFile.prototype.deleteFile = function () {
            var _this = this;
            var file = this.file();
            if (file) {
                var viewModel = new deleteItems([file]);
                viewModel.deletionTask.done(function () {
                    var filesUrl = appUrl.forFilesystemFiles(_this.activeFilesystem());
                    router.navigate(filesUrl);
                });
                app.showDialog(viewModel, filesystemEditFile.editFileSelector);
            }
            this.dirtyFlag().reset(); // Resync Changes
        };
        filesystemEditFile.prototype.renameFile = function () {
            var _this = this;
            var currentFileName = this.fileName();
            var dialog = new fileRenameDialog(currentFileName, this.activeFilesystem());
            dialog.onExit().done(function (newName) {
                _this.removeFromTopRecentFiles(currentFileName);
                router.navigate(appUrl.forEditFile(newName, _this.activeFilesystem()));
            });
            app.showDialog(dialog);
        };
        filesystemEditFile.prototype.removeFromTopRecentFiles = function (fileName) {
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().first(function (x) { return x.filesystemName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                recentFilesForCurFilesystem.recentFiles.remove(fileName);
            }
        };
        filesystemEditFile.prototype.getTopRecentFiles = function () {
            var _this = this;
            var currentFilesystemName = this.activeFilesystem().name;
            var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().first(function (x) { return x.filesystemName === currentFilesystemName; });
            if (recentFilesForCurFilesystem) {
                var value = recentFilesForCurFilesystem
                    .recentFiles()
                    .filter(function (x) {
                    return x !== _this.fileName();
                })
                    .slice(0, 5)
                    .map(function (fileId) {
                    return {
                        fileId: fileId,
                        fileUrl: appUrl.forEditFile(fileId, _this.activeFilesystem())
                    };
                });
                return value;
            }
            else {
                return [];
            }
        };
        filesystemEditFile.prototype.metadataChanged = function (meta) {
            if (meta) {
                //this.metaPropsToRestoreOnSave.length = 0;
                var metaDto = this.metadata().toDto();
                // We don't want to show certain reserved properties in the metadata text area.
                // Remove them from the DTO, restore them on save.
                var metaPropsToRemove = ["Raven-Last-Modified", "Raven-Creation-Date", "Last-Modified", "Creation-Date", "ETag", "RavenFS-Size"];
                for (var property in metaDto) {
                    if (metaDto.hasOwnProperty(property) && metaPropsToRemove.contains(property)) {
                        var value = metaDto[property];
                        if (typeof (value) != "string" && typeof (value) != "number") {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: JSON.stringify(value) });
                        }
                        else {
                            this.metaPropsToRestoreOnSave.push({ name: property, value: metaDto[property].toString() });
                        }
                        delete metaDto[property];
                    }
                }
                var metaString = this.stringify(metaDto);
                this.fileMetadataText(metaString);
            }
        };
        filesystemEditFile.prototype.appendRecentFile = function (fileId) {
            var _this = this;
            var existingRecentFilesStore = filesystemEditFile.recentDocumentsInFilesystem.first(function (x) { return x.filesystemName == _this.filesystemForEditedFile.name; });
            if (existingRecentFilesStore) {
                var existingDocumentInStore = existingRecentFilesStore.recentFiles.first(function (x) { return x === fileId; });
                if (!existingDocumentInStore) {
                    if (existingRecentFilesStore.recentFiles().length == 5) {
                        existingRecentFilesStore.recentFiles.pop();
                    }
                    existingRecentFilesStore.recentFiles.unshift(fileId);
                }
            }
            else {
                filesystemEditFile.recentDocumentsInFilesystem.push({ filesystemName: this.filesystemForEditedFile.name, recentFiles: ko.observableArray([fileId]) });
            }
        };
        filesystemEditFile.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };
        filesystemEditFile.editFileSelector = "#editFileContainer";
        filesystemEditFile.recentDocumentsInFilesystem = ko.observableArray();
        return filesystemEditFile;
    })(viewModelBase);
    return filesystemEditFile;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/filesystemFiles", ["require", "exports", "plugins/router", "common/appUrl", "durandal/app", "common/changesContext", "common/pagedList", "commands/filesystem/getFilesCommand", "commands/filesystem/getFilesystemRevisionsCommand", "viewmodels/filesystem/files/createFolderInFilesystem", "common/bindingHelpers/treeBindingHandler", "viewmodels/viewModelBase", "models/filesystem/folder", "common/bindingHelpers/fileUploadBindingHandler", "common/uploadQueueHelper", "viewmodels/filesystem/deleteFilesMatchingQueryConfirm", "commands/filesystem/searchByQueryCommand", "commands/filesystem/getFileSystemStatsCommand", "viewmodels/filesystem/files/filesystemEditFile", "viewmodels/filesystem/files/fileRenameDialog", "common/queryUtil"], function (require, exports, router, appUrl, app, changesContext, pagedList, getFilesystemFilesCommand, getFilesystemRevisionsCommand, createFolderInFilesystem, treeBindingHandler, viewModelBase, folder, fileUploadBindingHandler, uploadQueueHelper, deleteFilesMatchingQueryConfirm, searchByQueryCommand, getFileSystemStatsCommand, filesystemEditFile, fileRenameDialog, queryUtil) {
    var filesystemFiles = (function (_super) {
        __extends(filesystemFiles, _super);
        function filesystemFiles() {
            var _this = this;
            _super.call(this);
            this.allFilesPagedItems = ko.observable();
            this.selectedFilesIndices = ko.observableArray();
            this.selectedFolder = ko.observable();
            this.addedFolder = ko.observable();
            this.currentLevelSubdirectories = ko.observableArray();
            this.uploadFiles = ko.observable();
            this.uploadQueue = ko.observableArray();
            this.folderNotificationSubscriptions = {};
            this.isAnyFilesAutoSelected = ko.observable(false);
            this.isAllFilesAutoSelected = ko.observable(false);
            this.inRevisionsFolder = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.uploadQueue.subscribe(function (x) { return _this.newUpload(x); });
            fileUploadBindingHandler.install();
            treeBindingHandler.install();
            treeBindingHandler.includeRevisionsFunc = function () { return _this.activeFilesystem().activeBundles.contains("Versioning"); };
            this.selectedFilesText = ko.computed(function () {
                if (!!_this.selectedFilesIndices()) {
                    var documentsText = "file";
                    if (_this.selectedFilesIndices().length !== 1) {
                        documentsText += "s";
                    }
                    return documentsText;
                }
                return "";
            });
            this.filesCount = ko.computed(function () {
                if (!!_this.allFilesPagedItems()) {
                    var p = _this.allFilesPagedItems();
                    return p.totalResultCount();
                }
                return 0;
            });
            this.selectedFolderName = ko.computed(function () {
                if (!_this.selectedFolder())
                    return "Root Folder";
                var splittedName = _this.selectedFolder().split("/");
                return splittedName.last();
            });
            this.hasFiles = ko.computed(function () {
                if (!!_this.allFilesPagedItems()) {
                    var p = _this.allFilesPagedItems();
                    return p.totalResultCount() > 0;
                }
                return false;
            });
            this.hasAllFilesSelected = ko.computed(function () {
                var filesCount = _this.filesCount();
                return filesCount > 0 && filesCount === _this.selectedFilesIndices().length;
            });
            this.filesSelection = ko.computed(function () {
                var selected = _this.selectedFilesIndices();
                if (_this.hasAllFilesSelected()) {
                    return checkbox.Checked;
                }
                if (selected.length > 0) {
                    return checkbox.SomeChecked;
                }
                return checkbox.UnChecked;
            });
        }
        filesystemFiles.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.appUrls = appUrl.forCurrentFilesystem();
            this.hasAnyFilesSelected = ko.computed(function () { return _this.selectedFilesIndices().length > 0; });
            this.loadFiles();
            this.selectedFolder.subscribe(function (newValue) { return _this.folderChanged(newValue); });
            var storageKeyForFs = uploadQueueHelper.localStorageUploadQueueKey + this.activeFilesystem().name;
            if (window.localStorage.getItem(storageKeyForFs)) {
                this.uploadQueue(uploadQueueHelper.parseUploadQueue(window.localStorage.getItem(storageKeyForFs), this.activeFilesystem()));
            }
        };
        filesystemFiles.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.collapseUploadQueuePanel();
        };
        filesystemFiles.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        filesystemFiles.prototype.loadFiles = function () {
            this.allFilesPagedItems(this.createPagedList(this.selectedFolder()));
        };
        filesystemFiles.prototype.folderChanged = function (newFolder) {
            var _this = this;
            this.loadFiles();
            this.inRevisionsFolder(newFolder === filesystemFiles.revisionsFolderId);
            // treat notifications events
            if (!newFolder) {
                newFolder = "/";
            }
            if (!this.folderNotificationSubscriptions[newFolder] && changesContext.currentResourceChangesApi() != null) {
                this.folderNotificationSubscriptions[newFolder] = changesContext.currentResourceChangesApi()
                    .watchFsFolders(newFolder, function (e) {
                    var callbackFolder = new folder(newFolder);
                    if (!callbackFolder)
                        return;
                    switch (e.Action) {
                        case "Add":
                            {
                                var eventFolder = folder.getFolderFromFilePath(e.File);
                                if (!eventFolder || !treeBindingHandler.isNodeExpanded(filesystemFiles.treeSelector, callbackFolder.path)) {
                                    return;
                                }
                                //check if the file is new at the folder level to add it
                                if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                    _this.loadFiles();
                                }
                                else {
                                    //check if a new folder at this level was added so we add it to the tree
                                    var subPaths = eventFolder.getSubpathsFrom(callbackFolder.path);
                                    if (subPaths.length > 1 && !treeBindingHandler.nodeExists(filesystemFiles.treeSelector, subPaths[1].path)) {
                                        var newNode = {
                                            key: subPaths[1].path,
                                            title: subPaths[1].name,
                                            isLazy: true,
                                            isFolder: true,
                                        };
                                        _this.addedFolder(newNode);
                                    }
                                }
                                break;
                            }
                        case "Delete":
                            {
                                var eventFolder = folder.getFolderFromFilePath(e.File);
                                //check if the file is new at the folder level to remove it from the table
                                if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                    _this.loadFiles();
                                }
                                else {
                                    //reload node and its children
                                    treeBindingHandler.reloadNode(filesystemFiles.treeSelector, callbackFolder.path);
                                }
                                break;
                            }
                        case "Renaming":
                            {
                            }
                        case "Renamed":
                            {
                                //reload files to load the new names
                                if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                    _this.loadFiles();
                                }
                                break;
                            }
                        case "Update":
                            {
                                //check if the file is new at the folder level to add it
                                if (callbackFolder.isFileAtFolderLevel(e.File)) {
                                    _this.loadFiles();
                                }
                                break;
                            }
                        default:
                            console.error("unknown notification action");
                    }
                });
            }
        };
        filesystemFiles.prototype.createPagedList = function (directory) {
            var _this = this;
            var fetcher;
            if (directory === filesystemFiles.revisionsFolderId) {
                fetcher = function (skip, take) { return _this.fetchRevisions(skip, take); };
            }
            else {
                fetcher = function (skip, take) { return _this.fetchFiles(directory, skip, take); };
            }
            var list = new pagedList(fetcher);
            return list;
        };
        filesystemFiles.prototype.fetchFiles = function (directory, skip, take) {
            var task = new getFilesystemFilesCommand(appUrl.getFileSystem(), directory, skip, take).execute();
            return task;
        };
        filesystemFiles.prototype.fetchRevisions = function (skip, take) {
            var task = new getFilesystemRevisionsCommand(appUrl.getFileSystem(), skip, take).execute();
            return task;
        };
        filesystemFiles.prototype.fileSystemChanged = function (fs) {
            if (!!fs) {
                this.loadFiles();
            }
        };
        filesystemFiles.prototype.editSelectedFile = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };
        filesystemFiles.prototype.toggleSelectAll = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid) {
                if (this.hasAnyFilesSelected()) {
                    filesGrid.selectNone();
                }
                else {
                    filesGrid.selectSome();
                    this.isAnyFilesAutoSelected(this.hasAllFilesSelected() === false);
                }
            }
        };
        filesystemFiles.prototype.selectAll = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid && !!this.allFilesPagedItems()) {
                var p = this.allFilesPagedItems();
                filesGrid.selectAll(p.totalResultCount());
            }
        };
        filesystemFiles.prototype.selectNone = function () {
            var filesGrid = this.getFilesGrid();
            if (!!filesGrid) {
                filesGrid.selectNone();
            }
        };
        filesystemFiles.prototype.getFilesGrid = function () {
            var gridContents = $(filesystemFiles.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        filesystemFiles.prototype.createFolder = function () {
            var _this = this;
            var createFolderVm = new createFolderInFilesystem(this.currentLevelSubdirectories());
            createFolderVm.creationTask.done(function (folderName) {
                var parentDirectory = _this.selectedFolder() ? _this.selectedFolder() : "";
                var newNode = {
                    key: parentDirectory + "/" + folderName,
                    title: folderName,
                    isLazy: true,
                    isFolder: true,
                    addClass: treeBindingHandler.transientNodeStyle
                };
                _this.addedFolder(newNode);
            });
            app.showDialog(createFolderVm);
        };
        filesystemFiles.prototype.refresh = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                grid.refreshCollectionData();
            }
            this.selectNone();
        };
        filesystemFiles.prototype.deleteSelectedFiles = function () {
            if (this.hasAllFilesSelected()) {
                this.deleteFolder(false);
            }
            else {
                var grid = this.getFilesGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        filesystemFiles.prototype.downloadSelectedFiles = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1).first();
                var selectedFolder = this.selectedFolder();
                if (selectedFolder == null)
                    selectedFolder = "";
                var fs = this.activeFilesystem();
                var fileName = selectedItem.getId();
                var url = appUrl.forResourceQuery(fs) + "/files" + selectedFolder + "/" + encodeURIComponent(fileName);
                this.downloader.download(fs, url);
            }
        };
        filesystemFiles.prototype.renameSelectedFile = function () {
            var _this = this;
            var grid = this.getFilesGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1).first();
                var currentFileName = selectedItem.getId();
                var dialog = new fileRenameDialog(currentFileName, this.activeFilesystem());
                dialog.onExit().done(function (newName) {
                    var currentFilesystemName = _this.activeFilesystem().name;
                    var recentFilesForCurFilesystem = filesystemEditFile.recentDocumentsInFilesystem().first(function (x) { return x.filesystemName === currentFilesystemName; });
                    if (recentFilesForCurFilesystem) {
                        recentFilesForCurFilesystem.recentFiles.remove(currentFileName);
                    }
                    selectedItem.setId(newName);
                    grid.refreshCollectionData();
                });
                app.showDialog(dialog);
            }
        };
        filesystemFiles.prototype.clearUploadQueue = function () {
            window.localStorage.removeItem(uploadQueueHelper.localStorageUploadQueueKey + this.activeFilesystem().name);
            this.uploadQueue.removeAll();
        };
        filesystemFiles.prototype.navigateToFiles = function () {
            router.navigate(appUrl.forFilesystemFiles(this.activeFilesystem()));
        };
        filesystemFiles.prototype.uploadSuccess = function (x) {
            ko.postbox.publish("UploadFileStatusChanged", x);
            uploadQueueHelper.updateQueueStatus(x.id(), uploadQueueHelper.uploadedStatus, this.uploadQueue());
            this.uploadQueue(uploadQueueHelper.sortUploadQueue(this.uploadQueue()));
            var persistedFolder = folder.getFolderFromFilePath(x.fileName());
            if (persistedFolder) {
                treeBindingHandler.updateNodeHierarchyStyle(filesystemFiles.treeSelector, persistedFolder.path, "");
                treeBindingHandler.setNodeLoadStatus(filesystemFiles.treeSelector, persistedFolder.path, 0);
            }
            this.loadFiles();
        };
        filesystemFiles.prototype.uploadFailed = function (x) {
            ko.postbox.publish("UploadFileStatusChanged", x);
            uploadQueueHelper.updateQueueStatus(x.id(), uploadQueueHelper.failedStatus, this.uploadQueue());
            this.uploadQueue(uploadQueueHelper.sortUploadQueue(this.uploadQueue()));
        };
        filesystemFiles.prototype.newUpload = function (x) {
            if (x && x.length > 0) {
                uploadQueueHelper.updateLocalStorage(x, this.activeFilesystem());
                var waitingItems = x.filter(function (x) { return x.status() === uploadQueueHelper.queuedStatus || x.status() === uploadQueueHelper.uploadingStatus; });
                for (var i = 0; i < waitingItems.length; i++) {
                    var persistedFolder = folder.getFolderFromFilePath(waitingItems[i].fileName());
                    treeBindingHandler.setNodeLoadStatus(filesystemFiles.treeSelector, persistedFolder.path, 1);
                }
            }
        };
        filesystemFiles.prototype.toggleCollapseUploadQueue = function () {
            if ($(filesystemFiles.uploadQueuePanelToggleSelector).hasClass('opened')) {
                this.collapseUploadQueuePanel();
            }
            else {
                this.expandUploadQueuePanel();
            }
        };
        filesystemFiles.prototype.expandUploadQueuePanel = function () {
            $(filesystemFiles.uploadQueuePanelToggleSelector).addClass("opened").find('i').removeClass('fa-angle-double-up').addClass('fa-angle-double-down');
            $(filesystemFiles.uploadQueueSelector).removeClass("hidden");
            $(filesystemFiles.uploadQueuePanelCollapsedSelector).addClass("hidden");
            $(".upload-queue").removeClass("upload-queue-min");
        };
        filesystemFiles.prototype.collapseUploadQueuePanel = function () {
            $(filesystemFiles.uploadQueuePanelToggleSelector).removeClass('opened').find('i').removeClass('fa-angle-double-down').addClass('fa-angle-double-up');
            $(filesystemFiles.uploadQueueSelector).addClass("hidden");
            $(filesystemFiles.uploadQueuePanelCollapsedSelector).removeClass("hidden");
            $(".upload-queue").addClass("upload-queue-min");
        };
        filesystemFiles.prototype.deleteFolder = function (recursive) {
            var _this = this;
            if (recursive === void 0) { recursive = true; }
            if (!this.selectedFolder() && recursive) {
                // delete all files from filesystem
                new getFileSystemStatsCommand(this.activeFilesystem())
                    .execute()
                    .done(function (fs) {
                    _this.promptDeleteFilesMatchingQuery(fs.FileCount, "");
                });
            }
            else {
                // Run the query so that we have an idea of what we'll be deleting.
                var query;
                if (recursive) {
                    query = "__directoryName:" + queryUtil.escapeTerm(this.selectedFolder());
                }
                else {
                    var folder = !this.selectedFolder() ? "/" : this.selectedFolder();
                    query = "__directory:" + queryUtil.escapeTerm(folder);
                }
                new searchByQueryCommand(this.activeFilesystem(), query, 0, 1)
                    .execute()
                    .done(function (results) {
                    if (results.totalResultCount === 0) {
                        app.showMessage("There are no files matching your query.", "Nothing to do");
                    }
                    else {
                        _this.promptDeleteFilesMatchingQuery(results.totalResultCount, query);
                    }
                });
            }
        };
        filesystemFiles.prototype.promptDeleteFilesMatchingQuery = function (resultCount, query) {
            var _this = this;
            var viewModel = new deleteFilesMatchingQueryConfirm(query, resultCount, this.activeFilesystem());
            app.showDialog(viewModel);
            viewModel.deletionTask.done(function () { return _this.loadFiles(); });
        };
        filesystemFiles.revisionsFolderId = "/$$revisions$$";
        filesystemFiles.treeSelector = "#filesTree";
        filesystemFiles.gridSelector = "#filesGrid";
        filesystemFiles.uploadQueuePanelToggleSelector = "#uploadQueuePanelToggle";
        filesystemFiles.uploadQueueSelector = "#uploadQueue";
        filesystemFiles.uploadQueuePanelCollapsedSelector = "#uploadQueuePanelCollapsed";
        return filesystemFiles;
    })(viewModelBase);
    return filesystemFiles;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/searchDialogViewModel", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog"], function (require, exports, dialogViewModelBase, dialog) {
    /*
     * Base view model for search dialogs.
     * Extends from dialogViewModelBase
     * Provides some extra functionality:
     *    - Listens to Enter key and submits the form only if all the fields have been completed
     */
    var searchDialogViewModel = (function (_super) {
        __extends(searchDialogViewModel, _super);
        function searchDialogViewModel(inputs) {
            _super.call(this);
            this.inputs = inputs;
        }
        searchDialogViewModel.prototype.close = function () {
            dialog.close(this);
        };
        searchDialogViewModel.prototype.enterKeyPressed = function () {
            if (this.inputs == null)
                return false;
            var submit = this.enabled();
            if (submit) {
                _super.prototype.enterKeyPressed.call(this);
            }
            return submit;
        };
        searchDialogViewModel.prototype.enabled = function () {
            return this.checkRequired(true);
        };
        // check inputs 
        searchDialogViewModel.prototype.checkRequired = function (allRequired) {
            if (allRequired) {
                var result = true;
                this.inputs.forEach(function (input) {
                    if (input() == null || !input().trim()) {
                        result = false;
                    }
                });
                return result;
            }
            else {
                var result = false;
                this.inputs.forEach(function (input) {
                    if (input() != null && input().trim()) {
                        result = true;
                    }
                });
                return result;
            }
        };
        searchDialogViewModel.dialogSelector = ".messageBox";
        return searchDialogViewModel;
    })(dialogViewModelBase);
    return searchDialogViewModel;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/searchFileSizeRangeClause", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel"], function (require, exports, searchDialogViewModel) {
    var searchFileSizeRangeClause = (function (_super) {
        __extends(searchFileSizeRangeClause, _super);
        function searchFileSizeRangeClause() {
            _super.call(this, [ko.observable(""), ko.observable("")]);
            this.applyFilterTask = $.Deferred();
        }
        searchFileSizeRangeClause.prototype.applyFilter = function () {
            var filter = "__size_numeric:[" + this.convertInputStringToRangeValue(this.inputs[0]()) +
                " TO " + this.convertInputStringToRangeValue(this.inputs[1]()) + "]";
            this.applyFilterTask.resolve(filter);
            this.close();
        };
        searchFileSizeRangeClause.prototype.validateInput = function (input) {
            return !input.trim() || searchFileSizeRangeClause.inputRegexp.test(input);
        };
        searchFileSizeRangeClause.prototype.convertInputStringToRangeValue = function (input) {
            if (!input)
                return "*";
            var match = searchFileSizeRangeClause.inputRegexp.exec(input);
            var value = parseInt(match[1]);
            var loweredCaseMultiplier = match[2].toLowerCase();
            var multiplier = this.getMultiplier(loweredCaseMultiplier);
            value *= multiplier;
            return "Lx" + value.toString();
        };
        searchFileSizeRangeClause.prototype.getMultiplier = function (value) {
            if (!value)
                return 1;
            if (value.indexOf("kb") > -1)
                return 1024;
            if (value.indexOf("mb") > -1)
                return 1024 * 1024;
            if (value.indexOf("gb") > -1)
                return 1024 * 1024 * 1024;
            return 1;
        };
        searchFileSizeRangeClause.prototype.enabled = function () {
            return this.checkRequired(false)
                && this.validateInput(this.inputs[0]())
                && this.validateInput(this.inputs[1]());
        };
        searchFileSizeRangeClause.inputRegexp = /^(\d+)\s*(|[kmgKMG]b)$/;
        return searchFileSizeRangeClause;
    })(searchDialogViewModel);
    return searchFileSizeRangeClause;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/searchHasMetadataClause", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel", "commands/filesystem/searchByTermCommand", "common/bindingHelpers/autoCompleteBindingHandler"], function (require, exports, searchDialogViewModel, searchByTermCommand, autoCompleteBindingHandler) {
    var searchHasMetadataClause = (function (_super) {
        __extends(searchHasMetadataClause, _super);
        function searchHasMetadataClause(fs) {
            var _this = this;
            _super.call(this, [ko.observable(""), ko.observable("")]);
            this.fs = fs;
            this.applyFilterTask = $.Deferred();
            this.keySearchResults = ko.observableArray();
            this.inputs[0].throttle(250).subscribe(function (search) { return _this.fetchKeySearchResults(search); });
            autoCompleteBindingHandler.install();
        }
        searchHasMetadataClause.prototype.applyFilter = function () {
            this.applyFilterTask.resolve(this.inputs[0]() + ":" + this.inputs[1]());
            this.close();
        };
        searchHasMetadataClause.prototype.fetchKeySearchResults = function (query) {
            var _this = this;
            if (query.length >= 1) {
                new searchByTermCommand(this.fs, query).execute()
                    .done(function (x) {
                    x = x.filter(function (item, pos, arr) {
                        return item[0] != ("_") && !searchHasMetadataClause.ExcludedMetadataFields.contains(item);
                    });
                    _this.keySearchResults(x);
                });
            }
        };
        searchHasMetadataClause.prototype.setKey = function (key) {
            this.inputs[0](key);
        };
        searchHasMetadataClause.ExcludedMetadataFields = ["Content-Length", "Last-Modified", "Content-MD5", "RavenFS-Size", "Origin"];
        return searchHasMetadataClause;
    })(searchDialogViewModel);
    return searchHasMetadataClause;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/searchLastModifiedBetweenClause", ["require", "exports", "common/bindingHelpers/datePickerBindingHandler", "viewmodels/filesystem/files/searchDialogViewModel"], function (require, exports, datePickerBindingHandler, searchDialogViewModel) {
    var searchLastModifiedBetweenClause = (function (_super) {
        __extends(searchLastModifiedBetweenClause, _super);
        function searchLastModifiedBetweenClause() {
            var _this = this;
            _super.call(this, []);
            this.applyFilterTask = $.Deferred();
            this.fromDate = ko.observable();
            this.toDate = ko.observable();
            this.fromDateText = ko.observable();
            this.toDateText = ko.observable();
            this.inputs.push(this.fromDateText);
            this.inputs.push(this.toDateText);
            this.fromDate.subscribe(function (v) {
                return _this.fromDateText(_this.fromDate() != null ? _this.fromDate().format("YYYY-MM-DD HH-mm-ss") : "");
            });
            this.toDate.subscribe(function (v) {
                return _this.toDateText(_this.toDate() != null ? _this.toDate().format("YYYY-MM-DD HH-mm-ss") : "");
            });
            datePickerBindingHandler.install();
        }
        searchLastModifiedBetweenClause.prototype.applyFilter = function () {
            if (!this.fromDateText() || !this.toDateText())
                return false;
            var dates = "__modified:[" + this.fromDateText().trim().replaceAll(" ", "_")
                + " TO " + this.toDateText().trim().replaceAll(" ", "_") + "]";
            this.applyFilterTask.resolve(dates);
            this.close();
        };
        return searchLastModifiedBetweenClause;
    })(searchDialogViewModel);
    return searchLastModifiedBetweenClause;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/files/searchSingleInputClause", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel"], function (require, exports, searchDialogViewModel) {
    var searchSingleInputClause = (function (_super) {
        __extends(searchSingleInputClause, _super);
        function searchSingleInputClause(label) {
            _super.call(this, [ko.observable("")]);
            this.applyFilterTask = $.Deferred();
            this.label = "";
            this.label = label;
        }
        searchSingleInputClause.prototype.applyFilter = function () {
            this.applyFilterTask.resolve(this.inputs[0]());
            this.close();
        };
        searchSingleInputClause.prototype.enabled = function () {
            return this.checkRequired(true);
        };
        return searchSingleInputClause;
    })(searchDialogViewModel);
    return searchSingleInputClause;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/search/fieldRangeFilter", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel", "common/bindingHelpers/datePickerBindingHandler"], function (require, exports, searchDialogViewModel, datePickerBindingHandler) {
    var fieldRangeFilter = (function (_super) {
        __extends(fieldRangeFilter, _super);
        function fieldRangeFilter(label) {
            var _this = this;
            _super.call(this, [ko.observable("")]);
            this.filterOptions = ko.observableArray(["Numeric Double", "Numeric Int", "Alphabetical", "Datetime"]);
            this.selectedOption = ko.observable("Starts with");
            this.applyFilterTask = $.Deferred();
            this.label = "";
            this.from = ko.observable();
            this.to = ko.observable();
            this.fromDate = ko.observable();
            this.toDate = ko.observable();
            datePickerBindingHandler.install();
            this.label = label;
            this.from("");
            this.to("");
            this.fromDate.subscribe(function (v) {
                return _this.from(_this.fromDate() != null ? _this.fromDate().format("YYYY-MM-DDTHH:mm:00.0000000") : "");
            });
            this.toDate.subscribe(function (v) {
                return _this.to(_this.toDate() != null ? _this.toDate().format("YYYY-MM-DDTHH:mm:00.0000000") : "");
            });
        }
        fieldRangeFilter.prototype.applyFilter = function () {
            this.applyFilterTask.resolve(this.from(), this.to(), this.selectedOption());
            this.close();
        };
        fieldRangeFilter.prototype.enabled = function () {
            return true;
        };
        fieldRangeFilter.prototype.isDateTime = function () {
            return (this.selectedOption() === "Datetime");
        };
        return fieldRangeFilter;
    })(searchDialogViewModel);
    return fieldRangeFilter;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/search/fieldStringFilter", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel"], function (require, exports, searchDialogViewModel) {
    var fieldStringFilter = (function (_super) {
        __extends(fieldStringFilter, _super);
        function fieldStringFilter(label) {
            _super.call(this, [ko.observable("")]);
            this.filterOptions = ko.observableArray(["Starts with", "Ends with", "Contains", "Exact"]);
            this.selectedOption = ko.observable("Starts with");
            this.applyFilterTask = $.Deferred();
            this.label = "";
            this.label = label;
        }
        fieldStringFilter.prototype.applyFilter = function () {
            this.applyFilterTask.resolve(this.inputs[0](), this.selectedOption());
            this.close();
        };
        fieldStringFilter.prototype.enabled = function () {
            return this.checkRequired(true);
        };
        return fieldStringFilter;
    })(searchDialogViewModel);
    return fieldStringFilter;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/search/inMethodFilter", ["require", "exports", "viewmodels/filesystem/files/searchDialogViewModel"], function (require, exports, searchDialogViewModel) {
    var inMethodFilterElement = (function () {
        function inMethodFilterElement() {
            this.value = ko.observable('');
            this.focus = ko.observable(false);
        }
        return inMethodFilterElement;
    })();
    var inMethodFilter = (function (_super) {
        __extends(inMethodFilter, _super);
        function inMethodFilter(label) {
            var _this = this;
            _super.call(this, [ko.observable("")]);
            this.label = label;
            this.applyFilterTask = $.Deferred();
            this.elements = ko.observableArray();
            this.deleteEnabled = ko.computed(function () { return _this.elements().length > 1; });
            this.newElement();
        }
        inMethodFilter.prototype.applyFilter = function () {
            this.applyFilterTask.resolve(this.projectElementsToStringArray());
            this.close();
        };
        inMethodFilter.prototype.enabled = function () {
            return true;
        };
        inMethodFilter.prototype.newElement = function () {
            var element = new inMethodFilterElement();
            this.elements.push(element);
            element.focus(true);
        };
        inMethodFilter.prototype.removeElement = function (element) {
            this.elements.remove(element);
        };
        inMethodFilter.prototype.projectElementsToStringArray = function () {
            return this.elements().map(function (x) { return x.value(); });
        };
        return inMethodFilter;
    })(searchDialogViewModel);
    return inMethodFilter;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/search/resetIndexConfirm", ["require", "exports", "commands/filesystem/resetIndexCommand", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, resetIndexCommand, dialog, dialogViewModelBase) {
    var resetIndexConfirm = (function (_super) {
        __extends(resetIndexConfirm, _super);
        function resetIndexConfirm(fs) {
            _super.call(this);
            this.fs = fs;
            this.resetTask = $.Deferred();
        }
        resetIndexConfirm.prototype.resetIndex = function () {
            var _this = this;
            new resetIndexCommand(this.fs).execute().done(function () { return _this.resetTask.resolve(); }).fail(function () { return _this.resetTask.reject(); });
            dialog.close(this);
        };
        resetIndexConfirm.prototype.cancel = function () {
            this.resetTask.reject();
            dialog.close(this);
        };
        return resetIndexConfirm;
    })(dialogViewModelBase);
    return resetIndexConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/search/search", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/searchByQueryCommand", "common/pagedList", "viewmodels/filesystem/files/searchSingleInputClause", "viewmodels/filesystem/files/searchFileSizeRangeClause", "viewmodels/filesystem/files/searchHasMetadataClause", "viewmodels/filesystem/files/searchLastModifiedBetweenClause", "viewmodels/filesystem/deleteFilesMatchingQueryConfirm", "viewmodels/filesystem/search/resetIndexConfirm", "common/queryUtil"], function (require, exports, app, router, appUrl, viewModelBase, searchByQueryCommand, pagedList, searchSingleInputClause, searchFileSizeRangeClause, searchHasMetadataClause, searchLastModifiedBetweenClause, deleteFilesMatchingQueryConfirm, resetIndexConfirm, queryUtil) {
    var search = (function (_super) {
        __extends(search, _super);
        function search() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.searchUrl = appUrl.forCurrentDatabase().filesystemSearch;
            this.searchText = ko.observable("");
            this.allFilesPagedItems = ko.observable();
            this.selectedFilesIndices = ko.observableArray();
            this.searchText.extend({ throttle: 200 }).subscribe(function (s) { return _this.searchFiles(s); });
        }
        search.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.appUrls = appUrl.forCurrentFilesystem();
            this.activeFilesystem.subscribe(function (fs) {
                _this.searchFiles("");
                _this.searchText("");
            });
            this.loadFiles();
        };
        search.prototype.clear = function () {
            this.searchText("");
        };
        search.prototype.search = function () {
            this.searchFiles(this.searchText());
        };
        search.prototype.searchFiles = function (query) {
            this.allFilesPagedItems(this.createPagedList(query));
        };
        search.prototype.loadFiles = function () {
            this.allFilesPagedItems(this.createPagedList(""));
        };
        search.prototype.createPagedList = function (query) {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchFiles(query, skip, take); };
            return new pagedList(fetcher);
        };
        search.prototype.fetchFiles = function (query, skip, take) {
            var task = new searchByQueryCommand(appUrl.getFileSystem(), query, skip, take).execute();
            return task;
        };
        search.prototype.fileNameStartsWith = function () {
            var _this = this;
            var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename starts with: ");
            searchSingleInputClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput("__fileName:" + queryUtil.escapeTerm(input) + "*"); });
            app.showDialog(searchSingleInputClauseViewModel);
        };
        search.prototype.fileNameEndsWith = function () {
            var _this = this;
            var searchSingleInputClauseViewModel = new searchSingleInputClause("Filename ends with: ");
            searchSingleInputClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput("__rfileName:" + queryUtil.escapeTerm(String.prototype.reverse(input)) + "*"); });
            app.showDialog(searchSingleInputClauseViewModel);
        };
        search.prototype.fileSizeBetween = function () {
            var _this = this;
            var searchFileSizeRangeClauseViewModel = new searchFileSizeRangeClause();
            searchFileSizeRangeClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(input); });
            app.showDialog(searchFileSizeRangeClauseViewModel);
        };
        search.prototype.hasMetadata = function () {
            var _this = this;
            var searchHasMetadataClauseViewModel = new searchHasMetadataClause(this.activeFilesystem());
            searchHasMetadataClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(queryUtil.escapeTerm(input)); });
            app.showDialog(searchHasMetadataClauseViewModel);
        };
        search.prototype.inFolder = function () {
            var _this = this;
            var searchSingleInputClauseViewModel = new searchSingleInputClause("Folder path: ");
            searchSingleInputClauseViewModel
                .applyFilterTask
                .done(function (input) {
                if (!input.startsWith("/"))
                    input = "/" + input;
                var escaped = queryUtil.escapeTerm(input);
                _this.addToSearchInput("__directoryName:" + escaped);
            });
            app.showDialog(searchSingleInputClauseViewModel);
        };
        search.prototype.lastModifiedBetween = function () {
            var _this = this;
            var searchLastModifiedBetweenClauseViewModel = new searchLastModifiedBetweenClause();
            searchLastModifiedBetweenClauseViewModel
                .applyFilterTask
                .done(function (input) { return _this.addToSearchInput(input); });
            app.showDialog(searchLastModifiedBetweenClauseViewModel);
        };
        search.prototype.addToSearchInput = function (input) {
            var currentSearchText = this.searchText();
            if (currentSearchText != null && currentSearchText.trim().length > 0)
                currentSearchText += " AND ";
            this.searchText(currentSearchText + input);
        };
        search.prototype.deleteFilesMatchingQuery = function () {
            var _this = this;
            // Run the query so that we have an idea of what we'll be deleting.
            this.search();
            this.allFilesPagedItems()
                .fetch(0, 1)
                .done(function (results) {
                if (results.totalResultCount === 0) {
                    app.showMessage("There are no files matching your query.", "Nothing to do");
                }
                else {
                    _this.promptDeleteFilesMatchingQuery(results.totalResultCount);
                }
            });
        };
        search.prototype.promptDeleteFilesMatchingQuery = function (resultCount) {
            var _this = this;
            var viewModel = new deleteFilesMatchingQueryConfirm(this.searchText(), resultCount, this.activeFilesystem());
            app
                .showDialog(viewModel)
                .done(function () { return _this.search(); });
        };
        search.prototype.resetIndex = function () {
            var resetIndexVm = new resetIndexConfirm(this.activeFilesystem());
            app.showDialog(resetIndexVm);
        };
        search.gridSelector = "#filesGrid";
        return search;
    })(viewModelBase);
    return search;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/settings/settings", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var settings = (function (_super) {
        __extends(settings, _super);
        function settings() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.bundleMap = { versioning: "Versioning" };
            this.userDatabasePages = ko.observableArray([]);
            this.appUrls = appUrl.forCurrentDatabase();
            var versioningRoute = { route: 'filesystems/settings', moduleId: 'viewmodels/filesystem/settings/versioning', title: 'Versioning', nav: true, hash: appUrl.forCurrentFilesystem().filesystemVersioning };
            this.router = durandalRouter.createChildRouter()
                .map([
                versioningRoute
            ])
                .buildNavigationModel();
            this.router.guardRoute = function (instance, instruction) { return _this.getValidRoute(instance, instruction); };
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        /**
        * Checks whether the route can be navigated to. Returns true if it can be navigated to, or a redirect URI if it can't be navigated to.
        * This is used for preventing a navigating to system-only pages when the current databagse is non-system, and vice-versa.
        */
        settings.prototype.getValidRoute = function (instance, instruction) {
            var fs = this.activeFilesystem();
            var pathArr = instruction.fragment.split('/');
            var bundelName = pathArr[pathArr.length - 1].toLowerCase();
            var isLegalBundelName = (this.bundleMap[bundelName] != undefined);
            var isBundleExists = this.userDatabasePages.indexOf(this.bundleMap[bundelName]) > -1;
            if (isLegalBundelName && isBundleExists == false) {
                return appUrl.forCurrentDatabase().filesystemSettings();
            }
            return true;
        };
        settings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.userDatabasePages([]);
            var fs = this.activeFilesystem();
            var bundles = fs.activeBundles();
            bundles.forEach(function (bundle) {
                var bundleName = _this.bundleMap[bundle.toLowerCase()];
                if (bundleName != undefined) {
                    _this.userDatabasePages.push(bundleName);
                }
            });
        };
        settings.prototype.routeIsVisible = function (route) {
            var bundleTitle = route.title;
            if (this.userDatabasePages.indexOf(bundleTitle) !== -1) {
                return true;
            }
            return false;
        };
        return settings;
    })(viewModelBase);
    return settings;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/settings/versioning", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/filesystem/getVersioningCommand", "commands/filesystem/saveVersioningCommand"], function (require, exports, viewModelBase, appUrl, getVersioningCommand, saveVersioningCommand) {
    var versioning = (function (_super) {
        __extends(versioning, _super);
        function versioning() {
            _super.apply(this, arguments);
            this.versioning = ko.observable();
        }
        versioning.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                this.fetchVersioning(fs)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forFilesystemFiles(_this.activeFilesystem()) }); });
            }
            return deferred;
        };
        versioning.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.versioning]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        versioning.prototype.fetchVersioning = function (fs) {
            var _this = this;
            var task = new getVersioningCommand(fs).execute();
            task.done(function (versioning) { return _this.versioningLoaded(versioning); });
            return task;
        };
        versioning.prototype.saveChanges = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            if (fs) {
                var saveTask = new saveVersioningCommand(fs, this.versioning().toDto())
                    .execute()
                    .done(function (saveResult) {
                    _this.dirtyFlag().reset();
                });
            }
        };
        versioning.prototype.versioningLoaded = function (data) {
            this.versioning(data);
            this.dirtyFlag().reset();
        };
        return versioning;
    })(viewModelBase);
    return versioning;
});
define("viewmodels/filesystem/status/activityItems", ["require", "exports", "commands/filesystem/getSyncOutgoingActivitiesCommand", "commands/filesystem/getSyncIncomingActivitiesCommand"], function (require, exports, getSyncOutgoingActivitiesCommand, getSyncIncomingActivitiesCommand) {
    var activityItems = (function () {
        function activityItems(fs, activity, direction) {
            var _this = this;
            this.fs = fs;
            this.activity = activity;
            this.direction = direction;
            this.pageSize = 50;
            this.totalCount = ko.observable();
            this.pageIndex = ko.observable(0);
            this.items = ko.observableArray();
            this.maxPageIndex = ko.computed(function () {
                return Math.ceil(_this.totalCount() / _this.pageSize) - 1;
            });
            this.allPages = ko.computed(function () {
                var pages = [];
                for (var i = 0; i <= _this.maxPageIndex(); i++) {
                    pages.push({ pageNumber: (i + 1) });
                }
                return pages;
            });
            this.retrieveItems();
        }
        activityItems.prototype.moveToPage = function (index) {
            this.pageIndex(index);
            this.retrieveItems();
        };
        ;
        activityItems.prototype.refresh = function () {
            this.retrieveItems();
        };
        activityItems.prototype.retrieveItems = function () {
            var _this = this;
            var start = this.pageIndex() * this.pageSize;
            var command;
            switch (this.direction) {
                case 0 /* Outgoing */:
                    command = getSyncOutgoingActivitiesCommand;
                    break;
                case 1 /* Incoming */:
                    command = getSyncIncomingActivitiesCommand;
                    break;
                default:
                    throw new TypeError("Not supported synchronization direction type when attempting. Given type: " + this.direction);
            }
            new command(this.fs, this.activity, start, this.pageSize).execute()
                .done(function (x) {
                _this.totalCount(x.TotalCount);
                _this.items.removeAll();
                for (var i = 0; i < x.Items.length; i++) {
                    _this.items.push(x.Items[i]);
                }
            });
        };
        return activityItems;
    })();
    return activityItems;
});
define("viewmodels/filesystem/status/sourceSynchronizationInformation", ["require", "exports"], function (require, exports) {
    var sourceSynchronizationInformation = (function () {
        function sourceSynchronizationInformation(lastSynchronizedEtag, sourceServerUrl) {
            this.LastSourceFileEtag = ko.observable();
            this.SourceServerUrl = ko.observable();
            this.LastSourceFileEtag(lastSynchronizedEtag);
            this.SourceServerUrl(sourceServerUrl);
        }
        return sourceSynchronizationInformation;
    })();
    return sourceSynchronizationInformation;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/status/status", ["require", "exports", "common/appUrl", "common/changesContext", "viewmodels/viewModelBase", "commands/filesystem/synchronizeNowCommand", "viewmodels/filesystem/status/activityItems", "commands/filesystem/getConfigurationsByPrefixCommand", "commands/filesystem/getConfigurationByKeyCommand", "viewmodels/filesystem/status/sourceSynchronizationInformation"], function (require, exports, appUrl, changesContext, viewModelBase, synchronizeNowCommand, activityItems, getConfigurationNamesByPrefixCommand, getConfigurationByKeyCommand, sourceSynchronizationInformation) {
    var status = (function (_super) {
        __extends(status, _super);
        function status() {
            _super.apply(this, arguments);
            this.sourceSynchronizationInformations = ko.observableArray();
            this.isFsSyncUpToDate = true;
        }
        status.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.appUrls = appUrl.forCurrentFilesystem();
            this.pendingOutgoing = new activityItems(this.activeFilesystem(), 2 /* Pending */, 0 /* Outgoing */);
            this.activeOutgoing = new activityItems(this.activeFilesystem(), 1 /* Active */, 0 /* Outgoing */);
            this.finishedIncoming = new activityItems(this.activeFilesystem(), 3 /* Finished */, 1 /* Incoming */);
            this.activeIncoming = new activityItems(this.activeFilesystem(), 1 /* Active */, 1 /* Incoming */);
            this.retrieveLastSynchronizedEtags();
        };
        status.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchFsSync(function (e) { return _this.processFsSync(e); })];
        };
        status.prototype.showException = function (exception) {
            var prettifySpacing = 4;
            var configText = JSON.stringify(exception, null, prettifySpacing);
            this.confirmationMessage("Exception details", configText, ["Close"]);
        };
        status.prototype.retrieveLastSynchronizedEtags = function () {
            var _this = this;
            new getConfigurationNamesByPrefixCommand(this.activeFilesystem(), "Raven/Synchronization/Sources").execute().done(function (searchResult) {
                searchResult.ConfigNames.forEach(function (name) {
                    new getConfigurationByKeyCommand(_this.activeFilesystem(), name).execute().
                        done(function (sourceSyncInfoAsString) {
                        var syncInfo = JSON.parse(sourceSyncInfoAsString);
                        var match = ko.utils.arrayFirst(_this.sourceSynchronizationInformations(), function (item) { return (syncInfo.SourceServerUrl === item.SourceServerUrl()); });
                        if (!match) {
                            _this.sourceSynchronizationInformations.push(new sourceSynchronizationInformation(syncInfo.LastSourceFileEtag, syncInfo.SourceServerUrl));
                        }
                        else {
                            match.LastSourceFileEtag(syncInfo.LastSourceFileEtag);
                        }
                    });
                });
            });
        };
        status.prototype.processFsSync = function (e) {
            // treat notifications events
            this.isFsSyncUpToDate = false;
            switch (e.Direction) {
                case "Outgoing":
                    if (e.Action === "Enqueue") {
                        this.pendingOutgoing.refresh();
                    }
                    else if (e.Action === "Start" || e.Action === "Finish") {
                        this.activeOutgoing.refresh();
                        this.pendingOutgoing.refresh();
                    }
                    break;
                case "Incoming":
                    if (e.Action === "Start") {
                        this.activeIncoming.refresh();
                    }
                    else if (e.Action === "Finish") {
                        this.activeIncoming.refresh();
                        this.finishedIncoming.refresh();
                        this.retrieveLastSynchronizedEtags();
                    }
                    break;
                default:
                    break;
            }
        };
        status.prototype.synchronizeNow = function () {
            var fs = this.activeFilesystem();
            if (fs) {
                new synchronizeNowCommand(fs).execute();
            }
        };
        status.prototype.getActionDescription = function (action) {
            switch (action) {
                case 0 /* Enqueue */:
                    return 2 /* Pending */;
                case 1 /* Start */:
                    return 1 /* Active */;
                case 2 /* Finish */:
                    return 3 /* Finished */;
                default:
                    return 0 /* Unknown */;
            }
        };
        return status;
    })(viewModelBase);
    return status;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/synchronization/resolveConflict", ["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog"], function (require, exports, dialogViewModelBase, dialog) {
    var resolveConflict = (function (_super) {
        __extends(resolveConflict, _super);
        function resolveConflict(message, title) {
            _super.call(this);
            this.resolveTask = $.Deferred();
            this.resolveTaskStarted = false;
            this.message = ko.observable('');
            this.title = ko.observable('');
            this.message = message;
            this.title = title;
        }
        resolveConflict.prototype.cancel = function () {
            dialog.close(this);
        };
        resolveConflict.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.resolveTaskStarted) {
                this.resolveTask.reject();
            }
        };
        resolveConflict.prototype.resolve = function () {
            this.resolveTaskStarted = true;
            this.resolveTask.resolve();
            dialog.close(this);
        };
        return resolveConflict;
    })(dialogViewModelBase);
    return resolveConflict;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/synchronization/synchronization", ["require", "exports", "plugins/router", "viewmodels/viewModelBase", "common/appUrl"], function (require, exports, durandalRouter, viewModelBase, appUrl) {
    var synchronization = (function (_super) {
        __extends(synchronization, _super);
        function synchronization() {
            var _this = this;
            _super.call(this);
            this.appUrls = appUrl.forCurrentFilesystem();
            this.router = durandalRouter.createChildRouter()
                .map([
                { route: "filesystems/synchronization", moduleId: "viewmodels/filesystem/synchronization/synchronizationConflicts", title: "Conflicts", nav: true, hash: appUrl.forCurrentFilesystem().filesystemSynchronization },
                { route: "filesystems/synchronization/destinations", moduleId: "viewmodels/filesystem/synchronization/synchronizationDestinations", title: "Destinations", nav: true, hash: appUrl.forCurrentFilesystem().filesystemSynchronizationDestinations },
                { route: "filesystems/synchronization/configuration", moduleId: "viewmodels/filesystem/synchronization/synchronizationConfiguration", title: "Configuration", nav: true, hash: appUrl.forCurrentFilesystem().filesystemSynchronizationConfiguration }
            ])
                .buildNavigationModel();
            synchronization.statusRouter = this.router;
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        synchronization.prototype.canActivate = function (args) {
            return true;
        };
        return synchronization;
    })(viewModelBase);
    return synchronization;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/synchronization/synchronizationConfiguration", ["require", "exports", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/getSynchronizationConfigCommand", "commands/filesystem/saveConfigurationCommand", "models/filesystem/configurationKey"], function (require, exports, appUrl, viewModelBase, getSynchronizationConfigCommand, saveConfigurationCommand, configurationKey) {
    var synchronizationConfiguration = (function (_super) {
        __extends(synchronizationConfiguration, _super);
        function synchronizationConfiguration() {
            _super.apply(this, arguments);
            this.config = ko.observable();
            this.dirtyFlag = new ko.DirtyFlag([]);
        }
        synchronizationConfiguration.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                this.fetchConfig()
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forResources() }); });
            }
            return deferred;
        };
        synchronizationConfiguration.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.config]);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        };
        synchronizationConfiguration.prototype.saveChanges = function () {
            var _this = this;
            new saveConfigurationCommand(this.activeFilesystem(), new configurationKey(this.activeFilesystem(), "Raven/Synchronization/Config"), this.config().toDto())
                .execute()
                .done(function () { return _this.dirtyFlag().reset(); });
        };
        synchronizationConfiguration.prototype.fetchConfig = function () {
            var _this = this;
            return new getSynchronizationConfigCommand(this.activeFilesystem())
                .execute()
                .done(function (result) { return _this.config(result); });
        };
        return synchronizationConfiguration;
    })(viewModelBase);
    return synchronizationConfiguration;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/synchronization/synchronizationConflicts", ["require", "exports", "durandal/app", "viewmodels/viewModelBase", "common/changesContext", "viewmodels/filesystem/synchronization/resolveConflict", "models/filesystem/conflictItem", "commands/filesystem/getFilesConflictsCommand", "commands/filesystem/resolveConflictCommand"], function (require, exports, app, viewModelBase, changesContext, resolveConflict, conflictItem, getFilesConflictsCommand, resolveConflictCommand) {
    var synchronizationConflicts = (function (_super) {
        __extends(synchronizationConflicts, _super);
        function synchronizationConflicts() {
            var _this = this;
            _super.apply(this, arguments);
            this.conflicts = ko.observableArray();
            this.selectedConflicts = ko.observableArray();
            this.isConflictsVisible = ko.computed(function () { return _this.conflicts().length > 0; });
            this.isSelectAllValue = ko.observable(false);
        }
        synchronizationConflicts.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeFilesystemSubscription = this.activeFilesystem.subscribe(function (fs) { return _this.fileSystemChanged(fs); });
            this.loadConflicts();
        };
        synchronizationConflicts.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            this.activeFilesystemSubscription.dispose();
        };
        synchronizationConflicts.prototype.createNotifications = function () {
            var _this = this;
            return [changesContext.currentResourceChangesApi().watchFsConflicts(function (e) { return _this.processFsConflicts(e); })];
        };
        synchronizationConflicts.prototype.processFsConflicts = function (e) {
            // treat notifications events
            switch (e.Status) {
                case 0 /* Detected */:
                    {
                        this.addConflict(e);
                        break;
                    }
                case 1 /* Resolved */:
                    {
                        this.removeResolvedConflict(e);
                        break;
                    }
                default:
                    console.error("unknown notification action");
            }
        };
        synchronizationConflicts.prototype.addConflict = function (conflictUpdate) {
            var match = this.conflictsContains(conflictUpdate);
            if (!match) {
                this.conflicts.push(conflictItem.fromConflictNotificationDto(conflictUpdate));
            }
        };
        synchronizationConflicts.prototype.removeResolvedConflict = function (conflictUpdate) {
            var match = this.conflictsContains(conflictUpdate);
            if (match) {
                this.conflicts.remove(match);
                this.selectedConflicts.remove(match.fileName);
            }
            this.isSelectAllValue(false);
        };
        synchronizationConflicts.prototype.conflictsContains = function (e) {
            var match = ko.utils.arrayFirst(this.conflicts(), function (item) {
                return item.fileName === e.FileName;
            });
            return match;
        };
        synchronizationConflicts.prototype.loadConflicts = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            if (fs) {
                var deferred = $.Deferred();
                var conflictsTask = new getFilesConflictsCommand(fs).execute()
                    .done(function (x) { return _this.conflicts(x); });
                conflictsTask.done(function () { return deferred.resolve(); });
                return deferred;
            }
        };
        synchronizationConflicts.prototype.collapseAll = function () {
            $(".synchronization-group-content").collapse('hide');
        };
        synchronizationConflicts.prototype.expandAll = function () {
            $(".synchronization-group-content").collapse('show');
        };
        synchronizationConflicts.prototype.resolveWithLocalVersion = function () {
            var _this = this;
            var message = this.selectedConflicts().length == 1 ?
                "Are you sure you want to resolve the conflict for file <b>" + this.selectedConflicts()[0] + "</b> by choosing the local version?" :
                "Are you sure you want to resolve the conflict for <b>" + this.selectedConflicts().length + "</b> selected files by choosing the local version?";
            var resolveConflictViewModel = new resolveConflict(message, "Resolve conflict with local");
            resolveConflictViewModel
                .resolveTask
                .done(function (x) {
                var fs = _this.activeFilesystem();
                for (var i = 0; i < _this.selectedConflicts().length; i++) {
                    var conflict = _this.selectedConflicts()[i];
                    new resolveConflictCommand(conflict, 2, fs).execute().done(function () {
                        _this.selectedConflicts.remove(conflict);
                    });
                }
            });
            app.showDialog(resolveConflictViewModel);
        };
        synchronizationConflicts.prototype.resolveWithRemoteVersion = function () {
            var _this = this;
            var message = this.selectedConflicts().length == 1 ?
                "Are you sure you want to resolve the conflict for file <b>" + this.selectedConflicts()[0] + "</b> by choosing the remote version?" :
                "Are you sure you want to resolve the conflict for <b>" + this.selectedConflicts().length + "</b> selected files by choosing the remote version?";
            var resolveConflictViewModel = new resolveConflict(message, "Resolve conflict with remote");
            resolveConflictViewModel
                .resolveTask
                .done(function (x) {
                var fs = _this.activeFilesystem();
                for (var i = 0; i < _this.selectedConflicts().length; i++) {
                    var conflict = _this.selectedConflicts()[i];
                    new resolveConflictCommand(conflict, 1, fs).execute();
                }
            });
            app.showDialog(resolveConflictViewModel);
        };
        synchronizationConflicts.prototype.fileSystemChanged = function (fs) {
            if (fs) {
                this.loadConflicts();
            }
        };
        synchronizationConflicts.prototype.isSelectAll = function () {
            return this.isSelectAllValue();
        };
        synchronizationConflicts.prototype.toggleSelectAll = function () {
            this.isSelectAllValue(!this.isSelectAllValue());
            if (this.isSelectAllValue()) {
                this.selectedConflicts.pushAll(this.conflicts().map(function (x) { return x.fileName; }));
            }
            else {
                this.selectedConflicts.removeAll();
            }
        };
        return synchronizationConflicts;
    })(viewModelBase);
    return synchronizationConflicts;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/synchronization/synchronizationDestinations", ["require", "exports", "common/appUrl", "models/filesystem/synchronizationReplicationSetup", "models/filesystem/synchronizationDestination", "viewmodels/viewModelBase", "common/changesContext", "commands/filesystem/getDestinationsCommand", "commands/filesystem/getFileSystemStatsCommand", "commands/filesystem/saveDestinationCommand"], function (require, exports, appUrl, synchronizationReplicationSetup, synchronizationDestination, viewModelBase, changesContext, getDestinationsCommand, getFileSystemStatsCommand, saveDestinationCommand) {
    var synchronizationDestinations = (function (_super) {
        __extends(synchronizationDestinations, _super);
        function synchronizationDestinations() {
            _super.apply(this, arguments);
            this.dirtyFlag = new ko.DirtyFlag([]);
            this.replicationsSetup = ko.observable(new synchronizationReplicationSetup({ Destinations: [], Source: null }));
            this.saveIssued = false;
        }
        synchronizationDestinations.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                this.fetchDestinations()
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forFilesystemFiles(_this.activeFilesystem()) }); });
            }
            return deferred;
        };
        synchronizationDestinations.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (!this.subscription) {
                this.subscription = changesContext.currentResourceChangesApi()
                    .watchFsDestinations(function (e) {
                    if (e.Name.indexOf("Raven/Synchronization/Destinations") < 0)
                        return;
                    if (!_this.saveIssued && (e.Action == 0 /* Set */ || e.Action == 1 /* Delete */)) {
                        var canContinue = _this.canContinueIfNotDirty('Data has changed', 'Data has changed in the server. Do you want to refresh and overwrite your changes?');
                        canContinue.done(function () {
                            _this.fetchDestinations().done(function (x) {
                                _this.dirtyFlag().reset();
                            });
                        });
                    }
                    else
                        console.error("Unknown notification action.");
                });
            }
            this.dirtyFlag = new ko.DirtyFlag([this.replicationsSetup]);
            this.isSaveEnabled = ko.computed(function () {
                return _this.dirtyFlag().isDirty();
            });
        };
        synchronizationDestinations.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        synchronizationDestinations.prototype.saveChanges = function () {
            var _this = this;
            if (this.replicationsSetup().source()) {
                this.saveReplicationSetup();
            }
            else {
                var fs = this.activeFilesystem();
                if (fs) {
                    new getFileSystemStatsCommand(fs)
                        .execute()
                        .done(function (result) { return _this.prepareAndSaveReplicationSetup(result.DatabaseId); });
                }
            }
        };
        synchronizationDestinations.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };
        synchronizationDestinations.prototype.saveReplicationSetup = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            if (fs) {
                var self = this;
                this.saveIssued = true;
                new saveDestinationCommand(this.replicationsSetup().toDto(), fs)
                    .execute()
                    .done(function () {
                    console.log("Reseted dirty flag");
                    _this.dirtyFlag().reset();
                }).always(function () { return _this.saveIssued = false; });
                ;
            }
        };
        synchronizationDestinations.prototype.createNewDestination = function () {
            var fs = this.activeFilesystem();
            this.replicationsSetup().destinations.unshift(synchronizationDestination.empty(fs.name));
        };
        synchronizationDestinations.prototype.removeDestination = function (repl) {
            this.replicationsSetup().destinations.remove(repl);
        };
        synchronizationDestinations.prototype.fetchDestinations = function () {
            var _this = this;
            var deferred = $.Deferred();
            var fs = this.activeFilesystem();
            if (fs) {
                new getDestinationsCommand(fs)
                    .execute()
                    .done(function (data) { return _this.replicationsSetup(new synchronizationReplicationSetup({ Destinations: data.Destinations, Source: null })); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        return synchronizationDestinations;
    })(viewModelBase);
    return synchronizationDestinations;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/tasks/exportFilesystem", ["require", "exports", "viewmodels/viewModelBase", "common/bindingHelpers/aceEditorBindingHandler", "common/appUrl", "commands/auth/getSingleAuthTokenCommand", "common/messagePublisher"], function (require, exports, viewModelBase, aceEditorBindingHandler, appUrl, getSingleAuthTokenCommand, messagePublisher) {
    var exportDatabase = (function (_super) {
        __extends(exportDatabase, _super);
        function exportDatabase() {
            _super.call(this);
            this.batchSize = ko.observable(1024);
            this.noneDefualtFileName = ko.observable("");
            this.chooseDifferntFileName = ko.observable(false);
            this.authToken = ko.observable();
            aceEditorBindingHandler.install();
        }
        exportDatabase.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('YD9M1R');
            this.exportActionUrl = ko.computed(function () {
                var token = _this.authToken();
                return appUrl.forResourceQuery(_this.activeFilesystem()) + "/studio-tasks/exportFilesystem" + (token ? '?singleUseAuthToken=' + token : '');
            });
        };
        exportDatabase.prototype.startExport = function () {
            var _this = this;
            var smugglerOptions = {
                BatchSize: this.batchSize(),
                NoneDefualtFileName: this.noneDefualtFileName()
            };
            $("#SmugglerOptions").val(JSON.stringify(smugglerOptions));
            new getSingleAuthTokenCommand(this.activeFilesystem()).execute().done(function (token) {
                _this.authToken(token.Token);
                $("#fsExportDownloadForm").submit();
            }).fail(function (qXHR, textStatus, errorThrown) { return messagePublisher.reportError("Could not get Single Auth Token for export.", errorThrown); });
        };
        return exportDatabase;
    })(viewModelBase);
    return exportDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/tasks/importFilesystem", ["require", "exports", "viewmodels/viewModelBase", "common/messagePublisher", "commands/filesystem/importFilesystemCommand", "commands/operations/getOperationStatusCommand", "commands/filesystem/fsCheckSufficientDiskSpaceCommand"], function (require, exports, viewModelBase, messagePublisher, importFilesystemCommand, getOperationStatusCommand, fsCheckSufficientDiskSpaceCommand) {
    var importDatabase = (function (_super) {
        __extends(importDatabase, _super);
        function importDatabase() {
            _super.apply(this, arguments);
            this.batchSize = ko.observable(1024);
            this.stripReplicationInformation = ko.observable(false);
            this.shouldDisableVersioningBundle = ko.observable(false);
            this.hasFileSelected = ko.observable(false);
            this.importedFileName = ko.observable();
            this.isUploading = false;
            this.filePickerTag = "#importFilesystemFilePicker";
        }
        importDatabase.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.updateHelpLink("YD9M1R");
            var fs = this.activeFilesystem();
            var importStatus = fs.importStatus();
            if (importStatus === "Uploading 100%") {
                fs.importStatus("");
            }
        };
        importDatabase.prototype.canDeactivate = function (isClose) {
            _super.prototype.canDeactivate.call(this, isClose);
            if (this.isUploading) {
                this.confirmationMessage("Upload is in progress", "Please wait until uploading is complete.", ['OK']);
                return false;
            }
            return true;
        };
        importDatabase.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                ko.postbox.subscribe("UploadProgress", function (percentComplete) { return _this.activeFilesystem().importStatus("Uploading " + percentComplete.toFixed(2).replace(/\.0*$/, '') + "%"); }),
                ko.postbox.subscribe("ChangesApiReconnected", function (fs) {
                    fs.importStatus("");
                    fs.isImporting(false);
                    _this.isUploading = false;
                })
            ];
        };
        importDatabase.prototype.fileSelected = function (fileName) {
            var _this = this;
            var fs = this.activeFilesystem();
            var isFileSelected = !!$.trim(fileName);
            var importFileName = $(this.filePickerTag).val().split(/(\\|\/)/g).pop();
            if (isFileSelected) {
                var fileInput = document.querySelector(this.filePickerTag);
                new fsCheckSufficientDiskSpaceCommand(fileInput.files[0].size, this.activeFilesystem())
                    .execute()
                    .done(function () {
                    _this.hasFileSelected(isFileSelected);
                    _this.importedFileName(importFileName);
                    fs.importStatus("");
                })
                    .fail(function () {
                    fs.importStatus("No sufficient diskspace for import, consider using Raven.Smuggler.exe directly.");
                    _this.hasFileSelected(false);
                    _this.importedFileName("");
                });
            }
        };
        importDatabase.prototype.importFs = function () {
            var _this = this;
            var fs = this.activeFilesystem();
            fs.isImporting(true);
            this.isUploading = true;
            fs.importStatus("Uploading 0%");
            var formData = new FormData();
            var fileInput = document.querySelector(this.filePickerTag);
            formData.append("file", fileInput.files[0]);
            new importFilesystemCommand(formData, this.batchSize(), this.stripReplicationInformation(), this.shouldDisableVersioningBundle(), this.activeFilesystem())
                .execute()
                .done(function (result) {
                var operationId = result.OperationId;
                _this.waitForOperationToComplete(fs, operationId);
                fs.importStatus("Processing uploaded file");
            })
                .fail(function () { return fs.importStatus(""); })
                .always(function () { return _this.isUploading = false; });
        };
        importDatabase.prototype.waitForOperationToComplete = function (fs, operationId) {
            var _this = this;
            new getOperationStatusCommand(fs, operationId)
                .execute()
                .done(function (result) { return _this.importStatusRetrieved(fs, operationId, result); });
        };
        importDatabase.prototype.importStatusRetrieved = function (fs, operationId, result) {
            var _this = this;
            if (result.Completed) {
                if (result.ExceptionDetails == null && result.State != null && result.State.Progress != null) {
                    this.hasFileSelected(false);
                    $(this.filePickerTag).val('');
                    fs.importStatus("Last import was from '" + this.importedFileName() + "', " + result.State.Progress.toLocaleLowerCase());
                    messagePublisher.reportSuccess("Successfully imported data to " + fs.name);
                }
                else {
                    fs.importStatus("");
                    messagePublisher.reportError("Failed to import data!", result.ExceptionDetails);
                }
                fs.isImporting(false);
            }
            else {
                if (result.State && result.State.Progress) {
                    fs.importStatus("Processing uploaded file, " + result.State.Progress.toLocaleLowerCase());
                }
                setTimeout(function () { return _this.waitForOperationToComplete(fs, operationId); }, 1000);
            }
        };
        return importDatabase;
    })(viewModelBase);
    return importDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/filesystem/tasks/tasks", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var tasks = (function (_super) {
        __extends(tasks, _super);
        function tasks() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.appUrls = appUrl.forCurrentFilesystem();
            var importFilesystemUrl = ko.computed(function () { return appUrl.forImportFilesystem(_this.activeFilesystem()); });
            var exportFilesystemUrl = ko.computed(function () { return appUrl.forExportFilesystem(_this.activeFilesystem()); });
            var routeArray = [
                { route: ["filesystems/tasks", "filesystems/tasks/importFilesystem"], moduleId: "viewmodels/filesystem/tasks/importFilesystem", title: "Import File System", nav: true, hash: importFilesystemUrl },
                { route: "filesystems/tasks/exportFilesystem", moduleId: "viewmodels/filesystem/tasks/exportFilesystem", title: "Export File System", nav: true, hash: exportFilesystemUrl }
            ];
            this.router = durandalRouter.createChildRouter()
                .map(routeArray)
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return tasks;
    })(viewModelBase);
    return tasks;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/adminLogs", ["require", "exports", "durandal/app", "viewmodels/viewModelBase", "common/adminLogsClient", "common/fileDownloader", "viewmodels/manage/adminLogsConfigureDialog", "models/database/debug/adminLogsConfig", "commands/auth/getSingleAuthTokenCommand", "models/database/debug/adminLogsConfigEntry", 'common/appUrl', "viewmodels/shell"], function (require, exports, app, viewModelBase, adminLogsClient, fileDownloader, adminLogsConfigureDialog, adminLogsConfig, getSingleAuthTokenCommand, adminLogsConfigEntry, appUrl, shell) {
    var adminLogs = (function (_super) {
        __extends(adminLogs, _super);
        function adminLogs() {
            var _this = this;
            _super.apply(this, arguments);
            this.adminLogsClient = ko.observable(null);
            this.pendingLogs = [];
            this.keepDown = ko.observable(false);
            this.rawLogs = ko.observable([]);
            this.connected = ko.observable(false);
            this.entriesCount = ko.computed(function () { return _this.rawLogs().length; });
            this.adminLogsConfig = ko.observable();
            this.isForbidden = ko.observable();
        }
        adminLogs.prototype.canActivate = function (args) {
            this.isForbidden(shell.isGlobalAdmin() === false);
            return true;
        };
        adminLogs.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('57BGF7');
        };
        adminLogs.prototype.redraw = function () {
            if (this.pendingLogs.length > 0) {
                var pendingCopy = this.pendingLogs;
                this.pendingLogs = [];
                var logsAsText = "";
                pendingCopy.forEach(function (log) {
                    var line = log.TimeStamp + ";" + log.Level.toUpperCase() + ";" + log.Database + ";" + log.LoggerName + ";" + log.Message + (log.Exception || "") + "\n";
                    if (log.StackTrace != null) {
                        line += log.StackTrace + "\n\n";
                    }
                    logsAsText += line;
                });
                // text: allows us to escape values
                $("<div/>").text(logsAsText).appendTo("#rawLogsContainer pre");
                this.rawLogs().pushAll(pendingCopy);
                this.rawLogs.valueHasMutated();
                if (this.keepDown()) {
                    var logsPre = document.getElementById('adminLogsPre');
                    logsPre.scrollTop = logsPre.scrollHeight;
                }
            }
        };
        adminLogs.prototype.clearLogs = function () {
            this.pendingLogs = [];
            this.rawLogs([]);
            $("#rawLogsContainer pre").empty();
        };
        adminLogs.prototype.defaultLogsConfig = function () {
            var logConfig = new adminLogsConfig();
            logConfig.maxEntries(10000);
            logConfig.entries.push(new adminLogsConfigEntry("Raven.", "Debug", false));
            return logConfig;
        };
        adminLogs.prototype.configureConnection = function () {
            var _this = this;
            this.intervalId = setInterval(function () { this.redraw(); }.bind(this), 1000);
            var currentConfig = this.adminLogsConfig() ? this.adminLogsConfig().clone() : this.defaultLogsConfig();
            var adminLogsConfigViewModel = new adminLogsConfigureDialog(currentConfig);
            app.showDialog(adminLogsConfigViewModel);
            adminLogsConfigViewModel.configurationTask.done(function (x) {
                _this.adminLogsConfig(x);
                _this.reconnect();
            });
        };
        adminLogs.prototype.connect = function () {
            var _this = this;
            if (!!this.adminLogsClient()) {
                this.reconnect();
                return;
            }
            if (!this.adminLogsConfig()) {
                this.configureConnection();
                return;
            }
            var tokenDeferred = $.Deferred();
            if (!this.adminLogsConfig().singleAuthToken()) {
                new getSingleAuthTokenCommand(appUrl.getSystemDatabase(), true)
                    .execute()
                    .done(function (tokenObject) {
                    _this.adminLogsConfig().singleAuthToken(tokenObject);
                    tokenDeferred.resolve();
                })
                    .fail(function () {
                    app.showMessage("You are not authorized to trace this resource", "Authorization error");
                });
            }
            else {
                tokenDeferred.resolve();
            }
            tokenDeferred.done(function () {
                _this.adminLogsClient(new adminLogsClient(_this.adminLogsConfig().singleAuthToken().Token));
                _this.adminLogsClient().connect();
                var categoriesConfig = _this.adminLogsConfig().entries().map(function (e) { return e.toDto(); });
                _this.adminLogsClient().configureCategories(categoriesConfig);
                _this.adminLogsClient().connectionOpeningTask.done(function () {
                    _this.connected(true);
                    _this.adminLogsClient().watchAdminLogs(function (event) {
                        _this.onLogMessage(event);
                    });
                });
                _this.adminLogsConfig().singleAuthToken(null);
            });
        };
        adminLogs.prototype.disconnect = function () {
            var _this = this;
            if (!!this.adminLogsClient()) {
                this.adminLogsClient().dispose();
                return this.adminLogsClient().connectionClosingTask.then(function () {
                    _this.adminLogsClient(null);
                    _this.connected(false);
                });
            }
            else {
                app.showMessage("Cannot disconnect, connection does not exist", "Disconnect");
                return $.Deferred().reject();
            }
        };
        adminLogs.prototype.reconnect = function () {
            var _this = this;
            if (!this.adminLogsClient()) {
                this.connect();
            }
            else {
                this.disconnect().done(function () {
                    _this.connect();
                });
            }
        };
        adminLogs.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.logsContainer = document.getElementById("rawLogsContainer");
        };
        adminLogs.prototype.deactivate = function () {
            clearInterval(this.intervalId);
            if (this.adminLogsClient()) {
                this.adminLogsClient().dispose();
            }
        };
        adminLogs.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.disposeAdminLogsClient();
        };
        adminLogs.prototype.disposeAdminLogsClient = function () {
            var client = this.adminLogsClient();
            if (client) {
                client.dispose();
            }
        };
        adminLogs.prototype.onLogMessage = function (entry) {
            if (this.entriesCount() + this.pendingLogs.length < this.adminLogsConfig().maxEntries()) {
                this.pendingLogs.push(entry);
            }
            else {
                // stop logging
                var client = this.adminLogsClient();
                this.connected(false);
                client.dispose();
            }
        };
        adminLogs.prototype.exportLogs = function () {
            fileDownloader.downloadAsJson(this.rawLogs(), "logs.json");
        };
        adminLogs.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            if (this.keepDown()) {
                var logsPre = document.getElementById('adminLogsPre');
                logsPre.scrollTop = logsPre.scrollHeight;
            }
        };
        return adminLogs;
    })(viewModelBase);
    return adminLogs;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/adminLogsConfigureDialog", ["require", "exports", "durandal/app", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/auth/getSingleAuthTokenCommand", "models/database/debug/adminLogsConfigEntry", 'common/appUrl'], function (require, exports, app, dialog, dialogViewModelBase, getSingleAuthTokenCommand, adminLogsConfigEntry, appUrl) {
    var adminLogsConfigureDialog = (function (_super) {
        __extends(adminLogsConfigureDialog, _super);
        function adminLogsConfigureDialog(logConfig) {
            var _this = this;
            _super.call(this);
            this.logConfig = logConfig;
            this.configurationTask = $.Deferred();
            this.maxTableHeight = ko.observable();
            this.lineHeight = 51;
            this.maxTableHeight(Math.floor($(window).height() * 0.43));
            $(window).resize(function () {
                _this.maxTableHeight(Math.floor($(window).height() * 0.43));
                _this.alignBoxVertically();
            });
            this.isScrollNeeded = ko.computed(function () {
                var currentColumnsCount = _this.logConfig.entries().length;
                var currentColumnHeight = currentColumnsCount * _this.lineHeight;
                if (currentColumnHeight > _this.maxTableHeight()) {
                    return true;
                }
                return false;
            });
        }
        adminLogsConfigureDialog.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.form = $("#log-config-form");
        };
        adminLogsConfigureDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        adminLogsConfigureDialog.prototype.deactivate = function () {
            this.configurationTask.reject();
        };
        adminLogsConfigureDialog.prototype.startServerLogging = function () {
            var _this = this;
            var getTokenTask = new getSingleAuthTokenCommand(appUrl.getSystemDatabase(), true).execute();
            getTokenTask
                .done(function (tokenObject) {
                _this.logConfig.singleAuthToken(tokenObject);
                _this.configurationTask.resolve(_this.logConfig);
                dialog.close(_this);
            })
                .fail(function (e) {
                var response = JSON.parse(e.responseText);
                var msg = e.statusText;
                if ("Error" in response) {
                    msg += ": " + response.Error;
                }
                else if ("Reason" in response) {
                    msg += ": " + response.Reason;
                }
                app.showMessage(msg, "Error");
            });
        };
        adminLogsConfigureDialog.prototype.addCategory = function () {
            this.logConfig.entries.push(new adminLogsConfigEntry("Raven.", "Debug"));
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        adminLogsConfigureDialog.prototype.deleteRow = function (row) {
            this.logConfig.entries.remove(row);
            if (!this.isScrollNeeded()) {
                this.alignBoxVertically();
            }
        };
        adminLogsConfigureDialog.prototype.generateBindingInputId = function (index) {
            return 'binding-' + index;
        };
        return adminLogsConfigureDialog;
    })(dialogViewModelBase);
    return adminLogsConfigureDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/adminSettings", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "viewmodels/shell", "models/auth/license", "common/settingsAccessAuthorizer"], function (require, exports, durandalRouter, appUrl, viewModelBase, shell, license, settingsAccessAuthorizer) {
    var adminSettings = (function (_super) {
        __extends(adminSettings, _super);
        function adminSettings() {
            _super.call(this);
            this.router = null;
            this.isSystemDatabaseForbidden = ko.observable();
            this.settingsAccess = new settingsAccessAuthorizer();
            this.router = durandalRouter.createChildRouter()
                .makeRelative({
                fromParent: true,
                moduleId: 'viewmodels/manage'
            });
        }
        adminSettings.prototype.canActivate = function (args) {
            return true;
        };
        adminSettings.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            if (!license.licenseStatus()) {
                // we want to make sure license information is loaded before building route info
                return shell.fetchLicenseStatus()
                    .done(function () { return _this.init(); });
            }
            else {
                this.init();
            }
        };
        adminSettings.prototype.init = function () {
            var _this = this;
            this.docsForSystemUrl = appUrl.forDocuments(null, appUrl.getSystemDatabase());
            this.isSystemDatabaseForbidden((shell.isGlobalAdmin() || shell.canReadWriteSettings() || shell.canReadSettings()) === false);
            var canReadOrWrite = this.settingsAccess.canReadOrWrite();
            var isGlobalAdmin = shell.isGlobalAdmin();
            var apiKeyRoute = { route: ['', 'apiKeys'], moduleId: 'viewmodels/manage/apiKeys', title: 'API Keys', nav: true, hash: appUrl.forApiKeys(), enabled: canReadOrWrite };
            var windowsAuthRoute = { route: 'windowsAuth', moduleId: 'viewmodels/manage/windowsAuth', title: 'Windows Authentication', nav: true, hash: appUrl.forWindowsAuth(), enabled: canReadOrWrite };
            var clusterRoute = { route: 'cluster', moduleId: "viewmodels/manage/cluster", title: "Cluster", nav: true, hash: appUrl.forCluster(), enabled: canReadOrWrite };
            var globalConfigRoute = { route: 'globalConfig*details', moduleId: 'viewmodels/manage/globalConfig/globalConfig', title: 'Global Configuration', nav: true, hash: appUrl.forGlobalConfig(), enabled: canReadOrWrite };
            var serverSmuggling = { route: "serverSmuggling", moduleId: "viewmodels/manage/serverSmuggling", title: "Server Smuggling", nav: true, hash: appUrl.forServerSmugging(), enabled: isGlobalAdmin };
            var backupRoute = { route: 'backup', moduleId: 'viewmodels/manage/backup', title: 'Backup', nav: true, hash: appUrl.forBackup(), enabled: isGlobalAdmin };
            var compactRoute = { route: 'compact', moduleId: 'viewmodels/manage/compact', title: 'Compact', nav: true, hash: appUrl.forCompact(), enabled: isGlobalAdmin };
            var restoreRoute = { route: 'restore', moduleId: 'viewmodels/manage/restore', title: 'Restore', nav: true, hash: appUrl.forRestore(), enabled: isGlobalAdmin };
            var adminLogsRoute = { route: 'adminLogs', moduleId: 'viewmodels/manage/adminLogs', title: 'Admin Logs', nav: true, hash: appUrl.forAdminLogs(), enabled: isGlobalAdmin };
            var topologyRoute = { route: 'topology', moduleId: 'viewmodels/manage/topology', title: 'Server topology', nav: true, hash: appUrl.forServerTopology(), enabled: isGlobalAdmin };
            var trafficWatchRoute = { route: 'trafficWatch', moduleId: 'viewmodels/manage/trafficWatch', title: 'Traffic Watch', nav: true, hash: appUrl.forTrafficWatch(), enabled: isGlobalAdmin };
            var licenseInformation = { route: 'licenseInformation', moduleId: 'viewmodels/manage/licenseInformation', title: 'License Information', nav: true, hash: appUrl.forLicenseInformation(), enabled: canReadOrWrite };
            var debugInfoRoute = { route: 'debugInfo', moduleId: 'viewmodels/manage/infoPackage', title: 'Gather Debug Info', nav: true, hash: appUrl.forDebugInfo(), enabled: isGlobalAdmin };
            var ioTestRoute = { route: 'ioTest', moduleId: 'viewmodels/manage/ioTest', title: 'IO Test', nav: true, hash: appUrl.forIoTest(), enabled: isGlobalAdmin };
            var diskIoViewerRoute = { route: 'diskIoViewer', moduleId: 'viewmodels/manage/diskIoViewer', title: 'Disk IO Viewer', nav: true, hash: appUrl.forDiskIoViewer(), enabled: canReadOrWrite };
            var consoleRoute = { route: 'console', moduleId: "viewmodels/manage/console", title: "Administrator JS Console", nav: true, hash: appUrl.forAdminJsConsole(), enabled: isGlobalAdmin };
            var studioConfigRoute = { route: 'studioConfig', moduleId: 'viewmodels/manage/studioConfig', title: 'Studio Config', nav: true, hash: appUrl.forStudioConfig(), enabled: canReadOrWrite };
            var hotSpareRoute = { route: 'hotSpare', moduleId: 'viewmodels/manage/hotSpare', title: 'Hot Spare', nav: true, hash: appUrl.forHotSpare(), enabled: isGlobalAdmin };
            var routes = [
                apiKeyRoute,
                windowsAuthRoute,
                clusterRoute,
                globalConfigRoute,
                serverSmuggling,
                backupRoute,
                compactRoute,
                restoreRoute,
                adminLogsRoute,
                topologyRoute,
                trafficWatchRoute,
                licenseInformation,
                debugInfoRoute,
                ioTestRoute,
                diskIoViewerRoute,
                consoleRoute,
                studioConfigRoute
            ];
            if (license.licenseStatus().Attributes.hotSpare === "true")
                routes.push(hotSpareRoute);
            this.router = this.router
                .reset()
                .map(routes)
                .buildNavigationModel();
            adminSettings.adminSettingsRouter = this.router;
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        };
        adminSettings.prototype.navigateToSystemDatabase = function () {
            var _this = this;
            this.promptNavSystemDb(true).done(function () {
                var db = appUrl.getSystemDatabase();
                db.activate();
                var url = appUrl.forDocuments(null, db);
                _this.navigate(url);
            });
        };
        return adminSettings;
    })(viewModelBase);
    return adminSettings;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/apiKeys", ["require", "exports", "commands/auth/getApiKeysCommand", "commands/auth/saveApiKeysCommand", "models/auth/apiKey", "viewmodels/viewModelBase", "common/settingsAccessAuthorizer"], function (require, exports, getApiKeysCommand, saveApiKeysCommand, apiKey, viewModelBase, settingsAccessAuthorizer) {
    var apiKeys = (function (_super) {
        __extends(apiKeys, _super);
        function apiKeys() {
            var _this = this;
            _super.call(this);
            this.apiKeys = ko.observableArray();
            this.loadedApiKeys = ko.observableArray();
            this.searchText = ko.observable();
            this.noResults = ko.observable(false);
            this.settingsAccess = new settingsAccessAuthorizer();
            this.searchText.throttle(200).subscribe(function (value) { return _this.filterKeys(value); });
        }
        apiKeys.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            if (this.settingsAccess.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.fetchApiKeys().done(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        apiKeys.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('9CGJ4Y');
            this.dirtyFlag = new ko.DirtyFlag([this.apiKeys]);
            this.isSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        apiKeys.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            if (this.settingsAccess.isReadOnly()) {
                $('#manageApiKeys input:not(#apiKeysSearchInput)').attr('readonly', 'readonly');
                $('#manageApiKeys button').attr('disabled', 'true');
            }
            $("#manageApiKeys").on("keypress", 'input[name="databaseName"]', function (e) { return e.which !== 13; });
        };
        apiKeys.prototype.fetchApiKeys = function () {
            var _this = this;
            return new getApiKeysCommand()
                .execute()
                .done(function (results) {
                _this.apiKeys(results);
                _this.saveLoadedApiKeys(results);
                _this.apiKeys().forEach(function (key) {
                    _this.subscribeToObservableKeyName(key);
                });
            });
        };
        apiKeys.prototype.saveLoadedApiKeys = function (apiKeys) {
            // clones the apiKeys object
            this.loadedApiKeys = ko.mapping.fromJS(ko.mapping.toJS(apiKeys));
        };
        apiKeys.prototype.subscribeToObservableKeyName = function (key) {
            var _this = this;
            key.name.subscribe(function (previousApiKeyName) {
                var existingApiKeysExceptCurrent = _this.apiKeys().filter(function (k) { return k !== key && k.name() === previousApiKeyName; });
                if (existingApiKeysExceptCurrent.length === 1) {
                    existingApiKeysExceptCurrent[0].nameCustomValidity('');
                }
            }, this, "beforeChange");
            key.name.subscribe(function (newApiKeyName) {
                var errorMessage = '';
                var isApiKeyNameValid = newApiKeyName.indexOf("\\") === -1;
                var existingApiKeys = _this.apiKeys().filter(function (k) { return k !== key && k.name() === newApiKeyName; });
                if (!isApiKeyNameValid) {
                    errorMessage = "API Key name cannot contain '\\'";
                }
                else if (existingApiKeys.length > 0) {
                    errorMessage = "API key name already exists!";
                }
                key.nameCustomValidity(errorMessage);
            });
        };
        apiKeys.prototype.createNewApiKey = function () {
            var newApiKey = apiKey.empty();
            this.subscribeToObservableKeyName(newApiKey);
            newApiKey.generateSecret();
            this.apiKeys.unshift(newApiKey);
        };
        apiKeys.prototype.removeApiKey = function (key) {
            this.apiKeys.remove(key);
        };
        apiKeys.prototype.saveChanges = function () {
            var _this = this;
            this.apiKeys().forEach(function (key) { return key.setIdFromName(); });
            var apiKeysNamesArray = this.apiKeys().map(function (key) { return key.name(); });
            var deletedApiKeys = this.loadedApiKeys().filter(function (key) { return !apiKeysNamesArray.contains(key.name()); });
            deletedApiKeys.forEach(function (key) { return key.setIdFromName(); });
            new saveApiKeysCommand(this.apiKeys(), deletedApiKeys)
                .execute()
                .done(function (result) {
                _this.updateKeys(result);
                _this.saveLoadedApiKeys(_this.apiKeys());
                _this.dirtyFlag().reset();
            });
        };
        apiKeys.prototype.updateKeys = function (serverKeys) {
            this.apiKeys().forEach(function (key) {
                var serverKey = serverKeys.first(function (k) { return k.Key === key.getId(); });
                if (serverKey) {
                    key.__metadata.etag = serverKey.Etag;
                    key.__metadata.lastModified = serverKey.Metadata["Last-Modified"];
                }
            });
        };
        apiKeys.prototype.filterKeys = function (filter) {
            var filterLower = filter.toLowerCase().trim();
            var matches = 0;
            this.apiKeys().forEach(function (k) {
                var isEmpty = filterLower.length === 0;
                var isMatch = k.name() != null && k.name().toLowerCase().indexOf(filterLower) !== -1;
                var visible = isEmpty || isMatch;
                k.visible(visible);
                if (visible) {
                    matches++;
                }
            });
            this.noResults(matches === 0);
        };
        return apiKeys;
    })(viewModelBase);
    return apiKeys;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/backup", ["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell", "commands/maintenance/backupDatabaseCommand", "commands/filesystem/backupFilesystemCommand", "commands/counter/backupCounterStorageCommand", "commands/resources/getResourceDrives"], function (require, exports, viewModelBase, shell, backupDatabaseCommand, backupFilesystemCommand, backupCounterStorageCommand, getResourceDrives) {
    var resourceBackup = (function () {
        function resourceBackup(type, resources) {
            var _this = this;
            this.type = type;
            this.resources = resources;
            this.incremental = ko.observable(false);
            this.resourceName = ko.observable('');
            this.backupLocation = ko.observable('');
            this.backupStatusMessages = ko.observableArray();
            this.isBusy = ko.observable();
            this.keepDown = ko.observable(false);
            this.drivesForCurrentResource = ko.observable([]);
            this.displaySameDriveWarning = ko.computed(function () {
                var resourceDrives = _this.drivesForCurrentResource();
                var location = _this.backupLocation().toLowerCase().substr(0, 3);
                return resourceDrives.indexOf(location) !== -1;
            });
            this.has40Features = ko.computed(function () { return shell.has40Features(); });
            this.resourcesNames = ko.computed(function () { return resources().map(function (rs) { return rs.name; }); });
            this.fullTypeName = ko.computed(function () {
                var rs = resources();
                if (rs.length > 0) {
                    return rs[0].fullTypeName;
                }
                return null;
            });
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var foundRs = _this.resources().first(function (rs) { return newResourceName === rs.name && rs.type === _this.type; });
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = _this.fullTypeName() + " name doesn't exist!";
                }
                return errorMessage;
            });
            this.resourceName.throttle(200).subscribe(function (resource) {
                var foundRs = _this.resources().first(function (rs) { return resource === rs.name && rs.type === _this.type; });
                if (foundRs) {
                    new getResourceDrives(foundRs.name, foundRs.type.toString()).execute()
                        .done(function (drives) {
                        _this.drivesForCurrentResource(drives);
                    });
                }
                else {
                    _this.drivesForCurrentResource([]);
                }
            });
        }
        resourceBackup.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceBackup.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceBackup.prototype.updateBackupStatus = function (newBackupStatus) {
            this.backupStatusMessages(newBackupStatus.Messages);
            this.isBusy(newBackupStatus.IsRunning);
            this.forceKeepDown();
        };
        return resourceBackup;
    })();
    var backupDatabase = (function (_super) {
        __extends(backupDatabase, _super);
        function backupDatabase() {
            _super.apply(this, arguments);
            this.dbBackupOptions = new resourceBackup(TenantType.Database, shell.databases);
            this.fsBackupOptions = new resourceBackup(TenantType.FileSystem, shell.fileSystems);
            this.csBackupOptions = new resourceBackup(TenantType.CounterStorage, shell.counterStorages);
            this.isForbidden = ko.observable();
        }
        backupDatabase.prototype.canActivate = function (args) {
            this.isForbidden(!shell.isGlobalAdmin());
            return true;
        };
        backupDatabase.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("FT7RV6");
        };
        backupDatabase.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which !== 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which !== 13; });
        };
        backupDatabase.prototype.startDbBackup = function () {
            var backupOptions = this.dbBackupOptions;
            backupOptions.isBusy(true);
            var dbToBackup = shell.databases.first(function (db) { return db.name === backupOptions.resourceName(); });
            new backupDatabaseCommand(dbToBackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.dbBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        backupDatabase.prototype.startFsBackup = function () {
            var backupOptions = this.fsBackupOptions;
            backupOptions.isBusy(true);
            var fsToBackup = shell.fileSystems.first(function (fs) { return fs.name === backupOptions.resourceName(); });
            new backupFilesystemCommand(fsToBackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.fsBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        backupDatabase.prototype.startCsBackup = function () {
            var backupOptions = this.csBackupOptions;
            backupOptions.isBusy(true);
            var csToBackup = shell.counterStorages.first(function (cs) { return cs.name === backupOptions.resourceName(); });
            new backupCounterStorageCommand(csToBackup, backupOptions.backupLocation(), backupOptions.updateBackupStatus.bind(this.csBackupOptions), backupOptions.incremental())
                .execute()
                .always(function () { return backupOptions.isBusy(false); });
        };
        return backupDatabase;
    })(viewModelBase);
    return backupDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/cluster", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "commands/database/cluster/getClusterTopologyCommand", "common/messagePublisher", "models/database/cluster/nodeConnectionInfo", "viewmodels/manage/editNodeConnectionInfoDialog", "durandal/app", "commands/resources/getDatabaseStatsCommand", "commands/database/debug/getStatusDebugConfigCommand", "commands/database/cluster/extendRaftClusterCommand", "commands/database/cluster/initializeNewClusterCommand", "commands/database/cluster/leaveRaftClusterCommand", "commands/database/cluster/removeClusteringCommand", "commands/database/cluster/saveClusterConfigurationCommand", "commands/database/cluster/updateRaftClusterCommand", "commands/database/cluster/getClusterNodesStatusCommand", "viewmodels/shell", "common/bindingHelpers/autoRefreshBindingHandler", "models/auth/license", "common/settingsAccessAuthorizer", "commands/database/cluster/changeNodeVotingModeCommand"], function (require, exports, viewModelBase, appUrl, getClusterTopologyCommand, messagePublisher, nodeConnectionInfo, editNodeConnectionInfoDialog, app, getDatabaseStatsCommand, getStatusDebugConfigCommand, extendRaftClusterCommand, initializeNewClusterCommand, leaveRaftClusterCommand, removeClusteringCommand, saveClusterConfigurationCommand, updateRaftClusterCommand, getClusterNodesStatusCommand, shell, autoRefreshBindingHandler, license, settingsAccessAuthorizer, changeNodeVotingModeCommand) {
    var cluster = (function (_super) {
        __extends(cluster, _super);
        function cluster() {
            var _this = this;
            _super.call(this);
            this.topology = ko.observable();
            this.systemDatabaseId = ko.observable();
            this.serverUrl = ko.observable();
            this.canCreateCluster = ko.computed(function () { return !license.licenseStatus().IsCommercial || license.licenseStatus().Attributes.clustering === "true"; });
            this.developerLicense = ko.computed(function () { return !license.licenseStatus().IsCommercial; });
            this.settingsAccess = new settingsAccessAuthorizer();
            autoRefreshBindingHandler.install();
            this.clusterMode = ko.computed(function () {
                return _this.topology() && _this.topology().clusterMode();
            });
        }
        cluster.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            if (this.settingsAccess.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                var db = appUrl.getSystemDatabase();
                $.when(this.fetchClusterTopology(db), this.fetchDatabaseId(db), this.fetchServerUrl(db))
                    .done(function () {
                    deferred.resolve({ can: true });
                    if (_this.clusterMode()) {
                        _this.fetchStatus(db);
                    }
                })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forAdminSettings() }); });
            }
            return deferred;
        };
        cluster.prototype.refresh = function () {
            var _this = this;
            return this.fetchClusterTopology(appUrl.getSystemDatabase())
                .done(function () { return _this.fetchStatus(appUrl.getSystemDatabase()); });
        };
        cluster.prototype.fetchClusterTopology = function (db) {
            var _this = this;
            return new getClusterTopologyCommand(db)
                .execute()
                .done(function (topo) {
                _this.topology(topo);
            })
                .fail(function () { return messagePublisher.reportError("Unable to fetch cluster topology"); });
        };
        cluster.prototype.fetchDatabaseId = function (db) {
            var _this = this;
            return new getDatabaseStatsCommand(db)
                .execute()
                .done(function (stats) {
                _this.systemDatabaseId(stats.DatabaseId);
            });
        };
        cluster.prototype.fetchServerUrl = function (db) {
            var _this = this;
            return new getStatusDebugConfigCommand(db)
                .execute()
                .done(function (config) { return _this.serverUrl(config.ServerUrl); });
        };
        cluster.prototype.fetchStatus = function (db) {
            var _this = this;
            return new getClusterNodesStatusCommand(db)
                .execute()
                .done(function (status) { return _this.updateNodesStatus(status); });
        };
        cluster.prototype.updateNodesStatus = function (status) {
            var _this = this;
            status.forEach(function (nodeStatus) {
                var nci = _this.topology().allNodes().first(function (n) { return n.uri() === nodeStatus.Uri; });
                if (nci) {
                    nci.status(nodeStatus.Status);
                }
            });
        };
        cluster.prototype.addAnotherServerToCluster = function (forcedAdd) {
            var _this = this;
            var newNode = nodeConnectionInfo.empty();
            var dialog = new editNodeConnectionInfoDialog(newNode, false);
            dialog
                .onExit()
                .done(function (nci) {
                new extendRaftClusterCommand(appUrl.getSystemDatabase(), nci.toDto(), false, forcedAdd)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
            app.showDialog(dialog);
        };
        cluster.prototype.removeClustering = function () {
            var _this = this;
            this.confirmationMessage("Are you sure?", "You are about to clear cluster information on this server.")
                .done(function () {
                new removeClusteringCommand(appUrl.getSystemDatabase())
                    .execute()
                    .done(function () { return setTimeout(function () {
                    _this.refresh();
                    shell.clusterMode(false);
                }, 500); });
            });
        };
        cluster.prototype.createCluster = function () {
            var _this = this;
            var newNode = nodeConnectionInfo.empty();
            newNode.name(this.systemDatabaseId());
            newNode.uri(this.serverUrl());
            var dialog = new editNodeConnectionInfoDialog(newNode, false);
            dialog
                .onExit()
                .done(function (nci) {
                new extendRaftClusterCommand(appUrl.getSystemDatabase(), nci.toDto(), true, false)
                    .execute()
                    .done(function () {
                    shell.clusterMode(true);
                    setTimeout(function () { return _this.refresh(); }, 500);
                    new saveClusterConfigurationCommand({ EnableReplication: true }, appUrl.getSystemDatabase())
                        .execute();
                });
            });
            app.showDialog(dialog);
        };
        cluster.prototype.initializeNewCluster = function () {
            var _this = this;
            this.confirmationMessage("Are you sure?", "You are about to initialize new cluster on this server.")
                .done(function () {
                new initializeNewClusterCommand(appUrl.getSystemDatabase())
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        cluster.prototype.editNode = function (node) {
            var _this = this;
            var dialog = new editNodeConnectionInfoDialog(node, true);
            dialog.onExit()
                .done(function (nci) {
                new updateRaftClusterCommand(appUrl.getSystemDatabase(), nci)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
            app.showDialog(dialog);
        };
        cluster.prototype.leaveCluster = function (node) {
            var _this = this;
            this.confirmationMessage("Are you sure?", "You are removing node " + node.uri() + " from cluster.")
                .done(function () {
                new leaveRaftClusterCommand(appUrl.getSystemDatabase(), node.toDto())
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        cluster.prototype.promoteNodeToVoter = function (node) {
            var _this = this;
            var nodeAsDto = node.toDto();
            nodeAsDto.IsNoneVoter = false;
            this.confirmationMessage("Are you sure?", "You are promoting node " + node.uri() + " to voter.")
                .done(function () {
                new changeNodeVotingModeCommand(appUrl.getSystemDatabase(), nodeAsDto)
                    .execute()
                    .done(function () { return setTimeout(function () { return _this.refresh(); }, 500); });
            });
        };
        return cluster;
    })(viewModelBase);
    return cluster;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/compact", ["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell", "models/resources/database", "models/filesystem/filesystem", "commands/maintenance/startCompactCommand", "commands/filesystem/startCompactCommand"], function (require, exports, viewModelBase, shell, database, filesystem, startDbCompactCommand, startFsCompactCommand) {
    var resourceCompact = (function () {
        function resourceCompact(parent, type, resources) {
            var _this = this;
            this.parent = parent;
            this.type = type;
            this.resources = resources;
            this.resourceName = ko.observable('');
            this.compactStatusMessages = ko.observableArray();
            this.compactStatusLastUpdate = ko.observable();
            this.keepDown = ko.observable(false);
            this.resourcesNames = ko.computed(function () { return resources().map(function (rs) { return rs.name; }); });
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = '';
                var newResourceName = _this.resourceName();
                var foundRs = _this.resources().first(function (rs) { return newResourceName === rs.name; });
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = (_this.type === database.type ? "Database" : "File system") + " name doesn't exist!";
                }
                return errorMessage;
            });
        }
        resourceCompact.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceCompact.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceCompact.prototype.updateCompactStatus = function (newCompactStatus) {
            this.compactStatusMessages(newCompactStatus.Messages);
            this.compactStatusLastUpdate(newCompactStatus.LastProgressMessage);
            this.forceKeepDown();
            this.parent.isBusy(newCompactStatus.State === "Running");
        };
        return resourceCompact;
    })();
    var compact = (function (_super) {
        __extends(compact, _super);
        function compact() {
            _super.apply(this, arguments);
            this.dbCompactOptions = new resourceCompact(this, database.type, shell.databases);
            this.fsCompactOptions = new resourceCompact(this, filesystem.type, shell.fileSystems);
            this.isBusy = ko.observable();
            this.isForbidden = ko.observable();
        }
        compact.prototype.canActivate = function (args) {
            this.isForbidden(shell.isGlobalAdmin() === false);
            return true;
        };
        compact.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('7HZGOE');
        };
        compact.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which !== 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which !== 13; });
        };
        compact.prototype.startDbCompact = function () {
            this.isBusy(true);
            var self = this;
            new startDbCompactCommand(this.dbCompactOptions.resourceName(), self.dbCompactOptions.updateCompactStatus.bind(self.dbCompactOptions))
                .execute();
        };
        compact.prototype.startFsCompact = function () {
            this.isBusy(true);
            var self = this;
            new startFsCompactCommand(this.fsCompactOptions.resourceName(), self.fsCompactOptions.updateCompactStatus.bind(self.fsCompactOptions))
                .execute();
        };
        return compact;
    })(viewModelBase);
    return compact;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/console", ["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell", "common/bindingHelpers/aceEditorBindingHandler", "commands/maintenance/adminJsScriptCommand", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, shell, aceEditorBindingHandler, adminJsScriptCommand, settingsAccessAuthorizer) {
    var consoleJs = (function (_super) {
        __extends(consoleJs, _super);
        function consoleJs() {
            var _this = this;
            _super.call(this);
            this.resourceName = ko.observable('');
            this.isBusy = ko.observable();
            this.responseText = ko.observable();
            this.script = ko.observable();
            this.results = ko.observable();
            this.settingsAccess = new settingsAccessAuthorizer();
            this.predefinedSamples = ko.observableArray([]);
            this.predefinedSamples.push({ Name: "Get database stats", Code: "return database.Statistics;" });
            this.predefinedSamples.push({
                Name: "Get configuration values", Code: "return {" +
                    "\n	RaiseBatchLimit : database.Configuration.AvailableMemoryForRaisingBatchSizeLimit," +
                    "\n	ReduceBatchLimit: database.Configuration.MaxNumberOfItemsToReduceInSingleBatch" +
                    "\n};" });
            this.predefinedSamples.push({
                Name: "Change configuration on the fly", Code: "database.Configuration.DisableDocumentPreFetching = true;" +
                    "\ndatabase.Configuration.MaxNumberOfItemsToPreFetch = 1024;" +
                    "\ndatabase.Configuration.BulkImportBatchTimeout = System.TimeSpan.FromMinutes(13);"
            });
            this.predefinedSamples.push({ Name: "Run idle operations", Code: "database.RunIdleOperations();" });
            this.predefinedSamples.push({
                Name: "Put document",
                Code: "var doc = Raven.Json.Linq.RavenJObject.Parse('{ \"Name\" : \"Raven\" }');" +
                    "\nvar metadata = Raven.Json.Linq.RavenJObject.Parse('{ \"Raven-Entity-Name\" : \"Docs\" }');" +
                    "\n\ndatabase.Documents.Put('doc/1', null, doc, metadata, null);" });
            aceEditorBindingHandler.install();
            this.resourcesNames = ko.computed(function () { return shell.databases().map(function (rs) { return rs.name; }); });
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.resourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = '';
                var newResourceName = _this.resourceName();
                var foundRs = shell.databases().first(function (rs) { return newResourceName === rs.name && rs.type === TenantType.Database; });
                if (!foundRs && newResourceName.length > 0) {
                    errorMessage = "Database name doesn't exist!";
                }
                return errorMessage;
            });
        }
        consoleJs.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            $('form :input[name="databaseName"]').on("keypress", function (e) { return e.which != 13; });
            $('form :input[name="filesystemName"]').on("keypress", function (e) { return e.which != 13; });
        };
        consoleJs.prototype.executeJs = function () {
            var _this = this;
            this.isBusy(true);
            new adminJsScriptCommand(this.script(), this.resourceName())
                .execute()
                .done(function (result) {
                _this.results(JSON.stringify(result, null, 4));
            })
                .always(function () { _this.isBusy(false); });
        };
        consoleJs.prototype.fillWithSample = function (sample) {
            this.script(sample.Code);
        };
        return consoleJs;
    })(viewModelBase);
    return consoleJs;
});
/// <reference path="../../../Scripts/typings/d3/nvd3.d.ts" />
/// <reference path="../../../Scripts/typings/d3/d3.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/diskIoViewer", ["require", "exports", "viewmodels/viewModelBase", "d3/d3", "nvd3", "common/appUrl", "commands/maintenance/listDiskPerformanceRunsCommand", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/documents/deleteDocumentCommand", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, d3, nv, appUrl, listDiskPerformanceRunsCommand, getDocumentWithMetadataCommand, deleteDocumentCommand, settingsAccessAuthorizer) {
    var diskIoViewer = (function (_super) {
        __extends(diskIoViewer, _super);
        function diskIoViewer() {
            var _this = this;
            _super.call(this);
            this.isoFormat = d3.time.format.iso;
            this.settingsAccess = new settingsAccessAuthorizer();
            this.showChart = ko.observable(false);
            this.emptyReport = ko.observable(false);
            this.performanceRuns = ko.observableArray([]);
            this.currentPerformanceRun = ko.observable();
            this.currentPeformanceRunLabel = ko.computed(function () {
                var run = _this.currentPerformanceRun();
                if (run) {
                    return run.displayName;
                }
                return "select";
            });
            this.perDbReports = ko.observableArray([]);
            this.currentDbReport = ko.observable();
            this.currentDbReportLabel = ko.computed(function () {
                var report = _this.currentDbReport();
                if (report) {
                    return report.Name;
                }
                return "database name";
            });
            this.chart = null;
            this.chartData = [
                {
                    key: "Index Read",
                    values: []
                },
                {
                    key: "Index Write",
                    values: []
                },
                {
                    key: "Data read",
                    values: []
                },
                {
                    key: "Data write",
                    values: []
                }
            ];
            this.currentPerformanceRun.subscribe(function (v) {
                if (!v) {
                    _this.perDbReports([]);
                    _this.currentDbReport(undefined);
                    return;
                }
                new getDocumentWithMetadataCommand(v.documentId, appUrl.getSystemDatabase())
                    .execute()
                    .done(function (doc) {
                    _this.emptyReport(doc.Databases.length === 0);
                    _this.showChart(doc.Databases.length > 0);
                    _this.data = doc;
                    if (doc.Databases) {
                        _this.perDbReports(doc.Databases);
                        if (doc.Databases.length > 0) {
                            _this.currentDbReport(doc.Databases[0]);
                        }
                        else {
                            _this.currentDbReport(undefined);
                        }
                    }
                    else {
                        _this.perDbReports([]);
                    }
                });
            });
            this.currentDbReport.subscribe(function (v) {
                _this.updateData();
                _this.updateGraph();
            });
        }
        diskIoViewer.prototype.canActivate = function (args) {
            var _this = this;
            var deffered = $.Deferred();
            new listDiskPerformanceRunsCommand()
                .execute()
                .done(function (results) {
                _this.performanceRuns(results);
            })
                .always(function () { return deffered.resolve({ can: true }); });
            return deffered;
        };
        diskIoViewer.prototype.detached = function () {
            _super.prototype.detached.call(this);
            window.onresize = null; // FIX nvd3 event attached globally
        };
        diskIoViewer.prototype.updateData = function () {
            for (var i = 0; i < 4; i++) {
                this.chartData[i].values = [];
            }
            if (!this.currentDbReport()) {
                // workaround for empty data
                for (var i = 0; i < 4; i++) {
                    this.chartData[i].values.push({
                        x: new Date(),
                        y: 0
                    });
                }
                return;
            }
            var dbResults = this.currentDbReport().Results;
            for (var date in dbResults) {
                if (dbResults.hasOwnProperty(date)) {
                    var value = dbResults[date];
                    var parsedDate = this.isoFormat.parse(date);
                    var yValues = [0, 0, 0, 0];
                    for (var i = 0; i < value.length; i++) {
                        var item = value[i];
                        //find index in values array
                        if (item.WriteIoSizeInBytes > 0) {
                            if (item.PathType === "Data") {
                                yValues[3] = item.WriteIoSizeInBytes;
                            }
                            else if (item.PathType === "Index") {
                                yValues[1] = item.WriteIoSizeInBytes;
                            }
                        }
                        if (item.ReadIoSizeInBytes > 0) {
                            if (item.PathType === "Data") {
                                yValues[2] = item.ReadIoSizeInBytes;
                            }
                            else if (item.PathType === "Index") {
                                yValues[0] = item.ReadIoSizeInBytes;
                            }
                        }
                    }
                    for (var i = 0; i < yValues.length; i++) {
                        this.chartData[i].values.push({
                            x: parsedDate,
                            y: yValues[i] / 1024.0 / 1024.0
                        });
                    }
                }
            }
        };
        diskIoViewer.prototype.updateGraph = function () {
            var _this = this;
            if (this.chart === null) {
                nv.addGraph(function () {
                    var chart = nv.models.lineChart()
                        .margin({ left: 130 })
                        .useInteractiveGuideline(true)
                        .transitionDuration(350)
                        .showLegend(true)
                        .showYAxis(true)
                        .showXAxis(true)
                        .forceY([0]);
                    ;
                    chart.xAxis
                        .axisLabel('Time')
                        .tickFormat(function (d) { return d3.time.format('%H:%M:%S')(new Date(d)); });
                    chart.xScale(d3.time.scale());
                    chart.yAxis
                        .axisLabel('Mb')
                        .tickFormat(d3.format('.01f'));
                    nv.utils.windowResize(function () { chart.update(); });
                    return chart;
                }, function (chart) {
                    _this.chart = chart;
                    d3.select('#diskMonitorContainer svg')
                        .datum(_this.chartData)
                        .call(_this.chart);
                });
            }
            else {
                // just update data
                d3.select('#diskMonitorContainer svg')
                    .datum(this.chartData)
                    .call(this.chart);
            }
        };
        diskIoViewer.prototype.switchReport = function (report) {
            this.currentPerformanceRun(report);
        };
        diskIoViewer.prototype.switchDatabase = function (report) {
            this.currentDbReport(report);
        };
        diskIoViewer.prototype.deleteReport = function () {
            var _this = this;
            this.confirmationMessage("Are you sure?", "You are removing Disk IO Report: " + this.currentPeformanceRunLabel())
                .done(function () {
                var currentRun = _this.currentPerformanceRun();
                new deleteDocumentCommand(_this.currentPerformanceRun().documentId, appUrl.getSystemDatabase())
                    .execute();
                _this.performanceRuns.remove(currentRun);
                _this.currentPerformanceRun(null);
                _this.showChart(false);
            });
        };
        return diskIoViewer;
    })(viewModelBase);
    return diskIoViewer;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/editNodeConnectionInfoDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var editNodeConnectionInfoDialog = (function (_super) {
        __extends(editNodeConnectionInfoDialog, _super);
        function editNodeConnectionInfoDialog(node, isUpdate) {
            _super.call(this);
            this.nextTask = $.Deferred();
            this.nodeConnectionInfo = ko.observable();
            this.isUpdate = ko.observable();
            this.nodeConnectionInfo(node);
            this.isUpdate(isUpdate);
        }
        editNodeConnectionInfoDialog.prototype.onExit = function () {
            return this.nextTask.promise();
        };
        editNodeConnectionInfoDialog.prototype.cancel = function () {
            this.nextTask.reject();
            dialog.close(this);
        };
        editNodeConnectionInfoDialog.prototype.ok = function () {
            this.nextTask.resolve(this.nodeConnectionInfo());
            dialog.close(this);
        };
        return editNodeConnectionInfoDialog;
    })(dialogViewModelBase);
    return editNodeConnectionInfoDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/hotSpare", ["require", "exports", "viewmodels/viewModelBase", "commands/licensing/GetHotSpareInformation", "commands/licensing/testHotSpareCommand", "commands/licensing/activateHotSpareCommand", "viewmodels/shell"], function (require, exports, viewModelBase, getHotSpareInformation, testHotSpareCommand, activateHotSpareCommand, shell) {
    var hotSpare = (function (_super) {
        __extends(hotSpare, _super);
        function hotSpare() {
            _super.apply(this, arguments);
            this.runningOnExpiredLicense = ko.observable(false);
            this.activationMode = ko.observable();
            this.activationTime = ko.observable();
            this.remainingTestActivation = ko.observable();
            this.isActivationExpired = ko.observable(false);
        }
        hotSpare.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.fetchHotSpareInformation();
        };
        hotSpare.prototype.fetchHotSpareInformation = function () {
            var _this = this;
            new getHotSpareInformation().execute().done(function (res) {
                _this.activationMode(res.ActivationMode);
                _this.activationTime(res.ActivationTime);
                _this.remainingTestActivation(res.RemainingTestActivations);
            }).fail(function () {
                alert("Can't fetch license information");
            });
        };
        hotSpare.prototype.isTestEnabled = function () {
            return this.activationMode() === 'NotActivated';
        };
        hotSpare.prototype.testLicense = function () {
            new testHotSpareCommand().execute();
        };
        hotSpare.prototype.activateLicense = function () {
            var _this = this;
            var self = this;
            this.confirmationMessage("Hot Spare Activation", "This is a one time activation, valid for 96 hours, are you sure you want to activate the hot spare license?")
                .done(function () {
                new activateHotSpareCommand()
                    .execute()
                    .done(function () {
                    _this.fetchHotSpareInformation();
                    // refresh top navbar with 
                    shell.fetchStudioConfig();
                })
                    .fail(function (response) {
                    if (response.status === 403) {
                        self.isActivationExpired(true);
                    }
                });
            });
        };
        return hotSpare;
    })(viewModelBase);
    return hotSpare;
});
/// <reference path="../../../Scripts/typings/jszip/jszip.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/infoPackage", ["require", "exports", "durandal/app", 'd3/d3', 'nvd3', 'jszip/jszip', "common/messagePublisher", "common/appUrl", "common/svgDownloader", "common/fileDownloader", 'commands/database/debug/getInfoPackage', "viewmodels/viewModelBase", "viewmodels/manage/infoPackageImport", "viewmodels/shell"], function (require, exports, app, d3, nv, jszip, messagePublisher, appUrl, svgDownloader, fileDownloader, getInfoPackage, viewModelBase, infoPackageImport, shell) {
    var stackInfo = (function () {
        function stackInfo(ThreadIds, StackTrace) {
            var _this = this;
            this.ThreadIds = ThreadIds;
            this.StackTrace = StackTrace;
            this.boxHeight = function () {
                return _this.StackTrace.length * stackInfo.lineHeight + 2 * stackInfo.boxPadding;
            };
            this.stackWithShortcuts = function () {
                return _this.StackTrace.map(function (v) {
                    return {
                        short: stackInfo.shortName(v),
                        full: v
                    };
                });
            };
        }
        stackInfo.shortName = function (v) {
            var withoutArgs = v.replace(/\(.*?\)/g, '');
            if (withoutArgs.contains('+')) {
                return withoutArgs.replace(/.*\.(.*\+.*)/, '$1');
            }
            else {
                return withoutArgs.replace(/.*\.([^\.]+\.[^\.]+)$/, '$1');
            }
        };
        stackInfo.isUserCode = function (line) {
            return line.startsWith("Raven") || line.startsWith("Voron");
        };
        stackInfo.boxPadding = 8;
        stackInfo.lineHeight = 12;
        stackInfo.headerSize = 16;
        return stackInfo;
    })();
    var infoPackage = (function (_super) {
        __extends(infoPackage, _super);
        function infoPackage() {
            var _this = this;
            _super.call(this);
            this.node = null; // nodes selection
            this.link = null; // links selection
            this.nodes = [];
            this.svg = null;
            this.svgDefs = null;
            this.graph = null;
            this.infoPackage = ko.observable();
            this.infoPackageFilename = ko.observable();
            this.fetchException = ko.observable();
            this.fetchExceptionDetails = ko.observable();
            this.showMoreDetails = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showMoreDetailsButton = ko.computed(function () {
                var hasDetails = !!_this.fetchExceptionDetails();
                var detailsDisplayed = _this.showMoreDetails();
                return hasDetails && !detailsDisplayed;
            });
            this.stacksJson = ko.observable(null);
            this.hasFetchException = ko.computed(function () { return !!_this.fetchException(); });
            this.hasInfoPackage = ko.computed(function () { return !!_this.infoPackage(); });
            this.hasStackDump = ko.computed(function () { return !!_this.stacksJson(); });
            this.hasSaveAsPngSupport = ko.computed(function () {
                return !(navigator && navigator.msSaveBlob);
            });
            this.isForbidden = ko.observable();
            this.appUrls = appUrl.forCurrentDatabase();
            this.adminView = ko.computed(function () {
                var activeDb = _this.activeDatabase();
                var appUrls = _this.appUrls;
                return (!!activeDb && activeDb.isSystem || !!appUrls && appUrls.isAreaActive('admin')());
            });
            this.isForbidden(shell.isGlobalAdmin() === false);
        }
        infoPackage.prototype.canActivate = function (args) {
            return true;
        };
        infoPackage.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.updateHelpLink('KVLC4Y');
            var self = this;
            $("#stacksContainer").resize();
            this.diagonal = d3.svg.diagonal().projection(function (d) { return [self.xScale(d.x), self.yScale(d.y)]; });
        };
        infoPackage.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#stacksContainer").off('DynamicHeightSet');
            nv.tooltip.cleanup();
        };
        infoPackage.prototype.splitAndCollateStacks = function (stacks) {
            var _this = this;
            if (stacks.length === 1) {
                return stacks;
            }
            var grouped = d3.nest().key(function (d) { return d.StackTrace[0]; }).entries(stacks);
            // for each group find common stack
            return grouped.map(function (kv) {
                var sharedStacks = kv.values;
                var minDepth = d3.min(sharedStacks, function (s) { return s.StackTrace.length; });
                outer: for (var depth = 0; depth < minDepth; depth++) {
                    var currentStack = sharedStacks[0].StackTrace[depth];
                    for (var i = 1; i < sharedStacks.length; i++) {
                        if (currentStack !== sharedStacks[i].StackTrace[depth]) {
                            break outer;
                        }
                    }
                }
                // extract shared stack:
                var sharedStack = new stackInfo([], sharedStacks[0].StackTrace.slice(0, depth));
                // remove shared stack from all stacks and recurse
                var strippedStacks = sharedStacks.map(function (s) { return new stackInfo(s.ThreadIds, s.StackTrace.slice(depth)); }).filter(function (s) { return s.StackTrace.length > 0; });
                sharedStack.children = _this.splitAndCollateStacks(strippedStacks);
                sharedStack.ThreadIds = d3.merge(sharedStacks.map(function (s) { return s.ThreadIds; }));
                return sharedStack;
            });
        };
        infoPackage.prototype.cumulativeSumWithPadding = function (input, padding) {
            var currentSum = 0;
            var output = [0];
            for (var i = 0; i < input.length; i++) {
                var offset = padding + input[i];
                output.push(currentSum + offset);
                currentSum += offset;
            }
            return output;
        };
        infoPackage.prototype.getTooltip = function (data) {
            return data.full;
        };
        infoPackage.prototype.updateGraph = function (roots) {
            var _this = this;
            var self = this;
            $("#parallelStacks").empty();
            this.svgDefs = d3.select("#parallelStacks").append("defs");
            var zoom = d3.behavior.zoom().scale(0.25).scaleExtent([0.2, 1.5]).on("zoom", this.zoom.bind(self));
            this.svg = d3.select("#parallelStacks")
                .append("g")
                .call(zoom)
                .append("g");
            zoom.event(d3.select("#parallelStacks > g"));
            this.svg.append("rect")
                .attr("class", "overlay");
            this.node = this.svg.selectAll(".node");
            this.link = this.svg.selectAll(".link");
            var fakeRoot = new stackInfo([], []);
            fakeRoot.children = roots;
            this.graph = d3.layout.tree().nodeSize([infoPackage.maxBoxWidth + 20, 100]);
            this.nodes = this.graph.nodes(fakeRoot).filter(function (d) { return d.depth > 0; });
            var maxBoxHeightOnDepth = d3.nest()
                .key(function (d) { return d.depth; })
                .sortKeys(d3.ascending)
                .rollup(function (leaves) { return d3.max(leaves, function (l) { return l.boxHeight(); }); })
                .entries(this.nodes)
                .map(function (v) { return v.values; });
            var cumulative = this.cumulativeSumWithPadding(maxBoxHeightOnDepth, infoPackage.boxVerticalPadding);
            this.height = cumulative[cumulative.length - 1];
            var extent = d3.extent(this.nodes, function (node) { return node.x; });
            extent[1] += infoPackage.maxBoxWidth;
            this.width = extent[1] - extent[0];
            d3.select(".overlay")
                .attr("width", self.width)
                .attr("height", self.height);
            var halfBoxShift = infoPackage.maxBoxWidth / 2 + 10; // little padding
            this.xScale = d3.scale.linear().domain([extent[0] - halfBoxShift, extent[1] - halfBoxShift]).range([0, this.width]);
            this.yScale = d3.scale.linear().domain([0, this.height]).range([this.height, 0]);
            var yDepthScale = d3.scale.linear().domain(d3.range(1, cumulative.length + 2, 1)).range(cumulative);
            this.links = this.graph.links(this.nodes).map(function (link) {
                var targetY = yDepthScale(link.target.depth);
                var linkHeight = infoPackage.boxVerticalPadding - stackInfo.headerSize;
                return {
                    source: {
                        x: link.source.x,
                        y: targetY - linkHeight,
                        y0: link.source.y
                    },
                    target: {
                        x: link.target.x,
                        y: targetY,
                        y0: link.target.y
                    }
                };
            });
            this.node = this.node.data(this.nodes);
            this.link = this.link.data(this.links);
            var enteringNodes = this.node
                .enter()
                .append("g")
                .attr("transform", function (d) { return "translate(" + _this.xScale(d.x) + "," + _this.yScale(yDepthScale(d.depth)) + ")"; });
            enteringNodes
                .filter(function (d) { return d.children && d.children.length > 0; })
                .append("line")
                .attr("class", "link")
                .attr("x1", 0)
                .attr("x2", 0)
                .attr("y1", function (d) { return -d.boxHeight() - stackInfo.headerSize; })
                .attr("y2", function (d) { return -maxBoxHeightOnDepth[d.depth - 1] - stackInfo.headerSize; });
            var rect = enteringNodes.append('rect')
                .attr('class', 'box')
                .attr('x', -infoPackage.maxBoxWidth / 2)
                .attr('y', function (d) { return -1 * d.boxHeight() - stackInfo.headerSize; })
                .attr('width', infoPackage.maxBoxWidth)
                .attr('height', function (d) { return d.boxHeight() + stackInfo.headerSize; })
                .attr("fill", "red")
                .attr("rx", 5)
                .on('mouseout', function () { return nv.tooltip.cleanup(); });
            var clipPaths = this.svgDefs.selectAll('.stackClip').data(this.nodes);
            clipPaths
                .enter()
                .append("clipPath")
                .attr('class', 'stackClip')
                .attr('id', function (d, i) { return 'stack-clip-path-' + i; })
                .append('rect')
                .attr('x', -infoPackage.maxBoxWidth / 2)
                .attr('width', infoPackage.maxBoxWidth - 5) // we substract little padding
                .attr('y', function (d) { return -1 * d.boxHeight() - stackInfo.headerSize; })
                .attr('height', function (d) { return d.boxHeight() + stackInfo.headerSize; });
            enteringNodes
                .append("text")
                .attr('text-anchor', 'middle')
                .attr('y', function (d) { return -1 * d.boxHeight(); })
                .text(function (d) { return d.ThreadIds.length + " thread" + ((d.ThreadIds.length > 1) ? "s" : ''); });
            enteringNodes
                .append("line")
                .attr('class', 'headerLine')
                .attr('x1', -1 * infoPackage.maxBoxWidth / 2)
                .attr('x2', infoPackage.maxBoxWidth / 2)
                .attr('y1', function (d) { return -1 * d.boxHeight() + 4; })
                .attr('y2', function (d) { return -1 * d.boxHeight() + 4; });
            enteringNodes.filter(function (d) { return d.depth > 0; }).each(function (d, index) {
                var g = this;
                var offsetTop = d.boxHeight() - stackInfo.boxPadding - stackInfo.lineHeight;
                var textGroup = d3.select(g)
                    .append("g")
                    .attr('class', 'traces')
                    .style('clip-path', function (d) { return 'url(#stack-clip-path-' + index + ')'; });
                var stack = textGroup.selectAll('.trace').data(d.stackWithShortcuts().reverse());
                var reversedOriginalStack = d.StackTrace.reverse();
                stack
                    .enter()
                    .append('text')
                    .attr('x', -140 + stackInfo.boxPadding)
                    .attr('y', function (d, i) { return -offsetTop + stackInfo.lineHeight * i; })
                    .text(function (d) { return d.short; })
                    .classed('notUserCode', function (s, i) { return !stackInfo.isUserCode(reversedOriginalStack[i]); })
                    .on('mouseover', function (d) {
                    nv.tooltip.cleanup();
                    var offset = $(this).offset();
                    nv.tooltip.show([offset.left, offset.top], self.getTooltip(d), 'n', 25);
                });
            });
            var enteringLinks = this.link
                .enter()
                .append("g");
            enteringLinks
                .append("path")
                .attr("class", "link")
                .attr("d", this.diagonal);
        };
        infoPackage.prototype.zoom = function () {
            this.svg.attr("transform", "translate(" + d3.event.translate + ")scale(" + d3.event.scale + ")");
        };
        infoPackage.prototype.cleanup = function () {
            nv.tooltip.cleanup();
            $("#parallelStacks").empty();
            this.infoPackage(null);
            this.infoPackageFilename(null);
            this.fetchException(null);
            this.stacksJson(null);
        };
        infoPackage.prototype.createPackageWithStacks = function () {
            this.createPackage(true);
        };
        infoPackage.prototype.createPackageWithoutStacks = function () {
            this.createPackage(false);
        };
        infoPackage.prototype.createPackage = function (includeStacks) {
            var _this = this;
            this.showLoadingIndicator(true);
            this.cleanup();
            var activeDb = this.adminView() ? appUrl.getSystemDatabase() : (this.activeDatabase() || appUrl.getSystemDatabase());
            new getInfoPackage(activeDb, includeStacks)
                .execute()
                .done(function (data, filename) {
                _this.infoPackage(data);
                _this.infoPackageFilename(filename);
                var zip = new jszip(data);
                var stacks = zip.file("stacktraces.txt");
                if (stacks) {
                    var stacksText = stacks.asText();
                    var stacksJson = JSON.parse(stacksText);
                    _this.packageCreated(stacksJson);
                }
            })
                .always(function () { return _this.showLoadingIndicator(false); });
        };
        infoPackage.prototype.packageCreated = function (stacksJson) {
            this.showMoreDetails(false);
            if ('Error' in stacksJson) {
                this.fetchException('Unable to fetch info package: ' + stacksJson.Error);
                this.fetchExceptionDetails(stacksJson.Details);
                return;
            }
            else {
                this.fetchException(null);
                this.fetchExceptionDetails(null);
            }
            this.stacksJson(stacksJson);
            var collatedStacks = this.splitAndCollateStacks(stacksJson);
            this.updateGraph(collatedStacks);
        };
        infoPackage.prototype.saveAsSvg = function () {
            var _this = this;
            svgDownloader.downloadSvg(d3.select('#parallelStacks').node(), 'stacks.svg', function (svgClone) {
                _this.cleanupSvgCloneForSave(svgClone);
                return infoPackage.stacksCss;
            });
        };
        infoPackage.prototype.saveAsPng = function () {
            var _this = this;
            svgDownloader.downloadPng(d3.select('#parallelStacks').node(), 'stacks.png', function (svgClone) {
                _this.cleanupSvgCloneForSave(svgClone);
                return infoPackage.stacksCss;
            });
        };
        infoPackage.prototype.saveAsZip = function () {
            fileDownloader.downloadAsZip(this.infoPackage(), this.infoPackageFilename());
        };
        infoPackage.prototype.saveAsJson = function () {
            fileDownloader.downloadAsJson(this.stacksJson(), "stacks.json");
        };
        infoPackage.prototype.cleanupSvgCloneForSave = function (svgClone) {
            d3.select(svgClone).select("g").select("g").attr('transform', null);
            var overlay = d3.select(svgClone).select(".overlay");
            d3.select(svgClone).attr('viewBox', '0 0 ' + (this.width + 50) + ' ' + (this.height + 20));
            d3.select(svgClone).select(".overlay").remove();
        };
        infoPackage.prototype.chooseImportFile = function () {
            var _this = this;
            var dialog = new infoPackageImport();
            dialog.task()
                .done(function (importedData) {
                _this.cleanup();
                if (importedData) {
                    _this.packageCreated(importedData);
                }
                else {
                    messagePublisher.reportWarning("Stacktraces are not available in given file. Please create info package with stacktraces.");
                }
            })
                .fail(function (e) { return messagePublisher.reportError(e); });
            app.showDialog(dialog);
        };
        infoPackage.prototype.moreDetails = function () {
            this.showMoreDetails(true);
        };
        infoPackage.maxBoxWidth = 280;
        infoPackage.boxVerticalPadding = 60;
        infoPackage.stacksCss = "svg { background-color: white; }\n" +
            " * { box-sizing: border-box; }\n" +
            " svg text { font-style: normal; font-variant: normal; font-weight: normal; font-size: 12px; line-height: normal; font-family: Arial; }\n" +
            " * { box-sizing: border - box; }\n" +
            " .link { fill: none; stroke: rgb(204, 204, 204); stroke-width: 1.5px; }\n" +
            " .overlay { fill: none; pointer-events: all; }\n" +
            " .box { fill: rgb(21, 140, 186); fill-opacity: 0.2; stroke: rgb(21, 140, 186); stroke-width: 1.5px; }\n" +
            " .notUserCode { fill-opacity: 0.4; }\n" +
            " .headerLine { stroke: rgb(21, 140, 186); stroke-width: 1.5px; }";
        return infoPackage;
    })(viewModelBase);
    return infoPackage;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/infoPackageImport", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", 'jszip/jszip'], function (require, exports, dialog, dialogViewModelBase, jszip) {
    var importPackageImport = (function (_super) {
        __extends(importPackageImport, _super);
        function importPackageImport() {
            _super.apply(this, arguments);
            this.importTask = $.Deferred();
            this.hasFileSelected = ko.observable(false);
            this.isImporting = ko.observable(false);
        }
        importPackageImport.prototype.fileSelected = function (args) {
            this.hasFileSelected(true);
        };
        importPackageImport.prototype.doImport = function () {
            var _this = this;
            var fileInput = document.querySelector("#importFilePicker");
            var self = this;
            var file = fileInput.files[0];
            var reader = new FileReader();
            reader.onload = function () {
                // try to detect type
                var firstChar = new Uint8Array(this.result.slice(0, 1))[0];
                if (firstChar === 91) {
                    var rawJson = String.fromCharCode.apply(null, new Uint8Array(this.result));
                    var json = JSON.parse(rawJson);
                    self.dataImported(json);
                }
                else {
                    var zip = new jszip(this.result);
                    var stacks = zip.file("stacktraces.txt");
                    if (stacks) {
                        var stacksText = stacks.asText();
                        var stacksJson = JSON.parse(stacksText);
                        self.dataImported(stacksJson);
                    }
                    else {
                        self.dataImported(null);
                    }
                }
            };
            reader.onerror = function (error) { return _this.importTask.reject(error); };
            reader.readAsArrayBuffer(file);
        };
        importPackageImport.prototype.dataImported = function (result) {
            this.importTask.resolve(result);
            dialog.close(this);
        };
        importPackageImport.prototype.cancel = function () {
            dialog.close(this);
        };
        importPackageImport.prototype.task = function () {
            return this.importTask.promise();
        };
        return importPackageImport;
    })(dialogViewModelBase);
    return importPackageImport;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/ioTest", ["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getDocumentWithMetadataCommand", "commands/database/debug/getStatusDebugConfigCommand", "common/appUrl", "models/database/debug/performanceTestRequest", "models/database/debug/performanceTestResultWrapped", "commands/database/debug/ioTestCommand", "commands/operations/killRunningTaskCommand", 'd3/d3', 'nvd3', "viewmodels/shell"], function (require, exports, viewModelBase, getDocumentWithMetadataCommand, getStatusDebugConfigCommand, appUrl, performanceTestRequest, performanceTestResultWrapped, ioTestCommand, killRunningTaskCommand, d3, nv, shell) {
    var ioTest = (function (_super) {
        __extends(ioTest, _super);
        function ioTest() {
            var _this = this;
            _super.call(this);
            this.isBusy = ko.observable(false);
            this.ioTestRequest = performanceTestRequest.empty();
            this.testResult = ko.observable();
            this.currentStatus = ko.observable();
            this.lastCommand = null;
            this.fileSizeMb = ko.computed({
                read: function () { return _this.ioTestRequest.fileSize() / 1024 / 1024; },
                write: function (value) { return _this.ioTestRequest.fileSize(value * 1024 * 1024); }
            });
            this.chunkSizeKb = ko.computed({
                read: function () { return _this.ioTestRequest.chunkSize() / 1024; },
                write: function (value) { return _this.ioTestRequest.chunkSize(value * 1024); }
            });
            this.overTimeThroughputChart = null;
            this.overTimeThroughputChartData = [];
            this.overTimeLatencyChart = null;
            this.overTimeLatencyChartData = [];
            this.isForbidden = ko.observable();
            this.ioTestRequest.sequential(false);
            this.chunkSizeCustomValidityError = ko.computed(function () {
                var errorMessage = '';
                if (isNaN(_this.chunkSizeKb()) || _this.chunkSizeKb() % 4 != 0) {
                    errorMessage = "Chunk size must be multiple of 4";
                }
                return errorMessage;
            });
            this.isForbidden(!shell.isGlobalAdmin());
        }
        ioTest.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            if (this.isForbidden() === false) {
                new getStatusDebugConfigCommand(appUrl.getSystemDatabase())
                    .execute()
                    .done(function (results) {
                    return _this.ioTestRequest.threadCount(results.MaxNumberOfParallelProcessingTasks);
                })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        ioTest.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('ZL2H8E');
        };
        ioTest.prototype.onIoTestCompleted = function (result) {
            var _this = this;
            this.testResult(new performanceTestResultWrapped(result));
            this.overTimeThroughputChartData = [];
            this.overTimeLatencyChartData = [];
            if (this.testResult().hasReads()) {
                this.overTimeThroughputChartData.push({
                    key: 'Read throughput',
                    values: result.Result.ReadPerSecondHistory.map(function (v, idx) { return { x: idx, y: v / 1024 / 1024 }; })
                });
                this.overTimeLatencyChartData.push({
                    key: 'Read latency',
                    values: result.Result.AverageReadLatencyPerSecondHistory.map(function (v, idx) { return { x: idx, y: v }; })
                });
            }
            if (this.testResult().hasWrites()) {
                this.overTimeThroughputChartData.push({
                    key: 'Write throughput',
                    values: result.Result.WritePerSecondHistory.map(function (v, idx) { return { x: idx, y: v / 1024 / 1024 }; })
                });
                this.overTimeLatencyChartData.push({
                    key: 'Write latency',
                    values: result.Result.AverageWriteLatencyPerSecondHistory.map(function (v, idx) { return { x: idx, y: v }; })
                });
            }
            if (this.overTimeLatencyChart === null) {
                nv.addGraph(function () {
                    var chart = nv.models.lineChart()
                        .margin({ left: 130 })
                        .useInteractiveGuideline(true)
                        .transitionDuration(350)
                        .showLegend(true)
                        .showYAxis(true)
                        .showXAxis(true)
                        .forceY([0]);
                    ;
                    chart.xAxis
                        .axisLabel('Time [s]');
                    chart.yAxis
                        .axisLabel('ms')
                        .tickFormat(d3.format(',.2f'));
                    $(window).on('resize.ioTest', function (e) { return chart.update(); });
                    return chart;
                }, function (chart) {
                    _this.overTimeLatencyChart = chart;
                    d3.select('#overTimeLatencyContainer svg')
                        .datum(_this.overTimeLatencyChartData)
                        .call(_this.overTimeLatencyChart);
                });
            }
            else {
                d3.select('#overTimeLatencyContainer svg')
                    .datum(this.overTimeLatencyChartData)
                    .call(this.overTimeLatencyChart);
            }
            if (this.overTimeThroughputChart === null) {
                nv.addGraph(function () {
                    var chart = nv.models.lineChart()
                        .margin({ left: 130 })
                        .useInteractiveGuideline(true)
                        .transitionDuration(350)
                        .showLegend(true)
                        .showYAxis(true)
                        .showXAxis(true)
                        .forceY([0]);
                    ;
                    chart.xAxis
                        .axisLabel('Time [s]');
                    chart.yAxis
                        .axisLabel('MB/sec')
                        .tickFormat(d3.format(',.2f'));
                    nv.utils.windowResize(function () { chart.update(); });
                    return chart;
                }, function (chart) {
                    _this.overTimeThroughputChart = chart;
                    d3.select('#overTimeThroughputContainer svg')
                        .datum(_this.overTimeThroughputChartData)
                        .call(_this.overTimeThroughputChart);
                });
            }
            else {
                d3.select('#overTimeThroughputContainer svg')
                    .datum(this.overTimeThroughputChartData)
                    .call(this.overTimeThroughputChart);
            }
        };
        ioTest.prototype.killTask = function () {
            if (this.lastCommand !== null) {
                this.lastCommand.operationIdTask.done(function (operationId) {
                    new killRunningTaskCommand(appUrl.getSystemDatabase(), operationId).execute();
                });
            }
        };
        ioTest.prototype.startPerformanceTest = function () {
            var _this = this;
            this.isBusy(true);
            var self = this;
            var diskTestParams = this.ioTestRequest.toDto();
            this.lastCommand = new ioTestCommand(appUrl.getSystemDatabase(), diskTestParams, function (s) { if (s)
                _this.currentStatus(s.Progress); });
            this.lastCommand
                .execute()
                .done(function () {
                new getDocumentWithMetadataCommand("Raven/Disk/Performance", appUrl.getSystemDatabase())
                    .execute()
                    .done(function (result) {
                    _this.onIoTestCompleted(result);
                });
            })
                .always(function () { return _this.isBusy(false); });
        };
        ioTest.prototype.detached = function () {
            _super.prototype.detached.call(this);
            window.onresize = null; // FIX nvd3 event attached globally
        };
        return ioTest;
    })(viewModelBase);
    return ioTest;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/licenseInformation", ["require", "exports", "viewmodels/viewModelBase", "commands/auth/licenseCheckConnectivityCommand", "commands/auth/forceLicenseUpdate", "viewmodels/common/licensingStatus", "durandal/app", "models/auth/license", "commands/auth/getLicenseStatusCommand", "commands/auth/getSupportCoverageCommand", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, licenseCheckConnectivityCommand, forceLicenseUpdate, licensingStatus, app, license, getLicenseStatusCommand, getSupportCoverageCommand, settingsAccessAuthorizer) {
    var licenseInformation = (function (_super) {
        __extends(licenseInformation, _super);
        function licenseInformation() {
            _super.apply(this, arguments);
            this.settingsAccess = new settingsAccessAuthorizer();
            this.connectivityStatus = ko.observable("pending");
            this.isForbidden = ko.observable();
        }
        licenseInformation.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            if (this.settingsAccess.canReadOrWrite()) {
                this.checkConnectivity()
                    .done(function (result) {
                    _this.connectivityStatus(result ? "success" : "failed");
                })
                    .fail(function () { return _this.connectivityStatus("failed"); });
            }
        };
        licenseInformation.prototype.fetchLicenseStatus = function () {
            return new getLicenseStatusCommand()
                .execute()
                .done(function (result) {
                if (result.Status.contains("AGPL")) {
                    result.Status = "Development Only";
                }
                license.licenseStatus(result);
            });
        };
        licenseInformation.prototype.fetchSupportCoverage = function () {
            return new getSupportCoverageCommand()
                .execute()
                .done(function (result) {
                license.supportCoverage(result);
            });
        };
        licenseInformation.prototype.forceUpdate = function () {
            var _this = this;
            new forceLicenseUpdate().execute()
                .always(function () {
                $.when(_this.fetchLicenseStatus(), _this.fetchSupportCoverage())
                    .always(function () {
                    _this.showLicenseDialog();
                });
            });
        };
        licenseInformation.prototype.showLicenseDialog = function () {
            var dialog = new licensingStatus(license.licenseStatus(), license.supportCoverage(), license.hotSpare());
            app.showDialog(dialog);
        };
        licenseInformation.prototype.checkConnectivity = function () {
            return new licenseCheckConnectivityCommand().execute();
        };
        return licenseInformation;
    })(viewModelBase);
    return licenseInformation;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/restore", ["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell", "models/resources/database", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl", "models/filesystem/filesystem", "commands/maintenance/monitorRestoreCommand", "commands/maintenance/startRestoreCommand", "commands/filesystem/startRestoreCommand"], function (require, exports, viewModelBase, shell, database, getDocumentWithMetadataCommand, appUrl, filesystem, monitorRestoreCommand, startDbRestoreCommand, startFsRestoreCommand) {
    var resourceRestore = (function () {
        function resourceRestore(parent, type, resources) {
            var _this = this;
            this.parent = parent;
            this.type = type;
            this.resources = resources;
            this.defrag = ko.observable(false);
            this.backupLocation = ko.observable("");
            this.resourceLocation = ko.observable();
            this.indexesLocation = ko.observable();
            this.journalsLocation = ko.observable();
            this.resourceName = ko.observable();
            this.restoreStatusMessages = ko.observableArray();
            this.keepDown = ko.observable(false);
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var foundDb = resources.first(function (rs) { return newResourceName == rs.name; });
                if (!!foundDb && newResourceName.length > 0) {
                    errorMessage = (_this.type === database.type ? "Database" : "File System") + " name already exist!";
                }
                return errorMessage;
            });
        }
        resourceRestore.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            this.forceKeepDown();
        };
        resourceRestore.prototype.forceKeepDown = function () {
            if (this.keepDown()) {
                var body = document.getElementsByTagName("body")[0];
                body.scrollTop = body.scrollHeight;
            }
        };
        resourceRestore.prototype.updateRestoreStatus = function (newRestoreStatus) {
            this.restoreStatusMessages(newRestoreStatus.Messages);
            this.forceKeepDown();
            this.parent.isBusy(newRestoreStatus.State === "Running");
        };
        return resourceRestore;
    })();
    var restore = (function (_super) {
        __extends(restore, _super);
        function restore() {
            _super.apply(this, arguments);
            this.dbRestoreOptions = new resourceRestore(this, database.type, shell.databases);
            this.fsRestoreOptions = new resourceRestore(this, filesystem.type, shell.fileSystems);
            this.disableReplicationDestinations = ko.observable(false);
            this.generateNewDatabaseId = ko.observable(false);
            this.isForbidden = ko.observable();
            this.isBusy = ko.observable();
            this.anotherRestoreInProgres = ko.observable(false);
        }
        restore.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            this.isForbidden(shell.isGlobalAdmin() === false);
            if (this.isForbidden() === false) {
                this.isBusy(true);
                var db = appUrl.getSystemDatabase();
                var self = this;
                new getDocumentWithMetadataCommand("Raven/Restore/InProgress", db, true).execute()
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(db) }); })
                    .done(function (result) {
                    if (result) {
                        // looks like another restore is in progress
                        _this.anotherRestoreInProgres(true);
                        new monitorRestoreCommand($.Deferred(), function (s) {
                            self.dbRestoreOptions.updateRestoreStatus.bind(self.dbRestoreOptions)(s);
                            self.fsRestoreOptions.updateRestoreStatus.bind(self.fsRestoreOptions)(s);
                        })
                            .execute()
                            .always(function () {
                            $("#databaseRawLogsContainer").resize();
                            $("#filesystemRawLogsContainer").resize();
                            _this.anotherRestoreInProgres(false);
                        });
                    }
                    else {
                        _this.isBusy(false);
                    }
                    deferred.resolve({ can: true });
                });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        restore.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("FT7RV6");
        };
        restore.prototype.startDbRestore = function () {
            this.isBusy(true);
            var self = this;
            var restoreDatabaseDto = {
                BackupLocation: this.dbRestoreOptions.backupLocation(),
                DatabaseLocation: this.dbRestoreOptions.resourceLocation(),
                DatabaseName: this.dbRestoreOptions.resourceName(),
                DisableReplicationDestinations: this.disableReplicationDestinations(),
                GenerateNewDatabaseId: this.generateNewDatabaseId(),
                IndexesLocation: this.dbRestoreOptions.indexesLocation(),
                JournalsLocation: this.dbRestoreOptions.journalsLocation()
            };
            new startDbRestoreCommand(this.dbRestoreOptions.defrag(), restoreDatabaseDto, self.dbRestoreOptions.updateRestoreStatus.bind(self.dbRestoreOptions))
                .execute()
                .always(function () { return shell.reloadDatabases(); });
        };
        restore.prototype.startFsRestore = function () {
            this.isBusy(true);
            var self = this;
            var restoreFilesystemDto = {
                BackupLocation: this.fsRestoreOptions.backupLocation(),
                FilesystemLocation: this.fsRestoreOptions.resourceLocation(),
                FilesystemName: this.fsRestoreOptions.resourceName(),
                IndexesLocation: this.fsRestoreOptions.indexesLocation(),
                JournalsLocation: this.fsRestoreOptions.journalsLocation()
            };
            new startFsRestoreCommand(this.fsRestoreOptions.defrag(), restoreFilesystemDto, self.fsRestoreOptions.updateRestoreStatus.bind(self.fsRestoreOptions))
                .execute()
                .always(function () { return shell.reloadFileSystems(); });
        };
        return restore;
    })(viewModelBase);
    return restore;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/serverSmuggling", ["require", "exports", "viewmodels/viewModelBase", "viewmodels/shell", "models/resources/serverSmugglingItem", "models/database/cluster/serverConnectionInfo", "commands/operations/performSmugglingCommand", "common/appUrl", "common/jsonUtil", "common/serverSmugglingLocalStorage", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, shell, serverSmugglingItem, serverConnectionInfo, performSmugglingCommand, appUrl, jsonUtil, serverSmugglingLocalStorage, settingsAccessAuthorizer) {
    var serverSmuggling = (function (_super) {
        __extends(serverSmuggling, _super);
        function serverSmuggling() {
            var _this = this;
            _super.call(this);
            this.resources = ko.observableArray([]);
            this.selectedResources = ko.observableArray();
            this.targetServer = ko.observable(new serverConnectionInfo());
            this.settingsAccess = new settingsAccessAuthorizer();
            this.showJsonRequest = ko.observable(false);
            this.showCurlRequest = ko.observable(false);
            this.inProgress = ko.observable(false);
            this.resultsVisible = ko.observable(false);
            this.messages = ko.observableArray([]);
            var smi = shell.databases().filter(function (d) { return d.name !== "<system>"; }).map(function (d) { return new serverSmugglingItem(d); });
            this.resources(smi);
            this.hasResources = ko.computed(function () { return _this.resources().count() > 0; });
            this.noIncremental = ko.computed(function () { return _this.selectedResources().length === 0; });
            this.noStripReplication = ko.computed(function () {
                var resources = _this.selectedResources();
                var replicationCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
                return resources.length === 0 || replicationCount === 0;
            });
            this.noDisableVersioning = ko.computed(function () {
                var resources = _this.selectedResources();
                var versioningCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
                return resources.length === 0 || versioningCount === 0;
            });
            this.resourcesSelection = ko.computed(function () {
                var selectedResourcesCount = _this.selectedResources().length;
                if (selectedResourcesCount === _this.resources().length)
                    return checkbox.Checked;
                if (selectedResourcesCount > 0)
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
            this.incrementalSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                if (resources.length === 0)
                    return checkbox.UnChecked;
                var incrementalCount = resources.filter(function (x) { return x.incremental(); }).length;
                if (incrementalCount === resources.length)
                    return checkbox.Checked;
                if (incrementalCount > 0)
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
            this.stripReplicationSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                var replicationCount = resources.filter(function (x) { return x.stripReplicationInformation(); }).length;
                if (resources.length === 0 || replicationCount === 0)
                    return checkbox.UnChecked;
                var replicationBundleCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
                if (replicationBundleCount === replicationCount)
                    return checkbox.Checked;
                if (replicationBundleCount > 0)
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
            this.disableVersioningSelection = ko.computed(function () {
                var resources = _this.selectedResources();
                var versioningCount = resources.filter(function (x) { return x.shouldDisableVersioningBundle(); }).length;
                if (resources.length === 0 || versioningCount === 0)
                    return checkbox.UnChecked;
                var versioningBundleCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
                if (versioningBundleCount === versioningCount)
                    return checkbox.Checked;
                if (versioningBundleCount > 0)
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
            this.submitEnabled = ko.computed(function () {
                var progress = _this.inProgress();
                var selection = _this.selectedResources().length > 0;
                var url = _this.targetServer().url();
                return !progress && selection && !!url;
            });
            this.jsonRequest = ko.computed(function () {
                var request = _this.getJson();
                return jsonUtil.syntaxHighlight(request);
            });
            this.curlRequest = ko.computed(function () {
                var json = JSON.stringify(_this.getJson(), null, 0).replaceAll("\"", "\\\"");
                return "curl -i -H \"Accept: application/json\" -H \"Content-Type: application/json\" -X POST --data \"" + json + "\" " + appUrl.forServer() + "/admin/serverSmuggling";
            });
            this.jsonRequest.subscribe(function () { return _this.saveIntoLocalStorage(); });
            this.restoreFromLocalStorage();
        }
        serverSmuggling.prototype.restoreFromLocalStorage = function () {
            var savedValue = serverSmugglingLocalStorage.get();
            if (savedValue) {
                var self = this;
                var targetServer = this.targetServer();
                targetServer.url(savedValue.TargetServer.Url);
                targetServer.domain(savedValue.TargetServer.Domain);
                targetServer.username(savedValue.TargetServer.Username);
                targetServer.guessCredentialsType();
                // since resources might change over time we have to apply saved changes carefully. 
                savedValue.Config.forEach(function (savedConfig) {
                    var item = self.resources().first(function (r) { return r.resource.name === savedConfig.Name; });
                    if (item) {
                        self.selectedResources.push(item);
                        item.incremental(savedConfig.Incremental);
                        if (item.hasVersioningBundle()) {
                            item.shouldDisableVersioningBundle(savedConfig.ShouldDisableVersioningBundle);
                        }
                        if (item.hasReplicationBundle()) {
                            item.stripReplicationInformation(savedConfig.StripReplicationInformation);
                        }
                    }
                });
            }
        };
        serverSmuggling.prototype.toggleSelectAll = function () {
            if (this.selectedResources().length > 0) {
                this.selectedResources([]);
            }
            else {
                this.selectedResources(this.resources().slice(0));
            }
        };
        serverSmuggling.prototype.toggleSelectAllIncremental = function () {
            var resources = this.selectedResources();
            if (resources.length === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.incremental(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                resources[i].incremental(!hasSelected);
            }
        };
        serverSmuggling.prototype.toggleSelectAllStripReplication = function () {
            var resources = this.selectedResources();
            var replicationBundleCount = resources.filter(function (x) { return x.hasReplicationBundle(); }).length;
            if (resources.length === 0 || replicationBundleCount === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.stripReplicationInformation(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                if (resources[i].hasReplicationBundle()) {
                    resources[i].stripReplicationInformation(!hasSelected);
                }
            }
        };
        serverSmuggling.prototype.toggleSelectAllDisableVersioning = function () {
            var resources = this.selectedResources();
            var versioningBundleCount = resources.filter(function (x) { return x.hasVersioningBundle(); }).length;
            if (resources.length === 0 || versioningBundleCount === 0)
                return;
            var hasSelected = resources.filter(function (x) { return x.shouldDisableVersioningBundle(); }).length > 0;
            for (var i = 0; i < resources.length; i++) {
                if (resources[i].hasVersioningBundle()) {
                    resources[i].shouldDisableVersioningBundle(!hasSelected);
                }
            }
        };
        serverSmuggling.prototype.isSelected = function (item) {
            return this.selectedResources().indexOf(item) >= 0;
        };
        serverSmuggling.prototype.saveIntoLocalStorage = function () {
            var json = this.getJson();
            json.TargetServer.ApiKey = undefined;
            json.TargetServer.Password = undefined;
            serverSmugglingLocalStorage.setValue(json);
        };
        serverSmuggling.prototype.performMigration = function () {
            var _this = this;
            var request = this.getJson();
            this.messages([]);
            this.inProgress(true);
            this.resultsVisible(true);
            new performSmugglingCommand(request, appUrl.getSystemDatabase(), function (status) { return _this.updateProgress(status); })
                .execute()
                .always(function () { return _this.inProgress(false); });
        };
        serverSmuggling.prototype.getJson = function () {
            var targetServer = this.targetServer().toDto();
            var config = this.selectedResources().map(function (r) { return r.toDto(); });
            return {
                TargetServer: targetServer,
                Config: config
            };
        };
        serverSmuggling.prototype.updateProgress = function (progress) {
            this.messages(progress.Messages);
        };
        serverSmuggling.prototype.toggleJson = function () {
            this.showJsonRequest(!this.showJsonRequest());
        };
        serverSmuggling.prototype.toggleCurl = function () {
            this.showCurlRequest(!this.showCurlRequest());
        };
        return serverSmuggling;
    })(viewModelBase);
    return serverSmuggling;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/studioConfig", ["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getDocumentWithMetadataCommand", "common/appUrl", "models/database/documents/document", "common/serverBuildReminder", "common/eventSourceSettingStorage", "commands/database/documents/saveDocumentCommand", "models/resources/environmentColor", "viewmodels/shell", "common/numberFormattingStorage"], function (require, exports, viewModelBase, getDocumentWithMetadataCommand, appUrl, documentClass, serverBuildReminder, eventSourceSettingStorage, saveDocumentCommand, environmentColor, shell, numberFormattingStorage) {
    var studioConfig = (function (_super) {
        __extends(studioConfig, _super);
        function studioConfig() {
            var _this = this;
            _super.call(this);
            this.configDocument = ko.observable();
            this.warnWhenUsingSystemDatabase = ko.observable(true);
            this.disableEventSource = ko.observable(false);
            this.timeUntilRemindToUpgrade = ko.observable();
            this.isForbidden = ko.observable();
            this.browserFormatExample = 5050.99.toLocaleString();
            this.rawFormat = ko.observable();
            this.environmentColors = [
                new environmentColor("Default", "#f8f8f8"),
                new environmentColor("Development", "#80FF80"),
                new environmentColor("Staging", "#F5824D"),
                new environmentColor("Production", "#FF8585")
            ];
            this.selectedColor = ko.observable();
            this.documentId = shell.studioConfigDocumentId;
            this.systemDatabase = appUrl.getSystemDatabase();
            this.timeUntilRemindToUpgrade(serverBuildReminder.get());
            this.disableEventSource(eventSourceSettingStorage.get());
            this.mute = ko.computed(function () {
                var lastBuildCheck = _this.timeUntilRemindToUpgrade();
                var timestamp = Date.parse(lastBuildCheck);
                var isLegalDate = !isNaN(timestamp);
                return isLegalDate;
            });
            this.timeUntilRemindToUpgradeMessage = ko.computed(function () {
                if (_this.mute()) {
                    var lastBuildCheck = _this.timeUntilRemindToUpgrade();
                    var lastBuildCheckMoment = moment(lastBuildCheck);
                    return "muted for another " + lastBuildCheckMoment.add("days", 7).fromNow(true);
                }
                return "mute for a week";
            });
            var color = this.environmentColors.filter(function (color) { return color.name === shell.selectedEnvironmentColorStatic().name; });
            var selectedColor = !!color[0] ? color[0] : this.environmentColors[0];
            this.selectedColor(selectedColor);
            var self = this;
            this.selectedColor.subscribe(function (newValue) { return self.setEnvironmentColor(newValue); });
            this.isForbidden((shell.isGlobalAdmin() || shell.canReadWriteSettings() || shell.canReadSettings()) === false);
            this.isReadOnly = ko.computed(function () { return shell.isGlobalAdmin() === false && shell.canReadWriteSettings() === false && shell.canReadSettings(); });
            this.rawFormat(numberFormattingStorage.shouldUseRaw());
        }
        studioConfig.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            if (this.isForbidden() === false) {
                new getDocumentWithMetadataCommand(this.documentId, this.systemDatabase)
                    .execute()
                    .done(function (doc) {
                    _this.configDocument(doc);
                    _this.warnWhenUsingSystemDatabase(doc["WarnWhenUsingSystemDatabase"]);
                })
                    .fail(function () { return _this.configDocument(documentClass.empty()); })
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            else {
                deferred.resolve({ can: true });
            }
            return deferred;
        };
        studioConfig.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("4J5OUB");
        };
        studioConfig.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            var self = this;
            $(window).bind('storage', function (e) {
                if (e.originalEvent.key === serverBuildReminder.localStorageName) {
                    self.timeUntilRemindToUpgrade(serverBuildReminder.get());
                }
            });
            $("select").selectpicker();
            this.pickColor();
            $("#select-color li").each(function (index, element) {
                var color = _this.environmentColors[index];
                $(element).css("backgroundColor", color.backgroundColor);
            });
        };
        studioConfig.prototype.setEnvironmentColor = function (envColor) {
            var _this = this;
            var newDocument = this.configDocument();
            newDocument["EnvironmentColor"] = envColor.toDto();
            var saveTask = this.saveStudioConfig(newDocument);
            saveTask.done(function () {
                shell.selectedEnvironmentColorStatic(_this.selectedColor());
                _this.pickColor();
            });
        };
        studioConfig.prototype.setSystemDatabaseWarning = function (warnSetting) {
            var _this = this;
            if (this.warnWhenUsingSystemDatabase() !== warnSetting) {
                var newDocument = this.configDocument();
                this.warnWhenUsingSystemDatabase(warnSetting);
                newDocument["WarnWhenUsingSystemDatabase"] = warnSetting;
                var saveTask = this.saveStudioConfig(newDocument);
                saveTask.fail(function () { return _this.warnWhenUsingSystemDatabase(!warnSetting); });
            }
        };
        studioConfig.prototype.pickColor = function () {
            $("#select-color button").css("backgroundColor", this.selectedColor().backgroundColor);
        };
        studioConfig.prototype.setEventSourceDisabled = function (setting) {
            this.disableEventSource(setting);
            eventSourceSettingStorage.setValue(setting);
        };
        studioConfig.prototype.setUpgradeReminder = function (upgradeSetting) {
            serverBuildReminder.mute(upgradeSetting);
        };
        studioConfig.prototype.setNumberFormat = function (raw) {
            this.rawFormat(raw);
            numberFormattingStorage.save(raw);
        };
        studioConfig.prototype.saveStudioConfig = function (newDocument) {
            var _this = this;
            return new saveDocumentCommand(this.documentId, newDocument, this.systemDatabase)
                .execute()
                .done(function (saveResult) {
                _this.configDocument(newDocument);
                _this.configDocument().__metadata['etag'] = saveResult[0].Etag;
            });
        };
        return studioConfig;
    })(viewModelBase);
    return studioConfig;
});
/// <reference path="../../../Scripts/typings/d3/dagre.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/topology", ["require", "exports", "viewmodels/viewModelBase", "common/svgDownloader", "common/fileDownloader", "commands/resources/getGlobalReplicationTopology", 'd3/d3', 'dagre', "common/settingsAccessAuthorizer", "viewmodels/shell"], function (require, exports, viewModelBase, svgDownloader, fileDownloader, getGlobalReplicationTopology, d3, dagre, settingsAccessAuthorizer, shell) {
    var topology = (function (_super) {
        __extends(topology, _super);
        function topology() {
            var _this = this;
            _super.call(this);
            this.topology = ko.observable(null);
            this.topologyFiltered = ko.observable(null);
            this.currentLink = ko.observable(null);
            this.searchText = ko.observable();
            this.settingsAccess = new settingsAccessAuthorizer();
            this.fetchDb = ko.observable(true);
            this.fetchFs = ko.observable(true);
            this.fetchCs = ko.observable(shell.has40Features());
            this.showLoadingIndicator = ko.observable(false);
            this.showCsOption = shell.has40Features;
            this.colors = d3.scale.category10();
            this.line = d3.svg.line().x(function (d) { return d.x; }).y(function (d) { return d.y; });
            this.hasSaveAsPngSupport = ko.computed(function () {
                return !(navigator && navigator.msSaveBlob);
            });
            this.searchText.throttle(250).subscribe(function (value) { return _this.filter(value); });
        }
        topology.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('ES8PCB');
        };
        topology.prototype.attached = function () {
            _super.prototype.attached.call(this);
            d3.select(window).on("resize", this.resize.bind(this));
        };
        topology.prototype.compositionComplete = function () {
            this.resize();
        };
        topology.prototype.resize = function () {
            this.width = $("#replicationTopologySection").width() * 0.66;
        };
        topology.prototype.createReplicationTopology = function () {
            var _this = this;
            var self = this;
            var $replicationTopologSection = $("#replicationTopologySection");
            var $replicationTopologySvg = $("svg#replicationTopology");
            this.height = $replicationTopologSection.outerHeight() -
                ($replicationTopologySvg.offset().top - $replicationTopologSection.offset().top) -
                20;
            this.zoom = d3.behavior.zoom()
                .scaleExtent([0.2, 5])
                .on("zoom", function () {
                _this.svg.select('.graphZoom').attr("transform", "translate(" + d3.event.translate + ")scale(" + d3.event.scale + ")");
            });
            this.svg = d3.select("#replicationTopology")
                .style({ height: self.height + 'px' })
                .style({ width: self.width + 'px' })
                .attr("viewBox", "0 0 " + self.width + " " + self.height)
                .call(this.zoom);
            this.syncGraph();
        };
        topology.prototype.syncGraph = function () {
            var topologyGraph = new dagre.graphlib.Graph();
            topologyGraph.setGraph({});
            topologyGraph.setDefaultEdgeLabel(function () { return ""; });
            this.fillNodes(topologyGraph);
            dagre.layout(topologyGraph);
            this.renderTopology(topologyGraph);
        };
        topology.prototype.fillNodes = function (topologyGraph) {
            if (this.topologyFiltered().Databases) {
                this.topologyFiltered().Databases.SkippedResources.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "db",
                        url: s,
                        name: ' (skipped)',
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().Databases.Servers.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "db",
                        url: s.split("/databases/")[0],
                        name: s.split("/databases/")[1],
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().Databases.Connections.forEach(function (c) {
                    c.UiType = "db";
                    topologyGraph.setEdge(c.Source, c.Destination, c);
                });
            }
            if (this.topologyFiltered().FileSystems) {
                this.topologyFiltered().FileSystems.SkippedResources.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "fs",
                        url: s,
                        name: " (skipped)",
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().FileSystems.Servers.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "fs",
                        url: s.split("/fs/")[0],
                        name: s.split("/fs/")[1],
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().FileSystems.Connections.forEach(function (c) {
                    c.UiType = "fs";
                    topologyGraph.setEdge(c.Source, c.Destination, c);
                });
            }
            if (this.topologyFiltered().Counters) {
                this.topologyFiltered().Counters.SkippedResources.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "cs",
                        url: s,
                        name: " (skipped)",
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().Counters.Servers.forEach(function (s) {
                    topologyGraph.setNode(s, {
                        id: s,
                        rType: "cs",
                        url: s.split("/cs/")[0],
                        name: s.split("/cs/")[1],
                        width: 220,
                        height: 40
                    });
                });
                this.topologyFiltered().Counters.Connections.forEach(function (c) {
                    c.UiType = "cs";
                    topologyGraph.setEdge(c.Source, c.Destination, c);
                });
            }
        };
        topology.prototype.linkWithArrow = function (d) {
            var self = this;
            var pointCount = d.points.length;
            var d90 = Math.PI / 2, sourceX = d.points[pointCount - 2].x, sourceY = d.points[pointCount - 2].y, targetX = d.points[pointCount - 1].x, targetY = d.points[pointCount - 1].y;
            var theta = Math.atan2(targetY - sourceY, targetX - sourceX);
            return self.line(d.points) +
                "M" + targetX + "," + targetY +
                "l" + (3.5 * Math.cos(d90 - theta) - 10 * Math.cos(theta)) + "," + (-3.5 * Math.sin(d90 - theta) - 10 * Math.sin(theta)) +
                "L" + (targetX - 3.5 * Math.cos(d90 - theta) - 10 * Math.cos(theta)) + "," + (targetY + 3.5 * Math.sin(d90 - theta) - 10 * Math.sin(theta)) + "z";
        };
        topology.prototype.linkHasError = function (d) {
            return d.SourceToDestinationState !== "Online";
        };
        topology.prototype.iconText = function (rType) {
            switch (rType) {
                case "db":
                    return "&#xf1c0";
                case "fs":
                    return "&#xf1c5";
                case "cs":
                    return "&#xf163";
                default:
                    return "&#xf128";
            }
        };
        topology.prototype.renderTopology = function (topologyGraph) {
            var self = this;
            var graph = this.svg.selectAll('.graph').data([null]);
            var graphWidth = topologyGraph.graph().width;
            if (graphWidth === -Infinity) {
                graphWidth = 0;
            }
            graph.attr('transform', 'translate(' + ((self.width - graphWidth) / 2) + ',10)');
            this.dagreGraphSize = [topologyGraph.graph().width, topologyGraph.graph().height];
            var enteringGraph = graph.enter().append('g').attr('class', 'graph');
            var enteringGraphZoom = enteringGraph.append("g").attr("class", "graphZoom");
            enteringGraphZoom.append('g').attr('class', 'nodes');
            enteringGraphZoom.append('g').attr('class', 'edges');
            enteringGraph.attr('transform', 'translate(' + ((self.width - graphWidth) / 2) + ',10)');
            var mappedNodes = topologyGraph.nodes().map(function (n) { return topologyGraph.node(n); });
            var nodesDom = this.svg.select('.nodes').selectAll('g.node').data(mappedNodes, function (d) { return d.id; });
            nodesDom
                .exit()
                .transition()
                .style('opacity', 0)
                .remove();
            var nGroup = nodesDom.enter()
                .append('g')
                .attr('class', function (d) { return 'node ' + d.rType; })
                .attr('transform', function (d) { return 'translate(' + d.x + ',' + d.y + ')'; });
            nGroup.append('svg:rect')
                .attr('class', 'node')
                .attr('rx', 5)
                .attr("x", -110)
                .attr("y", -20)
                .attr('width', 220)
                .attr('height', 40);
            nGroup.append('svg:text')
                .attr("x", -100)
                .attr("y", 6)
                .attr('class', 'fa')
                .html(function (d) { return self.iconText(d.rType); });
            nGroup.append('svg:text')
                .attr('x', 10)
                .attr('y', -4)
                .attr('class', 'id')
                .text(function (d) { return d.url; });
            nGroup.append('svg:text')
                .attr('x', 10)
                .attr('y', 14)
                .attr('class', 'id')
                .text(function (d) { return d.name; });
            nodesDom
                .transition()
                .attr('transform', function (d) { return 'translate(' + d.x + ',' + d.y + ')'; });
            var mappedEdges = topologyGraph.edges().map(function (e) { return topologyGraph.edge(e); });
            var edgesDom = this.svg.select('.edges').selectAll('.link').data(mappedEdges, function (d) { return d.Source + "_" + d.Destination; });
            edgesDom
                .exit()
                .transition()
                .style('opacity', 0)
                .remove();
            edgesDom.enter()
                .append('path')
                .attr('class', 'link')
                .classed('error', self.linkHasError)
                .attr('d', function (d) { return self.linkWithArrow(d); })
                .on("click", function (d) {
                var currentSelection = d3.select(".selected").node();
                d3.selectAll(".selected").classed("selected", false);
                d3.select(this).classed('selected', currentSelection !== this);
                self.currentLink(currentSelection !== this ? d : null);
            });
            edgesDom
                .transition()
                .attr('d', function (d) { return self.linkWithArrow(d); });
        };
        topology.prototype.fetchTopology = function () {
            var _this = this;
            this.showLoadingIndicator(true);
            new getGlobalReplicationTopology(this.fetchDb(), this.fetchFs(), this.fetchCs())
                .execute()
                .done(function (topo) {
                _this.topology(topo);
                _this.topologyFiltered(topo);
                _this.createReplicationTopology();
            })
                .always(function () { return _this.showLoadingIndicator(false); });
        };
        topology.prototype.saveAsPng = function () {
            var _this = this;
            svgDownloader.downloadPng(d3.select('#replicationTopology').node(), 'replicationTopology.png', function (svg) {
                _this.preprocesSvgDownload(svg);
                return topology.inlineCss;
            });
        };
        topology.prototype.saveAsSvg = function () {
            var _this = this;
            svgDownloader.downloadSvg(d3.select('#replicationTopology').node(), 'replicationTopology.svg', function (svg) {
                _this.preprocesSvgDownload(svg);
                return topology.inlineCss;
            });
        };
        topology.prototype.preprocesSvgDownload = function (svg) {
            this.removeIconsProcessor(svg);
            var padding = 10;
            $(svg)
                .attr('width', (this.dagreGraphSize[0] + 2 * padding) + 'px')
                .attr('height', (this.dagreGraphSize[1] + 2 * padding) + 'px');
            $(".graph", svg).attr('transform', 'translate(' + padding + ',' + padding + ')');
            $(".graphZoom", svg).removeAttr('transform');
        };
        /*
        * Since we are using FontAwesome icons and they won't be avaialable in downloaded
        * svg, remove them from DOM
        * Also this procedure should fix issue with converting utf-8 data with window.atob.
        */
        topology.prototype.removeIconsProcessor = function (svg) {
            $("text.fa", svg).remove();
        };
        topology.prototype.saveAsJson = function () {
            fileDownloader.downloadAsJson(this.topology(), "topology.json");
        };
        topology.prototype.filter = function (criteria) {
            var filtered = jQuery.extend(true, {}, this.topology());
            this.resetZoom();
            if (!criteria) {
                this.topologyFiltered(filtered);
                this.syncGraph();
                return;
            }
            if (filtered.Databases) {
                filtered.Databases.SkippedResources = filtered.Databases.SkippedResources.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.Databases.Servers = filtered.Databases.Servers.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.Databases.Connections = filtered.Databases.Connections.filter(function (s) { return s.Source.indexOf(criteria) > -1 && s.Destination.indexOf(criteria) > -1; });
            }
            if (filtered.FileSystems) {
                filtered.FileSystems.SkippedResources = filtered.FileSystems.SkippedResources.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.FileSystems.Servers = filtered.FileSystems.Servers.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.FileSystems.Connections = filtered.FileSystems.Connections.filter(function (s) { return s.Source.indexOf(criteria) > -1 && s.Destination.indexOf(criteria) > -1; });
            }
            if (filtered.Counters) {
                filtered.Counters.SkippedResources = filtered.Counters.SkippedResources.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.Counters.Servers = filtered.Counters.Servers.filter(function (s) { return s.indexOf(criteria) > -1; });
                filtered.Counters.Connections = filtered.Counters.Connections.filter(function (s) { return s.Source.indexOf(criteria) > -1 && s.Destination.indexOf(criteria) > -1; });
            }
            this.topologyFiltered(filtered);
            this.syncGraph();
        };
        topology.prototype.resetZoom = function () {
            this.zoom.scale(1);
            this.zoom.translate([0, 0]);
            this.svg.select('.graphZoom')
                .transition()
                .attr('transform', 'translate(0,0)scale(1)');
        };
        topology.inlineCss = " svg { background-color: white; }" +
            " path.link { fill: none; stroke: #38b44a; stroke-width: 5px; cursor: default; } " +
            " path.link.error {  stroke: #df382c; } " +
            " svg:not(.active):not(.ctrl) path.link { cursor: pointer; } " +
            " path.link.hidden {  stroke-width: 0; } " +
            " rect.node {  stroke-width: 1.5px;  fill: rgba(243, 101, 35, 0.15); stroke: #d74c0c;  } " +
            " text { font: 12px sans-serif;  pointer-events: none;  } " +
            " text.id { text-anchor: middle;  font-weight: bold;  }";
        return topology;
    })(viewModelBase);
    return topology;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/trafficWatch", ["require", "exports", "durandal/app", "viewmodels/viewModelBase", "viewmodels/manage/watchTrafficConfigDialog", "common/trafficWatchClient", "commands/auth/getSingleAuthTokenCommand", "moment", "common/fileDownloader", "commands/database/query/enableQueryTimings", "viewmodels/shell"], function (require, exports, app, viewModelBase, watchTrafficConfigDialog, trafficWatchClient, getSingleAuthTokenCommand, moment, fileDownloader, enableQueryTimings, shell) {
    var trafficWatch = (function (_super) {
        __extends(trafficWatch, _super);
        function trafficWatch() {
            var _this = this;
            _super.call(this);
            this.logConfig = ko.observable();
            this.isConnected = ko.observable(false);
            this.recentEntries = ko.observableArray([]);
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;
            this.selectedLog = ko.observable();
            this.keepDown = ko.observable(false);
            this.watchedRequests = ko.observable(0);
            this.averageRequestDuration = ko.observable();
            this.summedRequestsDuration = 0;
            this.minRequestDuration = ko.observable(1000000);
            this.maxRequestDuration = ko.observable(0);
            this.startTraceTime = ko.observable();
            this.showLogDetails = ko.observable(false);
            this.isForbidden = ko.observable();
            this.filter = ko.observable();
            this.filterDuration = ko.observable();
            this.startTraceTimeHumanized = ko.computed(function () {
                var a = _this.now();
                if (!!_this.startTraceTime()) {
                    return _this.parseHumanReadableTimeString(_this.startTraceTime().toString(), true, false);
                }
                return "";
            });
            this.isForbidden(shell.isGlobalAdmin() === false);
            this.filter.throttle(250).subscribe(function () { return _this.filterEntries(); });
            this.filterDuration.throttle(250).subscribe(function () { return _this.filterEntries(); });
        }
        trafficWatch.prototype.filterEntries = function () {
            var _this = this;
            this.recentEntries().forEach(function (entry) {
                entry.Visible(_this.isVisible(entry));
            });
        };
        trafficWatch.prototype.canActivate = function (args) {
            return true;
        };
        trafficWatch.prototype.activate = function (args) {
            var widthUnit = 0.075;
            this.columnWidths = [
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit),
                ko.observable(100 * widthUnit * 7),
                ko.observable(100 * widthUnit)
            ];
            this.registerColumnResizing();
            this.updateHelpLink('EVEP6I');
        };
        trafficWatch.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.showLogDetails.subscribe(function (x) {
                $(".logRecords").toggleClass("logRecords-small");
            });
            this.updateCurrentNowTime();
            this.logRecordsElement = document.getElementById("logRecords");
        };
        trafficWatch.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(document).on("mousedown.logTableColumnResize", ".column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = _this.columnWidths[columnIndex]();
                startX = e.pageX;
                resizingColumn = true;
            });
            $(document).on("mouseup.logTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(document).on("mousemove.logTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var logsRecordsContainerWidth = $("#logRecordsContainer").width();
                    var targetColumnSize = startingWidth + 100 * (e.pageX - startX) / logsRecordsContainerWidth;
                    _this.columnWidths[columnIndex](targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        trafficWatch.prototype.configureConnection = function () {
            var _this = this;
            var configDialog = new watchTrafficConfigDialog();
            app.showDialog(configDialog);
            configDialog.configurationTask.done(function (x) {
                _this.logConfig(x);
                _this.enableTimingsTimer = setInterval(function () { return _this.enableQueryTiming(); }, 4.8 * 60 * 1000);
                _this.enableQueryTiming();
                _this.reconnect();
            });
        };
        trafficWatch.prototype.enableQueryTiming = function () {
            if (this.logConfig().Resource.isDatabase()) {
                new enableQueryTimings(this.logConfig().Resource).execute();
            }
        };
        trafficWatch.prototype.reconnect = function () {
            var _this = this;
            if (!this.watchClient) {
                if (!this.logConfig) {
                    app.showMessage("Cannot reconnect, please configure connection properly", "Connection Error");
                    return;
                }
                this.connect();
            }
            else {
                this.disconnect().done(function () {
                    _this.connect();
                });
            }
        };
        trafficWatch.prototype.connect = function () {
            var _this = this;
            if (!!this.watchClient) {
                this.reconnect();
                return;
            }
            if (!this.logConfig()) {
                this.configureConnection();
                return;
            }
            var tokenDeferred = $.Deferred();
            if (!this.logConfig().SingleAuthToken) {
                new getSingleAuthTokenCommand(this.logConfig().Resource, this.logConfig().WatchedResourceMode === "AdminView")
                    .execute()
                    .done(function (tokenObject) {
                    _this.logConfig().SingleAuthToken = tokenObject;
                    tokenDeferred.resolve();
                })
                    .fail(function (e) {
                    app.showMessage("You are not authorized to trace this resource", "Authorization error");
                });
            }
            else {
                tokenDeferred.resolve();
            }
            tokenDeferred.done(function () {
                _this.watchClient = new trafficWatchClient(_this.logConfig().ResourcePath, _this.logConfig().SingleAuthToken.Token);
                _this.watchClient.connect();
                _this.watchClient.connectionOpeningTask.done(function () {
                    _this.isConnected(true);
                    _this.watchClient.watchTraffic(function (event) {
                        _this.processHttpTraceMessage(event);
                    });
                    if (!_this.startTraceTime()) {
                        _this.startTraceTime(_this.now());
                    }
                });
                _this.logConfig().SingleAuthToken = null;
            });
        };
        trafficWatch.prototype.disconnect = function () {
            var _this = this;
            if (!!this.watchClient) {
                this.watchClient.disconnect();
                return this.watchClient.connectionClosingTask.done(function () {
                    _this.watchClient = null;
                    _this.isConnected(false);
                });
            }
            else {
                app.showMessage("Cannot disconnect, connection does not exist", "Disconnect");
                return $.Deferred().reject();
            }
        };
        trafficWatch.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (this.enableTimingsTimer) {
                clearInterval(this.enableTimingsTimer);
                this.enableTimingsTimer = null;
            }
            if (this.isConnected())
                this.disconnect();
        };
        trafficWatch.prototype.isVisible = function (logEntry) {
            if (this.filterDuration() && logEntry.Duration < parseInt(this.filterDuration())) {
                return false;
            }
            if (this.filter() && logEntry.Url.indexOf(this.filter()) === -1) {
                return false;
            }
            return true;
        };
        trafficWatch.prototype.processHttpTraceMessage = function (e) {
            var logObject;
            var mapTimings = function (value) {
                var result = [];
                if (!value) {
                    return result;
                }
                for (var key in value) {
                    if (value.hasOwnProperty(key)) {
                        result.push({ key: key, value: value[key] });
                    }
                }
                return result;
            };
            logObject = {
                Time: this.createHumanReadableTime(e.TimeStamp, false, true),
                Duration: e.ElapsedMilliseconds,
                Resource: e.TenantName,
                Method: e.HttpMethod,
                Url: e.RequestUri,
                CustomInfo: e.CustomInfo,
                TimeStampText: this.createHumanReadableTime(e.TimeStamp, true, false),
                QueryTimings: mapTimings(e.QueryTimings),
                Visible: ko.observable()
            };
            if (logObject.CustomInfo) {
                logObject.CustomInfo = decodeURIComponent(logObject.CustomInfo).replaceAll("\n", "<Br />").replaceAll("Inner Request", "<strong>Inner Request</strong>");
            }
            if (e.InnerRequestsCount > 0) {
                logObject.Url = "(" + e.InnerRequestsCount + " requests) " + logObject.Url;
            }
            if (this.recentEntries().length == this.logConfig().MaxEntries) {
                this.recentEntries.shift();
            }
            logObject.Visible(this.isVisible(logObject));
            this.recentEntries.push(logObject);
            if (this.keepDown()) {
                this.logRecordsElement.scrollTop = this.logRecordsElement.scrollHeight * 1.1;
            }
            this.watchedRequests(this.watchedRequests() + 1);
            this.summedRequestsDuration += e.ElapsedMilliseconds;
            this.averageRequestDuration((this.summedRequestsDuration / this.watchedRequests()).toFixed(2));
            this.minRequestDuration(this.minRequestDuration() > e.ElapsedMilliseconds ? e.ElapsedMilliseconds : this.minRequestDuration());
            this.maxRequestDuration(this.maxRequestDuration() < e.ElapsedMilliseconds ? e.ElapsedMilliseconds : this.maxRequestDuration());
        };
        trafficWatch.prototype.selectLog = function (log) {
            this.selectedLog(log);
            this.showLogDetails(true);
            $(".logRecords").addClass("logRecords-small");
        };
        trafficWatch.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            if (this.updateNowTimeoutHandle != 0)
                clearTimeout(this.updateNowTimeoutHandle);
            this.updateNowTimeoutHandle = setTimeout(function () { return _this.updateCurrentNowTime(); }, 1000);
        };
        trafficWatch.prototype.createHumanReadableTime = function (time, chainHumanized, chainDateTime) {
            var _this = this;
            if (chainHumanized === void 0) { chainHumanized = true; }
            if (chainDateTime === void 0) { chainDateTime = true; }
            if (time) {
                return ko.computed(function () {
                    return _this.parseHumanReadableTimeString(time, chainHumanized, chainDateTime);
                });
            }
            return ko.computed(function () { return time; });
        };
        trafficWatch.prototype.parseHumanReadableTimeString = function (time, chainHumanized, chainDateTime) {
            if (chainHumanized === void 0) { chainHumanized = true; }
            if (chainDateTime === void 0) { chainDateTime = true; }
            var dateMoment = moment(time);
            var humanized = "", formattedDateTime = "";
            var agoInMs = dateMoment.diff(this.now());
            if (chainHumanized)
                humanized = moment.duration(agoInMs).humanize(true);
            if (chainDateTime)
                formattedDateTime = dateMoment.format(" (ddd MMM DD YYYY HH:mm:ss.SS[GMT]ZZ)");
            return humanized + formattedDateTime;
        };
        trafficWatch.prototype.formatLogRecord = function (logRecord) {
            return 'Request #' + logRecord.RequestId.toString().paddingRight(' ', 4) + ' ' + logRecord.HttpMethod.paddingLeft(' ', 7) + ' - ' + logRecord.ElapsedMilliseconds.toString().paddingRight(' ', 5) + ' ms - ' + logRecord.TenantName.paddingLeft(' ', 10) + ' - ' + logRecord.ResponseStatusCode + ' - ' + logRecord.RequestUri;
        };
        trafficWatch.prototype.resetStats = function () {
            this.watchedRequests(0);
            this.averageRequestDuration("0");
            this.summedRequestsDuration = 0;
            this.minRequestDuration(1000000);
            this.maxRequestDuration(0);
            this.startTraceTime(null);
        };
        trafficWatch.prototype.exportTraffic = function () {
            fileDownloader.downloadAsJson(this.recentEntries(), "traffic.json");
        };
        trafficWatch.prototype.clearLogs = function () {
            this.recentEntries.removeAll();
        };
        trafficWatch.prototype.toggleKeepDown = function () {
            this.keepDown.toggle();
            if (this.keepDown()) {
                this.logRecordsElement.scrollTop = this.logRecordsElement.scrollHeight * 1.1;
            }
        };
        return trafficWatch;
    })(viewModelBase);
    return trafficWatch;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/watchTrafficConfigDialog", ["require", "exports", "durandal/app", "plugins/dialog", "viewmodels/dialogViewModelBase", "viewmodels/shell", "commands/auth/getSingleAuthTokenCommand", "common/appUrl"], function (require, exports, app, dialog, dialogViewModelBase, shell, getSingleAuthTokenCommand, appUrl) {
    var watchTrafficConfigDialog = (function (_super) {
        __extends(watchTrafficConfigDialog, _super);
        function watchTrafficConfigDialog() {
            var _this = this;
            _super.call(this);
            this.configurationTask = $.Deferred();
            this.watchedResourceMode = ko.observable("SingleResourceView");
            this.resourceName = ko.observable("");
            this.lastSearchedwatchedResourceName = ko.observable();
            this.resourceAutocompletes = ko.observableArray([]);
            this.maxEntries = ko.observable(1000);
            this.allResourcesNames = shell.resourcesNamesComputed();
            this.searchResults = ko.computed(function () {
                var newResourceName = _this.resourceName();
                return _this.allResourcesNames().filter(function (name) { return name.toLowerCase().indexOf(newResourceName.toLowerCase()) > -1; });
            });
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                var isSingleResourceView = _this.watchedResourceMode() === "SingleResourceView";
                if (isSingleResourceView) {
                    if (!newResourceName) {
                        errorMessage = "Resource name is required";
                    }
                    else {
                        var foundResource = _this.allResourcesNames().first(function (name) { return name === newResourceName; });
                        if (!foundResource) {
                            errorMessage = "Resource name doesn't exist!";
                        }
                    }
                }
                return errorMessage;
            });
        }
        watchTrafficConfigDialog.prototype.bindingComplete = function () {
            document.getElementById("watchedResource").focus();
        };
        watchTrafficConfigDialog.prototype.enterKeyPressed = function () {
            return true;
        };
        watchTrafficConfigDialog.prototype.fetchResourcesAutocompletes = function (search) {
            if (this.resourceName() === search) {
                if (this.resourceAutocompletes.length === 1 && this.resourceName() === this.resourceAutocompletes()[0]) {
                    this.resourceAutocompletes.removeAll();
                    return;
                }
                this.resourceAutocompletes(this.allResourcesNames().filter(function (name) { return name.toLowerCase().indexOf(search.toLowerCase()) === 0; }));
            }
        };
        watchTrafficConfigDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        watchTrafficConfigDialog.prototype.deactivate = function () {
            this.configurationTask.reject();
        };
        watchTrafficConfigDialog.prototype.confirmConfig = function () {
            var _this = this;
            var tracedResource;
            if (this.watchedResourceMode() === "SingleResourceView")
                tracedResource = shell.resources().first(function (rs) { return rs.name === _this.resourceName(); });
            tracedResource = tracedResource || appUrl.getSystemDatabase();
            var resourcePath = appUrl.forResourceQuery(tracedResource);
            var getTokenTask = new getSingleAuthTokenCommand(tracedResource, this.watchedResourceMode() === "AdminView").execute();
            getTokenTask
                .done(function (tokenObject) {
                _this.configurationTask.resolve({
                    Resource: tracedResource,
                    ResourceName: tracedResource.name,
                    ResourcePath: resourcePath,
                    MaxEntries: _this.maxEntries(),
                    WatchedResourceMode: _this.watchedResourceMode(),
                    SingleAuthToken: tokenObject
                });
                dialog.close(_this);
            })
                .fail(function (e) {
                var response = JSON.parse(e.responseText);
                var msg = e.statusText;
                if ("Error" in response) {
                    msg += ": " + response.Error;
                }
                else if ("Reason" in response) {
                    msg += ": " + response.Reason;
                }
                app.showMessage(msg, "Error");
            });
        };
        watchTrafficConfigDialog.prototype.generateBindingInputId = function (index) {
            return "binding-" + index;
        };
        return watchTrafficConfigDialog;
    })(dialogViewModelBase);
    return watchTrafficConfigDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/windowsAuth", ["require", "exports", "models/auth/windowsAuthSetup", "models/auth/windowsAuthData", "viewmodels/viewModelBase", "commands/auth/getWindowsAuthCommand", "commands/auth/saveWindowsAuthCommand", "common/settingsAccessAuthorizer"], function (require, exports, windowsAuthSetup, windowsAuthData, viewModelBase, getWindowsAuthCommand, saveWindowsAuthCommand, settingsAccessAuthorizer) {
    var windowsAuth = (function (_super) {
        __extends(windowsAuth, _super);
        function windowsAuth() {
            _super.apply(this, arguments);
            this.setup = ko.observable();
            this.isUsersSectionActive = ko.observable(true);
            this.settingsAccess = new settingsAccessAuthorizer();
        }
        windowsAuth.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            if (this.settingsAccess.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.setup(new windowsAuthSetup({ RequiredUsers: [], RequiredGroups: [] }));
                this.fetchWindowsAuth().always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        windowsAuth.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink('ZDGUY9');
            this.dirtyFlag = new ko.DirtyFlag([this.setup]);
            this.isSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        windowsAuth.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
            if (this.settingsAccess.isReadOnly()) {
                $('#manageWindowsAuth input').attr('readonly', 'readonly');
                $('#manageWindowsAuth button').attr('disabled', 'true');
            }
            $("#manageWindowsAuth").on("keypress", 'input[name="databaseName"]', function (e) { return e.which !== 13; });
        };
        windowsAuth.prototype.fetchWindowsAuth = function () {
            var _this = this;
            return new getWindowsAuthCommand()
                .execute()
                .done(function (result) { return _this.setup(result); });
        };
        windowsAuth.prototype.saveChanges = function () {
            var _this = this;
            new saveWindowsAuthCommand(this.setup().toDto())
                .execute()
                .done(function () { return _this.dirtyFlag().reset(); });
        };
        windowsAuth.prototype.addUserSettings = function () {
            var newAuthData = windowsAuthData.empty();
            windowsAuthSetup.subscribeToObservableName(newAuthData, this.setup().requiredUsers, windowsAuthSetup.ModeUser);
            this.setup().requiredUsers.push(newAuthData);
        };
        windowsAuth.prototype.removeUserSettings = function (data) {
            this.setup().requiredUsers.remove(data);
        };
        windowsAuth.prototype.addGroupSettings = function () {
            var newAuthData = windowsAuthData.empty();
            windowsAuthSetup.subscribeToObservableName(newAuthData, this.setup().requiredGroups, windowsAuthSetup.ModeGroup);
            this.setup().requiredGroups.push(newAuthData);
        };
        windowsAuth.prototype.removeGroupSettings = function (data) {
            this.setup().requiredGroups.remove(data);
        };
        return windowsAuth;
    })(viewModelBase);
    return windowsAuth;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfig", ["require", "exports", "viewmodels/viewModelBase", "common/appUrl", "viewmodels/manage/adminSettings", "models/auth/license"], function (require, exports, viewModelBase, appUrl, adminSettings, license) {
    var globalConfig = (function (_super) {
        __extends(globalConfig, _super);
        function globalConfig() {
            var _this = this;
            _super.call(this);
            this.router = adminSettings.adminSettingsRouter.createChildRouter()
                .map([
                { route: "globalConfig", moduleId: "viewmodels/manage/globalConfig/globalConfigPeriodicExport", title: "Periodic export", tooltip: "", nav: true, hash: appUrl.forGlobalConfigPeriodicExport() },
                { route: "globalConfigReplication", moduleId: "viewmodels/manage/globalConfig/globalConfigReplications", title: "Replication", tooltip: "Global replication settings", nav: true, hash: appUrl.forGlobalConfigReplication() },
                { route: "globalConfigSqlReplication", moduleId: "viewmodels/manage/globalConfig/globalConfigSqlReplication", title: "SQL Replication", tooltip: "Global SQL replication settings", nav: true, hash: appUrl.forGlobalConfigSqlReplication() },
                { route: "globalConfigQuotas", moduleId: "viewmodels/manage/globalConfig/globalConfigQuotas", title: "Quotas", tooltip: "Global quotas settings", nav: true, hash: appUrl.forGlobalConfigQuotas() },
                { route: "globalConfigCustomFunctions", moduleId: "viewmodels/manage/globalConfig/globalConfigCustomFunctions", title: "Custom functions", tooltip: "Global custom functions settings", nav: true, hash: appUrl.forGlobalConfigCustomFunctions() },
                { route: "globalConfigVersioning", moduleId: "viewmodels/manage/globalConfig/globalConfigVersioning", title: "Versioning", tooltip: "Global versioning settings", nav: true, hash: appUrl.forGlobalConfigVersioning() }
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.currentRouteTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        globalConfig.developerLicense = ko.computed(function () { return !license.licenseStatus().IsCommercial; });
        globalConfig.canUseGlobalConfigurations = ko.computed(function () { return !license.licenseStatus().IsCommercial || license.licenseStatus().Attributes.globalConfigurations === "true"; });
        return globalConfig;
    })(viewModelBase);
    return globalConfig;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigCustomFunctions", ["require", "exports", "common/bindingHelpers/aceEditorBindingHandler", "viewmodels/viewModelBase", "commands/database/documents/getCustomFunctionsCommand", "commands/database/documents/deleteDocumentCommand", "commands/database/documents/saveCustomFunctionsCommand", "models/database/documents/customFunctions", "common/jsonUtil", "common/messagePublisher", "common/appUrl", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, aceEditorBindingHandler, viewModelBase, getCustomFunctionsCommand, deleteDocumentCommand, saveCustomFunctionsCommand, customFunctions, jsonUtil, messagePublisher, appUrl, globalConfig, settingsAccessAuthorizer) {
    var globalConfigCustomFunctions = (function (_super) {
        __extends(globalConfigCustomFunctions, _super);
        function globalConfigCustomFunctions() {
            var _this = this;
            _super.call(this);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.activated = ko.observable(false);
            this.settingsAccess = new settingsAccessAuthorizer();
            aceEditorBindingHandler.install();
            this.documentText = ko.observable("");
            this.fetchCustomFunctions();
            this.dirtyFlag = new ko.DirtyFlag([this.documentText], false, jsonUtil.newLineNormalizingHashFunction);
            this.isSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.dirtyFlag().isDirty(); });
        }
        globalConfigCustomFunctions.prototype.attached = function () {
            _super.prototype.attached.call(this);
            $("#customFunctionsExample").popover({
                html: true,
                trigger: "hover",
                content: "Examples:<pre>exports.greet = <span class=\"code-keyword\">function</span>(name) {<br/>    <span class=\"code-keyword\">return</span> <span class=\"code-string\">\"Hello \" + name + \"!\"</span>;<br/>}</pre>"
            });
        };
        globalConfigCustomFunctions.prototype.compositionComplete = function () {
            var _this = this;
            _super.prototype.compositionComplete.call(this);
            var editorElement = $(".custom-functions-form .editor");
            if (editorElement.length > 0) {
                this.docEditor = ko.utils.domData.get(editorElement[0], "aceEditor");
            }
            $("#customFunctionsEditor").on("DynamicHeightSet", function () { return _this.docEditor.resize(); });
            this.fetchCustomFunctions();
        };
        globalConfigCustomFunctions.prototype.detached = function () {
            _super.prototype.detached.call(this);
            $("#customFunctionsEditor").off("DynamicHeightSet");
        };
        globalConfigCustomFunctions.prototype.fetchCustomFunctions = function () {
            var _this = this;
            var fetchTask = new getCustomFunctionsCommand(appUrl.getSystemDatabase(), true).execute();
            fetchTask.done(function (cf) {
                _this.documentText(cf.functions);
                _this.activated(true);
                _this.dirtyFlag().reset();
            }).fail(function (xhr) {
                if (xhr.status === 404) {
                    _this.activated(false);
                }
            });
        };
        globalConfigCustomFunctions.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigCustomFunctions.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            if (deleteConfig) {
                new deleteDocumentCommand("Raven/Global/Javascript/Functions", appUrl.getSystemDatabase())
                    .execute()
                    .done(function () { return messagePublisher.reportSuccess("Global Settings were successfully saved!"); })
                    .fail(function (response) { return messagePublisher.reportError("Failed to save global settings!", response.responseText, response.statusText); });
            }
            else {
                var annotations = this.docEditor.getSession().getAnnotations();
                var hasErrors = false;
                for (var i = 0; i < annotations.length; i++) {
                    if (annotations[i].type === "error") {
                        hasErrors = true;
                        break;
                    }
                }
                if (!hasErrors) {
                    var cf = new customFunctions({
                        Functions: this.documentText()
                    });
                    var saveTask = new saveCustomFunctionsCommand(appUrl.getSystemDatabase(), cf, true).execute();
                    saveTask.done(function () { return _this.dirtyFlag().reset(); });
                }
                else {
                    messagePublisher.reportError("Errors in the functions file", "Please correct the errors in the file in order to save it.");
                }
            }
        };
        globalConfigCustomFunctions.prototype.activateConfig = function () {
            this.activated(true);
            this.docEditor.resize();
        };
        globalConfigCustomFunctions.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for custom functions?", "Are you sure?")
                .done(function () {
                _this.documentText("");
                _this.activated(false);
                _this.syncChanges(true);
                _this.dirtyFlag().reset();
            });
        };
        return globalConfigCustomFunctions;
    })(viewModelBase);
    return globalConfigCustomFunctions;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigPeriodicExport", ["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getPeriodicExportSetupCommand", "commands/maintenance/savePeriodicExportSetupCommand", "models/database/documents/document", "commands/database/documents/deleteDocumentCommand", "models/database/documents/periodicExportSetup", "common/appUrl", "commands/database/globalConfig/getEffectiveSettingsCommand", "commands/database/globalConfig/saveGlobalSettingsCommand", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, getPeriodicExportSetupCommand, savePeriodicExportSetupCommand, document, deleteDocumentCommand, periodicExportSetup, appUrl, getEffectiveSettingsCommand, saveGlobalSettingsCommand, globalConfig, settingsAccessAuthorizer) {
    var globalConfigPeriodicExport = (function (_super) {
        __extends(globalConfigPeriodicExport, _super);
        function globalConfigPeriodicExport() {
            _super.apply(this, arguments);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.settingsDocument = ko.observable();
            this.activated = ko.observable(false);
            this.settingsAccess = new settingsAccessAuthorizer();
            this.backupSetup = ko.observable();
        }
        globalConfigPeriodicExport.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            this.backupSetup(new periodicExportSetup());
            var deferred = $.Deferred();
            var db = appUrl.getSystemDatabase();
            if (db) {
                if (this.settingsAccess.isForbidden()) {
                    deferred.resolve({ can: true });
                }
                else {
                    $.when(this.fetchPeriodicExportSetup(db), this.fetchPeriodicExportAccountsSettings(db))
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forAdminSettings() }); });
                }
            }
            return deferred;
        };
        globalConfigPeriodicExport.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            var self = this;
            this.dirtyFlag = new ko.DirtyFlag([this.backupSetup]);
            this.isSaveEnabled = ko.computed(function () {
                var isNotReadOnly = !_this.settingsAccess.isReadOnly();
                var onDisk = self.backupSetup().onDiskExportEnabled();
                var remote = self.backupSetup().remoteUploadEnabled();
                var hasAnyOption = onDisk || remote;
                var isDirty = _this.dirtyFlag().isDirty();
                return isNotReadOnly && hasAnyOption && isDirty;
            });
        };
        globalConfigPeriodicExport.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.bindPopover();
            this.bindHintWatchers();
        };
        globalConfigPeriodicExport.prototype.bindPopover = function () {
            $("#onDiskHint").popover({
                html: true,
                container: $("body"),
                trigger: "hover",
                content: "Database name will be appended to path in target configuration. <br /><br />" +
                    "For value: <code>C:\\exports\\</code> target path will be: <code>C:\\exports\\{databaseName}</code>"
            });
            $(".folderHint").popover({
                html: true,
                container: $("body"),
                trigger: "hover",
                content: "Folder name will be replaced with database name being exported in local configuration."
            });
        };
        globalConfigPeriodicExport.prototype.fetchPeriodicExportSetup = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getPeriodicExportSetupCommand(db, true)
                .execute()
                .done(function (result) {
                _this.backupSetup().fromDto(result);
                _this.activated(true);
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        globalConfigPeriodicExport.prototype.fetchPeriodicExportAccountsSettings = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getEffectiveSettingsCommand(db)
                .execute()
                .done(function (doc) {
                _this.settingsDocument(doc);
                _this.backupSetup().fromDatabaseSettingsDto(doc.toDto(true));
            })
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        globalConfigPeriodicExport.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigPeriodicExport.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            var db = appUrl.getSystemDatabase();
            if (db) {
                if (deleteConfig) {
                    new deleteDocumentCommand("Raven/Global/Backup/Periodic/Setup", appUrl.getSystemDatabase())
                        .execute();
                    this.deleteSettings(db);
                }
                else {
                    var saveTask = new savePeriodicExportSetupCommand(this.backupSetup(), db, true).execute();
                    saveTask.done(function (resultArray) {
                        var newEtag = resultArray[0].ETag;
                        _this.backupSetup().setEtag(newEtag);
                        _this.settingsDocument().__metadata["@etag"] = newEtag;
                        _this.dirtyFlag().reset(); // Resync Changes
                    });
                }
            }
        };
        globalConfigPeriodicExport.prototype.bindHintWatchers = function () {
            var _this = this;
            this.backupSetup().remoteUploadEnabled.subscribe(function () { return _this.bindPopover(); });
            this.backupSetup().type.subscribe(function () { return _this.bindPopover(); });
        };
        globalConfigPeriodicExport.prototype.deleteSettings = function (db) {
            var _this = this;
            var settingsDocument = this.settingsDocument();
            settingsDocument["@metadata"] = this.settingsDocument().__metadata;
            settingsDocument["@metadata"]["@etag"] = this.settingsDocument().__metadata["@etag"];
            var copyOfSettings = settingsDocument.toDto(true);
            delete copyOfSettings["Settings"]["Raven/AWSAccessKey"];
            delete copyOfSettings["Settings"]["Raven/AzureStorageAccount"];
            delete copyOfSettings["SecuredSettings"]["Raven/AWSSecretKey"];
            delete copyOfSettings["SecuredSettings"]["Raven/AzureStorageKey"];
            var doc = new document(copyOfSettings);
            this.backupSetup(new periodicExportSetup());
            this.bindHintWatchers();
            this.backupSetup().fromDatabaseSettingsDto(copyOfSettings);
            var saveTask = new saveGlobalSettingsCommand(db, doc).execute();
            saveTask.done(function (saveResult) {
                _this.backupSetup().setEtag(saveResult.ETag);
                _this.settingsDocument().__metadata["@etag"] = saveResult.ETag;
                _this.dirtyFlag().reset(); //Resync Changes
            });
        };
        globalConfigPeriodicExport.prototype.activateConfig = function () {
            this.activated(true);
            this.bindPopover();
        };
        globalConfigPeriodicExport.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for periodic export?", "Are you sure?")
                .done(function () {
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        return globalConfigPeriodicExport;
    })(viewModelBase);
    return globalConfigPeriodicExport;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigQuotas", ["require", "exports", "viewmodels/viewModelBase", "commands/database/globalConfig/getEffectiveSettingsCommand", "commands/database/globalConfig/saveGlobalSettingsCommand", "models/database/documents/document", "common/appUrl", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, getEffectiveSettingsCommand, saveGlobalSettingsCommand, document, appUrl, globalConfig, settingsAccessAuthorizer) {
    var globalConfigQuotas = (function (_super) {
        __extends(globalConfigQuotas, _super);
        function globalConfigQuotas() {
            _super.apply(this, arguments);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.settingsDocument = ko.observable();
            this.activated = ko.observable(false);
            this.settingsAccess = new settingsAccessAuthorizer();
            this.maximumSize = ko.observable();
            this.warningLimitThreshold = ko.observable();
            this.maxNumberOfDocs = ko.observable();
            this.warningThresholdForDocs = ko.observable();
        }
        globalConfigQuotas.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = appUrl.getSystemDatabase();
            if (db) {
                if (this.settingsAccess.isForbidden()) {
                    deferred.resolve({ can: true });
                }
                else {
                    // fetch current quotas from the database
                    this.fetchQuotas(db)
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(_this.activeDatabase()) }); });
                }
            }
            return deferred;
        };
        globalConfigQuotas.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.initializeDirtyFlag();
            this.isSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        globalConfigQuotas.prototype.fetchQuotas = function (db) {
            var _this = this;
            return new getEffectiveSettingsCommand(db)
                .execute()
                .done(function (doc) {
                _this.settingsDocument(doc);
                // we make decision based on first available property 
                var activated = !!doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                _this.activated(activated);
                if (activated) {
                    _this.maximumSize(doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] / 1024);
                    _this.warningLimitThreshold(doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] / 1024);
                    _this.maxNumberOfDocs(doc["Settings"]["Raven/Quotas/Documents/HardLimit"]);
                    _this.warningThresholdForDocs(doc["Settings"]["Raven/Quotas/Documents/SoftLimit"]);
                }
            });
        };
        globalConfigQuotas.prototype.initializeDirtyFlag = function () {
            this.dirtyFlag = new ko.DirtyFlag([
                this.maximumSize,
                this.warningLimitThreshold,
                this.maxNumberOfDocs,
                this.warningThresholdForDocs
            ]);
        };
        globalConfigQuotas.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigQuotas.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            var db = appUrl.getSystemDatabase();
            if (db) {
                var settingsDocument = this.settingsDocument();
                settingsDocument["@metadata"] = this.settingsDocument().__metadata;
                settingsDocument["@metadata"]["@etag"] = this.settingsDocument().__metadata["@etag"];
                var doc = new document(settingsDocument.toDto(true));
                if (deleteConfig) {
                    delete doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"];
                    delete doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"];
                    delete doc["Settings"]["Raven/Quotas/Documents/HardLimit"];
                    delete doc["Settings"]["Raven/Quotas/Documents/SoftLimit"];
                    this.maximumSize(null);
                    this.warningLimitThreshold(null);
                    this.maxNumberOfDocs(null);
                    this.warningThresholdForDocs(null);
                }
                else {
                    doc["Settings"]["Raven/Quotas/Size/HardLimitInKB"] = this.maximumSize() * 1024;
                    doc["Settings"]["Raven/Quotas/Size/SoftMarginInKB"] = this.warningLimitThreshold() * 1024;
                    doc["Settings"]["Raven/Quotas/Documents/HardLimit"] = this.maxNumberOfDocs();
                    doc["Settings"]["Raven/Quotas/Documents/SoftLimit"] = this.warningThresholdForDocs();
                }
                var saveTask = new saveGlobalSettingsCommand(db, doc).execute();
                saveTask.done(function (saveResult) {
                    _this.settingsDocument().__metadata["@etag"] = saveResult.ETag;
                    _this.dirtyFlag().reset(); //Resync Changes
                });
            }
        };
        globalConfigQuotas.prototype.activateConfig = function () {
            this.activated(true);
        };
        globalConfigQuotas.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for quotas?", "Please note that databases with quotas bundle enabled and without local configuration may crash.")
                .done(function () {
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        return globalConfigQuotas;
    })(viewModelBase);
    return globalConfigQuotas;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigReplications", ["require", "exports", "commands/database/documents/deleteDocumentCommand", "viewmodels/viewModelBase", "models/database/replication/replicationsSetup", "models/database/replication/replicationConfig", "models/database/replication/replicationDestination", "commands/resources/getDatabaseStatsCommand", "commands/database/globalConfig/getGlobalConfigReplicationsCommand", "commands/database/replication/saveReplicationDocumentCommand", "commands/database/replication/getAutomaticConflictResolutionDocumentCommand", "commands/database/replication/saveAutomaticConflictResolutionDocumentCommand", "common/appUrl", "common/messagePublisher", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, deleteDocumentCommand, viewModelBase, replicationsSetup, replicationConfig, replicationDestination, getDatabaseStatsCommand, getGlobalConfigReplicationsCommand, saveReplicationDocumentCommand, getAutomaticConflictResolutionDocumentCommand, saveAutomaticConflictResolutionDocumentCommand, appUrl, messagePublisher, globalConfig, settingsAccessAuthorizer) {
    var globalConfigReplications = (function (_super) {
        __extends(globalConfigReplications, _super);
        function globalConfigReplications() {
            _super.apply(this, arguments);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.replicationConfig = ko.observable(new replicationConfig({ DocumentConflictResolution: "None", AttachmentConflictResolution: "None" }));
            this.replicationsSetup = ko.observable(new replicationsSetup({ MergedDocument: { Destinations: [], Source: null } }));
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([]);
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag([]);
            this.settingsAccess = new settingsAccessAuthorizer();
            this.activated = ko.observable(false);
            this.skipIndexReplicationForAllDestinationsStatus = ko.observable();
            this.skipIndexReplicationForAll = ko.observable();
        }
        globalConfigReplications.prototype.refereshSkipIndexReplicationForAllDestinations = function () {
            var _this = this;
            if (this.skipIndexReplicationForAllSubscription != null)
                this.skipIndexReplicationForAllSubscription.dispose();
            var newStatus = this.getIndexReplicationStatusForAllDestinations();
            this.skipIndexReplicationForAll(newStatus === 'all');
            this.skipIndexReplicationForAllSubscription = this.skipIndexReplicationForAll.subscribe(function (newValue) { return _this.toggleIndexReplication(newValue); });
        };
        globalConfigReplications.prototype.getIndexReplicationStatusForAllDestinations = function () {
            var countOfSkipIndexReplication = 0;
            ko.utils.arrayForEach(this.replicationsSetup().destinations(), function (dest) {
                if (dest.skipIndexReplication()) {
                    countOfSkipIndexReplication++;
                }
            });
            // ReSharper disable once ConditionIsAlwaysConst
            if (countOfSkipIndexReplication === 0)
                return 'none';
            if (countOfSkipIndexReplication === this.replicationsSetup().destinations().length)
                return 'all';
            return 'mixed';
        };
        globalConfigReplications.prototype.canActivate = function (args) {
            var deferred = $.Deferred();
            var db = appUrl.getSystemDatabase();
            if (db) {
                if (this.settingsAccess.isForbidden()) {
                    deferred.resolve({ can: true });
                }
                else {
                    $.when(this.fetchAutomaticConflictResolution(db), this.fetchReplications(db))
                        .done(function () { return deferred.resolve({ can: true }); })
                        .fail(function () { return deferred.resolve({ redirect: appUrl.forSettings(db) }); });
                }
            }
            return deferred;
        };
        globalConfigReplications.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.bindPopover();
            this.refereshSkipIndexReplicationForAllDestinations();
        };
        globalConfigReplications.prototype.bindPopover = function () {
            $(".dbNameHint").popover({
                html: true,
                container: $("body"),
                trigger: "hover",
                content: "Database name will be replaced with database name being replicated in local configuration."
            });
        };
        globalConfigReplications.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([this.replicationConfig]);
            this.isConfigSaveEnabled = ko.computed(function () { return _this.replicationConfigDirtyFlag().isDirty(); });
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag([this.replicationsSetup, this.replicationsSetup().destinations(), this.replicationConfig, this.replicationsSetup().clientFailoverBehaviour, this.replicationsSetup().requestTimeSlaThreshold, this.replicationsSetup().showRequestTimeSlaThreshold]);
            this.isSetupSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.replicationsSetupDirtyFlag().isDirty(); });
            var combinedFlag = ko.computed(function () {
                var f1 = _this.replicationConfigDirtyFlag().isDirty();
                var f2 = _this.replicationsSetupDirtyFlag().isDirty();
                return f1 || f2;
            });
            this.dirtyFlag = new ko.DirtyFlag([combinedFlag]);
        };
        globalConfigReplications.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getAutomaticConflictResolutionDocumentCommand(db, true)
                .execute()
                .done(function (repConfig) {
                _this.replicationConfig(new replicationConfig(repConfig));
                _this.activated(true);
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        globalConfigReplications.prototype.fetchReplications = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            ko.postbox.subscribe('skip-index-replication', function () { return _this.refereshSkipIndexReplicationForAllDestinations(); });
            new getGlobalConfigReplicationsCommand(db)
                .execute()
                .done(function (repSetup) {
                _this.replicationsSetup(new replicationsSetup({ MergedDocument: repSetup }));
                _this.replicationsSetup().destinations().forEach(function (d) {
                    d.hasLocal(true);
                    d.hasGlobal(false);
                });
                _this.activated(true);
            })
                .always(function () { return deferred.resolve({ can: true }); });
            return deferred;
        };
        globalConfigReplications.prototype.createNewDestination = function () {
            this.replicationsSetup().destinations.unshift(replicationDestination.empty("{databaseName}"));
            this.refereshSkipIndexReplicationForAllDestinations();
            this.bindPopover();
        };
        globalConfigReplications.prototype.removeDestination = function (repl) {
            this.replicationsSetup().destinations.remove(repl);
        };
        globalConfigReplications.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigReplications.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            if (deleteConfig) {
                var task1 = new deleteDocumentCommand("Raven/Global/Replication/Config", appUrl.getSystemDatabase())
                    .execute();
                var task2 = new deleteDocumentCommand("Raven/Global/Replication/Destinations", appUrl.getSystemDatabase())
                    .execute();
                var combinedTask = $.when(task1, task2);
                combinedTask.done(function () { return messagePublisher.reportSuccess("Global Settings were successfully saved!"); });
                combinedTask.fail(function (response) { return messagePublisher.reportError("Failed to save global settings!", response.responseText, response.statusText); });
                this.resetUIToDefaultState();
            }
            else {
                if (this.isConfigSaveEnabled())
                    this.saveAutomaticConflictResolutionSettings();
                if (this.isSetupSaveEnabled()) {
                    if (this.replicationsSetup().source()) {
                        this.saveReplicationSetup();
                    }
                    else {
                        var db = appUrl.getSystemDatabase();
                        if (db) {
                            new getDatabaseStatsCommand(db)
                                .execute()
                                .done(function (result) {
                                _this.prepareAndSaveReplicationSetup(result.DatabaseId);
                            });
                        }
                    }
                }
            }
        };
        globalConfigReplications.prototype.toggleIndexReplication = function (skipReplicationValue) {
            this.replicationsSetup().destinations().forEach(function (dest) {
                dest.skipIndexReplication(skipReplicationValue);
            });
        };
        globalConfigReplications.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };
        globalConfigReplications.prototype.saveReplicationSetup = function () {
            var _this = this;
            var db = appUrl.getSystemDatabase();
            if (db) {
                new saveReplicationDocumentCommand(this.replicationsSetup().toDto(false), db, true)
                    .execute()
                    .done(function () { return _this.replicationsSetupDirtyFlag().reset(); });
            }
        };
        globalConfigReplications.prototype.saveAutomaticConflictResolutionSettings = function () {
            var _this = this;
            var db = appUrl.getSystemDatabase();
            if (db) {
                new saveAutomaticConflictResolutionDocumentCommand(this.replicationConfig().toDto(), db, true)
                    .execute()
                    .done(function () { return _this.replicationConfigDirtyFlag().reset(); });
            }
        };
        globalConfigReplications.prototype.activateConfig = function () {
            this.activated(true);
        };
        globalConfigReplications.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for replication?", "Are you sure?")
                .done(function () {
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        globalConfigReplications.prototype.resetUIToDefaultState = function () {
            this.replicationConfig().clear();
            var source = this.replicationsSetup().source();
            this.replicationsSetup().clear();
            this.replicationsSetup().source(source);
            this.replicationConfigDirtyFlag().reset();
            this.replicationsSetupDirtyFlag().reset();
        };
        return globalConfigReplications;
    })(viewModelBase);
    return globalConfigReplications;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigSqlReplication", ["require", "exports", "viewmodels/viewModelBase", "commands/database/documents/getDocumentWithMetadataCommand", "models/database/sqlReplication/sqlReplicationConnections", "models/database/sqlReplication/predefinedSqlConnection", "models/database/documents/document", "commands/database/documents/saveDocumentCommand", "common/appUrl", "common/messagePublisher", "commands/database/documents/deleteDocumentCommand", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, getDocumentWithMetadataCommand, sqlReplicationConnections, predefinedSqlConnection, document, saveDocumentCommand, appUrl, messagePublisher, deleteDocumentCommand, globalConfig, settingsAccessAuthorizer) {
    var globalConfigSqlReplication = (function (_super) {
        __extends(globalConfigSqlReplication, _super);
        function globalConfigSqlReplication() {
            _super.apply(this, arguments);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.htmlSelector = "#sqlReplicationConnectionsManagement";
            this.connections = ko.observable();
            this.activated = ko.observable(false);
            this.settingsAccess = new settingsAccessAuthorizer();
        }
        globalConfigSqlReplication.prototype.loadConnections = function () {
            var _this = this;
            return new getDocumentWithMetadataCommand("Raven/Global/SqlReplication/Connections", appUrl.getSystemDatabase())
                .execute()
                .done(function (x) {
                var dto = x.toDto(true);
                _this.connections(new sqlReplicationConnections({ MergedDocument: dto }));
                if (_this.connections().predefinedConnections().length > 0) {
                    _this.connections().predefinedConnections().forEach(function (x) { return _this.subscribeToSqlReplicationConnectionName(x); });
                }
                _this.activated(true);
            })
                .fail(function () {
                _this.connections(sqlReplicationConnections.empty());
            });
        };
        globalConfigSqlReplication.prototype.canActivate = function () {
            var deferred = $.Deferred();
            if (this.settingsAccess.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.loadConnections()
                    .always(function () { return deferred.resolve({ can: true }); });
            }
            return deferred;
        };
        globalConfigSqlReplication.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.dirtyFlag = new ko.DirtyFlag([this.connections]);
            this.isSaveEnabled = ko.computed(function () { return !_this.settingsAccess.isReadOnly() && _this.dirtyFlag().isDirty(); });
        };
        globalConfigSqlReplication.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigSqlReplication.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            if (deleteConfig) {
                new deleteDocumentCommand("Raven/Global/SqlReplication/Connections", appUrl.getSystemDatabase())
                    .execute()
                    .done(function () { return messagePublisher.reportSuccess("Global Settings were successfully saved!"); })
                    .fail(function (response) { return messagePublisher.reportError("Failed to save global settings!", response.responseText, response.statusText); });
            }
            else {
                var newDoc = new document(this.connections().toDto());
                this.attachReservedMetaProperties("Raven/Global/SqlReplication/Connections", newDoc.__metadata);
                var saveCommand = new saveDocumentCommand("Raven/Global/SqlReplication/Connections", newDoc, appUrl.getSystemDatabase());
                var saveTask = saveCommand.execute();
                saveTask.done(function () { return _this.dirtyFlag().reset(); });
            }
        };
        globalConfigSqlReplication.prototype.attachReservedMetaProperties = function (id, target) {
            target.etag = "";
            target.ravenEntityName = target.ravenEntityName || document.getEntityNameFromId(id);
            target.id = id;
        };
        globalConfigSqlReplication.prototype.getSqlReplicationConnectionsUrl = function () {
            return appUrl.forSqlReplicationConnections(appUrl.getSystemDatabase());
        };
        globalConfigSqlReplication.prototype.addSqlReplicationConnection = function () {
            var newPredefinedConnection = predefinedSqlConnection.empty();
            this.connections().predefinedConnections.splice(0, 0, newPredefinedConnection);
            this.subscribeToSqlReplicationConnectionName(newPredefinedConnection);
            newPredefinedConnection.name("New");
        };
        globalConfigSqlReplication.prototype.removeSqlReplicationConnection = function (connection) {
            this.connections().predefinedConnections.remove(connection);
        };
        globalConfigSqlReplication.prototype.subscribeToSqlReplicationConnectionName = function (con) {
            var _this = this;
            con.name.subscribe(function () {
                //Get the previous value of 'name' here before it's set to newValue
                $("input[name=\"name\"]")
                    .each(function (index, inputField) {
                    inputField.setCustomValidity("");
                });
            }, this, "beforeChange");
            con.name.subscribe(function (newName) {
                var message = "";
                if (newName === "") {
                    message = "Please fill out this field.";
                }
                else if (_this.isSqlPredefinedConnectionNameExists(newName)) {
                    message = "SQL Replication Connection name already exists.";
                }
                $("input[name=\"name\"]")
                    .filter(function () { return this.value === newName; })
                    .each(function (index, element) {
                    element.setCustomValidity(message);
                });
            });
        };
        globalConfigSqlReplication.prototype.isSqlPredefinedConnectionNameExists = function (connectionName) {
            return this.connections().predefinedConnections().count(function (x) { return x.name() === connectionName; }) > 1;
        };
        globalConfigSqlReplication.prototype.providerChanged = function (obj, event) {
            if (event.originalEvent) {
                var curConnectionString = !!obj.connectionString() ? obj.connectionString().trim() : "";
                if (curConnectionString === "" ||
                    sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ConnectionString === curConnectionString; })) {
                    var matchingConnectionStringPair = sqlReplicationConnections.sqlProvidersConnectionStrings.first(function (x) { return x.ProviderName == event.originalEvent.srcElement.selectedOptions[0].value; });
                    if (!!matchingConnectionStringPair) {
                        var matchingConnectionStringValue = matchingConnectionStringPair.ConnectionString;
                        obj.connectionString(matchingConnectionStringValue);
                    }
                }
            }
        };
        globalConfigSqlReplication.prototype.activateConfig = function () {
            this.activated(true);
        };
        globalConfigSqlReplication.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for sql replication?", "Are you sure?")
                .done(function () {
                _this.connections().predefinedConnections.removeAll();
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        return globalConfigSqlReplication;
    })(viewModelBase);
    return globalConfigSqlReplication;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/manage/globalConfig/globalConfigVersioning", ["require", "exports", "viewmodels/viewModelBase", "models/database/documents/versioningEntry", "common/appUrl", "commands/database/documents/getVersioningsCommand", "commands/database/documents/saveVersioningCommand", "viewmodels/manage/globalConfig/globalConfig", "common/settingsAccessAuthorizer"], function (require, exports, viewModelBase, versioningEntry, appUrl, getVersioningsCommand, saveVersioningCommand, globalConfig, settingsAccessAuthorizer) {
    var globalConfigVersioning = (function (_super) {
        __extends(globalConfigVersioning, _super);
        function globalConfigVersioning() {
            _super.apply(this, arguments);
            this.activated = ko.observable(false);
            this.developerLicense = globalConfig.developerLicense;
            this.canUseGlobalConfigurations = globalConfig.canUseGlobalConfigurations;
            this.versionings = ko.observableArray();
            this.settingsAccess = new settingsAccessAuthorizer();
        }
        globalConfigVersioning.prototype.canActivate = function (args) {
            _super.prototype.canActivate.call(this, args);
            var deferred = $.Deferred();
            var db = appUrl.getSystemDatabase();
            if (this.settingsAccess.isForbidden()) {
                deferred.resolve({ can: true });
            }
            else {
                this.fetchVersioningEntries(db)
                    .done(function () { return deferred.resolve({ can: true }); })
                    .fail(function () { return deferred.resolve({ redirect: appUrl.forDatabaseSettings(db) }); });
            }
            return deferred;
        };
        globalConfigVersioning.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.updateHelpLink("1UZ5WL");
            this.toRemove = [];
            this.dirtyFlag = new ko.DirtyFlag([this.versionings]);
            this.isSaveEnabled = ko.computed(function () { return _this.dirtyFlag().isDirty(); });
        };
        globalConfigVersioning.prototype.fetchVersioningEntries = function (db) {
            var _this = this;
            var task = new getVersioningsCommand(db, true).execute();
            task.done(function (versionings) { return _this.versioningsLoaded(versionings); });
            return task;
        };
        globalConfigVersioning.prototype.saveChanges = function () {
            this.syncChanges(false);
        };
        globalConfigVersioning.prototype.syncChanges = function (deleteConfig) {
            var _this = this;
            var db = appUrl.getSystemDatabase();
            if (deleteConfig) {
                var deleteTask = new saveVersioningCommand(db, [], this.versionings().map(function (v) { return v.toDto(true); }).concat(this.toRemove.map(function (v) { return v.toDto(true); })), true).execute();
                deleteTask.done(function (saveResult) {
                    _this.activated(false);
                    _this.versionings([]);
                    _this.versioningsSaved(saveResult);
                });
            }
            else {
                var saveTask = new saveVersioningCommand(db, this.versionings().map(function (v) { return v.toDto(true); }), this.toRemove.map(function (v) { return v.toDto(true); }), true).execute();
                saveTask.done(function (saveResult) {
                    _this.versioningsSaved(saveResult);
                });
            }
        };
        globalConfigVersioning.prototype.createNewVersioning = function () {
            this.versionings.push(new versioningEntry());
        };
        globalConfigVersioning.prototype.removeVersioning = function (entry) {
            if (entry.fromDatabase()) {
                // If this entry is in database schedule the removal
                this.toRemove.push(entry);
            }
            this.versionings.remove(entry);
        };
        globalConfigVersioning.prototype.versioningsLoaded = function (data) {
            this.versionings(data);
            this.dirtyFlag().reset();
            this.activated(data.length > 0);
        };
        globalConfigVersioning.prototype.versioningsSaved = function (saveResult) {
            for (var i = 0; i < this.versionings().length; i++) {
                this.versionings()[i].__metadata.etag = saveResult[i].Etag;
            }
            // After save the collection names are not editable
            this.versionings().forEach(function (v) {
                v.fromDatabase(true);
            });
            this.dirtyFlag().reset();
            this.toRemove = [];
        };
        globalConfigVersioning.prototype.activateConfig = function () {
            this.activated(true);
            this.versionings.push(this.defaultVersioningEntry());
        };
        globalConfigVersioning.prototype.defaultVersioningEntry = function () {
            var entry = new versioningEntry({
                Id: "DefaultConfiguration",
                MaxRevisions: 5,
                Exclude: false,
                ExcludeUnlessExplicit: false,
                PurgeOnDelete: false
            });
            entry.fromDatabase(true);
            return entry;
        };
        globalConfigVersioning.prototype.disactivateConfig = function () {
            var _this = this;
            this.confirmationMessage("Delete global configuration for versioning?", "Are you sure?")
                .done(function () {
                _this.activated(false);
                _this.syncChanges(true);
            });
        };
        globalConfigVersioning.prototype.makeExcluded = function (entry) {
            entry.exclude(true);
        };
        globalConfigVersioning.prototype.makeIncluded = function (entry) {
            entry.exclude(false);
        };
        return globalConfigVersioning;
    })(viewModelBase);
    return globalConfigVersioning;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createCounterStorage", ["require", "exports", "plugins/dialog", "viewmodels/resources/createResourceBase", "viewmodels/shell"], function (require, exports, dialog, createResourceBase, shell) {
    var createCounterStorage = (function (_super) {
        __extends(createCounterStorage, _super);
        function createCounterStorage(parent) {
            _super.call(this, shell.counterStorages, parent);
            this.resourceNameCapitalString = "Counter storage";
            this.resourceNameString = "counter storage";
            this.storageEngine("voron");
        }
        createCounterStorage.prototype.nextOrCreate = function () {
            this.creationTaskStarted = true;
            dialog.close(this.parent);
            this.creationTask.resolve(this.resourceName(), this.getActiveBundles(), this.resourcePath(), this.resourceTempPath());
            this.clearResourceName();
        };
        createCounterStorage.prototype.getActiveBundles = function () {
            var activeBundles = [];
            return activeBundles;
        };
        return createCounterStorage;
    })(createResourceBase);
    return createCounterStorage;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createDatabase", ["require", "exports", "common/appUrl", "plugins/dialog", "viewmodels/resources/createResourceBase", "commands/database/debug/getPluginsInfoCommand", "commands/resources/getDatabaseStatsCommand", "commands/database/debug/getStatusDebugConfigCommand", "commands/database/cluster/getClusterTopologyCommand", "viewmodels/shell"], function (require, exports, appUrl, dialog, createResourceBase, getPluginsInfoCommand, getDatabaseStatsCommand, getStatusDebugConfigCommand, getClusterTopologyCommand, shell) {
    var createDatabase = (function (_super) {
        __extends(createDatabase, _super);
        function createDatabase(parent) {
            var _this = this;
            _super.call(this, shell.databases, parent);
            this.resourceNameCapitalString = "Database";
            this.resourceNameString = "database";
            this.databaseIndexesPath = ko.observable("");
            this.isCompressionBundleEnabled = ko.observable(false);
            this.isEncryptionBundleEnabled = ko.observable(false);
            this.isExpirationBundleEnabled = ko.observable(false);
            this.isQuotasBundleEnabled = ko.observable(false);
            this.isReplicationBundleEnabled = ko.observable(false);
            this.isSqlReplicationBundleEnabled = ko.observable(false);
            this.isVersioningBundleEnabled = ko.observable(false);
            this.isPeriodicExportBundleEnabled = ko.observable(false); // Old Raven Studio has this enabled by default
            this.isScriptedIndexBundleEnabled = ko.observable(false);
            this.isIncrementalBackupChecked = ko.observable(false);
            this.isClusterWideVisible = ko.observable(false);
            this.isClusterWideChecked = ko.observable(true);
            this.alertTimeout = ko.observable("");
            this.alertRecurringTimeout = ko.observable("");
            this.customBundles = ko.observableArray();
            this.selectedCustomBundles = ko.observableArray([]);
            this.replicationBundleChangeDisabled = ko.computed(function () {
                var clusterMode = shell.clusterMode();
                var clusterWide = _this.isClusterWideChecked();
                return clusterMode && clusterWide;
            });
            if (!!this.licenseStatus() && this.licenseStatus().IsCommercial && this.licenseStatus().Attributes.periodicBackup !== "true") {
                this.isPeriodicExportBundleEnabled(false);
            }
            if (shell.clusterMode()) {
                this.isReplicationBundleEnabled(true);
            }
            this.indexesCustomValidityError = ko.computed(function () {
                var newPath = _this.databaseIndexesPath();
                var errorMessage = _this.isPathLegal(newPath, "Indexes");
                return errorMessage;
            });
            this.fetchCustomBundles();
            this.fetchAllowVoron();
            this.fetchClusterWideConfig();
        }
        createDatabase.prototype.fetchClusterWideConfig = function () {
            var _this = this;
            new getClusterTopologyCommand(appUrl.getSystemDatabase())
                .execute()
                .done(function (topology) {
                _this.isClusterWideVisible(topology.allNodes().length > 0);
            });
        };
        createDatabase.prototype.fetchCustomBundles = function () {
            var _this = this;
            new getPluginsInfoCommand(appUrl.getSystemDatabase())
                .execute()
                .done(function (result) {
                _this.customBundles(result.CustomBundles);
            });
        };
        createDatabase.prototype.fetchAllowVoron = function () {
            var _this = this;
            $.when(new getDatabaseStatsCommand(appUrl.getSystemDatabase()).execute(), new getStatusDebugConfigCommand(appUrl.getSystemDatabase()).execute()).done(function (stats, config) {
                _this.allowVoron(stats[0].Is64Bit || config[0].Storage.Voron.AllowOn32Bits);
            });
        };
        createDatabase.prototype.nextOrCreate = function () {
            this.creationTaskStarted = true;
            dialog.close(this.parent);
            this.creationTask.resolve(this.resourceName(), this.getActiveBundles(), this.resourcePath(), this.logsPath(), this.databaseIndexesPath(), this.resourceTempPath(), this.storageEngine(), this.isIncrementalBackupChecked(), this.alertTimeout(), this.alertRecurringTimeout(), this.isClusterWideChecked());
            this.clearResourceName();
        };
        createDatabase.prototype.isDatabaseNameExists = function (databaseName, databases) {
            databaseName = databaseName.toLowerCase();
            for (var i = 0; i < databases.length; i++) {
                if (databaseName === databases[i].name.toLowerCase()) {
                    return true;
                }
            }
            return false;
        };
        createDatabase.prototype.toggleCompressionBundle = function () {
            this.isCompressionBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleEncryptionBundle = function () {
            this.isEncryptionBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleExpirationBundle = function () {
            this.isExpirationBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleQuotasBundle = function () {
            this.isQuotasBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleReplicationBundle = function () {
            this.isReplicationBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleSqlReplicationBundle = function () {
            this.isSqlReplicationBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleVersioningBundle = function () {
            this.isVersioningBundleEnabled.toggle();
        };
        createDatabase.prototype.togglePeriodicExportBundle = function () {
            this.isPeriodicExportBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleScriptedIndexBundle = function () {
            this.isScriptedIndexBundleEnabled.toggle();
        };
        createDatabase.prototype.toggleCustomBundle = function (name) {
            if (this.selectedCustomBundles.contains(name)) {
                this.selectedCustomBundles.remove(name);
            }
            else {
                this.selectedCustomBundles.push(name);
            }
        };
        createDatabase.prototype.isCustomBundleEnabled = function (name) {
            return this.selectedCustomBundles().contains(name);
        };
        createDatabase.prototype.getActiveBundles = function () {
            var activeBundles = [];
            if (this.isCompressionBundleEnabled()) {
                activeBundles.push("Compression");
            }
            if (this.isEncryptionBundleEnabled()) {
                activeBundles.push("Encryption");
            }
            if (this.isExpirationBundleEnabled()) {
                activeBundles.push("DocumentExpiration");
            }
            if (this.isQuotasBundleEnabled()) {
                activeBundles.push("Quotas");
            }
            if (this.isReplicationBundleEnabled()) {
                activeBundles.push("Replication"); // TODO: Replication also needs to store 2 documents containing information about replication. See http://ravendb.net/docs/2.5/server/scaling-out/replication?version=2.5
            }
            if (this.isSqlReplicationBundleEnabled()) {
                activeBundles.push("SqlReplication");
            }
            if (this.isVersioningBundleEnabled()) {
                activeBundles.push("Versioning");
            }
            if (this.isPeriodicExportBundleEnabled()) {
                activeBundles.push("PeriodicExport");
            }
            if (this.isScriptedIndexBundleEnabled()) {
                activeBundles.push("ScriptedIndexResults");
            }
            activeBundles.pushAll(this.selectedCustomBundles());
            return activeBundles;
        };
        return createDatabase;
    })(createResourceBase);
    return createDatabase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createEncryption", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "commands/database/studio/getNewEncryptionKey", "commands/database/studio/checkEncryptionKey"], function (require, exports, dialog, dialogViewModelBase, getNewEncryptionKey, checkEncryptionKey) {
    var createEncryption = (function (_super) {
        __extends(createEncryption, _super);
        function createEncryption() {
            var _this = this;
            _super.call(this);
            this.creationEncryption = $.Deferred();
            this.creationEncryptionStarted = false;
            this.key = ko.observable();
            this.encryptionAlgorithm = ko.observable('Rijndael');
            this.encryptionBits = ko.observable(256);
            this.isEncryptedIndexes = ko.observable(true);
            this.base64Matcher = new RegExp("^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{4})([=]{1,2})?$");
            this.newEncryptionKey = new getNewEncryptionKey();
            this.newEncryptionKey
                .execute()
                .done(function (result) {
                _this.key(result);
            });
        }
        createEncryption.prototype.attached = function () {
            var _this = this;
            this.dialogSelectorName = "#CreateEncriptionDialog";
            _super.prototype.attached.call(this);
            var inputElement = $("#key")[0];
            this.key.subscribe(function (newKey) {
                if (!_this.base64Matcher.test(newKey.toString())) {
                    inputElement.setCustomValidity("The key must be in Base64 encoding format!");
                }
                else {
                    inputElement.setCustomValidity('');
                }
            });
        };
        createEncryption.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.creationEncryptionStarted) {
                this.creationEncryption.reject();
            }
        };
        createEncryption.prototype.cancel = function () {
            dialog.close(this);
        };
        createEncryption.prototype.refresh_encryption = function () {
            var _this = this;
            this.newEncryptionKey
                .execute()
                .done(function (result) {
                _this.key(result);
            });
        };
        createEncryption.prototype.save = function () {
            var _this = this;
            var key = this.key();
            var checkEncryption = new checkEncryptionKey(key);
            checkEncryption
                .execute()
                .done(function () {
                _this.creationEncryption.resolve(key, _this.encryptionAlgorithm(), _this.encryptionBits(), _this.isEncryptedIndexes());
                _this.creationEncryptionStarted = true;
                dialog.close(_this);
            });
        };
        return createEncryption;
    })(dialogViewModelBase);
    return createEncryption;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createEncryptionConfirmation", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var createEncryptionConfirmation = (function (_super) {
        __extends(createEncryptionConfirmation, _super);
        function createEncryptionConfirmation(savedKey) {
            _super.call(this);
            this.dialogPromise = $.Deferred();
            this.key = ko.observable();
            this.key(savedKey);
        }
        createEncryptionConfirmation.prototype.cancel = function () {
            dialog.close(this);
        };
        createEncryptionConfirmation.prototype.ok = function () {
            dialog.close(this);
        };
        createEncryptionConfirmation.prototype.deactivate = function () {
            this.dialogPromise.resolve();
        };
        createEncryptionConfirmation.prototype.clickKey = function () {
            $('#key').select();
        };
        return createEncryptionConfirmation;
    })(dialogViewModelBase);
    return createEncryptionConfirmation;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createFilesystem", ["require", "exports", "plugins/dialog", "common/appUrl", "viewmodels/resources/createResourceBase", "commands/resources/getDatabaseStatsCommand", "commands/database/debug/getStatusDebugConfigCommand", "viewmodels/shell"], function (require, exports, dialog, appUrl, createResourceBase, getDatabaseStatsCommand, getStatusDebugConfigCommand, shell) {
    var createFilesystem = (function (_super) {
        __extends(createFilesystem, _super);
        function createFilesystem(parent) {
            _super.call(this, shell.fileSystems, parent);
            this.resourceNameCapitalString = "File System";
            this.resourceNameString = "file system";
            this.isEncryptionBundleEnabled = ko.observable(false);
            this.isVersioningBundleEnabled = ko.observable(false);
            this.fetchAllowVoron();
        }
        createFilesystem.prototype.fetchAllowVoron = function () {
            var _this = this;
            $.when(new getDatabaseStatsCommand(appUrl.getSystemDatabase()).execute(), new getStatusDebugConfigCommand(appUrl.getSystemDatabase()).execute()).done(function (stats, config) {
                _this.allowVoron(stats[0].Is64Bit || config[0].Storage.Voron.AllowOn32Bits);
            });
        };
        createFilesystem.prototype.nextOrCreate = function () {
            this.creationTaskStarted = true;
            dialog.close(this.parent);
            this.creationTask.resolve(this.resourceName(), this.getActiveBundles(), this.resourcePath(), this.logsPath(), this.resourceTempPath(), this.storageEngine());
            this.clearResourceName();
        };
        createFilesystem.prototype.doesFileSystemNameExist = function (fileSystemName, fileSystems) {
            fileSystemName = fileSystemName.toLowerCase();
            for (var i = 0; i < fileSystems.length; i++) {
                if (fileSystemName === fileSystems[i].name.toLowerCase()) {
                    return true;
                }
            }
            return false;
        };
        createFilesystem.prototype.toggleEncryptionBundle = function () {
            this.isEncryptionBundleEnabled.toggle();
        };
        createFilesystem.prototype.toggleVersioningBundle = function () {
            this.isVersioningBundleEnabled.toggle();
        };
        createFilesystem.prototype.getActiveBundles = function () {
            var activeBundles = [];
            if (this.isEncryptionBundleEnabled()) {
                activeBundles.push("Encryption");
            }
            if (this.isVersioningBundleEnabled()) {
                activeBundles.push("Versioning");
            }
            return activeBundles;
        };
        return createFilesystem;
    })(createResourceBase);
    return createFilesystem;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createResource", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "viewmodels/resources/createDatabase", "viewmodels/resources/createFilesystem", "viewmodels/resources/createCounterStorage", "viewmodels/resources/createTimeSeries", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "viewmodels/shell"], function (require, exports, dialog, dialogViewModelBase, createDatabase, createFileSystem, createCounterStorage, createTimeSeries, database, fileSystem, counterStorage, timeSeries, shell) {
    var createResource = (function (_super) {
        __extends(createResource, _super);
        function createResource() {
            var _this = this;
            _super.call(this);
            this.databaseType = database.type;
            this.fileSystemType = fileSystem.type;
            this.counterStorageType = counterStorage.type;
            this.timeSeriesType = timeSeries.type;
            this.resourceType = ko.observable("Database");
            this.resourceTypes = ko.observableArray([
                { resourceType: this.databaseType, title: "Database", iconName: "fa fa-database fa-2x" },
                { resourceType: this.fileSystemType, title: "File System", iconName: "fa fa-file-image-o fa-2x" }
            ]);
            this.checkedResource = ko.observable(this.databaseType);
            if (shell.has40Features()) {
                this.resourceTypes.push({ resourceType: this.counterStorageType, title: "Counter Storage", iconName: "fa fa-sort-numeric-desc fa-2x" });
                this.resourceTypes.push({ resourceType: this.timeSeriesType, title: "Time Series", iconName: "fa fa-clock-o fa-2x" });
            }
            this.createDatabasePart = new createDatabase(this);
            this.createFileSystemPart = new createFileSystem(this);
            this.createCounterStoragePart = new createCounterStorage(this);
            this.createTimeSeriesPart = new createTimeSeries(this);
            this.checkedResource.subscribe(function (resourceType) {
                var resourceTypeText = "";
                switch (resourceType) {
                    case _this.databaseType:
                        _this.enableDbTab();
                        resourceTypeText = "Database";
                        break;
                    case _this.fileSystemType:
                        _this.enableFsTab();
                        resourceTypeText = "File System";
                        break;
                    case _this.counterStorageType:
                        _this.enableCsTab();
                        resourceTypeText = "Counter Storage";
                        break;
                    case _this.timeSeriesType:
                        _this.enableTsTab();
                        resourceTypeText = "Time Series";
                        break;
                    default:
                        break;
                }
                _this.resourceType(resourceTypeText);
            });
        }
        createResource.prototype.compositionComplete = function () {
            this.enableDbTab();
        };
        createResource.prototype.enableDbTab = function () {
            this.alterFormControls("#dbContainer", false);
            this.alterFormControls("#fsContainer", true);
            this.alterFormControls("#csContainer", true);
            this.alterFormControls("#tsContainer", true);
        };
        createResource.prototype.enableFsTab = function () {
            this.alterFormControls("#dbContainer", true);
            this.alterFormControls("#fsContainer", false);
            this.alterFormControls("#csContainer", true);
            this.alterFormControls("#tsContainer", true);
        };
        createResource.prototype.enableCsTab = function () {
            this.alterFormControls("#dbContainer", true);
            this.alterFormControls("#fsContainer", true);
            this.alterFormControls("#csContainer", false);
            this.alterFormControls("#tsContainer", true);
        };
        createResource.prototype.enableTsTab = function () {
            this.alterFormControls("#dbContainer", true);
            this.alterFormControls("#fsContainer", true);
            this.alterFormControls("#csContainer", true);
            this.alterFormControls("#tsContainer", false);
        };
        createResource.prototype.alterFormControls = function (formSelector, disabled) {
            $(formSelector + " input").prop("disabled", disabled);
            $(formSelector + " select").prop("disabled", disabled);
        };
        createResource.prototype.cancel = function () {
            dialog.close(this);
        };
        createResource.prototype.nextOrCreate = function () {
            switch (this.checkedResource()) {
                case this.databaseType:
                    this.createDatabasePart.nextOrCreate();
                    break;
                case this.fileSystemType:
                    this.createFileSystemPart.nextOrCreate();
                    break;
                case this.counterStorageType:
                    this.createCounterStoragePart.nextOrCreate();
                    break;
                case this.timeSeriesType:
                    this.createTimeSeriesPart.nextOrCreate();
                    break;
                default:
                    throw "Can't figure what to do!";
            }
        };
        return createResource;
    })(dialogViewModelBase);
    return createResource;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createResourceBase", ["require", "exports", "viewmodels/viewModelBase", "models/auth/license"], function (require, exports, viewModelBase, license) {
    var createResourceBase = (function (_super) {
        __extends(createResourceBase, _super);
        function createResourceBase(resources, parent) {
            var _this = this;
            _super.call(this);
            this.resources = resources;
            this.parent = parent;
            this.creationTask = $.Deferred();
            this.creationTaskStarted = false;
            this.resourceName = ko.observable("");
            this.resourcePath = ko.observable("");
            this.logsPath = ko.observable("");
            this.resourceTempPath = ko.observable("");
            this.resourceNameCapitalString = "";
            this.resourceNameString = "";
            this.storageEngine = ko.observable("");
            this.allowVoron = ko.observable(true);
            this.voronWarningVisible = ko.computed(function () { return !_this.allowVoron() && _this.storageEngine() === "voron"; });
            this.tempPathVisible = ko.computed(function () { return _this.storageEngine() === "voron"; });
            this.licenseStatus = license.licenseStatus;
            this.resourceName = createResourceBase.resourceName;
            this.nameCustomValidityError = ko.computed(function () {
                var errorMessage = "";
                var newResourceName = _this.resourceName();
                if (_this.doesCounterStorageNameExist(newResourceName, _this.resources())) {
                    errorMessage = _this.resourceNameCapitalString + " name already exists!";
                }
                else if ((errorMessage = _this.checkName(newResourceName)) !== "") { }
                return errorMessage;
            });
            this.pathCustomValidityError = ko.computed(function () {
                var newPath = _this.resourcePath();
                var errorMessage = _this.isPathLegal(newPath, "Path");
                return errorMessage;
            });
            this.logsCustomValidityError = ko.computed(function () {
                var newPath = _this.logsPath();
                var errorMessage = _this.isPathLegal(newPath, "Logs");
                return errorMessage;
            });
            this.tempCustomValidityError = ko.computed(function () {
                var newPath = _this.resourceTempPath();
                var errorMessage = _this.isPathLegal(newPath, "Temp");
                return errorMessage;
            });
        }
        createResourceBase.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.creationTaskStarted) {
                this.creationTask.reject();
            }
        };
        createResourceBase.prototype.clearResourceName = function () {
            this.resourceName("");
        };
        createResourceBase.prototype.isBundleActive = function (name) {
            var licenseStatus = this.licenseStatus();
            if (licenseStatus == null || licenseStatus.IsCommercial === false) {
                return true;
            }
            else {
                var value = licenseStatus.Attributes[name];
                return value === "true";
            }
        };
        createResourceBase.prototype.doesCounterStorageNameExist = function (resourceName, resources) {
            resourceName = resourceName.toLowerCase();
            for (var i = 0; i < resources.length; i++) {
                if (resourceName === resources[i].name.toLowerCase()) {
                    return true;
                }
            }
            return false;
        };
        createResourceBase.prototype.checkName = function (name) {
            var rg1 = /^[^\\/:\*\?"<>\|]+$/; // forbidden characters \ / : * ? " < > |
            var rg2 = /^\./; // cannot start with dot (.)
            var rg3 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i; // forbidden file names
            var maxLength = 260 - 30;
            var message = "";
            if (!$.trim(name)) {
                message = "Please fill out the " + this.resourceNameString + " name field!";
            }
            else if (name.length > maxLength) {
                message = "The " + this.resourceNameString + " length can't exceed " + maxLength + " characters!";
            }
            else if (!rg1.test(name)) {
                message = "The " + this.resourceNameString + " name can't contain any of the following characters: \ / : * ?" + ' " ' + "< > |";
            }
            else if (rg2.test(name)) {
                message = "The " + this.resourceNameString + " name can't start with a dot!";
            }
            else if (name[name.length - 1] === ".") {
                message = "The " + this.resourceNameString + " name can't end with a dot!";
            }
            else if (rg3.test(name)) {
                message = "The name '" + name + "' is forbidden for use!";
            }
            return message;
        };
        createResourceBase.prototype.isPathLegal = function (name, pathName) {
            var rg1 = /^[^*\?"<>\|]+$/; // forbidden characters \ * : ? " < > |
            var rg2 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i; // forbidden file names
            var errorMessage = "";
            if (!$.trim(name) == false) {
                if (name.length > 248) {
                    errorMessage = "The path name for the '" + pathName + "' can't exceed " + 248 + " characters!";
                }
                else if (!rg1.test(name)) {
                    errorMessage = "The " + pathName + " can't contain any of the following characters: * : ?" + ' " ' + "< > |";
                }
                else if (rg2.test(name)) {
                    errorMessage = "The name '" + name + "' is forbidden for use!";
                }
            }
            return errorMessage;
        };
        createResourceBase.resourceName = ko.observable("");
        return createResourceBase;
    })(viewModelBase);
    return createResourceBase;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/createTimeSeries", ["require", "exports", "plugins/dialog", "viewmodels/resources/createResourceBase", "viewmodels/shell"], function (require, exports, dialog, createResourceBase, shell) {
    var createTimeSeries = (function (_super) {
        __extends(createTimeSeries, _super);
        function createTimeSeries(parent) {
            _super.call(this, shell.timeSeries, parent);
            this.resourceNameCapitalString = "Time Series";
            this.resourceNameString = "time series";
            this.storageEngine("voron");
        }
        createTimeSeries.prototype.nextOrCreate = function () {
            this.creationTaskStarted = true;
            dialog.close(this.parent);
            this.creationTask.resolve(this.resourceName(), this.getActiveBundles(), this.resourcePath(), this.resourceTempPath());
            this.clearResourceName();
        };
        createTimeSeries.prototype.getActiveBundles = function () {
            var activeBundles = [];
            return activeBundles;
        };
        return createTimeSeries;
    })(createResourceBase);
    return createTimeSeries;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/databaseSettingsDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "durandal/activator"], function (require, exports, dialog, dialogViewModelBase, activator) {
    var databaseSettingsDialog = (function (_super) {
        __extends(databaseSettingsDialog, _super);
        function databaseSettingsDialog(bundles) {
            var _this = this;
            _super.call(this);
            this.dialogTask = $.Deferred();
            this.activeScreen = ko.observable("");
            this.content = activator.create();
            var quotasRoute = { moduleId: 'viewmodels/database/settings/quotas', title: 'Quotas', activate: true };
            var versioningRoute = { moduleId: 'viewmodels/database/settings/versioning', title: 'Versioning', activate: true };
            var sqlReplicationConnectionRoute = { moduleId: 'viewmodels/database/settings/sqlReplicationConnectionStringsManagement', title: 'SQL Replication Connection Strings', activate: true };
            // when the activeScreen name changes - load the viewmodel
            this.activeScreen.subscribe(function (newValue) {
                return require([newValue], function (model) {
                    _this.currentModel = new model();
                    _this.content.activateItem(_this.currentModel);
                });
            });
            this.routes = [];
            if (bundles.contains("Quotas")) {
                this.routes.push(quotasRoute);
            }
            if (bundles.contains("Versioning")) {
                this.routes.push(versioningRoute);
            }
            if (bundles.contains("SqlReplication")) {
                this.routes.push(sqlReplicationConnectionRoute);
            }
        }
        databaseSettingsDialog.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.activeScreen(this.routes[0].moduleId);
        };
        databaseSettingsDialog.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.dialogTask.resolve();
        };
        databaseSettingsDialog.prototype.canDeactivate = function () {
            return this.currentModel.canDeactivate(false);
        };
        databaseSettingsDialog.prototype.showView = function (moduleId) {
            var _this = this;
            var canDeactivate = this.canDeactivate();
            if (canDeactivate.done) {
                canDeactivate.done(function (answer) {
                    if (answer.can) {
                        _this.onSuccessfulDeactivation(moduleId);
                    }
                });
            }
            else if (canDeactivate === true) {
                this.onSuccessfulDeactivation(moduleId);
            }
        };
        databaseSettingsDialog.prototype.onSuccessfulDeactivation = function (moduleId) {
            this.currentModel.dirtyFlag().reset();
            this.activeScreen(moduleId);
        };
        databaseSettingsDialog.prototype.isActive = function (moduleId) {
            return moduleId === this.activeScreen();
        };
        databaseSettingsDialog.prototype.close = function () {
            dialog.close(this);
        };
        return databaseSettingsDialog;
    })(dialogViewModelBase);
    return databaseSettingsDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/deleteResourceConfirm", ["require", "exports", "commands/resources/deleteResourceCommand", "models/resources/database", "plugins/dialog", "common/appUrl", "viewmodels/dialogViewModelBase", "plugins/router"], function (require, exports, deleteResourceCommand, database, dialog, appUrl, dialogViewModelBase, router) {
    var deleteResourceConfirm = (function (_super) {
        __extends(deleteResourceConfirm, _super);
        function deleteResourceConfirm(resources) {
            _super.call(this);
            this.isKeepingFiles = ko.observable(true);
            this.resourcesToDelete = ko.observableArray();
            this.deleteTask = $.Deferred();
            if (resources.length === 0) {
                throw new Error("Must have at least one resource to delete.");
            }
            this.resourcesToDelete(resources);
            this.isDeletingDatabase = resources[0] instanceof database;
            if (this.isDeletingDatabase) {
                this.exportDatabaseUrl = appUrl.forExportDatabase(resources[0]);
            }
        }
        deleteResourceConfirm.prototype.keepFiles = function () {
            this.isKeepingFiles(true);
        };
        deleteResourceConfirm.prototype.deleteEverything = function () {
            this.isKeepingFiles(false);
        };
        deleteResourceConfirm.prototype.deleteDatabase = function () {
            var _this = this;
            new deleteResourceCommand(this.resourcesToDelete(), this.isKeepingFiles() === false)
                .execute()
                .done(function (results) { return _this.deleteTask.resolve(results); })
                .fail(function (details) { return _this.deleteTask.reject(details); });
            dialog.close(this);
        };
        deleteResourceConfirm.prototype.cancel = function () {
            this.deleteTask.reject();
            dialog.close(this);
        };
        deleteResourceConfirm.prototype.exportDatabase = function () {
            this.resourcesToDelete.first().activate();
            router.navigate(this.exportDatabaseUrl);
            dialog.close(this);
        };
        return deleteResourceConfirm;
    })(dialogViewModelBase);
    return deleteResourceConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/disableResourceToggleConfirm", ["require", "exports", "plugins/dialog", "commands/resources/disableResourceToggleCommand", "viewmodels/dialogViewModelBase", "viewmodels/shell"], function (require, exports, dialog, disableResourceToggleCommand, dialogViewModelBase, shell) {
    var disableResourceToggleConfirm = (function (_super) {
        __extends(disableResourceToggleConfirm, _super);
        function disableResourceToggleConfirm(resources, elementToFocusOnDismissal) {
            var _this = this;
            _super.call(this, elementToFocusOnDismissal);
            this.resources = resources;
            this.resourcesToDisable = ko.observableArray();
            this.disableToggleStarted = false;
            this.disableToggleTask = $.Deferred(); // Gives consumers a way to know when the async operation completes.
            this.desiredAction = ko.observable();
            if (resources.length === 0) {
                throw new Error("Must have at least one resource to disable.");
            }
            this.resourcesToDisable(resources);
            this.isSettingDisabled = !resources[0].disabled();
            this.deletionText = ko.computed(function () { return _this.isSettingDisabled ? "You're disabling" : "You're enabling"; });
            this.confirmDeletionText = ko.computed(function () { return _this.isSettingDisabled ? "Yep, disable" : "Yep, enable"; });
        }
        disableResourceToggleConfirm.prototype.toggleDisableReources = function () {
            var _this = this;
            var selected = this.resources.filter(function (rs) { return rs.isSelected(); })[0];
            if (!!selected)
                shell.disconnectFromResourceChangesApi();
            var disableToggleCommand = new disableResourceToggleCommand(this.resourcesToDisable(), this.isSettingDisabled);
            var disableToggleCommandTask = disableToggleCommand.execute();
            disableToggleCommandTask.done(function (results) {
                if (_this.resourcesToDisable().length === 1) {
                    results = [_this.resourcesToDisable()[0].name];
                }
                _this.disableToggleTask.resolve(results);
            });
            disableToggleCommandTask.fail(function (response) {
                if (!!selected)
                    selected.activate();
                _this.disableToggleTask.reject(response);
            });
            this.disableToggleStarted = true;
            dialog.close(this);
        };
        disableResourceToggleConfirm.prototype.cancel = function () {
            dialog.close(this);
        };
        disableResourceToggleConfirm.prototype.deactivate = function (args) {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never carried it out.
            if (!this.disableToggleStarted) {
                this.disableToggleTask.reject();
            }
        };
        return disableResourceToggleConfirm;
    })(dialogViewModelBase);
    return disableResourceToggleConfirm;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/filesystemSettingsDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase", "durandal/activator"], function (require, exports, dialog, dialogViewModelBase, activator) {
    var filesystemSettingsDialog = (function (_super) {
        __extends(filesystemSettingsDialog, _super);
        function filesystemSettingsDialog(bundles) {
            var _this = this;
            _super.call(this);
            this.dialogTask = $.Deferred();
            this.activeScreen = ko.observable("");
            this.content = activator.create();
            var versioningRoute = { moduleId: 'viewmodels/filesystem/settings/versioning', title: 'Versioning', activate: true };
            // when the activeScreen name changes - load the viewmodel
            this.activeScreen.subscribe(function (newValue) {
                return require([newValue], function (model) {
                    _this.currentModel = new model();
                    _this.content.activateItem(_this.currentModel);
                });
            });
            this.routes = [];
            if (bundles.contains("Versioning")) {
                this.routes.push(versioningRoute);
            }
        }
        filesystemSettingsDialog.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.activeScreen(this.routes[0].moduleId);
        };
        filesystemSettingsDialog.prototype.detached = function () {
            _super.prototype.detached.call(this);
            this.dialogTask.resolve();
        };
        filesystemSettingsDialog.prototype.canDeactivate = function () {
            return this.currentModel.canDeactivate(false);
        };
        filesystemSettingsDialog.prototype.showView = function (moduleId) {
            var _this = this;
            var canDeactivate = this.canDeactivate();
            if (canDeactivate.done) {
                canDeactivate.done(function (answer) {
                    if (answer.can) {
                        _this.onSuccessfulDeactivation(moduleId);
                    }
                });
            }
            else if (canDeactivate === true) {
                this.onSuccessfulDeactivation(moduleId);
            }
        };
        filesystemSettingsDialog.prototype.onSuccessfulDeactivation = function (moduleId) {
            this.currentModel.dirtyFlag().reset();
            this.activeScreen(moduleId);
        };
        filesystemSettingsDialog.prototype.isActive = function (moduleId) {
            return moduleId === this.activeScreen();
        };
        filesystemSettingsDialog.prototype.close = function () {
            dialog.close(this);
        };
        return filesystemSettingsDialog;
    })(dialogViewModelBase);
    return filesystemSettingsDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/resources/resources", ["require", "exports", "durandal/app", "common/appUrl", "viewmodels/viewModelBase", "viewmodels/shell", "models/resources/database", "commands/operations/getOperationAlertsCommand", "commands/operations/dismissAlertCommand", "viewmodels/resources/deleteResourceConfirm", "viewmodels/resources/disableResourceToggleConfirm", "commands/database/index/disableIndexingCommand", "commands/maintenance/toggleRejectDatabaseClients", "viewmodels/resources/createResource", "viewmodels/resources/createEncryption", "commands/resources/createDatabaseCommand", "viewmodels/resources/createEncryptionConfirmation", "viewmodels/resources/databaseSettingsDialog", "commands/resources/createDefaultSettingsCommand", "viewmodels/resources/filesystemSettingsDialog", "models/filesystem/filesystem", "commands/filesystem/createDefaultSettingsCommand", "commands/filesystem/createFilesystemCommand", "models/counter/counterStorage", "commands/resources/createCounterStorageCommand", "models/timeSeries/timeSeries", "commands/resources/createTimeSeriesCommand", "models/auth/license"], function (require, exports, app, appUrl, viewModelBase, shell, database, getOperationAlertsCommand, dismissAlertCommand, deleteResourceConfirm, disableResourceToggleConfirm, disableIndexingCommand, toggleRejectDatabaseClients, createResource, createEncryption, createDatabaseCommand, createEncryptionConfirmation, databaseSettingsDialog, createDefaultDbSettingsCommand, filesystemSettingsDialog, fileSystem, createDefaultFsSettingsCommand, createFilesystemCommand, counterStorage, createCounterStorageCommand, timeSeries, createTimeSeriesCommand, license) {
    var resources = (function (_super) {
        __extends(resources, _super);
        function resources() {
            var _this = this;
            _super.call(this);
            this.databases = ko.observableArray();
            this.fileSystems = ko.observableArray();
            this.counterStorages = ko.observableArray();
            this.timeSeries = ko.observableArray();
            this.searchText = ko.observable("");
            this.selectedResource = ko.observable();
            this.fileSystemsStatus = ko.observable("loading");
            this.optionsClicked = ko.observable(false);
            this.alerts = ko.observable([]);
            this.isGlobalAdmin = shell.isGlobalAdmin;
            this.clusterMode = ko.computed(function () { return shell.clusterMode(); });
            this.developerLicense = ko.computed(function () { return !license.licenseStatus() || !license.licenseStatus().IsCommercial; });
            this.showCreateCluster = ko.computed(function () { return !shell.clusterMode(); });
            this.canCreateCluster = ko.computed(function () { return license.licenseStatus() && (!license.licenseStatus().IsCommercial || license.licenseStatus().Attributes.clustering === "true"); });
            this.databaseType = database.type;
            this.fileSystemType = fileSystem.type;
            this.counterStorageType = counterStorage.type;
            this.timeSeriesType = timeSeries.type;
            this.visibleResource = ko.observable("");
            this.has40Features = ko.computed(function () { return shell.has40Features(); });
            this.visibleOptions = [
                { value: "", name: "Show all" },
                { value: database.type, name: "Show databases" },
                { value: fileSystem.type, name: "Show file systems" }
            ];
            if (this.has40Features()) {
                this.visibleOptions.pushAll([
                    { value: counterStorage.type, name: "Show counter storages" },
                    { value: timeSeries.type, name: "Show time series" }
                ]);
            }
            this.canNavigateToAdminSettings = ko.computed(function () { return shell.isGlobalAdmin() || shell.canReadWriteSettings() || shell.canReadSettings(); });
            this.databases = shell.databases;
            this.fileSystems = shell.fileSystems;
            this.counterStorages = shell.counterStorages;
            this.timeSeries = shell.timeSeries;
            this.resources = shell.resources;
            this.systemDb = appUrl.getSystemDatabase();
            this.appUrls = appUrl.forCurrentDatabase();
            this.searchText.extend({ throttle: 200 }).subscribe(function () { return _this.filterResources(); });
            var currentDatabase = this.activeDatabase();
            if (!!currentDatabase) {
                this.selectResource(currentDatabase, false);
            }
            var currentFileSystem = this.activeFilesystem();
            if (!!currentFileSystem) {
                this.selectResource(currentFileSystem, false);
            }
            var currentCounterStorage = this.activeCounterStorage();
            if (!!currentCounterStorage) {
                this.selectResource(currentCounterStorage, false);
            }
            var currentTimeSeries = this.activeTimeSeries();
            if (!!currentTimeSeries) {
                this.selectResource(currentTimeSeries, false);
            }
            var updatedUrl = appUrl.forResources();
            this.updateUrl(updatedUrl);
            this.resourcesSelection = ko.computed(function () {
                var resources = _this.resources();
                if (resources.length === 0) {
                    return checkbox.UnChecked;
                }
                var allSelected = true;
                var anySelected = false;
                for (var i = 0; i < resources.length; i++) {
                    var rs = resources[i];
                    if (rs.isDatabase() && rs.isSystem) {
                        continue;
                    }
                    if (rs.isChecked()) {
                        anySelected = true;
                    }
                    else {
                        allSelected = false;
                    }
                }
                if (allSelected) {
                    return checkbox.Checked;
                }
                else if (anySelected) {
                    return checkbox.SomeChecked;
                }
                return checkbox.UnChecked;
            });
            this.allCheckedResourcesDisabled = ko.computed(function () {
                var disabledStatus = null;
                for (var i = 0; i < _this.resources().length; i++) {
                    var rs = _this.resources()[i];
                    if (rs.isChecked()) {
                        if (disabledStatus == null) {
                            disabledStatus = rs.disabled();
                        }
                        else if (disabledStatus !== rs.disabled()) {
                            return null;
                        }
                    }
                }
                return disabledStatus;
            });
            this.isCheckboxVisible = ko.computed(function () {
                if (!_this.isGlobalAdmin())
                    return false;
                var resources = _this.resources();
                for (var i = 0; i < resources.length; i++) {
                    var rs = resources[i];
                    if (rs.isVisible())
                        return true;
                }
                return false;
            });
            this.fetchAlerts();
            this.visibleResource.subscribe(function () { return _this.filterResources(); });
            this.filterResources();
        }
        resources.prototype.fetchAlerts = function () {
            var _this = this;
            new getOperationAlertsCommand(appUrl.getSystemDatabase())
                .execute()
                .then(function (result) { return _this.alerts(result); });
        };
        // Override canActivate: we can always load this page, regardless of any system db prompt.
        resources.prototype.canActivate = function (args) {
            return true;
        };
        resources.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.updateHelpLink("Z8DC3Q");
            ko.postbox.publish("SetRawJSONUrl", appUrl.forDatabasesRawData());
            this.resourcesLoaded();
        };
        resources.prototype.resourcesLoaded = function () {
            /*
    
            Show it only when cluster bundle is not present
            // If we have no databases (except system db), show the "create a new database" screen.
            if (this.resources().length === 1 && this.isGlobalAdmin()) {
                this.newResource();
            } */
        };
        resources.prototype.filterResources = function () {
            var _this = this;
            var filter = this.searchText();
            var filterLower = filter.toLowerCase();
            this.databases().forEach(function (db) {
                var typeMatch = !_this.visibleResource() || _this.visibleResource() === database.type;
                var isMatch = (!filter || (db.name.toLowerCase().indexOf(filterLower) >= 0)) && db.name !== "<system>" && typeMatch;
                db.isVisible(isMatch);
            });
            this.databases().map(function (db) { return db.isChecked(!db.isVisible() ? false : db.isChecked()); });
            this.fileSystems().forEach(function (fs) {
                var typeMatch = !_this.visibleResource() || _this.visibleResource() === fileSystem.type;
                var isMatch = (!filter || (fs.name.toLowerCase().indexOf(filterLower) >= 0)) && typeMatch;
                fs.isVisible(isMatch);
            });
            this.fileSystems().map(function (fs) { return fs.isChecked(!fs.isVisible() ? false : fs.isChecked()); });
            this.counterStorages().forEach(function (cs) {
                var typeMatch = !_this.visibleResource() || _this.visibleResource() === counterStorage.type;
                var isMatch = (!filter || (cs.name.toLowerCase().indexOf(filterLower) >= 0)) && typeMatch;
                cs.isVisible(isMatch);
            });
            this.counterStorages().map(function (cs) { return cs.isChecked(!cs.isVisible() ? false : cs.isChecked()); });
            this.timeSeries().forEach(function (ts) {
                var typeMatch = !_this.visibleResource() || _this.visibleResource() === timeSeries.type;
                var isMatch = (!filter || (ts.name.toLowerCase().indexOf(filterLower) >= 0)) && typeMatch;
                ts.isVisible(isMatch);
            });
            this.timeSeries().map(function (ts) { return ts.isChecked(!ts.isVisible() ? false : ts.isChecked()); });
        };
        resources.prototype.getDocumentsUrl = function (db) {
            return appUrl.forDocuments(null, db);
        };
        resources.prototype.getFileSystemFilesUrl = function (fs) {
            return appUrl.forFilesystemFiles(fs);
        };
        resources.prototype.getCounterStorageCountersUrl = function (cs) {
            return appUrl.forCounterStorageCounters(null, cs);
        };
        resources.prototype.getTimeSeriesUrl = function (ts) {
            return appUrl.forTimeSeriesType(null, ts);
        };
        resources.prototype.selectResource = function (rs, activateResource) {
            if (activateResource === void 0) { activateResource = true; }
            if (this.optionsClicked() === false) {
                if (activateResource) {
                    rs.activate();
                }
                this.selectedResource(rs);
            }
            this.optionsClicked(false);
        };
        resources.prototype.deleteSelectedResources = function (resources) {
            var _this = this;
            if (resources.length > 0) {
                var confirmDeleteViewModel = new deleteResourceConfirm(resources);
                confirmDeleteViewModel.deleteTask.done(function (deletedResources) {
                    if (resources.length === 1) {
                        _this.onResourceDeleted(resources[0]);
                    }
                    else {
                        deletedResources.forEach(function (rs) { return _this.onResourceDeleted(rs); });
                    }
                });
                app.showDialog(confirmDeleteViewModel);
            }
        };
        resources.prototype.onResourceDeleted = function (resourceToDelete) {
            var resourcesArray = this.getResources(resourceToDelete.type);
            var resourceInArray = resourcesArray.first(function (db) { return db.name === resourceToDelete.name; });
            if (!!resourceInArray) {
                resourcesArray.remove(resourceInArray);
            }
            if ((this.resources().length > 0) && (this.resources().contains(this.selectedResource()) === false)) {
                ko.postbox.publish("SelectNone");
            }
        };
        resources.prototype.getResources = function (resourceType) {
            switch (resourceType) {
                case TenantType.Database:
                    return this.databases;
                case TenantType.FileSystem:
                    return this.fileSystems;
                case TenantType.CounterStorage:
                    return this.counterStorages;
                case TenantType.TimeSeries:
                    return this.timeSeries;
                default:
                    throw "Unknown type";
            }
        };
        resources.prototype.deleteCheckedResources = function () {
            var checkedResources = this.resources().filter(function (rs) { return rs.isChecked(); });
            this.deleteSelectedResources(checkedResources);
        };
        resources.prototype.toggleSelectAll = function () {
            var check = true;
            if (this.resourcesSelection() !== checkbox.UnChecked) {
                check = false;
            }
            for (var i = 0; i < this.resources().length; i++) {
                var rs = this.resources()[i];
                if (rs.isDatabase() && rs.isSystem) {
                    rs.isChecked(false);
                    continue;
                }
                if (rs.isVisible()) {
                    rs.isChecked(check);
                }
            }
        };
        resources.prototype.toggleCheckedResources = function () {
            var checkedResources = this.resources().filter(function (rs) { return rs.isChecked(); });
            this.toggleSelectedResources(checkedResources);
        };
        resources.prototype.toggleSelectedResources = function (resources) {
            var _this = this;
            if (resources.length > 0) {
                var action = !resources[0].disabled();
                var disableDatabaseToggleViewModel = new disableResourceToggleConfirm(resources);
                disableDatabaseToggleViewModel.disableToggleTask
                    .done(function (toggledResources) {
                    if (resources.length === 1) {
                        _this.onResourceDisabledToggle(resources[0], action);
                    }
                    else {
                        toggledResources.forEach(function (rs) {
                            _this.onResourceDisabledToggle(rs, action);
                        });
                    }
                });
                app.showDialog(disableDatabaseToggleViewModel);
            }
        };
        resources.prototype.onResourceDisabledToggle = function (rs, action) {
            if (!!rs) {
                rs.disabled(action);
                rs.isChecked(false);
                if (rs.isSelected() && rs.disabled() === false) {
                    rs.activate();
                }
            }
        };
        resources.prototype.disableDatabaseIndexing = function (db) {
            var action = !db.indexingDisabled();
            var actionText = db.indexingDisabled() ? "Enable" : "Disable";
            var message = this.confirmationMessage(actionText + " indexing?", "Are you sure?");
            message.done(function () {
                var task = new disableIndexingCommand(db.name, action).execute();
                task.done(function () { return db.indexingDisabled(action); });
            });
        };
        resources.prototype.toggleRejectDatabaseClients = function (db) {
            var action = !db.rejectClientsMode();
            var actionText = action ? "reject clients mode" : "accept clients mode";
            var message = this.confirmationMessage("Switch to " + actionText, "Are you sure?");
            message.done(function () {
                var task = new toggleRejectDatabaseClients(db.name, action).execute();
                task.done(function () { return db.rejectClientsMode(action); });
            });
        };
        resources.prototype.navigateToAdminSettings = function () {
            this.navigate(this.appUrls.adminSettings());
            shell.disconnectFromResourceChangesApi();
            shell.selectedEnvironmentColorStatic(shell.originalEnvironmentColor());
        };
        resources.prototype.navigateToCreateCluster = function () {
            this.navigate(this.appUrls.adminSettingsCluster());
            shell.disconnectFromResourceChangesApi();
        };
        resources.prototype.dismissAlert = function (uniqueKey) {
            new dismissAlertCommand(appUrl.getSystemDatabase(), uniqueKey).execute();
        };
        resources.prototype.urlForAlert = function (alert) {
            var index = this.alerts().indexOf(alert);
            return appUrl.forAlerts(appUrl.getSystemDatabase()) + "&item=" + index;
        };
        resources.prototype.newResource = function () {
            var _this = this;
            var createResourceViewModel = new createResource();
            createResourceViewModel.createDatabasePart
                .creationTask
                .done(function (databaseName, bundles, databasePath, databaseLogs, databaseIndexes, tempPath, storageEngine, incrementalBackup, alertTimeout, alertRecurringTimeout, clusterWide) {
                var settings = {
                    "Raven/ActiveBundles": bundles.join(";")
                };
                if (storageEngine) {
                    settings["Raven/StorageTypeName"] = storageEngine;
                }
                if (!clusterWide) {
                    settings["Raven-Non-Cluster-Database"] = "true";
                }
                if (incrementalBackup) {
                    if (storageEngine === "esent") {
                        settings["Raven/Esent/CircularLog"] = "false";
                    }
                    else {
                        settings["Raven/Voron/AllowIncrementalBackups"] = "true";
                    }
                }
                if (!_this.isEmptyStringOrWhitespace(tempPath)) {
                    settings["Raven/Voron/TempPath"] = tempPath;
                }
                if (alertTimeout !== "") {
                    settings["Raven/IncrementalBackup/AlertTimeoutHours"] = alertTimeout;
                }
                if (alertRecurringTimeout !== "") {
                    settings["Raven/IncrementalBackup/RecurringAlertTimeoutDays"] = alertRecurringTimeout;
                }
                settings["Raven/DataDir"] = (!_this.isEmptyStringOrWhitespace(databasePath)) ? databasePath : "~/" + databaseName;
                if (!_this.isEmptyStringOrWhitespace(databaseLogs)) {
                    settings["Raven/Esent/LogsPath"] = databaseLogs;
                }
                if (!_this.isEmptyStringOrWhitespace(databaseIndexes)) {
                    settings["Raven/IndexStoragePath"] = databaseIndexes;
                }
                _this.showDbCreationAdvancedStepsIfNecessary(databaseName, bundles, settings, clusterWide);
            });
            createResourceViewModel.createFileSystemPart
                .creationTask
                .done(function (fileSystemName, bundles, fileSystemPath, filesystemLogs, tempPath, storageEngine) {
                var settings = {
                    "Raven/ActiveBundles": bundles.join(";")
                };
                settings["Raven/FileSystem/DataDir"] = (!_this.isEmptyStringOrWhitespace(fileSystemPath)) ? fileSystemPath : "~\\FileSystems\\" + fileSystemName;
                if (storageEngine) {
                    settings["Raven/FileSystem/Storage"] = storageEngine;
                }
                if (!_this.isEmptyStringOrWhitespace(filesystemLogs)) {
                    settings["Raven/TransactionJournalsPath"] = filesystemLogs;
                }
                if (!_this.isEmptyStringOrWhitespace(tempPath)) {
                    settings["Raven/Voron/TempPath"] = tempPath;
                }
                _this.showFsCreationAdvancedStepsIfNecessary(fileSystemName, bundles, settings);
            });
            createResourceViewModel.createCounterStoragePart
                .creationTask
                .done(function (counterStorageName, bundles, counterStoragePath, tempPath) {
                var settings = {
                    "Raven/ActiveBundles": bundles.join(";")
                };
                settings["Raven/Counters/DataDir"] = (!_this.isEmptyStringOrWhitespace(counterStoragePath)) ? counterStoragePath : "~\\Counters\\" + counterStorageName;
                if (!_this.isEmptyStringOrWhitespace(tempPath)) {
                    settings["Raven/Voron/TempPath"] = tempPath;
                }
                _this.showCsCreationAdvancedStepsIfNecessary(counterStorageName, bundles, settings);
            });
            createResourceViewModel.createTimeSeriesPart
                .creationTask
                .done(function (timeSeriesName, bundles, timeSeriesPath, tempPath) {
                var settings = {
                    "Raven/ActiveBundles": bundles.join(";")
                };
                settings["Raven/TimeSeries/DataDir"] = (!_this.isEmptyStringOrWhitespace(timeSeriesPath)) ? timeSeriesPath : "~\\TimeSeries\\" + timeSeriesName;
                if (!_this.isEmptyStringOrWhitespace(tempPath)) {
                    settings["Raven/Voron/TempPath"] = tempPath;
                }
                _this.showTsCreationAdvancedStepsIfNecessary(timeSeriesName, bundles, settings);
            });
            app.showDialog(createResourceViewModel);
        };
        resources.prototype.showDbCreationAdvancedStepsIfNecessary = function (databaseName, bundles, settings, clusterWide) {
            var _this = this;
            var securedSettings = {};
            var savedKey;
            var encryptionDeferred = $.Deferred();
            if (bundles.contains("Encryption")) {
                var createEncryptionViewModel = new createEncryption();
                createEncryptionViewModel
                    .creationEncryption
                    .done(function (key, encryptionAlgorithm, encryptionBits, isEncryptedIndexes) {
                    savedKey = key;
                    securedSettings = {
                        'Raven/Encryption/Key': key,
                        'Raven/Encryption/Algorithm': _this.getEncryptionAlgorithmFullName(encryptionAlgorithm),
                        'Raven/Encryption/KeyBitsPreference': encryptionBits,
                        'Raven/Encryption/EncryptIndexes': isEncryptedIndexes
                    };
                    encryptionDeferred.resolve(securedSettings);
                });
                app.showDialog(createEncryptionViewModel);
            }
            else {
                encryptionDeferred.resolve();
            }
            encryptionDeferred.done(function () {
                new createDatabaseCommand(databaseName, settings, securedSettings)
                    .execute()
                    .done(function () {
                    var newDatabase = _this.addNewDatabase(databaseName, bundles, clusterWide);
                    _this.selectResource(newDatabase);
                    var encryptionConfirmationDialogPromise = $.Deferred();
                    if (!jQuery.isEmptyObject(securedSettings)) {
                        var createEncryptionConfirmationViewModel = new createEncryptionConfirmation(savedKey);
                        createEncryptionConfirmationViewModel.dialogPromise.done(function () { return encryptionConfirmationDialogPromise.resolve(); });
                        createEncryptionConfirmationViewModel.dialogPromise.fail(function () { return encryptionConfirmationDialogPromise.reject(); });
                        app.showDialog(createEncryptionConfirmationViewModel);
                    }
                    else {
                        encryptionConfirmationDialogPromise.resolve();
                    }
                    _this.createDefaultDatabaseSettings(newDatabase, bundles).always(function () {
                        if (bundles.contains("Quotas") || bundles.contains("Versioning") || bundles.contains("SqlReplication")) {
                            encryptionConfirmationDialogPromise.always(function () {
                                var settingsDialog = new databaseSettingsDialog(bundles);
                                app.showDialog(settingsDialog);
                            });
                        }
                    });
                });
            });
        };
        resources.prototype.addNewDatabase = function (databaseName, bundles, clusterWide) {
            var foundDatabase = this.databases.first(function (db) { return db.name == databaseName; });
            if (!foundDatabase) {
                var newDatabase = new database(databaseName, true, false, bundles, undefined, undefined, clusterWide);
                this.databases.unshift(newDatabase);
                this.filterResources();
                return newDatabase;
            }
            return foundDatabase;
        };
        resources.prototype.createDefaultDatabaseSettings = function (db, bundles) {
            var deferred = $.Deferred();
            new createDefaultDbSettingsCommand(db, bundles).execute()
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        resources.prototype.createDefaultFilesystemSettings = function (fs, bundles) {
            var deferred = $.Deferred();
            new createDefaultFsSettingsCommand(fs, bundles).execute()
                .always(function () { return deferred.resolve(); });
            return deferred;
        };
        resources.prototype.isEmptyStringOrWhitespace = function (str) {
            return !$.trim(str);
        };
        resources.prototype.getEncryptionAlgorithmFullName = function (encrytion) {
            var fullEncryptionName = null;
            switch (encrytion) {
                case "DES":
                    fullEncryptionName = "System.Security.Cryptography.DESCryptoServiceProvider, mscorlib";
                    break;
                case "RC2":
                    fullEncryptionName = "System.Security.Cryptography.RC2CryptoServiceProvider, mscorlib";
                    break;
                case "Rijndael":
                    fullEncryptionName = "System.Security.Cryptography.RijndaelManaged, mscorlib";
                    break;
                default:
                    fullEncryptionName = "System.Security.Cryptography.TripleDESCryptoServiceProvider, mscorlib";
            }
            return fullEncryptionName;
        };
        resources.prototype.showFsCreationAdvancedStepsIfNecessary = function (filesystemName, bundles, settings) {
            var _this = this;
            var securedSettings = {};
            var savedKey;
            var encryptionDeferred = $.Deferred();
            if (bundles.contains("Encryption")) {
                var createEncryptionViewModel = new createEncryption();
                createEncryptionViewModel
                    .creationEncryption
                    .done(function (key, encryptionAlgorithm, encryptionBits, isEncryptedIndexes) {
                    savedKey = key;
                    securedSettings = {
                        'Raven/Encryption/Key': key,
                        'Raven/Encryption/Algorithm': _this.getEncryptionAlgorithmFullName(encryptionAlgorithm),
                        'Raven/Encryption/KeyBitsPreference': encryptionBits,
                        'Raven/Encryption/EncryptIndexes': isEncryptedIndexes
                    };
                    encryptionDeferred.resolve(securedSettings);
                });
                app.showDialog(createEncryptionViewModel);
            }
            else {
                encryptionDeferred.resolve();
            }
            encryptionDeferred.done(function () {
                new createFilesystemCommand(filesystemName, settings, securedSettings)
                    .execute()
                    .done(function () {
                    var newFileSystem = _this.addNewFileSystem(filesystemName, bundles);
                    _this.selectResource(newFileSystem);
                    var encryptionConfirmationDialogPromise = $.Deferred();
                    if (!jQuery.isEmptyObject(securedSettings)) {
                        var createEncryptionConfirmationViewModel = new createEncryptionConfirmation(savedKey);
                        createEncryptionConfirmationViewModel.dialogPromise.done(function () { return encryptionConfirmationDialogPromise.resolve(); });
                        createEncryptionConfirmationViewModel.dialogPromise.fail(function () { return encryptionConfirmationDialogPromise.reject(); });
                        app.showDialog(createEncryptionConfirmationViewModel);
                    }
                    else {
                        encryptionConfirmationDialogPromise.resolve();
                    }
                    _this.createDefaultFilesystemSettings(newFileSystem, bundles).always(function () {
                        if (bundles.contains("Versioning")) {
                            encryptionConfirmationDialogPromise.always(function () {
                                var settingsDialog = new filesystemSettingsDialog(bundles);
                                app.showDialog(settingsDialog);
                            });
                        }
                    });
                });
            });
        };
        resources.prototype.addNewFileSystem = function (fileSystemName, bundles) {
            var foundFileSystem = this.fileSystems.first(function (fs) { return fs.name === fileSystemName; });
            if (!foundFileSystem) {
                var newFileSystem = new fileSystem(fileSystemName, true, false, false, bundles);
                this.fileSystems.unshift(newFileSystem);
                this.filterResources();
                return newFileSystem;
            }
            return foundFileSystem;
        };
        resources.prototype.showCsCreationAdvancedStepsIfNecessary = function (counterStorageName, bundles, settings) {
            var _this = this;
            new createCounterStorageCommand(counterStorageName, settings)
                .execute()
                .done(function () {
                var newCounterStorage = _this.addNewCounterStorage(counterStorageName, bundles);
                _this.selectResource(newCounterStorage);
            });
        };
        resources.prototype.addNewCounterStorage = function (counterStorageName, bundles) {
            var foundCounterStorage = this.counterStorages.first(function (cs) { return cs.name === counterStorageName; });
            if (!!foundCounterStorage)
                return foundCounterStorage;
            var newCounterStorage = new counterStorage(counterStorageName, true, false, bundles);
            this.counterStorages.unshift(newCounterStorage);
            this.filterResources();
            return newCounterStorage;
        };
        resources.prototype.showTsCreationAdvancedStepsIfNecessary = function (timeSeriesName, bundles, settings) {
            var _this = this;
            new createTimeSeriesCommand(timeSeriesName, settings)
                .execute()
                .done(function () {
                var newTimeSeries = _this.addNewTimeSeries(timeSeriesName, bundles);
                _this.selectResource(newTimeSeries);
            });
        };
        resources.prototype.addNewTimeSeries = function (timeSeriesName, bundles) {
            var foundTimeSeries = this.timeSeries.first(function (ts) { return ts.name === timeSeriesName; });
            if (!!foundTimeSeries)
                return foundTimeSeries;
            var newTimeSeries = new timeSeries(timeSeriesName, true, false, bundles);
            this.timeSeries.unshift(newTimeSeries);
            this.filterResources();
            return newTimeSeries;
        };
        return resources;
    })(viewModelBase);
    return resources;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/deleteKey", ["require", "exports", "plugins/dialog", "commands/timeSeries/deleteKeyCommand", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, deleteKeyCommand, dialogViewModelBase) {
    var deleteKey = (function (_super) {
        __extends(deleteKey, _super);
        function deleteKey(type, key, ts) {
            _super.call(this);
            this.type = type;
            this.key = key;
            this.ts = ts;
            this.deletionTask = $.Deferred();
            this.deletionStarted = false;
        }
        deleteKey.prototype.deleteKey = function () {
            var _this = this;
            new deleteKeyCommand(this.type, this.key, this.ts)
                .execute()
                .done(function (result) { return _this.deletionTask.resolve(result); })
                .fail(function (response) { return _this.deletionTask.reject(response); });
            this.deletionStarted = true;
            dialog.close(this);
        };
        deleteKey.prototype.cancel = function () {
            dialog.close(this);
        };
        deleteKey.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.deletionStarted) {
                this.deletionTask.reject();
            }
        };
        return deleteKey;
    })(dialogViewModelBase);
    return deleteKey;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/editPointDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var editPointDialog = (function (_super) {
        __extends(editPointDialog, _super);
        function editPointDialog(editedPoint, isNew) {
            var _this = this;
            _super.call(this);
            this.updateTask = $.Deferred();
            this.updateTaskStarted = false;
            this.editedPoint = ko.observable();
            this.editedPoint(editedPoint);
            this.isNew = ko.computed(function () { return isNew && _this.editedPoint().isNew(); });
        }
        editPointDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        editPointDialog.prototype.nextOrCreate = function () {
            this.updateTaskStarted = true;
            this.updateTask.resolve(this.editedPoint());
            dialog.close(this);
        };
        editPointDialog.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.updateTaskStarted) {
                this.updateTask.reject();
            }
        };
        return editPointDialog;
    })(dialogViewModelBase);
    return editPointDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/editTypeDialog", ["require", "exports", "plugins/dialog", "viewmodels/dialogViewModelBase"], function (require, exports, dialog, dialogViewModelBase) {
    var editTypeDialog = (function (_super) {
        __extends(editTypeDialog, _super);
        function editTypeDialog(editedType, isNew) {
            var _this = this;
            _super.call(this);
            this.updateTask = $.Deferred();
            this.updateTaskStarted = false;
            this.editedType = ko.observable();
            this.editedType(editedType);
            this.isNew = ko.computed(function () { return isNew && _this.editedType().isNew(); });
        }
        editTypeDialog.prototype.cancel = function () {
            dialog.close(this);
        };
        editTypeDialog.prototype.nextOrCreate = function () {
            this.updateTaskStarted = true;
            this.updateTask.resolve(this.editedType());
            dialog.close(this);
        };
        editTypeDialog.prototype.deactivate = function () {
            // If we were closed via X button or other dialog dismissal, reject the deletion task since
            // we never started it.
            if (!this.updateTaskStarted) {
                this.updateTask.reject();
            }
        };
        return editTypeDialog;
    })(dialogViewModelBase);
    return editTypeDialog;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/timeSeriesPoints", ["require", "exports", "durandal/app", "plugins/router", "common/pagedList", "common/appUrl", "commands/timeSeries/getKeyCommand", "commands/timeSeries/putPointCommand", "viewmodels/viewModelBase", "viewmodels/timeSeries/editPointDialog", "viewmodels/timeSeries/deleteKey", "models/timeSeries/pointChange", "models/timeSeries/timeSeriesPoint", "common/pagedResultSet", "commands/timeSeries/getPointsCommand"], function (require, exports, app, router, pagedList, appUrl, getKeyCommand, putPointCommand, viewModelBase, editPointDialog, deleteKey, pointChange, timeSeriesPoint, pagedResultSet, getPointsCommand) {
    var timeSeriesPoints = (function (_super) {
        __extends(timeSeriesPoints, _super);
        function timeSeriesPoints() {
            var _this = this;
            _super.call(this);
            this.viewType = 2 /* TimeSeries */;
            this.type = ko.observable();
            this.fields = ko.observableArray();
            this.key = ko.observable();
            this.pointsCount = ko.observable();
            this.minPoint = ko.observable();
            this.maxPoint = ko.observable();
            this.isFiltered = ko.observable(false);
            this.startPointFilter = ko.observable();
            this.endPointFilter = ko.observable();
            this.isAggregated = ko.observable(false);
            this.duration = ko.observable();
            this.durationType = ko.observable();
            this.pointsList = ko.observable();
            this.selectedPointsIndices = ko.observableArray();
            this.isAnyPointsAutoSelected = ko.observable(false);
            this.isAllPointsAutoSelected = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.isInitialized = timeSeriesPoints.isInitialized;
            this.hasPoints = ko.computed(function () {
                return _this.pointsCount() > 0;
            });
            this.hasAnyPointsSelected = ko.computed(function () { return _this.selectedPointsIndices().length > 0; });
            this.hasAllPointsSelected = ko.computed(function () {
                var numOfSelectedPoints = _this.selectedPointsIndices().length;
                if (!!_this.key() && numOfSelectedPoints !== 0) {
                    return numOfSelectedPoints === _this.pointsCount();
                }
                return false;
            });
            this.selectedPointsText = ko.computed(function () {
                if (!!_this.selectedPointsIndices()) {
                    var documentsText = "points";
                    return documentsText;
                }
                return "";
            });
            this.pointsSelection = ko.computed(function () {
                if (_this.hasAllPointsSelected())
                    return checkbox.Checked;
                if (_this.hasAnyPointsSelected())
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
            this.isFiltered.subscribe(function () {
                _this.refresh();
            });
            this.startPointFilter.subscribe(function () {
                _this.refresh();
            });
            this.endPointFilter.subscribe(function () {
                _this.refresh();
            });
            this.isAggregated.subscribe(function () {
                _this.refresh();
            });
            this.duration.subscribe(function () {
                _this.refresh();
            });
            this.durationType.subscribe(function () {
                _this.refresh();
            });
        }
        timeSeriesPoints.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a key name to view's URL, e.g. #timeSeries/timeSeries?key=Foo&timeSeriestorage=test
            if (args && args.type && args.key) {
                this.fetchKey(args.type, args.key).done(function (result) {
                    _this.type(result.Type.Type);
                    _this.fields(result.Type.Fields);
                    _this.key(result.Key);
                    _this.pointsCount(result.PointsCount);
                    _this.minPoint(result.MinPoint);
                    _this.startPointFilter(result.MinPoint);
                    _this.maxPoint(result.MaxPoint);
                    _this.endPointFilter(result.MaxPoint);
                    if (!_this.pointsList()) {
                        _this.pointsList(_this.createPointsPagedList());
                    }
                    timeSeriesPoints.isInitialized(true);
                });
            }
        };
        timeSeriesPoints.prototype.createPointsPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchPoints(skip, take, _this.isFiltered() ? _this.startPointFilter() : null, _this.isFiltered() ? _this.endPointFilter() : null); };
            var list = new pagedList(fetcher);
            list.collectionName = this.key();
            return list;
        };
        timeSeriesPoints.prototype.fetchPoints = function (skip, take, start, end) {
            var _this = this;
            var doneTask = $.Deferred();
            new getPointsCommand(this.activeTimeSeries(), skip, take, this.type(), this.fields(), this.key(), start, end).execute()
                .done(function (points) { return doneTask.resolve(new pagedResultSet(points, _this.isFiltered() ? _this.pointsList().itemCount() + points.length : _this.pointsCount())); })
                .fail(function (xhr) { return doneTask.reject(xhr); });
            return doneTask;
        };
        timeSeriesPoints.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        timeSeriesPoints.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            timeSeriesPoints.isInitialized(false);
        };
        timeSeriesPoints.prototype.createNotifications = function () {
            return [];
        };
        timeSeriesPoints.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        timeSeriesPoints.prototype.fetchKey = function (type, key) {
            var deferred = $.Deferred();
            new getKeyCommand(this.activeTimeSeries(), type, key).execute().done(function (result) { return deferred.resolve(result); });
            return deferred;
        };
        timeSeriesPoints.prototype.newPoint = function () {
            var _this = this;
            var changeVm = new editPointDialog(new pointChange(new timeSeriesPoint(this.type(), this.fields(), this.key(), moment().format(), this.fields().map(function (x) { return 0; })), true), true);
            changeVm.updateTask.done(function (change) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showDialog(changeVm);
        };
        timeSeriesPoints.prototype.refresh = function () {
            var grid = this.getPointsGrid();
            if (grid) {
                grid.refreshCollectionData();
            }
            var pointsList = this.pointsList();
            if (pointsList) {
                pointsList.invalidateCache();
            }
            this.selectNone();
        };
        timeSeriesPoints.prototype.changePoint = function () {
            var _this = this;
            var grid = this.getPointsGrid();
            if (!grid)
                return;
            var selectedPoint = grid.getSelectedItems(1).first();
            var change = new pointChange(selectedPoint);
            var pointChangeVM = new editPointDialog(change, false);
            pointChangeVM.updateTask.done(function (change, isNew) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showDialog(pointChangeVM);
        };
        timeSeriesPoints.prototype.toggleSelectAll = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid) {
                if (this.hasAnyPointsSelected()) {
                    pointsGrid.selectNone();
                }
                else {
                    pointsGrid.selectSome();
                    this.isAnyPointsAutoSelected(this.hasAllPointsSelected() === false);
                }
            }
        };
        timeSeriesPoints.prototype.selectAll = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid && !!this.key()) {
                pointsGrid.selectAll(this.pointsCount());
            }
        };
        timeSeriesPoints.prototype.selectNone = function () {
            var pointsGrid = this.getPointsGrid();
            if (!!pointsGrid) {
                pointsGrid.selectNone();
            }
        };
        timeSeriesPoints.prototype.deleteSelectedPoints = function () {
            if (this.hasAllPointsSelected()) {
                this.deleteKey();
            }
            else {
                var grid = this.getPointsGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        timeSeriesPoints.prototype.deleteKey = function () {
            var _this = this;
            var deleteKeyVm = new deleteKey(this.type(), this.key(), this.activeTimeSeries());
            deleteKeyVm.deletionTask.done(function () {
                _this.navigate(appUrl.forTimeSeriesType(_this.type(), _this.activeTimeSeries()));
            });
            app.showDialog(deleteKeyVm);
        };
        timeSeriesPoints.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        timeSeriesPoints.prototype.getPointsGrid = function () {
            var gridContents = $(timeSeriesPoints.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        timeSeriesPoints.gridSelector = "#pointsGrid";
        timeSeriesPoints.isInitialized = ko.observable(false);
        return timeSeriesPoints;
    })(viewModelBase);
    return timeSeriesPoints;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/timeSeriesStats", ["require", "exports", "viewmodels/viewModelBase"], function (require, exports, viewModelBase) {
    var timeSeriesStats = (function (_super) {
        __extends(timeSeriesStats, _super);
        function timeSeriesStats() {
            _super.apply(this, arguments);
        }
        timeSeriesStats.prototype.canActivate = function (args) {
            return true;
        };
        return timeSeriesStats;
    })(viewModelBase);
    return timeSeriesStats;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/timeSeriesTypes", ["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "viewmodels/timeSeries/editPointDialog", "models/timeSeries/pointChange", "models/timeSeries/timeSeriesPoint", "commands/timeSeries/getTypesCommand", "commands/timeSeries/putPointCommand", "viewmodels/viewModelBase"], function (require, exports, app, router, appUrl, editPointDialog, pointChange, timeSeriesPoint, getTypesCommand, putPointCommand, viewModelBase) {
    var timeSeriesTypes = (function (_super) {
        __extends(timeSeriesTypes, _super);
        function timeSeriesTypes() {
            var _this = this;
            _super.call(this);
            this.viewType = 2 /* TimeSeries */;
            this.types = ko.observableArray();
            this.selectedType = ko.observable().subscribeTo("ActivateType").distinctUntilChanged();
            this.currentType = ko.observable();
            this.currentTypePagedItems = ko.observable();
            this.selectedKeysIndices = ko.observableArray();
            this.isAnyKeysAutoSelected = ko.observable(false);
            this.isAllKeysAutoSelected = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.isInitialized = timeSeriesTypes.isInitialized;
            this.selectedType.subscribe(function (t) { return _this.selectedTypeChanged(t); });
            this.hasKeys = ko.computed(function () {
                var selectedType = _this.selectedType();
                if (!!selectedType) {
                    return _this.selectedType().keysCount() > 0;
                }
                return false;
            });
            this.hasAnyKeysSelected = ko.computed(function () { return _this.selectedKeysIndices().length > 0; });
            this.hasAllKeysSelected = ko.computed(function () {
                var numOfSelectedKeys = _this.selectedKeysIndices().length;
                if (!!_this.selectedType() && numOfSelectedKeys !== 0) {
                    return numOfSelectedKeys === _this.selectedType().keysCount();
                }
                return false;
            });
            this.selectedKeysText = ko.computed(function () {
                if (!!_this.selectedKeysIndices()) {
                    var documentsText = "time series";
                    return documentsText;
                }
                return "";
            });
            this.keysSelection = ko.computed(function () {
                if (_this.hasAllKeysSelected())
                    return checkbox.Checked;
                if (_this.hasAnyKeysSelected())
                    return checkbox.SomeChecked;
                return checkbox.UnChecked;
            });
        }
        timeSeriesTypes.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            //TODO: update this in documentation
            //this.updateHelpLink('G8CDCP');
            // We can optionally pass in a key name to view's URL, e.g. #timeSeries/timeSeries?key=Foo&timeSeriestorage=test
            if (args && args.type && args.key) {
                this.typeToSelect = args.type;
            }
            var ts = this.activeTimeSeries();
            this.fetchTypes(ts).done(function (results) {
                _this.typesLoaded(results, ts);
                timeSeriesTypes.isInitialized(true);
            });
        };
        timeSeriesTypes.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        timeSeriesTypes.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            timeSeriesTypes.isInitialized(false);
        };
        timeSeriesTypes.prototype.createNotifications = function () {
            return [];
        };
        timeSeriesTypes.prototype.createPostboxSubscriptions = function () {
            var _this = this;
            return [
                //    ko.postbox.subscribe("ChangePointValue", () => this.changePoint()),
                ko.postbox.subscribe("ChangesApiReconnected", function (ts) { return _this.reloadTimeSeriesData(ts); }),
                ko.postbox.subscribe("SortTypes", function () { return _this.sortTypes(); })
            ];
        };
        timeSeriesTypes.prototype.fetchTypes = function (ts) {
            var deferred = $.Deferred();
            new getTypesCommand(ts).execute().done(function (results) { return deferred.resolve(results); });
            return deferred;
        };
        timeSeriesTypes.prototype.typesLoaded = function (types, ts) {
            var _this = this;
            this.types(types);
            if (this.typeToSelect)
                this.types.first(function (g) { return g.name === _this.typeToSelect; }).activate();
            else {
                var allTypes = this.types();
                if (allTypes.length > 0) {
                    allTypes[0].activate();
                }
            }
        };
        timeSeriesTypes.prototype.newPoint = function () {
            var _this = this;
            var type = this.currentType();
            var changeVm = new editPointDialog(new pointChange(new timeSeriesPoint(type.name, type.fields, "", moment().format(), type.fields.map(function (x) { return 0; })), true), true);
            changeVm.updateTask.done(function (change) {
                new putPointCommand(change.type(), change.key(), change.at(), change.values(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showDialog(changeVm);
        };
        timeSeriesTypes.prototype.refresh = function () {
            var selectedKeyName = this.selectedType().name;
            this.refreshGridAndKey(selectedKeyName);
        };
        timeSeriesTypes.prototype.refreshGridAndKey = function (changedKeyName) {
            var type = this.selectedType();
            if (type.name === changedKeyName) {
                this.getKeysGrid().refreshCollectionData();
            }
            type.getKeys().invalidateCache();
            this.selectNone();
        };
        timeSeriesTypes.prototype.selectedTypeChanged = function (selected) {
            if (!!selected) {
                var pagedList = selected.getKeys();
                this.currentTypePagedItems(pagedList);
                this.currentType(selected);
            }
        };
        timeSeriesTypes.prototype.toggleSelectAll = function () {
            var pointsGrid = this.getKeysGrid();
            if (!!pointsGrid) {
                if (this.hasAnyKeysSelected()) {
                    pointsGrid.selectNone();
                }
                else {
                    pointsGrid.selectSome();
                    this.isAnyKeysAutoSelected(this.hasAllKeysSelected() === false);
                }
            }
        };
        timeSeriesTypes.prototype.selectAll = function () {
            var typesGrid = this.getKeysGrid();
            var type = this.selectedType();
            if (!!typesGrid && !!type) {
                typesGrid.selectAll(type.keysCount());
            }
        };
        timeSeriesTypes.prototype.selectNone = function () {
            var pointsGrid = this.getKeysGrid();
            if (!!pointsGrid) {
                pointsGrid.selectNone();
            }
        };
        timeSeriesTypes.prototype.deleteSelectedKeys = function () {
            if (this.hasAllKeysSelected()) {
            }
            else {
                var grid = this.getKeysGrid();
                if (grid) {
                    grid.deleteSelectedItems();
                }
            }
        };
        timeSeriesTypes.prototype.updateTypes = function (receivedTypes) {
            var _this = this;
            var deletedTypes = [];
            this.types().forEach(function (t) {
                if (!receivedTypes.first(function (receivedGroup) { return t.name === receivedGroup.name; })) {
                    deletedTypes.push(t);
                }
            });
            this.types.removeAll(deletedTypes);
            receivedTypes.forEach(function (receivedGroup) {
                var foundType = _this.types().first(function (t) { return t.name === receivedGroup.name; });
                if (!foundType) {
                    _this.types.push(receivedGroup);
                }
                else {
                    foundType.keysCount(receivedGroup.keysCount());
                }
            });
        };
        timeSeriesTypes.prototype.refreshKeysData = function () {
            var _this = this;
            var selectedGroup = this.selectedType();
            this.types().forEach(function (key) {
                if (key.name === selectedGroup.name) {
                    var docsGrid = _this.getKeysGrid();
                    if (!!docsGrid) {
                        docsGrid.refreshCollectionData();
                    }
                }
                else {
                    var pagedList = key.getKeys();
                    pagedList.invalidateCache();
                }
            });
        };
        timeSeriesTypes.prototype.refreshKeys = function () {
            var _this = this;
            var deferred = $.Deferred();
            var ts = this.activeTimeSeries();
            this.fetchTypes(ts).done(function (results) {
                _this.updateTypes(results);
                _this.refreshKeysData();
                deferred.resolve();
            });
            return deferred;
        };
        timeSeriesTypes.prototype.reloadTimeSeriesData = function (ts) {
            var _this = this;
            if (ts.name === this.activeTimeSeries().name) {
                this.refreshKeys().done(function () { return _this.refreshKeysData(); });
            }
        };
        timeSeriesTypes.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        timeSeriesTypes.prototype.getKeysGrid = function () {
            var gridContents = $(timeSeriesTypes.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        timeSeriesTypes.prototype.sortTypes = function () {
            this.types.sort(function (c1, c2) {
                return c1.name.toLowerCase() > c2.name.toLowerCase() ? 1 : -1;
            });
        };
        // Animation callbacks for the types list
        timeSeriesTypes.prototype.showTypeElement = function (element) {
            if (element.nodeType === 1 && timeSeriesTypes.isInitialized()) {
                $(element).hide().slideDown(500, function () {
                    ko.postbox.publish("SortTypes");
                    $(element).highlight();
                });
            }
        };
        timeSeriesTypes.prototype.hideTypeElement = function (element) {
            if (element.nodeType === 1) {
                $(element).slideUp(1000, function () { $(element).remove(); });
            }
        };
        timeSeriesTypes.gridSelector = "#keysGrid";
        timeSeriesTypes.isInitialized = ko.observable(false);
        return timeSeriesTypes;
    })(viewModelBase);
    return timeSeriesTypes;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/configuration/configuration", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var configuration = (function (_super) {
        __extends(configuration, _super);
        function configuration() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.appUrls = appUrl.forCurrentDatabase();
            var typesRoute = { route: ['timeSeries/configuration', 'timeSeries/configuration/types'], moduleId: 'viewmodels/timeSeries/configuration/types', title: 'Types Configuration', nav: true, hash: this.appUrls.timeSeriesConfigurationTypes };
            this.router = durandalRouter.createChildRouter()
                .map([
                typesRoute
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return configuration;
    })(viewModelBase);
    return configuration;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/configuration/types", ["require", "exports", "durandal/app", "plugins/router", "common/pagedList", "common/appUrl", "commands/timeSeries/putTypeCommand", "viewmodels/viewModelBase", "viewmodels/timeSeries/editTypeDialog", "models/timeSeries/timeSeriesType", "common/pagedResultSet", "commands/timeSeries/getTypesCommand", "models/timeSeries/typeChange"], function (require, exports, app, router, pagedList, appUrl, putTypeCommand, viewModelBase, editTypeDialog, timeSeriesType, pagedResultSet, getTypesCommand, typeChange) {
    var types = (function (_super) {
        __extends(types, _super);
        function types() {
            var _this = this;
            _super.call(this);
            this.viewType = 2 /* TimeSeries */;
            this.typesList = ko.observable();
            this.selectedTypesIndices = ko.observableArray();
            this.isAnyTypesAutoSelected = ko.observable(false);
            this.isAllTypesAutoSelected = ko.observable(false);
            this.showLoadingIndicator = ko.observable(false);
            this.showLoadingIndicatorThrottled = this.showLoadingIndicator.throttle(250);
            this.isInitialized = types.isInitialized;
            this.hasTypes = ko.computed(function () {
                var typesList = _this.typesList();
                return typesList && typesList.itemCount() > 0;
            });
            this.hasAnyTypesSelected = ko.computed(function () { return _this.selectedTypesIndices().length > 0; });
            this.canChangeType = ko.computed(function () {
                if (_this.selectedTypesIndices().length !== 1)
                    return false;
                var selectedItem = _this.getTypesGrid().getSelectedItems()[0];
                return selectedItem.keysCount() === 0;
            });
        }
        types.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.typesList(this.createTypesPagedList());
        };
        types.prototype.createTypesPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) { return _this.fetchTypes(skip, take); };
            var list = new pagedList(fetcher);
            return list;
        };
        types.prototype.fetchTypes = function (skip, take) {
            var deffered = $.Deferred();
            new getTypesCommand(this.activeTimeSeries()).execute()
                .done(function (types) {
                deffered.resolve(new pagedResultSet(types, 1024));
            });
            return deffered;
        };
        types.prototype.attached = function () {
            _super.prototype.attached.call(this);
            /*super.createKeyboardShortcut("F2", () => this.editSelectedTimeSeries(), timeSeries.gridSelector);*/
            // Q. Why do we have to setup the grid shortcuts here, when the grid already catches these shortcuts?
            // A. Because if the focus isn't on the grid, but on the docs page itself, we still need to catch the shortcuts.
            /*var docsPageSelector = ".documents-page";
            this.createKeyboardShortcut("DELETE", () => this.getDocumentsGrid().deleteSelectedItems(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, D", () => this.copySelectedDocs(), docsPageSelector);
            this.createKeyboardShortcut("Ctrl+C, I", () => this.copySelectedDocIds(), docsPageSelector);*/
        };
        types.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            types.isInitialized(false);
        };
        types.prototype.createNotifications = function () {
            return [];
        };
        types.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        types.prototype.newType = function () {
            var _this = this;
            var changeVm = new editTypeDialog(new typeChange(new timeSeriesType("", [""], 0, this.activeTimeSeries()), true), true);
            changeVm.updateTask.done(function (change) {
                new putTypeCommand(change.type(), change.fields(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showDialog(changeVm);
        };
        types.prototype.changeType = function () {
            var _this = this;
            var grid = this.getTypesGrid();
            if (!grid)
                return;
            var selectedType = grid.getSelectedItems(1).first();
            var change = new typeChange(selectedType);
            var typeChangeVM = new editTypeDialog(change, false);
            typeChangeVM.updateTask.done(function (change, isNew) {
                new putTypeCommand(change.type(), change.fields(), _this.activeTimeSeries())
                    .execute()
                    .done(function () { return _this.refresh(); });
            });
            app.showDialog(typeChangeVM);
        };
        types.prototype.refresh = function () {
            this.getTypesGrid().refreshCollectionData();
            this.typesList().invalidateCache();
        };
        types.prototype.deleteSelectedTypes = function () {
            var grid = this.getTypesGrid();
            if (grid) {
                grid.deleteSelectedItems();
            }
        };
        types.prototype.selectType = function (type, event) {
            if (!event || event.which !== 3) {
                type.activate();
                var timeSeriesTypeUrl = appUrl.forTimeSeriesType(type.name, this.activeTimeSeries());
                router.navigate(timeSeriesTypeUrl, false);
            }
        };
        types.prototype.getTypesGrid = function () {
            var gridContents = $(types.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }
            return null;
        };
        types.gridSelector = "#typesGrid";
        types.isInitialized = ko.observable(false);
        return types;
    })(viewModelBase);
    return types;
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define("viewmodels/timeSeries/configurations/configuration", ["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase"], function (require, exports, durandalRouter, appUrl, viewModelBase) {
    var configuration = (function (_super) {
        __extends(configuration, _super);
        function configuration() {
            var _this = this;
            _super.call(this);
            this.router = null;
            this.bundleMap = { types: "Types" };
            this.appUrls = appUrl.forCurrentDatabase();
            var typesRoute = { route: 'timeSeries/settings/types', moduleId: 'viewmodels/timeSeries/configuration/types', title: 'Types', nav: true, hash: appUrl.forCurrentTimeSeries().timeSeriesConfigurationTypes };
            this.router = durandalRouter.createChildRouter()
                .map([
                typesRoute
            ])
                .buildNavigationModel();
            appUrl.mapUnknownRoutes(this.router);
            this.activeSubViewTitle = ko.computed(function () {
                // Is there a better way to get the active route?
                var activeRoute = _this.router.navigationModel().first(function (r) { return r.isActive(); });
                return activeRoute != null ? activeRoute.title : "";
            });
        }
        return configuration;
    })(viewModelBase);
    return configuration;
});
define("widgets/virtualTable/cell", ["require", "exports", "widgets/virtualTable/settings"], function (require, exports, settings) {
    var cell = (function () {
        function cell(data, templateName) {
            this.templateName = templateName;
            this.data = ko.observable();
            this.resetFlag = false;
            this.update(data);
        }
        cell.prototype.update = function (data) {
            if (this.isNumber(data) && !settings.useRawFormat) {
                data = data.toLocaleString('en-US');
            }
            this.data(data);
        };
        cell.prototype.isNumber = function (o) {
            return typeof o === "number" && isFinite(o);
        };
        cell.prototype.reset = function () {
            this.data("");
            this.resetFlag = true;
        };
        cell.defaultTemplate = "defaultTemplate";
        cell.idTemplate = "idTemplate";
        cell.checkboxTemplate = "checkboxTemplate";
        cell.externalIdTemplate = "externalIdTemplate";
        cell.customTemplate = "customTemplate";
        cell.counterNameTemplate = "counterNameTemplate";
        cell.counterGroupTemplate = "counterGroupTemplate";
        cell.timeSeriesKeyTemplate = "timeSeriesKeyTemplate";
        return cell;
    })();
    return cell;
});
define("widgets/virtualTable/column", ["require", "exports"], function (require, exports) {
    var column = (function () {
        function column(binding, width, header) {
            this.width = ko.observable(0);
            this.binding = binding;
            this.header = header || binding;
            this.width(width);
        }
        return column;
    })();
    return column;
});
define("widgets/virtualTable/row", ["require", "exports", "widgets/virtualTable/cell", "common/execJs"], function (require, exports, cell, execJs) {
    var row = (function () {
        function row(addIdCell, viewModel) {
            this.viewModel = viewModel;
            this.top = ko.observable(0);
            this.rowIndex = ko.observable(0);
            this.isInUse = ko.observable(false);
            this.cellMap = { Id: null };
            this.collectionClass = ko.observable("");
            this.editUrl = ko.observable("");
            this.isChecked = ko.observable(false);
            this.compiledCustomFunctions = {};
            if (addIdCell) {
                this.addOrUpdateCellMap("Id", null);
            }
            this.viewModel.settings.customFunctions.subscribe(this.extractCustomFunctions);
            this.extractCustomFunctions(this.viewModel.settings.customFunctions());
        }
        row.prototype.calculateExternalIdCellColor = function (cellValue) {
            var cellCollectionName = cellValue.slice(0, cellValue.lastIndexOf("/")).toLocaleLowerCase();
            var matchingCollection = this.viewModel.settings.collections.first(function (c) { return c.name.toLocaleLowerCase() === cellCollectionName; });
            if (!!matchingCollection) {
                return matchingCollection.colorClass;
            }
            return "";
        };
        row.prototype.calculateExternalGroupCellColor = function (cellValue) {
            var matchingGroup = this.viewModel.settings.collections.first(function (c) { return c.name === cellValue; });
            if (!!matchingGroup) {
                return matchingGroup.colorClass;
            }
            return "";
        };
        row.prototype.extractCustomFunctions = function (newValue) {
            this.compiledCustomFunctions = new Function("var exports = {}; " + newValue.functions + "; return exports;")();
        };
        row.prototype.resetCells = function () {
            for (var prop in this.cellMap) {
                var cellVal = this.cellMap[prop];
                if (cellVal) {
                    cellVal.reset();
                }
            }
            this.collectionClass("");
            this.isChecked(false);
        };
        row.prototype.createPlaceholderCells = function (cellNames) {
            var _this = this;
            cellNames
                .filter(function (c) { return c != "Id"; })
                .forEach(function (c) { return _this.addOrUpdateCellMap(c, null); });
        };
        row.prototype.fillCells = function (rowData) {
            var _this = this;
            var customColumns = this.viewModel.settings.customColumns();
            this.isInUse(true);
            var rowProperties = rowData.getDocumentPropertyNames();
            if (customColumns.customMode()) {
                customColumns.columns().forEach(function (column, index) {
                    var binding = column.binding();
                    var context = {};
                    $.each(rowData, function (name, value) {
                        context[name] = value;
                    });
                    for (var p in _this.compiledCustomFunctions) {
                        context[p] = _this.compiledCustomFunctions[p];
                    }
                    var cellValue = execJs.createSimpleCallableCode("return " + binding + ";", context)();
                    var callValueAsString = (typeof cellValue === "object" && _this.getCellTemplateName(binding, rowData) !== cell.customTemplate) ? JSON.stringify(cellValue, null, 4) : cellValue;
                    _this.addOrUpdateCellMap(binding, callValueAsString);
                });
            }
            else {
                for (var i = 0; i < rowProperties.length; i++) {
                    var prop = rowProperties[i];
                    var cellValue = rowData[prop];
                    // pass json object when not custom template!
                    if (typeof cellValue === "object" && this.getCellTemplateName(prop, rowData) !== cell.customTemplate) {
                        cellValue = JSON.stringify(cellValue, null, 4) || "";
                    }
                    if (cellValue && cellValue.length > 300) {
                        cellValue = cellValue.substring(0, 300);
                    }
                    this.addOrUpdateCellMap(prop, cellValue);
                }
            }
            if (rowData.getId()) {
                this.addOrUpdateCellMap("Id", rowData.getId());
            }
        };
        row.prototype.addOrUpdateCellMap = function (propertyName, data) {
            if (!this.cellMap[propertyName]) {
                this.cellMap[propertyName] = new cell(data, this.getCellTemplateName(propertyName, data));
            }
            else {
                var cellVal = this.cellMap[propertyName];
                cellVal.update(data);
            }
        };
        row.prototype.getCellData = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                return cellVal.data;
            }
            return "";
        };
        row.prototype.getCellTemplate = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                if (cellVal.resetFlag) {
                    cellVal.templateName = this.getCellTemplateName(cellName, this.cellMap[cellName].data());
                    cellVal.resetFlag = false;
                    return cellVal.templateName;
                }
                else {
                    return cellVal.templateName;
                }
            }
            // Bug fix: http://issues.hibernatingrhinos.com/issue/RavenDB-2002
            // Calling .data() registers it as a Knockout dependency; updating this 
            // observable later will cause the cell to redraw, thus fixing the bug.
            if (this.cellMap && this.cellMap.Id) {
                this.cellMap["Id"].data();
            }
            return "nullTemplate";
        };
        row.prototype.getCellTemplateName = function (propertyName, data) {
            if (propertyName === "Id") {
                return cell.idTemplate;
            }
            else if (propertyName === "__IsChecked") {
                return cell.checkboxTemplate;
            }
            switch (this.viewModel.settings.viewType) {
                case 1 /* Counters */:
                    if (propertyName === "Counter Name") {
                        return cell.counterNameTemplate;
                    }
                    else if (this.viewModel.settings.isCounterAllGroupsGroup() && propertyName === "Group Name") {
                        return cell.counterGroupTemplate;
                    }
                    return cell.defaultTemplate;
                case 2 /* TimeSeries */:
                    if (propertyName === "Key") {
                        return cell.timeSeriesKeyTemplate;
                    }
                    return cell.defaultTemplate;
            }
            // see if this is an ID or external ID cell.
            if (!!data) {
                if (typeof data == "string") {
                    var cleanData = data.replace('/\t+/g', '')
                        .replace(/\s+/g, '')
                        .replace('/\n+/g', '');
                    if (/^\[{"[a-zA-Z0-9_-]+":/.test(cleanData) ||
                        //this handy REGEX for testing URLs was taken from http://stackoverflow.com/questions/8188645/javascript-regex-to-match-a-url-in-a-field-of-text
                        /(http|ftp|https):\/\/[\w-]+(\.[\w-]+)+([\w.,@?^=%&amp;:\/~+#-]*[\w@?^=%&amp;\/~+#-])?/.test(cleanData))
                        return cell.defaultTemplate;
                    if (/^\w+\/\w+/ig.test(data) && this.viewModel.collectionExists(data)) {
                        return cell.externalIdTemplate;
                    }
                }
                else if (!!data[propertyName] &&
                    typeof data[propertyName] == "string" &&
                    /\w+\/\w+/.test(data[propertyName])) {
                    return cell.externalIdTemplate;
                }
            }
            // note: we just inform here about custom template - without specific name of this template.
            var colParam = this.viewModel.settings.customColumns().findConfigFor(propertyName);
            if (colParam && colParam.template() !== cell.defaultTemplate && colParam.template() !== cell.counterGroupTemplate) {
                return cell.customTemplate;
            }
            return cell.defaultTemplate;
        };
        return row;
    })();
    return row;
});
define("widgets/virtualTable/settings", ["require", "exports", "common/numberFormattingStorage"], function (require, exports, numberFormattingStorage) {
    var settings = (function () {
        function settings() {
        }
        settings.watchStorage = function () {
            $(window).bind("storage", function (e) {
                if (e.originalEvent.key === numberFormattingStorage.localStorageName) {
                    settings.useRawFormat = e.originalEvent.newValue;
                }
            });
        };
        settings.useRawFormat = numberFormattingStorage.shouldUseRaw();
        return settings;
    })();
    settings.watchStorage();
    return settings;
});
define("widgets/virtualTable/viewmodel", ["require", "exports", "plugins/router", "durandal/app", "common/appUrl", "models/database/documents/document", "models/database/documents/collection", "models/counter/counterSummary", "models/counter/counterGroup", "viewmodels/common/deleteItems", "viewmodels/database/documents/copyDocuments", "widgets/virtualTable/row", "widgets/virtualTable/column", "models/database/documents/customColumnParams", "models/database/documents/customColumns", "models/database/documents/customFunctions", "models/timeSeries/timeSeriesKey", "models/timeSeries/timeSeriesType"], function (require, exports, router, app, appUrl, document, collection, counterSummary, counterGroup, deleteItems, copyDocuments, row, column, customColumnParams, customColumns, customFunctions, timeSeriesKey, timeSeriesType) {
    var ctor = (function () {
        function ctor() {
            this.$window = $(window);
            this.recycleRows = ko.observableArray();
            this.rowHeight = 38;
            this.borderHeight = 2;
            this.viewportHeight = ko.observable(0);
            this.virtualRowCount = ko.observable(0);
            this.columns = ko.observableArray();
            this.scrollThrottleTimeoutHandle = 0;
            this.firstVisibleRow = null;
            this.lastVisibleRow = null;
            this.itemsSourceSubscription = null;
            this.ensureColumnsAnimationFrameHandle = 0;
            this.headerVisible = ko.observable(false);
        }
        ctor.prototype.activate = function (settings) {
            var _this = this;
            var defaults = {
                dynamicHeightTargetSelector: "footer",
                dynamicHeightBottomMargin: 0,
                selectedIndices: ko.observableArray(),
                showCheckboxes: true,
                showIds: true,
                useContextMenu: true,
                maxHeight: 'none',
                customColumnParams: {},
                isIndexMapReduce: ko.observable(true),
                isCopyAllowed: true,
                contextMenuOptions: ["CopyItems", "CopyIDs", "Delete", "EditItem"],
                selectionEnabled: true,
                customColumns: ko.observable(customColumns.empty()),
                customFunctions: ko.observable(customFunctions.empty()),
                collections: ko.observableArray([]),
                rowsAreLoading: ko.observable(false),
                noResultsMessage: "No records found.",
                isAnyAutoSelected: ko.observable(false),
                isAllAutoSelected: ko.observable(false),
                viewType: 0 /* Documents */,
                isCounterAllGroupsGroup: ko.observable(false)
            };
            this.settings = $.extend(defaults, settings);
            this.bottomMargin = ko.computed(function () {
                // if header is visible we have to substruct it's height to avoid scroll
                var headerHeight = _this.headerVisible() ? 0 : 41;
                return headerHeight + (_this.settings.dynamicHeightBottomMargin || 0);
            });
            this.$window.resize(function () {
                _this.headerVisible($(".ko-grid-column-container", _this.grid).is(":visible"));
            });
            if (!!settings.isIndexMapReduce) {
                this.isIndexMapReduce = settings.isIndexMapReduce;
            }
            else {
                this.isIndexMapReduce = ko.observable(false);
            }
            this.getCollectionClassFromEntityNameMemoized = this.getCollectionClassFromEntityName.memoize(this);
            this.items = this.settings.itemsSource();
            this.focusableGridSelector = this.settings.gridSelector + " .ko-grid";
            this.virtualHeight = ko.computed(function () { return _this.rowHeight * _this.virtualRowCount(); });
            this.refreshIdAndCheckboxColumn();
            this.itemsSourceSubscription = this.settings.itemsSource.subscribe(function (list) {
                _this.recycleRows().forEach(function (r) {
                    r.resetCells();
                    _this.recycleRows.valueHasMutated();
                    _this.columns.valueHasMutated();
                    r.isInUse(false);
                });
                _this.items = list;
                _this.settings.selectedIndices.removeAll();
                _this.columns.remove(function (c) { return (c.binding !== "Id" && c.binding !== "__IsChecked"); });
                _this.gridViewport.scrollTop(0);
                _this.onGridScrolled();
                _this.refreshIdAndCheckboxColumn();
            });
            this.noResults = ko.computed(function () {
                var numOfRowsInUse = _this.recycleRows().filter(function (r) { return r.isInUse(); }).length;
                return numOfRowsInUse === 0 && !_this.settings.rowsAreLoading();
            });
            this.registerColumnResizing();
        };
        // Attached is called by Durandal when the view is attached to the DOM.
        // We use this to setup some UI-specific things like context menus, row creation, keyboard shortcuts, etc.
        ctor.prototype.attached = function () {
            var _this = this;
            this.grid = $(this.settings.gridSelector);
            if (this.grid.length !== 1) {
                // Don't throw an error here, because the user can cancel navigation, causing the element not to be found on the page. 
                //throw new Error("There should be 1 " + this.settings.gridSelector + " on the page, but found " + this.grid.length.toString());
                console.warn("There should be 1 " + this.settings.gridSelector + " on the page, but found " + this.grid.length.toString());
                return;
            }
            this.gridViewport = this.grid.find(".ko-grid-viewport-container");
            this.gridViewport.on('DynamicHeightSet', function () { return _this.onWindowHeightChanged(); });
            this.gridViewport.scroll(function () { return _this.onGridScrolled(); });
            this.setupKeyboardShortcuts();
            if (this.settings.useContextMenu) {
                this.setupContextMenu();
            }
        };
        ctor.prototype.detached = function () {
            $(this.settings.gridSelector).unbind('keydown.jwerty');
            this.gridViewport.off('DynamicHeightSet');
            if (this.itemsSourceSubscription) {
                this.itemsSourceSubscription.dispose();
            }
            this.unregisterColumnResizing();
        };
        ctor.prototype.calculateRecycleRowCount = function () {
            var requiredRowCount = Math.ceil(this.viewportHeight() / this.rowHeight);
            var rowCountWithPadding = requiredRowCount + 10;
            return rowCountWithPadding;
        };
        ctor.prototype.createRecycleRows = function (rowCount) {
            for (var i = this.recycleRows().length; i < rowCount; i++) {
                var newRow = new row(this.settings.showIds, this);
                newRow.createPlaceholderCells(this.columns().map(function (c) { return c.binding; }));
                newRow.rowIndex(i);
                var desiredTop = i * this.rowHeight;
                newRow.top(desiredTop);
                this.recycleRows.push(newRow);
            }
            for (var i = rowCount; i < this.recycleRows().length; i++) {
                var r = this.recycleRows()[i];
                r.isInUse(false);
            }
            app.trigger(this.settings.gridSelector + "RowsCreated", true);
        };
        ctor.prototype.onGridScrolled = function () {
            var _this = this;
            this.settings.rowsAreLoading(true);
            this.ensureRowsCoverViewport();
            this.scrollThrottleTimeoutHandle = this.requestAnimationFrame(function () { return _this.loadRowData(); }, this.scrollThrottleTimeoutHandle);
        };
        ctor.prototype.onWindowHeightChanged = function () {
            var _this = this;
            this.settings.rowsAreLoading(true);
            var newViewportHeight = this.gridViewport.height();
            this.viewportHeight(newViewportHeight);
            var desiredRowCount = this.calculateRecycleRowCount();
            this.createRecycleRows(desiredRowCount);
            this.ensureRowsCoverViewport();
            this.loadRowData();
            // Update row checked states.
            this.recycleRows().forEach(function (r) { return r.isChecked(_this.settings.selectedIndices().contains(r.rowIndex())); });
        };
        ctor.prototype.setupKeyboardShortcuts = function () {
            var _this = this;
            this.setupKeyboardShortcut("DELETE", function () { return _this.deleteSelectedItems(); });
            this.setupKeyboardShortcut("Ctrl+C,D", function () { return _this.copySelectedDocs(); });
            this.setupKeyboardShortcut("Ctrl+C,I", function () { return _this.copySelectedDocIds(); });
        };
        ctor.prototype.setupKeyboardShortcut = function (keys, handler) {
            jwerty.key(keys, function (e) {
                e.preventDefault();
                handler();
            }, this, this.settings.gridSelector);
        };
        ctor.prototype.setupContextMenu = function () {
            var _this = this;
            var untypedGrid = this.grid;
            untypedGrid.contextmenu({
                target: '#gridContextMenu',
                before: function (e) {
                    var target = e.target;
                    var rowTag = (target.className.indexOf("ko-grid-row") > -1) ? $(target) : $(e.target).parents(".ko-grid-row");
                    var rightClickedElement = rowTag.length ? ko.dataFor(rowTag[0]) : null;
                    if (_this.settings.showCheckboxes && !_this.isIndexMapReduce()) {
                        // Select any right-clicked row.
                        if (rightClickedElement && rightClickedElement.isChecked != null && !rightClickedElement.isChecked()) {
                            _this.toggleRowChecked(rightClickedElement, e.shiftKey);
                        }
                    }
                    else {
                        if (rightClickedElement) {
                            _this.settings.selectedIndices([rightClickedElement.rowIndex()]);
                        }
                    }
                    return true;
                }
            });
        };
        ctor.prototype.refreshIdAndCheckboxColumn = function () {
            var containsId = this.columns().first(function (x) { return x.binding === "Id"; });
            if (!containsId && !this.isIndexMapReduce()) {
                var containsCheckbox = this.columns().first(function (x) { return x.binding === "__IsChecked"; });
                if (!containsCheckbox && this.settings.showCheckboxes) {
                    this.columns.push(new column("__IsChecked", 38));
                }
                if (this.settings.showIds !== false) {
                    this.columns.push(new column("Id", ctor.idColumnWidth));
                }
                this.columns.valueHasMutated();
            }
            else if (containsId && this.isIndexMapReduce()) {
                this.columns.remove(function (c) { return c.binding === "Id" || c.binding === "__IsChecked"; });
                this.columns.valueHasMutated();
            }
        };
        ctor.prototype.loadRowData = function () {
            var _this = this;
            if (this.items && this.firstVisibleRow) {
                this.settings.rowsAreLoading(true);
                // The scrolling has paused for a minute. See if we have all the data needed.
                var firstVisibleIndex = this.firstVisibleRow.rowIndex();
                var fetchTask = this.items.fetch(firstVisibleIndex, this.recycleRows().length);
                fetchTask.done(function (resultSet) {
                    var firstVisibleRowIndexHasChanged = firstVisibleIndex !== _this.firstVisibleRow.rowIndex();
                    if (!firstVisibleRowIndexHasChanged) {
                        _this.virtualRowCount(resultSet.totalResultCount);
                        resultSet.items.forEach(function (r, i) { return _this.fillRow(r, i + firstVisibleIndex); });
                        // when we have few rows and we delete once of them there might be old rows that must be removed
                        _this.recycleRows().filter(function (r, i) { return i >= resultSet.items.length + firstVisibleIndex && r.isInUse(); }).map(function (r) { return r.isInUse(false); });
                        // Because processing all columns can take time for many columns, we
                        // asynchronously load the column information in the next animation frame.
                        _this.ensureColumnsAnimationFrameHandle = _this.requestAnimationFrame(function () { return _this.ensureColumnsForRows(resultSet.items); }, _this.ensureColumnsAnimationFrameHandle);
                    }
                });
            }
        };
        ctor.prototype.requestAnimationFrame = function (action, existingHandleToCancel) {
            var result;
            if (window.requestAnimationFrame) {
                if (existingHandleToCancel) {
                    window.cancelAnimationFrame(existingHandleToCancel);
                }
                result = window.requestAnimationFrame(action);
            }
            else if (window.msRequestAnimationFrame) {
                if (window.msCancelRequestAnimationFrame) {
                    window.msCancelRequestAnimationFrame(existingHandleToCancel);
                }
                result = window.msRequestAnimationFrame(action);
            }
            else {
                if (existingHandleToCancel) {
                    window.clearTimeout(existingHandleToCancel);
                }
                result = setTimeout(action, 1);
            }
            this.settings.rowsAreLoading(false);
            return result;
        };
        ctor.prototype.fillRow = function (rowData, rowIndex) {
            var rowAtIndex = ko.utils.arrayFirst(this.recycleRows(), function (r) { return r.rowIndex() === rowIndex; });
            if (rowAtIndex) {
                rowAtIndex.fillCells(rowData);
                var entityName = this.getEntityName(rowData);
                rowAtIndex.collectionClass(this.getCollectionClassFromEntityNameMemoized(rowData, entityName));
                var editUrl;
                if (rowData instanceof counterSummary) {
                    editUrl = appUrl.forEditCounter(appUrl.getResource(), rowData["Group Name"], rowData["Counter Name"]);
                }
                else if (rowData instanceof timeSeriesKey) {
                    editUrl = appUrl.forTimeSeriesKey(rowData["Type"], rowData["Key"], appUrl.getTimeSeries());
                }
                else {
                    editUrl = appUrl.forEditItem(!!rowData.getUrl() ? rowData.getUrl() : rowData["Id"], appUrl.getResource(), rowIndex, entityName);
                }
                rowAtIndex.editUrl(editUrl);
            }
        };
        ctor.prototype.editLastSelectedItem = function () {
            var selectedItem = this.getSelectedItems(1).first();
            if (selectedItem) {
                var collectionName = this.items.collectionName;
                var itemIndex = this.settings.selectedIndices().first();
                var editUrl;
                if (selectedItem instanceof counterSummary) {
                    editUrl = appUrl.forEditCounter(appUrl.getResource(), selectedItem["Group Name"], selectedItem["Counter Name"]);
                }
                else if (selectedItem instanceof timeSeriesKey) {
                    editUrl = appUrl.forTimeSeriesKey(selectedItem["Type"], selectedItem["Key"], appUrl.getTimeSeries());
                }
                else {
                    editUrl = appUrl.forEditItem(selectedItem.getUrl(), appUrl.getResource(), itemIndex, collectionName);
                }
                router.navigate(editUrl);
            }
        };
        ctor.prototype.getCollectionClassFromEntityName = function (rowData, entityName) {
            if (rowData instanceof document) {
                return collection.getCollectionCssClass(entityName, appUrl.getDatabase());
            }
            if (rowData instanceof counterSummary) {
                return counterGroup.getGroupCssClass(entityName, appUrl.getCounterStorage());
            }
            if (rowData instanceof timeSeriesKey) {
                return timeSeriesType.getTypeCssClass(entityName, appUrl.getTimeSeries());
            }
            return "";
        };
        ctor.prototype.getEntityName = function (rowData) {
            var obj = rowData;
            if (obj && obj.getEntityName) {
                if (obj instanceof document) {
                    var documentObj = obj;
                    return documentObj.getEntityName();
                }
                if (obj instanceof counterSummary) {
                    var counterSummaryObj = obj;
                    return counterSummaryObj.getEntityName();
                }
                if (obj instanceof timeSeriesKey) {
                    var timeSeriesKeyObj = obj;
                    return timeSeriesKeyObj.getEntityName();
                }
            }
            return null;
        };
        /*isCounterView(): boolean {
            var item = this.items.getItem(0);
            return item instanceof counterSummary;
        }
    
        isTimeSeriesView(): boolean {
            var item = this.items.getItem(0);
            return item instanceof timeSeriesKey;
        }*/
        ctor.prototype.getColumnWidth = function (binding, defaultColumnWidth) {
            if (defaultColumnWidth === void 0) { defaultColumnWidth = 100; }
            var customColumns = this.settings.customColumns();
            var customConfig = customColumns.findConfigFor(binding);
            if (customConfig && customColumns.customMode()) {
                return customConfig.width();
            }
            if (binding === "Id" && defaultColumnWidth > ctor.idColumnWidth) {
                return ctor.idColumnWidth;
            }
            return defaultColumnWidth;
        };
        ctor.prototype.getColumnName = function (binding) {
            if (this.settings.customColumns().hasOverrides()) {
                var customConfig = this.settings.customColumns().findConfigFor(binding);
                if (customConfig) {
                    return customConfig.header();
                }
            }
            else {
                var columns = this.settings.customColumns().columns();
                for (var i = 0; i < columns.length; i++) {
                    if (columns[i].binding() === binding) {
                        return columns[i].header();
                    }
                }
            }
            return binding;
        };
        ctor.prototype.ensureColumnsForRows = function (rows) {
            // Hot path.
            // This is called when items finish loading and are ready for display.
            // Keep allocations to a minimum.
            var columnsNeeded = {};
            var hasOverrides = this.settings.customColumns().hasOverrides();
            if (hasOverrides) {
                var colParams = this.settings.customColumns().columns();
                for (var i = 0, x = colParams.length; i < x; i++) {
                    columnsNeeded[colParams[i].binding()] = null;
                }
            }
            else {
                for (var i = 0, x = rows.length; i < x; i++) {
                    var currentRow = rows[i];
                    var rowProperties = currentRow.getDocumentPropertyNames();
                    for (var j = 0, y = rowProperties.length; j < y; j++) {
                        columnsNeeded[rowProperties[j]] = null;
                    }
                }
            }
            var existingColumns = this.columns();
            var desiredColumns = existingColumns.concat([]);
            for (var i = 0; i < existingColumns.length; i++) {
                var colName = existingColumns[i].binding;
                delete columnsNeeded[colName];
            }
            var idColumn = this.columns.first(function (x) { return x.binding === "Id"; });
            var idColumnExists = idColumn ? 1 : 0;
            var unneededColumns = [];
            ko.utils.arrayForEach(existingColumns, function (col) {
                if (col.binding !== "Id" && col.binding !== "__IsChecked" && !hasOverrides && rows.every(function (row) { return !row.getDocumentPropertyNames().contains(col.binding); })) {
                    unneededColumns.push(col.binding);
                }
            });
            desiredColumns = desiredColumns.filter(function (c) { return !unneededColumns.contains(c.binding); });
            this.settings.customColumns().columns.remove(function (c) { return unneededColumns.contains(c.binding()); });
            var columnsCurrentTotalWidth = 0;
            for (var i = 2; i < existingColumns.length; i++) {
                columnsCurrentTotalWidth += existingColumns[i].width();
            }
            var availiableWidth = this.grid.width() - 200 * idColumnExists - columnsCurrentTotalWidth;
            var freeWidth = availiableWidth;
            var fontSize = parseInt(this.grid.css("font-size"), 10);
            var columnCount = 0;
            for (var binding in columnsNeeded) {
                var curColWidth = (binding.length + 2) * fontSize;
                if (freeWidth - curColWidth < 0) {
                    break;
                }
                freeWidth -= curColWidth;
                columnCount++;
            }
            var freeWidthPerColumn = (freeWidth / (columnCount + 1));
            var firstRow = this.recycleRows().length > 0 ? this.recycleRows()[0] : null;
            for (var binding in columnsNeeded) {
                var curColWidth = (binding.length + 2) * fontSize + freeWidthPerColumn;
                var columnWidth = this.getColumnWidth(binding, curColWidth);
                availiableWidth -= columnWidth;
                if (availiableWidth <= 0) {
                    break;
                }
                var columnName = this.getColumnName(binding);
                // Give priority to any Name column. Put it after the check column (0) and Id (1) columns.
                var newColumn = new column(binding, columnWidth, columnName);
                if ((binding === "Name") && (!this.settings.customColumns().customMode())) {
                    desiredColumns.splice(2, 0, newColumn);
                }
                else {
                    desiredColumns.push(newColumn);
                }
                var curColumnConfig = this.settings.customColumns().findConfigFor(binding);
                if (!curColumnConfig && !!firstRow) {
                    var curColumnTemplate = firstRow.getCellTemplate(binding);
                    var newCustomColumn = new customColumnParams({
                        Binding: binding,
                        Header: binding,
                        Template: curColumnTemplate,
                        DefaultWidth: availiableWidth > 0 ? Math.floor(columnWidth) : 0
                    });
                    if ((binding === "Name") && (!this.settings.customColumns().customMode())) {
                        this.settings.customColumns().columns.splice(0, 0, newCustomColumn);
                    }
                    else {
                        this.settings.customColumns().columns.push(newCustomColumn);
                    }
                }
            }
            // Update the columns only if we have to.
            var columnsHaveChanged = desiredColumns.length !== existingColumns.length || desiredColumns.some(function (newCol, index) { return newCol.binding !== existingColumns[index].binding; });
            if (columnsHaveChanged) {
                this.columns(desiredColumns);
            }
        };
        ctor.prototype.ensureRowsCoverViewport = function () {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            var viewportTop = this.gridViewport.scrollTop();
            var viewportBottom = viewportTop + this.viewportHeight();
            var positionCheck = viewportTop;
            this.firstVisibleRow = null;
            var rowAtPosition = null;
            while (positionCheck < viewportBottom) {
                rowAtPosition = this.findRowAtY(positionCheck);
                if (!rowAtPosition) {
                    // If there's no row at this position, recycle one.
                    rowAtPosition = this.getOffscreenRow(viewportTop, viewportBottom);
                    // Find out what the new top of the row should be.
                    var rowIndex = Math.floor(positionCheck / this.rowHeight);
                    var desiredNewRowY = rowIndex * this.rowHeight;
                    rowAtPosition.top(desiredNewRowY);
                    rowAtPosition.rowIndex(rowIndex);
                    rowAtPosition.resetCells();
                    rowAtPosition.isChecked(this.settings.selectedIndices.indexOf(rowIndex) !== -1);
                }
                if (!this.firstVisibleRow) {
                    this.firstVisibleRow = rowAtPosition;
                }
                positionCheck = rowAtPosition.top() + this.rowHeight;
            }
            this.lastVisibleRow = rowAtPosition;
        };
        ctor.prototype.getOffscreenRow = function (viewportTop, viewportBottom) {
            // This is hot path, called multiple times when scrolling.
            // Keep allocations to a minimum.
            var rows = this.recycleRows();
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var rowTop = row.top();
                var rowBottom = rowTop + this.rowHeight;
                if (rowTop > viewportBottom || rowBottom < viewportTop) {
                    return row;
                }
            }
            throw new Error("Bug: couldn't find an offscreen row to recycle. viewportTop = " + viewportTop.toString() + ", viewportBottom = " + viewportBottom.toString() + ", recycle row count = " + rows.length.toString());
        };
        ctor.prototype.findRowAtY = function (y) {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            var rows = this.recycleRows();
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var rowTop = row.top();
                var rowBottom = rowTop + this.rowHeight;
                if (rowTop <= y && rowBottom > y) {
                    return row;
                }
            }
            return null;
        };
        ctor.prototype.getTemplateFor = function (columnName) {
            if (this.settings.customColumns().hasOverrides()) {
                var customConfig = this.settings.customColumns().findConfigFor(columnName);
                if (customConfig) {
                    return customConfig.template();
                }
            }
            return undefined;
        };
        ctor.prototype.toggleRowChecked = function (row, isShiftSelect) {
            var _this = this;
            if (isShiftSelect === void 0) { isShiftSelect = false; }
            if (this.settings.isAllAutoSelected()) {
                var cachedIndeices = this.items.getCachedIndices(this.settings.selectedIndices());
                this.settings.selectedIndices(cachedIndeices);
                this.recycleRows().forEach(function (r) { return r.isChecked(_this.settings.selectedIndices().contains(r.rowIndex())); });
                this.settings.isAllAutoSelected(false);
                this.settings.isAnyAutoSelected(true);
            }
            var rowIndex = row.rowIndex();
            var isChecked = row.isChecked();
            var firstIndex = this.settings.selectedIndices.first();
            var toggledIndices = isShiftSelect && this.settings.selectedIndices().length > 0 ? this.getRowIndicesRange(firstIndex, rowIndex) : [rowIndex];
            if (isChecked) {
                // Going from unchecked to checked.
                if (this.settings.selectedIndices.indexOf(rowIndex) === -1) {
                    toggledIndices
                        .filter(function (i) { return !_this.settings.selectedIndices.contains(i); })
                        .reverse()
                        .forEach(function (i) { return _this.settings.selectedIndices.unshift(i); });
                }
            }
            else {
                // Going from checked to unchecked.
                this.settings.selectedIndices.removeAll(toggledIndices);
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(_this.settings.selectedIndices().contains(r.rowIndex())); });
        };
        ctor.prototype.selectNone = function () {
            this.settings.selectedIndices([]);
            this.recycleRows().forEach(function (r) { return r.isChecked(false); });
            this.settings.isAnyAutoSelected(false);
            this.settings.isAllAutoSelected(false);
        };
        ctor.prototype.selectAll = function (documentCount) {
            var allIndices = [];
            /*this.settings.itemsSource().totalResultCount()*/
            for (var i = 0; i < documentCount; i++) {
                allIndices.push(i);
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(true); });
            this.settings.selectedIndices(allIndices);
            this.settings.isAnyAutoSelected(false);
            this.settings.isAllAutoSelected(true);
        };
        ctor.prototype.selectSome = function () {
            var allIndices = [];
            var firstVisibleRowNumber = this.firstVisibleRow.rowIndex();
            var lastVisibleRowNumber = this.lastVisibleRow.rowIndex();
            var numOfRowsInUse = this.recycleRows().filter(function (r) { return r.isInUse(); }).length;
            var actualNumberOfVisibleRows = Math.min(lastVisibleRowNumber - firstVisibleRowNumber, numOfRowsInUse);
            for (var i = firstVisibleRowNumber; i < firstVisibleRowNumber + actualNumberOfVisibleRows; i++) {
                allIndices.push(i);
            }
            this.recycleRows().forEach(function (r) { return r.isChecked(allIndices.contains(r.rowIndex())); });
            this.settings.selectedIndices(allIndices);
            this.settings.isAllAutoSelected(false);
        };
        ctor.prototype.getRowIndicesRange = function (firstRowIndex, secondRowIndex) {
            var isCountingDown = firstRowIndex > secondRowIndex;
            var indices = [];
            if (isCountingDown) {
                for (var i = firstRowIndex; i >= secondRowIndex; i--)
                    indices.unshift(i);
            }
            else {
                for (var i = firstRowIndex; i <= secondRowIndex; i++)
                    indices.unshift(i);
            }
            return indices;
        };
        ctor.prototype.editItem = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("EditItem", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.changeCounterValue = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("ChangeCounterValue", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.resetCounter = function () {
            if (this.settings.selectedIndices().length > 0) {
                ko.postbox.publish("ResetCounter", this.settings.selectedIndices()[0]);
            }
        };
        ctor.prototype.copySelectedDocs = function () {
            this.showCopyDocDialog(false);
        };
        ctor.prototype.copySelectedDocIds = function () {
            this.showCopyDocDialog(true);
        };
        ctor.prototype.showCopyDocDialog = function (idsOnly) {
            var selectedDocs = this.getSelectedItems();
            var copyDocumentsVm = new copyDocuments(selectedDocs, this.focusableGridSelector);
            copyDocumentsVm.isCopyingDocs(idsOnly === false);
            app.showDialog(copyDocumentsVm);
        };
        ctor.prototype.getSelectedItems = function (max) {
            if (!this.items || this.settings.selectedIndices().length === 0) {
                return [];
            }
            var sliced = max ? this.settings.selectedIndices.slice(0, max) : null;
            var maxSelectedIndices = sliced || this.settings.selectedIndices();
            return this.items.getCachedItemsAt(maxSelectedIndices);
        };
        ctor.prototype.refreshCollectionData = function () {
            this.settings.itemsSource.valueHasMutated();
            this.items.invalidateCache(); // Causes the cache of items to be discarded.
            this.onGridScrolled(); // Forces a re-fetch of the rows in view.
            this.onWindowHeightChanged();
        };
        ctor.prototype.getNumberOfCachedItems = function () {
            var items = this.items;
            if (!!items) {
                return this.items.itemCount();
            }
            return 0;
        };
        ctor.prototype.deleteSelectedItems = function () {
            var _this = this;
            var items = this.getSelectedItems();
            var deleteDocsVm = new deleteItems(items, this.focusableGridSelector);
            deleteDocsVm.deletionTask.done(function () {
                var deletedDocIndices = items.map(function (d) { return _this.items.indexOf(d); });
                deletedDocIndices.forEach(function (i) { return _this.settings.selectedIndices.remove(i); });
                _this.recycleRows().forEach(function (r) { return r.isChecked(_this.settings.selectedIndices().contains(r.rowIndex())); }); // Update row checked states.
                _this.recycleRows().filter(function (r) { return deletedDocIndices.indexOf(r.rowIndex()) >= 0; }).forEach(function (r) { return r.isInUse(false); });
                _this.items.invalidateCache(); // Causes the cache of items to be discarded.
                _this.onGridScrolled(); // Forces a re-fetch of the rows in view.
                // Forces recalculation of recycled rows, in order to eliminate "duplicate" after delete
                // note: won't run on delete of last document(s) of a collection in order to prevent race condition 
                // with changes api. Now we don't use changes api to update the documents list, so this isn't a problem.
                _this.onWindowHeightChanged();
            });
            app.showDialog(deleteDocsVm);
        };
        ctor.prototype.getDocumentHref = function (documentId) {
            if (typeof documentId == "string") {
                return appUrl.forEditItem(documentId, appUrl.getDatabase(), null, null);
            }
            else {
                return "#";
            }
        };
        ctor.prototype.getGroupHref = function (group) {
            if (typeof group == "string") {
                return appUrl.forCounterStorageCounters(group, appUrl.getCounterStorage());
            }
            else {
                return "#";
            }
        };
        ctor.prototype.selectGroup = function (groupName) {
            ko.postbox.publish("SelectGroup", groupName);
        };
        ctor.prototype.getColumnsNames = function () {
            var row = this.items.getAllCachedItems().first();
            return row ? row.getDocumentPropertyNames() : [];
        };
        ctor.prototype.collectionExists = function (collectionName) {
            var result = this.settings.collections()
                .map(function (c) {
                return collectionName.toLowerCase().substr(0, c.name.length) === c.name.toLowerCase();
            })
                .reduce(function (p, c) { return c || p; }, false);
            return result;
        };
        ctor.prototype.registerColumnResizing = function () {
            var _this = this;
            var resizingColumn = false;
            var startX = 0;
            var startingWidth = 0;
            var columnIndex = 0;
            $(this.settings.gridSelector).on("mousedown.virtualTableColumnResize", ".ko-grid-column-handle", function (e) {
                columnIndex = parseInt($(e.currentTarget).attr("column"));
                startingWidth = parseInt(_this.columns()[columnIndex].width().toString());
                startX = e.pageX;
                resizingColumn = true;
            });
            $(this.settings.gridSelector).on("mouseup.virtualTableColumnResize", "", function (e) {
                resizingColumn = false;
            });
            $(this.settings.gridSelector).on("mousemove.virtualTableColumnResize", "", function (e) {
                if (resizingColumn) {
                    var targetColumnSize = startingWidth + e.pageX - startX;
                    _this.columns()[columnIndex].width(targetColumnSize);
                    // Stop propagation of the event so the text selection doesn't fire up
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if (e.preventDefault)
                        e.preventDefault();
                    e.cancelBubble = true;
                    e.returnValue = false;
                    return false;
                }
            });
        };
        ctor.prototype.unregisterColumnResizing = function () {
            $(this.settings.gridSelector).off("mousedown.virtualTableColumnResize");
            $(this.settings.gridSelector).off("mouseup.virtualTableColumnResize");
            $(this.settings.gridSelector).off("mousemove.virtualTableColumnResize");
        };
        ctor.idColumnWidth = 200;
        return ctor;
    })();
    return ctor;
});
define("text!views/databaseStudioConfig.html", [], function () { return '<div class=\"panel panel-default form-horizontal\" role=\"form\">    <div class=\"panel-heading\">        Studio config        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <div class=\"clearfix\"></div>    </div>        <div class=\"form-group\" style=\"margin-top: 10px\">            <label class=\"col-md-4 control-label\">Environment</label>            <div class=\"col-md-2\">                <div id=\"select-color\" >                    <select class=\"form-control\" data-bind=\"options: environmentColors, value: selectedColor, optionsText: \'name\'\"></select>                </div>            </div>        </div>    </div>'; } )
define("text!views/dialog.html", [], function () { return '<!-- We\'re using custom HTML here because Durandal 2.0 doesn\'t work properly with Bootstrap 3. !--><div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: function () { $data.selectOption(null); }\" data-dismiss=\"modal\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h3 data-bind=\"text: title\"></h3>    </div>    <div class=\"modal-body\">        <p class=\"message\" data-bind=\"text: message\"></p>    </div>    <div class=\"modal-footer\" data-bind=\"foreach: options\">        <button class=\"btn\" data-bind=\"click: function () { $parent.selectOption($data); }, text: $data, css: { \'btn-primary\': $index() === ($parent.options.length -1), autofocus: $index() === ($parent.options.length -1) }\"></button>    </div></div>'; } )
define("text!views/shell.html", [], function () { return '<div><div id=\"shellHeader\" class=\"navbar navbar-fixed-top navbar-default\" role=\"navigation\" data-bind=\"style: {backgroundColor:  selectedColor().backgroundColor}\"><div data-bind=\"text: selectedEnvironmentText(), visible: canShowEnvironmentText()\" id=\"environment-header\">Environment</div><div class=\"container-fluid\"><div class=\"navbar-header\">    <button type=\"button\" class=\"navbar-toggle\" data-toggle=\"collapse\" data-target=\"#topNavBarCollapse\">        <span class=\"sr-only\">Toggle navigation</span>        <span class=\"icon-bar\"></span>        <span class=\"icon-bar\"></span>        <span class=\"icon-bar\"></span>    </button></div><div class=\"collapse navbar-collapse\" id=\"topNavBarCollapse\"><div data-bind=\"visible: appUrls.isAreaActive(\'admin\')\">    <ul class=\"nav navbar-nav\">        <li class=\"navbar-splitbutton active\">            <h3>Manage Your Server</h3>        </li>    </ul></div><div data-bind=\"visible: canShowDatabaseNavbar(), css: { \'menu-item-disabled\': isActiveDatabaseDisabled }\">    <ul class=\"nav navbar-nav\">        <li class=\"navbar-splitbutton\" data-bind=\"css: { active: appUrls.isActive(\'Documents\') }\">            <a data-bind=\"attr: { href: appUrls.documents }\">Documents</a>        </li>        <li class=\"dropdown\" data-bind=\"css: { active: appUrls.isActive(\'Documents\') }\">            <a class=\"dropdown-toggle caret-after\" data-toggle=\"dropdown\">                <b class=\"caret\"></b>            </a>            <ul class=\"dropdown-menu\">                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.documents }\">Documents</a>                </li>                <li data-bind=\"visible: $root.hasReplicationSupport()\">                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.conflicts }\">Conflicts</a>                </li>                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.patch }\">Patch</a>                </li>            </ul>        </li>        <li class=\"navbar-splitbutton\" data-bind=\"css: { active: appUrls.isActive(\'Indexes\') }\">            <a data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>        </li>        <li class=\"dropdown\" data-bind=\"css: { active: appUrls.isActive(\'Indexes\') }\">            <a class=\"dropdown-toggle caret-after\" data-toggle=\"dropdown\">                <b class=\"caret\"></b>            </a>            <ul class=\"dropdown-menu\">                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>                </li>                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.transformers }\">Transformers</a>                </li>            </ul>        </li>        <li class=\"navbar-splitbutton\" data-bind=\"css: { active: appUrls.isActive(\'Query\') }\">            <a data-bind=\"attr: { href: appUrls.query(null) }\">Query</a>        </li>        <li class=\"dropdown\" data-bind=\"css: { active: appUrls.isActive(\'Query\') }\">            <a class=\"dropdown-toggle caret-after\" data-toggle=\"dropdown\">                <b class=\"caret\"></b>            </a>            <ul class=\"dropdown-menu\">                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.query(null) }\">Query</a>                </li>                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.reporting }\">Reporting</a>                </li>                <li>                    <a tabindex=\"-1\" data-bind=\"attr: { href: appUrls.exploration }\">Data exploration</a>                </li>            </ul>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Tasks\') }\">            <a data-bind=\"attr: { href: appUrls.tasks }\">Tasks</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Settings\') }, visible: $root.isSystemConnected\">            <a data-bind=\"attr: { href: appUrls.settings }\">Settings</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Status\') }\">            <a data-bind=\"attr: { href: appUrls.status }\">Status</a>        </li>    </ul>    <ul class=\"nav navbar-nav\">        <li class=\"dropdown\">            <a class=\"dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">                <i class=\"fa fa-plus\"></i> New <b class=\"caret\"></b>            </a>            <ul class=\"dropdown-menu\">                <li>                    <a href=\"#\" data-bind=\"click: newDocument\">                        <i class=\"fa fa-file-o\"></i>                        <span>Document</span>                        <span class=\"text-muted keyboard-shortcut\">Ctrl+Alt+N</span>                    </a>                </li>                <li>                    <a data-bind=\"attr: { href: newIndexUrl }\">Index</a>                </li>                <li>                    <a data-bind=\"attr: { href: newTransformerUrl }\">Transformer</a>                </li>            </ul>        </li>    </ul>    <form class=\"navbar-form pull-left hidden-xs hidden-sm col-sm-2\">        <input id=\"goToDocInput\" title=\"Search for a document (Alt+G)\" accesskey=\"G\"               type=\"text\" class=\"form-control\" placeholder=\"Go to document\" data-bind=\"value: goToDocumentSearch, valueUpdate: \'afterkeydown\', disable: isActiveDatabaseDisabled\" autocomplete=\"off\"/>        <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#goToDocInput\', foreach: goToDocumentSearchResults\">            <li role=\"presentation\" data-bind=\"click: $root.goToDoc.bind($root, $data)\">                <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                    <span class=\"collection-color-strip\" data-bind=\"css: $root.getDocCssClass($data)\"></span>                    <span data-bind=\"text: $data[\'@metadata\'][\'@id\']\"></span>                </a>            </li>        </ul>    </form></div><div data-bind=\"visible: canShowFileSystemNavbar, css: { \'menu-item-disabled\': isActiveFileSystemDisabled }\">    <ul class=\"nav navbar-nav\">        <li data-bind=\"css: { active: appUrls.isActive(\'Files\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemFiles }\">Files</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Search\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemSearch }\">Search</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Synchronization\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemSynchronization }\">Synchronization</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Status\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemStatus }\">Status</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Tasks\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemTasks }\">Tasks</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Settings\') }, visible: canShowFileSystemSettings\">            <a data-bind=\"attr: { href: appUrls.filesystemSettings }\">Settings</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Configuration\') }\">            <a data-bind=\"attr: { href: appUrls.filesystemConfiguration }\">Configurations</a>        </li>    </ul></div><div data-bind=\"visible: canShowCountersNavbar, css: { \'menu-item-disabled\': isActiveCounterStorageDisabled }\">    <ul class=\" nav navbar-nav\">        <li data-bind=\"css: { active: appUrls.isActive(\'Counters\') }\">            <a data-bind=\"attr: { href: appUrls.counterStorageCounters }\">Counters</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Replication\') }\">            <a data-bind=\"attr: { href: appUrls.counterStorageReplication }\">Replication</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Stats\') }\">            <a data-bind=\"attr: { href: appUrls.counterStorageStats }\">Stats</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Configuration\') }\">            <a data-bind=\"attr: { href: appUrls.counterStorageConfiguration }\">Configuration</a>        </li>    </ul></div><div data-bind=\"visible: canShowTimeSeriesNavbar, css: { \'menu-item-disabled\': isActiveTimeSeriesDisabled }\">    <ul class=\" nav navbar-nav\">        <li data-bind=\"css: { active: appUrls.isActive(\'TimeSeries\') }\">            <a data-bind=\"attr: { href: appUrls.timeSeriesType }\">Time Series</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Stats\') }\">            <a data-bind=\"attr: { href: appUrls.timeSeriesStats }\">Stats</a>        </li>        <li data-bind=\"css: { active: appUrls.isActive(\'Configuration\') }\">            <a data-bind=\"attr: { href: appUrls.timeSeriesConfiguration }\">Configuration</a>        </li>    </ul></div><div>    <ul class=\"nav navbar-nav navbar-right hidden-xs hidden-sm databases\">        <li id=\"brand\" class=\"navbar-right\" style=\"padding-left: 5px;\">            <a class=\"navbar-brand\" href=\"http://ravendb.net\" target=\"_blank\">                <img style=\"height: 24px\" src=\"content/images/raven.png\"/>            </a>        </li>        <!-- ko if:showLogOutButton -->        <li id=\"logOut\" class=\"navbar-right\" style=\"padding-left: 5px; padding-top: 5px\">            <button data-bind=\"click: logOut\" class=\"btn btn-danger\"><i class=\"fa fa-sign-out\"></i> Log out</button>        </li>        <!-- /ko -->        <!-- ko if:showContinueTestButton() -->        <li id=\"continueTest\" class=\"navbar-right\" style=\"padding-left: 5px; padding-top: 5px\">            <button data-bind=\"click: continueTest\" class=\"btn btn-danger\"><i class=\"fa fa-play\"></i> Continue test</button>        </li>        <!-- /ko -->        <li class=\"dropdown navbar-right\" data-bind=\"visible: canShowResourcesNavbar()\">            <a id=\"tenantNameLink\" class=\"dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\" data-bind=\"with: lastActivatedResource, css: { \'resource-link-disabled\': $root.listedResources().length == 0 }\">                <span class=\"resource-link\" data-bind=\"style: { maxWidth: $root.maxResourceNameWidth() }\">                                    <span data-bind=\"text: name, attr: { title: name }\"></span>                                </span>                <b class=\"caret resource-caret\" data-bind=\"visible: $root.listedResources().length > 0\"></b>            </a>            <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" data-bind=\"foreach: listedResources, visible: $root.listedResources().length > 0\">                <li>                    <a href=\"#\" data-bind=\"click: $root.selectResource.bind($root, $data)\">                        <span data-bind=\"attr: { class: iconName }\"></span>                        <span data-bind=\"text: name\"></span>                    </a>                </li>            </ul>        </li>        <li id=\"featureName\" class=\"navbar-right\">            <div>                <ul class=\"nav navbar-nav\">                    <li class=\"vertical-navbar-menu-item\">                        <a style=\"padding-right: 0; padding-left: 0; padding-bottom: 0; display: inline-block;\" data-bind=\"attr: { href: getCurrentActiveFeatureHref() }\">                            <span data-bind=\"text: getCurrentActiveFeatureName()\"></span>&nbsp;                            <i class=\"fa fa-caret-right\" data-bind=\"visible: canShowResourcesNavbar\"></i>&nbsp;                        </a>                    </li>                    <li class=\"dropdown vertical-navbar-menu-item\" data-bind=\"visible: appUrls.isAreaActive(\'admin\')()\">                        <a class=\"dropdown-toggle vertical-bottom-container\" data-toggle=\"dropdown\">                            <i class=\"fa fa-caret-down\"></i>                        </a>                        <ul class=\"dropdown-menu\">                            <li>                                <a href=\"#resources\" data-bind=\"click: navigateToResources.bind($root)\">Resources</a>                            </li>                        </ul>                    </li>                </ul>            </div>        </li>        <li class=\"navbar-right\" data-bind=\"visible: recordedErrors().length > 0, click: $root.showErrorsDialog.bind($root)\" title=\"Show errors\" style=\"padding-left: 0;\">            <a href=\"#\" style=\"padding-left: 0; padding-right: 0\"><i class=\"fa fa-warning text-danger\"></i></a>        </li>    </ul></div></div></div></div><!--Alerts container--><div class=\"studio-alerts\" data-bind=\"with: currentAlert\">    <div class=\"alert alert-dismissable fade in\" data-bind=\"attr: { id: id }, css: { \'alert-info\': type === 0, \'alert-success\': type === 1, \'alert-warning\': type === 2, \'alert-danger\': type === 3 || type === 4 }, event: { \'closed.bs.alert\': $root.onAlertHidden.bind($root) }\">        <button type=\"button\" class=\"close\" data-dismiss=\"alert\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <div class=\"alert-text\">            <strong data-bind=\"text: title, attr:{title:title}\"></strong>        </div>        <span data-bind=\"visible: errorMessage, text: errorMessage\"></span>        <a class=\"alert alert-link\" href=\"#\" data-bind=\"visible: details, click: $root.showErrorsDialog.bind($root)\">Details</a>    </div></div><div id=\"splash-spinner\">    <div id=\"page-host\" class=\"page-host\" data-bind=\"router: { cacheViews: false }\"></div></div><footer style=\"z-index: 999\" id=\"shellFooter\">    <div class=\"navbar navbar-fixed-bottom navbar-inverse\">        <div class=\"navbar-inner\">            <ul class=\"nav navbar-nav\" data-bind=\"with: activeDatabase, visible: !isActiveDatabaseDisabled() && canShowDatabaseNavbar()\">                <li class=\"status-link\"><a data-bind=\"attr: { href: $root.appUrls.status }\" class=\"navbar-link\">Status</a></li>                <li data-bind=\"visible: $root.isLoadingStatistics()\" style=\"margin-top: 5px; margin-left: 5px;\">                    <span class=\"navbar-link\">Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                </li>                <!-- ko with: statistics -->                <li><a class=\"navbar-link\" data-bind=\"text: countOfDocumentsText, attr: { href: $parents[1].appUrls.documents }\"></a></li>                <li><a class=\"navbar-link\" data-bind=\"text: countOfFilteredIndexesText, attr: { href: $parents[1].appUrls.indexes, title: \'Total indexes including disabled and abandoned: \' + countOfIndexesText() }\"></a></li>                <li><a class=\"navbar-link\" data-bind=\"text: staleFilteredIndexesCountText, attr: { href: $parents[1].appUrls.status, title: \'Total stale indexes including disabled and abandoned: \' + staleIndexesCountText() }\"></a></li>                <li><a class=\"navbar-link\" data-bind=\"text: errorsCountText, attr: { href: $parents[1].appUrls.indexErrors }\"></a></li>                <!--<li><a class=\"navbar-link\" data-bind=\"text: ApproximateTaskCount + \' tasks\', attr: { href: $parents[1].appUrls.status }\"></a></li>-->                <li class=\"raw-url hidden-sm\" data-bind=\"visible: $root.rawUrlIsVisible\">                    <a class=\"navbar-link\" href=\"#\" target=\'_blank\' data-bind=\"text: $root.currentRawUrl, attr: { href: $root.currentRawUrl, title: $root.currentRawUrl }\" style=\"max-width: 180px; -ms-text-overflow: ellipsis; -o-text-overflow: ellipsis; text-overflow: ellipsis; overflow-x: hidden; white-space: nowrap;\"></a>                </li>                <!-- /ko -->            </ul>            <ul class=\"nav navbar-nav\" data-bind=\"with: activeFilesystem, visible: !isActiveFileSystemDisabled() && canShowFileSystemNavbar()\">                <li class=\"status-link\"><a data-bind=\"attr: { href: $root.appUrls.filesystemStatus }\" class=\"navbar-link\" href=\"#\">Status</a></li>                <li data-bind=\"visible: $root.isLoadingStatistics()\" style=\"margin-top: 5px; margin-left: 5px;\">                    <span class=\"navbar-link\">Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                </li>                <!-- ko with: statistics -->                <li><a data-bind=\"text: fileCountText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <li><a data-bind=\"text: requestsCountText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <li><a data-bind=\"text: meanDurationText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <!-- /ko -->            </ul>            <ul class=\"nav navbar-nav\" data-bind=\"with: activeCounterStorage, visible: !isActiveCounterStorageDisabled() && canShowCountersNavbar()\">                <li class=\"status-link\"><a data-bind=\"attr: { href: $root.appUrls.filesystemStatus }\" class=\"navbar-link\" href=\"#\">Status</a></li>                <li data-bind=\"visible: $root.isLoadingStatistics()\" style=\"margin-top: 5px; margin-left: 5px;\">                    <span class=\"navbar-link\">Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                </li>                <!-- ko with: statistics -->                <li><a data-bind=\"text: counterCountText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <li><a data-bind=\"text: groupCountText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <li><a data-bind=\"text: requestsPerSecondText, attr: { href: $parents[1].appUrls.filesystemFiles }\"></a></li>                <!-- /ko -->            </ul>            <ul class=\"nav navbar-nav\" data-bind=\"with: activeTimeSeries, visible: !isActiveTimeSeriesDisabled() && canShowTimeSeriesNavbar()\">                <li class=\"status-link\"><a data-bind=\"attr: { href: $root.appUrls.filesystemStatus }\" class=\"navbar-link\" href=\"#\">Status</a></li>                <li data-bind=\"visible: $root.isLoadingStatistics()\" style=\"margin-top: 5px; margin-left: 5px;\">                    <span class=\"navbar-link\">Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                </li>                <!-- ko with: statistics -->                <li><a data-bind=\"text: typesCountText\"></a></li>                <li><a data-bind=\"text: keysCountText\"></a></li>                <li><a data-bind=\"text: pointsCountText\"></a></li>                <!-- /ko -->            </ul>            <ul class=\"nav navbar-nav navbar-right\" style=\"margin-right: 10px;\">                <!-- ko with: $root -->                <li data-bind=\"visible: isInCluster()\"><a class=\"navbar-link\" href=\"#\" data-bind=\"click: navigateToClusterSettings\" title=\"This server is part of cluster\"><span class=\"text-danger\"><i class=\"fa fa-cubes\"></i> Cluster</span></a></li>                <li>                    <div class=\"btn-group\" role=\"group\">                        <button type=\"button\" data-bind=\"attr: { class: \'btn btn-default \' + licenseStatus() }, click: $root.showLicenseStatusDialog.bind($root)\">                            <div class=\"roundborder\">                                <i class=\"fa fa-file-text-o\"></i>                            </div>                        </button>                        <button type=\"button\" data-bind=\"attr: { class: \'btn btn-default \' + supportStatus() }, click: $root.showLicenseStatusDialog.bind($root)\">                            <div class=\"roundborder \">                                <i class=\"fa fa-comments-o\"></i>                            </div>                        </button>                    </div>                </li>                <li class=\"server-build-link\" data-bind=\"with: serverBuildVersion\"><a class=\"navbar-link\" data-bind=\"text: \'Server Build #\' + BuildVersion\"></a></li>                <li class=\"client-build-link\" data-bind=\"with: clientBuildVersion\"><a class=\"navbar-link\" data-bind=\"text: \'Client Build #\' + BuildVersion\"></a></li>                <!-- /ko -->            </ul>        </div>    </div></footer></div>'; } )
define("text!views/common/confirmationDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h3 class=\"modal-title\" data-bind=\"text: title\"></h3>    </div>    <div class=\"modal-body\">        <p class=\"confirmationMessage\" data-bind=\"text: confirmationMessage\"></p>    </div>    <div class=\"modal-footer\" data-bind=\"foreach: options\">        <button type=\"button\" class=\"btn\" data-bind=\"text: $data, click: $root.onOptionClicked.bind($root), css: { \'btn-default\': $index() !== $root.options.length - 1, \'btn-primary\': $index() === $root.options.length - 1 }\"></button>    </div></div>'; } )
define("text!views/common/deleteDocuments.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: documents().length === 1\">            <span>You\'re deleting</span>            <strong data-bind=\"with: documents()[0]\">                <span data-bind=\"text: __metadata.id\"></span>            </strong>        </p>        <div data-bind=\"visible: documents().length > 1\">            <span>You\'re deleting</span>            <strong data-bind=\"text: documents().length\"></strong>            <span>docs:</span>            <br />            <ul data-bind=\"foreach: documents\" style=\"max-height: 100px; overflow-y: auto;\">                <li data-bind=\"text: __metadata.id\"></li>            </ul>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteDocs\">Yep, delete</button>    </div></div>'; } )
define("text!views/common/deleteItems.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: items().length === 1\">            <span>You\'re deleting</span>            <strong>                <span data-bind=\"text: getDeletedItemName()\"></span>            </strong>        </p>        <div data-bind=\"visible: items().length > 1\">            <span>You\'re deleting</span>            <strong data-bind=\"text: items().length\"></strong>            <span>items:</span>            <br />            <ul data-bind=\"foreach: items\" style=\"max-height: 100px; overflow-y: auto;\">                <li data-bind=\"text: getId()\"></li>            </ul>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteItems\">Yep, delete</button>    </div></div>'; } )
define("text!views/common/enterApiKey.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Enter API Key</h4>    </div>    <div class=\"modal-body\">        <p>Please enter OAuth API Key that will be used to authenticate with the server.</p>        <form class=\"form-horizontal\" role=\"form\">            <div class=\"form-group\">                <label class=\"col-sm-3 control-label\">Key: </label>                <div class=\"col-sm-7\">                    <input type=\"text\" class=\"form-control \" placeholder=\"api key\" data-bind=\"value: apiKey\" />                </div>            </div>            <div class=\"form-group\">                <div class=\"col-sm-7 col-sm-offset-3\">                    <div class=\"checkbox\">                        <input type=\"checkbox\" class=\"styled\" id=\"remember_me\"                               data-bind=\"checked: saveInLocalStorage\">                        <label for=\"remember_me\">Remember me for this session</label>                    </div>                </div>            </div>        </form>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: ok\">OK</button>    </div></div>'; } )
define("text!views/common/latestBuildReminder.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h3 class=\"modal-title\">Latest Build Reminder</h3>    </div>    <div class=\"modal-body\" style=\"font-size: 1.3em\">        <!-- ko if: !latestServerBuildResult.Exception -->        <div>            A new build (<span data-bind=\"text: latestServerBuildResult.LatestBuild\"></span>) is available for <a href=\"http://ravendb.net/download\" target=\"_blank\">download</a>.        </div>        <!-- /ko -->        <!-- ko if: !!latestServerBuildResult.Exception -->        <div class=\"label label-danger\">An error occured!</div>        <div data-bind=\"text: latestServerBuildResult.Exception\"></div>        <!-- /ko -->        <div class=\"checkbox\">            <input type=\"checkbox\" class=\"styled\" id=\"mute\"                    data-bind=\"checked: mute\">            <label for=\"mute\">Mute for a week</label>        </div>    </div></div>'; } )
define("text!views/common/licensingStatus.html", [], function () { return '<div class=\"messageBox licensing-status\" tabindex=\"-1\" id=\"licenseSupportModal\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Licensing &amp; Support status</h4>    </div>    <div class=\"modal-body\">        <div class=\"row\">            <div class=\"col-sm-6\">                <div class=\"text-center\">                    <div data-bind=\"visible: isHotSpare\">                        <div class=\"roundborder hot-spare\">                            <i class=\"fa fa-clock-o\"></i>                        </div>                        <h3>LICENSE</h3>                        <h4>Hot Spare</h4>                    </div>                    <div data-bind=\"visible: isDevelopmentOnly\">                        <div class=\"roundborder dev-only\">                            <i class=\"fa fa-wrench\"></i>                        </div>                        <h3>LICENSE</h3>                        <h4>Development only</h4>                    </div>                    <div data-bind=\"visible: isNonExpiredCommercial\">                        <div class=\"roundborder commercial\">                            <i class=\"fa fa-file-text-o\"></i>                        </div>                        <h3>LICENSE</h3>                        <h4 data-bind=\"text: licenseStatusText\"></h4>                    </div>                    <div data-bind=\"visible: isExpired\">                        <div class=\"roundborder expired\">                            <i class=\"fa fa-hourglass-half\"></i>                        </div>                        <h3>LICENSE</h3>                        <h4 data-bind=\"text: licenseStatusText\"></h4>                    </div>                    <div data-bind=\"visible: licenseExpiresAt\" class=\"expire-date\">EXPIRES: <span data-bind=\"text: licenseExpiresAt\"></span></div>                </div>            </div>            <div class=\"col-sm-6\">                <div class=\"text-center\">                    <div data-bind=\"visible: supportStatus === \'NoSupport\'\">                        <div class=\"roundborder no-support\">                            <i class=\"fa fa-times\"></i>                        </div>                        <h3>SUPPORT</h3>                        <h4>No support</h4>                    </div>                    <div data-bind=\"visible: supportStatus === \'ProductionSupport\'\">                        <div class=\"roundborder production-support\">                            <i class=\"fa fa-life-ring\"></i>                        </div>                        <h3>SUPPORT</h3>                        <h4>Production support</h4>                    </div>                    <div data-bind=\"visible: supportStatus === \'ProfessionalSupport\'\">                        <div class=\"roundborder professional-support\">                            <i class=\"fa fa-life-ring\"></i>                        </div>                        <h3>SUPPORT</h3>                        <h4>Professional support</h4>                    </div>                    <div data-bind=\"visible: supportStatus === \'PartialSupport\'\">                        <div class=\"roundborder partial-support\">                            <i class=\"fa fa-life-ring\"></i>                        </div>                        <h3>SUPPORT</h3>                        <h4>Partial support</h4>                    </div>                    <div data-bind=\"visible: supportStatus === \'LicenseNotFound\'\">                        <div class=\"roundborder\">                            <i class=\"fa fa-question-circle\"></i>                        </div>                        <h3>SUPPORT</h3>                        <h4>License not found</h4>                    </div>                    <div data-bind=\"visible: supportExpiresAt\" class=\"expire-date\">EXPIRES: <span data-bind=\"text: supportExpiresAt\"></span></div>                </div>            </div>        </div>    </div>    <div class=\"bg-gray padding\">        <div class=\"row\">            <div class=\"col-sm-6\">                <div class=\"text-center\" data-bind=\"visible: isDevelopmentOnly\">                    <div class=\"border-danger padding\">                        <div class=\"icon-xl\"><i class=\"fa fa-exclamation-triangle\"></i></div>                        <h3 class=\"text-danger\">We couldn\'t find a license at <span class=\"path\" data-bind=\"text: licensePath\"></span></h3>                        <p class=\"text-muted\">                            Only Open Source / Development work is permitted<br />                            AGPL license restrictions apply                        </p>                    </div>                </div>                <div class=\"license-table\" data-bind=\"visible: !isDevelopmentOnly\">                    <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"alternate-align\">                        <tbody>                            <tr>                                <td>RAM</td>                                <td><strong><span data-bind=\"html: attrRam\"></span></strong></td>                            </tr>                            <tr>                                <td>CPU cores</td>                                <td><strong>Up to <strong data-bind=\"html: attrCpus\"></strong></strong></td>                            </tr>                            <!-- ko foreach: attributes -->                            <tr>                                <td data-bind=\"text: displayName\"></td>                                <td data-bind=\"html: value\"></td>                            </tr>                            <!-- /ko -->                        </tbody>                    </table>                </div>            </div>            <div class=\"col-sm-6\">                <div class=\"padding border-danger text-center\" data-bind=\"visible: isPartialSupport\">                    <div class=\"icon-xl\"><i class=\"fa fa-server\"></i> <i class=\"fa fa-server\"></i> <i class=\"fa fa-server text-danger\"></i></div>                    <h3 class=\"text-danger\">We have detected that you own more server than support licenses</h3>                    <p class=\"text-muted\">Please contact support to resolve this issue.</p>                    <a href=\"http://ravendb.net/contact\" target=\"_blank\" class=\"btn btn-primary btn-sm\">CONTACT</a>                </div>                <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"support-table\" data-bind=\"visible: isCommunitySupport\">                    <tbody>                        <tr>                            <td class=\"active\"><span class=\"text-muted\">CURRENT</span></td>                            <td colspan=\"2\"></td>                        </tr>                        <tr>                            <th scope=\"col\" class=\"active\" width=\"33.3%\">                                <h4>Community</h4>                                Essential support to get started                            </th>                            <th scope=\"col\" width=\"33.3%\">                                <h4>Professional</h4>                                Priority support for professional projects                            </th>                            <th scope=\"col\">                                <h4>Production</h4>                                Top-tier support for enterprise and mission-critical projects                            </th>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-group\"></i> Support via our comunity forum</td>                            <td colspan=\"2\"><i class=\"fa fa-users\"></i> Access to RavenDB core developers</td>                        </tr>                        <tr>                            <td class=\"active\"></td>                            <td colspan=\"2\"><i class=\"fa fa-phone\"></i> Email & Phone support</td>                        </tr>                        <tr>                            <td class=\"active\"></td>                            <td>                                <i class=\"fa fa-clock-o\"></i>Next day SLA<br>                                <small class=\"text-muted\">Sun-Thu | 8:00-18:00 GMT+2</small>                            </td>                            <td><i class=\"fa fa-clock-o\"></i>2 hour SLA <small class=\"text-muted\">We\'re there for you 24/7</small></td>                        </tr>                        <tr>                            <td class=\"active\"><a href=\"https://groups.google.com/forum/#!forum/ravendb\" target=\"_blank\" class=\"btn btn-info btn-block\">ACCESS</a></td>                            <td colspan=\"2\"><a href=\"http://ravendb.net/support\" target=\"_blank\" class=\"btn btn-primary btn-block\">UPGRADE</a></td>                        </tr>                    </tbody>                </table>                <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"support-table\" data-bind=\"visible: isProfessionalSupport\">                    <tbody>                        <tr>                            <td class=\"active\"><span class=\"text-muted\">CURRENT</span></td>                            <td></td>                        </tr>                        <tr>                            <th scope=\"col\" width=\"50%\">                                <h4>Professional</h4>                                Priority support for professional projects                            </th>                            <th scope=\"col\">                                <h4>Production</h4>                                Top-tier support for enterprise and mission-critical projects                            </th>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-users\"></i> Access to RavenDB core developers</td>                            <td><i class=\"fa fa-users\"></i> Access to RavenDB core developers</td>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-phone\"></i> Email & Phone support</td>                            <td><i class=\"fa fa-phone\"></i> Email & Phone support</td>                        </tr>                        <tr>                            <td class=\"active\">                                <i class=\"fa fa-clock-o\"></i>Next day SLA<br>                                <small class=\"text-muted\">Sun-Thu | 8:00-18:00 GMT+2</small>                            </td>                            <td><i class=\"fa fa-clock-o\"></i>2 hour SLA <small class=\"text-muted\">We\'re there for you 24/7</small></td>                        </tr>                        <tr>                            <td class=\"active\"><a href=\"http://ravendb.net/support/supportrequest\" target=\"_blank\" class=\"btn btn-primary btn-block\">SEND TICKET</a></td>                            <td><a href=\"http://ravendb.net/support\" target=\"_blank\" class=\"btn btn-primary btn-block\">UPGRADE</a></td>                        </tr>                    </tbody>                </table>                <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"support-table\" data-bind=\"visible: isProductionSupport\">                    <tbody>                        <tr>                            <td class=\"active\"><span class=\"text-muted\">CURRENT</span></td>                        </tr>                        <tr>                            <th class=\"active\" scope=\"col\">                                <h4>Production</h4>                                Top-tier support for enterprise and mission-critical projects                            </th>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-users\"></i> Access to RavenDB core developers</td>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-phone\"></i> Email & Phone support</td>                        </tr>                        <tr>                            <td class=\"active\"><i class=\"fa fa-clock-o\"></i>2 hour SLA <small class=\"text-muted\">We\'re there for you 24/7</small></td>                        </tr>                        <tr>                            <td class=\"active\"><a href=\"http://ravendb.net/support/supportrequest\" target=\"_blank\" class=\"btn btn-primary btn-block\">SEND TICKET</a></td>                        </tr>                    </tbody>                </table>            </div>        </div>    </div></div>'; } )
define("text!views/common/recentErrors.html", [], function () { return '<div id=\"recentErrorsDialog\" class=\"messageBox\" tabindex=\"-1\" data-bind=\"style: {width: width()+\'px\', height: height()+\'px\', \'marginLeft\': left()+\'px\', \'marginTop\': top()+\'px\'}\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Recent Errors</h4>    </div>    <div class=\"modal-body\" data-bind=\"style: { height: bodyHeight() + \'px\' }\">        <div class=\"panel-group error-details-accordion\" data-bind=\"foreach: errors\">            <div class=\"panel\" data-bind=\"css: { \'panel-danger\': $data.type == $root.getDangerAlertType(), \'panel-warning\': $data.type == $root.getWarningAlertType() }\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#errorDetailsCollapse\' + $index() }\">                            <strong data-bind=\"text: title\"></strong>                            <span data-bind=\"visible: errorMessage, text: errorMessage, attr: { title: errorMessage }\"></span>                        </a>                    </h4>                </div>                <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'errorDetailsCollapse\' + $index() }\">                    <div class=\"panel-body\">                        <pre class=\"error-details\" data-bind=\"visible: !!$root.getErrorDetails($data), text: $root.getErrorDetails($data)\"></pre>                        <pre class=\"http-status\" data-bind=\"visible: !!httpStatusText, text: httpStatusText\"></pre>                                                <button class=\"btn btn-default\" data-bind=\"visible: !!buttonName && !!onClickAction, text: buttonName, click: onClickAction\"></button>                    </div>                </div>            </div>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: clear\">Clear</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: close\">Close</button>        <a class=\"pull-left\" href=\"http://issues.hibernatingrhinos.com/issues/RavenDB?#newissue=yes\" target=\"_blank\">Report a defect</a>        <span class=\"dialogResizer\"></span>    </div></div>'; } )
define("text!views/common/selectColumns.html", [], function () { return '<form id=\"selectColumnsForm\" data-bind=\"submit: changeCurrentColumns\">    <div class=\"messageBox\" tabindex=\"-1\" style=\"width: 715px;\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Choose columns</h4>        </div>        <div class=\"modal-body\">            <div class=\"row\">                <div class=\"form-horizontal\" role=\"form\">                    <div class=\"form-group\">                        <label for=\"inputEmail3\" class=\"col-sm-2 control-label\">                            Columns                        </label>                        <div class=\"col-sm-10\">                            <div class=\"btn-group\">                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: !customColumns.customMode()\">Automatic</span>                                    <span data-bind=\"visible: customColumns.customMode\">Custom</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\">                                    <li><a href=\"#\" data-bind=\"click: customScheme.bind($data, false)\">Automatic</a></li>                                    <li><a href=\"#\" data-bind=\"click: customScheme.bind($data, true)\">Custom</a></li>                                </ul>                            </div>                            <p class=\"help-block\" data-bind=\"visible: !customColumns.customMode()\">                                <i class=\"fa fa-info-circle text-info\"></i> RavenDB will choose columns based on visible documents                            </p>                        </div>                    </div>                </div>            </div>            <table class=\"table table-hover\" data-bind=\"visible: customColumns.customMode(), css: { \'custom-table\': isScrollNeeded }\">                <thead>                    <tr style=\"height: 39px\">                        <th class=\"col-md-3\">Binding</th>                        <th class=\"col-md-3\">Title</th>                        <th class=\"col-md-2\">Default Width</th>                        <th class=\"col-md-2\">Options</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: customColumns.columns, css: { \'custom-table-tbody\': isScrollNeeded }\">                    <tr>                        <td class=\"col-md-3\">                            <div style=\"position: relative\">                                <input class=\"form-control\" required type=\"text\" data-bind=\"customValidity: bindingCustomValidity, attr: { \'id\': $root.generateBindingInputId($index()) }, value: binding, valueUpdate: \'afterkeydown\', event: { keydown: $root.consumeUpDownArrowKeys.bind($root), click: $root.consumeClick.bind($root) }\" autocomplete=\"off\" />                                <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#\' + $root.generateBindingInputId($index()), foreach: $root.autoCompleteResults\">                                    <li role=\"presentation\" data-bind=\"click: $root.completeTheWord.bind($root)\">                                        <a href=\"#\" data-bind=\"click: $root.completeTheWord.bind($root)\">                                            <span data-bind=\"text: $data\"></span>                                        </a>                                    </li>                                </ul>                            </div>                        </td>                        <td class=\"col-md-3\"><input class=\"form-control\" required type=\"text\" data-bind=\"value: header, valueUpdate: \'afterkeydown\'\" /></td>                        <td class=\"col-md-2\"><input class=\"form-control\" required type=\"number\" data-bind=\"value: width, valueUpdate: \'afterkeydown\'\" /></td>                        <td class=\"col-md-2\">                            <div class=\"btn-group\">                                <button data-bind=\"click: $root.moveUp.bind($root, $data), disable: ($index() === 0)\" type=\"button\" class=\"btn btn-default\"><i class=\"fa fa-arrow-up\"></i></button>                                <button data-bind=\"click: $root.moveDown.bind($root, $data), disable: ($index() === $root.customColumns.columns().length - 1)\" type=\"button\" class=\"btn btn-default\"><i class=\"fa fa-arrow-down\"></i></button>                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.deleteRow.bind($root, $data)\"><i class=\"fa fa-trash-o\"></i></button>                            </div>                        </td>                    </tr>                </tbody>            </table>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\">Ok</button>            <button type=\"button\" class=\"btn pull-left btn-default\" data-bind=\"click: insertNewRow, visible: customColumns.customMode()\">                <i class=\"fa fa-plus\"></i> New column            </button>            <button type=\"button\" class=\"btn pull-left btn-default\" data-bind=\"click: saveAsDefault\">                <i class=\"fa fa-save\"></i> Save as default            </button>        </div>    </div></form>'; } )
define("text!views/common/selectCsvColumns.html", [], function () { return '<form id=\"selectCsvColumnsForm\" data-bind=\"submit: exportCsv\">    <div class=\"messageBox\" tabindex=\"-1\" style=\"width: 715px;\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Choose columns for CSV export</h4>        </div>        <div class=\"modal-body\">            <table class=\"table table-hover\" data-bind=\"css: { \'custom-table\': isScrollNeeded }\">                <thead>                    <tr style=\"height: 39px\">                        <th class=\"col-md-8\">Name</th>                        <th class=\"col-md-2\">Options</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: csvColumns, css: { \'custom-table-tbody\': isScrollNeeded }\">                    <tr>                        <td class=\"col-md-8\">                            <div style=\"position: relative\">                                <input class=\"form-control\" required type=\"text\" data-bind=\"attr: { \'id\': $root.generateBindingInputId($index()) }, value: name, valueUpdate: \'afterkeydown\', event: { keydown: $root.consumeUpDownArrowKeys.bind($root), click: $root.consumeClick.bind($root) }\" autocomplete=\"off\" />                                <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#\' + $root.generateBindingInputId($index()), foreach: $root.autoCompleteResults\">                                    <li role=\"presentation\" data-bind=\"click: $root.completeTheWord.bind($root)\">                                        <a href=\"#\" data-bind=\"click: $root.completeTheWord.bind($root)\">                                            <span data-bind=\"text: $data\"></span>                                        </a>                                    </li>                                </ul>                            </div>                        </td>                        <td class=\"col-md-2\">                            <div class=\"btn-group\">                                <button data-bind=\"click: $root.moveUp.bind($root, $data), disable: ($index() === 0)\" type=\"button\" class=\"btn btn-default\"><i class=\"fa fa-arrow-up\"></i></button>                                <button data-bind=\"click: $root.moveDown.bind($root, $data), disable: ($index() === $root.csvColumns().length - 1)\" type=\"button\" class=\"btn btn-default\"><i class=\"fa fa-arrow-down\"></i></button>                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.deleteRow.bind($root, $data)\"><i class=\"fa fa-trash-o\"></i></button>                            </div>                        </td>                    </tr>                </tbody>            </table>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: $root.csvColumns().length > 0\">Export</button>            <button type=\"button\" class=\"btn pull-left btn-default\" data-bind=\"click: insertNewRow\">                <i class=\"fa fa-plus\"></i> New column            </button>        </div>    </div></form>'; } )
define("text!views/common/showDataDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\"  data-bind=\"style: { width: width() + \'px\', height: height() + \'px\', \'marginLeft\': left() + \'px\', \'marginTop\': top() + \'px\' }\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close glyphicon glyphicon-remove remove-icon\" data-bind=\"click: close\" aria-hidden=\"true\"></button>        <h4 class=\"modal-title\" data-bind=\"text: title\"></h4>    </div>    <div class=\"modal-body\">        <h4><strong>CTRL+C, </strong>to copy</h4>        <div>            <textarea data-bind=\"text: inputData, event: { focus: function () { $(this).select(); } },style : { height: (bodyHeight() - 110) + \'px\' } \" id=\"inputData\" rows=\"13\" style=\"width: 100%; resize: none\" class=\"form-control\" autofocus=\"autofocus\"></textarea>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: close\">Close</button>        <span class=\"dialogResizer\"></span>    </div></div>'; } )
define("text!views/common/upgrade.html", [], function () { return '<div id=\"upgradeContainer\">    <div class=\"text-center\">        <div data-bind=\"visible: inProgress\" class=\"alert alert-warning\" role=\"alert\">            <br /><br />            <h2>Opening database...</h2>            <h4>The database is currently being loaded.</h4>            <br /><br />            <i class=\"fa fa-spin fa-5x fa-cog\"></i>        </div>                <div data-bind=\"visible: !inProgress()\" class=\"alert alert-info\" role=\"alert\">            <br /><br />            <h2>Database has been loaded</h2>            <br /><br />            <i class=\"fa fa-check fa-5x\"></i>        </div>    </div></div>'; } )
define("text!views/common/viewSystemDatabaseConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Are you sure?</h4>    </div>    <div class=\"modal-body\">        <strong><span data-bind=\"text: confirmationMessage\"></span></strong>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        <button type=\"button\" class=\"btn btn-danger\" autofocus data-bind=\"click: viewSystemDatabase\">OK</button>    </div></div>'; } )
define("text!views/counter/counters.html", [], function () { return '<div class=\"documents-page no-outline\" tabindex=\"0\">    <div id=\"documents-page-container\" class=\"col-md-2\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" style=\"padding-left: 0;\">        <ul class=\"nav nav-pills nav-stacked document-collections has-show-all-items\" data-bind=\"template: { foreach: groups, afterAdd: showGroupElement, beforeRemove: hideGroupElement }\">            <li data-bind=\"event: { mousedown: $parent.selectGroup.bind($parent) }, css: { active: $data === $parent.selectedGroup() }, visible: countersCount() > 0 || isAllGroupsGroup\">                <a href=\"javascript: void(0)\">                    <div class=\"collection-name\">                        <div class=\"collection-text pull-left collection-color-strip\" data-bind=\"css: colorClass\"></div>                        <span class=\"collection-text pull-left collection-name-part\" data-bind=\" text: name, attr: { title: name }\"></span>                        <span class=\"collection-text pull-left text-muted\" data-bind=\"visible: !isAllGroupsGroup, css: { \'text-muted\': $data !== $parent.selectedGroup() }, text:  \'&nbsp;(\' + countersCountWithThousandsSeparator() + \')\', updateHighlighting: countersCount()\"></span>                    </div>                </a>            </li>        </ul>    </div>    <div class=\"col-md-10\">        <div class=\"btn-toolbar\" role=\"toolbar\">            <!-- TODO: create a page in documentation -->            <!--<a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>-->            <!-- TODO: replace with new checkboxTriple format -->            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-link\" style=\"margin-left: 10px;\" data-bind=\"click: toggleSelectAll, visible: hasCounters\" title=\"Select all or none\">                    <img data-bind=\"checkboxTriple: countersSelection\"/>                </button>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-primary\" accesskey=\"n\" data-bind=\"click: newCounter\" title=\"Create a new counter (Alt+N)\">                    <i class=\"fa fa-plus\"></i> New counter                </button>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refresh, disable: showLoadingIndicator() || isInitialized() == false\" title=\"Refresh from server\">                    <i class=\"fa fa-refresh\"></i>                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyCountersSelected\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: edit, enable: selectedCounterIndices().length == 1\" title=\"Edit Counter\">                    <i class=\"fa fa-pencil\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: change, enable: selectedCounterIndices().length == 1\" title=\"Change Counter Value\">                    <i class=\"fa fa-wrench\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: reset, enable: selectedCounterIndices().length == 1\" title=\"Reset Counter\">                    <i class=\"fa fa-recycle\"></i>                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyCountersSelected\">                <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteSelectedCounters\" title=\"Delete Counter\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>            <div class=\"btn-group\">                <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicatorThrottled\"><i class=\"fa fa-spinner fa-spin fa-2x\"></i>                </div>            </div>        </div>        <div class=\"documents-select\" data-bind=\"with: selectedGroup, visible: hasAnyCountersSelected() && !hasAllCountersSelected() && isAnyCountersAutoSelected()\">            <i class=\"fa fa-info-circle\"></i> You have selected <strong data-bind=\"text: $root.selectedCounterIndices().length\"></strong> <span data-bind=\"text: $root.selectedCountersText\"></span>.            <span class=\"select-link\" data-bind=\"click: $root.selectAll.bind($root)\">                Select all <strong data-bind=\"text: countersCount\"></strong> counters in <span data-bind=\"text: name\"></span>            </span>        </div>        <div class=\"documents-select\" data-bind=\"with: selectedGroup, visible: hasAllCountersSelected() && isAllCountersAutoSelected()\">            You have selected <strong data-bind=\"text: $root.selectedCounterIndices().length\"></strong> <span data-bind=\"text: $root.selectedCountersText\"></span> in            <span data-bind=\"text: name\"></span>.            <span class=\"select-link\" data-bind=\"click: $root.selectNone.bind($root)\">                Clear selection            </span>        </div>        <div id=\"countersGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: currentGroupPagedItems, selectedIndices: selectedCounterIndices, gridSelector: \'#countersGrid\', collections: groups, rowsAreLoading: showLoadingIndicator, isAnyAutoSelected: isAnyCountersAutoSelected, isAllAutoSelected: isAllCountersAutoSelected, contextMenuOptions: [\'ChangeValue\', \'Reset\', \'Delete\'], showIds: false, noResultsMessage: \'No counters were found!\', viewType: viewType, isCounterAllGroupsGroup: isAllGroupsGroup }\"></div>    </div></div>'; } )
define("text!views/counter/counterStorageConfiguration.html", [], function () { return '<!DOCTYPE html><html xmlns=\"http://www.w3.org/1999/xhtml\"><head>    <title></title></head><body></body></html>'; } )
define("text!views/counter/counterStorageCounters.html", [], function () { return '<div class=\"documents-page\">    <div class=\"col-md-2\">        <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: counterGroups\">            <li data-bind=\"click: $root.selectGroup.bind($root, $data), css: { active: $root.selectedCounterGroup() === $data }\">                <a href=\"javascript:void(0)\">                    <span data-bind=\"text: name\"></span>                    <span data-bind=\"text: \'(\' + numOfCounters() + \')\', css: { \'text-muted\': $data !== $parent.selectedCounterGroup() }\"></span>                </a>            </li>        </ul>    </div>    <div class=\"col-md-10\" data-bind=\"with: selectedCounterGroup\">        <div class=\"btn btn-group\">            <button class=\"btn btn-primary\" data-bind=\"click:function(){$root.addOrEditCounter();}\">Add New Counter</button>        </div>        <div class=\"panel-group\" id=\"accordion\" data-bind=\"foreach: counters\">            <div class=\"panel panel-default\" style=\"padding-top: 0\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title md-col-3\" style=\"padding-top: 10px; display:inline-block\">                        <a data-bind=\"text: id, attr: { href: \'#counterCollapse\' + $index() }\"></a>                    </h4>                    <button class=\"btn btn-primary md-col-3\" data-bind=\"click:function(){$root.addOrEditCounter($data);}\">Edit Counter</button>                    <button class=\"btn btn-primary md-col-3\" data-bind=\"click:function(){$root.resetCounter($data);}\">Reset Counter</button>                </div>                <div class=\"panel-collapse\" data-bind=\"attr:{id:\'counterCollapse\' + $index()}\">                    <div class=\"panel panel-default\">                        <div class=\"panel-body\">                            <div> <strong>Overall Total: </strong> <span data-bind=\"text:overallTotal\"></span></div>                            <div> <strong>Group: </strong> <span data-bind=\"text:group\"></span></div>                            <div> <strong>Servers: </strong></div>                            <ul data-bind=\"foreach: servers\">                                <li>                                    <span class=\"col-md-4\"> <strong>Server URL: </strong><span data-bind=\"text:name()\"></span></span>                                    <span class=\"col-md-4\"> <strong>Positive: </strong><span data-bind=\"text:posCount()\"></span></span>                                    <span class=\"col-md-4\"> <strong>Negative: </strong><span data-bind=\"text:negCount()\"></span></span>                                </li>                            </ul>                        </div>                    </div>                </div>            </div>        </div>    </div>    <div id=\"countersGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: currentCountersPagedItems, selectedIndices: selectedCountersIndices, gridSelector: \'#countersGrid\' }\"></div></div>'; } )
define("text!views/counter/counterStorageReplication.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                Counter Storages            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeCounterStorage\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.counterStorageCounters }\"></a>    </li>    <li class=\"active\">        <span>Replication</span>    </li></ul><div class=\"settings-container row\">    <form data-bind=\"submit: $root.saveChanges\" class=\"form-horizontal\" role=\"form\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <div class=\"pull-left panel-title\">Replication Destinations</div>                <div class=\"pull-right\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewDestination\" title=\"Add a replication destination (Alt+N)\" accesskey=\"N\">                        <i class=\"fa fa-plus\"></i> Add destination                    </button>                    <!-- ko if: true -->                         <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refreshFromServer\" title=\"Refresh from server, discarding any unsaved changes\" >                            <i class=\"fa fa-refresh\"></i>                        </button>                    <!-- /ko -->                     <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save replication destinations (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                </div>                <div class=\"clearfix\"></div>            </div>            <div class=\"panel-body\" data-bind=\"with: replicationsSetup\">                <div class=\"alert alert-warning\" data-bind=\"visible: destinations().length === 0\">                    You\'ve got no replication destinations set up.                </div>                <div data-bind=\"foreach: destinations\">                    <div class=\"panel replication-destination-card\" data-bind=\"css: { \'panel-danger\': !isValid(), \'panel-info\': isValid() && !disabled(), \'panel-warning\': isValid() && disabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span data-bind=\"text: name\"></span>                                <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\" data-bind=\"click: $root.removeDestination.bind($root)\" title=\"Remove this replication destination\"><i class=\"fa fa-times\"></i></button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">&nbsp;</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\" data-toggle=\"buttons\">                                        <label class=\"btn btn-primary\" data-bind=\"click: enable, css: { active: !disabled() }\">                                            <input type=\"radio\" name=\"options\"> Enabled                                        </label>                                        <label class=\"btn btn-primary\" data-bind=\"click: disable, css: { active: disabled }\">                                            <input type=\"radio\" name=\"options\"> Disabled                                        </label>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Url</label>                                <div class=\"col-sm-7\">                                    <input required type=\"url\" class=\"form-control\" data-bind=\"value: serverUrl, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Counter Storage</label>                                <div class=\"col-sm-7\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: counterStorageName, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Credentials</label>                                <div class=\"col-sm-7\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleUserCredentials, css: {active:isUserCredentials}\">User</button>                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleApiKeyCredentials, css: {active:isApiKeyCredentials}\">ApiKey</button>                                </div>                            </div>                            <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                                <div class=\"form-group\">                                    <label class=\"col-sm-4 control-label\">User Name</label>                                    <div class=\"col-sm-7\">                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, valueUpdate: \'afterkeydown\'\" />                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-sm-4 control-label\">Password</label>                                    <div class=\"col-sm-7\">                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, valueUpdate: \'afterkeydown\'\" />                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-sm-4 control-label\">Domain</label>                                    <div class=\"col-sm-7\">                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: domain, valueUpdate: \'afterkeydown\'\" />                                    </div>                                </div>                            </div>                            <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                                <div class=\"form-group\">                                    <label class=\"col-sm-4 control-label\">API Key</label>                                    <div class=\"col-sm-7\">                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, valueUpdate: \'afterkeydown\'\" />                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/counter/counterStorageStats.html", [], function () { return '<!DOCTYPE html><html xmlns=\"http://www.w3.org/1999/xhtml\"><head>    <title></title></head><body></body></html>'; } )
define("text!views/counter/deleteCounterStorageConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>        <h4 class=\"modal-title\">Delete Counter Storage?</h4>    </div>    <div class=\"modal-body\">        <p>Delete options:</p>        <div class=\"btn-group\" data-toggle=\"buttons\">            <label class=\"btn btn-primary active\" title=\"Keep the physical counter storage files on disk\" data-bind=\"click: keepFiles\">                <input type=\"radio\" name=\"options\" id=\"option1\"> Keep files            </label>            <label class=\"btn btn-primary\" title=\"Delete all files from disk which related to this counter storage\" data-bind=\"click: deleteEverything\">                <input type=\"radio\" name=\"options\" id=\"option2\"> Delete everything            </label>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteCounterStorage\">Yep, delete</button>    </div></div>'; } )
define("text!views/counter/deleteGroup.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\" data-bind=\"with: group\">        <span>You\'re deleting </span>        <strong data-bind=\"text: name\"></strong>        <strong data-bind=\"visible: countersCount() > 1, text: \'(\' + countersCount() + \' counters)\'\"></strong>        <strong data-bind=\"visible: countersCount() == 1, text: \'(\' + countersCount() + \' counter)\'\"></strong>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteGroup\">Yep, delete</button>    </div></div>'; } )
define("text!views/counter/editCounter.html", [], function () { return '<div id=\"editCounterContainer\" tabindex=\"-1\">    <!-- We use a container with a tab index so that we can capture keyboard events, such as CTRL+S. -->    <ul class=\"breadcrumb\">        <li>            <h3>                <a href=\"#resources\">                    Counters                </a>            </h3>        </li>        <li>            <a href=\"#\" data-bind=\"text: groupName, attr: { href: groupLink }\"></a>        </li>        <li class=\"active\">            <span data-bind=\"text: counterName\"></span>        </li>    </ul>    <div class=\"btn-toolbar\">        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refresh\" title=\"Refresh (Alt+R)\" accesskey=\"R\"><i class=\"fa fa-refresh\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: change\" title=\"Change Counter Value\"><i class=\"fa fa-wrench\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: reset\" title=\"Reset Counter\"><i class=\"fa fa-recycle\"></i></button>            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteCounter\" title=\"Delete Counter (Alt+Shift+Del)\"><i class=\"fa fa-trash-o\"></i></button>        </div>    </div>    <br/>    <div class=\"row\">        <div class=\"col-md-6\">            <div class=\"col-md-12\" data-bind=\"with: counter\">                <h4><strong>Summary</strong></h4>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Counter Name</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: $root.counterName\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Group Name</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: $root.groupName\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Total</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: total\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Local Server ID</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: localServerId\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Last Updated by Server</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: lastUpdateByServer\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Number of Servers</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: numOfServers\"></span>                    </div>                </div>            </div>            <div class=\"col-md-12\">                <hr/>            </div>            <div class=\"col-md-12\" data-bind=\"with: counter\">                <h4><strong>Values by Server</strong></h4>                <table class=\"table table-striped\">                    <thead>                    <tr>                        <th>Server ID</th>                        <th>Value</th>                        <th>Etag</th>                    </tr>                    </thead>                    <tbody data-bind=\"foreach: serverValues\">                    <tr>                        <td data-bind=\"text: serverId\"></td>                        <td data-bind=\"text: value\"></td>                        <td data-bind=\"text: etag\"></td>                    </tr>                    </tbody>                </table>            </div>        </div>        <div class=\"form-group panel panel-default col-md-4 counter-recent side-panel\" data-bind=\"visible: topRecentCounters().length > 0\">            <div class=\"col-md-12 counter-recent-internal\">                <h4><strong>Recent Counters</strong></h4>                <hr />                <div class=\"counters-links-container\" data-bind=\"foreach: topRecentCounters\">                    <a class=\"counter-link\" data-bind=\"attr: { href: counterUrl }\">                        <span data-bind=\"text: groupName\"></span>                        <span style=\"color: green; font-weight: bold\"> | </span>                        <span data-bind=\"text: counterName\"></span>                    </a>                </div>            </div>        </div>    </div>    </div>'; } )
define("text!views/counter/editCounterDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 52%\">    <form data-bind=\"submit: nextOrCreate\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>            <h4 class=\"modal-title\" data-bind=\"text: (isNew() ? \'Create \' : \'Update \' ) + \'Counter\'\"></h4>        </div>        <div class=\"modal-body\">            <div class=\"row\">                <div data-bind=\"with: editedCounter\">                    <div class=\"col-md-12\" data-bind=\"visible: isNew() == false\">                        <label class=\"col-md-4\" for=\"total\">Current Value:</label>                        <div class=\"col-md-7\">                            <span id=\"total\" data-bind=\"text: getValue()\"></span>                        </div>                    </div>                    <div class=\"col-md-12\">                        <label class=\"col-md-4\" for=\"group\">Group:</label>                        <div class=\"col-md-7\">                            <!-- ko if: isNew() -->                            <input required type=\"text\" class=\"form-control\" id=\"group\" placeholder=\"Group Name\" data-bind=\"value: group, customValidity: groupCustomValidityError, valueUpdate: \'afterkeydown\'\">                            <!-- /ko -->                            <!-- ko ifnot: isNew() -->                            <span data-bind=\"text: group\"></span>                            <!-- /ko -->                        </div>                    </div>                    <div class=\"col-md-12\">                        <label class=\"col-md-4\" for=\"counterName\">Counter Name:</label>                        <div class=\"col-md-7\">                            <!-- ko if: isNew() -->                            <input required type=\"text\" class=\"form-control\" id=\"counterName\" placeholder=\"Counter Name\" data-bind=\"value: counterName, customValidity: counterNameCustomValidityError, valueUpdate: \'afterkeydown\'\">                            <!-- /ko -->                            <!-- ko ifnot: isNew() -->                            <span data-bind=\"text: counterName\"></span>                            <!-- /ko -->                        </div>                    </div>                    <div class=\"col-md-12\">                        <label class=\"col-md-4\" for=\"counterDelta\" data-bind=\"text: isNew() ? \'Initial Value\' : \'Change\'\"></label>                        <div class=\"col-md-7\" style=\"display: inline-block\">                            <input required type=\"number\" class=\"form-control\" id=\"counterDelta\" placeholder=\"Counter Delta\" data-bind=\"value: delta, customValidity: deltaCustomValidityError, valueUpdate: \'afterkeydown\'\">                        </div>                    </div>                    <div class=\"col-md-12\" data-bind=\"visible: isNew()\">                        *Using an existing group and counter name will update its value                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"text: isNew() ? \'Create\' : \'Update\'\"></button>        </div>    </form></div>'; } )
define("text!views/counter/editCounterStorage.html", [], function () { return '<!DOCTYPE html><html xmlns=\"http://www.w3.org/1999/xhtml\"><head>    <title></title></head><body></body></html>'; } )
define("text!views/database/conflicts/conflicts.html", [], function () { return '<div class=\"conflicts-page\">    <div class=\"row\">        <div class=\"btn-toolbar\">            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-danger dropdown-toggle\" data-toggle=\"dropdown\" data-trigger=\"hover\" data-placement=\"bottom\" data-bind=\"enable: hasAnyConflict()\">                    Resolve all conflicts <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li><a href=\"#\" data-bind=\"click: resolveToLocal\">Resolve to local <span data-bind=\"visible: serverConflictResolution() === \'ResolveToLocal\'\">(default)</span></a></li>                    <li><a href=\"#\" data-bind=\"click: resolveToNewestRemote\">Resolve to newest remote <span data-bind=\"visible: serverConflictResolution() === \'ResolveToRemote\'\">(default)</span></li>                    <li><a href=\"#\" data-bind=\"click: resolveToLatest\">Resolve to latest <span data-bind=\"visible: serverConflictResolution() === \'ResolveToLatest\'\">(default)</span></a></li>                </ul>            </div>        </div>    </div>    <div class=\"row\">        <div class=\"col-md-12\">            <div id=\"conflictsGrid\" data-bind=\"widget: {                kind: \'virtualTable\', itemsSource: currentConflictsPagedItems,                selectedIndices: selectedDocumentIndices, gridSelector: \'#conflictsGrid\',                showCheckboxes: false, useContextMenu: false,                customColumns: currentColumns,                noResultsMessage: \'No conflicts found\'            }\"></div>        </div>    </div></div><script type=\"text/html\" id=\"versions-template\">    <span data-bind=\"foreach: { data: $data, as: \'version\' }\">        <a href=\"#\" data-bind=\"attr: { href: $root.getUrlForConflict(version), title: \'Document Id: \' + version.id + \' Server Id: \' + $root.getServerUrlForVersion(version) }\">            <span data-bind=\"text: ($index() + 1) + $root.getTextForVersion(version)\"></span>        </a>    </span></script>'; } )
define("text!views/database/documents/copyDocuments.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\"  data-bind=\"style: { width: width() + \'px\', height: height() + \'px\', \'marginLeft\': left() + \'px\', \'marginTop\': top() + \'px\' }\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\"></button>        <h4 class=\"modal-title\">Copy documents</h4>    </div>    <div class=\"modal-body\" data-bind=\"style: { height: bodyHeight() + \'px\' }\">        <h4><strong>CTRL+C, Enter</strong><span> to copy</span></h4>        <ul class=\"nav nav-tabs\">            <li data-bind=\"click: activateDocs, css: { active: isCopyingDocs}\"><a style=\"cursor: pointer;\">Docs</a></li>            <li data-bind=\"click: activateIds, css: { active: isCopyingDocs() === false }\"><a style=\"cursor: pointer;\">Ids</a></li>        </ul>        <textarea id=\"documentsText\" rows=\"10\" style=\"width: 100%; resize: none\" class=\"form-control\" autofocus=\"autofocus\" data-bind=\"style : { height: (bodyHeight() - 110) + \'px\' }, value: documentsOrIdsText, event: { focus: function () { $(this).select(); } }\" />    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: close\">Close</button>        <span class=\"dialogResizer\"></span>    </div></div>'; } )
define("text!views/database/documents/deleteCollection.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\" data-bind=\"with: collection\">        <span>You\'re deleting </span>        <strong data-bind=\"text: name\"></strong>        <strong data-bind=\"visible: documentCount() > 1, text: \'(\' + documentCount().toLocaleString() + \' documents)\'\"></strong>        <strong data-bind=\"visible: documentCount() == 1, text: \'(\' + documentCount() + \' document)\'\"></strong>        <div              data-bind=\"visible: $root.isAllDocuments\">            *Note that system documents won\'t be deleted.<br />            In order to do that go to the system documents section.        </div>    </div>        <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteCollection\">Yep, delete</button>    </div></div>'; } )
define("text!views/database/documents/documents.html", [], function () { return '<div id=\"documents-pane\" class=\"documents-page no-outline row\" tabindex=\"0\" data-bind=\"enableResize: true\">    <!-- ko if: alerts().length > 0 -->        <!-- ko foreach: alerts -->    <div class=\"alert alert-dismissable\" role=\"alert\" data-bind=\"css: { \'alert-warning\': alertLevel === \'Warning\', \'alert-danger\': alertLevel === \'Error\' }\">        <button data-bind=\"click: $root.dismissAlert.bind($root, uniqueKey)\" type=\"button\" class=\"close\" data-dismiss=\"alert\"><span aria-hidden=\"true\">&times;</span><span class=\"sr-only\">Close</span></button>        <a data-bind=\"attr: { href: $root.urlForAlert($data) }\"><span data-bind=\"text: title\"></span></a>    </div>        <!-- /ko -->    <!-- /ko -->    <div id=\"documents-page-container\" class=\"col-md-2 col-resizable-target\">                <ul class=\"nav nav-pills nav-stacked document-collections\" data-bind=\"with: allDocumentsCollection\">            <li class=\"all-docs-collection\" data-bind=\"event: { mousedown: $parent.selectCollection.bind($parent) }, css: { active: $data === $parent.selectedCollection() }\">                <a href=\"javascript:void(0)\">                    <div class=\"collection-name\">                        <div class=\"collection-text pull-left collection-color-strip\" data-bind=\"css: colorClass\"></div>                        <span class=\"collection-text pull-left collection-name-part\" data-bind=\"text: prettyLabel(name), attr: {title: name}\"></span>                    </div>                </a>            </li>        </ul>        <ul class=\"nav nav-pills nav-stacked document-collections\" data-bind=\"foreach: collectionsExceptAllDocs, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" style=\"padding-left: 0;\">            <li data-bind=\"event: { mousedown: $parent.selectCollection.bind($parent) }, css: { active: $data === $parent.selectedCollection() }, visible: documentCount() > 0 || isAllDocuments || isSystemDocuments\">                <a href=\"javascript:void(0)\">                    <div class=\"collection-name\">                        <div class=\"collection-text pull-left collection-color-strip\" data-bind=\"css: colorClass\"></div>                        <span class=\"collection-text pull-left collection-name-part\" data-bind=\"text: prettyLabel(name), attr: {title: name}\"></span>                        <span class=\"collection-text pull-left text-muted\" data-bind=\"visible: !isSystemDocuments && !isAllDocuments, css: { \'text-muted\': $data !== $parent.selectedCollection() }, text:  \'&nbsp;(\' + documentsCountWithThousandsSeparator() + \')\', updateHighlighting: documentCount()\"></span>                    </div>                </a>            </li>        </ul>        <div class=\"collection-resize\"></div>    </div>    <div class=\"col-md-10 col-resizable-affected\" id=\"documents-page-right-container\">        <div class=\"btn-toolbar\" role=\"toolbar\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <div class=\"btn-group\">                <div class=\"checkbox\">                    <input id=\"documents_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: documentsSelection, event: { change: toggleSelectAll }, visible: hasDocuments\" title=\"Select all or none\">                    <label for=\"documents_all\">&nbsp;</label>                </div>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-primary\" accesskey=\"n\" data-bind=\"click: newDocument\" title=\"Create new document (Alt+N)\">                    <i class=\"fa fa-plus\"></i> New document                </button>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refresh, disable: showLoadingIndicator() || isInitialized() == false\" title=\"Refresh from server\">                    <i class=\"fa fa-refresh\"></i>                </button>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: selectColumns, visible: hasDocuments\" title=\"Choose columns...\">                    <i class=\"fa fa-table\"></i>                </button>            </div>            <div class=\"btn-group\">                <button data-bind=\"visible: isRegularCollection, click: exportCsv\" class=\"btn btn-default\" title=\"Export CSV\">                    <i class=\"fa fa-file-excel-o\"></i>                </button>                <button type=\"button\" data-bind=\"visible: isRegularCollection\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\" aria-expanded=\"false\">                    <span class=\"caret\"></span>                    <span class=\"sr-only\">Toggle Dropdown</span>                </button>                <ul class=\"dropdown-menu\">                    <li><a href=\"#\" data-bind=\"click: selectCsvColumns\">Choose columns and export CSV...</a></li>                </ul>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyDocumentsSelected\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: editSelectedDoc, enable: selectedDocumentIndices().length == 1\" title=\"Edit selected document (F2)\">                    <i class=\"fa fa-pencil\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: generateDocCode, enable: selectedDocumentIndices().length == 1 \" title=\"Generate Class\">                    <i class=\"fa\">C#</i>                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyDocumentsSelected\">                <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteSelectedDocs\" title=\"Delete selected documents (DEL)\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyDocumentsSelected() && canCopyAllSelected()\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: copySelectedDocs\" title=\"Copy selected documents (Ctrl+C, D)\">                    <i class=\"fa fa-copy\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: copySelectedDocIds\" title=\"Copy selected document IDs (Ctrl+C, I)\">                    <i class=\"fa fa-clipboard\"></i>                </button>            </div>            <div class=\"btn-group\">                <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicatorThrottled\"><i class=\"fa fa-spinner fa-spin fa-2x\"></i>            </div>        </div>        </div>        <div class=\"documents-select\" data-bind=\"with: selectedCollection, visible: !isSystemDocumentsCollection() && hasAnyDocumentsSelected() && !hasAllDocumentsSelected() && isAnyDocumentsAutoSelected()\">            <i class=\"fa fa-info-circle\"></i> You have selected <strong data-bind=\"text: $root.selectedDocumentIndices().length.toLocaleString()\"></strong> <span data-bind=\"text: $root.selectedDocumentsText\"></span>.            <span class=\"select-link\" data-bind=\"click: $root.selectAll.bind($root)\">                Select all <strong data-bind=\"text: documentCount().toLocaleString()\"></strong> documents in <span data-bind=\"text: name\"></span>            </span>        </div>        <div class=\"documents-select\" data-bind=\"with: selectedCollection, visible: !isSystemDocumentsCollection() && hasAllDocumentsSelected() && isAllDocumentsAutoSelected()\">            You have selected <strong data-bind=\"text: $root.selectedDocumentIndices().length.toLocaleString()\"></strong> <span data-bind=\"text: $root.selectedDocumentsText\"></span> in            <span data-bind=\"text: name\"></span>.            <span class=\"select-link\" data-bind=\"click: $root.selectNone.bind($root)\">                Clear selection            </span>        </div>        <div id=\"documentsGrid\" data-bind=\"widget: { kind: \'virtualTable\', customColumns: currentColumnsParams, itemsSource: currentCollectionPagedItems, selectedIndices: selectedDocumentIndices, gridSelector: \'#documentsGrid\', customFunctions: currentCustomFunctions, collections: collections, rowsAreLoading: showLoadingIndicator, isAnyAutoSelected: isAnyDocumentsAutoSelected, isAllAutoSelected: isAllDocumentsAutoSelected, noResultsMessage: \'No documents were found!\', container: \'#documents-page-right-container\' }\"></div>    </div></div>'; } )
define("text!views/database/documents/editDocument.html", [], function () { return '<div id=\"editDocumentContainer\" tabindex=\"-1\">    <!-- We use a container with a tab index so that we can capture keyboard events, such as CTRL+S. -->    <ul class=\"breadcrumb\" style=\"display: inline-block\">        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <li style=\"display: inline-block\">            <h3 style=\"display: inline-block\">                <i class=\"fa fa-file-text-o\" style=\"display: inline-block\"></i>                <a href=\"#documents\" data-bind=\"click: navigateToDocuments, text:isInDocMode()?\'Documents\':\'Indexes\'\" style=\"display: inline-block\">                </a>            </h3>        </li>        <!-- ko ifnot: isCreatingNewDocument -->        <li data-bind=\"with: metadata\" style=\"display: inline-block\">            <a style=\"display: inline-block\" href=\"#\" data-bind=\"text: $root.isInDocMode()?$root.generateCollectionName(ravenEntityName, true):$root.queryIndex,                attr: {                    title: ravenEntityName,                    href: \'#documents?collection=\' + $root.generateCollectionName(ravenEntityName)                },                click: $root.navigateToCollection.bind($root, $root.generateCollectionName(ravenEntityName))\"></a>        </li>        <!-- /ko -->        <li class=\"active\" data-bind=\"text: docTitle, attr:{ title: docTitle }\" style=\"display: inline-block\">        </li>    </ul>    <form data-bind=\"submit: saveDocument\">        <div class=\"btn-toolbar\">            <!-- We duplicate following sections to fix issues with missing rounded borders when creating new document -->            <div data-bind=\"if: loadedDocumentName\">                <div class=\"btn-group\">                    <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save (Alt+S)\"><i class=\"fa fa-save\"></i></button>                    <a class=\"btn btn-default\" data-bind=\"click: refreshDocument\" title=\"Refresh (Alt+R)\" accesskey=\"R\"><i class=\"fa fa-refresh\"></i></a>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable:!isNewLineFriendlyMode() || $root.isEditingMetadata(), click: formatDocument\" title=\"Format document (Alt+[)\" accesskey=\"[\"><i class=\"fa fa-align-justify\"></i></button>                    <button class=\"btn btn-default\" title=\"Toggle newline mode (Alt+N)\" data-toggle=\"button\" accesskey=\"N\" data-bind=\"enable: !$root.isEditingMetadata(), click: toggleNewlineMode\">                        <span class=\"text-lowercase\" data-bind=\"text: newLineToggle\"></span>                    </button>                    <button class=\"btn btn-default\" title=\"Toggle auto-collapse\" data-toggle=\"button\" data-bind=\"enable: !$root.isEditingMetadata(), click: toggleAutoCollapse\">                        <span class=\"fa fa-plus-square-o\"></span>                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: generateCode, enable: !isSaveEnabled()\" title=\"Generate Class\"><i class=\"fa\">C#</i></button>                    <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteDocument\" title=\"Delete (Alt+shift+Del)\"><i class=\"fa fa-trash-o\"></i></button>                </div>            </div>            <div data-bind=\"ifnot: loadedDocumentName\">                <div class=\"btn-group\">                    <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save (Alt+S)\"><i class=\"fa fa-save\"></i></button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable:!isNewLineFriendlyMode() || $root.isEditingMetadata(), click: formatDocument\" title=\"Format document (Alt+[)\" accesskey=\"[\"><i class=\"fa fa-align-justify\"></i></button>                    <button class=\"btn btn-default\" title=\"Toggle newline mode (Alt+N)\" data-toggle=\"button\" accesskey=\"N\" data-bind=\"enable: !$root.isEditingMetadata(), click: toggleNewlineMode\">                        <span class=\"text-lowercase\" data-bind=\"text: newLineToggle\"></span>                    </button>                    <button class=\"btn btn-default\" title=\"Toggle auto-collapse\" data-toggle=\"button\" data-bind=\"enable: $root.isEditingMetadata() === false, click: toggleAutoCollapse\">                        <span class=\"fa fa-plus-square-o\"></span>                    </button>                </div>            </div>            <div class=\"btn-group\" data-bind=\"with: docsList\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.firstDocument.bind($root), attr:{ disabled: $root.isFirstDocumenNavtDisabled }\" title=\"Go to first document (Alt+Home)\"><i class=\"fa fa-step-backward\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.previousDocumentOrLast.bind($root)\" title=\"Go to previous document (Alt+PageUp)\"><i class=\"fa fa-caret-left\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"text: \'Doc \' + (currentItemIndex() + 1).toLocaleString() + \' of \' + totalResultCount().toLocaleString()\"></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.nextDocumentOrFirst.bind($root)\" title=\"Go to next document (Alt+PageDown)\"><i class=\"fa fa-caret-right\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.lastDocument.bind($root), attr:{ disabled: $root.isLastDocumentNavDisabled }\" title=\"Go to last document (Alt+End)\"><i class=\"fa fa-step-forward\"></i></button>            </div>        </div>        <br />        <div class=\"row\">            <div class=\"col-md-7\">                <div class=\"row\" data-bind=\"with: metadata\">                    <div class=\"col-md-12\">                        <input id=\"documentName\" type=\"text\" class=\"form-control\" data-placement=\"top\" accesskey=\"i\" placeholder=\"document id\" data-bind=\"textInput: $root.userSpecifiedId, attr: { disabled: $root.isBusy }\" />                    </div>                </div>                <br />                <div class=\"row\">                    <div class=\"col-md-12\">                        <ul class=\"nav nav-tabs\">                            <li accesskey=\"d\" data-bind=\"click: activateDoc, css: { active: isEditingMetadata() === false }\" title=\"Edit document data (Alt+Shift+D)\"><a style=\"cursor: pointer;\">Data</a></li>                            <li accesskey=\"m\" data-bind=\"click: activateMeta, css: { active: isEditingMetadata }\" title=\"Edit document metadata (Alt+Shift+M)\"><a style=\"cursor: pointer;\">Metadata</a></li>                        </ul>                        <pre id=\"docEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: text, fontSize:\'16px\', lang: \'ace/mode/json\'}, attr: { disabled: isBusy }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 20 }\"></pre>                    </div>                </div>            </div>            <div class=\"form-group panel panel-default col-md-5 document-addition-data side-panel\">                <div class=\"col-md-12\" data-bind=\"with: metadata\">                    <h4><strong>Metadata</strong></h4>                    <hr />                    <div class=\"row\">                        <label class=\"col-md-5\">Raven-Entity-Name</label>                        <span class=\"col-md-7\" data-bind=\"text: $root.prettyLabel(ravenEntityName), attr: {title:ravenEntityName}\"></span>                    </div>                    <div class=\"row\">                        <label class=\"col-md-5\">Etag</label>                        <span class=\"col-md-7\" data-bind=\"text: etag, attr: {title:etag}\"></span>                    </div>                    <div class=\"row\">                        <label class=\"col-md-5\">Last-Modified</label>                        <span class=\"col-md-7\" data-bind=\"text: lastModifiedFullDate, attr: {title:lastModifiedFullDate}\"></span>                    </div>                    <div class=\"row\">                        <label class=\"col-md-5\">Size in KB</label>                        <span class=\"col-md-7\" data-bind=\"text: $root.documentSize() , attr: {title:$root.documentSize() }\"></span>                    </div>                </div>                <div class=\"col-md-12 query-metadata\" data-bind=\"visible:topRecentDocuments().length > 0\">                    <h4><strong>Recent Documents</strong></h4>                    <hr />                    <div class=\"documents-links-container\" data-bind=\"foreach: topRecentDocuments\">                        <a class=\"document-link\" data-bind=\"attr: { href: docUrl }\">                            <span class=\"collection-color-strip\" data-bind=\"css: $root.getColorClass($data.docId)\">&nbsp;</span>                            <span data-bind=\"text: docId, attr : {title : fullDocId }\"></span>                        </a>                    </div>                </div>                <div class=\"col-md-12 query-metadata\" data-bind=\"visible: relatedDocumentHrefs().length >0\">                    <h4><strong>Related Documents</strong></h4>                    <hr />                    <div class=\"documents-links-container\" data-bind=\"foreach: relatedDocumentHrefs\">                        <a class=\"document-link\" data-bind=\"attr: { href:href }\">                            <span class=\"collection-color-strip\" data-bind=\"css: $root.getColorClass($data.id)\">&nbsp;</span>                            <span data-bind=\"text: id\"></span>                        </a>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/database/exploration/exploration.html", [], function () { return '<div id=\"explorationContainer\">    <ul class=\"breadcrumb\">        <li>            <h3>                      <a href=\"#resources\">                    Databases                </a>            </h3>        </li>        <li data-bind=\"with: activeDatabase\">            <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>        </li>        <li>            <a data-bind=\"attr: { href: appUrls.exploration }\">Data exploration</a>        </li>    </ul>    <div class=\"btn-toolbar\" role=\"toolbar\">        <div class=\"btn-group\">            <button class=\"btn btn-primary\" title=\"Run\" accesskey=\"R\" data-bind=\"click: runExploration, enable: runEnabled\">                <i class=\"fa fa-play\"></i>            </button>            <button data-bind=\"click: exportCsv, visible: runEnabled\" class=\"btn btn-default\" title=\"Export CSV\">                <i class=\"fa fa-file-excel-o\"></i>            </button>            <button class=\"btn btn-danger\" data-bind=\"click: killTask, visible: isBusy\"><i class=\"fa fa-bolt\"></i> Cancel</button>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Collection\" data-bind=\"disable: isBusy\">                <span data-bind=\"text: selectedCollectionLabel\"></span> <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: collections\">                <li data-bind=\"click: $root.explorationRequest.collection.bind($root, $data)\">                    <a href=\"#\" data-bind=\"text: $data\"></a>                </li>            </ul>        </div>        <div class=\"pull-right text-danger padd-top-10 padd-right-10\">            <i class=\"fa fa-exclamation-triangle\"></i> This operation is resource intensive!        </div>    </div><br />    <form class=\"form-horizontal\" role=\"form\">        <div class=\"row\">            <div class=\"col-md-4\">                <div class=\"form-group\">                    <label class=\"col-sm-6 control-label\">Timeout (seconds):</label>                    <div class=\"col-md-4\">                        <input data-bind=\"value: explorationRequest.timeoutSeconds, disable: isBusy\" type=\"number\" required class=\"form-control\"/>                    </div>                </div>            </div>            <div class=\"col-md-3\">                <div class=\"form-group\">                    <label class=\"col-sm-6 control-label\">Max documents to scan:</label>                    <div class=\"col-md-4\">                        <input data-bind=\"value: explorationRequest.pageSize, disable: isBusy\" type=\"number\" required class=\"form-control\" />                    </div>                </div>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-1 control-label\" data-placement=\"auto right\" data-original-title=\"LINQ\" data-trigger=\"hover\">                LINQ            </label>            <div class=\"col-md-11\">                <div class=\"row\">                    <div class=\"col-md-11\">                        <pre class=\"form-control editor\" data-bind=\"aceEditor: { code: explorationRequest.linq, allowResize: true, minHeight: 100, maxHeight: 400, lang:\'ace/mode/csharp\'}, valueUpdate: \'afterkeydown\'\"></pre>                    </div>                </div>            </div>        </div>        <div class=\"form-group\">            <div class=\"form-group col-md-12\">                <label class=\"control-label col-md-1\">                    Results                </label>                <div class=\"col-md-11 exploration-results\">                    <div id=\"explorationResultsPanel\" class=\"panel panel-default\">                        <div class=\"panel-body\" data-bind=\"visible: isLoading\">                            <span>Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                        </div>                        <div class=\"panel-body\">                            <div id=\"explorationResultsGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: queryResults, gridSelector: \'#explorationResultsGrid\', useContextMenu: false, showCheckboxes: false, noResultsMessage: \'No results were found.\'}\"></div>                        </div>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/database/indexes/cancelSideBySizeConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text: title\"></h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: indexNames.length === 1\">You\'re canceling side-by-side index <strong data-bind=\"text: indexNames[0]\"></strong></p>        <p data-bind=\"visible: indexNames.length > 1\">You\'re canceling  <strong data-bind=\"text: indexNames.length\"></strong> side-by-side indexes.</p>        <ul data-bind=\"visible: indexNames.length > 1, foreach: indexNames\" style=\"max-height: 200px; overflow-y: auto;\">            <li data-bind=\"text: $data\"></li>        </ul>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: cancelIndexes\">Yep, Cancel</button>    </div></div>'; } )
define("text!views/database/indexes/copyIndexDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text:isPaste ? \'Paste Index\':\'Copy Index\'\"></h4>    </div>    <div class=\"modal-body\">        <h4>            <strong data-bind=\"text: isPaste === true ? \'CTRL+V, Enter \': \'CTRL+C, Enter \'\"></strong>            <span data-bind=\"text: isPaste === true ? \'to save\' : \'to copy\'\"></span>        </h4>        <div>            <pre data-bind=\"aceEditor: { code: indexJSON, lang: \'ace/mode/json\', getFocus: true, selectAll: true, readOnly: !isPaste, bubbleEscKey: true }\" style=\"width: 600px; height: 400px;\"></pre>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close, visible: isPaste\">Close</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: saveIndex, text: isPaste ? \'Save Index\' : \'Close\', enable: indexJSON(), attr: { title: isPaste ? \'Save the index (Alt+S)\' : \'\', accesskey: isPaste ? \'S\' : \'\' }\"></button>    </div></div>'; } )
define("text!views/database/indexes/deleteIndexesConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text: title\"></h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: indexNames.length === 1\">You\'re deleting <strong data-bind=\"text: indexNames[0]\"></strong></p>        <p data-bind=\"visible: indexNames.length > 1\">You\'re deleting <strong data-bind=\"text: indexNames.length\"></strong> indexes</p>        <ul data-bind=\"visible: indexNames.length > 1, foreach: indexNames\" style=\"max-height: 200px; overflow-y: auto;\">            <li data-bind=\"text: $data\"></li>        </ul>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteIndexes\">Yep, delete</button>    </div></div>'; } )
define("text!views/database/indexes/editIndex.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li>        <a data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>    </li>    <li class=\"active\">        <span data-bind=\"text: currentIndexName\"></span>    </li></ul><section id=\"editIndex\">    <div class=\"btn-toolbar\">        <div class=\"btn-group\">            <button class=\"btn btn-primary\" data-bind=\"click: save, visible: !editedIndex().isSideBySideIndex(), enable: $root.isSaveEnabled()\" title=\"Save the index (Alt+S)\" accesskey=\"S\">                <i class=\"fa fa-save\"></i>            </button>            <button type=\"button\"  data-bind=\"visible: !editedIndex().isSideBySideIndex()\"class=\"btn btn-primary dropdown-toggle\" data-toggle=\"dropdown\" aria-expanded=\"false\">                <span class=\"caret\"></span>                <span class=\"sr-only\">Toggle Dropdown</span>            </button>            <ul class=\"dropdown-menu\"  role=\"menu\">                <li>                    <a title=\"Replaces current index when specified condition is met\" href=\"#\" data-bind=\"visible: !editedIndex().isSideBySideIndex() && !editedIndex().isTestIndex(), click: replaceIndex\">                        <i class=\"fa fa-reply fa-fw\"></i> Side-by-side indexing                    </a>                </li>                <li>                    <a title=\"Create test index based on limited data set\" href=\"#\" data-bind=\"visible: !editedIndex().isTestIndex() && !editedIndex().isSideBySideIndex(), click: tryIndex\">                        <i class=\"fa fa-eyedropper fa-fw\"></i> Try index                    </a>                </li>                <li>                    <a href=\"#\" title=\"Saves test index as persistent\" data-bind=\"visible: editedIndex().isTestIndex(), click: makePermanent\">                        <i class=\"fa fa-save\"></i> Make permanent                    </a>                </li>            </ul>        </div>        <div class=\"btn-group\" data-bind=\"visible: !editedIndex().isSideBySideIndex()\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" accesskey=\"A\">                <i class=\"fa fa-plus\"></i>                Add                <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\">                <li  data-bind=\"click: addMap\" title=\"Add a map function (Alt+M)\" accesskey=\"M\"><a href=\"#\">Map</a></li>                <li  data-bind=\"click: addReduce, visible: !hasExistingReduce()\" title=\"Add a reduce function (Alt+R)\" accesskey=\"R\"><a href=\"#\">Reduce</a></li>                <li  data-bind=\"click: addField\" title=\"Add a field (Alt+G)\" accesskey=\"G\"><a href=\"#\">Field</a></li>                <li  data-bind=\"click: addSpatialField\" title=\"Add a spatial field (Alt+P)\" accesskey=\"P\"><a href=\"#\">Spatial Field</a></li>                <li data-bind=\"visible: !editMaxIndexOutputsPerDocument(), click: function() {$root.editMaxIndexOutputsPerDocument(true); return false;}\"><a href=\"#\">Max Index Outputs</a></li>                <li data-bind=\"visible: !$root.editedIndex().storeAllFields(), click: function() {$root.editedIndex().setOrRemoveStoreAllFields(true); return false;}\"><a href=\"#\">Store All Fields</a></li>            </ul>        </div>        <div class=\"btn-group\" data-bind=\"visible: !editedIndex().isSideBySideIndex()\">            <button class=\"btn btn-default\" title=\"Format maps and reduce\" data-bind=\"click: formatIndex\">                <i class=\"fa fa-magic\"></i>            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: isEditingExistingIndex\">                <a class=\"btn btn-default\" title=\"Query the index (Alt+Q)\" accesskey=\"Q\"   data-bind=\"attr: { href: queryUrl }\">                    <i class=\"fa fa-play\"></i>                </a>              <a class=\"btn btn-default\" title=\"Navigate to index terms\"  data-bind=\"attr: { href: termsUrl }\">Terms</a>        </div>        <div class=\"btn-group\" data-bind=\"visible: isEditingExistingIndex\">            <button class=\"btn btn-default\" title=\"Copy edited index to clipboard\" data-bind=\"click: copyIndex, enable: !$root.isSaveEnabled()\">                <i class=\"fa fa-files-o\"></i>            </button>            <button class=\"btn btn-default\" title=\"Generate C# index definition\" data-bind=\"click: createCSharpCode, enable: !$root.isSaveEnabled()\">                <i class=\"fa\">C#</i>            </button>            <button class=\"btn btn-default\" title=\"Refresh from server, discarding any unsaved changes\" data-bind=\"click: refreshIndex\">                <i class=\"fa fa-refresh\"></i>            </button>            <button class=\"btn btn-danger\" title=\"Delete index\" data-bind=\"visible: !editedIndex().isSideBySideIndex(), click: deleteIndex\">                <i class=\"fa fa-trash-o\"></i>            </button>        </div>                <button class=\"btn btn-danger\" title=\"Cancel side-by-side index\" data-bind=\"visible: editedIndex().isSideBySideIndex(), click: cancelSideBySideIndex\">            <i class=\"fa fa-undo\"></i>        </button>    </div>    <div class=\"container-fluid\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" style=\"overflow: auto\">        <br />        <form class=\"form-horizontal\" role=\"form\" data-bind=\"with: editedIndex\">                        <div class=\"alert alert-warning\" data-bind=\"visible: isTestIndex()\" role=\"alert\">This is test index which operates on limited data set. Press &quot;Make permanent&quot; to override the original index.</div>                        <div class=\"form-group\">                <label for=\"indexName\" class=\"col-md-1 control-label\"  title=\" index name (Alt+I)\">Name</label>                <div class=\"col-md-11\">                    <div class=\"row\">                        <div class=\"col-md-11\">                            <input type=\"text\" class=\"form-control\" id=\"indexName\" placeholder=\"Index name\" accesskey=\"I\" data-bind=\"value: name, valueUpdate: \'afterkeyup\', enable: !isTestIndex() && !isSideBySideIndex()\" />                        </div>                    </div>                </div>            </div>            <div class=\"form-group\">                <label id=\"indexMapsLabel\" style=\"cursor: help;\" class=\"col-md-1 control-label\" data-placement=\"auto right\" data-original-title=\"Index maps\" data-trigger=\"hover\">                    Maps                    <i class=\"fa fa-info-circle text-info\"></i>                </label>                <div data-bind=\"foreach: maps\" class=\"col-md-11\">                    <div class=\"row\">                        <div class=\"col-md-11\">                            <pre id=\"indexEditor\" class=\"form-control map\" data-bind=\"aceEditor: { readOnly: $parent.isSideBySideIndex(), code: $data, allowResize: true, getFocus: true, lang: \'ace/mode/ravenMapLinq\', completer: $root.indexAutoCompleter.indexMapCompleter, typeName: \'map\', completerHostObject: $root.indexAutoCompleter }\"></pre>                        </div>                        <button type=\"button\" class=\"close pull-left\" data-bind=\"visible: $index() !== 0 || $root.hasMultipleMaps(), click: $root.removeMap.bind($root, $index())\" aria-hidden=\"true\" title=\"Remove this map from the index\">                            <i class=\"fa fa-times\"></i>                        </button>                    </div>                    <br />                </div>            </div>            <div class=\"form-group\" data-bind=\"if: reduce\">                <label id=\"indexReduceLabel\" style=\"cursor: help;\" for=\"indexReduce\" class=\"col-md-1 control-label\" data-placement=\"auto right\" data-original-title=\"Index reduce\" data-trigger=\"hover\">                    Reduce                    <i class=\"fa fa-info-circle text-info\"></i>                </label>                <div class=\"col-md-11\">                    <div class=\"row\">                        <div class=\"col-md-11\">                            <pre class=\"form-control map\" data-bind=\"aceEditor: { readOnly: isSideBySideIndex(), code: reduce, allowResize: true, lang: \'ace/mode/ravenReduceLinq\', completer: $root.indexAutoCompleter.indexReduceCompleter, typeName: \'reduce\', completerHostObject: $root.indexAutoCompleter }\"></pre>                        </div>                        <button type=\"button\" class=\"close pull-left\" data-bind=\"click: $root.removeReduce.bind($root)\" aria-hidden=\"true\" title=\"Remove the Reduce function from the index\">                            <i class=\"fa fa-times\"></i>                        </button>                    </div>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: luceneFields().length > 0\">                <label class=\"col-md-1 control-label\">                    Fields                </label>                <div class=\"col-md-11\">                    <div class=\"row\" data-bind=\"foreach: luceneFields\">                        <div class=\"col-xs-12 col-sm-9 col-md-6 col-lg-5\">                            <div class=\"panel panel-default\">                                <div class=\"panel-heading\">                                    <div class=\"col-md-11\">                                        <input name=\"fieldName\" type=\"text\" class=\"form-control\" placeholder=\"FieldName\" data-bind=\"enable: !$parent.isSideBySideIndex(), attr: { id:\'fieldName\' + $index() }, value: name, valueUpdate: \'afterkeydown\'\" autocomplete=\"off\" />                                        <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#\' + \'fieldName\' + $index(),foreach: fieldNameAutocompletes\">                                            <li role=\"presentation\" data-bind=\"click: $parent.setName.bind($parent,$data)\">                                                <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                                    <span data-bind=\" text:$data\"></span>                                                </a>                                            </li>                                        </ul>                                    </div>                                    <div>                                        <button type=\"button\" class=\"close\" aria-hidden=\"true\"><i class=\"fa fa-times\" data-bind=\"click: $root.removeLuceneField.bind($root, $index())\" aria-hidden=\"true\" title=\"Remove this field from the index\"></i></button>                                    </div>                                    <div class=\"clearfix\"></div>                                </div>                                <div class=\"panel-body fieldControl\">                                    <div class=\"col-md-6 fieldControl\">                                        <label class=\"control-label col-md-5 fieldControl\">Storage</label>                                        <div class=\"col-md-7 fieldControl\">                                            <select class=\"form-control fieldControl\" data-bind=\"options: [\'Yes\', \'No\'], value: stores, enable: !$parent.isSideBySideIndex()\"></select>                                        </div>                                        <label class=\"control-label col-md-5 fieldControl\">Sort</label>                                        <div class=\"col-md-7 fieldControl\">                                            <select class=\"form-control fieldControl\" data-bind=\"options: [\'None\', \'String\', \'Int\', \'Float\', \'Long\', \'Double\', \'Short\', \'Custom\', \'Byte\', \'StringVal\'], value: sort, enable: !$parent.isSideBySideIndex()\"></select>                                        </div>                                        <label class=\"control-label col-md-5 fieldControl\">TermVector</label>                                        <div class=\"col-md-7 fieldControl\">                                            <select class=\"form-control fieldControl\" data-bind=\"options: [\'No\', \'Yes\', \'WithPositions\', \'WithOffsets\', \'WithPositionsAndOffsets\'], value: termVector, enable: !$parent.isSideBySideIndex()\"></select>                                        </div>                                    </div>                                    <div class=\"col-md-6 fieldControl\">                                        <label class=\"control-label col-md-5 fieldControl\">Indexing</label>                                        <div class=\"col-md-7 fieldControl\">                                            <select class=\"form-control fieldControl\" data-bind=\"options: [\'No\', \'Analyzed\', \'NotAnalyzed\', \'Default\'], value: indexing, enable: !$parent.isSideBySideIndex()\"></select>                                        </div>                                        <label class=\"control-label col-md-5 fieldControl\" data-bind=\"visible: indexing() === \'Analyzed\'\">Analyzer</label>                                        <div class=\"col-md-7 fieldControl\" data-bind=\"visible: indexing() === \'Analyzed\'\">                                            <input type=\"text\" class=\"form-control fieldControl\" data-bind=\"value: analyzer, valueUpdate: \'afterkeyup\', attr: { title: analyzer }, enable: !$parent.isSideBySideIndex()\" />                                        </div>                                        <label class=\"control-label col-md-5  fieldControl\" title=\"The suggestion distance algorithm\">Suggestion</label>                                        <div class=\"col-md-7 fieldControl\">                                            <button type=\"button\" data-bind=\"enable: !$parent.isSideBySideIndex()\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                <span data-bind=\"visible: suggestionEnabled()\">Yes</span>                                                <span data-bind=\"visible: !suggestionEnabled()\">No</span>                                                <span class=\"caret\"></span>                                            </button>                                            <ul class=\"dropdown-menu\" role=\"menu\">                                                <li><a href=\"#\" data-bind=\"click: suggestionEnabled.bind(suggestionEnabled, true)\">Yes</a></li>                                                <li><a href=\"#\" data-bind=\"click: suggestionEnabled.bind(suggestionEnabled, false)\">No</a></li>                                            </ul>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: spatialFields().length > 0\">                <label class=\"col-sm-1 control-label\">                    Spatial Fields                </label>                <div class=\"col-md-10\" data-bind=\"foreach: spatialFields, visible: spatialFields().length > 0\">                    <div>                        <div class=\"panel panel-default\">                            <div class=\"panel-heading\">                                <div class=\"col-md-11\">                                    <input type=\"text\" class=\"form-control\" data-bind=\"enable: !$parent.isSideBySideIndex(), value: name, valueUpdate: \'afterkeyup\'\" placeholder=\"spatial field name\" />                                </div>                                <div>                                    <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeSpatialField.bind($root, $index())\" title=\"Remove this spatial field from the index\">                                        <i class=\"fa fa-times\"></i>                                    </button>                                </div>                                <div class=\"clearfix\"></div>                            </div>                            <div class=\"panel-body\">                                <div class=\"col-md-6\">                                    <label class=\"control-label col-md-3\">Type</label>                                    <div class=\"col-md-9\">                                        <select class=\"form-control\" data-bind=\"options: [\'Geography\', \'Cartesian\'], value: type, enable: !$parent.isSideBySideIndex()\"></select>                                    </div>                                    <label class=\"control-label col-md-3\">Strategy</label>                                    <div class=\"col-md-9\">                                        <select class=\"form-control\" data-bind=\"options: availableStrategies, value: strategy, enable: !$parent.isSideBySideIndex()\"></select>                                    </div>                                    <label class=\"control-label col-md-3\">Min X</label>                                    <div class=\"col-md-9\">                                        <input type=\"number\" class=\"form-control\" data-bind=\"numericValue: minX, valueUpdate: \'afterkeyup\', enable: canSpecifyCoordinates, enable: !$parent.isSideBySideIndex()\" />                                    </div>                                    <label class=\"control-label col-md-3\">Max X</label>                                    <div class=\"col-md-9\">                                        <input type=\"number\" class=\"form-control\" data-bind=\"numericValue: maxX, valueUpdate: \'afterkeyup\', enable: canSpecifyCoordinates, enable: !$parent.isSideBySideIndex()\" />                                    </div>                                    <label class=\"control-label col-md-3\">Precision</label>                                    <div class=\"col-md-9\">                                        <p class=\"form-control-static\" style=\"white-space: nowrap\" data-bind=\"text: precision, enable: !$parent.isSideBySideIndex()\"></p>                                    </div>                                </div>                                <div class=\"col-md-6\">                                    <div class=\"row\">                                        <label class=\"control-label col-md-4\" data-bind=\"visible: canSpecifyUnits\">Circle Radius Units</label>                                        <div class=\"col-md-4\" data-bind=\"visible: canSpecifyUnits\">                                            <select class=\"form-control\" data-bind=\"options: [\'Kilometers\', \'Miles\'], value: circleRadiusUnits, enable: !$parent.isSideBySideIndex()\"></select>                                        </div>                                    </div>                                    <div class=\"row\">                                        <label class=\"control-label col-md-4\" data-bind=\"visible: canSpecifyTreeLevel\">Max Tree Level</label>                                        <div class=\"col-md-4\" data-bind=\"visible: canSpecifyTreeLevel\">                                            <input type=\"number\" class=\"form-control\" data-bind=\"numericValue: maxTreeLevel, valueUpdate: \'afterkeyup\', enable: !$parent.isSideBySideIndex()\" />                                        </div>                                    </div>                                    <div class=\"row\">                                        <label class=\"control-label col-md-4\">Min Y</label>                                        <div class=\"col-md-4\">                                            <input type=\"number\" class=\"form-control\" data-bind=\"numericValue: minY, valueUpdate: \'afterkeyup\', enable: canSpecifyCoordinates() && !$parent.isSideBySideIndex()\" />                                        </div>                                    </div>                                    <div class=\"row\">                                        <label class=\"control-label col-md-4\">Max Y</label>                                        <div class=\"col-md-4\">                                            <input type=\"number\" class=\"form-control\" data-bind=\"numericValue: maxY, valueUpdate: \'afterkeyup\', enable: canSpecifyCoordinates() && !$parent.isSideBySideIndex()\" />                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: maxIndexOutputsPerDocument() > 0 || $root.editMaxIndexOutputsPerDocument() == true\">                <label class=\"col-sm-1 control-label\">Max Index Outputs Per Document</label>                <div class=\"col-md-2\">                    <input class=\"form-control\" required type=\"number\" data-bind=\"numericValue: maxIndexOutputsPerDocument, valueUpdate: \'afterkeyup\', enable: !isSideBySideIndex()\" min=\"0\" />                </div>                <div class=\"col-md-1\">                    <button type=\"button\" class=\"close pull-left\" data-bind=\"click: $root.removeMaxIndexOutputs.bind($root)\" aria-hidden=\"true\" title=\"Nullify Max Index Outputs Per Document And Hide The Control\">                        <i class=\"fa fa-times\"></i>                    </button>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: storeAllFields()\">                <label class=\"col-sm-1 control-label\">Store All Fields</label>                <div class=\"col-md-2\" style=\"padding-top: 3px\">                    <button style=\"height: 30px;\" type=\"button\" class=\"close pull-left\" data-bind=\"click: $root.removeStoreAllFields.bind($root)\" aria-hidden=\"true\" title=\"Remove Store All Fields And Hide The Control\">                        <i class=\"fa fa-times\"></i>                    </button>                </div>            </div>                        <div class=\"form-group\" data-bind=\"visible: $root.scriptedIndex\" id=\"indexScriptPopover\">                <label class=\"col-sm-1 control-label index-script-label\" data-placement=\"auto right\" data-original-title=\"Index Script\" data-trigger=\"hover\">                    Index Script                    <i class=\"fa fa-info-circle text-info\"></i>                </label>                <div class=\"col-sm-11\" data-bind=\"with: $root.scriptedIndex\">                    <div class=\"row\">                        <div class=\"col-md-11\">                            <pre class=\"form-control map\" data-bind=\"aceEditor: { readOnly: $parent.isSideBySideIndex(), code: indexScript, allowResize: true, lang: \'ace/mode/javascript\', completer: $root.scriptedIndexCompleter, completerHostObject: $root.scriptedIndexCompleter, typeName: \'map\' }\"></pre>                        </div>                    </div>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: $root.scriptedIndex\" id=\"deleteScriptPopover\">                <label class=\"col-sm-1 control-label delete-script-label\" data-placement=\"auto right\" data-original-title=\"Delete Script\" data-trigger=\"hover\">                    Delete Script                    <i class=\"fa fa-info-circle text-info\"></i>                </label>                <div class=\"col-sm-11\" data-bind=\"with: $root.scriptedIndex\">                    <div class=\"row\">                        <div class=\"col-md-11\">                            <pre class=\"form-control map\" data-bind=\"aceEditor: { readOnly: $parent.isSideBySideIndex(), code: deleteScript, allowResize: true, lang: \'ace/mode/javascript\', completer: $root.scriptedIndexCompleter, completerHostObject: $root.scriptedIndexCompleeter, typeName: \'map\' }\"></pre>                        </div>                    </div>                </div>            </div>        </form>    </div></section>'; } )
define("text!views/database/indexes/indexes.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        Indexes    </li></ul><div id=\"indexesContainer\">    <div class=\"btn-toolbar\" role=\"toolbar\" data-bind=\"visible: sortedGroups().length > 0\">        <div class=\"btn-group\">            <a class=\"btn btn-primary\" data-bind=\"attr: { href: newIndexUrl }\" title=\"Add a new index (Alt+N)\" accesskey=\"N\">                <i class=\"fa fa-plus\"></i> New index            </a>        </div>        <div class=\"btn-group\">            <a class=\"btn btn-default\" data-bind=\"attr: { href: queryUrl }\" title=\"New query (Alt+Q)\" accesskey=\"Q\"><i class=\"fa fa-search\"></i> Query</a>            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"visible: recentQueries().length > 0\" title=\"Recent queries\">                <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: recentQueries, visible: recentQueries().length > 0\">                <li>                    <a href=\"#\" data-bind=\"attr: { href: $root.getRecentQueryUrl($data) }\">                        <span data-bind=\"text: IndexName\"></span>                        <span data-bind=\"visible: QueryText\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: QueryText\"></span>                        </span>                        <span data-bind=\"visible: Sorts.length > 0, with: Sorts\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: \'sort by \' + $root.getRecentQuerySortText($data)\"></span>                        </span>                        <span data-bind=\"with: TransformerQuery\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: \'transform by \' + transformerName + \' \' + $root.getStoredQueryTransformerParameters(queryParams)\"></span>                        </span>                        <span data-bind=\"visible: UseAndOperator\">                            <span class=\"text-muted\">|</span>                            <span>AND operator</span>                        </span>                        <span data-bind=\"visible: ShowFields\">                            <span class=\"text-muted\">|</span>                            <span>show fields</span>                        </span>                        <span data-bind=\"visible: IndexEntries\">                            <span class=\"text-muted\">|</span>                            <span>index entries</span>                        </span>                    </a>                </li>            </ul>        </div>        <div class=\"btn-group\">            <a class=\"btn btn-default\" data-bind=\"click: toggleExpandAll , attr: { state: btnState, title: btnTitle }\" accesskey=\"=\">                <i data-bind=\"css: { \'fa-plus-square-o\': btnState, \'fa-minus-square-o\': !btnState() }\" class=\"fa\"></i>            </a>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                <i class=\"fa fa-clipboard\"></i><span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <li data-bind=\"click: pasteIndex\"><a href=\"#\" title=\"Paste index from JSON (Alt+V)\" accesskey=\"v\">Paste Single Index</a></li>                <li class=\"divider\"></li>                <li data-bind=\"click: copyIndexesAndTransformers\"><a href=\"#\" title=\"Paste index from JSON (Alt+V)\">Copy All Indexes And Transformers</a></li>                <li data-bind=\"click: pasteIndexesAndTransformers\"><a href=\"#\" title=\"Paste index from JSON (Alt+V)\">Paste Indexes And Transformers</a></li>            </ul>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Set lock mode to all indexes\">                <i class=\"fa fa-unlock-alt\"></i><span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <li data-bind=\"click: setLockModeAllIndexes.bind($root, \'Unlock\', \'Unlock\')\">                    <a href=\"#\" title=\"Unlock All Indexes\" data-bind=\"css: {\'index-lock-mode-disabled\': lockModeCommon() === \'Unlock\'}\">Unlock All Indexes</a>                </li>                <li class=\"divider\"></li>                <li data-bind=\"click: setLockModeAllIndexes.bind($root, \'SideBySide\', \'Lock Side By Side\')\">                    <a href=\"#\" title=\"Lock (Side By Side) All Indexes\" data-bind=\"css: {\'index-lock-mode-disabled\': lockModeCommon() === \'SideBySide\'}\">Lock (Side By Side) All Indexes</a>                </li>                <li data-bind=\"click: setLockModeAllIndexes.bind($root, \'LockedIgnore\', \'Lock\')\">                    <a href=\"#\" title=\"Lock All Indexes\" data-bind=\"css: {\'index-lock-mode-disabled\': lockModeCommon() === \'LockedIgnore\'}\">Lock All Indexes</a>                </li>                <li data-bind=\"click: setLockModeAllIndexes.bind($root, \'LockedError\', \'Locked Error\')\">                    <a href=\"#\" title=\"Lock All Indexes (Error)\" data-bind=\"css: {\'index-lock-mode-disabled\': lockModeCommon() === \'LockedError\'}\">Lock All Indexes (Error)</a>                </li>            </ul>        </div>        <div class=\"btn-group\">            <a class=\"btn btn-default\" title=\"Index merge suggestions\" data-bind=\"attr: { href: appUrls.megeSuggestions }\">                <i class=\"fa fa-compress\"></i>            </a>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-danger dropdown-toggle\" data-toggle=\"dropdown\">                <i class=\"fa fa-trash-o\"></i> <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <li data-bind=\"click: deleteIdleIndexes\"><a href=\"#\">Delete Idle Indexes</a></li>                <li data-bind=\"click: deleteDisabledIndexes\"><a href=\"#\">Delete Disabled Indexes</a></li>                <li data-bind=\"click: deleteAbandonedIndexes\"><a href=\"#\">Delete Abandoned Indexes</a></li>                <li class=\"divider\"></li>                <li data-bind=\"click: deleteAllIndexes\"><a href=\"#\">Delete All Indexes</a></li>            </ul>        </div>        <div class=\"btn-group\" data-bind=\"visible: corruptedIndexes().length > 0\">            <a class=\"btn btn-danger\" title=\"Try to recover corrupted indexes by reopening them (the index reset might apply)\" data-bind=\"click: tryRecoverCorruptedIndexes\">                Try to recover corrupted indexes            </a>        </div>        <div class=\"btn-group\">            <input type=\"text\" accesskey=\"/\" class=\"form-control\" placeholder=\"Search\" data-bind=\"value: searchText, valueUpdate: \'afterkeydown\', visible: indexGroups().length > 0\" title=\"Search for an index (Alt+/)\">        </div>    </div>    <div class=\"text-center\" data-bind=\"visible: sortedGroups().length == 0\">        <h3 class=\"page-header text-info\">            <div>No indexes have been created for this database.</div>            <div>                <small>                          Go ahead and <a href=\"#\" data-bind=\"attr: { href: newIndexUrl }\">create an index</a>                          , <a href=\"#\" data-bind=\"click: pasteIndex\">paste single index</a> or <a href=\"#\" data-bind=\"click: pasteIndexesAndTransformers\">paste a bulk</a> of indexes and transformers definitions from JSON.                      </small>            </div>        </h3>    </div>    <div data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" style=\"overflow: auto\">        <div class=\"indexes-container panel-group index-panel\" id=\"indexesAccordion\" data-bind=\"foreach: sortedGroups\">            <div class=\"index-group panel panel-default index-panel\" data-bind=\"visible: !groupHidden()\">                <div class=\"panel-heading\">                    <div class=\"pull-right indexGroup-actions-container\">                        <button class=\"btn btn-danger btn-sm\" data-bind=\"click: $root.deleteIndexGroup.bind($root), attr: { title: \'Delete all \' + entityName + \' indexes\' }\">                            <i class=\"fa fa-trash-o\"></i>                        </button>                    </div>                    <h4 class=\"panel-title\">                        <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#indexCollapse\' + $index() }\">                            <span data-bind=\"text: entityName\"></span>                            <span class=\"text-muted\" data-bind=\"text: \'(\' + indexes().length + \')\'\"></span>                        </a>                    </h4>                </div>                <div class=\"index-group-content panel-collapse in index-panel\" data-bind=\"attr: { id: \'indexCollapse\' + $index() }\">                    <div class=\"panel-body index-panel\" data-bind=\"foreach: indexes\" style=\"overflow: visible;\">                        <div class=\"index-panel panel panel-default\" data-bind=\"visible: !filteredOut()\">                            <div class=\"panel-body\">                                <h4 class=\"pull-left\">                                    <a data-bind=\"attr: { href: queryUrl }\" title=\"Query this index\">                                        <span data-bind=\"text: name, css: { \'text-warning\': isIdle() || isTestIndex(), \'text-muted\': isDisabled, \'text-danger\': isAbandoned() || isInvalid() || isErrored() }\"></span>                                    </a>                                    <small class=\"text-warning\" data-bind=\"visible: isIdle\">Idle</small>                                    <small class=\"text-danger\" data-bind=\"visible: isAbandoned\">Abandoned</small>                                    <small class=\"badge\" data-bind=\"visible: isInvalid\"><i class=\"fa fa-bug\" /> Invalid index</small>                                    <small class=\"badge\" data-bind=\"visible: isErrored\"><i class=\"fa fa-bug\" /> Corrupted index</small>                                    <small class=\"badge\" data-bind=\"visible: isTestIndex\">Test index</small>                                    <small class=\"badge\" data-bind=\"visible: willReplaceIndex()\"><i class=\"fa fa-arrow-left\"></i> Side-by-side index (new)</small>                                    <small class=\"badge\" data-bind=\"visible: willBeReplacedByIndex()\"><i class=\"fa fa-arrow-right\"></i> Side-by-side index (old)</small>                                    <small class=\"text-muted\" data-bind=\"visible: isDisabled\">Disabled</small>                                    <small class=\"text-muted\" data-bind=\"text: \' - \' + docsCount.toLocaleString() + \' entries\' \" title=\" the number of documents affected by this index\"></small>                                    <small data-bind=\"visible: isMapReduce\">                                        (<span data-bind=\"css: { \'text-warning\': isIdle() || isTestIndex(), \'text-muted\': isDisabled, \'text-danger\': isAbandoned() || isInvalid() || isErrored() }\">MapReduce</span>)                                    </small>                                    &nbsp;                                    <label>                                        <i class=\"fa\" data-bind=\"css: { \'fa-unlock\': lockMode() === \'Unlock\', \'text-danger\': lockMode() === \'LockedError\', \'fa-lock\': lockMode() === \'LockedIgnore\' || lockMode() === \'SideBySide\' || lockMode() === \'LockedError\' }\"></i>                                    </label>                                </h4>                                <div class=\"pull-right\">                                    <a data-bind=\"visible: willReplaceIndex(), click: $root.forceSideBySide.bind($root)\" href=\"#\" class=\"btn btn-danger\" title=\"Force side-by-side replacement right now\">                                        <i class=\"fa fa-bolt fa-fw\"></i>                                    </a>                                    <a class=\"btn btn-default indexes-controls\" data-placement=\"bottom\" data-bind=\"attr: { href: editUrl }\">                                        <i class=\"fa fa-pencil\"></i>                                    </a>                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-placement=\"left\">                                            <i class=\"fa fa-cogs\"></i>                                        </button>                                        <ul class=\"dropdown-menu index-dropdown\" role=\"menu\">                                            <li role=\"presentation\">                                                <a class=\"btn\" title=\"Copy the index\" data-bind=\"click: $root.copyIndex.bind($root)\">                                                    <i class=\"fa fa-copy fa-fw\"></i> Copy index                                                </a>                                            </li>                                            <li role=\"presentation\" data-bind=\"visible: !isSideBySideIndex()\">                                                <a class=\"btn\" title=\"Delete the index\" data-bind=\"click: $root.deleteIndex.bind($root)\">                                                    <i class=\"fa fa-trash-o fa-fw text-danger\"></i> Delete Index                                                </a>                                            </li>                                            <li role=\"presentation\" data-bind=\"visible: isSideBySideIndex()\">                                                <a class=\"btn\" title=\"Delete the index\" data-bind=\"click: $root.cancelIndex.bind($root)\">                                                    <i class=\"fa fa-undo fa-fw text-danger\"></i> Cancel side-by-side index                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn indexes-controls\" title=\"Reset the index\" data-bind=\"click: $root.resetIndex.bind($root)\">                                                    <i class=\"fa fa-undo fa-fw\"></i> Reset Index                                                </a>                                            </li>                                            <li role=\"presentation\" class=\"divider\"></li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.unlockIndex.bind($root, $data), css: { active: lockMode() === \'Unlock\' }\" title=\"Unlocked: The index is unlocked for changes; apps can modify it, e.g. via IndexCreation.CreateIndexes().\">                                                    <i class=\"fa fa-unlock fa-fw\"></i>                                                    Unlocked                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.lockSideBySide.bind($root, $data), css: { active: lockMode() === \'SideBySide\' }\" title=\"Side-by-side: The index is locked for changes; apps cannot modify it. Programmatic attempts to modify the index will be ignored. Index be can only updated via side-by-side index.\">                                                    <i class=\"fa fa-lock fa-fw\"></i>                                                    Locked (side-by-side)                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.lockIndex.bind($root, $data), css: { active: lockMode() === \'LockedIgnore\' }\" title=\"Locked: The index is locked for changes; apps cannot modify it. Programmatic attempts to modify the index will be ignored.\">                                                    <i class=\"fa fa-lock fa-fw\"></i>                                                    Locked                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.lockErrorIndex.bind($root, $data), css: { active: lockMode() === \'LockedError\' }\" title=\"Locked + Error: The index is locked for changes; apps cannot modify it. An error will be thrown if an app attempts to modify it.\">                                                    <i class=\"fa fa-lock text-danger fa-fw\"></i>                                                    Locked (Error)                                                </a>                                            </li>                                            <li role=\"presentation\" class=\"divider\"></li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.normalPriority.bind($root, $data), css: { active: priority === \'Normal\' }\" title=\"Normal\">                                                    <i class=\"fa fa-check fa-fw\"></i>                                                    Normal                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.idlePriority.bind($root, $data), css: { active: isIdle() }\" title=\"Idle\">                                                    <i class=\"fa fa-arrow-down fa-fw\"></i>                                                    Idle                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.disabledPriority.bind($root, $data), css: { active: isDisabled() }\" title=\"Disabled\">                                                    <i class=\"fa fa-ban fa-fw\"></i>                                                    Disabled                                                </a>                                            </li>                                            <li role=\"presentation\">                                                <a class=\"btn\" data-bind=\"click: $root.abandonedPriority.bind($root, $data), css: { active: isAbandoned() }\" title=\"Abandoned\">                                                    <i class=\"fa fa-arrow-left fa-fw\"></i>                                                    Abandoned                                                </a>                                            </li>                                        </ul>                                    </div>                                    <!--            COMMENTED OUT: In 3.0, this doesn\'t seem to do anything. Is it because the index creation and saving has changed?            <button class=\"btn btn-default btn-warning\" title=\"Make index persistent\" data-bind=\"click: $root.makeIndexPersistent.bind($root, $data), visible: isOnRam() !== \'false\'\">                <i class=\"fa fa-download\"></i>            </button>-->                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/indexes/indexesAndTransformersClipboardDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text:isPaste ? \'Paste Indexes And Transformers\':\'Copy Indexes And Transformers\'\"></h4>    </div>    <div class=\"modal-body\">        <h4>            <strong data-bind=\"text: isPaste === true ? \'CTRL+V, Enter \': \'CTRL+C, Enter \'\"></strong>            <span data-bind=\"text: isPaste === true ? \'to save\' : \'to copy\'\"></span>        </h4>        <div>            <pre data-bind=\"aceEditor: { code: json, lang: \'ace/mode/json\', getFocus: true, selectAll: true, readOnly: !isPaste, bubbleEscKey: true }\" style=\"width: 600px; height: 400px;\"></pre>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close, visible: isPaste\">Close</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: saveAll, text: isPaste ? \'Save All\' : \'Close\', enable: !!json(), attr: { title: isPaste ? \'Save All (Alt+S)\' : \'\', accesskey: isPaste ? \'S\' : \'\' }\"></button>    </div></div>'; } )
define("text!views/database/indexes/indexesShell.html", [], function () { return '<section data-bind=\"router: { cacheViews: false } \"></section>'; } )
define("text!views/database/indexes/indexLockAllConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h3 class=\"modal-title\">Are you sure?</h3>    </div>    <div class=\"modal-body\">        <p class=\"confirmationMessage\" data-bind=\"text: message\"></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: indexLockAllIndex\">Yes</button>    </div></div>'; } )
define("text!views/database/indexes/indexMergeSuggestions.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        <a data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>    </li>    <li class=\"active\">        <span>Merge Suggestions</span>    </li></ul><div id=\"index-merge-suggestions\" class=\"panel-group\">    <div class=\"panel index-group panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title text-info\">Suggestions</h4>        </div>        <div class=\"panel-body\" data-bind=\"visible: suggestions().length == 0\">            <h3 class=\"text-info text-center\"> No merge suggestions availiable.</h3>        </div>        <div class=\"panel-group panel-body\" id=\"accordion\" data-bind=\"foreach: suggestions, visible: suggestions().length > 0\">            <div data-bind=\"if: $data.isSurpassingIndex()\">                <div class=\"panel panel-default\">                    <div class=\"panel-heading\">                        <h4 class=\"panel-title\">                            <a data-toggle=\"collapse\" data-parent=\"#accordion\" data-bind=\"text: \'Surpassing index: \' + surpassingIndex, attr: { href: \'#collapse\' + $index() }\"></a>                        </h4>                    </div>                    <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'collapse\' + $index() }\">                        <div class=\"panel-body\">                            <div>Indexes to delete:</div>                            <ul data-bind=\"foreach: canDelete\">                                <li data-bind=\"text: $data\"></li>                            </ul>                            <button data-bind=\"click: $root.deleteIndexes.bind($root, $index())\" class=\"btn btn-primary\">Delete indexes</button>                        </div>                    </div>                </div>            </div>            <div data-bind=\"ifnot: $data.isSurpassingIndex()\">                <div class=\"panel panel-default\">                    <div class=\"panel-heading\">                        <h4 class=\"panel-title\">                            <a data-toggle=\"collapse\" data-parent=\"#accordion\" data-bind=\"text: \'Merge Suggestion for \' + collection, attr: { href: \'#collapse\' + $index() }\"></a>                        </h4>                    </div>                    <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'collapse\' + $index() }\">                        <div class=\"panel-body\">                            <div>Indexes to merge:</div>                            <ul data-bind=\"foreach: canMerge\">                                <li data-bind=\"text: $data\"></li>                            </ul>                            <a class=\"btn btn-primary\" href=\"#\" data-bind=\"attr: { id: id, href: $root.mergedIndexUrl(id()) }, click: $root.saveMergedIndex.bind($root, id(), $data)\" title=\"Navigate to Save Merged Index\"><i class=\"fa fa-external-link\"></i> Show Merged Index</a>                        </div>                    </div>                </div>            </div>        </div>    </div>    <div class=\"panel index-group panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title text-info\">                <a href=\"#collapseUnmergable\" data-toggle=\"collapse\">Unmergable (<span data-bind=\"text: unmergables().length\"></span>)</a>            </h4>        </div>        <div class=\"panel-collapse collapse\" id=\"collapseUnmergable\">            <div class=\"panel-body\">                <table class=\"table table-striped table-condensed\">                    <thead>                        <tr>                            <th>Index name</th>                            <th>Reason</th>                        </tr>                    </thead>                    <tbody data-bind=\"foreach: unmergables\">                        <tr>                            <td data-bind=\"text: indexName\"></td>                            <td data-bind=\"text: reason\"></td>                        </tr>                    </tbody>                </table>            </div>        </div>    </div>    <div class=\"panel index-group panel-default\" data-bind=\"visible: idleOrAbandonedIndexes().length > 0\">        <div class=\"panel-heading\">            <div class=\"pull-right indexGroup-actions-container\">                <button class=\"btn btn-danger btn-sm\" data-bind=\"click: deleteAllIdleOrAbandoned\" title=\"Delete all idle or abandoned indexes\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>            <h4 class=\"panel-title text-info\">Idle or Abandoned</h4>        </div>        <div class=\"panel-body\">            <ul data-bind=\"foreach: idleOrAbandonedIndexes\">                <li>                    <span class=\"index-name\" data-bind=\"text: Name\"></span>                    <span>: </span>                    <span data-bind=\"text: Priority\"></span>                    <button class=\"btn btn-danger btn-sm\" data-bind=\"click: $root.deleteIndex.bind($root, Name)\" title=\"Delete index\">                        <i class=\"fa fa-trash-o\"></i>                    </button>                </li>            </ul>        </div>    </div>    <div class=\"panel index-group panel-default\" data-bind=\"visible: notUsedForLastWeek().length > 0\">        <div class=\"panel-heading\">            <div class=\"pull-right indexGroup-actions-container\">                <button class=\"btn btn-danger btn-sm\" data-bind=\"click: deleteAllNotUsedForWeek\" title=\"Delete all indexes not used within last week\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>            <h4 class=\"panel-title text-info\">Not used for over a week.</h4>        </div>        <div class=\"panel-body\">            <ul data-bind=\"foreach: notUsedForLastWeek\">                <li>                    <span class=\"index-name\" data-bind=\"text: Name\"></span>                    <span>: </span>                    <span data-bind=\"text: LastQueryTimestampText\"></span>                    <button class=\"btn btn-danger btn-sm\" data-bind=\"click: $root.deleteIndex.bind($root, Name)\" title=\"Delete index\">                        <i class=\"fa fa-trash-o\"></i>                    </button>                </li>            </ul>        </div>    </div></div>'; } )
define("text!views/database/indexes/indexTerms.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li>        <a data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>    </li>    <li class=\"active\">        <a data-bind=\"text: indexName, attr: { href: appUrls.editIndex(indexName) }\"></a>    </li>    <li class=\"active\">        <span>Terms</span>    </li></ul><div class=\"panel-group panel-group-wo-margin\" id=\"accordion\" data-bind=\"foreach: fields, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title\">                <a data-toggle=\"collapse\" data-parent=\"#accordion\" data-bind=\"text: name, attr: { href: \'#termCollapse\' + $index() }\"></a>                <span class=\"text-warning padd-left-10\" data-bind=\"visible: hasMoreTerms\"><i class=\"fa fa-warning\"></i> Displaying only first <span data-bind=\"text: $root.termsPageLimit - 1\"></span> terms </span>            </h4>        </div>        <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'termCollapse\' + $index() }\">            <div class=\"panel-body\">                <ul class=\"list-group\" data-bind=\"foreach: terms\">                    <li class=\"list-group-item\" data-bind=\"text: $data\"></li>                </ul>                                <h4 class=\"text-center\" data-bind=\"visible: !terms().length\">No entries were found</h4>            </div>        </div>    </div></div>'; } )
define("text!views/database/indexes/replaceIndexDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 700px\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Replace index: <span data-bind=\"text: indexName\"></span></h4>    </div>    <div class=\"modal-body\">        <form class=\"form-horizontal\" role=\"form\" id=\"replaceIndexForm\">            <div class=\"form-group\">                <label class=\"col-md-4 control-label\">Replace conditions:</label>                <div class=\"col-md-8\">                    <button type=\"button\" class=\"btn btn-default\" disabled=\"disabled\">                        <i data-bind=\"css: { \'fa-check\': true }\" class=\"fa fa-fw\"></i> When new index become non-stale                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleEtagMode\">                        <i data-bind=\"css: { \'fa-check\': etagMode() }\" class=\"fa fa-fw\"></i> index etag reaches last indexed etag of old index                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleDateMode\">                        <i data-bind=\"css: { \'fa-check\': dateMode() }\" class=\"fa fa-fw\"></i> date is reached                    </button>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: dateMode()\">                <label class=\"col-md-4 control-label\">Replacement date (UTC):</label>                <div class=\"col-md-8\">                    <div class=\"input-group date\" data-bind=\"datePicker: replaceDate\">                        <input id=\"replaceDate\" type=\"text\"  class=\"form-control\" data-bind=\"value: replaceDateText\" />                        <span class=\"input-group-addon\">                            <span class=\"fa fa-calendar\"></span>                        </span>                    </div>                </div>            </div>        </form>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Close</button>        <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"click: saveReplace, enable: saveEnabled\">Save</button>    </div></div>'; } )
define("text!views/database/indexes/resetIndexConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Reset index?</h4>    </div>    <div class=\"modal-body\">        <p>You\'re resetting <strong data-bind=\"text: indexName\"></strong></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: resetIndex\">Yep, reset</button>    </div></div>'; } )
define("text!views/database/indexes/scriptedIndexes.html", [], function () { return '<div id=\"scriptedIndexes\" class=\"innerWindowsStyle\">    <form id=\"scriptedIndexesForm\" data-bind=\"submit: saveChanges\" class=\" form-horizontal\" role=\"form\">        <nav class=\"navbar navbar-default\" role=\"navigation\">            <div class=\"collapse navbar-collapse\">                <div class=\"navbar-form navbar-right\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"visible: inactiveIndexNames().length > 0\" title=\" Create Script\">                            <i class=\"fa fa-plus\"></i> Create Script for Index: <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <!-- ko foreach: inactiveIndexNames -->                            <li data-bind=\"click: $root.createScriptedIndex.bind($root, $data)\"><a href=\"#\" data-bind=\"text: $data\"></a></li>                            <!-- /ko -->                        </ul>                    </div>                    <div class=\"btn-group\">                        <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" accesskey=\"S\" data-bind=\"enable: isSaveEnabled\"><i class=\"fa fa-save\"></i></button>                    </div>                </div>            </div>        </nav>        <div class=\"alert alert-warning\" data-bind=\"visible: activeScriptedIndexes().length === 0\">            You\'ve got no scripted indexes set up.        </div>        <div class=\"panel-group\" id=\"accordion\" data-bind=\"foreach: activeScriptedIndexes\">            <div class=\"panel panel-default panel-info\">                <div class=\"panel-heading\">                    <a data-toggle=\"collapse\" data-parent=\"#accordion\" data-bind=\"attr: { \'href\': \'#scriptedIndex\' + $index() }\">                        <h3 class=\"panel-title\">                            <span data-bind=\"text: indexName\"></span>                            <span class=\"close\" aria-hidden=\"true\" data-bind=\"click: $root.removeScriptedIndex.bind($root, $data)\" title=\"Remove this Scripted Index\"><i class=\"fa fa-times\"></i></span>                        </h3>                    </a>                </div>                <div data-bind=\"attr: { \'id\': \'scriptedIndex\' + $index() }, css: { \'collapse\': ($index() !== $parent.firstIndex()), \'in\': ($index() === $parent.firstIndex()) }\" class=\"panel-collapse collapse\">                    <div class=\"panel-body\">                        <div class=\"form-group\">                            <label style=\"cursor: help;\" class=\"col-md-2 control-label index-script-label\" data-placement=\"auto right\" data-original-title=\"Index Script\" data-trigger=\"hover\">                                Index Script                                <i class=\"fa fa-info-circle text-info\"></i>                            </label>                            <div class=\"col-md-10\">                                <div class=\"row\">                                    <div class=\"col-md-11\">                                        <pre class=\"form-control map\" data-bind=\"aceEditor: { code: indexScript, lang: \'ace/mode/javascript\' }\"></pre>                                    </div>                                </div>                            </div>                        </div>                        <div class=\"form-group\">                            <label style=\"cursor: help;\" class=\"col-md-2 control-label delete-script-label\" data-placement=\"auto right\" data-original-title=\"Delete Script\" data-trigger=\"hover\">                                Delete Script                                <i class=\"fa fa-info-circle text-info\"></i>                            </label>                            <div class=\"col-md-10\">                                <div class=\"row\">                                    <div class=\"col-md-11\">                                        <pre class=\"form-control map\" data-bind=\"aceEditor: { code: deleteScript, lang: \'ace/mode/javascript\' }\"></pre>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/database/patch/executePatchConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Are you sure?</h4>    </div>    <div class=\"modal-body\">        <p><strong>Are you sure you want to apply this patch to all matching documents?</strong></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" autofocus data-bind=\"click: executePatch\">OK</button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>    </div></div>'; } )
define("text!views/database/patch/patch.html", [], function () { return '<div id=\"patchContainer\" class=\"no-outline\" tabindex=\"0\">    <!-- tabindex = 0 to that it can receive focus and catch keyboard shortcuts -->    <div class=\"btn-toolbar\" role=\"toolbar\">        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <div class=\"btn-group\" data-bind=\"with: patchDocument\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.isPatchingInProgress()\">                <span data-bind=\"text: patchOnOption\"></span>                <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <!-- ko foreach: patchOptions -->                <li data-bind=\"click: $root.setSelectedPatchOnOption.bind($root, $data)\"><a href=\"#\" data-bind=\"text: $data\"></a></li>                <!-- /ko -->            </ul>        </div>        <div class=\"btn-group\">            <button data-bind=\"enable: savedPatches().length && !$root.isPatchingInProgress()\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Load a saved patch\"><i class=\"fa fa-arrow-up\"></i> Load <span class=\"caret\"></span></button>            <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: savedPatches\">                <li>                    <a href=\"#\" data-bind=\"click: $root.usePatch.bind($root, $data)\">                        <span data-bind=\"text: name()\"></span>                    </a>                </li>            </ul>            <button class=\"btn btn-default\" title=\"Save the current patch\" data-bind=\"click: savePatch, enable: patchDocument().script() && !$root.isPatchingInProgress()\"><i class=\"fa fa-save\"></i> Save</button>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"visible: recentPatches().length > 0, disable: $root.isPatchingInProgress()\">                Recent Patches <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: recentPatches\">                <li>                    <a href=\"#\" data-bind=\"click: $root.useRecentPatch.bind($root, $data), attr: { title: Script }\">                        <span data-bind=\"text: PatchOnOption\"></span>                        <span data-bind=\"visible: SelectedItem\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: SelectedItem\"></span>                        </span>                        <span data-bind=\"visible: Query\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: Query\"></span>                        </span>                    </a>                </li>            </ul>        </div>        <div class=\"btn-group\">            <button class=\"btn btn-default\" title=\"Test the patch script on the selected document (Alt+T)\" data-bind=\"click: testPatch, enable: isExecuteAllowed() && !$root.isPatchingInProgress() && !$root.isMapReduceIndexSelected()\" accesskey=\"T\"><i class=\"fa fa-question-circle\"></i> Test</button>        </div>        <div class=\"btn-group\" data-bind=\"visible: patchDocument() && patchDocument().isDocumentPatch()\">            <button class=\"btn btn-primary\" title=\"Execute Patch (Alt+P)\" accesskey=\"P\" data-bind=\"click: executePatchOnSingle, enable: isExecuteAllowed() && !$root.isPatchingInProgress()\">                <i class=\"fa fa-play\"></i> Patch            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: (!!patchDocument()) && (patchDocument().isCollectionPatch() || patchDocument().isIndexPatch())\">            <button class=\"btn btn-primary\" title=\"Patch the selected documents (Alt+S)\" accesskey=\"S\" data-bind=\"click: executePatchOnSelected, enable: isExecuteAllowed() && !$root.isPatchingInProgress() && !$root.isMapReduceIndexSelected()\">                <i class=\"fa fa-play\"></i> Patch Selected            </button>            <button class=\"btn btn-primary\" title=\"Patch all matching documents (Alt+A)\" data-bind=\"click: executePatchOnAll, enable: patchDocument().script() && !$root.isPatchingInProgress()\" accesskey=\"A\">                <i class=\"fa fa-forward\"></i> Patch All            </button>        </div>    </div>    <br/>    <div class=\"row\">        <div class=\"col-md-12\">            <div class=\"progress\" data-bind=\"css: { active: isPatchingInProgress() }, visible: showPatchingProgress() && (!!patchDocument()) && (patchDocument().isDocumentPatch() == false)\">                <div class=\"progress-bar progress-bar-striped\" role=\"progressbar\" aria-valuemin=\"0\" aria-valuemax=\"100\" data-bind=\"style: {width: patchingProgressPercentage}, aria-valuenow: patchingProgress\">                    <span data-bind=\"text: patchingProgressText\" />                </div>            </div>        </div>    </div>    <div class=\"form-inline\">        <div class=\"form-group\" data-bind=\"visible: (!!patchDocument()) && patchDocument().isDocumentPatch()\">            <label for=\"documentToPatch\" class=\"padd-right-10\">Document to Patch:</label>            <span data-bind=\"with: patchDocument\">                <input id=\"documentToPatch\" accesskey=\"I\" type=\"text\" class=\"form-control\" data-bind=\"value: selectedItem, valueUpdate: \'afterkeydown\', event: { keyup: $root.loadDocumentToTest.bind($root, $element.value) }, enable: !$root.isPatchingInProgress()\" />            </span>        </div>        <div class=\"form-group\" data-bind=\"visible: (!!patchDocument()) && patchDocument().isCollectionPatch()\">            <label for=\"collectionToPatch\" class=\"padd-right-10\">Collection to Patch:</label>            <div class=\"btn-group\" data-bind=\"with: patchDocument\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.isPatchingInProgress()\">                    <span data-bind=\"text: selectedItem\"></span>                    <span class=\"caret\" data-bind=\"visible: $root.collections().length > 1\"></span>                </button>                <ul id=\"collectionToPatch\" class=\"dropdown-menu\" role=\"menu\">                    <!-- ko foreach: $root.collectionToSelect -->                    <li data-bind=\"click: $root.setSelectedCollection.bind($root, $data)\"><a href=\"#\" data-bind=\"text: $data.name\"></a></li>                    <!-- /ko -->                </ul>            </div>        </div>        <div class=\"form-group\" data-bind=\"visible: (!!patchDocument()) && patchDocument().isIndexPatch()\">            <label for=\"indexToPatch\" class=\"padd-right-10\">Index to Patch:</label>            <div class=\"btn-group\" data-bind=\"with: patchDocument\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.isPatchingInProgress()\">                    <span data-bind=\"text: selectedItem\"></span>                    <span class=\"caret\" data-bind=\"visible: $root.indices().length > 1\"></span>                </button>                <ul id=\"indexToPatch\" class=\"dropdown-menu\" role=\"menu\">                    <!-- ko foreach: $root.indicesToSelect -->                    <li data-bind=\"click: $root.useIndex.bind($root, $data.name)\"><a href=\"#\" data-bind=\"text: name\"></a></li>                    <!-- /ko -->                </ul>                <span class=\"text-height-fix padd-left-10\" data-bind=\"visible: $root.isMapReduceIndexSelected()\">                    <i class=\"fa fa-warning text-warning\"></i> <span class=\"text-muted\">\'Test\' and \'Patch selected\' are not available for MapReduce indexes.</span>                </span>            </div>        </div>    </div>    <div class=\"form-horizontal\">        <div class=\"form-group\" data-bind=\"visible: (!!patchDocument()) && patchDocument().isIndexPatch()\">            <div class=\"col-md-9\">                <div>                    <label id=\"indexQueryLabelContainer\" data-placement=\"auto right\" data-original-title=\"Index Query\" data-trigger=\"hover\">                        <span id=\"indexQueryLabel\" class=\"help-cursor\">                            Query                            <i class=\"fa fa-info-circle text-info\"></i>                        </span>                        <a target=\"_blank\" class=\"control-label-additional-link\" title=\"External link: Lucene query syntax documentation\" href=\"http://lucene.apache.org/core/2_9_4/queryparsersyntax.html\"><!--<i class=\"fa fa-question-circle text-muted\"></i>-->Lucene Docs</a>                    </label>                </div>                <div>                    <div data-bind=\"with: patchDocument\">                        <pre id=\"queryEditor\" class=\"form-control editor\" data-bind=\"aceEditor: { code: $root.queryText, readOnly: $root.isPatchingInProgress(), typeName:\'queryText\', allowResize: true, minHeight: 160, maxHeight: 200, completer:$root.queryCompleter, lang:\'ace/mode/lucene\', completerHostObject:$root}, valueUpdate: \'afterkeydown\'\"></pre>                    </div>                </div>            </div>        </div>        <div class=\"form-group\">            <div class=\"col-md-9\">                <div>                    <label id=\"patchScriptsLabelContainer\" data-placement=\"auto right\" data-original-title=\"Patch Scripts\" data-trigger=\"hover\">                        <span id=\"patchScriptsLabel\" class=\"help-cursor\">                            Patch Scripts                            <i class=\"fa fa-info-circle text-info\"></i>                        </span>                    </label>                </div>                <div>                    <div class=\"row\">                        <div class=\"col-md-12\" data-bind=\"with: patchDocument\">                            <pre class=\"form-control\" data-bind=\"aceEditor: { code: script, lang:\'ace/mode/javascript\', readOnly: $root.isPatchingInProgress() }\" style=\"height: 160px;\"></pre>                        </div>                    </div>                </div>            </div>            <div class=\"col-md-3\">                <div>                    <label>Parameters</label>                </div>                <div data-bind=\"with: patchDocument\">                    <table class=\"table table-striped table-hover\" data-bind=\"visible: parameters().length > 0\">                        <thead>                            <tr>                                <td width=\"35%\">Name</td>                                <td>Value</td>                                <td></td>                            </tr>                        </thead>                        <tbody data-bind=\"foreach: parameters\">                            <tr>                                <td>                                    <input id=\"parametersName\" type=\"text\" class=\"form-control\" data-bind=\"value: key, disable: $root.isPatchingInProgress()\" />                                </td>                                <td>                                    <input id=\"parametersValue\" type=\"text\" class=\"form-control\" data-bind=\"value: value, disable: $root.isPatchingInProgress()\" />                                </td>                                <td>                                    <button type=\"button\" class=\"close\" data-bind=\"click: $parent.removeParameter.bind($parent, $data), disable: $root.isPatchingInProgress()\" title=\"Remove this parameter\">                                        <i class=\"fa fa-times\"></i>                                    </button>                                </td>                            </tr>                        </tbody>                    </table>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createParameter, disable: $root.isPatchingInProgress()\" title=\"Add a parameter\">                        <i class=\"fa fa-plus\"></i> Add parameter                    </button>                </div>            </div>        </div>        <div id=\"matchingDocumentsContainer\" class=\"form-group\" data-bind=\"if: showDocumentsPreview\">            <div class=\"col-md-12\">                <div>                    <label>Matching Documents</label>                </div>                <div id=\"matchingDocumentsGrid\" style=\"width: 100%; height: 300px\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: currentCollectionPagedItems, dynamicHeightBottomMargin: 0, dynamicHeightTargetSelector: \'#beforeAfterContainer\', selectedIndices: selectedDocumentIndices, gridSelector: \'#matchingDocumentsGrid\' }\"></div>            </div>        </div>        <div class=\"form-group patch-preview-flag\" data-bind=\"with: $root\" id=\"beforeAfterContainer\">            <div class=\"col-md-6\">                <div>                    <label>Before Patch</label>                </div>                <div>                    <div class=\"row\">                        <div class=\"col-md-12\">                            <ul class=\"nav nav-tabs\">                                <li data-bind=\"click: activateBeforeDoc, css: { active: beforePatchDocMode()}\" title=\"View document data\"><a style=\"cursor: pointer;\">Data</a></li>                                <li data-bind=\"click: activateBeforeMeta, css: { active: !beforePatchDocMode() }\" title=\"View document metadata\"><a style=\"cursor: pointer;\">Metadata</a></li>                            </ul>                            <pre id=\"beforePatchEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: beforePatch, readOnly: true }\" style=\"height: 260px;\"></pre>                        </div>                    </div>                </div>            </div>            <div class=\"col-md-6\">                <div>                    <label>After Patch</label>                </div>                <div>                    <div class=\"row\">                        <div class=\"col-md-12\">                            <ul class=\"nav nav-tabs\">                                <li data-bind=\"click: activateAfterDoc, css: { active: afterPatchDocMode()}\" title=\"View document data\"><a style=\"cursor: pointer;\">Data</a></li>                                <li data-bind=\"click: activateAfterMeta, css: { active: !afterPatchDocMode() }\" title=\"View document metadata\"><a style=\"cursor: pointer;\">Metadata</a></li>                            </ul>                            <pre id=\"afterPatchEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: afterPatch, readOnly: true }\" style=\"height: 260px;\"></pre>                        </div>                    </div>                </div>                <div class=\"tabs-container\">                    <ul class=\"nav nav-tabs\" role=\"tablist\">                        <li class=\"active\"><a href=\"#patchPut\" role=\"tab\" data-toggle=\"tab\">Document Puts <span data-bind=\"text: putDocuments().length\" class=\"badge\"></span></a></li>                        <li><a href=\"#patchLoad\" role=\"tab\" data-toggle=\"tab\">Document Loads <span data-bind=\"text: loadedDocuments().length\" class=\"badge\"></span></a></li>                        <li><a href=\"#patchDebug\" role=\"tab\" data-toggle=\"tab\">Output <span data-bind=\"text: outputLog().length\" class=\"badge\"></span></a></li>                    </ul>                    <!-- Tab panes -->                    <div class=\"tab-content\">                        <div class=\"tab-pane active jsContent\" id=\"patchPut\" data-bind=\"foreach: putDocuments\">                            <pre data-bind=\"html: $data\"></pre>                        </div>                        <div class=\"tab-pane\" id=\"patchLoad\" data-bind=\"visible: loadedDocuments().length\">                            <pre><!-- ko foreach: loadedDocuments --><span data-bind=\"text: $data\"></span><!-- /ko --></pre>                        </div>                        <div class=\"tab-pane\" id=\"patchDebug\" data-bind=\"visible: outputLog().length\">                            <pre><!-- ko foreach: outputLog --><span data-bind=\"text: $data\"></span><!-- /ko --></pre>                        </div>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/patch/savePatch.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: saveThePatch\" id=\"select-columns-form\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Save Patch</h4>        </div>                <div class=\"modal-body\">            <div class=\"row\">                <div class=\"col-md-12\">                    <div class=\"form-group\">                        <label for=\"name\">Please enter a name</label>                        <input name=\"name\" class=\"form-control\" required type=\"text\" data-bind=\"value: patchName, valueUpdate: \'afterkeydown\'\" />                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" class=\"btn btn-primary\">OK</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/database/query/deleteDocumentsMatchingQueryConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Delete docs matching query?</h4>    </div>    <div class=\"modal-body\">        <p>            <span>You\'re deleting </span>            <strong data-bind=\"text: totalDocCount.toLocaleString()\"></strong>            <span data-bind=\"visible: totalDocCount > 1, text: \' documents\'\"></span>            <span data-bind=\"visible: totalDocCount == 1, text: \' document\'\"></span>        </p>        <p class=\"text-muted\">These are all the docs in <strong data-bind=\"text: indexName\"></strong> <span data-bind=\"visible: queryText\">matching <pre data-bind=\"text: queryText, visible: queryText\"></pre></span></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteDocs\">Yep, delete</button>    </div></div>'; } )
define("text!views/database/query/query.html", [], function () { return '<div id=\"queryContainer\" class=\"no-outline\" tabindex=\"0\">    <ul class=\"breadcrumb\">        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <li>            <h3>                <a href=\"#resources\">                    Databases                </a>            </h3>        </li>        <li data-bind=\"with: activeDatabase\">            <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>        </li>        <li>            <a data-bind=\"attr: { href: appUrls.indexes }\">Indexes</a>        </li>        <li>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    <span data-bind=\"text: selectedIndexLabel\"></span>                    <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\" style=\"max-height: 300px; overflow-x: hidden;\">                    <!-- ko foreach: indexesExceptCurrent -->                    <li data-bind=\"click: $root.setSelectedIndex.bind($root, $data.name)\"><a href=\"#\" data-bind=\"text: $data.name\"></a></li>                    <!-- /ko -->                    <li class=\"divider\" data-bind=\"visible: collectionNamesExceptCurrent().length\"></li>                    <!-- ko foreach: collectionNamesExceptCurrent -->                    <li data-bind=\"click: $root.setSelectedIndex.bind($root, \'dynamic/\' + $data)\">                        <a href=\"#\">                            <span data-bind=\"text: \'dynamic/\' + $data\"></span>                        </a>                    </li>                    <!-- /ko -->                    <li class=\"divider\"></li>                    <li data-bind=\"click: $root.setSelectedIndex.bind($root, \'dynamic\')\">                        <a href=\"#\">                            <span>dynamic/All Documents</span>                        </a>                    </li>                </ul>            </div>        </li>        <li class=\"active\">Query</li>    </ul>    <div class=\"btn-toolbar\" role=\"toolbar\">        <div class=\"btn-group\">            <button class=\"btn btn-primary\" title=\"Run the query (Alt+R)\" data-bind=\"enable: hasSelectedIndex, click: runQuery\"><i class=\"fa fa-play\"></i></button>            <button class=\"btn btn-default\" data-bind=\"enable: hasSelectedIndex, click: toggleCacheEnable, attr: { title:isCacheDisable() ? \'Enable Cache\' : \'Disable Cache\' }\">                <span data-bind=\"text: isCacheDisable() ? \'Cache Disabled\' : \'Cache Enabled\'\"></span>            </button>        </div>        <div class=\"btn-group\">            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Add a transformer or sorting option to the query (Alt+A)\" accesskey=\"A\" data-trigger=\"hover\" data-placement=\"bottom\" data-bind=\"enable: hasSelectedIndex\">                    <i class=\"fa fa-plus\"></i> Add <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li data-bind=\"visible: selectedIndex() !== \'All Documents\'\"><a href=\"#\" data-bind=\"click: addSortBy\" accesskey=\"O\" title=\"Add a field to sort by (Alt+O)\">Sort By</a></li>                    <li data-bind=\"visible: !transformer()\"><a href=\"#\" data-bind=\"click: addTransformer\" accesskey=\"T\" title=\"Add a transformer to transform the results of the query (Alt+T)\" data-placement=\"right\">Transformer</a></li>                </ul>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Configure advanced query options\" data-bind=\"enable: hasSelectedIndex\">                    <i class=\"fa fa-cog\"></i> <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li style=\"padding-left: 20px;\">                        <div class=\"btn-group\">                            <label class=\"btn btn-default\" data-bind=\"click: setOperatorOr, css: { active: isDefaultOperatorOr }\" title=\"Set the default operator to OR\">Or</label>                            <label class=\"btn btn-default\" data-bind=\"click: setOperatorAnd, css: { active: !isDefaultOperatorOr() }\" title=\"Set the default operator to AND\">And</label>                        </div>                    </li>                    <li><a href=\"#\"><label class=\"btn btn-default\" data-bind=\"click: toggleShowFields, css: { active: showFields }\" title=\"Show fields from matching index entries instead of matching documents\">Show Fields</label></a></li>                    <li><a href=\"#\"><label class=\"btn btn-default\" data-bind=\"click: toggleIndexEntries, css: { active: indexEntries }\" title=\"Show the raw index entries instead of matching documents\">Index Entries</label></a></li>                </ul>            </div>            <a class=\"btn btn-default\" data-bind=\"attr: { href: editIndexUrl }, visible: hasEditableIndex\" title=\"Edit the index (F2)\"><i class=\"fa fa-pencil\"></i></a>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: selectColumns\" title=\"Choose columns...\">                <i class=\"fa fa-table\"></i>            </button>            <button data-bind=\"visible: isStaticIndexSelected, click: exportCsv\" class=\"btn btn-default\" title=\"Export CSV\">                <i class=\"fa fa-file-excel-o\"></i>            </button>            <a class=\"btn btn-default\" data-bind=\"attr: { href: visualizerUrl }, visible: isIndexMapReduce\" title=\"Map-Reduce visualizer\">                <i class=\"fa fa-sitemap\"></i>            </a>            <a class=\"btn btn-default\" data-bind=\"attr: { href: indexPerfStatsUrl }\" title=\"Indexing performance statistics\">                <i class=\"fa fa-line-chart\" />            </a>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"visible: recentQueries().length > 0\">                Recent Queries <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: recentQueries\">                <li>                    <a href=\"#\" data-bind=\"click: $root.runRecentQuery.bind($root, $data)\">                        <span data-bind=\"text: IndexName === \'dynamic\' ? \'All Documents\' : IndexName\"></span>                        <span data-bind=\"visible: QueryText\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: QueryText\"></span>                        </span>                        <span data-bind=\"visible: Sorts.length, with: Sorts\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: \'sort by \' + $root.getRecentQuerySortText($data)\"></span>                        </span>                        <span data-bind=\"with: TransformerQuery\">                            <span class=\"text-muted\">|</span>                            <span data-bind=\"text: \'transform by \' + transformerName + \' \' + $root.getStoredQueryTransformerParameters(queryParams)\"></span>                        </span>                        <span data-bind=\"visible: UseAndOperator\">                            <span class=\"text-muted\">|</span>                            <span>AND operator</span>                        </span>                        <span data-bind=\"visible: ShowFields\">                            <span class=\"text-muted\">|</span>                            <span>show fields</span>                        </span>                        <span data-bind=\"visible: IndexEntries\">                            <span class=\"text-muted\">|</span>                            <span>index entries</span>                        </span>                    </a>                </li>            </ul>        </div>        <div class=\"btn-group\">            <a class=\"btn btn-default\" data-bind=\"attr: { href: termsUrl }, enable: hasSelectedIndex\" title=\"Navigate to index terms\">Terms</a>            <a class=\"btn btn-default\" data-bind=\"enable: !!queryStats(), click: openQueryStats\" title=\"Show Query Stats In Dialog\">Query Stats</a>        </div>        <div class=\"btn-group\" title=\"Delete all documents matching the query (map index only)\">            <button class=\" btn btn-danger\" data-bind=\"enable: enableDeleteButton, click: deleteDocsMatchingQuery\">                <i class=\"fa fa-trash-o\"></i>            </button>        </div>    </div>    <br />    <div class=\"alert alert-warning\" data-bind=\"visible: isTestIndex()\" role=\"alert\">This is test index which operates on limited data set.</div>    <form class=\"form-horizontal container-fluid\" role=\"form\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">        <div class=\"form-group\">            <label id=\"indexQueryLabelContainer\" class=\"col-sm-1 control-label\" data-placement=\"auto right\" data-original-title=\"Index Query\" data-trigger=\"hover\">                <span id=\"indexQueryLabel\" class=\"help-cursor\">                    Query                    <i class=\"fa fa-info-circle text-info\"></i>                </span>                <a target=\"_blank\" class=\"label control-label-additional-link\" title=\"External link: Lucene query syntax documentation\" href=\"http://lucene.apache.org/core/2_9_4/queryparsersyntax.html\"><!--<i class=\"fa fa-question-circle text-muted\"></i>-->Lucene Docs</a>            </label>            <div class=\"col-md-11\">                <div class=\"row\">                    <div class=\"col-md-11\">                        <pre id=\"queryEditor\" class=\"form-control editor\" data-bind=\"aceEditor: { code: queryText, allowResize: true, minHeight: 60, maxHeight: 200, completer:queryCompleter, typeName:\'query\', lang:\'ace/mode/lucene\', completerHostObject:$root}, valueUpdate: \'afterkeydown\'\"></pre>                    </div>                </div>            </div>        </div>        <div class=\"form-group\" data-bind=\"visible: sortBys().length > 0\">            <label class=\"col-sm-1 control-label\">                Sort By            </label>            <div class=\"col-md-9\" >                <div class=\"row\" data-bind=\"foreach: sortBys\">                    <div class=\"col-xs-12 col-sm-9 col-md-6 col-lg-5\">                        <div class=\"panel panel-default\">                            <div class=\"panel-body\">                                <div>                                    <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeSortBy.bind($root, $data)\" title=\"Remove this sort from the query\">                                        <i class=\"fa fa-times\"></i>                                    </button>                                </div>                                <div class=\"col-md-11\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"text: fieldNameOrDefault\"></span> <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: $root.indexFields\">                                            <li><a href=\"#\" data-bind=\"text: $data, click: $parent.fieldName.bind($data)\"></a></li>                                        </ul>                                    </div>                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: isAscending() && !isRange()\">Ascending</span>                                            <span data-bind=\"visible: !isAscending() && !isRange()\">Descending</span>                                            <span data-bind=\"visible: isAscending() && isRange()\">Range Ascending</span>                                            <span data-bind=\"visible: !isAscending() && isRange()\">Range Descending</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li data-bind=\"if: !(isAscending() && isRange() == false)\">                                                <a href=\"#\" data-bind=\"click: function() { $data.isAscending(true); $data.isRange(false); }\">Ascending</a>                                            </li>                                            <li data-bind=\"if: !(isAscending() == false && isRange() == false)\">                                                <a href=\"#\" data-bind=\"click: function() { $data.isAscending(false); $data.isRange(false); }\">Descending</a>                                            </li>                                            <li data-bind=\"if: !(isAscending() && isRange()) \">                                                <a href=\"#\" data-bind=\"click: function() { $data.isAscending(true); $data.isRange(true); }\">Range Ascending</a>                                            </li>                                            <li data-bind=\"if: !(isAscending() == false && isRange())\">                                                <a href=\"#\" data-bind=\"click: function() { $data.isAscending(false); $data.isRange(true); }\">Range Descending</a>                                            </li>                                        </ul>                                    </div>                                </div>                                <div class=\"clearfix\"></div>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"form-group\" data-bind=\"visible: transformer()\">            <label id=\"indexQueryLabelContainer\" style=\"overflow-x: hidden; text-overflow: ellipsis;\" class=\"col-md-1 control-label\">                <span>Transformer</span>            </label>            <div class=\"col-xs-12 col-sm-9 col-md-6 col-lg-3\">                <div class=\"panel panel-default\">                    <div class=\"panel-heading\">                        <div class=\"row\">                            <div class=\"btn-group col-md-11\">                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"text: transformer() && transformer().name() ? transformer().name() : \'Select a transformer\'\"></span> <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: $root.allTransformers\">                                    <li><a href=\"#\" data-bind=\"text: name, click: $root.selectTransformer.bind($root, $data)\"></a></li>                                </ul>                            </div>                            <div class=\"col-md-1\">                                <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeTransformer.bind($root)\" title=\"Remove this transformer from the query\">                                    <i class=\"fa fa-times\"></i>                                </button>                            </div>                        </div>                                                <div class=\"clearfix\"></div>                    </div>                    <!-- ko if: typeof transformer() !== \"undefined\" && transformer() != null && transformer().extractInputs().length > 0 -->                    <div class=\"panel-body\">                        <div class=\"btn-group col-md-12\">                            <div class=\"clearfix\"></div>                            <div id=\"transformerParams\" data-bind=\"foreach: transformer().extractInputs()\">                                <div class=\"form-group row\">                                    <label data-bind=\"text: \'Parameter: \'+ $data.name\" class=\"control-label\"></label>                                    <input type=\"text\" class=\"form-control transformer_param_flag\" data-bind=\"attr: { name: $data.name, required: $data.hasDefault ? null : \'\' }, event: { change: $root.runQuery.bind($root) }\" />                                </div>                            </div>                        </div>                    </div>                    <!-- /ko -->                </div>            </div>        </div>        <!-- Warning should apear here-->        <div class=\"form-group\" data-bind=\"visible:containsAsterixQuery()\">            <span class=\"col-md-offset-1 col-md-10 text-danger\"><i class=\"fa fa-warning\"></i>  The Query contains \'*.*\', did you intend to use \'*:*\' ?</span>        </div>        <div class=\"form-group\" data-bind=\"visible:isWarning()\">            <span class=\"col-md-offset-1 col-md-10 text-danger\" ><i class=\"fa fa-warning\"></i> <span data-bind=\"text:warningText()\"></span></span>        </div>        <!-- query predefined filters should apear here -->        <div class=\"form-group\">            <label class=\"col-sm-1 control-label\" data-placement=\"auto right\" data-original-title=\"Index Query Search Filters\" data-trigger=\"hover\">                <span  class=\"help-cursor\">                    Query Filters                </span>                        </label>            <div class=\"col-md-11\">                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"text:searchField()\"></span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: $root.indexFields\">                        <li><a href=\"#\" data-bind=\"text: $data, click:$root.searchField.bind($data)\"></a></li>                    </ul>                    <button id=\"fieldStartsWith\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fieldNameStartsWith, enable: searchField() !==\'Select field\'\">String filter</button>                    <!-- Support numric, string and date time ranges-->                    <button id=\"inRange\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fieldValueRange, enable: searchField() !==\'Select field\'\">Range filter</button>                    <button id=\"inCollection\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fieldValueInMethod, enable: searchField() !==\'Select field\'\">In filter</button>                </div>            </div>                   </div>                    <div class=\"form-group\">            <div class=\"form-group\" data-bind=\"if:(isTestIndex() == false && !!$root.queryStats() && $root.queryStats().IsStale == true)\">                <span class=\"col-md-offset-1 text-danger\">Index is stale, possibly not all results presented</span>            </div>        </div>        <div class=\"form-group\">            <div class=\"form-group col-md-12\">                <label id=\"indexQueryLabelContainer\" class=\"control-label col-md-1\">                    Results                </label>                <div class=\"col-md-11 query-results\">                    <div id=\"queryResultsPanel\" class=\"panel panel-default\">                        <div class=\"panel-body\" data-bind=\"visible: isLoading\">                            <span>Loading, please wait <i class=\"fa fa-spinner fa-spin fa-2x\"></i></span>                        </div>                        <div class=\"panel-body\">                            <div class=\"suggestion-box\" data-bind=\"visible: $root.showSuggestions, foreach: $root.indexSuggestions\">                                <div>Did you mean <a data-bind=\"text: Suggestion, click: $root.applySuggestion.bind($root, $data)\" href=\"#\"></a> for the field <i data-bind=\"text: FieldName\"></i>?</div>                            </div>                            <div id=\"queryResultsGrid\" data-bind=\"widget: { kind: \'virtualTable\', customColumns: currentColumnsParams, itemsSource: queryResults, selectedIndices: selectedResultIndices, gridSelector: \'#queryResultsGrid\', useContextMenu: true, showIds: true, showCheckboxes: true, maxHeight: \'450px\', isIndexMapReduce: isIndexMapReduce, collections: collections, contextMenuOptions: [\'copyitems\', \'CopyIDs\' , \'Delete\', \'EditItem\' ], noResultsMessage: \'No results were found. For NotAnalyzed fields you may wish to search with [[term]].\'}\"></div>                        </div>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/database/query/queryShell.html", [], function () { return '<section data-bind=\"router: { cacheViews: false }\"></section>'; } )
define("text!views/database/query/queryStatsDialog.html", [], function () { return '<div class=\"messageBox query-stats\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Query Stats</h4>    </div>    <div class=\"modal-body\" data-bind=\"with: queryStats\">                <form class=\"form-horizontal\" role=\"form\">            <div class=\"form-group\">                <label class=\"col-md-3 control-label\">Total Results:</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\" data-bind=\"text: !!TotalResults ? TotalResults.toLocaleString() : 0\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-3 control-label\">Status</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\" data-bind=\"text: IsStale ? \'Stale\' : \'Up to date\'\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-3 control-label\">Duration:</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\" data-bind=\"text: ((typeof (DurationMilliseconds) !== \'undefined\') ? DurationMilliseconds : \'??? \') + \'ms\'\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-3 control-label\">Index:</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\"><a target=\"_blank\" data-bind=\"text: IndexName, attr: { href: $root.selectedIndexEditUrl}\">Click Here</a></p>                </div>            </div>            <div class=\"form-group\" data-bind=\"visible: $root.didDynamicChangeIndex\">                <label class=\"col-md-3 control-label\">Index Explain:</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\">                        <a target=\'_blank\' data-bind=\"attr: { href: $root.rawJsonUrl  + \'&explain=true\'}\">Click Here</a>                    </p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-3 control-label\">ETag:</label>                <div class=\"col-md-9\">                    <p class=\"form-control-static\" data-bind=\"text: ResultEtag\"></p>                </div>            </div>        </form>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: cancel\">OK</button>    </div></div>'; } )
define("text!views/database/reporting/reporting.html", [], function () { return '<div id=\"reportingContainer\">    <ul class=\"breadcrumb\">        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <li>            <h3>                      <a href=\"#resources\">                    Databases                </a>            </h3>        </li>        <li data-bind=\"with: activeDatabase\">            <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>        </li>        <li>            <a data-bind=\"attr: { href: appUrls.reporting }\">Reporting</a>        </li>        <li>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    <span data-bind=\"text: selectedIndexLabel\"></span>                    <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: indexNames\">                    <li data-bind=\"click: $root.setSelectedIndex.bind($root, $data)\"><a href=\"#\" data-bind=\"text: $data\"></a></li>                </ul>            </div>        </li>    </ul><div class=\"btn-toolbar\" role=\"toolbar\">    <div class=\"btn-group\">        <button class=\"btn btn-primary\" title=\"Run the report (Alt+R)\" accesskey=\"R\" data-bind=\"enable: hasSelectedIndex, click: runReport\">            <i class=\"fa fa-play\"></i>        </button>    </div>    <div class=\"btn-group\">        <a class=\"btn btn-default\" title=\"Edit the index (F2)\" data-bind=\"attr: { href: editSelectedIndexUrl }\">            <i class=\" fa fa-pencil\"></i>        </a>    </div>    <button class=\"btn btn-default\" title=\"Toggle cache enabled\" data-bind=\"enable: hasSelectedIndex, click: toggleCacheEnable\">        <span data-bind=\"text: isCacheDisable()?\'Enable cache\':\'Disable cache\'\"></span>    </button>    <div class=\"btn-group\">        <button class=\"btn btn-default\" title=\"Export CSV\" data-bind=\"click: exportCsv, enable: isExportEnabled\">            <i class=\"fa fa-file-excel-o\"></i>        </button>    </div></div><br /><form class=\"form-horizontal\" role=\"form\">    <div class=\"form-group\">        <label class=\"col-sm-1 control-label\">Group By</label>        <div class=\"col-md-5\">            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" accesskey=\"G\" title=\"Group By (Alt+G)\">                    <span data-bind=\"text: selectedFieldLabel\"></span> <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: availableFields\">                    <li data-bind=\"click: $root.setSelectedField.bind($root, $data)\">                        <a href=\"#\" data-bind=\"text: $data\"></a>                    </li>                </ul>            </div>        </div>    </div>    <div class=\"form-group\">        <label class=\"col-sm-1 control-label\">            Values        </label>        <div class=\"col-md-6\">            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Add a value (Alt+V)\" accesskey=\"V\">                    <i class=\"fa fa-plus\"></i> Add a value <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: availableFields\">                    <li data-bind=\"click: $root.addValue.bind($root, $data)\">                        <a href=\"#\" data-bind=\"text: $data\"></a>                    </li>                </ul>            </div>            <br />            <br />            <ul class=\"list-group col-md-6\" data-bind=\"foreach: addedValues, visible: addedValues().length > 0\">                <li class=\"list-group-item\">                    <strong data-bind=\"text: aggregationField\"></strong>                    <span class=\"text-muted\"> summarized by </span>                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"text: aggregationLabel\"></span> <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li data-bind=\"click: setAggregationToCount\"><a href=\"#\">Count</a></li>                            <li data-bind=\"click: setAggregationToMin\"><a href=\"#\">Min</a></li>                            <li data-bind=\"click: setAggregationToMax\"><a href=\"#\">Max</a></li>                            <li data-bind=\"click: setAggregationToSum\"><a href=\"#\">Sum</a></li>                            <li data-bind=\"click: setAggregationToAverage\"><a href=\"#\">Average</a></li>                        </ul>                    </div>                    <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeValue.bind($root, $data)\" aria-hidden=\"true\" title=\"Remove this value\">                        <i class=\"fa fa-times\"></i>                    </button>                </li>            </ul>        </div>        <div>        </div>    </div>    <div class=\"form-group\">        <label class=\"col-sm-1 control-label\">            <span id=\"filterQueryLabel\" class=\"help-cursor\">                Filter                <i class=\"fa fa-info-circle text-info\"></i>            </span>        </label>        <div class=\"col-md-11\">            <button class=\"btn btn-default\" title=\"Add a filter (Alt+F)\" data-bind=\"visible: !hasFilter(), click: hasFilter.bind(true)\">                <i class=\"fa fa-plus\"></i> Add a filter            </button>            <div data-bind=\"visible: hasFilter\">                <div class=\"row\">                    <div class=\"col-md-11\">                        <pre class=\"form-control\" style=\"height: 100px;\" data-bind=\"aceEditor: { code: filter }\"></pre>                    </div>                    <div class=\"col-md-1\">                        <button type=\"button\" class=\"close pull-left\" title=\"Remove the filter\" data-bind=\"click: hasFilter.bind(hasFilter, false)\">                            <i class=\"fa fa-times\"></i>                        </button>                    </div>                </div>            </div>        </div>    </div>    <div id=\"reportGridGroup\" class=\"form-group\">        <label id=\"indexQueryLabelContainer\" class=\"col-sm-1 control-label\">            Results        </label>        <div class=\"col-md-11\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <table class=\"table table-condensed\" style=\"margin-bottom: 0;\">                        <thead>                            <tr>                                <th>Total</th>                                <th>Duration</th>                            </tr>                        </thead>                        <tbody>                            <tr data-bind=\"visible: reportResults\">                                <td data-bind=\"text: totalQueryResults\"></td>                                <td data-bind=\"text: queryDuration\"></td>                            </tr>                        </tbody>                    </table>                </div>                <div class=\"panel-body\">                    <div id=\"reportResultsGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: reportResults, gridSelector: \'#reportResultsGrid\', useContextMenu: false, showIds: false, showCheckboxes: false, maxHeight: \'300px\' }\"></div>                </div>            </div>        </div>    </div></form></div>'; } )
define("text!views/database/settings/customFunctionsEditor.html", [], function () { return '<form class=\"custom-functions-form\" id=\"customFunctionsContainer\">        <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"click: saveChanges, enable: isSaveEnabled\" title=\"Save Changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <span style=\"margin-left: 10px\">                You can create custom JavaScript functions here by writing simple <a href=\"http://wiki.commonjs.org/wiki/Modules/1.1\" target=\"about:blank\">commonjs module</a>.            </span>            <span id=\"customFunctionsExample\" class=\"help-cursor\" data-placement=\"auto right\" data-original-title=\"Custom function example\" data-trigger=\"hover\">            See the example. <i class=\"fa fa-info-circle text-info\"></i>            </span>        </div>        <div class=\"panel-body\">            <div data-bind=\"visible: hasGlobal\">                <div><strong>Global configuration: </strong></div>                <pre class=\"editor\" id=\"globalCustomFunctionsEditor\" data-bind=\"aceEditor: { code: globalDocumentText, readOnly: true, fontSize: \'18px\', lang: \'ace/mode/javascript\' }\"></pre>                <div><strong>Local configuration: </strong></div>            </div>                        <pre class=\"editor\" id=\"customFunctionsEditor\" data-bind=\"aceEditor: { code: documentText, fontSize: \'18px\', lang: \'ace/mode/javascript\' }\"></pre>        </div>    </div></form>                                                                                                           '; } )
define("text!views/database/settings/databaseSettings.html", [], function () { return '<div id=\"databaseSettingsContainer\">    <div data-bind=\"visible: isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <form data-bind=\"submit: saveChanges, visible: !isForbidden()\">        <div class=\"panel panel-default panel-wo-margin\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled, visible: isEditingEnabled\" title=\"Save Changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: editDatabaseSettings, visible: !isEditingEnabled()\" title=\"Edit Database Settings\"><i class=\"fa fa-pencil\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refreshFromServer\" title=\"Refresh from server, discarding any unsaved changes\">                    <i class=\"fa fa-refresh\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: formatDocument\" title=\"Format document (Alt+[)\" accesskey=\"[\">                    <i class=\"fa fa-align-justify\"></i>                </button>            </div>            <div class=\"panel-body\">                <ul class=\"nav nav-tabs\">                    <li accesskey=\"d\" data-bind=\"click: activateDoc, css: { active: isEditingMetadata() === false }\" title=\"Edit document data (Alt+Shift+D)\"><a style=\"cursor: pointer;\">Data</a></li>                    <li accesskey=\"m\" data-bind=\"click: activateMeta, css: { active: isEditingMetadata }\" title=\"Edit document metadata (Alt+Shift+M)\"><a style=\"cursor: pointer;\">Metadata</a></li>                </ul>                <pre id=\"dbDocEditor\" data-bind=\"aceEditor: { code: text, lang: \'ace/mode/json\', readOnly: !isEditingEnabled() }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#databaseSettingsContainer\' }\"></pre>            </div>        </div>    </form></div>'; } )
define("text!views/database/settings/editSqlReplication.html", [], function () { return '<section id=\"editSQLReplication\">    <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: save\">        <div class=\"btn-toolbar edit-sql-replication-actions\">            <div class=\"btn-group\">                <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: $root.isSaveEnabled()\" title=\"Save the index (Alt+S)\" accesskey=\"S\">                    <i class=\"fa fa-save\"></i>                </button>                <button class=\"btn btn-default\" title=\"Refresh from server, discarding any unsaved changes\" data-bind=\"click: refreshSqlReplication\">                    <i class=\"fa fa-refresh\"></i>                </button>            </div>            <div class=\"btn-group tools\">                <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" accesskey=\"A\">                    Tools                    <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\">                    <li>                        <a href=\"#\" title=\"View current SQL Replication stats\" data-bind=\"click: showStats\">Stats</a>                    </li>                    <li>                        <a href=\"#\" title=\"Simulate SQL Replication\" data-bind=\"click: simulateSqlReplication\">                            Simulation                        </a>                    </li>                    <li>                        <a href=\"#\" title=\"Open Sql Replication Statistics and Metrics View\" data-bind=\"attr: {href: $root.sqlReplicationStatsAndMetricsHref }\" target=\"_blank\">                            Metrics Page <i class=\"fa fa-external-link\"></i>                        </a>                    </li>                </ul>            </div>            <div class=\"btn-group\">                <button class=\"btn btn-warning\" title=\"Reset SQL Replication\" data-bind=\"click: resetSqlReplication\">                    <span class=\"glyphicon glyphicon-retweet\"></span>                </button>                <button class=\"btn btn-danger\" title=\"Delete SQL Replication\" data-bind=\"click: deleteSqlReplication\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>        </div>        <div data-bind=\"with: $root.editedReplication\" style=\"padding-top: 5px;\">            <div class=\"col-sm-6\">                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\" title=\"SQL Replication name (alt+i)\">Name</label>                    <div class=\"col-sm-8\">                        <input tabindex=\"3\" required type=\"text\" class=\"form-control\" name=\"name\" data-bind=\"value: name, valueUpdate: \'afterkeydown\', attr: { placeholder: \'New SQL Replication\'}\" />                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">Source Document Collection</label>                    <div class=\"col-sm-8\">                        <input tabindex=\"2\" required name=\"ravenEntityName\" type=\"text\" class=\"form-control\" data-bind=\"value: ravenEntityName, valueUpdate: \'afterkeydown\', attr: { \'id\': \'sourceDocumentCollection\'}\" />                        <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#sourceDocumentCollection\', foreach: searchResults\">                            <li role=\"presentation\" data-bind=\"click: $parent.saveNewRavenEntityName.bind($parent, $data)\">                                <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                    <span data-bind=\"text: $data\"></span>                                </a>                            </li>                        </ul>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">SQL Replication Tables</label>                    <div class=\"col-sm-8\">                        <table tabindex=\"8\" class=\"table table-striped table-hover table-sql-replication-tables\">                            <thead>                            <tr class=\"sql-replication-tables-header\">                                <td>Table Name</td>                                <td>Document Key</td>                                <td>Insert only</td>                                <td class=\"narrowest-column\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addNewTable\" title=\"Add new table\">                                        <i class=\"fa fa-plus\"></i>                                    </button>                                </td>                            </tr>                            </thead>                            <tbody data-bind=\"foreach: sqlReplicationTables\">                            <tr>                                <td>                                    <div>                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: tableName, valueUpdate: \'afterkeydown\'\"/>                                    </div>                                </td>                                <td>                                    <div>                                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: documentKeyColumn, valueUpdate: \'afterkeydown\'\"/>                                    </div>                                </td>                                <td>                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: insertOnly()\">Yes</span>                                            <span data-bind=\"visible: !insertOnly()\">No</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: function() { $data.insertOnly(true); }\">Yes</a></li>                                            <li><a href=\"#\" data-bind=\"click: function() { $data.insertOnly(false); }\">No</a></li>                                        </ul>                                    </div>                                </td>                                <td>                                    <button type=\"button\" style=\"display: inline-block;\" class=\"close\" data-bind=\"visible: $parent.sqlReplicationTables().length > 1, click: $parent.removeTable.bind($parent)\" title=\"Remove this table\">                                        <i class=\"fa fa-times\"></i>                                    </button>                                </td>                            </tr>                            </tbody>                        </table>                        <div class=\"alert alert-warning\" role=\"�lert\" data-bind=\"visible: hasAnyInsertOnlyOption\">                            <i class=\"fa fa-warning\"></i> Insert only option is only valid for tables in which documents are not modified. If they do updates, they would result in errors/duplicate information.                         </div>                    </div>                </div>            </div>            <div class=\"col-sm-6\">                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\" title=\"Sql Replication Mode (Alt+I)\">Replication</label>                    <div class=\"col-sm-8\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: !disabled()\">Enabled</span>                                <span data-bind=\"visible: disabled\">Disabled</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                            </ul>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\" data-bind=\"text: connectionStringSourceFieldName\"></label>                    <div class=\"col-sm-8\">                        <textarea tabindex=\"6\" rows=\"4\" required class=\"form-control\" data-bind=\"attr:{required:connectionStringType() !== $data.PREDEFINED_CONNECTION_STRING_NAME?\'required\':null},visible:connectionStringType() !== $data.PREDEFINED_CONNECTION_STRING_NAME ,value: connectionStringValue, valueUpdate: \'afterkeydown\'\" />                        <select tabindex=\"7\" required class=\"form-control\" data-bind=\"options:$root.availableConnectionStrings, optionsText:\'name\', optionsValue:\'name\', attr:{required:connectionStringType() === $data.PREDEFINED_CONNECTION_STRING_NAME?\'required\':null},visible:connectionStringType() === $data.PREDEFINED_CONNECTION_STRING_NAME ,value: connectionStringType() === $data.PREDEFINED_CONNECTION_STRING_NAME?connectionStringValue:null, valueUpdate: \'afterkeydown\'\" />                    </div>                </div>                <div class=\"form-group\">                    <button class=\"col-sm-4 btn btn-default\"  data-bind=\"attr:{title:(\'Switch to \' + ($root.isBasicView()?\'Advanced\':\'Basic\') + \' mode\')}, css: { active: !$root.isBasicView() }, click: $root.toggleBasicMode.bind($root)\">                        <i class=\"fa fa-cogs\"></i> Show advanced settings                    </button>                </div>                <div class=\"form-group\" data-bind=\"visible:$root.isBasicView() == false\">                    <label class=\"col-sm-4 control-label\">SQL Provider</label>                    <div class=\"col-sm-8\">                        <select tabindex=\"5\" required class=\"form-control\"                                data-bind=\"attr:{required:connectionStringType() !== $data.PREDEFINED_CONNECTION_STRING_NAME?\'required\':null}, event:{change:$root.providerChanged.bind($root)}, options: [\'System.Data.SqlClient\',\'System.Data.SqlServerCe.4.0\',\'System.Data.OleDb\',\'System.Data.OracleClient\',\'MySql.Data.MySqlClient\',\'System.Data.SqlServerCe.3.5\',\'Npgsql\'], optionsCaption: \'\', value: factoryName\"></select>                    </div>                </div>                <div class=\"form-group\" data-bind=\"visible:$root.isBasicView() == false\">                    <label class=\"col-sm-4 control-label\">Connection String Source</label>                    <div class=\"col-sm-8\">                        <select tabindex=\"7\" required class=\"form-control\"                                data-bind=\"options: $data.availableConnectionStringTypes, optionsCaption: \'\', value: connectionStringType\"></select>                    </div>                </div>                <div class=\"form-group\" data-bind=\"visible:$root.isBasicView() == false\">                    <!--<label class=\"col-sm-4 control-label\">Disable Parameterized Deletes</label>                    <div class=\"checkbox col-sm-8\">                        <input type=\"checkbox\" data-bind=\"checked: parameterizeDeletesDisabled\" />                    </div>-->                    <label class=\"col-sm-4 control-label\">Parameterized Deletes</label>                    <div class=\"col-sm-8\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: !parameterizeDeletesDisabled()\">Enable parameterized deletes</span>                                <span data-bind=\"visible: parameterizeDeletesDisabled\">Disable parameterized deletes</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: parameterizeDeletesDisabled.bind(parameterizeDeletesDisabled, false)\">Enable parameterized deletes</a></li>                                <li><a href=\"#\" data-bind=\"click: parameterizeDeletesDisabled.bind(parameterizeDeletesDisabled, true)\">Disable parameterized deletes</a></li>                            </ul>                        </div>                    </div>                </div>                <div class=\"form-group\" data-bind=\"visible:$root.isBasicView() === false &&  (isSqlServerKindOfFactory(factoryName()) === true  || (connectionStringType() == PREDEFINED_CONNECTION_STRING_NAME && !!$root.availableConnectionStrings.first(function(x){ return x.name() == connectionStringValue() && isSqlServerKindOfFactory(x.factoryName()) == true})))\">                    <label class=\"col-sm-4 control-label\">Query Recompiling</label>                    <div class=\"col-sm-8\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: forceSqlServerQueryRecompile\">Force recompile</span>                                <span data-bind=\"visible: !forceSqlServerQueryRecompile()\">Don\'t force recompile</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: forceSqlServerQueryRecompile.bind(forceSqlServerQueryRecompile, true)\">Force recompile</a></li>                                <li><a href=\"#\" data-bind=\"click: forceSqlServerQueryRecompile.bind(forceSqlServerQueryRecompile, false)\">Don\'t force recompile</a></li>                            </ul>                        </div>                    </div>                </div>                <div class=\"form-group\" data-bind=\"visible:$root.isBasicView() === false\">                    <label class=\"col-sm-4 control-label\">Table Quotation</label>                    <div class=\"col-sm-8\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: quoteTables()\">Use quotes for tables</span>                                <span data-bind=\"visible: !quoteTables()\">Don\'t use quotes for tables</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: quoteTables.bind(quoteTables, true)\">Use quotes for tables</a></li>                                <li><a href=\"#\" data-bind=\"click: quoteTables.bind(quoteTables, false)\">Don\'t use quotes for tables</a></li>                            </ul>                        </div>                    </div>                </div>            </div>            <br />                    </div>        <div data-bind=\"visible: $root.editedReplication\">            <div class=\"form-group sql-replication-script-container col-sm-12\">                <label class=\"col-sm-1 control-label script-label\" data-placement=\"auto right\" data-trigger=\"hover\">                    <span class=\"help-cursor\">                        Script                        <i class=\"fa fa-info-circle text-info\"></i>                    </span>                </label>                <div class=\"col-sm-11\">                    <pre tabindex=\"9\" id=\"sqlReplicationEditor\" class=\"form-control editor\" data-bind=\"aceEditor: { code: $root.script, lang: \'ace/mode/javascript\' }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 40 }\"></pre>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/settings/periodicExport.html", [], function () { return '<div id=\"periodicExportPage\">    <div data-bind=\"visible: isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div data-bind=\"visible: backupSetup().unsupported()\">        <div class=\"alert alert-danger\">Too many periodic export types found. This isn\'t a supported scenario for the UI.</div>    </div>    <div data-bind=\"visible: !backupSetup().unsupported() && !isForbidden(), with: backupSetup\">        <form data-bind=\"submit: $root.saveChanges.bind($root)\" id=\"save-periodic-backup-form\" class=\"form-horizontal\" role=\"form\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: $root.isSaveEnabled()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                    <span data-bind=\"visible: $root.hasGlobalValues()\">                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useLocal.bind($root), visible: $root.usingGlobal()\">                            <i class=\"fa fa-arrow-down\"></i> Override global configuration                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useGlobal.bind($root), visible: $root.usingGlobal() == false\">                            <i class=\"fa fa-arrow-up\"></i> Use global configuration                        </button>                    </span>                </div>                <div class=\"panel-body\">                    <div class=\"form-group\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"col-md-4 text-right\">                            <strong>Property</strong>                        </div>                        <div class=\"col-md-4\">                            <strong>Effective value</strong>                        </div>                        <div class=\"col-md-4\">                            <strong>Global value</strong>                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"disabled\" class=\"col-sm-4 control-label\">Enabled:</label>                        <div class=\"col-sm-4\">                            <div class=\"btn-group\">                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-bind=\"enable: $root.usingGlobal() == false\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: disabled() == false\">Yes</span>                                    <span data-bind=\"visible: disabled()\">No</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\">                                    <li><a href=\"#\" data-bind=\"click: disabled.bind(disabled, false)\">Yes</a></li>                                    <li><a href=\"#\" data-bind=\"click: disabled.bind(disabled, true)\">No</a></li>                                </ul>                            </div>                            <p class=\"text-info\" data-bind=\"visible: $root.exportDisabled\">                                <i class=\"fa fa-info-circle text-info\"></i> Periodic Export is currently deactivated.                            </p>                        </div>                        <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                            <div class=\"btn-group\">                                <button disabled=\"disabled\" type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: $root.globalBackupSetup().disabled() == false\">Yes</span>                                    <span data-bind=\"visible: $root.globalBackupSetup().disabled()\">No</span>                                    <span class=\"caret\"></span>                                </button>                            </div>                        </div>                    </div>                    <div id=\"periodicExportContainer\">                        <div class=\"form-group\">                            <label for=\"onDiskExport\" class=\"col-sm-4 control-label\">On disk export:</label>                            <div class=\"col-sm-4\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: $root.usingGlobal() == false\">                                        <span data-bind=\"visible: onDiskExportEnabled\">Yes</span>                                        <span data-bind=\"visible: !onDiskExportEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: onDiskExportEnabled.bind(onDiskExportEnabled, true)\">Yes</a></li>                                        <li><a href=\"#\" data-bind=\"click: onDiskExportEnabled.bind(onDiskExportEnabled, false)\">No</a></li>                                    </ul>                                </div>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" disabled=\"disabled\" data-toggle=\"dropdown\">                                        <span data-bind=\"visible: $root.globalBackupSetup().onDiskExportEnabled\">Yes</span>                                        <span data-bind=\"visible: !$root.globalBackupSetup().onDiskExportEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                </div>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: $root.showOnDiskExportRow()\">                            <label for=\"onDiskLocation\" class=\"col-sm-4 control-label\">File System Folder:</label>                            <div class=\"col-sm-4\" >                                <div data-bind=\"visible: onDiskExportEnabled\">                                    <input type=\"text\" id=\"onDiskLocation\" placeholder=\"full path\" required class=\"form-control\" data-bind=\"value: localFolderName, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\" />                                    </div>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues() && $root.globalBackupSetup().onDiskExportEnabled()\">                                <input type=\"text\" disabled=\"disabled\" placeholder=\"full path\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().localFolderName\" />                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"serverUpload\" class=\"col-sm-4 control-label\">Upload to remote server:</label>                            <div class=\"col-sm-4\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: $root.usingGlobal() == false\">                                        <span data-bind=\"visible: remoteUploadEnabled\">Yes</span>                                        <span data-bind=\"visible: !remoteUploadEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: remoteUploadEnabled.bind(remoteUploadEnabled, true)\">Yes</a></li>                                        <li><a href=\"#\" data-bind=\"click: remoteUploadEnabled.bind(remoteUploadEnabled, false)\">No</a></li>                                    </ul>                                </div>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <div class=\"btn-group\">                                    <button disabled=\"disabled\" type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                        <span data-bind=\"visible: $root.globalBackupSetup().remoteUploadEnabled\">Yes</span>                                        <span data-bind=\"visible: !$root.globalBackupSetup().remoteUploadEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                </div>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: remoteUploadEnabled, visible: remoteUploadEnabled\">                            <div class=\"col-sm-4\">                                <select class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, options: availablePeriodicExports, optionsText: \'label\', optionsValue: \'value\', value: type\"></select>                            </div>                            <div class=\"col-sm-4\">                                <input type=\"text\" required class=\"form-control\" data-bind=\"value: mainValue, enable: $root.usingGlobal() == false, customValidity: mainValueCustomValidity, valueUpdate: \'afterkeydown\', attr: { placeholder: mainPlaceholder }\"/>                                <div data-bind=\"visible: isGlaceirVault\">                                    <a target=\"_blank\" href=\"http://docs.aws.amazon.com/amazonglacier/latest/dev/creating-vaults.html\">Vault Name Restrictions and Limitations</a>                                </div>                                <div data-bind=\"visible: isS3Bucket\">                                    <a target=\"_blank\" href=\"http://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html\">Bucket Name Restrictions and Limitations</a>                                </div>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <input type=\"text\" disabled=\"disabled\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().mainValue, attr: { placeholder: mainPlaceholder }\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: isS3Bucket, visible: isS3Bucket\">                            <label for=\"awsRemoteFolder\" class=\"col-sm-4 control-label\">Folder name:</label>                            <div class=\"col-sm-4\">                                <input id=\"awsRemoteFolder\" type=\"text\" placeholder=\"Remote folder name\" class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, value: s3RemoteFolderName, valueUpdate: \'afterkeydown\'\" />                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <input disabled=\"disabled\" type=\"text\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().s3RemoteFolderName\" />                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsAccessKey\" class=\"col-sm-4 control-label\">AWS Access Key:</label>                            <div class=\"col-sm-4\">                                <input required id=\"awsAccessKey\" type=\"text\" class=\"form-control\" data-bind=\"value: awsAccessKey, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\"/>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <input disabled=\"disabled\" type=\"text\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().awsAccessKey\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsSecretKey\" class=\"col-sm-4 control-label\">AWS Secret Key:</label>                            <div class=\"col-sm-3\">                                <input required id=\"awsSecretKey\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"value: awsSecretKey, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\"/>                            </div>                            <div class=\"col-sm-1 control-label\" style=\"text-align: left\" data-bind=\"if: awsSecretKeyDecryptionFailed\">                                <span id=\"awsDecryptFailureSpan\" class=\"help-cursor\" data-placement=\"auto right\" data-original-title=\"Why this input is empty?\" data-trigger=\"hover\">                                    <i class=\"fa fa-warning text-info\"></i>                                </span>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <input disabled=\"disabled\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().awsSecretKey\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsRegionEndpoint\" class=\"col-sm-4 control-label\">AWS Region Endpoint:</label>                            <div class=\"col-sm-4\">                                <select required id=\"awsRegionEndpoint\" class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, options: availableAwsRegionEndpoints, optionsText: \'label\', optionsValue: \'value\', optionsCaption: \'\', value: awsRegionEndpoint\"></select>                            </div>                            <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                <select disabled=\"disabled\" class=\"form-control\" data-bind=\"options: availableAwsRegionEndpoints, optionsText: \'label\', optionsValue: \'value\', optionsCaption: \'\', value: $root.globalBackupSetup().awsRegionEndpoint\"></select>                            </div>                        </div>                        <div data-bind=\"if: additionalAzureInfoRequired\">                            <div class=\"form-group\">                                <label for=\"remoteFolder\" class=\"col-sm-4 control-label\">Folder name:</label>                                <div class=\"col-sm-4\">                                    <input id=\"remoteFolder\" type=\"text\" placeholder=\"Remote folder name\" class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, value: azureRemoteFolderName, valueUpdate: \'afterkeydown\'\"/>                                </div>                                <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                    <input disabled=\"disabled\" type=\"text\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().azureRemoteFolderName\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"azureStorageAccount\" class=\"col-sm-4 control-label\">Azure Storage Account:</label>                                <div class=\"col-sm-4\">                                    <input required id=\"azureStorageAccount\" type=\"text\" class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, value: azureStorageAccount, valueUpdate: \'afterkeydown\'\"/>                                </div>                                <div class=\"col-sm-4\" data-bind=\"visible: $root.hasGlobalValues\">                                    <input disabled=\"disabled\" type=\"text\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().azureStorageAccount\"/>                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"azureStorageKey\" class=\"col-sm-4 control-label\">Azure Storage Key:</label>                                <div class=\"col-sm-3\">                                    <input required id=\"azureStorageKey\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, value: azureStorageKey, valueUpdate: \'afterkeydown\'\"/>                                </div>                                <div class=\"col-sm-1 control-label\" style=\"text-align: left\" data-bind=\"if: azureStorageKeyDecryptionFailed\">                                    <span id=\"azureDecryptFailureSpan\" class=\"help-cursor\" data-placement=\"auto right\" data-original-title=\"Why this input is empty?\" data-trigger=\"hover\">                                        <i class=\"fa fa-warning text-info\"></i>                                    </span>                                </div>                                <div class=\"col-sm-3\" data-bind=\"visible: $root.hasGlobalValues\">                                    <input disabled=\"disabled\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"value: $root.globalBackupSetup().azureStorageKey\"/>                                </div>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"incrementalBackupInterval\" class=\"col-sm-4 control-label\">Incremental Backup Interval:</label>                            <div class=\"col-sm-1\">                                <input id=\"incrementalBackupInterval\" required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"enable: $root.usingGlobal() == false, numericValue: incrementalBackupInterval, valueUpdate: \'afterkeydown\'\" />                            </div>                            <div class=\"col-sm-3\">                                <select class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, options: availableIntervalUnits, value: incrementalBackupIntervalUnit\"></select>                            </div>                            <div class=\"col-sm-1\" data-bind=\"visible: $root.hasGlobalValues\">                                <input disabled=\"disabled\" type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: $root.globalBackupSetup().incrementalBackupInterval\" />                            </div>                            <div class=\"col-sm-3\" data-bind=\"visible: $root.hasGlobalValues\">                                <select disabled=\"disabled\" class=\"form-control\" data-bind=\"options: availableIntervalUnits, value: $root.globalBackupSetup().incrementalBackupIntervalUnit\"></select>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"fullBackupInterval\" class=\"col-sm-4 control-label\">Full Backup Interval:</label>                            <div class=\"col-sm-1\">                                <input id=\"fullBackupInterval\" required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"enable: $root.usingGlobal() == false, numericValue: fullBackupInterval, valueUpdate: \'afterkeydown\'\" />                            </div>                            <div class=\"col-sm-3\">                                <select class=\"form-control\" data-bind=\"enable: $root.usingGlobal() == false, options: availableIntervalUnits, value: fullBackupIntervalUnit\"></select>                            </div>                            <div class=\"col-sm-1\" data-bind=\"visible: $root.hasGlobalValues\">                                <input disabled=\"disabled\" type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: $root.globalBackupSetup().fullBackupInterval\" />                            </div>                            <div class=\"col-sm-3\" data-bind=\"visible: $root.hasGlobalValues\">                                <select disabled=\"disabled\" class=\"form-control\" data-bind=\"options: availableIntervalUnits, value: $root.globalBackupSetup().fullBackupIntervalUnit\"></select>                            </div>                        </div>                    </div>                </div>            </div>        </form>    </div></div>'; } )
define("text!views/database/settings/quotas.html", [], function () { return '<div id=\"quotasPage\">    <div data-bind=\"visible: isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <form class=\"quotas-settings-form form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges.bind($root), visible: isForbidden() == false\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: $root.isSaveEnabled()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                <span data-bind=\"visible: hasGlobalValues()\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useLocal.bind($root), visible: usingGlobal()\">                        <i class=\"fa fa-arrow-down\"></i> Override global configuration                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useGlobal.bind($root), visible: usingGlobal() == false\">                        <i class=\"fa fa-arrow-up\"></i> Use global configuration                    </button>                </span>            </div>            <div class=\"panel-body\">                <div class=\"form-group\" data-bind=\"visible: hasGlobalValues\">                    <label class=\"col-md-2 text-right\">                        Property                    </label>                    <label class=\"col-md-4\">                        Effective value                    </label>                    <label class=\"col-md-5\">                       Global value                    </label>                </div>                <div class=\"form-group\">                    <label class=\"col-md-2 control-label\">Maximum size:</label>                    <div class=\"col-md-4\">                        <div class=\"input-group\" data-bind=\"with: maximumSize\">                            <input required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: effectiveValue, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\" />                            <span class=\"input-group-addon\">MB</span>                        </div>                    </div>                    <div class=\"col-md-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"input-group\" data-bind=\"with: maximumSize\">                            <input required type=\"number\" disabled=\"disabled\" class=\"form-control\" data-bind=\"numericValue: globalValue\" />                            <span class=\"input-group-addon\">MB</span>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-md-2 control-label\">Warning limit threshold:</label>                    <div class=\"col-md-4\">                        <div class=\"input-group\" data-bind=\"with: warningLimitThreshold\">                            <input required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: effectiveValue, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\" />                            <span class=\"input-group-addon\">MB</span>                        </div>                    </div>                    <div class=\"col-md-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"input-group\" data-bind=\"with: warningLimitThreshold\">                            <input required type=\"number\" disabled=\"disabled\" class=\"form-control\" data-bind=\"numericValue: globalValue\" />                            <span class=\"input-group-addon\">MB</span>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-md-2 control-label\">Maximum number of docs:</label>                    <div class=\"col-md-4\">                        <div class=\"input-group\" data-bind=\"with: maxNumberOfDocs\">                            <input required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: effectiveValue, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\" />                            <span class=\"input-group-addon\">docs</span>                        </div>                    </div>                    <div class=\"col-md-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"input-group\" data-bind=\"with: maxNumberOfDocs\">                            <input required type=\"number\" disabled=\"disabled\" class=\"form-control\" data-bind=\"numericValue: globalValue\" />                            <span class=\"input-group-addon\">docs</span>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-md-2 control-label\">Warning threshold for docs:</label>                    <div class=\"col-md-4\">                        <div class=\"input-group\" data-bind=\"with: warningThresholdForDocs\">                            <input required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: effectiveValue, enable: $root.usingGlobal() == false, valueUpdate: \'afterkeydown\'\" />                            <span class=\"input-group-addon\">docs</span>                        </div>                    </div>                    <div class=\"col-md-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"input-group\" data-bind=\"with: warningThresholdForDocs\">                            <input required type=\"number\" disabled=\"disabled\" class=\"form-control\" data-bind=\"numericValue: globalValue\" />                            <span class=\"input-group-addon\">docs</span>                        </div>                    </div>                </div>            </div>        </div>    </form></div>'; } )
define("text!views/database/settings/replications.html", [], function () { return '<form data-bind=\"submit: $root.saveChanges\" class=\" form-horizontal\" role=\"form\">    <div class=\"panel-group\">        <div class=\"panel panel-default\" data-bind=\"if: replicationEnabled(), visible: replicationEnabled()\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSetupSaveEnabled() || isConfigSaveEnabled()\" title=\"Save changes (Alt+S)\" accesskey=\"S\">                    <i class=\"fa fa-save\"></i>                </button>                <span data-bind=\"visible: hasGlobalValues()\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useLocal.bind($root), visible: usingGlobal()\">                        <i class=\"fa fa-arrow-down\"></i> Override global configuration                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useGlobal.bind($root), visible: !usingGlobal()\">                        <i class=\"fa fa-arrow-up\"></i> Use global configuration                    </button>                </span>            </div>            <div class=\"panel-body\">                <div class=\"form-group\" data-bind=\"visible: hasGlobalValues\">                    <div class=\"col-md-2 text-right\">                        <strong>Property</strong>                    </div>                    <div class=\"col-md-5\">                        <strong>Effective value</strong>                    </div>                    <div class=\"col-md-5\">                        <strong>Global value</strong>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"conflictResolutionSelector\" class=\"col-sm-2 control-label\">Client failover behaviour</label>                    <div class=\"col-sm-5\">                        <div class=\"btn-group\" data-bind=\"with: replicationsSetup\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: !$root.usingGlobal()\">                                <span data-bind=\"visible: !clientFailoverBehaviour()\">Let client decide</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadsFromSecondaries\'\">Allow reads from secondaries</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached\'\">Allow reads from secondaries when request time SLA threshold is reached</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadsFromSecondariesAndWritesToSecondaries\'\">Allow reads from secondaries and writes to secondaries</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'FailImmediately\'\">Fail immediately</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'ReadFromAllServers\'\">Read from all servers</span>                                <span data-bind=\"visible: readFromAllButSwitchWhenRequestTimeSlaThresholdIsReached()\">Read from all servers but switch when request time SLA threshold is reached</span>                                <span data-bind=\"visible: readFromAllAllowWriteToSecondaries()\">Read from all servers and allow write to secondaries</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'\')\">Let client decide</a></li>                                <li class=\"divider\"></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondaries\')\">Allow reads from secondaries</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached\')\">Allow reads from secondaries when request time SLA threshold is reached</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondariesAndWritesToSecondaries\')\">Allow reads from secondaries and writes to secondaries</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'FailImmediately\')\">Fail immediately</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'ReadFromAllServers\')\">Read from all servers</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached,ReadFromAllServers\')\">Read from all servers but switch when request time SLA threshold is reached</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondariesAndWritesToSecondaries,ReadFromAllServers\')\">Read from all servers and allow write to secondaries</a></li>                            </ul>                        </div>                    </div>                    <div class=\"col-sm-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"btn-group\" data-bind=\"with: replicationsSetup\">                            <button disabled=\"disabled\" type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: !$root.globalClientFailoverBehaviour()\">Let client decide</span>                                <span data-bind=\"visible: $root.globalClientFailoverBehaviour() === \'AllowReadsFromSecondaries\'\">Allow reads from secondaries</span>                                <span data-bind=\"visible: $root.globalClientFailoverBehaviour() === \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached\'\">Allow reads from secondaries when request time SLA threshold is reached</span>                                <span data-bind=\"visible: $root.globalClientFailoverBehaviour() === \'AllowReadsFromSecondariesAndWritesToSecondaries\'\">Allow reads from secondaries and writes to secondaries</span>                                <span data-bind=\"visible: $root.globalClientFailoverBehaviour() === \'FailImmediately\'\">Fail immediately</span>                                <span data-bind=\"visible: $root.globalClientFailoverBehaviour() === \'ReadFromAllServers\'\">Read from all servers</span>                                <span data-bind=\"visible: $root.globalReadFromAllButSwitchWhenRequestTimeSlaThresholdIsReached\">Read from all servers but switch when request time SLA threshold is reached</span>                                <span data-bind=\"visible: $root.globalReadFromAllAllowWriteToSecondaries\">Read from all servers and allow write to secondaries</span>                                <span class=\"caret\"></span>                            </button>                        </div>                    </div>                </div>                <div data-bind=\"if: showRequestTimeoutRow()\">                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Request Time SLA Threshold</label>                        <div class=\"col-sm-5\">                            <div class=\"input-group\" data-bind=\"visible: replicationsSetup().showRequestTimeSlaThreshold()\">                                <input type=\"number\" class=\"form-control\" data-bind=\"value: replicationsSetup().requestTimeSlaThreshold, valueUpdate: \'afterkeydown\', enable: $root.usingGlobal() == false\" />                                <span class=\"input-group-addon\">ms</span>                            </div>                        </div>                        <div class=\"col-sm-5\" data-bind=\"visible: $root.hasGlobalValues() && $root.globalClientRequestTimeSlaThreshold()\">                            <div class=\"input-group\">                                <input disabled=\"disabled\" type=\"number\" class=\"form-control\" data-bind=\"value: $root.globalClientRequestTimeSlaThreshold()\" />                                <span class=\"input-group-addon\">ms</span>                            </div>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"conflictResolutionSelector\" class=\"col-sm-2 control-label\">Conflict resolution</label>                    <div class=\"col-sm-5\">                        <div class=\"btn-group\" data-bind=\"with: replicationConfig\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: $root.usingGlobal() == false\">                                <span data-bind=\"visible: documentConflictResolution() === \'None\'\">None</span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLocal\'\">Resolve to <strong>local</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToRemote\'\">Resolve to <strong>remote</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLatest\'\">Resolve to <strong>latest</strong></span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'None\')\">None</a></li>                                <li class=\"divider\"></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToLocal\')\">Resolve to <strong>local</strong></a></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToRemote\')\">Resolve to <strong>remote</strong></a></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToLatest\')\">Resolve to <strong>latest</strong></a></li>                            </ul>                        </div>                    </div>                    <div class=\"col-sm-5\" data-bind=\"visible: $root.hasGlobalValues\">                        <div class=\"btn-group\" data-bind=\"with: globalReplicationConfig\">                            <button type=\"button\" disabled=\"disabled\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: documentConflictResolution() === \'None\'\">None</span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLocal\'\">Resolve to <strong>local</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToRemote\'\">Resolve to <strong>remote</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLatest\'\">Resolve to <strong>latest</strong></span>                                <span class=\"caret\"></span>                            </button>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <div class=\"col-sm-10 col-sm-offset-2\">                        <div class=\"checkbox\">                            <input type=\"checkbox\" id=\"globalSkipIndexReplicatonForAll\" class=\"styled\" data-bind=\"checked: skipIndexReplicationForAll, enable: !$root.usingGlobal()\">                            <label for=\"globalSkipIndexReplicatonForAll\">Skip index replication for all destinations</label>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Replication destinations</label>                    <div class=\"col-sm-10\">                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: $root.usingGlobal() == false, click: createNewDestination\" title=\"Add a replication destination (Alt+N)\" accesskey=\"N\">                            <i class=\"fa fa-plus\"></i> Add destination                        </button>                        <br />                        <br />                        <div data-bind=\"with: replicationsSetup\">                            <p data-bind=\"visible: destinations().length === 0\">                                <i class=\"fa fa-warning text-warning\"></i> <span class=\"text-muted\">You haven\'t created any replication destinations.</span>                            </p>                            <div data-bind=\"foreach: destinations\">                                <div class=\"panel\" data-bind=\"css: { \'panel-warning\': disabled(), \'panel-default\': !disabled(), \'panel-info\': hasLocal() == false && hasGlobal() }\">                                    <div class=\"panel-heading\">                                        <h3 class=\"panel-title\">                                            <span title=\"Overridden from global configuration\" data-bind=\"visible: hasLocal() == false && hasGlobal()\"><i class=\"fa fa-arrow-down\"></i></span>                                            <span data-bind=\"text: name\"></span>                                            <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\"><i class=\"fa fa-times\" data-bind=\"click: $root.removeDestination.bind($root), visible: hasGlobal() == false\" title=\"Remove this replication destination\"></i></button>                                        </h3>                                    </div>                                    <div class=\"panel-body\">                                        <div class=\"form-group\" data-bind=\"visible: hasGlobal()\">                                            <label class=\"col-sm-2 control-label\">Override:</label>                                            <div class=\"col-sm-9\">                                                <div class=\"btn-group\">                                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: $root.usingGlobal() == false\">                                                        <span data-bind=\"visible: canEdit()\">Yes</span>                                                        <span data-bind=\"visible: canEdit()==false\">No</span>                                                        <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, true)\">Yes</a></li>                                                        <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, false)\">No</a></li>                                                    </ul>                                                </div>                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <div class=\"col-sm-offset-2 col-sm-9\">                                                <div class=\"btn-group\">                                                    <button type=\"button\" data-bind=\"enable: canEdit\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                        <span data-bind=\"visible: !disabled()\">Enabled</span>                                                        <span data-bind=\"visible: disabled\">Disabled</span>                                                        <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                                        <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                                                    </ul>                                                </div>                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-2 control-label\">Url</label>                                            <div class=\"col-sm-9\">                                                <input required type=\"url\" class=\"form-control\" title=\"The address of the server to replicate to\" data-bind=\"value: url, enable: hasGlobal() == false, valueUpdate: \'afterkeydown\'\" />                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-2 control-label\">Database</label>                                            <div class=\"col-sm-9\">                                                <input required type=\"text\" class=\"form-control\" title=\"The name of the database on the destination server to replicate to\" data-bind=\"value: database, enable: hasGlobal() == false, valueUpdate: \'afterkeydown\'\" />                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-2 control-label\">Credentials</label>                                            <div class=\"col-sm-9\">                                                <div class=\"btn-group\">                                                    <button type=\"button\" data-bind=\"enable: canEdit\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                        <span data-bind=\"visible: !isUserCredentials() && !isApiKeyCredentials()\">None</span>                                                        <span data-bind=\"visible: isUserCredentials\">User</span>                                                        <span data-bind=\"visible: isApiKeyCredentials\">API key</span>                                                        <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <li><a href=\"#\" data-bind=\"click: useNoCredentials\">None</a></li>                                                        <li class=\"divider\"></li>                                                        <li><a href=\"#\" data-bind=\"click: useUserCredentials\">User</a></li>                                                        <li><a href=\"#\" data-bind=\"click: useApiKeyCredentials\">API key</a></li>                                                    </ul>                                                </div>                                            </div>                                        </div>                                        <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">User Name</label>                                                <div class=\"col-sm-9\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, enable: canEdit, valueUpdate: \'afterkeydown\'\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">Password</label>                                                <div class=\"col-sm-9\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, enable: canEdit, valueUpdate: \'afterkeydown\'\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">Domain</label>                                                <div class=\"col-sm-9\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: domain, enable: canEdit, valueUpdate: \'afterkeydown\'\" />                                                </div>                                            </div>                                        </div>                                        <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">API Key</label>                                                <div class=\"col-sm-9\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, enable: canEdit, valueUpdate: \'afterkeydown\'\" />                                                </div>                                            </div>                                        </div>                                        <a data-bind=\"click: toggleIsAdvancedShows\" class=\"advanced-replication-settings\">                                            Advanced Options                                        </a>                                        <div data-bind=\"visible: false\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">Client Visible Url</label>                                                <div class=\"col-sm-9\">                                                    <input type=\"text\" class=\"form-control\" data-bind=\"value: clientVisibleUrl, enable: canEdit, valueUpdate: \'afterkeydown\'\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <div class=\"col-sm-9 col-sm-offset-2\">                                                    <div class=\"checkbox\">                                                        <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: skipIndexReplication, enable: canEdit, attr: { id: \'skipIndexReplicaton_\' + $index() } \">                                                        <label data-bind=\"attr: { for: \'skipIndexReplicaton_\' + $index() }\">Skip Index Replication</label>                                                    </div>                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <div class=\"col-sm-9 col-sm-offset-2\">                                                    <div class=\"checkbox\">                                                        <input type=\"checkbox\" class=\"styled\" id=\"mute\" data-bind=\"attr: { id: \'collection_rep_\' + $index() }, checked: enableReplicateOnlyFromCollections, enable: canEdit\">                                                        <label data-bind=\"attr: { for: \'collection_rep_\' + $index() }\">Customize collection replication</label>                                                    </div>                                                    <div data-bind=\"visible: enableReplicateOnlyFromCollections\">                                                        <table class=\"table table-striped\">                                                            <thead>                                                                <tr>                                                                    <th class=\"col-sm-2\">Replicate collection</th>                                                                    <th class=\"col-sm-2\">Transform / Filter</th>                                                                    <th class=\"col-sm-8\">Script <i class=\"fa fa-info-circle text-info scriptPopover\"></i></th>                                                                </tr>                                                            </thead>                                                            <tbody data-bind=\"foreach: $parents[1].collections\">                                                                <tr>                                                                    <td>                                                                        <div class=\"checkbox\">                                                                            <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: $parent.specifiedCollectionsNames, value: $data.name, event: { change: $root.onReplicateToCollectionClick.bind($root, $parents[0], $data.name) }, enable: $parent.canEdit, attr: { id: \'per_collection_\' + $parentContext.$index() + \'_\' + $index() }\">                                                                            <label data-bind=\"text: name, attr: { for: \'per_collection_\' + $parentContext.$index() + \'_\' + $index() }\"></label>                                                                        </div>                                                                    </td>                                                                    <td>                                                                        <div data-bind=\"visible: $parent.isSelected($data)\">                                                                            <div class=\"checkbox\">                                                                                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: $parent.withScripts, value: $data.name, event: { change: $root.onTransformCollectionClick.bind($root, $parents[0], $data.name) }, enable: $parent.canEdit, attr: { id: \'per_coll_filter_\' + $parentContext.$index() + \'_\' + $index() }\">                                                                                <label data-bind=\"attr: { for: \'per_coll_filter_\' + $parentContext.$index() + \'_\' + $index() }\">&nbsp;</label>                                                                            </div>                                                                        </div>                                                                    </td>                                                                    <td data-bind=\"if: $parent.hasScript($data) \">                                                                        <pre class=\"form-control\" data-bind=\"aceEditor: { lang: \'ace/mode/javascript\', readOnly: false, code: $parent.findEditor($data), allowResize: true }\"></pre>                                                                    </td>                                                                </tr>                                                            </tbody>                                                        </table>                                                    </div>                                                </div>                                            </div>                                            <div class=\"form-group\" style=\"margin-left: 10px\">                                                <button data-bind=\"click:function(event,data){ $parents[1].sendReplicateCommand($data,$parents[1]); }\" type=\"submit\" class=\"btn btn-primary\" title=\"Replicate to this destination\">                                                    <i class=\"fa fa-share-square-o\"></i>                                                </button>                                                <span style=\"margin-left: 5px;\">                                                    Force replicate all indexes and transformers                                                </span>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">Failover</label>                                                <div class=\"col-sm-9\">                                                    <div class=\"btn-group\">                                                        <button type=\"button\" data-bind=\"enable: canEdit\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                            <span data-bind=\"visible: !ignoredClient()\">Include</span>                                                            <span data-bind=\"visible: ignoredClient\">Skip</span>                                                            <span class=\"caret\"></span>                                                        </button>                                                        <ul class=\"dropdown-menu\" role=\"menu\">                                                            <li><a href=\"#\" data-bind=\"click: includeFailover\">Include</a></li>                                                            <li><a href=\"#\" data-bind=\"click: skipFailover\">Skip</a></li>                                                        </ul>                                                    </div>                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-2 control-label\">Transitive Replication</label>                                                <div class=\"col-sm-9\">                                                    <div class=\"btn-group\">                                                        <button type=\"button\" data-bind=\"enable: canEdit\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                            <span data-bind=\"visible: transitiveReplicationBehavior() === \'None\'\">Changed only</span>                                                            <span data-bind=\"visible: transitiveReplicationBehavior() === \'Replicate\'\">Changed and replicated</span>                                                            <span class=\"caret\"></span>                                                        </button>                                                        <ul class=\"dropdown-menu\" role=\"menu\">                                                            <li><a href=\"#\" data-bind=\"click: transitiveReplicationBehavior.bind(transitiveReplicationBehavior, \'None\')\">Changed only</a></li>                                                            <li><a href=\"#\" data-bind=\"click: transitiveReplicationBehavior.bind(transitiveReplicationBehavior, \'Replicate\')\">Changed and replicated</a></li>                                                        </ul>                                                    </div>                                                </div>                                            </div>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"if: replicationEnabled(), visible: replicationEnabled()\">            <div class=\"panel-heading\">                <button type=\"submit\" data-bind=\"click:sendResolveAllConflictsCommand\" class=\"btn btn-primary\" title=\"Resolve all existing conflict acording to the current conflict resolution\">                    <i class=\"fa fa-send\"></i>                </button>                <span style=\"margin-left: 5px;\">                    Resolve all existing conflict acording to the current conflict resolution                </span>            </div>            <div class=\"panel-heading\">                <button type=\"submit\" data-bind=\"click:sendReplicateAllCommand, enable: !isSetupSaveEnabled()\" class=\"btn btn-primary\" title=\"Replicate Indexes To All Destinations\">                    <i class=\"fa fa-send\"></i>                </button>                <span style=\"margin-left: 5px;\">                    Replicate all indexes and transformers to configured destinations                </span>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"if: replicationEnabled(), visible: replicationEnabled()\">            <div class=\"panel-heading\">                <button data-bind=\"click: saveServerPrefixForHiLo, enable: isServerPrefixForHiLoSaveEnabled\" type=\"submit\" class=\"btn btn-primary\" title=\"Save changes\">                    <i class=\"fa fa-save\"></i>                </button>                <span style=\"margin-left: 5px;\">                    Server HiLo prefix                </span>            </div>            <div class=\"panel-body\">                <div class=\"form-group\">                    <label for=\"conflictResolutionSelector\" class=\"col-sm-2 control-label\">                        Prefix                        <a class=\"control-label-additional-link\" href=\"http://ravendb.net/docs/server/scaling-out/replication/client-integration\" target=\"_blank\">                            Prefix docs                        </a>                    </label>                    <div class=\"col-sm-3\">                        <input name=\"prefix\" class=\"form-control\" type=\"text\" data-bind=\"value: prefixForHilo, valueUpdate: \'afterkeydown\'\">                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"if:!replicationEnabled(), visible: !replicationEnabled()\">            <div class=\"panel-heading\">                Replication is disabled            </div>            <div class=\"panel-body\">                <div class=\"text-center\">                    <h3 class=\"text-info\">Replication bundle is disabled. <small>Go ahead and <a href=\"#\" data-bind=\"click: enableReplication\">enable replication for this database</a>.</small></h3>                </div>            </div>        </div>    </div>    <div class=\"clearfix\"></div></form>'; } )
define("text!views/database/settings/settings.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        <span>Settings</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">        <a data-bind=\"attr: { href: hash }, text: title\"></a>    </li></ul><div class=\"settings-container row\">    <div class=\"col-md-2\">        <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">            <li data-bind=\"css: { active: (isActive() || $root.isEditingSqlReplication($root.router.navigationModel,route) === true) }, visible: $parent.routeIsVisible($data)\">                <a data-bind=\"attr: { href: hash }, text: title\"></a>            </li>        </ul>    </div>    <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/database/settings/sqlReplicationConnectionStringsManagement.html", [], function () { return '<form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: $root.save\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <span data-bind=\"visible: hasGlobalValues()\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useLocal.bind($root), visible: usingGlobal()\">                    <i class=\"fa fa-arrow-down\"></i> Override global configuration                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useGlobal.bind($root), visible: !usingGlobal()\">                    <i class=\"fa fa-arrow-up\"></i> Use global configuration                </button>            </span>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: !$root.usingGlobal(), click: addSqlReplicationConnection\" title=\"Add connection string (Alt+N)\" accesskey=\"N\">                <i class=\"fa fa-plus\"></i> New connection string            </button>        </div>        <br />        <div class=\"panel-body\" data-bind=\"with: connections\">            <!--<div class=\"alert alert-warning\" data-bind=\"visible: predefinedConnections().length === 0\">                You\'ve got no sql replication connections set up.            </div>-->            <p data-bind=\"visible: predefinedConnections().length === 0\">                <i class=\"fa fa-warning text-warning\"></i> You\'ve got no SQL replication connections set up.            </p>            <div data-bind=\"foreach: predefinedConnections\">                <div class=\"col-sm-6\">                    <div class=\"sql-replication-connection-card panel panel-default\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span title=\"Overridden from global configuration\" data-bind=\"visible: !hasLocal() && hasGlobal()\"><i class=\"fa fa-arrow-down\"></i></span>                                <span data-bind=\"text: name() || \'[untitled]\'\"></span>                                <button type=\"button\" class=\"close\" aria-hidden=\"true\" data-bind=\"click: function(){$root.removeSqlReplicationConnection($data)}, visible: !hasGlobal()\" title=\"Remove this connection\"><i class=\"fa fa-times\"></i></button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\" data-bind=\"visible: hasGlobal()\">                                <label class=\"col-sm-4 control-label\">Override:</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: !$root.usingGlobal()\">                                            <span data-bind=\"visible: canEdit()\">Yes</span>                                            <span data-bind=\"visible: !canEdit()\">No</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, true)\">Yes</a></li>                                            <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, false)\">No</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Name</label>                                <div class=\"col-sm-7\">                                    <input required name=\"name\" class=\"form-control\" type=\"text\" data-bind=\"value:name, valueUpdate: \'afterkeydown\', enable: canEdit\"/>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">SQL Provider</label>                                <div class=\"col-sm-7\">                                    <select tabindex=\"5\" required class=\"form-control\" data-bind=\"event:{change:function(val,event){$root.providerChanged($data,event);return true;}},options: [\'System.Data.SqlClient\',\'System.Data.SqlServerCe.4.0\',\'System.Data.OleDb\',\'System.Data.OracleClient\',\'MySql.Data.MySqlClient\',\'System.Data.SqlServerCe.3.5\',\'Npgsql\'], optionsCaption: \'\', value: factoryName, enable: canEdit\"></select>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Connection String</label>                                <div class=\"col-sm-7\">                                    <textarea tabindex=\"6\" rows=\"4\" required class=\"form-control\" data-bind=\"value: connectionString, valueUpdate: \'afterkeydown\', enable: canEdit\"></textarea>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-7 col-sm-offset-4\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: connectionTestState() === CONNECTION_STATE_STAND_BY, click: testConnection.bind($data, $root.activeDatabase)\">Test connection</button>                                    <br/>                                    <div class=\"progress progress-striped\" data-bind=\"visible:connectionTestState() !== CONNECTION_STATE_STAND_BY\">                                        <div class=\"progress-bar progress-bar-info\" data-bind=\"css: {\'active\':connectionTestState() === CONNECTION_STATE_CONNECTING}\" role=\"progressbar\" aria-valuenow=\"100\" aria-valuemin=\"0\" aria-valuemax=\"100\" style=\"width: 100%; float: left;\">                                            <span class=\"sr-only\" data-bind=\"text:connectionTestState() === CONNECTION_STATE_CONNECTING?\'Connecting\':\'Connected\'\"></span>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/database/settings/sqlReplications.html", [], function () { return '<div id=\"sqlReplications\" class=\"innerWindowsStyle\">    <div class=\"panel panel-default\" id=\"sqlReplicationsForm\">        <div class=\"panel-heading\">            <a class=\"btn btn-default\" data-bind=\"attr: {href:getSqlReplicationUrl(\'\')}\" title=\"New SQL Replication (Alt+N)\" accesskey=\"N\">                <i class=\"fa fa-plus\"></i> New SQL replication            </a>            <a class=\"btn btn-default\" data-bind=\"attr: {href:getSqlReplicationConnectionStringsUrl()}\" accesskey=\"M\" title=\"Manage SQL connection strings (Alt+M)\">                <i class=\"fa fa-database\"></i> Manage connection strings            </a>        </div>        <div class=\"panel-body\">            <div data-bind=\"visible: replications().length === 0\">                <br />                <p>                    <i class=\"fa fa-warning text-warning\"></i> You\'ve got no SQL replications set up.                </p>            </div>            <div class=\"replications-container\" data-bind=\"foreach: replications\">                <div class=\"replication\">                    <h4 data-bind=\"attr: { \'href\': \'\', titel:\'$root.appUrl.forEditSqlReplication(name, this.activeDatabase()) \'}\">                        <a data-bind=\"text: name, attr: { href: $root.getSqlReplicationUrl(name()) }\"></a>                    </h4>                    <label>                        <i class=\"fa fa-power-off\" data-bind=\"attr: { title:disabled() === false?\'SQL Replication is Enabled\':\'SQL Replication is Disabled\'},css:{enabledReplication:disabled() === false,disabledReplication:disabled() === true }\"></i>                    </label>                    <div class=\"sqlRepActions pull-right\">                        <div class=\"btn-group\">                            <a class=\"dropdown-toggle\" data-toggle=\"dropdown\">                                <i class=\"fa fa-cogs fa-lg\"></i>                            </a>                            <ul class=\"dropdown-menu index-dropdown\" role=\"menu\">                                <li role=\"presentation\">                                    <a href=\"#\" title=\"Delete the replication\" data-bind=\"click: $root.removeSqlReplication.bind($root,$data)\">                                        <i class=\"fa fa-trash-o text-danger\"></i> Delete                                    </a>                                </li>                                <li role=\"presentation\">                                    <a href=\"#\" title=\"Show replication stats dialog\" data-bind=\"click: $root.showStats.bind($root,Name)\">                                        Show stats                                    </a>                                </li>                                <li role=\"presentation\">                                    <a href=\"#\" title=\"Reset current replication\" data-bind=\"click: function(){ $root.resetSqlReplication(Name)}\">                                        Reset                                    </a>                                </li>                            </ul>                        </div>                    </div>                </div>            </div>        </div>    </div>    </div>'; } )
define("text!views/database/settings/versioning.html", [], function () { return '<section>    <form class=\"versioning-settings-form form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges.bind($root)\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: isSaveEnabled\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                <span data-bind=\"visible: hasGlobalValues()\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useLocal.bind($root), visible: usingGlobal()\">                        <i class=\"fa fa-arrow-down\"></i> Override global configuration                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.useGlobal.bind($root), visible: !usingGlobal()\">                        <i class=\"fa fa-arrow-up\"></i> Use global configuration                    </button>                </span>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewVersioning, visible: !usingGlobal()\" title=\"Add a replication destination (Alt+N)\" accesskey=\"N\">                    <i class=\"fa fa-plus\"></i> Add versioning                </button>            </div>            <div class=\"panel-body\">                <div class=\"alert alert-warning\" data-bind=\"visible: versionings().length == 0\">                    You\'ve got no versioning set up.                </div>                <div data-bind=\"foreach: versionings\">                    <div class=\"panel versioning-card\" data-bind=\"css: { \'panel-warning\': mergedDocument().disabled, \'panel-default\': !mergedDocument().disabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span> Versioning setup <span data-bind=\"visible: mergedDocument().collection().length>0\">for</span> <span data-bind=\"text: mergedDocument().collection\"></span></span>                                <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\" data-bind=\"click: $root.removeVersioning.bind($root), visible: mergedDocument().removable() && !$root.usingGlobal() && !globalExists()\" title=\"remove this versioning\"><i class=\"fa fa-times\"></i></button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\" data-bind=\"visible: globalExists() && !$root.usingGlobal()\">                                <label class=\"col-sm-4 control-label\">Override:</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: $root.usingGlobal() == false\">                                            <span data-bind=\"visible: localExists()\">Yes</span>                                            <span data-bind=\"visible: !localExists()\">No</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, true)\">Yes</a></li>                                            <li><a href=\"#\" data-bind=\"click: $root.override.bind($root, false)\">No</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Collection</label>                                <div class=\"col-sm-8\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: mergedDocument().collection, enable: mergedDocument().removable && !mergedDocument().fromDatabase() && localExists(), valueUpdate: \'afterkeyup\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Max Revisions</label>                                <div class=\"col-sm-8\">                                    <input required type=\"number\" class=\"form-control\" data-bind=\"numericValue: mergedDocument().maxRevisions, valueUpdate: \'afterkeyup\', enable: !$root.usingGlobal() && localExists()\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Excluded</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: !$root.usingGlobal() && localExists()\">                                            <span data-bind=\"visible: !mergedDocument().exclude()\">Include</span>                                            <span data-bind=\"visible: mergedDocument().exclude\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"with: mergedDocument\">                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, true)\">Exclude</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Exclude unless explicit</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: !$root.usingGlobal() && localExists()\">                                            <span data-bind=\"visible: !mergedDocument().excludeUnlessExplicit()\">Include</span>                                            <span data-bind=\"visible: mergedDocument().excludeUnlessExplicit\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"with: mergedDocument\">                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, true)\">Exclude</a></li>                                        </ul>                        </div>                    </div>                </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Purge on delete</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"enable: !$root.usingGlobal() && localExists()\">                                            <span data-bind=\"visible: !mergedDocument().purgeOnDelete()\">No</span>                                            <span data-bind=\"visible: mergedDocument().purgeOnDelete\">Yes</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"with: mergedDocument\">                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, false)\">No</a></li>                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, true)\">Yes</a></li>                                        </ul>            </div>        </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/alerts.html", [], function () { return '<section id=\"alertsContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div style=\"height: 65%; position: relative;\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: saveAlerts\" title=\"Save changes\"><i class=\"fa fa-save\"></i></button>        <button title=\"Refresh alerts\" class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 10000, onRefresh: $root.fetchAlerts.bind($root), active:autoRefreshEnabled }\"></button>        <div class=\"btn-group\" data-toggle=\"buttons\">            <label class=\"btn btn-default active\" data-bind=\"click: setFilterAll\">                <input type=\"radio\" name=\"options\" id=\"option1\"> <span data-bind=\"text: \'All alerts (\' + allAlerts().length + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterUnread\">                <input type=\"radio\" name=\"options\" id=\"option1\"> <span data-bind=\"text: \'Unread (\' + unreadAlertCount() + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterRead\">                <input type=\"radio\" name=\"options\" id=\"option2\"> <span data-bind=\"text: \'Read (\' + readAlertCount() + \')\'\"></span>            </label>        </div>        <div class=\"btn-group\" data-bind=\"with: selectedAlert\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: observed() === false, click: $root.toggleSelectedReadState.bind($root)\">Mark as read</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: observed, click: $root.toggleSelectedReadState.bind($root)\">Mark as unread</button>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    <i class=\"fa fa-trash-o\"></i>                    <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li><a href=\"#\" data-bind=\"click: $root.deleteSelectedAlert.bind($root)\">Delete selected alert</a></li>                    <li><a href=\"#\" data-bind=\"click: $root.deleteReadAlerts.bind($root)\">Delete read alerts</a></li>                    <li><a href=\"#\" data-bind=\"click: $root.deleteAllAlerts.bind($root)\">Delete all alerts</a></li>                </ul>            </div>        </div>        <div id=\"alertTableContainer\" style=\"overflow-y: auto; position: absolute; bottom: 10px; top: 40px; width: 100%;\">            <table class=\"table table-condensed single-line-row-table\" data-bind=\"event: { keydown: tableKeyDown }\" tabindex=\"1\">                <thead>                    <tr>                        <th class=\"col-md-1\">Level</th>                        <th class=\"col-md-2\">Reported</th>                        <th class=\"col-md-4\">Title</th>                        <th class=\"col-md-4\" >Message</th>                        <th class=\"col-md-2\">Exception</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: allAlerts, delegatedHandler: \'click\'\">                    <tr data-click=\"selectAlert\" data-bind=\"visible: isVisible, css: { \'unread\': observed() === false, \'selected\': $data === $parent.selectedAlert(), warning: alertLevel === \'Warn\', danger: alertLevel === \'Error\' || alertLevel === \'Fatal\' }\">                        <td data-bind=\"text: alertLevel\"></td>                        <td data-bind=\"text: createdAtHumanized, attr: { title: createdAtHumanized }\"></td>                        <td data-bind=\"text: title, attr: { title: title }\"></td>                        <td data-bind=\"text: message, attr: { title: message }\"></td>                        <td data-bind=\"text: exception, attr: { title: exception }\"></td>                    </tr>                </tbody>            </table>        </div>    </div>    <div class=\"alerts-details panel panel-default\" style=\"height: 30%;\">        <div class=\"row\">            <label class=\"col-md-1 text-right control-label\">Title</label>            <div class=\"col-md-10\" data-bind=\"with: selectedAlert\">                <p class=\"form-control-static\" data-bind=\"text: title\"></p>            </div>        </div>        <div class=\"row\">            <label class=\"col-md-1 text-right control-label\">Message</label>            <div class=\"col-md-10\" data-bind=\"with: selectedAlert\">                <p class=\"form-control-static\" data-bind=\"text: message\"></p>            </div>        </div>        <div class=\"row\">            <label class=\"col-md-1 text-right control-label\">Exception</label>            <div class=\"col-md-10\" data-bind=\"with: selectedAlert\">                <p class=\"form-control-static exceptionText\" data-bind=\"text: exception\"></p>            </div>        </div>        <div class=\"row\">            <label class=\"col-md-1 text-right control-label\">Reported</label>            <div class=\"col-md-10\" data-bind=\"with: selectedAlert\">                <p class=\"form-control-static\" data-bind=\"text: createdAtHumanized\"></p>            </div>        </div>        <div class=\"row\">            <label class=\"col-md-1 text-right control-label\">Level</label>            <div class=\"col-md-10\" data-bind=\"with: selectedAlert\">                <p class=\"form-control-static\" data-bind=\"text: alertLevel, css: { \'text-warning\': alertLevel === \'Warn\', \'text-danger\': alertLevel === \'Error\' || alertLevel === \'Fatal\' }\"></p>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/indexErrors.html", [], function () { return '<section id=\"indexErrorsContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div class=\"page-header\" data-bind=\"visible: hasFetchedErrors() && allIndexErrors().length === 0\">        <h3><i class=\"fa fa-info-circle text-info\"></i> No errors have occurred.</h3>    </div>    <div style=\"height: 65%; position: relative;\" data-bind=\"visible: hasFetchedErrors() && allIndexErrors().length > 0\">        <div id=\"indexErrorsTableContainer\" style=\"overflow-y: auto; position: absolute; bottom: 10px; top: 80px; width: 100%;\">            <table class=\"table table-condensed single-line-row-table\" data-bind=\"event: { keydown: tableKeyDown }\" tabindex=\"1\">                <thead>                    <tr>                        <th style=\"width: 10%\">Index</th>                        <th style=\"width: 10%\">Action</th>                        <th style=\"width: 10%\">Timestamp</th>                        <th style=\"width: 60%\">Error</th>                        <th style=\"width: 10%\">Document</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: allIndexErrors, delegatedHandler: \'click\'\">                    <tr data-click=\"selectIndexError\" data-bind=\"css: { \'selected\': $data === $parent.selectedIndexError() }\">                        <td data-bind=\"text: IndexName, attr: { title: IndexName }\"></td>                        <td data-bind=\"text: Action, attr: { title: Action }\"></td>                        <td data-bind=\"text: TimestampHumanized, attr: { title: TimestampHumanized }\"></td>                        <td data-bind=\"text: Error, attr: { title: Error }\"></td>                        <td data-bind=\"text: Document, attr: { title: Document }\"></td>                    </tr>                </tbody>            </table>        </div>    </div>    <div class=\"index-errors-details panel panel-default\" style=\"height: 30%;\" data-bind=\"visible: hasFetchedErrors && allIndexErrors().length > 0\">        <form class=\"form-horizontal\" role=\"form\">            <div class=\"form-group\">                <label class=\"col-md-1 text-right control-label\">Index</label>                <div class=\"col-md-10\" data-bind=\"with: selectedIndexError\">                    <p class=\"form-control-static\" data-bind=\"text: IndexName\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-1 text-right control-label\">Error</label>                <div class=\"col-md-10\" data-bind=\"with: selectedIndexError\">                    <p class=\"form-control-static exceptionText\" data-bind=\"text: Error\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-1 text-right control-label\">Timestamp</label>                <div class=\"col-md-10\" data-bind=\"with: selectedIndexError\">                    <p class=\"form-control-static\" data-bind=\"text: TimestampHumanized\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-1 text-right control-label\">Index</label>                <div class=\"col-md-10\" data-bind=\"with: selectedIndexError\">                    <p class=\"form-control-static\" data-bind=\"text: IndexName\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-1 text-right control-label\">Document</label>                <div class=\"col-md-10\" data-bind=\"with: selectedIndexError\">                    <p class=\"form-control-static\" data-bind=\"text: Document\"></p>                </div>            </div>        </form>    </div></section>'; } )
define("text!views/database/status/logs.html", [], function () { return '<section id=\"logsContainer\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div class=\"form-horizontal\">        <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 2000, onRefresh: $root.fetchLogs.bind($root), active:true }\"></button>        <div class=\"btn-group\" data-toggle=\"buttons\">            <label class=\"btn btn-default active\" data-bind=\"click: setFilterAll\">                <input type=\"radio\" name=\"options\" id=\"option1\"> <span data-bind=\"text: \'All logs (\' + allLogs().length + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterDebug\">                <input type=\"radio\" name=\"options\" id=\"option2\"> <span data-bind=\"text: \'Debug (\' + debugLogCount() + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterInfo\">                <input type=\"radio\" name=\"options\" id=\"option3\"> <span data-bind=\"text: \'Info (\' + infoLogCount() + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterWarning\">                <input type=\"radio\" name=\"options\" id=\"option4\"> <span data-bind=\"text: \'Warnings (\' + warningLogCount() + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterError\">                <input type=\"radio\" name=\"options\" id=\"option5\"> <span data-bind=\"text: \'Errors (\' + errorLogCount() + \')\'\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setFilterFatal\">                <input type=\"radio\" name=\"options\" id=\"option6\"> <span data-bind=\"text: \'Fatal (\' + fatalLogCount() + \')\'\"></span>            </label>        </div>        <form class=\"form-inline\" style=\"display: inline-block; vertical-align: middle;\">            <input type=\"text\" class=\"form-control col-md-8\" placeholder=\"Filter\" data-bind=\"value: searchText, valueUpdate: \'afterkeydown\', submit: function(){ }\" />        </form>        <div class=\"form-group\" id=\"logTableContainer\"  data-bind=\"event: { keydown: tableKeyDown }\">            <div style=\"padding-top: 10px;\" data-bind=\"foreach: filteredLoggers\">                <button class=\"btn btn-default\" data-bind=\"click: $root.unHidelogType.bind($root)\">                    <span data-bind=\"text: $data\"></span>                    <span class=\"fa fa-times\"></span>                </button>            </div>            <div class=\"form-group logHeaders\">                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[0]() + \'%\' }\">                    <span class=\"column-handle\" column=\"0\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'level\')\">Level</span>                    <span data-bind=\"visible: $root.sortColumn() === \'level\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[1]() + \'%\' }\">                    <span class=\"column-handle\" column=\"1\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'dateRaw\')\">Logged</span>                    <span data-bind=\"visible: $root.sortColumn() === \'dateRaw\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[2]() + \'%\' }\">                    <span class=\"column-handle\" column=\"2\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'message\')\">Message</span>                    <span data-bind=\"visible: $root.sortColumn() === \'message\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[3]() + \'%\' }\">                    <span class=\"column-handle\" column=\"3\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'exception\')\">Exception</span>                    <span data-bind=\"visible: $root.sortColumn() === \'exception\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[4]() + \'%\' }\">                    <span class=\"column-handle\" column=\"4\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'loggerName\')\">Log name</span>                    <span data-bind=\"visible: $root.sortColumn() === \'loggerName\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>            </div>            <div id=\"logRecords\" class=\"form-group logRecords no-outline\" style=\"white-space: nowrap\" tabindex=\"-1\" data-bind=\"foreach: allLogs, delegatedHandler: \'click\'\">                <div id=\"logRecordsContainer\" style=\"display: block; white-space: nowrap\" class=\"logRecord log-table-row\" data-click=\"selectLog\" data-bind=\"visible: isVisible, css: { warning: level() === \'Warn\', danger: level() === \'Error\' || level() === \'Fatal\', \'selected\': $data === $parent.selectedLog() }\">                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[0]() + \'%\' }, text: level\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[1]() + \'%\' }, text: humanizedTimestamp, attr: { title: humanizedTimestamp() + timeStampText() }\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[2]() + \'%\' }, text: message, attr: { title: message }\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[3]() + \'%\' }, text: exception() ? exception : \'-\', attr: { title: exception }\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[4]() + \'%\' }, text: loggerName\"></div>                </div>            </div>            <div class=\"form-group log-details panel panel-default\" data-bind=\"visible: showLogDetails\">                <form class=\"col-md-10\" role=\"form\" data-bind=\"with: selectedLog\">                    <div class=\"row\">                        <label class=\"col-md-2 text-right control-label\">Message</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: message\"></p>                        </div>                    </div>                    <div class=\"row\">                        <label class=\"col-md-2 text-right control-label\">Exception</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static exceptionText\" data-bind=\"text: exception() || \'-\'\"></p>                        </div>                    </div>                    <div class=\"row\">                        <label class=\"col-md-2 text-right control-label\">Logged</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: humanizedTimestamp() + timeStampText()\"></p>                        </div>                    </div>                    <div class=\"row\">                        <label class=\"col-md-2 text-right control-label\">Level</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: level, css: { \'text-warning\': level() === \'Warn\', \'text-danger\': level() === \'Error\' || level() === \'Fatal\' }\"></p>                        </div>                    </div>                    <div class=\"row\">                        <label class=\"col-md-2 text-right control-label\">Logger</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: loggerName\"></p>                        </div>                    </div>                </form>                <div class=\"pull-right\">                    <div class=\"btn-group\">                        <button class=\"btn btn-default\" data-bind=\"click: $root.unSelectLog.bind($root)\">Hide</button>                    </div>                    <div class=\"btn-group pull-right\" data-bind=\"with: selectedLog\">                        <button class=\"btn btn-default\" data-bind=\"click: $root.hideLogType.bind($root), attr:{title: \'Filter out all \' + loggerName().split(\'.\').slice(2).join(\'.\') + \' logs\'}\">                            <i class=\"fa fa-filter\"></i>                        </button>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/replicationStats.html", [], function () { return '<style>    #replicationStatsContainer {        font: 11px sans-serif;    }    .axis path,    .axis line {        fill: none;        stroke: #000;        shape-rendering: crispEdges;    }    .bar {        fill: steelblue;    }    rect.outputCounts, rect.inputCounts {        fill-opacity: 0.8;        cursor: pointer;    }    rect.outputCounts:hover, rect.inputCounts:hover {        fill-opacity: 1;    }    .legend rect {        cursor: pointer;    }    .legend_bg {        fill: white;        fill-opacity: 0.8;        stroke: #ccc;    }    .legendHidden {        fill-opacity: 0.2;       }</style><section id=\"replicationTopologySection\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div class=\"page-header\" data-bind=\"visible: hasNoReplStatsAvailable\">        <h3><i class=\"fa fa-info-circle text-info\"></i> No replication stats available. <small>Either this database isn\'t <a target=\"_blank\" href=\"http://ravendb.net/docs/server/scaling-out/replication\">setup for replication</a>, or replication hasn\'t occurred in some time.</small></h3>    </div>    <div data-bind=\"with: replStatsDoc\">        <div class=\"panel-group\" id=\"indexStatsAccordion\" data-bind=\"foreach: Stats, visible: Stats.length > 0\">            <div class=\"panel\" data-bind=\"css:{\'panel-default\':!isHotFailure, \'panel-danger\':isHotFailure}\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <a data-toggle=\"collapse\" href=\"#collapseOne\" data-bind=\"text: Url, attr: { href: \'#statsIndexCollapse\' + $index() }\"></a>                    </h4>                </div>                <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'statsIndexCollapse\' + $index() }\">                    <div class=\"panel-body\">                        <div class=\"form-horizontal\" role=\"form\">                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last success</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastSuccessTimestampHumanized\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last failure</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastFailureTimestampHumanized\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last failure message</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastError\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Failures</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: FailureCount\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last replicated ETag</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastReplicatedEtag\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last replicated Attachment ETag</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastReplicatedAttachmentEtag\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last checked ETag</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastEtagCheckedForReplication\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-2 control-label\">Last heartbeat</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: LastHeartbeatReceivedHumanized\"></p>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div>    <div>        <div class=\"panel-group\" data-bind=\"visible: hasReplicationEnabled\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title pull-left\">                        <a data-toggle=\"collapse\" href=\"#replicationSetupCollapse\">Replication topology</a>                    </h4>                    <div class=\"pull-right btn-group\" data-bind=\"if: topology\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Download topology\">                            <i class=\"fa fa-download\"></i>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li data-bind=\"click: saveAsPng, visible: hasSaveAsPngSupport\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> PNG</a></li>                            <li data-bind=\"click: saveAsSvg\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> SVG</a></li>                            <li data-bind=\"click: saveAsJson\"><a href=\"#\"><i class=\"fa fa-file-text-o\"></i> JSON</a></li>                        </ul>                    </div>                    <div class=\"clearfix\"></div>                </div>                <div class=\"panel-collapse collapse\" id=\"replicationSetupCollapse\">                    <div class=\"panel-body\">                        <div class=\"row\" data-bind=\"with: topology\">                            <div class=\"col-xs-8\">                                <svg id=\"replicationTopology\"  transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>                            </div>                            <div class=\"col-xs-4\">                                <div data-bind=\"if: !$root.currentLink()\">                                    <div class=\"well\">                                        <i class=\"fa fa-info-circle text-info\"></i> Click on the <strong>arrows between nodes</strong> to display information about the replication relationship.                                        <br/>                                        <br/>                                        <div>                                            <dl>                                                <dt class=\"text-success\"><i class=\"fa fa-long-arrow-right fa-2x\"></i></dt>                                                <dd>Endpoints can communicate each other</dd>                                            </dl>                                            <dl>                                                <dt class=\"text-danger\"><i class=\"fa fa-long-arrow-right fa-2x\"></i></dt>                                                <dd>Error in communication</dd>                                            </dl>                                        </div>                                    </div>                                </div>                                <div data-bind=\"with: $root.currentLink\">                                    <div class=\"panel panel-default\">                                        <div class=\"panel-body\">                                            <table class=\"table table-striped table-condensed\">                                                <thead>                                                    <tr>                                                        <th>Property</th>                                                        <th>Value</th>                                                    </tr>                                                </thead>                                                <tbody>                                                    <tr>                                                        <td>Source</td>                                                        <td data-bind=\"text: Source\"></td>                                                    </tr>                                                    <tr>                                                        <td>Destination</td>                                                        <td data-bind=\"text: Destination\"></td>                                                    </tr>                                                    <tr>                                                        <td>SendServerId</td>                                                        <td data-bind=\"text: SendServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>StoredServerId</td>                                                        <td data-bind=\"text: StoredServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastAttachmentEtag</td>                                                        <td data-bind=\"text: LastAttachmentEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastDocumentEtag</td>                                                        <td data-bind=\"text: LastDocumentEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>ReplicationBehavior</td>                                                        <td data-bind=\"text: ReplicationBehavior\"></td>                                                    </tr>                                                    <tr>                                                        <td>SourceToDestinationState</td>                                                        <td data-bind=\"text: SourceToDestinationState\"></td>                                                    </tr>                                                    <tr>                                                        <td>DestinationToSourceState</td>                                                        <td data-bind=\"text: DestinationToSourceState\"></td>                                                    </tr>                                                    <tr data-bind=\"if: Errors\">                                                        <td>Errors</td>                                                        <td>                                                            <ul data-bind=\"foreach: Errors\">                                                                <li data-bind=\"text: $data\"></li>                                                            </ul>                                                        </td>                                                    </tr>                                                </tbody>                                            </table>                                        </div>                                    </div>                                </div>                            </div>                        </div>                        <div data-bind=\"visible: topology() == null\">                            <button data-bind=\"click: fetchTopology\" class=\"btn btn-default\"><i class=\"fa fa-sitemap\"></i> Fetch topology</button>                            <p class=\"help-block\">                                <i class=\"fa fa-warning text-warning\"></i> This operation may be time consuming.                            </p>                            <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicator\" class=\"glyphicon\" style=\"width: 30px; height: 30px;\"><span class=\"fa fa-spinner fa-spin fa-2x\"></span></div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel-group\" data-bind=\"visible: hasReplicationEnabled\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title pull-left\">                        <a data-bind=\"click: replicationStatToggle\" data-toggle=\"collapse\" href=\"#replicationPerfStatsCollapse\">Replication Performance Statistics</a>                    </h4>                    <div class=\"clearfix\"></div>                </div>                <div class=\"panel-collapse collapse\" id=\"replicationPerfStatsCollapse\">                    <div class=\"panel-body\">                        <div id=\"replicationStatsContainer\" style=\"width: 100%; height: 600px\" >                            <svg id=\"replicationStatsGraph\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\">                                                            </svg>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/runningTasks.html", [], function () { return '<style>.column {    display: inline-block;    cursor: pointer;    overflow: hidden;    white-space: nowrap;    word-wrap: break-word;    text-overflow: ellipsis;    border-bottom: 1px solid #ddd;    margin: 0px;    padding: 5px;    height: 30px;}.selected .column {    background-color: #ffffcc;}</style><section id=\"runningTasksContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div style=\"height: 65%; position: relative;\">        <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 5000, onRefresh: $root.fetchTasks.bind($root), active:true }\"></button>        <div class=\"btn-group\" data-toggle=\"buttons\">            <label class=\"btn btn-default active\" data-bind=\"click: setFilterTypeAll\">                <input type=\"radio\" name=\"options\" id=\"option0\"> <span data-bind=\"text: \'All tasks (\' + allTasks().length + \')\'\"></span>            </label>            <!-- ko foreach: taskTypes -->            <label class=\"btn btn-default\" data-bind=\"click: $root.setFilterType.bind($root, name)\">                <input type=\"radio\" name=\"options\"> <span data-bind=\"text:  name + \' (\' + count + \')\'\"></span>            </label>            <!-- /ko -->        </div>        <form class=\"form-inline\" style=\"display: inline-block; vertical-align: middle;\">            <input type=\"text\" class=\"form-control col-md-8\" placeholder=\"Search\" data-bind=\"value: searchText, valueUpdate: \'afterkeydown\'\" />        </form>        <div id=\"runningTasksTableContainer\" class=\"no-outline\" tabindex=\"-1\" data-bind=\"event: { keydown: tableKeyDown }\" style=\"overflow-y: auto; overflow-x: auto; position: absolute; top: 40px; bottom: 10px; width: 100%;\">            <div style=\"white-space: nowrap\">                <div class=\"column column-header\"  style=\"width: 40px;\">                    <span>Kill</span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[0]() + \'px\' }\">                    <span class=\"column-handle\" column=\"0\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'taskType\')\">Type</span>                    <span data-bind=\"visible: $root.sortColumn() === \'taskType\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[1]() + \'px\' }\">                    <span class=\"column-handle\" column=\"1\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'description\')\">Description</span>                    <span data-bind=\"visible: $root.sortColumn() === \'description\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[2]() + \'px\' }\">                    <span class=\"column-handle\" column=\"2\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'fullStatus\')\">Status</span>                    <span data-bind=\"visible: $root.sortColumn() === \'fullStatus\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[3]() + \'px\' }\">                    <span class=\"column-handle\" column=\"3\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'startTime\')\">Started</span>                    <span data-bind=\"visible: $root.sortColumn() === \'startTime\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>                <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[4]() + \'px\' }\">                    <span class=\"column-handle\" column=\"4\"></span>                    <span data-bind=\"click: $root.sortBy.bind($root, \'exceptionText\')\">Exception</span>                    <span data-bind=\"visible: $root.sortColumn() === \'exceptionText\'\">                        <span class=\"fa fa-sort-asc\" data-bind=\"visible: $root.sortAsc\"></span>                        <span class=\"fa fa-sort-desc\" data-bind=\"visible: !$root.sortAsc()\"></span>                    </span>                </div>            </div>            <div id=\"runningTasksItemsContainer\" style=\"white-space: nowrap\" data-bind=\"foreach: filteredAndSortedTasks, delegatedHandler: \'click\'\">                <div style=\"display: block; white-space: nowrap\" class=\"logRecord log-table-row\" data-click=\"selectTask\" data-bind=\"css: { warning: canceled, danger: faulted, \'selected\': $data === $parent.selectedTask() }\">                    <div class=\"column\" style=\"width: 40px\"><button data-bind=\"visible: killable, click: $root.taskKill.bind($root, $data)\" class=\"btn btn-link negative-margin\"><i class=\"fa fa-bolt\"></i></button>                    </div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[0]() + \'px\' }, text: taskType\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[1]() + \'px\' }, text: description\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[2]() + \'px\' }, text: fullStatus\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[3]() + \'px\' }, text: timeStampText, attr: { title: timeStampText }\"></div>                    <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[4]() + \'px\' }\"><i data-bind=\"visible: exceptionText\" class=\"fa fa-exclamation\"></i>                    </div>                </div>            </div>        </div>    </div>    <div class=\"log-details panel panel-default\">        <form class=\"form-horizontal\" role=\"form\">            <div class=\"form-group\">                <label class=\"col-md-2 text-right control-label\">Type</label>                <div class=\"col-md-10\" data-bind=\"with: selectedTask\">                    <p class=\"form-control-static\" data-bind=\"text: taskType\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-2 text-right control-label\">Started</label>                <div class=\"col-md-10\" data-bind=\"with: selectedTask\">                    <p class=\"form-control-static\" data-bind=\"text: timeStampText\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-2 text-right control-label\">Status</label>                <div class=\"col-md-10\" data-bind=\"with: selectedTask\">                    <p class=\"form-control-static\" data-bind=\"text: fullStatus\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-2 text-right control-label\">Description</label>                <div class=\"col-md-10\" data-bind=\"with: selectedTask\">                    <p class=\"form-control-static\" data-bind=\"text: description\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-2 text-right control-label\">Exception</label>                <div class=\"col-md-10\" data-bind=\"with: selectedTask\">                    <p class=\"form-control-static\">                        <span data-bind=\"text: exceptionText\"></span>                    </p>                </div>            </div>            <div class=\"clearfix\"></div>        </form>        <div class=\"clearfix\"></div>    </div></section>'; } )
define("text!views/database/status/sqlReplicationPerfStats.html", [], function () { return '<style>    #replicationStatsContainer {        font: 11px sans-serif;    }    .axis path,    .axis line {        fill: none;        stroke: #000;        shape-rendering: crispEdges;    }    .bar {        fill: steelblue;    }    rect.outputCounts, rect.inputCounts {        fill-opacity: 0.8;        cursor: pointer;    }    rect.outputCounts:hover, rect.inputCounts:hover {        fill-opacity: 1;    }    .legend rect {        cursor: pointer;    }    .legend_bg {        fill: white;        fill-opacity: 0.8;        stroke: #ccc;    }    .legendHidden {        fill-opacity: 0.2;       }</style><section id=\"replicationStatsContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div class=\"page-header\" data-bind=\"visible: hasReplicationEnabled() == false\">        <h3><i class=\"fa fa-info-circle text-info\"></i> No SQL Replication stats available. <small>Either this database isn\'t <a target=\"_blank\" href=\"http://ravendb.net/docs/server/scaling-out/sql-replication\">setup for sql replication</a>, or sql replication hasn\'t occurred in some time.</small></h3>    </div>        <div data-bind=\"visible: hasReplicationEnabled()\">        <svg id=\"replicationStatsGraph\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>    </div></section>'; } )
define("text!views/database/status/sqlReplicationSimulationDialog.html", [], function () { return '<div class=\"messageBox replicationSimulation\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: $root.cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text:\'SQL Replication Simulation For: \' + simulatedSqlReplication.name()\"></h4>    </div>    <div class=\"modal-body replicationSimulationData\">        <div class=\"modal-body\">            <nav class=\"navbar navbar-default\" role=\"navigation\">                <div class=\"collapse navbar-collapse btn-toolbar\">                    <div class=\"col-sm-4\">                        <input id=\"docIdInput\" title=\"Document to run simulation with\" placeholder=\"Document ID\"                               type=\"text\" class=\"form-control\" data-bind=\"value: documentId, valueUpdate: \'afterkeydown\'\" autocomplete=\"off\" />                        <div data-bind=\"visible:isAutoCompleteVisible\">                            <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\"                                data-bind=\"autoComplete: \'#docIdInput\', foreach: documentAutocompletes\">                                <li role=\"presentation\" data-bind=\"click:function(){$root.documentIdSubmitted($data[\'@metadata\'][\'@id\']); }\">                                    <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                        <span class=\"collection-color-strip\" data-bind=\"css: $root.getDocCssClass($data)\"></span>                                        <span data-bind=\"text: $data[\'@metadata\'][\'@id\']\"></span>                                    </a>                                </li>                            </ul>                        </div>                    </div>                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-info\" data-bind=\"click:$root.getResults.bind($root,false)\">Simulate!</button>                        <button type=\"button\" class=\"btn btn-info dropdown-toggle\" data-toggle=\"dropdown\">                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\">                            <li>                                <a href=\"#\" data-bind=\"click:$root.getResults.bind($root,true)\">Test transaction</a>                            </li>                        </ul>                    </div>                                        <div class=\"progress progress-striped\" data-bind=\"visible:waitingToAnswer\" style=\"padding:0; \">                        <div class=\"progress-bar progress-bar-info active\" role=\"progressbar\" aria-valuenow=\"100\" aria-valuemin=\"0\" aria-valuemax=\"100\" style=\"width: 100%; float:left;\">                        </div>                    </div>                </div>            </nav>            <div class=\"form-group\" data-bind=\"if: !rolledbackTransactionPerformed(), visible: !rolledbackTransactionPerformed()\">                <label class=\"col-sm-2 control-label\" data-bind=\"visible: !!lastAlert()\">                    Simulation Error:                </label>                <pre class=\"col-sm-10\" data-bind=\"visible: !!lastAlert(), text: lastAlert\"></pre>            </div>            <div data-bind=\"if: rolledbackTransactionPerformed(), visible: rolledbackTransactionPerformed()\">                <div class=\"form-group\">                    <label class=\"col-sm-2  control-label\">                        Test Transaction Status:                    </label>                    <label class=\"col-sm-10\" data-bind=\"text:rolledBackTransactionPassed()==true?\'Passed\':\'Failed\', css:{\'text-success\':rolledBackTransactionPassed() === true, \'text-danger\':rolledBackTransactionPassed() === false}\"></label>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2  control-label\" data-bind=\"visible:rolledBackTransactionPassed()==false|| !!lastAlert()\">                        Test Transaction Error:                    </label>                    <span class=\"col-sm-10\" data-bind=\"visible:rolledBackTransactionPassed() == false || !!lastAlert(), text:lastAlert\"></span>                </div>            </div>            <div class=\"form-group\">                <!-- ko if: simulationResults().length> 0 -->                <label class=\"col-sm-12  control-label\" data-bind=\"text:rolledbackTransactionPerformed()?\'Test SQL Commands:\':\'Simulated SQL Commands:\'\">                </label>                <div class=\"col-sm-12 simulationResultContainer list-group\" data-bind=\"foreach:simulationResults\">                    <a class=\"col-sm-offset-1 col-sm-10 sqlQuery\" data-bind=\"click:function(){$data.showParamsValues.toggle();}, css:{\'parametrizedQuery\':($root.rolledbackTransactionPerformed() == true && (commandText().indexOf(\'INSERT\') === 0 || $root.simulatedSqlReplication.parameterizeDeletesDisabled() === false))}\">                        <span data-bind=\"text:commandText\"></span>                    </a>                </div>                <!-- /ko -->                <div class=\"col-sm-12\" data-bind=\"visible: simulationResults().length== 0\">                    <h4>                        No results                    </h4>                </div>            </div>        </div>    </div>    <div class=\"modal-footer\">        <span class=\"pull-left\" data-bind=\"text:rolledbackTransactionPerformed() == true?\'* Press command to toggle parameters view\':\'* This is debug data, real queries will be parametrised. Try pressing the test transaction button\'\"></span>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: $root.cancel\">OK</button>    </div></div>'; } )
define("text!views/database/status/sqlReplicationStatsDialog.html", [], function () { return '<div class=\"messageBox sqlReplication-Stats\" tabindex=\"-1\" >    <div class=\"modal-header\">        <button type=\"button\" class=\"close btn btn-default\" data-bind=\"click: fetchSqlReplicationStats\"><i class=\"fa fa-refresh\"></i></button>        <h4 class=\"modal-title\" data-bind=\"text:\'SQL Replication Stats For: \' + sqlReplicationName\"></h4>    </div>    <div class=\"modal-body replicationStatsData\">        <div class=\"modal-body\">            <!-- ko if: !!repData()  -->            <div class=\"form-horizontal\" role=\"form\" data-bind=\"with: repData\">                <div class=\"form-group\">                    <label class=\"col-lg-2 col-md-4 control-label\">Last error time</label>                    <div class=\"col-lg-10 col-md-8\">                        <p class=\"form-control-static\" data-bind=\"text: LastErrorTime\"></p>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-lg-2 col-md-4 control-label\">Script error count</label>                    <div class=\"col-lg-10 col-md-8\">                        <p class=\"form-control-static\" data-bind=\"text: ScriptErrorCount\"></p>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-lg-2 col-md-4 control-label\">Script success count</label>                    <div class=\"col-lg-10 col-md-8\">                        <p class=\"form-control-static\" data-bind=\"text: ScriptSuccessCount\"></p>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-lg-2 col-md-4 control-label\">Write error count</label>                    <div class=\"col-lg-10 col-md-8\">                        <p class=\"form-control-static\" data-bind=\"text: WriteErrorCount\"></p>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-lg-2 col-md-4 control-label\">Success count</label>                    <div class=\"col-lg-10 col-md-8\">                        <p class=\"form-control-static\" data-bind=\"text: SuccessCount\"></p>                    </div>                </div>                <!-- ko with: LastAlert -->                <div class=\"panel-group\" id=\"statsAccordion\">                    <div class=\"panel panel-warning\">                        <div class=\"panel-heading\">                            <a data-toggle=\"collapse\" data-parent=\"#statsAccordion\" href=\"#collapseLastError\">                                <h3 class=\"panel-title\">                                    Last Alert                                </h3>                            </a>                        </div>                        <div id=\"collapseLastError\" class=\"panel-collapse collapse\">                            <div class=\"panel-body\">                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Title</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Title\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Created at</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: CreatedAt\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Observed</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Observed\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Message</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Message\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Alert level</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: AlertLevel\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Exception</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\"><span class=\"exceptionText\" data-bind=\"text: Exception\"></span></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-lg-2 control-label\">Unique key</label>                                    <div class=\"col-lg-10\">                                        <p class=\"form-control-static\" data-bind=\"text: UniqueKey\"></p>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>                <!-- /ko -->            </div>            <!-- /ko -->            <div class=\"alert alert-warning text-center\" data-bind=\"visible: !repData()\">                <h3>                    No stats available                </h3>            </div>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: $root.cancel\">OK</button>    </div></div>'; } )
define("text!views/database/status/statistics.html", [], function () { return '<section data-bind=\"if: stats, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <form class=\"form-horizontal col-md-10\" role=\"form\">        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Storage Engine</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().StorageEngine\"></p>            </div>        </div><div class=\"form-group\">            <label class=\"col-md-2 control-label\">Last Document Etag</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().LastDocEtag\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Last Attachment Etag</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().LastAttachmentEtag\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Index Count</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().CountOfIndexes\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Document Count</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().CountOfDocumentsLocale\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Stale Indexes</label>            <ul class=\"col-md-10 list-unstyled\" style=\"margin-top: 8px\" data-bind=\"visible: noStaleIndexes\">                <li>-</li>            </ul>            <ul class=\"col-md-10 list-unstyled\" style=\"margin-top: 8px\" data-bind=\"foreach: stats().StaleIndexes, visible: noStaleIndexes() == false\">                <!-- ko if:$root.disabledIndexes().indexOf($data) == -1 -->                <li data-bind=\"text: $data\"></li>                <!-- /ko -->            </ul>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Index Batch Size</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().CurrentNumberOfItemsToIndexInSingleBatchLocale\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Last Indexing Time</label>            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: stats().LastIndexingDateTime\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Reduce Batch Size</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: stats().CurrentNumberOfItemsToReduceInSingleBatchLocale\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Indexes</label>            <div class=\"col-md-10\">                <div class=\"panel-group\" id=\"indexStatsAccordion\" data-bind=\"foreach: indexes\">                    <div class=\"panel panel-default\">                        <div class=\"panel-heading\">                            <h4 class=\"panel-title\">                                <a data-toggle=\"collapse\" href=\"#collapseOne\" data-bind=\"attr: { href: \'#statsIndexCollapse\' + $index() }\">                                    <span data-bind=\"text: Name\"></span>                                    <span class=\"text-muted\">- <span data-bind=\"text: DocsCountLocale\"></span> entries</span>                                </a>                            </h4>                        </div>                        <div class=\"panel-collapse collapse\" data-bind=\"attr: { id: \'statsIndexCollapse\' + $index() }\">                            <div class=\"panel-body\">                                <div class=\"form-horizontal\" role=\"form\">                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Created</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: CreatedTimestampText\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Entries</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: DocsCountLocale\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\" data-bind=\"if: IndexingAttempts !== null\">                                        <label class=\"col-md-2 control-label\">Runs</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"css: { \'text-success\': IndexingAttempts === IndexingSuccesses, \'text-warning\': IndexingSuccesses > 0 && IndexingSuccesses < IndexingAttempts, \'text-danger\': IndexingAttempts > 0 && IndexingSuccesses === 0 }\">                                                <span data-bind=\"visible: IndexingAttempts > 0, text: IndexingAttemptsLocale\"></span>                                                <span data-bind=\"visible: IndexingAttempts === 0\" class=\"text-muted\">Never run</span>                                                <span data-bind=\"visible: IndexingAttempts === IndexingSuccesses && IndexingAttempts > 0\">- all successful</span>                                                <span data-bind=\"visible: IndexingSuccesses > 0 && IndexingSuccesses < IndexingAttempts, text: \'attempts \' + (IndexingAttempts - IndexingSuccesses) + \' failed\'\"></span>                                                <span data-bind=\"visible: IndexingAttempts > 0 && IndexingSuccesses === 0\">- all failed</span>                                            </p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Docs indexed from</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LastIndexedTimestampText\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\" data-bind=\"if: IndexingAttempts !== null\">                                        <label class=\"col-md-2 control-label\">Maps reduced from</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"css: { \'text-success\': ReduceIndexingAttempts === ReduceIndexingSuccesses, \'text-warning\': ReduceIndexingSuccesses > 0 && ReduceIndexingSuccesses < ReduceIndexingAttempts, \'text-danger\': ReduceIndexingAttempts > 0 && ReduceIndexingSuccesses === 0 }\">                                                <span data-bind=\"visible: ReduceIndexingAttempts > 0, text: ReduceIndexingAttemptsLocale\"></span>                                                <span data-bind=\"visible: !ReduceIndexingAttempts\" class=\"text-muted\">Never run</span>                                                <span data-bind=\"visible: ReduceIndexingAttempts === ReduceIndexingSuccesses && ReduceIndexingAttempts > 0\">- all successful</span>                                                <span data-bind=\"visible: ReduceIndexingSuccesses > 0 && ReduceIndexingSuccesses < ReduceIndexingAttempts, text: \'attempts, \' + (ReduceIndexingAttempts - ReduceIndexingSuccesses) + \' failed\'\"></span>                                                <span data-bind=\"visible: ReduceIndexingAttempts > 0 && ReduceIndexingSuccesses === 0\">- all failed</span>                                            </p>                                        </div>                                    </div>                                    <div class=\"form-group\" data-bind=\"visible: IsMapReduce\">                                        <label class=\"col-md-2 control-label\">Last reduce</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LastReducedTimestampText\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Last query</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LastQueryTimestampText\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Priority</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: Priority, css: { \'text-muted\': Priority === \'Idle\', \'text-warning\': Priority === \'Disabled\', \'text-danger\': Priority === \'Abandoned\' }\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Last indexed Etag</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LastIndexedEtag\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\" data-bind=\"visible: IsMapReduce\">                                        <label class=\"col-md-2 control-label\">Last reduced Etag</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LastReducedEtag\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Is on RAM</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: IsOnRam\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Lock</label>                                        <div class=\"col-md-8\">                                            <p class=\"form-control-static\" data-bind=\"text: LockMode\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Entities</label>                                        <div class=\"col-md-8\" data-bind=\"with: ForEntityName\">                                            <p class=\"form-control-static\" data-bind=\"text: $data.join(\', \')\"></p>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/status.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        <span>Status</span>    </li>    <li class=\"active\">        <span data-bind=\"text: currentRouteTitle\"></span>    </li></ul><div>    <div class=\"row\">        <div class=\"col-md-2\">            <ul class=\"nav nav-list nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">                <li data-bind=\"css: { active: isActive }, visible: $parent.routeIsVisible($data)\">                    <a data-bind=\"attr: { href: hash }, text: title\"></a>                </li>            </ul>        </div>        <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">        </div>    </div></div>'; } )
define("text!views/database/status/userInfo.html", [], function () { return '<section data-bind=\"with: data\">    <form class=\"form-horizontal\" role=\"form\">        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Remark</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: Remark\"></p>            </div>        </div>        <div class=\"form-group\" data-bind=\"visible: User\">            <label class=\"col-md-2 control-label\">User</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: User\"></p>            </div>        </div>        <div class=\"form-group\" data-bind=\"visible: Principal\">            <label class=\"col-md-2 control-label\">Principal</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: Principal\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Admin (global)</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: IsAdminGlobal\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-2 control-label\">Admin (current database)</label>            <div class=\"col-md-10\">                <p class=\"form-control-static\" data-bind=\"text: IsAdminCurrentDb\"></p>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/visualizer.html", [], function () { return '<style>    .nodeRect {        fill-opacity: .4 !important;        stroke: #777;        stroke-width: 1.5px;    }    rect.hover {        stroke-width: 2px;        stroke: black;        fill-opacity: .95 !important;        stroke-opacity: .95 !important;    }    .nodeRect.highlight {        fill-opacity: 0.8 !important;    }    .nodeCheck {        stroke-width: 2px;        stroke: black;        fill: white;    }        .nodeCheck.selected {            stroke-width: 2px;            stroke: black;            fill: black;        }    .hidden {        display: none;    }    circle {        fill: #fff;        stroke: steelblue;        stroke-width: 1.5px;    }    g {        font: 10px sans-serif;        cursor: pointer;    }    .link {        fill: none;        stroke: #ccc;        stroke-width: 1.5px;    }        .link.selected {            fill: none;            stroke: black;            stroke-width: 2.5px;        }    .selectable-tooltip {        pointer-events: all;        -webkit-touch-callout: all;        -webkit-user-select: all;        -khtml-user-select: all;        -moz-user-select: all;        -ms-user-select: all;        user-select: all;    }    text {        pointer-events: none;        text-anchor: middle;    }</style><section id=\"visualizerSection\">    <div class=\"form-inline\" role=\"form\">        <h4 class=\"pull-left\" data-bind=\"visible: !onlineMode()\">Offline analysis for index <strong data-bind=\"text: indexName\"></strong>&nbsp;&nbsp;</h4>        <div class=\"btn-group\" data-bind=\"visible: onlineMode\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                <span data-bind=\"text: indexName\"></span>                <span class=\"caret\"></span>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <!-- ko foreach: indexes -->                <li data-bind=\"click: $root.setSelectedIndex.bind($root, $data.name)\"><a href=\"#\" data-bind=\"text: $data.name\"></a></li>                <!-- /ko -->                <!-- ko if: indexes().length === 0 -->                <li data-bind=\"click: function() { }\"><a href=\"#\">No indexes available</a></li>                <!-- /ko -->            </ul>        </div>        <div class=\"form-group\" data-bind=\"visible: hasIndexSelected() && onlineMode()\">            <div class=\"input-group\">                <input id=\"docKey\" type=\"text\" data-bind=\"value: docKey, valueUpdate: \'afterkeydown\'\" class=\"form-control\" placeholder=\"Document Id\" autocomplete=\"off\">                <span class=\"input-group-btn\" style=\"width: auto\">                    <button data-bind=\"click: addDocKey.bind($root, docKey())\" class=\"btn btn-default\" type=\"button\">                        <i class=\"fa fa-plus\"></i>                    </button>                </span>            </div>            <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#docKey\', foreach: docKeysSearchResults\">                <li role=\"presentation\" data-bind=\"click: $root.selectDocKey.bind($root, $data)\">                    <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                        <span data-bind=\"text: $data\"></span>                    </a>                </li>            </ul>        </div>        <div class=\"form-group\" data-bind=\"visible: hasIndexSelected() && onlineMode()\">            <div class=\"input-group\">                <input id=\"itemKey\" type=\"text\" data-bind=\"value: reduceKey, valueUpdate: \'afterkeydown\'\" class=\"form-control\" placeholder=\"Reduce key\" autocomplete=\"off\">                <span class=\"input-group-btn\" style=\"width: auto\">                    <button data-bind=\"click: addReduceKey.bind($root, reduceKey()), visible: hasIndexSelected\" class=\"btn btn-default\" type=\"button\"><i class=\"fa fa-plus\"></i></button>                </span>            </div>            <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#itemKey\', foreach: reduceKeysSearchResults\">                <li role=\"presentation\" data-bind=\"click: $root.selectReduceKey.bind($root, $data)\">                    <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                        <span data-bind=\"text: $data\"></span>                    </a>                </li>            </ul>        </div>        <div class=\"btn-group\">            <a class=\"btn btn-default\" data-bind=\"attr: { href: editIndexUrl }, visible: hasIndexSelected() && onlineMode()\" title=\"Edit index\"><i class=\"fa fa-pencil\"></i></a>            <a class=\"btn btn-default\" data-bind=\"attr: { href: runQueryUrl }, visible: hasIndexSelected() && onlineMode()\" title=\"Run query\">                <i class=\"fa fa-play\"></i>            </a>            <button id=\"fullScreenButton\" class=\"btn btn-default\" data-bind=\"visible: hasIndexSelected, click: toggleFullscreen\" title=\"Fullscreen mode\">                <i class=\"fa fa-expand\"></i>            </button>            <button id=\"keysDialogBtn\" class=\"btn btn-default\" title=\"Displays information about inserted document ids and reduce keys\" data-bind=\"visible: hasIndexSelected, click: displayKeyInfo, enable: docKeysOrReduceKeysSelected\">                <i class=\"fa fa-key\"></i>            </button>            <button class=\"btn btn-default\" title=\"Clear\" data-bind=\"visible: hasIndexSelected, click: clearChart\">                <i class=\"fa fa-refresh\"></i>            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: hasIndexSelected\">            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                <i class=\"fa fa-download\"></i>            </button>            <ul class=\"dropdown-menu\" role=\"menu\">                <li data-bind=\"click: saveAsPng, visible: hasSaveAsPngSupport\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> PNG</a></li>                <li data-bind=\"click: saveAsSvg\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> SVG</a></li>                <li data-bind=\"click: saveAsJson\"><a href=\"#\"><i class=\"fa fa-file-text-o\"></i> JSON</a></li>            </ul>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: chooseImportFile\" title=\"Import visualization\">                <i class=\"fa fa-upload\"></i>            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: hasIndexSelected\">            <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicator\" class=\"glyphicon\" style=\"width: 30px; height: 30px;\"><span class=\"fa fa-spinner fa-spin fa-2x\"></span></div>        </div>    </div>    <svg data-bind=\"visible: hasIndexSelected\" id=\"visHeader\" style=\"height: 25px\" height=\"25\"></svg>    <div id=\"visualizerContainer\" style=\"height: 600px; width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" class=\"with-3d-shadow with-transitions\">        <div>            <svg id=\"visualizer\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>        </div>    </div></section>'; } )
define("text!views/database/status/visualizerImport.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" >    <form data-bind=\"submit: doImport\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Import visualization</h4>        </div>        <div class=\"modal-body\">                        Select file to import. <br />            <input id=\"importFilePicker\" type=\"file\" data-bind=\"event: { change: fileSelected }\" />                    </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: hasFileSelected\"><i class=\"fa fa-upload\"></i> Import</button>        </div>    </form></div>'; } )
define("text!views/database/status/visualizerKeys.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 500px;\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Document Ids and Reduce Keys</h4>    </div>    <div class=\"modal-body\" data-bind=\"with: visualizer\">        <div class=\"row\">            <div class=\"col-lg-6\">                <h4>Document Ids (<span data-bind=\"text: docKeys().length\"></span>)</h4>                <ul data-bind=\"foreach: docKeys\">                    <li data-bind=\"text: $data\"></li>                </ul>            </div>            <div class=\"col-lg-6\">                <h4>Reduce Keys (<span data-bind=\"text: reduceKeys().length\"></span>)</h4>                <ul data-bind=\"foreach: reduceKeys\">                    <li data-bind=\"text: $data\"></li>                </ul>            </div>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/debug/statusDebug.html", [], function () { return '<div id=\"status-debug\">    <ul class=\"nav nav-tabs\" data-bind=\"foreach: router.navigationModel\">        <li data-bind=\"css: { active: isActive }\">             <a   data-bind=\"attr: { href: hash, title: tooltip }, text: title\"></a>        </li>    </ul>    <div class=\"tab-content\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/database/status/debug/statusDebugChanges.html", [], function () { return '<div class=\"panel-group\" data-bind=\"foreach: data\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title\">                <a data-toggle=\"collapse\"  data-bind=\"text: \'Connection identifier: \' + Id, attr: { href: \'#statsDebugChangesCollapse\' + $index() }\"></a>            </h4>        </div>        <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'statsDebugChangesCollapse\' + $index() }\">            <div class=\"panel-body\">                <div class=\"form-horizontal\" role=\"form\">                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Connected</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: Connected\"></p>                        </div>                    </div>                    <div data-bind=\"with: DocumentStore\">                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch all documents</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchAllDocuments\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch all indexes</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchAllIndexes\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch all transformers</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchAllTransformers\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch all replication conflicts</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchAllReplicationConflicts\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched indexes</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedIndexes.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedIndexes\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched documents</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedDocuments.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedDocuments\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched document prefixes</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedDocumentPrefixes.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedDocumentPrefixes\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched documents in collection</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedDocumentsInCollection.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedDocumentsInCollection\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched documents of type</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedDocumentsOfType.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedDocumentsOfType\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched bulk inserts</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\">                                    <span data-bind=\"if: WatchedBulkInserts.length == 0\">-</span>                                    <ul data-bind=\"foreach: WatchedBulkInserts\">                                        <li data-bind=\"text: $data\"></li>                                    </ul>                                </p>                            </div>                        </div>                    </div>                    <!--<div data-bind=\"with: FileSystem\">                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch conflicts</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchConflicts\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch sync</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchSync\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch cancellations</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchedCancellations\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watch config</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WatchConfig\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Watched folders</label>                            <div class=\"col-md-10\">                                <p data-bind=\"if: WatchedFolders.length == 0\">-</p>                                <ul data-bind=\"foreach: WatchedFolders\">                                    <li data-bind=\"text: $data\"></li>                                </ul>                            </div>                        </div>                    </div>-->                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/status/debug/statusDebugConfig.html", [], function () { return '<section>    <div data-bind=\"visible: isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this section!</h4>    </div>    <form data-bind=\"visible: isForbidden() == false\" class=\"form-horizontal\" role=\"form\">        <div class=\"form-group\">            <div class=\"col-md-12\">                <div>                    <div class=\"row\">                        <div class=\"col-md-12\">                            <pre id=\"statusDebugConfigEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: data, readOnly: true }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 35 }\"></pre>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/debug/statusDebugCurrentlyIndexing.html", [], function () { return '<section>    <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 2000, onRefresh: $root.fetchCurrentlyIndexing.bind($root), active:true }\"></button>    <form class=\"form-horizontal\" role=\"form\" data-bind=\"with: data\">        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Number of currently working indexes</label>            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: NumberOfCurrentlyWorkingIndexes\"></p>            </div>        </div>        <!-- ko foreach:Indexes -->        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Index name</label>            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: IndexName\"></p>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Is map/reduce?</label>            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: IsMapReduce\"></p>            </div>        </div>        <!-- ko foreach:CurrentOperations-->        <div class=\"form-group\">            <!-- ko if: $index() == 0 -->            <label class=\"col-md-2 control-label\">Current operations</label>            <label class=\"col-md-2 control-label\" data-bind=\"text: Operation\"></label>            <!-- /ko -->            <!-- ko if: $index() != 0 -->            <label class=\"col-md-2 col-md-offset-2 control-label\" data-bind=\"text: Operation\"></label>            <!-- /ko -->            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: NumberOfProcessingItems\"></p>            </div>        </div>        <!-- /ko -->        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Priority</label>            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: Priority\"></p>            </div>        </div>        <!-- ko foreach:OverallIndexingRate-->        <div class=\"form-group\">                        <!-- ko if: $index() == 0 -->            <label class=\"col-md-2 control-label\">Overall indexing rate</label>            <label class=\"col-md-2 control-label\" data-bind=\"text: Operation\"></label>            <!-- /ko -->            <!-- ko if: $index() != 0 -->            <label class=\"col-md-2 col-md-offset-2 control-label\" data-bind=\"text: Operation\"></label>            <!-- /ko -->            <div class=\"col-md-8\">                <p class=\"form-control-static\" data-bind=\"text: Rate\"></p>            </div>        </div>        <!-- /ko -->        <hr />        <!-- /ko -->    </form></section>'; } )
define("text!views/database/status/debug/statusDebugDataSubscriptions.html", [], function () { return '<div class=\"panel panel-default\" data-bind=\"visible: data.length === 0\">    <div class=\"panel-heading\">        Data subscriptions    </div>    <div class=\"panel-body\">        <div class=\"text-center\">            <h3 class=\"text-info\">No data subscriptions for now. </h3>        </div>    </div></div><div class=\"panel-group\" data-bind=\"foreach: data\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title\">                <a data-toggle=\"collapse\" data-bind=\"text: \'Id: \' + SubscriptionId, attr: { href: \'#statsDebugDataSubscriptionsCollapse\' + $index() }\"></a>            </h4>        </div>        <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'statsDebugDataSubscriptionsCollapse\' + $index() }\">            <div class=\"panel-body\">                <div class=\"form-horizontal\" role=\"form\">                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Acknowledged ETag</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: AckEtag\"></p>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Last batch sent time</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: TimeOfSendingLastBatch\"></p>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Last client activity time</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: TimeOfLastClientActivity\"></p>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Criteria</label>                    </div>                    <div data-bind=\"with: Criteria\">                        <div class=\"form-group\" data-bind=\"visible: KeyStartsWith !== null\">                            <label class=\"col-md-3 col-md-offset-2\">Document key has to starts with</label>                            <div class=\"col-md-7\">                                <p data-bind=\"text: KeyStartsWith\"></p>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: BelongsToAnyCollection !== null\">                            <label class=\"col-md-3 col-md-offset-2\">Document has to belong to any of collections</label>                            <div class=\"col-md-7\">                                <p data-bind=\"text: BelongsToAnyCollection\"></p>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: PropertiesMatch !== null\">                            <label class=\"col-md-3 col-md-offset-2\">Properties must match values</label>                            <div class=\"col-md-7\" data-bind=\"foreach: PropertiesMatch\">                                <p>                                    <span data-bind=\"text: Key\"></span>&nbsp;:&nbsp;<span data-bind=\"text: Value\"></span>                                </p>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: PropertiesNotMatch !== null\">                            <label class=\"col-md-3 col-md-offset-2\">Properties cannot match values</label>                            <div class=\"col-md-7\" data-bind=\"foreach: PropertiesNotMatch\">                                <p>                                    <span data-bind=\"text: Key\"></span> : <span data-bind=\"text: Value\"></span>                                </p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-3 col-md-offset-2\">Start ETag</label>                            <div class=\"col-md-7\">                                <p data-bind=\"text: StartEtag\"></p>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Open</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: IsOpen\"></p>                        </div>                    </div>                    <div data-bind=\"visible: IsOpen\">                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Connection options</label>                        </div>                        <div data-bind=\"with: ConnectionOptions\">                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Id</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: ConnectionId\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Client alive notification interval</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: ClientAliveNotificationInterval\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Time to wait before connection retry</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: TimeToWaitBeforeConnectionRetry\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Opening strategy</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: Strategy\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Pulling request timeout</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: PullingRequestTimeout\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Ignore subscribers errors</label>                                <div class=\"col-md-7\">                                    <p data-bind=\"text: IgnoreSubscribersErrors\"></p>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-md-3 col-md-offset-2\">Batch options</label>                            </div>                            <div data-bind=\"with: BatchOptions\">                                <div class=\"form-group\">                                    <label class=\"col-md-2 col-md-offset-3\">AcknowledgmentTimeout</label>                                    <div class=\"col-md-2\">                                        <p data-bind=\"text: AcknowledgmentTimeout\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-md-2 col-md-offset-3\">Max document count</label>                                    <div class=\"col-md-2\">                                        <p data-bind=\"text: MaxDocCount\"></p>                                    </div>                                </div>                                <div class=\"form-group\">                                    <label class=\"col-md-2 col-md-offset-3\">Max batch size</label>                                    <div class=\"col-md-2\">                                        <p data-bind=\"text: MaxSize\"></p>                                    </div>                                </div>                            </div>                                                    </div>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/status/debug/statusDebugDocrefs.html", [], function () { return '<section>    <form class=\"form-horizontal\" role=\"form\">        <div class=\"form-group row\">            <label class=\"col-md-2 control-label\">Show documents referencing</label>            <div class=\"col-md-4\">                <div class=\"input-group\">                    <input type=\"text\" data-bind=\"value: docId, valueUpdate: \'afterkeydown\'\" class=\"form-control\">                    <span class=\"input-group-btn\">                        <button data-bind=\"click: fetchDocRefs, enable: canSearch\" class=\"btn btn-default\"><i class=\"fa fa-search\"></i></button>                    </span>                </div>            </div>        </div>    </form>        <div class=\"status-debug-docrefs-page\" data-bind=\"if: currentDocRefs\">        <div class=\"row\">            <div class=\"col-md-12\">                <div id=\"docRefsGrid\" data-bind=\"widget: {                    kind: \'virtualTable\',                     itemsSource: currentDocRefs,                    gridSelector: \'#docRefsGrid\',                    showCheckboxes: false,                     useContextMenu: false,                    customColumns: columns                }\"></div>            </div>        </div>    </div>    </section>'; } )
define("text!views/database/status/debug/statusDebugExplainReplication.html", [], function () { return '<section>    <div class=\"page-header\" data-bind=\"visible: destinations().length == 0\">        <h3><i class=\"fa fa-info-circle text-info\"></i> No replication destinations were detected. <small>Maybe this database isn\'t <a target=\"_blank\" href=\"http://ravendb.net/docs/server/scaling-out/replication\">setup for replication</a>.</small></h3>    </div>    <form class=\"form-horizontal\" role=\"form\" data-bind=\"visible: destinations().length > 0\">        <div>            <div class=\"form-group\">                <label class=\"col-sm-2 control-label\">Replication destination:</label>                <div class=\"col-sm-10\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"text: selectedDestionationText\"></span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\">                            <!-- ko foreach: destinations -->                            <li data-bind=\"click: $root.setSelectedDestination.bind($root, $data)\"><a href=\"#\" data-bind=\"text: $data.Database + \' on \' + $data.Url\"></a></li>                            <!-- /ko -->                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-2 control-label\">Document ID:</label>                <div class=\"col-sm-10\">                    <div class=\"btn-group\">                        <input id=\"explainDocId\" type=\"text\" class=\"form-control\" placeholder=\"Document ID\" data-bind=\"value: documentId, valueUpdate: \'afterkeydown\'\" autocomplete=\"off\" />                        <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#explainDocId\', foreach: documentIdSearchResults\">                            <li role=\"presentation\" data-bind=\"click: $root.selectDocument.bind($root, $data)\">                                <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                    <span data-bind=\"text: $data[\'@metadata\'][\'@id\']\"></span>                                </a>                            </li>                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <div class=\"col-sm-10 col-sm-offset-2\">                    <button class=\"btn btn-default\" data-bind=\"click: explain, enable: buttonEnabled\">Explain</button>                </div>            </div>        </div>        <div data-bind=\"with: explanation\">            <hr />            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Message:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\"><strong data-bind=\"text: Message\"></strong></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Key:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Key\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Etag:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Etag\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Destination url:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Destination.Url\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Destination database name:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Destination.DatabaseName\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Destination server instance ID:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Destination.ServerInstanceId\"></p>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Destination last document Etag:</label>                <div class=\"col-sm-8\">                    <p class=\"form-control-static\" data-bind=\"text: Destination.LastDocumentEtag\"></p>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/debug/statusDebugIdentities.html", [], function () { return '<section>    <div class=\"status-debug-docrefs-page\" data-bind=\"if: currentIdentities\">        <div class=\"row\">            <div class=\"col-md-12\">                <div id=\"identitiesGrid\" data-bind=\"widget: {    kind: \'virtualTable\', itemsSource: currentIdentities,    gridSelector: \'#identitiesGrid\',    showCheckboxes: false, useContextMenu: false,    customColumns: columns, showIds: false}\">                </div>            </div>        </div>    </div>    </section>'; } )
define("text!views/database/status/debug/statusDebugIndexFields.html", [], function () { return '<section>    <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: fetchIndexFields\">        <div class=\"col-md-12\">            <div>                <div class=\"row\">                    <div class=\"col-md-6\">                        <div class=\"form-group\">                            <label id=\"indexDefLabelContainer\" class=\"control-label\" data-placement=\"auto right\" data-original-title=\"Index definition\" data-trigger=\"hover\">                                <span id=\"indexDefLabel\" class=\"help-cursor\">                                    Index definition                                    <i class=\"fa fa-info-circle text-info\"></i>                                </span>                            </label>                            <pre id=\"statusDebugIndexFieldsEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: indexStr }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 35 }\"></pre>                        </div>                    </div>                    <div class=\"col-md-6\">                        <button type=\"submit\" class=\"btn btn-default\"><i class=\"fa fa-table\"></i> Show index fields</button>                        <hr />                        <!-- ko with: result -->                        <ol data-bind=\"foreach: FieldNames\">                            <li data-bind=\"text: $data\"></li>                        </ol>                        <!-- /ko -->                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/debug/statusDebugMetrics.html", [], function () { return '<section data-bind=\"with: data\">    <div class=\"panel-group\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#metricsGeneral\">General</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\"  id=\"metricsGeneral\" >                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Docs writes/second</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: DocsWritesPerSecond\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Indexed/second</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: IndexedPerSecond\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Reduced/second</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: ReducedPerSecond\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Requests/second</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsPerSecond\"></p>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#metricsRequest\">Requests</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" id=\"metricsRequest\" >                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Count</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: Requests.Count\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Counter</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsDuration.Counter\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Mean rate</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: Requests.MeanRate\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Min</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsDuration.Min\"></p>                            </div>                            <label class=\"col-md-2 control-label\">One minute rate</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: Requests.OneMinuteRate\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Max</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsDuration.Max\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Five minute rate</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: Requests.FiveMinuteRate\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Mean</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsDuration.Mean\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Fifteen minute rate</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: Requests.FifteenMinuteRate\"></p>                            </div>                            <label class=\"col-md-2 control-label\">Std dev.</label>                            <div class=\"col-md-4\">                                <p class=\"form-control-static\" data-bind=\"text: RequestsDuration.Stdev\"></p>                            </div>                            <label class=\"col-md-2 col-md-offset-6 control-label\">Percentiles</label>                            <div class=\"col-md-4\"></div>                            <div data-bind=\"foreach: $parent.requestPercentiles\">                                <label class=\"col-md-2 col-md-offset-6 control-label\" data-bind=\"text: key\"></label>                                <div class=\"col-md-4\">                                    <p class=\"form-control-static\" data-bind=\"text: value\"></p>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#metricsStaleIndexes\">Stale Indexes</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" id=\"metricsStaleIndexes\" >                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group\">                            <div class=\"col-md-6\">                                <h4>Maps</h4>                                <label class=\"col-md-4 control-label\">Counter</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexMaps.Counter\"></p>                                </div>                                 <label class=\"col-md-4 control-label\">Min</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexMaps.Min\"></p>                                </div>                                 <label class=\"col-md-4 control-label\">Max</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexMaps.Max\"></p>                                </div>                                  <label class=\"col-md-4 control-label\">Mean</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexMaps.Mean\"></p>                                </div>                                 <label class=\"col-md-4 control-label\">Std dev.</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexMaps.Stdev\"></p>                                </div>                                <label class=\"col-md-4 control-label\">Percentiles</label>                                <div class=\"clearfix\"></div>                                <div class=\"left\" data-bind=\"foreach: $parent.staleIndexMapsPercentiles\">                                    <label class=\"col-md-4 control-label\" data-bind=\"text: key\"></label>                                    <div class=\"col-md-8\">                                        <p class=\"form-control-static\" data-bind=\"text: value\"></p>                                    </div>                                </div>                            </div>                            <div class=\"col-md-6\">                                <h4>Reduces</h4>                                <label class=\"col-md-4 control-label\">Counter</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexReduces.Counter\"></p>                                </div>                                 <label class=\"col-md-4 control-label\">Min</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexReduces.Min\"></p>                                </div>                                 <label class=\"col-md-4 control-label\">Max</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexReduces.Max\"></p>                                </div>                                <label class=\"col-md-4 control-label\">Mean</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexReduces.Mean\"></p>                                </div>                                <label class=\"col-md-4 control-label\">Std dev.</label>                                <div class=\"col-md-8\">                                    <p class=\"form-control-static\" data-bind=\"text: StaleIndexReduces.Stdev\"></p>                                </div>                                <label class=\"col-md-4 control-label\">Percentiles</label>                                <div class=\"clearfix\"></div>                                <div class=\"left\" data-bind=\"foreach: $parent.staleIndexReducesPercentiles\">                                    <label class=\"col-md-4 control-label\" data-bind=\"text: key\"></label>                                    <div class=\"col-md-8\">                                        <p class=\"form-control-static\" data-bind=\"text: value\"></p>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#metricsGauges\">Gauges</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\"  id=\"metricsGauges\" >                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\" data-bind=\"foreach: $root.gauges\">                        <!-- ko foreach: values -->                        <div class=\"form-group\">                            <label class=\"col-md-6 control-label\" data-bind=\"text: $parent.key + \' \' + key \"></label>                            <div class=\"col-md-6\">                                <p class=\"form-control-static\" data-bind=\"text: value\"></p>                            </div>                        </div>                        <!-- /ko -->                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#metricsReplication\">Replication</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" id=\"metricsReplication\"  >                <div class=\"panel-body\">                    <ul class=\"nav nav-tabs\" data-bind=\"foreach: $root.replicationDestinations\">                        <li data-bind=\"css: { active: $index() == 0 }\"><a data-bind=\"attr: { href: \'#replication-tab-\' + $index() }, text: $data\" data-toggle=\"tab\"></a></li>                    </ul>                    <div class=\"tab-content\" data-bind=\"foreach: $root.replicationDestinations\">                        <div data-bind=\"attr: { id: \'replication-tab-\' + $index() }, css: { active: $index() == 0 }\" class=\"tab-pane\">                            <div class=\"panel-group\">                                <div class=\"panel panel-default\">                                    <div class=\"panel-heading\">                                        <h4 class=\"panel-title\">                                            <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#metrics-rep-\' + $index() + \'-bs\' }\">Batch Size</a>                                        </h4>                                    </div>                                    <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'metrics-rep-\' + $index() + \'-bs\' }\">                                        <div class=\"panel-body\">                                            <div class=\"form-horizontal\" role=\"form\" >                                                <div class=\"col-md-6\" data-bind=\"with: $root.data().ReplicationBatchSizeMeter[$data]\">                                                     <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Count</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Count\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Mean rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: MeanRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">One minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: OneMinuteRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Five minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: FiveMinuteRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Fifteen minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: FifteenMinuteRate\"></p>                                                        </div>                                                    </div>                                                </div>                                                <div class=\"col-md-6\" data-bind=\"with: $root.data().ReplicationBatchSizeHistogram[$data]\">                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Counter</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Counter\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Min</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Min\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Max</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Max\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Mean</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Mean\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Std dev.</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Stdev\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Percentiles</label>                                                        <div class=\"clearfix\"></div>                                                        <div class=\"left\" data-bind=\"foreach: $root.extractPercentiles(Percentiles)\">                                                            <label class=\"col-md-4 control-label\" data-bind=\"text: key\"></label>                                                            <div class=\"col-md-8\">                                                                <p class=\"form-control-static\" data-bind=\"text: value\"></p>                                                            </div>                                                        </div>                                                    </div>                                                </div>                                            </div>                                        </div>                                    </div>                                </div>                                <div class=\"panel panel-default\">                                    <div class=\"panel-heading\">                                        <h4 class=\"panel-title\">                                            <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#metrics-rep-\' + $index() + \'-duration\' }\">Duration</a>                                        </h4>                                    </div>                                    <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'metrics-rep-\' + $index() + \'-duration\' }\">                                        <div class=\"panel-body\">                                            <div class=\"form-horizontal\" role=\"form\" >                                                <div class=\"col-md-6\" data-bind=\"with: $root.data().ReplicationDurationMeter[$data]\">                                                     <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Count</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Count\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Mean rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: MeanRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">One minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: OneMinuteRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Five minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: FiveMinuteRate\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Fifteen minute rate</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: FifteenMinuteRate\"></p>                                                        </div>                                                    </div>                                                </div>                                                <div class=\"col-md-6\" data-bind=\"with: $root.data().ReplicationDurationHistogram[$data]\">                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Counter</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Counter\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Min</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Min\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Max</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Max\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Mean</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Mean\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Std dev.</label>                                                        <div class=\"col-md-8\">                                                            <p class=\"form-control-static\" data-bind=\"text: Stdev\"></p>                                                        </div>                                                    </div>                                                    <div class=\"form-group\">                                                        <label class=\"col-md-4 control-label\">Percentiles</label>                                                        <div class=\"clearfix\"></div>                                                        <div class=\"left\" data-bind=\"foreach: $root.extractPercentiles(Percentiles)\">                                                            <label class=\"col-md-4 control-label\" data-bind=\"text: key\"></label>                                                            <div class=\"col-md-8\">                                                                <p class=\"form-control-static\" data-bind=\"text: value\"></p>                                                            </div>                                                        </div>                                                    </div>                                                </div>                                            </div>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/debug/statusDebugPersistAutoIndex.html", [], function () { return '<section>    <form class=\"form-horizontal\" role=\"form\">        <div class=\"form-group\">            <div class=\"form-group\">                <label class=\"col-sm-2 control-label\">Index to persist:</label>                <div class=\"col-sm-10\">                     <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"text: indexName\"></span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\">                            <!-- ko foreach: onRamIndexes -->                            <li data-bind=\"click: $root.setSelectedIndex.bind($root, $data.name)\"><a href=\"#\" data-bind=\"text: $data.name\"></a></li>                            <!-- /ko -->                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <div class=\"col-sm-10 col-sm-offset-2\">                    <button class=\"btn btn-default\" data-bind=\"click: persistIndex, enable: buttonEnabled\"><i class=\"fa fa-hdd-o\"></i> Persist index</button>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/debug/statusDebugQueries.html", [], function () { return '<div>    <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 2000, onRefresh: $root.fetchCurrentQueries.bind($root), active:true }\"></button>    <hr />    <div class=\"panel-group\" data-bind=\"foreach: data\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\"  data-bind=\"text: indexName, attr: { href: \'#statsDebugQueriesCollapse\' + $index() }\"></a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'statsDebugQueriesCollapse\' + $index() }\">                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\" data-bind=\"foreach: queries\">                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Start time</label>                            <div class=\"col-md-6\">                                <p class=\"form-control-static\" data-bind=\"text: startTime\"></p>                            </div>                            <div class=\"col-md-4 text-right\">                                <button class=\"btn btn-danger\" data-bind=\"click: $root.killQuery.bind($root, queryId)\"><i class=\"fa fa-bolt\"></i> Kill this query</button>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Duration</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: duration\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Query Info</label>                            <div class=\"col-md-10\">                                <pre class=\"form-control\" data-bind=\"aceEditor: { code: queryInfo, readOnly: true }\"  style=\"height: 350px;\" ></pre>                            </div>                        </div>                        <hr />                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/status/debug/statusDebugRoutes.html", [], function () { return '<section>    <div data-bind=\"visible: isGlobalAdmin() == false\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this section!</h4>    </div>    <form data-bind=\"visible: isGlobalAdmin\" class=\"form-horizontal\" role=\"form\">        <div class=\"form-group\">            <div class=\"col-md-12\">                <div>                    <div class=\"row\">                        <div class=\"col-md-12\">                            <pre id=\"statusDebugRoutesEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: data, readOnly: true }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 35 }\"></pre>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/database/status/debug/statusDebugSqlReplication.html", [], function () { return '<section>    <div data-bind=\"visible: !data() || data().length == 0\" class=\"text-center\">        <h4 class=\"page-header text-info\">No SQL Replication debug info to display!</h4>    </div>    <div id=\"sqlReplicationStats\" class=\"panel-group\" data-bind=\"foreach: data\">        <div class=\"sqlStatsPanel panel panel-default\">            <div class=\"panel-heading\">                <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#statsDebugSqlRepCollapse\' + $index() }\">                    <h4 class=\"panel-title\" data-bind=\"text:name\"></h4>                </a>            </div>            <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'statsDebugSqlRepCollapse\' + $index() }\">                <div class=\"panel-body\">                    <div class=\"form-horizontal statisticsSection\" role=\"form\" data-bind=\"with:statistics\">                        <div class=\"form-group sectionTitle\">                            <strong class=\"control-label\">Statistics:</strong>                        </div>                        <div class=\"form-group col-sm-12\">                            <label class=\"col-md-2 control-label\">Last error time</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: LastErrorTime\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Script error count</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: ScriptErrorCount\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Script success count</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: ScriptSuccessCount\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Write error count</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: WriteErrorCount\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-2 control-label\">Success count</label>                            <div class=\"col-md-10\">                                <p class=\"form-control-static\" data-bind=\"text: SuccessCount\"></p>                            </div>                        </div>                        <!-- ko with: LastAlert -->                        <div class=\"panel panel-warning\" data-bind=\"attr:{id:\'statsAccordion\' + $index()}\">                            <div class=\"panel-heading\">                                <a data-toggle=\"collapse\" data-bind=\"attr:{\'data-parent\':\'#statsAccordion\' + $index(), \'href\':\'#collapselasterror\' + $index()}\">                                    <h3 class=\"panel-title\">                                        Last Alert                                    </h3>                                </a>                            </div>                            <div class=\"panel-collapse collapse\" data-bind=\"attr:{id:\'collapselasterror\' + $index()}\">                                <div class=\"panel-body\">                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Title</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: Title\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Created at</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: CreatedAt\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Observed</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: Observed\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Message</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: Message\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Alert level</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: AlertLevel\"></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Exception</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\"><span class=\"exceptionText\" data-bind=\"text: Exception\"></span></p>                                        </div>                                    </div>                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Unique key</label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: UniqueKey\"></p>                                        </div>                                    </div>                                </div>                            </div>                        </div>                        <!-- /ko -->                    </div>                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group sectionTitle\">                            <strong class=\"control-label\">Metrics:</strong>                        </div>                        <div class=\"form-group subSectionTitle\">                            <strong class=\"control-label\">Rate Metrics:</strong>                        </div>                        <div from-group>                            <div class=\"meterTable col-sm-12\">                                <span class=\"headers\">                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[0]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"0\"></span>                                        <span>Table</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[1]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"1\"></span>                                        <span>Metric Name</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[2]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"2\"></span>                                        <span>Count</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[3]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"3\"></span>                                        <span>MeanRate</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[4]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"4\"></span>                                        <span>OneMinuteRate</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[5]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"5\"></span>                                        <span>FiveMinuteRate</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[6]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"6\"></span>                                        <span>FifteenMinuteRate</span>                                    </div>                                </span>                                <div style=\"white-space: nowrap\" data-bind=\"foreach: rateMetrics\">                                    <div style=\"display: block; white-space: nowrap\" class=\"logRecord log-table-row\">                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[0]() + \'em\' }, text: Table\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[1]() + \'em\' }, text: Name\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[2]() + \'em\' }, text: Count\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[3]() + \'em\' }, text: MeanRate\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[4]() + \'em\' }, text: OneMinuteRate\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[5]() + \'em\' }, text: FiveMinuteRate\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[6]() + \'em\' }, text: FifteenMinuteRate\"></div>                                    </div>                                </div>                            </div>                            <div class=\"form-group subSectionTitle\">                                <strong class=\"control-label\">Histograms:</strong>                            </div>                            <div class=\"meterTable col-sm-12\">                                <span class=\"headers\">                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[7]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"7\"></span>                                        <span>Table</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[8]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"8\"></span>                                        <span>Metric Name</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[9]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"9\"></span>                                        <span>Counter</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[10]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"10\"></span>                                        <span>Max</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[11]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"11\"></span>                                        <span>Min</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[12]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"12\"></span>                                        <span>Mean</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[13]() + \'em\' }\">                                        <span class=\"column-handle\" column=\"13\"></span>                                        <span>Stdev</span>                                    </div>                                    <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[14]() + \'em\' }\">                                        <span>Percentiles</span>                                        <div style=\"width:100%\">                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[15]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"15\"></span>                                                <span>50%</span>                                            </div>                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[16]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"16\"></span>                                                <span>75%</span>                                            </div>                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[17]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"17\"></span>                                                <span>95%</span>                                            </div>                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[18]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"18\"></span>                                                <span>99%</span>                                            </div>                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[19]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"19\"></span>                                                <span>99.9%</span>                                            </div>                                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[20]() + \'em\' }\">                                                <span class=\"column-handle\" column=\"20\"></span>                                                <span>99.99%</span>                                            </div>                                        </div>                                    </div>                                </span>                                <div style=\"white-space: nowrap\" data-bind=\"foreach: histogramMetrics\">                                    <div style=\"display: block; white-space: nowrap\" class=\"logRecord log-table-row\">                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[7]() + \'em\' }, text: Table\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[8]() + \'em\' }, text: Name\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[9]() + \'em\' }, text: Counter\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[10]() + \'em\' }, text: Max\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[11]() + \'em\' }, text: Min\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[12]() + \'em\' }, text: Min\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[13]() + \'em\' }, text: Stdev\"></div>                                        <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[14]() + \'em\' }, if:!!Percentiles[0]\">                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[15]() + \'em\' }, text: Percentiles[0].percentileValue\"></div>                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[16]() + \'em\' }, text: Percentiles[1].percentileValue\"></div>                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[17]() + \'em\' }, text: Percentiles[2].percentileValue\"></div>                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[18]() + \'em\' }, text: Percentiles[3].percentileValue\"></div>                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[19]() + \'em\' }, text: Percentiles[4].percentileValue\"></div>                                            <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[20]() + \'em\' }, text: Percentiles[5].percentileValue\"></div>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/debug/statusDebugTasks.html", [], function () { return '<section>    <form class=\"form-horizontal\" role=\"form\">        <div data-bind=\"visible: !data() || data().length == 0\" class=\"text-center\">            <h4 class=\"page-header text-info\">No Tasks debug info to display!</h4>        </div>        <div data-bind=\"if: data() && data().length > 0\">            <a class=\"btn btn-default\" target=\"_blank\" href=\"#\" data-bind=\"attr: { href: detailsUrl }\"><i class=\"fa fa-download\"></i> Download raw data</a>            <table class=\"table table-striped\">                <thead>                    <tr>                        <th>Type</th>                        <th>Index Id</th>                        <th>Index Name</th>                        <th>Task Count</th>                        <th>Min Date</th>                        <th>Max Date</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: data\">                    <tr>                        <td data-bind=\"text: Type\"></td>                        <td data-bind=\"text: IndexId\"></td>                        <td data-bind=\"text: IndexName\"></td>                        <td data-bind=\"text: Count\"></td>                        <td data-bind=\"text: MinDate\"></td>                        <td data-bind=\"text: MaxDate\"></td>                    </tr>                </tbody>            </table>            </div>            </form></section>'; } )
define("text!views/database/status/debug/statusDebugWebSocket.html", [], function () { return '<section>    <button data-bind=\"click: test\" class=\"btn btn-primary\">Test web sockets</button>        <h4>Results:</h4>    <pre data-bind=\"text: results\">    </pre>    </section>'; } )
define("text!views/database/status/indexing/indexBatchSize.html", [], function () { return '<section class=\"metricsBatchSizeContainer\">    <h3>        Indexing Batch Size        <small><a href=\"#\" target=\"_blank\" title=\"URL for raw index batch size metrics. Opens in a new tab.\" data-bind=\"text: indexBatchSizeQueryUrl, attr: { href: indexBatchSizeQueryUrl }\"></a></small>    </h3>        <div id=\"actualIndexingBatchSizeContainer\" class=\"with-3d-shadow with-transitions\">        <svg data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\"></svg>    </div></section>'; } )
define("text!views/database/status/indexing/indexing.html", [], function () { return '<div id=\"metrics\">    <ul class=\"nav nav-tabs\" data-bind=\"foreach: router.navigationModel\">        <li data-bind=\"css: { active: isActive }\">             <a   data-bind=\"attr: { href: hash, title: tooltip }, text: title\"></a>        </li>    </ul>    <div class=\"tab-content\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/database/status/indexing/indexPerformance.html", [], function () { return '<style>    .batchRange rect {        opacity: 0.4;    }    .batchRange rect:hover {        opacity: 0.6;    }    .rect_legend {        width: 15px;        height: 15px;        border: solid 1px black;        display: inline-block;    }    .op {        cursor: pointer;    }    #metricsIndexingGraph text {         pointer-events: none;        -webkit-touch-callout: none;        -webkit-user-select: none;        -khtml-user-select: none;        -moz-user-select: none;        -ms-user-select: none;        user-select: none;    }    .op rect {        opacity: 1;    }    .op:hover rect {        opacity: 0.8;    }    .filter_op {        fill: #aaa;        opacity: 0.4;        cursor: pointer;    }    .filter_op:hover {        opacity: 1;    }    .batchRange {        cursor: pointer;    }    .selectable-tooltip .close, .selectable-tooltip .handle {        pointer-events: all;        -webkit-touch-callout: all;        -webkit-user-select: all;        -khtml-user-select: all;        -moz-user-select: all;        -ms-user-select: all;        user-select: all;    }    line.reduce_split_line {        stroke:black;        stroke-width: 1;        opacity: 0.5;    }    .prefetch_op rect {        opacity: 0.6;        fill: #2ca02c;        cursor: pointer;    }    .prefetch_op rect:hover {        opacity: 0.7;    }    .LoadDocument { fill: #17becf; background-color: #17becf }    .Linq_MapExecution { fill: #1f77b4; background-color: #1f77b4 }    .Linq_ReduceLinqExecution { fill: #ff7f0e; background-color: #ff7f0e }    .Lucene_DeleteExistingDocument { fill: #2ca02c; background-color: #2ca02c }    .Lucene_ConvertToLuceneDocument { fill: #d62728; background-color: #d62728 }    .Lucene_AddDocument { fill: #9467bd; background-color: #9467bd }    .Lucene_FlushToDisk { fill: #8c564b; background-color: #8c564b }    .Lucene_RecreateSearcher { fill: #e377c2; background-color: #e377c2 }    .Map_DeleteMappedResults { fill: #aec7e8; background-color: #aec7e8 }    .Map_ConvertToRavenJObject { fill: #e0e5a3; background-color: #e0e5a3 }    .Map_PutMappedResults { fill: #98df8a; background-color: #98df8a }    .Map_ScheduleReductions { fill: #ffbb78; background-color: #ffbb78 }        .Reduce_GetItemsToReduce { fill: #c5b0d5; background-color: #c5b0d5 }    .Reduce_DeleteScheduledReductions { fill: #c49c94; background-color: #c49c94 }    .Reduce_ScheduleReductions { fill: #f7b6d2; background-color: #f7b6d2 }    .Reduce_GetMappedResults { fill: #bcbd22; background-color: #bcbd22 }    .Reduce_RemoveReduceResults { fill: #dbdb8d; background-color: #dbdb8d }    .StorageCommit { fill: #9edae5; background-color: #9edae5 }    .UpdateDocumentReferences { fill: #ff9896; background-color: #ff9896 }    .Extension_Suggestions { fill: #55fa9a; background-color: #55fa9a; }    .ParallelBlock {       background:url(\'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAI0lEQVQIW2NkwAT/GdHE/gP5jMiCYAGQIpggXAAmiCIAEgQAAE4FBbECyZcAAAAASUVORK5CYII=\') repeat;    }    #reduceStatsInfo .mto_indent, #mapStatsInfo .mto_indent {        padding-left: 25px;    }    .no_data {        font-size: 18px;        font-weight: bold;    }</style><section id=\"indexingPerformance\" style=\"position: relative\">    <div>        <div class=\"btn-group\">            <button class=\"btn btn-default\" data-bind=\"event: { mousedown: startZoomIn, mouseup: stopZoomIn }\" title=\"Zoom in\"><i class=\"fa fa-search-plus\"></i></button>            <button class=\"btn btn-default\" data-bind=\"event: { mousedown: startZoomOut, mouseup: stopZoomOut }\" title=\"Zoom out\"><i class=\"fa fa-search-minus\"></i></button>            </div>        <div class=\"btn-group\">            <select id=\"visibleIndexesSelector\" data-bind=\"options: allIndexNames, selectedOptions: selectedIndexNames\" multiple=\"multiple\"></select>        </div>                <div class=\"btn-group\">            <button id=\"fullScreenButton\" class=\"btn btn-default\" data-bind=\"click: toggleFullscreen\" title=\"Fullscreen mode\">                <i class=\"fa fa-expand\"></i>            </button>        </div>    </div>    <section id=\"metricsContainer\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">        <svg id=\"indexPerformanceGraph\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>    </section></section>'; } )
define("text!views/database/status/indexing/indexPrefetches.html", [], function () { return '<section class=\"metricsContainer\">        <h3>Prefetches</h3>        <div id=\"prefetchesContainer\" class=\"with-3d-shadow with-transitions\" >        <svg data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\"></svg>    </div></section>'; } )
define("text!views/database/status/indexing/indexStats.html", [], function () { return '<style>    #indexStatsContainer {        font: 11px sans-serif;    }    .axis path,    .axis line {        fill: none;        stroke: #000;        shape-rendering: crispEdges;    }    .bar {        fill: steelblue;    }    rect.outputCounts, rect.inputCounts {        fill-opacity: 0.8;        cursor: pointer;    }    rect.outputCounts:hover, rect.inputCounts:hover {        fill-opacity: 1;    }    .legend rect {        cursor: pointer;    }    .legend_bg {        fill: white;        fill-opacity: 0.8;        stroke: #ccc;    }    .legendHidden {        fill-opacity: 0.2;       }    .selectable-tooltip {        pointer-events: all;        -webkit-touch-callout: all;        -webkit-user-select: all;        -khtml-user-select: all;        -moz-user-select: all;        -ms-user-select: all;        user-select: all;        padding: 10px;    }</style><section id=\"indexStatsContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <svg id=\"indexStatsGraph\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg></section>'; } )
define("text!views/database/status/indexing/perfDialogs/filteredOutIndexInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Filtered out index info</h4>    </div>    <div class=\"modal-body\">        All documents in batch was filtered out for index. No data for indexing.    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/indexing/perfDialogs/mapBatchInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Map batch information</h4>    </div>    <div class=\"modal-body\">        <table class=\"table table-condensed table-striped\" data-bind=\"with: indexingBatchInfo\">            <thead>                <tr>                    <th>Batch type</th>                    <th data-bind=\"text: BatchType\"></th>                </tr>            </thead>            <tbody>                <tr>                    <td>Indexes to work on</td>                    <td>                        <ul data-bind=\"foreach: IndexesToWorkOn\">                            <li data-bind=\"text: $data\"></li>                        </ul>                    </td>                </tr>                <tr>                    <td>Total document count</td>                    <td data-bind=\"text: TotalDocumentCount.toLocaleString(2)\"></td>                </tr>                <tr>                    <td>Total document size</td>                    <td data-bind=\"text: TotalDocumentSize.toLocaleString(2)\"></td>                </tr>                <tr>                    <td>Started at</td>                    <td data-bind=\"text: StartedAt\"></td>                </tr>                <tr>                    <td>Total duration [ms]</td>                    <td data-bind=\"text: TotalDurationMs.toLocaleString(2)\"></td>                </tr>            </tbody>        </table>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/indexing/perfDialogs/mapStatsInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Indexing information</h4>    </div>    <div class=\"modal-body\">        <table id=\"mapStatsInfo\" class=\"table table-condensed table-striped\" data-bind=\"with: perfStats\">            <thead>                <tr>                    <th>Index stats:</th>                    <th data-bind=\"text: indexName, attr: { colspan: CacheThreadCount }\"></th>                </tr>            </thead>            <tbody data-bind=\"with: stats\">                <tr>                    <td>Operation</td>                    <td data-bind=\"text: Operation, attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Input count</td>                    <td data-bind=\"text: InputCount.toLocaleString(2), attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Items count</td>                    <td data-bind=\"text: ItemsCount.toLocaleString(2), attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Output count</td>                    <td data-bind=\"text: OutputCount.toLocaleString(2), attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Duration</td>                    <td data-bind=\"text: Duration, attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Started</td>                    <td data-bind=\"text: Started, attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>                <!-- ko foreach: Operations -->                <!-- ko if:CacheIsSingleThread -->                <tr class=\"sto\">                    <td><span data-bind=\"attr: { \'class\': \'rect_legend \' + Name }\"></span> <span data-bind=\"text: Name + \' [ms]\'\"></span></td>                    <td data-bind=\"text: DurationMs, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <!-- /ko -->                <!-- ko if:!CacheIsSingleThread -->                <tr>                    <td><span class=\"rect_legend ParallelBlock\"></span> Parallel block</td>                    <!-- ko foreach: BatchedOperations -->                    <td data-bind=\"text: \'Thread #\' + ( 1 + $index())\"></td>                    <!-- /ko -->                </tr>                <tr class=\"mto\">                    <td class=\"mto_indent\">Thread start delay [ms]</td>                    <!-- ko foreach: BatchedOperations -->                    <td data-bind=\"text: StartDelay\"></td>                    <!-- /ko -->                </tr>                <!-- ko foreach: BatchedOperations[0].Operations -->                <tr class=\"mto\">                    <td class=\"mto_indent\"><span data-bind=\"attr: { \'class\': \'rect_legend \' + Name }\"></span> <span data-bind=\"text: Name + \' [ms]\'\"></span></td>                    <!-- ko foreach: $parent.BatchedOperations -->                    <td data-bind=\"text: Operations[$parentContext.$index()].DurationMs\"></td>                    <!-- /ko -->                </tr>                <!-- /ko -->                <!-- /ko -->                <!-- /ko -->                <tr>                    <td>Waiting time since<br /> last batch completed</td>                    <td data-bind=\"text: WaitingTimeSinceLastBatchCompleted, attr: { colspan: $parent.CacheThreadCount }\"></td>                </tr>            </tbody>        </table>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/indexing/perfDialogs/prefetchInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Prefetching information</h4>    </div>    <div class=\"modal-body\">        <table class=\"table table-condensed table-striped\" data-bind=\"with: data\">            <tbody>                <tr>                    <td>Started at</td>                    <td data-bind=\"text: Timestamp\"></td>                </tr>                <tr>                    <td>Total duration [ms]</td>                    <td data-bind=\"text: DurationMs.toLocaleString(2)\"></td>                </tr>                <tr>                    <td>Size</td>                    <td data-bind=\"text: Size.toLocaleString(2)\"></td>                </tr>                <tr>                    <td>Retries</td>                    <td data-bind=\"text: Retries.toLocaleString(2)\"></td>                </tr>            </tbody>        </table>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/indexing/perfDialogs/reduceBatchInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Reduce batch information</h4>    </div>    <div class=\"modal-body\">        <table class=\"table table-condensed table-striped\" data-bind=\"with: reducingBatchInfo\">            <tbody>                <tr>                    <td>Indexes to work on</td>                    <td>                        <ul data-bind=\"foreach: IndexesToWorkOn\">                            <li data-bind=\"text: $data\"></li>                        </ul>                    </td>                </tr>                <tr>                    <td>Started at</td>                    <td data-bind=\"text: StartedAt\"></td>                </tr>                <tr>                    <td>Total duration [ms]</td>                    <td data-bind=\"text: TotalDurationMs.toLocaleString(2)\"></td>                </tr>                <tr>                    <td>Time since first reduce<br />in batch completed [ms]</td>                    <td data-bind=\"text: TimeSinceFirstReduceInBatchCompletedMs.toLocaleString(2)\"></td>                </tr>            </tbody>        </table>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/indexing/perfDialogs/reduceStatsInfo.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Reducing information</h4>    </div>    <div class=\"modal-body\">        <table id=\"reduceStatsInfo\" class=\"table table-condensed table-striped\" data-bind=\"with: perfStats\">            <thead>                <tr>                    <th>Reduce of index stats:</th>                    <th data-bind=\"text: parent.indexName, attr: { colspan: CacheThreadCount }\"></th>                </tr>            </thead>            <tbody>                <!-- ko if: parent.stats.ReduceType == \'None\' -->                <tr data-bind=\"if: parent.stats.ReduceType == \'None\'\">                    <td>Operation</td>                    <td>Post reduce operation</td>                </tr>                <!-- /ko -->                <!-- ko if: parent.stats.ReduceType != \'None\' -->                <tr>                    <td>Reduce type</td>                    <td data-bind=\"text: parent.stats.ReduceType, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <tr data-bind=\"if: parent.stats.ReduceType != \'None\'\">                    <td>Level</td>                    <td data-bind=\"text: Level, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <tr data-bind=\"if: parent.stats.ReduceType != \'None\'\">                    <td>Items count</td>                    <td data-bind=\"text: ItemsCount.toLocaleString(2), attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <tr data-bind=\"if: parent.stats.ReduceType != \'None\'\">                    <td>Output count</td>                    <td data-bind=\"text: OutputCount.toLocaleString(2), attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <!-- /ko -->                <tr>                    <td>Duration</td>                    <td data-bind=\"text: Duration, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <tr>                    <td>Started</td>                    <td data-bind=\"text: Started, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <!-- ko foreach: Operations -->                <!-- ko if:CacheIsSingleThread -->                <tr class=\"sto\">                    <td><span data-bind=\"attr: { \'class\': \'rect_legend \' + Name }\"></span> <span data-bind=\"    text: Name + \' [ms]\'\"></span></td>                    <td data-bind=\"text: DurationMs, attr: { colspan: $root.perfStats.CacheThreadCount }\"></td>                </tr>                <!-- /ko -->                <!-- ko if:!CacheIsSingleThread -->                <tr>                    <td>Parallel block</td>                    <!-- ko foreach: BatchedOperations -->                    <td data-bind=\"text: \'Thread #\' + (1 + $index())\"></td>                    <!-- /ko -->                </tr>                <tr class=\"mto\">                    <td class=\"mto_indent\">Thread start delay [ms]</td>                    <!-- ko foreach: BatchedOperations -->                    <td data-bind=\"text: StartDelay\"></td>                    <!-- /ko -->                </tr>                <!-- ko foreach: BatchedOperations[0].Operations -->                <tr class=\"mto\">                    <td class=\"mto_indent\"><span data-bind=\"attr: { \'class\': \'rect_legend \' + Name }\"></span> <span data-bind=\"    text: Name + \' [ms]\'\"></span></td>                    <!-- ko foreach: $parent.BatchedOperations -->                    <td data-bind=\"text: Operations[$parentContext.$index()].DurationMs\"></td>                    <!-- /ko -->                </tr>                <!-- /ko -->                <!-- /ko -->                <!-- /ko -->            </tbody>        </table>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>    </div></div>'; } )
define("text!views/database/status/requests/requests.html", [], function () { return '<div id=\"metrics\">    <ul class=\"nav nav-tabs\" data-bind=\"foreach: router.navigationModel\">        <li data-bind=\"css: { active: isActive }\">             <a   data-bind=\"attr: { href: hash, title: tooltip }, text: title\"></a>        </li>    </ul>    <div class=\"tab-content\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/database/status/requests/requestsCount.html", [], function () { return '<section class=\"metricsContainer\">    <h3>        Request Counter        <small><a href=\"#\" target=\"_blank\" title=\"URL for raw request metrics. Opens in a new tab.\" data-bind=\"text: requestsMetricsUrl, attr: { href: requestsMetricsUrl }\"></a></small>    </h3>    <div id=\"requestCounterContainer\" class=\"with-3d-shadow with-transitions\">        <svg height=\"300\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\"></svg>    </div></section>'; } )
define("text!views/database/status/requests/requestTracing.html", [], function () { return '<section id=\"requestTracingContainer\" style=\"width: 100%\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">    <div style=\"height: 65%; position: relative;\">        <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 10000, onRefresh: $root.fetchRequestTracing.bind($root), active:true }\">        </button>        <div class=\"btn-group\" data-toggle=\"buttons\">            <label class=\"btn btn-default active\" data-bind=\"click: setStatusAll\">                <input type=\"radio\" name=\"options\" id=\"option1\"> All requests <span class=\"badge\" data-bind=\"text: allEntries().length\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setStatusSuccess\">                <input type=\"radio\" name=\"options\" id=\"option1\"> Success <span class=\"badge\" data-bind=\"text: successCount()\"></span>            </label>            <label class=\"btn btn-default\" data-bind=\"click: setStatusFailed\">                <input type=\"radio\" name=\"options\" id=\"option2\"> Failure <span class=\"badge\" data-bind=\"text: failedCount()\"></span>            </label>        </div>        <div id=\"requestTracingTableContainer\" style=\"overflow-y: auto; position: absolute; bottom: 10px; top: 50px; width: 100%; \">            <table class=\"single-line-row-table table table-condensed\" data-bind=\"event: { keydown: tableKeyDown }\" tabindex=\"1\">                <thead>                    <tr>                        <th class=\"col-md-2\">Method</th>                        <th class=\"col-md-2\">Status code</th>                        <th class=\"col-md-6\">Uri</th>                        <th class=\"col-md-2\">Execution time</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: allEntries, delegatedHandler: \'click\'\">                    <tr class=\"logRecord\" data-click=\"selectEntry\" data-bind=\"visible: IsVisible, css: { warning: StatusCode >= 400, \'selected\': $data === $parent.selectedEntry() }\">                        <td data-bind=\"text: Method\"></td>                        <td data-bind=\"text: StatusCode\"></td>                        <td data-bind=\"text: Uri, attr: { title: Uri }\"></td>                        <td data-bind=\"text: ExecutionTime\"></td>                    </tr>                </tbody>            </table>        </div>    </div>    <div class=\"panel panel-default flex-container\" style=\"height: 34%; overflow-y: auto\" data-bind=\"with: selectedEntry\">        <div class=\"panel-heading\">Headers</div>        <div class=\"panel-body flex-grow\">            <form class=\"form-horizontal\" data-bind=\"foreach: RequestHeaders\" style=\"position: \">                <div class=\"row\">                    <label class=\"col-md-3 text-right control-label\" data-bind=\"text: Name\">Message</label>                    <div class=\"col-md-9\">                        <p class=\"form-control-static\" data-bind=\"text: Values.join(\', \')\"></p>                    </div>                </div>            </form>        </div>    </div></section>'; } )
define("text!views/database/status/storage/statusStorage.html", [], function () { return '<div id=\"status-storage\">    <ul class=\"nav nav-tabs\" data-bind=\"foreach: router.navigationModel\">        <li data-bind=\"css: { active: isActive }\">             <a   data-bind=\"attr: { href: hash, title: tooltip }, text: title\"></a>        </li>    </ul>    <div class=\"tab-content\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/database/status/storage/statusStorageBreakdown.html", [], function () { return '<section>    <div data-bind=\"visible: isGlobalAdmin() == false\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this section!</h4>    </div>    <div data-bind=\"visible: isGlobalAdmin\" class=\"panel-group\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#storageBreakdown\">Detailed storage breakdown</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" id=\"storageBreakdown\" data-bind=\"with: data\">                <div class=\"panel-body\">                    <pre>                    <!-- ko foreach: $parent -->                        <span data-bind=\"html: $data\"></span><br />                    <!-- /ko -->                    </pre>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/storage/statusStorageCollections.html", [], function () { return '<section>    <div class=\"panel panel-danger\">        <div class=\"panel-heading\">            <h3 class=\"panel-title\">This operation is resource intensive!</h3>        </div>        <div class=\"panel-body\">            <p class=\"text-center\">                <button class=\"btn btn-danger\" data-bind=\"click: fetchDocCounts, enable: canSearch\">Compute docs counts</button>            </p>        </div>    </div>    <div data-bind=\"with: data\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h3 class=\"panel-title\">General Data</h3>            </div>            <div class=\"panel-body\">                <div class=\"row\">                    <label class=\"col-md-5 text-right form-control-static\">Time to generate</label>                    <span class=\"col-md-5 form-control-static\" data-bind=\"text: $root.formatTimeSpan(timeToGenerate)\"></span>                </div>                <div class=\"row\">                    <label class=\"col-md-5 text-right form-control-static\">Total Count</label>                    <span class=\"col-md-5 form-control-static\" data-bind=\"text: total\"></span>                </div>                <div class=\"row\">                    <label class=\"col-md-5 text-right form-control-static\">Total Size</label>                    <span class=\"col-md-5 form-control-static\" data-bind=\"text: $root.formatBytesToSize(totalSize)\"></span>                </div>                <div class=\"row\">                    <label class=\"col-md-5 text-right form-control-static\">Tombstones</label>                    <span class=\"col-md-5 form-control-static\" data-bind=\"text: tombstones\"></span>                </div>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h3 class=\"panel-title\">Basic stats</h3>            </div>            <div class=\"panel-body\">                <table class=\"table table-condensed table-striped\">                    <thead>                        <tr>                            <th>Name</th>                            <th>Amount</th>                            <th>Size</th>                            <th>Min</th>                            <th>Max</th>                            <th>Mean</th>                            <th>Std dev.</th>                        </tr>                    </thead>                    <tbody>                        <!-- ko foreach:collections -->                        <tr>                            <td data-bind=\"text: name\"></td>                            <td data-bind=\"text: stats.Counter\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(size)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Min)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Max)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Mean)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Stdev)\"></td>                        </tr>                        <!-- /ko -->                        <tr data-bind=\"with: system\">                            <td>System</td>                            <td data-bind=\"text: Stats.Counter\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(TotalSize)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Min)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Max)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Mean)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Stdev)\"></td>                        </tr>                        <tr data-bind=\"with: noCollection\">                            <td>No collection</td>                            <td data-bind=\"text: Stats.Counter\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(TotalSize)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Min)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Max)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Mean)\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Stdev)\"></td>                        </tr>                    </tbody>                </table>            </div>        </div>        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h3 class=\"panel-title\">Percentiles</h3>            </div>            <div class=\"panel-body\">                <table class=\"table table-condensed table-striped\">                    <thead>                        <tr>                            <th>Name</th>                            <th>50%</th>                            <th>75%</th>                            <th>95%</th>                            <th>99%</th>                            <th>99.9%</th>                            <th>99.99%</th>                        </tr>                    </thead>                    <tbody>                        <!-- ko foreach:collections -->                        <tr>                            <td data-bind=\"text: name\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'50%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'75%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'95%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'99%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'99.9%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(stats.Percentiles[\'99.99%\'])\"></td>                        </tr>                        <!-- /ko -->                        <tr data-bind=\"with: system\">                            <td>System</td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'50%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'75%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'95%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99.9%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99.99%\'])\"></td>                        </tr>                        <tr data-bind=\"with: noCollection\">                            <td>No collection</td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'50%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'75%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'95%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99.9%\'])\"></td>                            <td data-bind=\"text: $root.formatBytesToSize(Stats.Percentiles[\'99.99%\'])\"></td>                        </tr>                    </tbody>                </table>            </div>        </div>         <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h3 class=\"panel-title\">Top docs</h3>            </div>            <div class=\"panel-body\">                <table class=\"table table-condensed table-striped\">                    <thead>                        <tr>                            <th>Collection name</th>                            <th>Document ID</th>                            <th>Document Size</th>                        </tr>                    </thead>                    <tbody>                        <!-- ko foreach:collections -->                        <!-- ko foreach:topDocs -->                        <tr data-bind=\"css: { \'topBorder\': $index() == 0 }\">                            <!-- ko if: $index() == 0 -->                            <td style=\"background-color: white\" data-bind=\"attr: { rowspan: $parent.topDocs.length }\"><span data-bind=\"    text: $parent.name\"></span></td>                            <!-- /ko -->                            <td data-bind=\"text: DocId\"></td>                            <td data-bind=\"text: Size\"></td>                        </tr>                        <!-- /ko -->                        <!-- /ko -->                        <!-- /ko -->                    </tbody>                </table>            </div>        </div>    </div></section>'; } )
define("text!views/database/status/storage/statusStorageOnDisk.html", [], function () { return '<section>    <div class=\"panel-group\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#storageSpaceUsed\">Space used</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\"  id=\"storageSpaceUsed\"  data-bind=\"with: onDiskStats\">                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Transactional storage used size</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: TransactionalStorageUsedSizeHumaneSize\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Transactional storage allocated size</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: TransactionalStorageAllocatedSizeHumaneSize\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Index storage size</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: IndexStorageSizeHumane\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Total database size</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: TotalDatabaseSizeHumane\"></p>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"visible: isConfigForbidden() == false\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a data-toggle=\"collapse\" href=\"#onDiskPaths\">Paths</a>                </h4>            </div>            <div class=\"panel-collapse collapse in\" id=\"onDiskPaths\" data-bind=\"with: config\">                <div class=\"panel-body\">                    <div class=\"form-horizontal\" role=\"form\">                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Transactional logs location:</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: $root.journalsStoragePath()\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Data location:</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: DataDirectory\"></p>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-md-4 control-label\">Indexes location:</label>                            <div class=\"col-md-8\">                                <p class=\"form-control-static\" data-bind=\"text: IndexStoragePath\"></p>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/database/tasks/createSampleData.html", [], function () { return '<div class=\"panel panel-default\">    <div class=\"panel-heading\">        <h3 class=\"panel-title\">Create Sample Data</h3>    </div>    <div class=\"panel-body\">        <p>            <i class=\"fa fa-info-circle text-info\"></i>            Creating sample data requires an empty database.<br />        </p>        <div class=\"btn-group\">            <button class=\"btn btn-primary\" data-bind=\"click: generateSampleData\">                <i class=\"fa fa-magic\"></i>                Create sample data            </button>        </div>        <button class=\"btn btn-link\" data-bind=\"click: showSampleDataClass , enable: isEnable\">            <!--<i class=\"fa fa-file-code-o\"></i>-->            Show sample data classes        </button>        <div class=\"progress progress-striped active\" data-bind=\"visible: isBusy\">            <div class=\"progress-bar\" role=\"progressbar\" aria-valuenow=\"100\" aria-valuemin=\"0\" aria-valuemax=\"100\" style=\"width: 100%\"></div>        </div>    </div></div>'; } )
define("text!views/database/tasks/csvImport.html", [], function () { return '<div id=\"csvImportContainer\" class=\"panel panel-default\">    <div class=\"panel-heading\">        <h3 class=\"panel-title\">Csv Import</h3>    </div>    <div class=\"panel-body\">        <p>            <i class=\"fa fa-info-circle text-info\"></i>            Import a csv file into a collection. Each column will be treated as a property.        </p>        <div id=\"addfileContainer\">            <span class=\"btn btn-primary btn-file\">                Choose file <input id=\"csvFilePicker\" type=\"file\" data-bind=\"event: { change: fileSelected }, disable: isImporting\" />            </span>            <button class=\"btn btn-primary\" data-bind=\"click: importCsv, enable: hasFileSelected\">                <i class=\"fa fa-save\"></i>                CSV Import            </button>            <br />            <div class=\"progress progress-striped active\" data-bind=\"visible: isImporting\">                <div class=\"progress-bar\" role=\"progressbar\" style=\"width: 100%\"></div>            </div>        </div>    </div></div>'; } )
define("text!views/database/tasks/exportDatabase.html", [], function () { return '<div class=\"panel panel-default\">        <div class=\"panel-heading\">        <h3 class=\"panel-title\">Export Database</h3>    </div>    <div class=\"panel-body\">        <h4 class=\"pull-left\">Export your database to a .ravendump file.</h4>        <br />        <br />        <div class=\"btn-toolbar\">            <div>                <form class=\"navbar-form navbar-left\" data-bind=\"submit: startExport\">                    <button class=\"btn btn-primary\" type=\"submit\" download>                        <i class=\"fa fa-download\"></i>                        Export Database                    </button>                    <div class=\"checkbox\" data-bind=\"visible: !chooseDifferntFileName()\">                        <input type=\"checkbox\" class=\"styled\" id=\"different_name\" data-bind=\"checked: chooseDifferntFileName\">                        <label for=\"different_name\">Override file name</label>                    </div>                    <div data-bind=\"visible: chooseDifferntFileName\" class=\"form-group\">                        <input class=\"form-control\" type=\"text\" name=\"fileName\" data-bind=\"visible: chooseDifferntFileName, value: noneDefualtFileName\" title=\"Insert a file name to override the default one.\" placeholder=\"Exported File Name\">                    </div>                </form>                <form id=\"dbExportDownloadForm\" method=\"post\" data-bind=\"attr: { action: exportActionUrl }\" target=\"about:blank\">                    <input type=\"hidden\" id=\"SmugglerOptions\" name=\"SmugglerOptions\"/>                </form>            </div>        </div>        <br />        <ul class=\"nav nav-tabs\">            <li class=\"active\" data-bind=\"css: { active: !showAdvancedOptions()}\">                <a href=\"#\" data-bind=\"click:function(){this.selectOptions()}\">General</a>            </li>            <li data-bind=\"css: { active: showAdvancedOptions()}\">                <a href=\"#\" data-bind=\"click:function(){this.selectAdvancedOptions()}\">Advanced</a>            </li>        </ul>        <br />        <div class=\"btn-group btn-group-vertical\" data-bind=\"visible:!showAdvancedOptions()\">            <label class=\"btn btn-default\" data-bind=\"css:{active:includeDocuments() == true}, click: function(){$root.includeDocuments(!$root.includeDocuments()); return true;}, clickBubble: false\">                <i data-bind=\"css: {\'fa-check\': includeDocuments }\" class=\"fa fa-fw\"></i> Include Documents            </label>            <label class=\"btn btn-default\" data-bind=\"css:{active:includeIndexes() == true}, click: function(){$root.includeIndexes(!$root.includeIndexes()); return true;}, clickBubble: false\">                <i data-bind=\"css: {\'fa-check\': includeIndexes }\" class=\"fa fa-fw\"></i> Include Indexes            </label>            <label class=\"btn btn-default\" data-bind=\"css:{active:includeAttachments() == true}, click: function(){$root.includeAttachments(!$root.includeAttachments()); return true;}, clickBubble: false\">                <i data-bind=\"css: {\'fa-check\': includeAttachments }\" class=\"fa fa-fw\"></i> Include Attachments            </label>            <label class=\"btn btn-default\" data-bind=\"css:{active:includeTransformers() == true}, click: function(){$root.includeTransformers(!$root.includeTransformers()); return true;}, clickBubble: false\">                <i data-bind=\"css: {\'fa-check\': includeTransformers }\" class=\"fa fa-fw\"></i> Include Transformers            </label>            <label class=\"btn btn-default\" data-bind=\"css:{active:removeAnalyzers() == true}, click: function(){$root.removeAnalyzers(!$root.removeAnalyzers()); return true; }, clickBubble: false\">                <i data-bind=\"css: {\'fa-check\': removeAnalyzers }\" class=\"fa fa-fw\"></i> Remove Analyzers            </label>        </div>        <div data-bind=\"visible:showAdvancedOptions()\">            <form class=\"form-horizontal\" role=\"form\">                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Batch size</label>                    <div class=\"col-sm-2\">                        <input class=\"form-control\" type=\"number\" min=\"0\" data-bind=\"value: batchSize\" />                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Expired documents</label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: includeExpiredDocuments\">Include</span>                                <span data-bind=\"visible: !includeExpiredDocuments()\">Exclude</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: includeExpiredDocuments.bind(includeExpiredDocuments, true)\">Include</a></li>                                <li><a href=\"#\" data-bind=\"click: includeExpiredDocuments.bind(includeExpiredDocuments, false)\">Exclude</a></li>                            </ul>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Collections</label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: includeAllCollections\">All collections</span>                                <span data-bind=\"visible: !includeAllCollections()\">Specified collections only</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: includeAllCollections.bind(includeAllCollections, true)\">All collections</a></li>                                <li><a href=\"#\" data-bind=\"click: includeAllCollections.bind(includeAllCollections, false)\">Specified collections only</a></li>                            </ul>                        </div>                        <div style=\"margin-left: 20px;\" data-bind=\"visible: !includeAllCollections()\">                            <p class=\"form-control-static\">Choose collections:</p>                            <div class=\"btn-group-vertical\" data-bind=\"foreach: includedCollections\">                                <label class=\"btn btn-default\" data-bind=\"click: isIncluded.bind(isIncluded, !isIncluded()), css: { active: isIncluded }\">                                    <i class=\"fa fa-fw\" data-bind=\"css: { \'fa-check\': isIncluded }\"></i>                                    <span data-bind=\"text: collection\"></span>                                </label>                            </div>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">                        <span>Filters</span>                        <span class=\"help-block\">                            <a class=\"text-muted\" title=\"External link: RavenDB filters documentation\" target=\"_blank\" href=\"http://ravendb.net/docs/article-page/3.0/csharp/server/administration/exporting-and-importing-data\">                                <i class=\"fa fa-question-circle\"></i>                                Help                            </a>                        </span>                    </label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\">                            <table class=\"table table-striped\" style=\"max-width: 500px;\" data-bind=\"visible: filters().length\">                                <thead>                                    <tr>                                        <th style=\"width: 40%;\">Path</th>                                        <th style=\"width: 45%;\">Value</th>                                        <th style=\"width: 10%\">Should Match</th>                                        <th>&nbsp;</th>                                    </tr>                                </thead>                                <tbody data-bind=\"foreach: filters\">                                    <tr>                                        <td><input class=\"form-control\" type=\"text\" data-bind=\"value: path\" /></td>                                        <td><input class=\"form-control\" type=\"text\" data-bind=\"value: value\" /></td>                                        <td>                                            <div class=\"btn-group\">                                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                    <span data-bind=\"visible: shouldMatch\">Should match</span>                                                    <span data-bind=\"visible: !shouldMatch()\">Should <strong>not</strong> match</span>                                                    <span class=\"caret\"></span>                                                </button>                                                <ul class=\"dropdown-menu\" role=\"menu\">                                                    <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: shouldMatch.bind(shouldMatch, true)\">Should match</a></li>                                                    <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: shouldMatch.bind(shouldMatch, false)\">Should <strong>not</strong> match</a></li>                                                </ul>                                            </div>                                        </td>                                        <td>                                            <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeFilter.bind($root, $data)\" title=\"Remove this filter\">                                                <i class=\"fa fa-times\"></i>                                            </button>                                        </td>                                    </tr>                                </tbody>                            </table>                            <button class=\"btn btn-default\" data-bind=\"click: addFilter\">                                <i class=\"fa fa-plus\"></i> Add filter                            </button>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label id=\"transformScriptHelp\" class=\"col-sm-2 control-label\">                        Transform script&nbsp;                        <i class=\"fa fa-info-circle help-cursor\"></i>                    </label>                    <div class=\"col-sm-10\">                        <pre style=\"height: 200px;\" data-bind=\"aceEditor: { code: transformScript, lang: \'ace/mode/javascript\' }\">                        </pre>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Command</label>                    <div class=\"col-sm-10\">                        <pre data-bind=\"text: exportCommand\"></pre>                        </div>                </div>            </form>        </div>    </div></div>'; } )
define("text!views/database/tasks/importDatabase.html", [], function () { return '<div class=\"panel panel-default\">    <div class=\"panel-heading\">        <h3 class=\"panel-title\">Import Database</h3>    </div>    <div class=\"panel-body\">        <h4>Import data from a .ravendump file into the current database.</h4>        <p class=\"help-block\">            <i class=\"fa fa-warning text-warning\"></i>            Importing will overwrite any existing documents and indexes.        </p>        <div data-bind=\"with: activeDatabase\">            <span class=\"btn btn-primary btn-file\" data-bind=\"disable: isImporting\">                Choose file <input id=\"importDatabaseFilePicker\" type=\"file\" data-bind=\"event: { change: $root.fileSelected.bind($root, $element.value) }\" />            </span>            <button class=\"btn btn-primary\" data-bind=\"click: $root.importDb.bind($root), disable: !$root.hasFileSelected() || isImporting()\">                <i class=\"fa fa-upload\"></i>                Import Database            </button>            <p data-bind=\"visible: $root.hasFileSelected()\">                Selected file name: <span data-bind=\"text: $root.importedFileName()\"></span>            </p>            <br />                       <p data-bind=\"text: importStatus, visible: $root.isUploading || isImporting() || !!importStatus()\"></p>            <br />            <div class=\"progress progress-striped active\" data-bind=\"visible: isImporting\">                <div class=\"progress-bar\" role=\"progressbar\" style=\"width: 100%\"></div>            </div>        </div>        <ul class=\"nav nav-tabs\">            <li data-bind=\"click: selectOptions, css: { active: !showAdvancedOptions() }\"><a href=\"#\">General</a></li>            <li data-bind=\"click: selectAdvancedOptions, css: { active: showAdvancedOptions }\"><a href=\"#\">Advanced</a></li>        </ul>        <br />        <div class=\"btn-group-vertical\" data-bind=\"visible: !showAdvancedOptions()\">            <label class=\"btn btn-default\" style=\"\" data-bind=\"click: includeDocuments.toggle.bind(includeDocuments), css: { active: includeDocuments }\">                <i data-bind=\"css: {\'fa-check\': includeDocuments }\" class=\"fa fa-fw\"></i> Include documents            </label>            <label class=\"btn btn-default\" style=\"\"  data-bind=\"click: includeIndexes.toggle.bind(includeIndexes), css: { active: includeIndexes }\">                <i data-bind=\"css: {\'fa-check\': includeIndexes }\" class=\"fa fa-fw\"></i> Include indexes            </label>            <label class=\"btn btn-default text-left\" style=\"\" data-bind=\"click: includeAttachments.toggle.bind(includeAttachments), css: { active: includeAttachments }\">                <i data-bind=\"css: {\'fa-check\': includeAttachments }\" class=\"fa fa-fw\"></i> Include attachments            </label>            <label class=\"btn btn-default text-left\" style=\"\" data-bind=\"click: includeTransformers.toggle.bind(includeTransformers), css: { active: includeTransformers }\">                <i data-bind=\"css: {\'fa-check\': includeTransformers }\" class=\"fa fa-fw\"></i> Include transformers            </label>            <label class=\"btn btn-default text-left\" style=\"\" data-bind=\"click: removeAnalyzers.toggle.bind(removeAnalyzers), css: { active: removeAnalyzers }\">                <i data-bind=\"css: {\'fa-check\': removeAnalyzers }\" class=\"fa fa-fw\"></i> Remove analyzers            </label>        </div>        <div data-bind=\"visible: showAdvancedOptions\">            <form class=\"form-horizontal\" role=\"form\">                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Batch size</label>                    <div class=\"col-sm-2\">                        <input class=\"form-control\" type=\"number\" min=\"0\" data-bind=\"value: batchSize\" />                    </div>                </div>                <div class=\"form-group\">                    <div class=\"col-sm-offset-2 col-sm-5\">                        <div class=\"checkbox\">                            <input type=\"checkbox\" class=\"styled\" id=\"strip_replicaton_info\" data-bind=\"checked: stripReplicationInformation\">                            <label for=\"strip_replicaton_info\">Strip replication information from documents metadata</label>                        </div>                        <div class=\"checkbox\">                            <input type=\"checkbox\" class=\"styled\" id=\"disable_versioning\" data-bind=\"checked: shouldDisableVersioningBundle\">                            <label for=\"disable_versioning\">Disable versioning bundle during import</label>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Expired documents</label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: includeExpiredDocuments\">Include</span>                                <span data-bind=\"visible: !includeExpiredDocuments()\">Exclude</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: includeExpiredDocuments.bind(includeExpiredDocuments, true)\">Include</a></li>                                <li><a href=\"#\" data-bind=\"click: includeExpiredDocuments.bind(includeExpiredDocuments, false)\">Exclude</a></li>                            </ul>                        </div>                    </div>                </div>                <!--This doesn\'tappear to be supported in Raven 3.-->                <!--<div class=\"form-group\">                <label class=\"col-sm-2 control-label\">Collections</label>                <div class=\"col-sm-10\">                    <div class=\"btn-group\">                        <label class=\"btn btn-primary\" data-bind=\"click: includeAllCollections.bind(includeAllCollections, true), css: { active: includeAllCollections }\">                            All collections                        </label>                        <label class=\"btn btn-primary\" data-bind=\"click: includeAllCollections.bind(includeAllCollections, false), css: { active: !includeAllCollections() }\">                            Specified collections only                        </label>                    </div>                    <div style=\"margin-left: 20px;\" data-bind=\"visible: !includeAllCollections()\">                        <p class=\"form-control-static\">Choose collections:</p>                        <div class=\"btn-group-vertical\" data-bind=\"foreach: includedCollections\">                            <label class=\"btn btn-primary\" data-bind=\"text: collection, click: isIncluded.bind(isIncluded, !isIncluded()), css: { active: isIncluded }\"></label>                        </div>                    </div>                </div>            </div>-->                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">                        <span>Filters</span>                        <span class=\"help-block\">                            <a class=\"text-muted\" title=\"External link: RavenDB filters documentation\" target=\"_blank\" href=\"http://ravendb.net/docs/article-page/3.0/csharp/server/administration/exporting-and-importing-data\">                                <i class=\"fa fa-question-circle\"></i>                                Help                            </a>                        </span>                    </label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\">                            <table class=\"table table-striped\" style=\"max-width: 500px;\" data-bind=\"visible: filters().length\">                                <thead>                                    <tr>                                        <th style=\"width: 40%;\">Path</th>                                        <th style=\"width: 45%;\">Value</th>                                        <th style=\"width: 10%\">Should Match</th>                                        <th>&nbsp;</th>                                    </tr>                                </thead>                                <tbody data-bind=\"foreach: filters\">                                    <tr>                                        <td><input class=\"form-control\" type=\"text\" data-bind=\"value: Path\" /></td>                                        <td><input class=\"form-control\" type=\"text\" data-bind=\"value: Values[0]\" /></td>                                        <td>                                                                                            <div class=\"btn-group\">                                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                    <span data-bind=\"visible: ShouldMatchObservable\">Should match</span>                                                    <span data-bind=\"visible: !ShouldMatchObservable()\">Should <strong>not</strong> match</span>                                                    <span class=\"caret\"></span>                                                </button>                                                <ul class=\"dropdown-menu\" role=\"menu\">                                                    <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: ShouldMatchObservable.bind(ShouldMatchObservable, true)\">Should match</a></li>                                                    <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: ShouldMatchObservable.bind(ShouldMatchObservable, false)\">Should <strong>not</strong> match</a></li>                                                </ul>                                            </div>                                        </td>                                        <td>                                            <button type=\"button\" class=\"close\" data-bind=\"click: $root.removeFilter.bind($root, $data)\" title=\"Remove this filter\">                                                <i class=\"fa fa-times\"></i>                                            </button>                                        </td>                                    </tr>                                </tbody>                            </table>                            <button class=\"btn btn-default\" data-bind=\"click: addFilter\">                                <i class=\"fa fa-plus\"></i> Add filter                            </button>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label id=\"transformScriptHelp\" class=\"col-sm-2 control-label\">                        Transform script&nbsp;                        <i class=\"fa fa-info-circle help-cursor text-info\"></i>                    </label>                    <div class=\"col-sm-10\">                        <pre style=\"height: 200px;\" data-bind=\"aceEditor: { code: transformScript, lang: \'ace/mode/javascript\' }\">                        </pre>                    </div>                </div>            </form>                    </div>    </div></div>'; } )
define("text!views/database/tasks/subscriptionsTask.html", [], function () { return '<div class=\"panel panel-default\" data-bind=\"visible: data().length === 0\">    <div class=\"panel-heading\">        Data subscriptions    </div>    <div class=\"panel-body\">        <div class=\"text-center\">            <h3 class=\"text-info\">No data subscriptions for now. </h3>        </div>    </div></div><div class=\"panel-group\" data-bind=\"foreach: data\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <h4 class=\"panel-title\">                <a data-toggle=\"collapse\" data-bind=\"text: \'Id: \' + subscriptionId, attr: { href: \'#statsSubscriptionsCollapse\' + $index() }\"></a>            </h4>        </div>        <div class=\"panel-collapse collapse in\" data-bind=\"attr: { id: \'statsSubscriptionsCollapse\' + $index() }\">            <div class=\"panel-body\">                <div class=\"form-horizontal\" role=\"form\">                    <div class=\"form-group\">                        <label class=\"col-md-2 control-label\">Acknowledged Etag</label>                        <div class=\"col-md-10\">                            <p class=\"form-control-static\" data-bind=\"text: ackEtag\"></p>                        </div>                    </div>                    <form class=\"form-group\" data-bind=\"submit: $root.setSubscriptionEtag.bind($root, $data)\">                        <label class=\"col-md-2 control-label\">Set Acknowledged Etag</label>                        <div class=\"col-md-5\">                            <div class=\"input-group\">                                <input required class=\"form-control\" size=\"38\" data-bind=\"value: newAckEtag, valueUpdate: \'afterkeyup\', customValidity: etagCustomValidityError\">                                <span class=\"input-group-btn\">                                    <button class=\"btn btn-default\" type=\"submit\" data-bind=\"disable: isButtonDisabled\">Set</button>                                </span>                            </div>                        </div>                    </form>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/tasks/tasks.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        <span>Tasks</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">    </li></ul><div>    <div class=\"row\">        <div class=\"col-md-2\">            <ul class=\"nav nav-pills nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">                <li data-bind=\"css: { active: isActive }\">                    <a data-bind=\"attr: { href: hash }, text: title\"></a>                </li>            </ul>        </div>        <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">                    </div>    </div></div>'; } )
define("text!views/database/tasks/toggleIndexing.html", [], function () { return '<div class=\"panel panel-default\">    <div class=\"panel-heading\">        <h3 class=\"panel-title\">Toggle Indexing</h3>    </div>    <div class=\"panel-body\">        <div class=\"row\">            <div class=\"col-xs-12\">                <div class=\"btn-group\" data-toggle=\"buttons\">                    <label class=\"btn btn-default\" data-bind=\"click: enableIndexing, css: { active: isIndexingEnabled, disabled: isIndexingEnabled }\">                        <input type=\"checkbox\"> <i class=\"fa fa-play\"></i> Enable indexing                    </label>                    <label class=\"btn btn-default\" data-bind=\"click: disableIndexing, css: { active: canDisableIndexing() === false, disabled: canDisableIndexing() === false }\">                        <input type=\"checkbox\"> <i class=\"fa fa-stop text-danger\"></i> Disable indexing                    </label>                </div>            </div>        </div>        <br />        <div class=\"row\">            <div class=\"col-xs-12\">                <div class=\"btn-group\" data-toggle=\"buttons\">                    <label class=\"btn btn-default\" data-bind=\"click: enableReducing, css: { active: isReducingEnabled, disabled: isReducingEnabled }\">                        <input type=\"checkbox\"> <i class=\"fa fa-play\"></i> Enable reducing                    </label>                    <label class=\"btn btn-default\" data-bind=\"click: disableReducing, css: { active: isReducingEnabled() === false, disabled: isReducingEnabled() === false }\">                        <input type=\"checkbox\"> <i class=\"fa fa-stop text-danger\"></i> Disable reducing                    </label>                </div>            </div>        </div>        <br />        <div>            <span class=\"text-muted\">Current Status:</span>&nbsp;<span data-bind=\"text: indexingStatusText\"></span>        </div>    </div></div>'; } )
define("text!views/database/transformers/copyTransformerDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\" data-bind=\"text: isPaste === true ? \'Paste Transformer\' : \'Copy Transformer\'\"></h4>    </div>    <div class=\"modal-body\">        <h4>            <strong data-bind=\"text: isPaste === true ? \'CTRL+V, Enter \': \'CTRL+C, Enter \'\"></strong>            <span data-bind=\"text: isPaste === true ? \'to save\' : \'to copy\'\"></span>        </h4>        <div>            <pre data-bind=\"aceEditor: { code: transformerJSON, lang: \'ace/mode/json\', getFocus: true, selectAll: true, readOnly: !isPaste, bubbleEscKey: true }\" style=\"width: 600px; height: 200px;\"></pre>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close, visible: isPaste === true\">Cancel</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: saveTransformer, text: isPaste === true ? \'Save Transformer\' : \'Close\', enable: !!transformerJSON(), attr: { accesskey: isPaste ? \'S\' : \'\', title: isPaste ? \'Save transformer (Alt+S)\' : \'\' }\"></button>    </div></div>'; } )
define("text!views/database/transformers/deleteTransformerConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Delete transformer?</h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: transformersNames.length === 1\">You\'re deleting <strong data-bind=\"text: transformersNames[0]\"></strong></p>        <p data-bind=\"visible: transformersNames.length > 1\">You\'re deleting <strong data-bind=\"text: transformersNames.length\"></strong> transformers</p>        <ul data-bind=\"visible: transformersNames.length > 1, foreach: transformersNames\" style=\"max-height: 200px; overflow-y: auto;\">            <li data-bind=\"text: $data\"></li>        </ul>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteTransformers\">Yep, delete</button>    </div></div>'; } )
define("text!views/database/transformers/editTransformer.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li>        <a data-bind=\"attr: { href: appUrls.transformers }\">Transformers</a>    </li>    <li class=\"active\">        <span data-bind=\"text: transformerName, visible: transformerName() != null\"></span>        <span data-bind=\"visible: transformerName() == null\">New Transformer</span>    </li></ul><section id=\"editTransformerContainer\">    <div class=\"btn-toolbar\">        <div class=\"btn-group\">            <button class=\"btn btn-primary\" data-bind=\"click: saveTransformer, enable: $root.isSaveEnabled()\" title=\"Save the transformer (Alt+S)\" accesskey=\"S\">                <i class=\"fa fa-save\"></i>            </button>        </div>        <div class=\"btn-group\">            <button class=\"btn btn-default\" title=\"Format transformer\" data-bind=\"click: formatTransformer\">                <i class=\"fa fa-magic\"></i>            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: isEditingExistingTransformer\">            <button class=\"btn btn-default\" title=\"Refresh from server, discarding any unsaved changes\" data-bind=\"click: refreshTransformer\">                <i class=\"fa fa-refresh\"></i>            </button>            <button class=\"btn btn-danger\" data-bind=\"click: deleteTransformer\" title=\"Delete the transformer (Alt+Shift+Delete)\">                <i class=\"fa fa-trash-o\"></i>            </button>        </div>    </div>    <br />    <form class=\"form-horizontal\" role=\"form\" data-bind=\"with: editedTransformer, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\">        <div class=\"form-group\">            <label for=\"transformerName\" class=\"col-md-1 control-label\" title=\"Transformer name(Alt+I)\">Name</label>            <div class=\"col-md-11\">                <div class=\"row\">                    <div class=\"col-md-11\">                        <input type=\"text\" class=\"form-control\" id=\"transformerName\" placeholder=\"Transformer name\" accesskey=\"I\" data-bind=\"value: name, valueUpdate: \'afterkeyup\'\" />                    </div>                </div>            </div>        </div>        <div class=\"form-group\">            <label id=\"transformerResultsLabel\" style=\"cursor: help;\" class=\"col-sm-1 control-label\" data-placement=\"auto top\" data-original-title=\"Transform\" data-trigger=\"hover\">                Transform                <i class=\"fa fa-info-circle text-info\"></i>            </label>            <div class=\"col-md-11\">                <div class=\"row\">                    <div class=\"col-md-11\">                        <pre id=\"transformerAceEditor\" class=\"form-control transformer\" data-bind=\"aceEditor: { code: transformResults, getFocus: true}, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 65 }\"></pre>                    </div>                </div>                <br />            </div>        </div>    </form></section>'; } )
define("text!views/database/transformers/saveTransformerWithNewNameConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Can not rename and save a transformer</h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"text: message\"></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: saveTransformer\">OK</button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>    </div></div>'; } )
define("text!views/database/transformers/transformers.html", [], function () { return '<ul class=\"breadcrumb\">    <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>    <li>        <h3>            <a href=\"#resources\">                Databases            </a>        </h3>    </li>    <li data-bind=\"with: activeDatabase\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.documents }\"></a>    </li>    <li class=\"active\">        Transformers    </li></ul><div id=\"transformersContainer\">    <div class=\"btn-toolbar\" role=\"toolbar\">        <div class=\"btn-group\">            <a class=\"btn btn-primary\" data-bind=\"attr: { href: newTransformerUrl }\" title=\"Add a new transformer (Alt+N)\" accesskey=\"N\">                <i class=\"fa fa-plus\"></i> New transformer            </a>        </div>        <div class=\"btn-group\" data-bind=\"visible: transformersGroups().length > 0\">            <button class=\"btn btn-default\" data-bind=\"click: toggleExpandAll, attr: { title: expandCollapseTitle }\" accesskey=\"=\">                <i data-bind=\"css: { \'fa-plus-square-o\': !allTransformersExpanded(), \'fa-minus-square-o\': allTransformersExpanded }\" class=\"fa fa-fw\"></i>            </button>        </div>        <div class=\"btn-group\">            <button class=\"btn btn-default\" data-bind=\"click: pasteTransformer\" title=\"Paste transformer from JSON (Alt+V)\" accesskey=\"V\">                <i class=\"fa fa-paste\"></i>            </button>        </div>        <div class=\"btn-group\" data-bind=\"visible: transformersGroups().length > 0\">            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteAllTransformers\" title=\"Delete All Transformers\"><i class=\"fa fa-trash-o\"></i></button>        </div>    </div>    <div data-bind=\"visible: transformersGroups().length == 0\" class=\"text-center\">        <h4 class=\"page-header text-info\">No transformers have been created for this database. <a href=\"#\" data-bind=\"attr: { href: newTransformerUrl }\">Create one now</a>.</h4>    </div>    <br />    <div data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\' }\" style=\"overflow: auto\">        <div class=\"transformers-container panel-group\" id=\"transformersAccordion\" data-bind=\"foreach: transformersGroups\">            <div class=\"transformers-group panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <a data-toggle=\"collapse\" data-bind=\"attr: { href: \'#transformerCollapse\' + $index() }\">                            <span data-bind=\"text: entityName\"></span>                            <span class=\"text-muted\" data-bind=\"text: \'(\' + transformers().length + \')\'\"></span>                        </a>                    </h4>                </div>                <div class=\"index-group-content panel-collapse in\" data-bind=\"attr: { id: \'transformerCollapse\' + $index() }\">                    <div style=\"overflow: visible;\">                        <div class=\"panel-body\" data-bind=\"foreach: transformers\">                            <div class=\"index panel panel-default\">                                <div class=\"panel-body\">                                    <h4 class=\"pull-left\">                                        <a data-bind=\"attr: { href: editUrl }\">                                            <span data-bind=\"text: name\"></span>                                            <i class=\"fa\" data-bind=\"css: { \'fa-lock\': isLocked(), \'fa-unlock\': !isLocked() }\"></i>                                        </a>                                    </h4>                                    <div class=\"pull-right index-actions-container\">                                        <a class=\"btn btn-default\" title=\"Edit the transformer\" data-bind=\"attr: { href: editUrl }\">                                            <i class=\"fa fa-pencil\"></i>                                        </a>                                        <a class=\"btn btn-default\" title=\"Copy the transformer\" data-bind=\"click: $root.copyTransformer.bind($root)\">                                            <i class=\"fa fa-copy\"></i>                                        </a>                                        <a class=\"btn btn-default\" title=\"Lock/Un-Lock Transformer\" data-bind=\"click: $root.updateTransformerLockMode.bind($root, $data)\">                                            <i class=\"fa\" data-bind=\"css: { \'fa-lock\': isLocked(), \'fa-unlock\': !isLocked() }\"></i>                                        </a>                                        <button class=\"btn btn-danger\" title=\"Delete the transformer\" data-bind=\"click: $root.deleteTransformer.bind($root)\">                                            <i class=\"fa fa-trash-o\"></i>                                        </button>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/database/transformers/transformersShell.html", [], function () { return '<section data-bind=\"router: { cacheViews: false }\"></section>'; } )
define("text!views/filesystem/deleteFilesMatchingQueryConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Delete files matching query?</h4>    </div>    <div class=\"modal-body\">        <p>You\'re deleting <strong data-bind=\"text: totalFilesCount\"></strong> <span data-bind=\"visible: totalFilesCount === 1\">file.</span> <span data-bind=\"visible: totalFilesCount !== 1\">files.</span></p>        <br />        <p class=\"text-muted\">These are all the files <span data-bind=\"visible: queryText\">matching <pre data-bind=\"text: queryText, visible: queryText\"></pre></span></p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteFiles\">Yep, delete</button>    </div></div>'; } )
define("text!views/filesystem/configurations/configuration.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Configurations</span>    </li></ul><div id=\"configurationContainer\" tabindex=\"-1\">    <div class=\"text-center\" data-bind=\"visible: keys().length == 0\">        <h3>No configuration keys have been created for this file system. <small>Go ahead and <a href=\"#\" data-bind=\"click: newConfigurationKey\">create a configuration key</a>.</small></h3>    </div>    <div class=\"row\" data-bind=\"visible: keys().length > 0\">        <div class=\"col-md-3\">            <ul class=\"nav nav-pills nav-stacked keys-collection\" data-bind=\"foreach: keys\">                <li data-bind=\"event: { mousedown: $parent.selectKey.bind($parent) }, css: { active: $data === $parent.selectedKey() }\">                    <a href=\"#\" data-bind=\"text: key, click: $parent.selectKey.bind($parent)\"></a>                </li>            </ul>        </div>        <div id=\"settingsContainer\" class=\"col-md-9\">            <form data-bind=\"submit: save\">                <div class=\"row\">                    <div class=\"btn-toolbar\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: newConfigurationKey\" title=\"Create new key (Alt+N)\" accesskey=\"N\">                                <i class=\"fa fa-plus\"></i>                            </button>                        </div>                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: save, enable: isSaveEnabled\" title=\"Save (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refreshConfig\" title=\"Refresh (Alt+R)\" accesskey=\"R\"><i class=\"fa fa-refresh\"></i></button>                            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteConfiguration\" title=\"Delete (Alt+shift+Del)\" accesskey=\"D\"><i class=\"fa fa-trash-o\"></i></button>                        </div>                    </div>                </div>                <div class=\"row\"> &nbsp; </div>                <div class=\"row\">                    <pre id=\"configurationEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: text, fontSize:\'16px\', lang: \'ace/mode/json\'}, attr: { disabled: isBusy }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 20 }\"></pre>                </div>            </form>        </div>    </div></div>'; } )
define("text!views/filesystem/configurations/createConfigurationKey.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: create\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Create Configuration Key</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label for=\"databaseName\">Name</label>                <input required type=\"text\" class=\"form-control\" id=\"configurationKeyName\" placeholder=\"Configuration Key Name\" data-bind=\"value: configurationKeyName, hasfocus: configurationKeyNameFocus\">            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\">Create</button>        </div>    </form></div>'; } )
define("text!views/filesystem/configurations/deleteConfigurationKeys.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\">        <p data-bind=\"visible: keys().length === 1\">            <span>You\'re deleting</span>            <strong data-bind=\"with: keys()[0]\">                <span data-bind=\"text: key\"></span>            </strong>        </p>        <div data-bind=\"visible: keys().length > 1\">            <span>You\'re deleting</span>            <strong data-bind=\"text: keys().length\"></strong>            <span>items:</span>            <br />            <ul data-bind=\"foreach: keys\" style=\"max-height: 100px; overflow-y: auto;\">                <li data-bind=\"text: key\"></li>            </ul>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteKeys\">Yep, delete</button>    </div></div>'; } )
define("text!views/filesystem/files/createFolderInFilesystem.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: create\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Create folder</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label for=\"folderName\">Name</label>                <input required type=\"text\" class=\"form-control\" id=\"filesystemName\" placeholder=\"Folder Name\" data-bind=\"value: folderName\">            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\">Create</button>        </div>    </form></div>'; } )
define("text!views/filesystem/files/fileRenameDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 715px;\">    <div class=\" modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Rename file: <span data-bind=\"oldFilename\"></span></h4>    </div>    <div class=\"modal-body\">        <div class=\"row\">            <form class=\"form-horizontal\" role=\"form\">                <div class=\"form-group\">                    <div class=\"col-sm-4 control-label\">New file name:</div>                    <div class=\"col-sm-8\">                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: newName\" />                    </div>                </div>            </form>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        <button class=\"btn btn-primary\" data-bind=\"click: rename\">Ok</button>    </div></div>'; } )
define("text!views/filesystem/files/filesystemEditFile.html", [], function () { return '<div id=\"editFileContainer\" tabindex=\"-1\">    <!-- We use a container with a tab index so that we can capture keyboard events, such as CTRL+S. -->    <ul class=\"breadcrumb\">        <li>            <h3>                <a href=\"#resources\">                    File Systems                </a>            </h3>        </li>        <li class=\"active\">            <span data-bind=\"text: fileName\"></span>        </li>    </ul>    <div class=\"btn-toolbar\">        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: saveFileMetadata\" title=\"Save (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refreshFile\" title=\"Refresh (Alt+R)\" accesskey=\"R\"><i class=\"fa fa-refresh\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: downloadFile\" title=\"Download file (Alt+O)\" accesskey=\"O\"><i class=\"fa fa-download\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: renameFile\" title=\"Rename file (Alt+E)\" accesskey=\"E\"><i class=\"fa fa-random\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: deleteFile\" title=\"Delete (Alt+shift+Del)\"><i class=\"fa fa-trash-o\"></i></button>        </div>    </div>    <br />    <div class=\"row\">        <div class=\"col-md-7\">            <div class=\"row\">                <div class=\"col-md-12\">                    <input type=\"text\" class=\"form-control\" title=\"File name\" data-bind=\"value: fileName, attr: { disabled: true }\" />                </div>            </div>            <br />            <div class=\"row\">                <div class=\"col-md-12\">                    <ul class=\"nav nav-tabs\">                        <li accesskey=\"m\" title=\"Edit file metadata (Alt+M)\">File Metadata</li>                    </ul>                    <pre id=\"fileMetadataEditor\" class=\"form-control\" data-bind=\"aceEditor: { code: fileMetadataText, fontSize:\'16px\', lang: \'ace/mode/json\'}, attr: { disabled: isBusy }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: 20 }\"></pre>                </div>            </div>        </div>        <div class=\"form-group panel panel-default col-md-5 document-addition-data side-panel\">            <div class=\"col-md-12\" data-bind=\"with: metadata\">                <h4><strong>Metadata</strong></h4>                <hr />                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Etag</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: etag\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Creation-Date</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: creationDate\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>Last-Modified</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: lastModified\"></span>                    </div>                </div>                <div class=\"row\">                    <div class=\"col-md-5\">                        <label>RavenFS-Size</label>                    </div>                    <div class=\"col-md-7\">                        <span data-bind=\"text: ravenFSSize\"></span>                    </div>                </div>            </div>            <div class=\"col-md-12 query-metadata\" data-bind=\"visible:topRecentFiles().length > 0\">                <h4><strong>Recent Files</strong></h4>                <hr />                <div class=\"documents-links-container\" data-bind=\"foreach: topRecentFiles\">                    <a class=\"document-link\" data-bind=\"attr: { href: fileUrl }\">                        <span class=\"collection-color-strip system-documents-collection\">&nbsp;</span>                        <span data-bind=\"text: fileId\"></span>                    </a>                </div>            </div>                    </div>    </div></div>'; } )
define("text!views/filesystem/files/filesystemFiles.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeFilesystem\">        <span data-bind=\"text: name\" />    </li>    <li class=\"active\">        <span>All</span>    </li></ul><div id=\"fileSystemFilesPage\">    <div class=\"row\">        <div class=\"col-md-2\">            <div class=\"btn-toolbar\" role=\"toolbar\">                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-primary\" accesskey=\"n\" data-bind=\"click: createFolder, enable: inRevisionsFolder() == false\" title=\"Create folder\">                        <span>New Folder</span>                    </button>                </div>                <div class=\"btn-group\" data-bind=\"visible: inRevisionsFolder() == false\">                    <span class=\"btn btn-primary btn-file\" title=\"Upload files\">                        <i class=\"fa fa-upload\"></i>                        <input id=\"upload\" name=\"upload\" type=\"file\" multiple=\"multiple\" data-bind=\"value: uploadFiles, fileUpload: { files: uploadFiles, directory: selectedFolder, uploads: uploadQueue, before: expandUploadQueuePanel, success: uploadSuccess.bind($data), fail: uploadFailed.bind($data) }\" />                    </span>                </div>                <div class=\"btn-group\" data-bind=\"visible: !inRevisionsFolder()\">                    <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteFolder\" title=\"Remove folder\">                        <i class=\"fa fa-trash-o\"></i>                    </button>                </div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: refresh, disable: showLoadingIndicator()\" title=\"Refresh files from server\">                        <i class=\"fa fa-refresh\"></i>                    </button>                </div>            </div>            <span><br/></span>            <div data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'#uploadQueueContainer\', bottomMargin: 5}\">                <div id=\"filesTree\" data-bind=\"tree: { selectedNode: selectedFolder, addedNode: addedFolder, currentLevelNodes: currentLevelSubdirectories }\"></div>            </div>        </div>        <div class=\"col-md-10\">            <div class=\"btn-toolbar\" role=\"toolbar\">                <div class=\"btn-group\" data-bind=\"style: { visibility : hasFiles() ? \'visible\' : \'hidden\' }\">                    <div class=\"checkbox\">                        <input id=\"files_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: filesSelection, event: { change: toggleSelectAll }\" title=\"Select all or none\">                        <label for=\"files_all\">&nbsp;</label>                    </div>                </div>                <div class=\"btn-group\" data-bind=\"visible: hasAnyFilesSelected\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: selectedFilesIndices().length == 1, click: editSelectedFile\" title=\"Edit selected document (F2)\">                        <i class=\"fa fa-pencil\"></i>                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: selectedFilesIndices().length == 1, click: downloadSelectedFiles\" title=\"Download selected document\">                        <i class=\"fa fa-download\"></i>                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: selectedFilesIndices().length == 1 , click: renameSelectedFile\" title=\"Rename file\">                        <i class=\"fa fa-random\"></i>                    </button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: deleteSelectedFiles\" title=\"Delete selected documents (DEL)\">                        <i class=\"fa fa-trash-o\"></i>                    </button>                </div>                <div class=\"btn-group\">                    <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicatorThrottled\"><i class=\"fa fa-spinner fa-spin fa-2x\"></i></div>                </div>            </div>            <div class=\"files-select\" data-bind=\"visible: hasAnyFilesSelected() && !hasAllFilesSelected() && isAnyFilesAutoSelected()\">                <i class=\"fa fa-info-circle\"></i> You have selected <strong data-bind=\"text: selectedFilesIndices().length.toLocaleString()\"></strong> <span data-bind=\"text: selectedFilesText\"></span>.                <span class=\"select-link\" data-bind=\"click: selectAll.bind($root)\">                    Select all <strong data-bind=\"text: filesCount\"></strong> files in <span data-bind=\"text: selectedFolderName\"></span>                </span>            </div>            <div class=\"files-select\" data-bind=\"visible: hasAllFilesSelected() && isAllFilesAutoSelected()\">                You have selected <strong data-bind=\"text: selectedFilesIndices().length\"></strong> <span data-bind=\"text: selectedFilesText\"></span> in                <span data-bind=\"text: selectedFolderName\"></span>.                <span class=\"select-link\" data-bind=\"click: selectNone.bind($root)\">                    Clear selection                </span>            </div>            <div id=\"filesGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: allFilesPagedItems, selectedIndices: selectedFilesIndices, dynamicHeightTargetSelector: \'#uploadQueueContainer\', gridSelector: \'#filesGrid\', contextMenuOptions: [\'Delete\'], isAnyAutoSelected: isAnyFilesAutoSelected, isAllAutoSelected: isAllFilesAutoSelected, rowsAreLoading: showLoadingIndicator, noResultsMessage: \'No files were found!\' }\"></div>        </div>    </div>    <div class=\"row\">        <div id=\"uploadQueueContainer\" class=\"upload-queue panel panel-info\">            <div class=\"panel-heading\">                <span><i class=\"fa fa-upload bold blue\"></i>&nbsp;Upload Queue</span>                <a href=\"#\" id=\"uploadQueuePanelToggle\" class=\"opened pull-right\" style=\"float:right; padding-right:10px\" data-bind=\"click: toggleCollapseUploadQueue\"><i class=\"fa fa-angle-double-down bold red\"></i></a>            </div>            <div class=\"panel-body\">                <div id=\"uploadQueue\" >                    <button id=\"clearUploadQueue\" class=\"btn btn-default btn-sm pull-right\" title=\"Clear queue\"                             data-trigger=\"hover\" data-toggle=\"tooltip\" data-bind=\"click: $root.clearUploadQueue.bind($root)\">                        <i class=\"fa fa-ban\"></i>                    </button>                    <div style=\"height:230px; overflow-y:auto\">                        <table class=\"table table-striped table-hover\">                            <thead>                                <tr class=\"database-header\">                                    <td class=\"ko-grid-column-header\"><b>File</b></td>                                    <td class=\"ko-grid-column-header\"><b>Status</b></td>                                </tr>                            </thead>                            <tbody data-bind=\"foreach: $root.uploadQueue\">                                <tr class=\"database-row\">                                    <td data-bind=\"text: fileName\"></td>                                    <td data-bind=\"text: status\"></td>                                </tr>                            </tbody>                        </table>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/filesystem/files/searchFileSizeRangeClause.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add search filter</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group form-inline\">                <label>File size between: </label>                <input type=\"text\" class=\"form-control\" data-bind=\"value: inputs[0], valueUpdate: \'afterkeydown\'\" />                <label> and: </label>                <input type=\"text\" class=\"form-control\" data-bind=\"value: inputs[1], valueUpdate: \'afterkeydown\'\" />            </div>            <div class=\"form-group help-block\">                <p><b>Hint:</b> use <b>Kb</b>, <b>Mb</b> or <b>Gb</b> to specify unit (case insensitive). Default: Bytes</p>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/files/searchHasMetadataClause.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form class=\"form-horizontal\" data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add search filter</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Key:</label>                <div class=\"col-sm-8\">                    <input id=\"keysAutocomplete\" type=\"text\" class=\"form-control\" placeholder=\"Search key...\" data-bind=\"value: inputs[0], valueUpdate: \'afterkeydown\'\" autocomplete=\"off\" />                    <ul style=\"display: none\" class=\"dropdown-menu\" role=\"menu\" data-bind=\"autoComplete: \'#keysAutocomplete\', foreach: keySearchResults\">                        <li role=\"presentation\" data-bind=\"click: $root.setKey.bind($root, $data)\">                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                <span data-bind=\"text: $data\"></span>                            </a>                        </li>                    </ul>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">value:</label>                <div class=\"col-sm-8\">                    <input type=\"text\" class=\"form-control\" data-bind=\"value: inputs[1], valueUpdate: \'afterkeydown\'\" />                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/files/searchLastModifiedBetweenClause.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form class=\"form-horizontal\" data-bind=\"submit: applyFilter\" style=\"width: 450px\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add search filter</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Between: </label>                <div class=\"col-sm-8\">                    <div class=\"input-group date\" id=\"fromDate\" data-bind=\"datePicker: fromDate, datepickerOptions: { endDateElement: \'toDate\' }\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: inputs[0]\" />                        <span class=\"input-group-addon\">                            <span class=\"fa fa-calendar\"></span>                        </span>                    </div>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\"> and: </label>                <div class=\"col-sm-8\">                    <div class=\"input-group date\" id=\"toDate\" data-bind=\"datePicker: toDate, datepickerOptions: { startDateElement: \'fromDate\' }\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: inputs[1]\" />                        <span class=\"input-group-addon\">                            <span class=\"fa fa-calendar\"></span>                        </span>                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/files/searchSingleInputClause.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">        <form data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add search filter</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label data-bind=\"text: label\"></label>                <input type=\"text\" id=\"firstInput\" class=\"form-control\" data-bind=\"value: inputs[0], valueUpdate: \'afterkeydown\'\"/>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/search/fieldRangeFilter.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add range filter </h4>        </div>                <div class=\"modal-body\">            <div data-bind=\"visible: isDateTime(), if: isDateTime()\">                <div class=\"form-group\">                    <label for=\"fromDate\">From: </label>                    <div class=\"input-group date\" id=\"fromDate\" data-bind=\"datePicker: fromDate, datepickerOptions: { endDateElement: \'toDate\'}\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: from, valueUpdate: \'afterkeydown\'\" autofocus />                        <span class=\"input-group-addon\" data-bind=\"visible:isDateTime()\">                            <span class=\"fa fa-calendar\"></span>                        </span>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"toDate\">To: </label>                    <div class=\"input-group date\" id=\"toDate\" data-bind=\"datePicker: toDate, datepickerOptions: { startDateElement: \'fromDate\' }\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: to, valueUpdate: \'afterkeydown\'\" autofocus />                        <span class=\"input-group-addon\" data-bind=\"visible:isDateTime()\">                            <span class=\"fa fa-calendar\"></span>                        </span>                    </div>                </div>            </div>            <div data-bind=\"visible: !isDateTime(), if: !isDateTime()\">                <div class=\"form-group\">                    <label for=\"fromDate\">From: </label>                    <input type=\"text\" class=\"form-control\" data-bind=\"value: from, valueUpdate: \'afterkeydown\'\" autofocus />                </div>                <div class=\"form-group\">                    <label for=\"toDate\">To: </label>                    <input type=\"text\" class=\"form-control\" data-bind=\"value: to, valueUpdate: \'afterkeydown\'\" autofocus />                </div>            </div>            <div class=\"form-group\">                <label>Search type</label>                <select type=\"text\" class=\"form-control\" data-bind=\"options: filterOptions, value: selectedOption\" />            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/search/fieldStringFilter.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add string field filter </h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label data-bind=\"text: label\"></label>                <input type=\"text\" id=\"firstInput\" class=\"form-control\" data-bind=\"value: inputs[0], valueUpdate: \'afterkeydown\'\" autofocus />            </div>            <div class=\"form-group\">                <label>Search type</label>                <select type=\"text\" class=\"form-control\" data-bind=\"options: filterOptions, value: selectedOption\" />            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/search/inMethodFilter.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: applyFilter\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"false\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Add in method filter </h4>        </div>        <div class=\"modal-body\">            <table class=\"table table-striped table-hover\">                <thead>                    <tr>                        <th>Value</th>                        <th width=\"20%\"></th>                    </tr>                </thead>                <tbody data-bind=\"foreach: elements\">                    <tr>                        <td>                            <input type=\"text\" class=\"form-control\" data-bind=\"value: value, hasFocus: focus\" />                        </td>                        <td>                            <button type=\"button\" class=\"close\" data-bind=\"click: $parent.removeElement.bind($parent, $data), visible: $parent.deleteEnabled\" title=\"Remove this value\">                                <i class=\"fa fa-times\"></i>                            </button>                        </td>                    </tr>                </tbody>            </table>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: newElement\" title=\"Add a parameter\">                <i class=\"fa fa-plus\"></i> Add Search Value            </button>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" data-bind=\"enable: enabled()\" class=\"btn btn-primary\">Apply</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/filesystem/search/search.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Search</span>    </li></ul><div class=\"filesystems-page\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <form class=\"form-inline\">                <div class=\"row\" style=\"padding: 0 10px 0 10px\">                    <div class=\"input-group col-lg-12\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: searchText\" placeholder=\"E.g. __fileName:MyFile* AND __directoryName:/subfolder\" />                        <a class=\"input-group-addon\" style=\"width: 30px\" href=\"#\" data-bind=\"click: clear\">                           <i class=\"fa fa-times\"></i>                        </a>                        <a href=\"#\" data-bind=\"click: search\" class=\"input-group-addon\" style=\"width: 30px\">                            <i class=\"fa fa-search\"></i>                        </a>                        <a href=\"#\" class=\"btn btn-danger input-group-addon\" style=\"width: 30px\" title=\"Delete all files matching the query\" data-bind=\"click: deleteFilesMatchingQuery\">                            <i class=\"fa fa-trash-o\"></i>                        </a>                    </div>                </div>            </form>            <br />            <h5>Add search filter</h5>            <div class=\"row\" style=\"padding: 0 10px 0 10px\">                <div class=\"btn-group\">                    <button id=\"fileNameStartsWith\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fileNameStartsWith\">File name starts with...</button>                    <button id=\"fileNameEndsWith\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fileNameEndsWith\">File name ends with...</button>                    <button id=\"fileSizeBetween\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: fileSizeBetween\">File size...</button>                    <button id=\"hasMetadata\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: hasMetadata\">Has metadata...</button>                    <button id=\"inFolder\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: inFolder\">In folder...</button>                    <button id=\"lastModifiedBetween\" type=\"button\" class=\"btn btn-default\" data-bind=\"click: lastModifiedBetween\">Last modified...</button>                </div>                <button class=\"btn btn-danger pull-right \" data-bind=\"click: resetIndex\" title=\"Reset index\"><i class=\"fa fa-undo\"></i> Reset index</button>            </div>        </div>        <div class=\"panel-body\" style=\"position: relative\">            <div id=\"filesGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: allFilesPagedItems, selectedIndices: selectedFilesIndices, gridSelector: \'#filesGrid\', contextMenuOptions: [], selectionEnabled: false }\"></div>        </div>    </div></div>'; } )
define("text!views/filesystem/settings/settings.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Settings</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">        <a data-bind=\"attr: { href: hash }, text: title\"></a>    </li></ul><div class=\"settings-container row\">    <div class=\"col-md-2\">        <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">            <li data-bind=\"css: { active: isActive() }, visible: $parent.routeIsVisible($data)\">                <a data-bind=\"attr: { href: hash }, text: title\"></a>            </li>        </ul>    </div>    <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/filesystem/settings/versioning.html", [], function () { return '<section>    <form class=\"versioning-settings-form form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges.bind($root)\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: isSaveEnabled\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            </div>            <div class=\"panel-body\">                <div data-bind=\"with: versioning\">                    <div class=\"panel versioning-card\" data-bind=\"css: { \'panel-warning\': disabled, \'panel-default\': !disabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span> Versioning setup</span>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <div class=\"col-sm-4 control-label\">Max Revisions</div>                                <div class=\"col-sm-8\">                                    <input required type=\"number\" class=\"form-control\" data-bind=\"numericValue: maxRevisions, valueUpdate: \'afterkeyup\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-4 control-label\">Excluded</div>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: !exclude()\">Include</span>                                            <span data-bind=\"visible: exclude\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, true)\">Exclude</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-4 control-label\">Exclude unless explicit</div>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: !excludeUnlessExplicit()\">Include</span>                                            <span data-bind=\"visible: excludeUnlessExplicit\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, true)\">Exclude</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-4 control-label\">Purge on delete</div>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: !purgeOnDelete()\">No</span>                                            <span data-bind=\"visible: purgeOnDelete\">Yes</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, false)\">No</a></li>                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, true)\">Yes</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-4 control-label\">Reset on rename</div>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: !resetOnRename()\">No</span>                                            <span data-bind=\"visible: resetOnRename()\">Yes</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: resetOnRename.bind($root, false)\">No</a></li>                                            <li><a href=\"#\" data-bind=\"click: resetOnRename.bind($root, true)\">Yes</a></li>                                        </ul>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/filesystem/status/resetIndexConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Reset index?</h4>    </div>    <div class=\"modal-body\">        <p>You\'re resetting file system index.</p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: resetIndex\">Yep, reset</button>    </div></div>'; } )
define("text!views/filesystem/status/status.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Status</span>    </li></ul><div class=\"filesystems-page\">    <button class=\"btn btn-primary\" data-bind=\"click: synchronizeNow\" title=\"Sync Now (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-refresh\"></i> Sync Now</button>    <br />    <br />    <div class=\"row\">        <div class=\"col-md-6\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <span>Outgoing synchronizations</span>                        <span class=\"glyphicon glyphicon-export\"></span>                    </h4>                </div>                <div class=\"panel-body\">                    <div>                        <ul class=\"nav nav-tabs\">                            <li class=\"active\">                                <a data-toggle=\"tab\" href=\"#outgoing-active\">                                    <span>Active</span>                                    <span class=\"text-muted\" data-bind=\"text: \'(\'+ activeOutgoing.totalCount() +\')\'\"></span>                                </a>                            </li>                            <li>                                <a data-toggle=\"tab\" href=\"#outgoing-pending\">                                    <span>Pending</span>                                    <span class=\"text-muted\" data-bind=\"text: \'(\'+ pendingOutgoing.totalCount() +\')\'\"></span>                                </a>                            </li>                        </ul>                        <div class=\"tab-content\">                            <div id=\"outgoing-active\" class=\"tab-pane fade in active\">                                <div data-bind=\"with: activeOutgoing\">                                    <div style=\"height: 392px; overflow-y: auto\">                                        <table class=\"table table-striped table-hover\">                                            <thead>                                                <tr class=\"database-header\">                                                    <td class=\"ko-grid-column-header\"><b>File Name</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Etag</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Destination Url</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Type</b></td>                                                </tr>                                            </thead>                                            <tbody data-bind=\"foreach: items\">                                                <tr class=\"database-row\">                                                    <td data-bind=\"text: FileName\"></td>                                                    <td data-bind=\"text: FileEtag\"></td>                                                    <td data-bind=\"text: DestinationUrl\"></td>                                                    <td data-bind=\"text: TypeDescription\"></td>                                                </tr>                                            </tbody>                                        </table>                                    </div>                                    <div>                                        <ul class=\"pagination pull-right\" data-bind=\"foreach: allPages, visible: allPages().length > 1\">                                            <li data-bind=\"css: { active: pageNumber === ($parent.pageIndex() + 1) }\"><a href=\"#\" data-bind=\"text: pageNumber, click: function() { $parent.moveToPage(pageNumber-1); }\"></a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div id=\"outgoing-pending\" class=\"tab-pane fade\">                                <div data-bind=\"with: pendingOutgoing\">                                    <div style=\"height: 392px; overflow-y: auto\">                                        <table class=\"table table-striped table-hover\">                                            <thead>                                                <tr class=\"database-header\">                                                    <td class=\"ko-grid-column-header\"><b>File Name</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Etag</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Destination Url</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Type</b></td>                                                </tr>                                            </thead>                                            <tbody data-bind=\"foreach: items\">                                                <tr class=\"database-row\">                                                    <td data-bind=\"text: FileName\"></td>                                                    <td data-bind=\"text: FileEtag\"></td>                                                    <td data-bind=\"text: DestinationUrl\"></td>                                                    <td data-bind=\"text: TypeDescription\"></td>                                                </tr>                                            </tbody>                                        </table>                                    </div>                                    <div>                                        <ul class=\"pagination pull-right\" data-bind=\"foreach: allPages, visible: allPages().length > 1\">                                            <li data-bind=\"css: { active: pageNumber === ($parent.pageIndex() + 1) }\"><a href=\"#\" data-bind=\"text: pageNumber, click: function() { $parent.moveToPage(pageNumber-1); }\"></a></li>                                        </ul>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"col-md-6\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <span class=\"glyphicon glyphicon-import\"></span>                        <span>Incoming synchronizations</span>                    </h4>                </div>                <div class=\"panel-body\">                    <div>                        <ul class=\"nav nav-tabs\">                            <li class=\"active\">                                <a data-toggle=\"tab\" href=\"#incoming-active\">                                    <span>Active</span>                                    <span class=\"text-muted\" data-bind=\"text: \'(\'+ activeIncoming.totalCount() +\')\'\"></span>                                </a>                            </li>                            <li>                                <a data-toggle=\"tab\" href=\"#incoming-finished\">                                    <span>Finished</span>                                    <span class=\"text-muted\" data-bind=\"text: \'(\'+ finishedIncoming.totalCount() +\')\'\"></span>                                </a>                            </li>                        </ul>                        <div class=\"tab-content\">                            <div id=\"incoming-active\" class=\"tab-pane fade in active\">                                <div data-bind=\"with: activeIncoming\">                                    <div style=\"height: 392px; overflow-y: auto\">                                        <table class=\"table table-striped table-hover\">                                            <thead>                                                <tr class=\"database-header\">                                                    <td class=\"ko-grid-column-header\"><b>File Name</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Etag</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Destination Url</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Type</b></td>                                                </tr>                                            </thead>                                            <tbody data-bind=\"foreach: items\">                                                <tr class=\"database-row\">                                                    <td data-bind=\"text: FileName\"></td>                                                    <td data-bind=\"text: FileEtag\"></td>                                                    <td data-bind=\"text: DestinationUrl\"></td>                                                    <td data-bind=\"text: TypeDescription\"></td>                                                </tr>                                            </tbody>                                        </table>                                    </div>                                    <div>                                        <ul class=\"pagination pull-right\" data-bind=\"foreach: allPages, visible: allPages().length > 1\">                                            <li data-bind=\"css: { active: pageNumber === ($parent.pageIndex() + 1) }\"><a href=\"#\" data-bind=\"text: pageNumber, click: function() { $parent.moveToPage(pageNumber-1); }\"></a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div id=\"incoming-finished\" class=\"tab-pane fade\">                                <div data-bind=\"with: finishedIncoming\">                                    <div style=\"height: 392px; overflow-y: auto\">                                        <table class=\"table table-striped table-hover\">                                            <thead>                                                <tr class=\"database-header\">                                                    <td class=\"ko-grid-column-header\"><b>File Name</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Etag</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Type</b></td>                                                    <td class=\"ko-grid-column-header\"><b>Exception</b></td>                                                </tr>                                            </thead>                                            <tbody data-bind=\"foreach: items\">                                                <tr class=\"database-row\">                                                    <td data-bind=\"text: FileName\"></td>                                                    <td data-bind=\"text: FileEtag\"></td>                                                    <td data-bind=\"text: TypeDescription\"></td>                                                    <td>                                                        <span data-bind=\"visible: AdditionalInfo() != null\"><a href=\"#\" data-bind=\"click: function() { $root.showException(AdditionalInfo()); }\">Details</a></span>                                                    </td>                                                </tr>                                            </tbody>                                        </table>                                    </div>                                    <div>                                        <ul class=\"pagination pull-right\" data-bind=\"foreach: allPages, visible: allPages().length > 1\">                                            <li data-bind=\"css: { active: pageNumber === ($parent.pageIndex() + 1) }\"><a href=\"#\" data-bind=\"text: pageNumber, click: function() { $parent.moveToPage(pageNumber-1); }\"></a></li>                                        </ul>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div>    <div class=\"row\">        <div class=\"col-md-12\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <h4 class=\"panel-title\">                        <span>Synchronization sources</span>                    </h4>                </div>                <div class=\"panel-body\">                    <div>                        <ul class=\"nav nav-tabs\" data-bind=\"foreach: sourceSynchronizationInformations\">                            <li data-bind=\"css: { active:  $index() == 0 }\">                                <a data-toggle=\"tab\" data-bind=\"text: SourceServerUrl, attr: { href: \'#source-tabs-\' + $index() }\"></a>                            </li>                        </ul>                        <div class=\"tab-content\" data-bind=\"foreach: sourceSynchronizationInformations\">                            <div class=\"tab-pane fade in\" data-bind=\"attr: { id: \'source-tabs-\' + $index() }, css: { active:  $index() == 0 }\">                                <form class=\"form-horizontal col-md-12\" role=\"form\">                                    <div class=\"form-group\">                                        <label class=\"col-md-2 control-label\">Etag of last synchronized file: </label>                                        <div class=\"col-md-10\">                                            <p class=\"form-control-static\" data-bind=\"text: LastSourceFileEtag\"></p>                                        </div>                                    </div>                                </form>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/filesystem/synchronization/resolveConflict.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: resolve\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\" data-bind=\"text: title\"></h4>        </div>        <div class=\"modal-body\">            <span data-bind=\"html: message\"></span>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-warning\">Resolve</button>        </div>    </form></div>'; } )
define("text!views/filesystem/synchronization/synchronization.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Synchronization</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">    </li></ul><div>    <div class=\"settings-container row\">        <div class=\"col-md-2\">            <ul class=\"nav nav-list nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">                <li data-bind=\"css: { active: isActive }\">                    <a data-bind=\"attr: { href: hash }, text: title\"></a>                </li>            </ul>        </div>        <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">        </div>    </div></div>'; } )
define("text!views/filesystem/synchronization/synchronizationConfiguration.html", [], function () { return '<form data-bind=\"submit: $root.saveChanges.bind($root)\" id=\"save-periodic-backup-form\" class=\"form-horizontal\" role=\"form\">    <div class=\"btn-toolbar\">        <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: $root.isSaveEnabled()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>    </div>    <br />    <div class=\"form-group\">        <label for=\"fileConflictResolution\" class=\"col-sm-3 control-label\">File conflict resolution:</label>        <div class=\"col-sm-3\">            <div class=\"btn-group\" data-bind=\"with: config\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    <span data-bind=\"visible: fileConflictResolution() === \'None\'\">None</span>                    <span data-bind=\"visible: fileConflictResolution() === \'ResolveToLocal\'\">Resolve to local</span>                    <span data-bind=\"visible: fileConflictResolution() === \'ResolveToRemote\'\">Resolve to remote</span>                    <span data-bind=\"visible: fileConflictResolution() === \'ResolveToLatest\'\">Resolve to latest</span>                    <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li><a href=\"#\" data-bind=\"click: fileConflictResolution.bind(fileConflictResolution, \'None\')\">None</a></li>                    <li><a href=\"#\" data-bind=\"click: fileConflictResolution.bind(fileConflictResolution, \'ResolveToLocal\')\">Resolve to local</a></li>                    <li><a href=\"#\" data-bind=\"click: fileConflictResolution.bind(fileConflictResolution, \'ResolveToRemote\')\">Resolve to remote</a></li>                    <li><a href=\"#\" data-bind=\"click: fileConflictResolution.bind(fileConflictResolution, \'ResolveToLatest\')\">Resolve to latest</a></li>                </ul>            </div>        </div>    </div>    <div class=\"form-group\">        <label for=\"maxNumberOfSynchronizationsPerDestination\" class=\"col-sm-3 control-label\">Max number of synchronizations per destination:</label>        <div class=\"col-sm-3\">            <input type=\"number\" id=\"maxNumberOfSynchronizationsPerDestination\" required class=\"form-control\" data-bind=\"value: config().maxNumberOfSynchronizationsPerDestination, valueUpdate: \'afterkeydown\'\" />        </div>    </div>    <div class=\"form-group\" data-bind=\"with: config\">        <label for=\"lockInterval\" class=\"col-sm-3 control-label\">Synchronization lock timeout:</label>        <div class=\"col-sm-1\">            <input id=\"lockInterval\" required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: synchronizationLockTimeout, valueUpdate: \'afterkeydown\'\" />        </div>        <div class=\"col-sm-2\">            <select class=\"form-control\" data-bind=\"options: availableIntervalUnits, value: synchronizationLockTimeoutUnit\"></select>        </div>    </div></form>'; } )
define("text!views/filesystem/synchronization/synchronizationConflicts.html", [], function () { return '<div class=\"synchronization-group panel panel-default\" >    <div class=\"panel-heading\">        <h4 class=\"panel-title\">            <a class=\"pull-right text-height-fix\" data-toggle=\"collapse\" data-bind=\"attr: { href: \'#conflictsCollapse\' }\">                <span>Conflicts</span>                <span class=\"text-muted\" data-bind=\"text: \'(\' + conflicts().length + \')\'\"></span>            </a>            <span class=\"btn-group\">                <div class=\"checkbox\">                    <input id=\"files_all\" class=\"styled\" type=\"checkbox\" data-bind=\"value: isSelectAll, event: { change: toggleSelectAll }\" title=\"Select all or none\">                    <label for=\"files_all\">&nbsp;</label>                </div>            </span>            <span class=\"btn-group\">                <button class=\"btn btn-primary\" data-bind=\"click: resolveWithLocalVersion\">Resolve with Local</button>                <button class=\"btn btn-primary\" data-bind=\"click: resolveWithRemoteVersion\">Resolve with Remote</button>            </span>        </h4>        <div class=\"clearfix\"></div>    </div>    <div class=\"synchronization-group-content panel-collapse in\" data-bind=\"attr: { id: \'conflictsCollapse\' }\">        <div class=\"panel-body\">            <div class=\"alert alert-warning\" data-bind=\"visible: conflicts().length === 0\">                You\'ve got no conflicts.            </div>            <table class=\"table-no-borders\">                <tbody data-bind=\"foreach: conflicts\">                    <tr>                        <td style=\"padding: 0\">                            <h5 class=\"pull-left\">                                <a>                                    <input type=\"checkbox\" data-bind=\"attr: { value: fileName }, checked: $root.selectedConflicts\" />                                    <span data-bind=\"text: fileName\"></span>                                </a>                            </h5>                        </td>                    </tr>                </tbody>            </table>        </div>    </div></div>'; } )
define("text!views/filesystem/synchronization/synchronizationDestinations.html", [], function () { return '<form data-bind=\"submit: $root.saveChanges\" class=\" form-horizontal\" role=\"form\">    <div class=\"panel panel-default\">        <div class=\"panel-heading\">            <div class=\"pull-left panel-title\">Synchronization Destinations</div>            <div class=\"pull-right\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewDestination\" title=\"Add a replication destination (Alt+N)\" accesskey=\"N\">                    <i class=\"fa fa-plus\"></i> Add destination                </button>                <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save synchronization destinations (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            </div>            <div class=\"clearfix\"></div>        </div>        <div class=\"panel-body\" data-bind=\"with: replicationsSetup\">            <div class=\"alert alert-warning\" data-bind=\"visible: destinations().length === 0\">                You\'ve got no synchronization destinations set up.            </div>            <div data-bind=\"foreach: destinations\">                <div class=\"panel replication-destination-card\" data-bind=\"css: { \'panel-danger\': !isValid(), \'panel-info\': isValid() && !disabled(), \'panel-warning\': isValid() && disabled() }\">                    <div class=\"panel-heading\">                        <h3 class=\"panel-title\">                            <span data-bind=\"text: name\"></span>                            <button data-bind=\"click: $root.removeDestination.bind($root)\" title=\"Remove this replication destination\" type=\"button\" class=\"close\" aria-hidden=\"true\"><i class=\"fa fa-times\"></i></button>                        </h3>                    </div>                    <div class=\"panel-body\">                        <div class=\"form-group\">                            <label class=\"col-sm-4 control-label\">Disabled</label>                            <div class=\"col-sm-7\">                                <div class=\"btn-group\" data-toggle=\"buttons\">                                    <label class=\"btn btn-primary\" data-bind=\"click: enable, css: { active: !disabled() }\">                                        <input type=\"radio\" name=\"options\"> Enabled                                    </label>                                    <label class=\"btn btn-primary\" data-bind=\"click: disable, css: { active: disabled }\">                                        <input type=\"radio\" name=\"options\"> Disabled                                    </label>                                </div>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-4 control-label\">Url</label>                            <div class=\"col-sm-7\">                                <input required type=\"url\" class=\"form-control\" data-bind=\"value: url, valueUpdate: \'afterkeydown\'\" />                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-4 control-label\">File System</label>                            <div class=\"col-sm-7\">                                <input required type=\"text\" class=\"form-control\" data-bind=\"value: filesystem, valueUpdate: \'afterkeydown\'\" />                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-4 control-label\">Credentials</label>                            <div class=\"col-sm-7\">                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleUserCredentials, css: {active:isUserCredentials}\">User</button>                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleApiKeyCredentials, css: {active:isApiKeyCredentials}\">ApiKey</button>                            </div>                        </div>                        <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">User Name</label>                                <div class=\"col-sm-7\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Password</label>                                <div class=\"col-sm-7\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Domain</label>                                <div class=\"col-sm-7\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: domain, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                        </div>                        <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">API Key</label>                                <div class=\"col-sm-7\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, valueUpdate: \'afterkeydown\'\" />                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/filesystem/tasks/exportFilesystem.html", [], function () { return '<div class=\"panel panel-default\">        <div class=\"panel-heading\">        <h3 class=\"panel-title\">Export File System</h3>    </div>    <div class=\"panel-body\">        <h4 class=\"pull-left\">Export your file system to a .ravendump file.</h4>        <br />        <br />        <div class=\"btn-toolbar\">            <div>                <form class=\"navbar-form navbar-left\" data-bind=\"submit: startExport\">                    <button class=\"btn btn-primary\" type=\"submit\" download>                        <i class=\"fa fa-download\"></i>                        Export File System                    </button>                    <div class=\"checkbox\" data-bind=\"visible: !chooseDifferntFileName()\">                        <input type=\"checkbox\" class=\"styled\" id=\"different_name\" data-bind=\"checked: chooseDifferntFileName\">                        <label for=\"different_name\">Override file name</label>                    </div>                    <div data-bind=\"visible: chooseDifferntFileName\" class=\"form-group\">                        <input class=\"form-control\" type=\"text\" name=\"fileName\" data-bind=\"visible: chooseDifferntFileName, value: noneDefualtFileName\" title=\"Insert a file name to ovveride the default one.\" placeholder=\"Exported File Name\">                    </div>                </form>                                <form id=\"fsExportDownloadForm\" method=\"post\" data-bind=\"attr:{action:exportActionUrl}\" target=\"about:blank\">                    <input type=\"hidden\" id=\"SmugglerOptions\" name=\"SmugglerOptions\" />                </form>            </div>        </div>        <br />        <div>            <form class=\"form-horizontal\" role=\"form\">                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Batch size</label>                    <div class=\"col-sm-2\">                        <input class=\"form-control\" type=\"number\" min=\"0\" data-bind=\"value: batchSize\" />                    </div>                </div>            </form>        </div>    </div></div>'; } )
define("text!views/filesystem/tasks/importFilesystem.html", [], function () { return '<div class=\"panel panel-default\">    <div class=\"panel-heading\">        <h3 class=\"panel-title\">Import File System</h3>    </div>    <div class=\"panel-body\">        <h4>Import data from a .ravendump file into the current file system.</h4>        <p class=\"help-block\">            <i class=\"fa fa-warning text-warning\"></i>            Importing will overwrite any existing data.        </p>        <div data-bind=\"with: activeFilesystem\">            <span class=\"btn btn-primary btn-file\" data-bind=\"disable: isImporting\">                Choose file <input id=\"importFilesystemFilePicker\" type=\"file\" data-bind=\"event: { change: $root.fileSelected.bind($root, $element.value) }\"/>            </span>            <button class=\"btn btn-primary\" data-bind=\"click: $root.importFs.bind($root), disable: !$root.hasFileSelected() || isImporting()\">                <i class=\"fa fa-upload\"></i>                Import File System            </button>            <p data-bind=\"visible: $root.hasFileSelected()\">                Selected file name: <span data-bind=\"text: $root.importedFileName()\"></span>            </p>            <br />             <p data-bind=\"text: importStatus, visible: $root.isUploading || isImporting() || !!importStatus()\"></p>            <br />            <div class=\"progress progress-striped active\" data-bind=\"visible: isImporting\">                <div class=\"progress-bar\" role=\"progressbar\" style=\"width: 100%\"></div>            </div>        </div>        <br />        <div>            <form class=\"form-horizontal\" role=\"form\">                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Batch size</label>                    <div class=\"col-sm-2\">                        <input class=\"form-control\" type=\"number\" min=\"0\" data-bind=\"value: batchSize\"/>                    </div>                </div>            </form>            <div class=\"form-group\">                <div class=\"col-sm-offset-2 col-sm-5\">                    <div class=\"checkbox\">                        <input type=\"checkbox\" class=\"styled\" id=\"strip_replicaton_info_fs\" data-bind=\"checked: stripReplicationInformation\">                        <label for=\"strip_replicaton_info_fs\">Strip synchronization information from files metadata</label>                    </div>                    <div class=\"checkbox\">                        <input type=\"checkbox\" class=\"styled\" id=\"disable_versioning_fs\" data-bind=\"checked: shouldDisableVersioningBundle\">                        <label for=\"disable_versioning_fs\">Disable versioning bundle during import</label>                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/filesystem/tasks/tasks.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                File Systems            </a>        </h3>    </li>    <li data-bind=\"with: activeFilesystem\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.filesystemFiles }\"></a>    </li>    <li class=\"active\">        <span>Tasks</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">        <a data-bind=\"attr: { href: hash }, text: title\"></a>    </li></ul><div>    <div class=\"row\">        <div class=\"col-md-2\">            <ul class=\"nav nav-pills nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">                <li data-bind=\"css: { active: isActive }\">                    <a data-bind=\"attr: { href: hash }, text: title\"></a>                </li>            </ul>        </div>        <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">                    </div>    </div></div>'; } )
define("text!views/manage/adminLogs.html", [], function () { return '<div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><section id=\"logsContainer\" style=\"width: 100%; position: relative\" data-bind=\"visible: !isForbidden()\">    <div class=\"panel panel-default panel-wo-margin\">        <div class=\"panel-heading panel-info\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: configureConnection\">Configure Connection</button>                    <button type=\"button\" class=\"btn btn-success\" data-bind=\"click: reconnect\">Reconnect</button>                    <button type=\"button\" class=\"btn btn-warning\" data-bind=\"click: disconnect\">Disconnect</button>                </div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">Keep Down</button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: clearLogs\">Clear</button>                </div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: exportLogs\">Export</button>                </div>            </div>        </div>        <div class=\"panel-body\">            <div class=\"text-center\" data-bind=\"visible: !adminLogsConfig()\">                <h3 class=\"text-info\">                    Click on \'Configure Connection\' to start.                </h3>            </div>            <div id=\"rawLogsContainer\" class=\"bs-code-output skip-header\" data-bind=\"visible: adminLogsConfig, if: adminLogsConfig\">                <pre id=\"adminLogsPre\" style=\"overflow: auto\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#logsContainer\'  }\"></pre>                <div class=\"header small col-sm-11\">                    <strong>Status: </strong><label data-bind=\"text: ($root.connected() == true ? \'Connected\' : \'Disconnected\'), css: { \'text-danger\': $root.connected() == false, \'text-success\': $root.connected() == true }\"></label>                    <strong>; Entries: <label data-bind=\"text: $root.entriesCount()\"></label></strong>                    <strong>; Buffer Size: </strong><label data-bind=\"text: $root.adminLogsConfig().maxEntries\"></label>                </div>            </div>                   </div>    </div></section>'; } )
define("text!views/manage/adminLogsConfigureDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 715px;\">    <form data-bind=\"submit: startServerLogging\" id=\"log-config-form\" onkeypress=\"return event.keyCode != 13;\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Custom logging</h4>        </div>                <div class=\"modal-body\">                        <div class=\"row\">                <div class=\"col-md-12\">                    <div class=\"form-group\">                        <label for=\"maxEntries\" class=\"col-sm-4 control-label\">Max log entries</label>                        <div class=\"col-sm-8\">                          <input type=\"number\" data-bind=\"value: logConfig.maxEntries\" class=\"form-control\" id=\"maxEntries\" required max=\"30000\" min=\"1\">                        </div>                    </div>                </div>            </div>            <table class=\"table table-hover\" data-bind=\"css: { \'custom-table\': isScrollNeeded }\">                <thead>                    <tr>                        <th class=\"col-md-4\">Category</th>                        <th class=\"col-md-4\">Level</th>                        <th class=\"col-md-4\">Stack traces?</th>                        <th class=\"col-md-2\">Options</th>                    </tr>                </thead>                <tbody data-bind=\"foreach: logConfig.entries, css: { \'custom-table-tbody\': isScrollNeeded }\">                    <tr>                        <td class=\"col-md-4\"><input class=\"form-control\" required type=\"text\" data-bind=\"value: category\" /></td>                        <td class=\"col-md-4\">                            <select class=\"form-control\" required data-bind=\"value: level\">                                <option value=\"Debug\">Debug</option>                                <option value=\"Info\">Info</option>                                <option value=\"Warn\">Warning</option>                                <option value=\"Error\">Error</option>                                <option value=\"Fatal\">Fatal</option>                            </select>                        </td>                        <td class=\"col-md-4\">                            <div class=\"checkbox\">                                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: includeStackTrace, attr: { id : \'stack_\' + $index() }\">                                <label data-bind=\"attr: { for: \'stack_\' + $index() }\">&nbsp;</label>                            </div>                        </td>                        <td class=\"col-md-2\">                            <div class=\"btn-group\">                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.deleteRow.bind($root, $data)\"><i class=\"fa fa-trash-o\"></i></button>                            </div>                        </td>                    </tr>                </tbody>            </table>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" class=\"btn btn-primary\">Ok</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"button\" class=\"btn pull-left btn-default\" data-bind=\"click: addCategory\">                <i class=\"fa fa-plus\"></i> Add new category            </button>        </div>    </form></div>'; } )
define("text!views/manage/adminSettings.html", [], function () { return '<br/><div class=\"settings-container row admin-settings-container\">    <div class=\"col-md-2\">        <ul class=\"nav nav-pills nav-stacked\">            <!-- ko foreach: router.navigationModel -->            <li data-bind=\"css: { active: isActive(), disabled: !enabled }\">                <a data-bind=\"attr: { href: hash }, text: title\"></a>            </li>            <!-- /ko -->            <li>                <a href=\"#\" data-bind=\"click: navigateToSystemDatabase, visible: isSystemDatabaseForbidden() == false\">To System Database</a>            </li>        </ul>    </div>    <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/manage/apiKeys.html", [], function () { return '<form class=\"form-horizontal\" role=\"form\" id=\"manageApiKeys\" data-bind=\"submit: saveChanges\">    <div data-bind=\"visible: settingsAccess.isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !settingsAccess.isForbidden()\">        <div class=\"panel-heading\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <div class=\"btn-group\">                <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            </div>            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewApiKey, disable: settingsAccess.isReadOnly\" title=\"Create a new API Key (Alt+N)\" accesskey=\"N\">                    <i class=\"fa fa-plus\"></i> Create a new API Key                </button>            </div>            <div class=\"btn-group\">                <div class=\"input-group\" data-bind=\"visible: apiKeys().length > 0\">                    <span class=\"input-group-addon\"><i class=\"fa fa-search\"></i></span>                    <input type=\"text\" id=\"apiKeysSearchInput\" class=\"form-control\" placeholder=\"Search for API Keys\" data-bind=\"value: searchText, valueUpdate: \'afterkeydown\'\"/>                </div>            </div>        </div>        <div class=\"panel-body\">            <div class=\"text-center\" data-bind=\"visible: apiKeys().length == 0\">                <h3 class=\"text-info\">You don\'t have any API keys. <small>Go ahead and <a href=\"#\" data-bind=\"click: createNewApiKey\">create one now</a>.</small>                </h3>            </div>            <div class=text-center data-bind=\"visible: noResults\">                <h3 class=\"text-info\">No results found</h3>            </div>            <div data-bind=\"foreach: apiKeys\">                <div class=\"panel api-key-card\" data-bind=\"visible: visible, css: { \'panel-info\': enabled() && isValid($index), \'panel-warning\': !enabled() && isValid($index), \'panel-danger\': !isValid($index) }\">                    <div class=\"panel-heading\">                        <h3 class=\"panel-title\">                            <span data-bind=\"text: name() ? name() : \'[new api key]\'\"></span>                            <button type=\"button\" style=\"display: inline-block;\" class=\"close\" data-bind=\"click: $root.removeApiKey.bind($root), visible: $root.settingsAccess.isReadOnly() == false\" title=\"Remove this API key\">                                <i class=\"fa fa-times\"></i>                            </button>                        </h3>                    </div>                    <div class=\"panel-body\">                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Name</label>                            <div class=\"col-sm-8\">                                <input type=\"text\" class=\"form-control\" required placeholder=\"API Key name\" data-bind=\"value: name, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidity\"/>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Secret</label>                            <div class=\"col-sm-8\">                                <input type=\"text\" class=\"form-control\" readonly data-bind=\"value: secret\" onmouseup=\"this.select(); return false;\"/>                                <a href=\"javascript:void(0)\" data-bind=\"click: generateSecret, visible: !$root.settingsAccess.isReadOnly()\">Generate</a>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Full API Key</label>                            <div class=\"col-sm-8\">                                <input type=\"text\" class=\"form-control\" readonly data-bind=\"value: fullApiKey\" onmouseup=\"this.select(); return false;\">                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Connection String</label>                            <div class=\"col-sm-8\">                                <input type=\"text\" class=\"form-control\" readonly data-bind=\"value: connectionString\" onmouseup=\"this.select(); return false;\"/>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Direct Link</label>                            <div class=\"col-sm-8\">                                <input type=\"text\" class=\"form-control\" readonly data-bind=\"value: directLink\" onmouseup=\"this.select(); return false;\"/>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Enabled</label>                            <div class=\"col-sm-8\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                        <span data-bind=\"visible: enabled\">Enabled</span>                                        <span data-bind=\"visible: !enabled()\">Disabled</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                        <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                                    </ul>                                </div>                            </div>                        </div>                        <div class=\"form-group\">                            <label class=\"col-sm-3 control-label\">Resources</label>                            <div class=\"col-sm-9\">                                                                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addEmptyDatabase, visible: databases().length == 0\" title=\"Add Resource\">                                    <i class=\"fa fa-plus\"></i> Add Resource                                </button>                                <table class=\"table table-striped table-hover\">                                    <thead>                                    <tr class=\"database-header\" data-bind=\"visible: databases().length > 0\">                                        <td>Name</td>                                        <td>Type</td>                                        <td>                                            <button type=\"button\" style=\"display: inline-block; font-size: 14px;\" class=\"close\" title=\"Add resource\" data-bind=\"click: addEmptyDatabase, visible: !$root.settingsAccess.isReadOnly()\">                                                <i class=\"fa fa-plus\"></i>                                            </button>                                        </td>                                    </tr>                                    </thead>                                    <tbody data-bind=\"foreach: databases\">                                    <tr>                                        <td>                                            <div class=\"btn-group\">                                                <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"databaseName\" class=\"form-control\" data-bind=\"value: tenantId, valueUpdate: \'afterkeydown\', customValidity: tenantCustomValidityError, attr: { \'id\': \'databaseName\' + $parentContext.$index() + $index() }\" placeholder=\"Resource name\">                                                <ul class=\"dropdown-menu scrollable-resource-menu\" style=\"display: none; overflow-x: hidden\" data-bind=\"autoComplete: \'#databaseName\' + $parentContext.$index() + $index(), foreach: searchResults\">                                                    <li role=\"presentation\" data-bind=\"click: $parent.tenantId.bind($data)\">                                                        <a role=\"menuitem\" tabindex=\"-1\" href=\"#\" class=\"nounderline\" data-bind=\"attr : { \'title\': $data }\">                                                            <span data-bind=\"text: $data\"></span>                                                        </a>                                                    </li>                                                </ul>                                            </div>                                        </td>                                        <td>                                            <div class=\"btn-group\">                                                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                    <span data-bind=\"text: currentAccessType\"></span> <span class=\"caret\"></span>                                                </button>                                                <ul class=\"dropdown-menu\" role=\"menu\">                                                    <!-- ko foreach: getTypes() -->                                                    <li><a href=\"#\" class=\"nounderline\" data-bind=\"text: $data, click: $parent.currentAccessType.bind($parent)\"></a></li>                                                    <!-- /ko -->                                                </ul>                                            </div>                                        </td>                                        <td>                                            <button type=\"button\" style=\"display: inline-block; float: none; vertical-align: middle; font-size: 14px;\" class=\"close\" data-bind=\"visible: $parent.databases().length > 1, click: $parent.removeDatabase.bind($parent, $data), visible: !$root.settingsAccess.isReadOnly()\" title=\"Remove this resource\">                                                <i class=\"fa fa-times\"></i>                                            </button>                                        </td>                                    </tr>                                    </tbody>                                </table>                                <div style=\"color: red\" data-bind=\"visible: needToShowSystemDatabaseWarning\">                                    * This API Key provides access to all resources but not to the System Database                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/backup.html", [], function () { return '<div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><div data-bind=\"visible: isForbidden() == false\">    <div role=\"tabpanel\">        <ul class=\"nav nav-tabs\" role=\"tablist\">            <li role=\"presentation\" class=\"active\"><a href=\"#backupDb\" aria-controls=\"backupDb\" role=\"tab\" data-toggle=\"tab\">Database</a></li>            <li role=\"presentation\"><a href=\"#backupFs\" aria-controls=\"backupFs\" role=\"tab\" data-toggle=\"tab\">File System</a></li>            <li role=\"presentation\" data-bind=\"visible: $root.has40Features\"><a href=\"#backupCs\" aria-controls=\"backupCs\" role=\"tab\" data-toggle=\"tab\">Counter Storage</a></li>        </ul>        <div class=\"tab-content\">            <div role=\"tabpanel\" class=\"tab-pane active\" id=\"backupDb\">                <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startDbBackup\">                    <div class=\"panel panel-default\" data-bind=\"with: dbBackupOptions\">                        <div class=\"panel-heading\">                            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: isBusy\">                                <i class=\"fa fa-cloud-upload fa-fw\" data-bind=\"visible: !isBusy()\"></i>                                <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: isBusy\"></i>                                <span>Start Database Backup</span>                            </button>                            <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label for=\"databaseName\" class=\"col-sm-2 control-label\">Database to backup</label>                                <div class=\"col-sm-5\">                                    <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"databaseName\" class=\"form-control\" id=\"databaseName\" data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"Database name to backup\">                                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#databaseName\', foreach: searchResults\">                                        <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                                <span data-bind=\"text: $data\"></span>                                            </a>                                        </li>                                    </ul>                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"location\" class=\"col-sm-2 control-label\">Location</label>                                <div class=\"col-sm-5\">                                    <input type=\"text\" required class=\"form-control\" id=\"location\" data-bind=\"value: backupLocation, valueUpdate: \'afterkeydown\'\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: displaySameDriveWarning\">                                <div class=\"col-sm-5 col-sm-offset-2 text-warning\">                                    <i class=\"fa fa-warning\"></i> Backup to database drive is not suggested.                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" data-bind=\"checked: incremental\" class=\"styled\" id=\"incrementalDb\">                                        <label for=\"incrementalDb\">Incremental</label>                                    </div>                                </div>                            </div>                            <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: backupStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                                <pre><!-- ko foreach: backupStatusMessages --><div data-bind=\"css: { \'text-danger\': Severity === \'Error\' }, text: \'[\' + Timestamp + \']\t\' + Severity + \':\t\' + Message\"></div><!-- /ko --></pre>                            </div>                        </div>                    </div>                </form>            </div>            <div role=\"tabpanel\" class=\"tab-pane\" id=\"backupFs\">                <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startFsBackup\">                    <div class=\"panel panel-default\" data-bind=\"with: fsBackupOptions\">                        <div class=\"panel-heading\">                            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: isBusy\">                                <i class=\"fa fa-cloud-upload fa-fw\" data-bind=\"visible: !isBusy()\"></i>                                <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: isBusy\"></i>                                <span>Start File System Backup</span>                            </button>                            <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label for=\"filesystemName\" class=\"col-sm-2 control-label\">File System to backup</label>                                <div class=\"col-sm-5\">                                    <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"filesystemName\" class=\"form-control\" id=\"filesystemName\" data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"Filesystem name to backup\">                                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#filesystemName\', foreach: searchResults\">                                        <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                                <span data-bind=\"text: $data\"></span>                                            </a>                                        </li>                                    </ul>                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"location\" class=\"col-sm-2 control-label\">Location</label>                                <div class=\"col-sm-5\">                                    <input type=\"text\" required class=\"form-control\" id=\"Text1\" data-bind=\"value: backupLocation, valueUpdate: \'afterkeydown\'\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: displaySameDriveWarning\">                                <div class=\"col-sm-5 col-sm-offset-2 text-warning\">                                    <i class=\"fa fa-warning\"></i> Backup to filesystem drive is not suggested.                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" data-bind=\"checked: incremental\" class=\"styled\"  id=\"incrementalFs\">                                        <label for=\"incrementalFs\">Incremental</label>                                    </div>                                </div>                            </div>                            <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: backupStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                                <pre><!-- ko foreach: backupStatusMessages --><div data-bind=\"css: { \'text-danger\': Severity === \'Error\' }, text: \'[\' + Timestamp + \']\t\' + Severity + \':\t\' + Message\"></div><!-- /ko --></pre>                            </div>                        </div>                    </div>                </form>            </div>            <div role=\"tabpanel\" class=\"tab-pane\" id=\"backupCs\">                <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startCsBackup\">                    <div class=\"panel panel-default\" data-bind=\"with: csBackupOptions\">                        <div class=\"panel-heading\">                            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: isBusy\">                                <i class=\"fa fa-cloud-upload fa-fw\" data-bind=\"visible: !isBusy()\"></i>                                <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: isBusy\"></i>                                <span>Start Counter Storage Backup</span>                            </button>                            <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label for=\"countersstoragename\" class=\"col-sm-2 control-label\">Counter Storage to backup</label>                                <div class=\"col-sm-5\">                                    <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"countersstoragename\" class=\"form-control\" id=\"countersstoragename\" data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"Filesystem name to backup\">                                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#countersstoragename\', foreach: searchResults\">                                        <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                                <span data-bind=\"text: $data\"></span>                                            </a>                                        </li>                                    </ul>                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"location\" class=\"col-sm-2 control-label\">Location</label>                                <div class=\"col-sm-5\">                                    <input type=\"text\" required class=\"form-control\" data-bind=\"value: backupLocation, valueUpdate: \'afterkeydown\'\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: displaySameDriveWarning\">                                <div class=\"col-sm-5 col-sm-offset-2 text-warning\">                                    <i class=\"fa fa-warning\"></i> Backup to counter storage drive is not suggested.                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" data-bind=\"checked: incremental\" class=\"styled\" id=\"incrementalCs\">                                        <label for=\"incrementalCs\">Incremental</label>                                    </div>                                </div>                            </div>                            <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: backupStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                                <pre><!-- ko foreach: backupStatusMessages --><div data-bind=\"css: { \'text-danger\': Severity === \'Error\' }, text: \'[\' + Timestamp + \']\t\' + Severity + \':\t\' + Message\"></div><!-- /ko --></pre>                            </div>                        </div>                    </div>                </form>            </div>        </div>    </div></div>'; } )
define("text!views/manage/cluster.html", [], function () { return '<section>    <div data-bind=\"visible: settingsAccess.isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div data-bind=\"if: !settingsAccess.isForbidden()\">        <div data-bind=\"if: !clusterMode()\">            <div class=\"panel panel-default\" data-bind=\"visible: $root.canCreateCluster()\">                <div class=\"panel-heading\">                    <div class=\"pull-right text-warning\" data-bind=\"visible: developerLicense\">Enterprise feature</div>                    Cluster configuration                </div>                <div class=\"panel-body\">                    <div class=\"text-center\">                        <h3 class=\"text-info\">                            This server is not part of cluster. <small data-bind=\"visible: !settingsAccess.isReadOnly()\">Go ahead and <a href=\"#\" data-bind=\"click: createCluster\">create cluster</a>.</small>                        </h3>                    </div>                                    </div>            </div>            <div class=\"panel panel-default\" data-bind=\"visible: !$root.canCreateCluster()\">                <div class=\"panel-heading\">                    Cluster configuration                </div>                <div class=\"panel-body\">                    <div class=\"text-center\">                        <h3 class=\"text-info\">                            Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                        </h3>                    </div>                </div>            </div>        </div>        <div data-bind=\"if: clusterMode()\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <button data-bind=\"click: addAnotherServerToCluster.bind($root, false), disable: settingsAccess.isReadOnly()\" class=\"btn btn-default\"><i class=\"fa fa-plus\"></i> Add another server to cluster</button>                    <button class=\"btn btn-default\" data-bind=\"autoRefresh: { duration: 5000, onRefresh: $root.refresh.bind($root), active:false }\"></button>                    <div class=\"btn-group\">                        <button id=\"operationType\" type=\"button\" class=\"btn btn-danger dropdown-toggle\" data-bind=\"visible: !settingsAccess.isReadOnly()\" data-toggle=\"dropdown\">                            <span>Emergency cluster operations</span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li><a href=\"#\" data-bind=\"click: initializeNewCluster\">Cecede from cluster - rude step down into a single node cluster</a></li>                            <li><a href=\"#\" data-bind=\"click: addAnotherServerToCluster.bind($root, true)\">Kidnap node into cluster - force a node to the current cluster</a></li>                            <li><a href=\"#\" data-bind=\"click: removeClustering.bind($root)\">Remove cluster entirely and perform cleanup.</a></li>                        </ul>                    </div>                </div>                <div class=\"panel-body\">                    <table class=\"table table-condensed table-striped\">                        <thead>                            <tr>                                <th>State</th>                                <th>Status</th>                                <th>Server Url</th>                                <th data-bind=\"visible: !$root.settingsAccess.isReadOnly()\">Edit</th>                                <th data-bind=\"visible: !$root.settingsAccess.isReadOnly()\">Delete</th>                            </tr>                        </thead>                        <tbody data-bind=\"foreach: topology().allNodes\">                            <tr>                                <td>                                    <label data-bind=\"text: state\"></label>                                </td>                                <td>                                    <div data-bind=\"visible: status() === \'Online\'\" class=\" text-success\"><span class=\"fa fa-check\"></span> Online</div>                                    <div data-bind=\"visible: status() === \'Offline\'\" class=\" text-danger\"><span class=\"fa fa-times\"></span> Offline</div>                                    <div data-bind=\"visible: status() === \'WrongCredentials\'\" class=\"text-danger\"><span class=\"fa fa-exclamation-circle\"></span> Wrong Credentials</div>                                    <div data-bind=\"visible: status() === \'Loading\'\"><span class=\"fa fa-spin fa-spinner\"></span> Loading...</div>                                </td>                                <td data-bind=\"text: uri\"></td>                                <td data-bind=\"visible: !$root.settingsAccess.isReadOnly()\">                                    <button data-bind=\"click: $root.editNode.bind($root)\" class=\"btn btn-default\"><i class=\"fa fa-edit\"></i> Edit</button>                                    <button data-bind=\"visible: state() === \'non-voting\', click: $root.promoteNodeToVoter.bind($root)\" class=\"btn btn-default\"><i class=\"fa fa-arrow-up\"></i> Promote</button>                                 </td>                                <td data-bind=\"visible: !$root.settingsAccess.isReadOnly()\"><button data-bind=\"click: $root.leaveCluster.bind($root)\" class=\"btn btn-danger\"><i class=\"fa fa-trash\"></i> Leave cluster</button></td>                            </tr>                        </tbody>                    </table>                </div>            </div>        </div>    </div>    </section>'; } )
define("text!views/manage/compact.html", [], function () { return '<div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><div role=\"tabpanel\" data-bind=\"visible: isForbidden() == false\">    <ul class=\"nav nav-tabs\" role=\"tablist\">        <li role=\"presentation\" class=\"active\"><a href=\"#compactDb\" aria-controls=\"compactDb\" role=\"tab\" data-toggle=\"tab\">Database</a></li>        <li role=\"presentation\"><a href=\"#compactFs\" aria-controls=\"compactFs\" role=\"tab\" data-toggle=\"tab\">File system</a></li>    </ul>    <div class=\"tab-content\">        <div role=\"tabpanel\" class=\"tab-pane active\" id=\"compactDb\">            <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startDbCompact\">                <div class=\"panel panel-default\" data-bind=\"with: dbCompactOptions\">                    <div class=\"panel-heading panel-info\">                        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                        <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: $parent.isBusy\">                            <i class=\"fa fa-hdd-o fa-fw\" data-bind=\"visible: !$parent.isBusy()\"></i>                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: $parent.isBusy\"></i>                            <span>Start Compaction</span>                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: $parent.isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                    </div>                    <div class=\"panel-body\">                        <div class=\"form-group\">                            <label for=\"databaseName\" class=\"col-sm-2 control-label\">Database to compact</label>                            <div class=\"col-sm-5\">                                <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"databaseName\" class=\"form-control\" id=\"databaseName\" data-bind=\"value: resourceName, disable: $parent.isBusy, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"Database name to compact\">                                <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#databaseName\', foreach: searchResults\">                                    <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                        <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                            <span data-bind=\"text: $data\"></span>                                        </a>                                    </li>                                </ul>                            </div>                        </div>                        <div class=\"form-group\">                            <div class=\"col-sm-9 col-sm-offset-2\">                                <i class=\"fa fa-warning text-warning\"></i> <span class=\"text-muted\">Compaction takes database offline.</span>                            </div>                        </div>                        <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: compactStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                            <pre><!-- ko foreach: compactStatusMessages --><div data-bind=\"text: $data\"></div><!-- /ko --><div data-bind=\"text: \'Last Progress Message: \' + compactStatusLastUpdate(), visible: $parent.isBusy()\"></div></pre>                        </div>                    </div>                </div>                            </form>        </div>        <div role=\"tabpanel\" class=\"tab-pane\" id=\"compactFs\">            <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startFsCompact\">                <div class=\"panel panel-default\" data-bind=\"with: fsCompactOptions\">                    <div class=\"panel-heading panel-info\">                        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                        <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: $parent.isBusy\">                            <i class=\"fa fa-hdd-o fa-fw\" data-bind=\"visible: !$parent.isBusy()\"></i>                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: $parent.isBusy\"></i>                            <span>Start Compaction</span>                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: $parent.isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                    </div>                    <div class=\"panel-body\">                        <div class=\"form-group\">                            <label for=\"filesystemName\" class=\"col-sm-2 control-label\">Filesystem to compact</label>                            <div class=\"col-sm-5\">                                <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"filesystemName\" class=\"form-control\" id=\"filesystemName\" data-bind=\"value: resourceName, disable: $parent.isBusy, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"File System name to compact\">                                <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#filesystemName\', foreach: searchResults\">                                    <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                        <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                            <span data-bind=\"text: $data\"></span>                                        </a>                                    </li>                                </ul>                            </div>                        </div>                        <div class=\"form-group\">                            <div class=\"col-sm-9 col-sm-offset-2\">                                <i class=\"fa fa-warning text-warning\"></i> <span class=\"text-muted\">Compaction takes file system offline.</span>                            </div>                        </div>                        <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: compactStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                            <pre><!-- ko foreach: compactStatusMessages --><div data-bind=\"text: $data\"></div><!-- /ko --><div data-bind=\"text: \'Last Progress Message: \' + compactStatusLastUpdate(), visible: $parent.isBusy()\"></div></pre>                        </div>                    </div>                </div>            </form>        </div>    </div></div>'; } )
define("text!views/manage/console.html", [], function () { return '<section>    <div data-bind=\"visible: !settingsAccess.canWrite()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: executeJs, visible: settingsAccess.canWrite()\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <div class=\"btn-group\">                    <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"disable: isBusy\">                        <i class=\"fa fa-gears\"></i>                        <span>Execute command</span>                    </button>                </div>            </div>            <div class=\"panel-body\">                <div class=\"alert alert-danger\"><i class=\"fa fa-exclamation-triangle\"></i> The execution of the admin script might crash you server, unless you know what you are doing.                </div>                <div class=\"form-group\">                    <label for=\"databaseName\" class=\"col-sm-2 control-label\">Target database</label>                    <div class=\"col-sm-3\">                        <input tabindex=\"2\" required type=\"text\" name=\"databaseName\" class=\"form-control\" id=\"databaseName\" data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" placeholder=\"Database name\">                        <ul class=\"dropdown-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#databaseName\', foreach: searchResults\">                            <li role=\"presentation\" data-bind=\"click: $parent.resourceName.bind($parent, $data)\">                                <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                    <span data-bind=\"text: $data\"></span>                                </a>                            </li>                        </ul>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"predefinedScripts\" class=\"col-sm-2 control-label\">Predefined scripts</label>                    <div class=\"col-sm-3\">                        <div class=\"col-sm-10 input-group\">                            <div class=\"btn-group\">                                <button id=\"operationType\" type=\"button\" data-bind=\"disable: $root.isBusy\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span>Choose</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: predefinedSamples\">                                    <li><a href=\"#\" data-bind=\"click: $root.fillWithSample.bind($root, $data), text: Name\"></a></li>                                </ul>                            </div>                        </div>                    </div>                </div>                <div class=\"row\">                    <label for=\"script\" class=\"col-sm-2 control-label\">Script</label>                    <div class=\"col-sm-9\">                        <pre class=\"form-control\" data-bind=\"aceEditor: { code: script }\" style=\"height: 160px;\"></pre>                    </div>                </div>                <div class=\"row\">                    <label for=\"results\" class=\"col-sm-2 control-label\">Results:</label>                    <div class=\"col-sm-9\">                        <pre class=\"form-control\" data-bind=\"aceEditor: { code: results, readOnly: true }\" style=\"height: 160px;\"></pre>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/manage/diskIoViewer.html", [], function () { return '<div data-bind=\"visible: !settingsAccess.canReadOrWrite()\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><div class=\"panel panel-default panel-wo-margin\" id=\"disk-io-viewer\" data-bind=\"visible: settingsAccess.canReadOrWrite\">    <div class=\"panel-heading panel-info\">        Disk IO Viewer    </div>    <div class=\"panel-body\">        <div class=\"text-center\" data-bind=\"visible: performanceRuns().length == 0\">            <h3 class=\"text-info\">There are no results to display. <small>Use Raven.Monitor.exe to generate Disk IO Performance report. </small></h3>        </div>        <div class=\"form-inline\">            <div class=\"form-group\" data-bind=\"visible: performanceRuns().length > 0\">                <label>Report:</label>                <div class=\"input-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"text: currentPeformanceRunLabel\">Select</span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: performanceRuns\">                        <li><a href=\"#\" data-bind=\"click: $root.switchReport.bind($root), text: displayName\"></a></li>                    </ul>                </div>                <button data-bind=\"click: deleteReport, visible: currentPerformanceRun()\" class=\"btn btn-danger\"><i class=\"fa fa-trash-o\"></i></button>            </div>            <div class=\"form-group padd-left-10\" data-bind=\"visible: perDbReports().length > 0\">                <label>Database:</label>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"text: currentDbReportLabel\"></span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: perDbReports\">                        <li><a href=\"#\" data-bind=\"click: $root.switchDatabase.bind($root), text: Name\"></a></li>                    </ul>                </div>            </div>        </div>        <div class=\"text-center\" data-bind=\"visible: emptyReport\">            <h3 class=\"text-info\">                <i class=\"fa fa-warning\"></i>  This report does not contain any data.            </h3>        </div>        <div data-bind=\"if: showChart()\" id=\"diskMonitorContainer\" class=\"with-3d-shadow with-transitions\">            <svg height=\"300\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#disk-io-viewer\' }\"></svg>        </div>    </div></div>'; } )
define("text!views/manage/editNodeConnectionInfoDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 600px\">    <form data-bind=\"submit: ok\" class=\"form-horizontal\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Define cluster node connection info</h4>        </div>        <div class=\"modal-body\" data-bind=\"with: nodeConnectionInfo\">            <div>                <p><i class=\"fa fa-info-circle\"></i> Please enter url and credentials (optionally). Url should be reachable from any other node within cluster.</p>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Url</label>                <div class=\"col-sm-7\">                    <input type=\"text\" class=\"form-control\" data-bind=\"value: uri, valueUpdate: \'afterkeydown\', enable: !$root.isUpdate()\"/>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Credentials</label>                <div class=\"col-sm-7\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"visible: !isUserCredentials() && !isApiKeyCredentials()\">None</span>                            <span data-bind=\"visible: isUserCredentials\">User</span>                            <span data-bind=\"visible: isApiKeyCredentials\">API key</span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li><a href=\"#\" data-bind=\"click: useNoCredentials\">None</a></li>                            <li class=\"divider\"></li>                            <li><a href=\"#\" data-bind=\"click: useUserCredentials\">User</a></li>                            <li><a href=\"#\" data-bind=\"click: useApiKeyCredentials\">API key</a></li>                        </ul>                    </div>                </div>            </div>            <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">User Name</label>                    <div class=\"col-sm-7\">                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, valueUpdate: \'afterkeydown\'\"/>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">Password</label>                    <div class=\"col-sm-7\">                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, valueUpdate: \'afterkeydown\'\"/>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">Domain</label>                    <div class=\"col-sm-7\">                        <input type=\"text\" class=\"form-control\" data-bind=\"value: domain, valueUpdate: \'afterkeydown\'\"/>                    </div>                </div>            </div>            <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                <div class=\"form-group\">                    <label class=\"col-sm-4 control-label\">API Key</label>                    <div class=\"col-sm-7\">                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, valueUpdate: \'afterkeydown\'\"/>                    </div>                </div>            </div>            <div class=\"form-group\">                <div class=\"col-sm-7 col-sm-offset-4\">                    <div class=\"checkbox\">                        <input type=\"checkbox\" id=\"votingNode\" class=\"form-control\" data-bind=\"checked: !isNonVoting()\" />                        <label for=\"votingNode\">Voting node</label>                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>            <button type=\"submit\" class=\"btn btn-primary\">Save Changes</button>        </div>    </form></div>'; } )
define("text!views/manage/hotSpare.html", [], function () { return '<div class=\"panel panel-default form-horizontal\" role=\"form\">    <div class=\"panel-heading\">        Hot Spare        <div class=\"clearfix\"></div>    </div>    <div class=\"panel-body\">        <form class=\"form-horizontal\" role=\"form\">            <div class=\"form-group\">                <label class=\"col-sm-3 control-label\">Activation Mode:</label>                <div class=\"col-sm-5 input-group text-height-fix\" data-bind=\"text: activationMode()\"></div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-3 control-label\">Activation Time:</label>                <div class=\"col-sm-5 input-group text-height-fix\" data-bind=\"text: activationTime()\"></div>            </div>            <div class=\"form-group\" data-bind=\"visible: isTestEnabled\">                <label class=\"col-sm-3 control-label\">Remaining Test Activation:</label>                <div class=\"col-sm-5 input-group text-height-fix\" data-bind=\"text: remainingTestActivation()\"></div>            </div>            <div class=\"form-group\" data-bind=\"visible: isTestEnabled\">                <label class=\"col-sm-3 control-label\">Test Hot Spare</label>                <button type=\"button\" data-bind=\"click: testLicense\" class=\"col-sm-2 btn btn-primary\">Test</button>            </div>            <div class=\"form-group\">                <label class=\"col-sm-3 control-label\">Activate Hot Spare</label>                <button type=\"button\" data-bind=\"click: activateLicense\" class=\"col-sm-2 btn btn-primary\">Activate</button>            </div>            <div class=\"form-group\" data-bind=\"visible: isActivationExpired\">                <a class=\"col-sm-offset-3\" href=\"http://ravendb.net/buy\">Activation expired please buy a new license</a>            </div>        </form>    </div></div>'; } )
define("text!views/manage/infoPackage.html", [], function () { return '<style type=\"text/css\">    .link {  fill: none;  stroke: #ccc;  stroke-width: 1.5px;}.overlay {  fill: none;  pointer-events: all;}text.trace {    font-size: 11px;}.box {    fill: #158CBA;    fill-opacity: 0.2;    stroke: #158CBA;    stroke-width: 1.5px;}.notUserCode {    fill-opacity: 0.4;}.headerLine {    stroke: #158CBA;    stroke-width: 1.5px;}</style><div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><section id=\"infoPackageSection\" data-bind=\"visible: !isForbidden()\">    <div class=\"panel panel-default panel-wo-margin\">        <div class=\"panel-heading\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <div class=\"btn-group\" data-bind=\"visible: adminView()\">                <button class=\"btn btn-default\" title=\"Creates package with stacktrace (available ONLY for system database)\" data-bind=\"click: createPackageWithStacks\">                    <span data-bind=\"visible: showLoadingIndicator\" class=\"fa fa-spinner fa-spin fa-fw\"></span>                    Create info package with stacktraces <span class=\"text-danger\">(This is resource consuming)</span>                </button>                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    <span class=\"caret\"></span>                    <span class=\"sr-only\">Toggle Dropdown</span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li>                        <a data-bind=\"click: createPackageWithoutStacks\" class=\"btn btn-link\">                            Create info package <i class=\"fa fa-file-zip-o\"></i>                        </a>                    </li>                </ul>            </div>            <div class=\"btn-group\" data-bind=\"visible: adminView()==false\">                <button data-bind=\"click: createPackageWithoutStacks\" class=\"btn btn-default\">                    Create info package <i class=\"fa fa-file-zip-o\"></i>                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasStackDump\">                <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                    Download parallel stacks <i class=\"fa fa-download\"></i> <span class=\"caret\"></span>                </button>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li data-bind=\"click: saveAsPng, visible: hasSaveAsPngSupport\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> PNG</a></li>                    <li data-bind=\"click: saveAsSvg\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> SVG</a></li>                    <li data-bind=\"click: saveAsJson\"><a href=\"#\"><i class=\"fa fa-file-text-o\"></i> JSON</a></li>                </ul>            </div>            <button data-bind=\"click: saveAsZip, visible: hasInfoPackage\" type=\"button\" class=\"btn btn-default\">                Download info package <i class=\"fa fa-download\"></i>            </button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: chooseImportFile\">                Import <i class=\"fa fa-upload\"></i>            </button>        </div>        <div class=\"panel-body\">            <div class=\"alert alert-danger\" style=\"margin-top: 10px\" role=\"alert\" data-bind=\"visible: hasFetchException\">                <span data-bind=\"text: fetchException\"></span> <a href=\"#\" data-bind=\"click: moreDetails, visible: showMoreDetailsButton()\">More details</a>                <div data-bind=\"visible: showMoreDetails(), text: fetchExceptionDetails\"></div>            </div>            <div id=\"stacksContainer\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#infoPackageSection\' }\" style=\"width: 100%\" class=\"with-3d-shadow with-transitions\">                <svg id=\"parallelStacks\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>            </div>        </div>    </div></section>'; } )
define("text!views/manage/infoPackageImport.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" >    <form data-bind=\"submit: doImport\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Import info package</h4>        </div>        <div class=\"modal-body\">                        Select file to import. <br />            <input id=\"importFilePicker\" type=\"file\" data-bind=\"event: { change: fileSelected }\" />                    </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Close</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: hasFileSelected\"><i class=\"fa fa-upload\"></i> Import</button>        </div>    </form></div>'; } )
define("text!views/manage/ioTest.html", [], function () { return '<div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startPerformanceTest, visible: isForbidden() == false\">    <div class=\"panel panel-default\">        <div class=\"panel-heading panel-info\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <button type=\"submit\" title=\"IO Test\" class=\"btn btn-primary\" data-bind=\"disable: isBusy\">                <i class=\"fa fa-gears fa-fw\"></i>                <span>Test Disk Performance</span>            </button>            <button class=\"btn btn-danger\" data-bind=\"click: killTask, visible: isBusy\"><i class=\"fa fa-bolt fa-fw\"></i> Cancel test</button>        </div>        <div class=\"panel-body\">            <div>                <div class=\"form-group\" data-bind=\"visible: isBusy\">                    <label for=\"path\" class=\"col-sm-2 control-label\">Current status:</label>                    <div class=\"col-sm-5 text-height-fix\" data-bind=\"text: currentStatus\"></div>                </div>                <div class=\"form-group\">                    <label for=\"path\" class=\"col-sm-2 control-label\">Temporary dir location</label>                    <div class=\"col-sm-5\">                        <input data-bind=\"value: ioTestRequest.path, disable: isBusy\" type=\"text\" required class=\"form-control\" id=\"path\" placeholder=\"C:&#92;temp&#92;\">                    </div>                </div>                <div class=\"form-group\">                    <label for=\"fileSize\" class=\"col-sm-2 control-label\">File size</label>                    <div class=\"col-sm-5\">                        <div class=\"input-group\">                            <input data-bind=\"value: fileSizeMb, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"fileSize\" placeholder=\"1024\">                            <span class=\"input-group-addon\">MB</span>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"testType\" class=\"col-sm-2 control-label\">Test Type</label>                    <div class=\"col-sm-10\">                        <div class=\"btn-group\" data-bind=\"with: ioTestRequest\">                            <button id=\"testType\" type=\"button\" data-bind=\"disable: $root.isBusy\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                <span data-bind=\"visible: testType() === \'generic\'\">Simple</span>                                <span data-bind=\"visible: testType() === \'batch\'\">Batch</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: testType.bind(testType, \'generic\')\">Simple</a></li>                                <li><a href=\"#\" data-bind=\"click: testType.bind(testType, \'batch\')\">Batch</a></li>                            </ul>                        </div>                    </div>                </div>                <div data-bind=\"visible: ioTestRequest.testType() == \'generic\'\">                    <div class=\"form-group\">                        <label for=\"operationType\" class=\"col-sm-2 control-label\">Operation Type</label>                        <div class=\"col-sm-10\">                            <div class=\"btn-group\" data-bind=\"with: ioTestRequest\">                                <button id=\"operationType\" type=\"button\" data-bind=\"disable: $root.isBusy\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: operationType() === \'Read\'\">Read</span>                                    <span data-bind=\"visible: operationType() === \'Write\'\">Write</span>                                    <span data-bind=\"visible: operationType() === \'Mix\'\">Read and write</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\">                                    <li><a href=\"#\" data-bind=\"click: operationType.bind(operationType, \'Read\')\">Read</a></li>                                    <li><a href=\"#\" data-bind=\"click: operationType.bind(operationType, \'Write\')\">Write</a></li>                                    <li><a href=\"#\" data-bind=\"click: operationType.bind(operationType, \'Mix\')\">Read and write</a></li>                                </ul>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"operationType\" class=\"col-sm-2 control-label\">Buffering</label>                        <div class=\"col-sm-10\">                            <div class=\"btn-group\" data-bind=\"with: ioTestRequest\">                                <button id=\"bufferingType\" type=\"button\" data-bind=\"disable: $root.isBusy\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: bufferingType() === \'None\'\">None</span>                                    <span data-bind=\"visible: bufferingType() === \'ReadAndWrite\'\">Read and write</span>                                    <span data-bind=\"visible: bufferingType() === \'Read\'\">Read</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\">                                    <li><a href=\"#\" data-bind=\"click: bufferingType.bind(operationType, \'None\')\">None</a></li>                                    <li><a href=\"#\" data-bind=\"click: bufferingType.bind(operationType, \'ReadAndWrite\')\">Read and write</a></li>                                    <li><a href=\"#\" data-bind=\"click: bufferingType.bind(operationType, \'Read\')\">Read</a></li>                                </ul>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"sequential\" class=\"col-sm-2 control-label\">Sequential</label>                        <div class=\"col-sm-10\">                            <div class=\"btn-group\" data-bind=\"with: ioTestRequest\">                                <button id=\"sequential\" type=\"button\" data-bind=\"disable: $root.isBusy\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                    <span data-bind=\"visible: sequential()\">Yes</span>                                    <span data-bind=\"visible: sequential() == false\">No</span>                                    <span class=\"caret\"></span>                                </button>                                <ul class=\"dropdown-menu\" role=\"menu\">                                    <li><a href=\"#\" data-bind=\"click: sequential.bind(sequential, true)\">Yes</a></li>                                    <li><a href=\"#\" data-bind=\"click: sequential.bind(sequential, false)\">No</a></li>                                </ul>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"threadCount\" class=\"col-sm-2 control-label\">Thread count</label>                        <div class=\"col-sm-5\">                            <input data-bind=\"value: ioTestRequest.threadCount, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"threadCount\">                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"timeToRunInSeconds\" class=\"col-sm-2 control-label\">Time to run</label>                        <div class=\"col-sm-5\">                            <div class=\"input-group\">                                <input data-bind=\"value: ioTestRequest.timeToRunInSeconds, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"timeToRunInSeconds\">                                <span class=\"input-group-addon\">sec</span>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"chunkSize\" class=\"col-sm-2 control-label\">Chunk size</label>                        <div class=\"col-sm-5\">                            <div class=\"input-group\">                                <input data-bind=\"value: chunkSizeKb, disable: isBusy, customValidity: chunkSizeCustomValidityError\" type=\"number\" required class=\"form-control\" id=\"chunkSize\">                                <span class=\"input-group-addon\">KB</span>                            </div>                        </div>                    </div>                </div>                <div data-bind=\"visible: ioTestRequest.testType() == \'batch\'\">                    <div class=\"form-group\">                        <label for=\"numberOfDocuments\" class=\"col-sm-2 control-label\">Number of documents</label>                        <div class=\"col-sm-5\">                            <input data-bind=\"value: ioTestRequest.numberOfDocuments, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"numberOfDocuments\">                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"sizeOfDocuments\" class=\"col-sm-2 control-label\">Size of documents</label>                        <div class=\"col-sm-5\">                            <input data-bind=\"value: ioTestRequest.sizeOfDocuments, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"sizeOfDocuments\">                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"numberOfDocumentsInBatch\" class=\"col-sm-2 control-label\">Number of documents<br/>in batch</label>                        <div class=\"col-sm-5\">                            <input data-bind=\"value: ioTestRequest.numberOfDocumentsInBatch, disable: isBusy\" type=\"number\" required class=\"form-control\" id=\"numberOfDocumentsInBatch\">                        </div>                    </div>                    <div class=\"form-group\">                        <label for=\"waitBetweenBatches\" class=\"col-sm-2 control-label\">Wait between batches (ms)</label>                        <div class=\"col-sm-5\">                            <input data-bind=\"value: ioTestRequest.waitBetweenBatches, disable: isBusy\" type=\"number\" required max=\"100\" min=\"0\" class=\"form-control\" id=\"waitBetweenBatches\">                        </div>                    </div>                </div>            </div>        </div>    </div>        <div data-bind=\"with: testResult\">        <div class=\"panel panel-default\" >            <div class=\"panel-heading panel-info\">                Disk Performance Test Results            </div>            <div class=\"panel-body\">                <!-- ko if:hasReads -->                <table class=\"table-condensed table-striped table-hover table\">                    <tr>                        <th>Total read:</th>                        <th>Test time:</th>                        <th>Avg read:</th>                        <th>&nbsp;</th>                    </tr>                    <tr>                        <td data-bind=\"text: totalRead\"></td>                        <td data-bind=\"text: testTime\"></td>                        <td data-bind=\"text: avgRead\"></td>                        <td>&nbsp;</td>                    </tr>                    <tr>                        <th>Read latency min</th>                        <th>Read latency mean</th>                        <th>Read latency stdev</th>                        <th>Read latency max</th>                    </tr>                    <tr>                        <td data-bind=\"text: readLatencyMin\"></td>                        <td data-bind=\"text: readLatencyMean\"></td>                        <td data-bind=\"text: readLatencyStdev\"></td>                        <td data-bind=\"text: readLatencyMax\"></td>                    </tr>                </table>                <!-- /ko -->                <!-- ko if:hasWrites -->                <table class=\"table-condensed table-striped table-hover table\">                    <tr>                        <th>Total write:</th>                        <th>Test time:</th>                        <th>Avg write:</th>                        <th>&nbsp;</th>                    </tr>                    <tr>                        <td data-bind=\"text: totalWrite\"></td>                        <td data-bind=\"text: testTime\"></td>                        <td data-bind=\"text: avgWrite\"></td>                        <td>&nbsp;</td>                    </tr>                    <tr>                        <th>Write latency min</th>                        <th>Write latency mean</th>                        <th>Write latency stdev</th>                        <th>Write latency max</th>                    </tr>                    <tr>                        <td data-bind=\"text: writeLatencyMin\"></td>                        <td data-bind=\"text: writeLatencyMean\"></td>                        <td data-bind=\"text: writeLatencyStdev\"></td>                        <td data-bind=\"text: writeLatencyMax\"></td>                    </tr>                </table>                <!-- /ko -->                <!-- ko if:hasReads -->                <table  class=\"table-condensed table-striped table-hover table\">                    <tr>                        <th data-bind=\"attr: { colspan: readPercentiles().length } \">Read latency percentiles</th>                    </tr>                    <tr data-bind=\"foreach: readPercentiles\">                        <td data-bind=\"text: key\"></td>                    </tr>                    <tr data-bind=\"foreach: readPercentiles\">                        <td data-bind=\"text: value\"></td>                    </tr>                </table>                <!-- /ko -->                <!-- ko if:hasWrites -->                <table  class=\"table-condensed table-striped table-hover table\">                    <tr>                        <th data-bind=\"attr: { colspan: writePercentiles().length } \">Write latency percentiles</th>                    </tr>                    <tr data-bind=\"foreach: writePercentiles\">                        <td data-bind=\"text: key\"></td>                    </tr>                    <tr data-bind=\"foreach: writePercentiles\">                        <td data-bind=\"text: value\"></td>                    </tr>                </table>                <!-- /ko -->                                <h4>Disk throughput</h4>                <div id=\"overTimeThroughputContainer\" class=\"with-3d-shadow with-transitions\">                    <svg height=\"400\"></svg>                </div>                                <h4>Disk latency</h4>                <div id=\"overTimeLatencyContainer\" class=\"with-3d-shadow with-transitions\">                    <svg height=\"400\"></svg>                </div>            </div>         </div>    </div></form>'; } )
define("text!views/manage/licenseInformation.html", [], function () { return '<div data-bind=\"visible: !settingsAccess.canReadOrWrite()\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><div class=\"panel panel-default form-horizontal\" role=\"form\" data-bind=\"visible: settingsAccess.canReadOrWrite\">    <div class=\"panel-heading\">        License information        <div class=\"clearfix\"></div>    </div>    <div class=\"panel-body\">        <div class=\"form-group\">            <label class=\"col-md-3 control-label\">Connectivity status to license server</label>            <div class=\"col-md-9 text-height-fix\">                <span data-bind=\"visible: connectivityStatus() === \'pending\'\"><i class=\"fa fa-spin fa-spinner\"></i> Checking...</span>                <span class=\"text-success\" data-bind=\"visible: connectivityStatus() === \'success\'\"><i class=\"fa fa-check fa-fw\"></i> Success</span>                <span class=\"text-success\" data-bind=\"visible: connectivityStatus() === \'failure\'\"><i class=\"fa fa-bolt fa-fw\"></i> Can\'t connect to licensing server</span>                            </div>        </div>        <div class=\"form-group\">            <div class=\"col-md-9 col-md-offset-3 text-height-fix\">                <button data-bind=\"click: forceUpdate\" class=\"btn btn-default\"><i class=\"fa fa-bolt fa-fw\"></i> Force license update</button>            </div>        </div>    </div></div>'; } )
define("text!views/manage/restore.html", [], function () { return '<div data-bind=\"visible: anotherRestoreInProgres() && isForbidden() == false\" class=\"badge pull-right\"><span class=\"fa fa-spinner fa-spin\"></span>Another restore in progress</div><div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><div role=\"tabpanel\" data-bind=\"visible: isForbidden() == false\">    <ul class=\"nav nav-tabs\" role=\"tablist\">        <li role=\"presentation\" class=\"active\"><a href=\"#restoreDb\" aria-controls=\"restoreDb\" role=\"tab\" data-toggle=\"tab\">Database</a></li>        <li role=\"presentation\"><a href=\"#restoreFs\" aria-controls=\"restoreFs\" role=\"tab\" data-toggle=\"tab\">File System</a></li>        <li role=\"presentation\" data-bind=\"visible: $root.has40Features\"><a href=\"#restoreCs\" aria-controls=\"restoreFs\" role=\"tab\" data-toggle=\"tab\">Counter Storage</a></li>    </ul>    <div class=\"tab-content\">        <div role=\"tabpanel\" class=\"tab-pane active\" id=\"restoreDb\">            <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startDbRestore\">                <div class=\"panel panel-default\" data-bind=\"with: dbRestoreOptions\">                    <div class=\"panel-heading panel-info\">                        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                        <button type=\"submit\" title=\"Start restore\" class=\"btn btn-primary\" data-bind=\"disable: $parent.isBusy\">                            <i class=\"fa fa-cloud-download fa-fw\" data-bind=\"visible: !$parent.isBusy()\"></i>                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: $parent.isBusy\"></i>                            <span>Restore Database</span>                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: $parent.isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                    </div>                    <div class=\"panel-body\">                        <div>                            <div class=\"form-group\">                                <label for=\"backupLocation\" class=\"col-sm-2 control-label\">Backup Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: backupLocation, disable: $parent.isBusy\" type=\"text\" required class=\"form-control\" id=\"backupLocation\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"databaseLocation\" class=\"col-sm-2 control-label\">Database Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: resourceLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"databaseLocation\" placeholder=\"C:&#92;path-to-your-restored-database-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"indexesLocation\" class=\"col-sm-2 control-label\">Indexes Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: indexesLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"indexesLocation\" placeholder=\"(Optional) Path to location where indexes should be placed\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"journalsLocation\" class=\"col-sm-2 control-label\">Logs Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: journalsLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"journalsLocation\" placeholder=\"(Optional) Path to location where logs/journals should be placed\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"databaseName\" class=\"col-sm-2 control-label\">Database Name</label>                                <div class=\"col-sm-3\">                                    <input data-bind=\"value: resourceName, disable: $parent.isBusy, customValidity: nameCustomValidityError\" type=\"text\" required class=\"form-control\" id=\"databaseName\" placeholder=\"Restored database name\" autocomplete=\"off\">                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" id=\"defrag_db\" data-bind=\"checked: defrag, disable: $parent.isBusy\">                                        <label for=\"defrag_db\">Defrag</label>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" id=\"disable_replication_dest_db\" data-bind=\"checked: $parent.disableReplicationDestinations, disable: $parent.isBusy\">                                        <label for=\"disable_replication_dest_db\">Disable replication destinations</label>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" id=\"gen_db_id\" data-bind=\"checked: $parent.generateNewDatabaseId, disable: $parent.isBusy\">                                        <label for=\"gen_db_id\">Generate new database id</label>                                    </div>                                </div>                            </div>                        </div>                        <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: restoreStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                            <pre><!-- ko foreach: restoreStatusMessages --><div data-bind=\"text: $data\"></div><!-- /ko --></pre>                        </div>                    </div>                </div>                            </form>        </div>        <div role=\"tabpanel\" class=\"tab-pane\" id=\"restoreFs\">            <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startFsRestore\">                <div class=\"panel panel-default\" data-bind=\"with: fsRestoreOptions\">                    <div class=\"panel-heading panel-info\">                        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                        <button type=\"submit\" title=\"Start restore\" class=\"btn btn-primary\" data-bind=\"disable: $parent.isBusy\">                            <i class=\"fa fa-cloud-download fa-fw\" data-bind=\"visible: !$parent.isBusy()\"></i>                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: $parent.isBusy\"></i>                            <span>Restore Filesystem</span>                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: $parent.isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                    </div>                    <div class=\"panel-body\">                        <div>                            <div class=\"form-group\">                                <label for=\"backupLocation\" class=\"col-sm-2 control-label\">Backup Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: backupLocation, disable: $parent.isBusy\" type=\"text\" required class=\"form-control\" id=\"Text1\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemLocation\" class=\"col-sm-2 control-label\">File System Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: resourceLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"filesystemLocation\" placeholder=\"C:&#92;path-to-your-restored-file-system-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemIndexesLocation\" class=\"col-sm-2 control-label\">Indexes Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: indexesLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"filesystemIndexesLocation\" placeholder=\"(Optional) Path to location where indexes should be placed\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemJournalsLocation\" class=\"col-sm-2 control-label\">Logs Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: journalsLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"filesystemJournalsLocation\" placeholder=\"(Optional) Path to location where logs/journals should be placed\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemName\" class=\"col-sm-2 control-label\">File System Name</label>                                <div class=\"col-sm-3\">                                    <input data-bind=\"value: resourceName, disable: $parent.isBusy, customValidity: nameCustomValidityError\" type=\"text\" required class=\"form-control\" id=\"filesystemName\" placeholder=\"Restored file system name\" autocomplete=\"off\">                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" id=\"defrag_fs\" data-bind=\"checked: defrag, disable: $parent.isBusy\">                                        <label for=\"defrag_fs\">Defrag</label>                                    </div>                                </div>                            </div>                        </div>                        <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: restoreStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                            <pre><!-- ko foreach: restoreStatusMessages --><div data-bind=\"text: $data\"></div><!-- /ko --></pre>                        </div>                    </div>                </div>            </form>        </div>        <div role=\"tabpanel\" class=\"tab-pane\" id=\"restoreCs\">            <form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: startFsRestore\">                <div class=\"panel panel-default\" data-bind=\"with: fsRestoreOptions\">                    <div class=\"panel-heading panel-info\">                        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>                        <button type=\"submit\" title=\"Start restore\" class=\"btn btn-primary\" data-bind=\"disable: $parent.isBusy\">                            <i class=\"fa fa-cloud-download fa-fw\" data-bind=\"visible: !$parent.isBusy()\"></i>                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: $parent.isBusy\"></i>                            <span>Restore Counter Storage</span>                        </button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"visible: $parent.isBusy, css: { \'active\': keepDown }, click: function () { this.toggleKeepDown() }\">KeepDown</button>                    </div>                    <div class=\"panel-body\">                        <div>                            <div class=\"form-group\">                                <label for=\"backupLocation\" class=\"col-sm-2 control-label\">Backup Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: backupLocation, disable: $parent.isBusy\" type=\"text\" required class=\"form-control\" id=\"Text1\" placeholder=\"C:&#92;path-to-your-backup-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemLocation\" class=\"col-sm-2 control-label\">Counter Storage Location</label>                                <div class=\"col-sm-5\">                                    <input data-bind=\"value: resourceLocation, disable: $parent.isBusy\" type=\"text\" class=\"form-control\" id=\"filesystemLocation\" placeholder=\"C:&#92;path-to-your-restored-counter-storage-folder\">                                </div>                            </div>                            <div class=\"form-group\">                                <label for=\"filesystemName\" class=\"col-sm-2 control-label\">Counter Storage Name</label>                                <div class=\"col-sm-3\">                                    <input data-bind=\"value: resourceName, disable: $parent.isBusy, customValidity: nameCustomValidityError\" type=\"text\" required class=\"form-control\" id=\"filesystemName\" placeholder=\"Restored counter storage name\" autocomplete=\"off\">                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-offset-2 col-sm-5\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" id=\"defrag_cs\" data-bind=\"checked: defrag, disable: $parent.isBusy\">                                        <label for=\"defrag_cs\">Defrag</label>                                    </div>                                </div>                            </div>                        </div>                        <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: restoreStatusMessages().length, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', bottomMargin: \'10px\' }\">                            <pre><!-- ko foreach: restoreStatusMessages --><div data-bind=\"text: $data\"></div><!-- /ko --></pre>                        </div>                    </div>                </div>            </form>        </div>    </div></div>'; } )
define("text!views/manage/serverSmuggling.html", [], function () { return '<form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: performMigration\">    <div data-bind=\"visible: !settingsAccess.canWrite()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: settingsAccess.canWrite()\">        <div class=\"panel-heading panel-info\">            Server Smuggling        </div>        <div class=\"panel-body\">            <div data-bind=\"with: targetServer\">                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Destination server Url</label>                    <div class=\"col-sm-5\">                        <input required type=\"text\" class=\"form-control\" data-bind=\"value: url, valueUpdate: \'afterkeydown\', disable: $root.inProgress()\"/>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-2 control-label\">Credentials</label>                    <div class=\"col-sm-5\">                        <div class=\"btn-group\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.inProgress()\">                                <span data-bind=\"visible: !isUserCredentials() && !isApiKeyCredentials()\">None</span>                                <span data-bind=\"visible: isUserCredentials\">User</span>                                <span data-bind=\"visible: isApiKeyCredentials\">API key</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: useNoCredentials\">None</a></li>                                <li class=\"divider\"></li>                                <li><a href=\"#\" data-bind=\"click: useUserCredentials\">User</a></li>                                <li><a href=\"#\" data-bind=\"click: useApiKeyCredentials\">API key</a></li>                            </ul>                        </div>                    </div>                </div>                <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">User Name</label>                        <div class=\"col-sm-5\">                            <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, valueUpdate: \'afterkeydown\', disable: $root.inProgress()\"/>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Password</label>                        <div class=\"col-sm-5\">                            <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, valueUpdate: \'afterkeydown\', disable: $root.inProgress()\"/>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Domain</label>                        <div class=\"col-sm-5\">                            <input type=\"text\" class=\"form-control\" data-bind=\"value: domain, valueUpdate: \'afterkeydown\', disable: $root.inProgress()\"/>                        </div>                    </div>                </div>                <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">API Key</label>                        <div class=\"col-sm-5\">                            <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, valueUpdate: \'afterkeydown\', disable: $root.inProgress()\"/>                        </div>                    </div>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-sm-2 control-label\">                    Select databases to migrate <br/>                </label>                <div class=\"col-sm-9\">                    <div>                        <table class=\"table table-condensed table-striped\">                            <thead>                            <tr>                                <th class=\"text-center\">                                    <div class=\"checkbox\">                                        <input id=\"resources_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: resourcesSelection, event: { change: toggleSelectAll }, visible: hasResources, disable: $root.inProgress()\" title=\"Select all or none\">                                        <label for=\"resources_all\">&nbsp;</label>                                    </div>                                </th>                                <th>Name</th>                                <th class=\"text-center\">                                    <div class=\"checkbox\">                                        <input id=\"incr_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: incrementalSelection, event: { change: toggleSelectAllIncremental }, visible: hasResources, disable: $root.inProgress() || noIncremental()\" title=\"Select all or none\">                                        <label for=\"incr_all\">Incremental</label>                                    </div>                                </th>                                <th class=\"text-center\">                                    <div class=\"checkbox\">                                        <input id=\"rep_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: stripReplicationSelection, event: { change: toggleSelectAllStripReplication }, visible: hasResources, disable: $root.inProgress() || noStripReplication()\" title=\"Select all or none\">                                        <label for=\"rep_all\">Strip replication information</label>                                    </div>                                </th>                                <th class=\"text-center\">                                    <div class=\"checkbox\">                                        <input id=\"versioning_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: disableVersioningSelection, event: { change: toggleSelectAllDisableVersioning }, visible: hasResources, disable: $root.inProgress() || noDisableVersioning()\" title=\"Select all or none\">                                        <label for=\"versioning_all\">Disable versioning bundle</label>                                    </div>                                </th>                            </tr>                            </thead>                            <tbody data-bind=\"foreach: resources\">                            <tr>                                <td class=\"text-center\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\"  data-bind=\"attr: { id: \'selection_\' + $index() }, value: $data, checked: $root.selectedResources \">                                        <label data-bind=\"attr: { for: \'selection_\' + $index() } \">&nbsp;</label>                                    </div>                                </td>                                <td>                                    <label data-bind=\"attr: { for: \'selection_\' + $index() }, text: resource.name \" style=\"font-weight: normal\"></label>                                </td>                                <td class=\"text-center\">                                    <div class=\"checkbox\">                                        <input type=\"checkbox\" class=\"styled\" aria-label=\"Incremental\"                                                data-bind=\"disable: $root.inProgress() || !$root.isSelected($data), checked: incremental, attr: { id: \'incr_\' + $index() }\">                                        <label data-bind=\"attr: { for: \'incr_\' + $index() }\">&nbsp;</label>                                    </div>                                </td>                                <td class=\"text-center\">                                    <div class=\"checkbox\" data-bind=\"visible: hasReplicationBundle\">                                        <input type=\"checkbox\" class=\"styled\" aria-label=\"Strip replication information\"                                                data-bind=\"disable: $root.inProgress() || !$root.isSelected($data), checked: stripReplicationInformation, attr: { id : \'strip_rep_\' + $index() }\">                                        <label data-bind=\"attr: { for: \'strip_rep_\' + $index() }\">&nbsp;</label>                                    </div>                                </td>                                <td class=\"text-center\">                                    <div class=\"checkbox\" data-bind=\"visible: hasVersioningBundle\">                                        <input type=\"checkbox\" class=\"styled\" aria-label=\"Should disable versioning bundle\"                                                data-bind=\"disable: $root.inProgress() || !$root.isSelected($data), checked: shouldDisableVersioningBundle, attr: { id: \'versioning_\' + $index() }\">                                        <label data-bind=\"attr: { for: \'versioning_\' + $index() }\">&nbsp;</label>                                    </div>                                </td>                            </tr>                            </tbody>                        </table>                    </div>                </div>                <div class=\"form-group\">                    <div class=\"col-sm-offset-2 col-sm-9\">                        <button type=\"submit\" class=\"btn btn-warning\" data-bind=\"enable: submitEnabled\"><span data-bind=\"visible: $root.inProgress()\" class=\"fa fa-spinner fa-spin\"></span> Run migration</button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleJson, text: (showJsonRequest() ? \'Hide JSON\' : \'Show JSON\')\"></button>                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleCurl, text: (showCurlRequest() ? \'Hide curl\' : \'Show curl\')\"></button>                    </div>                </div>                <div data-bind=\"visible: showCurlRequest()\">                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Curl Command</label>                        <div class=\"col-sm-9\">                            <pre data-bind=\"text: curlRequest\"></pre>                        </div>                    </div>                </div>                <div data-bind=\"visible: showJsonRequest()\">                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Method</label>                        <div class=\"col-sm-9\">                            <input type=\"text\" class=\"form-control\" readonly=\"readonly\" value=\"POST\"/>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-sm-2 control-label\">Payload</label>                        <div class=\"col-sm-9\">                            <pre class=\"jsContent\" data-bind=\"html: jsonRequest\"></pre>                        </div>                    </div>                </div>                <div class=\"bs-code-output\" data-title=\"Log\" data-bind=\"visible: resultsVisible\">                    <pre><!-- ko foreach: $root.messages --><div data-bind=\"text: $data\"></div><!-- /ko --></pre>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/studioConfig.html", [], function () { return '<div class=\"panel panel-default form-horizontal\" role=\"form\">    <div class=\"panel-heading\">        <span class=\"text-height-fix\">Studio config</span>        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <div class=\"clearfix\"></div>    </div>    <div class=\"panel-body\">        <div data-bind=\"visible: !isForbidden()\">            <h4>Server-wide settings</h4>            <hr />            <div class=\"form-group\">                <label class=\"col-md-4 control-label\">Warn when using System Database</label>                <div class=\"col-md-8\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: isReadOnly\">                            <span data-bind=\"visible: warnWhenUsingSystemDatabase\">On</span>                            <span data-bind=\"visible: !warnWhenUsingSystemDatabase()\">Off</span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li><a href=\"#\" data-bind=\"click: setSystemDatabaseWarning.bind($root, true)\">On</a></li>                            <li><a href=\"#\" data-bind=\"click: setSystemDatabaseWarning.bind($root, false)\">Off</a></li>                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <label class=\"col-md-4 control-label\">Environment</label>                <div class=\"col-md-2\">                    <div id=\"select-color\">                        <select class=\"form-control\" data-bind=\"options: environmentColors, value: selectedColor, optionsText: \'name\', disable: isReadOnly\"></select>                    </div>                </div>            </div>        </div>                <h4>Local settings</h4>        <hr />        <div class=\"form-group\">            <label class=\"col-md-4 control-label\" data-bind=\"text: \'Mute remind to upgrade (\' + timeUntilRemindToUpgradeMessage() + \')\'\"></label>            <div class=\"col-md-8\">                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"visible: mute\">On</span>                        <span data-bind=\"visible: !mute()\">Off</span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\">                        <li><a href=\"#\" data-bind=\"click: setUpgradeReminder.bind($root, true)\">On</a></li>                        <li><a href=\"#\" data-bind=\"click: setUpgradeReminder.bind($root, false)\">Off</a></li>                    </ul>                </div>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Disable event source</label>            <div class=\"col-md-8\">                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"visible: disableEventSource\">Yes</span>                        <span data-bind=\"visible: !disableEventSource()\">No</span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\">                        <li><a href=\"#\" data-bind=\"click: setEventSourceDisabled.bind($root, true)\">Yes</a></li>                        <li><a href=\"#\" data-bind=\"click: setEventSourceDisabled.bind($root, false)\">No</a></li>                    </ul>                </div>            </div>        </div>        <div class=\"form-group\">            <label class=\"col-md-4 control-label\">Number format</label>            <div class=\"col-md-8\">                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                        <span data-bind=\"visible: rawFormat\">Raw format (ex. 5055.99)</span>                        <span data-bind=\"visible: !rawFormat()\">Browser default format (ex. <span data-bind=\"text: browserFormatExample\"></span>)</span>                        <span class=\"caret\"></span>                    </button>                    <ul class=\"dropdown-menu\" role=\"menu\">                        <li><a href=\"#\" data-bind=\"click: setNumberFormat.bind($root, true)\">Raw format (ex. 5055.99)</a></li>                        <li><a href=\"#\" data-bind=\"click: setNumberFormat.bind($root, false)\">Browser default format (ex. <span data-bind=\"text: browserFormatExample\"></span>)</a></li>                    </ul>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/manage/topology.html", [], function () { return '<style>    #replicationStatsContainer {        font: 11px sans-serif;    }    svg {        border: solid 1px #eee;    }    .axis path,    .axis line {        fill: none;        stroke: #000;        shape-rendering: crispEdges;    }    .bar {        fill: steelblue;    }    rect.outputCounts, rect.inputCounts {        fill-opacity: 0.8;        cursor: pointer;    }    rect.outputCounts:hover, rect.inputCounts:hover {        fill-opacity: 1;    }    .legend rect {        cursor: pointer;    }    .legend_bg {        fill: white;        fill-opacity: 0.8;        stroke: #ccc;    }    .legendHidden {        fill-opacity: 0.2;       }    #replicationTopologySection text.fa {        font-family: FontAwesome;        font-size: 18px;    }</style><section id=\"replicationTopologySection\">    <div data-bind=\"visible: !settingsAccess.canWrite()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div>        <div data-bind=\"visible: settingsAccess.canWrite()\">            <div class=\"panel panel-default panel-wo-margin\" data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#replicationTopologySection\' }\">                <div class=\"panel-heading\">                    <span data-bind=\"visible: !topology()\">                        <button data-bind=\"click: fetchTopology\" class=\"btn btn-primary\">                            <i class=\"fa fa-spinner fa-spin fa-fw\" data-bind=\"visible: showLoadingIndicator\"></i>                            <i class=\"fa fa-sitemap fa-fw\" data-bind=\"visible: !showLoadingIndicator()\"></i> Fetch topology                        </button>                        <i class=\"fa fa-warning text-warning padd-left-10\"></i> <span class=\"text-muted\"> This operation may be time consuming.</span>                    </span>                    <div class=\"btn-group\" data-bind=\"if: topology\">                        <div class=\"input-group\">                            <span class=\"input-group-addon\"><i class=\"fa fa-search\"></i></span>                            <input type=\"text\" class=\"form-control\" placeholder=\"Search by name\" data-bind=\"value: $root.searchText, valueUpdate: \'afterkeydown\'\" />                        </div>                    </div>                    <div class=\"pull-right btn-group\" data-bind=\"if: topology\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" title=\"Download topology\">                            <i class=\"fa fa-download\"></i>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li data-bind=\"click: saveAsPng, visible: hasSaveAsPngSupport\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> PNG</a></li>                            <li data-bind=\"click: saveAsSvg\"><a href=\"#\"><i class=\"fa fa-file-image-o\"></i> SVG</a></li>                            <li data-bind=\"click: saveAsJson\"><a href=\"#\"><i class=\"fa fa-file-text-o\"></i> JSON</a></li>                        </ul>                    </div>                    <div class=\"clearfix\"></div>                </div>                <div class=\"panel-collapse\">                    <div class=\"panel-body\">                        <div class=\"row padd-top-10\" data-bind=\"with: topology\">                            <div class=\"col-xs-8\">                                <svg id=\"replicationTopology\" transform=\"translate(0.5,0.5)\" preserveAspectRatio=\"xMinYMin slice\"></svg>                            </div>                            <div class=\"col-xs-4\">                                <div data-bind=\"if: !$root.currentLink()\">                                    <div class=\"well\">                                        <i class=\"fa fa-info-circle text-info\"></i> Click on the <strong>arrows between nodes</strong> to display information about the replication relationship.                                        <br />                                        <br />                                        <div>                                            <dl>                                                <dt class=\"text-success\"><i class=\"fa fa-long-arrow-right fa-2x\"></i></dt>                                                <dd>Endpoints can communicate each other</dd>                                            </dl>                                            <dl>                                                <dt class=\"text-danger\"><i class=\"fa fa-long-arrow-right fa-2x\"></i></dt>                                                <dd>Error in communication</dd>                                            </dl>                                        </div>                                        <br />                                        <i class=\"fa fa-info-circle text-info\"></i> Use the mouse to pan by clicking on the background; you may also use the mousewheel to zoom.                                    </div>                                </div>                                <div data-bind=\"with: $root.currentLink\">                                    <!-- ko if: UiType == \'db\' -->                                    <div class=\"panel panel-default\">                                        <div class=\"panel-body\">                                            <table class=\"table table-striped table-condensed\">                                                <thead>                                                    <tr>                                                        <th>Property</th>                                                        <th>Value</th>                                                    </tr>                                                </thead>                                                <tbody>                                                    <tr>                                                        <td>Source</td>                                                        <td data-bind=\"text: Source\"></td>                                                    </tr>                                                    <tr>                                                        <td>Destination</td>                                                        <td data-bind=\"text: Destination\"></td>                                                    </tr>                                                    <tr>                                                        <td>SendServerId</td>                                                        <td data-bind=\"text: SendServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>StoredServerId</td>                                                        <td data-bind=\"text: StoredServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastAttachmentEtag</td>                                                        <td data-bind=\"text: LastAttachmentEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastDocumentEtag</td>                                                        <td data-bind=\"text: LastDocumentEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>ReplicationBehavior</td>                                                        <td data-bind=\"text: ReplicationBehavior\"></td>                                                    </tr>                                                    <tr>                                                        <td>SourceToDestinationState</td>                                                        <td data-bind=\"text: SourceToDestinationState\"></td>                                                    </tr>                                                    <tr>                                                        <td>DestinationToSourceState</td>                                                        <td data-bind=\"text: DestinationToSourceState\"></td>                                                    </tr>                                                    <tr data-bind=\"if: Errors\">                                                        <td>Errors</td>                                                        <td>                                                            <ul data-bind=\"foreach: Errors\">                                                                <li data-bind=\"text: $data\"></li>                                                            </ul>                                                        </td>                                                    </tr>                                                </tbody>                                            </table>                                        </div>                                    </div>                                    <!-- /ko -->                                    <!-- ko if: UiType == \'fs\' -->                                    <div class=\"panel panel-default\">                                        <div class=\"panel-body\">                                            <table class=\"table table-striped table-condensed\">                                                <thead>                                                    <tr>                                                        <th>Property</th>                                                        <th>Value</th>                                                    </tr>                                                </thead>                                                <tbody>                                                    <tr>                                                        <td>Source</td>                                                        <td data-bind=\"text: Source\"></td>                                                    </tr>                                                    <tr>                                                        <td>Destination</td>                                                        <td data-bind=\"text: Destination\"></td>                                                    </tr>                                                    <tr>                                                        <td>SendServerId</td>                                                        <td data-bind=\"text: SendServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>StoredServerId</td>                                                        <td data-bind=\"text: StoredServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastSourceFileEtag</td>                                                        <td data-bind=\"text: LastSourceFileEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>SourceToDestinationState</td>                                                        <td data-bind=\"text: SourceToDestinationState\"></td>                                                    </tr>                                                    <tr>                                                        <td>DestinationToSourceState</td>                                                        <td data-bind=\"text: DestinationToSourceState\"></td>                                                    </tr>                                                    <tr data-bind=\"if: Errors\">                                                        <td>Errors</td>                                                        <td>                                                            <ul data-bind=\"foreach: Errors\">                                                                <li data-bind=\"text: $data\"></li>                                                            </ul>                                                        </td>                                                    </tr>                                                </tbody>                                            </table>                                        </div>                                    </div>                                    <!-- /ko -->                                    <!-- ko if: UiType == \'cs\' -->                                    <div class=\"panel panel-default\">                                        <div class=\"panel-body\">                                            <table class=\"table table-striped table-condensed\">                                                <thead>                                                    <tr>                                                        <th>Property</th>                                                        <th>Value</th>                                                    </tr>                                                </thead>                                                <tbody>                                                    <tr>                                                        <td>Source</td>                                                        <td data-bind=\"text: Source\"></td>                                                    </tr>                                                    <tr>                                                        <td>Destination</td>                                                        <td data-bind=\"text: Destination\"></td>                                                    </tr>                                                    <tr>                                                        <td>SendServerId</td>                                                        <td data-bind=\"text: SendServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>StoredServerId</td>                                                        <td data-bind=\"text: StoredServerId\"></td>                                                    </tr>                                                    <tr>                                                        <td>LastSourceFileEtag</td>                                                        <td data-bind=\"text: LastEtag\"></td>                                                    </tr>                                                    <tr>                                                        <td>SourceToDestinationState</td>                                                        <td data-bind=\"text: SourceToDestinationState\"></td>                                                    </tr>                                                    <tr>                                                        <td>DestinationToSourceState</td>                                                        <td data-bind=\"text: DestinationToSourceState\"></td>                                                    </tr>                                                    <tr data-bind=\"if: Errors\">                                                        <td>Errors</td>                                                        <td>                                                            <ul data-bind=\"foreach: Errors\">                                                                <li data-bind=\"text: $data\"></li>                                                            </ul>                                                        </td>                                                    </tr>                                                </tbody>                                            </table>                                        </div>                                    </div>                                    <!-- /ko -->                                </div>                            </div>                        </div>                        <div data-bind=\"visible: !topology()\">                            <form class=\"form-horizontal\" role=\"form\">                                <div class=\"form-group\">                                    <label class=\"col-sm-2 control-label\">Resource types</label>                                    <div class=\"col-sm-5\">                                        <div class=\"checkbox\">                                            <input type=\"checkbox\" class=\"styled\" id=\"topo_db\" data-bind=\"checked: fetchDb\">                                            <label for=\"topo_db\">Databases</label>                                        </div>                                        <div class=\"checkbox\">                                            <input type=\"checkbox\" class=\"styled\" id=\"topo_fs\" data-bind=\"checked: fetchFs\">                                            <label for=\"topo_fs\">File systems</label>                                        </div>                                        <div class=\"checkbox\" data-bind=\"visible: $root.showCsOption()\">                                            <input type=\"checkbox\" class=\"styled\" id=\"topo_cs\" data-bind=\"checked: fetchCs\">                                            <label for=\"topo_cs\">Counters</label>                                        </div>                                    </div>                                </div>                            </form>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/manage/trafficWatch.html", [], function () { return '<div data-bind=\"visible: isForbidden\" class=\"text-center\">    <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4></div><section id=\"watchTrafficContainer\" data-bind=\"visible: isForbidden() == false\">    <div class=\"panel panel-default panel-wo-margin\">        <div class=\"panel-heading panel-info\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: configureConnection\">Configure Connection</button>                    <button type=\"button\" class=\"btn btn-success\" data-bind=\"click: reconnect\">Reconnect</button>                    <button type=\"button\" class=\"btn btn-warning\" data-bind=\"click: disconnect\">Disconnect</button>                </div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"css: { \'active\': keepDown }, click: toggleKeepDown\">Keep Down</button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: resetStats\">Reset Stats</button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: clearLogs\">Clear</button>                </div>                <div class=\"btn-group\">                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: exportTraffic\">Export</button>                </div>                <div class=\"padd-top-10 form-inline\">                    <div class=\"form-group\">                        <label>Url:</label>                        <input type=\"text\" class=\"form-control\" data-bind=\"value: filter, valueUpdate: \'afterkeydown\'\" placeholder=\"Filter results by url\">                    </div>                    <div class=\"form-group padd-left-10\">                        <label>Duration:</label>                        <div class=\"input-group\">                            <input type=\"number\" class=\"form-control pull-left\" data-bind=\"value: filterDuration, valueUpdate: \'afterkeydown\'\" placeholder=\"duration > than\">                            <span class=\"input-group-addon\">ms</span>                        </div>                    </div>                </div>            </div>        </div>        <div class=\"panel-body\">            <div class=\"text-center\" data-bind=\"visible: !logConfig()\">                <h3 class=\"text-info\">                    Click on \'Configure Connection\' to start.                </h3>            </div>            <div data-bind=\"if: logConfig\">                <div class=\"bs-status\" data-bind=\"with: logConfig\">                    <strong>Status: </strong><label data-bind=\"text: ($root.isConnected()?\'Connected\':\'Disconnected\'), css:{\'text-danger\':$root.isConnected() == false, \'text-success\':$root.isConnected()==true}\"></label>                    <strong>; Watched Resource: </strong><label data-bind=\"text: ResourceName\"></label>                    <strong>; Trace Start Time :</strong><label data-bind=\"text: $root.startTraceTimeHumanized()\"></label>                    <strong>; Requests: </strong><label data-bind=\"text: $root.watchedRequests()\"></label>                    <strong>; Avg Request Duration: </strong><label data-bind=\"text: $root.averageRequestDuration()\"></label>                    <strong>; Min Request Duration: </strong><label data-bind=\"text: $root.minRequestDuration()\"></label>                    <strong>; Max Request Duration: </strong><label data-bind=\"text: $root.maxRequestDuration()\"></label>                    <strong>; Buffer Size: </strong><label data-bind=\"text: MaxEntries\"></label>                </div>                <div data-bind=\"dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#watchTrafficContainer\' }\" style=\"position: relative\">                    <div class=\"form-group\" id=\"logTableContainer\" >                        <div class=\"form-group logHeaders\">                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[0]() + \'%\' }\">                                <span class=\"column-handle\" column=\"0\"></span>                                <span>Time</span>                            </div>                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[1]() + \'%\' }\">                                <span class=\"column-handle\" column=\"1\"></span>                                <span>Duration</span>                            </div>                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[2]() + \'%\' }\">                                <span class=\"column-handle\" column=\"2\"></span>                                <span>Resource</span>                            </div>                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[3]() + \'%\' }\">                                <span class=\"column-handle\" column=\"3\"></span>                                <span>Method</span>                            </div>                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[4]() + \'%\' }\">                                <span class=\"column-handle\" column=\"4\"></span>                                <span>Url</span>                            </div>                            <div class=\"column column-header\" data-bind=\"style: { width: $root.columnWidths[5]() + \'%\' }\">                                <span class=\"column-handle\" column=\"5\"></span>                                <span>Has Additional Data</span>                            </div>                        </div>                        <div id=\"logRecords\" class=\"form-group logRecords\" style=\"white-space: nowrap\" data-bind=\"foreach: recentEntries, delegatedHandler: \'click\'\">                            <div id=\"logRecordsContainer\" style=\"display: block; white-space: nowrap\" class=\"logRecord log-table-row\" data-click=\"selectLog\" data-bind=\"\'selected\': $data === $parent.selectedLog(), visible: Visible\">                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[0]() + \'%\' }, text: TimeStampText, attr: {title:Time}\"></div>                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[1]() + \'%\' }, text: Duration, attr: { title: Duration }\"></div>                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[2]() + \'%\' }, text: Resource, attr: { title: Resource }\"></div>                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[3]() + \'%\' }, text: Method, attr: { title: Method }\"></div>                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[4]() + \'%\' }, text: Url\"></div>                                <div class=\"column\" data-bind=\"style: { width: $root.columnWidths[5]() + \'%\' }, text: !!CustomInfo?\'Y\':\'N\'\"></div>                            </div>                        </div>                        <div class=\"form-group log-details panel panel-default panel-wo-margin\" data-bind=\"visible: showLogDetails\">                            <div class=\"col-md-11\" data-bind=\"with: selectedLog\">                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Time</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"text: TimeStampText() +\' \' + Time()\"></p>                                    </div>                                </div>                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Duration</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Duration\"></p>                                    </div>                                </div>                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Resource Name</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Resource\"></p>                                    </div>                                </div>                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Method</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Method\"></p>                                    </div>                                </div>                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Url</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"text: Url\"></p>                                    </div>                                </div>                                <div class=\"row\">                                    <label class=\"col-md-2 text-right control-label\">Custom Info</label>                                    <div class=\"col-md-10\">                                        <p class=\"form-control-static\" data-bind=\"html: !!CustomInfo ? CustomInfo : \'-\'\"></p>                                    </div>                                </div>                            </div>                            <div class=\"col-sm-1\">                                <button class=\"btn btn-default\" data-bind=\"click: function(){ this.showLogDetails(false) }\">Hide</button>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></section>'; } )
define("text!views/manage/watchTrafficConfigDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 715px;\">    <form class=\"form-horizontal\" data-bind=\"submit: confirmConfig\" id=\"log-config-form\" onkeypress=\"return event.keyCode !== 13;\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">                <i class=\"fa fa-times\"></i>            </button>            <h4 class=\"modal-title\">Custom logging</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label for=\"maxEntries\" class=\"control-label col-sm-4\">Maximum number of displayed entries</label>                <div class=\"col-sm-8\">                    <input type=\"number\" data-bind=\"value: maxEntries\" class=\"form-control\" id=\"maxEntries\" required max=\"30000\" min=\"1\">                </div>            </div>            <div class=\"form-group\">                <label class=\"control-label col-sm-4\">Watched Resource:</label>            </div>            <div class=\"form-group\">                <label class=\"control-label col-sm-4\" title=\"Resource name, might be a database and/or a filesystem\">                    <input type=\"radio\" value=\"SingleResourceView\" data-bind=\"checked: watchedResourceMode\"/> Single Resource                </label>                <div class=\"col-sm-8\">                    <input id=\"watchedResource\" placeholder=\"Resource name\" type=\"text\" class=\"form-control\"                           data-bind=\"enable: watchedResourceMode() === \'SingleResourceView\', customValidity: nameCustomValidityError, value: resourceName, valueUpdate: \'afterkeydown\'\" autocomplete=\"off\"/>                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\"                        data-bind=\"autoComplete: \'#watchedResource\', foreach: searchResults\">                        <li role=\"presentation\" data-bind=\"click: function(){ $root.resourceName($data); }\">                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\">                                <span role=\"menuitem\" tabindex=\"-1\">                                    <span data-bind=\"text: $data\"></span>                                </span>                            </a>                        </li>                    </ul>                </div>            </div>            <div class=\"form-group\">                <label title=\"Allows watching all incoming Requests. Requires Machine Admin authentication\" class=\"text-danger control-label col-sm-4\">                    <input type=\"radio\" value=\"AdminView\" data-bind=\"checked: watchedResourceMode\" /> All (Admin View)                </label>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"submit\" class=\"btn btn-primary\">Connect</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>        </div>    </form></div>'; } )
define("text!views/manage/windowsAuth.html", [], function () { return '<form class=\"form-horizontal\" id=\"manageWindowsAuth\" role=\"form\" data-bind=\"submit: saveChanges\">    <div data-bind=\"visible: settingsAccess.isForbidden\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: settingsAccess.isForbidden() == false\">        <div class=\"panel-heading\">            <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled\" title=\"Save changes(Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addUserSettings, disable: settingsAccess.isReadOnly, visible: isUsersSectionActive\" title=\"Add user settings(Alt+N)\" accesskey=\"N\"><i class=\"fa fa-plus\"></i> Add User Settings</button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addGroupSettings, disable: settingsAccess.isReadOnly, visible: !isUsersSectionActive()\"><i class=\"fa fa-plus\"></i> Add Group Settings</button>        </div>        <div class=\"panel-body\">            <ul class=\"nav nav-tabs\">                <li class=\"active\"><a href=\"#windowsAuthUsers\" data-toggle=\"tab\" data-bind=\"click: isUsersSectionActive.bind($root, true)\">Users</a></li>                <li><a href=\"#windowsAuthGroups\" data-toggle=\"tab\" data-bind=\"click: isUsersSectionActive.bind($root, false)\">Groups</a></li>            </ul>            <div class=\"tab-content win-auth-tabs\">            <div class=\"tab-pane fade in active\" id=\"windowsAuthUsers\">                <div class=\"text-center\" data-bind=\"visible: setup().requiredUsers().length === 0\">                    <h3 class=\"text-info\">                        You don\'t have any User Settings defined.                    </h3>                </div>                <div data-bind=\"foreach: setup().requiredUsers\" class=\"win-auth-panels-container\">                    <div class=\"panel win-auth-card\" data-bind=\"css: { \'panel-info\': enabled, \'panel-warning\': !enabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span><span data-bind=\"text: name() ? name() : \'untitled\'\"></span></span>                                <button type=\"button\" style=\"display: inline-block;\" class=\"close\" title=\"Remove\" data-bind=\"click: $parent.removeUserSettings.bind($parent), visible: $root.settingsAccess.isReadOnly() == false\">                                    <i class=\"fa fa-times\"></i>                                </button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Name</label>                                <div class=\"col-sm-9\">                                    <input type=\"text\" class=\"form-control\" required data-bind=\"value: name, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidity\"/>                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: invalidName\">                                <div class=\"col-sm-9 col-sm-offset-3 text-warning\">                                    <i class=\"fa fa-warning\"></i> Supplied username does not exist in domain                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: verificationInProgress\">                                <div class=\"col-sm-9 col-sm-offset-3\">                                    <i class=\"fa fa-spin fa-spinner\"></i> Verifing username and domain                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Enabled</label>                                <div class=\"col-sm-9\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: enabled\">Enabled</span>                                            <span data-bind=\"visible: !enabled()\">Disabled</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                            <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Resources</label>                                <div class=\"col-sm-9\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addEmptyDatabase, visible: databases().length == 0\" title=\"Add resource\">                                        <i class=\"fa fa-plus\"></i> Add resource                                    </button>                                    <table class=\"table table-striped table-hover\">                                        <thead>                                        <tr class=\"database-header\" data-bind=\"visible: databases().length > 0\">                                            <td>Name</td>                                            <td>Type</td>                                            <td>                                                <button type=\"button\" style=\"display: inline-block; font-size: 14px;\" class=\"close\" title=\"Add resource\" data-bind=\"click: addEmptyDatabase, visible: $root.settingsAccess.isReadOnly() == false\">                                                    <i class=\"fa fa-plus\"></i>                                                </button>                                            </td>                                        </tr>                                        </thead>                                        <tbody data-bind=\"foreach: databases\">                                        <tr>                                            <td>                                                <div class=\"btn-group\">                                                    <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"databaseName\" class=\"form-control\" data-bind=\"value: tenantId, valueUpdate: \'afterkeydown\', customValidity: tenantCustomValidityError, attr: { \'id\': \'userDatabaseName\' + $parentContext.$index() + $index() }\" placeholder=\"Resource name\">                                                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#userDatabaseName\' + $parentContext.$index() + $index(), foreach: searchResults\">                                                        <li role=\"presentation\" data-bind=\"click: $parent.tenantId.bind($data)\">                                                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\" class=\"nounderline\">                                                                <span data-bind=\"text: $data\"></span>                                                            </a>                                                        </li>                                                    </ul>                                                </div>                                            </td>                                            <td>                                                <div class=\"btn-group\">                                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                        <span data-bind=\"text: currentAccessType\"></span> <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <!-- ko foreach:getTypes() -->                                                        <li><a href=\"#\" class=\"nounderline\" data-bind=\"text: $data, click: $parent.currentAccessType.bind($parent)\"></a></li>                                                        <!-- /ko -->                                                    </ul>                                                </div>                                            </td>                                            <td>                                                <button type=\"button\" class=\"close\" data-bind=\"visible: $parent.databases().length > 1, click: $parent.removeDatabase.bind($parent, $data), visible: $root.settingsAccess.isReadOnly() == false\" title=\"Remove this resource\">                                                    <i class=\"fa fa-times\"></i>                                                </button>                                            </td>                                        </tr>                                        </tbody>                                    </table>                                </div>                            </div>                        </div>                    </div>                </div>            </div>            <div class=\"tab-pane fade\" id=\"windowsAuthGroups\">                <div class=\"text-center\" data-bind=\"visible: setup().requiredGroups().length === 0\">                    <h3 class=\"text-info\">                        You don\'t have any Group Settings defined.                    </h3>                </div>                <div data-bind=\"foreach: setup().requiredGroups\" class=\"win-auth-panels-container\">                    <div class=\"panel win-auth-card\" data-bind=\"css: { \'panel-info\': enabled, \'panel-warning\': !enabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span><span data-bind=\"text: name() ? name() : \'untitled\'\"></span></span>                                <button type=\"button\" style=\"display: inline-block;\" class=\"close\" title=\"Remove\" data-bind=\"click: $parent.removeGroupSettings.bind($parent), visible: $root.settingsAccess.isReadOnly() == false\">                                    <i class=\"fa fa-times\"></i>                                </button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Name</label>                                <div class=\"col-sm-9\">                                    <input type=\"text\" class=\"form-control\" required data-bind=\"value: name, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidity\"/>                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: invalidName\">                                <div class=\"col-sm-9 col-sm-offset-3 text-warning\">                                    <i class=\"fa fa-warning\"></i> Supplied group does not exist in domain                                </div>                            </div>                            <div class=\"form-group\" data-bind=\"visible: verificationInProgress\">                                <div class=\"col-sm-9 col-sm-offset-3\">                                    <i class=\"fa fa-spin fa-spinner\"></i> Verifing group and domain                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Enabled</label>                                <div class=\"col-sm-9\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                            <span data-bind=\"visible: enabled\">Enabled</span>                                            <span data-bind=\"visible: !enabled()\">Disabled</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                            <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-3 control-label\">Resources</label>                                <div class=\"col-sm-9\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addEmptyDatabase, visible: databases().length == 0\" title=\"Add resource\">                                        <i class=\"fa fa-plus\"></i> Add resource                                    </button>                                    <table class=\"table table-striped table-hover\">                                        <thead>                                        <tr class=\"database-header\" data-bind=\"visible: databases().length > 0\">                                            <td>Name</td>                                            <td>Type</td>                                            <td>                                                <button type=\"button\" style=\"display: inline-block; font-size: 14px;\" class=\"close\" title=\"Add resource\" data-bind=\"click: addEmptyDatabase, visible: $root.settingsAccess.isReadOnly() == false\">                                                    <i class=\"fa fa-plus\"></i>                                                </button>                                            </td>                                        </tr>                                        </thead>                                        <tbody data-bind=\"foreach: databases\">                                        <tr>                                            <td>                                                <div class=\"btn-group\">                                                    <input tabindex=\"2\" autocomplete=\"off\" required type=\"text\" name=\"databaseName\" class=\"form-control\" data-bind=\"value: tenantId, valueUpdate: \'afterkeydown\', customValidity: tenantCustomValidityError, attr: { \'id\': \'groupDatabaseName\' + $parentContext.$index() + $index() }\" placeholder=\"Resource name\">                                                    <ul class=\"dropdown-menu scrollable-resource-menu\" role=\"menu\" style=\"display: none;\" data-bind=\"autoComplete: \'#groupDatabaseName\' + $parentContext.$index() + $index(), foreach: searchResults\">                                                        <li role=\"presentation\" data-bind=\"click: $parent.tenantId.bind($data)\">                                                            <a role=\"menuitem\" tabindex=\"-1\" href=\"#\" class=\"nounderline\">                                                                <span data-bind=\"text: $data\"></span>                                                            </a>                                                        </li>                                                    </ul>                                                </div>                                            </td>                                            <td>                                                <div class=\"btn-group\">                                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                                                        <span data-bind=\"text: currentAccessType\"></span> <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <!-- ko foreach:getTypes() -->                                                        <li><a href=\"#\" class=\"nounderline\" data-bind=\"text: $data, click: $parent.currentAccessType.bind($parent)\"></a></li>                                                        <!-- /ko -->                                                    </ul>                                                </div>                                            </td>                                            <td>                                                <button type=\"button\" class=\"close\" style=\"display: inline-block; font-size: 14px;\" data-bind=\"visible: $parent.databases().length > 1, click: $parent.removeDatabase.bind($parent, $data), visible: $root.settingsAccess.isReadOnly() == false\" title=\"Remove this resource\">                                                    <i class=\"fa fa-times\"></i>                                                </button>                                            </td>                                        </tr>                                        </tbody>                                    </table>                                </div>                            </div>                        </div>                    </div>                </div>            </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/globalConfig/globalConfig.html", [], function () { return '<div id=\"global-config\">    <ul class=\"nav nav-tabs\" data-bind=\"foreach: router.navigationModel\">        <li data-bind=\"css: { active: isActive }\">             <a   data-bind=\"attr: { href: hash, title: tooltip }, text: title\"></a>        </li>    </ul>    <div class=\"tab-content\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/manage/globalConfig/globalConfigCustomFunctions.html", [], function () { return '<form class=\"custom-functions-form\" id=\"customFunctionsContainer\">    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && !$root.canUseGlobalConfigurations()\">        <div class=\"panel-body\">            <div class=\"text-center\">                <h3 class=\"text-info\">                    Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                </h3>            </div>        </div>    </div>    <div class=\"panel panel-default panel-wo-margin\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && $root.canUseGlobalConfigurations()\">        <div class=\"panel-heading\">            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"click: saveChanges, enable: isSaveEnabled, visible: activated()\" title=\"Save Changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-plus\"></i> Create global configuration for Custom Functions            </button>            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-trash\"></i> Remove global configuration for Custom Functions            </button>            <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>        </div>        <div class=\"panel-body\">            <div class=\"text-center\" data-bind=\"visible: !activated()\">                <h3 class=\"text-info\">Global configuration for Custom functions is not currently defined.</h3>            </div>            <div data-bind=\"visible: activated()\">                <div>                    <p>You can create custom JavaScript functions here by writing simple <a href=\"http://wiki.commonjs.org/wiki/Modules/1.1\" target=\"about:blank\">commonjs module</a>.                        <span id=\"customFunctionsExample\" class=\"help-cursor\" data-placement=\"auto right\" data-original-title=\"Custom function example\" data-trigger=\"hover\">                            See the example. <i class=\"fa fa-info-circle text-info\"></i>                        </span>                    </p>                </div>                <div data-bind=\"if: activated()\">                    <pre class=\"editor\" id=\"customFunctionsEditor\" data-bind=\"aceEditor: { code: documentText, readOnly: $root.settingsAccess.isReadOnly(), fontSize: \'18px\', lang: \'ace/mode/javascript\' }, dynamicHeight: { resizeTrigger: window.ravenStudioWindowHeight(), target: \'footer\', container: \'#global-config\' }\"></pre>                    </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/globalConfig/globalConfigPeriodicExport.html", [], function () { return '<section data-bind=\"with: backupSetup\">    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div data-bind=\"visible: !$root.settingsAccess.isForbidden() && unsupported()\">        <div class=\"alert alert-danger\">Too many periodic export types found. This isn\'t a supported scenario for the UI.</div>    </div>    <div data-bind=\"if: !$root.settingsAccess.isForbidden() && !unsupported()\">        <div class=\"panel panel-default\" data-bind=\"visible: !$root.canUseGlobalConfigurations()\">            <div class=\"panel-body\">                <div class=\"text-center\">                    <h3 class=\"text-info\">                        Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                    </h3>                </div>            </div>        </div>        <form data-bind=\"visible: $root.canUseGlobalConfigurations(), submit: $root.saveChanges.bind($root)\" id=\"save-periodic-backup-form\" class=\"form-horizontal\" role=\"form\">            <div class=\"panel panel-default\">                <div class=\"panel-heading\">                    <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: $root.isSaveEnabled(), visible: $root.activated()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !$root.activated(), disable: $root.settingsAccess.isReadOnly()\">                        <i class=\"fa fa-plus\"></i> Create global configuration for Periodic Export                    </button>                    <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: $root.activated(), disable: $root.settingsAccess.isReadOnly()\">                        <i class=\"fa fa-trash\"></i> Remove global configuration for Periodic Export                    </button>                    <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>                </div>                <div class=\"panel-body\">                    <div class=\"text-center\" data-bind=\"visible: !$root.activated()\">                        <h3 class=\"text-info\">Global configuration for Periodic Export is not currently defined.</h3>                    </div>                    <div data-bind=\"if: $root.activated()\" id=\"periodicExportContainer\">                        <div class=\"form-group\">                            <label for=\"disabled\" class=\"col-sm-3 control-label\">Enabled:</label>                            <div class=\"col-sm-3\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                        <span data-bind=\"visible: disabled()==false\">Yes</span>                                        <span data-bind=\"visible: disabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: disabled.bind(disabled, false)\">Yes</a></li>                                        <li><a href=\"#\" data-bind=\"click: disabled.bind(disabled, true)\">No</a></li>                                    </ul>                                </div>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"onDiskExport\" class=\"col-sm-3 control-label\">On disk export:</label>                            <div class=\"col-sm-3\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                        <span data-bind=\"visible: onDiskExportEnabled\">Yes</span>                                        <span data-bind=\"visible: !onDiskExportEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: onDiskExportEnabled.bind(onDiskExportEnabled, true)\">Yes</a></li>                                        <li><a href=\"#\" data-bind=\"click: onDiskExportEnabled.bind(onDiskExportEnabled, false)\">No</a></li>                                    </ul>                                </div>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: onDiskExportEnabled\">                            <label for=\"onDiskLocation\" class=\"col-sm-3 control-label\">File System Folder:</label>                            <div class=\"col-sm-3\" data-bind=\"if: onDiskExportEnabled\">                                <input type=\"text\" id=\"onDiskLocation\" placeholder=\"full path\" required class=\"form-control\" data-bind=\"value: localFolderName, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                            <div class=\"col-sm-6\">                                <span id=\"onDiskHint\" class=\"help-cursor\" data-placement=\"auto left\" data-original-title=\"Note\" data-trigger=\"hover\">                                        <i class=\"fa fa-info-circle text-info\"></i>                                    </span>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"serverUpload\" class=\"col-sm-3 control-label\">Upload to remote server:</label>                            <div class=\"col-sm-3\">                                <div class=\"btn-group\">                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                        <span data-bind=\"visible: remoteUploadEnabled\">Yes</span>                                        <span data-bind=\"visible: !remoteUploadEnabled()\">No</span>                                        <span class=\"caret\"></span>                                    </button>                                    <ul class=\"dropdown-menu\" role=\"menu\">                                        <li><a href=\"#\" data-bind=\"click: remoteUploadEnabled.bind(remoteUploadEnabled, true)\">Yes</a></li>                                        <li><a href=\"#\" data-bind=\"click: remoteUploadEnabled.bind(remoteUploadEnabled, false)\">No</a></li>                                    </ul>                                </div>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: remoteUploadEnabled, visible: remoteUploadEnabled\">                            <div class=\"col-sm-3\">                                <select class=\"form-control\" data-bind=\"options: availablePeriodicExports, optionsText: \'label\', optionsValue: \'value\', value: type, disable: $root.settingsAccess.isReadOnly()\"></select>                            </div>                            <div class=\"col-sm-3\">                                <input type=\"text\" required class=\"form-control\" data-bind=\"value: mainValue, customValidity: mainValueCustomValidity, valueUpdate: \'afterkeydown\', attr: { placeholder: mainPlaceholder }, disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: isS3Bucket\">                            <label for=\"awsRemoteFolder\" class=\"col-sm-3 control-label\">Folder name:</label>                            <div class=\"col-sm-3\">                                <input id=\"awsRemoteFolder\" type=\"text\" disabled=\"disabled\" value=\"{databaseName}\" class=\"form-control\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\" />                            </div>                            <div class=\"col-sm-1\">                                <span class=\"folderHint help-cursor\" data-placement=\"auto left\" data-original-title=\"Note\" data-trigger=\"hover\">                                        <i class=\"fa fa-info-circle text-info\"></i>                                    </span>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsAccessKey\" class=\"col-sm-3 control-label\">AWS Access Key:</label>                            <div class=\"col-sm-3\">                                <input required id=\"awsAccessKey\" type=\"text\" class=\"form-control\" data-bind=\"value: awsAccessKey, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsSecretKey\" class=\"col-sm-3 control-label\">AWS Secret Key:</label>                            <div class=\"col-sm-3\">                                <input required id=\"awsSecretKey\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"value: awsSecretKey, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAwsInfoRequired, visible: additionalAwsInfoRequired\">                            <label for=\"awsRegionEndpoint\" class=\"col-sm-3 control-label\">AWS Region Endpoint:</label>                            <div class=\"col-sm-3\">                                <select required id=\"awsRegionEndpoint\" class=\"form-control\" data-bind=\"options: availableAwsRegionEndpoints, optionsText: \'label\', optionsValue: \'value\', optionsCaption: \'\', value: awsRegionEndpoint, disable: $root.settingsAccess.isReadOnly()\"></select>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"visible: additionalAzureInfoRequired\">                            <label for=\"remoteFolder\" class=\"col-sm-3 control-label\">Folder name:</label>                            <div class=\"col-sm-3\">                                <input id=\"remoteFolder\" type=\"text\" value=\"{databaseName}\" class=\"form-control\" disabled=\"disabled\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\" />                            </div>                            <div class=\"col-sm-1\">                                <span class=\"folderHint help-cursor\" data-placement=\"auto left\" data-original-title=\"Note\" data-trigger=\"hover\">                                    <i class=\"fa fa-info-circle text-info\"></i>                                </span>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAzureInfoRequired, visible: additionalAzureInfoRequired\">                            <label for=\"azureStorageAccount\" class=\"col-sm-3 control-label\">Azure Storage Account:</label>                            <div class=\"col-sm-3\">                                <input required id=\"azureStorageAccount\" type=\"text\" class=\"form-control\" data-bind=\"value: azureStorageAccount, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                        </div>                        <div class=\"form-group\" data-bind=\"if: additionalAzureInfoRequired, visible: additionalAzureInfoRequired\">                            <label for=\"azureStorageKey\" class=\"col-sm-3 control-label\">Azure Storage Key:</label>                            <div class=\"col-sm-3\">                                <input required id=\"azureStorageKey\" type=\"password\" autocomplete=\"off\" class=\"form-control\" data-bind=\"value: azureStorageKey, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"incrementalBackupInterval\" class=\"col-sm-3 control-label\">Incremental Backup Interval:</label>                            <div class=\"col-sm-1\">                                <input id=\"incrementalBackupInterval\" required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: incrementalBackupInterval, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                            <div class=\"col-sm-2\">                                <select class=\"form-control\" data-bind=\"options: availableIntervalUnits, value: incrementalBackupIntervalUnit, disable: $root.settingsAccess.isReadOnly()\"></select>                            </div>                        </div>                        <div class=\"form-group\">                            <label for=\"fullBackupInterval\" class=\"col-sm-3 control-label\">Full Backup Interval:</label>                            <div class=\"col-sm-1\">                                <input id=\"fullBackupInterval\" required type=\"number\" class=\"form-control\" min=\"0\" data-bind=\"numericValue: fullBackupInterval, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                            </div>                            <div class=\"col-sm-2\">                                <select class=\"form-control\" data-bind=\"options: availableIntervalUnits, value: fullBackupIntervalUnit, disable: $root.settingsAccess.isReadOnly()\"></select>                            </div>                        </div>                    </div>                </div>            </div>        </form>    </div></section>'; } )
define("text!views/manage/globalConfig/globalConfigQuotas.html", [], function () { return '<section>    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <form class=\"quotas-settings-form form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges.bind($root), if: !$root.settingsAccess.isForbidden()\">        <div class=\"panel panel-default\" data-bind=\"visible: !$root.canUseGlobalConfigurations()\">            <div class=\"panel-body\">                <div class=\"text-center\">                    <h3 class=\"text-info\">                        Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                    </h3>                </div>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"visible: $root.canUseGlobalConfigurations()\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: $root.isSaveEnabled(), visible: activated()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !activated(), disable: $root.settingsAccess.isReadOnly()\">                    <i class=\"fa fa-plus\"></i> Create global configuration for Quotas                </button>                <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: activated(), disable: $root.settingsAccess.isReadOnly()\">                    <i class=\"fa fa-trash\"></i> Remove global configuration for Quotas                </button>                <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>            </div>            <div class=\"panel-body\">                <div class=\"text-center\" data-bind=\"visible: !activated()\">                    <h3 class=\"text-info\">Global configuration for Quotas is not currently defined.</h3>                </div>                <div data-bind=\"if: activated()\">                    <div class=\"form-group\">                        <label class=\"col-md-3 control-label\">Maximum size:</label>                        <div class=\"col-md-9\">                            <div class=\"input-group\">                                <input required type=\"number\" min=\"0\" class=\"form-control\" data-bind=\"numericValue: maximumSize,valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                                <span class=\"input-group-addon\">MB</span>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-3 control-label\">Warning limit threshold:</label>                        <div class=\"col-md-9\">                            <div class=\"input-group\">                                <input required type=\"number\" min=\"0\" class=\"form-control\" data-bind=\"numericValue: warningLimitThreshold, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                                <span class=\"input-group-addon\">MB</span>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-3 control-label\">Maximum number of docs:</label>                        <div class=\"col-md-9\">                            <div class=\"input-group\">                                <input required type=\"number\" min=\"0\" class=\"form-control\" data-bind=\"numericValue: maxNumberOfDocs,valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                                <span class=\"input-group-addon\">docs</span>                            </div>                        </div>                    </div>                    <div class=\"form-group\">                        <label class=\"col-md-3 control-label\">Warning threshold for docs:</label>                        <div class=\"col-md-9\">                            <div class=\"input-group\">                                <input required type=\"number\" min=\"0\" class=\"form-control\" data-bind=\"numericValue: warningThresholdForDocs,valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                                <span class=\"input-group-addon\">docs</span>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/manage/globalConfig/globalConfigReplications.html", [], function () { return '<form data-bind=\"submit: $root.saveChanges\" class=\" form-horizontal\" role=\"form\">    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && !$root.canUseGlobalConfigurations()\">        <div class=\"panel-body\">            <div class=\"text-center\">                <h3 class=\"text-info\">                    Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                </h3>            </div>        </div>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && $root.canUseGlobalConfigurations()\">        <div class=\"panel-heading\">            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSetupSaveEnabled() || isConfigSaveEnabled(), visible: activated()\" title=\"Save changes (Alt+S)\" accesskey=\"S\">                <i class=\"fa fa-save\"></i>            </button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-plus\"></i> Create global configuration for Replication            </button>            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-trash\"></i> Remove global configuration for Replication            </button>            <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>        </div>        <div class=\"panel-body\">            <div class=\"text-center\" data-bind=\"visible: !activated()\">                <h3 class=\"text-info\">Global configuration for Replication is not currently defined.</h3>            </div>            <div data-bind=\"if: activated()\">                <div class=\"form-group\">                    <label class=\"col-sm-3 control-label\">Client failover behaviour</label>                    <div class=\"col-sm-9\">                        <div class=\"btn-group\" data-bind=\"with: replicationsSetup\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                <span data-bind=\"visible: !clientFailoverBehaviour()\">Let client decide</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadsFromSecondaries\'\">Allow reads from secondaries</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached\'\">Allow read from secondaries when request time SLA threshold is reached</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'AllowReadsFromSecondariesAndWritesToSecondaries\'\">Allow reads from secondaries and writes to secondaries</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'FailImmediately\'\">Fail immediately</span>                                <span data-bind=\"visible: clientFailoverBehaviour() === \'ReadFromAllServers\'\">Read from all servers</span>                                <span data-bind=\"visible: readFromAllButSwitchWhenRequestTimeSlaThresholdIsReached()\">Read from all servers but switch when request time SLA threshold is reached</span>                                <span data-bind=\"visible: readFromAllAllowWriteToSecondaries()\">Read from all servers and allow write to secondaries</span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'\')\">Let client decide</a></li>                                <li class=\"divider\"></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondaries\')\">Allow reads from secondaries</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached\')\">Allow read from secondaries when request time SLA threshold is reached</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondariesAndWritesToSecondaries\')\">Allow reads from secondaries and writes to secondaries</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'FailImmediately\')\">Fail immediately</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'ReadFromAllServers\')\">Read from all servers</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadFromSecondariesWhenRequestTimeSlaThresholdIsReached,ReadFromAllServers\')\">Read from all servers but switch when request time SLA threshold is reached</a></li>                                <li><a href=\"#\" data-bind=\"click: clientFailoverBehaviour.bind(clientFailoverBehaviour, \'AllowReadsFromSecondariesAndWritesToSecondaries,ReadFromAllServers\')\">Read from all servers and allow write to secondaries</a></li>                            </ul>                        </div>                    </div>                </div>                <div data-bind=\"visible: replicationsSetup().showRequestTimeSlaThreshold()\">                    <div class=\"form-group\">                        <label class=\"col-sm-3 control-label\">Request Time SLA Threshold</label>                        <div class=\"col-sm-2\">                            <div class=\"input-group\">                                <input type=\"number\" class=\"form-control\" data-bind=\"value: replicationsSetup().requestTimeSlaThreshold, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                <span class=\"input-group-addon\">ms</span>                            </div>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label for=\"conflictResolutionSelector\" class=\"col-sm-3 control-label\">Conflict resolution</label>                    <div class=\"col-sm-9\">                        <div class=\"btn-group\" data-bind=\"with: replicationConfig\">                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                <span data-bind=\"visible: documentConflictResolution() === \'None\'\">None</span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLocal\'\">Resolve to <strong>local</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToRemote\'\">Resolve to <strong>remote</strong></span>                                <span data-bind=\"visible: documentConflictResolution() === \'ResolveToLatest\'\">Resolve to <strong>latest</strong></span>                                <span class=\"caret\"></span>                            </button>                            <ul class=\"dropdown-menu\" role=\"menu\">                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'None\')\">None</a></li>                                <li class=\"divider\"></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToLocal\')\">Resolve to <strong>local</strong></a></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToRemote\')\">Resolve to <strong>remote</strong></a></li>                                <li><a href=\"#\" data-bind=\"click: documentConflictResolution.bind(documentConflictResolution, \'ResolveToLatest\')\">Resolve to <strong>latest</strong></a></li>                            </ul>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <div class=\"col-sm-9 col-sm-offset-3\">                        <div class=\"checkbox\">                            <input type=\"checkbox\" id=\"globalSkipIndexReplicatonForAll\" class=\"styled\" data-bind=\"checked: skipIndexReplicationForAll, disable: $root.settingsAccess.isReadOnly()\">                            <label for=\"globalSkipIndexReplicatonForAll\">Skip index replication for all destinations</label>                        </div>                    </div>                </div>                <div class=\"form-group\">                    <label class=\"col-sm-3 control-label\">Replication destinations</label>                    <div class=\"col-sm-9\">                        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewDestination, disable: $root.settingsAccess.isReadOnly()\" title=\"Add a replication destination (Alt+N)\" accesskey=\"N\">                            <i class=\"fa fa-plus\"></i> Add destination                        </button>                        <br />                        <br />                        <div data-bind=\"with: replicationsSetup\">                            <p data-bind=\"visible: destinations().length === 0\">                                <i class=\"fa fa-warning text-warning\"></i> <span class=\"text-muted\">You haven\'t created any replication destinations.</span>                            </p>                            <div data-bind=\"foreach: destinations\">                                <div class=\"panel replication-destination-card\" data-bind=\"css: { \'panel-warning\': disabled(), \'panel-default\': !disabled() }\">                                    <div class=\"panel-heading\">                                        <h3 class=\"panel-title\">                                            <span data-bind=\"text: name\"></span>                                            <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\"><i class=\"fa fa-times\" data-bind=\"click: $root.removeDestination.bind($root)\" title=\"Remove this replication destination\"></i></button>                                        </h3>                                    </div>                                    <div class=\"panel-body\">                                        <div class=\"form-group\">                                            <div class=\"col-sm-offset-4 col-sm-7\">                                                <div class=\"btn-group\">                                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                                        <span data-bind=\"visible: !disabled()\">Enabled</span>                                                        <span data-bind=\"visible: disabled\">Disabled</span>                                                        <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <li><a href=\"#\" data-bind=\"click: enable\">Enabled</a></li>                                                        <li><a href=\"#\" data-bind=\"click: disable\">Disabled</a></li>                                                    </ul>                                                </div>                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-4 control-label\">Url</label>                                            <div class=\"col-sm-7\">                                                <input required type=\"url\" class=\"form-control\" title=\"The address of the server to replicate to\" data-bind=\"value: url, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-4 control-label\">Database</label>                                            <div class=\"col-sm-6\">                                                <input required type=\"text\" class=\"form-control\" title=\"The name of the database on the destination server to replicate to\" disabled=\"disabled\" value=\"{databaseName}\" />                                            </div>                                            <div class=\"col-sm-1\">                                                <span class=\"dbNameHint help-cursor\" data-placement=\"auto left\" data-original-title=\"Note\" data-trigger=\"hover\">                                                    <i class=\"fa fa-info-circle text-info\"></i>                                                </span>                                            </div>                                        </div>                                        <div class=\"form-group\">                                            <label class=\"col-sm-4 control-label\">Credentials</label>                                            <div class=\"col-sm-7\">                                                <div class=\"btn-group\">                                                    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                                        <span data-bind=\"visible: !isUserCredentials() && !isApiKeyCredentials()\">None</span>                                                        <span data-bind=\"visible: isUserCredentials\">User</span>                                                        <span data-bind=\"visible: isApiKeyCredentials\">API key</span>                                                        <span class=\"caret\"></span>                                                    </button>                                                    <ul class=\"dropdown-menu\" role=\"menu\">                                                        <li><a href=\"#\" data-bind=\"click: useNoCredentials\">None</a></li>                                                        <li class=\"divider\"></li>                                                        <li><a href=\"#\" data-bind=\"click: useUserCredentials\">User</a></li>                                                        <li><a href=\"#\" data-bind=\"click: useApiKeyCredentials\">API key</a></li>                                                    </ul>                                                </div>                                            </div>                                        </div>                                        <div data-bind=\"if: isUserCredentials, visible: isUserCredentials\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">User Name</label>                                                <div class=\"col-sm-7\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: username, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">Password</label>                                                <div class=\"col-sm-7\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: password, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">Domain</label>                                                <div class=\"col-sm-7\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: domain, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                                </div>                                            </div>                                        </div>                                        <div data-bind=\"if: isApiKeyCredentials, visible: isApiKeyCredentials\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">API Key</label>                                                <div class=\"col-sm-7\">                                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: apiKey, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                                </div>                                            </div>                                        </div>                                        <a data-bind=\"click: toggleIsAdvancedShows\" class=\"advanced-replication-settings\">                                            Advanced Options                                        </a>                                        <div data-bind=\"visible: false\">                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">Client Visible Url</label>                                                <div class=\"col-sm-7\">                                                    <input type=\"text\" class=\"form-control\" data-bind=\"value: clientVisibleUrl, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\" />                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <div class=\"col-sm-7 col-sm-offset-4\">                                                    <div class=\"checkbox\">                                                        <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: skipIndexReplication, disable: $root.settingsAccess.isReadOnly(), attr: { id: \'skipIndexReplicaton_\' + $index() } \">                                                        <label data-bind=\"attr: { for: \'skipIndexReplicaton_\' + $index() }\">Skip Index Replication</label>                                                    </div>                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">Failover</label>                                                <div class=\"col-sm-7\">                                                    <div class=\"btn-group\">                                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                                            <span data-bind=\"visible: !ignoredClient()\">Include</span>                                                            <span data-bind=\"visible: ignoredClient\">Skip</span>                                                            <span class=\"caret\"></span>                                                        </button>                                                        <ul class=\"dropdown-menu\" role=\"menu\">                                                            <li><a href=\"#\" data-bind=\"click: includeFailover\">Include</a></li>                                                            <li><a href=\"#\" data-bind=\"click: skipFailover\">Skip</a></li>                                                        </ul>                                                    </div>                                                </div>                                            </div>                                            <div class=\"form-group\">                                                <label class=\"col-sm-4 control-label\">Transitive Replication</label>                                                <div class=\"col-sm-7\">                                                    <div class=\"btn-group\">                                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                                            <span data-bind=\"visible: transitiveReplicationBehavior() === \'None\'\">Changed only</span>                                                            <span data-bind=\"visible: transitiveReplicationBehavior() === \'Replicate\'\">Changed and replicated</span>                                                            <span class=\"caret\"></span>                                                        </button>                                                        <ul class=\"dropdown-menu\" role=\"menu\">                                                            <li><a href=\"#\" data-bind=\"click: transitiveReplicationBehavior.bind(transitiveReplicationBehavior, \'None\')\">Changed only</a></li>                                                            <li><a href=\"#\" data-bind=\"click: transitiveReplicationBehavior.bind(transitiveReplicationBehavior, \'Replicate\')\">Changed and replicated</a></li>                                                        </ul>                                                    </div>                                                </div>                                            </div>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/globalConfig/globalConfigSqlReplication.html", [], function () { return '<form class=\"form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges\">    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && !$root.canUseGlobalConfigurations()\">        <div class=\"panel-body\">            <div class=\"text-center\">                <h3 class=\"text-info\">                    Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                </h3>            </div>        </div>    </div>    <div class=\"panel panel-default\" data-bind=\"visible: !$root.settingsAccess.isForbidden() && $root.canUseGlobalConfigurations()\">        <div class=\"panel-heading\">            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"enable: isSaveEnabled, visible: activated()\" title=\"Save changes (Alt+S)\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-plus\"></i> Create global configuration for SQL Replication            </button>            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: activated(), disable: $root.settingsAccess.isReadOnly()\">                <i class=\"fa fa-trash\"></i> Remove global configuration for SQL Replication            </button>            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: addSqlReplicationConnection, visible: activated(), disable: $root.settingsAccess.isReadOnly()\" title=\"Add connection string (Alt+N)\" accesskey=\"N\">                <i class=\"fa fa-plus\"></i> New connection string            </button>            <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>        </div>        <div class=\"panel-body\" data-bind=\"with: connections\">            <div class=\"text-center\" data-bind=\"visible: !$root.activated()\">                <h3 class=\"text-info\">Global configuration for SQL Replication is not currently defined.</h3>            </div>            <div class=\"text-center\" data-bind=\"visible: predefinedConnections().length === 0 && $root.activated()\">                <h3 class=\"text-info\">You\'ve got no SQL replication connections set up.</h3>            </div>            <div data-bind=\"foreach: predefinedConnections\">                <div class=\"col-sm-6\">                    <div class=\"sql-replication-connection-card panel panel-default\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span data-bind=\"text: name() || \'[untitled]\'\"></span>                                <button type=\"button\" class=\"close\" aria-hidden=\"true\" data-bind=\"click: $root.removeSqlReplicationConnection.bind($root, $data), visible: !$root.settingsAccess.isReadOnly()\" title=\"Remove this connection\"><i class=\"fa fa-times\"></i></button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Name</label>                                <div class=\"col-sm-7\">                                    <input required name=\"name\" class=\"form-control\" type=\"text\" data-bind=\"value:name, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"/>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">SQL Provider</label>                                <div class=\"col-sm-7\">                                    <select tabindex=\"5\" required class=\"form-control\" data-bind=\"disable: $root.settingsAccess.isReadOnly(), event:{change:function(val,event){$root.providerChanged($data,event);return true;}},options: [\'System.Data.SqlClient\',\'System.Data.SqlServerCe.4.0\',\'System.Data.OleDb\',\'System.Data.OracleClient\',\'MySql.Data.MySqlClient\',\'System.Data.SqlServerCe.3.5\',\'Npgsql\'], optionsCaption: \'\', value: factoryName\"></select>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Connection String</label>                                <div class=\"col-sm-7\">                                    <textarea tabindex=\"6\" rows=\"4\" required class=\"form-control\" data-bind=\"value: connectionString, valueUpdate: \'afterkeydown\', disable: $root.settingsAccess.isReadOnly()\"></textarea>                                </div>                            </div>                            <div class=\"form-group\">                                <div class=\"col-sm-7 col-sm-offset-4\">                                    <button type=\"button\" class=\"btn btn-default\" data-bind=\"enable: connectionTestState() === CONNECTION_STATE_STAND_BY, click: testConnection.bind($data, $root.activeDatabase)\">Test connection</button>                                    <br/>                                    <div class=\"progress progress-striped\" data-bind=\"visible:connectionTestState() !== CONNECTION_STATE_STAND_BY\">                                        <div class=\"progress-bar progress-bar-info\" data-bind=\"css: {\'active\':true}\" role=\"progressbar\" aria-valuenow=\"100\" aria-valuemin=\"0\" aria-valuemax=\"100\" style=\"width: 100%; float: left;\">                                            <span class=\"sr-only\" data-bind=\"text:connectionTestState() === CONNECTION_STATE_CONNECTING?\'Connecting\':\'Connected\'\"></span>                                        </div>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </div></form>'; } )
define("text!views/manage/globalConfig/globalConfigVersioning.html", [], function () { return '<section>    <div data-bind=\"visible: $root.settingsAccess.isForbidden()\" class=\"text-center\">        <h4 class=\"page-header text-info\">You are not authorized to view this page!</h4>    </div>    <form class=\"versioning-settings-form form-horizontal\" role=\"form\" data-bind=\"submit: $root.saveChanges.bind($root), if: !$root.settingsAccess.isForbidden()\">        <div class=\"panel panel-default\" data-bind=\"visible: !$root.canUseGlobalConfigurations()\">            <div class=\"panel-body\">                <div class=\"text-center\">                    <h3 class=\"text-info\">                        Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a>.                    </h3>                </div>            </div>        </div>        <div class=\"panel panel-default\" data-bind=\"visible: $root.canUseGlobalConfigurations()\">            <div class=\"panel-heading\">                <button type=\"submit\" class=\"btn btn-primary\" title=\"Save Changes(Alt+S)\" data-bind=\"enable: isSaveEnabled, visible: activated()\" accesskey=\"S\"><i class=\"fa fa-save\"></i></button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $root.activateConfig.bind($root), visible: !activated(), disable: $root.settingsAccess.isReadOnly()\">                    <i class=\"fa fa-plus\"></i> Create global configuration for Versioning                </button>                <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: $root.disactivateConfig.bind($root), visible: activated(), disable: $root.settingsAccess.isReadOnly()\">                    <i class=\"fa fa-trash\"></i> Remove global configuration for Versioning                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: createNewVersioning, visible: activated(), disable: $root.settingsAccess.isReadOnly()\" title=\"Add a versioning (Alt+N)\" accesskey=\"N\">                    <i class=\"fa fa-plus\"></i> Add versioning                </button>                <div class=\"pull-right padd-top-10 text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</div>            </div>            <div class=\"panel-body\">                <div class=\"text-center\" data-bind=\"visible: !activated()\">                    <h3 class=\"text-info\">Global configuration for Versioning is not currently defined.</h3>                </div>                <div data-bind=\"foreach: versionings \">                    <div class=\"panel versioning-card\" data-bind=\"css: { \'panel-warning\': disabled, \'panel-default\': !disabled() }\">                        <div class=\"panel-heading\">                            <h3 class=\"panel-title\">                                <span> Versioning setup <span data-bind=\"visible: !!collection() && collection().length>0\">for</span> <span data-bind=\"text: collection\"></span></span>                                <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\" data-bind=\"click: $root.removeVersioning.bind($root), visible: removable() && !$root.settingsAccess.isReadOnly()\" title=\"Remove this versioning\"><i class=\"fa fa-times\"></i></button>                            </h3>                        </div>                        <div class=\"panel-body\">                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Collection</label>                                <div class=\"col-sm-8\">                                    <input required type=\"text\" class=\"form-control\" data-bind=\"value: collection, enable: removable() && !fromDatabase() && !$root.settingsAccess.isReadOnly(), valueUpdate: \'afterkeyup\'\"/>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Max Revisions</label>                                <div class=\"col-sm-8\">                                    <input required type=\"number\" class=\"form-control\" data-bind=\"numericValue: maxRevisions, valueUpdate: \'afterkeyup\', disable: $root.settingsAccess.isReadOnly()\"/>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Excluded</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                            <span data-bind=\"visible: !exclude()\">Include</span>                                            <span data-bind=\"visible: exclude\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: exclude.bind($root, true)\">Exclude</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Excluded unless explicit</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                            <span data-bind=\"visible: !excludeUnlessExplicit()\">Include</span>                                            <span data-bind=\"visible: excludeUnlessExplicit\">Exclude</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, false)\">Include</a></li>                                            <li><a href=\"#\" data-bind=\"click: excludeUnlessExplicit.bind($root, true)\">Exclude</a></li>                                        </ul>                                    </div>                                </div>                            </div>                            <div class=\"form-group\">                                <label class=\"col-sm-4 control-label\">Purge on delete</label>                                <div class=\"col-sm-7\">                                    <div class=\"btn-group\">                                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" data-bind=\"disable: $root.settingsAccess.isReadOnly()\">                                            <span data-bind=\"visible: !purgeOnDelete()\">No</span>                                            <span data-bind=\"visible: purgeOnDelete\">Yes</span>                                            <span class=\"caret\"></span>                                        </button>                                        <ul class=\"dropdown-menu\" role=\"menu\">                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, false)\">No</a></li>                                            <li><a href=\"#\" data-bind=\"click: purgeOnDelete.bind($root, true)\">Yes</a></li>                                        </ul>                                    </div>                                </div>                            </div>                        </div>                    </div>                </div>            </div>        </div>    </form></section>'; } )
define("text!views/resources/createCounterStorage.html", [], function () { return '<div class=\"col-md-7\">    <div class=\"form-group\">        <label for=\"counterStorageName\">Name</label>        <input autofocus data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" type=\"text\" class=\"form-control\" id=\"counterStorageName\" autocomplete=\"off\" placeholder=\"Counter Storage Name\">    </div>    <div class=\"panel-group\" id=\"accordion\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#csAdvancedOptions\">                        Advanced Settings                    </a>                </h4>            </div>            <div id=\"csAdvancedOptions\" class=\"panel-collapse collapse\">                <div class=\"panel-body\">                    <div class=\"alert alert-danger\" role=\"alert\" data-bind=\"visible: voronWarningVisible\">                        Running Voron in 32-bit mode is error prone. Set Raven/Voron/AllowOn32Bits in configuration to allow Voron on 32-bit.                    </div>                    <div class=\"form-group\">                        <label for=\"counterStoragePath\">Path</label>                        <input data-bind=\"value: resourcePath, customValidity: pathCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/Counters/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"counterStoragePath\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\" data-bind=\"visible: tempPathVisible\">                        <label for=\"counterStorageTempPath\">Temp</label>                        <input data-bind=\"value: resourceTempPath, customValidity: tempCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"counterStorageTempPath\" autocomplete=\"off\">                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/resources/createDatabase.html", [], function () { return '<div class=\"col-md-7\">    <div class=\"form-group\">        <label for=\"databaseName\">Name</label>        <input autofocus data-bind=\"value: resourceName, customValidity: nameCustomValidityError, valueUpdate: \'afterkeydown\'\" type=\"text\" class=\" autofocus form-control\" id=\"databaseName\" autocomplete=\"off\" placeholder=\"Database Name\">    </div>    <div class=\"panel-group\" id=\"accordion\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#dbAdvancedOptions\">                        Advanced Settings                    </a>                </h4>            </div>            <div id=\"dbAdvancedOptions\" class=\"panel-collapse collapse\">                <div class=\"panel-body\">                    <div class=\"form-group\">                        <label for=\"storageEngine\">Storage Engine</label>                        <select data-bind=\"value: storageEngine\" class=\"form-control\" id=\"storageEngine\">                            <option value=\"\">Default</option>                            <option value=\"esent\">Esent</option>                            <option value=\"voron\">Voron</option>                        </select>                    </div>                    <div class=\"alert alert-danger\" role=\"alert\" data-bind=\"visible: voronWarningVisible\">                        Running Voron in 32-bit mode is error prone. Set Raven/Voron/AllowOn32Bits in configuration to allow Voron on 32-bit.                    </div>                    <div class=\"form-group\">                        <label for=\"databasePath\">Path</label>                        <input data-bind=\"value: resourcePath, customValidity: pathCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"databasePath\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\">                        <label for=\"databaseLogs\">Logs</label>                        <input data-bind=\"value: logsPath, customValidity: logsCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"databaseLogs\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\">                        <label for=\"databaseIndexes\">Indexes</label>                        <input data-bind=\"value: databaseIndexesPath, customValidity: indexesCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + (!!resourceName() ? resourceName() : \'{Database Name}\') + \'/Indexes/\' }\" type=\"text\" class=\"form-control\" id=\"databaseIndexes\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\" data-bind=\"visible: tempPathVisible\">                        <label for=\"databaseTempPath\">Temp</label>                        <input data-bind=\"value: resourceTempPath, customValidity: tempCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"databaseTempPath\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\">                        <div class=\"checkbox\">                            <input type=\"checkbox\" class=\"styled\" id=\"incremental_backup_db\" data-bind=\"checked: isIncrementalBackupChecked\">                            <label for=\"incremental_backup_db\">Incremental Backup</label>                        </div>                        <label for=\"alertTimeout\" data-bind=\"visible: isIncrementalBackupChecked\">Alert timeout in hours:</label>                        <select data-bind=\"value: alertTimeout, visible: isIncrementalBackupChecked\" title=\"Sets the span of time allowed before issuing an alert for missing backup(in hours).\" class=\"form-control\" id=\"alertTimeout\">                            <option value=\"4\">4</option>                            <option value=\"8\">8</option>                            <option value=\"12\">12</option>                            <option value=\"\">24</option>                            <option value=\"48\">48</option>                            <option value=\"72\">72</option>                        </select>                        <label for=\"alertRecurringTimeout\" data-bind=\"visible: isIncrementalBackupChecked\">Alert recurring Timeout in days:</label>                        <select data-bind=\"value: alertRecurringTimeout, visible: isIncrementalBackupChecked\" title=\"Sets the span of time allowed before issuing multiple alerts on the same missed backup(in days).\" class=\"form-control\" id=\"alertRecurringTimeout\">                            <option value=\"1\">1</option>                            <option value=\"2\">2</option>                            <option value=\"4\">4</option>                            <option value=\"\">7</option>                            <option value=\"14\">14</option>                        </select>                    </div>                    <div class=\"form-group\" data-bind=\"if: isClusterWideVisible, visible: isClusterWideVisible\">                        <div class=\"checkbox\">                            <input type=\"checkbox\" class=\"styled\" id=\"clusterWide\" data-bind=\"checked: isClusterWideChecked\">                            <label for=\"clusterWide\">Cluster-wide</label>                        </div>                    </div>                    <div class=\"alert alert-info\" data-bind=\"if: isClusterWideVisible, visible: isClusterWideVisible() && !isClusterWideChecked()\">                        Notice: Global configuration will be inherited by newly created database even when you unselect cluster-wide database.                    </div>                </div>            </div>        </div>    </div></div><div class=\"col-md-5\">    <div>        <label>Bundles</label>    </div>    <div class=\"btn-group-vertical\" data-toggle=\"buttons-checkbox\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleCompressionBundle, css: { \'license-inactive\': !isBundleActive(\'compression\') }\">            <i data-bind=\"css: {\'fa-check\': isCompressionBundleEnabled }\" class=\"fa fa-fw\"></i> Compression        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleEncryptionBundle, css: { \'license-inactive\': !isBundleActive(\'encryption\') }\">            <i data-bind=\"css: {\'fa-check\': isEncryptionBundleEnabled }\" class=\"fa fa-fw\"></i> Encryption        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleExpirationBundle, css: { \'license-inactive\': !isBundleActive(\'documentExpiration\') }\">            <i data-bind=\"css: {\'fa-check\': isExpirationBundleEnabled }\" class=\"fa fa-fw\"></i> Expiration        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: togglePeriodicExportBundle, css: { \'license-inactive\': !isBundleActive(\'periodicBackup\') }\">            <i data-bind=\"css: {\'fa-check\': isPeriodicExportBundleEnabled }\" class=\"fa fa-fw\"></i> Periodic Export        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleQuotasBundle, css: { \'license-inactive\': !isBundleActive(\'quotas\') }\">            <i data-bind=\"css: {\'fa-check\': isQuotasBundleEnabled }\" class=\"fa fa-fw\"></i> Quotas        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleReplicationBundle, disable: replicationBundleChangeDisabled(), css: { \'license-inactive\': !isBundleActive(\'replication\') }\">            <i data-bind=\"css: {\'fa-check\': isReplicationBundleEnabled }\" class=\"fa fa-fw\"></i> Replication        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleScriptedIndexBundle\">            <i data-bind=\"css: {\'fa-check\': isScriptedIndexBundleEnabled }\" class=\"fa fa-fw\"></i> Scripted Index        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleSqlReplicationBundle\">            <i data-bind=\"css: {\'fa-check\': isSqlReplicationBundleEnabled }\" class=\"fa fa-fw\"></i> SQL Replication        </button>        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleVersioningBundle, css: { \'license-inactive\': !isBundleActive(\'versioning\') }\">            <i data-bind=\"css: {\'fa-check\': isVersioningBundleEnabled }\" class=\"fa fa-fw\"></i> Versioning        </button>        <!-- ko foreach: customBundles -->            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: $parent.toggleCustomBundle.bind($parent, $data)\">                <i data-bind=\"css: {\'fa-check\': $parent.selectedCustomBundles().indexOf($data)>=0 }\" class=\"fa fa-fw\"></i> <span data-bind=\"text: $data\"></span>            </button>        <!-- /ko -->    </div></div>          '; } )
define("text!views/resources/createEncryption.html", [], function () { return '<div id=\"CreateEncriptionDialog\" class=\"messageBox\" tabindex=\"-1\">    <form data-bind=\"submit: save\" class=\"form-horizontal\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>            <h4 class=\"modal-title\">Create Encryption</h4>        </div>        <div class=\"modal-body\">            <div class=\"form-group\">                <label for=\"keyName\" class=\"col-sm-4 control-label\">Select Key <br /><small class=\"text-muted\">(Base64 Encoding)</small></label>                <div class=\"col-sm-8\">                    <div class=\"input-group\">                        <input required pattern=\"^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{4})([=]{1,2})?$\" type=\"text\" class=\"form-control\" id=\"key\" placeholder=\"Key\" data-bind=\"value: key\" autocomplete=\"off\" />                        <span class=\"input-group-btn\">                            <button class=\"btn btn-default\" type=\"button\" data-bind=\"click: refresh_encryption\">                                <i class=\"fa fa-refresh\"></i>                            </button>                        </span>                    </div>                    <p class=\"help-block\">                        <!--You can use the auto generated key or enter one of your own.-->                        <br />                        <i class=\"fa fa-warning text-warning\"></i> Save this key in a safe place. If you lose this key you could lose access to your data.                    </p>                </div>            </div>            <div class=\"form-group\">                <label for=\"encryptionAlgorithm\" class=\"col-sm-4 control-label\">Encryption Algorithm</label>                <div class=\"col-sm-8\">                    <!--<select class=\"form-control\" required data-bind=\"options: [\'DES\', \'RC2\', \'Rijndael\', \'Triple DES\'], value: encryptionAlgorithm, valueAllowUnset: false, hasFocus: algorithmFocus\"></select>-->                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"text: encryptionAlgorithm\"></span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: [\'DES\', \'RC2\', \'Rijndael\', \'Triple DES\']\">                            <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"text: $data, click: $root.encryptionAlgorithm.bind($root.encryptionAlgorithm, $data)\"></a></li>                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <label for=\"encryptionBits\" class=\"col-sm-4 control-label\">Encryption Key Bits</label>                <div class=\"col-sm-8\">                    <!--<select class=\"form-control\" required data-bind=\"options: [\'128\', \'192\', \'256\'], value: encryptionBits, valueAllowUnset: false, hasFocus: bitsFocus\"></select>-->                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"text: encryptionBits\"></span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\" data-bind=\"foreach: [\'128\', \'192\', \'256\']\">                            <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"text: $data, click: $root.encryptionBits.bind($root.encryptionBits, $data)\"></a></li>                        </ul>                    </div>                </div>            </div>            <div class=\"form-group\">                <label for=\"encryptedIndexes\" class=\"col-sm-4 control-label\">Indexes</label>                <div class=\"col-sm-8\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"visible: isEncryptedIndexes\">Encrypt</span>                            <span data-bind=\"visible: !isEncryptedIndexes()\">Don\'t encrypt</span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: isEncryptedIndexes.bind(isEncryptedIndexes, true)\">Encrypt</a></li>                            <li><a href=\"#\" style=\"text-decoration: none;\" data-bind=\"click: isEncryptedIndexes.bind(isEncryptedIndexes, false)\">Unencrypt</a></li>                        </ul>                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\">Save</button>        </div>    </form></div>'; } )
define("text!views/resources/createEncryptionConfirmation.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h3 class=\"modal-title\">Save Your Encryption Key</h3>    </div>    <div class=\"modal-body\">        <h4>            <strong>Save this key in a safe place.</strong>        </h4>        <div>            <input readonly=\"readonly\" type=\"text\" class=\"form-control input-lg col-sm-9\" id=\"key\" placeholder=\"Key\" autofocus=\"\" data-bind=\"value: key, click: clickKey\">        </div>        <div class=\"clearfix\"></div>        <br />        <p>            <i class=\"fa fa-warning text-warning\"></i> If you lose your key, you could lose access to your data.        </p>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: ok\">OK</button>    </div></div>'; } )
define("text!views/resources/createFilesystem.html", [], function () { return '<div class=\"col-md-7\">    <div class=\"form-group\">        <label for=\"fileSystemName\">Name</label>        <input autofocus data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" type=\"text\" class=\"form-control\" id=\"fileSystemName\" autocomplete=\"off\" placeholder=\"File System Name\">    </div>    <div class=\"panel-group\" id=\"accordion\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#fsAdvancedOptions\">                        Advanced Settings                    </a>                </h4>            </div>            <div id=\"fsAdvancedOptions\" class=\"panel-collapse collapse\">                <div class=\"panel-body\">                    <div class=\"form-group\">                        <label for=\"storageEngine\">Storage Engine</label>                        <select data-bind=\"value: storageEngine\" class=\"form-control\" id=\"storageEngine\">                            <option value=\"\">Default</option>                            <option value=\"esent\">Esent</option>                            <option value=\"voron\">Voron</option>                        </select>                    </div>                    <div class=\"alert alert-danger\" role=\"alert\" data-bind=\"visible: voronWarningVisible\">                        Running Voron in 32-bit mode is error prone. Set Raven/Voron/AllowOn32Bits in configuration to allow Voron on 32-bit.                    </div>                    <div class=\"form-group\">                        <label for=\"filesystemPath\">Path</label>                        <input data-bind=\"value: resourcePath, customValidity: pathCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/FileSystems/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"filesystemPath\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\">                        <label for=\"filesystemLogs\">Logs</label>                        <input data-bind=\"value: logsPath, customValidity: logsCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/FileSystems/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"filesystemLogs\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\" data-bind=\"visible: tempPathVisible\">                        <label for=\"fileSystemTempPath\">Temp</label>                        <input data-bind=\"value: resourceTempPath, customValidity: tempCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"fileSystemTempPath\" autocomplete=\"off\">                    </div>                </div>            </div>        </div>    </div></div><div class=\"col-md-5\">    <div>        <label>Bundles</label>    </div>    <div class=\"btn-group-vertical\" data-toggle=\"buttons-checkbox\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleEncryptionBundle, css: { \'license-inactive\': !isBundleActive(\'encryption\') }\">            <i data-bind=\"css: { \'fa-check\': isEncryptionBundleEnabled }\" class=\"fa fa-fw\"></i> Encryption        </button>         <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleVersioningBundle, css: { \'license-inactive\': !isBundleActive(\'versioning\') }\">            <i data-bind=\"css: { \'fa-check\': isVersioningBundleEnabled }\" class=\"fa fa-fw\"></i> Versioning        </button>    </div></div>'; } )
define("text!views/resources/createResource.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"min-width: 500px;\">    <form id=\"createDbForm\" data-bind=\"submit: nextOrCreate\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>            <h4 class=\"modal-title\">Create                 <span data-bind=\"text: resourceType\" style=\"font-weight: bold\"></span>            </h4>        </div>        <div class=\"modal-body\" style=\"max-height: 600px; overflow: auto\">            <div class=\"form-group\">                <label>Resource type</label>                <div class=\"text-center\">                    <div class=\"btn-group\" data-toggle=\"buttons\">                        <!--ko foreach: resourceTypes-->                        <label class=\"btn\" data-bind=\"attr: { title: $data.title }, css: { \'btn-primary\': $data.resourceType === $parent.checkedResource(), \'btn-default\': $data.resourceType !== $parent.checkedResource() }\">                            <input type=\"radio\" name=\"radio\" data-bind=\"checkedValue: $data.resourceType, bsChecked: $parent.checkedResource\">                            &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i data-bind=\"attr: { class: $data.iconName }\">&nbsp;&nbsp;&nbsp;</i><br/>                        </label>                        <!-- /ko -->                    </div>                </div>            </div>            <div id=\"dbContainer\" class=\"row\" style=\"min-height: 460px;\" data-bind=\"with: createDatabasePart, visible: checkedResource() == databaseType\">                <div data-bind=\"compose: \'resources/createDatabase.html\'\"></div>            </div>            <div id=\"fsContainer\" class=\"row\" style=\"min-height: 460px;\" data-bind=\"with: createFileSystemPart, visible: checkedResource() == fileSystemType\">                <div data-bind=\"compose: \'resources/createFilesystem.html\'\"></div>            </div>            <div id=\"csContainer\" class=\"row\" style=\"min-height: 460px;\" data-bind=\"with: createCounterStoragePart, visible: checkedResource() == counterStorageType\">                <div data-bind=\"compose: \'resources/createCounterStorage.html\'\"></div>            </div>            <div id=\"tsContainer\" class=\"row\" style=\"min-height: 460px;\" data-bind=\"with: createTimeSeriesPart, visible: checkedResource() == timeSeriesType\">                <div data-bind=\"compose: \'resources/createTimeSeries.html\'\"></div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\">Create</button>        </div>    </form></div>'; } )
define("text!views/resources/createTimeSeries.html", [], function () { return '<div class=\"col-md-7\">    <div class=\"form-group\">        <label for=\"timeSeriesName\">Name</label>        <input autofocus data-bind=\"value: resourceName, valueUpdate: \'afterkeydown\', customValidity: nameCustomValidityError\" type=\"text\" class=\"form-control\" id=\"timeSeriesName\" autocomplete=\"off\" placeholder=\"Time Series Name\">    </div>    <div class=\"panel-group\" id=\"accordion\">        <div class=\"panel panel-default\">            <div class=\"panel-heading\">                <h4 class=\"panel-title\">                    <a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#tsAdvancedOptions\">                        Advanced Settings                    </a>                </h4>            </div>            <div id=\"tsAdvancedOptions\" class=\"panel-collapse collapse\">                <div class=\"panel-body\">                    <div class=\"alert alert-danger\" role=\"alert\" data-bind=\"visible: voronWarningVisible\">                        Running Voron in 32-bit mode is error prone. Set Raven/Voron/AllowOn32Bits in configuration to allow Voron on 32-bit.                    </div>                    <div class=\"form-group\">                        <label for=\"timeSeriesPath\">Path</label>                        <input data-bind=\"value: resourcePath, customValidity: pathCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/TimeSeries/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"timeSeriesPath\" autocomplete=\"off\">                    </div>                    <div class=\"form-group\" data-bind=\"visible: tempPathVisible\">                        <label for=\"timeSeriesTempPath\">Temp</label>                        <input data-bind=\"value: resourceTempPath, customValidity: tempCustomValidityError, valueUpdate: \'afterkeydown\', attr: { placeholder: \'~/\' + resourceName() }\" type=\"text\" class=\"form-control\" id=\"timeSeriesTempPath\" autocomplete=\"off\">                    </div>                </div>            </div>        </div>    </div></div>'; } )
define("text!views/resources/databaseSettingsDialog.html", [], function () { return '<style>    .settings-container {        overflow-x: hidden;        overflow-y: auto;        min-width: 75vw;        min-height: 60vh;        max-height: 60vh;    }</style><div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Database Settings</h4>    </div>    <div class=\"modal-body\">        <div class=\"settings-container row\">            <div class=\"col-md-2\">                <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: routes\">                    <li data-bind=\"css: { active: $root.isActive(moduleId) }\">                        <a href=\"#\" data-bind=\"text: title, click: $root.showView.bind($root, moduleId)\"></a>                    </li>                </ul>            </div>            <div class=\"col-md-10 composite-view\" data-bind=\"compose: content\"></div>            <div class=\"clearfix\"></div>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Close</button>    </div></div>'; } )
define("text!views/resources/deleteResourceConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\">        <div data-bind=\"visible: resourcesToDelete().length === 1\">            <span>You\'re deleting </span>            <strong data-bind=\"with: resourcesToDelete()[0]\">                <span data-bind=\"text: name\"></span>            </strong>        </div>        <div data-bind=\"visible: resourcesToDelete().length > 1\">            <span>You\'re deleting </span>            <strong data-bind=\"text: resourcesToDelete().length\"></strong>             resources:            <br />            <ul data-bind=\"foreach: resourcesToDelete\" style=\"max-height: 100px; overflow-y: auto;\">                <li><span data-bind=\"text: name\"></span> (<span data-bind=\"text: fullTypeName\"></span>)</li>            </ul>        </div>        <div data-bind=\"if: isDeletingDatabase\">            <div data-bind=\"visible: resourcesToDelete().length === 1\">                You can <a href=\"#\" data-bind=\"click: exportDatabase\">export the database</a> if needed.<br />            </div>            <div data-bind=\"visible: resourcesToDelete().length > 1\">                You can <a href=\"#\" data-bind=\"click: exportDatabase\">export these databases</a> if needed.<br />            </div>        </div>        <br />        <form class=\"form-horizontal\">            <div class=\"form-group\">                <label class=\"col-sm-4 control-label\">Delete options</label>                <div class=\"col-sm-8\">                    <div class=\"btn-group\">                        <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">                            <span data-bind=\"visible: isKeepingFiles\">Keep files on disk</span>                            <span data-bind=\"visible: !isKeepingFiles()\">Delete everything</span>                            <span class=\"caret\"></span>                        </button>                        <ul class=\"dropdown-menu\" role=\"menu\">                            <li><a href=\"#\" data-bind=\"click: keepFiles\" title=\"Keep the physical database files on disk\">Keep files on disk</a></li>                            <li><a href=\"#\" data-bind=\"click: deleteEverything\" title=\"Delete all the on-disk files for this database\">Delete everything</a></li>                        </ul>                    </div>                    <p class=\"help-block\" data-bind=\"visible: resourcesToDelete().length > 1\">                        <i class=\"fa fa-info-circle\"></i> Delete options will be applied to all of the deleted resources.                    </p>                </div>            </div>        </form>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteDatabase\">Yep, delete</button>    </div></div>'; } )
define("text!views/resources/disableResourceToggleConfirm.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\">        <div data-bind=\"visible: resourcesToDisable().length === 1\">            <span data-bind=\"text: deletionText\"></span>            <strong data-bind=\"with: resourcesToDisable()[0]\">                <span data-bind=\"text: name\"></span>            </strong>        </div>        <div data-bind=\"visible: resourcesToDisable().length > 1\">            <span data-bind=\"text: deletionText\"></span>            <strong data-bind=\"text: resourcesToDisable().length\"></strong>            resources            <br />            <ul data-bind=\"foreach: resourcesToDisable\" style=\"max-height: 100px; overflow-y: auto;\">                <li><span data-bind=\"text: name\"></span> (<span data-bind=\"text: fullTypeName\"></span>)</li>            </ul>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: toggleDisableReources, text: confirmDeletionText\"></button>    </div></div>'; } )
define("text!views/resources/filesystemSettingsDialog.html", [], function () { return '<style>    .settings-container {        overflow-x: hidden;        overflow-y: auto;        min-width: 75vw;        min-height: 60vh;        max-height: 60vh;    }</style><div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: close\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Filesystem Settings</h4>    </div>    <div class=\"modal-body\">        <div class=\"settings-container row\">            <div class=\"col-md-2\">                <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: routes\">                    <li data-bind=\"css: { active: $root.isActive(moduleId) }\">                        <a href=\"#\" data-bind=\"text: title, click: $root.showView.bind($root, moduleId)\"></a>                    </li>                </ul>            </div>            <div class=\"col-md-10 composite-view\" data-bind=\"compose: content\"></div>            <div class=\"clearfix\"></div>        </div>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: close\">Close</button>    </div></div>'; } )
define("text!views/resources/resources.html", [], function () { return '<section id=\"resources-page\">    <div class=\"text-center\" data-bind=\"visible: resources().length == 1\">        <div class=\"row\">            <div data-bind=\"css: { \'col-md-5\' : showCreateCluster(), \'col-md-12\': !showCreateCluster() }\">                <h2 class=\"page-header text-info\">                    <i class=\"fa fa-database fa-5x\"></i>                    <br />                    <br/>                    <span data-bind=\"visible: has40Features()\">No databases, file systems, counter storages or time series are available.</span>                    <span data-bind=\"visible: !has40Features()\">No databases or file systems are available.</span>                     <br />                    <small data-bind=\"visible: isGlobalAdmin() || canNavigateToAdminSettings()\">                        <span data-bind=\"visible: isGlobalAdmin\">                            Go ahead and <a href=\"#\" data-bind=\"click: newResource\">create one now</a> or                         </span>                        <span data-bind=\"visible: canNavigateToAdminSettings\">                            <a href=\"#\" data-bind=\"click: navigateToAdminSettings\">manage your server</a>                        </span>                    </small>                </h2>            </div>            <div class=\"col-md-2\" data-bind=\"visible: showCreateCluster()\">                <h2 class=\"page-header text-info\" style=\"border-bottom: 0\">                    <i class=\"fa fa-5x\">&nbsp;</i><br />                    OR                </h2>            </div>            <div class=\"col-md-5\" data-bind=\"visible: showCreateCluster()\">                <h2 class=\"page-header text-info\" data-bind=\"visible: canCreateCluster()\">                    <i class=\"fa fa-cubes fa-5x\"></i>                    <br/>                    <br/>                    Create cluster <br/>                    <small>Go ahead and <a href=\"#\" data-bind=\"click: navigateToCreateCluster\">create cluster</a></small><br/>                    <small class=\"text-warning\" data-bind=\"visible: $root.developerLicense()\">Enterprise feature</small>                </h2>                <h2 class=\"page-header text-info\" style=\"color: gray\" data-bind=\"visible: !canCreateCluster()\">                    <i class=\"fa fa-cubes fa-5x\"></i>                    <br />                    <br />                    Create cluster <br />                    <small>Enterprise feature. <a href=\"http://ravendb.net/buy\" target=\"_blank\">Upgrade your license</a></small>                </h2>            </div>        </div>    </div>    <ul class=\"breadcrumb\" data-bind=\"visible: resources().length > 1\">        <a target=\"_blank\" data-bind=\"attr: { href: $root.currentHelpLink }, visible: $root.currentHelpLink\" class=\"global_help_link\"><i class=\"fa fa-question-circle fa-2x\"></i></a>        <li>            <h3>                <a href=\"#resources\" data-bind=\"visible: has40Features()\">                    Databases, File Systems, Counter Storages and Time Series                </a>                <a href=\"#resources\" data-bind=\"visible: !has40Features()\">                    Databases and File Systems                </a>            </h3>        </li>    </ul>    <!-- ko if: alerts().length > 0 -->    <!-- ko foreach: alerts -->    <div class=\"alert alert-dismissable\" role=\"alert\" data-bind=\"css: { \'alert-warning\': alertLevel === \'Warning\', \'alert-danger\': alertLevel === \'Error\' }\">        <button data-bind=\"click: $root.dismissAlert.bind($root, uniqueKey)\" type=\"button\" class=\"close\" data-dismiss=\"alert\"><span aria-hidden=\"true\">&times;</span><span class=\"sr-only\">Close</span></button>        <a data-bind=\"attr: { href: $root.urlForAlert($data) }\"><span data-bind=\"text: title\"></span></a>    </div>    <!-- /ko -->    <!-- /ko -->    <div class=\"btn-toolbar\" data-bind=\"visible: resources().length > 1\">        <div class=\"btn-group\">            <div class=\"checkbox\">                <input id=\"resources_all\" class=\"styled\" type=\"checkbox\" data-bind=\"checkboxTriple: resourcesSelection, event: { change: toggleSelectAll }, visible: isCheckboxVisible\" title=\"Select all or none\">                <label for=\"resources_all\">&nbsp;</label>            </div>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: newResource, visible: isGlobalAdmin\" accesskey=\"n\" title=\"Create a new resource. (Alt+N)\">                <i class=\"fa fa-plus\"></i> New Resource            </button>        </div>        <div class=\"btn-group\">            <input type=\"text\" accesskey=\"/\" class=\"form-control\" placeholder=\"Search\" data-bind=\"value: searchText, valueUpdate: \'afterkeydown\', visible: resources().length > 0\" title=\"Search for a database or file system (Alt+/)\">        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-danger\" data-bind=\"click: deleteCheckedResources, visible: resourcesSelection()\" title=\"Delete selected databases or file systems\"><i class=\"fa fa-trash-o\"></i></button>        </div>        <div class=\"btn-group\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: toggleCheckedResources, visible: allCheckedResourcesDisabled() != null, attr: { title: allCheckedResourcesDisabled() ? \'Enable selected resources\' : \'Disable selected resources\' }\">                <i class=\"fa\" data-bind=\"css: { \'fa-lock\': !allCheckedResourcesDisabled(), \'fa-unlock\': allCheckedResourcesDisabled }\"></i>            </button>        </div>        <div class=\"btn-group\">            <select data-bind=\"options: visibleOptions, optionsText: \'name\', optionsValue: \'value\', value: visibleResource\" class=\"form-control\"></select>        </div>        <div class=\"btn-group pull-right\">            <button class=\"btn btn-warning\" data-bind=\"click: navigateToAdminSettings, visible: canNavigateToAdminSettings\">Manage Your Server</button>        </div>    </div>    <div class=\"resources-container\" data-bind=\"foreach: resources\">        <!--ko if: $data.isDatabase() -->        <div class=\"resource\" data-bind=\"visible: isVisible, click: function () { $root.selectResource($data); return true; }, css: { selected: isSelected, disabled: disabled, isLoaded: isLoaded() }\">            <div class=\"checkbox\" data-bind=\"visible: $root.isGlobalAdmin(), click: function(){ $root.optionsClicked(true); return true; }, style: { display: isChecked() ? \'block\' : \'\' }\">                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: isChecked, attr: { id: \'resource_\' + $index() }\">                <label data-bind=\"attr: { for: \'resource_\' + $index() }\">&nbsp;</label>            </div>            <div class=\"details\">                <h4>                    <!-- ko if: !$data.disabled() && $data.isLicensed() -->                    <a class=\"resource-name\" data-bind=\"attr: { href: $root.getDocumentsUrl($data) }\"><i data-bind=\"attr: { class: iconName }\" class=\"fa\"></i> db/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                    <!-- ko if: $data.disabled() || !$data.isLicensed() -->                    <a class=\"resource-name\"><i data-bind=\"attr: { class: iconName }\"></i> db/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                </h4>                <br />                <!-- ko if: !$data.disabled() && $data.isLicensed() -->                <span class=\"stats\" data-bind=\"style: { visibility: !!statistics() ? \'visible\' : \'hidden\' }, text: itemCountText\"></span>                <!-- /ko -->                <!-- ko if: $data.disabled -->                <span class=\"stats-disabled\">Disabled</span>                <!-- /ko -->                <!-- ko if: !$data.disabled() && !$data.isLicensed() -->                <span class=\"stats-disabled\">Unlicensed</span>                <!-- /ko -->            </div>            <!-- ko if: $data.indexingDisabled -->            <span class=\"indexing-disabled\"><i class=\"fa fa-warning text-warning\" title=\"Indexing Disabled\"></i></span>            <!-- /ko -->            <!-- ko if: $data.rejectClientsMode -->            <span class=\"clients-rejected\"><i class=\"fa fa-warning text-warning\" title=\"Clients rejected mode\"></i></span>            <!-- /ko -->            <div class=\"btn-group\" data-bind=\"click: function(){ $root.optionsClicked(true); }, visible: $root.isGlobalAdmin()\">                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\">                    <i class=\"fa fa-cogs fa-lg\"></i>                </a>                <ul class=\"dropdown-menu dropdown-menu-right\" role=\"menu\">                    <li class=\"dropdown-submenu\">                        <a tabindex=\"-1\" href=\"#\">                            <i class=\"fa fa-arrow-down\"></i>                            <span>Takedowns</span>                        </a>                        <ul class=\"dropdown-menu\">                            <li role=\"presentation\">                                <a data-bind=\"click: function () { $root.toggleSelectedResources([$data]); return true; }, attr: { title: disabled() ? \'Enable Selected Database\' : \'Disable Selected Database\' }\">                                    <i class=\"fa fa-fw\" data-bind=\"css: { \'fa-lock\': !disabled(), \'fa-unlock\': disabled }\"></i>                                    <span data-bind=\"text: disabled() ? \'Enable\' : \'Disable\'\"></span>                                </a>                            </li>                            <li role=\"presentation\">                                <a data-bind=\"click: function () { $root.disableDatabaseIndexing($data); return true; }, attr: { title: indexingDisabled() ? \'Enable Indexing\' : \'Disable Indexing\' }\">                                    <i class=\"fa fa-fw fa-bolt\" data-bind=\"style: { color: indexingDisabled() ?  \'red\' : \'green\' }\"></i>                                    <span data-bind=\"text: indexingDisabled() ? \'Enable indexing\' : \'Disable indexing\'\"></span>                                </a>                            </li>                            <li role=\"presentation\">                                <a data-bind=\"click: function () { $root.toggleRejectDatabaseClients($data); return true; }, attr: { title: rejectClientsMode() ? \'Accept clients mode\' : \'Reject clients mode\' }\">                                    <i class=\"fa fa-fw fa-umbrella\" data-bind=\"style: { color: rejectClientsMode() ? \'red\' : \'green\' }\"></i>                                    <span data-bind=\"text: rejectClientsMode() ? \'Accept clients mode\' : \'Reject clients mode\'\"></span>                                </a>                            </li>                        </ul>                    </li>                    <li class=\"divider\"></li>                    <li role=\"presentation\" class=\"delete-resource\">                        <a title=\"Delete selected database\" data-bind=\"click: function () { $root.deleteSelectedResources([$data]); return true; }\">                            <i class=\"fa fa-fw fa-trash-o\"></i>                            <span>Delete</span>                        </a>                    </li>                </ul>            </div>        </div>        <!-- /ko -->        <!-- ko if: $data.isFileSystem() -->        <div class=\"resource fs\" data-bind=\"visible: isVisible, click: function () { $root.selectResource($data); return true; }, css: { selected: isSelected, disabled: disabled, isLoaded: isLoaded() }\">            <div class=\"checkbox\" data-bind=\"visible: $root.isGlobalAdmin(), click: function () { $root.optionsClicked(true); return true; }, style: { display: isChecked() ? \'block\' : \'\' }\">                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: isChecked, attr: { id: \'resource_\' + $index() }\">                <label data-bind=\"attr: { for: \'resource_\' + $index() }\">&nbsp;</label>            </div>            <div class=\"details\">                <h4>                    <!-- ko if: !$data.disabled() && $data.isLicensed() -->                    <a class=\"resource-name\" data-bind=\"attr: { href: $root.getFileSystemFilesUrl($data) }\"><i data-bind=\"attr: { class: iconName }\"></i> fs/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                    <!-- ko if: $data.disabled() || !$data.isLicensed() -->                    <a class=\"resource-name\"><i data-bind=\"attr: { class: iconName }\"></i> fs/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                </h4>                <br />                <!-- ko if: !$data.disabled() && $data.isLicensed() -->                <span class=\"stats\" data-bind=\"style: { visibility: !!statistics() ? \'visible\' : \'hidden\' }, text: itemCountText\"></span>                <!-- /ko -->                <!-- ko if: $data.disabled -->                <span class=\"stats-disabled\">Disabled</span>                <!-- /ko -->                <!-- ko if: !$data.disabled() && !$data.isLicensed() -->                <span class=\"stats-disabled\">Unlicensed</span>                <!-- /ko -->            </div>            <div class=\"btn-group\" data-bind=\"click: function () { $root.optionsClicked(true); }, visible: $root.isGlobalAdmin()\">                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\">                    <i class=\"fa fa-cogs fa-lg\"></i>                </a>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li role=\"presentation\">                        <a data-bind=\"click: function () { $root.toggleSelectedResources([$data]); return true; }, attr: { title: disabled() ? \'Enable Selected Database\' : \'Disable Selected Database\' }\">                            <i class=\"fa fa-fw\" data-bind=\"css: { \'fa-lock\': !disabled(), \'fa-unlock\': disabled }\"></i>                            <span data-bind=\"text: disabled() ? \'Enable\' : \'Disable\'\"></span>                        </a>                    </li>                    <li class=\"divider\"></li>                    <li role=\"presentation\" class=\"delete-resource\">                        <a title=\"Delete selected filesystem\" data-bind=\"click: function () { $root.deleteSelectedResources([$data]); return true; }\">                            <i class=\"fa fa-fw fa-trash-o\"></i>                            <span>Delete</span>                        </a>                    </li>                </ul>            </div>        </div>        <!-- /ko -->        <!-- ko if: $data.isCounterStorage() -->        <div class=\"resource cs\" data-bind=\"visible: isVisible, click: function () { $root.selectResource($data); return true; }, css: { selected: isSelected, disabled: disabled }\">            <div class=\"checkbox\" data-bind=\"visible: $root.isGlobalAdmin(), click: function () { $root.optionsClicked(true); return true; }, style: { display: isChecked() ? \'block\' : \'\' }\">                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: isChecked, attr: { id: \'resource_\' + $index() }\">                <label data-bind=\"attr: { for: \'resource_\' + $index() }\">&nbsp;</label>            </div>            <div class=\"details\">                <h4>                    <!-- ko if: !$data.disabled() && $data.isLicensed() -->                    <a class=\"resource-name\" data-bind=\"attr: { href: $root.getCounterStorageCountersUrl($data) }\"><i data-bind=\"attr: { class: iconName }\"></i> cs/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                    <!-- ko if: $data.disabled() || !$data.isLicensed() -->                    <a class=\"resource-name\"><i data-bind=\"attr: { class: iconName }\"></i> fs/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                </h4>                <br />                <!-- ko if: !$data.disabled() && $data.isLicensed() -->                <!--TODO: implement stats-->                <span class=\"stats\" data-bind=\"style: { visibility: !!statistics() ? \'visible\' : \'hidden\' }, text: itemCountText\"></span>                <!-- /ko -->                <!-- ko if: $data.disabled -->                <span class=\"stats-disabled\">Disabled</span>                <!-- /ko -->                <!-- ko if: !$data.disabled() && !$data.isLicensed() -->                <span class=\"stats-disabled\">Unlicensed</span>                <!-- /ko -->            </div>            <div class=\"btn-group\" data-bind=\"click: function () { $root.optionsClicked(true); }, visible: $root.isGlobalAdmin()\">                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\">                    <i class=\"fa fa-cogs fa-lg\"></i>                </a>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li role=\"presentation\">                        <a data-bind=\"click: function () { $root.toggleSelectedResources([$data]); return true; }, attr: { title: disabled() ? \'Enable Selected Counter Storage\' : \'Disable Selected Counter Storage\' }\">                            <i class=\"fa fa-fw\" data-bind=\"css: { \'fa-lock\': !disabled(), \'fa-unlock\': disabled }\"></i>                            <span data-bind=\"text: disabled() ? \'Enable\' : \'Disable\'\"></span>                        </a>                    </li>                    <li class=\"divider\"></li>                    <li role=\"presentation\" class=\"delete-resource\">                        <a title=\"Delete Selected Counter Storage\" data-bind=\"click: function () { $root.deleteSelectedResources([$data]); return true; }\">                            <i class=\"fa fa-fw fa-trash-o\"></i>                            <span>Delete</span>                        </a>                    </li>                </ul>            </div>        </div>        <!-- /ko -->        <!-- ko if: $data.isTimeSeries() -->        <div class=\"resource ts\" data-bind=\"visible: isVisible, click: function () { $root.selectResource($data); return true; }, css: { selected: isSelected, disabled: disabled }\">            <div class=\"checkbox\" data-bind=\"visible: $root.isGlobalAdmin(), click: function () { $root.optionsClicked(true); return true; }, style: { display: isChecked() ? \'block\' : \'\' }\">                <input type=\"checkbox\" class=\"styled\" data-bind=\"checked: isChecked, attr: { id: \'resource_\' + $index() }\">                <label data-bind=\"attr: { for: \'resource_\' + $index() }\">&nbsp;</label>            </div>            <div class=\"details\">                <h4>                    <!-- ko if: !$data.disabled() && $data.isLicensed() -->                    <a class=\"resource-name\" data-bind=\"attr: { href: $root.getTimeSeriesUrl($data) }\"><i data-bind=\"attr: { class: iconName }\"></i> ts/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                    <!-- ko if: $data.disabled() || !$data.isLicensed() -->                    <a class=\"resource-name\"><i data-bind=\"attr: { class: iconName }\"></i> fs/<span data-bind=\"text: name\"></span></a>                    <!-- /ko -->                </h4>                <br />                <!-- ko if: !$data.disabled() && $data.isLicensed() -->                <!--TODO: implement stats-->                <span class=\"stats\" data-bind=\"style: { visibility: !!statistics() ? \'visible\' : \'hidden\' }, text: itemCountText\"></span>                <!-- /ko -->                <!-- ko if: $data.disabled -->                <span class=\"stats-disabled\">Disabled</span>                <!-- /ko -->                <!-- ko if: !$data.disabled() && !$data.isLicensed() -->                <span class=\"stats-disabled\">Unlicensed</span>                <!-- /ko -->            </div>            <div class=\"btn-group\" data-bind=\"click: function () { $root.optionsClicked(true); }, visible: $root.isGlobalAdmin()\">                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\">                    <i class=\"fa fa-cogs fa-lg\"></i>                </a>                <ul class=\"dropdown-menu\" role=\"menu\">                    <li role=\"presentation\">                        <a data-bind=\"click: function () { $root.toggleSelectedResources([$data]); return true; }, attr: { title: disabled() ? \'Enable Selected Time Series\' : \'Disable Selected Time Series\' }\">                            <i class=\"fa fa-fw\" data-bind=\"css: { \'fa-lock\': !disabled(), \'fa-unlock\': disabled }\"></i>                            <span data-bind=\"text: disabled() ? \'Enable\' : \'Disable\'\"></span>                        </a>                    </li>                    <li class=\"divider\"></li>                    <li role=\"presentation\" class=\"delete-resource\">                        <a title=\"Delete Selected Time Series\" data-bind=\"click: function () { $root.deleteSelectedResources([$data]); return true; }\">                            <i class=\"fa fa-fw fa-trash-o\"></i>                            <span>Delete</span>                        </a>                    </li>                </ul>            </div>        </div>        <!-- /ko -->    </div></section>'; } )
define("text!views/timeSeries/deleteKey.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\">    <div class=\"modal-header\">        <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\">            <i class=\"fa fa-times\"></i>        </button>        <h4 class=\"modal-title\">Sure?</h4>    </div>    <div class=\"modal-body\" data-bind=\"with: key\">        <span>You\'re deleting the </span>        \'<strong data-bind=\"text: Key\"></strong>\' key, in \'<strong data-bind=\"text: Type\"></strong>\' type.        <strong data-bind=\"visible: Points > 1, text: \'(\' + Points + \' points)\'\"></strong>        <strong data-bind=\"visible: Points == 1, text: \'(\' + Points + \' point)\'\"></strong>    </div>    <div class=\"modal-footer\">        <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Nevermind</button>        <button type=\"button\" class=\"btn btn-primary\" data-bind=\"click: deleteKey\">Yep, delete</button>    </div></div>'; } )
define("text!views/timeSeries/editPointDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 52%\">    <form data-bind=\"submit: nextOrCreate\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>            <h4 class=\"modal-title\" data-bind=\"text: (isNew() ? \'Append\' : \'Update\' ) + \' Point\'\"></h4>        </div>        <div class=\"modal-body\">            <div class=\"row\">                <div data-bind=\"with: editedPoint\">                    <div class=\"col-md-12\">                        <label class=\"col-md-4\">Type:</label>                        <div class=\"col-md-7\">                            <span data-bind=\"text: type\"></span>                        </div>                    </div>                    <div class=\"col-md-12\">                        <label class=\"col-md-4\" for=\"key\">Key:</label>                        <div class=\"col-md-7\">                            <!-- ko if: isNew() -->                            <input required type=\"text\" class=\"form-control\" id=\"key\" placeholder=\"Key\" data-bind=\"value: key, customValidity: keyCustomValidityError, valueUpdate: \'afterkeydown\'\">                            <!-- /ko -->                            <!-- ko ifnot: isNew() -->                            <span data-bind=\"text: key\"></span>                            <!-- /ko -->                        </div>                    </div>                    <div class=\"col-md-12\">                        <label class=\"col-md-4\" for=\"at\">At:</label>                        <div class=\"col-md-7\" style=\"display: inline-block\">                            <input required type=\"text\" class=\"form-control\" id=\"at\" placeholder=\"At\" data-bind=\"value: at, customValidity: atCustomValidityError, valueUpdate: \'afterkeydown\'\">                        </div>                    </div>                    <div class=\"col-md-12\" data-bind=\"foreach: fields\">                        <label class=\"col-md-4\" data-bind=\"text: $data, for: $data\"></label>                        <div class=\"col-md-7\" style=\"display: inline-block\">                            <input required type=\"number\" class=\"form-control\" data-bind=\"attr: { placeholder: $data, id: $data }, value: $parent.values()[$index()], valueUpdate: \'afterkeydown\'\">                        </div>                    </div>                    <div class=\"col-md-12\" data-bind=\"visible: isNew()\">                        *Using an existing point will update its value                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"text: isNew() ? \'Append\' : \'Update\'\"></button>        </div>    </form></div>'; } )
define("text!views/timeSeries/editTypeDialog.html", [], function () { return '<div class=\"messageBox\" tabindex=\"-1\" style=\"width: 52%\">    <form data-bind=\"submit: nextOrCreate\">        <div class=\"modal-header\">            <button type=\"button\" class=\"close\" data-bind=\"click: cancel\" aria-hidden=\"true\"></button>            <h4 class=\"modal-title\" data-bind=\"text: (isNew() ? \'New\' : \'Update\') + \' Type\'\"></h4>        </div>        <div class=\"modal-body\">            <div class=\"row\">                <div data-bind=\"with: editedType\">                    <div class=\"col-md-12\">                        <label class=\"col-md-3\" for=\"type\">Type:</label>                        <div class=\"col-md-7\" for=\"type\">                            <input required type=\"text\" class=\"form-control\" id=\"type\" placeholder=\"Type Name\" data-bind=\"value: type, customValidity: typeCustomValidityError, valueUpdate: \'afterkeydown\'\">                        </div>                    </div>                    <div class=\"col-md-12\" data-bind=\"foreach: fields\">                        <label class=\"col-md-3\" data-bind=\"text: \'Field \' + ($index() + 1) + \':\', for: \'field\' + $index()\"></label>                        <div class=\"col-md-7\" style=\"display: inline-block\">                            <input required type=\"text\" class=\"form-control\" data-bind=\"attr: { placeholder: \'Field \' + ($index() + 1), id: \'field\' + $index() }, value: $parent.fields()[$index()], valueUpdate: \'afterkeydown\'\">                        </div>                        <div class=\"col-md-1\" style=\"display: inline-block\">                            <button type=\"button\" style=\"font-size: 14px;\" class=\"red\" title=\"Add field\" data-bind=\"click: $parent.removeField.bind($parent, $data), visible: $parent.fields().length > 1\">                                <i class=\"fa fa-remove\"></i>                            </button>                        </div>                    </div>                    <div class=\"col-md-12\" style=\"text-align: center;margin-left: 5px;margin-top: 4px;\">                        <button type=\"button\" style=\"font-size: 14px;\" class=\"btn btn-default\" title=\"Add field\" data-bind=\"click: addField\">                            <i class=\"fa fa-plus\"></i> Add Field                        </button>                    </div>                </div>            </div>        </div>        <div class=\"modal-footer\">            <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: cancel\">Cancel</button>            <button type=\"submit\" class=\"btn btn-primary\" data-bind=\"text: isNew() ? \'Append\' : \'Update\'\"></button>        </div>    </form></div>'; } )
define("text!views/timeSeries/configuration/configuration.html", [], function () { return '<ul class=\"breadcrumb\">    <li>        <h3>            <a href=\"#resources\">                Time Series            </a>        </h3>    </li>    <li class=\"active\" data-bind=\"with: activeTimeSeries\">        <a data-bind=\"text: name, attr: { href: $parent.appUrls.timeSeriesType }\"></a>    </li>    <li class=\"active\">        <span>Configurations</span>    </li>    <li class=\"active\" data-bind=\"text: activeSubViewTitle\">        <a data-bind=\"attr: { href: hash }, text: title\"></a>    </li></ul><div class=\"settings-container row\">    <div class=\"col-md-2\">        <ul class=\"nav nav-pills nav-stacked\" data-bind=\"foreach: router.navigationModel\">            <li data-bind=\"css: { active: isActive() }\">                <a data-bind=\"attr: { href: hash }, text: title\"></a>            </li>        </ul>    </div>    <div class=\"col-md-10\" data-bind=\"router: { cacheViews: false }\">    </div></div>'; } )
define("text!views/timeSeries/configuration/types.html", [], function () { return '<div id=\"typesConfigurationPage\">    <div class=\"col-md-12\">        <div class=\"btn-toolbar\" role=\"toolbar\">            <div class=\"btn-group\">                <button type=\"button\" class=\"btn btn-primary\" accesskey=\"n\" data-bind=\"click: newType\" title=\"Create a new type (Alt+N)\">                    <i class=\"fa fa-plus\"></i> New Type                </button>            </div>            <div class=\"btn-group\" data-bind=\"visible: hasAnyTypesSelected\">                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: changeType, enable: canChangeType\" title=\"Change Type Value\">                    <i class=\"fa fa-pencil\"></i>                </button>                <button type=\"button\" class=\"btn btn-default\" data-bind=\"click: deleteSelectedTypes, attr: { title: selectedTypesIndices().length == 1 ? \'Delete Type\' : \'Delete Types\' }\">                    <i class=\"fa fa-trash-o\"></i>                </button>            </div>            <div class=\"btn-group\">                <div id=\"loadingIndicator\" data-bind=\"visible: showLoadingIndicatorThrottled\">                    <i class=\"fa fa-spinner fa-spin fa-2x\"></i>                </div>            </div>        </div>        <div id=\"typesGrid\" data-bind=\"widget: { kind: \'virtualTable\', itemsSource: typesList, selectedIndices: selectedTypesIndices, gridSelector: \'#typesGrid\', rowsAreLoading: showLoadingIndicator, contextMenuOptions: [\'ChangeValue\', \'Delete\'], showIds: false, noResultsMessage: \'No types were found!\', viewType: viewType }\"></div>    </div></div>'; } )